# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by lib/unicore/mktables from the Unicode
# database, Version 17.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.


return <<'END';
00000
NULL

00000
NUL

00001
START OF HEADING

00001
SOH

00002
START OF TEXT

00002
STX

00003
END OF TEXT

00003
ETX

00004
END OF TRANSMISSION

00004
EOT

00005
ENQUIRY

00005
ENQ

00006
ACKNOWLEDGE

00006
ACK

00007
ALERT

00007
BEL

00008
BACKSPACE

00008
BS

00009
CHARACTER TABULATION

00009
HORIZONTAL TABULATION

00009
HT

00009
TAB

0000A
LINE FEED

0000A
NEW LINE

0000A
END OF LINE

0000A
LINE FEED (LF)

0000A
EOL

0000A
LF

0000A
NL

0000B
LINE TABULATION

0000B
VERTICAL TABULATION

0000B
VT

0000C
FORM FEED

0000C
FORM FEED (FF)

0000C
FF

0000D
CARRIAGE RETURN

0000D
CARRIAGE RETURN (CR)

0000D
CR

0000E
SHIFT OUT

0000E
LOCKING-SHIFT ONE

0000E
SO

0000F
SHIFT IN

0000F
LOCKING-SHIFT ZERO

0000F
SI

00010
DATA LINK ESCAPE

00010
DLE

00011
DEVICE CONTROL ONE

00011
DC1

00012
DEVICE CONTROL TWO

00012
DC2

00013
DEVICE CONTROL THREE

00013
DC3

00014
DEVICE CONTROL FOUR

00014
DC4

00015
NEGATIVE ACKNOWLEDGE

00015
NAK

00016
SYNCHRONOUS IDLE

00016
SYN

00017
END OF TRANSMISSION BLOCK

00017
ETB

00018
CANCEL

00018
CAN

00019
END OF MEDIUM

00019
EM

00019
EOM

0001A
SUBSTITUTE

0001A
SUB

0001B
ESCAPE

0001B
ESC

0001C
INFORMATION SEPARATOR FOUR

0001C
FILE SEPARATOR

0001C
FS

0001D
INFORMATION SEPARATOR THREE

0001D
GROUP SEPARATOR

0001D
GS

0001E
INFORMATION SEPARATOR TWO

0001E
RECORD SEPARATOR

0001E
RS

0001F
INFORMATION SEPARATOR ONE

0001F
UNIT SEPARATOR

0001F
US

00020
SPACE

00020
SP

00021
EXCLAMATION MARK

00022
QUOTATION MARK

00023
NUMBER SIGN

00024
DOLLAR SIGN

00025
PERCENT SIGN

00026
AMPERSAND

00027
APOSTROPHE

00028
LEFT PARENTHESIS

00029
RIGHT PARENTHESIS

0002A
ASTERISK

0002B
PLUS SIGN

0002C
COMMA

0002D
HYPHEN-MINUS

0002E
FULL STOP

0002F
SOLIDUS

00030
DIGIT ZERO

00031
DIGIT ONE

00032
DIGIT TWO

00033
DIGIT THREE

00034
DIGIT FOUR

00035
DIGIT FIVE

00036
DIGIT SIX

00037
DIGIT SEVEN

00038
DIGIT EIGHT

00039
DIGIT NINE

0003A
COLON

0003B
SEMICOLON

0003C
LESS-THAN SIGN

0003D
EQUALS SIGN

0003E
GREATER-THAN SIGN

0003F
QUESTION MARK

00040
COMMERCIAL AT

00041
LATIN CAPITAL LETTER A

00042
LATIN CAPITAL LETTER B

00043
LATIN CAPITAL LETTER C

00044
LATIN CAPITAL LETTER D

00045
LATIN CAPITAL LETTER E

00046
LATIN CAPITAL LETTER F

00047
LATIN CAPITAL LETTER G

00048
LATIN CAPITAL LETTER H

00049
LATIN CAPITAL LETTER I

0004A
LATIN CAPITAL LETTER J

0004B
LATIN CAPITAL LETTER K

0004C
LATIN CAPITAL LETTER L

0004D
LATIN CAPITAL LETTER M

0004E
LATIN CAPITAL LETTER N

0004F
LATIN CAPITAL LETTER O

00050
LATIN CAPITAL LETTER P

00051
LATIN CAPITAL LETTER Q

00052
LATIN CAPITAL LETTER R

00053
LATIN CAPITAL LETTER S

00054
LATIN CAPITAL LETTER T

00055
LATIN CAPITAL LETTER U

00056
LATIN CAPITAL LETTER V

00057
LATIN CAPITAL LETTER W

00058
LATIN CAPITAL LETTER X

00059
LATIN CAPITAL LETTER Y

0005A
LATIN CAPITAL LETTER Z

0005B
LEFT SQUARE BRACKET

0005C
REVERSE SOLIDUS

0005D
RIGHT SQUARE BRACKET

0005E
CIRCUMFLEX ACCENT

0005F
LOW LINE

00060
GRAVE ACCENT

00061
LATIN SMALL LETTER A

00062
LATIN SMALL LETTER B

00063
LATIN SMALL LETTER C

00064
LATIN SMALL LETTER D

00065
LATIN SMALL LETTER E

00066
LATIN SMALL LETTER F

00067
LATIN SMALL LETTER G

00068
LATIN SMALL LETTER H

00069
LATIN SMALL LETTER I

0006A
LATIN SMALL LETTER J

0006B
LATIN SMALL LETTER K

0006C
LATIN SMALL LETTER L

0006D
LATIN SMALL LETTER M

0006E
LATIN SMALL LETTER N

0006F
LATIN SMALL LETTER O

00070
LATIN SMALL LETTER P

00071
LATIN SMALL LETTER Q

00072
LATIN SMALL LETTER R

00073
LATIN SMALL LETTER S

00074
LATIN SMALL LETTER T

00075
LATIN SMALL LETTER U

00076
LATIN SMALL LETTER V

00077
LATIN SMALL LETTER W

00078
LATIN SMALL LETTER X

00079
LATIN SMALL LETTER Y

0007A
LATIN SMALL LETTER Z

0007B
LEFT CURLY BRACKET

0007C
VERTICAL LINE

0007D
RIGHT CURLY BRACKET

0007E
TILDE

0007F
DELETE

0007F
DEL

00080
PADDING CHARACTER

00080
PAD

00081
HIGH OCTET PRESET

00081
HOP

00082
BREAK PERMITTED HERE

00082
BPH

00083
NO BREAK HERE

00083
NBH

00084
INDEX

00084
IND

00085
NEXT LINE

00085
NEXT LINE (NEL)

00085
NEL

00086
START OF SELECTED AREA

00086
SSA

00087
END OF SELECTED AREA

00087
ESA

00088
CHARACTER TABULATION SET

00088
HORIZONTAL TABULATION SET

00088
HTS

00089
CHARACTER TABULATION WITH JUSTIFICATION

00089
HORIZONTAL TABULATION WITH JUSTIFICATION

00089
HTJ

0008A
LINE TABULATION SET

0008A
VERTICAL TABULATION SET

0008A
VTS

0008B
PARTIAL LINE FORWARD

0008B
PARTIAL LINE DOWN

0008B
PLD

0008C
PARTIAL LINE BACKWARD

0008C
PARTIAL LINE UP

0008C
PLU

0008D
REVERSE LINE FEED

0008D
REVERSE INDEX

0008D
RI

0008E
SINGLE SHIFT TWO

0008E
SINGLE-SHIFT-2

0008E
SS2

0008F
SINGLE SHIFT THREE

0008F
SINGLE-SHIFT-3

0008F
SS3

00090
DEVICE CONTROL STRING

00090
DCS

00091
PRIVATE USE ONE

00091
PRIVATE USE-1

00091
PU1

00092
PRIVATE USE TWO

00092
PRIVATE USE-2

00092
PU2

00093
SET TRANSMIT STATE

00093
STS

00094
CANCEL CHARACTER

00094
CCH

00095
MESSAGE WAITING

00095
MW

00096
START OF GUARDED AREA

00096
START OF PROTECTED AREA

00096
SPA

00097
END OF GUARDED AREA

00097
END OF PROTECTED AREA

00097
EPA

00098
START OF STRING

00098
SOS

00099
SINGLE GRAPHIC CHARACTER INTRODUCER

00099
SGC

0009A
SINGLE CHARACTER INTRODUCER

0009A
SCI

0009B
CONTROL SEQUENCE INTRODUCER

0009B
CSI

0009C
STRING TERMINATOR

0009C
ST

0009D
OPERATING SYSTEM COMMAND

0009D
OSC

0009E
PRIVACY MESSAGE

0009E
PM

0009F
APPLICATION PROGRAM COMMAND

0009F
APC

000A0
NO-BREAK SPACE

000A0
NBSP

000A1
INVERTED EXCLAMATION MARK

000A2
CENT SIGN

000A3
POUND SIGN

000A4
CURRENCY SIGN

000A5
YEN SIGN

000A6
BROKEN BAR

000A7
SECTION SIGN

000A8
DIAERESIS

000A9
COPYRIGHT SIGN

000AA
FEMININE ORDINAL INDICATOR

000AB
LEFT-POINTING DOUBLE ANGLE QUOTATION MARK

000AC
NOT SIGN

000AD
SOFT HYPHEN

000AD
SHY

000AE
REGISTERED SIGN

000AF
MACRON

000B0
DEGREE SIGN

000B1
PLUS-MINUS SIGN

000B2
SUPERSCRIPT TWO

000B3
SUPERSCRIPT THREE

000B4
ACUTE ACCENT

000B5
MICRO SIGN

000B6
PILCROW SIGN

000B7
MIDDLE DOT

000B8
CEDILLA

000B9
SUPERSCRIPT ONE

000BA
MASCULINE ORDINAL INDICATOR

000BB
RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK

000BC
VULGAR FRACTION ONE QUARTER

000BD
VULGAR FRACTION ONE HALF

000BE
VULGAR FRACTION THREE QUARTERS

000BF
INVERTED QUESTION MARK

000C0
LATIN CAPITAL LETTER A WITH GRAVE

000C1
LATIN CAPITAL LETTER A WITH ACUTE

000C2
LATIN CAPITAL LETTER A WITH CIRCUMFLEX

000C3
LATIN CAPITAL LETTER A WITH TILDE

000C4
LATIN CAPITAL LETTER A WITH DIAERESIS

000C5
LATIN CAPITAL LETTER A WITH RING ABOVE

000C6
LATIN CAPITAL LETTER AE

000C7
LATIN CAPITAL LETTER C WITH CEDILLA

000C8
LATIN CAPITAL LETTER E WITH GRAVE

000C9
LATIN CAPITAL LETTER E WITH ACUTE

000CA
LATIN CAPITAL LETTER E WITH CIRCUMFLEX

000CB
LATIN CAPITAL LETTER E WITH DIAERESIS

000CC
LATIN CAPITAL LETTER I WITH GRAVE

000CD
LATIN CAPITAL LETTER I WITH ACUTE

000CE
LATIN CAPITAL LETTER I WITH CIRCUMFLEX

000CF
LATIN CAPITAL LETTER I WITH DIAERESIS

000D0
LATIN CAPITAL LETTER ETH

000D1
LATIN CAPITAL LETTER N WITH TILDE

000D2
LATIN CAPITAL LETTER O WITH GRAVE

000D3
LATIN CAPITAL LETTER O WITH ACUTE

000D4
LATIN CAPITAL LETTER O WITH CIRCUMFLEX

000D5
LATIN CAPITAL LETTER O WITH TILDE

000D6
LATIN CAPITAL LETTER O WITH DIAERESIS

000D7
MULTIPLICATION SIGN

000D8
LATIN CAPITAL LETTER O WITH STROKE

000D9
LATIN CAPITAL LETTER U WITH GRAVE

000DA
LATIN CAPITAL LETTER U WITH ACUTE

000DB
LATIN CAPITAL LETTER U WITH CIRCUMFLEX

000DC
LATIN CAPITAL LETTER U WITH DIAERESIS

000DD
LATIN CAPITAL LETTER Y WITH ACUTE

000DE
LATIN CAPITAL LETTER THORN

000DF
LATIN SMALL LETTER SHARP S

000E0
LATIN SMALL LETTER A WITH GRAVE

000E1
LATIN SMALL LETTER A WITH ACUTE

000E2
LATIN SMALL LETTER A WITH CIRCUMFLEX

000E3
LATIN SMALL LETTER A WITH TILDE

000E4
LATIN SMALL LETTER A WITH DIAERESIS

000E5
LATIN SMALL LETTER A WITH RING ABOVE

000E6
LATIN SMALL LETTER AE

000E7
LATIN SMALL LETTER C WITH CEDILLA

000E8
LATIN SMALL LETTER E WITH GRAVE

000E9
LATIN SMALL LETTER E WITH ACUTE

000EA
LATIN SMALL LETTER E WITH CIRCUMFLEX

000EB
LATIN SMALL LETTER E WITH DIAERESIS

000EC
LATIN SMALL LETTER I WITH GRAVE

000ED
LATIN SMALL LETTER I WITH ACUTE

000EE
LATIN SMALL LETTER I WITH CIRCUMFLEX

000EF
LATIN SMALL LETTER I WITH DIAERESIS

000F0
LATIN SMALL LETTER ETH

000F1
LATIN SMALL LETTER N WITH TILDE

000F2
LATIN SMALL LETTER O WITH GRAVE

000F3
LATIN SMALL LETTER O WITH ACUTE

000F4
LATIN SMALL LETTER O WITH CIRCUMFLEX

000F5
LATIN SMALL LETTER O WITH TILDE

000F6
LATIN SMALL LETTER O WITH DIAERESIS

000F7
DIVISION SIGN

000F8
LATIN SMALL LETTER O WITH STROKE

000F9
LATIN SMALL LETTER U WITH GRAVE

000FA
LATIN SMALL LETTER U WITH ACUTE

000FB
LATIN SMALL LETTER U WITH CIRCUMFLEX

000FC
LATIN SMALL LETTER U WITH DIAERESIS

000FD
LATIN SMALL LETTER Y WITH ACUTE

000FE
LATIN SMALL LETTER THORN

000FF
LATIN SMALL LETTER Y WITH DIAERESIS

00100
LATIN CAPITAL LETTER A WITH MACRON

00101
LATIN SMALL LETTER A WITH MACRON

00102
LATIN CAPITAL LETTER A WITH BREVE

00103
LATIN SMALL LETTER A WITH BREVE

00104
LATIN CAPITAL LETTER A WITH OGONEK

00105
LATIN SMALL LETTER A WITH OGONEK

00106
LATIN CAPITAL LETTER C WITH ACUTE

00107
LATIN SMALL LETTER C WITH ACUTE

00108
LATIN CAPITAL LETTER C WITH CIRCUMFLEX

00109
LATIN SMALL LETTER C WITH CIRCUMFLEX

0010A
LATIN CAPITAL LETTER C WITH DOT ABOVE

0010B
LATIN SMALL LETTER C WITH DOT ABOVE

0010C
LATIN CAPITAL LETTER C WITH CARON

0010D
LATIN SMALL LETTER C WITH CARON

0010E
LATIN CAPITAL LETTER D WITH CARON

0010F
LATIN SMALL LETTER D WITH CARON

00110
LATIN CAPITAL LETTER D WITH STROKE

00111
LATIN SMALL LETTER D WITH STROKE

00112
LATIN CAPITAL LETTER E WITH MACRON

00113
LATIN SMALL LETTER E WITH MACRON

00114
LATIN CAPITAL LETTER E WITH BREVE

00115
LATIN SMALL LETTER E WITH BREVE

00116
LATIN CAPITAL LETTER E WITH DOT ABOVE

00117
LATIN SMALL LETTER E WITH DOT ABOVE

00118
LATIN CAPITAL LETTER E WITH OGONEK

00119
LATIN SMALL LETTER E WITH OGONEK

0011A
LATIN CAPITAL LETTER E WITH CARON

0011B
LATIN SMALL LETTER E WITH CARON

0011C
LATIN CAPITAL LETTER G WITH CIRCUMFLEX

0011D
LATIN SMALL LETTER G WITH CIRCUMFLEX

0011E
LATIN CAPITAL LETTER G WITH BREVE

0011F
LATIN SMALL LETTER G WITH BREVE

00120
LATIN CAPITAL LETTER G WITH DOT ABOVE

00121
LATIN SMALL LETTER G WITH DOT ABOVE

00122
LATIN CAPITAL LETTER G WITH CEDILLA

00123
LATIN SMALL LETTER G WITH CEDILLA

00124
LATIN CAPITAL LETTER H WITH CIRCUMFLEX

00125
LATIN SMALL LETTER H WITH CIRCUMFLEX

00126
LATIN CAPITAL LETTER H WITH STROKE

00127
LATIN SMALL LETTER H WITH STROKE

00128
LATIN CAPITAL LETTER I WITH TILDE

00129
LATIN SMALL LETTER I WITH TILDE

0012A
LATIN CAPITAL LETTER I WITH MACRON

0012B
LATIN SMALL LETTER I WITH MACRON

0012C
LATIN CAPITAL LETTER I WITH BREVE

0012D
LATIN SMALL LETTER I WITH BREVE

0012E
LATIN CAPITAL LETTER I WITH OGONEK

0012F
LATIN SMALL LETTER I WITH OGONEK

00130
LATIN CAPITAL LETTER I WITH DOT ABOVE

00131
LATIN SMALL LETTER DOTLESS I

00132
LATIN CAPITAL LIGATURE IJ

00133
LATIN SMALL LIGATURE IJ

00134
LATIN CAPITAL LETTER J WITH CIRCUMFLEX

00135
LATIN SMALL LETTER J WITH CIRCUMFLEX

00136
LATIN CAPITAL LETTER K WITH CEDILLA

00137
LATIN SMALL LETTER K WITH CEDILLA

00138
LATIN SMALL LETTER KRA

00139
LATIN CAPITAL LETTER L WITH ACUTE

0013A
LATIN SMALL LETTER L WITH ACUTE

0013B
LATIN CAPITAL LETTER L WITH CEDILLA

0013C
LATIN SMALL LETTER L WITH CEDILLA

0013D
LATIN CAPITAL LETTER L WITH CARON

0013E
LATIN SMALL LETTER L WITH CARON

0013F
LATIN CAPITAL LETTER L WITH MIDDLE DOT

00140
LATIN SMALL LETTER L WITH MIDDLE DOT

00141
LATIN CAPITAL LETTER L WITH STROKE

00142
LATIN SMALL LETTER L WITH STROKE

00143
LATIN CAPITAL LETTER N WITH ACUTE

00144
LATIN SMALL LETTER N WITH ACUTE

00145
LATIN CAPITAL LETTER N WITH CEDILLA

00146
LATIN SMALL LETTER N WITH CEDILLA

00147
LATIN CAPITAL LETTER N WITH CARON

00148
LATIN SMALL LETTER N WITH CARON

00149
LATIN SMALL LETTER N PRECEDED BY APOSTROPHE

0014A
LATIN CAPITAL LETTER ENG

0014B
LATIN SMALL LETTER ENG

0014C
LATIN CAPITAL LETTER O WITH MACRON

0014D
LATIN SMALL LETTER O WITH MACRON

0014E
LATIN CAPITAL LETTER O WITH BREVE

0014F
LATIN SMALL LETTER O WITH BREVE

00150
LATIN CAPITAL LETTER O WITH DOUBLE ACUTE

00151
LATIN SMALL LETTER O WITH DOUBLE ACUTE

00152
LATIN CAPITAL LIGATURE OE

00153
LATIN SMALL LIGATURE OE

00154
LATIN CAPITAL LETTER R WITH ACUTE

00155
LATIN SMALL LETTER R WITH ACUTE

00156
LATIN CAPITAL LETTER R WITH CEDILLA

00157
LATIN SMALL LETTER R WITH CEDILLA

00158
LATIN CAPITAL LETTER R WITH CARON

00159
LATIN SMALL LETTER R WITH CARON

0015A
LATIN CAPITAL LETTER S WITH ACUTE

0015B
LATIN SMALL LETTER S WITH ACUTE

0015C
LATIN CAPITAL LETTER S WITH CIRCUMFLEX

0015D
LATIN SMALL LETTER S WITH CIRCUMFLEX

0015E
LATIN CAPITAL LETTER S WITH CEDILLA

0015F
LATIN SMALL LETTER S WITH CEDILLA

00160
LATIN CAPITAL LETTER S WITH CARON

00161
LATIN SMALL LETTER S WITH CARON

00162
LATIN CAPITAL LETTER T WITH CEDILLA

00163
LATIN SMALL LETTER T WITH CEDILLA

00164
LATIN CAPITAL LETTER T WITH CARON

00165
LATIN SMALL LETTER T WITH CARON

00166
LATIN CAPITAL LETTER T WITH STROKE

00167
LATIN SMALL LETTER T WITH STROKE

00168
LATIN CAPITAL LETTER U WITH TILDE

00169
LATIN SMALL LETTER U WITH TILDE

0016A
LATIN CAPITAL LETTER U WITH MACRON

0016B
LATIN SMALL LETTER U WITH MACRON

0016C
LATIN CAPITAL LETTER U WITH BREVE

0016D
LATIN SMALL LETTER U WITH BREVE

0016E
LATIN CAPITAL LETTER U WITH RING ABOVE

0016F
LATIN SMALL LETTER U WITH RING ABOVE

00170
LATIN CAPITAL LETTER U WITH DOUBLE ACUTE

00171
LATIN SMALL LETTER U WITH DOUBLE ACUTE

00172
LATIN CAPITAL LETTER U WITH OGONEK

00173
LATIN SMALL LETTER U WITH OGONEK

00174
LATIN CAPITAL LETTER W WITH CIRCUMFLEX

00175
LATIN SMALL LETTER W WITH CIRCUMFLEX

00176
LATIN CAPITAL LETTER Y WITH CIRCUMFLEX

00177
LATIN SMALL LETTER Y WITH CIRCUMFLEX

00178
LATIN CAPITAL LETTER Y WITH DIAERESIS

00179
LATIN CAPITAL LETTER Z WITH ACUTE

0017A
LATIN SMALL LETTER Z WITH ACUTE

0017B
LATIN CAPITAL LETTER Z WITH DOT ABOVE

0017C
LATIN SMALL LETTER Z WITH DOT ABOVE

0017D
LATIN CAPITAL LETTER Z WITH CARON

0017E
LATIN SMALL LETTER Z WITH CARON

0017F
LATIN SMALL LETTER LONG S

00180
LATIN SMALL LETTER B WITH STROKE

00181
LATIN CAPITAL LETTER B WITH HOOK

00182
LATIN CAPITAL LETTER B WITH TOPBAR

00183
LATIN SMALL LETTER B WITH TOPBAR

00184
LATIN CAPITAL LETTER TONE SIX

00185
LATIN SMALL LETTER TONE SIX

00186
LATIN CAPITAL LETTER OPEN O

00187
LATIN CAPITAL LETTER C WITH HOOK

00188
LATIN SMALL LETTER C WITH HOOK

00189
LATIN CAPITAL LETTER AFRICAN D

0018A
LATIN CAPITAL LETTER D WITH HOOK

0018B
LATIN CAPITAL LETTER D WITH TOPBAR

0018C
LATIN SMALL LETTER D WITH TOPBAR

0018D
LATIN SMALL LETTER TURNED DELTA

0018E
LATIN CAPITAL LETTER REVERSED E

0018F
LATIN CAPITAL LETTER SCHWA

00190
LATIN CAPITAL LETTER OPEN E

00191
LATIN CAPITAL LETTER F WITH HOOK

00192
LATIN SMALL LETTER F WITH HOOK

00193
LATIN CAPITAL LETTER G WITH HOOK

00194
LATIN CAPITAL LETTER GAMMA

00195
LATIN SMALL LETTER HV

00196
LATIN CAPITAL LETTER IOTA

00197
LATIN CAPITAL LETTER I WITH STROKE

00198
LATIN CAPITAL LETTER K WITH HOOK

00199
LATIN SMALL LETTER K WITH HOOK

0019A
LATIN SMALL LETTER L WITH BAR

0019B
LATIN SMALL LETTER LAMBDA WITH STROKE

0019C
LATIN CAPITAL LETTER TURNED M

0019D
LATIN CAPITAL LETTER N WITH LEFT HOOK

0019E
LATIN SMALL LETTER N WITH LONG RIGHT LEG

0019F
LATIN CAPITAL LETTER O WITH MIDDLE TILDE

001A0
LATIN CAPITAL LETTER O WITH HORN

001A1
LATIN SMALL LETTER O WITH HORN

001A2
LATIN CAPITAL LETTER GHA

001A2
LATIN CAPITAL LETTER OI

001A3
LATIN SMALL LETTER GHA

001A3
LATIN SMALL LETTER OI

001A4
LATIN CAPITAL LETTER P WITH HOOK

001A5
LATIN SMALL LETTER P WITH HOOK

001A6
LATIN LETTER YR

001A7
LATIN CAPITAL LETTER TONE TWO

001A8
LATIN SMALL LETTER TONE TWO

001A9
LATIN CAPITAL LETTER ESH

001AA
LATIN LETTER REVERSED ESH LOOP

001AB
LATIN SMALL LETTER T WITH PALATAL HOOK

001AC
LATIN CAPITAL LETTER T WITH HOOK

001AD
LATIN SMALL LETTER T WITH HOOK

001AE
LATIN CAPITAL LETTER T WITH RETROFLEX HOOK

001AF
LATIN CAPITAL LETTER U WITH HORN

001B0
LATIN SMALL LETTER U WITH HORN

001B1
LATIN CAPITAL LETTER UPSILON

001B2
LATIN CAPITAL LETTER V WITH HOOK

001B3
LATIN CAPITAL LETTER Y WITH HOOK

001B4
LATIN SMALL LETTER Y WITH HOOK

001B5
LATIN CAPITAL LETTER Z WITH STROKE

001B6
LATIN SMALL LETTER Z WITH STROKE

001B7
LATIN CAPITAL LETTER EZH

001B8
LATIN CAPITAL LETTER EZH REVERSED

001B9
LATIN SMALL LETTER EZH REVERSED

001BA
LATIN SMALL LETTER EZH WITH TAIL

001BB
LATIN LETTER TWO WITH STROKE

001BC
LATIN CAPITAL LETTER TONE FIVE

001BD
LATIN SMALL LETTER TONE FIVE

001BE
LATIN LETTER INVERTED GLOTTAL STOP WITH STROKE

001BF
LATIN LETTER WYNN

001C0
LATIN LETTER DENTAL CLICK

001C1
LATIN LETTER LATERAL CLICK

001C2
LATIN LETTER ALVEOLAR CLICK

001C3
LATIN LETTER RETROFLEX CLICK

001C4
LATIN CAPITAL LETTER DZ WITH CARON

001C5
LATIN CAPITAL LETTER D WITH SMALL LETTER Z WITH CARON

001C6
LATIN SMALL LETTER DZ WITH CARON

001C7
LATIN CAPITAL LETTER LJ

001C8
LATIN CAPITAL LETTER L WITH SMALL LETTER J

001C9
LATIN SMALL LETTER LJ

001CA
LATIN CAPITAL LETTER NJ

001CB
LATIN CAPITAL LETTER N WITH SMALL LETTER J

001CC
LATIN SMALL LETTER NJ

001CD
LATIN CAPITAL LETTER A WITH CARON

001CE
LATIN SMALL LETTER A WITH CARON

001CF
LATIN CAPITAL LETTER I WITH CARON

001D0
LATIN SMALL LETTER I WITH CARON

001D1
LATIN CAPITAL LETTER O WITH CARON

001D2
LATIN SMALL LETTER O WITH CARON

001D3
LATIN CAPITAL LETTER U WITH CARON

001D4
LATIN SMALL LETTER U WITH CARON

001D5
LATIN CAPITAL LETTER U WITH DIAERESIS AND MACRON

001D6
LATIN SMALL LETTER U WITH DIAERESIS AND MACRON

001D7
LATIN CAPITAL LETTER U WITH DIAERESIS AND ACUTE

001D8
LATIN SMALL LETTER U WITH DIAERESIS AND ACUTE

001D9
LATIN CAPITAL LETTER U WITH DIAERESIS AND CARON

001DA
LATIN SMALL LETTER U WITH DIAERESIS AND CARON

001DB
LATIN CAPITAL LETTER U WITH DIAERESIS AND GRAVE

001DC
LATIN SMALL LETTER U WITH DIAERESIS AND GRAVE

001DD
LATIN SMALL LETTER TURNED E

001DE
LATIN CAPITAL LETTER A WITH DIAERESIS AND MACRON

001DF
LATIN SMALL LETTER A WITH DIAERESIS AND MACRON

001E0
LATIN CAPITAL LETTER A WITH DOT ABOVE AND MACRON

001E1
LATIN SMALL LETTER A WITH DOT ABOVE AND MACRON

001E2
LATIN CAPITAL LETTER AE WITH MACRON

001E3
LATIN SMALL LETTER AE WITH MACRON

001E4
LATIN CAPITAL LETTER G WITH STROKE

001E5
LATIN SMALL LETTER G WITH STROKE

001E6
LATIN CAPITAL LETTER G WITH CARON

001E7
LATIN SMALL LETTER G WITH CARON

001E8
LATIN CAPITAL LETTER K WITH CARON

001E9
LATIN SMALL LETTER K WITH CARON

001EA
LATIN CAPITAL LETTER O WITH OGONEK

001EB
LATIN SMALL LETTER O WITH OGONEK

001EC
LATIN CAPITAL LETTER O WITH OGONEK AND MACRON

001ED
LATIN SMALL LETTER O WITH OGONEK AND MACRON

001EE
LATIN CAPITAL LETTER EZH WITH CARON

001EF
LATIN SMALL LETTER EZH WITH CARON

001F0
LATIN SMALL LETTER J WITH CARON

001F1
LATIN CAPITAL LETTER DZ

001F2
LATIN CAPITAL LETTER D WITH SMALL LETTER Z

001F3
LATIN SMALL LETTER DZ

001F4
LATIN CAPITAL LETTER G WITH ACUTE

001F5
LATIN SMALL LETTER G WITH ACUTE

001F6
LATIN CAPITAL LETTER HWAIR

001F7
LATIN CAPITAL LETTER WYNN

001F8
LATIN CAPITAL LETTER N WITH GRAVE

001F9
LATIN SMALL LETTER N WITH GRAVE

001FA
LATIN CAPITAL LETTER A WITH RING ABOVE AND ACUTE

001FB
LATIN SMALL LETTER A WITH RING ABOVE AND ACUTE

001FC
LATIN CAPITAL LETTER AE WITH ACUTE

001FD
LATIN SMALL LETTER AE WITH ACUTE

001FE
LATIN CAPITAL LETTER O WITH STROKE AND ACUTE

001FF
LATIN SMALL LETTER O WITH STROKE AND ACUTE

00200
LATIN CAPITAL LETTER A WITH DOUBLE GRAVE

00201
LATIN SMALL LETTER A WITH DOUBLE GRAVE

00202
LATIN CAPITAL LETTER A WITH INVERTED BREVE

00203
LATIN SMALL LETTER A WITH INVERTED BREVE

00204
LATIN CAPITAL LETTER E WITH DOUBLE GRAVE

00205
LATIN SMALL LETTER E WITH DOUBLE GRAVE

00206
LATIN CAPITAL LETTER E WITH INVERTED BREVE

00207
LATIN SMALL LETTER E WITH INVERTED BREVE

00208
LATIN CAPITAL LETTER I WITH DOUBLE GRAVE

00209
LATIN SMALL LETTER I WITH DOUBLE GRAVE

0020A
LATIN CAPITAL LETTER I WITH INVERTED BREVE

0020B
LATIN SMALL LETTER I WITH INVERTED BREVE

0020C
LATIN CAPITAL LETTER O WITH DOUBLE GRAVE

0020D
LATIN SMALL LETTER O WITH DOUBLE GRAVE

0020E
LATIN CAPITAL LETTER O WITH INVERTED BREVE

0020F
LATIN SMALL LETTER O WITH INVERTED BREVE

00210
LATIN CAPITAL LETTER R WITH DOUBLE GRAVE

00211
LATIN SMALL LETTER R WITH DOUBLE GRAVE

00212
LATIN CAPITAL LETTER R WITH INVERTED BREVE

00213
LATIN SMALL LETTER R WITH INVERTED BREVE

00214
LATIN CAPITAL LETTER U WITH DOUBLE GRAVE

00215
LATIN SMALL LETTER U WITH DOUBLE GRAVE

00216
LATIN CAPITAL LETTER U WITH INVERTED BREVE

00217
LATIN SMALL LETTER U WITH INVERTED BREVE

00218
LATIN CAPITAL LETTER S WITH COMMA BELOW

00219
LATIN SMALL LETTER S WITH COMMA BELOW

0021A
LATIN CAPITAL LETTER T WITH COMMA BELOW

0021B
LATIN SMALL LETTER T WITH COMMA BELOW

0021C
LATIN CAPITAL LETTER YOGH

0021D
LATIN SMALL LETTER YOGH

0021E
LATIN CAPITAL LETTER H WITH CARON

0021F
LATIN SMALL LETTER H WITH CARON

00220
LATIN CAPITAL LETTER N WITH LONG RIGHT LEG

00221
LATIN SMALL LETTER D WITH CURL

00222
LATIN CAPITAL LETTER OU

00223
LATIN SMALL LETTER OU

00224
LATIN CAPITAL LETTER Z WITH HOOK

00225
LATIN SMALL LETTER Z WITH HOOK

00226
LATIN CAPITAL LETTER A WITH DOT ABOVE

00227
LATIN SMALL LETTER A WITH DOT ABOVE

00228
LATIN CAPITAL LETTER E WITH CEDILLA

00229
LATIN SMALL LETTER E WITH CEDILLA

0022A
LATIN CAPITAL LETTER O WITH DIAERESIS AND MACRON

0022B
LATIN SMALL LETTER O WITH DIAERESIS AND MACRON

0022C
LATIN CAPITAL LETTER O WITH TILDE AND MACRON

0022D
LATIN SMALL LETTER O WITH TILDE AND MACRON

0022E
LATIN CAPITAL LETTER O WITH DOT ABOVE

0022F
LATIN SMALL LETTER O WITH DOT ABOVE

00230
LATIN CAPITAL LETTER O WITH DOT ABOVE AND MACRON

00231
LATIN SMALL LETTER O WITH DOT ABOVE AND MACRON

00232
LATIN CAPITAL LETTER Y WITH MACRON

00233
LATIN SMALL LETTER Y WITH MACRON

00234
LATIN SMALL LETTER L WITH CURL

00235
LATIN SMALL LETTER N WITH CURL

00236
LATIN SMALL LETTER T WITH CURL

00237
LATIN SMALL LETTER DOTLESS J

00238
LATIN SMALL LETTER DB DIGRAPH

00239
LATIN SMALL LETTER QP DIGRAPH

0023A
LATIN CAPITAL LETTER A WITH STROKE

0023B
LATIN CAPITAL LETTER C WITH STROKE

0023C
LATIN SMALL LETTER C WITH STROKE

0023D
LATIN CAPITAL LETTER L WITH BAR

0023E
LATIN CAPITAL LETTER T WITH DIAGONAL STROKE

0023F
LATIN SMALL LETTER S WITH SWASH TAIL

00240
LATIN SMALL LETTER Z WITH SWASH TAIL

00241
LATIN CAPITAL LETTER GLOTTAL STOP

00242
LATIN SMALL LETTER GLOTTAL STOP

00243
LATIN CAPITAL LETTER B WITH STROKE

00244
LATIN CAPITAL LETTER U BAR

00245
LATIN CAPITAL LETTER TURNED V

00246
LATIN CAPITAL LETTER E WITH STROKE

00247
LATIN SMALL LETTER E WITH STROKE

00248
LATIN CAPITAL LETTER J WITH STROKE

00249
LATIN SMALL LETTER J WITH STROKE

0024A
LATIN CAPITAL LETTER SMALL Q WITH HOOK TAIL

0024B
LATIN SMALL LETTER Q WITH HOOK TAIL

0024C
LATIN CAPITAL LETTER R WITH STROKE

0024D
LATIN SMALL LETTER R WITH STROKE

0024E
LATIN CAPITAL LETTER Y WITH STROKE

0024F
LATIN SMALL LETTER Y WITH STROKE

00250
LATIN SMALL LETTER TURNED A

00251
LATIN SMALL LETTER ALPHA

00252
LATIN SMALL LETTER TURNED ALPHA

00253
LATIN SMALL LETTER B WITH HOOK

00254
LATIN SMALL LETTER OPEN O

00255
LATIN SMALL LETTER C WITH CURL

00256
LATIN SMALL LETTER D WITH TAIL

00257
LATIN SMALL LETTER D WITH HOOK

00258
LATIN SMALL LETTER REVERSED E

00259
LATIN SMALL LETTER SCHWA

0025A
LATIN SMALL LETTER SCHWA WITH HOOK

0025B
LATIN SMALL LETTER OPEN E

0025C
LATIN SMALL LETTER REVERSED OPEN E

0025D
LATIN SMALL LETTER REVERSED OPEN E WITH HOOK

0025E
LATIN SMALL LETTER CLOSED REVERSED OPEN E

0025F
LATIN SMALL LETTER DOTLESS J WITH STROKE

00260
LATIN SMALL LETTER G WITH HOOK

00261
LATIN SMALL LETTER SCRIPT G

00262
LATIN LETTER SMALL CAPITAL G

00263
LATIN SMALL LETTER GAMMA

00264
LATIN SMALL LETTER RAMS HORN

00265
LATIN SMALL LETTER TURNED H

00266
LATIN SMALL LETTER H WITH HOOK

00267
LATIN SMALL LETTER HENG WITH HOOK

00268
LATIN SMALL LETTER I WITH STROKE

00269
LATIN SMALL LETTER IOTA

0026A
LATIN LETTER SMALL CAPITAL I

0026B
LATIN SMALL LETTER L WITH MIDDLE TILDE

0026C
LATIN SMALL LETTER L WITH BELT

0026D
LATIN SMALL LETTER L WITH RETROFLEX HOOK

0026E
LATIN SMALL LETTER LEZH

0026F
LATIN SMALL LETTER TURNED M

00270
LATIN SMALL LETTER TURNED M WITH LONG LEG

00271
LATIN SMALL LETTER M WITH HOOK

00272
LATIN SMALL LETTER N WITH LEFT HOOK

00273
LATIN SMALL LETTER N WITH RETROFLEX HOOK

00274
LATIN LETTER SMALL CAPITAL N

00275
LATIN SMALL LETTER BARRED O

00276
LATIN LETTER SMALL CAPITAL OE

00277
LATIN SMALL LETTER CLOSED OMEGA

00278
LATIN SMALL LETTER PHI

00279
LATIN SMALL LETTER TURNED R

0027A
LATIN SMALL LETTER TURNED R WITH LONG LEG

0027B
LATIN SMALL LETTER TURNED R WITH HOOK

0027C
LATIN SMALL LETTER R WITH LONG LEG

0027D
LATIN SMALL LETTER R WITH TAIL

0027E
LATIN SMALL LETTER R WITH FISHHOOK

0027F
LATIN SMALL LETTER REVERSED R WITH FISHHOOK

00280
LATIN LETTER SMALL CAPITAL R

00281
LATIN LETTER SMALL CAPITAL INVERTED R

00282
LATIN SMALL LETTER S WITH HOOK

00283
LATIN SMALL LETTER ESH

00284
LATIN SMALL LETTER DOTLESS J WITH STROKE AND HOOK

00285
LATIN SMALL LETTER SQUAT REVERSED ESH

00286
LATIN SMALL LETTER ESH WITH CURL

00287
LATIN SMALL LETTER TURNED T

00288
LATIN SMALL LETTER T WITH RETROFLEX HOOK

00289
LATIN SMALL LETTER U BAR

0028A
LATIN SMALL LETTER UPSILON

0028B
LATIN SMALL LETTER V WITH HOOK

0028C
LATIN SMALL LETTER TURNED V

0028D
LATIN SMALL LETTER TURNED W

0028E
LATIN SMALL LETTER TURNED Y

0028F
LATIN LETTER SMALL CAPITAL Y

00290
LATIN SMALL LETTER Z WITH RETROFLEX HOOK

00291
LATIN SMALL LETTER Z WITH CURL

00292
LATIN SMALL LETTER EZH

00293
LATIN SMALL LETTER EZH WITH CURL

00294
LATIN LETTER GLOTTAL STOP

00295
LATIN LETTER PHARYNGEAL VOICED FRICATIVE

00296
LATIN LETTER INVERTED GLOTTAL STOP

00297
LATIN LETTER STRETCHED C

00298
LATIN LETTER BILABIAL CLICK

00299
LATIN LETTER SMALL CAPITAL B

0029A
LATIN SMALL LETTER CLOSED OPEN E

0029B
LATIN LETTER SMALL CAPITAL G WITH HOOK

0029C
LATIN LETTER SMALL CAPITAL H

0029D
LATIN SMALL LETTER J WITH CROSSED-TAIL

0029E
LATIN SMALL LETTER TURNED K

0029F
LATIN LETTER SMALL CAPITAL L

002A0
LATIN SMALL LETTER Q WITH HOOK

002A1
LATIN LETTER GLOTTAL STOP WITH STROKE

002A2
LATIN LETTER REVERSED GLOTTAL STOP WITH STROKE

002A3
LATIN SMALL LETTER DZ DIGRAPH

002A4
LATIN SMALL LETTER DEZH DIGRAPH

002A5
LATIN SMALL LETTER DZ DIGRAPH WITH CURL

002A6
LATIN SMALL LETTER TS DIGRAPH

002A7
LATIN SMALL LETTER TESH DIGRAPH

002A8
LATIN SMALL LETTER TC DIGRAPH WITH CURL

002A9
LATIN SMALL LETTER FENG DIGRAPH

002AA
LATIN SMALL LETTER LS DIGRAPH

002AB
LATIN SMALL LETTER LZ DIGRAPH

002AC
LATIN LETTER BILABIAL PERCUSSIVE

002AD
LATIN LETTER BIDENTAL PERCUSSIVE

002AE
LATIN SMALL LETTER TURNED H WITH FISHHOOK

002AF
LATIN SMALL LETTER TURNED H WITH FISHHOOK AND TAIL

002B0
MODIFIER LETTER SMALL H

002B1
MODIFIER LETTER SMALL H WITH HOOK

002B2
MODIFIER LETTER SMALL J

002B3
MODIFIER LETTER SMALL R

002B4
MODIFIER LETTER SMALL TURNED R

002B5
MODIFIER LETTER SMALL TURNED R WITH HOOK

002B6
MODIFIER LETTER SMALL CAPITAL INVERTED R

002B7
MODIFIER LETTER SMALL W

002B8
MODIFIER LETTER SMALL Y

002B9
MODIFIER LETTER PRIME

002BA
MODIFIER LETTER DOUBLE PRIME

002BB
MODIFIER LETTER TURNED COMMA

002BC
MODIFIER LETTER APOSTROPHE

002BD
MODIFIER LETTER REVERSED COMMA

002BE
MODIFIER LETTER RIGHT HALF RING

002BF
MODIFIER LETTER LEFT HALF RING

002C0
MODIFIER LETTER GLOTTAL STOP

002C1
MODIFIER LETTER REVERSED GLOTTAL STOP

002C2
MODIFIER LETTER LEFT ARROWHEAD

002C3
MODIFIER LETTER RIGHT ARROWHEAD

002C4
MODIFIER LETTER UP ARROWHEAD

002C5
MODIFIER LETTER DOWN ARROWHEAD

002C6
MODIFIER LETTER CIRCUMFLEX ACCENT

002C7
CARON

002C8
MODIFIER LETTER VERTICAL LINE

002C9
MODIFIER LETTER MACRON

002CA
MODIFIER LETTER ACUTE ACCENT

002CB
MODIFIER LETTER GRAVE ACCENT

002CC
MODIFIER LETTER LOW VERTICAL LINE

002CD
MODIFIER LETTER LOW MACRON

002CE
MODIFIER LETTER LOW GRAVE ACCENT

002CF
MODIFIER LETTER LOW ACUTE ACCENT

002D0
MODIFIER LETTER TRIANGULAR COLON

002D1
MODIFIER LETTER HALF TRIANGULAR COLON

002D2
MODIFIER LETTER CENTRED RIGHT HALF RING

002D3
MODIFIER LETTER CENTRED LEFT HALF RING

002D4
MODIFIER LETTER UP TACK

002D5
MODIFIER LETTER DOWN TACK

002D6
MODIFIER LETTER PLUS SIGN

002D7
MODIFIER LETTER MINUS SIGN

002D8
BREVE

002D9
DOT ABOVE

002DA
RING ABOVE

002DB
OGONEK

002DC
SMALL TILDE

002DD
DOUBLE ACUTE ACCENT

002DE
MODIFIER LETTER RHOTIC HOOK

002DF
MODIFIER LETTER CROSS ACCENT

002E0
MODIFIER LETTER SMALL GAMMA

002E1
MODIFIER LETTER SMALL L

002E2
MODIFIER LETTER SMALL S

002E3
MODIFIER LETTER SMALL X

002E4
MODIFIER LETTER SMALL REVERSED GLOTTAL STOP

002E5
MODIFIER LETTER EXTRA-HIGH TONE BAR

002E6
MODIFIER LETTER HIGH TONE BAR

002E7
MODIFIER LETTER MID TONE BAR

002E8
MODIFIER LETTER LOW TONE BAR

002E9
MODIFIER LETTER EXTRA-LOW TONE BAR

002EA
MODIFIER LETTER YIN DEPARTING TONE MARK

002EB
MODIFIER LETTER YANG DEPARTING TONE MARK

002EC
MODIFIER LETTER VOICING

002ED
MODIFIER LETTER UNASPIRATED

002EE
MODIFIER LETTER DOUBLE APOSTROPHE

002EF
MODIFIER LETTER LOW DOWN ARROWHEAD

002F0
MODIFIER LETTER LOW UP ARROWHEAD

002F1
MODIFIER LETTER LOW LEFT ARROWHEAD

002F2
MODIFIER LETTER LOW RIGHT ARROWHEAD

002F3
MODIFIER LETTER LOW RING

002F4
MODIFIER LETTER MIDDLE GRAVE ACCENT

002F5
MODIFIER LETTER MIDDLE DOUBLE GRAVE ACCENT

002F6
MODIFIER LETTER MIDDLE DOUBLE ACUTE ACCENT

002F7
MODIFIER LETTER LOW TILDE

002F8
MODIFIER LETTER RAISED COLON

002F9
MODIFIER LETTER BEGIN HIGH TONE

002FA
MODIFIER LETTER END HIGH TONE

002FB
MODIFIER LETTER BEGIN LOW TONE

002FC
MODIFIER LETTER END LOW TONE

002FD
MODIFIER LETTER SHELF

002FE
MODIFIER LETTER OPEN SHELF

002FF
MODIFIER LETTER LOW LEFT ARROW

00300
COMBINING GRAVE ACCENT

00301
COMBINING ACUTE ACCENT

00302
COMBINING CIRCUMFLEX ACCENT

00303
COMBINING TILDE

00304
COMBINING MACRON

00305
COMBINING OVERLINE

00306
COMBINING BREVE

00307
COMBINING DOT ABOVE

00308
COMBINING DIAERESIS

00309
COMBINING HOOK ABOVE

0030A
COMBINING RING ABOVE

0030B
COMBINING DOUBLE ACUTE ACCENT

0030C
COMBINING CARON

0030D
COMBINING VERTICAL LINE ABOVE

0030E
COMBINING DOUBLE VERTICAL LINE ABOVE

0030F
COMBINING DOUBLE GRAVE ACCENT

00310
COMBINING CANDRABINDU

00311
COMBINING INVERTED BREVE

00312
COMBINING TURNED COMMA ABOVE

00313
COMBINING COMMA ABOVE

00314
COMBINING REVERSED COMMA ABOVE

00315
COMBINING COMMA ABOVE RIGHT

00316
COMBINING GRAVE ACCENT BELOW

00317
COMBINING ACUTE ACCENT BELOW

00318
COMBINING LEFT TACK BELOW

00319
COMBINING RIGHT TACK BELOW

0031A
COMBINING LEFT ANGLE ABOVE

0031B
COMBINING HORN

0031C
COMBINING LEFT HALF RING BELOW

0031D
COMBINING UP TACK BELOW

0031E
COMBINING DOWN TACK BELOW

0031F
COMBINING PLUS SIGN BELOW

00320
COMBINING MINUS SIGN BELOW

00321
COMBINING PALATALIZED HOOK BELOW

00322
COMBINING RETROFLEX HOOK BELOW

00323
COMBINING DOT BELOW

00324
COMBINING DIAERESIS BELOW

00325
COMBINING RING BELOW

00326
COMBINING COMMA BELOW

00327
COMBINING CEDILLA

00328
COMBINING OGONEK

00329
COMBINING VERTICAL LINE BELOW

0032A
COMBINING BRIDGE BELOW

0032B
COMBINING INVERTED DOUBLE ARCH BELOW

0032C
COMBINING CARON BELOW

0032D
COMBINING CIRCUMFLEX ACCENT BELOW

0032E
COMBINING BREVE BELOW

0032F
COMBINING INVERTED BREVE BELOW

00330
COMBINING TILDE BELOW

00331
COMBINING MACRON BELOW

00332
COMBINING LOW LINE

00333
COMBINING DOUBLE LOW LINE

00334
COMBINING TILDE OVERLAY

00335
COMBINING SHORT STROKE OVERLAY

00336
COMBINING LONG STROKE OVERLAY

00337
COMBINING SHORT SOLIDUS OVERLAY

00338
COMBINING LONG SOLIDUS OVERLAY

00339
COMBINING RIGHT HALF RING BELOW

0033A
COMBINING INVERTED BRIDGE BELOW

0033B
COMBINING SQUARE BELOW

0033C
COMBINING SEAGULL BELOW

0033D
COMBINING X ABOVE

0033E
COMBINING VERTICAL TILDE

0033F
COMBINING DOUBLE OVERLINE

00340
COMBINING GRAVE TONE MARK

00341
COMBINING ACUTE TONE MARK

00342
COMBINING GREEK PERISPOMENI

00343
COMBINING GREEK KORONIS

00344
COMBINING GREEK DIALYTIKA TONOS

00345
COMBINING GREEK YPOGEGRAMMENI

00346
COMBINING BRIDGE ABOVE

00347
COMBINING EQUALS SIGN BELOW

00348
COMBINING DOUBLE VERTICAL LINE BELOW

00349
COMBINING LEFT ANGLE BELOW

0034A
COMBINING NOT TILDE ABOVE

0034B
COMBINING HOMOTHETIC ABOVE

0034C
COMBINING ALMOST EQUAL TO ABOVE

0034D
COMBINING LEFT RIGHT ARROW BELOW

0034E
COMBINING UPWARDS ARROW BELOW

0034F
COMBINING GRAPHEME JOINER

0034F
CGJ

00350
COMBINING RIGHT ARROWHEAD ABOVE

00351
COMBINING LEFT HALF RING ABOVE

00352
COMBINING FERMATA

00353
COMBINING X BELOW

00354
COMBINING LEFT ARROWHEAD BELOW

00355
COMBINING RIGHT ARROWHEAD BELOW

00356
COMBINING RIGHT ARROWHEAD AND UP ARROWHEAD BELOW

00357
COMBINING RIGHT HALF RING ABOVE

00358
COMBINING DOT ABOVE RIGHT

00359
COMBINING ASTERISK BELOW

0035A
COMBINING DOUBLE RING BELOW

0035B
COMBINING ZIGZAG ABOVE

0035C
COMBINING DOUBLE BREVE BELOW

0035D
COMBINING DOUBLE BREVE

0035E
COMBINING DOUBLE MACRON

0035F
COMBINING DOUBLE MACRON BELOW

00360
COMBINING DOUBLE TILDE

00361
COMBINING DOUBLE INVERTED BREVE

00362
COMBINING DOUBLE RIGHTWARDS ARROW BELOW

00363
COMBINING LATIN SMALL LETTER A

00364
COMBINING LATIN SMALL LETTER E

00365
COMBINING LATIN SMALL LETTER I

00366
COMBINING LATIN SMALL LETTER O

00367
COMBINING LATIN SMALL LETTER U

00368
COMBINING LATIN SMALL LETTER C

00369
COMBINING LATIN SMALL LETTER D

0036A
COMBINING LATIN SMALL LETTER H

0036B
COMBINING LATIN SMALL LETTER M

0036C
COMBINING LATIN SMALL LETTER R

0036D
COMBINING LATIN SMALL LETTER T

0036E
COMBINING LATIN SMALL LETTER V

0036F
COMBINING LATIN SMALL LETTER X

00370
GREEK CAPITAL LETTER HETA

00371
GREEK SMALL LETTER HETA

00372
GREEK CAPITAL LETTER ARCHAIC SAMPI

00373
GREEK SMALL LETTER ARCHAIC SAMPI

00374
GREEK NUMERAL SIGN

00375
GREEK LOWER NUMERAL SIGN

00376
GREEK CAPITAL LETTER PAMPHYLIAN DIGAMMA

00377
GREEK SMALL LETTER PAMPHYLIAN DIGAMMA

0037A
GREEK YPOGEGRAMMENI

0037B
GREEK SMALL REVERSED LUNATE SIGMA SYMBOL

0037C
GREEK SMALL DOTTED LUNATE SIGMA SYMBOL

0037D
GREEK SMALL REVERSED DOTTED LUNATE SIGMA SYMBOL

0037E
GREEK QUESTION MARK

0037F
GREEK CAPITAL LETTER YOT

00384
GREEK TONOS

00385
GREEK DIALYTIKA TONOS

00386
GREEK CAPITAL LETTER ALPHA WITH TONOS

00387
GREEK ANO TELEIA

00388
GREEK CAPITAL LETTER EPSILON WITH TONOS

00389
GREEK CAPITAL LETTER ETA WITH TONOS

0038A
GREEK CAPITAL LETTER IOTA WITH TONOS

0038C
GREEK CAPITAL LETTER OMICRON WITH TONOS

0038E
GREEK CAPITAL LETTER UPSILON WITH TONOS

0038F
GREEK CAPITAL LETTER OMEGA WITH TONOS

00390
GREEK SMALL LETTER IOTA WITH DIALYTIKA AND TONOS

00391
GREEK CAPITAL LETTER ALPHA

00392
GREEK CAPITAL LETTER BETA

00393
GREEK CAPITAL LETTER GAMMA

00394
GREEK CAPITAL LETTER DELTA

00395
GREEK CAPITAL LETTER EPSILON

00396
GREEK CAPITAL LETTER ZETA

00397
GREEK CAPITAL LETTER ETA

00398
GREEK CAPITAL LETTER THETA

00399
GREEK CAPITAL LETTER IOTA

0039A
GREEK CAPITAL LETTER KAPPA

0039B
GREEK CAPITAL LETTER LAMDA

0039C
GREEK CAPITAL LETTER MU

0039D
GREEK CAPITAL LETTER NU

0039E
GREEK CAPITAL LETTER XI

0039F
GREEK CAPITAL LETTER OMICRON

003A0
GREEK CAPITAL LETTER PI

003A1
GREEK CAPITAL LETTER RHO

003A3
GREEK CAPITAL LETTER SIGMA

003A4
GREEK CAPITAL LETTER TAU

003A5
GREEK CAPITAL LETTER UPSILON

003A6
GREEK CAPITAL LETTER PHI

003A7
GREEK CAPITAL LETTER CHI

003A8
GREEK CAPITAL LETTER PSI

003A9
GREEK CAPITAL LETTER OMEGA

003AA
GREEK CAPITAL LETTER IOTA WITH DIALYTIKA

003AB
GREEK CAPITAL LETTER UPSILON WITH DIALYTIKA

003AC
GREEK SMALL LETTER ALPHA WITH TONOS

003AD
GREEK SMALL LETTER EPSILON WITH TONOS

003AE
GREEK SMALL LETTER ETA WITH TONOS

003AF
GREEK SMALL LETTER IOTA WITH TONOS

003B0
GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND TONOS

003B1
GREEK SMALL LETTER ALPHA

003B2
GREEK SMALL LETTER BETA

003B3
GREEK SMALL LETTER GAMMA

003B4
GREEK SMALL LETTER DELTA

003B5
GREEK SMALL LETTER EPSILON

003B6
GREEK SMALL LETTER ZETA

003B7
GREEK SMALL LETTER ETA

003B8
GREEK SMALL LETTER THETA

003B9
GREEK SMALL LETTER IOTA

003BA
GREEK SMALL LETTER KAPPA

003BB
GREEK SMALL LETTER LAMDA

003BC
GREEK SMALL LETTER MU

003BD
GREEK SMALL LETTER NU

003BE
GREEK SMALL LETTER XI

003BF
GREEK SMALL LETTER OMICRON

003C0
GREEK SMALL LETTER PI

003C1
GREEK SMALL LETTER RHO

003C2
GREEK SMALL LETTER FINAL SIGMA

003C3
GREEK SMALL LETTER SIGMA

003C4
GREEK SMALL LETTER TAU

003C5
GREEK SMALL LETTER UPSILON

003C6
GREEK SMALL LETTER PHI

003C7
GREEK SMALL LETTER CHI

003C8
GREEK SMALL LETTER PSI

003C9
GREEK SMALL LETTER OMEGA

003CA
GREEK SMALL LETTER IOTA WITH DIALYTIKA

003CB
GREEK SMALL LETTER UPSILON WITH DIALYTIKA

003CC
GREEK SMALL LETTER OMICRON WITH TONOS

003CD
GREEK SMALL LETTER UPSILON WITH TONOS

003CE
GREEK SMALL LETTER OMEGA WITH TONOS

003CF
GREEK CAPITAL KAI SYMBOL

003D0
GREEK BETA SYMBOL

003D1
GREEK THETA SYMBOL

003D2
GREEK UPSILON WITH HOOK SYMBOL

003D3
GREEK UPSILON WITH ACUTE AND HOOK SYMBOL

003D4
GREEK UPSILON WITH DIAERESIS AND HOOK SYMBOL

003D5
GREEK PHI SYMBOL

003D6
GREEK PI SYMBOL

003D7
GREEK KAI SYMBOL

003D8
GREEK LETTER ARCHAIC KOPPA

003D9
GREEK SMALL LETTER ARCHAIC KOPPA

003DA
GREEK LETTER STIGMA

003DB
GREEK SMALL LETTER STIGMA

003DC
GREEK LETTER DIGAMMA

003DD
GREEK SMALL LETTER DIGAMMA

003DE
GREEK LETTER KOPPA

003DF
GREEK SMALL LETTER KOPPA

003E0
GREEK LETTER SAMPI

003E1
GREEK SMALL LETTER SAMPI

003E2
COPTIC CAPITAL LETTER SHEI

003E3
COPTIC SMALL LETTER SHEI

003E4
COPTIC CAPITAL LETTER FEI

003E5
COPTIC SMALL LETTER FEI

003E6
COPTIC CAPITAL LETTER KHEI

003E7
COPTIC SMALL LETTER KHEI

003E8
COPTIC CAPITAL LETTER HORI

003E9
COPTIC SMALL LETTER HORI

003EA
COPTIC CAPITAL LETTER GANGIA

003EB
COPTIC SMALL LETTER GANGIA

003EC
COPTIC CAPITAL LETTER SHIMA

003ED
COPTIC SMALL LETTER SHIMA

003EE
COPTIC CAPITAL LETTER DEI

003EF
COPTIC SMALL LETTER DEI

003F0
GREEK KAPPA SYMBOL

003F1
GREEK RHO SYMBOL

003F2
GREEK LUNATE SIGMA SYMBOL

003F3
GREEK LETTER YOT

003F4
GREEK CAPITAL THETA SYMBOL

003F5
GREEK LUNATE EPSILON SYMBOL

003F6
GREEK REVERSED LUNATE EPSILON SYMBOL

003F7
GREEK CAPITAL LETTER SHO

003F8
GREEK SMALL LETTER SHO

003F9
GREEK CAPITAL LUNATE SIGMA SYMBOL

003FA
GREEK CAPITAL LETTER SAN

003FB
GREEK SMALL LETTER SAN

003FC
GREEK RHO WITH STROKE SYMBOL

003FD
GREEK CAPITAL REVERSED LUNATE SIGMA SYMBOL

003FE
GREEK CAPITAL DOTTED LUNATE SIGMA SYMBOL

003FF
GREEK CAPITAL REVERSED DOTTED LUNATE SIGMA SYMBOL

00400
CYRILLIC CAPITAL LETTER IE WITH GRAVE

00401
CYRILLIC CAPITAL LETTER IO

00402
CYRILLIC CAPITAL LETTER DJE

00403
CYRILLIC CAPITAL LETTER GJE

00404
CYRILLIC CAPITAL LETTER UKRAINIAN IE

00405
CYRILLIC CAPITAL LETTER DZE

00406
CYRILLIC CAPITAL LETTER BYELORUSSIAN-UKRAINIAN I

00407
CYRILLIC CAPITAL LETTER YI

00408
CYRILLIC CAPITAL LETTER JE

00409
CYRILLIC CAPITAL LETTER LJE

0040A
CYRILLIC CAPITAL LETTER NJE

0040B
CYRILLIC CAPITAL LETTER TSHE

0040C
CYRILLIC CAPITAL LETTER KJE

0040D
CYRILLIC CAPITAL LETTER I WITH GRAVE

0040E
CYRILLIC CAPITAL LETTER SHORT U

0040F
CYRILLIC CAPITAL LETTER DZHE

00410
CYRILLIC CAPITAL LETTER A

00411
CYRILLIC CAPITAL LETTER BE

00412
CYRILLIC CAPITAL LETTER VE

00413
CYRILLIC CAPITAL LETTER GHE

00414
CYRILLIC CAPITAL LETTER DE

00415
CYRILLIC CAPITAL LETTER IE

00416
CYRILLIC CAPITAL LETTER ZHE

00417
CYRILLIC CAPITAL LETTER ZE

00418
CYRILLIC CAPITAL LETTER I

00419
CYRILLIC CAPITAL LETTER SHORT I

0041A
CYRILLIC CAPITAL LETTER KA

0041B
CYRILLIC CAPITAL LETTER EL

0041C
CYRILLIC CAPITAL LETTER EM

0041D
CYRILLIC CAPITAL LETTER EN

0041E
CYRILLIC CAPITAL LETTER O

0041F
CYRILLIC CAPITAL LETTER PE

00420
CYRILLIC CAPITAL LETTER ER

00421
CYRILLIC CAPITAL LETTER ES

00422
CYRILLIC CAPITAL LETTER TE

00423
CYRILLIC CAPITAL LETTER U

00424
CYRILLIC CAPITAL LETTER EF

00425
CYRILLIC CAPITAL LETTER HA

00426
CYRILLIC CAPITAL LETTER TSE

00427
CYRILLIC CAPITAL LETTER CHE

00428
CYRILLIC CAPITAL LETTER SHA

00429
CYRILLIC CAPITAL LETTER SHCHA

0042A
CYRILLIC CAPITAL LETTER HARD SIGN

0042B
CYRILLIC CAPITAL LETTER YERU

0042C
CYRILLIC CAPITAL LETTER SOFT SIGN

0042D
CYRILLIC CAPITAL LETTER E

0042E
CYRILLIC CAPITAL LETTER YU

0042F
CYRILLIC CAPITAL LETTER YA

00430
CYRILLIC SMALL LETTER A

00431
CYRILLIC SMALL LETTER BE

00432
CYRILLIC SMALL LETTER VE

00433
CYRILLIC SMALL LETTER GHE

00434
CYRILLIC SMALL LETTER DE

00435
CYRILLIC SMALL LETTER IE

00436
CYRILLIC SMALL LETTER ZHE

00437
CYRILLIC SMALL LETTER ZE

00438
CYRILLIC SMALL LETTER I

00439
CYRILLIC SMALL LETTER SHORT I

0043A
CYRILLIC SMALL LETTER KA

0043B
CYRILLIC SMALL LETTER EL

0043C
CYRILLIC SMALL LETTER EM

0043D
CYRILLIC SMALL LETTER EN

0043E
CYRILLIC SMALL LETTER O

0043F
CYRILLIC SMALL LETTER PE

00440
CYRILLIC SMALL LETTER ER

00441
CYRILLIC SMALL LETTER ES

00442
CYRILLIC SMALL LETTER TE

00443
CYRILLIC SMALL LETTER U

00444
CYRILLIC SMALL LETTER EF

00445
CYRILLIC SMALL LETTER HA

00446
CYRILLIC SMALL LETTER TSE

00447
CYRILLIC SMALL LETTER CHE

00448
CYRILLIC SMALL LETTER SHA

00449
CYRILLIC SMALL LETTER SHCHA

0044A
CYRILLIC SMALL LETTER HARD SIGN

0044B
CYRILLIC SMALL LETTER YERU

0044C
CYRILLIC SMALL LETTER SOFT SIGN

0044D
CYRILLIC SMALL LETTER E

0044E
CYRILLIC SMALL LETTER YU

0044F
CYRILLIC SMALL LETTER YA

00450
CYRILLIC SMALL LETTER IE WITH GRAVE

00451
CYRILLIC SMALL LETTER IO

00452
CYRILLIC SMALL LETTER DJE

00453
CYRILLIC SMALL LETTER GJE

00454
CYRILLIC SMALL LETTER UKRAINIAN IE

00455
CYRILLIC SMALL LETTER DZE

00456
CYRILLIC SMALL LETTER BYELORUSSIAN-UKRAINIAN I

00457
CYRILLIC SMALL LETTER YI

00458
CYRILLIC SMALL LETTER JE

00459
CYRILLIC SMALL LETTER LJE

0045A
CYRILLIC SMALL LETTER NJE

0045B
CYRILLIC SMALL LETTER TSHE

0045C
CYRILLIC SMALL LETTER KJE

0045D
CYRILLIC SMALL LETTER I WITH GRAVE

0045E
CYRILLIC SMALL LETTER SHORT U

0045F
CYRILLIC SMALL LETTER DZHE

00460
CYRILLIC CAPITAL LETTER OMEGA

00461
CYRILLIC SMALL LETTER OMEGA

00462
CYRILLIC CAPITAL LETTER YAT

00463
CYRILLIC SMALL LETTER YAT

00464
CYRILLIC CAPITAL LETTER IOTIFIED E

00465
CYRILLIC SMALL LETTER IOTIFIED E

00466
CYRILLIC CAPITAL LETTER LITTLE YUS

00467
CYRILLIC SMALL LETTER LITTLE YUS

00468
CYRILLIC CAPITAL LETTER IOTIFIED LITTLE YUS

00469
CYRILLIC SMALL LETTER IOTIFIED LITTLE YUS

0046A
CYRILLIC CAPITAL LETTER BIG YUS

0046B
CYRILLIC SMALL LETTER BIG YUS

0046C
CYRILLIC CAPITAL LETTER IOTIFIED BIG YUS

0046D
CYRILLIC SMALL LETTER IOTIFIED BIG YUS

0046E
CYRILLIC CAPITAL LETTER KSI

0046F
CYRILLIC SMALL LETTER KSI

00470
CYRILLIC CAPITAL LETTER PSI

00471
CYRILLIC SMALL LETTER PSI

00472
CYRILLIC CAPITAL LETTER FITA

00473
CYRILLIC SMALL LETTER FITA

00474
CYRILLIC CAPITAL LETTER IZHITSA

00475
CYRILLIC SMALL LETTER IZHITSA

00476
CYRILLIC CAPITAL LETTER IZHITSA WITH DOUBLE GRAVE ACCENT

00477
CYRILLIC SMALL LETTER IZHITSA WITH DOUBLE GRAVE ACCENT

00478
CYRILLIC CAPITAL LETTER UK

00479
CYRILLIC SMALL LETTER UK

0047A
CYRILLIC CAPITAL LETTER ROUND OMEGA

0047B
CYRILLIC SMALL LETTER ROUND OMEGA

0047C
CYRILLIC CAPITAL LETTER OMEGA WITH TITLO

0047D
CYRILLIC SMALL LETTER OMEGA WITH TITLO

0047E
CYRILLIC CAPITAL LETTER OT

0047F
CYRILLIC SMALL LETTER OT

00480
CYRILLIC CAPITAL LETTER KOPPA

00481
CYRILLIC SMALL LETTER KOPPA

00482
CYRILLIC THOUSANDS SIGN

00483
COMBINING CYRILLIC TITLO

00484
COMBINING CYRILLIC PALATALIZATION

00485
COMBINING CYRILLIC DASIA PNEUMATA

00486
COMBINING CYRILLIC PSILI PNEUMATA

00487
COMBINING CYRILLIC POKRYTIE

00488
COMBINING CYRILLIC HUNDRED THOUSANDS SIGN

00489
COMBINING CYRILLIC MILLIONS SIGN

0048A
CYRILLIC CAPITAL LETTER SHORT I WITH TAIL

0048B
CYRILLIC SMALL LETTER SHORT I WITH TAIL

0048C
CYRILLIC CAPITAL LETTER SEMISOFT SIGN

0048D
CYRILLIC SMALL LETTER SEMISOFT SIGN

0048E
CYRILLIC CAPITAL LETTER ER WITH TICK

0048F
CYRILLIC SMALL LETTER ER WITH TICK

00490
CYRILLIC CAPITAL LETTER GHE WITH UPTURN

00491
CYRILLIC SMALL LETTER GHE WITH UPTURN

00492
CYRILLIC CAPITAL LETTER GHE WITH STROKE

00493
CYRILLIC SMALL LETTER GHE WITH STROKE

00494
CYRILLIC CAPITAL LETTER GHE WITH MIDDLE HOOK

00495
CYRILLIC SMALL LETTER GHE WITH MIDDLE HOOK

00496
CYRILLIC CAPITAL LETTER ZHE WITH DESCENDER

00497
CYRILLIC SMALL LETTER ZHE WITH DESCENDER

00498
CYRILLIC CAPITAL LETTER ZE WITH DESCENDER

00499
CYRILLIC SMALL LETTER ZE WITH DESCENDER

0049A
CYRILLIC CAPITAL LETTER KA WITH DESCENDER

0049B
CYRILLIC SMALL LETTER KA WITH DESCENDER

0049C
CYRILLIC CAPITAL LETTER KA WITH VERTICAL STROKE

0049D
CYRILLIC SMALL LETTER KA WITH VERTICAL STROKE

0049E
CYRILLIC CAPITAL LETTER KA WITH STROKE

0049F
CYRILLIC SMALL LETTER KA WITH STROKE

004A0
CYRILLIC CAPITAL LETTER BASHKIR KA

004A1
CYRILLIC SMALL LETTER BASHKIR KA

004A2
CYRILLIC CAPITAL LETTER EN WITH DESCENDER

004A3
CYRILLIC SMALL LETTER EN WITH DESCENDER

004A4
CYRILLIC CAPITAL LIGATURE EN GHE

004A5
CYRILLIC SMALL LIGATURE EN GHE

004A6
CYRILLIC CAPITAL LETTER PE WITH MIDDLE HOOK

004A7
CYRILLIC SMALL LETTER PE WITH MIDDLE HOOK

004A8
CYRILLIC CAPITAL LETTER ABKHASIAN HA

004A9
CYRILLIC SMALL LETTER ABKHASIAN HA

004AA
CYRILLIC CAPITAL LETTER ES WITH DESCENDER

004AB
CYRILLIC SMALL LETTER ES WITH DESCENDER

004AC
CYRILLIC CAPITAL LETTER TE WITH DESCENDER

004AD
CYRILLIC SMALL LETTER TE WITH DESCENDER

004AE
CYRILLIC CAPITAL LETTER STRAIGHT U

004AF
CYRILLIC SMALL LETTER STRAIGHT U

004B0
CYRILLIC CAPITAL LETTER STRAIGHT U WITH STROKE

004B1
CYRILLIC SMALL LETTER STRAIGHT U WITH STROKE

004B2
CYRILLIC CAPITAL LETTER HA WITH DESCENDER

004B3
CYRILLIC SMALL LETTER HA WITH DESCENDER

004B4
CYRILLIC CAPITAL LIGATURE TE TSE

004B5
CYRILLIC SMALL LIGATURE TE TSE

004B6
CYRILLIC CAPITAL LETTER CHE WITH DESCENDER

004B7
CYRILLIC SMALL LETTER CHE WITH DESCENDER

004B8
CYRILLIC CAPITAL LETTER CHE WITH VERTICAL STROKE

004B9
CYRILLIC SMALL LETTER CHE WITH VERTICAL STROKE

004BA
CYRILLIC CAPITAL LETTER SHHA

004BB
CYRILLIC SMALL LETTER SHHA

004BC
CYRILLIC CAPITAL LETTER ABKHASIAN CHE

004BD
CYRILLIC SMALL LETTER ABKHASIAN CHE

004BE
CYRILLIC CAPITAL LETTER ABKHASIAN CHE WITH DESCENDER

004BF
CYRILLIC SMALL LETTER ABKHASIAN CHE WITH DESCENDER

004C0
CYRILLIC LETTER PALOCHKA

004C1
CYRILLIC CAPITAL LETTER ZHE WITH BREVE

004C2
CYRILLIC SMALL LETTER ZHE WITH BREVE

004C3
CYRILLIC CAPITAL LETTER KA WITH HOOK

004C4
CYRILLIC SMALL LETTER KA WITH HOOK

004C5
CYRILLIC CAPITAL LETTER EL WITH TAIL

004C6
CYRILLIC SMALL LETTER EL WITH TAIL

004C7
CYRILLIC CAPITAL LETTER EN WITH HOOK

004C8
CYRILLIC SMALL LETTER EN WITH HOOK

004C9
CYRILLIC CAPITAL LETTER EN WITH TAIL

004CA
CYRILLIC SMALL LETTER EN WITH TAIL

004CB
CYRILLIC CAPITAL LETTER KHAKASSIAN CHE

004CC
CYRILLIC SMALL LETTER KHAKASSIAN CHE

004CD
CYRILLIC CAPITAL LETTER EM WITH TAIL

004CE
CYRILLIC SMALL LETTER EM WITH TAIL

004CF
CYRILLIC SMALL LETTER PALOCHKA

004D0
CYRILLIC CAPITAL LETTER A WITH BREVE

004D1
CYRILLIC SMALL LETTER A WITH BREVE

004D2
CYRILLIC CAPITAL LETTER A WITH DIAERESIS

004D3
CYRILLIC SMALL LETTER A WITH DIAERESIS

004D4
CYRILLIC CAPITAL LIGATURE A IE

004D5
CYRILLIC SMALL LIGATURE A IE

004D6
CYRILLIC CAPITAL LETTER IE WITH BREVE

004D7
CYRILLIC SMALL LETTER IE WITH BREVE

004D8
CYRILLIC CAPITAL LETTER SCHWA

004D9
CYRILLIC SMALL LETTER SCHWA

004DA
CYRILLIC CAPITAL LETTER SCHWA WITH DIAERESIS

004DB
CYRILLIC SMALL LETTER SCHWA WITH DIAERESIS

004DC
CYRILLIC CAPITAL LETTER ZHE WITH DIAERESIS

004DD
CYRILLIC SMALL LETTER ZHE WITH DIAERESIS

004DE
CYRILLIC CAPITAL LETTER ZE WITH DIAERESIS

004DF
CYRILLIC SMALL LETTER ZE WITH DIAERESIS

004E0
CYRILLIC CAPITAL LETTER ABKHASIAN DZE

004E1
CYRILLIC SMALL LETTER ABKHASIAN DZE

004E2
CYRILLIC CAPITAL LETTER I WITH MACRON

004E3
CYRILLIC SMALL LETTER I WITH MACRON

004E4
CYRILLIC CAPITAL LETTER I WITH DIAERESIS

004E5
CYRILLIC SMALL LETTER I WITH DIAERESIS

004E6
CYRILLIC CAPITAL LETTER O WITH DIAERESIS

004E7
CYRILLIC SMALL LETTER O WITH DIAERESIS

004E8
CYRILLIC CAPITAL LETTER BARRED O

004E9
CYRILLIC SMALL LETTER BARRED O

004EA
CYRILLIC CAPITAL LETTER BARRED O WITH DIAERESIS

004EB
CYRILLIC SMALL LETTER BARRED O WITH DIAERESIS

004EC
CYRILLIC CAPITAL LETTER E WITH DIAERESIS

004ED
CYRILLIC SMALL LETTER E WITH DIAERESIS

004EE
CYRILLIC CAPITAL LETTER U WITH MACRON

004EF
CYRILLIC SMALL LETTER U WITH MACRON

004F0
CYRILLIC CAPITAL LETTER U WITH DIAERESIS

004F1
CYRILLIC SMALL LETTER U WITH DIAERESIS

004F2
CYRILLIC CAPITAL LETTER U WITH DOUBLE ACUTE

004F3
CYRILLIC SMALL LETTER U WITH DOUBLE ACUTE

004F4
CYRILLIC CAPITAL LETTER CHE WITH DIAERESIS

004F5
CYRILLIC SMALL LETTER CHE WITH DIAERESIS

004F6
CYRILLIC CAPITAL LETTER GHE WITH DESCENDER

004F7
CYRILLIC SMALL LETTER GHE WITH DESCENDER

004F8
CYRILLIC CAPITAL LETTER YERU WITH DIAERESIS

004F9
CYRILLIC SMALL LETTER YERU WITH DIAERESIS

004FA
CYRILLIC CAPITAL LETTER GHE WITH STROKE AND HOOK

004FB
CYRILLIC SMALL LETTER GHE WITH STROKE AND HOOK

004FC
CYRILLIC CAPITAL LETTER HA WITH HOOK

004FD
CYRILLIC SMALL LETTER HA WITH HOOK

004FE
CYRILLIC CAPITAL LETTER HA WITH STROKE

004FF
CYRILLIC SMALL LETTER HA WITH STROKE

00500
CYRILLIC CAPITAL LETTER KOMI DE

00501
CYRILLIC SMALL LETTER KOMI DE

00502
CYRILLIC CAPITAL LETTER KOMI DJE

00503
CYRILLIC SMALL LETTER KOMI DJE

00504
CYRILLIC CAPITAL LETTER KOMI ZJE

00505
CYRILLIC SMALL LETTER KOMI ZJE

00506
CYRILLIC CAPITAL LETTER KOMI DZJE

00507
CYRILLIC SMALL LETTER KOMI DZJE

00508
CYRILLIC CAPITAL LETTER KOMI LJE

00509
CYRILLIC SMALL LETTER KOMI LJE

0050A
CYRILLIC CAPITAL LETTER KOMI NJE

0050B
CYRILLIC SMALL LETTER KOMI NJE

0050C
CYRILLIC CAPITAL LETTER KOMI SJE

0050D
CYRILLIC SMALL LETTER KOMI SJE

0050E
CYRILLIC CAPITAL LETTER KOMI TJE

0050F
CYRILLIC SMALL LETTER KOMI TJE

00510
CYRILLIC CAPITAL LETTER REVERSED ZE

00511
CYRILLIC SMALL LETTER REVERSED ZE

00512
CYRILLIC CAPITAL LETTER EL WITH HOOK

00513
CYRILLIC SMALL LETTER EL WITH HOOK

00514
CYRILLIC CAPITAL LETTER LHA

00515
CYRILLIC SMALL LETTER LHA

00516
CYRILLIC CAPITAL LETTER RHA

00517
CYRILLIC SMALL LETTER RHA

00518
CYRILLIC CAPITAL LETTER YAE

00519
CYRILLIC SMALL LETTER YAE

0051A
CYRILLIC CAPITAL LETTER QA

0051B
CYRILLIC SMALL LETTER QA

0051C
CYRILLIC CAPITAL LETTER WE

0051D
CYRILLIC SMALL LETTER WE

0051E
CYRILLIC CAPITAL LETTER ALEUT KA

0051F
CYRILLIC SMALL LETTER ALEUT KA

00520
CYRILLIC CAPITAL LETTER EL WITH MIDDLE HOOK

00521
CYRILLIC SMALL LETTER EL WITH MIDDLE HOOK

00522
CYRILLIC CAPITAL LETTER EN WITH MIDDLE HOOK

00523
CYRILLIC SMALL LETTER EN WITH MIDDLE HOOK

00524
CYRILLIC CAPITAL LETTER PE WITH DESCENDER

00525
CYRILLIC SMALL LETTER PE WITH DESCENDER

00526
CYRILLIC CAPITAL LETTER SHHA WITH DESCENDER

00527
CYRILLIC SMALL LETTER SHHA WITH DESCENDER

00528
CYRILLIC CAPITAL LETTER EN WITH LEFT HOOK

00529
CYRILLIC SMALL LETTER EN WITH LEFT HOOK

0052A
CYRILLIC CAPITAL LETTER DZZHE

0052B
CYRILLIC SMALL LETTER DZZHE

0052C
CYRILLIC CAPITAL LETTER DCHE

0052D
CYRILLIC SMALL LETTER DCHE

0052E
CYRILLIC CAPITAL LETTER EL WITH DESCENDER

0052F
CYRILLIC SMALL LETTER EL WITH DESCENDER

00531
ARMENIAN CAPITAL LETTER AYB

00532
ARMENIAN CAPITAL LETTER BEN

00533
ARMENIAN CAPITAL LETTER GIM

00534
ARMENIAN CAPITAL LETTER DA

00535
ARMENIAN CAPITAL LETTER ECH

00536
ARMENIAN CAPITAL LETTER ZA

00537
ARMENIAN CAPITAL LETTER EH

00538
ARMENIAN CAPITAL LETTER ET

00539
ARMENIAN CAPITAL LETTER TO

0053A
ARMENIAN CAPITAL LETTER ZHE

0053B
ARMENIAN CAPITAL LETTER INI

0053C
ARMENIAN CAPITAL LETTER LIWN

0053D
ARMENIAN CAPITAL LETTER XEH

0053E
ARMENIAN CAPITAL LETTER CA

0053F
ARMENIAN CAPITAL LETTER KEN

00540
ARMENIAN CAPITAL LETTER HO

00541
ARMENIAN CAPITAL LETTER JA

00542
ARMENIAN CAPITAL LETTER GHAD

00543
ARMENIAN CAPITAL LETTER CHEH

00544
ARMENIAN CAPITAL LETTER MEN

00545
ARMENIAN CAPITAL LETTER YI

00546
ARMENIAN CAPITAL LETTER NOW

00547
ARMENIAN CAPITAL LETTER SHA

00548
ARMENIAN CAPITAL LETTER VO

00549
ARMENIAN CAPITAL LETTER CHA

0054A
ARMENIAN CAPITAL LETTER PEH

0054B
ARMENIAN CAPITAL LETTER JHEH

0054C
ARMENIAN CAPITAL LETTER RA

0054D
ARMENIAN CAPITAL LETTER SEH

0054E
ARMENIAN CAPITAL LETTER VEW

0054F
ARMENIAN CAPITAL LETTER TIWN

00550
ARMENIAN CAPITAL LETTER REH

00551
ARMENIAN CAPITAL LETTER CO

00552
ARMENIAN CAPITAL LETTER YIWN

00553
ARMENIAN CAPITAL LETTER PIWR

00554
ARMENIAN CAPITAL LETTER KEH

00555
ARMENIAN CAPITAL LETTER OH

00556
ARMENIAN CAPITAL LETTER FEH

00559
ARMENIAN MODIFIER LETTER LEFT HALF RING

0055A
ARMENIAN APOSTROPHE

0055B
ARMENIAN EMPHASIS MARK

0055C
ARMENIAN EXCLAMATION MARK

0055D
ARMENIAN COMMA

0055E
ARMENIAN QUESTION MARK

0055F
ARMENIAN ABBREVIATION MARK

00560
ARMENIAN SMALL LETTER TURNED AYB

00561
ARMENIAN SMALL LETTER AYB

00562
ARMENIAN SMALL LETTER BEN

00563
ARMENIAN SMALL LETTER GIM

00564
ARMENIAN SMALL LETTER DA

00565
ARMENIAN SMALL LETTER ECH

00566
ARMENIAN SMALL LETTER ZA

00567
ARMENIAN SMALL LETTER EH

00568
ARMENIAN SMALL LETTER ET

00569
ARMENIAN SMALL LETTER TO

0056A
ARMENIAN SMALL LETTER ZHE

0056B
ARMENIAN SMALL LETTER INI

0056C
ARMENIAN SMALL LETTER LIWN

0056D
ARMENIAN SMALL LETTER XEH

0056E
ARMENIAN SMALL LETTER CA

0056F
ARMENIAN SMALL LETTER KEN

00570
ARMENIAN SMALL LETTER HO

00571
ARMENIAN SMALL LETTER JA

00572
ARMENIAN SMALL LETTER GHAD

00573
ARMENIAN SMALL LETTER CHEH

00574
ARMENIAN SMALL LETTER MEN

00575
ARMENIAN SMALL LETTER YI

00576
ARMENIAN SMALL LETTER NOW

00577
ARMENIAN SMALL LETTER SHA

00578
ARMENIAN SMALL LETTER VO

00579
ARMENIAN SMALL LETTER CHA

0057A
ARMENIAN SMALL LETTER PEH

0057B
ARMENIAN SMALL LETTER JHEH

0057C
ARMENIAN SMALL LETTER RA

0057D
ARMENIAN SMALL LETTER SEH

0057E
ARMENIAN SMALL LETTER VEW

0057F
ARMENIAN SMALL LETTER TIWN

00580
ARMENIAN SMALL LETTER REH

00581
ARMENIAN SMALL LETTER CO

00582
ARMENIAN SMALL LETTER YIWN

00583
ARMENIAN SMALL LETTER PIWR

00584
ARMENIAN SMALL LETTER KEH

00585
ARMENIAN SMALL LETTER OH

00586
ARMENIAN SMALL LETTER FEH

00587
ARMENIAN SMALL LIGATURE ECH YIWN

00588
ARMENIAN SMALL LETTER YI WITH STROKE

00589
ARMENIAN FULL STOP

0058A
ARMENIAN HYPHEN

0058D
RIGHT-FACING ARMENIAN ETERNITY SIGN

0058E
LEFT-FACING ARMENIAN ETERNITY SIGN

0058F
ARMENIAN DRAM SIGN

00591
HEBREW ACCENT ETNAHTA

00592
HEBREW ACCENT SEGOL

00593
HEBREW ACCENT SHALSHELET

00594
HEBREW ACCENT ZAQEF QATAN

00595
HEBREW ACCENT ZAQEF GADOL

00596
HEBREW ACCENT TIPEHA

00597
HEBREW ACCENT REVIA

00598
HEBREW ACCENT ZARQA

00599
HEBREW ACCENT PASHTA

0059A
HEBREW ACCENT YETIV

0059B
HEBREW ACCENT TEVIR

0059C
HEBREW ACCENT GERESH

0059D
HEBREW ACCENT GERESH MUQDAM

0059E
HEBREW ACCENT GERSHAYIM

0059F
HEBREW ACCENT QARNEY PARA

005A0
HEBREW ACCENT TELISHA GEDOLA

005A1
HEBREW ACCENT PAZER

005A2
HEBREW ACCENT ATNAH HAFUKH

005A3
HEBREW ACCENT MUNAH

005A4
HEBREW ACCENT MAHAPAKH

005A5
HEBREW ACCENT MERKHA

005A6
HEBREW ACCENT MERKHA KEFULA

005A7
HEBREW ACCENT DARGA

005A8
HEBREW ACCENT QADMA

005A9
HEBREW ACCENT TELISHA QETANA

005AA
HEBREW ACCENT YERAH BEN YOMO

005AB
HEBREW ACCENT OLE

005AC
HEBREW ACCENT ILUY

005AD
HEBREW ACCENT DEHI

005AE
HEBREW ACCENT ZINOR

005AF
HEBREW MARK MASORA CIRCLE

005B0
HEBREW POINT SHEVA

005B1
HEBREW POINT HATAF SEGOL

005B2
HEBREW POINT HATAF PATAH

005B3
HEBREW POINT HATAF QAMATS

005B4
HEBREW POINT HIRIQ

005B5
HEBREW POINT TSERE

005B6
HEBREW POINT SEGOL

005B7
HEBREW POINT PATAH

005B8
HEBREW POINT QAMATS

005B9
HEBREW POINT HOLAM

005BA
HEBREW POINT HOLAM HASER FOR VAV

005BB
HEBREW POINT QUBUTS

005BC
HEBREW POINT DAGESH OR MAPIQ

005BD
HEBREW POINT METEG

005BE
HEBREW PUNCTUATION MAQAF

005BF
HEBREW POINT RAFE

005C0
HEBREW PUNCTUATION PASEQ

005C1
HEBREW POINT SHIN DOT

005C2
HEBREW POINT SIN DOT

005C3
HEBREW PUNCTUATION SOF PASUQ

005C4
HEBREW MARK UPPER DOT

005C5
HEBREW MARK LOWER DOT

005C6
HEBREW PUNCTUATION NUN HAFUKHA

005C7
HEBREW POINT QAMATS QATAN

005D0
HEBREW LETTER ALEF

005D1
HEBREW LETTER BET

005D2
HEBREW LETTER GIMEL

005D3
HEBREW LETTER DALET

005D4
HEBREW LETTER HE

005D5
HEBREW LETTER VAV

005D6
HEBREW LETTER ZAYIN

005D7
HEBREW LETTER HET

005D8
HEBREW LETTER TET

005D9
HEBREW LETTER YOD

005DA
HEBREW LETTER FINAL KAF

005DB
HEBREW LETTER KAF

005DC
HEBREW LETTER LAMED

005DD
HEBREW LETTER FINAL MEM

005DE
HEBREW LETTER MEM

005DF
HEBREW LETTER FINAL NUN

005E0
HEBREW LETTER NUN

005E1
HEBREW LETTER SAMEKH

005E2
HEBREW LETTER AYIN

005E3
HEBREW LETTER FINAL PE

005E4
HEBREW LETTER PE

005E5
HEBREW LETTER FINAL TSADI

005E6
HEBREW LETTER TSADI

005E7
HEBREW LETTER QOF

005E8
HEBREW LETTER RESH

005E9
HEBREW LETTER SHIN

005EA
HEBREW LETTER TAV

005EF
HEBREW YOD TRIANGLE

005F0
HEBREW LIGATURE YIDDISH DOUBLE VAV

005F1
HEBREW LIGATURE YIDDISH VAV YOD

005F2
HEBREW LIGATURE YIDDISH DOUBLE YOD

005F3
HEBREW PUNCTUATION GERESH

005F4
HEBREW PUNCTUATION GERSHAYIM

00600
ARABIC NUMBER SIGN

00601
ARABIC SIGN SANAH

00602
ARABIC FOOTNOTE MARKER

00603
ARABIC SIGN SAFHA

00604
ARABIC SIGN SAMVAT

00605
ARABIC NUMBER MARK ABOVE

00606
ARABIC-INDIC CUBE ROOT

00607
ARABIC-INDIC FOURTH ROOT

00608
ARABIC RAY

00609
ARABIC-INDIC PER MILLE SIGN

0060A
ARABIC-INDIC PER TEN THOUSAND SIGN

0060B
AFGHANI SIGN

0060C
ARABIC COMMA

0060D
ARABIC DATE SEPARATOR

0060E
ARABIC POETIC VERSE SIGN

0060F
ARABIC SIGN MISRA

00610
ARABIC SIGN SALLALLAHOU ALAYHE WASSALLAM

00611
ARABIC SIGN ALAYHE ASSALLAM

00612
ARABIC SIGN RAHMATULLAH ALAYHE

00613
ARABIC SIGN RADI ALLAHOU ANHU

00614
ARABIC SIGN TAKHALLUS

00615
ARABIC SMALL HIGH TAH

00616
ARABIC SMALL HIGH LIGATURE ALEF WITH YEH BARREE

00616
ARABIC SMALL HIGH LIGATURE ALEF WITH LAM WITH YEH

00617
ARABIC SMALL HIGH ZAIN

00618
ARABIC SMALL FATHA

00619
ARABIC SMALL DAMMA

0061A
ARABIC SMALL KASRA

0061B
ARABIC SEMICOLON

0061C
ARABIC LETTER MARK

0061C
ALM

0061D
ARABIC END OF TEXT MARK

0061E
ARABIC TRIPLE DOT PUNCTUATION MARK

0061F
ARABIC QUESTION MARK

00620
ARABIC LETTER KASHMIRI YEH

00621
ARABIC LETTER HAMZA

00622
ARABIC LETTER ALEF WITH MADDA ABOVE

00623
ARABIC LETTER ALEF WITH HAMZA ABOVE

00624
ARABIC LETTER WAW WITH HAMZA ABOVE

00625
ARABIC LETTER ALEF WITH HAMZA BELOW

00626
ARABIC LETTER YEH WITH HAMZA ABOVE

00627
ARABIC LETTER ALEF

00628
ARABIC LETTER BEH

00629
ARABIC LETTER TEH MARBUTA

0062A
ARABIC LETTER TEH

0062B
ARABIC LETTER THEH

0062C
ARABIC LETTER JEEM

0062D
ARABIC LETTER HAH

0062E
ARABIC LETTER KHAH

0062F
ARABIC LETTER DAL

00630
ARABIC LETTER THAL

00631
ARABIC LETTER REH

00632
ARABIC LETTER ZAIN

00633
ARABIC LETTER SEEN

00634
ARABIC LETTER SHEEN

00635
ARABIC LETTER SAD

00636
ARABIC LETTER DAD

00637
ARABIC LETTER TAH

00638
ARABIC LETTER ZAH

00639
ARABIC LETTER AIN

0063A
ARABIC LETTER GHAIN

0063B
ARABIC LETTER KEHEH WITH TWO DOTS ABOVE

0063C
ARABIC LETTER KEHEH WITH THREE DOTS BELOW

0063D
ARABIC LETTER FARSI YEH WITH INVERTED V

0063E
ARABIC LETTER FARSI YEH WITH TWO DOTS ABOVE

0063F
ARABIC LETTER FARSI YEH WITH THREE DOTS ABOVE

00640
ARABIC TATWEEL

00641
ARABIC LETTER FEH

00642
ARABIC LETTER QAF

00643
ARABIC LETTER KAF

00644
ARABIC LETTER LAM

00645
ARABIC LETTER MEEM

00646
ARABIC LETTER NOON

00647
ARABIC LETTER HEH

00648
ARABIC LETTER WAW

00649
ARABIC LETTER ALEF MAKSURA

0064A
ARABIC LETTER YEH

0064B
ARABIC FATHATAN

0064C
ARABIC DAMMATAN

0064D
ARABIC KASRATAN

0064E
ARABIC FATHA

0064F
ARABIC DAMMA

00650
ARABIC KASRA

00651
ARABIC SHADDA

00652
ARABIC SUKUN

00653
ARABIC MADDAH ABOVE

00654
ARABIC HAMZA ABOVE

00655
ARABIC HAMZA BELOW

00656
ARABIC SUBSCRIPT ALEF

00657
ARABIC INVERTED DAMMA

00658
ARABIC MARK NOON GHUNNA

00659
ARABIC ZWARAKAY

0065A
ARABIC VOWEL SIGN SMALL V ABOVE

0065B
ARABIC VOWEL SIGN INVERTED SMALL V ABOVE

0065C
ARABIC VOWEL SIGN DOT BELOW

0065D
ARABIC REVERSED DAMMA

0065E
ARABIC FATHA WITH TWO DOTS

0065F
ARABIC WAVY HAMZA BELOW

00660
ARABIC-INDIC DIGIT ZERO

00661
ARABIC-INDIC DIGIT ONE

00662
ARABIC-INDIC DIGIT TWO

00663
ARABIC-INDIC DIGIT THREE

00664
ARABIC-INDIC DIGIT FOUR

00665
ARABIC-INDIC DIGIT FIVE

00666
ARABIC-INDIC DIGIT SIX

00667
ARABIC-INDIC DIGIT SEVEN

00668
ARABIC-INDIC DIGIT EIGHT

00669
ARABIC-INDIC DIGIT NINE

0066A
ARABIC PERCENT SIGN

0066B
ARABIC DECIMAL SEPARATOR

0066C
ARABIC THOUSANDS SEPARATOR

0066D
ARABIC FIVE POINTED STAR

0066E
ARABIC LETTER DOTLESS BEH

0066F
ARABIC LETTER DOTLESS QAF

00670
ARABIC LETTER SUPERSCRIPT ALEF

00671
ARABIC LETTER ALEF WASLA

00672
ARABIC LETTER ALEF WITH WAVY HAMZA ABOVE

00673
ARABIC LETTER ALEF WITH WAVY HAMZA BELOW

00674
ARABIC LETTER HIGH HAMZA

00675
ARABIC LETTER HIGH HAMZA ALEF

00676
ARABIC LETTER HIGH HAMZA WAW

00677
ARABIC LETTER U WITH HAMZA ABOVE

00678
ARABIC LETTER HIGH HAMZA YEH

00679
ARABIC LETTER TTEH

0067A
ARABIC LETTER TTEHEH

0067B
ARABIC LETTER BEEH

0067C
ARABIC LETTER TEH WITH RING

0067D
ARABIC LETTER TEH WITH THREE DOTS ABOVE DOWNWARDS

0067E
ARABIC LETTER PEH

0067F
ARABIC LETTER TEHEH

00680
ARABIC LETTER BEHEH

00681
ARABIC LETTER HAH WITH HAMZA ABOVE

00682
ARABIC LETTER HAH WITH TWO DOTS VERTICAL ABOVE

00683
ARABIC LETTER NYEH

00684
ARABIC LETTER DYEH

00685
ARABIC LETTER HAH WITH THREE DOTS ABOVE

00686
ARABIC LETTER TCHEH

00687
ARABIC LETTER TCHEHEH

00688
ARABIC LETTER DDAL

00689
ARABIC LETTER DAL WITH RING

0068A
ARABIC LETTER DAL WITH DOT BELOW

0068B
ARABIC LETTER DAL WITH DOT BELOW AND SMALL TAH

0068C
ARABIC LETTER DAHAL

0068D
ARABIC LETTER DDAHAL

0068E
ARABIC LETTER DUL

0068F
ARABIC LETTER DAL WITH THREE DOTS ABOVE DOWNWARDS

00690
ARABIC LETTER DAL WITH FOUR DOTS ABOVE

00691
ARABIC LETTER RREH

00692
ARABIC LETTER REH WITH SMALL V

00693
ARABIC LETTER REH WITH RING

00694
ARABIC LETTER REH WITH DOT BELOW

00695
ARABIC LETTER REH WITH SMALL V BELOW

00696
ARABIC LETTER REH WITH DOT BELOW AND DOT ABOVE

00697
ARABIC LETTER REH WITH TWO DOTS ABOVE

00698
ARABIC LETTER JEH

00699
ARABIC LETTER REH WITH FOUR DOTS ABOVE

0069A
ARABIC LETTER SEEN WITH DOT BELOW AND DOT ABOVE

0069B
ARABIC LETTER SEEN WITH THREE DOTS BELOW

0069C
ARABIC LETTER SEEN WITH THREE DOTS BELOW AND THREE DOTS ABOVE

0069D
ARABIC LETTER SAD WITH TWO DOTS BELOW

0069E
ARABIC LETTER SAD WITH THREE DOTS ABOVE

0069F
ARABIC LETTER TAH WITH THREE DOTS ABOVE

006A0
ARABIC LETTER AIN WITH THREE DOTS ABOVE

006A1
ARABIC LETTER DOTLESS FEH

006A2
ARABIC LETTER FEH WITH DOT MOVED BELOW

006A3
ARABIC LETTER FEH WITH DOT BELOW

006A4
ARABIC LETTER VEH

006A5
ARABIC LETTER FEH WITH THREE DOTS BELOW

006A6
ARABIC LETTER PEHEH

006A7
ARABIC LETTER QAF WITH DOT ABOVE

006A8
ARABIC LETTER QAF WITH THREE DOTS ABOVE

006A9
ARABIC LETTER KEHEH

006AA
ARABIC LETTER SWASH KAF

006AB
ARABIC LETTER KAF WITH RING

006AC
ARABIC LETTER KAF WITH DOT ABOVE

006AD
ARABIC LETTER NG

006AE
ARABIC LETTER KAF WITH THREE DOTS BELOW

006AF
ARABIC LETTER GAF

006B0
ARABIC LETTER GAF WITH RING

006B1
ARABIC LETTER NGOEH

006B2
ARABIC LETTER GAF WITH TWO DOTS BELOW

006B3
ARABIC LETTER GUEH

006B4
ARABIC LETTER GAF WITH THREE DOTS ABOVE

006B5
ARABIC LETTER LAM WITH SMALL V

006B6
ARABIC LETTER LAM WITH DOT ABOVE

006B7
ARABIC LETTER LAM WITH THREE DOTS ABOVE

006B8
ARABIC LETTER LAM WITH THREE DOTS BELOW

006B9
ARABIC LETTER NOON WITH DOT BELOW

006BA
ARABIC LETTER NOON GHUNNA

006BB
ARABIC LETTER RNOON

006BC
ARABIC LETTER NOON WITH RING

006BD
ARABIC LETTER NOON WITH THREE DOTS ABOVE

006BE
ARABIC LETTER HEH DOACHASHMEE

006BF
ARABIC LETTER TCHEH WITH DOT ABOVE

006C0
ARABIC LETTER HEH WITH YEH ABOVE

006C1
ARABIC LETTER HEH GOAL

006C2
ARABIC LETTER HEH GOAL WITH HAMZA ABOVE

006C3
ARABIC LETTER TEH MARBUTA GOAL

006C4
ARABIC LETTER WAW WITH RING

006C5
ARABIC LETTER KIRGHIZ OE

006C6
ARABIC LETTER OE

006C7
ARABIC LETTER U

006C8
ARABIC LETTER YU

006C9
ARABIC LETTER KIRGHIZ YU

006CA
ARABIC LETTER WAW WITH TWO DOTS ABOVE

006CB
ARABIC LETTER VE

006CC
ARABIC LETTER FARSI YEH

006CD
ARABIC LETTER YEH WITH TAIL

006CE
ARABIC LETTER YEH WITH SMALL V

006CF
ARABIC LETTER WAW WITH DOT ABOVE

006D0
ARABIC LETTER E

006D1
ARABIC LETTER YEH WITH THREE DOTS BELOW

006D2
ARABIC LETTER YEH BARREE

006D3
ARABIC LETTER YEH BARREE WITH HAMZA ABOVE

006D4
ARABIC FULL STOP

006D5
ARABIC LETTER AE

006D6
ARABIC SMALL HIGH LIGATURE SAD WITH LAM WITH ALEF MAKSURA

006D7
ARABIC SMALL HIGH LIGATURE QAF WITH LAM WITH ALEF MAKSURA

006D8
ARABIC SMALL HIGH MEEM INITIAL FORM

006D9
ARABIC SMALL HIGH LAM ALEF

006DA
ARABIC SMALL HIGH JEEM

006DB
ARABIC SMALL HIGH THREE DOTS

006DC
ARABIC SMALL HIGH SEEN

006DD
ARABIC END OF AYAH

006DE
ARABIC START OF RUB EL HIZB

006DF
ARABIC SMALL HIGH ROUNDED ZERO

006E0
ARABIC SMALL HIGH UPRIGHT RECTANGULAR ZERO

006E1
ARABIC SMALL HIGH DOTLESS HEAD OF KHAH

006E2
ARABIC SMALL HIGH MEEM ISOLATED FORM

006E3
ARABIC SMALL LOW SEEN

006E4
ARABIC SMALL HIGH MADDA

006E5
ARABIC SMALL WAW

006E6
ARABIC SMALL YEH

006E7
ARABIC SMALL HIGH YEH

006E8
ARABIC SMALL HIGH NOON

006E9
ARABIC PLACE OF SAJDAH

006EA
ARABIC EMPTY CENTRE LOW STOP

006EB
ARABIC EMPTY CENTRE HIGH STOP

006EC
ARABIC ROUNDED HIGH STOP WITH FILLED CENTRE

006ED
ARABIC SMALL LOW MEEM

006EE
ARABIC LETTER DAL WITH INVERTED V

006EF
ARABIC LETTER REH WITH INVERTED V

006F0
EXTENDED ARABIC-INDIC DIGIT ZERO

006F1
EXTENDED ARABIC-INDIC DIGIT ONE

006F2
EXTENDED ARABIC-INDIC DIGIT TWO

006F3
EXTENDED ARABIC-INDIC DIGIT THREE

006F4
EXTENDED ARABIC-INDIC DIGIT FOUR

006F5
EXTENDED ARABIC-INDIC DIGIT FIVE

006F6
EXTENDED ARABIC-INDIC DIGIT SIX

006F7
EXTENDED ARABIC-INDIC DIGIT SEVEN

006F8
EXTENDED ARABIC-INDIC DIGIT EIGHT

006F9
EXTENDED ARABIC-INDIC DIGIT NINE

006FA
ARABIC LETTER SHEEN WITH DOT BELOW

006FB
ARABIC LETTER DAD WITH DOT BELOW

006FC
ARABIC LETTER GHAIN WITH DOT BELOW

006FD
ARABIC SIGN SINDHI AMPERSAND

006FE
ARABIC SIGN SINDHI POSTPOSITION MEN

006FF
ARABIC LETTER HEH WITH INVERTED V

00700
SYRIAC END OF PARAGRAPH

00701
SYRIAC SUPRALINEAR FULL STOP

00702
SYRIAC SUBLINEAR FULL STOP

00703
SYRIAC SUPRALINEAR COLON

00704
SYRIAC SUBLINEAR COLON

00705
SYRIAC HORIZONTAL COLON

00706
SYRIAC COLON SKEWED LEFT

00707
SYRIAC COLON SKEWED RIGHT

00708
SYRIAC SUPRALINEAR COLON SKEWED LEFT

00709
SYRIAC SUBLINEAR COLON SKEWED LEFT

00709
SYRIAC SUBLINEAR COLON SKEWED RIGHT

0070A
SYRIAC CONTRACTION

0070B
SYRIAC HARKLEAN OBELUS

0070C
SYRIAC HARKLEAN METOBELUS

0070D
SYRIAC HARKLEAN ASTERISCUS

0070F
SYRIAC ABBREVIATION MARK

00710
SYRIAC LETTER ALAPH

00711
SYRIAC LETTER SUPERSCRIPT ALAPH

00712
SYRIAC LETTER BETH

00713
SYRIAC LETTER GAMAL

00714
SYRIAC LETTER GAMAL GARSHUNI

00715
SYRIAC LETTER DALATH

00716
SYRIAC LETTER DOTLESS DALATH RISH

00717
SYRIAC LETTER HE

00718
SYRIAC LETTER WAW

00719
SYRIAC LETTER ZAIN

0071A
SYRIAC LETTER HETH

0071B
SYRIAC LETTER TETH

0071C
SYRIAC LETTER TETH GARSHUNI

0071D
SYRIAC LETTER YUDH

0071E
SYRIAC LETTER YUDH HE

0071F
SYRIAC LETTER KAPH

00720
SYRIAC LETTER LAMADH

00721
SYRIAC LETTER MIM

00722
SYRIAC LETTER NUN

00723
SYRIAC LETTER SEMKATH

00724
SYRIAC LETTER FINAL SEMKATH

00725
SYRIAC LETTER E

00726
SYRIAC LETTER PE

00727
SYRIAC LETTER REVERSED PE

00728
SYRIAC LETTER SADHE

00729
SYRIAC LETTER QAPH

0072A
SYRIAC LETTER RISH

0072B
SYRIAC LETTER SHIN

0072C
SYRIAC LETTER TAW

0072D
SYRIAC LETTER PERSIAN BHETH

0072E
SYRIAC LETTER PERSIAN GHAMAL

0072F
SYRIAC LETTER PERSIAN DHALATH

00730
SYRIAC PTHAHA ABOVE

00731
SYRIAC PTHAHA BELOW

00732
SYRIAC PTHAHA DOTTED

00733
SYRIAC ZQAPHA ABOVE

00734
SYRIAC ZQAPHA BELOW

00735
SYRIAC ZQAPHA DOTTED

00736
SYRIAC RBASA ABOVE

00737
SYRIAC RBASA BELOW

00738
SYRIAC DOTTED ZLAMA HORIZONTAL

00739
SYRIAC DOTTED ZLAMA ANGULAR

0073A
SYRIAC HBASA ABOVE

0073B
SYRIAC HBASA BELOW

0073C
SYRIAC HBASA-ESASA DOTTED

0073D
SYRIAC ESASA ABOVE

0073E
SYRIAC ESASA BELOW

0073F
SYRIAC RWAHA

00740
SYRIAC FEMININE DOT

00741
SYRIAC QUSHSHAYA

00742
SYRIAC RUKKAKHA

00743
SYRIAC TWO VERTICAL DOTS ABOVE

00744
SYRIAC TWO VERTICAL DOTS BELOW

00745
SYRIAC THREE DOTS ABOVE

00746
SYRIAC THREE DOTS BELOW

00747
SYRIAC OBLIQUE LINE ABOVE

00748
SYRIAC OBLIQUE LINE BELOW

00749
SYRIAC MUSIC

0074A
SYRIAC BARREKH

0074D
SYRIAC LETTER SOGDIAN ZHAIN

0074E
SYRIAC LETTER SOGDIAN KHAPH

0074F
SYRIAC LETTER SOGDIAN FE

00750
ARABIC LETTER BEH WITH THREE DOTS HORIZONTALLY BELOW

00751
ARABIC LETTER BEH WITH DOT BELOW AND THREE DOTS ABOVE

00752
ARABIC LETTER BEH WITH THREE DOTS POINTING UPWARDS BELOW

00753
ARABIC LETTER BEH WITH THREE DOTS POINTING UPWARDS BELOW AND TWO DOTS ABOVE

00754
ARABIC LETTER BEH WITH TWO DOTS BELOW AND DOT ABOVE

00755
ARABIC LETTER BEH WITH INVERTED SMALL V BELOW

00756
ARABIC LETTER BEH WITH SMALL V

00757
ARABIC LETTER HAH WITH TWO DOTS ABOVE

00758
ARABIC LETTER HAH WITH THREE DOTS POINTING UPWARDS BELOW

00759
ARABIC LETTER DAL WITH TWO DOTS VERTICALLY BELOW AND SMALL TAH

0075A
ARABIC LETTER DAL WITH INVERTED SMALL V BELOW

0075B
ARABIC LETTER REH WITH STROKE

0075C
ARABIC LETTER SEEN WITH FOUR DOTS ABOVE

0075D
ARABIC LETTER AIN WITH TWO DOTS ABOVE

0075E
ARABIC LETTER AIN WITH THREE DOTS POINTING DOWNWARDS ABOVE

0075F
ARABIC LETTER AIN WITH TWO DOTS VERTICALLY ABOVE

00760
ARABIC LETTER FEH WITH TWO DOTS BELOW

00761
ARABIC LETTER FEH WITH THREE DOTS POINTING UPWARDS BELOW

00762
ARABIC LETTER KEHEH WITH DOT ABOVE

00763
ARABIC LETTER KEHEH WITH THREE DOTS ABOVE

00764
ARABIC LETTER KEHEH WITH THREE DOTS POINTING UPWARDS BELOW

00765
ARABIC LETTER MEEM WITH DOT ABOVE

00766
ARABIC LETTER MEEM WITH DOT BELOW

00767
ARABIC LETTER NOON WITH TWO DOTS BELOW

00768
ARABIC LETTER NOON WITH SMALL TAH

00769
ARABIC LETTER NOON WITH SMALL V

0076A
ARABIC LETTER LAM WITH BAR

0076B
ARABIC LETTER REH WITH TWO DOTS VERTICALLY ABOVE

0076C
ARABIC LETTER REH WITH HAMZA ABOVE

0076D
ARABIC LETTER SEEN WITH TWO DOTS VERTICALLY ABOVE

0076E
ARABIC LETTER HAH WITH SMALL ARABIC LETTER TAH BELOW

0076F
ARABIC LETTER HAH WITH SMALL ARABIC LETTER TAH AND TWO DOTS

00770
ARABIC LETTER SEEN WITH SMALL ARABIC LETTER TAH AND TWO DOTS

00771
ARABIC LETTER REH WITH SMALL ARABIC LETTER TAH AND TWO DOTS

00772
ARABIC LETTER HAH WITH SMALL ARABIC LETTER TAH ABOVE

00773
ARABIC LETTER ALEF WITH EXTENDED ARABIC-INDIC DIGIT TWO ABOVE

00774
ARABIC LETTER ALEF WITH EXTENDED ARABIC-INDIC DIGIT THREE ABOVE

00775
ARABIC LETTER FARSI YEH WITH EXTENDED ARABIC-INDIC DIGIT TWO ABOVE

00776
ARABIC LETTER FARSI YEH WITH EXTENDED ARABIC-INDIC DIGIT THREE ABOVE

00777
ARABIC LETTER FARSI YEH WITH EXTENDED ARABIC-INDIC DIGIT FOUR BELOW

00778
ARABIC LETTER WAW WITH EXTENDED ARABIC-INDIC DIGIT TWO ABOVE

00779
ARABIC LETTER WAW WITH EXTENDED ARABIC-INDIC DIGIT THREE ABOVE

0077A
ARABIC LETTER YEH BARREE WITH EXTENDED ARABIC-INDIC DIGIT TWO ABOVE

0077B
ARABIC LETTER YEH BARREE WITH EXTENDED ARABIC-INDIC DIGIT THREE ABOVE

0077C
ARABIC LETTER HAH WITH EXTENDED ARABIC-INDIC DIGIT FOUR BELOW

0077D
ARABIC LETTER SEEN WITH EXTENDED ARABIC-INDIC DIGIT FOUR ABOVE

0077E
ARABIC LETTER SEEN WITH INVERTED V

0077F
ARABIC LETTER KAF WITH TWO DOTS ABOVE

00780
THAANA LETTER HAA

00781
THAANA LETTER SHAVIYANI

00782
THAANA LETTER NOONU

00783
THAANA LETTER RAA

00784
THAANA LETTER BAA

00785
THAANA LETTER LHAVIYANI

00786
THAANA LETTER KAAFU

00787
THAANA LETTER ALIFU

00788
THAANA LETTER VAAVU

00789
THAANA LETTER MEEMU

0078A
THAANA LETTER FAAFU

0078B
THAANA LETTER DHAALU

0078C
THAANA LETTER THAA

0078D
THAANA LETTER LAAMU

0078E
THAANA LETTER GAAFU

0078F
THAANA LETTER GNAVIYANI

00790
THAANA LETTER SEENU

00791
THAANA LETTER DAVIYANI

00792
THAANA LETTER ZAVIYANI

00793
THAANA LETTER TAVIYANI

00794
THAANA LETTER YAA

00795
THAANA LETTER PAVIYANI

00796
THAANA LETTER JAVIYANI

00797
THAANA LETTER CHAVIYANI

00798
THAANA LETTER TTAA

00799
THAANA LETTER HHAA

0079A
THAANA LETTER KHAA

0079B
THAANA LETTER THAALU

0079C
THAANA LETTER ZAA

0079D
THAANA LETTER SHEENU

0079E
THAANA LETTER SAADHU

0079F
THAANA LETTER DAADHU

007A0
THAANA LETTER TO

007A1
THAANA LETTER ZO

007A2
THAANA LETTER AINU

007A3
THAANA LETTER GHAINU

007A4
THAANA LETTER QAAFU

007A5
THAANA LETTER WAAVU

007A6
THAANA ABAFILI

007A7
THAANA AABAAFILI

007A8
THAANA IBIFILI

007A9
THAANA EEBEEFILI

007AA
THAANA UBUFILI

007AB
THAANA OOBOOFILI

007AC
THAANA EBEFILI

007AD
THAANA EYBEYFILI

007AE
THAANA OBOFILI

007AF
THAANA OABOAFILI

007B0
THAANA SUKUN

007B1
THAANA LETTER NAA

007C0
NKO DIGIT ZERO

007C1
NKO DIGIT ONE

007C2
NKO DIGIT TWO

007C3
NKO DIGIT THREE

007C4
NKO DIGIT FOUR

007C5
NKO DIGIT FIVE

007C6
NKO DIGIT SIX

007C7
NKO DIGIT SEVEN

007C8
NKO DIGIT EIGHT

007C9
NKO DIGIT NINE

007CA
NKO LETTER A

007CB
NKO LETTER EE

007CC
NKO LETTER I

007CD
NKO LETTER E

007CE
NKO LETTER U

007CF
NKO LETTER OO

007D0
NKO LETTER O

007D1
NKO LETTER DAGBASINNA

007D2
NKO LETTER N

007D3
NKO LETTER BA

007D4
NKO LETTER PA

007D5
NKO LETTER TA

007D6
NKO LETTER JA

007D7
NKO LETTER CHA

007D8
NKO LETTER DA

007D9
NKO LETTER RA

007DA
NKO LETTER RRA

007DB
NKO LETTER SA

007DC
NKO LETTER GBA

007DD
NKO LETTER FA

007DE
NKO LETTER KA

007DF
NKO LETTER LA

007E0
NKO LETTER NA WOLOSO

007E1
NKO LETTER MA

007E2
NKO LETTER NYA

007E3
NKO LETTER NA

007E4
NKO LETTER HA

007E5
NKO LETTER WA

007E6
NKO LETTER YA

007E7
NKO LETTER NYA WOLOSO

007E8
NKO LETTER JONA JA

007E9
NKO LETTER JONA CHA

007EA
NKO LETTER JONA RA

007EB
NKO COMBINING SHORT HIGH TONE

007EC
NKO COMBINING SHORT LOW TONE

007ED
NKO COMBINING SHORT RISING TONE

007EE
NKO COMBINING LONG DESCENDING TONE

007EF
NKO COMBINING LONG HIGH TONE

007F0
NKO COMBINING LONG LOW TONE

007F1
NKO COMBINING LONG RISING TONE

007F2
NKO COMBINING NASALIZATION MARK

007F3
NKO COMBINING DOUBLE DOT ABOVE

007F4
NKO HIGH TONE APOSTROPHE

007F5
NKO LOW TONE APOSTROPHE

007F6
NKO SYMBOL OO DENNEN

007F7
NKO SYMBOL GBAKURUNEN

007F8
NKO COMMA

007F9
NKO EXCLAMATION MARK

007FA
NKO LAJANYALAN

007FD
NKO DANTAYALAN

007FE
NKO DOROME SIGN

007FF
NKO TAMAN SIGN

00800
SAMARITAN LETTER ALAF

00801
SAMARITAN LETTER BIT

00802
SAMARITAN LETTER GAMAN

00803
SAMARITAN LETTER DALAT

00804
SAMARITAN LETTER IY

00805
SAMARITAN LETTER BAA

00806
SAMARITAN LETTER ZEN

00807
SAMARITAN LETTER IT

00808
SAMARITAN LETTER TIT

00809
SAMARITAN LETTER YUT

0080A
SAMARITAN LETTER KAAF

0080B
SAMARITAN LETTER LABAT

0080C
SAMARITAN LETTER MIM

0080D
SAMARITAN LETTER NUN

0080E
SAMARITAN LETTER SINGAAT

0080F
SAMARITAN LETTER IN

00810
SAMARITAN LETTER FI

00811
SAMARITAN LETTER TSAADIY

00812
SAMARITAN LETTER QUF

00813
SAMARITAN LETTER RISH

00814
SAMARITAN LETTER SHAN

00815
SAMARITAN LETTER TAAF

00816
SAMARITAN MARK IN

00817
SAMARITAN MARK IN-ALAF

00818
SAMARITAN MARK OCCLUSION

00819
SAMARITAN MARK DAGESH

0081A
SAMARITAN MODIFIER LETTER EPENTHETIC YUT

0081B
SAMARITAN MARK EPENTHETIC YUT

0081C
SAMARITAN VOWEL SIGN LONG E

0081D
SAMARITAN VOWEL SIGN E

0081E
SAMARITAN VOWEL SIGN OVERLONG AA

0081F
SAMARITAN VOWEL SIGN LONG AA

00820
SAMARITAN VOWEL SIGN AA

00821
SAMARITAN VOWEL SIGN OVERLONG A

00822
SAMARITAN VOWEL SIGN LONG A

00823
SAMARITAN VOWEL SIGN A

00824
SAMARITAN MODIFIER LETTER SHORT A

00825
SAMARITAN VOWEL SIGN SHORT A

00826
SAMARITAN VOWEL SIGN LONG U

00827
SAMARITAN VOWEL SIGN U

00828
SAMARITAN MODIFIER LETTER I

00829
SAMARITAN VOWEL SIGN LONG I

0082A
SAMARITAN VOWEL SIGN I

0082B
SAMARITAN VOWEL SIGN O

0082C
SAMARITAN VOWEL SIGN SUKUN

0082D
SAMARITAN MARK NEQUDAA

00830
SAMARITAN PUNCTUATION NEQUDAA

00831
SAMARITAN PUNCTUATION AFSAAQ

00832
SAMARITAN PUNCTUATION ANGED

00833
SAMARITAN PUNCTUATION BAU

00834
SAMARITAN PUNCTUATION ATMAAU

00835
SAMARITAN PUNCTUATION SHIYYAALAA

00836
SAMARITAN ABBREVIATION MARK

00837
SAMARITAN PUNCTUATION MELODIC QITSA

00838
SAMARITAN PUNCTUATION ZIQAA

00839
SAMARITAN PUNCTUATION QITSA

0083A
SAMARITAN PUNCTUATION ZAEF

0083B
SAMARITAN PUNCTUATION TURU

0083C
SAMARITAN PUNCTUATION ARKAANU

0083D
SAMARITAN PUNCTUATION SOF MASHFAAT

0083E
SAMARITAN PUNCTUATION ANNAAU

00840
MANDAIC LETTER HALQA

00841
MANDAIC LETTER AB

00842
MANDAIC LETTER AG

00843
MANDAIC LETTER AD

00844
MANDAIC LETTER AH

00845
MANDAIC LETTER USHENNA

00846
MANDAIC LETTER AZ

00847
MANDAIC LETTER IT

00848
MANDAIC LETTER ATT

00849
MANDAIC LETTER AKSA

0084A
MANDAIC LETTER AK

0084B
MANDAIC LETTER AL

0084C
MANDAIC LETTER AM

0084D
MANDAIC LETTER AN

0084E
MANDAIC LETTER AS

0084F
MANDAIC LETTER IN

00850
MANDAIC LETTER AP

00851
MANDAIC LETTER ASZ

00852
MANDAIC LETTER AQ

00853
MANDAIC LETTER AR

00854
MANDAIC LETTER ASH

00855
MANDAIC LETTER AT

00856
MANDAIC LETTER DUSHENNA

00857
MANDAIC LETTER KAD

00858
MANDAIC LETTER AIN

00859
MANDAIC AFFRICATION MARK

0085A
MANDAIC VOCALIZATION MARK

0085B
MANDAIC GEMINATION MARK

0085E
MANDAIC PUNCTUATION

00860
SYRIAC LETTER MALAYALAM NGA

00861
SYRIAC LETTER MALAYALAM JA

00862
SYRIAC LETTER MALAYALAM NYA

00863
SYRIAC LETTER MALAYALAM TTA

00864
SYRIAC LETTER MALAYALAM NNA

00865
SYRIAC LETTER MALAYALAM NNNA

00866
SYRIAC LETTER MALAYALAM BHA

00867
SYRIAC LETTER MALAYALAM RA

00868
SYRIAC LETTER MALAYALAM LLA

00869
SYRIAC LETTER MALAYALAM LLLA

0086A
SYRIAC LETTER MALAYALAM SSA

00870
ARABIC LETTER ALEF WITH ATTACHED FATHA

00871
ARABIC LETTER ALEF WITH ATTACHED TOP RIGHT FATHA

00872
ARABIC LETTER ALEF WITH RIGHT MIDDLE STROKE

00873
ARABIC LETTER ALEF WITH LEFT MIDDLE STROKE

00874
ARABIC LETTER ALEF WITH ATTACHED KASRA

00875
ARABIC LETTER ALEF WITH ATTACHED BOTTOM RIGHT KASRA

00876
ARABIC LETTER ALEF WITH ATTACHED ROUND DOT ABOVE

00877
ARABIC LETTER ALEF WITH ATTACHED RIGHT ROUND DOT

00878
ARABIC LETTER ALEF WITH ATTACHED LEFT ROUND DOT

00879
ARABIC LETTER ALEF WITH ATTACHED ROUND DOT BELOW

0087A
ARABIC LETTER ALEF WITH DOT ABOVE

0087B
ARABIC LETTER ALEF WITH ATTACHED TOP RIGHT FATHA AND DOT ABOVE

0087C
ARABIC LETTER ALEF WITH RIGHT MIDDLE STROKE AND DOT ABOVE

0087D
ARABIC LETTER ALEF WITH ATTACHED BOTTOM RIGHT KASRA AND DOT ABOVE

0087E
ARABIC LETTER ALEF WITH ATTACHED TOP RIGHT FATHA AND LEFT RING

0087F
ARABIC LETTER ALEF WITH RIGHT MIDDLE STROKE AND LEFT RING

00880
ARABIC LETTER ALEF WITH ATTACHED BOTTOM RIGHT KASRA AND LEFT RING

00881
ARABIC LETTER ALEF WITH ATTACHED RIGHT HAMZA

00882
ARABIC LETTER ALEF WITH ATTACHED LEFT HAMZA

00883
ARABIC TATWEEL WITH OVERSTRUCK HAMZA

00884
ARABIC TATWEEL WITH OVERSTRUCK WAW

00885
ARABIC TATWEEL WITH TWO DOTS BELOW

00886
ARABIC LETTER THIN YEH

00887
ARABIC BASELINE ROUND DOT

00888
ARABIC RAISED ROUND DOT

00889
ARABIC LETTER NOON WITH INVERTED SMALL V

0088A
ARABIC LETTER HAH WITH INVERTED SMALL V BELOW

0088B
ARABIC LETTER TAH WITH DOT BELOW

0088C
ARABIC LETTER TAH WITH THREE DOTS BELOW

0088D
ARABIC LETTER KEHEH WITH TWO DOTS VERTICALLY BELOW

0088E
ARABIC VERTICAL TAIL

0088F
ARABIC LETTER NOON WITH RING ABOVE

00890
ARABIC POUND MARK ABOVE

00891
ARABIC PIASTRE MARK ABOVE

00897
ARABIC PEPET

00898
ARABIC SMALL HIGH WORD AL-JUZ

00899
ARABIC SMALL LOW WORD ISHMAAM

0089A
ARABIC SMALL LOW WORD IMAALA

0089B
ARABIC SMALL LOW WORD TASHEEL

0089C
ARABIC MADDA WAAJIB

0089D
ARABIC SUPERSCRIPT ALEF MOKHASSAS

0089E
ARABIC DOUBLED MADDA

0089F
ARABIC HALF MADDA OVER MADDA

008A0
ARABIC LETTER BEH WITH SMALL V BELOW

008A1
ARABIC LETTER BEH WITH HAMZA ABOVE

008A2
ARABIC LETTER JEEM WITH TWO DOTS ABOVE

008A3
ARABIC LETTER TAH WITH TWO DOTS ABOVE

008A4
ARABIC LETTER FEH WITH DOT BELOW AND THREE DOTS ABOVE

008A5
ARABIC LETTER QAF WITH DOT BELOW

008A6
ARABIC LETTER LAM WITH DOUBLE BAR

008A7
ARABIC LETTER MEEM WITH THREE DOTS ABOVE

008A8
ARABIC LETTER YEH WITH TWO DOTS BELOW AND HAMZA ABOVE

008A9
ARABIC LETTER YEH WITH TWO DOTS BELOW AND DOT ABOVE

008AA
ARABIC LETTER REH WITH LOOP

008AB
ARABIC LETTER WAW WITH DOT WITHIN

008AC
ARABIC LETTER ROHINGYA YEH

008AD
ARABIC LETTER LOW ALEF

008AE
ARABIC LETTER DAL WITH THREE DOTS BELOW

008AF
ARABIC LETTER SAD WITH THREE DOTS BELOW

008B0
ARABIC LETTER GAF WITH INVERTED STROKE

008B1
ARABIC LETTER STRAIGHT WAW

008B2
ARABIC LETTER ZAIN WITH INVERTED V ABOVE

008B3
ARABIC LETTER AIN WITH THREE DOTS BELOW

008B4
ARABIC LETTER KAF WITH DOT BELOW

008B5
ARABIC LETTER QAF WITH DOT BELOW AND NO DOTS ABOVE

008B6
ARABIC LETTER BEH WITH SMALL MEEM ABOVE

008B7
ARABIC LETTER PEH WITH SMALL MEEM ABOVE

008B8
ARABIC LETTER TEH WITH SMALL TEH ABOVE

008B9
ARABIC LETTER REH WITH SMALL NOON ABOVE

008BA
ARABIC LETTER YEH WITH TWO DOTS BELOW AND SMALL NOON ABOVE

008BB
ARABIC LETTER AFRICAN FEH

008BC
ARABIC LETTER AFRICAN QAF

008BD
ARABIC LETTER AFRICAN NOON

008BE
ARABIC LETTER PEH WITH SMALL V

008BF
ARABIC LETTER TEH WITH SMALL V

008C0
ARABIC LETTER TTEH WITH SMALL V

008C1
ARABIC LETTER TCHEH WITH SMALL V

008C2
ARABIC LETTER KEHEH WITH SMALL V

008C3
ARABIC LETTER GHAIN WITH THREE DOTS ABOVE

008C4
ARABIC LETTER AFRICAN QAF WITH THREE DOTS ABOVE

008C5
ARABIC LETTER JEEM WITH THREE DOTS ABOVE

008C6
ARABIC LETTER JEEM WITH THREE DOTS BELOW

008C7
ARABIC LETTER LAM WITH SMALL ARABIC LETTER TAH ABOVE

008C8
ARABIC LETTER GRAF

008C9
ARABIC SMALL FARSI YEH

008CA
ARABIC SMALL HIGH FARSI YEH

008CB
ARABIC SMALL HIGH YEH BARREE WITH TWO DOTS BELOW

008CC
ARABIC SMALL HIGH WORD SAH

008CD
ARABIC SMALL HIGH ZAH

008CE
ARABIC LARGE ROUND DOT ABOVE

008CF
ARABIC LARGE ROUND DOT BELOW

008D0
ARABIC SUKUN BELOW

008D1
ARABIC LARGE CIRCLE BELOW

008D2
ARABIC LARGE ROUND DOT INSIDE CIRCLE BELOW

008D3
ARABIC SMALL LOW WAW

008D4
ARABIC SMALL HIGH WORD AR-RUB

008D5
ARABIC SMALL HIGH SAD

008D6
ARABIC SMALL HIGH AIN

008D7
ARABIC SMALL HIGH QAF

008D8
ARABIC SMALL HIGH NOON WITH KASRA

008D9
ARABIC SMALL LOW NOON WITH KASRA

008DA
ARABIC SMALL HIGH WORD ATH-THALATHA

008DB
ARABIC SMALL HIGH WORD AS-SAJDA

008DC
ARABIC SMALL HIGH WORD AN-NISF

008DD
ARABIC SMALL HIGH WORD SAKTA

008DE
ARABIC SMALL HIGH WORD QIF

008DF
ARABIC SMALL HIGH WORD WAQFA

008E0
ARABIC SMALL HIGH FOOTNOTE MARKER

008E1
ARABIC SMALL HIGH SIGN SAFHA

008E2
ARABIC DISPUTED END OF AYAH

008E3
ARABIC TURNED DAMMA BELOW

008E4
ARABIC CURLY FATHA

008E5
ARABIC CURLY DAMMA

008E6
ARABIC CURLY KASRA

008E7
ARABIC CURLY FATHATAN

008E8
ARABIC CURLY DAMMATAN

008E9
ARABIC CURLY KASRATAN

008EA
ARABIC TONE ONE DOT ABOVE

008EB
ARABIC TONE TWO DOTS ABOVE

008EC
ARABIC TONE LOOP ABOVE

008ED
ARABIC TONE ONE DOT BELOW

008EE
ARABIC TONE TWO DOTS BELOW

008EF
ARABIC TONE LOOP BELOW

008F0
ARABIC OPEN FATHATAN

008F1
ARABIC OPEN DAMMATAN

008F2
ARABIC OPEN KASRATAN

008F3
ARABIC SMALL HIGH WAW

008F4
ARABIC FATHA WITH RING

008F5
ARABIC FATHA WITH DOT ABOVE

008F6
ARABIC KASRA WITH DOT BELOW

008F7
ARABIC LEFT ARROWHEAD ABOVE

008F8
ARABIC RIGHT ARROWHEAD ABOVE

008F9
ARABIC LEFT ARROWHEAD BELOW

008FA
ARABIC RIGHT ARROWHEAD BELOW

008FB
ARABIC DOUBLE RIGHT ARROWHEAD ABOVE

008FC
ARABIC DOUBLE RIGHT ARROWHEAD ABOVE WITH DOT

008FD
ARABIC RIGHT ARROWHEAD ABOVE WITH DOT

008FE
ARABIC DAMMA WITH DOT

008FF
ARABIC MARK SIDEWAYS NOON GHUNNA

00900
DEVANAGARI SIGN INVERTED CANDRABINDU

00901
DEVANAGARI SIGN CANDRABINDU

00902
DEVANAGARI SIGN ANUSVARA

00903
DEVANAGARI SIGN VISARGA

00904
DEVANAGARI LETTER SHORT A

00905
DEVANAGARI LETTER A

00906
DEVANAGARI LETTER AA

00907
DEVANAGARI LETTER I

00908
DEVANAGARI LETTER II

00909
DEVANAGARI LETTER U

0090A
DEVANAGARI LETTER UU

0090B
DEVANAGARI LETTER VOCALIC R

0090C
DEVANAGARI LETTER VOCALIC L

0090D
DEVANAGARI LETTER CANDRA E

0090E
DEVANAGARI LETTER SHORT E

0090F
DEVANAGARI LETTER E

00910
DEVANAGARI LETTER AI

00911
DEVANAGARI LETTER CANDRA O

00912
DEVANAGARI LETTER SHORT O

00913
DEVANAGARI LETTER O

00914
DEVANAGARI LETTER AU

00915
DEVANAGARI LETTER KA

00916
DEVANAGARI LETTER KHA

00917
DEVANAGARI LETTER GA

00918
DEVANAGARI LETTER GHA

00919
DEVANAGARI LETTER NGA

0091A
DEVANAGARI LETTER CA

0091B
DEVANAGARI LETTER CHA

0091C
DEVANAGARI LETTER JA

0091D
DEVANAGARI LETTER JHA

0091E
DEVANAGARI LETTER NYA

0091F
DEVANAGARI LETTER TTA

00920
DEVANAGARI LETTER TTHA

00921
DEVANAGARI LETTER DDA

00922
DEVANAGARI LETTER DDHA

00923
DEVANAGARI LETTER NNA

00924
DEVANAGARI LETTER TA

00925
DEVANAGARI LETTER THA

00926
DEVANAGARI LETTER DA

00927
DEVANAGARI LETTER DHA

00928
DEVANAGARI LETTER NA

00929
DEVANAGARI LETTER NNNA

0092A
DEVANAGARI LETTER PA

0092B
DEVANAGARI LETTER PHA

0092C
DEVANAGARI LETTER BA

0092D
DEVANAGARI LETTER BHA

0092E
DEVANAGARI LETTER MA

0092F
DEVANAGARI LETTER YA

00930
DEVANAGARI LETTER RA

00931
DEVANAGARI LETTER RRA

00932
DEVANAGARI LETTER LA

00933
DEVANAGARI LETTER LLA

00934
DEVANAGARI LETTER LLLA

00935
DEVANAGARI LETTER VA

00936
DEVANAGARI LETTER SHA

00937
DEVANAGARI LETTER SSA

00938
DEVANAGARI LETTER SA

00939
DEVANAGARI LETTER HA

0093A
DEVANAGARI VOWEL SIGN OE

0093B
DEVANAGARI VOWEL SIGN OOE

0093C
DEVANAGARI SIGN NUKTA

0093D
DEVANAGARI SIGN AVAGRAHA

0093E
DEVANAGARI VOWEL SIGN AA

0093F
DEVANAGARI VOWEL SIGN I

00940
DEVANAGARI VOWEL SIGN II

00941
DEVANAGARI VOWEL SIGN U

00942
DEVANAGARI VOWEL SIGN UU

00943
DEVANAGARI VOWEL SIGN VOCALIC R

00944
DEVANAGARI VOWEL SIGN VOCALIC RR

00945
DEVANAGARI VOWEL SIGN CANDRA E

00946
DEVANAGARI VOWEL SIGN SHORT E

00947
DEVANAGARI VOWEL SIGN E

00948
DEVANAGARI VOWEL SIGN AI

00949
DEVANAGARI VOWEL SIGN CANDRA O

0094A
DEVANAGARI VOWEL SIGN SHORT O

0094B
DEVANAGARI VOWEL SIGN O

0094C
DEVANAGARI VOWEL SIGN AU

0094D
DEVANAGARI SIGN VIRAMA

0094E
DEVANAGARI VOWEL SIGN PRISHTHAMATRA E

0094F
DEVANAGARI VOWEL SIGN AW

00950
DEVANAGARI OM

00951
DEVANAGARI STRESS SIGN UDATTA

00952
DEVANAGARI STRESS SIGN ANUDATTA

00953
DEVANAGARI GRAVE ACCENT

00954
DEVANAGARI ACUTE ACCENT

00955
DEVANAGARI VOWEL SIGN CANDRA LONG E

00956
DEVANAGARI VOWEL SIGN UE

00957
DEVANAGARI VOWEL SIGN UUE

00958
DEVANAGARI LETTER QA

00959
DEVANAGARI LETTER KHHA

0095A
DEVANAGARI LETTER GHHA

0095B
DEVANAGARI LETTER ZA

0095C
DEVANAGARI LETTER DDDHA

0095D
DEVANAGARI LETTER RHA

0095E
DEVANAGARI LETTER FA

0095F
DEVANAGARI LETTER YYA

00960
DEVANAGARI LETTER VOCALIC RR

00961
DEVANAGARI LETTER VOCALIC LL

00962
DEVANAGARI VOWEL SIGN VOCALIC L

00963
DEVANAGARI VOWEL SIGN VOCALIC LL

00964
DEVANAGARI DANDA

00965
DEVANAGARI DOUBLE DANDA

00966
DEVANAGARI DIGIT ZERO

00967
DEVANAGARI DIGIT ONE

00968
DEVANAGARI DIGIT TWO

00969
DEVANAGARI DIGIT THREE

0096A
DEVANAGARI DIGIT FOUR

0096B
DEVANAGARI DIGIT FIVE

0096C
DEVANAGARI DIGIT SIX

0096D
DEVANAGARI DIGIT SEVEN

0096E
DEVANAGARI DIGIT EIGHT

0096F
DEVANAGARI DIGIT NINE

00970
DEVANAGARI ABBREVIATION SIGN

00971
DEVANAGARI SIGN HIGH SPACING DOT

00972
DEVANAGARI LETTER CANDRA A

00973
DEVANAGARI LETTER OE

00974
DEVANAGARI LETTER OOE

00975
DEVANAGARI LETTER AW

00976
DEVANAGARI LETTER UE

00977
DEVANAGARI LETTER UUE

00978
DEVANAGARI LETTER MARWARI DDA

00979
DEVANAGARI LETTER ZHA

0097A
DEVANAGARI LETTER HEAVY YA

0097B
DEVANAGARI LETTER GGA

0097C
DEVANAGARI LETTER JJA

0097D
DEVANAGARI LETTER GLOTTAL STOP

0097E
DEVANAGARI LETTER DDDA

0097F
DEVANAGARI LETTER BBA

00980
BENGALI ANJI

00981
BENGALI SIGN CANDRABINDU

00982
BENGALI SIGN ANUSVARA

00983
BENGALI SIGN VISARGA

00985
BENGALI LETTER A

00986
BENGALI LETTER AA

00987
BENGALI LETTER I

00988
BENGALI LETTER II

00989
BENGALI LETTER U

0098A
BENGALI LETTER UU

0098B
BENGALI LETTER VOCALIC R

0098C
BENGALI LETTER VOCALIC L

0098F
BENGALI LETTER E

00990
BENGALI LETTER AI

00993
BENGALI LETTER O

00994
BENGALI LETTER AU

00995
BENGALI LETTER KA

00996
BENGALI LETTER KHA

00997
BENGALI LETTER GA

00998
BENGALI LETTER GHA

00999
BENGALI LETTER NGA

0099A
BENGALI LETTER CA

0099B
BENGALI LETTER CHA

0099C
BENGALI LETTER JA

0099D
BENGALI LETTER JHA

0099E
BENGALI LETTER NYA

0099F
BENGALI LETTER TTA

009A0
BENGALI LETTER TTHA

009A1
BENGALI LETTER DDA

009A2
BENGALI LETTER DDHA

009A3
BENGALI LETTER NNA

009A4
BENGALI LETTER TA

009A5
BENGALI LETTER THA

009A6
BENGALI LETTER DA

009A7
BENGALI LETTER DHA

009A8
BENGALI LETTER NA

009AA
BENGALI LETTER PA

009AB
BENGALI LETTER PHA

009AC
BENGALI LETTER BA

009AD
BENGALI LETTER BHA

009AE
BENGALI LETTER MA

009AF
BENGALI LETTER YA

009B0
BENGALI LETTER RA

009B2
BENGALI LETTER LA

009B6
BENGALI LETTER SHA

009B7
BENGALI LETTER SSA

009B8
BENGALI LETTER SA

009B9
BENGALI LETTER HA

009BC
BENGALI SIGN NUKTA

009BD
BENGALI SIGN AVAGRAHA

009BE
BENGALI VOWEL SIGN AA

009BF
BENGALI VOWEL SIGN I

009C0
BENGALI VOWEL SIGN II

009C1
BENGALI VOWEL SIGN U

009C2
BENGALI VOWEL SIGN UU

009C3
BENGALI VOWEL SIGN VOCALIC R

009C4
BENGALI VOWEL SIGN VOCALIC RR

009C7
BENGALI VOWEL SIGN E

009C8
BENGALI VOWEL SIGN AI

009CB
BENGALI VOWEL SIGN O

009CC
BENGALI VOWEL SIGN AU

009CD
BENGALI SIGN VIRAMA

009CE
BENGALI LETTER KHANDA TA

009D7
BENGALI AU LENGTH MARK

009DC
BENGALI LETTER RRA

009DD
BENGALI LETTER RHA

009DF
BENGALI LETTER YYA

009E0
BENGALI LETTER VOCALIC RR

009E1
BENGALI LETTER VOCALIC LL

009E2
BENGALI VOWEL SIGN VOCALIC L

009E3
BENGALI VOWEL SIGN VOCALIC LL

009E6
BENGALI DIGIT ZERO

009E7
BENGALI DIGIT ONE

009E8
BENGALI DIGIT TWO

009E9
BENGALI DIGIT THREE

009EA
BENGALI DIGIT FOUR

009EB
BENGALI DIGIT FIVE

009EC
BENGALI DIGIT SIX

009ED
BENGALI DIGIT SEVEN

009EE
BENGALI DIGIT EIGHT

009EF
BENGALI DIGIT NINE

009F0
BENGALI LETTER RA WITH MIDDLE DIAGONAL

009F1
BENGALI LETTER RA WITH LOWER DIAGONAL

009F2
BENGALI RUPEE MARK

009F3
BENGALI RUPEE SIGN

009F4
BENGALI CURRENCY NUMERATOR ONE

009F5
BENGALI CURRENCY NUMERATOR TWO

009F6
BENGALI CURRENCY NUMERATOR THREE

009F7
BENGALI CURRENCY NUMERATOR FOUR

009F8
BENGALI CURRENCY NUMERATOR ONE LESS THAN THE DENOMINATOR

009F9
BENGALI CURRENCY DENOMINATOR SIXTEEN

009FA
BENGALI ISSHAR

009FB
BENGALI GANDA MARK

009FC
BENGALI LETTER VEDIC ANUSVARA

009FD
BENGALI ABBREVIATION SIGN

009FE
BENGALI SANDHI MARK

00A01
GURMUKHI SIGN ADAK BINDI

00A02
GURMUKHI SIGN BINDI

00A03
GURMUKHI SIGN VISARGA

00A05
GURMUKHI LETTER A

00A06
GURMUKHI LETTER AA

00A07
GURMUKHI LETTER I

00A08
GURMUKHI LETTER II

00A09
GURMUKHI LETTER U

00A0A
GURMUKHI LETTER UU

00A0F
GURMUKHI LETTER EE

00A10
GURMUKHI LETTER AI

00A13
GURMUKHI LETTER OO

00A14
GURMUKHI LETTER AU

00A15
GURMUKHI LETTER KA

00A16
GURMUKHI LETTER KHA

00A17
GURMUKHI LETTER GA

00A18
GURMUKHI LETTER GHA

00A19
GURMUKHI LETTER NGA

00A1A
GURMUKHI LETTER CA

00A1B
GURMUKHI LETTER CHA

00A1C
GURMUKHI LETTER JA

00A1D
GURMUKHI LETTER JHA

00A1E
GURMUKHI LETTER NYA

00A1F
GURMUKHI LETTER TTA

00A20
GURMUKHI LETTER TTHA

00A21
GURMUKHI LETTER DDA

00A22
GURMUKHI LETTER DDHA

00A23
GURMUKHI LETTER NNA

00A24
GURMUKHI LETTER TA

00A25
GURMUKHI LETTER THA

00A26
GURMUKHI LETTER DA

00A27
GURMUKHI LETTER DHA

00A28
GURMUKHI LETTER NA

00A2A
GURMUKHI LETTER PA

00A2B
GURMUKHI LETTER PHA

00A2C
GURMUKHI LETTER BA

00A2D
GURMUKHI LETTER BHA

00A2E
GURMUKHI LETTER MA

00A2F
GURMUKHI LETTER YA

00A30
GURMUKHI LETTER RA

00A32
GURMUKHI LETTER LA

00A33
GURMUKHI LETTER LLA

00A35
GURMUKHI LETTER VA

00A36
GURMUKHI LETTER SHA

00A38
GURMUKHI LETTER SA

00A39
GURMUKHI LETTER HA

00A3C
GURMUKHI SIGN NUKTA

00A3E
GURMUKHI VOWEL SIGN AA

00A3F
GURMUKHI VOWEL SIGN I

00A40
GURMUKHI VOWEL SIGN II

00A41
GURMUKHI VOWEL SIGN U

00A42
GURMUKHI VOWEL SIGN UU

00A47
GURMUKHI VOWEL SIGN EE

00A48
GURMUKHI VOWEL SIGN AI

00A4B
GURMUKHI VOWEL SIGN OO

00A4C
GURMUKHI VOWEL SIGN AU

00A4D
GURMUKHI SIGN VIRAMA

00A51
GURMUKHI SIGN UDAAT

00A59
GURMUKHI LETTER KHHA

00A5A
GURMUKHI LETTER GHHA

00A5B
GURMUKHI LETTER ZA

00A5C
GURMUKHI LETTER RRA

00A5E
GURMUKHI LETTER FA

00A66
GURMUKHI DIGIT ZERO

00A67
GURMUKHI DIGIT ONE

00A68
GURMUKHI DIGIT TWO

00A69
GURMUKHI DIGIT THREE

00A6A
GURMUKHI DIGIT FOUR

00A6B
GURMUKHI DIGIT FIVE

00A6C
GURMUKHI DIGIT SIX

00A6D
GURMUKHI DIGIT SEVEN

00A6E
GURMUKHI DIGIT EIGHT

00A6F
GURMUKHI DIGIT NINE

00A70
GURMUKHI TIPPI

00A71
GURMUKHI ADDAK

00A72
GURMUKHI IRI

00A73
GURMUKHI URA

00A74
GURMUKHI EK ONKAR

00A75
GURMUKHI SIGN YAKASH

00A76
GURMUKHI ABBREVIATION SIGN

00A81
GUJARATI SIGN CANDRABINDU

00A82
GUJARATI SIGN ANUSVARA

00A83
GUJARATI SIGN VISARGA

00A85
GUJARATI LETTER A

00A86
GUJARATI LETTER AA

00A87
GUJARATI LETTER I

00A88
GUJARATI LETTER II

00A89
GUJARATI LETTER U

00A8A
GUJARATI LETTER UU

00A8B
GUJARATI LETTER VOCALIC R

00A8C
GUJARATI LETTER VOCALIC L

00A8D
GUJARATI VOWEL CANDRA E

00A8F
GUJARATI LETTER E

00A90
GUJARATI LETTER AI

00A91
GUJARATI VOWEL CANDRA O

00A93
GUJARATI LETTER O

00A94
GUJARATI LETTER AU

00A95
GUJARATI LETTER KA

00A96
GUJARATI LETTER KHA

00A97
GUJARATI LETTER GA

00A98
GUJARATI LETTER GHA

00A99
GUJARATI LETTER NGA

00A9A
GUJARATI LETTER CA

00A9B
GUJARATI LETTER CHA

00A9C
GUJARATI LETTER JA

00A9D
GUJARATI LETTER JHA

00A9E
GUJARATI LETTER NYA

00A9F
GUJARATI LETTER TTA

00AA0
GUJARATI LETTER TTHA

00AA1
GUJARATI LETTER DDA

00AA2
GUJARATI LETTER DDHA

00AA3
GUJARATI LETTER NNA

00AA4
GUJARATI LETTER TA

00AA5
GUJARATI LETTER THA

00AA6
GUJARATI LETTER DA

00AA7
GUJARATI LETTER DHA

00AA8
GUJARATI LETTER NA

00AAA
GUJARATI LETTER PA

00AAB
GUJARATI LETTER PHA

00AAC
GUJARATI LETTER BA

00AAD
GUJARATI LETTER BHA

00AAE
GUJARATI LETTER MA

00AAF
GUJARATI LETTER YA

00AB0
GUJARATI LETTER RA

00AB2
GUJARATI LETTER LA

00AB3
GUJARATI LETTER LLA

00AB5
GUJARATI LETTER VA

00AB6
GUJARATI LETTER SHA

00AB7
GUJARATI LETTER SSA

00AB8
GUJARATI LETTER SA

00AB9
GUJARATI LETTER HA

00ABC
GUJARATI SIGN NUKTA

00ABD
GUJARATI SIGN AVAGRAHA

00ABE
GUJARATI VOWEL SIGN AA

00ABF
GUJARATI VOWEL SIGN I

00AC0
GUJARATI VOWEL SIGN II

00AC1
GUJARATI VOWEL SIGN U

00AC2
GUJARATI VOWEL SIGN UU

00AC3
GUJARATI VOWEL SIGN VOCALIC R

00AC4
GUJARATI VOWEL SIGN VOCALIC RR

00AC5
GUJARATI VOWEL SIGN CANDRA E

00AC7
GUJARATI VOWEL SIGN E

00AC8
GUJARATI VOWEL SIGN AI

00AC9
GUJARATI VOWEL SIGN CANDRA O

00ACB
GUJARATI VOWEL SIGN O

00ACC
GUJARATI VOWEL SIGN AU

00ACD
GUJARATI SIGN VIRAMA

00AD0
GUJARATI OM

00AE0
GUJARATI LETTER VOCALIC RR

00AE1
GUJARATI LETTER VOCALIC LL

00AE2
GUJARATI VOWEL SIGN VOCALIC L

00AE3
GUJARATI VOWEL SIGN VOCALIC LL

00AE6
GUJARATI DIGIT ZERO

00AE7
GUJARATI DIGIT ONE

00AE8
GUJARATI DIGIT TWO

00AE9
GUJARATI DIGIT THREE

00AEA
GUJARATI DIGIT FOUR

00AEB
GUJARATI DIGIT FIVE

00AEC
GUJARATI DIGIT SIX

00AED
GUJARATI DIGIT SEVEN

00AEE
GUJARATI DIGIT EIGHT

00AEF
GUJARATI DIGIT NINE

00AF0
GUJARATI ABBREVIATION SIGN

00AF1
GUJARATI RUPEE SIGN

00AF9
GUJARATI LETTER ZHA

00AFA
GUJARATI SIGN SUKUN

00AFB
GUJARATI SIGN SHADDA

00AFC
GUJARATI SIGN MADDAH

00AFD
GUJARATI SIGN THREE-DOT NUKTA ABOVE

00AFE
GUJARATI SIGN CIRCLE NUKTA ABOVE

00AFF
GUJARATI SIGN TWO-CIRCLE NUKTA ABOVE

00B01
ORIYA SIGN CANDRABINDU

00B02
ORIYA SIGN ANUSVARA

00B03
ORIYA SIGN VISARGA

00B05
ORIYA LETTER A

00B06
ORIYA LETTER AA

00B07
ORIYA LETTER I

00B08
ORIYA LETTER II

00B09
ORIYA LETTER U

00B0A
ORIYA LETTER UU

00B0B
ORIYA LETTER VOCALIC R

00B0C
ORIYA LETTER VOCALIC L

00B0F
ORIYA LETTER E

00B10
ORIYA LETTER AI

00B13
ORIYA LETTER O

00B14
ORIYA LETTER AU

00B15
ORIYA LETTER KA

00B16
ORIYA LETTER KHA

00B17
ORIYA LETTER GA

00B18
ORIYA LETTER GHA

00B19
ORIYA LETTER NGA

00B1A
ORIYA LETTER CA

00B1B
ORIYA LETTER CHA

00B1C
ORIYA LETTER JA

00B1D
ORIYA LETTER JHA

00B1E
ORIYA LETTER NYA

00B1F
ORIYA LETTER TTA

00B20
ORIYA LETTER TTHA

00B21
ORIYA LETTER DDA

00B22
ORIYA LETTER DDHA

00B23
ORIYA LETTER NNA

00B24
ORIYA LETTER TA

00B25
ORIYA LETTER THA

00B26
ORIYA LETTER DA

00B27
ORIYA LETTER DHA

00B28
ORIYA LETTER NA

00B2A
ORIYA LETTER PA

00B2B
ORIYA LETTER PHA

00B2C
ORIYA LETTER BA

00B2D
ORIYA LETTER BHA

00B2E
ORIYA LETTER MA

00B2F
ORIYA LETTER YA

00B30
ORIYA LETTER RA

00B32
ORIYA LETTER LA

00B33
ORIYA LETTER LLA

00B35
ORIYA LETTER VA

00B36
ORIYA LETTER SHA

00B37
ORIYA LETTER SSA

00B38
ORIYA LETTER SA

00B39
ORIYA LETTER HA

00B3C
ORIYA SIGN NUKTA

00B3D
ORIYA SIGN AVAGRAHA

00B3E
ORIYA VOWEL SIGN AA

00B3F
ORIYA VOWEL SIGN I

00B40
ORIYA VOWEL SIGN II

00B41
ORIYA VOWEL SIGN U

00B42
ORIYA VOWEL SIGN UU

00B43
ORIYA VOWEL SIGN VOCALIC R

00B44
ORIYA VOWEL SIGN VOCALIC RR

00B47
ORIYA VOWEL SIGN E

00B48
ORIYA VOWEL SIGN AI

00B4B
ORIYA VOWEL SIGN O

00B4C
ORIYA VOWEL SIGN AU

00B4D
ORIYA SIGN VIRAMA

00B55
ORIYA SIGN OVERLINE

00B56
ORIYA AI LENGTH MARK

00B57
ORIYA AU LENGTH MARK

00B5C
ORIYA LETTER RRA

00B5D
ORIYA LETTER RHA

00B5F
ORIYA LETTER YYA

00B60
ORIYA LETTER VOCALIC RR

00B61
ORIYA LETTER VOCALIC LL

00B62
ORIYA VOWEL SIGN VOCALIC L

00B63
ORIYA VOWEL SIGN VOCALIC LL

00B66
ORIYA DIGIT ZERO

00B67
ORIYA DIGIT ONE

00B68
ORIYA DIGIT TWO

00B69
ORIYA DIGIT THREE

00B6A
ORIYA DIGIT FOUR

00B6B
ORIYA DIGIT FIVE

00B6C
ORIYA DIGIT SIX

00B6D
ORIYA DIGIT SEVEN

00B6E
ORIYA DIGIT EIGHT

00B6F
ORIYA DIGIT NINE

00B70
ORIYA ISSHAR

00B71
ORIYA LETTER WA

00B72
ORIYA FRACTION ONE QUARTER

00B73
ORIYA FRACTION ONE HALF

00B74
ORIYA FRACTION THREE QUARTERS

00B75
ORIYA FRACTION ONE SIXTEENTH

00B76
ORIYA FRACTION ONE EIGHTH

00B77
ORIYA FRACTION THREE SIXTEENTHS

00B82
TAMIL SIGN ANUSVARA

00B83
TAMIL SIGN VISARGA

00B85
TAMIL LETTER A

00B86
TAMIL LETTER AA

00B87
TAMIL LETTER I

00B88
TAMIL LETTER II

00B89
TAMIL LETTER U

00B8A
TAMIL LETTER UU

00B8E
TAMIL LETTER E

00B8F
TAMIL LETTER EE

00B90
TAMIL LETTER AI

00B92
TAMIL LETTER O

00B93
TAMIL LETTER OO

00B94
TAMIL LETTER AU

00B95
TAMIL LETTER KA

00B99
TAMIL LETTER NGA

00B9A
TAMIL LETTER CA

00B9C
TAMIL LETTER JA

00B9E
TAMIL LETTER NYA

00B9F
TAMIL LETTER TTA

00BA3
TAMIL LETTER NNA

00BA4
TAMIL LETTER TA

00BA8
TAMIL LETTER NA

00BA9
TAMIL LETTER NNNA

00BAA
TAMIL LETTER PA

00BAE
TAMIL LETTER MA

00BAF
TAMIL LETTER YA

00BB0
TAMIL LETTER RA

00BB1
TAMIL LETTER RRA

00BB2
TAMIL LETTER LA

00BB3
TAMIL LETTER LLA

00BB4
TAMIL LETTER LLLA

00BB5
TAMIL LETTER VA

00BB6
TAMIL LETTER SHA

00BB7
TAMIL LETTER SSA

00BB8
TAMIL LETTER SA

00BB9
TAMIL LETTER HA

00BBE
TAMIL VOWEL SIGN AA

00BBF
TAMIL VOWEL SIGN I

00BC0
TAMIL VOWEL SIGN II

00BC1
TAMIL VOWEL SIGN U

00BC2
TAMIL VOWEL SIGN UU

00BC6
TAMIL VOWEL SIGN E

00BC7
TAMIL VOWEL SIGN EE

00BC8
TAMIL VOWEL SIGN AI

00BCA
TAMIL VOWEL SIGN O

00BCB
TAMIL VOWEL SIGN OO

00BCC
TAMIL VOWEL SIGN AU

00BCD
TAMIL SIGN VIRAMA

00BD0
TAMIL OM

00BD7
TAMIL AU LENGTH MARK

00BE6
TAMIL DIGIT ZERO

00BE7
TAMIL DIGIT ONE

00BE8
TAMIL DIGIT TWO

00BE9
TAMIL DIGIT THREE

00BEA
TAMIL DIGIT FOUR

00BEB
TAMIL DIGIT FIVE

00BEC
TAMIL DIGIT SIX

00BED
TAMIL DIGIT SEVEN

00BEE
TAMIL DIGIT EIGHT

00BEF
TAMIL DIGIT NINE

00BF0
TAMIL NUMBER TEN

00BF1
TAMIL NUMBER ONE HUNDRED

00BF2
TAMIL NUMBER ONE THOUSAND

00BF3
TAMIL DAY SIGN

00BF4
TAMIL MONTH SIGN

00BF5
TAMIL YEAR SIGN

00BF6
TAMIL DEBIT SIGN

00BF7
TAMIL CREDIT SIGN

00BF8
TAMIL AS ABOVE SIGN

00BF9
TAMIL RUPEE SIGN

00BFA
TAMIL NUMBER SIGN

00C00
TELUGU SIGN COMBINING CANDRABINDU ABOVE

00C01
TELUGU SIGN CANDRABINDU

00C02
TELUGU SIGN ANUSVARA

00C03
TELUGU SIGN VISARGA

00C04
TELUGU SIGN COMBINING ANUSVARA ABOVE

00C05
TELUGU LETTER A

00C06
TELUGU LETTER AA

00C07
TELUGU LETTER I

00C08
TELUGU LETTER II

00C09
TELUGU LETTER U

00C0A
TELUGU LETTER UU

00C0B
TELUGU LETTER VOCALIC R

00C0C
TELUGU LETTER VOCALIC L

00C0E
TELUGU LETTER E

00C0F
TELUGU LETTER EE

00C10
TELUGU LETTER AI

00C12
TELUGU LETTER O

00C13
TELUGU LETTER OO

00C14
TELUGU LETTER AU

00C15
TELUGU LETTER KA

00C16
TELUGU LETTER KHA

00C17
TELUGU LETTER GA

00C18
TELUGU LETTER GHA

00C19
TELUGU LETTER NGA

00C1A
TELUGU LETTER CA

00C1B
TELUGU LETTER CHA

00C1C
TELUGU LETTER JA

00C1D
TELUGU LETTER JHA

00C1E
TELUGU LETTER NYA

00C1F
TELUGU LETTER TTA

00C20
TELUGU LETTER TTHA

00C21
TELUGU LETTER DDA

00C22
TELUGU LETTER DDHA

00C23
TELUGU LETTER NNA

00C24
TELUGU LETTER TA

00C25
TELUGU LETTER THA

00C26
TELUGU LETTER DA

00C27
TELUGU LETTER DHA

00C28
TELUGU LETTER NA

00C2A
TELUGU LETTER PA

00C2B
TELUGU LETTER PHA

00C2C
TELUGU LETTER BA

00C2D
TELUGU LETTER BHA

00C2E
TELUGU LETTER MA

00C2F
TELUGU LETTER YA

00C30
TELUGU LETTER RA

00C31
TELUGU LETTER RRA

00C32
TELUGU LETTER LA

00C33
TELUGU LETTER LLA

00C34
TELUGU LETTER LLLA

00C35
TELUGU LETTER VA

00C36
TELUGU LETTER SHA

00C37
TELUGU LETTER SSA

00C38
TELUGU LETTER SA

00C39
TELUGU LETTER HA

00C3C
TELUGU SIGN NUKTA

00C3D
TELUGU SIGN AVAGRAHA

00C3E
TELUGU VOWEL SIGN AA

00C3F
TELUGU VOWEL SIGN I

00C40
TELUGU VOWEL SIGN II

00C41
TELUGU VOWEL SIGN U

00C42
TELUGU VOWEL SIGN UU

00C43
TELUGU VOWEL SIGN VOCALIC R

00C44
TELUGU VOWEL SIGN VOCALIC RR

00C46
TELUGU VOWEL SIGN E

00C47
TELUGU VOWEL SIGN EE

00C48
TELUGU VOWEL SIGN AI

00C4A
TELUGU VOWEL SIGN O

00C4B
TELUGU VOWEL SIGN OO

00C4C
TELUGU VOWEL SIGN AU

00C4D
TELUGU SIGN VIRAMA

00C55
TELUGU LENGTH MARK

00C56
TELUGU AI LENGTH MARK

00C58
TELUGU LETTER TSA

00C59
TELUGU LETTER DZA

00C5A
TELUGU LETTER RRRA

00C5C
TELUGU ARCHAIC SHRII

00C5D
TELUGU LETTER NAKAARA POLLU

00C60
TELUGU LETTER VOCALIC RR

00C61
TELUGU LETTER VOCALIC LL

00C62
TELUGU VOWEL SIGN VOCALIC L

00C63
TELUGU VOWEL SIGN VOCALIC LL

00C66
TELUGU DIGIT ZERO

00C67
TELUGU DIGIT ONE

00C68
TELUGU DIGIT TWO

00C69
TELUGU DIGIT THREE

00C6A
TELUGU DIGIT FOUR

00C6B
TELUGU DIGIT FIVE

00C6C
TELUGU DIGIT SIX

00C6D
TELUGU DIGIT SEVEN

00C6E
TELUGU DIGIT EIGHT

00C6F
TELUGU DIGIT NINE

00C77
TELUGU SIGN SIDDHAM

00C78
TELUGU FRACTION DIGIT ZERO FOR ODD POWERS OF FOUR

00C79
TELUGU FRACTION DIGIT ONE FOR ODD POWERS OF FOUR

00C7A
TELUGU FRACTION DIGIT TWO FOR ODD POWERS OF FOUR

00C7B
TELUGU FRACTION DIGIT THREE FOR ODD POWERS OF FOUR

00C7C
TELUGU FRACTION DIGIT ONE FOR EVEN POWERS OF FOUR

00C7D
TELUGU FRACTION DIGIT TWO FOR EVEN POWERS OF FOUR

00C7E
TELUGU FRACTION DIGIT THREE FOR EVEN POWERS OF FOUR

00C7F
TELUGU SIGN TUUMU

00C80
KANNADA SIGN SPACING CANDRABINDU

00C81
KANNADA SIGN CANDRABINDU

00C82
KANNADA SIGN ANUSVARA

00C83
KANNADA SIGN VISARGA

00C84
KANNADA SIGN SIDDHAM

00C85
KANNADA LETTER A

00C86
KANNADA LETTER AA

00C87
KANNADA LETTER I

00C88
KANNADA LETTER II

00C89
KANNADA LETTER U

00C8A
KANNADA LETTER UU

00C8B
KANNADA LETTER VOCALIC R

00C8C
KANNADA LETTER VOCALIC L

00C8E
KANNADA LETTER E

00C8F
KANNADA LETTER EE

00C90
KANNADA LETTER AI

00C92
KANNADA LETTER O

00C93
KANNADA LETTER OO

00C94
KANNADA LETTER AU

00C95
KANNADA LETTER KA

00C96
KANNADA LETTER KHA

00C97
KANNADA LETTER GA

00C98
KANNADA LETTER GHA

00C99
KANNADA LETTER NGA

00C9A
KANNADA LETTER CA

00C9B
KANNADA LETTER CHA

00C9C
KANNADA LETTER JA

00C9D
KANNADA LETTER JHA

00C9E
KANNADA LETTER NYA

00C9F
KANNADA LETTER TTA

00CA0
KANNADA LETTER TTHA

00CA1
KANNADA LETTER DDA

00CA2
KANNADA LETTER DDHA

00CA3
KANNADA LETTER NNA

00CA4
KANNADA LETTER TA

00CA5
KANNADA LETTER THA

00CA6
KANNADA LETTER DA

00CA7
KANNADA LETTER DHA

00CA8
KANNADA LETTER NA

00CAA
KANNADA LETTER PA

00CAB
KANNADA LETTER PHA

00CAC
KANNADA LETTER BA

00CAD
KANNADA LETTER BHA

00CAE
KANNADA LETTER MA

00CAF
KANNADA LETTER YA

00CB0
KANNADA LETTER RA

00CB1
KANNADA LETTER RRA

00CB2
KANNADA LETTER LA

00CB3
KANNADA LETTER LLA

00CB5
KANNADA LETTER VA

00CB6
KANNADA LETTER SHA

00CB7
KANNADA LETTER SSA

00CB8
KANNADA LETTER SA

00CB9
KANNADA LETTER HA

00CBC
KANNADA SIGN NUKTA

00CBD
KANNADA SIGN AVAGRAHA

00CBE
KANNADA VOWEL SIGN AA

00CBF
KANNADA VOWEL SIGN I

00CC0
KANNADA VOWEL SIGN II

00CC1
KANNADA VOWEL SIGN U

00CC2
KANNADA VOWEL SIGN UU

00CC3
KANNADA VOWEL SIGN VOCALIC R

00CC4
KANNADA VOWEL SIGN VOCALIC RR

00CC6
KANNADA VOWEL SIGN E

00CC7
KANNADA VOWEL SIGN EE

00CC8
KANNADA VOWEL SIGN AI

00CCA
KANNADA VOWEL SIGN O

00CCB
KANNADA VOWEL SIGN OO

00CCC
KANNADA VOWEL SIGN AU

00CCD
KANNADA SIGN VIRAMA

00CD5
KANNADA LENGTH MARK

00CD6
KANNADA AI LENGTH MARK

00CDC
KANNADA ARCHAIC SHRII

00CDD
KANNADA LETTER NAKAARA POLLU

00CDE
KANNADA LETTER LLLA

00CDE
KANNADA LETTER FA

00CE0
KANNADA LETTER VOCALIC RR

00CE1
KANNADA LETTER VOCALIC LL

00CE2
KANNADA VOWEL SIGN VOCALIC L

00CE3
KANNADA VOWEL SIGN VOCALIC LL

00CE6
KANNADA DIGIT ZERO

00CE7
KANNADA DIGIT ONE

00CE8
KANNADA DIGIT TWO

00CE9
KANNADA DIGIT THREE

00CEA
KANNADA DIGIT FOUR

00CEB
KANNADA DIGIT FIVE

00CEC
KANNADA DIGIT SIX

00CED
KANNADA DIGIT SEVEN

00CEE
KANNADA DIGIT EIGHT

00CEF
KANNADA DIGIT NINE

00CF1
KANNADA SIGN JIHVAMULIYA

00CF2
KANNADA SIGN UPADHMANIYA

00CF3
KANNADA SIGN COMBINING ANUSVARA ABOVE RIGHT

00D00
MALAYALAM SIGN COMBINING ANUSVARA ABOVE

00D01
MALAYALAM SIGN CANDRABINDU

00D02
MALAYALAM SIGN ANUSVARA

00D03
MALAYALAM SIGN VISARGA

00D04
MALAYALAM LETTER VEDIC ANUSVARA

00D05
MALAYALAM LETTER A

00D06
MALAYALAM LETTER AA

00D07
MALAYALAM LETTER I

00D08
MALAYALAM LETTER II

00D09
MALAYALAM LETTER U

00D0A
MALAYALAM LETTER UU

00D0B
MALAYALAM LETTER VOCALIC R

00D0C
MALAYALAM LETTER VOCALIC L

00D0E
MALAYALAM LETTER E

00D0F
MALAYALAM LETTER EE

00D10
MALAYALAM LETTER AI

00D12
MALAYALAM LETTER O

00D13
MALAYALAM LETTER OO

00D14
MALAYALAM LETTER AU

00D15
MALAYALAM LETTER KA

00D16
MALAYALAM LETTER KHA

00D17
MALAYALAM LETTER GA

00D18
MALAYALAM LETTER GHA

00D19
MALAYALAM LETTER NGA

00D1A
MALAYALAM LETTER CA

00D1B
MALAYALAM LETTER CHA

00D1C
MALAYALAM LETTER JA

00D1D
MALAYALAM LETTER JHA

00D1E
MALAYALAM LETTER NYA

00D1F
MALAYALAM LETTER TTA

00D20
MALAYALAM LETTER TTHA

00D21
MALAYALAM LETTER DDA

00D22
MALAYALAM LETTER DDHA

00D23
MALAYALAM LETTER NNA

00D24
MALAYALAM LETTER TA

00D25
MALAYALAM LETTER THA

00D26
MALAYALAM LETTER DA

00D27
MALAYALAM LETTER DHA

00D28
MALAYALAM LETTER NA

00D29
MALAYALAM LETTER NNNA

00D2A
MALAYALAM LETTER PA

00D2B
MALAYALAM LETTER PHA

00D2C
MALAYALAM LETTER BA

00D2D
MALAYALAM LETTER BHA

00D2E
MALAYALAM LETTER MA

00D2F
MALAYALAM LETTER YA

00D30
MALAYALAM LETTER RA

00D31
MALAYALAM LETTER RRA

00D32
MALAYALAM LETTER LA

00D33
MALAYALAM LETTER LLA

00D34
MALAYALAM LETTER LLLA

00D35
MALAYALAM LETTER VA

00D36
MALAYALAM LETTER SHA

00D37
MALAYALAM LETTER SSA

00D38
MALAYALAM LETTER SA

00D39
MALAYALAM LETTER HA

00D3A
MALAYALAM LETTER TTTA

00D3B
MALAYALAM SIGN VERTICAL BAR VIRAMA

00D3C
MALAYALAM SIGN CIRCULAR VIRAMA

00D3D
MALAYALAM SIGN AVAGRAHA

00D3E
MALAYALAM VOWEL SIGN AA

00D3F
MALAYALAM VOWEL SIGN I

00D40
MALAYALAM VOWEL SIGN II

00D41
MALAYALAM VOWEL SIGN U

00D42
MALAYALAM VOWEL SIGN UU

00D43
MALAYALAM VOWEL SIGN VOCALIC R

00D44
MALAYALAM VOWEL SIGN VOCALIC RR

00D46
MALAYALAM VOWEL SIGN E

00D47
MALAYALAM VOWEL SIGN EE

00D48
MALAYALAM VOWEL SIGN AI

00D4A
MALAYALAM VOWEL SIGN O

00D4B
MALAYALAM VOWEL SIGN OO

00D4C
MALAYALAM VOWEL SIGN AU

00D4D
MALAYALAM SIGN VIRAMA

00D4E
MALAYALAM LETTER DOT REPH

00D4F
MALAYALAM SIGN PARA

00D54
MALAYALAM LETTER CHILLU M

00D55
MALAYALAM LETTER CHILLU Y

00D56
MALAYALAM LETTER CHILLU LLL

00D57
MALAYALAM AU LENGTH MARK

00D58
MALAYALAM FRACTION ONE ONE-HUNDRED-AND-SIXTIETH

00D59
MALAYALAM FRACTION ONE FORTIETH

00D5A
MALAYALAM FRACTION THREE EIGHTIETHS

00D5B
MALAYALAM FRACTION ONE TWENTIETH

00D5C
MALAYALAM FRACTION ONE TENTH

00D5D
MALAYALAM FRACTION THREE TWENTIETHS

00D5E
MALAYALAM FRACTION ONE FIFTH

00D5F
MALAYALAM LETTER ARCHAIC II

00D60
MALAYALAM LETTER VOCALIC RR

00D61
MALAYALAM LETTER VOCALIC LL

00D62
MALAYALAM VOWEL SIGN VOCALIC L

00D63
MALAYALAM VOWEL SIGN VOCALIC LL

00D66
MALAYALAM DIGIT ZERO

00D67
MALAYALAM DIGIT ONE

00D68
MALAYALAM DIGIT TWO

00D69
MALAYALAM DIGIT THREE

00D6A
MALAYALAM DIGIT FOUR

00D6B
MALAYALAM DIGIT FIVE

00D6C
MALAYALAM DIGIT SIX

00D6D
MALAYALAM DIGIT SEVEN

00D6E
MALAYALAM DIGIT EIGHT

00D6F
MALAYALAM DIGIT NINE

00D70
MALAYALAM NUMBER TEN

00D71
MALAYALAM NUMBER ONE HUNDRED

00D72
MALAYALAM NUMBER ONE THOUSAND

00D73
MALAYALAM FRACTION ONE QUARTER

00D74
MALAYALAM FRACTION ONE HALF

00D75
MALAYALAM FRACTION THREE QUARTERS

00D76
MALAYALAM FRACTION ONE SIXTEENTH

00D77
MALAYALAM FRACTION ONE EIGHTH

00D78
MALAYALAM FRACTION THREE SIXTEENTHS

00D79
MALAYALAM DATE MARK

00D7A
MALAYALAM LETTER CHILLU NN

00D7B
MALAYALAM LETTER CHILLU N

00D7C
MALAYALAM LETTER CHILLU RR

00D7D
MALAYALAM LETTER CHILLU L

00D7E
MALAYALAM LETTER CHILLU LL

00D7F
MALAYALAM LETTER CHILLU K

00D81
SINHALA SIGN CANDRABINDU

00D82
SINHALA SIGN ANUSVARAYA

00D83
SINHALA SIGN VISARGAYA

00D85
SINHALA LETTER AYANNA

00D86
SINHALA LETTER AAYANNA

00D87
SINHALA LETTER AEYANNA

00D88
SINHALA LETTER AEEYANNA

00D89
SINHALA LETTER IYANNA

00D8A
SINHALA LETTER IIYANNA

00D8B
SINHALA LETTER UYANNA

00D8C
SINHALA LETTER UUYANNA

00D8D
SINHALA LETTER IRUYANNA

00D8E
SINHALA LETTER IRUUYANNA

00D8F
SINHALA LETTER ILUYANNA

00D90
SINHALA LETTER ILUUYANNA

00D91
SINHALA LETTER EYANNA

00D92
SINHALA LETTER EEYANNA

00D93
SINHALA LETTER AIYANNA

00D94
SINHALA LETTER OYANNA

00D95
SINHALA LETTER OOYANNA

00D96
SINHALA LETTER AUYANNA

00D9A
SINHALA LETTER ALPAPRAANA KAYANNA

00D9B
SINHALA LETTER MAHAAPRAANA KAYANNA

00D9C
SINHALA LETTER ALPAPRAANA GAYANNA

00D9D
SINHALA LETTER MAHAAPRAANA GAYANNA

00D9E
SINHALA LETTER KANTAJA NAASIKYAYA

00D9F
SINHALA LETTER SANYAKA GAYANNA

00DA0
SINHALA LETTER ALPAPRAANA CAYANNA

00DA1
SINHALA LETTER MAHAAPRAANA CAYANNA

00DA2
SINHALA LETTER ALPAPRAANA JAYANNA

00DA3
SINHALA LETTER MAHAAPRAANA JAYANNA

00DA4
SINHALA LETTER TAALUJA NAASIKYAYA

00DA5
SINHALA LETTER TAALUJA SANYOOGA NAAKSIKYAYA

00DA6
SINHALA LETTER SANYAKA JAYANNA

00DA7
SINHALA LETTER ALPAPRAANA TTAYANNA

00DA8
SINHALA LETTER MAHAAPRAANA TTAYANNA

00DA9
SINHALA LETTER ALPAPRAANA DDAYANNA

00DAA
SINHALA LETTER MAHAAPRAANA DDAYANNA

00DAB
SINHALA LETTER MUURDHAJA NAYANNA

00DAC
SINHALA LETTER SANYAKA DDAYANNA

00DAD
SINHALA LETTER ALPAPRAANA TAYANNA

00DAE
SINHALA LETTER MAHAAPRAANA TAYANNA

00DAF
SINHALA LETTER ALPAPRAANA DAYANNA

00DB0
SINHALA LETTER MAHAAPRAANA DAYANNA

00DB1
SINHALA LETTER DANTAJA NAYANNA

00DB3
SINHALA LETTER SANYAKA DAYANNA

00DB4
SINHALA LETTER ALPAPRAANA PAYANNA

00DB5
SINHALA LETTER MAHAAPRAANA PAYANNA

00DB6
SINHALA LETTER ALPAPRAANA BAYANNA

00DB7
SINHALA LETTER MAHAAPRAANA BAYANNA

00DB8
SINHALA LETTER MAYANNA

00DB9
SINHALA LETTER AMBA BAYANNA

00DBA
SINHALA LETTER YAYANNA

00DBB
SINHALA LETTER RAYANNA

00DBD
SINHALA LETTER DANTAJA LAYANNA

00DC0
SINHALA LETTER VAYANNA

00DC1
SINHALA LETTER TAALUJA SAYANNA

00DC2
SINHALA LETTER MUURDHAJA SAYANNA

00DC3
SINHALA LETTER DANTAJA SAYANNA

00DC4
SINHALA LETTER HAYANNA

00DC5
SINHALA LETTER MUURDHAJA LAYANNA

00DC6
SINHALA LETTER FAYANNA

00DCA
SINHALA SIGN AL-LAKUNA

00DCF
SINHALA VOWEL SIGN AELA-PILLA

00DD0
SINHALA VOWEL SIGN KETTI AEDA-PILLA

00DD1
SINHALA VOWEL SIGN DIGA AEDA-PILLA

00DD2
SINHALA VOWEL SIGN KETTI IS-PILLA

00DD3
SINHALA VOWEL SIGN DIGA IS-PILLA

00DD4
SINHALA VOWEL SIGN KETTI PAA-PILLA

00DD6
SINHALA VOWEL SIGN DIGA PAA-PILLA

00DD8
SINHALA VOWEL SIGN GAETTA-PILLA

00DD9
SINHALA VOWEL SIGN KOMBUVA

00DDA
SINHALA VOWEL SIGN DIGA KOMBUVA

00DDB
SINHALA VOWEL SIGN KOMBU DEKA

00DDC
SINHALA VOWEL SIGN KOMBUVA HAA AELA-PILLA

00DDD
SINHALA VOWEL SIGN KOMBUVA HAA DIGA AELA-PILLA

00DDE
SINHALA VOWEL SIGN KOMBUVA HAA GAYANUKITTA

00DDF
SINHALA VOWEL SIGN GAYANUKITTA

00DE6
SINHALA LITH DIGIT ZERO

00DE7
SINHALA LITH DIGIT ONE

00DE8
SINHALA LITH DIGIT TWO

00DE9
SINHALA LITH DIGIT THREE

00DEA
SINHALA LITH DIGIT FOUR

00DEB
SINHALA LITH DIGIT FIVE

00DEC
SINHALA LITH DIGIT SIX

00DED
SINHALA LITH DIGIT SEVEN

00DEE
SINHALA LITH DIGIT EIGHT

00DEF
SINHALA LITH DIGIT NINE

00DF2
SINHALA VOWEL SIGN DIGA GAETTA-PILLA

00DF3
SINHALA VOWEL SIGN DIGA GAYANUKITTA

00DF4
SINHALA PUNCTUATION KUNDDALIYA

00E01
THAI CHARACTER KO KAI

00E02
THAI CHARACTER KHO KHAI

00E03
THAI CHARACTER KHO KHUAT

00E04
THAI CHARACTER KHO KHWAI

00E05
THAI CHARACTER KHO KHON

00E06
THAI CHARACTER KHO RAKHANG

00E07
THAI CHARACTER NGO NGU

00E08
THAI CHARACTER CHO CHAN

00E09
THAI CHARACTER CHO CHING

00E0A
THAI CHARACTER CHO CHANG

00E0B
THAI CHARACTER SO SO

00E0C
THAI CHARACTER CHO CHOE

00E0D
THAI CHARACTER YO YING

00E0E
THAI CHARACTER DO CHADA

00E0F
THAI CHARACTER TO PATAK

00E10
THAI CHARACTER THO THAN

00E11
THAI CHARACTER THO NANGMONTHO

00E12
THAI CHARACTER THO PHUTHAO

00E13
THAI CHARACTER NO NEN

00E14
THAI CHARACTER DO DEK

00E15
THAI CHARACTER TO TAO

00E16
THAI CHARACTER THO THUNG

00E17
THAI CHARACTER THO THAHAN

00E18
THAI CHARACTER THO THONG

00E19
THAI CHARACTER NO NU

00E1A
THAI CHARACTER BO BAIMAI

00E1B
THAI CHARACTER PO PLA

00E1C
THAI CHARACTER PHO PHUNG

00E1D
THAI CHARACTER FO FA

00E1E
THAI CHARACTER PHO PHAN

00E1F
THAI CHARACTER FO FAN

00E20
THAI CHARACTER PHO SAMPHAO

00E21
THAI CHARACTER MO MA

00E22
THAI CHARACTER YO YAK

00E23
THAI CHARACTER RO RUA

00E24
THAI CHARACTER RU

00E25
THAI CHARACTER LO LING

00E26
THAI CHARACTER LU

00E27
THAI CHARACTER WO WAEN

00E28
THAI CHARACTER SO SALA

00E29
THAI CHARACTER SO RUSI

00E2A
THAI CHARACTER SO SUA

00E2B
THAI CHARACTER HO HIP

00E2C
THAI CHARACTER LO CHULA

00E2D
THAI CHARACTER O ANG

00E2E
THAI CHARACTER HO NOKHUK

00E2F
THAI CHARACTER PAIYANNOI

00E30
THAI CHARACTER SARA A

00E31
THAI CHARACTER MAI HAN-AKAT

00E32
THAI CHARACTER SARA AA

00E33
THAI CHARACTER SARA AM

00E34
THAI CHARACTER SARA I

00E35
THAI CHARACTER SARA II

00E36
THAI CHARACTER SARA UE

00E37
THAI CHARACTER SARA UEE

00E38
THAI CHARACTER SARA U

00E39
THAI CHARACTER SARA UU

00E3A
THAI CHARACTER PHINTHU

00E3F
THAI CURRENCY SYMBOL BAHT

00E40
THAI CHARACTER SARA E

00E41
THAI CHARACTER SARA AE

00E42
THAI CHARACTER SARA O

00E43
THAI CHARACTER SARA AI MAIMUAN

00E44
THAI CHARACTER SARA AI MAIMALAI

00E45
THAI CHARACTER LAKKHANGYAO

00E46
THAI CHARACTER MAIYAMOK

00E47
THAI CHARACTER MAITAIKHU

00E48
THAI CHARACTER MAI EK

00E49
THAI CHARACTER MAI THO

00E4A
THAI CHARACTER MAI TRI

00E4B
THAI CHARACTER MAI CHATTAWA

00E4C
THAI CHARACTER THANTHAKHAT

00E4D
THAI CHARACTER NIKHAHIT

00E4E
THAI CHARACTER YAMAKKAN

00E4F
THAI CHARACTER FONGMAN

00E50
THAI DIGIT ZERO

00E51
THAI DIGIT ONE

00E52
THAI DIGIT TWO

00E53
THAI DIGIT THREE

00E54
THAI DIGIT FOUR

00E55
THAI DIGIT FIVE

00E56
THAI DIGIT SIX

00E57
THAI DIGIT SEVEN

00E58
THAI DIGIT EIGHT

00E59
THAI DIGIT NINE

00E5A
THAI CHARACTER ANGKHANKHU

00E5B
THAI CHARACTER KHOMUT

00E81
LAO LETTER KO

00E82
LAO LETTER KHO SUNG

00E84
LAO LETTER KHO TAM

00E86
LAO LETTER PALI GHA

00E87
LAO LETTER NGO

00E88
LAO LETTER CO

00E89
LAO LETTER PALI CHA

00E8A
LAO LETTER SO TAM

00E8C
LAO LETTER PALI JHA

00E8D
LAO LETTER NYO

00E8E
LAO LETTER PALI NYA

00E8F
LAO LETTER PALI TTA

00E90
LAO LETTER PALI TTHA

00E91
LAO LETTER PALI DDA

00E92
LAO LETTER PALI DDHA

00E93
LAO LETTER PALI NNA

00E94
LAO LETTER DO

00E95
LAO LETTER TO

00E96
LAO LETTER THO SUNG

00E97
LAO LETTER THO TAM

00E98
LAO LETTER PALI DHA

00E99
LAO LETTER NO

00E9A
LAO LETTER BO

00E9B
LAO LETTER PO

00E9C
LAO LETTER PHO SUNG

00E9D
LAO LETTER FO FON

00E9D
LAO LETTER FO TAM

00E9E
LAO LETTER PHO TAM

00E9F
LAO LETTER FO FAY

00E9F
LAO LETTER FO SUNG

00EA0
LAO LETTER PALI BHA

00EA1
LAO LETTER MO

00EA2
LAO LETTER YO

00EA3
LAO LETTER RO

00EA3
LAO LETTER LO LING

00EA5
LAO LETTER LO

00EA5
LAO LETTER LO LOOT

00EA7
LAO LETTER WO

00EA8
LAO LETTER SANSKRIT SHA

00EA9
LAO LETTER SANSKRIT SSA

00EAA
LAO LETTER SO SUNG

00EAB
LAO LETTER HO SUNG

00EAC
LAO LETTER PALI LLA

00EAD
LAO LETTER O

00EAE
LAO LETTER HO TAM

00EAF
LAO ELLIPSIS

00EB0
LAO VOWEL SIGN A

00EB1
LAO VOWEL SIGN MAI KAN

00EB2
LAO VOWEL SIGN AA

00EB3
LAO VOWEL SIGN AM

00EB4
LAO VOWEL SIGN I

00EB5
LAO VOWEL SIGN II

00EB6
LAO VOWEL SIGN Y

00EB7
LAO VOWEL SIGN YY

00EB8
LAO VOWEL SIGN U

00EB9
LAO VOWEL SIGN UU

00EBA
LAO SIGN PALI VIRAMA

00EBB
LAO VOWEL SIGN MAI KON

00EBC
LAO SEMIVOWEL SIGN LO

00EBD
LAO SEMIVOWEL SIGN NYO

00EC0
LAO VOWEL SIGN E

00EC1
LAO VOWEL SIGN EI

00EC2
LAO VOWEL SIGN O

00EC3
LAO VOWEL SIGN AY

00EC4
LAO VOWEL SIGN AI

00EC6
LAO KO LA

00EC8
LAO TONE MAI EK

00EC9
LAO TONE MAI THO

00ECA
LAO TONE MAI TI

00ECB
LAO TONE MAI CATAWA

00ECC
LAO CANCELLATION MARK

00ECD
LAO NIGGAHITA

00ECE
LAO YAMAKKAN

00ED0
LAO DIGIT ZERO

00ED1
LAO DIGIT ONE

00ED2
LAO DIGIT TWO

00ED3
LAO DIGIT THREE

00ED4
LAO DIGIT FOUR

00ED5
LAO DIGIT FIVE

00ED6
LAO DIGIT SIX

00ED7
LAO DIGIT SEVEN

00ED8
LAO DIGIT EIGHT

00ED9
LAO DIGIT NINE

00EDC
LAO HO NO

00EDD
LAO HO MO

00EDE
LAO LETTER KHMU GO

00EDF
LAO LETTER KHMU NYO

00F00
TIBETAN SYLLABLE OM

00F01
TIBETAN MARK GTER YIG MGO TRUNCATED A

00F02
TIBETAN MARK GTER YIG MGO -UM RNAM BCAD MA

00F03
TIBETAN MARK GTER YIG MGO -UM GTER TSHEG MA

00F04
TIBETAN MARK INITIAL YIG MGO MDUN MA

00F05
TIBETAN MARK CLOSING YIG MGO SGAB MA

00F06
TIBETAN MARK CARET YIG MGO PHUR SHAD MA

00F07
TIBETAN MARK YIG MGO TSHEG SHAD MA

00F08
TIBETAN MARK SBRUL SHAD

00F09
TIBETAN MARK BSKUR YIG MGO

00F0A
TIBETAN MARK BKA- SHOG YIG MGO

00F0B
TIBETAN MARK INTERSYLLABIC TSHEG

00F0C
TIBETAN MARK DELIMITER TSHEG BSTAR

00F0D
TIBETAN MARK SHAD

00F0E
TIBETAN MARK NYIS SHAD

00F0F
TIBETAN MARK TSHEG SHAD

00F10
TIBETAN MARK NYIS TSHEG SHAD

00F11
TIBETAN MARK RIN CHEN SPUNGS SHAD

00F12
TIBETAN MARK RGYA GRAM SHAD

00F13
TIBETAN MARK CARET -DZUD RTAGS ME LONG CAN

00F14
TIBETAN MARK GTER TSHEG

00F15
TIBETAN LOGOTYPE SIGN CHAD RTAGS

00F16
TIBETAN LOGOTYPE SIGN LHAG RTAGS

00F17
TIBETAN ASTROLOGICAL SIGN SGRA GCAN -CHAR RTAGS

00F18
TIBETAN ASTROLOGICAL SIGN -KHYUD PA

00F19
TIBETAN ASTROLOGICAL SIGN SDONG TSHUGS

00F1A
TIBETAN SIGN RDEL DKAR GCIG

00F1B
TIBETAN SIGN RDEL DKAR GNYIS

00F1C
TIBETAN SIGN RDEL DKAR GSUM

00F1D
TIBETAN SIGN RDEL NAG GCIG

00F1E
TIBETAN SIGN RDEL NAG GNYIS

00F1F
TIBETAN SIGN RDEL DKAR RDEL NAG

00F20
TIBETAN DIGIT ZERO

00F21
TIBETAN DIGIT ONE

00F22
TIBETAN DIGIT TWO

00F23
TIBETAN DIGIT THREE

00F24
TIBETAN DIGIT FOUR

00F25
TIBETAN DIGIT FIVE

00F26
TIBETAN DIGIT SIX

00F27
TIBETAN DIGIT SEVEN

00F28
TIBETAN DIGIT EIGHT

00F29
TIBETAN DIGIT NINE

00F2A
TIBETAN DIGIT HALF ONE

00F2B
TIBETAN DIGIT HALF TWO

00F2C
TIBETAN DIGIT HALF THREE

00F2D
TIBETAN DIGIT HALF FOUR

00F2E
TIBETAN DIGIT HALF FIVE

00F2F
TIBETAN DIGIT HALF SIX

00F30
TIBETAN DIGIT HALF SEVEN

00F31
TIBETAN DIGIT HALF EIGHT

00F32
TIBETAN DIGIT HALF NINE

00F33
TIBETAN DIGIT HALF ZERO

00F34
TIBETAN MARK BSDUS RTAGS

00F35
TIBETAN MARK NGAS BZUNG NYI ZLA

00F36
TIBETAN MARK CARET -DZUD RTAGS BZHI MIG CAN

00F37
TIBETAN MARK NGAS BZUNG SGOR RTAGS

00F38
TIBETAN MARK CHE MGO

00F39
TIBETAN MARK TSA -PHRU

00F3A
TIBETAN MARK GUG RTAGS GYON

00F3B
TIBETAN MARK GUG RTAGS GYAS

00F3C
TIBETAN MARK ANG KHANG GYON

00F3D
TIBETAN MARK ANG KHANG GYAS

00F3E
TIBETAN SIGN YAR TSHES

00F3F
TIBETAN SIGN MAR TSHES

00F40
TIBETAN LETTER KA

00F41
TIBETAN LETTER KHA

00F42
TIBETAN LETTER GA

00F43
TIBETAN LETTER GHA

00F44
TIBETAN LETTER NGA

00F45
TIBETAN LETTER CA

00F46
TIBETAN LETTER CHA

00F47
TIBETAN LETTER JA

00F49
TIBETAN LETTER NYA

00F4A
TIBETAN LETTER TTA

00F4B
TIBETAN LETTER TTHA

00F4C
TIBETAN LETTER DDA

00F4D
TIBETAN LETTER DDHA

00F4E
TIBETAN LETTER NNA

00F4F
TIBETAN LETTER TA

00F50
TIBETAN LETTER THA

00F51
TIBETAN LETTER DA

00F52
TIBETAN LETTER DHA

00F53
TIBETAN LETTER NA

00F54
TIBETAN LETTER PA

00F55
TIBETAN LETTER PHA

00F56
TIBETAN LETTER BA

00F57
TIBETAN LETTER BHA

00F58
TIBETAN LETTER MA

00F59
TIBETAN LETTER TSA

00F5A
TIBETAN LETTER TSHA

00F5B
TIBETAN LETTER DZA

00F5C
TIBETAN LETTER DZHA

00F5D
TIBETAN LETTER WA

00F5E
TIBETAN LETTER ZHA

00F5F
TIBETAN LETTER ZA

00F60
TIBETAN LETTER -A

00F61
TIBETAN LETTER YA

00F62
TIBETAN LETTER RA

00F63
TIBETAN LETTER LA

00F64
TIBETAN LETTER SHA

00F65
TIBETAN LETTER SSA

00F66
TIBETAN LETTER SA

00F67
TIBETAN LETTER HA

00F68
TIBETAN LETTER A

00F69
TIBETAN LETTER KSSA

00F6A
TIBETAN LETTER FIXED-FORM RA

00F6B
TIBETAN LETTER KKA

00F6C
TIBETAN LETTER RRA

00F71
TIBETAN VOWEL SIGN AA

00F72
TIBETAN VOWEL SIGN I

00F73
TIBETAN VOWEL SIGN II

00F74
TIBETAN VOWEL SIGN U

00F75
TIBETAN VOWEL SIGN UU

00F76
TIBETAN VOWEL SIGN VOCALIC R

00F77
TIBETAN VOWEL SIGN VOCALIC RR

00F78
TIBETAN VOWEL SIGN VOCALIC L

00F79
TIBETAN VOWEL SIGN VOCALIC LL

00F7A
TIBETAN VOWEL SIGN E

00F7B
TIBETAN VOWEL SIGN EE

00F7C
TIBETAN VOWEL SIGN O

00F7D
TIBETAN VOWEL SIGN OO

00F7E
TIBETAN SIGN RJES SU NGA RO

00F7F
TIBETAN SIGN RNAM BCAD

00F80
TIBETAN VOWEL SIGN REVERSED I

00F81
TIBETAN VOWEL SIGN REVERSED II

00F82
TIBETAN SIGN NYI ZLA NAA DA

00F83
TIBETAN SIGN SNA LDAN

00F84
TIBETAN MARK HALANTA

00F85
TIBETAN MARK PALUTA

00F86
TIBETAN SIGN LCI RTAGS

00F87
TIBETAN SIGN YANG RTAGS

00F88
TIBETAN SIGN LCE TSA CAN

00F89
TIBETAN SIGN MCHU CAN

00F8A
TIBETAN SIGN GRU CAN RGYINGS

00F8B
TIBETAN SIGN GRU MED RGYINGS

00F8C
TIBETAN SIGN INVERTED MCHU CAN

00F8D
TIBETAN SUBJOINED SIGN LCE TSA CAN

00F8E
TIBETAN SUBJOINED SIGN MCHU CAN

00F8F
TIBETAN SUBJOINED SIGN INVERTED MCHU CAN

00F90
TIBETAN SUBJOINED LETTER KA

00F91
TIBETAN SUBJOINED LETTER KHA

00F92
TIBETAN SUBJOINED LETTER GA

00F93
TIBETAN SUBJOINED LETTER GHA

00F94
TIBETAN SUBJOINED LETTER NGA

00F95
TIBETAN SUBJOINED LETTER CA

00F96
TIBETAN SUBJOINED LETTER CHA

00F97
TIBETAN SUBJOINED LETTER JA

00F99
TIBETAN SUBJOINED LETTER NYA

00F9A
TIBETAN SUBJOINED LETTER TTA

00F9B
TIBETAN SUBJOINED LETTER TTHA

00F9C
TIBETAN SUBJOINED LETTER DDA

00F9D
TIBETAN SUBJOINED LETTER DDHA

00F9E
TIBETAN SUBJOINED LETTER NNA

00F9F
TIBETAN SUBJOINED LETTER TA

00FA0
TIBETAN SUBJOINED LETTER THA

00FA1
TIBETAN SUBJOINED LETTER DA

00FA2
TIBETAN SUBJOINED LETTER DHA

00FA3
TIBETAN SUBJOINED LETTER NA

00FA4
TIBETAN SUBJOINED LETTER PA

00FA5
TIBETAN SUBJOINED LETTER PHA

00FA6
TIBETAN SUBJOINED LETTER BA

00FA7
TIBETAN SUBJOINED LETTER BHA

00FA8
TIBETAN SUBJOINED LETTER MA

00FA9
TIBETAN SUBJOINED LETTER TSA

00FAA
TIBETAN SUBJOINED LETTER TSHA

00FAB
TIBETAN SUBJOINED LETTER DZA

00FAC
TIBETAN SUBJOINED LETTER DZHA

00FAD
TIBETAN SUBJOINED LETTER WA

00FAE
TIBETAN SUBJOINED LETTER ZHA

00FAF
TIBETAN SUBJOINED LETTER ZA

00FB0
TIBETAN SUBJOINED LETTER -A

00FB1
TIBETAN SUBJOINED LETTER YA

00FB2
TIBETAN SUBJOINED LETTER RA

00FB3
TIBETAN SUBJOINED LETTER LA

00FB4
TIBETAN SUBJOINED LETTER SHA

00FB5
TIBETAN SUBJOINED LETTER SSA

00FB6
TIBETAN SUBJOINED LETTER SA

00FB7
TIBETAN SUBJOINED LETTER HA

00FB8
TIBETAN SUBJOINED LETTER A

00FB9
TIBETAN SUBJOINED LETTER KSSA

00FBA
TIBETAN SUBJOINED LETTER FIXED-FORM WA

00FBB
TIBETAN SUBJOINED LETTER FIXED-FORM YA

00FBC
TIBETAN SUBJOINED LETTER FIXED-FORM RA

00FBE
TIBETAN KU RU KHA

00FBF
TIBETAN KU RU KHA BZHI MIG CAN

00FC0
TIBETAN CANTILLATION SIGN HEAVY BEAT

00FC1
TIBETAN CANTILLATION SIGN LIGHT BEAT

00FC2
TIBETAN CANTILLATION SIGN CANG TE-U

00FC3
TIBETAN CANTILLATION SIGN SBUB -CHAL

00FC4
TIBETAN SYMBOL DRIL BU

00FC5
TIBETAN SYMBOL RDO RJE

00FC6
TIBETAN SYMBOL PADMA GDAN

00FC7
TIBETAN SYMBOL RDO RJE RGYA GRAM

00FC8
TIBETAN SYMBOL PHUR PA

00FC9
TIBETAN SYMBOL NOR BU

00FCA
TIBETAN SYMBOL NOR BU NYIS -KHYIL

00FCB
TIBETAN SYMBOL NOR BU GSUM -KHYIL

00FCC
TIBETAN SYMBOL NOR BU BZHI -KHYIL

00FCE
TIBETAN SIGN RDEL NAG RDEL DKAR

00FCF
TIBETAN SIGN RDEL NAG GSUM

00FD0
TIBETAN MARK BKA- SHOG GI MGO RGYAN

00FD0
TIBETAN MARK BSKA- SHOG GI MGO RGYAN

00FD1
TIBETAN MARK MNYAM YIG GI MGO RGYAN

00FD2
TIBETAN MARK NYIS TSHEG

00FD3
TIBETAN MARK INITIAL BRDA RNYING YIG MGO MDUN MA

00FD4
TIBETAN MARK CLOSING BRDA RNYING YIG MGO SGAB MA

00FD5
RIGHT-FACING SVASTI SIGN

00FD6
LEFT-FACING SVASTI SIGN

00FD7
RIGHT-FACING SVASTI SIGN WITH DOTS

00FD8
LEFT-FACING SVASTI SIGN WITH DOTS

00FD9
TIBETAN MARK LEADING MCHAN RTAGS

00FDA
TIBETAN MARK TRAILING MCHAN RTAGS

01000
MYANMAR LETTER KA

01001
MYANMAR LETTER KHA

01002
MYANMAR LETTER GA

01003
MYANMAR LETTER GHA

01004
MYANMAR LETTER NGA

01005
MYANMAR LETTER CA

01006
MYANMAR LETTER CHA

01007
MYANMAR LETTER JA

01008
MYANMAR LETTER JHA

01009
MYANMAR LETTER NYA

0100A
MYANMAR LETTER NNYA

0100B
MYANMAR LETTER TTA

0100C
MYANMAR LETTER TTHA

0100D
MYANMAR LETTER DDA

0100E
MYANMAR LETTER DDHA

0100F
MYANMAR LETTER NNA

01010
MYANMAR LETTER TA

01011
MYANMAR LETTER THA

01012
MYANMAR LETTER DA

01013
MYANMAR LETTER DHA

01014
MYANMAR LETTER NA

01015
MYANMAR LETTER PA

01016
MYANMAR LETTER PHA

01017
MYANMAR LETTER BA

01018
MYANMAR LETTER BHA

01019
MYANMAR LETTER MA

0101A
MYANMAR LETTER YA

0101B
MYANMAR LETTER RA

0101C
MYANMAR LETTER LA

0101D
MYANMAR LETTER WA

0101E
MYANMAR LETTER SA

0101F
MYANMAR LETTER HA

01020
MYANMAR LETTER LLA

01021
MYANMAR LETTER A

01022
MYANMAR LETTER SHAN A

01023
MYANMAR LETTER I

01024
MYANMAR LETTER II

01025
MYANMAR LETTER U

01026
MYANMAR LETTER UU

01027
MYANMAR LETTER E

01028
MYANMAR LETTER MON E

01029
MYANMAR LETTER O

0102A
MYANMAR LETTER AU

0102B
MYANMAR VOWEL SIGN TALL AA

0102C
MYANMAR VOWEL SIGN AA

0102D
MYANMAR VOWEL SIGN I

0102E
MYANMAR VOWEL SIGN II

0102F
MYANMAR VOWEL SIGN U

01030
MYANMAR VOWEL SIGN UU

01031
MYANMAR VOWEL SIGN E

01032
MYANMAR VOWEL SIGN AI

01033
MYANMAR VOWEL SIGN MON II

01034
MYANMAR VOWEL SIGN MON O

01035
MYANMAR VOWEL SIGN E ABOVE

01036
MYANMAR SIGN ANUSVARA

01037
MYANMAR SIGN DOT BELOW

01038
MYANMAR SIGN VISARGA

01039
MYANMAR SIGN VIRAMA

0103A
MYANMAR SIGN ASAT

0103B
MYANMAR CONSONANT SIGN MEDIAL YA

0103C
MYANMAR CONSONANT SIGN MEDIAL RA

0103D
MYANMAR CONSONANT SIGN MEDIAL WA

0103E
MYANMAR CONSONANT SIGN MEDIAL HA

0103F
MYANMAR LETTER GREAT SA

01040
MYANMAR DIGIT ZERO

01041
MYANMAR DIGIT ONE

01042
MYANMAR DIGIT TWO

01043
MYANMAR DIGIT THREE

01044
MYANMAR DIGIT FOUR

01045
MYANMAR DIGIT FIVE

01046
MYANMAR DIGIT SIX

01047
MYANMAR DIGIT SEVEN

01048
MYANMAR DIGIT EIGHT

01049
MYANMAR DIGIT NINE

0104A
MYANMAR SIGN LITTLE SECTION

0104B
MYANMAR SIGN SECTION

0104C
MYANMAR SYMBOL LOCATIVE

0104D
MYANMAR SYMBOL COMPLETED

0104E
MYANMAR SYMBOL AFOREMENTIONED

0104F
MYANMAR SYMBOL GENITIVE

01050
MYANMAR LETTER SHA

01051
MYANMAR LETTER SSA

01052
MYANMAR LETTER VOCALIC R

01053
MYANMAR LETTER VOCALIC RR

01054
MYANMAR LETTER VOCALIC L

01055
MYANMAR LETTER VOCALIC LL

01056
MYANMAR VOWEL SIGN VOCALIC R

01057
MYANMAR VOWEL SIGN VOCALIC RR

01058
MYANMAR VOWEL SIGN VOCALIC L

01059
MYANMAR VOWEL SIGN VOCALIC LL

0105A
MYANMAR LETTER MON NGA

0105B
MYANMAR LETTER MON JHA

0105C
MYANMAR LETTER MON BBA

0105D
MYANMAR LETTER MON BBE

0105E
MYANMAR CONSONANT SIGN MON MEDIAL NA

0105F
MYANMAR CONSONANT SIGN MON MEDIAL MA

01060
MYANMAR CONSONANT SIGN MON MEDIAL LA

01061
MYANMAR LETTER SGAW KAREN SHA

01062
MYANMAR VOWEL SIGN SGAW KAREN EU

01063
MYANMAR TONE MARK SGAW KAREN HATHI

01064
MYANMAR TONE MARK SGAW KAREN KE PHO

01065
MYANMAR LETTER WESTERN PWO KAREN THA

01066
MYANMAR LETTER WESTERN PWO KAREN PWA

01067
MYANMAR VOWEL SIGN WESTERN PWO KAREN EU

01068
MYANMAR VOWEL SIGN WESTERN PWO KAREN UE

01069
MYANMAR SIGN WESTERN PWO KAREN TONE-1

0106A
MYANMAR SIGN WESTERN PWO KAREN TONE-2

0106B
MYANMAR SIGN WESTERN PWO KAREN TONE-3

0106C
MYANMAR SIGN WESTERN PWO KAREN TONE-4

0106D
MYANMAR SIGN WESTERN PWO KAREN TONE-5

0106E
MYANMAR LETTER EASTERN PWO KAREN NNA

0106F
MYANMAR LETTER EASTERN PWO KAREN YWA

01070
MYANMAR LETTER EASTERN PWO KAREN GHWA

01071
MYANMAR VOWEL SIGN GEBA KAREN I

01072
MYANMAR VOWEL SIGN KAYAH OE

01073
MYANMAR VOWEL SIGN KAYAH U

01074
MYANMAR VOWEL SIGN KAYAH EE

01075
MYANMAR LETTER SHAN KA

01076
MYANMAR LETTER SHAN KHA

01077
MYANMAR LETTER SHAN GA

01078
MYANMAR LETTER SHAN CA

01079
MYANMAR LETTER SHAN ZA

0107A
MYANMAR LETTER SHAN NYA

0107B
MYANMAR LETTER SHAN DA

0107C
MYANMAR LETTER SHAN NA

0107D
MYANMAR LETTER SHAN PHA

0107E
MYANMAR LETTER SHAN FA

0107F
MYANMAR LETTER SHAN BA

01080
MYANMAR LETTER SHAN THA

01081
MYANMAR LETTER SHAN HA

01082
MYANMAR CONSONANT SIGN SHAN MEDIAL WA

01083
MYANMAR VOWEL SIGN SHAN AA

01084
MYANMAR VOWEL SIGN SHAN E

01085
MYANMAR VOWEL SIGN SHAN E ABOVE

01086
MYANMAR VOWEL SIGN SHAN FINAL Y

01087
MYANMAR SIGN SHAN TONE-2

01088
MYANMAR SIGN SHAN TONE-3

01089
MYANMAR SIGN SHAN TONE-5

0108A
MYANMAR SIGN SHAN TONE-6

0108B
MYANMAR SIGN SHAN COUNCIL TONE-2

0108C
MYANMAR SIGN SHAN COUNCIL TONE-3

0108D
MYANMAR SIGN SHAN COUNCIL EMPHATIC TONE

0108E
MYANMAR LETTER RUMAI PALAUNG FA

0108F
MYANMAR SIGN RUMAI PALAUNG TONE-5

01090
MYANMAR SHAN DIGIT ZERO

01091
MYANMAR SHAN DIGIT ONE

01092
MYANMAR SHAN DIGIT TWO

01093
MYANMAR SHAN DIGIT THREE

01094
MYANMAR SHAN DIGIT FOUR

01095
MYANMAR SHAN DIGIT FIVE

01096
MYANMAR SHAN DIGIT SIX

01097
MYANMAR SHAN DIGIT SEVEN

01098
MYANMAR SHAN DIGIT EIGHT

01099
MYANMAR SHAN DIGIT NINE

0109A
MYANMAR SIGN KHAMTI TONE-1

0109B
MYANMAR SIGN KHAMTI TONE-3

0109C
MYANMAR VOWEL SIGN AITON A

0109D
MYANMAR VOWEL SIGN AITON AI

0109E
MYANMAR SYMBOL SHAN ONE

0109F
MYANMAR SYMBOL SHAN EXCLAMATION

010A0
GEORGIAN CAPITAL LETTER AN

010A1
GEORGIAN CAPITAL LETTER BAN

010A2
GEORGIAN CAPITAL LETTER GAN

010A3
GEORGIAN CAPITAL LETTER DON

010A4
GEORGIAN CAPITAL LETTER EN

010A5
GEORGIAN CAPITAL LETTER VIN

010A6
GEORGIAN CAPITAL LETTER ZEN

010A7
GEORGIAN CAPITAL LETTER TAN

010A8
GEORGIAN CAPITAL LETTER IN

010A9
GEORGIAN CAPITAL LETTER KAN

010AA
GEORGIAN CAPITAL LETTER LAS

010AB
GEORGIAN CAPITAL LETTER MAN

010AC
GEORGIAN CAPITAL LETTER NAR

010AD
GEORGIAN CAPITAL LETTER ON

010AE
GEORGIAN CAPITAL LETTER PAR

010AF
GEORGIAN CAPITAL LETTER ZHAR

010B0
GEORGIAN CAPITAL LETTER RAE

010B1
GEORGIAN CAPITAL LETTER SAN

010B2
GEORGIAN CAPITAL LETTER TAR

010B3
GEORGIAN CAPITAL LETTER UN

010B4
GEORGIAN CAPITAL LETTER PHAR

010B5
GEORGIAN CAPITAL LETTER KHAR

010B6
GEORGIAN CAPITAL LETTER GHAN

010B7
GEORGIAN CAPITAL LETTER QAR

010B8
GEORGIAN CAPITAL LETTER SHIN

010B9
GEORGIAN CAPITAL LETTER CHIN

010BA
GEORGIAN CAPITAL LETTER CAN

010BB
GEORGIAN CAPITAL LETTER JIL

010BC
GEORGIAN CAPITAL LETTER CIL

010BD
GEORGIAN CAPITAL LETTER CHAR

010BE
GEORGIAN CAPITAL LETTER XAN

010BF
GEORGIAN CAPITAL LETTER JHAN

010C0
GEORGIAN CAPITAL LETTER HAE

010C1
GEORGIAN CAPITAL LETTER HE

010C2
GEORGIAN CAPITAL LETTER HIE

010C3
GEORGIAN CAPITAL LETTER WE

010C4
GEORGIAN CAPITAL LETTER HAR

010C5
GEORGIAN CAPITAL LETTER HOE

010C7
GEORGIAN CAPITAL LETTER YN

010CD
GEORGIAN CAPITAL LETTER AEN

010D0
GEORGIAN LETTER AN

010D1
GEORGIAN LETTER BAN

010D2
GEORGIAN LETTER GAN

010D3
GEORGIAN LETTER DON

010D4
GEORGIAN LETTER EN

010D5
GEORGIAN LETTER VIN

010D6
GEORGIAN LETTER ZEN

010D7
GEORGIAN LETTER TAN

010D8
GEORGIAN LETTER IN

010D9
GEORGIAN LETTER KAN

010DA
GEORGIAN LETTER LAS

010DB
GEORGIAN LETTER MAN

010DC
GEORGIAN LETTER NAR

010DD
GEORGIAN LETTER ON

010DE
GEORGIAN LETTER PAR

010DF
GEORGIAN LETTER ZHAR

010E0
GEORGIAN LETTER RAE

010E1
GEORGIAN LETTER SAN

010E2
GEORGIAN LETTER TAR

010E3
GEORGIAN LETTER UN

010E4
GEORGIAN LETTER PHAR

010E5
GEORGIAN LETTER KHAR

010E6
GEORGIAN LETTER GHAN

010E7
GEORGIAN LETTER QAR

010E8
GEORGIAN LETTER SHIN

010E9
GEORGIAN LETTER CHIN

010EA
GEORGIAN LETTER CAN

010EB
GEORGIAN LETTER JIL

010EC
GEORGIAN LETTER CIL

010ED
GEORGIAN LETTER CHAR

010EE
GEORGIAN LETTER XAN

010EF
GEORGIAN LETTER JHAN

010F0
GEORGIAN LETTER HAE

010F1
GEORGIAN LETTER HE

010F2
GEORGIAN LETTER HIE

010F3
GEORGIAN LETTER WE

010F4
GEORGIAN LETTER HAR

010F5
GEORGIAN LETTER HOE

010F6
GEORGIAN LETTER FI

010F7
GEORGIAN LETTER YN

010F8
GEORGIAN LETTER ELIFI

010F9
GEORGIAN LETTER TURNED GAN

010FA
GEORGIAN LETTER AIN

010FB
GEORGIAN PARAGRAPH SEPARATOR

010FC
MODIFIER LETTER GEORGIAN NAR

010FD
GEORGIAN LETTER AEN

010FE
GEORGIAN LETTER HARD SIGN

010FF
GEORGIAN LETTER LABIAL SIGN

01100
HANGUL CHOSEONG KIYEOK

01101
HANGUL CHOSEONG SSANGKIYEOK

01102
HANGUL CHOSEONG NIEUN

01103
HANGUL CHOSEONG TIKEUT

01104
HANGUL CHOSEONG SSANGTIKEUT

01105
HANGUL CHOSEONG RIEUL

01106
HANGUL CHOSEONG MIEUM

01107
HANGUL CHOSEONG PIEUP

01108
HANGUL CHOSEONG SSANGPIEUP

01109
HANGUL CHOSEONG SIOS

0110A
HANGUL CHOSEONG SSANGSIOS

0110B
HANGUL CHOSEONG IEUNG

0110C
HANGUL CHOSEONG CIEUC

0110D
HANGUL CHOSEONG SSANGCIEUC

0110E
HANGUL CHOSEONG CHIEUCH

0110F
HANGUL CHOSEONG KHIEUKH

01110
HANGUL CHOSEONG THIEUTH

01111
HANGUL CHOSEONG PHIEUPH

01112
HANGUL CHOSEONG HIEUH

01113
HANGUL CHOSEONG NIEUN-KIYEOK

01114
HANGUL CHOSEONG SSANGNIEUN

01115
HANGUL CHOSEONG NIEUN-TIKEUT

01116
HANGUL CHOSEONG NIEUN-PIEUP

01117
HANGUL CHOSEONG TIKEUT-KIYEOK

01118
HANGUL CHOSEONG RIEUL-NIEUN

01119
HANGUL CHOSEONG SSANGRIEUL

0111A
HANGUL CHOSEONG RIEUL-HIEUH

0111B
HANGUL CHOSEONG KAPYEOUNRIEUL

0111C
HANGUL CHOSEONG MIEUM-PIEUP

0111D
HANGUL CHOSEONG KAPYEOUNMIEUM

0111E
HANGUL CHOSEONG PIEUP-KIYEOK

0111F
HANGUL CHOSEONG PIEUP-NIEUN

01120
HANGUL CHOSEONG PIEUP-TIKEUT

01121
HANGUL CHOSEONG PIEUP-SIOS

01122
HANGUL CHOSEONG PIEUP-SIOS-KIYEOK

01123
HANGUL CHOSEONG PIEUP-SIOS-TIKEUT

01124
HANGUL CHOSEONG PIEUP-SIOS-PIEUP

01125
HANGUL CHOSEONG PIEUP-SSANGSIOS

01126
HANGUL CHOSEONG PIEUP-SIOS-CIEUC

01127
HANGUL CHOSEONG PIEUP-CIEUC

01128
HANGUL CHOSEONG PIEUP-CHIEUCH

01129
HANGUL CHOSEONG PIEUP-THIEUTH

0112A
HANGUL CHOSEONG PIEUP-PHIEUPH

0112B
HANGUL CHOSEONG KAPYEOUNPIEUP

0112C
HANGUL CHOSEONG KAPYEOUNSSANGPIEUP

0112D
HANGUL CHOSEONG SIOS-KIYEOK

0112E
HANGUL CHOSEONG SIOS-NIEUN

0112F
HANGUL CHOSEONG SIOS-TIKEUT

01130
HANGUL CHOSEONG SIOS-RIEUL

01131
HANGUL CHOSEONG SIOS-MIEUM

01132
HANGUL CHOSEONG SIOS-PIEUP

01133
HANGUL CHOSEONG SIOS-PIEUP-KIYEOK

01134
HANGUL CHOSEONG SIOS-SSANGSIOS

01135
HANGUL CHOSEONG SIOS-IEUNG

01136
HANGUL CHOSEONG SIOS-CIEUC

01137
HANGUL CHOSEONG SIOS-CHIEUCH

01138
HANGUL CHOSEONG SIOS-KHIEUKH

01139
HANGUL CHOSEONG SIOS-THIEUTH

0113A
HANGUL CHOSEONG SIOS-PHIEUPH

0113B
HANGUL CHOSEONG SIOS-HIEUH

0113C
HANGUL CHOSEONG CHITUEUMSIOS

0113D
HANGUL CHOSEONG CHITUEUMSSANGSIOS

0113E
HANGUL CHOSEONG CEONGCHIEUMSIOS

0113F
HANGUL CHOSEONG CEONGCHIEUMSSANGSIOS

01140
HANGUL CHOSEONG PANSIOS

01141
HANGUL CHOSEONG IEUNG-KIYEOK

01142
HANGUL CHOSEONG IEUNG-TIKEUT

01143
HANGUL CHOSEONG IEUNG-MIEUM

01144
HANGUL CHOSEONG IEUNG-PIEUP

01145
HANGUL CHOSEONG IEUNG-SIOS

01146
HANGUL CHOSEONG IEUNG-PANSIOS

01147
HANGUL CHOSEONG SSANGIEUNG

01148
HANGUL CHOSEONG IEUNG-CIEUC

01149
HANGUL CHOSEONG IEUNG-CHIEUCH

0114A
HANGUL CHOSEONG IEUNG-THIEUTH

0114B
HANGUL CHOSEONG IEUNG-PHIEUPH

0114C
HANGUL CHOSEONG YESIEUNG

0114D
HANGUL CHOSEONG CIEUC-IEUNG

0114E
HANGUL CHOSEONG CHITUEUMCIEUC

0114F
HANGUL CHOSEONG CHITUEUMSSANGCIEUC

01150
HANGUL CHOSEONG CEONGCHIEUMCIEUC

01151
HANGUL CHOSEONG CEONGCHIEUMSSANGCIEUC

01152
HANGUL CHOSEONG CHIEUCH-KHIEUKH

01153
HANGUL CHOSEONG CHIEUCH-HIEUH

01154
HANGUL CHOSEONG CHITUEUMCHIEUCH

01155
HANGUL CHOSEONG CEONGCHIEUMCHIEUCH

01156
HANGUL CHOSEONG PHIEUPH-PIEUP

01157
HANGUL CHOSEONG KAPYEOUNPHIEUPH

01158
HANGUL CHOSEONG SSANGHIEUH

01159
HANGUL CHOSEONG YEORINHIEUH

0115A
HANGUL CHOSEONG KIYEOK-TIKEUT

0115B
HANGUL CHOSEONG NIEUN-SIOS

0115C
HANGUL CHOSEONG NIEUN-CIEUC

0115D
HANGUL CHOSEONG NIEUN-HIEUH

0115E
HANGUL CHOSEONG TIKEUT-RIEUL

0115F
HANGUL CHOSEONG FILLER

01160
HANGUL JUNGSEONG FILLER

01161
HANGUL JUNGSEONG A

01162
HANGUL JUNGSEONG AE

01163
HANGUL JUNGSEONG YA

01164
HANGUL JUNGSEONG YAE

01165
HANGUL JUNGSEONG EO

01166
HANGUL JUNGSEONG E

01167
HANGUL JUNGSEONG YEO

01168
HANGUL JUNGSEONG YE

01169
HANGUL JUNGSEONG O

0116A
HANGUL JUNGSEONG WA

0116B
HANGUL JUNGSEONG WAE

0116C
HANGUL JUNGSEONG OE

0116D
HANGUL JUNGSEONG YO

0116E
HANGUL JUNGSEONG U

0116F
HANGUL JUNGSEONG WEO

01170
HANGUL JUNGSEONG WE

01171
HANGUL JUNGSEONG WI

01172
HANGUL JUNGSEONG YU

01173
HANGUL JUNGSEONG EU

01174
HANGUL JUNGSEONG YI

01175
HANGUL JUNGSEONG I

01176
HANGUL JUNGSEONG A-O

01177
HANGUL JUNGSEONG A-U

01178
HANGUL JUNGSEONG YA-O

01179
HANGUL JUNGSEONG YA-YO

0117A
HANGUL JUNGSEONG EO-O

0117B
HANGUL JUNGSEONG EO-U

0117C
HANGUL JUNGSEONG EO-EU

0117D
HANGUL JUNGSEONG YEO-O

0117E
HANGUL JUNGSEONG YEO-U

0117F
HANGUL JUNGSEONG O-EO

01180
HANGUL JUNGSEONG O-E

01181
HANGUL JUNGSEONG O-YE

01182
HANGUL JUNGSEONG O-O

01183
HANGUL JUNGSEONG O-U

01184
HANGUL JUNGSEONG YO-YA

01185
HANGUL JUNGSEONG YO-YAE

01186
HANGUL JUNGSEONG YO-YEO

01187
HANGUL JUNGSEONG YO-O

01188
HANGUL JUNGSEONG YO-I

01189
HANGUL JUNGSEONG U-A

0118A
HANGUL JUNGSEONG U-AE

0118B
HANGUL JUNGSEONG U-EO-EU

0118C
HANGUL JUNGSEONG U-YE

0118D
HANGUL JUNGSEONG U-U

0118E
HANGUL JUNGSEONG YU-A

0118F
HANGUL JUNGSEONG YU-EO

01190
HANGUL JUNGSEONG YU-E

01191
HANGUL JUNGSEONG YU-YEO

01192
HANGUL JUNGSEONG YU-YE

01193
HANGUL JUNGSEONG YU-U

01194
HANGUL JUNGSEONG YU-I

01195
HANGUL JUNGSEONG EU-U

01196
HANGUL JUNGSEONG EU-EU

01197
HANGUL JUNGSEONG YI-U

01198
HANGUL JUNGSEONG I-A

01199
HANGUL JUNGSEONG I-YA

0119A
HANGUL JUNGSEONG I-O

0119B
HANGUL JUNGSEONG I-U

0119C
HANGUL JUNGSEONG I-EU

0119D
HANGUL JUNGSEONG I-ARAEA

0119E
HANGUL JUNGSEONG ARAEA

0119F
HANGUL JUNGSEONG ARAEA-EO

011A0
HANGUL JUNGSEONG ARAEA-U

011A1
HANGUL JUNGSEONG ARAEA-I

011A2
HANGUL JUNGSEONG SSANGARAEA

011A3
HANGUL JUNGSEONG A-EU

011A4
HANGUL JUNGSEONG YA-U

011A5
HANGUL JUNGSEONG YEO-YA

011A6
HANGUL JUNGSEONG O-YA

011A7
HANGUL JUNGSEONG O-YAE

011A8
HANGUL JONGSEONG KIYEOK

011A9
HANGUL JONGSEONG SSANGKIYEOK

011AA
HANGUL JONGSEONG KIYEOK-SIOS

011AB
HANGUL JONGSEONG NIEUN

011AC
HANGUL JONGSEONG NIEUN-CIEUC

011AD
HANGUL JONGSEONG NIEUN-HIEUH

011AE
HANGUL JONGSEONG TIKEUT

011AF
HANGUL JONGSEONG RIEUL

011B0
HANGUL JONGSEONG RIEUL-KIYEOK

011B1
HANGUL JONGSEONG RIEUL-MIEUM

011B2
HANGUL JONGSEONG RIEUL-PIEUP

011B3
HANGUL JONGSEONG RIEUL-SIOS

011B4
HANGUL JONGSEONG RIEUL-THIEUTH

011B5
HANGUL JONGSEONG RIEUL-PHIEUPH

011B6
HANGUL JONGSEONG RIEUL-HIEUH

011B7
HANGUL JONGSEONG MIEUM

011B8
HANGUL JONGSEONG PIEUP

011B9
HANGUL JONGSEONG PIEUP-SIOS

011BA
HANGUL JONGSEONG SIOS

011BB
HANGUL JONGSEONG SSANGSIOS

011BC
HANGUL JONGSEONG IEUNG

011BD
HANGUL JONGSEONG CIEUC

011BE
HANGUL JONGSEONG CHIEUCH

011BF
HANGUL JONGSEONG KHIEUKH

011C0
HANGUL JONGSEONG THIEUTH

011C1
HANGUL JONGSEONG PHIEUPH

011C2
HANGUL JONGSEONG HIEUH

011C3
HANGUL JONGSEONG KIYEOK-RIEUL

011C4
HANGUL JONGSEONG KIYEOK-SIOS-KIYEOK

011C5
HANGUL JONGSEONG NIEUN-KIYEOK

011C6
HANGUL JONGSEONG NIEUN-TIKEUT

011C7
HANGUL JONGSEONG NIEUN-SIOS

011C8
HANGUL JONGSEONG NIEUN-PANSIOS

011C9
HANGUL JONGSEONG NIEUN-THIEUTH

011CA
HANGUL JONGSEONG TIKEUT-KIYEOK

011CB
HANGUL JONGSEONG TIKEUT-RIEUL

011CC
HANGUL JONGSEONG RIEUL-KIYEOK-SIOS

011CD
HANGUL JONGSEONG RIEUL-NIEUN

011CE
HANGUL JONGSEONG RIEUL-TIKEUT

011CF
HANGUL JONGSEONG RIEUL-TIKEUT-HIEUH

011D0
HANGUL JONGSEONG SSANGRIEUL

011D1
HANGUL JONGSEONG RIEUL-MIEUM-KIYEOK

011D2
HANGUL JONGSEONG RIEUL-MIEUM-SIOS

011D3
HANGUL JONGSEONG RIEUL-PIEUP-SIOS

011D4
HANGUL JONGSEONG RIEUL-PIEUP-HIEUH

011D5
HANGUL JONGSEONG RIEUL-KAPYEOUNPIEUP

011D6
HANGUL JONGSEONG RIEUL-SSANGSIOS

011D7
HANGUL JONGSEONG RIEUL-PANSIOS

011D8
HANGUL JONGSEONG RIEUL-KHIEUKH

011D9
HANGUL JONGSEONG RIEUL-YEORINHIEUH

011DA
HANGUL JONGSEONG MIEUM-KIYEOK

011DB
HANGUL JONGSEONG MIEUM-RIEUL

011DC
HANGUL JONGSEONG MIEUM-PIEUP

011DD
HANGUL JONGSEONG MIEUM-SIOS

011DE
HANGUL JONGSEONG MIEUM-SSANGSIOS

011DF
HANGUL JONGSEONG MIEUM-PANSIOS

011E0
HANGUL JONGSEONG MIEUM-CHIEUCH

011E1
HANGUL JONGSEONG MIEUM-HIEUH

011E2
HANGUL JONGSEONG KAPYEOUNMIEUM

011E3
HANGUL JONGSEONG PIEUP-RIEUL

011E4
HANGUL JONGSEONG PIEUP-PHIEUPH

011E5
HANGUL JONGSEONG PIEUP-HIEUH

011E6
HANGUL JONGSEONG KAPYEOUNPIEUP

011E7
HANGUL JONGSEONG SIOS-KIYEOK

011E8
HANGUL JONGSEONG SIOS-TIKEUT

011E9
HANGUL JONGSEONG SIOS-RIEUL

011EA
HANGUL JONGSEONG SIOS-PIEUP

011EB
HANGUL JONGSEONG PANSIOS

011EC
HANGUL JONGSEONG YESIEUNG-KIYEOK

011EC
HANGUL JONGSEONG IEUNG-KIYEOK

011ED
HANGUL JONGSEONG YESIEUNG-SSANGKIYEOK

011ED
HANGUL JONGSEONG IEUNG-SSANGKIYEOK

011EE
HANGUL JONGSEONG SSANGYESIEUNG

011EE
HANGUL JONGSEONG SSANGIEUNG

011EF
HANGUL JONGSEONG YESIEUNG-KHIEUKH

011EF
HANGUL JONGSEONG IEUNG-KHIEUKH

011F0
HANGUL JONGSEONG YESIEUNG

011F1
HANGUL JONGSEONG YESIEUNG-SIOS

011F2
HANGUL JONGSEONG YESIEUNG-PANSIOS

011F3
HANGUL JONGSEONG PHIEUPH-PIEUP

011F4
HANGUL JONGSEONG KAPYEOUNPHIEUPH

011F5
HANGUL JONGSEONG HIEUH-NIEUN

011F6
HANGUL JONGSEONG HIEUH-RIEUL

011F7
HANGUL JONGSEONG HIEUH-MIEUM

011F8
HANGUL JONGSEONG HIEUH-PIEUP

011F9
HANGUL JONGSEONG YEORINHIEUH

011FA
HANGUL JONGSEONG KIYEOK-NIEUN

011FB
HANGUL JONGSEONG KIYEOK-PIEUP

011FC
HANGUL JONGSEONG KIYEOK-CHIEUCH

011FD
HANGUL JONGSEONG KIYEOK-KHIEUKH

011FE
HANGUL JONGSEONG KIYEOK-HIEUH

011FF
HANGUL JONGSEONG SSANGNIEUN

01200
ETHIOPIC SYLLABLE HA

01201
ETHIOPIC SYLLABLE HU

01202
ETHIOPIC SYLLABLE HI

01203
ETHIOPIC SYLLABLE HAA

01204
ETHIOPIC SYLLABLE HEE

01205
ETHIOPIC SYLLABLE HE

01206
ETHIOPIC SYLLABLE HO

01207
ETHIOPIC SYLLABLE HOA

01208
ETHIOPIC SYLLABLE LA

01209
ETHIOPIC SYLLABLE LU

0120A
ETHIOPIC SYLLABLE LI

0120B
ETHIOPIC SYLLABLE LAA

0120C
ETHIOPIC SYLLABLE LEE

0120D
ETHIOPIC SYLLABLE LE

0120E
ETHIOPIC SYLLABLE LO

0120F
ETHIOPIC SYLLABLE LWA

01210
ETHIOPIC SYLLABLE HHA

01211
ETHIOPIC SYLLABLE HHU

01212
ETHIOPIC SYLLABLE HHI

01213
ETHIOPIC SYLLABLE HHAA

01214
ETHIOPIC SYLLABLE HHEE

01215
ETHIOPIC SYLLABLE HHE

01216
ETHIOPIC SYLLABLE HHO

01217
ETHIOPIC SYLLABLE HHWA

01218
ETHIOPIC SYLLABLE MA

01219
ETHIOPIC SYLLABLE MU

0121A
ETHIOPIC SYLLABLE MI

0121B
ETHIOPIC SYLLABLE MAA

0121C
ETHIOPIC SYLLABLE MEE

0121D
ETHIOPIC SYLLABLE ME

0121E
ETHIOPIC SYLLABLE MO

0121F
ETHIOPIC SYLLABLE MWA

01220
ETHIOPIC SYLLABLE SZA

01221
ETHIOPIC SYLLABLE SZU

01222
ETHIOPIC SYLLABLE SZI

01223
ETHIOPIC SYLLABLE SZAA

01224
ETHIOPIC SYLLABLE SZEE

01225
ETHIOPIC SYLLABLE SZE

01226
ETHIOPIC SYLLABLE SZO

01227
ETHIOPIC SYLLABLE SZWA

01228
ETHIOPIC SYLLABLE RA

01229
ETHIOPIC SYLLABLE RU

0122A
ETHIOPIC SYLLABLE RI

0122B
ETHIOPIC SYLLABLE RAA

0122C
ETHIOPIC SYLLABLE REE

0122D
ETHIOPIC SYLLABLE RE

0122E
ETHIOPIC SYLLABLE RO

0122F
ETHIOPIC SYLLABLE RWA

01230
ETHIOPIC SYLLABLE SA

01231
ETHIOPIC SYLLABLE SU

01232
ETHIOPIC SYLLABLE SI

01233
ETHIOPIC SYLLABLE SAA

01234
ETHIOPIC SYLLABLE SEE

01235
ETHIOPIC SYLLABLE SE

01236
ETHIOPIC SYLLABLE SO

01237
ETHIOPIC SYLLABLE SWA

01238
ETHIOPIC SYLLABLE SHA

01239
ETHIOPIC SYLLABLE SHU

0123A
ETHIOPIC SYLLABLE SHI

0123B
ETHIOPIC SYLLABLE SHAA

0123C
ETHIOPIC SYLLABLE SHEE

0123D
ETHIOPIC SYLLABLE SHE

0123E
ETHIOPIC SYLLABLE SHO

0123F
ETHIOPIC SYLLABLE SHWA

01240
ETHIOPIC SYLLABLE QA

01241
ETHIOPIC SYLLABLE QU

01242
ETHIOPIC SYLLABLE QI

01243
ETHIOPIC SYLLABLE QAA

01244
ETHIOPIC SYLLABLE QEE

01245
ETHIOPIC SYLLABLE QE

01246
ETHIOPIC SYLLABLE QO

01247
ETHIOPIC SYLLABLE QOA

01248
ETHIOPIC SYLLABLE QWA

0124A
ETHIOPIC SYLLABLE QWI

0124B
ETHIOPIC SYLLABLE QWAA

0124C
ETHIOPIC SYLLABLE QWEE

0124D
ETHIOPIC SYLLABLE QWE

01250
ETHIOPIC SYLLABLE QHA

01251
ETHIOPIC SYLLABLE QHU

01252
ETHIOPIC SYLLABLE QHI

01253
ETHIOPIC SYLLABLE QHAA

01254
ETHIOPIC SYLLABLE QHEE

01255
ETHIOPIC SYLLABLE QHE

01256
ETHIOPIC SYLLABLE QHO

01258
ETHIOPIC SYLLABLE QHWA

0125A
ETHIOPIC SYLLABLE QHWI

0125B
ETHIOPIC SYLLABLE QHWAA

0125C
ETHIOPIC SYLLABLE QHWEE

0125D
ETHIOPIC SYLLABLE QHWE

01260
ETHIOPIC SYLLABLE BA

01261
ETHIOPIC SYLLABLE BU

01262
ETHIOPIC SYLLABLE BI

01263
ETHIOPIC SYLLABLE BAA

01264
ETHIOPIC SYLLABLE BEE

01265
ETHIOPIC SYLLABLE BE

01266
ETHIOPIC SYLLABLE BO

01267
ETHIOPIC SYLLABLE BWA

01268
ETHIOPIC SYLLABLE VA

01269
ETHIOPIC SYLLABLE VU

0126A
ETHIOPIC SYLLABLE VI

0126B
ETHIOPIC SYLLABLE VAA

0126C
ETHIOPIC SYLLABLE VEE

0126D
ETHIOPIC SYLLABLE VE

0126E
ETHIOPIC SYLLABLE VO

0126F
ETHIOPIC SYLLABLE VWA

01270
ETHIOPIC SYLLABLE TA

01271
ETHIOPIC SYLLABLE TU

01272
ETHIOPIC SYLLABLE TI

01273
ETHIOPIC SYLLABLE TAA

01274
ETHIOPIC SYLLABLE TEE

01275
ETHIOPIC SYLLABLE TE

01276
ETHIOPIC SYLLABLE TO

01277
ETHIOPIC SYLLABLE TWA

01278
ETHIOPIC SYLLABLE CA

01279
ETHIOPIC SYLLABLE CU

0127A
ETHIOPIC SYLLABLE CI

0127B
ETHIOPIC SYLLABLE CAA

0127C
ETHIOPIC SYLLABLE CEE

0127D
ETHIOPIC SYLLABLE CE

0127E
ETHIOPIC SYLLABLE CO

0127F
ETHIOPIC SYLLABLE CWA

01280
ETHIOPIC SYLLABLE XA

01281
ETHIOPIC SYLLABLE XU

01282
ETHIOPIC SYLLABLE XI

01283
ETHIOPIC SYLLABLE XAA

01284
ETHIOPIC SYLLABLE XEE

01285
ETHIOPIC SYLLABLE XE

01286
ETHIOPIC SYLLABLE XO

01287
ETHIOPIC SYLLABLE XOA

01288
ETHIOPIC SYLLABLE XWA

0128A
ETHIOPIC SYLLABLE XWI

0128B
ETHIOPIC SYLLABLE XWAA

0128C
ETHIOPIC SYLLABLE XWEE

0128D
ETHIOPIC SYLLABLE XWE

01290
ETHIOPIC SYLLABLE NA

01291
ETHIOPIC SYLLABLE NU

01292
ETHIOPIC SYLLABLE NI

01293
ETHIOPIC SYLLABLE NAA

01294
ETHIOPIC SYLLABLE NEE

01295
ETHIOPIC SYLLABLE NE

01296
ETHIOPIC SYLLABLE NO

01297
ETHIOPIC SYLLABLE NWA

01298
ETHIOPIC SYLLABLE NYA

01299
ETHIOPIC SYLLABLE NYU

0129A
ETHIOPIC SYLLABLE NYI

0129B
ETHIOPIC SYLLABLE NYAA

0129C
ETHIOPIC SYLLABLE NYEE

0129D
ETHIOPIC SYLLABLE NYE

0129E
ETHIOPIC SYLLABLE NYO

0129F
ETHIOPIC SYLLABLE NYWA

012A0
ETHIOPIC SYLLABLE GLOTTAL A

012A1
ETHIOPIC SYLLABLE GLOTTAL U

012A2
ETHIOPIC SYLLABLE GLOTTAL I

012A3
ETHIOPIC SYLLABLE GLOTTAL AA

012A4
ETHIOPIC SYLLABLE GLOTTAL EE

012A5
ETHIOPIC SYLLABLE GLOTTAL E

012A6
ETHIOPIC SYLLABLE GLOTTAL O

012A7
ETHIOPIC SYLLABLE GLOTTAL WA

012A8
ETHIOPIC SYLLABLE KA

012A9
ETHIOPIC SYLLABLE KU

012AA
ETHIOPIC SYLLABLE KI

012AB
ETHIOPIC SYLLABLE KAA

012AC
ETHIOPIC SYLLABLE KEE

012AD
ETHIOPIC SYLLABLE KE

012AE
ETHIOPIC SYLLABLE KO

012AF
ETHIOPIC SYLLABLE KOA

012B0
ETHIOPIC SYLLABLE KWA

012B2
ETHIOPIC SYLLABLE KWI

012B3
ETHIOPIC SYLLABLE KWAA

012B4
ETHIOPIC SYLLABLE KWEE

012B5
ETHIOPIC SYLLABLE KWE

012B8
ETHIOPIC SYLLABLE KXA

012B9
ETHIOPIC SYLLABLE KXU

012BA
ETHIOPIC SYLLABLE KXI

012BB
ETHIOPIC SYLLABLE KXAA

012BC
ETHIOPIC SYLLABLE KXEE

012BD
ETHIOPIC SYLLABLE KXE

012BE
ETHIOPIC SYLLABLE KXO

012C0
ETHIOPIC SYLLABLE KXWA

012C2
ETHIOPIC SYLLABLE KXWI

012C3
ETHIOPIC SYLLABLE KXWAA

012C4
ETHIOPIC SYLLABLE KXWEE

012C5
ETHIOPIC SYLLABLE KXWE

012C8
ETHIOPIC SYLLABLE WA

012C9
ETHIOPIC SYLLABLE WU

012CA
ETHIOPIC SYLLABLE WI

012CB
ETHIOPIC SYLLABLE WAA

012CC
ETHIOPIC SYLLABLE WEE

012CD
ETHIOPIC SYLLABLE WE

012CE
ETHIOPIC SYLLABLE WO

012CF
ETHIOPIC SYLLABLE WOA

012D0
ETHIOPIC SYLLABLE PHARYNGEAL A

012D1
ETHIOPIC SYLLABLE PHARYNGEAL U

012D2
ETHIOPIC SYLLABLE PHARYNGEAL I

012D3
ETHIOPIC SYLLABLE PHARYNGEAL AA

012D4
ETHIOPIC SYLLABLE PHARYNGEAL EE

012D5
ETHIOPIC SYLLABLE PHARYNGEAL E

012D6
ETHIOPIC SYLLABLE PHARYNGEAL O

012D8
ETHIOPIC SYLLABLE ZA

012D9
ETHIOPIC SYLLABLE ZU

012DA
ETHIOPIC SYLLABLE ZI

012DB
ETHIOPIC SYLLABLE ZAA

012DC
ETHIOPIC SYLLABLE ZEE

012DD
ETHIOPIC SYLLABLE ZE

012DE
ETHIOPIC SYLLABLE ZO

012DF
ETHIOPIC SYLLABLE ZWA

012E0
ETHIOPIC SYLLABLE ZHA

012E1
ETHIOPIC SYLLABLE ZHU

012E2
ETHIOPIC SYLLABLE ZHI

012E3
ETHIOPIC SYLLABLE ZHAA

012E4
ETHIOPIC SYLLABLE ZHEE

012E5
ETHIOPIC SYLLABLE ZHE

012E6
ETHIOPIC SYLLABLE ZHO

012E7
ETHIOPIC SYLLABLE ZHWA

012E8
ETHIOPIC SYLLABLE YA

012E9
ETHIOPIC SYLLABLE YU

012EA
ETHIOPIC SYLLABLE YI

012EB
ETHIOPIC SYLLABLE YAA

012EC
ETHIOPIC SYLLABLE YEE

012ED
ETHIOPIC SYLLABLE YE

012EE
ETHIOPIC SYLLABLE YO

012EF
ETHIOPIC SYLLABLE YOA

012F0
ETHIOPIC SYLLABLE DA

012F1
ETHIOPIC SYLLABLE DU

012F2
ETHIOPIC SYLLABLE DI

012F3
ETHIOPIC SYLLABLE DAA

012F4
ETHIOPIC SYLLABLE DEE

012F5
ETHIOPIC SYLLABLE DE

012F6
ETHIOPIC SYLLABLE DO

012F7
ETHIOPIC SYLLABLE DWA

012F8
ETHIOPIC SYLLABLE DDA

012F9
ETHIOPIC SYLLABLE DDU

012FA
ETHIOPIC SYLLABLE DDI

012FB
ETHIOPIC SYLLABLE DDAA

012FC
ETHIOPIC SYLLABLE DDEE

012FD
ETHIOPIC SYLLABLE DDE

012FE
ETHIOPIC SYLLABLE DDO

012FF
ETHIOPIC SYLLABLE DDWA

01300
ETHIOPIC SYLLABLE JA

01301
ETHIOPIC SYLLABLE JU

01302
ETHIOPIC SYLLABLE JI

01303
ETHIOPIC SYLLABLE JAA

01304
ETHIOPIC SYLLABLE JEE

01305
ETHIOPIC SYLLABLE JE

01306
ETHIOPIC SYLLABLE JO

01307
ETHIOPIC SYLLABLE JWA

01308
ETHIOPIC SYLLABLE GA

01309
ETHIOPIC SYLLABLE GU

0130A
ETHIOPIC SYLLABLE GI

0130B
ETHIOPIC SYLLABLE GAA

0130C
ETHIOPIC SYLLABLE GEE

0130D
ETHIOPIC SYLLABLE GE

0130E
ETHIOPIC SYLLABLE GO

0130F
ETHIOPIC SYLLABLE GOA

01310
ETHIOPIC SYLLABLE GWA

01312
ETHIOPIC SYLLABLE GWI

01313
ETHIOPIC SYLLABLE GWAA

01314
ETHIOPIC SYLLABLE GWEE

01315
ETHIOPIC SYLLABLE GWE

01318
ETHIOPIC SYLLABLE GGA

01319
ETHIOPIC SYLLABLE GGU

0131A
ETHIOPIC SYLLABLE GGI

0131B
ETHIOPIC SYLLABLE GGAA

0131C
ETHIOPIC SYLLABLE GGEE

0131D
ETHIOPIC SYLLABLE GGE

0131E
ETHIOPIC SYLLABLE GGO

0131F
ETHIOPIC SYLLABLE GGWAA

01320
ETHIOPIC SYLLABLE THA

01321
ETHIOPIC SYLLABLE THU

01322
ETHIOPIC SYLLABLE THI

01323
ETHIOPIC SYLLABLE THAA

01324
ETHIOPIC SYLLABLE THEE

01325
ETHIOPIC SYLLABLE THE

01326
ETHIOPIC SYLLABLE THO

01327
ETHIOPIC SYLLABLE THWA

01328
ETHIOPIC SYLLABLE CHA

01329
ETHIOPIC SYLLABLE CHU

0132A
ETHIOPIC SYLLABLE CHI

0132B
ETHIOPIC SYLLABLE CHAA

0132C
ETHIOPIC SYLLABLE CHEE

0132D
ETHIOPIC SYLLABLE CHE

0132E
ETHIOPIC SYLLABLE CHO

0132F
ETHIOPIC SYLLABLE CHWA

01330
ETHIOPIC SYLLABLE PHA

01331
ETHIOPIC SYLLABLE PHU

01332
ETHIOPIC SYLLABLE PHI

01333
ETHIOPIC SYLLABLE PHAA

01334
ETHIOPIC SYLLABLE PHEE

01335
ETHIOPIC SYLLABLE PHE

01336
ETHIOPIC SYLLABLE PHO

01337
ETHIOPIC SYLLABLE PHWA

01338
ETHIOPIC SYLLABLE TSA

01339
ETHIOPIC SYLLABLE TSU

0133A
ETHIOPIC SYLLABLE TSI

0133B
ETHIOPIC SYLLABLE TSAA

0133C
ETHIOPIC SYLLABLE TSEE

0133D
ETHIOPIC SYLLABLE TSE

0133E
ETHIOPIC SYLLABLE TSO

0133F
ETHIOPIC SYLLABLE TSWA

01340
ETHIOPIC SYLLABLE TZA

01341
ETHIOPIC SYLLABLE TZU

01342
ETHIOPIC SYLLABLE TZI

01343
ETHIOPIC SYLLABLE TZAA

01344
ETHIOPIC SYLLABLE TZEE

01345
ETHIOPIC SYLLABLE TZE

01346
ETHIOPIC SYLLABLE TZO

01347
ETHIOPIC SYLLABLE TZOA

01348
ETHIOPIC SYLLABLE FA

01349
ETHIOPIC SYLLABLE FU

0134A
ETHIOPIC SYLLABLE FI

0134B
ETHIOPIC SYLLABLE FAA

0134C
ETHIOPIC SYLLABLE FEE

0134D
ETHIOPIC SYLLABLE FE

0134E
ETHIOPIC SYLLABLE FO

0134F
ETHIOPIC SYLLABLE FWA

01350
ETHIOPIC SYLLABLE PA

01351
ETHIOPIC SYLLABLE PU

01352
ETHIOPIC SYLLABLE PI

01353
ETHIOPIC SYLLABLE PAA

01354
ETHIOPIC SYLLABLE PEE

01355
ETHIOPIC SYLLABLE PE

01356
ETHIOPIC SYLLABLE PO

01357
ETHIOPIC SYLLABLE PWA

01358
ETHIOPIC SYLLABLE RYA

01359
ETHIOPIC SYLLABLE MYA

0135A
ETHIOPIC SYLLABLE FYA

0135D
ETHIOPIC COMBINING GEMINATION AND VOWEL LENGTH MARK

0135E
ETHIOPIC COMBINING VOWEL LENGTH MARK

0135F
ETHIOPIC COMBINING GEMINATION MARK

01360
ETHIOPIC SECTION MARK

01361
ETHIOPIC WORDSPACE

01362
ETHIOPIC FULL STOP

01363
ETHIOPIC COMMA

01364
ETHIOPIC SEMICOLON

01365
ETHIOPIC COLON

01366
ETHIOPIC PREFACE COLON

01367
ETHIOPIC QUESTION MARK

01368
ETHIOPIC PARAGRAPH SEPARATOR

01369
ETHIOPIC DIGIT ONE

0136A
ETHIOPIC DIGIT TWO

0136B
ETHIOPIC DIGIT THREE

0136C
ETHIOPIC DIGIT FOUR

0136D
ETHIOPIC DIGIT FIVE

0136E
ETHIOPIC DIGIT SIX

0136F
ETHIOPIC DIGIT SEVEN

01370
ETHIOPIC DIGIT EIGHT

01371
ETHIOPIC DIGIT NINE

01372
ETHIOPIC NUMBER TEN

01373
ETHIOPIC NUMBER TWENTY

01374
ETHIOPIC NUMBER THIRTY

01375
ETHIOPIC NUMBER FORTY

01376
ETHIOPIC NUMBER FIFTY

01377
ETHIOPIC NUMBER SIXTY

01378
ETHIOPIC NUMBER SEVENTY

01379
ETHIOPIC NUMBER EIGHTY

0137A
ETHIOPIC NUMBER NINETY

0137B
ETHIOPIC NUMBER HUNDRED

0137C
ETHIOPIC NUMBER TEN THOUSAND

01380
ETHIOPIC SYLLABLE SEBATBEIT MWA

01381
ETHIOPIC SYLLABLE MWI

01382
ETHIOPIC SYLLABLE MWEE

01383
ETHIOPIC SYLLABLE MWE

01384
ETHIOPIC SYLLABLE SEBATBEIT BWA

01385
ETHIOPIC SYLLABLE BWI

01386
ETHIOPIC SYLLABLE BWEE

01387
ETHIOPIC SYLLABLE BWE

01388
ETHIOPIC SYLLABLE SEBATBEIT FWA

01389
ETHIOPIC SYLLABLE FWI

0138A
ETHIOPIC SYLLABLE FWEE

0138B
ETHIOPIC SYLLABLE FWE

0138C
ETHIOPIC SYLLABLE SEBATBEIT PWA

0138D
ETHIOPIC SYLLABLE PWI

0138E
ETHIOPIC SYLLABLE PWEE

0138F
ETHIOPIC SYLLABLE PWE

01390
ETHIOPIC TONAL MARK YIZET

01391
ETHIOPIC TONAL MARK DERET

01392
ETHIOPIC TONAL MARK RIKRIK

01393
ETHIOPIC TONAL MARK SHORT RIKRIK

01394
ETHIOPIC TONAL MARK DIFAT

01395
ETHIOPIC TONAL MARK KENAT

01396
ETHIOPIC TONAL MARK CHIRET

01397
ETHIOPIC TONAL MARK HIDET

01398
ETHIOPIC TONAL MARK DERET-HIDET

01399
ETHIOPIC TONAL MARK KURT

013A0
CHEROKEE LETTER A

013A1
CHEROKEE LETTER E

013A2
CHEROKEE LETTER I

013A3
CHEROKEE LETTER O

013A4
CHEROKEE LETTER U

013A5
CHEROKEE LETTER V

013A6
CHEROKEE LETTER GA

013A7
CHEROKEE LETTER KA

013A8
CHEROKEE LETTER GE

013A9
CHEROKEE LETTER GI

013AA
CHEROKEE LETTER GO

013AB
CHEROKEE LETTER GU

013AC
CHEROKEE LETTER GV

013AD
CHEROKEE LETTER HA

013AE
CHEROKEE LETTER HE

013AF
CHEROKEE LETTER HI

013B0
CHEROKEE LETTER HO

013B1
CHEROKEE LETTER HU

013B2
CHEROKEE LETTER HV

013B3
CHEROKEE LETTER LA

013B4
CHEROKEE LETTER LE

013B5
CHEROKEE LETTER LI

013B6
CHEROKEE LETTER LO

013B7
CHEROKEE LETTER LU

013B8
CHEROKEE LETTER LV

013B9
CHEROKEE LETTER MA

013BA
CHEROKEE LETTER ME

013BB
CHEROKEE LETTER MI

013BC
CHEROKEE LETTER MO

013BD
CHEROKEE LETTER MU

013BE
CHEROKEE LETTER NA

013BF
CHEROKEE LETTER HNA

013C0
CHEROKEE LETTER NAH

013C1
CHEROKEE LETTER NE

013C2
CHEROKEE LETTER NI

013C3
CHEROKEE LETTER NO

013C4
CHEROKEE LETTER NU

013C5
CHEROKEE LETTER NV

013C6
CHEROKEE LETTER QUA

013C7
CHEROKEE LETTER QUE

013C8
CHEROKEE LETTER QUI

013C9
CHEROKEE LETTER QUO

013CA
CHEROKEE LETTER QUU

013CB
CHEROKEE LETTER QUV

013CC
CHEROKEE LETTER SA

013CD
CHEROKEE LETTER S

013CE
CHEROKEE LETTER SE

013CF
CHEROKEE LETTER SI

013D0
CHEROKEE LETTER SO

013D1
CHEROKEE LETTER SU

013D2
CHEROKEE LETTER SV

013D3
CHEROKEE LETTER DA

013D4
CHEROKEE LETTER TA

013D5
CHEROKEE LETTER DE

013D6
CHEROKEE LETTER TE

013D7
CHEROKEE LETTER DI

013D8
CHEROKEE LETTER TI

013D9
CHEROKEE LETTER DO

013DA
CHEROKEE LETTER DU

013DB
CHEROKEE LETTER DV

013DC
CHEROKEE LETTER DLA

013DD
CHEROKEE LETTER TLA

013DE
CHEROKEE LETTER TLE

013DF
CHEROKEE LETTER TLI

013E0
CHEROKEE LETTER TLO

013E1
CHEROKEE LETTER TLU

013E2
CHEROKEE LETTER TLV

013E3
CHEROKEE LETTER TSA

013E4
CHEROKEE LETTER TSE

013E5
CHEROKEE LETTER TSI

013E6
CHEROKEE LETTER TSO

013E7
CHEROKEE LETTER TSU

013E8
CHEROKEE LETTER TSV

013E9
CHEROKEE LETTER WA

013EA
CHEROKEE LETTER WE

013EB
CHEROKEE LETTER WI

013EC
CHEROKEE LETTER WO

013ED
CHEROKEE LETTER WU

013EE
CHEROKEE LETTER WV

013EF
CHEROKEE LETTER YA

013F0
CHEROKEE LETTER YE

013F1
CHEROKEE LETTER YI

013F2
CHEROKEE LETTER YO

013F3
CHEROKEE LETTER YU

013F4
CHEROKEE LETTER YV

013F5
CHEROKEE LETTER MV

013F8
CHEROKEE SMALL LETTER YE

013F9
CHEROKEE SMALL LETTER YI

013FA
CHEROKEE SMALL LETTER YO

013FB
CHEROKEE SMALL LETTER YU

013FC
CHEROKEE SMALL LETTER YV

013FD
CHEROKEE SMALL LETTER MV

01400
CANADIAN SYLLABICS HYPHEN

01401
CANADIAN SYLLABICS E

01402
CANADIAN SYLLABICS AAI

01403
CANADIAN SYLLABICS I

01404
CANADIAN SYLLABICS II

01405
CANADIAN SYLLABICS O

01406
CANADIAN SYLLABICS OO

01407
CANADIAN SYLLABICS Y-CREE OO

01408
CANADIAN SYLLABICS CARRIER EE

01409
CANADIAN SYLLABICS CARRIER I

0140A
CANADIAN SYLLABICS A

0140B
CANADIAN SYLLABICS AA

0140C
CANADIAN SYLLABICS WE

0140D
CANADIAN SYLLABICS WEST-CREE WE

0140E
CANADIAN SYLLABICS WI

0140F
CANADIAN SYLLABICS WEST-CREE WI

01410
CANADIAN SYLLABICS WII

01411
CANADIAN SYLLABICS WEST-CREE WII

01412
CANADIAN SYLLABICS WO

01413
CANADIAN SYLLABICS WEST-CREE WO

01414
CANADIAN SYLLABICS WOO

01415
CANADIAN SYLLABICS WEST-CREE WOO

01416
CANADIAN SYLLABICS NASKAPI WOO

01417
CANADIAN SYLLABICS WA

01418
CANADIAN SYLLABICS WEST-CREE WA

01419
CANADIAN SYLLABICS WAA

0141A
CANADIAN SYLLABICS WEST-CREE WAA

0141B
CANADIAN SYLLABICS NASKAPI WAA

0141C
CANADIAN SYLLABICS AI

0141D
CANADIAN SYLLABICS Y-CREE W

0141E
CANADIAN SYLLABICS GLOTTAL STOP

0141F
CANADIAN SYLLABICS FINAL ACUTE

01420
CANADIAN SYLLABICS FINAL GRAVE

01421
CANADIAN SYLLABICS FINAL BOTTOM HALF RING

01422
CANADIAN SYLLABICS FINAL TOP HALF RING

01423
CANADIAN SYLLABICS FINAL RIGHT HALF RING

01424
CANADIAN SYLLABICS FINAL RING

01425
CANADIAN SYLLABICS FINAL DOUBLE ACUTE

01426
CANADIAN SYLLABICS FINAL DOUBLE SHORT VERTICAL STROKES

01427
CANADIAN SYLLABICS FINAL MIDDLE DOT

01428
CANADIAN SYLLABICS FINAL SHORT HORIZONTAL STROKE

01429
CANADIAN SYLLABICS FINAL PLUS

0142A
CANADIAN SYLLABICS FINAL DOWN TACK

0142B
CANADIAN SYLLABICS EN

0142C
CANADIAN SYLLABICS IN

0142D
CANADIAN SYLLABICS ON

0142E
CANADIAN SYLLABICS AN

0142F
CANADIAN SYLLABICS PE

01430
CANADIAN SYLLABICS PAAI

01431
CANADIAN SYLLABICS PI

01432
CANADIAN SYLLABICS PII

01433
CANADIAN SYLLABICS PO

01434
CANADIAN SYLLABICS POO

01435
CANADIAN SYLLABICS Y-CREE POO

01436
CANADIAN SYLLABICS CARRIER HEE

01437
CANADIAN SYLLABICS CARRIER HI

01438
CANADIAN SYLLABICS PA

01439
CANADIAN SYLLABICS PAA

0143A
CANADIAN SYLLABICS PWE

0143B
CANADIAN SYLLABICS WEST-CREE PWE

0143C
CANADIAN SYLLABICS PWI

0143D
CANADIAN SYLLABICS WEST-CREE PWI

0143E
CANADIAN SYLLABICS PWII

0143F
CANADIAN SYLLABICS WEST-CREE PWII

01440
CANADIAN SYLLABICS PWO

01441
CANADIAN SYLLABICS WEST-CREE PWO

01442
CANADIAN SYLLABICS PWOO

01443
CANADIAN SYLLABICS WEST-CREE PWOO

01444
CANADIAN SYLLABICS PWA

01445
CANADIAN SYLLABICS WEST-CREE PWA

01446
CANADIAN SYLLABICS PWAA

01447
CANADIAN SYLLABICS WEST-CREE PWAA

01448
CANADIAN SYLLABICS Y-CREE PWAA

01449
CANADIAN SYLLABICS P

0144A
CANADIAN SYLLABICS WEST-CREE P

0144B
CANADIAN SYLLABICS CARRIER H

0144C
CANADIAN SYLLABICS TE

0144D
CANADIAN SYLLABICS TAAI

0144E
CANADIAN SYLLABICS TI

0144F
CANADIAN SYLLABICS TII

01450
CANADIAN SYLLABICS TO

01451
CANADIAN SYLLABICS TOO

01452
CANADIAN SYLLABICS Y-CREE TOO

01453
CANADIAN SYLLABICS CARRIER DEE

01454
CANADIAN SYLLABICS CARRIER DI

01455
CANADIAN SYLLABICS TA

01456
CANADIAN SYLLABICS TAA

01457
CANADIAN SYLLABICS TWE

01458
CANADIAN SYLLABICS WEST-CREE TWE

01459
CANADIAN SYLLABICS TWI

0145A
CANADIAN SYLLABICS WEST-CREE TWI

0145B
CANADIAN SYLLABICS TWII

0145C
CANADIAN SYLLABICS WEST-CREE TWII

0145D
CANADIAN SYLLABICS TWO

0145E
CANADIAN SYLLABICS WEST-CREE TWO

0145F
CANADIAN SYLLABICS TWOO

01460
CANADIAN SYLLABICS WEST-CREE TWOO

01461
CANADIAN SYLLABICS TWA

01462
CANADIAN SYLLABICS WEST-CREE TWA

01463
CANADIAN SYLLABICS TWAA

01464
CANADIAN SYLLABICS WEST-CREE TWAA

01465
CANADIAN SYLLABICS NASKAPI TWAA

01466
CANADIAN SYLLABICS T

01467
CANADIAN SYLLABICS TTE

01468
CANADIAN SYLLABICS TTI

01469
CANADIAN SYLLABICS TTO

0146A
CANADIAN SYLLABICS TTA

0146B
CANADIAN SYLLABICS KE

0146C
CANADIAN SYLLABICS KAAI

0146D
CANADIAN SYLLABICS KI

0146E
CANADIAN SYLLABICS KII

0146F
CANADIAN SYLLABICS KO

01470
CANADIAN SYLLABICS KOO

01471
CANADIAN SYLLABICS Y-CREE KOO

01472
CANADIAN SYLLABICS KA

01473
CANADIAN SYLLABICS KAA

01474
CANADIAN SYLLABICS KWE

01475
CANADIAN SYLLABICS WEST-CREE KWE

01476
CANADIAN SYLLABICS KWI

01477
CANADIAN SYLLABICS WEST-CREE KWI

01478
CANADIAN SYLLABICS KWII

01479
CANADIAN SYLLABICS WEST-CREE KWII

0147A
CANADIAN SYLLABICS KWO

0147B
CANADIAN SYLLABICS WEST-CREE KWO

0147C
CANADIAN SYLLABICS KWOO

0147D
CANADIAN SYLLABICS WEST-CREE KWOO

0147E
CANADIAN SYLLABICS KWA

0147F
CANADIAN SYLLABICS WEST-CREE KWA

01480
CANADIAN SYLLABICS KWAA

01481
CANADIAN SYLLABICS WEST-CREE KWAA

01482
CANADIAN SYLLABICS NASKAPI KWAA

01483
CANADIAN SYLLABICS K

01484
CANADIAN SYLLABICS KW

01485
CANADIAN SYLLABICS SOUTH-SLAVEY KEH

01486
CANADIAN SYLLABICS SOUTH-SLAVEY KIH

01487
CANADIAN SYLLABICS SOUTH-SLAVEY KOH

01488
CANADIAN SYLLABICS SOUTH-SLAVEY KAH

01489
CANADIAN SYLLABICS CE

0148A
CANADIAN SYLLABICS CAAI

0148B
CANADIAN SYLLABICS CI

0148C
CANADIAN SYLLABICS CII

0148D
CANADIAN SYLLABICS CO

0148E
CANADIAN SYLLABICS COO

0148F
CANADIAN SYLLABICS Y-CREE COO

01490
CANADIAN SYLLABICS CA

01491
CANADIAN SYLLABICS CAA

01492
CANADIAN SYLLABICS CWE

01493
CANADIAN SYLLABICS WEST-CREE CWE

01494
CANADIAN SYLLABICS CWI

01495
CANADIAN SYLLABICS WEST-CREE CWI

01496
CANADIAN SYLLABICS CWII

01497
CANADIAN SYLLABICS WEST-CREE CWII

01498
CANADIAN SYLLABICS CWO

01499
CANADIAN SYLLABICS WEST-CREE CWO

0149A
CANADIAN SYLLABICS CWOO

0149B
CANADIAN SYLLABICS WEST-CREE CWOO

0149C
CANADIAN SYLLABICS CWA

0149D
CANADIAN SYLLABICS WEST-CREE CWA

0149E
CANADIAN SYLLABICS CWAA

0149F
CANADIAN SYLLABICS WEST-CREE CWAA

014A0
CANADIAN SYLLABICS NASKAPI CWAA

014A1
CANADIAN SYLLABICS C

014A2
CANADIAN SYLLABICS SAYISI TH

014A3
CANADIAN SYLLABICS ME

014A4
CANADIAN SYLLABICS MAAI

014A5
CANADIAN SYLLABICS MI

014A6
CANADIAN SYLLABICS MII

014A7
CANADIAN SYLLABICS MO

014A8
CANADIAN SYLLABICS MOO

014A9
CANADIAN SYLLABICS Y-CREE MOO

014AA
CANADIAN SYLLABICS MA

014AB
CANADIAN SYLLABICS MAA

014AC
CANADIAN SYLLABICS MWE

014AD
CANADIAN SYLLABICS WEST-CREE MWE

014AE
CANADIAN SYLLABICS MWI

014AF
CANADIAN SYLLABICS WEST-CREE MWI

014B0
CANADIAN SYLLABICS MWII

014B1
CANADIAN SYLLABICS WEST-CREE MWII

014B2
CANADIAN SYLLABICS MWO

014B3
CANADIAN SYLLABICS WEST-CREE MWO

014B4
CANADIAN SYLLABICS MWOO

014B5
CANADIAN SYLLABICS WEST-CREE MWOO

014B6
CANADIAN SYLLABICS MWA

014B7
CANADIAN SYLLABICS WEST-CREE MWA

014B8
CANADIAN SYLLABICS MWAA

014B9
CANADIAN SYLLABICS WEST-CREE MWAA

014BA
CANADIAN SYLLABICS NASKAPI MWAA

014BB
CANADIAN SYLLABICS M

014BC
CANADIAN SYLLABICS WEST-CREE M

014BD
CANADIAN SYLLABICS MH

014BE
CANADIAN SYLLABICS ATHAPASCAN M

014BF
CANADIAN SYLLABICS SAYISI M

014C0
CANADIAN SYLLABICS NE

014C1
CANADIAN SYLLABICS NAAI

014C2
CANADIAN SYLLABICS NI

014C3
CANADIAN SYLLABICS NII

014C4
CANADIAN SYLLABICS NO

014C5
CANADIAN SYLLABICS NOO

014C6
CANADIAN SYLLABICS Y-CREE NOO

014C7
CANADIAN SYLLABICS NA

014C8
CANADIAN SYLLABICS NAA

014C9
CANADIAN SYLLABICS NWE

014CA
CANADIAN SYLLABICS WEST-CREE NWE

014CB
CANADIAN SYLLABICS NWA

014CC
CANADIAN SYLLABICS WEST-CREE NWA

014CD
CANADIAN SYLLABICS NWAA

014CE
CANADIAN SYLLABICS WEST-CREE NWAA

014CF
CANADIAN SYLLABICS NASKAPI NWAA

014D0
CANADIAN SYLLABICS N

014D1
CANADIAN SYLLABICS CARRIER NG

014D2
CANADIAN SYLLABICS NH

014D3
CANADIAN SYLLABICS LE

014D4
CANADIAN SYLLABICS LAAI

014D5
CANADIAN SYLLABICS LI

014D6
CANADIAN SYLLABICS LII

014D7
CANADIAN SYLLABICS LO

014D8
CANADIAN SYLLABICS LOO

014D9
CANADIAN SYLLABICS Y-CREE LOO

014DA
CANADIAN SYLLABICS LA

014DB
CANADIAN SYLLABICS LAA

014DC
CANADIAN SYLLABICS LWE

014DD
CANADIAN SYLLABICS WEST-CREE LWE

014DE
CANADIAN SYLLABICS LWI

014DF
CANADIAN SYLLABICS WEST-CREE LWI

014E0
CANADIAN SYLLABICS LWII

014E1
CANADIAN SYLLABICS WEST-CREE LWII

014E2
CANADIAN SYLLABICS LWO

014E3
CANADIAN SYLLABICS WEST-CREE LWO

014E4
CANADIAN SYLLABICS LWOO

014E5
CANADIAN SYLLABICS WEST-CREE LWOO

014E6
CANADIAN SYLLABICS LWA

014E7
CANADIAN SYLLABICS WEST-CREE LWA

014E8
CANADIAN SYLLABICS LWAA

014E9
CANADIAN SYLLABICS WEST-CREE LWAA

014EA
CANADIAN SYLLABICS L

014EB
CANADIAN SYLLABICS WEST-CREE L

014EC
CANADIAN SYLLABICS MEDIAL L

014ED
CANADIAN SYLLABICS SE

014EE
CANADIAN SYLLABICS SAAI

014EF
CANADIAN SYLLABICS SI

014F0
CANADIAN SYLLABICS SII

014F1
CANADIAN SYLLABICS SO

014F2
CANADIAN SYLLABICS SOO

014F3
CANADIAN SYLLABICS Y-CREE SOO

014F4
CANADIAN SYLLABICS SA

014F5
CANADIAN SYLLABICS SAA

014F6
CANADIAN SYLLABICS SWE

014F7
CANADIAN SYLLABICS WEST-CREE SWE

014F8
CANADIAN SYLLABICS SWI

014F9
CANADIAN SYLLABICS WEST-CREE SWI

014FA
CANADIAN SYLLABICS SWII

014FB
CANADIAN SYLLABICS WEST-CREE SWII

014FC
CANADIAN SYLLABICS SWO

014FD
CANADIAN SYLLABICS WEST-CREE SWO

014FE
CANADIAN SYLLABICS SWOO

014FF
CANADIAN SYLLABICS WEST-CREE SWOO

01500
CANADIAN SYLLABICS SWA

01501
CANADIAN SYLLABICS WEST-CREE SWA

01502
CANADIAN SYLLABICS SWAA

01503
CANADIAN SYLLABICS WEST-CREE SWAA

01504
CANADIAN SYLLABICS NASKAPI SWAA

01505
CANADIAN SYLLABICS S

01506
CANADIAN SYLLABICS ATHAPASCAN S

01507
CANADIAN SYLLABICS SW

01508
CANADIAN SYLLABICS BLACKFOOT S

01509
CANADIAN SYLLABICS MOOSE-CREE SK

0150A
CANADIAN SYLLABICS NASKAPI SKW

0150B
CANADIAN SYLLABICS NASKAPI S-W

0150C
CANADIAN SYLLABICS NASKAPI SPWA

0150D
CANADIAN SYLLABICS NASKAPI STWA

0150E
CANADIAN SYLLABICS NASKAPI SKWA

0150F
CANADIAN SYLLABICS NASKAPI SCWA

01510
CANADIAN SYLLABICS SHE

01511
CANADIAN SYLLABICS SHI

01512
CANADIAN SYLLABICS SHII

01513
CANADIAN SYLLABICS SHO

01514
CANADIAN SYLLABICS SHOO

01515
CANADIAN SYLLABICS SHA

01516
CANADIAN SYLLABICS SHAA

01517
CANADIAN SYLLABICS SHWE

01518
CANADIAN SYLLABICS WEST-CREE SHWE

01519
CANADIAN SYLLABICS SHWI

0151A
CANADIAN SYLLABICS WEST-CREE SHWI

0151B
CANADIAN SYLLABICS SHWII

0151C
CANADIAN SYLLABICS WEST-CREE SHWII

0151D
CANADIAN SYLLABICS SHWO

0151E
CANADIAN SYLLABICS WEST-CREE SHWO

0151F
CANADIAN SYLLABICS SHWOO

01520
CANADIAN SYLLABICS WEST-CREE SHWOO

01521
CANADIAN SYLLABICS SHWA

01522
CANADIAN SYLLABICS WEST-CREE SHWA

01523
CANADIAN SYLLABICS SHWAA

01524
CANADIAN SYLLABICS WEST-CREE SHWAA

01525
CANADIAN SYLLABICS SH

01526
CANADIAN SYLLABICS YE

01527
CANADIAN SYLLABICS YAAI

01528
CANADIAN SYLLABICS YI

01529
CANADIAN SYLLABICS YII

0152A
CANADIAN SYLLABICS YO

0152B
CANADIAN SYLLABICS YOO

0152C
CANADIAN SYLLABICS Y-CREE YOO

0152D
CANADIAN SYLLABICS YA

0152E
CANADIAN SYLLABICS YAA

0152F
CANADIAN SYLLABICS YWE

01530
CANADIAN SYLLABICS WEST-CREE YWE

01531
CANADIAN SYLLABICS YWI

01532
CANADIAN SYLLABICS WEST-CREE YWI

01533
CANADIAN SYLLABICS YWII

01534
CANADIAN SYLLABICS WEST-CREE YWII

01535
CANADIAN SYLLABICS YWO

01536
CANADIAN SYLLABICS WEST-CREE YWO

01537
CANADIAN SYLLABICS YWOO

01538
CANADIAN SYLLABICS WEST-CREE YWOO

01539
CANADIAN SYLLABICS YWA

0153A
CANADIAN SYLLABICS WEST-CREE YWA

0153B
CANADIAN SYLLABICS YWAA

0153C
CANADIAN SYLLABICS WEST-CREE YWAA

0153D
CANADIAN SYLLABICS NASKAPI YWAA

0153E
CANADIAN SYLLABICS Y

0153F
CANADIAN SYLLABICS BIBLE-CREE Y

01540
CANADIAN SYLLABICS WEST-CREE Y

01541
CANADIAN SYLLABICS SAYISI YI

01542
CANADIAN SYLLABICS RE

01543
CANADIAN SYLLABICS R-CREE RE

01544
CANADIAN SYLLABICS WEST-CREE LE

01545
CANADIAN SYLLABICS RAAI

01546
CANADIAN SYLLABICS RI

01547
CANADIAN SYLLABICS RII

01548
CANADIAN SYLLABICS RO

01549
CANADIAN SYLLABICS ROO

0154A
CANADIAN SYLLABICS WEST-CREE LO

0154B
CANADIAN SYLLABICS RA

0154C
CANADIAN SYLLABICS RAA

0154D
CANADIAN SYLLABICS WEST-CREE LA

0154E
CANADIAN SYLLABICS RWAA

0154F
CANADIAN SYLLABICS WEST-CREE RWAA

01550
CANADIAN SYLLABICS R

01551
CANADIAN SYLLABICS WEST-CREE R

01552
CANADIAN SYLLABICS MEDIAL R

01553
CANADIAN SYLLABICS FE

01554
CANADIAN SYLLABICS FAAI

01555
CANADIAN SYLLABICS FI

01556
CANADIAN SYLLABICS FII

01557
CANADIAN SYLLABICS FO

01558
CANADIAN SYLLABICS FOO

01559
CANADIAN SYLLABICS FA

0155A
CANADIAN SYLLABICS FAA

0155B
CANADIAN SYLLABICS FWAA

0155C
CANADIAN SYLLABICS WEST-CREE FWAA

0155D
CANADIAN SYLLABICS F

0155E
CANADIAN SYLLABICS THE

0155F
CANADIAN SYLLABICS N-CREE THE

01560
CANADIAN SYLLABICS THI

01561
CANADIAN SYLLABICS N-CREE THI

01562
CANADIAN SYLLABICS THII

01563
CANADIAN SYLLABICS N-CREE THII

01564
CANADIAN SYLLABICS THO

01565
CANADIAN SYLLABICS THOO

01566
CANADIAN SYLLABICS THA

01567
CANADIAN SYLLABICS THAA

01568
CANADIAN SYLLABICS THWAA

01569
CANADIAN SYLLABICS WEST-CREE THWAA

0156A
CANADIAN SYLLABICS TH

0156B
CANADIAN SYLLABICS TTHE

0156C
CANADIAN SYLLABICS TTHI

0156D
CANADIAN SYLLABICS TTHO

0156E
CANADIAN SYLLABICS TTHA

0156F
CANADIAN SYLLABICS TTH

01570
CANADIAN SYLLABICS TYE

01571
CANADIAN SYLLABICS TYI

01572
CANADIAN SYLLABICS TYO

01573
CANADIAN SYLLABICS TYA

01574
CANADIAN SYLLABICS NUNAVIK HE

01575
CANADIAN SYLLABICS NUNAVIK HI

01576
CANADIAN SYLLABICS NUNAVIK HII

01577
CANADIAN SYLLABICS NUNAVIK HO

01578
CANADIAN SYLLABICS NUNAVIK HOO

01579
CANADIAN SYLLABICS NUNAVIK HA

0157A
CANADIAN SYLLABICS NUNAVIK HAA

0157B
CANADIAN SYLLABICS NUNAVIK H

0157C
CANADIAN SYLLABICS NUNAVUT H

0157D
CANADIAN SYLLABICS HK

0157E
CANADIAN SYLLABICS QAAI

0157F
CANADIAN SYLLABICS QI

01580
CANADIAN SYLLABICS QII

01581
CANADIAN SYLLABICS QO

01582
CANADIAN SYLLABICS QOO

01583
CANADIAN SYLLABICS QA

01584
CANADIAN SYLLABICS QAA

01585
CANADIAN SYLLABICS Q

01586
CANADIAN SYLLABICS TLHE

01587
CANADIAN SYLLABICS TLHI

01588
CANADIAN SYLLABICS TLHO

01589
CANADIAN SYLLABICS TLHA

0158A
CANADIAN SYLLABICS WEST-CREE RE

0158B
CANADIAN SYLLABICS WEST-CREE RI

0158C
CANADIAN SYLLABICS WEST-CREE RO

0158D
CANADIAN SYLLABICS WEST-CREE RA

0158E
CANADIAN SYLLABICS NGAAI

0158F
CANADIAN SYLLABICS NGI

01590
CANADIAN SYLLABICS NGII

01591
CANADIAN SYLLABICS NGO

01592
CANADIAN SYLLABICS NGOO

01593
CANADIAN SYLLABICS NGA

01594
CANADIAN SYLLABICS NGAA

01595
CANADIAN SYLLABICS NG

01596
CANADIAN SYLLABICS NNG

01597
CANADIAN SYLLABICS SAYISI SHE

01598
CANADIAN SYLLABICS SAYISI SHI

01599
CANADIAN SYLLABICS SAYISI SHO

0159A
CANADIAN SYLLABICS SAYISI SHA

0159B
CANADIAN SYLLABICS WOODS-CREE THE

0159C
CANADIAN SYLLABICS WOODS-CREE THI

0159D
CANADIAN SYLLABICS WOODS-CREE THO

0159E
CANADIAN SYLLABICS WOODS-CREE THA

0159F
CANADIAN SYLLABICS WOODS-CREE TH

015A0
CANADIAN SYLLABICS LHI

015A1
CANADIAN SYLLABICS LHII

015A2
CANADIAN SYLLABICS LHO

015A3
CANADIAN SYLLABICS LHOO

015A4
CANADIAN SYLLABICS LHA

015A5
CANADIAN SYLLABICS LHAA

015A6
CANADIAN SYLLABICS LH

015A7
CANADIAN SYLLABICS TH-CREE THE

015A8
CANADIAN SYLLABICS TH-CREE THI

015A9
CANADIAN SYLLABICS TH-CREE THII

015AA
CANADIAN SYLLABICS TH-CREE THO

015AB
CANADIAN SYLLABICS TH-CREE THOO

015AC
CANADIAN SYLLABICS TH-CREE THA

015AD
CANADIAN SYLLABICS TH-CREE THAA

015AE
CANADIAN SYLLABICS TH-CREE TH

015AF
CANADIAN SYLLABICS AIVILIK B

015B0
CANADIAN SYLLABICS BLACKFOOT E

015B1
CANADIAN SYLLABICS BLACKFOOT I

015B2
CANADIAN SYLLABICS BLACKFOOT O

015B3
CANADIAN SYLLABICS BLACKFOOT A

015B4
CANADIAN SYLLABICS BLACKFOOT WE

015B5
CANADIAN SYLLABICS BLACKFOOT WI

015B6
CANADIAN SYLLABICS BLACKFOOT WO

015B7
CANADIAN SYLLABICS BLACKFOOT WA

015B8
CANADIAN SYLLABICS BLACKFOOT NE

015B9
CANADIAN SYLLABICS BLACKFOOT NI

015BA
CANADIAN SYLLABICS BLACKFOOT NO

015BB
CANADIAN SYLLABICS BLACKFOOT NA

015BC
CANADIAN SYLLABICS BLACKFOOT KE

015BD
CANADIAN SYLLABICS BLACKFOOT KI

015BE
CANADIAN SYLLABICS BLACKFOOT KO

015BF
CANADIAN SYLLABICS BLACKFOOT KA

015C0
CANADIAN SYLLABICS SAYISI HE

015C1
CANADIAN SYLLABICS SAYISI HI

015C2
CANADIAN SYLLABICS SAYISI HO

015C3
CANADIAN SYLLABICS SAYISI HA

015C4
CANADIAN SYLLABICS CARRIER GHU

015C5
CANADIAN SYLLABICS CARRIER GHO

015C6
CANADIAN SYLLABICS CARRIER GHE

015C7
CANADIAN SYLLABICS CARRIER GHEE

015C8
CANADIAN SYLLABICS CARRIER GHI

015C9
CANADIAN SYLLABICS CARRIER GHA

015CA
CANADIAN SYLLABICS CARRIER RU

015CB
CANADIAN SYLLABICS CARRIER RO

015CC
CANADIAN SYLLABICS CARRIER RE

015CD
CANADIAN SYLLABICS CARRIER REE

015CE
CANADIAN SYLLABICS CARRIER RI

015CF
CANADIAN SYLLABICS CARRIER RA

015D0
CANADIAN SYLLABICS CARRIER WU

015D1
CANADIAN SYLLABICS CARRIER WO

015D2
CANADIAN SYLLABICS CARRIER WE

015D3
CANADIAN SYLLABICS CARRIER WEE

015D4
CANADIAN SYLLABICS CARRIER WI

015D5
CANADIAN SYLLABICS CARRIER WA

015D6
CANADIAN SYLLABICS CARRIER HWU

015D7
CANADIAN SYLLABICS CARRIER HWO

015D8
CANADIAN SYLLABICS CARRIER HWE

015D9
CANADIAN SYLLABICS CARRIER HWEE

015DA
CANADIAN SYLLABICS CARRIER HWI

015DB
CANADIAN SYLLABICS CARRIER HWA

015DC
CANADIAN SYLLABICS CARRIER THU

015DD
CANADIAN SYLLABICS CARRIER THO

015DE
CANADIAN SYLLABICS CARRIER THE

015DF
CANADIAN SYLLABICS CARRIER THEE

015E0
CANADIAN SYLLABICS CARRIER THI

015E1
CANADIAN SYLLABICS CARRIER THA

015E2
CANADIAN SYLLABICS CARRIER TTU

015E3
CANADIAN SYLLABICS CARRIER TTO

015E4
CANADIAN SYLLABICS CARRIER TTE

015E5
CANADIAN SYLLABICS CARRIER TTEE

015E6
CANADIAN SYLLABICS CARRIER TTI

015E7
CANADIAN SYLLABICS CARRIER TTA

015E8
CANADIAN SYLLABICS CARRIER PU

015E9
CANADIAN SYLLABICS CARRIER PO

015EA
CANADIAN SYLLABICS CARRIER PE

015EB
CANADIAN SYLLABICS CARRIER PEE

015EC
CANADIAN SYLLABICS CARRIER PI

015ED
CANADIAN SYLLABICS CARRIER PA

015EE
CANADIAN SYLLABICS CARRIER P

015EF
CANADIAN SYLLABICS CARRIER GU

015F0
CANADIAN SYLLABICS CARRIER GO

015F1
CANADIAN SYLLABICS CARRIER GE

015F2
CANADIAN SYLLABICS CARRIER GEE

015F3
CANADIAN SYLLABICS CARRIER GI

015F4
CANADIAN SYLLABICS CARRIER GA

015F5
CANADIAN SYLLABICS CARRIER KHU

015F6
CANADIAN SYLLABICS CARRIER KHO

015F7
CANADIAN SYLLABICS CARRIER KHE

015F8
CANADIAN SYLLABICS CARRIER KHEE

015F9
CANADIAN SYLLABICS CARRIER KHI

015FA
CANADIAN SYLLABICS CARRIER KHA

015FB
CANADIAN SYLLABICS CARRIER KKU

015FC
CANADIAN SYLLABICS CARRIER KKO

015FD
CANADIAN SYLLABICS CARRIER KKE

015FE
CANADIAN SYLLABICS CARRIER KKEE

015FF
CANADIAN SYLLABICS CARRIER KKI

01600
CANADIAN SYLLABICS CARRIER KKA

01601
CANADIAN SYLLABICS CARRIER KK

01602
CANADIAN SYLLABICS CARRIER NU

01603
CANADIAN SYLLABICS CARRIER NO

01604
CANADIAN SYLLABICS CARRIER NE

01605
CANADIAN SYLLABICS CARRIER NEE

01606
CANADIAN SYLLABICS CARRIER NI

01607
CANADIAN SYLLABICS CARRIER NA

01608
CANADIAN SYLLABICS CARRIER MU

01609
CANADIAN SYLLABICS CARRIER MO

0160A
CANADIAN SYLLABICS CARRIER ME

0160B
CANADIAN SYLLABICS CARRIER MEE

0160C
CANADIAN SYLLABICS CARRIER MI

0160D
CANADIAN SYLLABICS CARRIER MA

0160E
CANADIAN SYLLABICS CARRIER YU

0160F
CANADIAN SYLLABICS CARRIER YO

01610
CANADIAN SYLLABICS CARRIER YE

01611
CANADIAN SYLLABICS CARRIER YEE

01612
CANADIAN SYLLABICS CARRIER YI

01613
CANADIAN SYLLABICS CARRIER YA

01614
CANADIAN SYLLABICS CARRIER JU

01615
CANADIAN SYLLABICS SAYISI JU

01616
CANADIAN SYLLABICS CARRIER JO

01617
CANADIAN SYLLABICS CARRIER JE

01618
CANADIAN SYLLABICS CARRIER JEE

01619
CANADIAN SYLLABICS CARRIER JI

0161A
CANADIAN SYLLABICS SAYISI JI

0161B
CANADIAN SYLLABICS CARRIER JA

0161C
CANADIAN SYLLABICS CARRIER JJU

0161D
CANADIAN SYLLABICS CARRIER JJO

0161E
CANADIAN SYLLABICS CARRIER JJE

0161F
CANADIAN SYLLABICS CARRIER JJEE

01620
CANADIAN SYLLABICS CARRIER JJI

01621
CANADIAN SYLLABICS CARRIER JJA

01622
CANADIAN SYLLABICS CARRIER LU

01623
CANADIAN SYLLABICS CARRIER LO

01624
CANADIAN SYLLABICS CARRIER LE

01625
CANADIAN SYLLABICS CARRIER LEE

01626
CANADIAN SYLLABICS CARRIER LI

01627
CANADIAN SYLLABICS CARRIER LA

01628
CANADIAN SYLLABICS CARRIER DLU

01629
CANADIAN SYLLABICS CARRIER DLO

0162A
CANADIAN SYLLABICS CARRIER DLE

0162B
CANADIAN SYLLABICS CARRIER DLEE

0162C
CANADIAN SYLLABICS CARRIER DLI

0162D
CANADIAN SYLLABICS CARRIER DLA

0162E
CANADIAN SYLLABICS CARRIER LHU

0162F
CANADIAN SYLLABICS CARRIER LHO

01630
CANADIAN SYLLABICS CARRIER LHE

01631
CANADIAN SYLLABICS CARRIER LHEE

01632
CANADIAN SYLLABICS CARRIER LHI

01633
CANADIAN SYLLABICS CARRIER LHA

01634
CANADIAN SYLLABICS CARRIER TLHU

01635
CANADIAN SYLLABICS CARRIER TLHO

01636
CANADIAN SYLLABICS CARRIER TLHE

01637
CANADIAN SYLLABICS CARRIER TLHEE

01638
CANADIAN SYLLABICS CARRIER TLHI

01639
CANADIAN SYLLABICS CARRIER TLHA

0163A
CANADIAN SYLLABICS CARRIER TLU

0163B
CANADIAN SYLLABICS CARRIER TLO

0163C
CANADIAN SYLLABICS CARRIER TLE

0163D
CANADIAN SYLLABICS CARRIER TLEE

0163E
CANADIAN SYLLABICS CARRIER TLI

0163F
CANADIAN SYLLABICS CARRIER TLA

01640
CANADIAN SYLLABICS CARRIER ZU

01641
CANADIAN SYLLABICS CARRIER ZO

01642
CANADIAN SYLLABICS CARRIER ZE

01643
CANADIAN SYLLABICS CARRIER ZEE

01644
CANADIAN SYLLABICS CARRIER ZI

01645
CANADIAN SYLLABICS CARRIER ZA

01646
CANADIAN SYLLABICS CARRIER Z

01647
CANADIAN SYLLABICS CARRIER INITIAL Z

01648
CANADIAN SYLLABICS CARRIER DZU

01649
CANADIAN SYLLABICS CARRIER DZO

0164A
CANADIAN SYLLABICS CARRIER DZE

0164B
CANADIAN SYLLABICS CARRIER DZEE

0164C
CANADIAN SYLLABICS CARRIER DZI

0164D
CANADIAN SYLLABICS CARRIER DZA

0164E
CANADIAN SYLLABICS CARRIER SU

0164F
CANADIAN SYLLABICS CARRIER SO

01650
CANADIAN SYLLABICS CARRIER SE

01651
CANADIAN SYLLABICS CARRIER SEE

01652
CANADIAN SYLLABICS CARRIER SI

01653
CANADIAN SYLLABICS CARRIER SA

01654
CANADIAN SYLLABICS CARRIER SHU

01655
CANADIAN SYLLABICS CARRIER SHO

01656
CANADIAN SYLLABICS CARRIER SHE

01657
CANADIAN SYLLABICS CARRIER SHEE

01658
CANADIAN SYLLABICS CARRIER SHI

01659
CANADIAN SYLLABICS CARRIER SHA

0165A
CANADIAN SYLLABICS CARRIER SH

0165B
CANADIAN SYLLABICS CARRIER TSU

0165C
CANADIAN SYLLABICS CARRIER TSO

0165D
CANADIAN SYLLABICS CARRIER TSE

0165E
CANADIAN SYLLABICS CARRIER TSEE

0165F
CANADIAN SYLLABICS CARRIER TSI

01660
CANADIAN SYLLABICS CARRIER TSA

01661
CANADIAN SYLLABICS CARRIER CHU

01662
CANADIAN SYLLABICS CARRIER CHO

01663
CANADIAN SYLLABICS CARRIER CHE

01664
CANADIAN SYLLABICS CARRIER CHEE

01665
CANADIAN SYLLABICS CARRIER CHI

01666
CANADIAN SYLLABICS CARRIER CHA

01667
CANADIAN SYLLABICS CARRIER TTSU

01668
CANADIAN SYLLABICS CARRIER TTSO

01669
CANADIAN SYLLABICS CARRIER TTSE

0166A
CANADIAN SYLLABICS CARRIER TTSEE

0166B
CANADIAN SYLLABICS CARRIER TTSI

0166C
CANADIAN SYLLABICS CARRIER TTSA

0166D
CANADIAN SYLLABICS CHI SIGN

0166E
CANADIAN SYLLABICS FULL STOP

0166F
CANADIAN SYLLABICS QAI

01670
CANADIAN SYLLABICS NGAI

01671
CANADIAN SYLLABICS NNGI

01672
CANADIAN SYLLABICS NNGII

01673
CANADIAN SYLLABICS NNGO

01674
CANADIAN SYLLABICS NNGOO

01675
CANADIAN SYLLABICS NNGA

01676
CANADIAN SYLLABICS NNGAA

01677
CANADIAN SYLLABICS WOODS-CREE THWEE

01678
CANADIAN SYLLABICS WOODS-CREE THWI

01679
CANADIAN SYLLABICS WOODS-CREE THWII

0167A
CANADIAN SYLLABICS WOODS-CREE THWO

0167B
CANADIAN SYLLABICS WOODS-CREE THWOO

0167C
CANADIAN SYLLABICS WOODS-CREE THWA

0167D
CANADIAN SYLLABICS WOODS-CREE THWAA

0167E
CANADIAN SYLLABICS WOODS-CREE FINAL TH

0167F
CANADIAN SYLLABICS BLACKFOOT W

01680
OGHAM SPACE MARK

01681
OGHAM LETTER BEITH

01682
OGHAM LETTER LUIS

01683
OGHAM LETTER FEARN

01684
OGHAM LETTER SAIL

01685
OGHAM LETTER NION

01686
OGHAM LETTER UATH

01687
OGHAM LETTER DAIR

01688
OGHAM LETTER TINNE

01689
OGHAM LETTER COLL

0168A
OGHAM LETTER CEIRT

0168B
OGHAM LETTER MUIN

0168C
OGHAM LETTER GORT

0168D
OGHAM LETTER NGEADAL

0168E
OGHAM LETTER STRAIF

0168F
OGHAM LETTER RUIS

01690
OGHAM LETTER AILM

01691
OGHAM LETTER ONN

01692
OGHAM LETTER UR

01693
OGHAM LETTER EADHADH

01694
OGHAM LETTER IODHADH

01695
OGHAM LETTER EABHADH

01696
OGHAM LETTER OR

01697
OGHAM LETTER UILLEANN

01698
OGHAM LETTER IFIN

01699
OGHAM LETTER EAMHANCHOLL

0169A
OGHAM LETTER PEITH

0169B
OGHAM FEATHER MARK

0169C
OGHAM REVERSED FEATHER MARK

016A0
RUNIC LETTER FEHU FEOH FE F

016A1
RUNIC LETTER V

016A2
RUNIC LETTER URUZ UR U

016A3
RUNIC LETTER YR

016A4
RUNIC LETTER Y

016A5
RUNIC LETTER W

016A6
RUNIC LETTER THURISAZ THURS THORN

016A7
RUNIC LETTER ETH

016A8
RUNIC LETTER ANSUZ A

016A9
RUNIC LETTER OS O

016AA
RUNIC LETTER AC A

016AB
RUNIC LETTER AESC

016AC
RUNIC LETTER LONG-BRANCH-OSS O

016AD
RUNIC LETTER SHORT-TWIG-OSS O

016AE
RUNIC LETTER O

016AF
RUNIC LETTER OE

016B0
RUNIC LETTER ON

016B1
RUNIC LETTER RAIDO RAD REID R

016B2
RUNIC LETTER KAUNA

016B3
RUNIC LETTER CEN

016B4
RUNIC LETTER KAUN K

016B5
RUNIC LETTER G

016B6
RUNIC LETTER ENG

016B7
RUNIC LETTER GEBO GYFU G

016B8
RUNIC LETTER GAR

016B9
RUNIC LETTER WUNJO WYNN W

016BA
RUNIC LETTER HAGLAZ H

016BB
RUNIC LETTER HAEGL H

016BC
RUNIC LETTER LONG-BRANCH-HAGALL H

016BD
RUNIC LETTER SHORT-TWIG-HAGALL H

016BE
RUNIC LETTER NAUDIZ NYD NAUD N

016BF
RUNIC LETTER SHORT-TWIG-NAUD N

016C0
RUNIC LETTER DOTTED-N

016C1
RUNIC LETTER ISAZ IS ISS I

016C2
RUNIC LETTER E

016C3
RUNIC LETTER JERAN J

016C4
RUNIC LETTER GER

016C5
RUNIC LETTER LONG-BRANCH-AR AE

016C6
RUNIC LETTER SHORT-TWIG-AR A

016C7
RUNIC LETTER IWAZ EOH

016C8
RUNIC LETTER PERTHO PEORTH P

016C9
RUNIC LETTER ALGIZ EOLHX

016CA
RUNIC LETTER SOWILO S

016CB
RUNIC LETTER SIGEL LONG-BRANCH-SOL S

016CC
RUNIC LETTER SHORT-TWIG-SOL S

016CD
RUNIC LETTER C

016CE
RUNIC LETTER Z

016CF
RUNIC LETTER TIWAZ TIR TYR T

016D0
RUNIC LETTER SHORT-TWIG-TYR T

016D1
RUNIC LETTER D

016D2
RUNIC LETTER BERKANAN BEORC BJARKAN B

016D3
RUNIC LETTER SHORT-TWIG-BJARKAN B

016D4
RUNIC LETTER DOTTED-P

016D5
RUNIC LETTER OPEN-P

016D6
RUNIC LETTER EHWAZ EH E

016D7
RUNIC LETTER MANNAZ MAN M

016D8
RUNIC LETTER LONG-BRANCH-MADR M

016D9
RUNIC LETTER SHORT-TWIG-MADR M

016DA
RUNIC LETTER LAUKAZ LAGU LOGR L

016DB
RUNIC LETTER DOTTED-L

016DC
RUNIC LETTER INGWAZ

016DD
RUNIC LETTER ING

016DE
RUNIC LETTER DAGAZ DAEG D

016DF
RUNIC LETTER OTHALAN ETHEL O

016E0
RUNIC LETTER EAR

016E1
RUNIC LETTER IOR

016E2
RUNIC LETTER CWEORTH

016E3
RUNIC LETTER CALC

016E4
RUNIC LETTER CEALC

016E5
RUNIC LETTER STAN

016E6
RUNIC LETTER LONG-BRANCH-YR

016E7
RUNIC LETTER SHORT-TWIG-YR

016E8
RUNIC LETTER ICELANDIC-YR

016E9
RUNIC LETTER Q

016EA
RUNIC LETTER X

016EB
RUNIC SINGLE PUNCTUATION

016EC
RUNIC MULTIPLE PUNCTUATION

016ED
RUNIC CROSS PUNCTUATION

016EE
RUNIC ARLAUG SYMBOL

016EF
RUNIC TVIMADUR SYMBOL

016F0
RUNIC BELGTHOR SYMBOL

016F1
RUNIC LETTER K

016F2
RUNIC LETTER SH

016F3
RUNIC LETTER OO

016F4
RUNIC LETTER FRANKS CASKET OS

016F5
RUNIC LETTER FRANKS CASKET IS

016F6
RUNIC LETTER FRANKS CASKET EH

016F7
RUNIC LETTER FRANKS CASKET AC

016F8
RUNIC LETTER FRANKS CASKET AESC

01700
TAGALOG LETTER A

01701
TAGALOG LETTER I

01702
TAGALOG LETTER U

01703
TAGALOG LETTER KA

01704
TAGALOG LETTER GA

01705
TAGALOG LETTER NGA

01706
TAGALOG LETTER TA

01707
TAGALOG LETTER DA

01708
TAGALOG LETTER NA

01709
TAGALOG LETTER PA

0170A
TAGALOG LETTER BA

0170B
TAGALOG LETTER MA

0170C
TAGALOG LETTER YA

0170D
TAGALOG LETTER RA

0170E
TAGALOG LETTER LA

0170F
TAGALOG LETTER WA

01710
TAGALOG LETTER SA

01711
TAGALOG LETTER HA

01712
TAGALOG VOWEL SIGN I

01713
TAGALOG VOWEL SIGN U

01714
TAGALOG SIGN VIRAMA

01715
TAGALOG SIGN PAMUDPOD

0171F
TAGALOG LETTER ARCHAIC RA

01720
HANUNOO LETTER A

01721
HANUNOO LETTER I

01722
HANUNOO LETTER U

01723
HANUNOO LETTER KA

01724
HANUNOO LETTER GA

01725
HANUNOO LETTER NGA

01726
HANUNOO LETTER TA

01727
HANUNOO LETTER DA

01728
HANUNOO LETTER NA

01729
HANUNOO LETTER PA

0172A
HANUNOO LETTER BA

0172B
HANUNOO LETTER MA

0172C
HANUNOO LETTER YA

0172D
HANUNOO LETTER RA

0172E
HANUNOO LETTER LA

0172F
HANUNOO LETTER WA

01730
HANUNOO LETTER SA

01731
HANUNOO LETTER HA

01732
HANUNOO VOWEL SIGN I

01733
HANUNOO VOWEL SIGN U

01734
HANUNOO SIGN PAMUDPOD

01735
PHILIPPINE SINGLE PUNCTUATION

01736
PHILIPPINE DOUBLE PUNCTUATION

01740
BUHID LETTER A

01741
BUHID LETTER I

01742
BUHID LETTER U

01743
BUHID LETTER KA

01744
BUHID LETTER GA

01745
BUHID LETTER NGA

01746
BUHID LETTER TA

01747
BUHID LETTER DA

01748
BUHID LETTER NA

01749
BUHID LETTER PA

0174A
BUHID LETTER BA

0174B
BUHID LETTER MA

0174C
BUHID LETTER YA

0174D
BUHID LETTER RA

0174E
BUHID LETTER LA

0174F
BUHID LETTER WA

01750
BUHID LETTER SA

01751
BUHID LETTER HA

01752
BUHID VOWEL SIGN I

01753
BUHID VOWEL SIGN U

01760
TAGBANWA LETTER A

01761
TAGBANWA LETTER I

01762
TAGBANWA LETTER U

01763
TAGBANWA LETTER KA

01764
TAGBANWA LETTER GA

01765
TAGBANWA LETTER NGA

01766
TAGBANWA LETTER TA

01767
TAGBANWA LETTER DA

01768
TAGBANWA LETTER NA

01769
TAGBANWA LETTER PA

0176A
TAGBANWA LETTER BA

0176B
TAGBANWA LETTER MA

0176C
TAGBANWA LETTER YA

0176E
TAGBANWA LETTER LA

0176F
TAGBANWA LETTER WA

01770
TAGBANWA LETTER SA

01772
TAGBANWA VOWEL SIGN I

01773
TAGBANWA VOWEL SIGN U

01780
KHMER LETTER KA

01781
KHMER LETTER KHA

01782
KHMER LETTER KO

01783
KHMER LETTER KHO

01784
KHMER LETTER NGO

01785
KHMER LETTER CA

01786
KHMER LETTER CHA

01787
KHMER LETTER CO

01788
KHMER LETTER CHO

01789
KHMER LETTER NYO

0178A
KHMER LETTER DA

0178B
KHMER LETTER TTHA

0178C
KHMER LETTER DO

0178D
KHMER LETTER TTHO

0178E
KHMER LETTER NNO

0178F
KHMER LETTER TA

01790
KHMER LETTER THA

01791
KHMER LETTER TO

01792
KHMER LETTER THO

01793
KHMER LETTER NO

01794
KHMER LETTER BA

01795
KHMER LETTER PHA

01796
KHMER LETTER PO

01797
KHMER LETTER PHO

01798
KHMER LETTER MO

01799
KHMER LETTER YO

0179A
KHMER LETTER RO

0179B
KHMER LETTER LO

0179C
KHMER LETTER VO

0179D
KHMER LETTER SHA

0179E
KHMER LETTER SSO

0179F
KHMER LETTER SA

017A0
KHMER LETTER HA

017A1
KHMER LETTER LA

017A2
KHMER LETTER QA

017A3
KHMER INDEPENDENT VOWEL QAQ

017A4
KHMER INDEPENDENT VOWEL QAA

017A5
KHMER INDEPENDENT VOWEL QI

017A6
KHMER INDEPENDENT VOWEL QII

017A7
KHMER INDEPENDENT VOWEL QU

017A8
KHMER INDEPENDENT VOWEL QUK

017A9
KHMER INDEPENDENT VOWEL QUU

017AA
KHMER INDEPENDENT VOWEL QUUV

017AB
KHMER INDEPENDENT VOWEL RY

017AC
KHMER INDEPENDENT VOWEL RYY

017AD
KHMER INDEPENDENT VOWEL LY

017AE
KHMER INDEPENDENT VOWEL LYY

017AF
KHMER INDEPENDENT VOWEL QE

017B0
KHMER INDEPENDENT VOWEL QAI

017B1
KHMER INDEPENDENT VOWEL QOO TYPE ONE

017B2
KHMER INDEPENDENT VOWEL QOO TYPE TWO

017B3
KHMER INDEPENDENT VOWEL QAU

017B4
KHMER VOWEL INHERENT AQ

017B5
KHMER VOWEL INHERENT AA

017B6
KHMER VOWEL SIGN AA

017B7
KHMER VOWEL SIGN I

017B8
KHMER VOWEL SIGN II

017B9
KHMER VOWEL SIGN Y

017BA
KHMER VOWEL SIGN YY

017BB
KHMER VOWEL SIGN U

017BC
KHMER VOWEL SIGN UU

017BD
KHMER VOWEL SIGN UA

017BE
KHMER VOWEL SIGN OE

017BF
KHMER VOWEL SIGN YA

017C0
KHMER VOWEL SIGN IE

017C1
KHMER VOWEL SIGN E

017C2
KHMER VOWEL SIGN AE

017C3
KHMER VOWEL SIGN AI

017C4
KHMER VOWEL SIGN OO

017C5
KHMER VOWEL SIGN AU

017C6
KHMER SIGN NIKAHIT

017C7
KHMER SIGN REAHMUK

017C8
KHMER SIGN YUUKALEAPINTU

017C9
KHMER SIGN MUUSIKATOAN

017CA
KHMER SIGN TRIISAP

017CB
KHMER SIGN BANTOC

017CC
KHMER SIGN ROBAT

017CD
KHMER SIGN TOANDAKHIAT

017CE
KHMER SIGN KAKABAT

017CF
KHMER SIGN AHSDA

017D0
KHMER SIGN SAMYOK SANNYA

017D1
KHMER SIGN VIRIAM

017D2
KHMER SIGN COENG

017D3
KHMER SIGN BATHAMASAT

017D4
KHMER SIGN KHAN

017D5
KHMER SIGN BARIYOOSAN

017D6
KHMER SIGN CAMNUC PII KUUH

017D7
KHMER SIGN LEK TOO

017D8
KHMER SIGN BEYYAL

017D9
KHMER SIGN PHNAEK MUAN

017DA
KHMER SIGN KOOMUUT

017DB
KHMER CURRENCY SYMBOL RIEL

017DC
KHMER SIGN AVAKRAHASANYA

017DD
KHMER SIGN ATTHACAN

017E0
KHMER DIGIT ZERO

017E1
KHMER DIGIT ONE

017E2
KHMER DIGIT TWO

017E3
KHMER DIGIT THREE

017E4
KHMER DIGIT FOUR

017E5
KHMER DIGIT FIVE

017E6
KHMER DIGIT SIX

017E7
KHMER DIGIT SEVEN

017E8
KHMER DIGIT EIGHT

017E9
KHMER DIGIT NINE

017F0
KHMER SYMBOL LEK ATTAK SON

017F1
KHMER SYMBOL LEK ATTAK MUOY

017F2
KHMER SYMBOL LEK ATTAK PII

017F3
KHMER SYMBOL LEK ATTAK BEI

017F4
KHMER SYMBOL LEK ATTAK BUON

017F5
KHMER SYMBOL LEK ATTAK PRAM

017F6
KHMER SYMBOL LEK ATTAK PRAM-MUOY

017F7
KHMER SYMBOL LEK ATTAK PRAM-PII

017F8
KHMER SYMBOL LEK ATTAK PRAM-BEI

017F9
KHMER SYMBOL LEK ATTAK PRAM-BUON

01800
MONGOLIAN BIRGA

01801
MONGOLIAN ELLIPSIS

01802
MONGOLIAN COMMA

01803
MONGOLIAN FULL STOP

01804
MONGOLIAN COLON

01805
MONGOLIAN FOUR DOTS

01806
MONGOLIAN TODO SOFT HYPHEN

01807
MONGOLIAN SIBE SYLLABLE BOUNDARY MARKER

01808
MONGOLIAN MANCHU COMMA

01809
MONGOLIAN MANCHU FULL STOP

0180A
MONGOLIAN NIRUGU

0180B
MONGOLIAN FREE VARIATION SELECTOR ONE

0180B
FVS1

0180C
MONGOLIAN FREE VARIATION SELECTOR TWO

0180C
FVS2

0180D
MONGOLIAN FREE VARIATION SELECTOR THREE

0180D
FVS3

0180E
MONGOLIAN VOWEL SEPARATOR

0180E
MVS

0180F
MONGOLIAN FREE VARIATION SELECTOR FOUR

0180F
FVS4

01810
MONGOLIAN DIGIT ZERO

01811
MONGOLIAN DIGIT ONE

01812
MONGOLIAN DIGIT TWO

01813
MONGOLIAN DIGIT THREE

01814
MONGOLIAN DIGIT FOUR

01815
MONGOLIAN DIGIT FIVE

01816
MONGOLIAN DIGIT SIX

01817
MONGOLIAN DIGIT SEVEN

01818
MONGOLIAN DIGIT EIGHT

01819
MONGOLIAN DIGIT NINE

01820
MONGOLIAN LETTER A

01821
MONGOLIAN LETTER E

01822
MONGOLIAN LETTER I

01823
MONGOLIAN LETTER O

01824
MONGOLIAN LETTER U

01825
MONGOLIAN LETTER OE

01826
MONGOLIAN LETTER UE

01827
MONGOLIAN LETTER EE

01828
MONGOLIAN LETTER NA

01829
MONGOLIAN LETTER ANG

0182A
MONGOLIAN LETTER BA

0182B
MONGOLIAN LETTER PA

0182C
MONGOLIAN LETTER QA

0182D
MONGOLIAN LETTER GA

0182E
MONGOLIAN LETTER MA

0182F
MONGOLIAN LETTER LA

01830
MONGOLIAN LETTER SA

01831
MONGOLIAN LETTER SHA

01832
MONGOLIAN LETTER TA

01833
MONGOLIAN LETTER DA

01834
MONGOLIAN LETTER CHA

01835
MONGOLIAN LETTER JA

01836
MONGOLIAN LETTER YA

01837
MONGOLIAN LETTER RA

01838
MONGOLIAN LETTER WA

01839
MONGOLIAN LETTER FA

0183A
MONGOLIAN LETTER KA

0183B
MONGOLIAN LETTER KHA

0183C
MONGOLIAN LETTER TSA

0183D
MONGOLIAN LETTER ZA

0183E
MONGOLIAN LETTER HAA

0183F
MONGOLIAN LETTER ZRA

01840
MONGOLIAN LETTER LHA

01841
MONGOLIAN LETTER ZHI

01842
MONGOLIAN LETTER CHI

01843
MONGOLIAN LETTER TODO LONG VOWEL SIGN

01844
MONGOLIAN LETTER TODO E

01845
MONGOLIAN LETTER TODO I

01846
MONGOLIAN LETTER TODO O

01847
MONGOLIAN LETTER TODO U

01848
MONGOLIAN LETTER TODO OE

01849
MONGOLIAN LETTER TODO UE

0184A
MONGOLIAN LETTER TODO ANG

0184B
MONGOLIAN LETTER TODO BA

0184C
MONGOLIAN LETTER TODO PA

0184D
MONGOLIAN LETTER TODO QA

0184E
MONGOLIAN LETTER TODO GA

0184F
MONGOLIAN LETTER TODO MA

01850
MONGOLIAN LETTER TODO TA

01851
MONGOLIAN LETTER TODO DA

01852
MONGOLIAN LETTER TODO CHA

01853
MONGOLIAN LETTER TODO JA

01854
MONGOLIAN LETTER TODO TSA

01855
MONGOLIAN LETTER TODO YA

01856
MONGOLIAN LETTER TODO WA

01857
MONGOLIAN LETTER TODO KA

01858
MONGOLIAN LETTER TODO GAA

01859
MONGOLIAN LETTER TODO HAA

0185A
MONGOLIAN LETTER TODO JIA

0185B
MONGOLIAN LETTER TODO NIA

0185C
MONGOLIAN LETTER TODO DZA

0185D
MONGOLIAN LETTER SIBE E

0185E
MONGOLIAN LETTER SIBE I

0185F
MONGOLIAN LETTER SIBE IY

01860
MONGOLIAN LETTER SIBE UE

01861
MONGOLIAN LETTER SIBE U

01862
MONGOLIAN LETTER SIBE ANG

01863
MONGOLIAN LETTER SIBE KA

01864
MONGOLIAN LETTER SIBE GA

01865
MONGOLIAN LETTER SIBE HA

01866
MONGOLIAN LETTER SIBE PA

01867
MONGOLIAN LETTER SIBE SHA

01868
MONGOLIAN LETTER SIBE TA

01869
MONGOLIAN LETTER SIBE DA

0186A
MONGOLIAN LETTER SIBE JA

0186B
MONGOLIAN LETTER SIBE FA

0186C
MONGOLIAN LETTER SIBE GAA

0186D
MONGOLIAN LETTER SIBE HAA

0186E
MONGOLIAN LETTER SIBE TSA

0186F
MONGOLIAN LETTER SIBE ZA

01870
MONGOLIAN LETTER SIBE RAA

01871
MONGOLIAN LETTER SIBE CHA

01872
MONGOLIAN LETTER SIBE ZHA

01873
MONGOLIAN LETTER MANCHU I

01874
MONGOLIAN LETTER MANCHU KA

01875
MONGOLIAN LETTER MANCHU RA

01876
MONGOLIAN LETTER MANCHU FA

01877
MONGOLIAN LETTER MANCHU ZHA

01878
MONGOLIAN LETTER CHA WITH TWO DOTS

01880
MONGOLIAN LETTER ALI GALI ANUSVARA ONE

01881
MONGOLIAN LETTER ALI GALI VISARGA ONE

01882
MONGOLIAN LETTER ALI GALI DAMARU

01883
MONGOLIAN LETTER ALI GALI UBADAMA

01884
MONGOLIAN LETTER ALI GALI INVERTED UBADAMA

01885
MONGOLIAN LETTER ALI GALI BALUDA

01886
MONGOLIAN LETTER ALI GALI THREE BALUDA

01887
MONGOLIAN LETTER ALI GALI A

01888
MONGOLIAN LETTER ALI GALI I

01889
MONGOLIAN LETTER ALI GALI KA

0188A
MONGOLIAN LETTER ALI GALI NGA

0188B
MONGOLIAN LETTER ALI GALI CA

0188C
MONGOLIAN LETTER ALI GALI TTA

0188D
MONGOLIAN LETTER ALI GALI TTHA

0188E
MONGOLIAN LETTER ALI GALI DDA

0188F
MONGOLIAN LETTER ALI GALI NNA

01890
MONGOLIAN LETTER ALI GALI TA

01891
MONGOLIAN LETTER ALI GALI DA

01892
MONGOLIAN LETTER ALI GALI PA

01893
MONGOLIAN LETTER ALI GALI PHA

01894
MONGOLIAN LETTER ALI GALI SSA

01895
MONGOLIAN LETTER ALI GALI ZHA

01896
MONGOLIAN LETTER ALI GALI ZA

01897
MONGOLIAN LETTER ALI GALI AH

01898
MONGOLIAN LETTER TODO ALI GALI TA

01899
MONGOLIAN LETTER TODO ALI GALI ZHA

0189A
MONGOLIAN LETTER MANCHU ALI GALI GHA

0189B
MONGOLIAN LETTER MANCHU ALI GALI NGA

0189C
MONGOLIAN LETTER MANCHU ALI GALI CA

0189D
MONGOLIAN LETTER MANCHU ALI GALI JHA

0189E
MONGOLIAN LETTER MANCHU ALI GALI TTA

0189F
MONGOLIAN LETTER MANCHU ALI GALI DDHA

018A0
MONGOLIAN LETTER MANCHU ALI GALI TA

018A1
MONGOLIAN LETTER MANCHU ALI GALI DHA

018A2
MONGOLIAN LETTER MANCHU ALI GALI SSA

018A3
MONGOLIAN LETTER MANCHU ALI GALI CYA

018A4
MONGOLIAN LETTER MANCHU ALI GALI ZHA

018A5
MONGOLIAN LETTER MANCHU ALI GALI ZA

018A6
MONGOLIAN LETTER ALI GALI HALF U

018A7
MONGOLIAN LETTER ALI GALI HALF YA

018A8
MONGOLIAN LETTER MANCHU ALI GALI BHA

018A9
MONGOLIAN LETTER ALI GALI DAGALGA

018AA
MONGOLIAN LETTER MANCHU ALI GALI LHA

018B0
CANADIAN SYLLABICS OY

018B1
CANADIAN SYLLABICS AY

018B2
CANADIAN SYLLABICS AAY

018B3
CANADIAN SYLLABICS WAY

018B4
CANADIAN SYLLABICS POY

018B5
CANADIAN SYLLABICS PAY

018B6
CANADIAN SYLLABICS PWOY

018B7
CANADIAN SYLLABICS TAY

018B8
CANADIAN SYLLABICS KAY

018B9
CANADIAN SYLLABICS KWAY

018BA
CANADIAN SYLLABICS MAY

018BB
CANADIAN SYLLABICS NOY

018BC
CANADIAN SYLLABICS NAY

018BD
CANADIAN SYLLABICS LAY

018BE
CANADIAN SYLLABICS SOY

018BF
CANADIAN SYLLABICS SAY

018C0
CANADIAN SYLLABICS SHOY

018C1
CANADIAN SYLLABICS SHAY

018C2
CANADIAN SYLLABICS SHWOY

018C3
CANADIAN SYLLABICS YOY

018C4
CANADIAN SYLLABICS YAY

018C5
CANADIAN SYLLABICS RAY

018C6
CANADIAN SYLLABICS NWI

018C7
CANADIAN SYLLABICS OJIBWAY NWI

018C8
CANADIAN SYLLABICS NWII

018C9
CANADIAN SYLLABICS OJIBWAY NWII

018CA
CANADIAN SYLLABICS NWO

018CB
CANADIAN SYLLABICS OJIBWAY NWO

018CC
CANADIAN SYLLABICS NWOO

018CD
CANADIAN SYLLABICS OJIBWAY NWOO

018CE
CANADIAN SYLLABICS RWEE

018CF
CANADIAN SYLLABICS RWI

018D0
CANADIAN SYLLABICS RWII

018D1
CANADIAN SYLLABICS RWO

018D2
CANADIAN SYLLABICS RWOO

018D3
CANADIAN SYLLABICS RWA

018D4
CANADIAN SYLLABICS OJIBWAY P

018D5
CANADIAN SYLLABICS OJIBWAY T

018D6
CANADIAN SYLLABICS OJIBWAY K

018D7
CANADIAN SYLLABICS OJIBWAY C

018D8
CANADIAN SYLLABICS OJIBWAY M

018D9
CANADIAN SYLLABICS OJIBWAY N

018DA
CANADIAN SYLLABICS OJIBWAY S

018DB
CANADIAN SYLLABICS OJIBWAY SH

018DC
CANADIAN SYLLABICS EASTERN W

018DD
CANADIAN SYLLABICS WESTERN W

018DE
CANADIAN SYLLABICS FINAL SMALL RING

018DF
CANADIAN SYLLABICS FINAL RAISED DOT

018E0
CANADIAN SYLLABICS R-CREE RWE

018E1
CANADIAN SYLLABICS WEST-CREE LOO

018E2
CANADIAN SYLLABICS WEST-CREE LAA

018E3
CANADIAN SYLLABICS THWE

018E4
CANADIAN SYLLABICS THWA

018E5
CANADIAN SYLLABICS TTHWE

018E6
CANADIAN SYLLABICS TTHOO

018E7
CANADIAN SYLLABICS TTHAA

018E8
CANADIAN SYLLABICS TLHWE

018E9
CANADIAN SYLLABICS TLHOO

018EA
CANADIAN SYLLABICS SAYISI SHWE

018EB
CANADIAN SYLLABICS SAYISI SHOO

018EC
CANADIAN SYLLABICS SAYISI HOO

018ED
CANADIAN SYLLABICS CARRIER GWU

018EE
CANADIAN SYLLABICS CARRIER DENE GEE

018EF
CANADIAN SYLLABICS CARRIER GAA

018F0
CANADIAN SYLLABICS CARRIER GWA

018F1
CANADIAN SYLLABICS SAYISI JUU

018F2
CANADIAN SYLLABICS CARRIER JWA

018F3
CANADIAN SYLLABICS BEAVER DENE L

018F4
CANADIAN SYLLABICS BEAVER DENE R

018F5
CANADIAN SYLLABICS CARRIER DENTAL S

01900
LIMBU VOWEL-CARRIER LETTER

01901
LIMBU LETTER KA

01902
LIMBU LETTER KHA

01903
LIMBU LETTER GA

01904
LIMBU LETTER GHA

01905
LIMBU LETTER NGA

01906
LIMBU LETTER CA

01907
LIMBU LETTER CHA

01908
LIMBU LETTER JA

01909
LIMBU LETTER JHA

0190A
LIMBU LETTER YAN

0190B
LIMBU LETTER TA

0190C
LIMBU LETTER THA

0190D
LIMBU LETTER DA

0190E
LIMBU LETTER DHA

0190F
LIMBU LETTER NA

01910
LIMBU LETTER PA

01911
LIMBU LETTER PHA

01912
LIMBU LETTER BA

01913
LIMBU LETTER BHA

01914
LIMBU LETTER MA

01915
LIMBU LETTER YA

01916
LIMBU LETTER RA

01917
LIMBU LETTER LA

01918
LIMBU LETTER WA

01919
LIMBU LETTER SHA

0191A
LIMBU LETTER SSA

0191B
LIMBU LETTER SA

0191C
LIMBU LETTER HA

0191D
LIMBU LETTER GYAN

0191E
LIMBU LETTER TRA

01920
LIMBU VOWEL SIGN A

01921
LIMBU VOWEL SIGN I

01922
LIMBU VOWEL SIGN U

01923
LIMBU VOWEL SIGN EE

01924
LIMBU VOWEL SIGN AI

01925
LIMBU VOWEL SIGN OO

01926
LIMBU VOWEL SIGN AU

01927
LIMBU VOWEL SIGN E

01928
LIMBU VOWEL SIGN O

01929
LIMBU SUBJOINED LETTER YA

0192A
LIMBU SUBJOINED LETTER RA

0192B
LIMBU SUBJOINED LETTER WA

01930
LIMBU SMALL LETTER KA

01931
LIMBU SMALL LETTER NGA

01932
LIMBU SMALL LETTER ANUSVARA

01933
LIMBU SMALL LETTER TA

01934
LIMBU SMALL LETTER NA

01935
LIMBU SMALL LETTER PA

01936
LIMBU SMALL LETTER MA

01937
LIMBU SMALL LETTER RA

01938
LIMBU SMALL LETTER LA

01939
LIMBU SIGN MUKPHRENG

0193A
LIMBU SIGN KEMPHRENG

0193B
LIMBU SIGN SA-I

01940
LIMBU SIGN LOO

01944
LIMBU EXCLAMATION MARK

01945
LIMBU QUESTION MARK

01946
LIMBU DIGIT ZERO

01947
LIMBU DIGIT ONE

01948
LIMBU DIGIT TWO

01949
LIMBU DIGIT THREE

0194A
LIMBU DIGIT FOUR

0194B
LIMBU DIGIT FIVE

0194C
LIMBU DIGIT SIX

0194D
LIMBU DIGIT SEVEN

0194E
LIMBU DIGIT EIGHT

0194F
LIMBU DIGIT NINE

01950
TAI LE LETTER KA

01951
TAI LE LETTER XA

01952
TAI LE LETTER NGA

01953
TAI LE LETTER TSA

01954
TAI LE LETTER SA

01955
TAI LE LETTER YA

01956
TAI LE LETTER TA

01957
TAI LE LETTER THA

01958
TAI LE LETTER LA

01959
TAI LE LETTER PA

0195A
TAI LE LETTER PHA

0195B
TAI LE LETTER MA

0195C
TAI LE LETTER FA

0195D
TAI LE LETTER VA

0195E
TAI LE LETTER HA

0195F
TAI LE LETTER QA

01960
TAI LE LETTER KHA

01961
TAI LE LETTER TSHA

01962
TAI LE LETTER NA

01963
TAI LE LETTER A

01964
TAI LE LETTER I

01965
TAI LE LETTER EE

01966
TAI LE LETTER EH

01967
TAI LE LETTER U

01968
TAI LE LETTER OO

01969
TAI LE LETTER O

0196A
TAI LE LETTER UE

0196B
TAI LE LETTER E

0196C
TAI LE LETTER AUE

0196D
TAI LE LETTER AI

01970
TAI LE LETTER TONE-2

01971
TAI LE LETTER TONE-3

01972
TAI LE LETTER TONE-4

01973
TAI LE LETTER TONE-5

01974
TAI LE LETTER TONE-6

01980
NEW TAI LUE LETTER HIGH QA

01981
NEW TAI LUE LETTER LOW QA

01982
NEW TAI LUE LETTER HIGH KA

01983
NEW TAI LUE LETTER HIGH XA

01984
NEW TAI LUE LETTER HIGH NGA

01985
NEW TAI LUE LETTER LOW KA

01986
NEW TAI LUE LETTER LOW XA

01987
NEW TAI LUE LETTER LOW NGA

01988
NEW TAI LUE LETTER HIGH TSA

01989
NEW TAI LUE LETTER HIGH SA

0198A
NEW TAI LUE LETTER HIGH YA

0198B
NEW TAI LUE LETTER LOW TSA

0198C
NEW TAI LUE LETTER LOW SA

0198D
NEW TAI LUE LETTER LOW YA

0198E
NEW TAI LUE LETTER HIGH TA

0198F
NEW TAI LUE LETTER HIGH THA

01990
NEW TAI LUE LETTER HIGH NA

01991
NEW TAI LUE LETTER LOW TA

01992
NEW TAI LUE LETTER LOW THA

01993
NEW TAI LUE LETTER LOW NA

01994
NEW TAI LUE LETTER HIGH PA

01995
NEW TAI LUE LETTER HIGH PHA

01996
NEW TAI LUE LETTER HIGH MA

01997
NEW TAI LUE LETTER LOW PA

01998
NEW TAI LUE LETTER LOW PHA

01999
NEW TAI LUE LETTER LOW MA

0199A
NEW TAI LUE LETTER HIGH FA

0199B
NEW TAI LUE LETTER HIGH VA

0199C
NEW TAI LUE LETTER HIGH LA

0199D
NEW TAI LUE LETTER LOW FA

0199E
NEW TAI LUE LETTER LOW VA

0199F
NEW TAI LUE LETTER LOW LA

019A0
NEW TAI LUE LETTER HIGH HA

019A1
NEW TAI LUE LETTER HIGH DA

019A2
NEW TAI LUE LETTER HIGH BA

019A3
NEW TAI LUE LETTER LOW HA

019A4
NEW TAI LUE LETTER LOW DA

019A5
NEW TAI LUE LETTER LOW BA

019A6
NEW TAI LUE LETTER HIGH KVA

019A7
NEW TAI LUE LETTER HIGH XVA

019A8
NEW TAI LUE LETTER LOW KVA

019A9
NEW TAI LUE LETTER LOW XVA

019AA
NEW TAI LUE LETTER HIGH SUA

019AB
NEW TAI LUE LETTER LOW SUA

019B0
NEW TAI LUE VOWEL SIGN VOWEL SHORTENER

019B1
NEW TAI LUE VOWEL SIGN AA

019B2
NEW TAI LUE VOWEL SIGN II

019B3
NEW TAI LUE VOWEL SIGN U

019B4
NEW TAI LUE VOWEL SIGN UU

019B5
NEW TAI LUE VOWEL SIGN E

019B6
NEW TAI LUE VOWEL SIGN AE

019B7
NEW TAI LUE VOWEL SIGN O

019B8
NEW TAI LUE VOWEL SIGN OA

019B9
NEW TAI LUE VOWEL SIGN UE

019BA
NEW TAI LUE VOWEL SIGN AY

019BB
NEW TAI LUE VOWEL SIGN AAY

019BC
NEW TAI LUE VOWEL SIGN UY

019BD
NEW TAI LUE VOWEL SIGN OY

019BE
NEW TAI LUE VOWEL SIGN OAY

019BF
NEW TAI LUE VOWEL SIGN UEY

019C0
NEW TAI LUE VOWEL SIGN IY

019C1
NEW TAI LUE LETTER FINAL V

019C2
NEW TAI LUE LETTER FINAL NG

019C3
NEW TAI LUE LETTER FINAL N

019C4
NEW TAI LUE LETTER FINAL M

019C5
NEW TAI LUE LETTER FINAL K

019C6
NEW TAI LUE LETTER FINAL D

019C7
NEW TAI LUE LETTER FINAL B

019C8
NEW TAI LUE TONE MARK-1

019C9
NEW TAI LUE TONE MARK-2

019D0
NEW TAI LUE DIGIT ZERO

019D1
NEW TAI LUE DIGIT ONE

019D2
NEW TAI LUE DIGIT TWO

019D3
NEW TAI LUE DIGIT THREE

019D4
NEW TAI LUE DIGIT FOUR

019D5
NEW TAI LUE DIGIT FIVE

019D6
NEW TAI LUE DIGIT SIX

019D7
NEW TAI LUE DIGIT SEVEN

019D8
NEW TAI LUE DIGIT EIGHT

019D9
NEW TAI LUE DIGIT NINE

019DA
NEW TAI LUE THAM DIGIT ONE

019DE
NEW TAI LUE SIGN LAE

019DF
NEW TAI LUE SIGN LAEV

019E0
KHMER SYMBOL PATHAMASAT

019E1
KHMER SYMBOL MUOY KOET

019E2
KHMER SYMBOL PII KOET

019E3
KHMER SYMBOL BEI KOET

019E4
KHMER SYMBOL BUON KOET

019E5
KHMER SYMBOL PRAM KOET

019E6
KHMER SYMBOL PRAM-MUOY KOET

019E7
KHMER SYMBOL PRAM-PII KOET

019E8
KHMER SYMBOL PRAM-BEI KOET

019E9
KHMER SYMBOL PRAM-BUON KOET

019EA
KHMER SYMBOL DAP KOET

019EB
KHMER SYMBOL DAP-MUOY KOET

019EC
KHMER SYMBOL DAP-PII KOET

019ED
KHMER SYMBOL DAP-BEI KOET

019EE
KHMER SYMBOL DAP-BUON KOET

019EF
KHMER SYMBOL DAP-PRAM KOET

019F0
KHMER SYMBOL TUTEYASAT

019F1
KHMER SYMBOL MUOY ROC

019F2
KHMER SYMBOL PII ROC

019F3
KHMER SYMBOL BEI ROC

019F4
KHMER SYMBOL BUON ROC

019F5
KHMER SYMBOL PRAM ROC

019F6
KHMER SYMBOL PRAM-MUOY ROC

019F7
KHMER SYMBOL PRAM-PII ROC

019F8
KHMER SYMBOL PRAM-BEI ROC

019F9
KHMER SYMBOL PRAM-BUON ROC

019FA
KHMER SYMBOL DAP ROC

019FB
KHMER SYMBOL DAP-MUOY ROC

019FC
KHMER SYMBOL DAP-PII ROC

019FD
KHMER SYMBOL DAP-BEI ROC

019FE
KHMER SYMBOL DAP-BUON ROC

019FF
KHMER SYMBOL DAP-PRAM ROC

01A00
BUGINESE LETTER KA

01A01
BUGINESE LETTER GA

01A02
BUGINESE LETTER NGA

01A03
BUGINESE LETTER NGKA

01A04
BUGINESE LETTER PA

01A05
BUGINESE LETTER BA

01A06
BUGINESE LETTER MA

01A07
BUGINESE LETTER MPA

01A08
BUGINESE LETTER TA

01A09
BUGINESE LETTER DA

01A0A
BUGINESE LETTER NA

01A0B
BUGINESE LETTER NRA

01A0C
BUGINESE LETTER CA

01A0D
BUGINESE LETTER JA

01A0E
BUGINESE LETTER NYA

01A0F
BUGINESE LETTER NYCA

01A10
BUGINESE LETTER YA

01A11
BUGINESE LETTER RA

01A12
BUGINESE LETTER LA

01A13
BUGINESE LETTER VA

01A14
BUGINESE LETTER SA

01A15
BUGINESE LETTER A

01A16
BUGINESE LETTER HA

01A17
BUGINESE VOWEL SIGN I

01A18
BUGINESE VOWEL SIGN U

01A19
BUGINESE VOWEL SIGN E

01A1A
BUGINESE VOWEL SIGN O

01A1B
BUGINESE VOWEL SIGN AE

01A1E
BUGINESE PALLAWA

01A1F
BUGINESE END OF SECTION

01A20
TAI THAM LETTER HIGH KA

01A21
TAI THAM LETTER HIGH KHA

01A22
TAI THAM LETTER HIGH KXA

01A23
TAI THAM LETTER LOW KA

01A24
TAI THAM LETTER LOW KXA

01A25
TAI THAM LETTER LOW KHA

01A26
TAI THAM LETTER NGA

01A27
TAI THAM LETTER HIGH CA

01A28
TAI THAM LETTER HIGH CHA

01A29
TAI THAM LETTER LOW CA

01A2A
TAI THAM LETTER LOW SA

01A2B
TAI THAM LETTER LOW CHA

01A2C
TAI THAM LETTER NYA

01A2D
TAI THAM LETTER RATA

01A2E
TAI THAM LETTER HIGH RATHA

01A2F
TAI THAM LETTER DA

01A30
TAI THAM LETTER LOW RATHA

01A31
TAI THAM LETTER RANA

01A32
TAI THAM LETTER HIGH TA

01A33
TAI THAM LETTER HIGH THA

01A34
TAI THAM LETTER LOW TA

01A35
TAI THAM LETTER LOW THA

01A36
TAI THAM LETTER NA

01A37
TAI THAM LETTER BA

01A38
TAI THAM LETTER HIGH PA

01A39
TAI THAM LETTER HIGH PHA

01A3A
TAI THAM LETTER HIGH FA

01A3B
TAI THAM LETTER LOW PA

01A3C
TAI THAM LETTER LOW FA

01A3D
TAI THAM LETTER LOW PHA

01A3E
TAI THAM LETTER MA

01A3F
TAI THAM LETTER LOW YA

01A40
TAI THAM LETTER HIGH YA

01A41
TAI THAM LETTER RA

01A42
TAI THAM LETTER RUE

01A43
TAI THAM LETTER LA

01A44
TAI THAM LETTER LUE

01A45
TAI THAM LETTER WA

01A46
TAI THAM LETTER HIGH SHA

01A47
TAI THAM LETTER HIGH SSA

01A48
TAI THAM LETTER HIGH SA

01A49
TAI THAM LETTER HIGH HA

01A4A
TAI THAM LETTER LLA

01A4B
TAI THAM LETTER A

01A4C
TAI THAM LETTER LOW HA

01A4D
TAI THAM LETTER I

01A4E
TAI THAM LETTER II

01A4F
TAI THAM LETTER U

01A50
TAI THAM LETTER UU

01A51
TAI THAM LETTER EE

01A52
TAI THAM LETTER OO

01A53
TAI THAM LETTER LAE

01A54
TAI THAM LETTER GREAT SA

01A55
TAI THAM CONSONANT SIGN MEDIAL RA

01A56
TAI THAM CONSONANT SIGN MEDIAL LA

01A57
TAI THAM CONSONANT SIGN LA TANG LAI

01A58
TAI THAM SIGN MAI KANG LAI

01A59
TAI THAM CONSONANT SIGN FINAL NGA

01A5A
TAI THAM CONSONANT SIGN LOW PA

01A5B
TAI THAM CONSONANT SIGN HIGH RATHA OR LOW PA

01A5C
TAI THAM CONSONANT SIGN MA

01A5D
TAI THAM CONSONANT SIGN BA

01A5E
TAI THAM CONSONANT SIGN SA

01A60
TAI THAM SIGN SAKOT

01A61
TAI THAM VOWEL SIGN A

01A62
TAI THAM VOWEL SIGN MAI SAT

01A63
TAI THAM VOWEL SIGN AA

01A64
TAI THAM VOWEL SIGN TALL AA

01A65
TAI THAM VOWEL SIGN I

01A66
TAI THAM VOWEL SIGN II

01A67
TAI THAM VOWEL SIGN UE

01A68
TAI THAM VOWEL SIGN UUE

01A69
TAI THAM VOWEL SIGN U

01A6A
TAI THAM VOWEL SIGN UU

01A6B
TAI THAM VOWEL SIGN O

01A6C
TAI THAM VOWEL SIGN OA BELOW

01A6D
TAI THAM VOWEL SIGN OY

01A6E
TAI THAM VOWEL SIGN E

01A6F
TAI THAM VOWEL SIGN AE

01A70
TAI THAM VOWEL SIGN OO

01A71
TAI THAM VOWEL SIGN AI

01A72
TAI THAM VOWEL SIGN THAM AI

01A73
TAI THAM VOWEL SIGN OA ABOVE

01A74
TAI THAM SIGN MAI KANG

01A75
TAI THAM SIGN TONE-1

01A76
TAI THAM SIGN TONE-2

01A77
TAI THAM SIGN KHUEN TONE-3

01A78
TAI THAM SIGN KHUEN TONE-4

01A79
TAI THAM SIGN KHUEN TONE-5

01A7A
TAI THAM SIGN RA HAAM

01A7B
TAI THAM SIGN MAI SAM

01A7C
TAI THAM SIGN KHUEN-LUE KARAN

01A7F
TAI THAM COMBINING CRYPTOGRAMMIC DOT

01A80
TAI THAM HORA DIGIT ZERO

01A81
TAI THAM HORA DIGIT ONE

01A82
TAI THAM HORA DIGIT TWO

01A83
TAI THAM HORA DIGIT THREE

01A84
TAI THAM HORA DIGIT FOUR

01A85
TAI THAM HORA DIGIT FIVE

01A86
TAI THAM HORA DIGIT SIX

01A87
TAI THAM HORA DIGIT SEVEN

01A88
TAI THAM HORA DIGIT EIGHT

01A89
TAI THAM HORA DIGIT NINE

01A90
TAI THAM THAM DIGIT ZERO

01A91
TAI THAM THAM DIGIT ONE

01A92
TAI THAM THAM DIGIT TWO

01A93
TAI THAM THAM DIGIT THREE

01A94
TAI THAM THAM DIGIT FOUR

01A95
TAI THAM THAM DIGIT FIVE

01A96
TAI THAM THAM DIGIT SIX

01A97
TAI THAM THAM DIGIT SEVEN

01A98
TAI THAM THAM DIGIT EIGHT

01A99
TAI THAM THAM DIGIT NINE

01AA0
TAI THAM SIGN WIANG

01AA1
TAI THAM SIGN WIANGWAAK

01AA2
TAI THAM SIGN SAWAN

01AA3
TAI THAM SIGN KEOW

01AA4
TAI THAM SIGN HOY

01AA5
TAI THAM SIGN DOKMAI

01AA6
TAI THAM SIGN REVERSED ROTATED RANA

01AA7
TAI THAM SIGN MAI YAMOK

01AA8
TAI THAM SIGN KAAN

01AA9
TAI THAM SIGN KAANKUU

01AAA
TAI THAM SIGN SATKAAN

01AAB
TAI THAM SIGN SATKAANKUU

01AAC
TAI THAM SIGN HANG

01AAD
TAI THAM SIGN CAANG

01AB0
COMBINING DOUBLED CIRCUMFLEX ACCENT

01AB1
COMBINING DIAERESIS-RING

01AB2
COMBINING INFINITY

01AB3
COMBINING DOWNWARDS ARROW

01AB4
COMBINING TRIPLE DOT

01AB5
COMBINING X-X BELOW

01AB6
COMBINING WIGGLY LINE BELOW

01AB7
COMBINING OPEN MARK BELOW

01AB8
COMBINING DOUBLE OPEN MARK BELOW

01AB9
COMBINING LIGHT CENTRALIZATION STROKE BELOW

01ABA
COMBINING STRONG CENTRALIZATION STROKE BELOW

01ABB
COMBINING PARENTHESES ABOVE

01ABC
COMBINING DOUBLE PARENTHESES ABOVE

01ABD
COMBINING PARENTHESES BELOW

01ABE
COMBINING PARENTHESES OVERLAY

01ABF
COMBINING LATIN SMALL LETTER W BELOW

01AC0
COMBINING LATIN SMALL LETTER TURNED W BELOW

01AC1
COMBINING LEFT PARENTHESIS ABOVE LEFT

01AC2
COMBINING RIGHT PARENTHESIS ABOVE RIGHT

01AC3
COMBINING LEFT PARENTHESIS BELOW LEFT

01AC4
COMBINING RIGHT PARENTHESIS BELOW RIGHT

01AC5
COMBINING SQUARE BRACKETS ABOVE

01AC6
COMBINING NUMBER SIGN ABOVE

01AC7
COMBINING INVERTED DOUBLE ARCH ABOVE

01AC8
COMBINING PLUS SIGN ABOVE

01AC9
COMBINING DOUBLE PLUS SIGN ABOVE

01ACA
COMBINING DOUBLE PLUS SIGN BELOW

01ACB
COMBINING TRIPLE ACUTE ACCENT

01ACC
COMBINING LATIN SMALL LETTER INSULAR G

01ACD
COMBINING LATIN SMALL LETTER INSULAR R

01ACE
COMBINING LATIN SMALL LETTER INSULAR T

01ACF
COMBINING DOUBLE CARON

01AD0
COMBINING VERTICAL-LINE-ACUTE

01AD1
COMBINING GRAVE-VERTICAL-LINE

01AD2
COMBINING VERTICAL-LINE-GRAVE

01AD3
COMBINING ACUTE-VERTICAL-LINE

01AD4
COMBINING VERTICAL-LINE-MACRON

01AD5
COMBINING MACRON-VERTICAL-LINE

01AD6
COMBINING VERTICAL-LINE-ACUTE-GRAVE

01AD7
COMBINING VERTICAL-LINE-GRAVE-ACUTE

01AD8
COMBINING MACRON-ACUTE-GRAVE

01AD9
COMBINING SHARP SIGN

01ADA
COMBINING FLAT SIGN

01ADB
COMBINING DOWN TACK ABOVE

01ADC
COMBINING DIAERESIS WITH RAISED LEFT DOT

01ADD
COMBINING DOT-AND-RING BELOW

01AE0
COMBINING LEFT TACK ABOVE

01AE1
COMBINING RIGHT TACK ABOVE

01AE2
COMBINING MINUS SIGN ABOVE

01AE3
COMBINING INVERTED BRIDGE ABOVE

01AE4
COMBINING SQUARE ABOVE

01AE5
COMBINING SEAGULL ABOVE

01AE6
COMBINING DOUBLE ARCH BELOW

01AE7
COMBINING DOUBLE ARCH ABOVE

01AE8
COMBINING EQUALS SIGN ABOVE

01AE9
COMBINING LEFT ANGLE CENTRED ABOVE

01AEA
COMBINING UPWARDS ARROW ABOVE

01AEB
COMBINING DOUBLE RIGHTWARDS ARROW ABOVE

01B00
BALINESE SIGN ULU RICEM

01B01
BALINESE SIGN ULU CANDRA

01B02
BALINESE SIGN CECEK

01B03
BALINESE SIGN SURANG

01B04
BALINESE SIGN BISAH

01B05
BALINESE LETTER AKARA

01B06
BALINESE LETTER AKARA TEDUNG

01B07
BALINESE LETTER IKARA

01B08
BALINESE LETTER IKARA TEDUNG

01B09
BALINESE LETTER UKARA

01B0A
BALINESE LETTER UKARA TEDUNG

01B0B
BALINESE LETTER RA REPA

01B0C
BALINESE LETTER RA REPA TEDUNG

01B0D
BALINESE LETTER LA LENGA

01B0E
BALINESE LETTER LA LENGA TEDUNG

01B0F
BALINESE LETTER EKARA

01B10
BALINESE LETTER AIKARA

01B11
BALINESE LETTER OKARA

01B12
BALINESE LETTER OKARA TEDUNG

01B13
BALINESE LETTER KA

01B14
BALINESE LETTER KA MAHAPRANA

01B15
BALINESE LETTER GA

01B16
BALINESE LETTER GA GORA

01B17
BALINESE LETTER NGA

01B18
BALINESE LETTER CA

01B19
BALINESE LETTER CA LACA

01B1A
BALINESE LETTER JA

01B1B
BALINESE LETTER JA JERA

01B1C
BALINESE LETTER NYA

01B1D
BALINESE LETTER TA LATIK

01B1E
BALINESE LETTER TA MURDA MAHAPRANA

01B1F
BALINESE LETTER DA MURDA ALPAPRANA

01B20
BALINESE LETTER DA MURDA MAHAPRANA

01B21
BALINESE LETTER NA RAMBAT

01B22
BALINESE LETTER TA

01B23
BALINESE LETTER TA TAWA

01B24
BALINESE LETTER DA

01B25
BALINESE LETTER DA MADU

01B26
BALINESE LETTER NA

01B27
BALINESE LETTER PA

01B28
BALINESE LETTER PA KAPAL

01B29
BALINESE LETTER BA

01B2A
BALINESE LETTER BA KEMBANG

01B2B
BALINESE LETTER MA

01B2C
BALINESE LETTER YA

01B2D
BALINESE LETTER RA

01B2E
BALINESE LETTER LA

01B2F
BALINESE LETTER WA

01B30
BALINESE LETTER SA SAGA

01B31
BALINESE LETTER SA SAPA

01B32
BALINESE LETTER SA

01B33
BALINESE LETTER HA

01B34
BALINESE SIGN REREKAN

01B35
BALINESE VOWEL SIGN TEDUNG

01B36
BALINESE VOWEL SIGN ULU

01B37
BALINESE VOWEL SIGN ULU SARI

01B38
BALINESE VOWEL SIGN SUKU

01B39
BALINESE VOWEL SIGN SUKU ILUT

01B3A
BALINESE VOWEL SIGN RA REPA

01B3B
BALINESE VOWEL SIGN RA REPA TEDUNG

01B3C
BALINESE VOWEL SIGN LA LENGA

01B3D
BALINESE VOWEL SIGN LA LENGA TEDUNG

01B3E
BALINESE VOWEL SIGN TALING

01B3F
BALINESE VOWEL SIGN TALING REPA

01B40
BALINESE VOWEL SIGN TALING TEDUNG

01B41
BALINESE VOWEL SIGN TALING REPA TEDUNG

01B42
BALINESE VOWEL SIGN PEPET

01B43
BALINESE VOWEL SIGN PEPET TEDUNG

01B44
BALINESE ADEG ADEG

01B45
BALINESE LETTER KAF SASAK

01B46
BALINESE LETTER KHOT SASAK

01B47
BALINESE LETTER TZIR SASAK

01B48
BALINESE LETTER EF SASAK

01B49
BALINESE LETTER VE SASAK

01B4A
BALINESE LETTER ZAL SASAK

01B4B
BALINESE LETTER ASYURA SASAK

01B4C
BALINESE LETTER ARCHAIC JNYA

01B4E
BALINESE INVERTED CARIK SIKI

01B4F
BALINESE INVERTED CARIK PAREREN

01B50
BALINESE DIGIT ZERO

01B51
BALINESE DIGIT ONE

01B52
BALINESE DIGIT TWO

01B53
BALINESE DIGIT THREE

01B54
BALINESE DIGIT FOUR

01B55
BALINESE DIGIT FIVE

01B56
BALINESE DIGIT SIX

01B57
BALINESE DIGIT SEVEN

01B58
BALINESE DIGIT EIGHT

01B59
BALINESE DIGIT NINE

01B5A
BALINESE PANTI

01B5B
BALINESE PAMADA

01B5C
BALINESE WINDU

01B5D
BALINESE CARIK PAMUNGKAH

01B5E
BALINESE CARIK SIKI

01B5F
BALINESE CARIK PAREREN

01B60
BALINESE PAMENENG

01B61
BALINESE MUSICAL SYMBOL DONG

01B62
BALINESE MUSICAL SYMBOL DENG

01B63
BALINESE MUSICAL SYMBOL DUNG

01B64
BALINESE MUSICAL SYMBOL DANG

01B65
BALINESE MUSICAL SYMBOL DANG SURANG

01B66
BALINESE MUSICAL SYMBOL DING

01B67
BALINESE MUSICAL SYMBOL DAENG

01B68
BALINESE MUSICAL SYMBOL DEUNG

01B69
BALINESE MUSICAL SYMBOL DAING

01B6A
BALINESE MUSICAL SYMBOL DANG GEDE

01B6B
BALINESE MUSICAL SYMBOL COMBINING TEGEH

01B6C
BALINESE MUSICAL SYMBOL COMBINING ENDEP

01B6D
BALINESE MUSICAL SYMBOL COMBINING KEMPUL

01B6E
BALINESE MUSICAL SYMBOL COMBINING KEMPLI

01B6F
BALINESE MUSICAL SYMBOL COMBINING JEGOGAN

01B70
BALINESE MUSICAL SYMBOL COMBINING KEMPUL WITH JEGOGAN

01B71
BALINESE MUSICAL SYMBOL COMBINING KEMPLI WITH JEGOGAN

01B72
BALINESE MUSICAL SYMBOL COMBINING BENDE

01B73
BALINESE MUSICAL SYMBOL COMBINING GONG

01B74
BALINESE MUSICAL SYMBOL RIGHT-HAND OPEN DUG

01B75
BALINESE MUSICAL SYMBOL RIGHT-HAND OPEN DAG

01B76
BALINESE MUSICAL SYMBOL RIGHT-HAND CLOSED TUK

01B77
BALINESE MUSICAL SYMBOL RIGHT-HAND CLOSED TAK

01B78
BALINESE MUSICAL SYMBOL LEFT-HAND OPEN PANG

01B79
BALINESE MUSICAL SYMBOL LEFT-HAND OPEN PUNG

01B7A
BALINESE MUSICAL SYMBOL LEFT-HAND CLOSED PLAK

01B7B
BALINESE MUSICAL SYMBOL LEFT-HAND CLOSED PLUK

01B7C
BALINESE MUSICAL SYMBOL LEFT-HAND OPEN PING

01B7D
BALINESE PANTI LANTANG

01B7E
BALINESE PAMADA LANTANG

01B7F
BALINESE PANTI BAWAK

01B80
SUNDANESE SIGN PANYECEK

01B81
SUNDANESE SIGN PANGLAYAR

01B82
SUNDANESE SIGN PANGWISAD

01B83
SUNDANESE LETTER A

01B84
SUNDANESE LETTER I

01B85
SUNDANESE LETTER U

01B86
SUNDANESE LETTER AE

01B87
SUNDANESE LETTER O

01B88
SUNDANESE LETTER E

01B89
SUNDANESE LETTER EU

01B8A
SUNDANESE LETTER KA

01B8B
SUNDANESE LETTER QA

01B8C
SUNDANESE LETTER GA

01B8D
SUNDANESE LETTER NGA

01B8E
SUNDANESE LETTER CA

01B8F
SUNDANESE LETTER JA

01B90
SUNDANESE LETTER ZA

01B91
SUNDANESE LETTER NYA

01B92
SUNDANESE LETTER TA

01B93
SUNDANESE LETTER DA

01B94
SUNDANESE LETTER NA

01B95
SUNDANESE LETTER PA

01B96
SUNDANESE LETTER FA

01B97
SUNDANESE LETTER VA

01B98
SUNDANESE LETTER BA

01B99
SUNDANESE LETTER MA

01B9A
SUNDANESE LETTER YA

01B9B
SUNDANESE LETTER RA

01B9C
SUNDANESE LETTER LA

01B9D
SUNDANESE LETTER WA

01B9E
SUNDANESE LETTER SA

01B9F
SUNDANESE LETTER XA

01BA0
SUNDANESE LETTER HA

01BA1
SUNDANESE CONSONANT SIGN PAMINGKAL

01BA2
SUNDANESE CONSONANT SIGN PANYAKRA

01BA3
SUNDANESE CONSONANT SIGN PANYIKU

01BA4
SUNDANESE VOWEL SIGN PANGHULU

01BA5
SUNDANESE VOWEL SIGN PANYUKU

01BA6
SUNDANESE VOWEL SIGN PANAELAENG

01BA7
SUNDANESE VOWEL SIGN PANOLONG

01BA8
SUNDANESE VOWEL SIGN PAMEPET

01BA9
SUNDANESE VOWEL SIGN PANEULEUNG

01BAA
SUNDANESE SIGN PAMAAEH

01BAB
SUNDANESE SIGN VIRAMA

01BAC
SUNDANESE CONSONANT SIGN PASANGAN MA

01BAD
SUNDANESE CONSONANT SIGN PASANGAN WA

01BAE
SUNDANESE LETTER KHA

01BAF
SUNDANESE LETTER SYA

01BB0
SUNDANESE DIGIT ZERO

01BB1
SUNDANESE DIGIT ONE

01BB2
SUNDANESE DIGIT TWO

01BB3
SUNDANESE DIGIT THREE

01BB4
SUNDANESE DIGIT FOUR

01BB5
SUNDANESE DIGIT FIVE

01BB6
SUNDANESE DIGIT SIX

01BB7
SUNDANESE DIGIT SEVEN

01BB8
SUNDANESE DIGIT EIGHT

01BB9
SUNDANESE DIGIT NINE

01BBA
SUNDANESE AVAGRAHA

01BBB
SUNDANESE LETTER REU

01BBC
SUNDANESE LETTER LEU

01BBD
SUNDANESE LETTER ARCHAIC I

01BBD
SUNDANESE LETTER BHA

01BBE
SUNDANESE LETTER FINAL K

01BBF
SUNDANESE LETTER FINAL M

01BC0
BATAK LETTER A

01BC1
BATAK LETTER SIMALUNGUN A

01BC2
BATAK LETTER HA

01BC3
BATAK LETTER SIMALUNGUN HA

01BC4
BATAK LETTER MANDAILING HA

01BC5
BATAK LETTER BA

01BC6
BATAK LETTER KARO BA

01BC7
BATAK LETTER PA

01BC8
BATAK LETTER SIMALUNGUN PA

01BC9
BATAK LETTER NA

01BCA
BATAK LETTER MANDAILING NA

01BCB
BATAK LETTER WA

01BCC
BATAK LETTER SIMALUNGUN WA

01BCD
BATAK LETTER PAKPAK WA

01BCE
BATAK LETTER GA

01BCF
BATAK LETTER SIMALUNGUN GA

01BD0
BATAK LETTER JA

01BD1
BATAK LETTER DA

01BD2
BATAK LETTER RA

01BD3
BATAK LETTER SIMALUNGUN RA

01BD4
BATAK LETTER MA

01BD5
BATAK LETTER SIMALUNGUN MA

01BD6
BATAK LETTER SOUTHERN TA

01BD7
BATAK LETTER NORTHERN TA

01BD8
BATAK LETTER SA

01BD9
BATAK LETTER SIMALUNGUN SA

01BDA
BATAK LETTER MANDAILING SA

01BDB
BATAK LETTER YA

01BDC
BATAK LETTER SIMALUNGUN YA

01BDD
BATAK LETTER NGA

01BDE
BATAK LETTER LA

01BDF
BATAK LETTER SIMALUNGUN LA

01BE0
BATAK LETTER NYA

01BE1
BATAK LETTER CA

01BE2
BATAK LETTER NDA

01BE3
BATAK LETTER MBA

01BE4
BATAK LETTER I

01BE5
BATAK LETTER U

01BE6
BATAK SIGN TOMPI

01BE7
BATAK VOWEL SIGN E

01BE8
BATAK VOWEL SIGN PAKPAK E

01BE9
BATAK VOWEL SIGN EE

01BEA
BATAK VOWEL SIGN I

01BEB
BATAK VOWEL SIGN KARO I

01BEC
BATAK VOWEL SIGN O

01BED
BATAK VOWEL SIGN KARO O

01BEE
BATAK VOWEL SIGN U

01BEF
BATAK VOWEL SIGN U FOR SIMALUNGUN SA

01BF0
BATAK CONSONANT SIGN NG

01BF1
BATAK CONSONANT SIGN H

01BF2
BATAK PANGOLAT

01BF3
BATAK PANONGONAN

01BFC
BATAK SYMBOL BINDU NA METEK

01BFD
BATAK SYMBOL BINDU PINARBORAS

01BFE
BATAK SYMBOL BINDU JUDUL

01BFF
BATAK SYMBOL BINDU PANGOLAT

01C00
LEPCHA LETTER KA

01C01
LEPCHA LETTER KLA

01C02
LEPCHA LETTER KHA

01C03
LEPCHA LETTER GA

01C04
LEPCHA LETTER GLA

01C05
LEPCHA LETTER NGA

01C06
LEPCHA LETTER CA

01C07
LEPCHA LETTER CHA

01C08
LEPCHA LETTER JA

01C09
LEPCHA LETTER NYA

01C0A
LEPCHA LETTER TA

01C0B
LEPCHA LETTER THA

01C0C
LEPCHA LETTER DA

01C0D
LEPCHA LETTER NA

01C0E
LEPCHA LETTER PA

01C0F
LEPCHA LETTER PLA

01C10
LEPCHA LETTER PHA

01C11
LEPCHA LETTER FA

01C12
LEPCHA LETTER FLA

01C13
LEPCHA LETTER BA

01C14
LEPCHA LETTER BLA

01C15
LEPCHA LETTER MA

01C16
LEPCHA LETTER MLA

01C17
LEPCHA LETTER TSA

01C18
LEPCHA LETTER TSHA

01C19
LEPCHA LETTER DZA

01C1A
LEPCHA LETTER YA

01C1B
LEPCHA LETTER RA

01C1C
LEPCHA LETTER LA

01C1D
LEPCHA LETTER HA

01C1E
LEPCHA LETTER HLA

01C1F
LEPCHA LETTER VA

01C20
LEPCHA LETTER SA

01C21
LEPCHA LETTER SHA

01C22
LEPCHA LETTER WA

01C23
LEPCHA LETTER A

01C24
LEPCHA SUBJOINED LETTER YA

01C25
LEPCHA SUBJOINED LETTER RA

01C26
LEPCHA VOWEL SIGN AA

01C27
LEPCHA VOWEL SIGN I

01C28
LEPCHA VOWEL SIGN O

01C29
LEPCHA VOWEL SIGN OO

01C2A
LEPCHA VOWEL SIGN U

01C2B
LEPCHA VOWEL SIGN UU

01C2C
LEPCHA VOWEL SIGN E

01C2D
LEPCHA CONSONANT SIGN K

01C2E
LEPCHA CONSONANT SIGN M

01C2F
LEPCHA CONSONANT SIGN L

01C30
LEPCHA CONSONANT SIGN N

01C31
LEPCHA CONSONANT SIGN P

01C32
LEPCHA CONSONANT SIGN R

01C33
LEPCHA CONSONANT SIGN T

01C34
LEPCHA CONSONANT SIGN NYIN-DO

01C35
LEPCHA CONSONANT SIGN KANG

01C36
LEPCHA SIGN RAN

01C37
LEPCHA SIGN NUKTA

01C3B
LEPCHA PUNCTUATION TA-ROL

01C3C
LEPCHA PUNCTUATION NYET THYOOM TA-ROL

01C3D
LEPCHA PUNCTUATION CER-WA

01C3E
LEPCHA PUNCTUATION TSHOOK CER-WA

01C3F
LEPCHA PUNCTUATION TSHOOK

01C40
LEPCHA DIGIT ZERO

01C41
LEPCHA DIGIT ONE

01C42
LEPCHA DIGIT TWO

01C43
LEPCHA DIGIT THREE

01C44
LEPCHA DIGIT FOUR

01C45
LEPCHA DIGIT FIVE

01C46
LEPCHA DIGIT SIX

01C47
LEPCHA DIGIT SEVEN

01C48
LEPCHA DIGIT EIGHT

01C49
LEPCHA DIGIT NINE

01C4D
LEPCHA LETTER TTA

01C4E
LEPCHA LETTER TTHA

01C4F
LEPCHA LETTER DDA

01C50
OL CHIKI DIGIT ZERO

01C51
OL CHIKI DIGIT ONE

01C52
OL CHIKI DIGIT TWO

01C53
OL CHIKI DIGIT THREE

01C54
OL CHIKI DIGIT FOUR

01C55
OL CHIKI DIGIT FIVE

01C56
OL CHIKI DIGIT SIX

01C57
OL CHIKI DIGIT SEVEN

01C58
OL CHIKI DIGIT EIGHT

01C59
OL CHIKI DIGIT NINE

01C5A
OL CHIKI LETTER LA

01C5B
OL CHIKI LETTER AT

01C5C
OL CHIKI LETTER AG

01C5D
OL CHIKI LETTER ANG

01C5E
OL CHIKI LETTER AL

01C5F
OL CHIKI LETTER LAA

01C60
OL CHIKI LETTER AAK

01C61
OL CHIKI LETTER AAJ

01C62
OL CHIKI LETTER AAM

01C63
OL CHIKI LETTER AAW

01C64
OL CHIKI LETTER LI

01C65
OL CHIKI LETTER IS

01C66
OL CHIKI LETTER IH

01C67
OL CHIKI LETTER INY

01C68
OL CHIKI LETTER IR

01C69
OL CHIKI LETTER LU

01C6A
OL CHIKI LETTER UC

01C6B
OL CHIKI LETTER UD

01C6C
OL CHIKI LETTER UNN

01C6D
OL CHIKI LETTER UY

01C6E
OL CHIKI LETTER LE

01C6F
OL CHIKI LETTER EP

01C70
OL CHIKI LETTER EDD

01C71
OL CHIKI LETTER EN

01C72
OL CHIKI LETTER ERR

01C73
OL CHIKI LETTER LO

01C74
OL CHIKI LETTER OTT

01C75
OL CHIKI LETTER OB

01C76
OL CHIKI LETTER OV

01C77
OL CHIKI LETTER OH

01C78
OL CHIKI MU TTUDDAG

01C79
OL CHIKI GAAHLAA TTUDDAAG

01C7A
OL CHIKI MU-GAAHLAA TTUDDAAG

01C7B
OL CHIKI RELAA

01C7C
OL CHIKI PHAARKAA

01C7D
OL CHIKI AHAD

01C7E
OL CHIKI PUNCTUATION MUCAAD

01C7F
OL CHIKI PUNCTUATION DOUBLE MUCAAD

01C80
CYRILLIC SMALL LETTER ROUNDED VE

01C81
CYRILLIC SMALL LETTER LONG-LEGGED DE

01C82
CYRILLIC SMALL LETTER NARROW O

01C83
CYRILLIC SMALL LETTER WIDE ES

01C84
CYRILLIC SMALL LETTER TALL TE

01C85
CYRILLIC SMALL LETTER THREE-LEGGED TE

01C86
CYRILLIC SMALL LETTER TALL HARD SIGN

01C87
CYRILLIC SMALL LETTER TALL YAT

01C88
CYRILLIC SMALL LETTER UNBLENDED UK

01C89
CYRILLIC CAPITAL LETTER TJE

01C8A
CYRILLIC SMALL LETTER TJE

01C90
GEORGIAN MTAVRULI CAPITAL LETTER AN

01C91
GEORGIAN MTAVRULI CAPITAL LETTER BAN

01C92
GEORGIAN MTAVRULI CAPITAL LETTER GAN

01C93
GEORGIAN MTAVRULI CAPITAL LETTER DON

01C94
GEORGIAN MTAVRULI CAPITAL LETTER EN

01C95
GEORGIAN MTAVRULI CAPITAL LETTER VIN

01C96
GEORGIAN MTAVRULI CAPITAL LETTER ZEN

01C97
GEORGIAN MTAVRULI CAPITAL LETTER TAN

01C98
GEORGIAN MTAVRULI CAPITAL LETTER IN

01C99
GEORGIAN MTAVRULI CAPITAL LETTER KAN

01C9A
GEORGIAN MTAVRULI CAPITAL LETTER LAS

01C9B
GEORGIAN MTAVRULI CAPITAL LETTER MAN

01C9C
GEORGIAN MTAVRULI CAPITAL LETTER NAR

01C9D
GEORGIAN MTAVRULI CAPITAL LETTER ON

01C9E
GEORGIAN MTAVRULI CAPITAL LETTER PAR

01C9F
GEORGIAN MTAVRULI CAPITAL LETTER ZHAR

01CA0
GEORGIAN MTAVRULI CAPITAL LETTER RAE

01CA1
GEORGIAN MTAVRULI CAPITAL LETTER SAN

01CA2
GEORGIAN MTAVRULI CAPITAL LETTER TAR

01CA3
GEORGIAN MTAVRULI CAPITAL LETTER UN

01CA4
GEORGIAN MTAVRULI CAPITAL LETTER PHAR

01CA5
GEORGIAN MTAVRULI CAPITAL LETTER KHAR

01CA6
GEORGIAN MTAVRULI CAPITAL LETTER GHAN

01CA7
GEORGIAN MTAVRULI CAPITAL LETTER QAR

01CA8
GEORGIAN MTAVRULI CAPITAL LETTER SHIN

01CA9
GEORGIAN MTAVRULI CAPITAL LETTER CHIN

01CAA
GEORGIAN MTAVRULI CAPITAL LETTER CAN

01CAB
GEORGIAN MTAVRULI CAPITAL LETTER JIL

01CAC
GEORGIAN MTAVRULI CAPITAL LETTER CIL

01CAD
GEORGIAN MTAVRULI CAPITAL LETTER CHAR

01CAE
GEORGIAN MTAVRULI CAPITAL LETTER XAN

01CAF
GEORGIAN MTAVRULI CAPITAL LETTER JHAN

01CB0
GEORGIAN MTAVRULI CAPITAL LETTER HAE

01CB1
GEORGIAN MTAVRULI CAPITAL LETTER HE

01CB2
GEORGIAN MTAVRULI CAPITAL LETTER HIE

01CB3
GEORGIAN MTAVRULI CAPITAL LETTER WE

01CB4
GEORGIAN MTAVRULI CAPITAL LETTER HAR

01CB5
GEORGIAN MTAVRULI CAPITAL LETTER HOE

01CB6
GEORGIAN MTAVRULI CAPITAL LETTER FI

01CB7
GEORGIAN MTAVRULI CAPITAL LETTER YN

01CB8
GEORGIAN MTAVRULI CAPITAL LETTER ELIFI

01CB9
GEORGIAN MTAVRULI CAPITAL LETTER TURNED GAN

01CBA
GEORGIAN MTAVRULI CAPITAL LETTER AIN

01CBD
GEORGIAN MTAVRULI CAPITAL LETTER AEN

01CBE
GEORGIAN MTAVRULI CAPITAL LETTER HARD SIGN

01CBF
GEORGIAN MTAVRULI CAPITAL LETTER LABIAL SIGN

01CC0
SUNDANESE PUNCTUATION BINDU SURYA

01CC1
SUNDANESE PUNCTUATION BINDU PANGLONG

01CC2
SUNDANESE PUNCTUATION BINDU PURNAMA

01CC3
SUNDANESE PUNCTUATION BINDU CAKRA

01CC4
SUNDANESE PUNCTUATION BINDU LEU SATANGA

01CC5
SUNDANESE PUNCTUATION BINDU KA SATANGA

01CC6
SUNDANESE PUNCTUATION BINDU DA SATANGA

01CC7
SUNDANESE PUNCTUATION BINDU BA SATANGA

01CD0
VEDIC TONE KARSHANA

01CD1
VEDIC TONE SHARA

01CD2
VEDIC TONE PRENKHA

01CD3
VEDIC SIGN NIHSHVASA

01CD4
VEDIC SIGN YAJURVEDIC MIDLINE SVARITA

01CD5
VEDIC TONE YAJURVEDIC AGGRAVATED INDEPENDENT SVARITA

01CD6
VEDIC TONE YAJURVEDIC INDEPENDENT SVARITA

01CD7
VEDIC TONE YAJURVEDIC KATHAKA INDEPENDENT SVARITA

01CD8
VEDIC TONE CANDRA BELOW

01CD9
VEDIC TONE YAJURVEDIC KATHAKA INDEPENDENT SVARITA SCHROEDER

01CDA
VEDIC TONE DOUBLE SVARITA

01CDB
VEDIC TONE TRIPLE SVARITA

01CDC
VEDIC TONE KATHAKA ANUDATTA

01CDD
VEDIC TONE DOT BELOW

01CDE
VEDIC TONE TWO DOTS BELOW

01CDF
VEDIC TONE THREE DOTS BELOW

01CE0
VEDIC TONE RIGVEDIC KASHMIRI INDEPENDENT SVARITA

01CE1
VEDIC TONE ATHARVAVEDIC INDEPENDENT SVARITA

01CE2
VEDIC SIGN VISARGA SVARITA

01CE3
VEDIC SIGN VISARGA UDATTA

01CE4
VEDIC SIGN REVERSED VISARGA UDATTA

01CE5
VEDIC SIGN VISARGA ANUDATTA

01CE6
VEDIC SIGN REVERSED VISARGA ANUDATTA

01CE7
VEDIC SIGN VISARGA UDATTA WITH TAIL

01CE8
VEDIC SIGN VISARGA ANUDATTA WITH TAIL

01CE9
VEDIC SIGN ANUSVARA ANTARGOMUKHA

01CEA
VEDIC SIGN ANUSVARA BAHIRGOMUKHA

01CEB
VEDIC SIGN ANUSVARA VAMAGOMUKHA

01CEC
VEDIC SIGN ANUSVARA VAMAGOMUKHA WITH TAIL

01CED
VEDIC SIGN TIRYAK

01CEE
VEDIC SIGN HEXIFORM LONG ANUSVARA

01CEF
VEDIC SIGN LONG ANUSVARA

01CF0
VEDIC SIGN RTHANG LONG ANUSVARA

01CF1
VEDIC SIGN ANUSVARA UBHAYATO MUKHA

01CF2
VEDIC SIGN ARDHAVISARGA

01CF3
VEDIC SIGN ROTATED ARDHAVISARGA

01CF4
VEDIC TONE CANDRA ABOVE

01CF5
VEDIC SIGN JIHVAMULIYA

01CF6
VEDIC SIGN UPADHMANIYA

01CF7
VEDIC SIGN ATIKRAMA

01CF8
VEDIC TONE RING ABOVE

01CF9
VEDIC TONE DOUBLE RING ABOVE

01CFA
VEDIC SIGN DOUBLE ANUSVARA ANTARGOMUKHA

01D00
LATIN LETTER SMALL CAPITAL A

01D01
LATIN LETTER SMALL CAPITAL AE

01D02
LATIN SMALL LETTER TURNED AE

01D03
LATIN LETTER SMALL CAPITAL BARRED B

01D04
LATIN LETTER SMALL CAPITAL C

01D05
LATIN LETTER SMALL CAPITAL D

01D06
LATIN LETTER SMALL CAPITAL ETH

01D07
LATIN LETTER SMALL CAPITAL E

01D08
LATIN SMALL LETTER TURNED OPEN E

01D09
LATIN SMALL LETTER TURNED I

01D0A
LATIN LETTER SMALL CAPITAL J

01D0B
LATIN LETTER SMALL CAPITAL K

01D0C
LATIN LETTER SMALL CAPITAL L WITH STROKE

01D0D
LATIN LETTER SMALL CAPITAL M

01D0E
LATIN LETTER SMALL CAPITAL REVERSED N

01D0F
LATIN LETTER SMALL CAPITAL O

01D10
LATIN LETTER SMALL CAPITAL OPEN O

01D11
LATIN SMALL LETTER SIDEWAYS O

01D12
LATIN SMALL LETTER SIDEWAYS OPEN O

01D13
LATIN SMALL LETTER SIDEWAYS O WITH STROKE

01D14
LATIN SMALL LETTER TURNED OE

01D15
LATIN LETTER SMALL CAPITAL OU

01D16
LATIN SMALL LETTER TOP HALF O

01D17
LATIN SMALL LETTER BOTTOM HALF O

01D18
LATIN LETTER SMALL CAPITAL P

01D19
LATIN LETTER SMALL CAPITAL REVERSED R

01D1A
LATIN LETTER SMALL CAPITAL TURNED R

01D1B
LATIN LETTER SMALL CAPITAL T

01D1C
LATIN LETTER SMALL CAPITAL U

01D1D
LATIN SMALL LETTER SIDEWAYS U

01D1E
LATIN SMALL LETTER SIDEWAYS DIAERESIZED U

01D1F
LATIN SMALL LETTER SIDEWAYS TURNED M

01D20
LATIN LETTER SMALL CAPITAL V

01D21
LATIN LETTER SMALL CAPITAL W

01D22
LATIN LETTER SMALL CAPITAL Z

01D23
LATIN LETTER SMALL CAPITAL EZH

01D24
LATIN LETTER VOICED LARYNGEAL SPIRANT

01D25
LATIN LETTER AIN

01D26
GREEK LETTER SMALL CAPITAL GAMMA

01D27
GREEK LETTER SMALL CAPITAL LAMDA

01D28
GREEK LETTER SMALL CAPITAL PI

01D29
GREEK LETTER SMALL CAPITAL RHO

01D2A
GREEK LETTER SMALL CAPITAL PSI

01D2B
CYRILLIC LETTER SMALL CAPITAL EL

01D2C
MODIFIER LETTER CAPITAL A

01D2D
MODIFIER LETTER CAPITAL AE

01D2E
MODIFIER LETTER CAPITAL B

01D2F
MODIFIER LETTER CAPITAL BARRED B

01D30
MODIFIER LETTER CAPITAL D

01D31
MODIFIER LETTER CAPITAL E

01D32
MODIFIER LETTER CAPITAL REVERSED E

01D33
MODIFIER LETTER CAPITAL G

01D34
MODIFIER LETTER CAPITAL H

01D35
MODIFIER LETTER CAPITAL I

01D36
MODIFIER LETTER CAPITAL J

01D37
MODIFIER LETTER CAPITAL K

01D38
MODIFIER LETTER CAPITAL L

01D39
MODIFIER LETTER CAPITAL M

01D3A
MODIFIER LETTER CAPITAL N

01D3B
MODIFIER LETTER CAPITAL REVERSED N

01D3C
MODIFIER LETTER CAPITAL O

01D3D
MODIFIER LETTER CAPITAL OU

01D3E
MODIFIER LETTER CAPITAL P

01D3F
MODIFIER LETTER CAPITAL R

01D40
MODIFIER LETTER CAPITAL T

01D41
MODIFIER LETTER CAPITAL U

01D42
MODIFIER LETTER CAPITAL W

01D43
MODIFIER LETTER SMALL A

01D44
MODIFIER LETTER SMALL TURNED A

01D45
MODIFIER LETTER SMALL ALPHA

01D46
MODIFIER LETTER SMALL TURNED AE

01D47
MODIFIER LETTER SMALL B

01D48
MODIFIER LETTER SMALL D

01D49
MODIFIER LETTER SMALL E

01D4A
MODIFIER LETTER SMALL SCHWA

01D4B
MODIFIER LETTER SMALL OPEN E

01D4C
MODIFIER LETTER SMALL TURNED OPEN E

01D4D
MODIFIER LETTER SMALL G

01D4E
MODIFIER LETTER SMALL TURNED I

01D4F
MODIFIER LETTER SMALL K

01D50
MODIFIER LETTER SMALL M

01D51
MODIFIER LETTER SMALL ENG

01D52
MODIFIER LETTER SMALL O

01D53
MODIFIER LETTER SMALL OPEN O

01D54
MODIFIER LETTER SMALL TOP HALF O

01D55
MODIFIER LETTER SMALL BOTTOM HALF O

01D56
MODIFIER LETTER SMALL P

01D57
MODIFIER LETTER SMALL T

01D58
MODIFIER LETTER SMALL U

01D59
MODIFIER LETTER SMALL SIDEWAYS U

01D5A
MODIFIER LETTER SMALL TURNED M

01D5B
MODIFIER LETTER SMALL V

01D5C
MODIFIER LETTER SMALL AIN

01D5D
MODIFIER LETTER SMALL BETA

01D5E
MODIFIER LETTER SMALL GREEK GAMMA

01D5F
MODIFIER LETTER SMALL DELTA

01D60
MODIFIER LETTER SMALL GREEK PHI

01D61
MODIFIER LETTER SMALL CHI

01D62
LATIN SUBSCRIPT SMALL LETTER I

01D63
LATIN SUBSCRIPT SMALL LETTER R

01D64
LATIN SUBSCRIPT SMALL LETTER U

01D65
LATIN SUBSCRIPT SMALL LETTER V

01D66
GREEK SUBSCRIPT SMALL LETTER BETA

01D67
GREEK SUBSCRIPT SMALL LETTER GAMMA

01D68
GREEK SUBSCRIPT SMALL LETTER RHO

01D69
GREEK SUBSCRIPT SMALL LETTER PHI

01D6A
GREEK SUBSCRIPT SMALL LETTER CHI

01D6B
LATIN SMALL LETTER UE

01D6C
LATIN SMALL LETTER B WITH MIDDLE TILDE

01D6D
LATIN SMALL LETTER D WITH MIDDLE TILDE

01D6E
LATIN SMALL LETTER F WITH MIDDLE TILDE

01D6F
LATIN SMALL LETTER M WITH MIDDLE TILDE

01D70
LATIN SMALL LETTER N WITH MIDDLE TILDE

01D71
LATIN SMALL LETTER P WITH MIDDLE TILDE

01D72
LATIN SMALL LETTER R WITH MIDDLE TILDE

01D73
LATIN SMALL LETTER R WITH FISHHOOK AND MIDDLE TILDE

01D74
LATIN SMALL LETTER S WITH MIDDLE TILDE

01D75
LATIN SMALL LETTER T WITH MIDDLE TILDE

01D76
LATIN SMALL LETTER Z WITH MIDDLE TILDE

01D77
LATIN SMALL LETTER TURNED G

01D78
MODIFIER LETTER CYRILLIC EN

01D79
LATIN SMALL LETTER INSULAR G

01D7A
LATIN SMALL LETTER TH WITH STRIKETHROUGH

01D7B
LATIN SMALL CAPITAL LETTER I WITH STROKE

01D7C
LATIN SMALL LETTER IOTA WITH STROKE

01D7D
LATIN SMALL LETTER P WITH STROKE

01D7E
LATIN SMALL CAPITAL LETTER U WITH STROKE

01D7F
LATIN SMALL LETTER UPSILON WITH STROKE

01D80
LATIN SMALL LETTER B WITH PALATAL HOOK

01D81
LATIN SMALL LETTER D WITH PALATAL HOOK

01D82
LATIN SMALL LETTER F WITH PALATAL HOOK

01D83
LATIN SMALL LETTER G WITH PALATAL HOOK

01D84
LATIN SMALL LETTER K WITH PALATAL HOOK

01D85
LATIN SMALL LETTER L WITH PALATAL HOOK

01D86
LATIN SMALL LETTER M WITH PALATAL HOOK

01D87
LATIN SMALL LETTER N WITH PALATAL HOOK

01D88
LATIN SMALL LETTER P WITH PALATAL HOOK

01D89
LATIN SMALL LETTER R WITH PALATAL HOOK

01D8A
LATIN SMALL LETTER S WITH PALATAL HOOK

01D8B
LATIN SMALL LETTER ESH WITH PALATAL HOOK

01D8C
LATIN SMALL LETTER V WITH PALATAL HOOK

01D8D
LATIN SMALL LETTER X WITH PALATAL HOOK

01D8E
LATIN SMALL LETTER Z WITH PALATAL HOOK

01D8F
LATIN SMALL LETTER A WITH RETROFLEX HOOK

01D90
LATIN SMALL LETTER ALPHA WITH RETROFLEX HOOK

01D91
LATIN SMALL LETTER D WITH HOOK AND TAIL

01D92
LATIN SMALL LETTER E WITH RETROFLEX HOOK

01D93
LATIN SMALL LETTER OPEN E WITH RETROFLEX HOOK

01D94
LATIN SMALL LETTER REVERSED OPEN E WITH RETROFLEX HOOK

01D95
LATIN SMALL LETTER SCHWA WITH RETROFLEX HOOK

01D96
LATIN SMALL LETTER I WITH RETROFLEX HOOK

01D97
LATIN SMALL LETTER OPEN O WITH RETROFLEX HOOK

01D98
LATIN SMALL LETTER ESH WITH RETROFLEX HOOK

01D99
LATIN SMALL LETTER U WITH RETROFLEX HOOK

01D9A
LATIN SMALL LETTER EZH WITH RETROFLEX HOOK

01D9B
MODIFIER LETTER SMALL TURNED ALPHA

01D9C
MODIFIER LETTER SMALL C

01D9D
MODIFIER LETTER SMALL C WITH CURL

01D9E
MODIFIER LETTER SMALL ETH

01D9F
MODIFIER LETTER SMALL REVERSED OPEN E

01DA0
MODIFIER LETTER SMALL F

01DA1
MODIFIER LETTER SMALL DOTLESS J WITH STROKE

01DA2
MODIFIER LETTER SMALL SCRIPT G

01DA3
MODIFIER LETTER SMALL TURNED H

01DA4
MODIFIER LETTER SMALL I WITH STROKE

01DA5
MODIFIER LETTER SMALL IOTA

01DA6
MODIFIER LETTER SMALL CAPITAL I

01DA7
MODIFIER LETTER SMALL CAPITAL I WITH STROKE

01DA8
MODIFIER LETTER SMALL J WITH CROSSED-TAIL

01DA9
MODIFIER LETTER SMALL L WITH RETROFLEX HOOK

01DAA
MODIFIER LETTER SMALL L WITH PALATAL HOOK

01DAB
MODIFIER LETTER SMALL CAPITAL L

01DAC
MODIFIER LETTER SMALL M WITH HOOK

01DAD
MODIFIER LETTER SMALL TURNED M WITH LONG LEG

01DAE
MODIFIER LETTER SMALL N WITH LEFT HOOK

01DAF
MODIFIER LETTER SMALL N WITH RETROFLEX HOOK

01DB0
MODIFIER LETTER SMALL CAPITAL N

01DB1
MODIFIER LETTER SMALL BARRED O

01DB2
MODIFIER LETTER SMALL PHI

01DB3
MODIFIER LETTER SMALL S WITH HOOK

01DB4
MODIFIER LETTER SMALL ESH

01DB5
MODIFIER LETTER SMALL T WITH PALATAL HOOK

01DB6
MODIFIER LETTER SMALL U BAR

01DB7
MODIFIER LETTER SMALL UPSILON

01DB8
MODIFIER LETTER SMALL CAPITAL U

01DB9
MODIFIER LETTER SMALL V WITH HOOK

01DBA
MODIFIER LETTER SMALL TURNED V

01DBB
MODIFIER LETTER SMALL Z

01DBC
MODIFIER LETTER SMALL Z WITH RETROFLEX HOOK

01DBD
MODIFIER LETTER SMALL Z WITH CURL

01DBE
MODIFIER LETTER SMALL EZH

01DBF
MODIFIER LETTER SMALL THETA

01DC0
COMBINING DOTTED GRAVE ACCENT

01DC1
COMBINING DOTTED ACUTE ACCENT

01DC2
COMBINING SNAKE BELOW

01DC3
COMBINING SUSPENSION MARK

01DC4
COMBINING MACRON-ACUTE

01DC5
COMBINING GRAVE-MACRON

01DC6
COMBINING MACRON-GRAVE

01DC7
COMBINING ACUTE-MACRON

01DC8
COMBINING GRAVE-ACUTE-GRAVE

01DC9
COMBINING ACUTE-GRAVE-ACUTE

01DCA
COMBINING LATIN SMALL LETTER R BELOW

01DCB
COMBINING BREVE-MACRON

01DCC
COMBINING MACRON-BREVE

01DCD
COMBINING DOUBLE CIRCUMFLEX ABOVE

01DCE
COMBINING OGONEK ABOVE

01DCF
COMBINING ZIGZAG BELOW

01DD0
COMBINING IS BELOW

01DD1
COMBINING UR ABOVE

01DD2
COMBINING US ABOVE

01DD3
COMBINING LATIN SMALL LETTER FLATTENED OPEN A ABOVE

01DD4
COMBINING LATIN SMALL LETTER AE

01DD5
COMBINING LATIN SMALL LETTER AO

01DD6
COMBINING LATIN SMALL LETTER AV

01DD7
COMBINING LATIN SMALL LETTER C CEDILLA

01DD8
COMBINING LATIN SMALL LETTER INSULAR D

01DD9
COMBINING LATIN SMALL LETTER ETH

01DDA
COMBINING LATIN SMALL LETTER G

01DDB
COMBINING LATIN LETTER SMALL CAPITAL G

01DDC
COMBINING LATIN SMALL LETTER K

01DDD
COMBINING LATIN SMALL LETTER L

01DDE
COMBINING LATIN LETTER SMALL CAPITAL L

01DDF
COMBINING LATIN LETTER SMALL CAPITAL M

01DE0
COMBINING LATIN SMALL LETTER N

01DE1
COMBINING LATIN LETTER SMALL CAPITAL N

01DE2
COMBINING LATIN LETTER SMALL CAPITAL R

01DE3
COMBINING LATIN SMALL LETTER R ROTUNDA

01DE4
COMBINING LATIN SMALL LETTER S

01DE5
COMBINING LATIN SMALL LETTER LONG S

01DE6
COMBINING LATIN SMALL LETTER Z

01DE7
COMBINING LATIN SMALL LETTER ALPHA

01DE8
COMBINING LATIN SMALL LETTER B

01DE9
COMBINING LATIN SMALL LETTER BETA

01DEA
COMBINING LATIN SMALL LETTER SCHWA

01DEB
COMBINING LATIN SMALL LETTER F

01DEC
COMBINING LATIN SMALL LETTER L WITH DOUBLE MIDDLE TILDE

01DED
COMBINING LATIN SMALL LETTER O WITH LIGHT CENTRALIZATION STROKE

01DEE
COMBINING LATIN SMALL LETTER P

01DEF
COMBINING LATIN SMALL LETTER ESH

01DF0
COMBINING LATIN SMALL LETTER U WITH LIGHT CENTRALIZATION STROKE

01DF1
COMBINING LATIN SMALL LETTER W

01DF2
COMBINING LATIN SMALL LETTER A WITH DIAERESIS

01DF3
COMBINING LATIN SMALL LETTER O WITH DIAERESIS

01DF4
COMBINING LATIN SMALL LETTER U WITH DIAERESIS

01DF5
COMBINING UP TACK ABOVE

01DF6
COMBINING KAVYKA ABOVE RIGHT

01DF7
COMBINING KAVYKA ABOVE LEFT

01DF8
COMBINING DOT ABOVE LEFT

01DF9
COMBINING WIDE INVERTED BRIDGE BELOW

01DFA
COMBINING DOT BELOW LEFT

01DFB
COMBINING DELETION MARK

01DFC
COMBINING DOUBLE INVERTED BREVE BELOW

01DFD
COMBINING ALMOST EQUAL TO BELOW

01DFE
COMBINING LEFT ARROWHEAD ABOVE

01DFF
COMBINING RIGHT ARROWHEAD AND DOWN ARROWHEAD BELOW

01E00
LATIN CAPITAL LETTER A WITH RING BELOW

01E01
LATIN SMALL LETTER A WITH RING BELOW

01E02
LATIN CAPITAL LETTER B WITH DOT ABOVE

01E03
LATIN SMALL LETTER B WITH DOT ABOVE

01E04
LATIN CAPITAL LETTER B WITH DOT BELOW

01E05
LATIN SMALL LETTER B WITH DOT BELOW

01E06
LATIN CAPITAL LETTER B WITH LINE BELOW

01E07
LATIN SMALL LETTER B WITH LINE BELOW

01E08
LATIN CAPITAL LETTER C WITH CEDILLA AND ACUTE

01E09
LATIN SMALL LETTER C WITH CEDILLA AND ACUTE

01E0A
LATIN CAPITAL LETTER D WITH DOT ABOVE

01E0B
LATIN SMALL LETTER D WITH DOT ABOVE

01E0C
LATIN CAPITAL LETTER D WITH DOT BELOW

01E0D
LATIN SMALL LETTER D WITH DOT BELOW

01E0E
LATIN CAPITAL LETTER D WITH LINE BELOW

01E0F
LATIN SMALL LETTER D WITH LINE BELOW

01E10
LATIN CAPITAL LETTER D WITH CEDILLA

01E11
LATIN SMALL LETTER D WITH CEDILLA

01E12
LATIN CAPITAL LETTER D WITH CIRCUMFLEX BELOW

01E13
LATIN SMALL LETTER D WITH CIRCUMFLEX BELOW

01E14
LATIN CAPITAL LETTER E WITH MACRON AND GRAVE

01E15
LATIN SMALL LETTER E WITH MACRON AND GRAVE

01E16
LATIN CAPITAL LETTER E WITH MACRON AND ACUTE

01E17
LATIN SMALL LETTER E WITH MACRON AND ACUTE

01E18
LATIN CAPITAL LETTER E WITH CIRCUMFLEX BELOW

01E19
LATIN SMALL LETTER E WITH CIRCUMFLEX BELOW

01E1A
LATIN CAPITAL LETTER E WITH TILDE BELOW

01E1B
LATIN SMALL LETTER E WITH TILDE BELOW

01E1C
LATIN CAPITAL LETTER E WITH CEDILLA AND BREVE

01E1D
LATIN SMALL LETTER E WITH CEDILLA AND BREVE

01E1E
LATIN CAPITAL LETTER F WITH DOT ABOVE

01E1F
LATIN SMALL LETTER F WITH DOT ABOVE

01E20
LATIN CAPITAL LETTER G WITH MACRON

01E21
LATIN SMALL LETTER G WITH MACRON

01E22
LATIN CAPITAL LETTER H WITH DOT ABOVE

01E23
LATIN SMALL LETTER H WITH DOT ABOVE

01E24
LATIN CAPITAL LETTER H WITH DOT BELOW

01E25
LATIN SMALL LETTER H WITH DOT BELOW

01E26
LATIN CAPITAL LETTER H WITH DIAERESIS

01E27
LATIN SMALL LETTER H WITH DIAERESIS

01E28
LATIN CAPITAL LETTER H WITH CEDILLA

01E29
LATIN SMALL LETTER H WITH CEDILLA

01E2A
LATIN CAPITAL LETTER H WITH BREVE BELOW

01E2B
LATIN SMALL LETTER H WITH BREVE BELOW

01E2C
LATIN CAPITAL LETTER I WITH TILDE BELOW

01E2D
LATIN SMALL LETTER I WITH TILDE BELOW

01E2E
LATIN CAPITAL LETTER I WITH DIAERESIS AND ACUTE

01E2F
LATIN SMALL LETTER I WITH DIAERESIS AND ACUTE

01E30
LATIN CAPITAL LETTER K WITH ACUTE

01E31
LATIN SMALL LETTER K WITH ACUTE

01E32
LATIN CAPITAL LETTER K WITH DOT BELOW

01E33
LATIN SMALL LETTER K WITH DOT BELOW

01E34
LATIN CAPITAL LETTER K WITH LINE BELOW

01E35
LATIN SMALL LETTER K WITH LINE BELOW

01E36
LATIN CAPITAL LETTER L WITH DOT BELOW

01E37
LATIN SMALL LETTER L WITH DOT BELOW

01E38
LATIN CAPITAL LETTER L WITH DOT BELOW AND MACRON

01E39
LATIN SMALL LETTER L WITH DOT BELOW AND MACRON

01E3A
LATIN CAPITAL LETTER L WITH LINE BELOW

01E3B
LATIN SMALL LETTER L WITH LINE BELOW

01E3C
LATIN CAPITAL LETTER L WITH CIRCUMFLEX BELOW

01E3D
LATIN SMALL LETTER L WITH CIRCUMFLEX BELOW

01E3E
LATIN CAPITAL LETTER M WITH ACUTE

01E3F
LATIN SMALL LETTER M WITH ACUTE

01E40
LATIN CAPITAL LETTER M WITH DOT ABOVE

01E41
LATIN SMALL LETTER M WITH DOT ABOVE

01E42
LATIN CAPITAL LETTER M WITH DOT BELOW

01E43
LATIN SMALL LETTER M WITH DOT BELOW

01E44
LATIN CAPITAL LETTER N WITH DOT ABOVE

01E45
LATIN SMALL LETTER N WITH DOT ABOVE

01E46
LATIN CAPITAL LETTER N WITH DOT BELOW

01E47
LATIN SMALL LETTER N WITH DOT BELOW

01E48
LATIN CAPITAL LETTER N WITH LINE BELOW

01E49
LATIN SMALL LETTER N WITH LINE BELOW

01E4A
LATIN CAPITAL LETTER N WITH CIRCUMFLEX BELOW

01E4B
LATIN SMALL LETTER N WITH CIRCUMFLEX BELOW

01E4C
LATIN CAPITAL LETTER O WITH TILDE AND ACUTE

01E4D
LATIN SMALL LETTER O WITH TILDE AND ACUTE

01E4E
LATIN CAPITAL LETTER O WITH TILDE AND DIAERESIS

01E4F
LATIN SMALL LETTER O WITH TILDE AND DIAERESIS

01E50
LATIN CAPITAL LETTER O WITH MACRON AND GRAVE

01E51
LATIN SMALL LETTER O WITH MACRON AND GRAVE

01E52
LATIN CAPITAL LETTER O WITH MACRON AND ACUTE

01E53
LATIN SMALL LETTER O WITH MACRON AND ACUTE

01E54
LATIN CAPITAL LETTER P WITH ACUTE

01E55
LATIN SMALL LETTER P WITH ACUTE

01E56
LATIN CAPITAL LETTER P WITH DOT ABOVE

01E57
LATIN SMALL LETTER P WITH DOT ABOVE

01E58
LATIN CAPITAL LETTER R WITH DOT ABOVE

01E59
LATIN SMALL LETTER R WITH DOT ABOVE

01E5A
LATIN CAPITAL LETTER R WITH DOT BELOW

01E5B
LATIN SMALL LETTER R WITH DOT BELOW

01E5C
LATIN CAPITAL LETTER R WITH DOT BELOW AND MACRON

01E5D
LATIN SMALL LETTER R WITH DOT BELOW AND MACRON

01E5E
LATIN CAPITAL LETTER R WITH LINE BELOW

01E5F
LATIN SMALL LETTER R WITH LINE BELOW

01E60
LATIN CAPITAL LETTER S WITH DOT ABOVE

01E61
LATIN SMALL LETTER S WITH DOT ABOVE

01E62
LATIN CAPITAL LETTER S WITH DOT BELOW

01E63
LATIN SMALL LETTER S WITH DOT BELOW

01E64
LATIN CAPITAL LETTER S WITH ACUTE AND DOT ABOVE

01E65
LATIN SMALL LETTER S WITH ACUTE AND DOT ABOVE

01E66
LATIN CAPITAL LETTER S WITH CARON AND DOT ABOVE

01E67
LATIN SMALL LETTER S WITH CARON AND DOT ABOVE

01E68
LATIN CAPITAL LETTER S WITH DOT BELOW AND DOT ABOVE

01E69
LATIN SMALL LETTER S WITH DOT BELOW AND DOT ABOVE

01E6A
LATIN CAPITAL LETTER T WITH DOT ABOVE

01E6B
LATIN SMALL LETTER T WITH DOT ABOVE

01E6C
LATIN CAPITAL LETTER T WITH DOT BELOW

01E6D
LATIN SMALL LETTER T WITH DOT BELOW

01E6E
LATIN CAPITAL LETTER T WITH LINE BELOW

01E6F
LATIN SMALL LETTER T WITH LINE BELOW

01E70
LATIN CAPITAL LETTER T WITH CIRCUMFLEX BELOW

01E71
LATIN SMALL LETTER T WITH CIRCUMFLEX BELOW

01E72
LATIN CAPITAL LETTER U WITH DIAERESIS BELOW

01E73
LATIN SMALL LETTER U WITH DIAERESIS BELOW

01E74
LATIN CAPITAL LETTER U WITH TILDE BELOW

01E75
LATIN SMALL LETTER U WITH TILDE BELOW

01E76
LATIN CAPITAL LETTER U WITH CIRCUMFLEX BELOW

01E77
LATIN SMALL LETTER U WITH CIRCUMFLEX BELOW

01E78
LATIN CAPITAL LETTER U WITH TILDE AND ACUTE

01E79
LATIN SMALL LETTER U WITH TILDE AND ACUTE

01E7A
LATIN CAPITAL LETTER U WITH MACRON AND DIAERESIS

01E7B
LATIN SMALL LETTER U WITH MACRON AND DIAERESIS

01E7C
LATIN CAPITAL LETTER V WITH TILDE

01E7D
LATIN SMALL LETTER V WITH TILDE

01E7E
LATIN CAPITAL LETTER V WITH DOT BELOW

01E7F
LATIN SMALL LETTER V WITH DOT BELOW

01E80
LATIN CAPITAL LETTER W WITH GRAVE

01E81
LATIN SMALL LETTER W WITH GRAVE

01E82
LATIN CAPITAL LETTER W WITH ACUTE

01E83
LATIN SMALL LETTER W WITH ACUTE

01E84
LATIN CAPITAL LETTER W WITH DIAERESIS

01E85
LATIN SMALL LETTER W WITH DIAERESIS

01E86
LATIN CAPITAL LETTER W WITH DOT ABOVE

01E87
LATIN SMALL LETTER W WITH DOT ABOVE

01E88
LATIN CAPITAL LETTER W WITH DOT BELOW

01E89
LATIN SMALL LETTER W WITH DOT BELOW

01E8A
LATIN CAPITAL LETTER X WITH DOT ABOVE

01E8B
LATIN SMALL LETTER X WITH DOT ABOVE

01E8C
LATIN CAPITAL LETTER X WITH DIAERESIS

01E8D
LATIN SMALL LETTER X WITH DIAERESIS

01E8E
LATIN CAPITAL LETTER Y WITH DOT ABOVE

01E8F
LATIN SMALL LETTER Y WITH DOT ABOVE

01E90
LATIN CAPITAL LETTER Z WITH CIRCUMFLEX

01E91
LATIN SMALL LETTER Z WITH CIRCUMFLEX

01E92
LATIN CAPITAL LETTER Z WITH DOT BELOW

01E93
LATIN SMALL LETTER Z WITH DOT BELOW

01E94
LATIN CAPITAL LETTER Z WITH LINE BELOW

01E95
LATIN SMALL LETTER Z WITH LINE BELOW

01E96
LATIN SMALL LETTER H WITH LINE BELOW

01E97
LATIN SMALL LETTER T WITH DIAERESIS

01E98
LATIN SMALL LETTER W WITH RING ABOVE

01E99
LATIN SMALL LETTER Y WITH RING ABOVE

01E9A
LATIN SMALL LETTER A WITH RIGHT HALF RING

01E9B
LATIN SMALL LETTER LONG S WITH DOT ABOVE

01E9C
LATIN SMALL LETTER LONG S WITH DIAGONAL STROKE

01E9D
LATIN SMALL LETTER LONG S WITH HIGH STROKE

01E9E
LATIN CAPITAL LETTER SHARP S

01E9F
LATIN SMALL LETTER DELTA

01EA0
LATIN CAPITAL LETTER A WITH DOT BELOW

01EA1
LATIN SMALL LETTER A WITH DOT BELOW

01EA2
LATIN CAPITAL LETTER A WITH HOOK ABOVE

01EA3
LATIN SMALL LETTER A WITH HOOK ABOVE

01EA4
LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND ACUTE

01EA5
LATIN SMALL LETTER A WITH CIRCUMFLEX AND ACUTE

01EA6
LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND GRAVE

01EA7
LATIN SMALL LETTER A WITH CIRCUMFLEX AND GRAVE

01EA8
LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND HOOK ABOVE

01EA9
LATIN SMALL LETTER A WITH CIRCUMFLEX AND HOOK ABOVE

01EAA
LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND TILDE

01EAB
LATIN SMALL LETTER A WITH CIRCUMFLEX AND TILDE

01EAC
LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND DOT BELOW

01EAD
LATIN SMALL LETTER A WITH CIRCUMFLEX AND DOT BELOW

01EAE
LATIN CAPITAL LETTER A WITH BREVE AND ACUTE

01EAF
LATIN SMALL LETTER A WITH BREVE AND ACUTE

01EB0
LATIN CAPITAL LETTER A WITH BREVE AND GRAVE

01EB1
LATIN SMALL LETTER A WITH BREVE AND GRAVE

01EB2
LATIN CAPITAL LETTER A WITH BREVE AND HOOK ABOVE

01EB3
LATIN SMALL LETTER A WITH BREVE AND HOOK ABOVE

01EB4
LATIN CAPITAL LETTER A WITH BREVE AND TILDE

01EB5
LATIN SMALL LETTER A WITH BREVE AND TILDE

01EB6
LATIN CAPITAL LETTER A WITH BREVE AND DOT BELOW

01EB7
LATIN SMALL LETTER A WITH BREVE AND DOT BELOW

01EB8
LATIN CAPITAL LETTER E WITH DOT BELOW

01EB9
LATIN SMALL LETTER E WITH DOT BELOW

01EBA
LATIN CAPITAL LETTER E WITH HOOK ABOVE

01EBB
LATIN SMALL LETTER E WITH HOOK ABOVE

01EBC
LATIN CAPITAL LETTER E WITH TILDE

01EBD
LATIN SMALL LETTER E WITH TILDE

01EBE
LATIN CAPITAL LETTER E WITH CIRCUMFLEX AND ACUTE

01EBF
LATIN SMALL LETTER E WITH CIRCUMFLEX AND ACUTE

01EC0
LATIN CAPITAL LETTER E WITH CIRCUMFLEX AND GRAVE

01EC1
LATIN SMALL LETTER E WITH CIRCUMFLEX AND GRAVE

01EC2
LATIN CAPITAL LETTER E WITH CIRCUMFLEX AND HOOK ABOVE

01EC3
LATIN SMALL LETTER E WITH CIRCUMFLEX AND HOOK ABOVE

01EC4
LATIN CAPITAL LETTER E WITH CIRCUMFLEX AND TILDE

01EC5
LATIN SMALL LETTER E WITH CIRCUMFLEX AND TILDE

01EC6
LATIN CAPITAL LETTER E WITH CIRCUMFLEX AND DOT BELOW

01EC7
LATIN SMALL LETTER E WITH CIRCUMFLEX AND DOT BELOW

01EC8
LATIN CAPITAL LETTER I WITH HOOK ABOVE

01EC9
LATIN SMALL LETTER I WITH HOOK ABOVE

01ECA
LATIN CAPITAL LETTER I WITH DOT BELOW

01ECB
LATIN SMALL LETTER I WITH DOT BELOW

01ECC
LATIN CAPITAL LETTER O WITH DOT BELOW

01ECD
LATIN SMALL LETTER O WITH DOT BELOW

01ECE
LATIN CAPITAL LETTER O WITH HOOK ABOVE

01ECF
LATIN SMALL LETTER O WITH HOOK ABOVE

01ED0
LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND ACUTE

01ED1
LATIN SMALL LETTER O WITH CIRCUMFLEX AND ACUTE

01ED2
LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND GRAVE

01ED3
LATIN SMALL LETTER O WITH CIRCUMFLEX AND GRAVE

01ED4
LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND HOOK ABOVE

01ED5
LATIN SMALL LETTER O WITH CIRCUMFLEX AND HOOK ABOVE

01ED6
LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND TILDE

01ED7
LATIN SMALL LETTER O WITH CIRCUMFLEX AND TILDE

01ED8
LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND DOT BELOW

01ED9
LATIN SMALL LETTER O WITH CIRCUMFLEX AND DOT BELOW

01EDA
LATIN CAPITAL LETTER O WITH HORN AND ACUTE

01EDB
LATIN SMALL LETTER O WITH HORN AND ACUTE

01EDC
LATIN CAPITAL LETTER O WITH HORN AND GRAVE

01EDD
LATIN SMALL LETTER O WITH HORN AND GRAVE

01EDE
LATIN CAPITAL LETTER O WITH HORN AND HOOK ABOVE

01EDF
LATIN SMALL LETTER O WITH HORN AND HOOK ABOVE

01EE0
LATIN CAPITAL LETTER O WITH HORN AND TILDE

01EE1
LATIN SMALL LETTER O WITH HORN AND TILDE

01EE2
LATIN CAPITAL LETTER O WITH HORN AND DOT BELOW

01EE3
LATIN SMALL LETTER O WITH HORN AND DOT BELOW

01EE4
LATIN CAPITAL LETTER U WITH DOT BELOW

01EE5
LATIN SMALL LETTER U WITH DOT BELOW

01EE6
LATIN CAPITAL LETTER U WITH HOOK ABOVE

01EE7
LATIN SMALL LETTER U WITH HOOK ABOVE

01EE8
LATIN CAPITAL LETTER U WITH HORN AND ACUTE

01EE9
LATIN SMALL LETTER U WITH HORN AND ACUTE

01EEA
LATIN CAPITAL LETTER U WITH HORN AND GRAVE

01EEB
LATIN SMALL LETTER U WITH HORN AND GRAVE

01EEC
LATIN CAPITAL LETTER U WITH HORN AND HOOK ABOVE

01EED
LATIN SMALL LETTER U WITH HORN AND HOOK ABOVE

01EEE
LATIN CAPITAL LETTER U WITH HORN AND TILDE

01EEF
LATIN SMALL LETTER U WITH HORN AND TILDE

01EF0
LATIN CAPITAL LETTER U WITH HORN AND DOT BELOW

01EF1
LATIN SMALL LETTER U WITH HORN AND DOT BELOW

01EF2
LATIN CAPITAL LETTER Y WITH GRAVE

01EF3
LATIN SMALL LETTER Y WITH GRAVE

01EF4
LATIN CAPITAL LETTER Y WITH DOT BELOW

01EF5
LATIN SMALL LETTER Y WITH DOT BELOW

01EF6
LATIN CAPITAL LETTER Y WITH HOOK ABOVE

01EF7
LATIN SMALL LETTER Y WITH HOOK ABOVE

01EF8
LATIN CAPITAL LETTER Y WITH TILDE

01EF9
LATIN SMALL LETTER Y WITH TILDE

01EFA
LATIN CAPITAL LETTER MIDDLE-WELSH LL

01EFB
LATIN SMALL LETTER MIDDLE-WELSH LL

01EFC
LATIN CAPITAL LETTER MIDDLE-WELSH V

01EFD
LATIN SMALL LETTER MIDDLE-WELSH V

01EFE
LATIN CAPITAL LETTER Y WITH LOOP

01EFF
LATIN SMALL LETTER Y WITH LOOP

01F00
GREEK SMALL LETTER ALPHA WITH PSILI

01F01
GREEK SMALL LETTER ALPHA WITH DASIA

01F02
GREEK SMALL LETTER ALPHA WITH PSILI AND VARIA

01F03
GREEK SMALL LETTER ALPHA WITH DASIA AND VARIA

01F04
GREEK SMALL LETTER ALPHA WITH PSILI AND OXIA

01F05
GREEK SMALL LETTER ALPHA WITH DASIA AND OXIA

01F06
GREEK SMALL LETTER ALPHA WITH PSILI AND PERISPOMENI

01F07
GREEK SMALL LETTER ALPHA WITH DASIA AND PERISPOMENI

01F08
GREEK CAPITAL LETTER ALPHA WITH PSILI

01F09
GREEK CAPITAL LETTER ALPHA WITH DASIA

01F0A
GREEK CAPITAL LETTER ALPHA WITH PSILI AND VARIA

01F0B
GREEK CAPITAL LETTER ALPHA WITH DASIA AND VARIA

01F0C
GREEK CAPITAL LETTER ALPHA WITH PSILI AND OXIA

01F0D
GREEK CAPITAL LETTER ALPHA WITH DASIA AND OXIA

01F0E
GREEK CAPITAL LETTER ALPHA WITH PSILI AND PERISPOMENI

01F0F
GREEK CAPITAL LETTER ALPHA WITH DASIA AND PERISPOMENI

01F10
GREEK SMALL LETTER EPSILON WITH PSILI

01F11
GREEK SMALL LETTER EPSILON WITH DASIA

01F12
GREEK SMALL LETTER EPSILON WITH PSILI AND VARIA

01F13
GREEK SMALL LETTER EPSILON WITH DASIA AND VARIA

01F14
GREEK SMALL LETTER EPSILON WITH PSILI AND OXIA

01F15
GREEK SMALL LETTER EPSILON WITH DASIA AND OXIA

01F18
GREEK CAPITAL LETTER EPSILON WITH PSILI

01F19
GREEK CAPITAL LETTER EPSILON WITH DASIA

01F1A
GREEK CAPITAL LETTER EPSILON WITH PSILI AND VARIA

01F1B
GREEK CAPITAL LETTER EPSILON WITH DASIA AND VARIA

01F1C
GREEK CAPITAL LETTER EPSILON WITH PSILI AND OXIA

01F1D
GREEK CAPITAL LETTER EPSILON WITH DASIA AND OXIA

01F20
GREEK SMALL LETTER ETA WITH PSILI

01F21
GREEK SMALL LETTER ETA WITH DASIA

01F22
GREEK SMALL LETTER ETA WITH PSILI AND VARIA

01F23
GREEK SMALL LETTER ETA WITH DASIA AND VARIA

01F24
GREEK SMALL LETTER ETA WITH PSILI AND OXIA

01F25
GREEK SMALL LETTER ETA WITH DASIA AND OXIA

01F26
GREEK SMALL LETTER ETA WITH PSILI AND PERISPOMENI

01F27
GREEK SMALL LETTER ETA WITH DASIA AND PERISPOMENI

01F28
GREEK CAPITAL LETTER ETA WITH PSILI

01F29
GREEK CAPITAL LETTER ETA WITH DASIA

01F2A
GREEK CAPITAL LETTER ETA WITH PSILI AND VARIA

01F2B
GREEK CAPITAL LETTER ETA WITH DASIA AND VARIA

01F2C
GREEK CAPITAL LETTER ETA WITH PSILI AND OXIA

01F2D
GREEK CAPITAL LETTER ETA WITH DASIA AND OXIA

01F2E
GREEK CAPITAL LETTER ETA WITH PSILI AND PERISPOMENI

01F2F
GREEK CAPITAL LETTER ETA WITH DASIA AND PERISPOMENI

01F30
GREEK SMALL LETTER IOTA WITH PSILI

01F31
GREEK SMALL LETTER IOTA WITH DASIA

01F32
GREEK SMALL LETTER IOTA WITH PSILI AND VARIA

01F33
GREEK SMALL LETTER IOTA WITH DASIA AND VARIA

01F34
GREEK SMALL LETTER IOTA WITH PSILI AND OXIA

01F35
GREEK SMALL LETTER IOTA WITH DASIA AND OXIA

01F36
GREEK SMALL LETTER IOTA WITH PSILI AND PERISPOMENI

01F37
GREEK SMALL LETTER IOTA WITH DASIA AND PERISPOMENI

01F38
GREEK CAPITAL LETTER IOTA WITH PSILI

01F39
GREEK CAPITAL LETTER IOTA WITH DASIA

01F3A
GREEK CAPITAL LETTER IOTA WITH PSILI AND VARIA

01F3B
GREEK CAPITAL LETTER IOTA WITH DASIA AND VARIA

01F3C
GREEK CAPITAL LETTER IOTA WITH PSILI AND OXIA

01F3D
GREEK CAPITAL LETTER IOTA WITH DASIA AND OXIA

01F3E
GREEK CAPITAL LETTER IOTA WITH PSILI AND PERISPOMENI

01F3F
GREEK CAPITAL LETTER IOTA WITH DASIA AND PERISPOMENI

01F40
GREEK SMALL LETTER OMICRON WITH PSILI

01F41
GREEK SMALL LETTER OMICRON WITH DASIA

01F42
GREEK SMALL LETTER OMICRON WITH PSILI AND VARIA

01F43
GREEK SMALL LETTER OMICRON WITH DASIA AND VARIA

01F44
GREEK SMALL LETTER OMICRON WITH PSILI AND OXIA

01F45
GREEK SMALL LETTER OMICRON WITH DASIA AND OXIA

01F48
GREEK CAPITAL LETTER OMICRON WITH PSILI

01F49
GREEK CAPITAL LETTER OMICRON WITH DASIA

01F4A
GREEK CAPITAL LETTER OMICRON WITH PSILI AND VARIA

01F4B
GREEK CAPITAL LETTER OMICRON WITH DASIA AND VARIA

01F4C
GREEK CAPITAL LETTER OMICRON WITH PSILI AND OXIA

01F4D
GREEK CAPITAL LETTER OMICRON WITH DASIA AND OXIA

01F50
GREEK SMALL LETTER UPSILON WITH PSILI

01F51
GREEK SMALL LETTER UPSILON WITH DASIA

01F52
GREEK SMALL LETTER UPSILON WITH PSILI AND VARIA

01F53
GREEK SMALL LETTER UPSILON WITH DASIA AND VARIA

01F54
GREEK SMALL LETTER UPSILON WITH PSILI AND OXIA

01F55
GREEK SMALL LETTER UPSILON WITH DASIA AND OXIA

01F56
GREEK SMALL LETTER UPSILON WITH PSILI AND PERISPOMENI

01F57
GREEK SMALL LETTER UPSILON WITH DASIA AND PERISPOMENI

01F59
GREEK CAPITAL LETTER UPSILON WITH DASIA

01F5B
GREEK CAPITAL LETTER UPSILON WITH DASIA AND VARIA

01F5D
GREEK CAPITAL LETTER UPSILON WITH DASIA AND OXIA

01F5F
GREEK CAPITAL LETTER UPSILON WITH DASIA AND PERISPOMENI

01F60
GREEK SMALL LETTER OMEGA WITH PSILI

01F61
GREEK SMALL LETTER OMEGA WITH DASIA

01F62
GREEK SMALL LETTER OMEGA WITH PSILI AND VARIA

01F63
GREEK SMALL LETTER OMEGA WITH DASIA AND VARIA

01F64
GREEK SMALL LETTER OMEGA WITH PSILI AND OXIA

01F65
GREEK SMALL LETTER OMEGA WITH DASIA AND OXIA

01F66
GREEK SMALL LETTER OMEGA WITH PSILI AND PERISPOMENI

01F67
GREEK SMALL LETTER OMEGA WITH DASIA AND PERISPOMENI

01F68
GREEK CAPITAL LETTER OMEGA WITH PSILI

01F69
GREEK CAPITAL LETTER OMEGA WITH DASIA

01F6A
GREEK CAPITAL LETTER OMEGA WITH PSILI AND VARIA

01F6B
GREEK CAPITAL LETTER OMEGA WITH DASIA AND VARIA

01F6C
GREEK CAPITAL LETTER OMEGA WITH PSILI AND OXIA

01F6D
GREEK CAPITAL LETTER OMEGA WITH DASIA AND OXIA

01F6E
GREEK CAPITAL LETTER OMEGA WITH PSILI AND PERISPOMENI

01F6F
GREEK CAPITAL LETTER OMEGA WITH DASIA AND PERISPOMENI

01F70
GREEK SMALL LETTER ALPHA WITH VARIA

01F71
GREEK SMALL LETTER ALPHA WITH OXIA

01F72
GREEK SMALL LETTER EPSILON WITH VARIA

01F73
GREEK SMALL LETTER EPSILON WITH OXIA

01F74
GREEK SMALL LETTER ETA WITH VARIA

01F75
GREEK SMALL LETTER ETA WITH OXIA

01F76
GREEK SMALL LETTER IOTA WITH VARIA

01F77
GREEK SMALL LETTER IOTA WITH OXIA

01F78
GREEK SMALL LETTER OMICRON WITH VARIA

01F79
GREEK SMALL LETTER OMICRON WITH OXIA

01F7A
GREEK SMALL LETTER UPSILON WITH VARIA

01F7B
GREEK SMALL LETTER UPSILON WITH OXIA

01F7C
GREEK SMALL LETTER OMEGA WITH VARIA

01F7D
GREEK SMALL LETTER OMEGA WITH OXIA

01F80
GREEK SMALL LETTER ALPHA WITH PSILI AND YPOGEGRAMMENI

01F81
GREEK SMALL LETTER ALPHA WITH DASIA AND YPOGEGRAMMENI

01F82
GREEK SMALL LETTER ALPHA WITH PSILI AND VARIA AND YPOGEGRAMMENI

01F83
GREEK SMALL LETTER ALPHA WITH DASIA AND VARIA AND YPOGEGRAMMENI

01F84
GREEK SMALL LETTER ALPHA WITH PSILI AND OXIA AND YPOGEGRAMMENI

01F85
GREEK SMALL LETTER ALPHA WITH DASIA AND OXIA AND YPOGEGRAMMENI

01F86
GREEK SMALL LETTER ALPHA WITH PSILI AND PERISPOMENI AND YPOGEGRAMMENI

01F87
GREEK SMALL LETTER ALPHA WITH DASIA AND PERISPOMENI AND YPOGEGRAMMENI

01F88
GREEK CAPITAL LETTER ALPHA WITH PSILI AND PROSGEGRAMMENI

01F89
GREEK CAPITAL LETTER ALPHA WITH DASIA AND PROSGEGRAMMENI

01F8A
GREEK CAPITAL LETTER ALPHA WITH PSILI AND VARIA AND PROSGEGRAMMENI

01F8B
GREEK CAPITAL LETTER ALPHA WITH DASIA AND VARIA AND PROSGEGRAMMENI

01F8C
GREEK CAPITAL LETTER ALPHA WITH PSILI AND OXIA AND PROSGEGRAMMENI

01F8D
GREEK CAPITAL LETTER ALPHA WITH DASIA AND OXIA AND PROSGEGRAMMENI

01F8E
GREEK CAPITAL LETTER ALPHA WITH PSILI AND PERISPOMENI AND PROSGEGRAMMENI

01F8F
GREEK CAPITAL LETTER ALPHA WITH DASIA AND PERISPOMENI AND PROSGEGRAMMENI

01F90
GREEK SMALL LETTER ETA WITH PSILI AND YPOGEGRAMMENI

01F91
GREEK SMALL LETTER ETA WITH DASIA AND YPOGEGRAMMENI

01F92
GREEK SMALL LETTER ETA WITH PSILI AND VARIA AND YPOGEGRAMMENI

01F93
GREEK SMALL LETTER ETA WITH DASIA AND VARIA AND YPOGEGRAMMENI

01F94
GREEK SMALL LETTER ETA WITH PSILI AND OXIA AND YPOGEGRAMMENI

01F95
GREEK SMALL LETTER ETA WITH DASIA AND OXIA AND YPOGEGRAMMENI

01F96
GREEK SMALL LETTER ETA WITH PSILI AND PERISPOMENI AND YPOGEGRAMMENI

01F97
GREEK SMALL LETTER ETA WITH DASIA AND PERISPOMENI AND YPOGEGRAMMENI

01F98
GREEK CAPITAL LETTER ETA WITH PSILI AND PROSGEGRAMMENI

01F99
GREEK CAPITAL LETTER ETA WITH DASIA AND PROSGEGRAMMENI

01F9A
GREEK CAPITAL LETTER ETA WITH PSILI AND VARIA AND PROSGEGRAMMENI

01F9B
GREEK CAPITAL LETTER ETA WITH DASIA AND VARIA AND PROSGEGRAMMENI

01F9C
GREEK CAPITAL LETTER ETA WITH PSILI AND OXIA AND PROSGEGRAMMENI

01F9D
GREEK CAPITAL LETTER ETA WITH DASIA AND OXIA AND PROSGEGRAMMENI

01F9E
GREEK CAPITAL LETTER ETA WITH PSILI AND PERISPOMENI AND PROSGEGRAMMENI

01F9F
GREEK CAPITAL LETTER ETA WITH DASIA AND PERISPOMENI AND PROSGEGRAMMENI

01FA0
GREEK SMALL LETTER OMEGA WITH PSILI AND YPOGEGRAMMENI

01FA1
GREEK SMALL LETTER OMEGA WITH DASIA AND YPOGEGRAMMENI

01FA2
GREEK SMALL LETTER OMEGA WITH PSILI AND VARIA AND YPOGEGRAMMENI

01FA3
GREEK SMALL LETTER OMEGA WITH DASIA AND VARIA AND YPOGEGRAMMENI

01FA4
GREEK SMALL LETTER OMEGA WITH PSILI AND OXIA AND YPOGEGRAMMENI

01FA5
GREEK SMALL LETTER OMEGA WITH DASIA AND OXIA AND YPOGEGRAMMENI

01FA6
GREEK SMALL LETTER OMEGA WITH PSILI AND PERISPOMENI AND YPOGEGRAMMENI

01FA7
GREEK SMALL LETTER OMEGA WITH DASIA AND PERISPOMENI AND YPOGEGRAMMENI

01FA8
GREEK CAPITAL LETTER OMEGA WITH PSILI AND PROSGEGRAMMENI

01FA9
GREEK CAPITAL LETTER OMEGA WITH DASIA AND PROSGEGRAMMENI

01FAA
GREEK CAPITAL LETTER OMEGA WITH PSILI AND VARIA AND PROSGEGRAMMENI

01FAB
GREEK CAPITAL LETTER OMEGA WITH DASIA AND VARIA AND PROSGEGRAMMENI

01FAC
GREEK CAPITAL LETTER OMEGA WITH PSILI AND OXIA AND PROSGEGRAMMENI

01FAD
GREEK CAPITAL LETTER OMEGA WITH DASIA AND OXIA AND PROSGEGRAMMENI

01FAE
GREEK CAPITAL LETTER OMEGA WITH PSILI AND PERISPOMENI AND PROSGEGRAMMENI

01FAF
GREEK CAPITAL LETTER OMEGA WITH DASIA AND PERISPOMENI AND PROSGEGRAMMENI

01FB0
GREEK SMALL LETTER ALPHA WITH VRACHY

01FB1
GREEK SMALL LETTER ALPHA WITH MACRON

01FB2
GREEK SMALL LETTER ALPHA WITH VARIA AND YPOGEGRAMMENI

01FB3
GREEK SMALL LETTER ALPHA WITH YPOGEGRAMMENI

01FB4
GREEK SMALL LETTER ALPHA WITH OXIA AND YPOGEGRAMMENI

01FB6
GREEK SMALL LETTER ALPHA WITH PERISPOMENI

01FB7
GREEK SMALL LETTER ALPHA WITH PERISPOMENI AND YPOGEGRAMMENI

01FB8
GREEK CAPITAL LETTER ALPHA WITH VRACHY

01FB9
GREEK CAPITAL LETTER ALPHA WITH MACRON

01FBA
GREEK CAPITAL LETTER ALPHA WITH VARIA

01FBB
GREEK CAPITAL LETTER ALPHA WITH OXIA

01FBC
GREEK CAPITAL LETTER ALPHA WITH PROSGEGRAMMENI

01FBD
GREEK KORONIS

01FBE
GREEK PROSGEGRAMMENI

01FBF
GREEK PSILI

01FC0
GREEK PERISPOMENI

01FC1
GREEK DIALYTIKA AND PERISPOMENI

01FC2
GREEK SMALL LETTER ETA WITH VARIA AND YPOGEGRAMMENI

01FC3
GREEK SMALL LETTER ETA WITH YPOGEGRAMMENI

01FC4
GREEK SMALL LETTER ETA WITH OXIA AND YPOGEGRAMMENI

01FC6
GREEK SMALL LETTER ETA WITH PERISPOMENI

01FC7
GREEK SMALL LETTER ETA WITH PERISPOMENI AND YPOGEGRAMMENI

01FC8
GREEK CAPITAL LETTER EPSILON WITH VARIA

01FC9
GREEK CAPITAL LETTER EPSILON WITH OXIA

01FCA
GREEK CAPITAL LETTER ETA WITH VARIA

01FCB
GREEK CAPITAL LETTER ETA WITH OXIA

01FCC
GREEK CAPITAL LETTER ETA WITH PROSGEGRAMMENI

01FCD
GREEK PSILI AND VARIA

01FCE
GREEK PSILI AND OXIA

01FCF
GREEK PSILI AND PERISPOMENI

01FD0
GREEK SMALL LETTER IOTA WITH VRACHY

01FD1
GREEK SMALL LETTER IOTA WITH MACRON

01FD2
GREEK SMALL LETTER IOTA WITH DIALYTIKA AND VARIA

01FD3
GREEK SMALL LETTER IOTA WITH DIALYTIKA AND OXIA

01FD6
GREEK SMALL LETTER IOTA WITH PERISPOMENI

01FD7
GREEK SMALL LETTER IOTA WITH DIALYTIKA AND PERISPOMENI

01FD8
GREEK CAPITAL LETTER IOTA WITH VRACHY

01FD9
GREEK CAPITAL LETTER IOTA WITH MACRON

01FDA
GREEK CAPITAL LETTER IOTA WITH VARIA

01FDB
GREEK CAPITAL LETTER IOTA WITH OXIA

01FDD
GREEK DASIA AND VARIA

01FDE
GREEK DASIA AND OXIA

01FDF
GREEK DASIA AND PERISPOMENI

01FE0
GREEK SMALL LETTER UPSILON WITH VRACHY

01FE1
GREEK SMALL LETTER UPSILON WITH MACRON

01FE2
GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND VARIA

01FE3
GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND OXIA

01FE4
GREEK SMALL LETTER RHO WITH PSILI

01FE5
GREEK SMALL LETTER RHO WITH DASIA

01FE6
GREEK SMALL LETTER UPSILON WITH PERISPOMENI

01FE7
GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND PERISPOMENI

01FE8
GREEK CAPITAL LETTER UPSILON WITH VRACHY

01FE9
GREEK CAPITAL LETTER UPSILON WITH MACRON

01FEA
GREEK CAPITAL LETTER UPSILON WITH VARIA

01FEB
GREEK CAPITAL LETTER UPSILON WITH OXIA

01FEC
GREEK CAPITAL LETTER RHO WITH DASIA

01FED
GREEK DIALYTIKA AND VARIA

01FEE
GREEK DIALYTIKA AND OXIA

01FEF
GREEK VARIA

01FF2
GREEK SMALL LETTER OMEGA WITH VARIA AND YPOGEGRAMMENI

01FF3
GREEK SMALL LETTER OMEGA WITH YPOGEGRAMMENI

01FF4
GREEK SMALL LETTER OMEGA WITH OXIA AND YPOGEGRAMMENI

01FF6
GREEK SMALL LETTER OMEGA WITH PERISPOMENI

01FF7
GREEK SMALL LETTER OMEGA WITH PERISPOMENI AND YPOGEGRAMMENI

01FF8
GREEK CAPITAL LETTER OMICRON WITH VARIA

01FF9
GREEK CAPITAL LETTER OMICRON WITH OXIA

01FFA
GREEK CAPITAL LETTER OMEGA WITH VARIA

01FFB
GREEK CAPITAL LETTER OMEGA WITH OXIA

01FFC
GREEK CAPITAL LETTER OMEGA WITH PROSGEGRAMMENI

01FFD
GREEK OXIA

01FFE
GREEK DASIA

02000
EN QUAD

02001
EM QUAD

02002
EN SPACE

02003
EM SPACE

02004
THREE-PER-EM SPACE

02005
FOUR-PER-EM SPACE

02006
SIX-PER-EM SPACE

02007
FIGURE SPACE

02008
PUNCTUATION SPACE

02009
THIN SPACE

0200A
HAIR SPACE

0200B
ZERO WIDTH SPACE

0200B
ZWSP

0200C
ZERO WIDTH NON-JOINER

0200C
ZWNJ

0200D
ZERO WIDTH JOINER

0200D
ZWJ

0200E
LEFT-TO-RIGHT MARK

0200E
LRM

0200F
RIGHT-TO-LEFT MARK

0200F
RLM

02010
HYPHEN

02011
NON-BREAKING HYPHEN

02012
FIGURE DASH

02013
EN DASH

02014
EM DASH

02015
HORIZONTAL BAR

02016
DOUBLE VERTICAL LINE

02017
DOUBLE LOW LINE

02018
LEFT SINGLE QUOTATION MARK

02019
RIGHT SINGLE QUOTATION MARK

0201A
SINGLE LOW-9 QUOTATION MARK

0201B
SINGLE HIGH-REVERSED-9 QUOTATION MARK

0201C
LEFT DOUBLE QUOTATION MARK

0201D
RIGHT DOUBLE QUOTATION MARK

0201E
DOUBLE LOW-9 QUOTATION MARK

0201F
DOUBLE HIGH-REVERSED-9 QUOTATION MARK

02020
DAGGER

02021
DOUBLE DAGGER

02022
BULLET

02023
TRIANGULAR BULLET

02024
ONE DOT LEADER

02025
TWO DOT LEADER

02026
HORIZONTAL ELLIPSIS

02027
HYPHENATION POINT

02028
LINE SEPARATOR

02029
PARAGRAPH SEPARATOR

0202A
LEFT-TO-RIGHT EMBEDDING

0202A
LRE

0202B
RIGHT-TO-LEFT EMBEDDING

0202B
RLE

0202C
POP DIRECTIONAL FORMATTING

0202C
PDF

0202D
LEFT-TO-RIGHT OVERRIDE

0202D
LRO

0202E
RIGHT-TO-LEFT OVERRIDE

0202E
RLO

0202F
NARROW NO-BREAK SPACE

0202F
NNBSP

02030
PER MILLE SIGN

02031
PER TEN THOUSAND SIGN

02032
PRIME

02033
DOUBLE PRIME

02034
TRIPLE PRIME

02035
REVERSED PRIME

02036
REVERSED DOUBLE PRIME

02037
REVERSED TRIPLE PRIME

02038
CARET

02039
SINGLE LEFT-POINTING ANGLE QUOTATION MARK

0203A
SINGLE RIGHT-POINTING ANGLE QUOTATION MARK

0203B
REFERENCE MARK

0203C
DOUBLE EXCLAMATION MARK

0203D
INTERROBANG

0203E
OVERLINE

0203F
UNDERTIE

02040
CHARACTER TIE

02041
CARET INSERTION POINT

02042
ASTERISM

02043
HYPHEN BULLET

02044
FRACTION SLASH

02045
LEFT SQUARE BRACKET WITH QUILL

02046
RIGHT SQUARE BRACKET WITH QUILL

02047
DOUBLE QUESTION MARK

02048
QUESTION EXCLAMATION MARK

02049
EXCLAMATION QUESTION MARK

0204A
TIRONIAN SIGN ET

0204B
REVERSED PILCROW SIGN

0204C
BLACK LEFTWARDS BULLET

0204D
BLACK RIGHTWARDS BULLET

0204E
LOW ASTERISK

0204F
REVERSED SEMICOLON

02050
CLOSE UP

02051
TWO ASTERISKS ALIGNED VERTICALLY

02052
COMMERCIAL MINUS SIGN

02053
SWUNG DASH

02054
INVERTED UNDERTIE

02055
FLOWER PUNCTUATION MARK

02056
THREE DOT PUNCTUATION

02057
QUADRUPLE PRIME

02058
FOUR DOT PUNCTUATION

02059
FIVE DOT PUNCTUATION

0205A
TWO DOT PUNCTUATION

0205B
FOUR DOT MARK

0205C
DOTTED CROSS

0205D
TRICOLON

0205E
VERTICAL FOUR DOTS

0205F
MEDIUM MATHEMATICAL SPACE

0205F
MMSP

02060
WORD JOINER

02060
WJ

02061
FUNCTION APPLICATION

02062
INVISIBLE TIMES

02063
INVISIBLE SEPARATOR

02064
INVISIBLE PLUS

02066
LEFT-TO-RIGHT ISOLATE

02066
LRI

02067
RIGHT-TO-LEFT ISOLATE

02067
RLI

02068
FIRST STRONG ISOLATE

02068
FSI

02069
POP DIRECTIONAL ISOLATE

02069
PDI

0206A
INHIBIT SYMMETRIC SWAPPING

0206B
ACTIVATE SYMMETRIC SWAPPING

0206C
INHIBIT ARABIC FORM SHAPING

0206D
ACTIVATE ARABIC FORM SHAPING

0206E
NATIONAL DIGIT SHAPES

0206F
NOMINAL DIGIT SHAPES

02070
SUPERSCRIPT ZERO

02071
SUPERSCRIPT LATIN SMALL LETTER I

02074
SUPERSCRIPT FOUR

02075
SUPERSCRIPT FIVE

02076
SUPERSCRIPT SIX

02077
SUPERSCRIPT SEVEN

02078
SUPERSCRIPT EIGHT

02079
SUPERSCRIPT NINE

0207A
SUPERSCRIPT PLUS SIGN

0207B
SUPERSCRIPT MINUS

0207C
SUPERSCRIPT EQUALS SIGN

0207D
SUPERSCRIPT LEFT PARENTHESIS

0207E
SUPERSCRIPT RIGHT PARENTHESIS

0207F
SUPERSCRIPT LATIN SMALL LETTER N

02080
SUBSCRIPT ZERO

02081
SUBSCRIPT ONE

02082
SUBSCRIPT TWO

02083
SUBSCRIPT THREE

02084
SUBSCRIPT FOUR

02085
SUBSCRIPT FIVE

02086
SUBSCRIPT SIX

02087
SUBSCRIPT SEVEN

02088
SUBSCRIPT EIGHT

02089
SUBSCRIPT NINE

0208A
SUBSCRIPT PLUS SIGN

0208B
SUBSCRIPT MINUS

0208C
SUBSCRIPT EQUALS SIGN

0208D
SUBSCRIPT LEFT PARENTHESIS

0208E
SUBSCRIPT RIGHT PARENTHESIS

02090
LATIN SUBSCRIPT SMALL LETTER A

02091
LATIN SUBSCRIPT SMALL LETTER E

02092
LATIN SUBSCRIPT SMALL LETTER O

02093
LATIN SUBSCRIPT SMALL LETTER X

02094
LATIN SUBSCRIPT SMALL LETTER SCHWA

02095
LATIN SUBSCRIPT SMALL LETTER H

02096
LATIN SUBSCRIPT SMALL LETTER K

02097
LATIN SUBSCRIPT SMALL LETTER L

02098
LATIN SUBSCRIPT SMALL LETTER M

02099
LATIN SUBSCRIPT SMALL LETTER N

0209A
LATIN SUBSCRIPT SMALL LETTER P

0209B
LATIN SUBSCRIPT SMALL LETTER S

0209C
LATIN SUBSCRIPT SMALL LETTER T

020A0
EURO-CURRENCY SIGN

020A1
COLON SIGN

020A2
CRUZEIRO SIGN

020A3
FRENCH FRANC SIGN

020A4
LIRA SIGN

020A5
MILL SIGN

020A6
NAIRA SIGN

020A7
PESETA SIGN

020A8
RUPEE SIGN

020A9
WON SIGN

020AA
NEW SHEQEL SIGN

020AB
DONG SIGN

020AC
EURO SIGN

020AD
KIP SIGN

020AE
TUGRIK SIGN

020AF
DRACHMA SIGN

020B0
GERMAN PENNY SIGN

020B1
PESO SIGN

020B2
GUARANI SIGN

020B3
AUSTRAL SIGN

020B4
HRYVNIA SIGN

020B5
CEDI SIGN

020B6
LIVRE TOURNOIS SIGN

020B7
SPESMILO SIGN

020B8
TENGE SIGN

020B9
INDIAN RUPEE SIGN

020BA
TURKISH LIRA SIGN

020BB
NORDIC MARK SIGN

020BC
MANAT SIGN

020BD
RUBLE SIGN

020BE
LARI SIGN

020BF
BITCOIN SIGN

020C0
SOM SIGN

020C1
SAUDI RIYAL SIGN

020D0
COMBINING LEFT HARPOON ABOVE

020D1
COMBINING RIGHT HARPOON ABOVE

020D2
COMBINING LONG VERTICAL LINE OVERLAY

020D3
COMBINING SHORT VERTICAL LINE OVERLAY

020D4
COMBINING ANTICLOCKWISE ARROW ABOVE

020D5
COMBINING CLOCKWISE ARROW ABOVE

020D6
COMBINING LEFT ARROW ABOVE

020D7
COMBINING RIGHT ARROW ABOVE

020D8
COMBINING RING OVERLAY

020D9
COMBINING CLOCKWISE RING OVERLAY

020DA
COMBINING ANTICLOCKWISE RING OVERLAY

020DB
COMBINING THREE DOTS ABOVE

020DC
COMBINING FOUR DOTS ABOVE

020DD
COMBINING ENCLOSING CIRCLE

020DE
COMBINING ENCLOSING SQUARE

020DF
COMBINING ENCLOSING DIAMOND

020E0
COMBINING ENCLOSING CIRCLE BACKSLASH

020E1
COMBINING LEFT RIGHT ARROW ABOVE

020E2
COMBINING ENCLOSING SCREEN

020E3
COMBINING ENCLOSING KEYCAP

020E4
COMBINING ENCLOSING UPWARD POINTING TRIANGLE

020E5
COMBINING REVERSE SOLIDUS OVERLAY

020E6
COMBINING DOUBLE VERTICAL STROKE OVERLAY

020E7
COMBINING ANNUITY SYMBOL

020E8
COMBINING TRIPLE UNDERDOT

020E9
COMBINING WIDE BRIDGE ABOVE

020EA
COMBINING LEFTWARDS ARROW OVERLAY

020EB
COMBINING LONG DOUBLE SOLIDUS OVERLAY

020EC
COMBINING RIGHTWARDS HARPOON WITH BARB DOWNWARDS

020ED
COMBINING LEFTWARDS HARPOON WITH BARB DOWNWARDS

020EE
COMBINING LEFT ARROW BELOW

020EF
COMBINING RIGHT ARROW BELOW

020F0
COMBINING ASTERISK ABOVE

02100
ACCOUNT OF

02101
ADDRESSED TO THE SUBJECT

02102
DOUBLE-STRUCK CAPITAL C

02103
DEGREE CELSIUS

02104
CENTRE LINE SYMBOL

02105
CARE OF

02106
CADA UNA

02107
EULER CONSTANT

02108
SCRUPLE

02109
DEGREE FAHRENHEIT

0210A
SCRIPT SMALL G

0210B
SCRIPT CAPITAL H

0210C
BLACK-LETTER CAPITAL H

0210D
DOUBLE-STRUCK CAPITAL H

0210E
PLANCK CONSTANT

0210F
PLANCK CONSTANT OVER TWO PI

02110
SCRIPT CAPITAL I

02111
BLACK-LETTER CAPITAL I

02112
SCRIPT CAPITAL L

02113
SCRIPT SMALL L

02114
L B BAR SYMBOL

02115
DOUBLE-STRUCK CAPITAL N

02116
NUMERO SIGN

02117
SOUND RECORDING COPYRIGHT

02118
WEIERSTRASS ELLIPTIC FUNCTION

02118
SCRIPT CAPITAL P

02119
DOUBLE-STRUCK CAPITAL P

0211A
DOUBLE-STRUCK CAPITAL Q

0211B
SCRIPT CAPITAL R

0211C
BLACK-LETTER CAPITAL R

0211D
DOUBLE-STRUCK CAPITAL R

0211E
PRESCRIPTION TAKE

0211F
RESPONSE

02120
SERVICE MARK

02121
TELEPHONE SIGN

02122
TRADE MARK SIGN

02123
VERSICLE

02124
DOUBLE-STRUCK CAPITAL Z

02125
OUNCE SIGN

02126
OHM SIGN

02127
INVERTED OHM SIGN

02128
BLACK-LETTER CAPITAL Z

02129
TURNED GREEK SMALL LETTER IOTA

0212A
KELVIN SIGN

0212B
ANGSTROM SIGN

0212C
SCRIPT CAPITAL B

0212D
BLACK-LETTER CAPITAL C

0212E
ESTIMATED SYMBOL

0212F
SCRIPT SMALL E

02130
SCRIPT CAPITAL E

02131
SCRIPT CAPITAL F

02132
TURNED CAPITAL F

02133
SCRIPT CAPITAL M

02134
SCRIPT SMALL O

02135
ALEF SYMBOL

02136
BET SYMBOL

02137
GIMEL SYMBOL

02138
DALET SYMBOL

02139
INFORMATION SOURCE

0213A
ROTATED CAPITAL Q

0213B
FACSIMILE SIGN

0213C
DOUBLE-STRUCK SMALL PI

0213D
DOUBLE-STRUCK SMALL GAMMA

0213E
DOUBLE-STRUCK CAPITAL GAMMA

0213F
DOUBLE-STRUCK CAPITAL PI

02140
DOUBLE-STRUCK N-ARY SUMMATION

02141
TURNED SANS-SERIF CAPITAL G

02142
TURNED SANS-SERIF CAPITAL L

02143
REVERSED SANS-SERIF CAPITAL L

02144
TURNED SANS-SERIF CAPITAL Y

02145
DOUBLE-STRUCK ITALIC CAPITAL D

02146
DOUBLE-STRUCK ITALIC SMALL D

02147
DOUBLE-STRUCK ITALIC SMALL E

02148
DOUBLE-STRUCK ITALIC SMALL I

02149
DOUBLE-STRUCK ITALIC SMALL J

0214A
PROPERTY LINE

0214B
TURNED AMPERSAND

0214C
PER SIGN

0214D
AKTIESELSKAB

0214E
TURNED SMALL F

0214F
SYMBOL FOR SAMARITAN SOURCE

02150
VULGAR FRACTION ONE SEVENTH

02151
VULGAR FRACTION ONE NINTH

02152
VULGAR FRACTION ONE TENTH

02153
VULGAR FRACTION ONE THIRD

02154
VULGAR FRACTION TWO THIRDS

02155
VULGAR FRACTION ONE FIFTH

02156
VULGAR FRACTION TWO FIFTHS

02157
VULGAR FRACTION THREE FIFTHS

02158
VULGAR FRACTION FOUR FIFTHS

02159
VULGAR FRACTION ONE SIXTH

0215A
VULGAR FRACTION FIVE SIXTHS

0215B
VULGAR FRACTION ONE EIGHTH

0215C
VULGAR FRACTION THREE EIGHTHS

0215D
VULGAR FRACTION FIVE EIGHTHS

0215E
VULGAR FRACTION SEVEN EIGHTHS

0215F
FRACTION NUMERATOR ONE

02160
ROMAN NUMERAL ONE

02161
ROMAN NUMERAL TWO

02162
ROMAN NUMERAL THREE

02163
ROMAN NUMERAL FOUR

02164
ROMAN NUMERAL FIVE

02165
ROMAN NUMERAL SIX

02166
ROMAN NUMERAL SEVEN

02167
ROMAN NUMERAL EIGHT

02168
ROMAN NUMERAL NINE

02169
ROMAN NUMERAL TEN

0216A
ROMAN NUMERAL ELEVEN

0216B
ROMAN NUMERAL TWELVE

0216C
ROMAN NUMERAL FIFTY

0216D
ROMAN NUMERAL ONE HUNDRED

0216E
ROMAN NUMERAL FIVE HUNDRED

0216F
ROMAN NUMERAL ONE THOUSAND

02170
SMALL ROMAN NUMERAL ONE

02171
SMALL ROMAN NUMERAL TWO

02172
SMALL ROMAN NUMERAL THREE

02173
SMALL ROMAN NUMERAL FOUR

02174
SMALL ROMAN NUMERAL FIVE

02175
SMALL ROMAN NUMERAL SIX

02176
SMALL ROMAN NUMERAL SEVEN

02177
SMALL ROMAN NUMERAL EIGHT

02178
SMALL ROMAN NUMERAL NINE

02179
SMALL ROMAN NUMERAL TEN

0217A
SMALL ROMAN NUMERAL ELEVEN

0217B
SMALL ROMAN NUMERAL TWELVE

0217C
SMALL ROMAN NUMERAL FIFTY

0217D
SMALL ROMAN NUMERAL ONE HUNDRED

0217E
SMALL ROMAN NUMERAL FIVE HUNDRED

0217F
SMALL ROMAN NUMERAL ONE THOUSAND

02180
ROMAN NUMERAL ONE THOUSAND C D

02181
ROMAN NUMERAL FIVE THOUSAND

02182
ROMAN NUMERAL TEN THOUSAND

02183
ROMAN NUMERAL REVERSED ONE HUNDRED

02184
LATIN SMALL LETTER REVERSED C

02185
ROMAN NUMERAL SIX LATE FORM

02186
ROMAN NUMERAL FIFTY EARLY FORM

02187
ROMAN NUMERAL FIFTY THOUSAND

02188
ROMAN NUMERAL ONE HUNDRED THOUSAND

02189
VULGAR FRACTION ZERO THIRDS

0218A
TURNED DIGIT TWO

0218B
TURNED DIGIT THREE

02190
LEFTWARDS ARROW

02191
UPWARDS ARROW

02192
RIGHTWARDS ARROW

02193
DOWNWARDS ARROW

02194
LEFT RIGHT ARROW

02195
UP DOWN ARROW

02196
NORTH WEST ARROW

02197
NORTH EAST ARROW

02198
SOUTH EAST ARROW

02199
SOUTH WEST ARROW

0219A
LEFTWARDS ARROW WITH STROKE

0219B
RIGHTWARDS ARROW WITH STROKE

0219C
LEFTWARDS WAVE ARROW

0219D
RIGHTWARDS WAVE ARROW

0219E
LEFTWARDS TWO HEADED ARROW

0219F
UPWARDS TWO HEADED ARROW

021A0
RIGHTWARDS TWO HEADED ARROW

021A1
DOWNWARDS TWO HEADED ARROW

021A2
LEFTWARDS ARROW WITH TAIL

021A3
RIGHTWARDS ARROW WITH TAIL

021A4
LEFTWARDS ARROW FROM BAR

021A5
UPWARDS ARROW FROM BAR

021A6
RIGHTWARDS ARROW FROM BAR

021A7
DOWNWARDS ARROW FROM BAR

021A8
UP DOWN ARROW WITH BASE

021A9
LEFTWARDS ARROW WITH HOOK

021AA
RIGHTWARDS ARROW WITH HOOK

021AB
LEFTWARDS ARROW WITH LOOP

021AC
RIGHTWARDS ARROW WITH LOOP

021AD
LEFT RIGHT WAVE ARROW

021AE
LEFT RIGHT ARROW WITH STROKE

021AF
DOWNWARDS ZIGZAG ARROW

021B0
UPWARDS ARROW WITH TIP LEFTWARDS

021B1
UPWARDS ARROW WITH TIP RIGHTWARDS

021B2
DOWNWARDS ARROW WITH TIP LEFTWARDS

021B3
DOWNWARDS ARROW WITH TIP RIGHTWARDS

021B4
RIGHTWARDS ARROW WITH CORNER DOWNWARDS

021B5
DOWNWARDS ARROW WITH CORNER LEFTWARDS

021B6
ANTICLOCKWISE TOP SEMICIRCLE ARROW

021B7
CLOCKWISE TOP SEMICIRCLE ARROW

021B8
NORTH WEST ARROW TO LONG BAR

021B9
LEFTWARDS ARROW TO BAR OVER RIGHTWARDS ARROW TO BAR

021BA
ANTICLOCKWISE OPEN CIRCLE ARROW

021BB
CLOCKWISE OPEN CIRCLE ARROW

021BC
LEFTWARDS HARPOON WITH BARB UPWARDS

021BD
LEFTWARDS HARPOON WITH BARB DOWNWARDS

021BE
UPWARDS HARPOON WITH BARB RIGHTWARDS

021BF
UPWARDS HARPOON WITH BARB LEFTWARDS

021C0
RIGHTWARDS HARPOON WITH BARB UPWARDS

021C1
RIGHTWARDS HARPOON WITH BARB DOWNWARDS

021C2
DOWNWARDS HARPOON WITH BARB RIGHTWARDS

021C3
DOWNWARDS HARPOON WITH BARB LEFTWARDS

021C4
RIGHTWARDS ARROW OVER LEFTWARDS ARROW

021C5
UPWARDS ARROW LEFTWARDS OF DOWNWARDS ARROW

021C6
LEFTWARDS ARROW OVER RIGHTWARDS ARROW

021C7
LEFTWARDS PAIRED ARROWS

021C8
UPWARDS PAIRED ARROWS

021C9
RIGHTWARDS PAIRED ARROWS

021CA
DOWNWARDS PAIRED ARROWS

021CB
LEFTWARDS HARPOON OVER RIGHTWARDS HARPOON

021CC
RIGHTWARDS HARPOON OVER LEFTWARDS HARPOON

021CD
LEFTWARDS DOUBLE ARROW WITH STROKE

021CE
LEFT RIGHT DOUBLE ARROW WITH STROKE

021CF
RIGHTWARDS DOUBLE ARROW WITH STROKE

021D0
LEFTWARDS DOUBLE ARROW

021D1
UPWARDS DOUBLE ARROW

021D2
RIGHTWARDS DOUBLE ARROW

021D3
DOWNWARDS DOUBLE ARROW

021D4
LEFT RIGHT DOUBLE ARROW

021D5
UP DOWN DOUBLE ARROW

021D6
NORTH WEST DOUBLE ARROW

021D7
NORTH EAST DOUBLE ARROW

021D8
SOUTH EAST DOUBLE ARROW

021D9
SOUTH WEST DOUBLE ARROW

021DA
LEFTWARDS TRIPLE ARROW

021DB
RIGHTWARDS TRIPLE ARROW

021DC
LEFTWARDS SQUIGGLE ARROW

021DD
RIGHTWARDS SQUIGGLE ARROW

021DE
UPWARDS ARROW WITH DOUBLE STROKE

021DF
DOWNWARDS ARROW WITH DOUBLE STROKE

021E0
LEFTWARDS DASHED ARROW

021E1
UPWARDS DASHED ARROW

021E2
RIGHTWARDS DASHED ARROW

021E3
DOWNWARDS DASHED ARROW

021E4
LEFTWARDS ARROW TO BAR

021E5
RIGHTWARDS ARROW TO BAR

021E6
LEFTWARDS WHITE ARROW

021E7
UPWARDS WHITE ARROW

021E8
RIGHTWARDS WHITE ARROW

021E9
DOWNWARDS WHITE ARROW

021EA
UPWARDS WHITE ARROW FROM BAR

021EB
UPWARDS WHITE ARROW ON PEDESTAL

021EC
UPWARDS WHITE ARROW ON PEDESTAL WITH HORIZONTAL BAR

021ED
UPWARDS WHITE ARROW ON PEDESTAL WITH VERTICAL BAR

021EE
UPWARDS WHITE DOUBLE ARROW

021EF
UPWARDS WHITE DOUBLE ARROW ON PEDESTAL

021F0
RIGHTWARDS WHITE ARROW FROM WALL

021F1
NORTH WEST ARROW TO CORNER

021F2
SOUTH EAST ARROW TO CORNER

021F3
UP DOWN WHITE ARROW

021F4
RIGHT ARROW WITH SMALL CIRCLE

021F5
DOWNWARDS ARROW LEFTWARDS OF UPWARDS ARROW

021F6
THREE RIGHTWARDS ARROWS

021F7
LEFTWARDS ARROW WITH VERTICAL STROKE

021F8
RIGHTWARDS ARROW WITH VERTICAL STROKE

021F9
LEFT RIGHT ARROW WITH VERTICAL STROKE

021FA
LEFTWARDS ARROW WITH DOUBLE VERTICAL STROKE

021FB
RIGHTWARDS ARROW WITH DOUBLE VERTICAL STROKE

021FC
LEFT RIGHT ARROW WITH DOUBLE VERTICAL STROKE

021FD
LEFTWARDS OPEN-HEADED ARROW

021FE
RIGHTWARDS OPEN-HEADED ARROW

021FF
LEFT RIGHT OPEN-HEADED ARROW

02200
FOR ALL

02201
COMPLEMENT

02202
PARTIAL DIFFERENTIAL

02203
THERE EXISTS

02204
THERE DOES NOT EXIST

02205
EMPTY SET

02206
INCREMENT

02207
NABLA

02208
ELEMENT OF

02209
NOT AN ELEMENT OF

0220A
SMALL ELEMENT OF

0220B
CONTAINS AS MEMBER

0220C
DOES NOT CONTAIN AS MEMBER

0220D
SMALL CONTAINS AS MEMBER

0220E
END OF PROOF

0220F
N-ARY PRODUCT

02210
N-ARY COPRODUCT

02211
N-ARY SUMMATION

02212
MINUS SIGN

02213
MINUS-OR-PLUS SIGN

02214
DOT PLUS

02215
DIVISION SLASH

02216
SET MINUS

02217
ASTERISK OPERATOR

02218
RING OPERATOR

02219
BULLET OPERATOR

0221A
SQUARE ROOT

0221B
CUBE ROOT

0221C
FOURTH ROOT

0221D
PROPORTIONAL TO

0221E
INFINITY

0221F
RIGHT ANGLE

02220
ANGLE

02221
MEASURED ANGLE

02222
SPHERICAL ANGLE

02223
DIVIDES

02224
DOES NOT DIVIDE

02225
PARALLEL TO

02226
NOT PARALLEL TO

02227
LOGICAL AND

02228
LOGICAL OR

02229
INTERSECTION

0222A
UNION

0222B
INTEGRAL

0222C
DOUBLE INTEGRAL

0222D
TRIPLE INTEGRAL

0222E
CONTOUR INTEGRAL

0222F
SURFACE INTEGRAL

02230
VOLUME INTEGRAL

02231
CLOCKWISE INTEGRAL

02232
CLOCKWISE CONTOUR INTEGRAL

02233
ANTICLOCKWISE CONTOUR INTEGRAL

02234
THEREFORE

02235
BECAUSE

02236
RATIO

02237
PROPORTION

02238
DOT MINUS

02239
EXCESS

0223A
GEOMETRIC PROPORTION

0223B
HOMOTHETIC

0223C
TILDE OPERATOR

0223D
REVERSED TILDE

0223E
INVERTED LAZY S

0223F
SINE WAVE

02240
WREATH PRODUCT

02241
NOT TILDE

02242
MINUS TILDE

02243
ASYMPTOTICALLY EQUAL TO

02244
NOT ASYMPTOTICALLY EQUAL TO

02245
APPROXIMATELY EQUAL TO

02246
APPROXIMATELY BUT NOT ACTUALLY EQUAL TO

02247
NEITHER APPROXIMATELY NOR ACTUALLY EQUAL TO

02248
ALMOST EQUAL TO

02249
NOT ALMOST EQUAL TO

0224A
ALMOST EQUAL OR EQUAL TO

0224B
TRIPLE TILDE

0224C
ALL EQUAL TO

0224D
EQUIVALENT TO

0224E
GEOMETRICALLY EQUIVALENT TO

0224F
DIFFERENCE BETWEEN

02250
APPROACHES THE LIMIT

02251
GEOMETRICALLY EQUAL TO

02252
APPROXIMATELY EQUAL TO OR THE IMAGE OF

02253
IMAGE OF OR APPROXIMATELY EQUAL TO

02254
COLON EQUALS

02255
EQUALS COLON

02256
RING IN EQUAL TO

02257
RING EQUAL TO

02258
CORRESPONDS TO

02259
ESTIMATES

0225A
EQUIANGULAR TO

0225B
STAR EQUALS

0225C
DELTA EQUAL TO

0225D
EQUAL TO BY DEFINITION

0225E
MEASURED BY

0225F
QUESTIONED EQUAL TO

02260
NOT EQUAL TO

02261
IDENTICAL TO

02262
NOT IDENTICAL TO

02263
STRICTLY EQUIVALENT TO

02264
LESS-THAN OR EQUAL TO

02265
GREATER-THAN OR EQUAL TO

02266
LESS-THAN OVER EQUAL TO

02267
GREATER-THAN OVER EQUAL TO

02268
LESS-THAN BUT NOT EQUAL TO

02269
GREATER-THAN BUT NOT EQUAL TO

0226A
MUCH LESS-THAN

0226B
MUCH GREATER-THAN

0226C
BETWEEN

0226D
NOT EQUIVALENT TO

0226E
NOT LESS-THAN

0226F
NOT GREATER-THAN

02270
NEITHER LESS-THAN NOR EQUAL TO

02271
NEITHER GREATER-THAN NOR EQUAL TO

02272
LESS-THAN OR EQUIVALENT TO

02273
GREATER-THAN OR EQUIVALENT TO

02274
NEITHER LESS-THAN NOR EQUIVALENT TO

02275
NEITHER GREATER-THAN NOR EQUIVALENT TO

02276
LESS-THAN OR GREATER-THAN

02277
GREATER-THAN OR LESS-THAN

02278
NEITHER LESS-THAN NOR GREATER-THAN

02279
NEITHER GREATER-THAN NOR LESS-THAN

0227A
PRECEDES

0227B
SUCCEEDS

0227C
PRECEDES OR EQUAL TO

0227D
SUCCEEDS OR EQUAL TO

0227E
PRECEDES OR EQUIVALENT TO

0227F
SUCCEEDS OR EQUIVALENT TO

02280
DOES NOT PRECEDE

02281
DOES NOT SUCCEED

02282
SUBSET OF

02283
SUPERSET OF

02284
NOT A SUBSET OF

02285
NOT A SUPERSET OF

02286
SUBSET OF OR EQUAL TO

02287
SUPERSET OF OR EQUAL TO

02288
NEITHER A SUBSET OF NOR EQUAL TO

02289
NEITHER A SUPERSET OF NOR EQUAL TO

0228A
SUBSET OF WITH NOT EQUAL TO

0228B
SUPERSET OF WITH NOT EQUAL TO

0228C
MULTISET

0228D
MULTISET MULTIPLICATION

0228E
MULTISET UNION

0228F
SQUARE IMAGE OF

02290
SQUARE ORIGINAL OF

02291
SQUARE IMAGE OF OR EQUAL TO

02292
SQUARE ORIGINAL OF OR EQUAL TO

02293
SQUARE CAP

02294
SQUARE CUP

02295
CIRCLED PLUS

02296
CIRCLED MINUS

02297
CIRCLED TIMES

02298
CIRCLED DIVISION SLASH

02299
CIRCLED DOT OPERATOR

0229A
CIRCLED RING OPERATOR

0229B
CIRCLED ASTERISK OPERATOR

0229C
CIRCLED EQUALS

0229D
CIRCLED DASH

0229E
SQUARED PLUS

0229F
SQUARED MINUS

022A0
SQUARED TIMES

022A1
SQUARED DOT OPERATOR

022A2
RIGHT TACK

022A3
LEFT TACK

022A4
DOWN TACK

022A5
UP TACK

022A6
ASSERTION

022A7
MODELS

022A8
TRUE

022A9
FORCES

022AA
TRIPLE VERTICAL BAR RIGHT TURNSTILE

022AB
DOUBLE VERTICAL BAR DOUBLE RIGHT TURNSTILE

022AC
DOES NOT PROVE

022AD
NOT TRUE

022AE
DOES NOT FORCE

022AF
NEGATED DOUBLE VERTICAL BAR DOUBLE RIGHT TURNSTILE

022B0
PRECEDES UNDER RELATION

022B1
SUCCEEDS UNDER RELATION

022B2
NORMAL SUBGROUP OF

022B3
CONTAINS AS NORMAL SUBGROUP

022B4
NORMAL SUBGROUP OF OR EQUAL TO

022B5
CONTAINS AS NORMAL SUBGROUP OR EQUAL TO

022B6
ORIGINAL OF

022B7
IMAGE OF

022B8
MULTIMAP

022B9
HERMITIAN CONJUGATE MATRIX

022BA
INTERCALATE

022BB
XOR

022BC
NAND

022BD
NOR

022BE
RIGHT ANGLE WITH ARC

022BF
RIGHT TRIANGLE

022C0
N-ARY LOGICAL AND

022C1
N-ARY LOGICAL OR

022C2
N-ARY INTERSECTION

022C3
N-ARY UNION

022C4
DIAMOND OPERATOR

022C5
DOT OPERATOR

022C6
STAR OPERATOR

022C7
DIVISION TIMES

022C8
BOWTIE

022C9
LEFT NORMAL FACTOR SEMIDIRECT PRODUCT

022CA
RIGHT NORMAL FACTOR SEMIDIRECT PRODUCT

022CB
LEFT SEMIDIRECT PRODUCT

022CC
RIGHT SEMIDIRECT PRODUCT

022CD
REVERSED TILDE EQUALS

022CE
CURLY LOGICAL OR

022CF
CURLY LOGICAL AND

022D0
DOUBLE SUBSET

022D1
DOUBLE SUPERSET

022D2
DOUBLE INTERSECTION

022D3
DOUBLE UNION

022D4
PITCHFORK

022D5
EQUAL AND PARALLEL TO

022D6
LESS-THAN WITH DOT

022D7
GREATER-THAN WITH DOT

022D8
VERY MUCH LESS-THAN

022D9
VERY MUCH GREATER-THAN

022DA
LESS-THAN EQUAL TO OR GREATER-THAN

022DB
GREATER-THAN EQUAL TO OR LESS-THAN

022DC
EQUAL TO OR LESS-THAN

022DD
EQUAL TO OR GREATER-THAN

022DE
EQUAL TO OR PRECEDES

022DF
EQUAL TO OR SUCCEEDS

022E0
DOES NOT PRECEDE OR EQUAL

022E1
DOES NOT SUCCEED OR EQUAL

022E2
NOT SQUARE IMAGE OF OR EQUAL TO

022E3
NOT SQUARE ORIGINAL OF OR EQUAL TO

022E4
SQUARE IMAGE OF OR NOT EQUAL TO

022E5
SQUARE ORIGINAL OF OR NOT EQUAL TO

022E6
LESS-THAN BUT NOT EQUIVALENT TO

022E7
GREATER-THAN BUT NOT EQUIVALENT TO

022E8
PRECEDES BUT NOT EQUIVALENT TO

022E9
SUCCEEDS BUT NOT EQUIVALENT TO

022EA
NOT NORMAL SUBGROUP OF

022EB
DOES NOT CONTAIN AS NORMAL SUBGROUP

022EC
NOT NORMAL SUBGROUP OF OR EQUAL TO

022ED
DOES NOT CONTAIN AS NORMAL SUBGROUP OR EQUAL

022EE
VERTICAL ELLIPSIS

022EF
MIDLINE HORIZONTAL ELLIPSIS

022F0
UP RIGHT DIAGONAL ELLIPSIS

022F1
DOWN RIGHT DIAGONAL ELLIPSIS

022F2
ELEMENT OF WITH LONG HORIZONTAL STROKE

022F3
ELEMENT OF WITH VERTICAL BAR AT END OF HORIZONTAL STROKE

022F4
SMALL ELEMENT OF WITH VERTICAL BAR AT END OF HORIZONTAL STROKE

022F5
ELEMENT OF WITH DOT ABOVE

022F6
ELEMENT OF WITH OVERBAR

022F7
SMALL ELEMENT OF WITH OVERBAR

022F8
ELEMENT OF WITH UNDERBAR

022F9
ELEMENT OF WITH TWO HORIZONTAL STROKES

022FA
CONTAINS WITH LONG HORIZONTAL STROKE

022FB
CONTAINS WITH VERTICAL BAR AT END OF HORIZONTAL STROKE

022FC
SMALL CONTAINS WITH VERTICAL BAR AT END OF HORIZONTAL STROKE

022FD
CONTAINS WITH OVERBAR

022FE
SMALL CONTAINS WITH OVERBAR

022FF
Z NOTATION BAG MEMBERSHIP

02300
DIAMETER SIGN

02301
ELECTRIC ARROW

02302
HOUSE

02303
UP ARROWHEAD

02304
DOWN ARROWHEAD

02305
PROJECTIVE

02306
PERSPECTIVE

02307
WAVY LINE

02308
LEFT CEILING

02309
RIGHT CEILING

0230A
LEFT FLOOR

0230B
RIGHT FLOOR

0230C
BOTTOM RIGHT CROP

0230D
BOTTOM LEFT CROP

0230E
TOP RIGHT CROP

0230F
TOP LEFT CROP

02310
REVERSED NOT SIGN

02311
SQUARE LOZENGE

02312
ARC

02313
SEGMENT

02314
SECTOR

02315
TELEPHONE RECORDER

02316
POSITION INDICATOR

02317
VIEWDATA SQUARE

02318
PLACE OF INTEREST SIGN

02319
TURNED NOT SIGN

0231A
WATCH

0231B
HOURGLASS

0231C
TOP LEFT CORNER

0231D
TOP RIGHT CORNER

0231E
BOTTOM LEFT CORNER

0231F
BOTTOM RIGHT CORNER

02320
TOP HALF INTEGRAL

02321
BOTTOM HALF INTEGRAL

02322
FROWN

02323
SMILE

02324
UP ARROWHEAD BETWEEN TWO HORIZONTAL BARS

02325
OPTION KEY

02326
ERASE TO THE RIGHT

02327
X IN A RECTANGLE BOX

02328
KEYBOARD

02329
LEFT-POINTING ANGLE BRACKET

0232A
RIGHT-POINTING ANGLE BRACKET

0232B
ERASE TO THE LEFT

0232C
BENZENE RING

0232D
CYLINDRICITY

0232E
ALL AROUND-PROFILE

0232F
SYMMETRY

02330
TOTAL RUNOUT

02331
DIMENSION ORIGIN

02332
CONICAL TAPER

02333
SLOPE

02334
COUNTERBORE

02335
COUNTERSINK

02336
APL FUNCTIONAL SYMBOL I-BEAM

02337
APL FUNCTIONAL SYMBOL SQUISH QUAD

02338
APL FUNCTIONAL SYMBOL QUAD EQUAL

02339
APL FUNCTIONAL SYMBOL QUAD DIVIDE

0233A
APL FUNCTIONAL SYMBOL QUAD DIAMOND

0233B
APL FUNCTIONAL SYMBOL QUAD JOT

0233C
APL FUNCTIONAL SYMBOL QUAD CIRCLE

0233D
APL FUNCTIONAL SYMBOL CIRCLE STILE

0233E
APL FUNCTIONAL SYMBOL CIRCLE JOT

0233F
APL FUNCTIONAL SYMBOL SLASH BAR

02340
APL FUNCTIONAL SYMBOL BACKSLASH BAR

02341
APL FUNCTIONAL SYMBOL QUAD SLASH

02342
APL FUNCTIONAL SYMBOL QUAD BACKSLASH

02343
APL FUNCTIONAL SYMBOL QUAD LESS-THAN

02344
APL FUNCTIONAL SYMBOL QUAD GREATER-THAN

02345
APL FUNCTIONAL SYMBOL LEFTWARDS VANE

02346
APL FUNCTIONAL SYMBOL RIGHTWARDS VANE

02347
APL FUNCTIONAL SYMBOL QUAD LEFTWARDS ARROW

02348
APL FUNCTIONAL SYMBOL QUAD RIGHTWARDS ARROW

02349
APL FUNCTIONAL SYMBOL CIRCLE BACKSLASH

0234A
APL FUNCTIONAL SYMBOL DOWN TACK UNDERBAR

0234B
APL FUNCTIONAL SYMBOL DELTA STILE

0234C
APL FUNCTIONAL SYMBOL QUAD DOWN CARET

0234D
APL FUNCTIONAL SYMBOL QUAD DELTA

0234E
APL FUNCTIONAL SYMBOL DOWN TACK JOT

0234F
APL FUNCTIONAL SYMBOL UPWARDS VANE

02350
APL FUNCTIONAL SYMBOL QUAD UPWARDS ARROW

02351
APL FUNCTIONAL SYMBOL UP TACK OVERBAR

02352
APL FUNCTIONAL SYMBOL DEL STILE

02353
APL FUNCTIONAL SYMBOL QUAD UP CARET

02354
APL FUNCTIONAL SYMBOL QUAD DEL

02355
APL FUNCTIONAL SYMBOL UP TACK JOT

02356
APL FUNCTIONAL SYMBOL DOWNWARDS VANE

02357
APL FUNCTIONAL SYMBOL QUAD DOWNWARDS ARROW

02358
APL FUNCTIONAL SYMBOL QUOTE UNDERBAR

02359
APL FUNCTIONAL SYMBOL DELTA UNDERBAR

0235A
APL FUNCTIONAL SYMBOL DIAMOND UNDERBAR

0235B
APL FUNCTIONAL SYMBOL JOT UNDERBAR

0235C
APL FUNCTIONAL SYMBOL CIRCLE UNDERBAR

0235D
APL FUNCTIONAL SYMBOL UP SHOE JOT

0235E
APL FUNCTIONAL SYMBOL QUOTE QUAD

0235F
APL FUNCTIONAL SYMBOL CIRCLE STAR

02360
APL FUNCTIONAL SYMBOL QUAD COLON

02361
APL FUNCTIONAL SYMBOL UP TACK DIAERESIS

02362
APL FUNCTIONAL SYMBOL DEL DIAERESIS

02363
APL FUNCTIONAL SYMBOL STAR DIAERESIS

02364
APL FUNCTIONAL SYMBOL JOT DIAERESIS

02365
APL FUNCTIONAL SYMBOL CIRCLE DIAERESIS

02366
APL FUNCTIONAL SYMBOL DOWN SHOE STILE

02367
APL FUNCTIONAL SYMBOL LEFT SHOE STILE

02368
APL FUNCTIONAL SYMBOL TILDE DIAERESIS

02369
APL FUNCTIONAL SYMBOL GREATER-THAN DIAERESIS

0236A
APL FUNCTIONAL SYMBOL COMMA BAR

0236B
APL FUNCTIONAL SYMBOL DEL TILDE

0236C
APL FUNCTIONAL SYMBOL ZILDE

0236D
APL FUNCTIONAL SYMBOL STILE TILDE

0236E
APL FUNCTIONAL SYMBOL SEMICOLON UNDERBAR

0236F
APL FUNCTIONAL SYMBOL QUAD NOT EQUAL

02370
APL FUNCTIONAL SYMBOL QUAD QUESTION

02371
APL FUNCTIONAL SYMBOL DOWN CARET TILDE

02372
APL FUNCTIONAL SYMBOL UP CARET TILDE

02373
APL FUNCTIONAL SYMBOL IOTA

02374
APL FUNCTIONAL SYMBOL RHO

02375
APL FUNCTIONAL SYMBOL OMEGA

02376
APL FUNCTIONAL SYMBOL ALPHA UNDERBAR

02377
APL FUNCTIONAL SYMBOL EPSILON UNDERBAR

02378
APL FUNCTIONAL SYMBOL IOTA UNDERBAR

02379
APL FUNCTIONAL SYMBOL OMEGA UNDERBAR

0237A
APL FUNCTIONAL SYMBOL ALPHA

0237B
NOT CHECK MARK

0237C
RIGHT ANGLE WITH DOWNWARDS ZIGZAG ARROW

0237D
SHOULDERED OPEN BOX

0237E
BELL SYMBOL

0237F
VERTICAL LINE WITH MIDDLE DOT

02380
INSERTION SYMBOL

02381
CONTINUOUS UNDERLINE SYMBOL

02382
DISCONTINUOUS UNDERLINE SYMBOL

02383
EMPHASIS SYMBOL

02384
COMPOSITION SYMBOL

02385
WHITE SQUARE WITH CENTRE VERTICAL LINE

02386
ENTER SYMBOL

02387
ALTERNATIVE KEY SYMBOL

02388
HELM SYMBOL

02389
CIRCLED HORIZONTAL BAR WITH NOTCH

0238A
CIRCLED TRIANGLE DOWN

0238B
BROKEN CIRCLE WITH NORTHWEST ARROW

0238C
UNDO SYMBOL

0238D
MONOSTABLE SYMBOL

0238E
HYSTERESIS SYMBOL

0238F
OPEN-CIRCUIT-OUTPUT H-TYPE SYMBOL

02390
OPEN-CIRCUIT-OUTPUT L-TYPE SYMBOL

02391
PASSIVE-PULL-DOWN-OUTPUT SYMBOL

02392
PASSIVE-PULL-UP-OUTPUT SYMBOL

02393
DIRECT CURRENT SYMBOL FORM TWO

02394
SOFTWARE-FUNCTION SYMBOL

02395
APL FUNCTIONAL SYMBOL QUAD

02396
DECIMAL SEPARATOR KEY SYMBOL

02397
PREVIOUS PAGE

02398
NEXT PAGE

02399
PRINT SCREEN SYMBOL

0239A
CLEAR SCREEN SYMBOL

0239B
LEFT PARENTHESIS UPPER HOOK

0239C
LEFT PARENTHESIS EXTENSION

0239D
LEFT PARENTHESIS LOWER HOOK

0239E
RIGHT PARENTHESIS UPPER HOOK

0239F
RIGHT PARENTHESIS EXTENSION

023A0
RIGHT PARENTHESIS LOWER HOOK

023A1
LEFT SQUARE BRACKET UPPER CORNER

023A2
LEFT SQUARE BRACKET EXTENSION

023A3
LEFT SQUARE BRACKET LOWER CORNER

023A4
RIGHT SQUARE BRACKET UPPER CORNER

023A5
RIGHT SQUARE BRACKET EXTENSION

023A6
RIGHT SQUARE BRACKET LOWER CORNER

023A7
LEFT CURLY BRACKET UPPER HOOK

023A8
LEFT CURLY BRACKET MIDDLE PIECE

023A9
LEFT CURLY BRACKET LOWER HOOK

023AA
CURLY BRACKET EXTENSION

023AB
RIGHT CURLY BRACKET UPPER HOOK

023AC
RIGHT CURLY BRACKET MIDDLE PIECE

023AD
RIGHT CURLY BRACKET LOWER HOOK

023AE
INTEGRAL EXTENSION

023AF
HORIZONTAL LINE EXTENSION

023B0
UPPER LEFT OR LOWER RIGHT CURLY BRACKET SECTION

023B1
UPPER RIGHT OR LOWER LEFT CURLY BRACKET SECTION

023B2
SUMMATION TOP

023B3
SUMMATION BOTTOM

023B4
TOP SQUARE BRACKET

023B5
BOTTOM SQUARE BRACKET

023B6
BOTTOM SQUARE BRACKET OVER TOP SQUARE BRACKET

023B7
RADICAL SYMBOL BOTTOM

023B8
LEFT VERTICAL BOX LINE

023B9
RIGHT VERTICAL BOX LINE

023BA
HORIZONTAL SCAN LINE-1

023BB
HORIZONTAL SCAN LINE-3

023BC
HORIZONTAL SCAN LINE-7

023BD
HORIZONTAL SCAN LINE-9

023BE
DENTISTRY SYMBOL LIGHT VERTICAL AND TOP RIGHT

023BF
DENTISTRY SYMBOL LIGHT VERTICAL AND BOTTOM RIGHT

023C0
DENTISTRY SYMBOL LIGHT VERTICAL WITH CIRCLE

023C1
DENTISTRY SYMBOL LIGHT DOWN AND HORIZONTAL WITH CIRCLE

023C2
DENTISTRY SYMBOL LIGHT UP AND HORIZONTAL WITH CIRCLE

023C3
DENTISTRY SYMBOL LIGHT VERTICAL WITH TRIANGLE

023C4
DENTISTRY SYMBOL LIGHT DOWN AND HORIZONTAL WITH TRIANGLE

023C5
DENTISTRY SYMBOL LIGHT UP AND HORIZONTAL WITH TRIANGLE

023C6
DENTISTRY SYMBOL LIGHT VERTICAL AND WAVE

023C7
DENTISTRY SYMBOL LIGHT DOWN AND HORIZONTAL WITH WAVE

023C8
DENTISTRY SYMBOL LIGHT UP AND HORIZONTAL WITH WAVE

023C9
DENTISTRY SYMBOL LIGHT DOWN AND HORIZONTAL

023CA
DENTISTRY SYMBOL LIGHT UP AND HORIZONTAL

023CB
DENTISTRY SYMBOL LIGHT VERTICAL AND TOP LEFT

023CC
DENTISTRY SYMBOL LIGHT VERTICAL AND BOTTOM LEFT

023CD
SQUARE FOOT

023CE
RETURN SYMBOL

023CF
EJECT SYMBOL

023D0
VERTICAL LINE EXTENSION

023D1
METRICAL BREVE

023D2
METRICAL LONG OVER SHORT

023D3
METRICAL SHORT OVER LONG

023D4
METRICAL LONG OVER TWO SHORTS

023D5
METRICAL TWO SHORTS OVER LONG

023D6
METRICAL TWO SHORTS JOINED

023D7
METRICAL TRISEME

023D8
METRICAL TETRASEME

023D9
METRICAL PENTASEME

023DA
EARTH GROUND

023DB
FUSE

023DC
TOP PARENTHESIS

023DD
BOTTOM PARENTHESIS

023DE
TOP CURLY BRACKET

023DF
BOTTOM CURLY BRACKET

023E0
TOP TORTOISE SHELL BRACKET

023E1
BOTTOM TORTOISE SHELL BRACKET

023E2
WHITE TRAPEZIUM

023E3
BENZENE RING WITH CIRCLE

023E4
STRAIGHTNESS

023E5
FLATNESS

023E6
AC CURRENT

023E7
ELECTRICAL INTERSECTION

023E8
DECIMAL EXPONENT SYMBOL

023E9
BLACK RIGHT-POINTING DOUBLE TRIANGLE

023EA
BLACK LEFT-POINTING DOUBLE TRIANGLE

023EB
BLACK UP-POINTING DOUBLE TRIANGLE

023EC
BLACK DOWN-POINTING DOUBLE TRIANGLE

023ED
BLACK RIGHT-POINTING DOUBLE TRIANGLE WITH VERTICAL BAR

023EE
BLACK LEFT-POINTING DOUBLE TRIANGLE WITH VERTICAL BAR

023EF
BLACK RIGHT-POINTING TRIANGLE WITH DOUBLE VERTICAL BAR

023F0
ALARM CLOCK

023F1
STOPWATCH

023F2
TIMER CLOCK

023F3
HOURGLASS WITH FLOWING SAND

023F4
BLACK MEDIUM LEFT-POINTING TRIANGLE

023F5
BLACK MEDIUM RIGHT-POINTING TRIANGLE

023F6
BLACK MEDIUM UP-POINTING TRIANGLE

023F7
BLACK MEDIUM DOWN-POINTING TRIANGLE

023F8
DOUBLE VERTICAL BAR

023F9
BLACK SQUARE FOR STOP

023FA
BLACK CIRCLE FOR RECORD

023FB
POWER SYMBOL

023FC
POWER ON-OFF SYMBOL

023FD
POWER ON SYMBOL

023FE
POWER SLEEP SYMBOL

023FF
OBSERVER EYE SYMBOL

02400
SYMBOL FOR NULL

02401
SYMBOL FOR START OF HEADING

02402
SYMBOL FOR START OF TEXT

02403
SYMBOL FOR END OF TEXT

02404
SYMBOL FOR END OF TRANSMISSION

02405
SYMBOL FOR ENQUIRY

02406
SYMBOL FOR ACKNOWLEDGE

02407
SYMBOL FOR BELL

02408
SYMBOL FOR BACKSPACE

02409
SYMBOL FOR HORIZONTAL TABULATION

0240A
SYMBOL FOR LINE FEED

0240B
SYMBOL FOR VERTICAL TABULATION

0240C
SYMBOL FOR FORM FEED

0240D
SYMBOL FOR CARRIAGE RETURN

0240E
SYMBOL FOR SHIFT OUT

0240F
SYMBOL FOR SHIFT IN

02410
SYMBOL FOR DATA LINK ESCAPE

02411
SYMBOL FOR DEVICE CONTROL ONE

02412
SYMBOL FOR DEVICE CONTROL TWO

02413
SYMBOL FOR DEVICE CONTROL THREE

02414
SYMBOL FOR DEVICE CONTROL FOUR

02415
SYMBOL FOR NEGATIVE ACKNOWLEDGE

02416
SYMBOL FOR SYNCHRONOUS IDLE

02417
SYMBOL FOR END OF TRANSMISSION BLOCK

02418
SYMBOL FOR CANCEL

02419
SYMBOL FOR END OF MEDIUM

0241A
SYMBOL FOR SUBSTITUTE

0241B
SYMBOL FOR ESCAPE

0241C
SYMBOL FOR FILE SEPARATOR

0241D
SYMBOL FOR GROUP SEPARATOR

0241E
SYMBOL FOR RECORD SEPARATOR

0241F
SYMBOL FOR UNIT SEPARATOR

02420
SYMBOL FOR SPACE

02421
SYMBOL FOR DELETE

02422
BLANK SYMBOL

02423
OPEN BOX

02424
SYMBOL FOR NEWLINE

02425
SYMBOL FOR DELETE FORM TWO

02426
SYMBOL FOR SUBSTITUTE FORM TWO

02427
SYMBOL FOR DELETE SQUARE CHECKER BOARD FORM

02428
SYMBOL FOR DELETE RECTANGULAR CHECKER BOARD FORM

02429
SYMBOL FOR DELETE MEDIUM SHADE FORM

02440
OCR HOOK

02441
OCR CHAIR

02442
OCR FORK

02443
OCR INVERTED FORK

02444
OCR BELT BUCKLE

02445
OCR BOW TIE

02446
OCR BRANCH BANK IDENTIFICATION

02447
OCR AMOUNT OF CHECK

02448
MICR ON US SYMBOL

02448
OCR DASH

02449
MICR DASH SYMBOL

02449
OCR CUSTOMER ACCOUNT NUMBER

0244A
OCR DOUBLE BACKSLASH

02460
CIRCLED DIGIT ONE

02461
CIRCLED DIGIT TWO

02462
CIRCLED DIGIT THREE

02463
CIRCLED DIGIT FOUR

02464
CIRCLED DIGIT FIVE

02465
CIRCLED DIGIT SIX

02466
CIRCLED DIGIT SEVEN

02467
CIRCLED DIGIT EIGHT

02468
CIRCLED DIGIT NINE

02469
CIRCLED NUMBER TEN

0246A
CIRCLED NUMBER ELEVEN

0246B
CIRCLED NUMBER TWELVE

0246C
CIRCLED NUMBER THIRTEEN

0246D
CIRCLED NUMBER FOURTEEN

0246E
CIRCLED NUMBER FIFTEEN

0246F
CIRCLED NUMBER SIXTEEN

02470
CIRCLED NUMBER SEVENTEEN

02471
CIRCLED NUMBER EIGHTEEN

02472
CIRCLED NUMBER NINETEEN

02473
CIRCLED NUMBER TWENTY

02474
PARENTHESIZED DIGIT ONE

02475
PARENTHESIZED DIGIT TWO

02476
PARENTHESIZED DIGIT THREE

02477
PARENTHESIZED DIGIT FOUR

02478
PARENTHESIZED DIGIT FIVE

02479
PARENTHESIZED DIGIT SIX

0247A
PARENTHESIZED DIGIT SEVEN

0247B
PARENTHESIZED DIGIT EIGHT

0247C
PARENTHESIZED DIGIT NINE

0247D
PARENTHESIZED NUMBER TEN

0247E
PARENTHESIZED NUMBER ELEVEN

0247F
PARENTHESIZED NUMBER TWELVE

02480
PARENTHESIZED NUMBER THIRTEEN

02481
PARENTHESIZED NUMBER FOURTEEN

02482
PARENTHESIZED NUMBER FIFTEEN

02483
PARENTHESIZED NUMBER SIXTEEN

02484
PARENTHESIZED NUMBER SEVENTEEN

02485
PARENTHESIZED NUMBER EIGHTEEN

02486
PARENTHESIZED NUMBER NINETEEN

02487
PARENTHESIZED NUMBER TWENTY

02488
DIGIT ONE FULL STOP

02489
DIGIT TWO FULL STOP

0248A
DIGIT THREE FULL STOP

0248B
DIGIT FOUR FULL STOP

0248C
DIGIT FIVE FULL STOP

0248D
DIGIT SIX FULL STOP

0248E
DIGIT SEVEN FULL STOP

0248F
DIGIT EIGHT FULL STOP

02490
DIGIT NINE FULL STOP

02491
NUMBER TEN FULL STOP

02492
NUMBER ELEVEN FULL STOP

02493
NUMBER TWELVE FULL STOP

02494
NUMBER THIRTEEN FULL STOP

02495
NUMBER FOURTEEN FULL STOP

02496
NUMBER FIFTEEN FULL STOP

02497
NUMBER SIXTEEN FULL STOP

02498
NUMBER SEVENTEEN FULL STOP

02499
NUMBER EIGHTEEN FULL STOP

0249A
NUMBER NINETEEN FULL STOP

0249B
NUMBER TWENTY FULL STOP

0249C
PARENTHESIZED LATIN SMALL LETTER A

0249D
PARENTHESIZED LATIN SMALL LETTER B

0249E
PARENTHESIZED LATIN SMALL LETTER C

0249F
PARENTHESIZED LATIN SMALL LETTER D

024A0
PARENTHESIZED LATIN SMALL LETTER E

024A1
PARENTHESIZED LATIN SMALL LETTER F

024A2
PARENTHESIZED LATIN SMALL LETTER G

024A3
PARENTHESIZED LATIN SMALL LETTER H

024A4
PARENTHESIZED LATIN SMALL LETTER I

024A5
PARENTHESIZED LATIN SMALL LETTER J

024A6
PARENTHESIZED LATIN SMALL LETTER K

024A7
PARENTHESIZED LATIN SMALL LETTER L

024A8
PARENTHESIZED LATIN SMALL LETTER M

024A9
PARENTHESIZED LATIN SMALL LETTER N

024AA
PARENTHESIZED LATIN SMALL LETTER O

024AB
PARENTHESIZED LATIN SMALL LETTER P

024AC
PARENTHESIZED LATIN SMALL LETTER Q

024AD
PARENTHESIZED LATIN SMALL LETTER R

024AE
PARENTHESIZED LATIN SMALL LETTER S

024AF
PARENTHESIZED LATIN SMALL LETTER T

024B0
PARENTHESIZED LATIN SMALL LETTER U

024B1
PARENTHESIZED LATIN SMALL LETTER V

024B2
PARENTHESIZED LATIN SMALL LETTER W

024B3
PARENTHESIZED LATIN SMALL LETTER X

024B4
PARENTHESIZED LATIN SMALL LETTER Y

024B5
PARENTHESIZED LATIN SMALL LETTER Z

024B6
CIRCLED LATIN CAPITAL LETTER A

024B7
CIRCLED LATIN CAPITAL LETTER B

024B8
CIRCLED LATIN CAPITAL LETTER C

024B9
CIRCLED LATIN CAPITAL LETTER D

024BA
CIRCLED LATIN CAPITAL LETTER E

024BB
CIRCLED LATIN CAPITAL LETTER F

024BC
CIRCLED LATIN CAPITAL LETTER G

024BD
CIRCLED LATIN CAPITAL LETTER H

024BE
CIRCLED LATIN CAPITAL LETTER I

024BF
CIRCLED LATIN CAPITAL LETTER J

024C0
CIRCLED LATIN CAPITAL LETTER K

024C1
CIRCLED LATIN CAPITAL LETTER L

024C2
CIRCLED LATIN CAPITAL LETTER M

024C3
CIRCLED LATIN CAPITAL LETTER N

024C4
CIRCLED LATIN CAPITAL LETTER O

024C5
CIRCLED LATIN CAPITAL LETTER P

024C6
CIRCLED LATIN CAPITAL LETTER Q

024C7
CIRCLED LATIN CAPITAL LETTER R

024C8
CIRCLED LATIN CAPITAL LETTER S

024C9
CIRCLED LATIN CAPITAL LETTER T

024CA
CIRCLED LATIN CAPITAL LETTER U

024CB
CIRCLED LATIN CAPITAL LETTER V

024CC
CIRCLED LATIN CAPITAL LETTER W

024CD
CIRCLED LATIN CAPITAL LETTER X

024CE
CIRCLED LATIN CAPITAL LETTER Y

024CF
CIRCLED LATIN CAPITAL LETTER Z

024D0
CIRCLED LATIN SMALL LETTER A

024D1
CIRCLED LATIN SMALL LETTER B

024D2
CIRCLED LATIN SMALL LETTER C

024D3
CIRCLED LATIN SMALL LETTER D

024D4
CIRCLED LATIN SMALL LETTER E

024D5
CIRCLED LATIN SMALL LETTER F

024D6
CIRCLED LATIN SMALL LETTER G

024D7
CIRCLED LATIN SMALL LETTER H

024D8
CIRCLED LATIN SMALL LETTER I

024D9
CIRCLED LATIN SMALL LETTER J

024DA
CIRCLED LATIN SMALL LETTER K

024DB
CIRCLED LATIN SMALL LETTER L

024DC
CIRCLED LATIN SMALL LETTER M

024DD
CIRCLED LATIN SMALL LETTER N

024DE
CIRCLED LATIN SMALL LETTER O

024DF
CIRCLED LATIN SMALL LETTER P

024E0
CIRCLED LATIN SMALL LETTER Q

024E1
CIRCLED LATIN SMALL LETTER R

024E2
CIRCLED LATIN SMALL LETTER S

024E3
CIRCLED LATIN SMALL LETTER T

024E4
CIRCLED LATIN SMALL LETTER U

024E5
CIRCLED LATIN SMALL LETTER V

024E6
CIRCLED LATIN SMALL LETTER W

024E7
CIRCLED LATIN SMALL LETTER X

024E8
CIRCLED LATIN SMALL LETTER Y

024E9
CIRCLED LATIN SMALL LETTER Z

024EA
CIRCLED DIGIT ZERO

024EB
NEGATIVE CIRCLED NUMBER ELEVEN

024EC
NEGATIVE CIRCLED NUMBER TWELVE

024ED
NEGATIVE CIRCLED NUMBER THIRTEEN

024EE
NEGATIVE CIRCLED NUMBER FOURTEEN

024EF
NEGATIVE CIRCLED NUMBER FIFTEEN

024F0
NEGATIVE CIRCLED NUMBER SIXTEEN

024F1
NEGATIVE CIRCLED NUMBER SEVENTEEN

024F2
NEGATIVE CIRCLED NUMBER EIGHTEEN

024F3
NEGATIVE CIRCLED NUMBER NINETEEN

024F4
NEGATIVE CIRCLED NUMBER TWENTY

024F5
DOUBLE CIRCLED DIGIT ONE

024F6
DOUBLE CIRCLED DIGIT TWO

024F7
DOUBLE CIRCLED DIGIT THREE

024F8
DOUBLE CIRCLED DIGIT FOUR

024F9
DOUBLE CIRCLED DIGIT FIVE

024FA
DOUBLE CIRCLED DIGIT SIX

024FB
DOUBLE CIRCLED DIGIT SEVEN

024FC
DOUBLE CIRCLED DIGIT EIGHT

024FD
DOUBLE CIRCLED DIGIT NINE

024FE
DOUBLE CIRCLED NUMBER TEN

024FF
NEGATIVE CIRCLED DIGIT ZERO

02500
BOX DRAWINGS LIGHT HORIZONTAL

02501
BOX DRAWINGS HEAVY HORIZONTAL

02502
BOX DRAWINGS LIGHT VERTICAL

02503
BOX DRAWINGS HEAVY VERTICAL

02504
BOX DRAWINGS LIGHT TRIPLE DASH HORIZONTAL

02505
BOX DRAWINGS HEAVY TRIPLE DASH HORIZONTAL

02506
BOX DRAWINGS LIGHT TRIPLE DASH VERTICAL

02507
BOX DRAWINGS HEAVY TRIPLE DASH VERTICAL

02508
BOX DRAWINGS LIGHT QUADRUPLE DASH HORIZONTAL

02509
BOX DRAWINGS HEAVY QUADRUPLE DASH HORIZONTAL

0250A
BOX DRAWINGS LIGHT QUADRUPLE DASH VERTICAL

0250B
BOX DRAWINGS HEAVY QUADRUPLE DASH VERTICAL

0250C
BOX DRAWINGS LIGHT DOWN AND RIGHT

0250D
BOX DRAWINGS DOWN LIGHT AND RIGHT HEAVY

0250E
BOX DRAWINGS DOWN HEAVY AND RIGHT LIGHT

0250F
BOX DRAWINGS HEAVY DOWN AND RIGHT

02510
BOX DRAWINGS LIGHT DOWN AND LEFT

02511
BOX DRAWINGS DOWN LIGHT AND LEFT HEAVY

02512
BOX DRAWINGS DOWN HEAVY AND LEFT LIGHT

02513
BOX DRAWINGS HEAVY DOWN AND LEFT

02514
BOX DRAWINGS LIGHT UP AND RIGHT

02515
BOX DRAWINGS UP LIGHT AND RIGHT HEAVY

02516
BOX DRAWINGS UP HEAVY AND RIGHT LIGHT

02517
BOX DRAWINGS HEAVY UP AND RIGHT

02518
BOX DRAWINGS LIGHT UP AND LEFT

02519
BOX DRAWINGS UP LIGHT AND LEFT HEAVY

0251A
BOX DRAWINGS UP HEAVY AND LEFT LIGHT

0251B
BOX DRAWINGS HEAVY UP AND LEFT

0251C
BOX DRAWINGS LIGHT VERTICAL AND RIGHT

0251D
BOX DRAWINGS VERTICAL LIGHT AND RIGHT HEAVY

0251E
BOX DRAWINGS UP HEAVY AND RIGHT DOWN LIGHT

0251F
BOX DRAWINGS DOWN HEAVY AND RIGHT UP LIGHT

02520
BOX DRAWINGS VERTICAL HEAVY AND RIGHT LIGHT

02521
BOX DRAWINGS DOWN LIGHT AND RIGHT UP HEAVY

02522
BOX DRAWINGS UP LIGHT AND RIGHT DOWN HEAVY

02523
BOX DRAWINGS HEAVY VERTICAL AND RIGHT

02524
BOX DRAWINGS LIGHT VERTICAL AND LEFT

02525
BOX DRAWINGS VERTICAL LIGHT AND LEFT HEAVY

02526
BOX DRAWINGS UP HEAVY AND LEFT DOWN LIGHT

02527
BOX DRAWINGS DOWN HEAVY AND LEFT UP LIGHT

02528
BOX DRAWINGS VERTICAL HEAVY AND LEFT LIGHT

02529
BOX DRAWINGS DOWN LIGHT AND LEFT UP HEAVY

0252A
BOX DRAWINGS UP LIGHT AND LEFT DOWN HEAVY

0252B
BOX DRAWINGS HEAVY VERTICAL AND LEFT

0252C
BOX DRAWINGS LIGHT DOWN AND HORIZONTAL

0252D
BOX DRAWINGS LEFT HEAVY AND RIGHT DOWN LIGHT

0252E
BOX DRAWINGS RIGHT HEAVY AND LEFT DOWN LIGHT

0252F
BOX DRAWINGS DOWN LIGHT AND HORIZONTAL HEAVY

02530
BOX DRAWINGS DOWN HEAVY AND HORIZONTAL LIGHT

02531
BOX DRAWINGS RIGHT LIGHT AND LEFT DOWN HEAVY

02532
BOX DRAWINGS LEFT LIGHT AND RIGHT DOWN HEAVY

02533
BOX DRAWINGS HEAVY DOWN AND HORIZONTAL

02534
BOX DRAWINGS LIGHT UP AND HORIZONTAL

02535
BOX DRAWINGS LEFT HEAVY AND RIGHT UP LIGHT

02536
BOX DRAWINGS RIGHT HEAVY AND LEFT UP LIGHT

02537
BOX DRAWINGS UP LIGHT AND HORIZONTAL HEAVY

02538
BOX DRAWINGS UP HEAVY AND HORIZONTAL LIGHT

02539
BOX DRAWINGS RIGHT LIGHT AND LEFT UP HEAVY

0253A
BOX DRAWINGS LEFT LIGHT AND RIGHT UP HEAVY

0253B
BOX DRAWINGS HEAVY UP AND HORIZONTAL

0253C
BOX DRAWINGS LIGHT VERTICAL AND HORIZONTAL

0253D
BOX DRAWINGS LEFT HEAVY AND RIGHT VERTICAL LIGHT

0253E
BOX DRAWINGS RIGHT HEAVY AND LEFT VERTICAL LIGHT

0253F
BOX DRAWINGS VERTICAL LIGHT AND HORIZONTAL HEAVY

02540
BOX DRAWINGS UP HEAVY AND DOWN HORIZONTAL LIGHT

02541
BOX DRAWINGS DOWN HEAVY AND UP HORIZONTAL LIGHT

02542
BOX DRAWINGS VERTICAL HEAVY AND HORIZONTAL LIGHT

02543
BOX DRAWINGS LEFT UP HEAVY AND RIGHT DOWN LIGHT

02544
BOX DRAWINGS RIGHT UP HEAVY AND LEFT DOWN LIGHT

02545
BOX DRAWINGS LEFT DOWN HEAVY AND RIGHT UP LIGHT

02546
BOX DRAWINGS RIGHT DOWN HEAVY AND LEFT UP LIGHT

02547
BOX DRAWINGS DOWN LIGHT AND UP HORIZONTAL HEAVY

02548
BOX DRAWINGS UP LIGHT AND DOWN HORIZONTAL HEAVY

02549
BOX DRAWINGS RIGHT LIGHT AND LEFT VERTICAL HEAVY

0254A
BOX DRAWINGS LEFT LIGHT AND RIGHT VERTICAL HEAVY

0254B
BOX DRAWINGS HEAVY VERTICAL AND HORIZONTAL

0254C
BOX DRAWINGS LIGHT DOUBLE DASH HORIZONTAL

0254D
BOX DRAWINGS HEAVY DOUBLE DASH HORIZONTAL

0254E
BOX DRAWINGS LIGHT DOUBLE DASH VERTICAL

0254F
BOX DRAWINGS HEAVY DOUBLE DASH VERTICAL

02550
BOX DRAWINGS DOUBLE HORIZONTAL

02551
BOX DRAWINGS DOUBLE VERTICAL

02552
BOX DRAWINGS DOWN SINGLE AND RIGHT DOUBLE

02553
BOX DRAWINGS DOWN DOUBLE AND RIGHT SINGLE

02554
BOX DRAWINGS DOUBLE DOWN AND RIGHT

02555
BOX DRAWINGS DOWN SINGLE AND LEFT DOUBLE

02556
BOX DRAWINGS DOWN DOUBLE AND LEFT SINGLE

02557
BOX DRAWINGS DOUBLE DOWN AND LEFT

02558
BOX DRAWINGS UP SINGLE AND RIGHT DOUBLE

02559
BOX DRAWINGS UP DOUBLE AND RIGHT SINGLE

0255A
BOX DRAWINGS DOUBLE UP AND RIGHT

0255B
BOX DRAWINGS UP SINGLE AND LEFT DOUBLE

0255C
BOX DRAWINGS UP DOUBLE AND LEFT SINGLE

0255D
BOX DRAWINGS DOUBLE UP AND LEFT

0255E
BOX DRAWINGS VERTICAL SINGLE AND RIGHT DOUBLE

0255F
BOX DRAWINGS VERTICAL DOUBLE AND RIGHT SINGLE

02560
BOX DRAWINGS DOUBLE VERTICAL AND RIGHT

02561
BOX DRAWINGS VERTICAL SINGLE AND LEFT DOUBLE

02562
BOX DRAWINGS VERTICAL DOUBLE AND LEFT SINGLE

02563
BOX DRAWINGS DOUBLE VERTICAL AND LEFT

02564
BOX DRAWINGS DOWN SINGLE AND HORIZONTAL DOUBLE

02565
BOX DRAWINGS DOWN DOUBLE AND HORIZONTAL SINGLE

02566
BOX DRAWINGS DOUBLE DOWN AND HORIZONTAL

02567
BOX DRAWINGS UP SINGLE AND HORIZONTAL DOUBLE

02568
BOX DRAWINGS UP DOUBLE AND HORIZONTAL SINGLE

02569
BOX DRAWINGS DOUBLE UP AND HORIZONTAL

0256A
BOX DRAWINGS VERTICAL SINGLE AND HORIZONTAL DOUBLE

0256B
BOX DRAWINGS VERTICAL DOUBLE AND HORIZONTAL SINGLE

0256C
BOX DRAWINGS DOUBLE VERTICAL AND HORIZONTAL

0256D
BOX DRAWINGS LIGHT ARC DOWN AND RIGHT

0256E
BOX DRAWINGS LIGHT ARC DOWN AND LEFT

0256F
BOX DRAWINGS LIGHT ARC UP AND LEFT

02570
BOX DRAWINGS LIGHT ARC UP AND RIGHT

02571
BOX DRAWINGS LIGHT DIAGONAL UPPER RIGHT TO LOWER LEFT

02572
BOX DRAWINGS LIGHT DIAGONAL UPPER LEFT TO LOWER RIGHT

02573
BOX DRAWINGS LIGHT DIAGONAL CROSS

02574
BOX DRAWINGS LIGHT LEFT

02575
BOX DRAWINGS LIGHT UP

02576
BOX DRAWINGS LIGHT RIGHT

02577
BOX DRAWINGS LIGHT DOWN

02578
BOX DRAWINGS HEAVY LEFT

02579
BOX DRAWINGS HEAVY UP

0257A
BOX DRAWINGS HEAVY RIGHT

0257B
BOX DRAWINGS HEAVY DOWN

0257C
BOX DRAWINGS LIGHT LEFT AND HEAVY RIGHT

0257D
BOX DRAWINGS LIGHT UP AND HEAVY DOWN

0257E
BOX DRAWINGS HEAVY LEFT AND LIGHT RIGHT

0257F
BOX DRAWINGS HEAVY UP AND LIGHT DOWN

02580
UPPER HALF BLOCK

02581
LOWER ONE EIGHTH BLOCK

02582
LOWER ONE QUARTER BLOCK

02583
LOWER THREE EIGHTHS BLOCK

02584
LOWER HALF BLOCK

02585
LOWER FIVE EIGHTHS BLOCK

02586
LOWER THREE QUARTERS BLOCK

02587
LOWER SEVEN EIGHTHS BLOCK

02588
FULL BLOCK

02589
LEFT SEVEN EIGHTHS BLOCK

0258A
LEFT THREE QUARTERS BLOCK

0258B
LEFT FIVE EIGHTHS BLOCK

0258C
LEFT HALF BLOCK

0258D
LEFT THREE EIGHTHS BLOCK

0258E
LEFT ONE QUARTER BLOCK

0258F
LEFT ONE EIGHTH BLOCK

02590
RIGHT HALF BLOCK

02591
LIGHT SHADE

02592
MEDIUM SHADE

02593
DARK SHADE

02594
UPPER ONE EIGHTH BLOCK

02595
RIGHT ONE EIGHTH BLOCK

02596
QUADRANT LOWER LEFT

02597
QUADRANT LOWER RIGHT

02598
QUADRANT UPPER LEFT

02599
QUADRANT UPPER LEFT AND LOWER LEFT AND LOWER RIGHT

0259A
QUADRANT UPPER LEFT AND LOWER RIGHT

0259B
QUADRANT UPPER LEFT AND UPPER RIGHT AND LOWER LEFT

0259C
QUADRANT UPPER LEFT AND UPPER RIGHT AND LOWER RIGHT

0259D
QUADRANT UPPER RIGHT

0259E
QUADRANT UPPER RIGHT AND LOWER LEFT

0259F
QUADRANT UPPER RIGHT AND LOWER LEFT AND LOWER RIGHT

025A0
BLACK SQUARE

025A1
WHITE SQUARE

025A2
WHITE SQUARE WITH ROUNDED CORNERS

025A3
WHITE SQUARE CONTAINING BLACK SMALL SQUARE

025A4
SQUARE WITH HORIZONTAL FILL

025A5
SQUARE WITH VERTICAL FILL

025A6
SQUARE WITH ORTHOGONAL CROSSHATCH FILL

025A7
SQUARE WITH UPPER LEFT TO LOWER RIGHT FILL

025A8
SQUARE WITH UPPER RIGHT TO LOWER LEFT FILL

025A9
SQUARE WITH DIAGONAL CROSSHATCH FILL

025AA
BLACK SMALL SQUARE

025AB
WHITE SMALL SQUARE

025AC
BLACK RECTANGLE

025AD
WHITE RECTANGLE

025AE
BLACK VERTICAL RECTANGLE

025AF
WHITE VERTICAL RECTANGLE

025B0
BLACK PARALLELOGRAM

025B1
WHITE PARALLELOGRAM

025B2
BLACK UP-POINTING TRIANGLE

025B3
WHITE UP-POINTING TRIANGLE

025B4
BLACK UP-POINTING SMALL TRIANGLE

025B5
WHITE UP-POINTING SMALL TRIANGLE

025B6
BLACK RIGHT-POINTING TRIANGLE

025B7
WHITE RIGHT-POINTING TRIANGLE

025B8
BLACK RIGHT-POINTING SMALL TRIANGLE

025B9
WHITE RIGHT-POINTING SMALL TRIANGLE

025BA
BLACK RIGHT-POINTING POINTER

025BB
WHITE RIGHT-POINTING POINTER

025BC
BLACK DOWN-POINTING TRIANGLE

025BD
WHITE DOWN-POINTING TRIANGLE

025BE
BLACK DOWN-POINTING SMALL TRIANGLE

025BF
WHITE DOWN-POINTING SMALL TRIANGLE

025C0
BLACK LEFT-POINTING TRIANGLE

025C1
WHITE LEFT-POINTING TRIANGLE

025C2
BLACK LEFT-POINTING SMALL TRIANGLE

025C3
WHITE LEFT-POINTING SMALL TRIANGLE

025C4
BLACK LEFT-POINTING POINTER

025C5
WHITE LEFT-POINTING POINTER

025C6
BLACK DIAMOND

025C7
WHITE DIAMOND

025C8
WHITE DIAMOND CONTAINING BLACK SMALL DIAMOND

025C9
FISHEYE

025CA
LOZENGE

025CB
WHITE CIRCLE

025CC
DOTTED CIRCLE

025CD
CIRCLE WITH VERTICAL FILL

025CE
BULLSEYE

025CF
BLACK CIRCLE

025D0
CIRCLE WITH LEFT HALF BLACK

025D1
CIRCLE WITH RIGHT HALF BLACK

025D2
CIRCLE WITH LOWER HALF BLACK

025D3
CIRCLE WITH UPPER HALF BLACK

025D4
CIRCLE WITH UPPER RIGHT QUADRANT BLACK

025D5
CIRCLE WITH ALL BUT UPPER LEFT QUADRANT BLACK

025D6
LEFT HALF BLACK CIRCLE

025D7
RIGHT HALF BLACK CIRCLE

025D8
INVERSE BULLET

025D9
INVERSE WHITE CIRCLE

025DA
UPPER HALF INVERSE WHITE CIRCLE

025DB
LOWER HALF INVERSE WHITE CIRCLE

025DC
UPPER LEFT QUADRANT CIRCULAR ARC

025DD
UPPER RIGHT QUADRANT CIRCULAR ARC

025DE
LOWER RIGHT QUADRANT CIRCULAR ARC

025DF
LOWER LEFT QUADRANT CIRCULAR ARC

025E0
UPPER HALF CIRCLE

025E1
LOWER HALF CIRCLE

025E2
BLACK LOWER RIGHT TRIANGLE

025E3
BLACK LOWER LEFT TRIANGLE

025E4
BLACK UPPER LEFT TRIANGLE

025E5
BLACK UPPER RIGHT TRIANGLE

025E6
WHITE BULLET

025E7
SQUARE WITH LEFT HALF BLACK

025E8
SQUARE WITH RIGHT HALF BLACK

025E9
SQUARE WITH UPPER LEFT DIAGONAL HALF BLACK

025EA
SQUARE WITH LOWER RIGHT DIAGONAL HALF BLACK

025EB
WHITE SQUARE WITH VERTICAL BISECTING LINE

025EC
WHITE UP-POINTING TRIANGLE WITH DOT

025ED
UP-POINTING TRIANGLE WITH LEFT HALF BLACK

025EE
UP-POINTING TRIANGLE WITH RIGHT HALF BLACK

025EF
LARGE CIRCLE

025F0
WHITE SQUARE WITH UPPER LEFT QUADRANT

025F1
WHITE SQUARE WITH LOWER LEFT QUADRANT

025F2
WHITE SQUARE WITH LOWER RIGHT QUADRANT

025F3
WHITE SQUARE WITH UPPER RIGHT QUADRANT

025F4
WHITE CIRCLE WITH UPPER LEFT QUADRANT

025F5
WHITE CIRCLE WITH LOWER LEFT QUADRANT

025F6
WHITE CIRCLE WITH LOWER RIGHT QUADRANT

025F7
WHITE CIRCLE WITH UPPER RIGHT QUADRANT

025F8
UPPER LEFT TRIANGLE

025F9
UPPER RIGHT TRIANGLE

025FA
LOWER LEFT TRIANGLE

025FB
WHITE MEDIUM SQUARE

025FC
BLACK MEDIUM SQUARE

025FD
WHITE MEDIUM SMALL SQUARE

025FE
BLACK MEDIUM SMALL SQUARE

025FF
LOWER RIGHT TRIANGLE

02600
BLACK SUN WITH RAYS

02601
CLOUD

02602
UMBRELLA

02603
SNOWMAN

02604
COMET

02605
BLACK STAR

02606
WHITE STAR

02607
LIGHTNING

02608
THUNDERSTORM

02609
SUN

0260A
ASCENDING NODE

0260B
DESCENDING NODE

0260C
CONJUNCTION

0260D
OPPOSITION

0260E
BLACK TELEPHONE

0260F
WHITE TELEPHONE

02610
BALLOT BOX

02611
BALLOT BOX WITH CHECK

02612
BALLOT BOX WITH X

02613
SALTIRE

02614
UMBRELLA WITH RAIN DROPS

02615
HOT BEVERAGE

02616
WHITE SHOGI PIECE

02617
BLACK SHOGI PIECE

02618
SHAMROCK

02619
REVERSED ROTATED FLORAL HEART BULLET

0261A
BLACK LEFT POINTING INDEX

0261B
BLACK RIGHT POINTING INDEX

0261C
WHITE LEFT POINTING INDEX

0261D
WHITE UP POINTING INDEX

0261E
WHITE RIGHT POINTING INDEX

0261F
WHITE DOWN POINTING INDEX

02620
SKULL AND CROSSBONES

02621
CAUTION SIGN

02622
RADIOACTIVE SIGN

02623
BIOHAZARD SIGN

02624
CADUCEUS

02625
ANKH

02626
ORTHODOX CROSS

02627
CHI RHO

02628
CROSS OF LORRAINE

02629
CROSS OF JERUSALEM

0262A
STAR AND CRESCENT

0262B
FARSI SYMBOL

0262C
ADI SHAKTI

0262D
HAMMER AND SICKLE

0262E
PEACE SYMBOL

0262F
YIN YANG

02630
TRIGRAM FOR HEAVEN

02631
TRIGRAM FOR LAKE

02632
TRIGRAM FOR FIRE

02633
TRIGRAM FOR THUNDER

02634
TRIGRAM FOR WIND

02635
TRIGRAM FOR WATER

02636
TRIGRAM FOR MOUNTAIN

02637
TRIGRAM FOR EARTH

02638
WHEEL OF DHARMA

02639
WHITE FROWNING FACE

0263A
WHITE SMILING FACE

0263B
BLACK SMILING FACE

0263C
WHITE SUN WITH RAYS

0263D
FIRST QUARTER MOON

0263E
LAST QUARTER MOON

0263F
MERCURY

02640
FEMALE SIGN

02641
EARTH

02642
MALE SIGN

02643
JUPITER

02644
SATURN

02645
URANUS

02646
NEPTUNE

02647
PLUTO

02648
ARIES

02649
TAURUS

0264A
GEMINI

0264B
CANCER

0264C
LEO

0264D
VIRGO

0264E
LIBRA

0264F
SCORPIUS

02650
SAGITTARIUS

02651
CAPRICORN

02652
AQUARIUS

02653
PISCES

02654
WHITE CHESS KING

02655
WHITE CHESS QUEEN

02656
WHITE CHESS ROOK

02657
WHITE CHESS BISHOP

02658
WHITE CHESS KNIGHT

02659
WHITE CHESS PAWN

0265A
BLACK CHESS KING

0265B
BLACK CHESS QUEEN

0265C
BLACK CHESS ROOK

0265D
BLACK CHESS BISHOP

0265E
BLACK CHESS KNIGHT

0265F
BLACK CHESS PAWN

02660
BLACK SPADE SUIT

02661
WHITE HEART SUIT

02662
WHITE DIAMOND SUIT

02663
BLACK CLUB SUIT

02664
WHITE SPADE SUIT

02665
BLACK HEART SUIT

02666
BLACK DIAMOND SUIT

02667
WHITE CLUB SUIT

02668
HOT SPRINGS

02669
QUARTER NOTE

0266A
EIGHTH NOTE

0266B
BEAMED EIGHTH NOTES

0266C
BEAMED SIXTEENTH NOTES

0266D
MUSIC FLAT SIGN

0266E
MUSIC NATURAL SIGN

0266F
MUSIC SHARP SIGN

02670
WEST SYRIAC CROSS

02671
EAST SYRIAC CROSS

02672
UNIVERSAL RECYCLING SYMBOL

02673
RECYCLING SYMBOL FOR TYPE-1 PLASTICS

02674
RECYCLING SYMBOL FOR TYPE-2 PLASTICS

02675
RECYCLING SYMBOL FOR TYPE-3 PLASTICS

02676
RECYCLING SYMBOL FOR TYPE-4 PLASTICS

02677
RECYCLING SYMBOL FOR TYPE-5 PLASTICS

02678
RECYCLING SYMBOL FOR TYPE-6 PLASTICS

02679
RECYCLING SYMBOL FOR TYPE-7 PLASTICS

0267A
RECYCLING SYMBOL FOR GENERIC MATERIALS

0267B
BLACK UNIVERSAL RECYCLING SYMBOL

0267C
RECYCLED PAPER SYMBOL

0267D
PARTIALLY-RECYCLED PAPER SYMBOL

0267E
PERMANENT PAPER SIGN

0267F
WHEELCHAIR SYMBOL

02680
DIE FACE-1

02681
DIE FACE-2

02682
DIE FACE-3

02683
DIE FACE-4

02684
DIE FACE-5

02685
DIE FACE-6

02686
WHITE CIRCLE WITH DOT RIGHT

02687
WHITE CIRCLE WITH TWO DOTS

02688
BLACK CIRCLE WITH WHITE DOT RIGHT

02689
BLACK CIRCLE WITH TWO WHITE DOTS

0268A
MONOGRAM FOR YANG

0268B
MONOGRAM FOR YIN

0268C
DIGRAM FOR GREATER YANG

0268D
DIGRAM FOR LESSER YIN

0268E
DIGRAM FOR LESSER YANG

0268F
DIGRAM FOR GREATER YIN

02690
WHITE FLAG

02691
BLACK FLAG

02692
HAMMER AND PICK

02693
ANCHOR

02694
CROSSED SWORDS

02695
STAFF OF AESCULAPIUS

02696
SCALES

02697
ALEMBIC

02698
FLOWER

02699
GEAR

0269A
STAFF OF HERMES

0269B
ATOM SYMBOL

0269C
FLEUR-DE-LIS

0269D
OUTLINED WHITE STAR

0269E
THREE LINES CONVERGING RIGHT

0269F
THREE LINES CONVERGING LEFT

026A0
WARNING SIGN

026A1
HIGH VOLTAGE SIGN

026A2
DOUBLED FEMALE SIGN

026A3
DOUBLED MALE SIGN

026A4
INTERLOCKED FEMALE AND MALE SIGN

026A5
MALE AND FEMALE SIGN

026A6
MALE WITH STROKE SIGN

026A7
MALE WITH STROKE AND MALE AND FEMALE SIGN

026A8
VERTICAL MALE WITH STROKE SIGN

026A9
HORIZONTAL MALE WITH STROKE SIGN

026AA
MEDIUM WHITE CIRCLE

026AB
MEDIUM BLACK CIRCLE

026AC
MEDIUM SMALL WHITE CIRCLE

026AD
MARRIAGE SYMBOL

026AE
DIVORCE SYMBOL

026AF
UNMARRIED PARTNERSHIP SYMBOL

026B0
COFFIN

026B1
FUNERAL URN

026B2
NEUTER

026B3
CERES

026B4
PALLAS

026B5
JUNO

026B6
VESTA

026B7
CHIRON

026B8
BLACK MOON LILITH

026B9
SEXTILE

026BA
SEMISEXTILE

026BB
QUINCUNX

026BC
SESQUIQUADRATE

026BD
SOCCER BALL

026BE
BASEBALL

026BF
SQUARED KEY

026C0
WHITE DRAUGHTS MAN

026C1
WHITE DRAUGHTS KING

026C2
BLACK DRAUGHTS MAN

026C3
BLACK DRAUGHTS KING

026C4
SNOWMAN WITHOUT SNOW

026C5
SUN BEHIND CLOUD

026C6
RAIN

026C7
BLACK SNOWMAN

026C8
THUNDER CLOUD AND RAIN

026C9
TURNED WHITE SHOGI PIECE

026CA
TURNED BLACK SHOGI PIECE

026CB
WHITE DIAMOND IN SQUARE

026CC
CROSSING LANES

026CD
DISABLED CAR

026CE
OPHIUCHUS

026CF
PICK

026D0
CAR SLIDING

026D1
HELMET WITH WHITE CROSS

026D2
CIRCLED CROSSING LANES

026D3
CHAINS

026D4
NO ENTRY

026D5
ALTERNATE ONE-WAY LEFT WAY TRAFFIC

026D6
BLACK TWO-WAY LEFT WAY TRAFFIC

026D7
WHITE TWO-WAY LEFT WAY TRAFFIC

026D8
BLACK LEFT LANE MERGE

026D9
WHITE LEFT LANE MERGE

026DA
DRIVE SLOW SIGN

026DB
HEAVY WHITE DOWN-POINTING TRIANGLE

026DC
LEFT CLOSED ENTRY

026DD
SQUARED SALTIRE

026DE
FALLING DIAGONAL IN WHITE CIRCLE IN BLACK SQUARE

026DF
BLACK TRUCK

026E0
RESTRICTED LEFT ENTRY-1

026E1
RESTRICTED LEFT ENTRY-2

026E2
ASTRONOMICAL SYMBOL FOR URANUS

026E3
HEAVY CIRCLE WITH STROKE AND TWO DOTS ABOVE

026E4
PENTAGRAM

026E5
RIGHT-HANDED INTERLACED PENTAGRAM

026E6
LEFT-HANDED INTERLACED PENTAGRAM

026E7
INVERTED PENTAGRAM

026E8
BLACK CROSS ON SHIELD

026E9
SHINTO SHRINE

026EA
CHURCH

026EB
CASTLE

026EC
HISTORIC SITE

026ED
GEAR WITHOUT HUB

026EE
GEAR WITH HANDLES

026EF
MAP SYMBOL FOR LIGHTHOUSE

026F0
MOUNTAIN

026F1
UMBRELLA ON GROUND

026F2
FOUNTAIN

026F3
FLAG IN HOLE

026F4
FERRY

026F5
SAILBOAT

026F6
SQUARE FOUR CORNERS

026F7
SKIER

026F8
ICE SKATE

026F9
PERSON WITH BALL

026FA
TENT

026FB
JAPANESE BANK SYMBOL

026FC
HEADSTONE GRAVEYARD SYMBOL

026FD
FUEL PUMP

026FE
CUP ON BLACK SQUARE

026FF
WHITE FLAG WITH HORIZONTAL MIDDLE BLACK STRIPE

02700
BLACK SAFETY SCISSORS

02701
UPPER BLADE SCISSORS

02702
BLACK SCISSORS

02703
LOWER BLADE SCISSORS

02704
WHITE SCISSORS

02705
WHITE HEAVY CHECK MARK

02706
TELEPHONE LOCATION SIGN

02707
TAPE DRIVE

02708
AIRPLANE

02709
ENVELOPE

0270A
RAISED FIST

0270B
RAISED HAND

0270C
VICTORY HAND

0270D
WRITING HAND

0270E
LOWER RIGHT PENCIL

0270F
PENCIL

02710
UPPER RIGHT PENCIL

02711
WHITE NIB

02712
BLACK NIB

02713
CHECK MARK

02714
HEAVY CHECK MARK

02715
MULTIPLICATION X

02716
HEAVY MULTIPLICATION X

02717
BALLOT X

02718
HEAVY BALLOT X

02719
OUTLINED GREEK CROSS

0271A
HEAVY GREEK CROSS

0271B
OPEN CENTRE CROSS

0271C
HEAVY OPEN CENTRE CROSS

0271D
LATIN CROSS

0271E
SHADOWED WHITE LATIN CROSS

0271F
OUTLINED LATIN CROSS

02720
MALTESE CROSS

02721
STAR OF DAVID

02722
FOUR TEARDROP-SPOKED ASTERISK

02723
FOUR BALLOON-SPOKED ASTERISK

02724
HEAVY FOUR BALLOON-SPOKED ASTERISK

02725
FOUR CLUB-SPOKED ASTERISK

02726
BLACK FOUR POINTED STAR

02727
WHITE FOUR POINTED STAR

02728
SPARKLES

02729
STRESS OUTLINED WHITE STAR

0272A
CIRCLED WHITE STAR

0272B
OPEN CENTRE BLACK STAR

0272C
BLACK CENTRE WHITE STAR

0272D
OUTLINED BLACK STAR

0272E
HEAVY OUTLINED BLACK STAR

0272F
PINWHEEL STAR

02730
SHADOWED WHITE STAR

02731
HEAVY ASTERISK

02732
OPEN CENTRE ASTERISK

02733
EIGHT SPOKED ASTERISK

02734
EIGHT POINTED BLACK STAR

02735
EIGHT POINTED PINWHEEL STAR

02736
SIX POINTED BLACK STAR

02737
EIGHT POINTED RECTILINEAR BLACK STAR

02738
HEAVY EIGHT POINTED RECTILINEAR BLACK STAR

02739
TWELVE POINTED BLACK STAR

0273A
SIXTEEN POINTED ASTERISK

0273B
TEARDROP-SPOKED ASTERISK

0273C
OPEN CENTRE TEARDROP-SPOKED ASTERISK

0273D
HEAVY TEARDROP-SPOKED ASTERISK

0273E
SIX PETALLED BLACK AND WHITE FLORETTE

0273F
BLACK FLORETTE

02740
WHITE FLORETTE

02741
EIGHT PETALLED OUTLINED BLACK FLORETTE

02742
CIRCLED OPEN CENTRE EIGHT POINTED STAR

02743
HEAVY TEARDROP-SPOKED PINWHEEL ASTERISK

02744
SNOWFLAKE

02745
TIGHT TRIFOLIATE SNOWFLAKE

02746
HEAVY CHEVRON SNOWFLAKE

02747
SPARKLE

02748
HEAVY SPARKLE

02749
BALLOON-SPOKED ASTERISK

0274A
EIGHT TEARDROP-SPOKED PROPELLER ASTERISK

0274B
HEAVY EIGHT TEARDROP-SPOKED PROPELLER ASTERISK

0274C
CROSS MARK

0274D
SHADOWED WHITE CIRCLE

0274E
NEGATIVE SQUARED CROSS MARK

0274F
LOWER RIGHT DROP-SHADOWED WHITE SQUARE

02750
UPPER RIGHT DROP-SHADOWED WHITE SQUARE

02751
LOWER RIGHT SHADOWED WHITE SQUARE

02752
UPPER RIGHT SHADOWED WHITE SQUARE

02753
BLACK QUESTION MARK ORNAMENT

02754
WHITE QUESTION MARK ORNAMENT

02755
WHITE EXCLAMATION MARK ORNAMENT

02756
BLACK DIAMOND MINUS WHITE X

02757
HEAVY EXCLAMATION MARK SYMBOL

02758
LIGHT VERTICAL BAR

02759
MEDIUM VERTICAL BAR

0275A
HEAVY VERTICAL BAR

0275B
HEAVY SINGLE TURNED COMMA QUOTATION MARK ORNAMENT

0275C
HEAVY SINGLE COMMA QUOTATION MARK ORNAMENT

0275D
HEAVY DOUBLE TURNED COMMA QUOTATION MARK ORNAMENT

0275E
HEAVY DOUBLE COMMA QUOTATION MARK ORNAMENT

0275F
HEAVY LOW SINGLE COMMA QUOTATION MARK ORNAMENT

02760
HEAVY LOW DOUBLE COMMA QUOTATION MARK ORNAMENT

02761
CURVED STEM PARAGRAPH SIGN ORNAMENT

02762
HEAVY EXCLAMATION MARK ORNAMENT

02763
HEAVY HEART EXCLAMATION MARK ORNAMENT

02764
HEAVY BLACK HEART

02765
ROTATED HEAVY BLACK HEART BULLET

02766
FLORAL HEART

02767
ROTATED FLORAL HEART BULLET

02768
MEDIUM LEFT PARENTHESIS ORNAMENT

02769
MEDIUM RIGHT PARENTHESIS ORNAMENT

0276A
MEDIUM FLATTENED LEFT PARENTHESIS ORNAMENT

0276B
MEDIUM FLATTENED RIGHT PARENTHESIS ORNAMENT

0276C
MEDIUM LEFT-POINTING ANGLE BRACKET ORNAMENT

0276D
MEDIUM RIGHT-POINTING ANGLE BRACKET ORNAMENT

0276E
HEAVY LEFT-POINTING ANGLE QUOTATION MARK ORNAMENT

0276F
HEAVY RIGHT-POINTING ANGLE QUOTATION MARK ORNAMENT

02770
HEAVY LEFT-POINTING ANGLE BRACKET ORNAMENT

02771
HEAVY RIGHT-POINTING ANGLE BRACKET ORNAMENT

02772
LIGHT LEFT TORTOISE SHELL BRACKET ORNAMENT

02773
LIGHT RIGHT TORTOISE SHELL BRACKET ORNAMENT

02774
MEDIUM LEFT CURLY BRACKET ORNAMENT

02775
MEDIUM RIGHT CURLY BRACKET ORNAMENT

02776
DINGBAT NEGATIVE CIRCLED DIGIT ONE

02777
DINGBAT NEGATIVE CIRCLED DIGIT TWO

02778
DINGBAT NEGATIVE CIRCLED DIGIT THREE

02779
DINGBAT NEGATIVE CIRCLED DIGIT FOUR

0277A
DINGBAT NEGATIVE CIRCLED DIGIT FIVE

0277B
DINGBAT NEGATIVE CIRCLED DIGIT SIX

0277C
DINGBAT NEGATIVE CIRCLED DIGIT SEVEN

0277D
DINGBAT NEGATIVE CIRCLED DIGIT EIGHT

0277E
DINGBAT NEGATIVE CIRCLED DIGIT NINE

0277F
DINGBAT NEGATIVE CIRCLED NUMBER TEN

02780
DINGBAT CIRCLED SANS-SERIF DIGIT ONE

02781
DINGBAT CIRCLED SANS-SERIF DIGIT TWO

02782
DINGBAT CIRCLED SANS-SERIF DIGIT THREE

02783
DINGBAT CIRCLED SANS-SERIF DIGIT FOUR

02784
DINGBAT CIRCLED SANS-SERIF DIGIT FIVE

02785
DINGBAT CIRCLED SANS-SERIF DIGIT SIX

02786
DINGBAT CIRCLED SANS-SERIF DIGIT SEVEN

02787
DINGBAT CIRCLED SANS-SERIF DIGIT EIGHT

02788
DINGBAT CIRCLED SANS-SERIF DIGIT NINE

02789
DINGBAT CIRCLED SANS-SERIF NUMBER TEN

0278A
DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT ONE

0278B
DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT TWO

0278C
DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT THREE

0278D
DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT FOUR

0278E
DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT FIVE

0278F
DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT SIX

02790
DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT SEVEN

02791
DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT EIGHT

02792
DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT NINE

02793
DINGBAT NEGATIVE CIRCLED SANS-SERIF NUMBER TEN

02794
HEAVY WIDE-HEADED RIGHTWARDS ARROW

02795
HEAVY PLUS SIGN

02796
HEAVY MINUS SIGN

02797
HEAVY DIVISION SIGN

02798
HEAVY SOUTH EAST ARROW

02799
HEAVY RIGHTWARDS ARROW

0279A
HEAVY NORTH EAST ARROW

0279B
DRAFTING POINT RIGHTWARDS ARROW

0279C
HEAVY ROUND-TIPPED RIGHTWARDS ARROW

0279D
TRIANGLE-HEADED RIGHTWARDS ARROW

0279E
HEAVY TRIANGLE-HEADED RIGHTWARDS ARROW

0279F
DASHED TRIANGLE-HEADED RIGHTWARDS ARROW

027A0
HEAVY DASHED TRIANGLE-HEADED RIGHTWARDS ARROW

027A1
BLACK RIGHTWARDS ARROW

027A2
THREE-D TOP-LIGHTED RIGHTWARDS ARROWHEAD

027A3
THREE-D BOTTOM-LIGHTED RIGHTWARDS ARROWHEAD

027A4
BLACK RIGHTWARDS ARROWHEAD

027A5
HEAVY BLACK CURVED DOWNWARDS AND RIGHTWARDS ARROW

027A6
HEAVY BLACK CURVED UPWARDS AND RIGHTWARDS ARROW

027A7
SQUAT BLACK RIGHTWARDS ARROW

027A8
HEAVY CONCAVE-POINTED BLACK RIGHTWARDS ARROW

027A9
RIGHT-SHADED WHITE RIGHTWARDS ARROW

027AA
LEFT-SHADED WHITE RIGHTWARDS ARROW

027AB
BACK-TILTED SHADOWED WHITE RIGHTWARDS ARROW

027AC
FRONT-TILTED SHADOWED WHITE RIGHTWARDS ARROW

027AD
HEAVY LOWER RIGHT-SHADOWED WHITE RIGHTWARDS ARROW

027AE
HEAVY UPPER RIGHT-SHADOWED WHITE RIGHTWARDS ARROW

027AF
NOTCHED LOWER RIGHT-SHADOWED WHITE RIGHTWARDS ARROW

027B0
CURLY LOOP

027B1
NOTCHED UPPER RIGHT-SHADOWED WHITE RIGHTWARDS ARROW

027B2
CIRCLED HEAVY WHITE RIGHTWARDS ARROW

027B3
WHITE-FEATHERED RIGHTWARDS ARROW

027B4
BLACK-FEATHERED SOUTH EAST ARROW

027B5
BLACK-FEATHERED RIGHTWARDS ARROW

027B6
BLACK-FEATHERED NORTH EAST ARROW

027B7
HEAVY BLACK-FEATHERED SOUTH EAST ARROW

027B8
HEAVY BLACK-FEATHERED RIGHTWARDS ARROW

027B9
HEAVY BLACK-FEATHERED NORTH EAST ARROW

027BA
TEARDROP-BARBED RIGHTWARDS ARROW

027BB
HEAVY TEARDROP-SHANKED RIGHTWARDS ARROW

027BC
WEDGE-TAILED RIGHTWARDS ARROW

027BD
HEAVY WEDGE-TAILED RIGHTWARDS ARROW

027BE
OPEN-OUTLINED RIGHTWARDS ARROW

027BF
DOUBLE CURLY LOOP

027C0
THREE DIMENSIONAL ANGLE

027C1
WHITE TRIANGLE CONTAINING SMALL WHITE TRIANGLE

027C2
PERPENDICULAR

027C3
OPEN SUBSET

027C4
OPEN SUPERSET

027C5
LEFT S-SHAPED BAG DELIMITER

027C6
RIGHT S-SHAPED BAG DELIMITER

027C7
OR WITH DOT INSIDE

027C8
REVERSE SOLIDUS PRECEDING SUBSET

027C9
SUPERSET PRECEDING SOLIDUS

027CA
VERTICAL BAR WITH HORIZONTAL STROKE

027CB
MATHEMATICAL RISING DIAGONAL

027CC
LONG DIVISION

027CD
MATHEMATICAL FALLING DIAGONAL

027CE
SQUARED LOGICAL AND

027CF
SQUARED LOGICAL OR

027D0
WHITE DIAMOND WITH CENTRED DOT

027D1
AND WITH DOT

027D2
ELEMENT OF OPENING UPWARDS

027D3
LOWER RIGHT CORNER WITH DOT

027D4
UPPER LEFT CORNER WITH DOT

027D5
LEFT OUTER JOIN

027D6
RIGHT OUTER JOIN

027D7
FULL OUTER JOIN

027D8
LARGE UP TACK

027D9
LARGE DOWN TACK

027DA
LEFT AND RIGHT DOUBLE TURNSTILE

027DB
LEFT AND RIGHT TACK

027DC
LEFT MULTIMAP

027DD
LONG RIGHT TACK

027DE
LONG LEFT TACK

027DF
UP TACK WITH CIRCLE ABOVE

027E0
LOZENGE DIVIDED BY HORIZONTAL RULE

027E1
WHITE CONCAVE-SIDED DIAMOND

027E2
WHITE CONCAVE-SIDED DIAMOND WITH LEFTWARDS TICK

027E3
WHITE CONCAVE-SIDED DIAMOND WITH RIGHTWARDS TICK

027E4
WHITE SQUARE WITH LEFTWARDS TICK

027E5
WHITE SQUARE WITH RIGHTWARDS TICK

027E6
MATHEMATICAL LEFT WHITE SQUARE BRACKET

027E7
MATHEMATICAL RIGHT WHITE SQUARE BRACKET

027E8
MATHEMATICAL LEFT ANGLE BRACKET

027E9
MATHEMATICAL RIGHT ANGLE BRACKET

027EA
MATHEMATICAL LEFT DOUBLE ANGLE BRACKET

027EB
MATHEMATICAL RIGHT DOUBLE ANGLE BRACKET

027EC
MATHEMATICAL LEFT WHITE TORTOISE SHELL BRACKET

027ED
MATHEMATICAL RIGHT WHITE TORTOISE SHELL BRACKET

027EE
MATHEMATICAL LEFT FLATTENED PARENTHESIS

027EF
MATHEMATICAL RIGHT FLATTENED PARENTHESIS

027F0
UPWARDS QUADRUPLE ARROW

027F1
DOWNWARDS QUADRUPLE ARROW

027F2
ANTICLOCKWISE GAPPED CIRCLE ARROW

027F3
CLOCKWISE GAPPED CIRCLE ARROW

027F4
RIGHT ARROW WITH CIRCLED PLUS

027F5
LONG LEFTWARDS ARROW

027F6
LONG RIGHTWARDS ARROW

027F7
LONG LEFT RIGHT ARROW

027F8
LONG LEFTWARDS DOUBLE ARROW

027F9
LONG RIGHTWARDS DOUBLE ARROW

027FA
LONG LEFT RIGHT DOUBLE ARROW

027FB
LONG LEFTWARDS ARROW FROM BAR

027FC
LONG RIGHTWARDS ARROW FROM BAR

027FD
LONG LEFTWARDS DOUBLE ARROW FROM BAR

027FE
LONG RIGHTWARDS DOUBLE ARROW FROM BAR

027FF
LONG RIGHTWARDS SQUIGGLE ARROW

02800
BRAILLE PATTERN BLANK

02801
BRAILLE PATTERN DOTS-1

02802
BRAILLE PATTERN DOTS-2

02803
BRAILLE PATTERN DOTS-12

02804
BRAILLE PATTERN DOTS-3

02805
BRAILLE PATTERN DOTS-13

02806
BRAILLE PATTERN DOTS-23

02807
BRAILLE PATTERN DOTS-123

02808
BRAILLE PATTERN DOTS-4

02809
BRAILLE PATTERN DOTS-14

0280A
BRAILLE PATTERN DOTS-24

0280B
BRAILLE PATTERN DOTS-124

0280C
BRAILLE PATTERN DOTS-34

0280D
BRAILLE PATTERN DOTS-134

0280E
BRAILLE PATTERN DOTS-234

0280F
BRAILLE PATTERN DOTS-1234

02810
BRAILLE PATTERN DOTS-5

02811
BRAILLE PATTERN DOTS-15

02812
BRAILLE PATTERN DOTS-25

02813
BRAILLE PATTERN DOTS-125

02814
BRAILLE PATTERN DOTS-35

02815
BRAILLE PATTERN DOTS-135

02816
BRAILLE PATTERN DOTS-235

02817
BRAILLE PATTERN DOTS-1235

02818
BRAILLE PATTERN DOTS-45

02819
BRAILLE PATTERN DOTS-145

0281A
BRAILLE PATTERN DOTS-245

0281B
BRAILLE PATTERN DOTS-1245

0281C
BRAILLE PATTERN DOTS-345

0281D
BRAILLE PATTERN DOTS-1345

0281E
BRAILLE PATTERN DOTS-2345

0281F
BRAILLE PATTERN DOTS-12345

02820
BRAILLE PATTERN DOTS-6

02821
BRAILLE PATTERN DOTS-16

02822
BRAILLE PATTERN DOTS-26

02823
BRAILLE PATTERN DOTS-126

02824
BRAILLE PATTERN DOTS-36

02825
BRAILLE PATTERN DOTS-136

02826
BRAILLE PATTERN DOTS-236

02827
BRAILLE PATTERN DOTS-1236

02828
BRAILLE PATTERN DOTS-46

02829
BRAILLE PATTERN DOTS-146

0282A
BRAILLE PATTERN DOTS-246

0282B
BRAILLE PATTERN DOTS-1246

0282C
BRAILLE PATTERN DOTS-346

0282D
BRAILLE PATTERN DOTS-1346

0282E
BRAILLE PATTERN DOTS-2346

0282F
BRAILLE PATTERN DOTS-12346

02830
BRAILLE PATTERN DOTS-56

02831
BRAILLE PATTERN DOTS-156

02832
BRAILLE PATTERN DOTS-256

02833
BRAILLE PATTERN DOTS-1256

02834
BRAILLE PATTERN DOTS-356

02835
BRAILLE PATTERN DOTS-1356

02836
BRAILLE PATTERN DOTS-2356

02837
BRAILLE PATTERN DOTS-12356

02838
BRAILLE PATTERN DOTS-456

02839
BRAILLE PATTERN DOTS-1456

0283A
BRAILLE PATTERN DOTS-2456

0283B
BRAILLE PATTERN DOTS-12456

0283C
BRAILLE PATTERN DOTS-3456

0283D
BRAILLE PATTERN DOTS-13456

0283E
BRAILLE PATTERN DOTS-23456

0283F
BRAILLE PATTERN DOTS-123456

02840
BRAILLE PATTERN DOTS-7

02841
BRAILLE PATTERN DOTS-17

02842
BRAILLE PATTERN DOTS-27

02843
BRAILLE PATTERN DOTS-127

02844
BRAILLE PATTERN DOTS-37

02845
BRAILLE PATTERN DOTS-137

02846
BRAILLE PATTERN DOTS-237

02847
BRAILLE PATTERN DOTS-1237

02848
BRAILLE PATTERN DOTS-47

02849
BRAILLE PATTERN DOTS-147

0284A
BRAILLE PATTERN DOTS-247

0284B
BRAILLE PATTERN DOTS-1247

0284C
BRAILLE PATTERN DOTS-347

0284D
BRAILLE PATTERN DOTS-1347

0284E
BRAILLE PATTERN DOTS-2347

0284F
BRAILLE PATTERN DOTS-12347

02850
BRAILLE PATTERN DOTS-57

02851
BRAILLE PATTERN DOTS-157

02852
BRAILLE PATTERN DOTS-257

02853
BRAILLE PATTERN DOTS-1257

02854
BRAILLE PATTERN DOTS-357

02855
BRAILLE PATTERN DOTS-1357

02856
BRAILLE PATTERN DOTS-2357

02857
BRAILLE PATTERN DOTS-12357

02858
BRAILLE PATTERN DOTS-457

02859
BRAILLE PATTERN DOTS-1457

0285A
BRAILLE PATTERN DOTS-2457

0285B
BRAILLE PATTERN DOTS-12457

0285C
BRAILLE PATTERN DOTS-3457

0285D
BRAILLE PATTERN DOTS-13457

0285E
BRAILLE PATTERN DOTS-23457

0285F
BRAILLE PATTERN DOTS-123457

02860
BRAILLE PATTERN DOTS-67

02861
BRAILLE PATTERN DOTS-167

02862
BRAILLE PATTERN DOTS-267

02863
BRAILLE PATTERN DOTS-1267

02864
BRAILLE PATTERN DOTS-367

02865
BRAILLE PATTERN DOTS-1367

02866
BRAILLE PATTERN DOTS-2367

02867
BRAILLE PATTERN DOTS-12367

02868
BRAILLE PATTERN DOTS-467

02869
BRAILLE PATTERN DOTS-1467

0286A
BRAILLE PATTERN DOTS-2467

0286B
BRAILLE PATTERN DOTS-12467

0286C
BRAILLE PATTERN DOTS-3467

0286D
BRAILLE PATTERN DOTS-13467

0286E
BRAILLE PATTERN DOTS-23467

0286F
BRAILLE PATTERN DOTS-123467

02870
BRAILLE PATTERN DOTS-567

02871
BRAILLE PATTERN DOTS-1567

02872
BRAILLE PATTERN DOTS-2567

02873
BRAILLE PATTERN DOTS-12567

02874
BRAILLE PATTERN DOTS-3567

02875
BRAILLE PATTERN DOTS-13567

02876
BRAILLE PATTERN DOTS-23567

02877
BRAILLE PATTERN DOTS-123567

02878
BRAILLE PATTERN DOTS-4567

02879
BRAILLE PATTERN DOTS-14567

0287A
BRAILLE PATTERN DOTS-24567

0287B
BRAILLE PATTERN DOTS-124567

0287C
BRAILLE PATTERN DOTS-34567

0287D
BRAILLE PATTERN DOTS-134567

0287E
BRAILLE PATTERN DOTS-234567

0287F
BRAILLE PATTERN DOTS-1234567

02880
BRAILLE PATTERN DOTS-8

02881
BRAILLE PATTERN DOTS-18

02882
BRAILLE PATTERN DOTS-28

02883
BRAILLE PATTERN DOTS-128

02884
BRAILLE PATTERN DOTS-38

02885
BRAILLE PATTERN DOTS-138

02886
BRAILLE PATTERN DOTS-238

02887
BRAILLE PATTERN DOTS-1238

02888
BRAILLE PATTERN DOTS-48

02889
BRAILLE PATTERN DOTS-148

0288A
BRAILLE PATTERN DOTS-248

0288B
BRAILLE PATTERN DOTS-1248

0288C
BRAILLE PATTERN DOTS-348

0288D
BRAILLE PATTERN DOTS-1348

0288E
BRAILLE PATTERN DOTS-2348

0288F
BRAILLE PATTERN DOTS-12348

02890
BRAILLE PATTERN DOTS-58

02891
BRAILLE PATTERN DOTS-158

02892
BRAILLE PATTERN DOTS-258

02893
BRAILLE PATTERN DOTS-1258

02894
BRAILLE PATTERN DOTS-358

02895
BRAILLE PATTERN DOTS-1358

02896
BRAILLE PATTERN DOTS-2358

02897
BRAILLE PATTERN DOTS-12358

02898
BRAILLE PATTERN DOTS-458

02899
BRAILLE PATTERN DOTS-1458

0289A
BRAILLE PATTERN DOTS-2458

0289B
BRAILLE PATTERN DOTS-12458

0289C
BRAILLE PATTERN DOTS-3458

0289D
BRAILLE PATTERN DOTS-13458

0289E
BRAILLE PATTERN DOTS-23458

0289F
BRAILLE PATTERN DOTS-123458

028A0
BRAILLE PATTERN DOTS-68

028A1
BRAILLE PATTERN DOTS-168

028A2
BRAILLE PATTERN DOTS-268

028A3
BRAILLE PATTERN DOTS-1268

028A4
BRAILLE PATTERN DOTS-368

028A5
BRAILLE PATTERN DOTS-1368

028A6
BRAILLE PATTERN DOTS-2368

028A7
BRAILLE PATTERN DOTS-12368

028A8
BRAILLE PATTERN DOTS-468

028A9
BRAILLE PATTERN DOTS-1468

028AA
BRAILLE PATTERN DOTS-2468

028AB
BRAILLE PATTERN DOTS-12468

028AC
BRAILLE PATTERN DOTS-3468

028AD
BRAILLE PATTERN DOTS-13468

028AE
BRAILLE PATTERN DOTS-23468

028AF
BRAILLE PATTERN DOTS-123468

028B0
BRAILLE PATTERN DOTS-568

028B1
BRAILLE PATTERN DOTS-1568

028B2
BRAILLE PATTERN DOTS-2568

028B3
BRAILLE PATTERN DOTS-12568

028B4
BRAILLE PATTERN DOTS-3568

028B5
BRAILLE PATTERN DOTS-13568

028B6
BRAILLE PATTERN DOTS-23568

028B7
BRAILLE PATTERN DOTS-123568

028B8
BRAILLE PATTERN DOTS-4568

028B9
BRAILLE PATTERN DOTS-14568

028BA
BRAILLE PATTERN DOTS-24568

028BB
BRAILLE PATTERN DOTS-124568

028BC
BRAILLE PATTERN DOTS-34568

028BD
BRAILLE PATTERN DOTS-134568

028BE
BRAILLE PATTERN DOTS-234568

028BF
BRAILLE PATTERN DOTS-1234568

028C0
BRAILLE PATTERN DOTS-78

028C1
BRAILLE PATTERN DOTS-178

028C2
BRAILLE PATTERN DOTS-278

028C3
BRAILLE PATTERN DOTS-1278

028C4
BRAILLE PATTERN DOTS-378

028C5
BRAILLE PATTERN DOTS-1378

028C6
BRAILLE PATTERN DOTS-2378

028C7
BRAILLE PATTERN DOTS-12378

028C8
BRAILLE PATTERN DOTS-478

028C9
BRAILLE PATTERN DOTS-1478

028CA
BRAILLE PATTERN DOTS-2478

028CB
BRAILLE PATTERN DOTS-12478

028CC
BRAILLE PATTERN DOTS-3478

028CD
BRAILLE PATTERN DOTS-13478

028CE
BRAILLE PATTERN DOTS-23478

028CF
BRAILLE PATTERN DOTS-123478

028D0
BRAILLE PATTERN DOTS-578

028D1
BRAILLE PATTERN DOTS-1578

028D2
BRAILLE PATTERN DOTS-2578

028D3
BRAILLE PATTERN DOTS-12578

028D4
BRAILLE PATTERN DOTS-3578

028D5
BRAILLE PATTERN DOTS-13578

028D6
BRAILLE PATTERN DOTS-23578

028D7
BRAILLE PATTERN DOTS-123578

028D8
BRAILLE PATTERN DOTS-4578

028D9
BRAILLE PATTERN DOTS-14578

028DA
BRAILLE PATTERN DOTS-24578

028DB
BRAILLE PATTERN DOTS-124578

028DC
BRAILLE PATTERN DOTS-34578

028DD
BRAILLE PATTERN DOTS-134578

028DE
BRAILLE PATTERN DOTS-234578

028DF
BRAILLE PATTERN DOTS-1234578

028E0
BRAILLE PATTERN DOTS-678

028E1
BRAILLE PATTERN DOTS-1678

028E2
BRAILLE PATTERN DOTS-2678

028E3
BRAILLE PATTERN DOTS-12678

028E4
BRAILLE PATTERN DOTS-3678

028E5
BRAILLE PATTERN DOTS-13678

028E6
BRAILLE PATTERN DOTS-23678

028E7
BRAILLE PATTERN DOTS-123678

028E8
BRAILLE PATTERN DOTS-4678

028E9
BRAILLE PATTERN DOTS-14678

028EA
BRAILLE PATTERN DOTS-24678

028EB
BRAILLE PATTERN DOTS-124678

028EC
BRAILLE PATTERN DOTS-34678

028ED
BRAILLE PATTERN DOTS-134678

028EE
BRAILLE PATTERN DOTS-234678

028EF
BRAILLE PATTERN DOTS-1234678

028F0
BRAILLE PATTERN DOTS-5678

028F1
BRAILLE PATTERN DOTS-15678

028F2
BRAILLE PATTERN DOTS-25678

028F3
BRAILLE PATTERN DOTS-125678

028F4
BRAILLE PATTERN DOTS-35678

028F5
BRAILLE PATTERN DOTS-135678

028F6
BRAILLE PATTERN DOTS-235678

028F7
BRAILLE PATTERN DOTS-1235678

028F8
BRAILLE PATTERN DOTS-45678

028F9
BRAILLE PATTERN DOTS-145678

028FA
BRAILLE PATTERN DOTS-245678

028FB
BRAILLE PATTERN DOTS-1245678

028FC
BRAILLE PATTERN DOTS-345678

028FD
BRAILLE PATTERN DOTS-1345678

028FE
BRAILLE PATTERN DOTS-2345678

028FF
BRAILLE PATTERN DOTS-12345678

02900
RIGHTWARDS TWO-HEADED ARROW WITH VERTICAL STROKE

02901
RIGHTWARDS TWO-HEADED ARROW WITH DOUBLE VERTICAL STROKE

02902
LEFTWARDS DOUBLE ARROW WITH VERTICAL STROKE

02903
RIGHTWARDS DOUBLE ARROW WITH VERTICAL STROKE

02904
LEFT RIGHT DOUBLE ARROW WITH VERTICAL STROKE

02905
RIGHTWARDS TWO-HEADED ARROW FROM BAR

02906
LEFTWARDS DOUBLE ARROW FROM BAR

02907
RIGHTWARDS DOUBLE ARROW FROM BAR

02908
DOWNWARDS ARROW WITH HORIZONTAL STROKE

02909
UPWARDS ARROW WITH HORIZONTAL STROKE

0290A
UPWARDS TRIPLE ARROW

0290B
DOWNWARDS TRIPLE ARROW

0290C
LEFTWARDS DOUBLE DASH ARROW

0290D
RIGHTWARDS DOUBLE DASH ARROW

0290E
LEFTWARDS TRIPLE DASH ARROW

0290F
RIGHTWARDS TRIPLE DASH ARROW

02910
RIGHTWARDS TWO-HEADED TRIPLE DASH ARROW

02911
RIGHTWARDS ARROW WITH DOTTED STEM

02912
UPWARDS ARROW TO BAR

02913
DOWNWARDS ARROW TO BAR

02914
RIGHTWARDS ARROW WITH TAIL WITH VERTICAL STROKE

02915
RIGHTWARDS ARROW WITH TAIL WITH DOUBLE VERTICAL STROKE

02916
RIGHTWARDS TWO-HEADED ARROW WITH TAIL

02917
RIGHTWARDS TWO-HEADED ARROW WITH TAIL WITH VERTICAL STROKE

02918
RIGHTWARDS TWO-HEADED ARROW WITH TAIL WITH DOUBLE VERTICAL STROKE

02919
LEFTWARDS ARROW-TAIL

0291A
RIGHTWARDS ARROW-TAIL

0291B
LEFTWARDS DOUBLE ARROW-TAIL

0291C
RIGHTWARDS DOUBLE ARROW-TAIL

0291D
LEFTWARDS ARROW TO BLACK DIAMOND

0291E
RIGHTWARDS ARROW TO BLACK DIAMOND

0291F
LEFTWARDS ARROW FROM BAR TO BLACK DIAMOND

02920
RIGHTWARDS ARROW FROM BAR TO BLACK DIAMOND

02921
NORTH WEST AND SOUTH EAST ARROW

02922
NORTH EAST AND SOUTH WEST ARROW

02923
NORTH WEST ARROW WITH HOOK

02924
NORTH EAST ARROW WITH HOOK

02925
SOUTH EAST ARROW WITH HOOK

02926
SOUTH WEST ARROW WITH HOOK

02927
NORTH WEST ARROW AND NORTH EAST ARROW

02928
NORTH EAST ARROW AND SOUTH EAST ARROW

02929
SOUTH EAST ARROW AND SOUTH WEST ARROW

0292A
SOUTH WEST ARROW AND NORTH WEST ARROW

0292B
RISING DIAGONAL CROSSING FALLING DIAGONAL

0292C
FALLING DIAGONAL CROSSING RISING DIAGONAL

0292D
SOUTH EAST ARROW CROSSING NORTH EAST ARROW

0292E
NORTH EAST ARROW CROSSING SOUTH EAST ARROW

0292F
FALLING DIAGONAL CROSSING NORTH EAST ARROW

02930
RISING DIAGONAL CROSSING SOUTH EAST ARROW

02931
NORTH EAST ARROW CROSSING NORTH WEST ARROW

02932
NORTH WEST ARROW CROSSING NORTH EAST ARROW

02933
WAVE ARROW POINTING DIRECTLY RIGHT

02934
ARROW POINTING RIGHTWARDS THEN CURVING UPWARDS

02935
ARROW POINTING RIGHTWARDS THEN CURVING DOWNWARDS

02936
ARROW POINTING DOWNWARDS THEN CURVING LEFTWARDS

02937
ARROW POINTING DOWNWARDS THEN CURVING RIGHTWARDS

02938
RIGHT-SIDE ARC CLOCKWISE ARROW

02939
LEFT-SIDE ARC ANTICLOCKWISE ARROW

0293A
TOP ARC ANTICLOCKWISE ARROW

0293B
BOTTOM ARC ANTICLOCKWISE ARROW

0293C
TOP ARC CLOCKWISE ARROW WITH MINUS

0293D
TOP ARC ANTICLOCKWISE ARROW WITH PLUS

0293E
LOWER RIGHT SEMICIRCULAR CLOCKWISE ARROW

0293F
LOWER LEFT SEMICIRCULAR ANTICLOCKWISE ARROW

02940
ANTICLOCKWISE CLOSED CIRCLE ARROW

02941
CLOCKWISE CLOSED CIRCLE ARROW

02942
RIGHTWARDS ARROW ABOVE SHORT LEFTWARDS ARROW

02943
LEFTWARDS ARROW ABOVE SHORT RIGHTWARDS ARROW

02944
SHORT RIGHTWARDS ARROW ABOVE LEFTWARDS ARROW

02945
RIGHTWARDS ARROW WITH PLUS BELOW

02946
LEFTWARDS ARROW WITH PLUS BELOW

02947
RIGHTWARDS ARROW THROUGH X

02948
LEFT RIGHT ARROW THROUGH SMALL CIRCLE

02949
UPWARDS TWO-HEADED ARROW FROM SMALL CIRCLE

0294A
LEFT BARB UP RIGHT BARB DOWN HARPOON

0294B
LEFT BARB DOWN RIGHT BARB UP HARPOON

0294C
UP BARB RIGHT DOWN BARB LEFT HARPOON

0294D
UP BARB LEFT DOWN BARB RIGHT HARPOON

0294E
LEFT BARB UP RIGHT BARB UP HARPOON

0294F
UP BARB RIGHT DOWN BARB RIGHT HARPOON

02950
LEFT BARB DOWN RIGHT BARB DOWN HARPOON

02951
UP BARB LEFT DOWN BARB LEFT HARPOON

02952
LEFTWARDS HARPOON WITH BARB UP TO BAR

02953
RIGHTWARDS HARPOON WITH BARB UP TO BAR

02954
UPWARDS HARPOON WITH BARB RIGHT TO BAR

02955
DOWNWARDS HARPOON WITH BARB RIGHT TO BAR

02956
LEFTWARDS HARPOON WITH BARB DOWN TO BAR

02957
RIGHTWARDS HARPOON WITH BARB DOWN TO BAR

02958
UPWARDS HARPOON WITH BARB LEFT TO BAR

02959
DOWNWARDS HARPOON WITH BARB LEFT TO BAR

0295A
LEFTWARDS HARPOON WITH BARB UP FROM BAR

0295B
RIGHTWARDS HARPOON WITH BARB UP FROM BAR

0295C
UPWARDS HARPOON WITH BARB RIGHT FROM BAR

0295D
DOWNWARDS HARPOON WITH BARB RIGHT FROM BAR

0295E
LEFTWARDS HARPOON WITH BARB DOWN FROM BAR

0295F
RIGHTWARDS HARPOON WITH BARB DOWN FROM BAR

02960
UPWARDS HARPOON WITH BARB LEFT FROM BAR

02961
DOWNWARDS HARPOON WITH BARB LEFT FROM BAR

02962
LEFTWARDS HARPOON WITH BARB UP ABOVE LEFTWARDS HARPOON WITH BARB DOWN

02963
UPWARDS HARPOON WITH BARB LEFT BESIDE UPWARDS HARPOON WITH BARB RIGHT

02964
RIGHTWARDS HARPOON WITH BARB UP ABOVE RIGHTWARDS HARPOON WITH BARB DOWN

02965
DOWNWARDS HARPOON WITH BARB LEFT BESIDE DOWNWARDS HARPOON WITH BARB RIGHT

02966
LEFTWARDS HARPOON WITH BARB UP ABOVE RIGHTWARDS HARPOON WITH BARB UP

02967
LEFTWARDS HARPOON WITH BARB DOWN ABOVE RIGHTWARDS HARPOON WITH BARB DOWN

02968
RIGHTWARDS HARPOON WITH BARB UP ABOVE LEFTWARDS HARPOON WITH BARB UP

02969
RIGHTWARDS HARPOON WITH BARB DOWN ABOVE LEFTWARDS HARPOON WITH BARB DOWN

0296A
LEFTWARDS HARPOON WITH BARB UP ABOVE LONG DASH

0296B
LEFTWARDS HARPOON WITH BARB DOWN BELOW LONG DASH

0296C
RIGHTWARDS HARPOON WITH BARB UP ABOVE LONG DASH

0296D
RIGHTWARDS HARPOON WITH BARB DOWN BELOW LONG DASH

0296E
UPWARDS HARPOON WITH BARB LEFT BESIDE DOWNWARDS HARPOON WITH BARB RIGHT

0296F
DOWNWARDS HARPOON WITH BARB LEFT BESIDE UPWARDS HARPOON WITH BARB RIGHT

02970
RIGHT DOUBLE ARROW WITH ROUNDED HEAD

02971
EQUALS SIGN ABOVE RIGHTWARDS ARROW

02972
TILDE OPERATOR ABOVE RIGHTWARDS ARROW

02973
LEFTWARDS ARROW ABOVE TILDE OPERATOR

02974
RIGHTWARDS ARROW ABOVE TILDE OPERATOR

02975
RIGHTWARDS ARROW ABOVE ALMOST EQUAL TO

02976
LESS-THAN ABOVE LEFTWARDS ARROW

02977
LEFTWARDS ARROW THROUGH LESS-THAN

02978
GREATER-THAN ABOVE RIGHTWARDS ARROW

02979
SUBSET ABOVE RIGHTWARDS ARROW

0297A
LEFTWARDS ARROW THROUGH SUBSET

0297B
SUPERSET ABOVE LEFTWARDS ARROW

0297C
LEFT FISH TAIL

0297D
RIGHT FISH TAIL

0297E
UP FISH TAIL

0297F
DOWN FISH TAIL

02980
TRIPLE VERTICAL BAR DELIMITER

02981
Z NOTATION SPOT

02982
Z NOTATION TYPE COLON

02983
LEFT WHITE CURLY BRACKET

02984
RIGHT WHITE CURLY BRACKET

02985
LEFT WHITE PARENTHESIS

02986
RIGHT WHITE PARENTHESIS

02987
Z NOTATION LEFT IMAGE BRACKET

02988
Z NOTATION RIGHT IMAGE BRACKET

02989
Z NOTATION LEFT BINDING BRACKET

0298A
Z NOTATION RIGHT BINDING BRACKET

0298B
LEFT SQUARE BRACKET WITH UNDERBAR

0298C
RIGHT SQUARE BRACKET WITH UNDERBAR

0298D
LEFT SQUARE BRACKET WITH TICK IN TOP CORNER

0298E
RIGHT SQUARE BRACKET WITH TICK IN BOTTOM CORNER

0298F
LEFT SQUARE BRACKET WITH TICK IN BOTTOM CORNER

02990
RIGHT SQUARE BRACKET WITH TICK IN TOP CORNER

02991
LEFT ANGLE BRACKET WITH DOT

02992
RIGHT ANGLE BRACKET WITH DOT

02993
LEFT ARC LESS-THAN BRACKET

02994
RIGHT ARC GREATER-THAN BRACKET

02995
DOUBLE LEFT ARC GREATER-THAN BRACKET

02996
DOUBLE RIGHT ARC LESS-THAN BRACKET

02997
LEFT BLACK TORTOISE SHELL BRACKET

02998
RIGHT BLACK TORTOISE SHELL BRACKET

02999
DOTTED FENCE

0299A
VERTICAL ZIGZAG LINE

0299B
MEASURED ANGLE OPENING LEFT

0299C
RIGHT ANGLE VARIANT WITH SQUARE

0299D
MEASURED RIGHT ANGLE WITH DOT

0299E
ANGLE WITH S INSIDE

0299F
ACUTE ANGLE

029A0
SPHERICAL ANGLE OPENING LEFT

029A1
SPHERICAL ANGLE OPENING UP

029A2
TURNED ANGLE

029A3
REVERSED ANGLE

029A4
ANGLE WITH UNDERBAR

029A5
REVERSED ANGLE WITH UNDERBAR

029A6
OBLIQUE ANGLE OPENING UP

029A7
OBLIQUE ANGLE OPENING DOWN

029A8
MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING UP AND RIGHT

029A9
MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING UP AND LEFT

029AA
MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING DOWN AND RIGHT

029AB
MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING DOWN AND LEFT

029AC
MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING RIGHT AND UP

029AD
MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING LEFT AND UP

029AE
MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING RIGHT AND DOWN

029AF
MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING LEFT AND DOWN

029B0
REVERSED EMPTY SET

029B1
EMPTY SET WITH OVERBAR

029B2
EMPTY SET WITH SMALL CIRCLE ABOVE

029B3
EMPTY SET WITH RIGHT ARROW ABOVE

029B4
EMPTY SET WITH LEFT ARROW ABOVE

029B5
CIRCLE WITH HORIZONTAL BAR

029B6
CIRCLED VERTICAL BAR

029B7
CIRCLED PARALLEL

029B8
CIRCLED REVERSE SOLIDUS

029B9
CIRCLED PERPENDICULAR

029BA
CIRCLE DIVIDED BY HORIZONTAL BAR AND TOP HALF DIVIDED BY VERTICAL BAR

029BB
CIRCLE WITH SUPERIMPOSED X

029BC
CIRCLED ANTICLOCKWISE-ROTATED DIVISION SIGN

029BD
UP ARROW THROUGH CIRCLE

029BE
CIRCLED WHITE BULLET

029BF
CIRCLED BULLET

029C0
CIRCLED LESS-THAN

029C1
CIRCLED GREATER-THAN

029C2
CIRCLE WITH SMALL CIRCLE TO THE RIGHT

029C3
CIRCLE WITH TWO HORIZONTAL STROKES TO THE RIGHT

029C4
SQUARED RISING DIAGONAL SLASH

029C5
SQUARED FALLING DIAGONAL SLASH

029C6
SQUARED ASTERISK

029C7
SQUARED SMALL CIRCLE

029C8
SQUARED SQUARE

029C9
TWO JOINED SQUARES

029CA
TRIANGLE WITH DOT ABOVE

029CB
TRIANGLE WITH UNDERBAR

029CC
S IN TRIANGLE

029CD
TRIANGLE WITH SERIFS AT BOTTOM

029CE
RIGHT TRIANGLE ABOVE LEFT TRIANGLE

029CF
LEFT TRIANGLE BESIDE VERTICAL BAR

029D0
VERTICAL BAR BESIDE RIGHT TRIANGLE

029D1
BOWTIE WITH LEFT HALF BLACK

029D2
BOWTIE WITH RIGHT HALF BLACK

029D3
BLACK BOWTIE

029D4
TIMES WITH LEFT HALF BLACK

029D5
TIMES WITH RIGHT HALF BLACK

029D6
WHITE HOURGLASS

029D7
BLACK HOURGLASS

029D8
LEFT WIGGLY FENCE

029D9
RIGHT WIGGLY FENCE

029DA
LEFT DOUBLE WIGGLY FENCE

029DB
RIGHT DOUBLE WIGGLY FENCE

029DC
INCOMPLETE INFINITY

029DD
TIE OVER INFINITY

029DE
INFINITY NEGATED WITH VERTICAL BAR

029DF
DOUBLE-ENDED MULTIMAP

029E0
SQUARE WITH CONTOURED OUTLINE

029E1
INCREASES AS

029E2
SHUFFLE PRODUCT

029E3
EQUALS SIGN AND SLANTED PARALLEL

029E4
EQUALS SIGN AND SLANTED PARALLEL WITH TILDE ABOVE

029E5
IDENTICAL TO AND SLANTED PARALLEL

029E6
GLEICH STARK

029E7
THERMODYNAMIC

029E8
DOWN-POINTING TRIANGLE WITH LEFT HALF BLACK

029E9
DOWN-POINTING TRIANGLE WITH RIGHT HALF BLACK

029EA
BLACK DIAMOND WITH DOWN ARROW

029EB
BLACK LOZENGE

029EC
WHITE CIRCLE WITH DOWN ARROW

029ED
BLACK CIRCLE WITH DOWN ARROW

029EE
ERROR-BARRED WHITE SQUARE

029EF
ERROR-BARRED BLACK SQUARE

029F0
ERROR-BARRED WHITE DIAMOND

029F1
ERROR-BARRED BLACK DIAMOND

029F2
ERROR-BARRED WHITE CIRCLE

029F3
ERROR-BARRED BLACK CIRCLE

029F4
RULE-DELAYED

029F5
REVERSE SOLIDUS OPERATOR

029F6
SOLIDUS WITH OVERBAR

029F7
REVERSE SOLIDUS WITH HORIZONTAL STROKE

029F8
BIG SOLIDUS

029F9
BIG REVERSE SOLIDUS

029FA
DOUBLE PLUS

029FB
TRIPLE PLUS

029FC
LEFT-POINTING CURVED ANGLE BRACKET

029FD
RIGHT-POINTING CURVED ANGLE BRACKET

029FE
TINY

029FF
MINY

02A00
N-ARY CIRCLED DOT OPERATOR

02A01
N-ARY CIRCLED PLUS OPERATOR

02A02
N-ARY CIRCLED TIMES OPERATOR

02A03
N-ARY UNION OPERATOR WITH DOT

02A04
N-ARY UNION OPERATOR WITH PLUS

02A05
N-ARY SQUARE INTERSECTION OPERATOR

02A06
N-ARY SQUARE UNION OPERATOR

02A07
TWO LOGICAL AND OPERATOR

02A08
TWO LOGICAL OR OPERATOR

02A09
N-ARY TIMES OPERATOR

02A0A
MODULO TWO SUM

02A0B
SUMMATION WITH INTEGRAL

02A0C
QUADRUPLE INTEGRAL OPERATOR

02A0D
FINITE PART INTEGRAL

02A0E
INTEGRAL WITH DOUBLE STROKE

02A0F
INTEGRAL AVERAGE WITH SLASH

02A10
CIRCULATION FUNCTION

02A11
ANTICLOCKWISE INTEGRATION

02A12
LINE INTEGRATION WITH RECTANGULAR PATH AROUND POLE

02A13
LINE INTEGRATION WITH SEMICIRCULAR PATH AROUND POLE

02A14
LINE INTEGRATION NOT INCLUDING THE POLE

02A15
INTEGRAL AROUND A POINT OPERATOR

02A16
QUATERNION INTEGRAL OPERATOR

02A17
INTEGRAL WITH LEFTWARDS ARROW WITH HOOK

02A18
INTEGRAL WITH TIMES SIGN

02A19
INTEGRAL WITH INTERSECTION

02A1A
INTEGRAL WITH UNION

02A1B
INTEGRAL WITH OVERBAR

02A1C
INTEGRAL WITH UNDERBAR

02A1D
JOIN

02A1E
LARGE LEFT TRIANGLE OPERATOR

02A1F
Z NOTATION SCHEMA COMPOSITION

02A20
Z NOTATION SCHEMA PIPING

02A21
Z NOTATION SCHEMA PROJECTION

02A22
PLUS SIGN WITH SMALL CIRCLE ABOVE

02A23
PLUS SIGN WITH CIRCUMFLEX ACCENT ABOVE

02A24
PLUS SIGN WITH TILDE ABOVE

02A25
PLUS SIGN WITH DOT BELOW

02A26
PLUS SIGN WITH TILDE BELOW

02A27
PLUS SIGN WITH SUBSCRIPT TWO

02A28
PLUS SIGN WITH BLACK TRIANGLE

02A29
MINUS SIGN WITH COMMA ABOVE

02A2A
MINUS SIGN WITH DOT BELOW

02A2B
MINUS SIGN WITH FALLING DOTS

02A2C
MINUS SIGN WITH RISING DOTS

02A2D
PLUS SIGN IN LEFT HALF CIRCLE

02A2E
PLUS SIGN IN RIGHT HALF CIRCLE

02A2F
VECTOR OR CROSS PRODUCT

02A30
MULTIPLICATION SIGN WITH DOT ABOVE

02A31
MULTIPLICATION SIGN WITH UNDERBAR

02A32
SEMIDIRECT PRODUCT WITH BOTTOM CLOSED

02A33
SMASH PRODUCT

02A34
MULTIPLICATION SIGN IN LEFT HALF CIRCLE

02A35
MULTIPLICATION SIGN IN RIGHT HALF CIRCLE

02A36
CIRCLED MULTIPLICATION SIGN WITH CIRCUMFLEX ACCENT

02A37
MULTIPLICATION SIGN IN DOUBLE CIRCLE

02A38
CIRCLED DIVISION SIGN

02A39
PLUS SIGN IN TRIANGLE

02A3A
MINUS SIGN IN TRIANGLE

02A3B
MULTIPLICATION SIGN IN TRIANGLE

02A3C
INTERIOR PRODUCT

02A3D
RIGHTHAND INTERIOR PRODUCT

02A3E
Z NOTATION RELATIONAL COMPOSITION

02A3F
AMALGAMATION OR COPRODUCT

02A40
INTERSECTION WITH DOT

02A41
UNION WITH MINUS SIGN

02A42
UNION WITH OVERBAR

02A43
INTERSECTION WITH OVERBAR

02A44
INTERSECTION WITH LOGICAL AND

02A45
UNION WITH LOGICAL OR

02A46
UNION ABOVE INTERSECTION

02A47
INTERSECTION ABOVE UNION

02A48
UNION ABOVE BAR ABOVE INTERSECTION

02A49
INTERSECTION ABOVE BAR ABOVE UNION

02A4A
UNION BESIDE AND JOINED WITH UNION

02A4B
INTERSECTION BESIDE AND JOINED WITH INTERSECTION

02A4C
CLOSED UNION WITH SERIFS

02A4D
CLOSED INTERSECTION WITH SERIFS

02A4E
DOUBLE SQUARE INTERSECTION

02A4F
DOUBLE SQUARE UNION

02A50
CLOSED UNION WITH SERIFS AND SMASH PRODUCT

02A51
LOGICAL AND WITH DOT ABOVE

02A52
LOGICAL OR WITH DOT ABOVE

02A53
DOUBLE LOGICAL AND

02A54
DOUBLE LOGICAL OR

02A55
TWO INTERSECTING LOGICAL AND

02A56
TWO INTERSECTING LOGICAL OR

02A57
SLOPING LARGE OR

02A58
SLOPING LARGE AND

02A59
LOGICAL OR OVERLAPPING LOGICAL AND

02A5A
LOGICAL AND WITH MIDDLE STEM

02A5B
LOGICAL OR WITH MIDDLE STEM

02A5C
LOGICAL AND WITH HORIZONTAL DASH

02A5D
LOGICAL OR WITH HORIZONTAL DASH

02A5E
LOGICAL AND WITH DOUBLE OVERBAR

02A5F
LOGICAL AND WITH UNDERBAR

02A60
LOGICAL AND WITH DOUBLE UNDERBAR

02A61
SMALL VEE WITH UNDERBAR

02A62
LOGICAL OR WITH DOUBLE OVERBAR

02A63
LOGICAL OR WITH DOUBLE UNDERBAR

02A64
Z NOTATION DOMAIN ANTIRESTRICTION

02A65
Z NOTATION RANGE ANTIRESTRICTION

02A66
EQUALS SIGN WITH DOT BELOW

02A67
IDENTICAL WITH DOT ABOVE

02A68
TRIPLE HORIZONTAL BAR WITH DOUBLE VERTICAL STROKE

02A69
TRIPLE HORIZONTAL BAR WITH TRIPLE VERTICAL STROKE

02A6A
TILDE OPERATOR WITH DOT ABOVE

02A6B
TILDE OPERATOR WITH RISING DOTS

02A6C
SIMILAR MINUS SIMILAR

02A6D
CONGRUENT WITH DOT ABOVE

02A6E
EQUALS WITH ASTERISK

02A6F
ALMOST EQUAL TO WITH CIRCUMFLEX ACCENT

02A70
APPROXIMATELY EQUAL OR EQUAL TO

02A71
EQUALS SIGN ABOVE PLUS SIGN

02A72
PLUS SIGN ABOVE EQUALS SIGN

02A73
EQUALS SIGN ABOVE TILDE OPERATOR

02A74
DOUBLE COLON EQUAL

02A75
TWO CONSECUTIVE EQUALS SIGNS

02A76
THREE CONSECUTIVE EQUALS SIGNS

02A77
EQUALS SIGN WITH TWO DOTS ABOVE AND TWO DOTS BELOW

02A78
EQUIVALENT WITH FOUR DOTS ABOVE

02A79
LESS-THAN WITH CIRCLE INSIDE

02A7A
GREATER-THAN WITH CIRCLE INSIDE

02A7B
LESS-THAN WITH QUESTION MARK ABOVE

02A7C
GREATER-THAN WITH QUESTION MARK ABOVE

02A7D
LESS-THAN OR SLANTED EQUAL TO

02A7E
GREATER-THAN OR SLANTED EQUAL TO

02A7F
LESS-THAN OR SLANTED EQUAL TO WITH DOT INSIDE

02A80
GREATER-THAN OR SLANTED EQUAL TO WITH DOT INSIDE

02A81
LESS-THAN OR SLANTED EQUAL TO WITH DOT ABOVE

02A82
GREATER-THAN OR SLANTED EQUAL TO WITH DOT ABOVE

02A83
LESS-THAN OR SLANTED EQUAL TO WITH DOT ABOVE RIGHT

02A84
GREATER-THAN OR SLANTED EQUAL TO WITH DOT ABOVE LEFT

02A85
LESS-THAN OR APPROXIMATE

02A86
GREATER-THAN OR APPROXIMATE

02A87
LESS-THAN AND SINGLE-LINE NOT EQUAL TO

02A88
GREATER-THAN AND SINGLE-LINE NOT EQUAL TO

02A89
LESS-THAN AND NOT APPROXIMATE

02A8A
GREATER-THAN AND NOT APPROXIMATE

02A8B
LESS-THAN ABOVE DOUBLE-LINE EQUAL ABOVE GREATER-THAN

02A8C
GREATER-THAN ABOVE DOUBLE-LINE EQUAL ABOVE LESS-THAN

02A8D
LESS-THAN ABOVE SIMILAR OR EQUAL

02A8E
GREATER-THAN ABOVE SIMILAR OR EQUAL

02A8F
LESS-THAN ABOVE SIMILAR ABOVE GREATER-THAN

02A90
GREATER-THAN ABOVE SIMILAR ABOVE LESS-THAN

02A91
LESS-THAN ABOVE GREATER-THAN ABOVE DOUBLE-LINE EQUAL

02A92
GREATER-THAN ABOVE LESS-THAN ABOVE DOUBLE-LINE EQUAL

02A93
LESS-THAN ABOVE SLANTED EQUAL ABOVE GREATER-THAN ABOVE SLANTED EQUAL

02A94
GREATER-THAN ABOVE SLANTED EQUAL ABOVE LESS-THAN ABOVE SLANTED EQUAL

02A95
SLANTED EQUAL TO OR LESS-THAN

02A96
SLANTED EQUAL TO OR GREATER-THAN

02A97
SLANTED EQUAL TO OR LESS-THAN WITH DOT INSIDE

02A98
SLANTED EQUAL TO OR GREATER-THAN WITH DOT INSIDE

02A99
DOUBLE-LINE EQUAL TO OR LESS-THAN

02A9A
DOUBLE-LINE EQUAL TO OR GREATER-THAN

02A9B
DOUBLE-LINE SLANTED EQUAL TO OR LESS-THAN

02A9C
DOUBLE-LINE SLANTED EQUAL TO OR GREATER-THAN

02A9D
SIMILAR OR LESS-THAN

02A9E
SIMILAR OR GREATER-THAN

02A9F
SIMILAR ABOVE LESS-THAN ABOVE EQUALS SIGN

02AA0
SIMILAR ABOVE GREATER-THAN ABOVE EQUALS SIGN

02AA1
DOUBLE NESTED LESS-THAN

02AA2
DOUBLE NESTED GREATER-THAN

02AA3
DOUBLE NESTED LESS-THAN WITH UNDERBAR

02AA4
GREATER-THAN OVERLAPPING LESS-THAN

02AA5
GREATER-THAN BESIDE LESS-THAN

02AA6
LESS-THAN CLOSED BY CURVE

02AA7
GREATER-THAN CLOSED BY CURVE

02AA8
LESS-THAN CLOSED BY CURVE ABOVE SLANTED EQUAL

02AA9
GREATER-THAN CLOSED BY CURVE ABOVE SLANTED EQUAL

02AAA
SMALLER THAN

02AAB
LARGER THAN

02AAC
SMALLER THAN OR EQUAL TO

02AAD
LARGER THAN OR EQUAL TO

02AAE
EQUALS SIGN WITH BUMPY ABOVE

02AAF
PRECEDES ABOVE SINGLE-LINE EQUALS SIGN

02AB0
SUCCEEDS ABOVE SINGLE-LINE EQUALS SIGN

02AB1
PRECEDES ABOVE SINGLE-LINE NOT EQUAL TO

02AB2
SUCCEEDS ABOVE SINGLE-LINE NOT EQUAL TO

02AB3
PRECEDES ABOVE EQUALS SIGN

02AB4
SUCCEEDS ABOVE EQUALS SIGN

02AB5
PRECEDES ABOVE NOT EQUAL TO

02AB6
SUCCEEDS ABOVE NOT EQUAL TO

02AB7
PRECEDES ABOVE ALMOST EQUAL TO

02AB8
SUCCEEDS ABOVE ALMOST EQUAL TO

02AB9
PRECEDES ABOVE NOT ALMOST EQUAL TO

02ABA
SUCCEEDS ABOVE NOT ALMOST EQUAL TO

02ABB
DOUBLE PRECEDES

02ABC
DOUBLE SUCCEEDS

02ABD
SUBSET WITH DOT

02ABE
SUPERSET WITH DOT

02ABF
SUBSET WITH PLUS SIGN BELOW

02AC0
SUPERSET WITH PLUS SIGN BELOW

02AC1
SUBSET WITH MULTIPLICATION SIGN BELOW

02AC2
SUPERSET WITH MULTIPLICATION SIGN BELOW

02AC3
SUBSET OF OR EQUAL TO WITH DOT ABOVE

02AC4
SUPERSET OF OR EQUAL TO WITH DOT ABOVE

02AC5
SUBSET OF ABOVE EQUALS SIGN

02AC6
SUPERSET OF ABOVE EQUALS SIGN

02AC7
SUBSET OF ABOVE TILDE OPERATOR

02AC8
SUPERSET OF ABOVE TILDE OPERATOR

02AC9
SUBSET OF ABOVE ALMOST EQUAL TO

02ACA
SUPERSET OF ABOVE ALMOST EQUAL TO

02ACB
SUBSET OF ABOVE NOT EQUAL TO

02ACC
SUPERSET OF ABOVE NOT EQUAL TO

02ACD
SQUARE LEFT OPEN BOX OPERATOR

02ACE
SQUARE RIGHT OPEN BOX OPERATOR

02ACF
CLOSED SUBSET

02AD0
CLOSED SUPERSET

02AD1
CLOSED SUBSET OR EQUAL TO

02AD2
CLOSED SUPERSET OR EQUAL TO

02AD3
SUBSET ABOVE SUPERSET

02AD4
SUPERSET ABOVE SUBSET

02AD5
SUBSET ABOVE SUBSET

02AD6
SUPERSET ABOVE SUPERSET

02AD7
SUPERSET BESIDE SUBSET

02AD8
SUPERSET BESIDE AND JOINED BY DASH WITH SUBSET

02AD9
ELEMENT OF OPENING DOWNWARDS

02ADA
PITCHFORK WITH TEE TOP

02ADB
TRANSVERSAL INTERSECTION

02ADC
FORKING

02ADD
NONFORKING

02ADE
SHORT LEFT TACK

02ADF
SHORT DOWN TACK

02AE0
SHORT UP TACK

02AE1
PERPENDICULAR WITH S

02AE2
VERTICAL BAR TRIPLE RIGHT TURNSTILE

02AE3
DOUBLE VERTICAL BAR LEFT TURNSTILE

02AE4
VERTICAL BAR DOUBLE LEFT TURNSTILE

02AE5
DOUBLE VERTICAL BAR DOUBLE LEFT TURNSTILE

02AE6
LONG DASH FROM LEFT MEMBER OF DOUBLE VERTICAL

02AE7
SHORT DOWN TACK WITH OVERBAR

02AE8
SHORT UP TACK WITH UNDERBAR

02AE9
SHORT UP TACK ABOVE SHORT DOWN TACK

02AEA
DOUBLE DOWN TACK

02AEB
DOUBLE UP TACK

02AEC
DOUBLE STROKE NOT SIGN

02AED
REVERSED DOUBLE STROKE NOT SIGN

02AEE
DOES NOT DIVIDE WITH REVERSED NEGATION SLASH

02AEF
VERTICAL LINE WITH CIRCLE ABOVE

02AF0
VERTICAL LINE WITH CIRCLE BELOW

02AF1
DOWN TACK WITH CIRCLE BELOW

02AF2
PARALLEL WITH HORIZONTAL STROKE

02AF3
PARALLEL WITH TILDE OPERATOR

02AF4
TRIPLE VERTICAL BAR BINARY RELATION

02AF5
TRIPLE VERTICAL BAR WITH HORIZONTAL STROKE

02AF6
TRIPLE COLON OPERATOR

02AF7
TRIPLE NESTED LESS-THAN

02AF8
TRIPLE NESTED GREATER-THAN

02AF9
DOUBLE-LINE SLANTED LESS-THAN OR EQUAL TO

02AFA
DOUBLE-LINE SLANTED GREATER-THAN OR EQUAL TO

02AFB
TRIPLE SOLIDUS BINARY RELATION

02AFC
LARGE TRIPLE VERTICAL BAR OPERATOR

02AFD
DOUBLE SOLIDUS OPERATOR

02AFE
WHITE VERTICAL BAR

02AFF
N-ARY WHITE VERTICAL BAR

02B00
NORTH EAST WHITE ARROW

02B01
NORTH WEST WHITE ARROW

02B02
SOUTH EAST WHITE ARROW

02B03
SOUTH WEST WHITE ARROW

02B04
LEFT RIGHT WHITE ARROW

02B05
LEFTWARDS BLACK ARROW

02B06
UPWARDS BLACK ARROW

02B07
DOWNWARDS BLACK ARROW

02B08
NORTH EAST BLACK ARROW

02B09
NORTH WEST BLACK ARROW

02B0A
SOUTH EAST BLACK ARROW

02B0B
SOUTH WEST BLACK ARROW

02B0C
LEFT RIGHT BLACK ARROW

02B0D
UP DOWN BLACK ARROW

02B0E
RIGHTWARDS ARROW WITH TIP DOWNWARDS

02B0F
RIGHTWARDS ARROW WITH TIP UPWARDS

02B10
LEFTWARDS ARROW WITH TIP DOWNWARDS

02B11
LEFTWARDS ARROW WITH TIP UPWARDS

02B12
SQUARE WITH TOP HALF BLACK

02B13
SQUARE WITH BOTTOM HALF BLACK

02B14
SQUARE WITH UPPER RIGHT DIAGONAL HALF BLACK

02B15
SQUARE WITH LOWER LEFT DIAGONAL HALF BLACK

02B16
DIAMOND WITH LEFT HALF BLACK

02B17
DIAMOND WITH RIGHT HALF BLACK

02B18
DIAMOND WITH TOP HALF BLACK

02B19
DIAMOND WITH BOTTOM HALF BLACK

02B1A
DOTTED SQUARE

02B1B
BLACK LARGE SQUARE

02B1C
WHITE LARGE SQUARE

02B1D
BLACK VERY SMALL SQUARE

02B1E
WHITE VERY SMALL SQUARE

02B1F
BLACK PENTAGON

02B20
WHITE PENTAGON

02B21
WHITE HEXAGON

02B22
BLACK HEXAGON

02B23
HORIZONTAL BLACK HEXAGON

02B24
BLACK LARGE CIRCLE

02B25
BLACK MEDIUM DIAMOND

02B26
WHITE MEDIUM DIAMOND

02B27
BLACK MEDIUM LOZENGE

02B28
WHITE MEDIUM LOZENGE

02B29
BLACK SMALL DIAMOND

02B2A
BLACK SMALL LOZENGE

02B2B
WHITE SMALL LOZENGE

02B2C
BLACK HORIZONTAL ELLIPSE

02B2D
WHITE HORIZONTAL ELLIPSE

02B2E
BLACK VERTICAL ELLIPSE

02B2F
WHITE VERTICAL ELLIPSE

02B30
LEFT ARROW WITH SMALL CIRCLE

02B31
THREE LEFTWARDS ARROWS

02B32
LEFT ARROW WITH CIRCLED PLUS

02B33
LONG LEFTWARDS SQUIGGLE ARROW

02B34
LEFTWARDS TWO-HEADED ARROW WITH VERTICAL STROKE

02B35
LEFTWARDS TWO-HEADED ARROW WITH DOUBLE VERTICAL STROKE

02B36
LEFTWARDS TWO-HEADED ARROW FROM BAR

02B37
LEFTWARDS TWO-HEADED TRIPLE DASH ARROW

02B38
LEFTWARDS ARROW WITH DOTTED STEM

02B39
LEFTWARDS ARROW WITH TAIL WITH VERTICAL STROKE

02B3A
LEFTWARDS ARROW WITH TAIL WITH DOUBLE VERTICAL STROKE

02B3B
LEFTWARDS TWO-HEADED ARROW WITH TAIL

02B3C
LEFTWARDS TWO-HEADED ARROW WITH TAIL WITH VERTICAL STROKE

02B3D
LEFTWARDS TWO-HEADED ARROW WITH TAIL WITH DOUBLE VERTICAL STROKE

02B3E
LEFTWARDS ARROW THROUGH X

02B3F
WAVE ARROW POINTING DIRECTLY LEFT

02B40
EQUALS SIGN ABOVE LEFTWARDS ARROW

02B41
REVERSE TILDE OPERATOR ABOVE LEFTWARDS ARROW

02B42
LEFTWARDS ARROW ABOVE REVERSE ALMOST EQUAL TO

02B43
RIGHTWARDS ARROW THROUGH GREATER-THAN

02B44
RIGHTWARDS ARROW THROUGH SUPERSET

02B45
LEFTWARDS QUADRUPLE ARROW

02B46
RIGHTWARDS QUADRUPLE ARROW

02B47
REVERSE TILDE OPERATOR ABOVE RIGHTWARDS ARROW

02B48
RIGHTWARDS ARROW ABOVE REVERSE ALMOST EQUAL TO

02B49
TILDE OPERATOR ABOVE LEFTWARDS ARROW

02B4A
LEFTWARDS ARROW ABOVE ALMOST EQUAL TO

02B4B
LEFTWARDS ARROW ABOVE REVERSE TILDE OPERATOR

02B4C
RIGHTWARDS ARROW ABOVE REVERSE TILDE OPERATOR

02B4D
DOWNWARDS TRIANGLE-HEADED ZIGZAG ARROW

02B4E
SHORT SLANTED NORTH ARROW

02B4F
SHORT BACKSLANTED SOUTH ARROW

02B50
WHITE MEDIUM STAR

02B51
BLACK SMALL STAR

02B52
WHITE SMALL STAR

02B53
BLACK RIGHT-POINTING PENTAGON

02B54
WHITE RIGHT-POINTING PENTAGON

02B55
HEAVY LARGE CIRCLE

02B56
HEAVY OVAL WITH OVAL INSIDE

02B57
HEAVY CIRCLE WITH CIRCLE INSIDE

02B58
HEAVY CIRCLE

02B59
HEAVY CIRCLED SALTIRE

02B5A
SLANTED NORTH ARROW WITH HOOKED HEAD

02B5B
BACKSLANTED SOUTH ARROW WITH HOOKED TAIL

02B5C
SLANTED NORTH ARROW WITH HORIZONTAL TAIL

02B5D
BACKSLANTED SOUTH ARROW WITH HORIZONTAL TAIL

02B5E
BENT ARROW POINTING DOWNWARDS THEN NORTH EAST

02B5F
SHORT BENT ARROW POINTING DOWNWARDS THEN NORTH EAST

02B60
LEFTWARDS TRIANGLE-HEADED ARROW

02B61
UPWARDS TRIANGLE-HEADED ARROW

02B62
RIGHTWARDS TRIANGLE-HEADED ARROW

02B63
DOWNWARDS TRIANGLE-HEADED ARROW

02B64
LEFT RIGHT TRIANGLE-HEADED ARROW

02B65
UP DOWN TRIANGLE-HEADED ARROW

02B66
NORTH WEST TRIANGLE-HEADED ARROW

02B67
NORTH EAST TRIANGLE-HEADED ARROW

02B68
SOUTH EAST TRIANGLE-HEADED ARROW

02B69
SOUTH WEST TRIANGLE-HEADED ARROW

02B6A
LEFTWARDS TRIANGLE-HEADED DASHED ARROW

02B6B
UPWARDS TRIANGLE-HEADED DASHED ARROW

02B6C
RIGHTWARDS TRIANGLE-HEADED DASHED ARROW

02B6D
DOWNWARDS TRIANGLE-HEADED DASHED ARROW

02B6E
CLOCKWISE TRIANGLE-HEADED OPEN CIRCLE ARROW

02B6F
ANTICLOCKWISE TRIANGLE-HEADED OPEN CIRCLE ARROW

02B70
LEFTWARDS TRIANGLE-HEADED ARROW TO BAR

02B71
UPWARDS TRIANGLE-HEADED ARROW TO BAR

02B72
RIGHTWARDS TRIANGLE-HEADED ARROW TO BAR

02B73
DOWNWARDS TRIANGLE-HEADED ARROW TO BAR

02B76
NORTH WEST TRIANGLE-HEADED ARROW TO BAR

02B77
NORTH EAST TRIANGLE-HEADED ARROW TO BAR

02B78
SOUTH EAST TRIANGLE-HEADED ARROW TO BAR

02B79
SOUTH WEST TRIANGLE-HEADED ARROW TO BAR

02B7A
LEFTWARDS TRIANGLE-HEADED ARROW WITH DOUBLE VERTICAL STROKE

02B7A
LEFTWARDS TRIANGLE-HEADED ARROW WITH DOUBLE HORIZONTAL STROKE

02B7B
UPWARDS TRIANGLE-HEADED ARROW WITH DOUBLE HORIZONTAL STROKE

02B7C
RIGHTWARDS TRIANGLE-HEADED ARROW WITH DOUBLE VERTICAL STROKE

02B7C
RIGHTWARDS TRIANGLE-HEADED ARROW WITH DOUBLE HORIZONTAL STROKE

02B7D
DOWNWARDS TRIANGLE-HEADED ARROW WITH DOUBLE HORIZONTAL STROKE

02B7E
HORIZONTAL TAB KEY

02B7F
VERTICAL TAB KEY

02B80
LEFTWARDS TRIANGLE-HEADED ARROW OVER RIGHTWARDS TRIANGLE-HEADED ARROW

02B81
UPWARDS TRIANGLE-HEADED ARROW LEFTWARDS OF DOWNWARDS TRIANGLE-HEADED ARROW

02B82
RIGHTWARDS TRIANGLE-HEADED ARROW OVER LEFTWARDS TRIANGLE-HEADED ARROW

02B83
DOWNWARDS TRIANGLE-HEADED ARROW LEFTWARDS OF UPWARDS TRIANGLE-HEADED ARROW

02B84
LEFTWARDS TRIANGLE-HEADED PAIRED ARROWS

02B85
UPWARDS TRIANGLE-HEADED PAIRED ARROWS

02B86
RIGHTWARDS TRIANGLE-HEADED PAIRED ARROWS

02B87
DOWNWARDS TRIANGLE-HEADED PAIRED ARROWS

02B88
LEFTWARDS BLACK CIRCLED WHITE ARROW

02B89
UPWARDS BLACK CIRCLED WHITE ARROW

02B8A
RIGHTWARDS BLACK CIRCLED WHITE ARROW

02B8B
DOWNWARDS BLACK CIRCLED WHITE ARROW

02B8C
ANTICLOCKWISE TRIANGLE-HEADED RIGHT U-SHAPED ARROW

02B8D
ANTICLOCKWISE TRIANGLE-HEADED BOTTOM U-SHAPED ARROW

02B8E
ANTICLOCKWISE TRIANGLE-HEADED LEFT U-SHAPED ARROW

02B8F
ANTICLOCKWISE TRIANGLE-HEADED TOP U-SHAPED ARROW

02B90
RETURN LEFT

02B91
RETURN RIGHT

02B92
NEWLINE LEFT

02B93
NEWLINE RIGHT

02B94
FOUR CORNER ARROWS CIRCLING ANTICLOCKWISE

02B95
RIGHTWARDS BLACK ARROW

02B96
EQUALS SIGN WITH INFINITY ABOVE

02B97
SYMBOL FOR TYPE A ELECTRONICS

02B98
THREE-D TOP-LIGHTED LEFTWARDS EQUILATERAL ARROWHEAD

02B99
THREE-D RIGHT-LIGHTED UPWARDS EQUILATERAL ARROWHEAD

02B9A
THREE-D TOP-LIGHTED RIGHTWARDS EQUILATERAL ARROWHEAD

02B9B
THREE-D LEFT-LIGHTED DOWNWARDS EQUILATERAL ARROWHEAD

02B9C
BLACK LEFTWARDS EQUILATERAL ARROWHEAD

02B9D
BLACK UPWARDS EQUILATERAL ARROWHEAD

02B9E
BLACK RIGHTWARDS EQUILATERAL ARROWHEAD

02B9F
BLACK DOWNWARDS EQUILATERAL ARROWHEAD

02BA0
DOWNWARDS TRIANGLE-HEADED ARROW WITH LONG TIP LEFTWARDS

02BA1
DOWNWARDS TRIANGLE-HEADED ARROW WITH LONG TIP RIGHTWARDS

02BA2
UPWARDS TRIANGLE-HEADED ARROW WITH LONG TIP LEFTWARDS

02BA3
UPWARDS TRIANGLE-HEADED ARROW WITH LONG TIP RIGHTWARDS

02BA4
LEFTWARDS TRIANGLE-HEADED ARROW WITH LONG TIP UPWARDS

02BA5
RIGHTWARDS TRIANGLE-HEADED ARROW WITH LONG TIP UPWARDS

02BA6
LEFTWARDS TRIANGLE-HEADED ARROW WITH LONG TIP DOWNWARDS

02BA7
RIGHTWARDS TRIANGLE-HEADED ARROW WITH LONG TIP DOWNWARDS

02BA8
BLACK CURVED DOWNWARDS AND LEFTWARDS ARROW

02BA9
BLACK CURVED DOWNWARDS AND RIGHTWARDS ARROW

02BAA
BLACK CURVED UPWARDS AND LEFTWARDS ARROW

02BAB
BLACK CURVED UPWARDS AND RIGHTWARDS ARROW

02BAC
BLACK CURVED LEFTWARDS AND UPWARDS ARROW

02BAD
BLACK CURVED RIGHTWARDS AND UPWARDS ARROW

02BAE
BLACK CURVED LEFTWARDS AND DOWNWARDS ARROW

02BAF
BLACK CURVED RIGHTWARDS AND DOWNWARDS ARROW

02BB0
RIBBON ARROW DOWN LEFT

02BB1
RIBBON ARROW DOWN RIGHT

02BB2
RIBBON ARROW UP LEFT

02BB3
RIBBON ARROW UP RIGHT

02BB4
RIBBON ARROW LEFT UP

02BB5
RIBBON ARROW RIGHT UP

02BB6
RIBBON ARROW LEFT DOWN

02BB7
RIBBON ARROW RIGHT DOWN

02BB8
UPWARDS WHITE ARROW FROM BAR WITH HORIZONTAL BAR

02BB9
UP ARROWHEAD IN A RECTANGLE BOX

02BBA
OVERLAPPING WHITE SQUARES

02BBB
OVERLAPPING WHITE AND BLACK SQUARES

02BBC
OVERLAPPING BLACK SQUARES

02BBD
BALLOT BOX WITH LIGHT X

02BBE
CIRCLED X

02BBF
CIRCLED BOLD X

02BC0
BLACK SQUARE CENTRED

02BC1
BLACK DIAMOND CENTRED

02BC2
TURNED BLACK PENTAGON

02BC3
HORIZONTAL BLACK OCTAGON

02BC4
BLACK OCTAGON

02BC5
BLACK MEDIUM UP-POINTING TRIANGLE CENTRED

02BC6
BLACK MEDIUM DOWN-POINTING TRIANGLE CENTRED

02BC7
BLACK MEDIUM LEFT-POINTING TRIANGLE CENTRED

02BC8
BLACK MEDIUM RIGHT-POINTING TRIANGLE CENTRED

02BC9
NEPTUNE FORM TWO

02BCA
TOP HALF BLACK CIRCLE

02BCB
BOTTOM HALF BLACK CIRCLE

02BCC
LIGHT FOUR POINTED BLACK CUSP

02BCD
ROTATED LIGHT FOUR POINTED BLACK CUSP

02BCE
WHITE FOUR POINTED CUSP

02BCF
ROTATED WHITE FOUR POINTED CUSP

02BD0
SQUARE POSITION INDICATOR

02BD1
UNCERTAINTY SIGN

02BD2
GROUP MARK

02BD3
PLUTO FORM TWO

02BD4
PLUTO FORM THREE

02BD5
PLUTO FORM FOUR

02BD6
PLUTO FORM FIVE

02BD7
TRANSPLUTO

02BD8
PROSERPINA

02BD9
ASTRAEA

02BDA
HYGIEA

02BDB
PHOLUS

02BDC
NESSUS

02BDD
WHITE MOON SELENA

02BDE
BLACK DIAMOND ON CROSS

02BDF
TRUE LIGHT MOON ARTA

02BE0
CUPIDO

02BE1
HADES

02BE2
ZEUS

02BE3
KRONOS

02BE4
APOLLON

02BE5
ADMETOS

02BE6
VULCANUS

02BE7
POSEIDON

02BE8
LEFT HALF BLACK STAR

02BE9
RIGHT HALF BLACK STAR

02BEA
STAR WITH LEFT HALF BLACK

02BEB
STAR WITH RIGHT HALF BLACK

02BEC
LEFTWARDS TWO-HEADED ARROW WITH TRIANGLE ARROWHEADS

02BED
UPWARDS TWO-HEADED ARROW WITH TRIANGLE ARROWHEADS

02BEE
RIGHTWARDS TWO-HEADED ARROW WITH TRIANGLE ARROWHEADS

02BEF
DOWNWARDS TWO-HEADED ARROW WITH TRIANGLE ARROWHEADS

02BF0
ERIS FORM ONE

02BF1
ERIS FORM TWO

02BF2
SEDNA

02BF3
RUSSIAN ASTROLOGICAL SYMBOL VIGINTILE

02BF4
RUSSIAN ASTROLOGICAL SYMBOL NOVILE

02BF5
RUSSIAN ASTROLOGICAL SYMBOL QUINTILE

02BF6
RUSSIAN ASTROLOGICAL SYMBOL BINOVILE

02BF7
RUSSIAN ASTROLOGICAL SYMBOL SENTAGON

02BF8
RUSSIAN ASTROLOGICAL SYMBOL TREDECILE

02BF9
EQUALS SIGN WITH INFINITY BELOW

02BFA
UNITED SYMBOL

02BFB
SEPARATED SYMBOL

02BFC
DOUBLED SYMBOL

02BFD
PASSED SYMBOL

02BFE
REVERSED RIGHT ANGLE

02BFF
HELLSCHREIBER PAUSE SYMBOL

02C00
GLAGOLITIC CAPITAL LETTER AZU

02C01
GLAGOLITIC CAPITAL LETTER BUKY

02C02
GLAGOLITIC CAPITAL LETTER VEDE

02C03
GLAGOLITIC CAPITAL LETTER GLAGOLI

02C04
GLAGOLITIC CAPITAL LETTER DOBRO

02C05
GLAGOLITIC CAPITAL LETTER YESTU

02C06
GLAGOLITIC CAPITAL LETTER ZHIVETE

02C07
GLAGOLITIC CAPITAL LETTER DZELO

02C08
GLAGOLITIC CAPITAL LETTER ZEMLJA

02C09
GLAGOLITIC CAPITAL LETTER IZHE

02C0A
GLAGOLITIC CAPITAL LETTER INITIAL IZHE

02C0B
GLAGOLITIC CAPITAL LETTER I

02C0C
GLAGOLITIC CAPITAL LETTER DJERVI

02C0D
GLAGOLITIC CAPITAL LETTER KAKO

02C0E
GLAGOLITIC CAPITAL LETTER LJUDIJE

02C0F
GLAGOLITIC CAPITAL LETTER MYSLITE

02C10
GLAGOLITIC CAPITAL LETTER NASHI

02C11
GLAGOLITIC CAPITAL LETTER ONU

02C12
GLAGOLITIC CAPITAL LETTER POKOJI

02C13
GLAGOLITIC CAPITAL LETTER RITSI

02C14
GLAGOLITIC CAPITAL LETTER SLOVO

02C15
GLAGOLITIC CAPITAL LETTER TVRIDO

02C16
GLAGOLITIC CAPITAL LETTER UKU

02C17
GLAGOLITIC CAPITAL LETTER FRITU

02C18
GLAGOLITIC CAPITAL LETTER HERU

02C19
GLAGOLITIC CAPITAL LETTER OTU

02C1A
GLAGOLITIC CAPITAL LETTER PE

02C1B
GLAGOLITIC CAPITAL LETTER SHTA

02C1C
GLAGOLITIC CAPITAL LETTER TSI

02C1D
GLAGOLITIC CAPITAL LETTER CHRIVI

02C1E
GLAGOLITIC CAPITAL LETTER SHA

02C1F
GLAGOLITIC CAPITAL LETTER YERU

02C20
GLAGOLITIC CAPITAL LETTER YERI

02C21
GLAGOLITIC CAPITAL LETTER YATI

02C22
GLAGOLITIC CAPITAL LETTER SPIDERY HA

02C23
GLAGOLITIC CAPITAL LETTER YU

02C24
GLAGOLITIC CAPITAL LETTER SMALL YUS

02C25
GLAGOLITIC CAPITAL LETTER SMALL YUS WITH TAIL

02C26
GLAGOLITIC CAPITAL LETTER YO

02C27
GLAGOLITIC CAPITAL LETTER IOTATED SMALL YUS

02C28
GLAGOLITIC CAPITAL LETTER BIG YUS

02C29
GLAGOLITIC CAPITAL LETTER IOTATED BIG YUS

02C2A
GLAGOLITIC CAPITAL LETTER FITA

02C2B
GLAGOLITIC CAPITAL LETTER IZHITSA

02C2C
GLAGOLITIC CAPITAL LETTER SHTAPIC

02C2D
GLAGOLITIC CAPITAL LETTER TROKUTASTI A

02C2E
GLAGOLITIC CAPITAL LETTER LATINATE MYSLITE

02C2F
GLAGOLITIC CAPITAL LETTER CAUDATE CHRIVI

02C30
GLAGOLITIC SMALL LETTER AZU

02C31
GLAGOLITIC SMALL LETTER BUKY

02C32
GLAGOLITIC SMALL LETTER VEDE

02C33
GLAGOLITIC SMALL LETTER GLAGOLI

02C34
GLAGOLITIC SMALL LETTER DOBRO

02C35
GLAGOLITIC SMALL LETTER YESTU

02C36
GLAGOLITIC SMALL LETTER ZHIVETE

02C37
GLAGOLITIC SMALL LETTER DZELO

02C38
GLAGOLITIC SMALL LETTER ZEMLJA

02C39
GLAGOLITIC SMALL LETTER IZHE

02C3A
GLAGOLITIC SMALL LETTER INITIAL IZHE

02C3B
GLAGOLITIC SMALL LETTER I

02C3C
GLAGOLITIC SMALL LETTER DJERVI

02C3D
GLAGOLITIC SMALL LETTER KAKO

02C3E
GLAGOLITIC SMALL LETTER LJUDIJE

02C3F
GLAGOLITIC SMALL LETTER MYSLITE

02C40
GLAGOLITIC SMALL LETTER NASHI

02C41
GLAGOLITIC SMALL LETTER ONU

02C42
GLAGOLITIC SMALL LETTER POKOJI

02C43
GLAGOLITIC SMALL LETTER RITSI

02C44
GLAGOLITIC SMALL LETTER SLOVO

02C45
GLAGOLITIC SMALL LETTER TVRIDO

02C46
GLAGOLITIC SMALL LETTER UKU

02C47
GLAGOLITIC SMALL LETTER FRITU

02C48
GLAGOLITIC SMALL LETTER HERU

02C49
GLAGOLITIC SMALL LETTER OTU

02C4A
GLAGOLITIC SMALL LETTER PE

02C4B
GLAGOLITIC SMALL LETTER SHTA

02C4C
GLAGOLITIC SMALL LETTER TSI

02C4D
GLAGOLITIC SMALL LETTER CHRIVI

02C4E
GLAGOLITIC SMALL LETTER SHA

02C4F
GLAGOLITIC SMALL LETTER YERU

02C50
GLAGOLITIC SMALL LETTER YERI

02C51
GLAGOLITIC SMALL LETTER YATI

02C52
GLAGOLITIC SMALL LETTER SPIDERY HA

02C53
GLAGOLITIC SMALL LETTER YU

02C54
GLAGOLITIC SMALL LETTER SMALL YUS

02C55
GLAGOLITIC SMALL LETTER SMALL YUS WITH TAIL

02C56
GLAGOLITIC SMALL LETTER YO

02C57
GLAGOLITIC SMALL LETTER IOTATED SMALL YUS

02C58
GLAGOLITIC SMALL LETTER BIG YUS

02C59
GLAGOLITIC SMALL LETTER IOTATED BIG YUS

02C5A
GLAGOLITIC SMALL LETTER FITA

02C5B
GLAGOLITIC SMALL LETTER IZHITSA

02C5C
GLAGOLITIC SMALL LETTER SHTAPIC

02C5D
GLAGOLITIC SMALL LETTER TROKUTASTI A

02C5E
GLAGOLITIC SMALL LETTER LATINATE MYSLITE

02C5F
GLAGOLITIC SMALL LETTER CAUDATE CHRIVI

02C60
LATIN CAPITAL LETTER L WITH DOUBLE BAR

02C61
LATIN SMALL LETTER L WITH DOUBLE BAR

02C62
LATIN CAPITAL LETTER L WITH MIDDLE TILDE

02C63
LATIN CAPITAL LETTER P WITH STROKE

02C64
LATIN CAPITAL LETTER R WITH TAIL

02C65
LATIN SMALL LETTER A WITH STROKE

02C66
LATIN SMALL LETTER T WITH DIAGONAL STROKE

02C67
LATIN CAPITAL LETTER H WITH DESCENDER

02C68
LATIN SMALL LETTER H WITH DESCENDER

02C69
LATIN CAPITAL LETTER K WITH DESCENDER

02C6A
LATIN SMALL LETTER K WITH DESCENDER

02C6B
LATIN CAPITAL LETTER Z WITH DESCENDER

02C6C
LATIN SMALL LETTER Z WITH DESCENDER

02C6D
LATIN CAPITAL LETTER ALPHA

02C6E
LATIN CAPITAL LETTER M WITH HOOK

02C6F
LATIN CAPITAL LETTER TURNED A

02C70
LATIN CAPITAL LETTER TURNED ALPHA

02C71
LATIN SMALL LETTER V WITH RIGHT HOOK

02C72
LATIN CAPITAL LETTER W WITH HOOK

02C73
LATIN SMALL LETTER W WITH HOOK

02C74
LATIN SMALL LETTER V WITH CURL

02C75
LATIN CAPITAL LETTER HALF H

02C76
LATIN SMALL LETTER HALF H

02C77
LATIN SMALL LETTER TAILLESS PHI

02C78
LATIN SMALL LETTER E WITH NOTCH

02C79
LATIN SMALL LETTER TURNED R WITH TAIL

02C7A
LATIN SMALL LETTER O WITH LOW RING INSIDE

02C7B
LATIN LETTER SMALL CAPITAL TURNED E

02C7C
LATIN SUBSCRIPT SMALL LETTER J

02C7D
MODIFIER LETTER CAPITAL V

02C7E
LATIN CAPITAL LETTER S WITH SWASH TAIL

02C7F
LATIN CAPITAL LETTER Z WITH SWASH TAIL

02C80
COPTIC CAPITAL LETTER ALFA

02C81
COPTIC SMALL LETTER ALFA

02C82
COPTIC CAPITAL LETTER VIDA

02C83
COPTIC SMALL LETTER VIDA

02C84
COPTIC CAPITAL LETTER GAMMA

02C85
COPTIC SMALL LETTER GAMMA

02C86
COPTIC CAPITAL LETTER DALDA

02C87
COPTIC SMALL LETTER DALDA

02C88
COPTIC CAPITAL LETTER EIE

02C89
COPTIC SMALL LETTER EIE

02C8A
COPTIC CAPITAL LETTER SOU

02C8B
COPTIC SMALL LETTER SOU

02C8C
COPTIC CAPITAL LETTER ZATA

02C8D
COPTIC SMALL LETTER ZATA

02C8E
COPTIC CAPITAL LETTER HATE

02C8F
COPTIC SMALL LETTER HATE

02C90
COPTIC CAPITAL LETTER THETHE

02C91
COPTIC SMALL LETTER THETHE

02C92
COPTIC CAPITAL LETTER IAUDA

02C93
COPTIC SMALL LETTER IAUDA

02C94
COPTIC CAPITAL LETTER KAPA

02C95
COPTIC SMALL LETTER KAPA

02C96
COPTIC CAPITAL LETTER LAULA

02C97
COPTIC SMALL LETTER LAULA

02C98
COPTIC CAPITAL LETTER MI

02C99
COPTIC SMALL LETTER MI

02C9A
COPTIC CAPITAL LETTER NI

02C9B
COPTIC SMALL LETTER NI

02C9C
COPTIC CAPITAL LETTER KSI

02C9D
COPTIC SMALL LETTER KSI

02C9E
COPTIC CAPITAL LETTER O

02C9F
COPTIC SMALL LETTER O

02CA0
COPTIC CAPITAL LETTER PI

02CA1
COPTIC SMALL LETTER PI

02CA2
COPTIC CAPITAL LETTER RO

02CA3
COPTIC SMALL LETTER RO

02CA4
COPTIC CAPITAL LETTER SIMA

02CA5
COPTIC SMALL LETTER SIMA

02CA6
COPTIC CAPITAL LETTER TAU

02CA7
COPTIC SMALL LETTER TAU

02CA8
COPTIC CAPITAL LETTER UA

02CA9
COPTIC SMALL LETTER UA

02CAA
COPTIC CAPITAL LETTER FI

02CAB
COPTIC SMALL LETTER FI

02CAC
COPTIC CAPITAL LETTER KHI

02CAD
COPTIC SMALL LETTER KHI

02CAE
COPTIC CAPITAL LETTER PSI

02CAF
COPTIC SMALL LETTER PSI

02CB0
COPTIC CAPITAL LETTER OOU

02CB1
COPTIC SMALL LETTER OOU

02CB2
COPTIC CAPITAL LETTER DIALECT-P ALEF

02CB3
COPTIC SMALL LETTER DIALECT-P ALEF

02CB4
COPTIC CAPITAL LETTER OLD COPTIC AIN

02CB5
COPTIC SMALL LETTER OLD COPTIC AIN

02CB6
COPTIC CAPITAL LETTER CRYPTOGRAMMIC EIE

02CB7
COPTIC SMALL LETTER CRYPTOGRAMMIC EIE

02CB8
COPTIC CAPITAL LETTER DIALECT-P KAPA

02CB9
COPTIC SMALL LETTER DIALECT-P KAPA

02CBA
COPTIC CAPITAL LETTER DIALECT-P NI

02CBB
COPTIC SMALL LETTER DIALECT-P NI

02CBC
COPTIC CAPITAL LETTER CRYPTOGRAMMIC NI

02CBD
COPTIC SMALL LETTER CRYPTOGRAMMIC NI

02CBE
COPTIC CAPITAL LETTER OLD COPTIC OOU

02CBF
COPTIC SMALL LETTER OLD COPTIC OOU

02CC0
COPTIC CAPITAL LETTER SAMPI

02CC1
COPTIC SMALL LETTER SAMPI

02CC2
COPTIC CAPITAL LETTER CROSSED SHEI

02CC3
COPTIC SMALL LETTER CROSSED SHEI

02CC4
COPTIC CAPITAL LETTER OLD COPTIC SHEI

02CC5
COPTIC SMALL LETTER OLD COPTIC SHEI

02CC6
COPTIC CAPITAL LETTER OLD COPTIC ESH

02CC7
COPTIC SMALL LETTER OLD COPTIC ESH

02CC8
COPTIC CAPITAL LETTER AKHMIMIC KHEI

02CC9
COPTIC SMALL LETTER AKHMIMIC KHEI

02CCA
COPTIC CAPITAL LETTER DIALECT-P HORI

02CCB
COPTIC SMALL LETTER DIALECT-P HORI

02CCC
COPTIC CAPITAL LETTER OLD COPTIC HORI

02CCD
COPTIC SMALL LETTER OLD COPTIC HORI

02CCE
COPTIC CAPITAL LETTER OLD COPTIC HA

02CCF
COPTIC SMALL LETTER OLD COPTIC HA

02CD0
COPTIC CAPITAL LETTER L-SHAPED HA

02CD1
COPTIC SMALL LETTER L-SHAPED HA

02CD2
COPTIC CAPITAL LETTER OLD COPTIC HEI

02CD3
COPTIC SMALL LETTER OLD COPTIC HEI

02CD4
COPTIC CAPITAL LETTER OLD COPTIC HAT

02CD5
COPTIC SMALL LETTER OLD COPTIC HAT

02CD6
COPTIC CAPITAL LETTER OLD COPTIC GANGIA

02CD7
COPTIC SMALL LETTER OLD COPTIC GANGIA

02CD8
COPTIC CAPITAL LETTER OLD COPTIC DJA

02CD9
COPTIC SMALL LETTER OLD COPTIC DJA

02CDA
COPTIC CAPITAL LETTER OLD COPTIC SHIMA

02CDB
COPTIC SMALL LETTER OLD COPTIC SHIMA

02CDC
COPTIC CAPITAL LETTER OLD NUBIAN SHIMA

02CDD
COPTIC SMALL LETTER OLD NUBIAN SHIMA

02CDE
COPTIC CAPITAL LETTER OLD NUBIAN NGI

02CDF
COPTIC SMALL LETTER OLD NUBIAN NGI

02CE0
COPTIC CAPITAL LETTER OLD NUBIAN NYI

02CE1
COPTIC SMALL LETTER OLD NUBIAN NYI

02CE2
COPTIC CAPITAL LETTER OLD NUBIAN WAU

02CE3
COPTIC SMALL LETTER OLD NUBIAN WAU

02CE4
COPTIC SYMBOL KAI

02CE5
COPTIC SYMBOL MI RO

02CE6
COPTIC SYMBOL PI RO

02CE7
COPTIC SYMBOL STAUROS

02CE8
COPTIC SYMBOL TAU RO

02CE9
COPTIC SYMBOL KHI RO

02CEA
COPTIC SYMBOL SHIMA SIMA

02CEB
COPTIC CAPITAL LETTER CRYPTOGRAMMIC SHEI

02CEC
COPTIC SMALL LETTER CRYPTOGRAMMIC SHEI

02CED
COPTIC CAPITAL LETTER CRYPTOGRAMMIC GANGIA

02CEE
COPTIC SMALL LETTER CRYPTOGRAMMIC GANGIA

02CEF
COPTIC COMBINING NI ABOVE

02CF0
COPTIC COMBINING SPIRITUS ASPER

02CF1
COPTIC COMBINING SPIRITUS LENIS

02CF2
COPTIC CAPITAL LETTER BOHAIRIC KHEI

02CF3
COPTIC SMALL LETTER BOHAIRIC KHEI

02CF9
COPTIC OLD NUBIAN FULL STOP

02CFA
COPTIC OLD NUBIAN DIRECT QUESTION MARK

02CFB
COPTIC OLD NUBIAN INDIRECT QUESTION MARK

02CFC
COPTIC OLD NUBIAN VERSE DIVIDER

02CFD
COPTIC FRACTION ONE HALF

02CFE
COPTIC FULL STOP

02CFF
COPTIC MORPHOLOGICAL DIVIDER

02D00
GEORGIAN SMALL LETTER AN

02D01
GEORGIAN SMALL LETTER BAN

02D02
GEORGIAN SMALL LETTER GAN

02D03
GEORGIAN SMALL LETTER DON

02D04
GEORGIAN SMALL LETTER EN

02D05
GEORGIAN SMALL LETTER VIN

02D06
GEORGIAN SMALL LETTER ZEN

02D07
GEORGIAN SMALL LETTER TAN

02D08
GEORGIAN SMALL LETTER IN

02D09
GEORGIAN SMALL LETTER KAN

02D0A
GEORGIAN SMALL LETTER LAS

02D0B
GEORGIAN SMALL LETTER MAN

02D0C
GEORGIAN SMALL LETTER NAR

02D0D
GEORGIAN SMALL LETTER ON

02D0E
GEORGIAN SMALL LETTER PAR

02D0F
GEORGIAN SMALL LETTER ZHAR

02D10
GEORGIAN SMALL LETTER RAE

02D11
GEORGIAN SMALL LETTER SAN

02D12
GEORGIAN SMALL LETTER TAR

02D13
GEORGIAN SMALL LETTER UN

02D14
GEORGIAN SMALL LETTER PHAR

02D15
GEORGIAN SMALL LETTER KHAR

02D16
GEORGIAN SMALL LETTER GHAN

02D17
GEORGIAN SMALL LETTER QAR

02D18
GEORGIAN SMALL LETTER SHIN

02D19
GEORGIAN SMALL LETTER CHIN

02D1A
GEORGIAN SMALL LETTER CAN

02D1B
GEORGIAN SMALL LETTER JIL

02D1C
GEORGIAN SMALL LETTER CIL

02D1D
GEORGIAN SMALL LETTER CHAR

02D1E
GEORGIAN SMALL LETTER XAN

02D1F
GEORGIAN SMALL LETTER JHAN

02D20
GEORGIAN SMALL LETTER HAE

02D21
GEORGIAN SMALL LETTER HE

02D22
GEORGIAN SMALL LETTER HIE

02D23
GEORGIAN SMALL LETTER WE

02D24
GEORGIAN SMALL LETTER HAR

02D25
GEORGIAN SMALL LETTER HOE

02D27
GEORGIAN SMALL LETTER YN

02D2D
GEORGIAN SMALL LETTER AEN

02D30
TIFINAGH LETTER YA

02D31
TIFINAGH LETTER YAB

02D32
TIFINAGH LETTER YABH

02D33
TIFINAGH LETTER YAG

02D34
TIFINAGH LETTER YAGHH

02D35
TIFINAGH LETTER BERBER ACADEMY YAJ

02D36
TIFINAGH LETTER YAJ

02D37
TIFINAGH LETTER YAD

02D38
TIFINAGH LETTER YADH

02D39
TIFINAGH LETTER YADD

02D3A
TIFINAGH LETTER YADDH

02D3B
TIFINAGH LETTER YEY

02D3C
TIFINAGH LETTER YAF

02D3D
TIFINAGH LETTER YAK

02D3E
TIFINAGH LETTER TUAREG YAK

02D3F
TIFINAGH LETTER YAKHH

02D40
TIFINAGH LETTER YAH

02D41
TIFINAGH LETTER BERBER ACADEMY YAH

02D42
TIFINAGH LETTER TUAREG YAH

02D43
TIFINAGH LETTER YAHH

02D44
TIFINAGH LETTER YAA

02D45
TIFINAGH LETTER YAKH

02D46
TIFINAGH LETTER TUAREG YAKH

02D47
TIFINAGH LETTER YAQ

02D48
TIFINAGH LETTER TUAREG YAQ

02D49
TIFINAGH LETTER YI

02D4A
TIFINAGH LETTER YAZH

02D4B
TIFINAGH LETTER AHAGGAR YAZH

02D4C
TIFINAGH LETTER TUAREG YAZH

02D4D
TIFINAGH LETTER YAL

02D4E
TIFINAGH LETTER YAM

02D4F
TIFINAGH LETTER YAN

02D50
TIFINAGH LETTER TUAREG YAGN

02D51
TIFINAGH LETTER TUAREG YANG

02D52
TIFINAGH LETTER YAP

02D53
TIFINAGH LETTER YU

02D54
TIFINAGH LETTER YAR

02D55
TIFINAGH LETTER YARR

02D56
TIFINAGH LETTER YAGH

02D57
TIFINAGH LETTER TUAREG YAGH

02D58
TIFINAGH LETTER AYER YAGH

02D59
TIFINAGH LETTER YAS

02D5A
TIFINAGH LETTER YASS

02D5B
TIFINAGH LETTER YASH

02D5C
TIFINAGH LETTER YAT

02D5D
TIFINAGH LETTER YATH

02D5E
TIFINAGH LETTER YACH

02D5F
TIFINAGH LETTER YATT

02D60
TIFINAGH LETTER YAV

02D61
TIFINAGH LETTER YAW

02D62
TIFINAGH LETTER YAY

02D63
TIFINAGH LETTER YAZ

02D64
TIFINAGH LETTER TAWELLEMET YAZ

02D65
TIFINAGH LETTER YAZZ

02D66
TIFINAGH LETTER YE

02D67
TIFINAGH LETTER YO

02D6F
TIFINAGH MODIFIER LETTER LABIALIZATION MARK

02D70
TIFINAGH SEPARATOR MARK

02D7F
TIFINAGH CONSONANT JOINER

02D80
ETHIOPIC SYLLABLE LOA

02D81
ETHIOPIC SYLLABLE MOA

02D82
ETHIOPIC SYLLABLE ROA

02D83
ETHIOPIC SYLLABLE SOA

02D84
ETHIOPIC SYLLABLE SHOA

02D85
ETHIOPIC SYLLABLE BOA

02D86
ETHIOPIC SYLLABLE TOA

02D87
ETHIOPIC SYLLABLE COA

02D88
ETHIOPIC SYLLABLE NOA

02D89
ETHIOPIC SYLLABLE NYOA

02D8A
ETHIOPIC SYLLABLE GLOTTAL OA

02D8B
ETHIOPIC SYLLABLE ZOA

02D8C
ETHIOPIC SYLLABLE DOA

02D8D
ETHIOPIC SYLLABLE DDOA

02D8E
ETHIOPIC SYLLABLE JOA

02D8F
ETHIOPIC SYLLABLE THOA

02D90
ETHIOPIC SYLLABLE CHOA

02D91
ETHIOPIC SYLLABLE PHOA

02D92
ETHIOPIC SYLLABLE POA

02D93
ETHIOPIC SYLLABLE GGWA

02D94
ETHIOPIC SYLLABLE GGWI

02D95
ETHIOPIC SYLLABLE GGWEE

02D96
ETHIOPIC SYLLABLE GGWE

02DA0
ETHIOPIC SYLLABLE SSA

02DA1
ETHIOPIC SYLLABLE SSU

02DA2
ETHIOPIC SYLLABLE SSI

02DA3
ETHIOPIC SYLLABLE SSAA

02DA4
ETHIOPIC SYLLABLE SSEE

02DA5
ETHIOPIC SYLLABLE SSE

02DA6
ETHIOPIC SYLLABLE SSO

02DA8
ETHIOPIC SYLLABLE CCA

02DA9
ETHIOPIC SYLLABLE CCU

02DAA
ETHIOPIC SYLLABLE CCI

02DAB
ETHIOPIC SYLLABLE CCAA

02DAC
ETHIOPIC SYLLABLE CCEE

02DAD
ETHIOPIC SYLLABLE CCE

02DAE
ETHIOPIC SYLLABLE CCO

02DB0
ETHIOPIC SYLLABLE ZZA

02DB1
ETHIOPIC SYLLABLE ZZU

02DB2
ETHIOPIC SYLLABLE ZZI

02DB3
ETHIOPIC SYLLABLE ZZAA

02DB4
ETHIOPIC SYLLABLE ZZEE

02DB5
ETHIOPIC SYLLABLE ZZE

02DB6
ETHIOPIC SYLLABLE ZZO

02DB8
ETHIOPIC SYLLABLE CCHA

02DB9
ETHIOPIC SYLLABLE CCHU

02DBA
ETHIOPIC SYLLABLE CCHI

02DBB
ETHIOPIC SYLLABLE CCHAA

02DBC
ETHIOPIC SYLLABLE CCHEE

02DBD
ETHIOPIC SYLLABLE CCHE

02DBE
ETHIOPIC SYLLABLE CCHO

02DC0
ETHIOPIC SYLLABLE QYA

02DC1
ETHIOPIC SYLLABLE QYU

02DC2
ETHIOPIC SYLLABLE QYI

02DC3
ETHIOPIC SYLLABLE QYAA

02DC4
ETHIOPIC SYLLABLE QYEE

02DC5
ETHIOPIC SYLLABLE QYE

02DC6
ETHIOPIC SYLLABLE QYO

02DC8
ETHIOPIC SYLLABLE KYA

02DC9
ETHIOPIC SYLLABLE KYU

02DCA
ETHIOPIC SYLLABLE KYI

02DCB
ETHIOPIC SYLLABLE KYAA

02DCC
ETHIOPIC SYLLABLE KYEE

02DCD
ETHIOPIC SYLLABLE KYE

02DCE
ETHIOPIC SYLLABLE KYO

02DD0
ETHIOPIC SYLLABLE XYA

02DD1
ETHIOPIC SYLLABLE XYU

02DD2
ETHIOPIC SYLLABLE XYI

02DD3
ETHIOPIC SYLLABLE XYAA

02DD4
ETHIOPIC SYLLABLE XYEE

02DD5
ETHIOPIC SYLLABLE XYE

02DD6
ETHIOPIC SYLLABLE XYO

02DD8
ETHIOPIC SYLLABLE GYA

02DD9
ETHIOPIC SYLLABLE GYU

02DDA
ETHIOPIC SYLLABLE GYI

02DDB
ETHIOPIC SYLLABLE GYAA

02DDC
ETHIOPIC SYLLABLE GYEE

02DDD
ETHIOPIC SYLLABLE GYE

02DDE
ETHIOPIC SYLLABLE GYO

02DE0
COMBINING CYRILLIC LETTER BE

02DE1
COMBINING CYRILLIC LETTER VE

02DE2
COMBINING CYRILLIC LETTER GHE

02DE3
COMBINING CYRILLIC LETTER DE

02DE4
COMBINING CYRILLIC LETTER ZHE

02DE5
COMBINING CYRILLIC LETTER ZE

02DE6
COMBINING CYRILLIC LETTER KA

02DE7
COMBINING CYRILLIC LETTER EL

02DE8
COMBINING CYRILLIC LETTER EM

02DE9
COMBINING CYRILLIC LETTER EN

02DEA
COMBINING CYRILLIC LETTER O

02DEB
COMBINING CYRILLIC LETTER PE

02DEC
COMBINING CYRILLIC LETTER ER

02DED
COMBINING CYRILLIC LETTER ES

02DEE
COMBINING CYRILLIC LETTER TE

02DEF
COMBINING CYRILLIC LETTER HA

02DF0
COMBINING CYRILLIC LETTER TSE

02DF1
COMBINING CYRILLIC LETTER CHE

02DF2
COMBINING CYRILLIC LETTER SHA

02DF3
COMBINING CYRILLIC LETTER SHCHA

02DF4
COMBINING CYRILLIC LETTER FITA

02DF5
COMBINING CYRILLIC LETTER ES-TE

02DF6
COMBINING CYRILLIC LETTER A

02DF7
COMBINING CYRILLIC LETTER IE

02DF8
COMBINING CYRILLIC LETTER DJERV

02DF9
COMBINING CYRILLIC LETTER MONOGRAPH UK

02DFA
COMBINING CYRILLIC LETTER YAT

02DFB
COMBINING CYRILLIC LETTER YU

02DFC
COMBINING CYRILLIC LETTER IOTIFIED A

02DFD
COMBINING CYRILLIC LETTER LITTLE YUS

02DFE
COMBINING CYRILLIC LETTER BIG YUS

02DFF
COMBINING CYRILLIC LETTER IOTIFIED BIG YUS

02E00
RIGHT ANGLE SUBSTITUTION MARKER

02E01
RIGHT ANGLE DOTTED SUBSTITUTION MARKER

02E02
LEFT SUBSTITUTION BRACKET

02E03
RIGHT SUBSTITUTION BRACKET

02E04
LEFT DOTTED SUBSTITUTION BRACKET

02E05
RIGHT DOTTED SUBSTITUTION BRACKET

02E06
RAISED INTERPOLATION MARKER

02E07
RAISED DOTTED INTERPOLATION MARKER

02E08
DOTTED TRANSPOSITION MARKER

02E09
LEFT TRANSPOSITION BRACKET

02E0A
RIGHT TRANSPOSITION BRACKET

02E0B
RAISED SQUARE

02E0C
LEFT RAISED OMISSION BRACKET

02E0D
RIGHT RAISED OMISSION BRACKET

02E0E
EDITORIAL CORONIS

02E0F
PARAGRAPHOS

02E10
FORKED PARAGRAPHOS

02E11
REVERSED FORKED PARAGRAPHOS

02E12
HYPODIASTOLE

02E13
DOTTED OBELOS

02E14
DOWNWARDS ANCORA

02E15
UPWARDS ANCORA

02E16
DOTTED RIGHT-POINTING ANGLE

02E17
DOUBLE OBLIQUE HYPHEN

02E18
INVERTED INTERROBANG

02E19
PALM BRANCH

02E1A
HYPHEN WITH DIAERESIS

02E1B
TILDE WITH RING ABOVE

02E1C
LEFT LOW PARAPHRASE BRACKET

02E1D
RIGHT LOW PARAPHRASE BRACKET

02E1E
TILDE WITH DOT ABOVE

02E1F
TILDE WITH DOT BELOW

02E20
LEFT VERTICAL BAR WITH QUILL

02E21
RIGHT VERTICAL BAR WITH QUILL

02E22
TOP LEFT HALF BRACKET

02E23
TOP RIGHT HALF BRACKET

02E24
BOTTOM LEFT HALF BRACKET

02E25
BOTTOM RIGHT HALF BRACKET

02E26
LEFT SIDEWAYS U BRACKET

02E27
RIGHT SIDEWAYS U BRACKET

02E28
LEFT DOUBLE PARENTHESIS

02E29
RIGHT DOUBLE PARENTHESIS

02E2A
TWO DOTS OVER ONE DOT PUNCTUATION

02E2B
ONE DOT OVER TWO DOTS PUNCTUATION

02E2C
SQUARED FOUR DOT PUNCTUATION

02E2D
FIVE DOT MARK

02E2E
REVERSED QUESTION MARK

02E2F
VERTICAL TILDE

02E30
RING POINT

02E31
WORD SEPARATOR MIDDLE DOT

02E32
TURNED COMMA

02E33
RAISED DOT

02E34
RAISED COMMA

02E35
TURNED SEMICOLON

02E36
DAGGER WITH LEFT GUARD

02E37
DAGGER WITH RIGHT GUARD

02E38
TURNED DAGGER

02E39
TOP HALF SECTION SIGN

02E3A
TWO-EM DASH

02E3B
THREE-EM DASH

02E3C
STENOGRAPHIC FULL STOP

02E3D
VERTICAL SIX DOTS

02E3E
WIGGLY VERTICAL LINE

02E3F
CAPITULUM

02E40
DOUBLE HYPHEN

02E41
REVERSED COMMA

02E42
DOUBLE LOW-REVERSED-9 QUOTATION MARK

02E43
DASH WITH LEFT UPTURN

02E44
DOUBLE SUSPENSION MARK

02E45
INVERTED LOW KAVYKA

02E46
INVERTED LOW KAVYKA WITH KAVYKA ABOVE

02E47
LOW KAVYKA

02E48
LOW KAVYKA WITH DOT

02E49
DOUBLE STACKED COMMA

02E4A
DOTTED SOLIDUS

02E4B
TRIPLE DAGGER

02E4C
MEDIEVAL COMMA

02E4D
PARAGRAPHUS MARK

02E4E
PUNCTUS ELEVATUS MARK

02E4F
CORNISH VERSE DIVIDER

02E50
CROSS PATTY WITH RIGHT CROSSBAR

02E51
CROSS PATTY WITH LEFT CROSSBAR

02E52
TIRONIAN SIGN CAPITAL ET

02E53
MEDIEVAL EXCLAMATION MARK

02E54
MEDIEVAL QUESTION MARK

02E55
LEFT SQUARE BRACKET WITH STROKE

02E56
RIGHT SQUARE BRACKET WITH STROKE

02E57
LEFT SQUARE BRACKET WITH DOUBLE STROKE

02E58
RIGHT SQUARE BRACKET WITH DOUBLE STROKE

02E59
TOP HALF LEFT PARENTHESIS

02E5A
TOP HALF RIGHT PARENTHESIS

02E5B
BOTTOM HALF LEFT PARENTHESIS

02E5C
BOTTOM HALF RIGHT PARENTHESIS

02E5D
OBLIQUE HYPHEN

02E80
CJK RADICAL REPEAT

02E81
CJK RADICAL CLIFF

02E82
CJK RADICAL SECOND ONE

02E83
CJK RADICAL SECOND TWO

02E84
CJK RADICAL SECOND THREE

02E85
CJK RADICAL PERSON

02E86
CJK RADICAL BOX

02E87
CJK RADICAL TABLE

02E88
CJK RADICAL KNIFE ONE

02E89
CJK RADICAL KNIFE TWO

02E8A
CJK RADICAL DIVINATION

02E8B
CJK RADICAL SEAL

02E8C
CJK RADICAL SMALL ONE

02E8D
CJK RADICAL SMALL TWO

02E8E
CJK RADICAL LAME ONE

02E8F
CJK RADICAL LAME TWO

02E90
CJK RADICAL LAME THREE

02E91
CJK RADICAL LAME FOUR

02E92
CJK RADICAL SNAKE

02E93
CJK RADICAL THREAD

02E94
CJK RADICAL SNOUT ONE

02E95
CJK RADICAL SNOUT TWO

02E96
CJK RADICAL HEART ONE

02E97
CJK RADICAL HEART TWO

02E98
CJK RADICAL HAND

02E99
CJK RADICAL RAP

02E9B
CJK RADICAL CHOKE

02E9C
CJK RADICAL SUN

02E9D
CJK RADICAL MOON

02E9E
CJK RADICAL DEATH

02E9F
CJK RADICAL MOTHER

02EA0
CJK RADICAL CIVILIAN

02EA1
CJK RADICAL WATER ONE

02EA2
CJK RADICAL WATER TWO

02EA3
CJK RADICAL FIRE

02EA4
CJK RADICAL PAW ONE

02EA5
CJK RADICAL PAW TWO

02EA6
CJK RADICAL SIMPLIFIED HALF TREE TRUNK

02EA7
CJK RADICAL COW

02EA8
CJK RADICAL DOG

02EA9
CJK RADICAL JADE

02EAA
CJK RADICAL BOLT OF CLOTH

02EAB
CJK RADICAL EYE

02EAC
CJK RADICAL SPIRIT ONE

02EAD
CJK RADICAL SPIRIT TWO

02EAE
CJK RADICAL BAMBOO

02EAF
CJK RADICAL SILK

02EB0
CJK RADICAL C-SIMPLIFIED SILK

02EB1
CJK RADICAL NET ONE

02EB2
CJK RADICAL NET TWO

02EB3
CJK RADICAL NET THREE

02EB4
CJK RADICAL NET FOUR

02EB5
CJK RADICAL MESH

02EB6
CJK RADICAL SHEEP

02EB7
CJK RADICAL RAM

02EB8
CJK RADICAL EWE

02EB9
CJK RADICAL OLD

02EBA
CJK RADICAL BRUSH ONE

02EBB
CJK RADICAL BRUSH TWO

02EBC
CJK RADICAL MEAT

02EBD
CJK RADICAL MORTAR

02EBE
CJK RADICAL GRASS ONE

02EBF
CJK RADICAL GRASS TWO

02EC0
CJK RADICAL GRASS THREE

02EC1
CJK RADICAL TIGER

02EC2
CJK RADICAL CLOTHES

02EC3
CJK RADICAL WEST ONE

02EC4
CJK RADICAL WEST TWO

02EC5
CJK RADICAL C-SIMPLIFIED SEE

02EC6
CJK RADICAL SIMPLIFIED HORN

02EC7
CJK RADICAL HORN

02EC8
CJK RADICAL C-SIMPLIFIED SPEECH

02EC9
CJK RADICAL C-SIMPLIFIED SHELL

02ECA
CJK RADICAL FOOT

02ECB
CJK RADICAL C-SIMPLIFIED CART

02ECC
CJK RADICAL SIMPLIFIED WALK

02ECD
CJK RADICAL WALK ONE

02ECE
CJK RADICAL WALK TWO

02ECF
CJK RADICAL CITY

02ED0
CJK RADICAL C-SIMPLIFIED GOLD

02ED1
CJK RADICAL LONG ONE

02ED2
CJK RADICAL LONG TWO

02ED3
CJK RADICAL C-SIMPLIFIED LONG

02ED4
CJK RADICAL C-SIMPLIFIED GATE

02ED5
CJK RADICAL MOUND ONE

02ED6
CJK RADICAL MOUND TWO

02ED7
CJK RADICAL RAIN

02ED8
CJK RADICAL BLUE

02ED9
CJK RADICAL C-SIMPLIFIED TANNED LEATHER

02EDA
CJK RADICAL C-SIMPLIFIED LEAF

02EDB
CJK RADICAL C-SIMPLIFIED WIND

02EDC
CJK RADICAL C-SIMPLIFIED FLY

02EDD
CJK RADICAL EAT ONE

02EDE
CJK RADICAL EAT TWO

02EDF
CJK RADICAL EAT THREE

02EE0
CJK RADICAL C-SIMPLIFIED EAT

02EE1
CJK RADICAL HEAD

02EE2
CJK RADICAL C-SIMPLIFIED HORSE

02EE3
CJK RADICAL BONE

02EE4
CJK RADICAL GHOST

02EE5
CJK RADICAL C-SIMPLIFIED FISH

02EE6
CJK RADICAL C-SIMPLIFIED BIRD

02EE7
CJK RADICAL C-SIMPLIFIED SALT

02EE8
CJK RADICAL SIMPLIFIED WHEAT

02EE9
CJK RADICAL SIMPLIFIED YELLOW

02EEA
CJK RADICAL C-SIMPLIFIED FROG

02EEB
CJK RADICAL J-SIMPLIFIED EVEN

02EEC
CJK RADICAL C-SIMPLIFIED EVEN

02EED
CJK RADICAL J-SIMPLIFIED TOOTH

02EEE
CJK RADICAL C-SIMPLIFIED TOOTH

02EEF
CJK RADICAL J-SIMPLIFIED DRAGON

02EF0
CJK RADICAL C-SIMPLIFIED DRAGON

02EF1
CJK RADICAL TURTLE

02EF2
CJK RADICAL J-SIMPLIFIED TURTLE

02EF3
CJK RADICAL C-SIMPLIFIED TURTLE

02F00
KANGXI RADICAL ONE

02F01
KANGXI RADICAL LINE

02F02
KANGXI RADICAL DOT

02F03
KANGXI RADICAL SLASH

02F04
KANGXI RADICAL SECOND

02F05
KANGXI RADICAL HOOK

02F06
KANGXI RADICAL TWO

02F07
KANGXI RADICAL LID

02F08
KANGXI RADICAL MAN

02F09
KANGXI RADICAL LEGS

02F0A
KANGXI RADICAL ENTER

02F0B
KANGXI RADICAL EIGHT

02F0C
KANGXI RADICAL DOWN BOX

02F0D
KANGXI RADICAL COVER

02F0E
KANGXI RADICAL ICE

02F0F
KANGXI RADICAL TABLE

02F10
KANGXI RADICAL OPEN BOX

02F11
KANGXI RADICAL KNIFE

02F12
KANGXI RADICAL POWER

02F13
KANGXI RADICAL WRAP

02F14
KANGXI RADICAL SPOON

02F15
KANGXI RADICAL RIGHT OPEN BOX

02F16
KANGXI RADICAL HIDING ENCLOSURE

02F17
KANGXI RADICAL TEN

02F18
KANGXI RADICAL DIVINATION

02F19
KANGXI RADICAL SEAL

02F1A
KANGXI RADICAL CLIFF

02F1B
KANGXI RADICAL PRIVATE

02F1C
KANGXI RADICAL AGAIN

02F1D
KANGXI RADICAL MOUTH

02F1E
KANGXI RADICAL ENCLOSURE

02F1F
KANGXI RADICAL EARTH

02F20
KANGXI RADICAL SCHOLAR

02F21
KANGXI RADICAL GO

02F22
KANGXI RADICAL GO SLOWLY

02F23
KANGXI RADICAL EVENING

02F24
KANGXI RADICAL BIG

02F25
KANGXI RADICAL WOMAN

02F26
KANGXI RADICAL CHILD

02F27
KANGXI RADICAL ROOF

02F28
KANGXI RADICAL INCH

02F29
KANGXI RADICAL SMALL

02F2A
KANGXI RADICAL LAME

02F2B
KANGXI RADICAL CORPSE

02F2C
KANGXI RADICAL SPROUT

02F2D
KANGXI RADICAL MOUNTAIN

02F2E
KANGXI RADICAL RIVER

02F2F
KANGXI RADICAL WORK

02F30
KANGXI RADICAL ONESELF

02F31
KANGXI RADICAL TURBAN

02F32
KANGXI RADICAL DRY

02F33
KANGXI RADICAL SHORT THREAD

02F34
KANGXI RADICAL DOTTED CLIFF

02F35
KANGXI RADICAL LONG STRIDE

02F36
KANGXI RADICAL TWO HANDS

02F37
KANGXI RADICAL SHOOT

02F38
KANGXI RADICAL BOW

02F39
KANGXI RADICAL SNOUT

02F3A
KANGXI RADICAL BRISTLE

02F3B
KANGXI RADICAL STEP

02F3C
KANGXI RADICAL HEART

02F3D
KANGXI RADICAL HALBERD

02F3E
KANGXI RADICAL DOOR

02F3F
KANGXI RADICAL HAND

02F40
KANGXI RADICAL BRANCH

02F41
KANGXI RADICAL RAP

02F42
KANGXI RADICAL SCRIPT

02F43
KANGXI RADICAL DIPPER

02F44
KANGXI RADICAL AXE

02F45
KANGXI RADICAL SQUARE

02F46
KANGXI RADICAL NOT

02F47
KANGXI RADICAL SUN

02F48
KANGXI RADICAL SAY

02F49
KANGXI RADICAL MOON

02F4A
KANGXI RADICAL TREE

02F4B
KANGXI RADICAL LACK

02F4C
KANGXI RADICAL STOP

02F4D
KANGXI RADICAL DEATH

02F4E
KANGXI RADICAL WEAPON

02F4F
KANGXI RADICAL DO NOT

02F50
KANGXI RADICAL COMPARE

02F51
KANGXI RADICAL FUR

02F52
KANGXI RADICAL CLAN

02F53
KANGXI RADICAL STEAM

02F54
KANGXI RADICAL WATER

02F55
KANGXI RADICAL FIRE

02F56
KANGXI RADICAL CLAW

02F57
KANGXI RADICAL FATHER

02F58
KANGXI RADICAL DOUBLE X

02F59
KANGXI RADICAL HALF TREE TRUNK

02F5A
KANGXI RADICAL SLICE

02F5B
KANGXI RADICAL FANG

02F5C
KANGXI RADICAL COW

02F5D
KANGXI RADICAL DOG

02F5E
KANGXI RADICAL PROFOUND

02F5F
KANGXI RADICAL JADE

02F60
KANGXI RADICAL MELON

02F61
KANGXI RADICAL TILE

02F62
KANGXI RADICAL SWEET

02F63
KANGXI RADICAL LIFE

02F64
KANGXI RADICAL USE

02F65
KANGXI RADICAL FIELD

02F66
KANGXI RADICAL BOLT OF CLOTH

02F67
KANGXI RADICAL SICKNESS

02F68
KANGXI RADICAL DOTTED TENT

02F69
KANGXI RADICAL WHITE

02F6A
KANGXI RADICAL SKIN

02F6B
KANGXI RADICAL DISH

02F6C
KANGXI RADICAL EYE

02F6D
KANGXI RADICAL SPEAR

02F6E
KANGXI RADICAL ARROW

02F6F
KANGXI RADICAL STONE

02F70
KANGXI RADICAL SPIRIT

02F71
KANGXI RADICAL TRACK

02F72
KANGXI RADICAL GRAIN

02F73
KANGXI RADICAL CAVE

02F74
KANGXI RADICAL STAND

02F75
KANGXI RADICAL BAMBOO

02F76
KANGXI RADICAL RICE

02F77
KANGXI RADICAL SILK

02F78
KANGXI RADICAL JAR

02F79
KANGXI RADICAL NET

02F7A
KANGXI RADICAL SHEEP

02F7B
KANGXI RADICAL FEATHER

02F7C
KANGXI RADICAL OLD

02F7D
KANGXI RADICAL AND

02F7E
KANGXI RADICAL PLOW

02F7F
KANGXI RADICAL EAR

02F80
KANGXI RADICAL BRUSH

02F81
KANGXI RADICAL MEAT

02F82
KANGXI RADICAL MINISTER

02F83
KANGXI RADICAL SELF

02F84
KANGXI RADICAL ARRIVE

02F85
KANGXI RADICAL MORTAR

02F86
KANGXI RADICAL TONGUE

02F87
KANGXI RADICAL OPPOSE

02F88
KANGXI RADICAL BOAT

02F89
KANGXI RADICAL STOPPING

02F8A
KANGXI RADICAL COLOR

02F8B
KANGXI RADICAL GRASS

02F8C
KANGXI RADICAL TIGER

02F8D
KANGXI RADICAL INSECT

02F8E
KANGXI RADICAL BLOOD

02F8F
KANGXI RADICAL WALK ENCLOSURE

02F90
KANGXI RADICAL CLOTHES

02F91
KANGXI RADICAL WEST

02F92
KANGXI RADICAL SEE

02F93
KANGXI RADICAL HORN

02F94
KANGXI RADICAL SPEECH

02F95
KANGXI RADICAL VALLEY

02F96
KANGXI RADICAL BEAN

02F97
KANGXI RADICAL PIG

02F98
KANGXI RADICAL BADGER

02F99
KANGXI RADICAL SHELL

02F9A
KANGXI RADICAL RED

02F9B
KANGXI RADICAL RUN

02F9C
KANGXI RADICAL FOOT

02F9D
KANGXI RADICAL BODY

02F9E
KANGXI RADICAL CART

02F9F
KANGXI RADICAL BITTER

02FA0
KANGXI RADICAL MORNING

02FA1
KANGXI RADICAL WALK

02FA2
KANGXI RADICAL CITY

02FA3
KANGXI RADICAL WINE

02FA4
KANGXI RADICAL DISTINGUISH

02FA5
KANGXI RADICAL VILLAGE

02FA6
KANGXI RADICAL GOLD

02FA7
KANGXI RADICAL LONG

02FA8
KANGXI RADICAL GATE

02FA9
KANGXI RADICAL MOUND

02FAA
KANGXI RADICAL SLAVE

02FAB
KANGXI RADICAL SHORT TAILED BIRD

02FAC
KANGXI RADICAL RAIN

02FAD
KANGXI RADICAL BLUE

02FAE
KANGXI RADICAL WRONG

02FAF
KANGXI RADICAL FACE

02FB0
KANGXI RADICAL LEATHER

02FB1
KANGXI RADICAL TANNED LEATHER

02FB2
KANGXI RADICAL LEEK

02FB3
KANGXI RADICAL SOUND

02FB4
KANGXI RADICAL LEAF

02FB5
KANGXI RADICAL WIND

02FB6
KANGXI RADICAL FLY

02FB7
KANGXI RADICAL EAT

02FB8
KANGXI RADICAL HEAD

02FB9
KANGXI RADICAL FRAGRANT

02FBA
KANGXI RADICAL HORSE

02FBB
KANGXI RADICAL BONE

02FBC
KANGXI RADICAL TALL

02FBD
KANGXI RADICAL HAIR

02FBE
KANGXI RADICAL FIGHT

02FBF
KANGXI RADICAL SACRIFICIAL WINE

02FC0
KANGXI RADICAL CAULDRON

02FC1
KANGXI RADICAL GHOST

02FC2
KANGXI RADICAL FISH

02FC3
KANGXI RADICAL BIRD

02FC4
KANGXI RADICAL SALT

02FC5
KANGXI RADICAL DEER

02FC6
KANGXI RADICAL WHEAT

02FC7
KANGXI RADICAL HEMP

02FC8
KANGXI RADICAL YELLOW

02FC9
KANGXI RADICAL MILLET

02FCA
KANGXI RADICAL BLACK

02FCB
KANGXI RADICAL EMBROIDERY

02FCC
KANGXI RADICAL FROG

02FCD
KANGXI RADICAL TRIPOD

02FCE
KANGXI RADICAL DRUM

02FCF
KANGXI RADICAL RAT

02FD0
KANGXI RADICAL NOSE

02FD1
KANGXI RADICAL EVEN

02FD2
KANGXI RADICAL TOOTH

02FD3
KANGXI RADICAL DRAGON

02FD4
KANGXI RADICAL TURTLE

02FD5
KANGXI RADICAL FLUTE

02FF0
IDEOGRAPHIC DESCRIPTION CHARACTER LEFT TO RIGHT

02FF1
IDEOGRAPHIC DESCRIPTION CHARACTER ABOVE TO BELOW

02FF2
IDEOGRAPHIC DESCRIPTION CHARACTER LEFT TO MIDDLE AND RIGHT

02FF3
IDEOGRAPHIC DESCRIPTION CHARACTER ABOVE TO MIDDLE AND BELOW

02FF4
IDEOGRAPHIC DESCRIPTION CHARACTER FULL SURROUND

02FF5
IDEOGRAPHIC DESCRIPTION CHARACTER SURROUND FROM ABOVE

02FF6
IDEOGRAPHIC DESCRIPTION CHARACTER SURROUND FROM BELOW

02FF7
IDEOGRAPHIC DESCRIPTION CHARACTER SURROUND FROM LEFT

02FF8
IDEOGRAPHIC DESCRIPTION CHARACTER SURROUND FROM UPPER LEFT

02FF9
IDEOGRAPHIC DESCRIPTION CHARACTER SURROUND FROM UPPER RIGHT

02FFA
IDEOGRAPHIC DESCRIPTION CHARACTER SURROUND FROM LOWER LEFT

02FFB
IDEOGRAPHIC DESCRIPTION CHARACTER OVERLAID

02FFC
IDEOGRAPHIC DESCRIPTION CHARACTER SURROUND FROM RIGHT

02FFD
IDEOGRAPHIC DESCRIPTION CHARACTER SURROUND FROM LOWER RIGHT

02FFE
IDEOGRAPHIC DESCRIPTION CHARACTER HORIZONTAL REFLECTION

02FFF
IDEOGRAPHIC DESCRIPTION CHARACTER ROTATION

03000
IDEOGRAPHIC SPACE

03001
IDEOGRAPHIC COMMA

03002
IDEOGRAPHIC FULL STOP

03003
DITTO MARK

03004
JAPANESE INDUSTRIAL STANDARD SYMBOL

03005
IDEOGRAPHIC ITERATION MARK

03006
IDEOGRAPHIC CLOSING MARK

03007
IDEOGRAPHIC NUMBER ZERO

03008
LEFT ANGLE BRACKET

03009
RIGHT ANGLE BRACKET

0300A
LEFT DOUBLE ANGLE BRACKET

0300B
RIGHT DOUBLE ANGLE BRACKET

0300C
LEFT CORNER BRACKET

0300D
RIGHT CORNER BRACKET

0300E
LEFT WHITE CORNER BRACKET

0300F
RIGHT WHITE CORNER BRACKET

03010
LEFT BLACK LENTICULAR BRACKET

03011
RIGHT BLACK LENTICULAR BRACKET

03012
POSTAL MARK

03013
GETA MARK

03014
LEFT TORTOISE SHELL BRACKET

03015
RIGHT TORTOISE SHELL BRACKET

03016
LEFT WHITE LENTICULAR BRACKET

03017
RIGHT WHITE LENTICULAR BRACKET

03018
LEFT WHITE TORTOISE SHELL BRACKET

03019
RIGHT WHITE TORTOISE SHELL BRACKET

0301A
LEFT WHITE SQUARE BRACKET

0301B
RIGHT WHITE SQUARE BRACKET

0301C
WAVE DASH

0301D
REVERSED DOUBLE PRIME QUOTATION MARK

0301E
DOUBLE PRIME QUOTATION MARK

0301F
LOW DOUBLE PRIME QUOTATION MARK

03020
POSTAL MARK FACE

03021
HANGZHOU NUMERAL ONE

03022
HANGZHOU NUMERAL TWO

03023
HANGZHOU NUMERAL THREE

03024
HANGZHOU NUMERAL FOUR

03025
HANGZHOU NUMERAL FIVE

03026
HANGZHOU NUMERAL SIX

03027
HANGZHOU NUMERAL SEVEN

03028
HANGZHOU NUMERAL EIGHT

03029
HANGZHOU NUMERAL NINE

0302A
IDEOGRAPHIC LEVEL TONE MARK

0302B
IDEOGRAPHIC RISING TONE MARK

0302C
IDEOGRAPHIC DEPARTING TONE MARK

0302D
IDEOGRAPHIC ENTERING TONE MARK

0302E
HANGUL SINGLE DOT TONE MARK

0302F
HANGUL DOUBLE DOT TONE MARK

03030
WAVY DASH

03031
VERTICAL KANA REPEAT MARK

03032
VERTICAL KANA REPEAT WITH VOICED SOUND MARK

03033
VERTICAL KANA REPEAT MARK UPPER HALF

03034
VERTICAL KANA REPEAT WITH VOICED SOUND MARK UPPER HALF

03035
VERTICAL KANA REPEAT MARK LOWER HALF

03036
CIRCLED POSTAL MARK

03037
IDEOGRAPHIC TELEGRAPH LINE FEED SEPARATOR SYMBOL

03038
HANGZHOU NUMERAL TEN

03039
HANGZHOU NUMERAL TWENTY

0303A
HANGZHOU NUMERAL THIRTY

0303B
VERTICAL IDEOGRAPHIC ITERATION MARK

0303C
MASU MARK

0303D
PART ALTERNATION MARK

0303E
IDEOGRAPHIC VARIATION INDICATOR

0303F
IDEOGRAPHIC HALF FILL SPACE

03041
HIRAGANA LETTER SMALL A

03042
HIRAGANA LETTER A

03043
HIRAGANA LETTER SMALL I

03044
HIRAGANA LETTER I

03045
HIRAGANA LETTER SMALL U

03046
HIRAGANA LETTER U

03047
HIRAGANA LETTER SMALL E

03048
HIRAGANA LETTER E

03049
HIRAGANA LETTER SMALL O

0304A
HIRAGANA LETTER O

0304B
HIRAGANA LETTER KA

0304C
HIRAGANA LETTER GA

0304D
HIRAGANA LETTER KI

0304E
HIRAGANA LETTER GI

0304F
HIRAGANA LETTER KU

03050
HIRAGANA LETTER GU

03051
HIRAGANA LETTER KE

03052
HIRAGANA LETTER GE

03053
HIRAGANA LETTER KO

03054
HIRAGANA LETTER GO

03055
HIRAGANA LETTER SA

03056
HIRAGANA LETTER ZA

03057
HIRAGANA LETTER SI

03058
HIRAGANA LETTER ZI

03059
HIRAGANA LETTER SU

0305A
HIRAGANA LETTER ZU

0305B
HIRAGANA LETTER SE

0305C
HIRAGANA LETTER ZE

0305D
HIRAGANA LETTER SO

0305E
HIRAGANA LETTER ZO

0305F
HIRAGANA LETTER TA

03060
HIRAGANA LETTER DA

03061
HIRAGANA LETTER TI

03062
HIRAGANA LETTER DI

03063
HIRAGANA LETTER SMALL TU

03064
HIRAGANA LETTER TU

03065
HIRAGANA LETTER DU

03066
HIRAGANA LETTER TE

03067
HIRAGANA LETTER DE

03068
HIRAGANA LETTER TO

03069
HIRAGANA LETTER DO

0306A
HIRAGANA LETTER NA

0306B
HIRAGANA LETTER NI

0306C
HIRAGANA LETTER NU

0306D
HIRAGANA LETTER NE

0306E
HIRAGANA LETTER NO

0306F
HIRAGANA LETTER HA

03070
HIRAGANA LETTER BA

03071
HIRAGANA LETTER PA

03072
HIRAGANA LETTER HI

03073
HIRAGANA LETTER BI

03074
HIRAGANA LETTER PI

03075
HIRAGANA LETTER HU

03076
HIRAGANA LETTER BU

03077
HIRAGANA LETTER PU

03078
HIRAGANA LETTER HE

03079
HIRAGANA LETTER BE

0307A
HIRAGANA LETTER PE

0307B
HIRAGANA LETTER HO

0307C
HIRAGANA LETTER BO

0307D
HIRAGANA LETTER PO

0307E
HIRAGANA LETTER MA

0307F
HIRAGANA LETTER MI

03080
HIRAGANA LETTER MU

03081
HIRAGANA LETTER ME

03082
HIRAGANA LETTER MO

03083
HIRAGANA LETTER SMALL YA

03084
HIRAGANA LETTER YA

03085
HIRAGANA LETTER SMALL YU

03086
HIRAGANA LETTER YU

03087
HIRAGANA LETTER SMALL YO

03088
HIRAGANA LETTER YO

03089
HIRAGANA LETTER RA

0308A
HIRAGANA LETTER RI

0308B
HIRAGANA LETTER RU

0308C
HIRAGANA LETTER RE

0308D
HIRAGANA LETTER RO

0308E
HIRAGANA LETTER SMALL WA

0308F
HIRAGANA LETTER WA

03090
HIRAGANA LETTER WI

03091
HIRAGANA LETTER WE

03092
HIRAGANA LETTER WO

03093
HIRAGANA LETTER N

03094
HIRAGANA LETTER VU

03095
HIRAGANA LETTER SMALL KA

03096
HIRAGANA LETTER SMALL KE

03099
COMBINING KATAKANA-HIRAGANA VOICED SOUND MARK

0309A
COMBINING KATAKANA-HIRAGANA SEMI-VOICED SOUND MARK

0309B
KATAKANA-HIRAGANA VOICED SOUND MARK

0309C
KATAKANA-HIRAGANA SEMI-VOICED SOUND MARK

0309D
HIRAGANA ITERATION MARK

0309E
HIRAGANA VOICED ITERATION MARK

0309F
HIRAGANA DIGRAPH YORI

030A0
KATAKANA-HIRAGANA DOUBLE HYPHEN

030A1
KATAKANA LETTER SMALL A

030A2
KATAKANA LETTER A

030A3
KATAKANA LETTER SMALL I

030A4
KATAKANA LETTER I

030A5
KATAKANA LETTER SMALL U

030A6
KATAKANA LETTER U

030A7
KATAKANA LETTER SMALL E

030A8
KATAKANA LETTER E

030A9
KATAKANA LETTER SMALL O

030AA
KATAKANA LETTER O

030AB
KATAKANA LETTER KA

030AC
KATAKANA LETTER GA

030AD
KATAKANA LETTER KI

030AE
KATAKANA LETTER GI

030AF
KATAKANA LETTER KU

030B0
KATAKANA LETTER GU

030B1
KATAKANA LETTER KE

030B2
KATAKANA LETTER GE

030B3
KATAKANA LETTER KO

030B4
KATAKANA LETTER GO

030B5
KATAKANA LETTER SA

030B6
KATAKANA LETTER ZA

030B7
KATAKANA LETTER SI

030B8
KATAKANA LETTER ZI

030B9
KATAKANA LETTER SU

030BA
KATAKANA LETTER ZU

030BB
KATAKANA LETTER SE

030BC
KATAKANA LETTER ZE

030BD
KATAKANA LETTER SO

030BE
KATAKANA LETTER ZO

030BF
KATAKANA LETTER TA

030C0
KATAKANA LETTER DA

030C1
KATAKANA LETTER TI

030C2
KATAKANA LETTER DI

030C3
KATAKANA LETTER SMALL TU

030C4
KATAKANA LETTER TU

030C5
KATAKANA LETTER DU

030C6
KATAKANA LETTER TE

030C7
KATAKANA LETTER DE

030C8
KATAKANA LETTER TO

030C9
KATAKANA LETTER DO

030CA
KATAKANA LETTER NA

030CB
KATAKANA LETTER NI

030CC
KATAKANA LETTER NU

030CD
KATAKANA LETTER NE

030CE
KATAKANA LETTER NO

030CF
KATAKANA LETTER HA

030D0
KATAKANA LETTER BA

030D1
KATAKANA LETTER PA

030D2
KATAKANA LETTER HI

030D3
KATAKANA LETTER BI

030D4
KATAKANA LETTER PI

030D5
KATAKANA LETTER HU

030D6
KATAKANA LETTER BU

030D7
KATAKANA LETTER PU

030D8
KATAKANA LETTER HE

030D9
KATAKANA LETTER BE

030DA
KATAKANA LETTER PE

030DB
KATAKANA LETTER HO

030DC
KATAKANA LETTER BO

030DD
KATAKANA LETTER PO

030DE
KATAKANA LETTER MA

030DF
KATAKANA LETTER MI

030E0
KATAKANA LETTER MU

030E1
KATAKANA LETTER ME

030E2
KATAKANA LETTER MO

030E3
KATAKANA LETTER SMALL YA

030E4
KATAKANA LETTER YA

030E5
KATAKANA LETTER SMALL YU

030E6
KATAKANA LETTER YU

030E7
KATAKANA LETTER SMALL YO

030E8
KATAKANA LETTER YO

030E9
KATAKANA LETTER RA

030EA
KATAKANA LETTER RI

030EB
KATAKANA LETTER RU

030EC
KATAKANA LETTER RE

030ED
KATAKANA LETTER RO

030EE
KATAKANA LETTER SMALL WA

030EF
KATAKANA LETTER WA

030F0
KATAKANA LETTER WI

030F1
KATAKANA LETTER WE

030F2
KATAKANA LETTER WO

030F3
KATAKANA LETTER N

030F4
KATAKANA LETTER VU

030F5
KATAKANA LETTER SMALL KA

030F6
KATAKANA LETTER SMALL KE

030F7
KATAKANA LETTER VA

030F8
KATAKANA LETTER VI

030F9
KATAKANA LETTER VE

030FA
KATAKANA LETTER VO

030FB
KATAKANA MIDDLE DOT

030FC
KATAKANA-HIRAGANA PROLONGED SOUND MARK

030FD
KATAKANA ITERATION MARK

030FE
KATAKANA VOICED ITERATION MARK

030FF
KATAKANA DIGRAPH KOTO

03105
BOPOMOFO LETTER B

03106
BOPOMOFO LETTER P

03107
BOPOMOFO LETTER M

03108
BOPOMOFO LETTER F

03109
BOPOMOFO LETTER D

0310A
BOPOMOFO LETTER T

0310B
BOPOMOFO LETTER N

0310C
BOPOMOFO LETTER L

0310D
BOPOMOFO LETTER G

0310E
BOPOMOFO LETTER K

0310F
BOPOMOFO LETTER H

03110
BOPOMOFO LETTER J

03111
BOPOMOFO LETTER Q

03112
BOPOMOFO LETTER X

03113
BOPOMOFO LETTER ZH

03114
BOPOMOFO LETTER CH

03115
BOPOMOFO LETTER SH

03116
BOPOMOFO LETTER R

03117
BOPOMOFO LETTER Z

03118
BOPOMOFO LETTER C

03119
BOPOMOFO LETTER S

0311A
BOPOMOFO LETTER A

0311B
BOPOMOFO LETTER O

0311C
BOPOMOFO LETTER E

0311D
BOPOMOFO LETTER EH

0311E
BOPOMOFO LETTER AI

0311F
BOPOMOFO LETTER EI

03120
BOPOMOFO LETTER AU

03121
BOPOMOFO LETTER OU

03122
BOPOMOFO LETTER AN

03123
BOPOMOFO LETTER EN

03124
BOPOMOFO LETTER ANG

03125
BOPOMOFO LETTER ENG

03126
BOPOMOFO LETTER ER

03127
BOPOMOFO LETTER I

03128
BOPOMOFO LETTER U

03129
BOPOMOFO LETTER IU

0312A
BOPOMOFO LETTER V

0312B
BOPOMOFO LETTER NG

0312C
BOPOMOFO LETTER GN

0312D
BOPOMOFO LETTER IH

0312E
BOPOMOFO LETTER O WITH DOT ABOVE

0312F
BOPOMOFO LETTER NN

03131
HANGUL LETTER KIYEOK

03132
HANGUL LETTER SSANGKIYEOK

03133
HANGUL LETTER KIYEOK-SIOS

03134
HANGUL LETTER NIEUN

03135
HANGUL LETTER NIEUN-CIEUC

03136
HANGUL LETTER NIEUN-HIEUH

03137
HANGUL LETTER TIKEUT

03138
HANGUL LETTER SSANGTIKEUT

03139
HANGUL LETTER RIEUL

0313A
HANGUL LETTER RIEUL-KIYEOK

0313B
HANGUL LETTER RIEUL-MIEUM

0313C
HANGUL LETTER RIEUL-PIEUP

0313D
HANGUL LETTER RIEUL-SIOS

0313E
HANGUL LETTER RIEUL-THIEUTH

0313F
HANGUL LETTER RIEUL-PHIEUPH

03140
HANGUL LETTER RIEUL-HIEUH

03141
HANGUL LETTER MIEUM

03142
HANGUL LETTER PIEUP

03143
HANGUL LETTER SSANGPIEUP

03144
HANGUL LETTER PIEUP-SIOS

03145
HANGUL LETTER SIOS

03146
HANGUL LETTER SSANGSIOS

03147
HANGUL LETTER IEUNG

03148
HANGUL LETTER CIEUC

03149
HANGUL LETTER SSANGCIEUC

0314A
HANGUL LETTER CHIEUCH

0314B
HANGUL LETTER KHIEUKH

0314C
HANGUL LETTER THIEUTH

0314D
HANGUL LETTER PHIEUPH

0314E
HANGUL LETTER HIEUH

0314F
HANGUL LETTER A

03150
HANGUL LETTER AE

03151
HANGUL LETTER YA

03152
HANGUL LETTER YAE

03153
HANGUL LETTER EO

03154
HANGUL LETTER E

03155
HANGUL LETTER YEO

03156
HANGUL LETTER YE

03157
HANGUL LETTER O

03158
HANGUL LETTER WA

03159
HANGUL LETTER WAE

0315A
HANGUL LETTER OE

0315B
HANGUL LETTER YO

0315C
HANGUL LETTER U

0315D
HANGUL LETTER WEO

0315E
HANGUL LETTER WE

0315F
HANGUL LETTER WI

03160
HANGUL LETTER YU

03161
HANGUL LETTER EU

03162
HANGUL LETTER YI

03163
HANGUL LETTER I

03164
HANGUL FILLER

03165
HANGUL LETTER SSANGNIEUN

03166
HANGUL LETTER NIEUN-TIKEUT

03167
HANGUL LETTER NIEUN-SIOS

03168
HANGUL LETTER NIEUN-PANSIOS

03169
HANGUL LETTER RIEUL-KIYEOK-SIOS

0316A
HANGUL LETTER RIEUL-TIKEUT

0316B
HANGUL LETTER RIEUL-PIEUP-SIOS

0316C
HANGUL LETTER RIEUL-PANSIOS

0316D
HANGUL LETTER RIEUL-YEORINHIEUH

0316E
HANGUL LETTER MIEUM-PIEUP

0316F
HANGUL LETTER MIEUM-SIOS

03170
HANGUL LETTER MIEUM-PANSIOS

03171
HANGUL LETTER KAPYEOUNMIEUM

03172
HANGUL LETTER PIEUP-KIYEOK

03173
HANGUL LETTER PIEUP-TIKEUT

03174
HANGUL LETTER PIEUP-SIOS-KIYEOK

03175
HANGUL LETTER PIEUP-SIOS-TIKEUT

03176
HANGUL LETTER PIEUP-CIEUC

03177
HANGUL LETTER PIEUP-THIEUTH

03178
HANGUL LETTER KAPYEOUNPIEUP

03179
HANGUL LETTER KAPYEOUNSSANGPIEUP

0317A
HANGUL LETTER SIOS-KIYEOK

0317B
HANGUL LETTER SIOS-NIEUN

0317C
HANGUL LETTER SIOS-TIKEUT

0317D
HANGUL LETTER SIOS-PIEUP

0317E
HANGUL LETTER SIOS-CIEUC

0317F
HANGUL LETTER PANSIOS

03180
HANGUL LETTER SSANGIEUNG

03181
HANGUL LETTER YESIEUNG

03182
HANGUL LETTER YESIEUNG-SIOS

03183
HANGUL LETTER YESIEUNG-PANSIOS

03184
HANGUL LETTER KAPYEOUNPHIEUPH

03185
HANGUL LETTER SSANGHIEUH

03186
HANGUL LETTER YEORINHIEUH

03187
HANGUL LETTER YO-YA

03188
HANGUL LETTER YO-YAE

03189
HANGUL LETTER YO-I

0318A
HANGUL LETTER YU-YEO

0318B
HANGUL LETTER YU-YE

0318C
HANGUL LETTER YU-I

0318D
HANGUL LETTER ARAEA

0318E
HANGUL LETTER ARAEAE

03190
IDEOGRAPHIC ANNOTATION LINKING MARK

03191
IDEOGRAPHIC ANNOTATION REVERSE MARK

03192
IDEOGRAPHIC ANNOTATION ONE MARK

03193
IDEOGRAPHIC ANNOTATION TWO MARK

03194
IDEOGRAPHIC ANNOTATION THREE MARK

03195
IDEOGRAPHIC ANNOTATION FOUR MARK

03196
IDEOGRAPHIC ANNOTATION TOP MARK

03197
IDEOGRAPHIC ANNOTATION MIDDLE MARK

03198
IDEOGRAPHIC ANNOTATION BOTTOM MARK

03199
IDEOGRAPHIC ANNOTATION FIRST MARK

0319A
IDEOGRAPHIC ANNOTATION SECOND MARK

0319B
IDEOGRAPHIC ANNOTATION THIRD MARK

0319C
IDEOGRAPHIC ANNOTATION FOURTH MARK

0319D
IDEOGRAPHIC ANNOTATION HEAVEN MARK

0319E
IDEOGRAPHIC ANNOTATION EARTH MARK

0319F
IDEOGRAPHIC ANNOTATION MAN MARK

031A0
BOPOMOFO LETTER BU

031A1
BOPOMOFO LETTER ZI

031A2
BOPOMOFO LETTER JI

031A3
BOPOMOFO LETTER GU

031A4
BOPOMOFO LETTER EE

031A5
BOPOMOFO LETTER ENN

031A6
BOPOMOFO LETTER OO

031A7
BOPOMOFO LETTER ONN

031A8
BOPOMOFO LETTER IR

031A9
BOPOMOFO LETTER ANN

031AA
BOPOMOFO LETTER INN

031AB
BOPOMOFO LETTER UNN

031AC
BOPOMOFO LETTER IM

031AD
BOPOMOFO LETTER NGG

031AE
BOPOMOFO LETTER AINN

031AF
BOPOMOFO LETTER AUNN

031B0
BOPOMOFO LETTER AM

031B1
BOPOMOFO LETTER OM

031B2
BOPOMOFO LETTER ONG

031B3
BOPOMOFO LETTER INNN

031B4
BOPOMOFO FINAL LETTER P

031B5
BOPOMOFO FINAL LETTER T

031B6
BOPOMOFO FINAL LETTER K

031B7
BOPOMOFO FINAL LETTER H

031B8
BOPOMOFO LETTER GH

031B9
BOPOMOFO LETTER LH

031BA
BOPOMOFO LETTER ZY

031BB
BOPOMOFO FINAL LETTER G

031BC
BOPOMOFO LETTER GW

031BD
BOPOMOFO LETTER KW

031BE
BOPOMOFO LETTER OE

031BF
BOPOMOFO LETTER AH

031C0
CJK STROKE T

031C1
CJK STROKE WG

031C2
CJK STROKE XG

031C3
CJK STROKE BXG

031C4
CJK STROKE SW

031C5
CJK STROKE HZZ

031C6
CJK STROKE HZG

031C7
CJK STROKE HP

031C8
CJK STROKE HZWG

031C9
CJK STROKE SZWG

031CA
CJK STROKE HZT

031CB
CJK STROKE HZZP

031CC
CJK STROKE HPWG

031CD
CJK STROKE HZW

031CE
CJK STROKE HZZZ

031CF
CJK STROKE N

031D0
CJK STROKE H

031D1
CJK STROKE S

031D2
CJK STROKE P

031D3
CJK STROKE SP

031D4
CJK STROKE D

031D5
CJK STROKE HZ

031D6
CJK STROKE HG

031D7
CJK STROKE SZ

031D8
CJK STROKE SWZ

031D9
CJK STROKE ST

031DA
CJK STROKE SG

031DB
CJK STROKE PD

031DC
CJK STROKE PZ

031DD
CJK STROKE TN

031DE
CJK STROKE SZZ

031DF
CJK STROKE SWG

031E0
CJK STROKE HXWG

031E1
CJK STROKE HZZZG

031E2
CJK STROKE PG

031E3
CJK STROKE Q

031E4
CJK STROKE HXG

031E5
CJK STROKE SZP

031EF
IDEOGRAPHIC DESCRIPTION CHARACTER SUBTRACTION

031F0
KATAKANA LETTER SMALL KU

031F1
KATAKANA LETTER SMALL SI

031F2
KATAKANA LETTER SMALL SU

031F3
KATAKANA LETTER SMALL TO

031F4
KATAKANA LETTER SMALL NU

031F5
KATAKANA LETTER SMALL HA

031F6
KATAKANA LETTER SMALL HI

031F7
KATAKANA LETTER SMALL HU

031F8
KATAKANA LETTER SMALL HE

031F9
KATAKANA LETTER SMALL HO

031FA
KATAKANA LETTER SMALL MU

031FB
KATAKANA LETTER SMALL RA

031FC
KATAKANA LETTER SMALL RI

031FD
KATAKANA LETTER SMALL RU

031FE
KATAKANA LETTER SMALL RE

031FF
KATAKANA LETTER SMALL RO

03200
PARENTHESIZED HANGUL KIYEOK

03201
PARENTHESIZED HANGUL NIEUN

03202
PARENTHESIZED HANGUL TIKEUT

03203
PARENTHESIZED HANGUL RIEUL

03204
PARENTHESIZED HANGUL MIEUM

03205
PARENTHESIZED HANGUL PIEUP

03206
PARENTHESIZED HANGUL SIOS

03207
PARENTHESIZED HANGUL IEUNG

03208
PARENTHESIZED HANGUL CIEUC

03209
PARENTHESIZED HANGUL CHIEUCH

0320A
PARENTHESIZED HANGUL KHIEUKH

0320B
PARENTHESIZED HANGUL THIEUTH

0320C
PARENTHESIZED HANGUL PHIEUPH

0320D
PARENTHESIZED HANGUL HIEUH

0320E
PARENTHESIZED HANGUL KIYEOK A

0320F
PARENTHESIZED HANGUL NIEUN A

03210
PARENTHESIZED HANGUL TIKEUT A

03211
PARENTHESIZED HANGUL RIEUL A

03212
PARENTHESIZED HANGUL MIEUM A

03213
PARENTHESIZED HANGUL PIEUP A

03214
PARENTHESIZED HANGUL SIOS A

03215
PARENTHESIZED HANGUL IEUNG A

03216
PARENTHESIZED HANGUL CIEUC A

03217
PARENTHESIZED HANGUL CHIEUCH A

03218
PARENTHESIZED HANGUL KHIEUKH A

03219
PARENTHESIZED HANGUL THIEUTH A

0321A
PARENTHESIZED HANGUL PHIEUPH A

0321B
PARENTHESIZED HANGUL HIEUH A

0321C
PARENTHESIZED HANGUL CIEUC U

0321D
PARENTHESIZED KOREAN CHARACTER OJEON

0321E
PARENTHESIZED KOREAN CHARACTER O HU

03220
PARENTHESIZED IDEOGRAPH ONE

03221
PARENTHESIZED IDEOGRAPH TWO

03222
PARENTHESIZED IDEOGRAPH THREE

03223
PARENTHESIZED IDEOGRAPH FOUR

03224
PARENTHESIZED IDEOGRAPH FIVE

03225
PARENTHESIZED IDEOGRAPH SIX

03226
PARENTHESIZED IDEOGRAPH SEVEN

03227
PARENTHESIZED IDEOGRAPH EIGHT

03228
PARENTHESIZED IDEOGRAPH NINE

03229
PARENTHESIZED IDEOGRAPH TEN

0322A
PARENTHESIZED IDEOGRAPH MOON

0322B
PARENTHESIZED IDEOGRAPH FIRE

0322C
PARENTHESIZED IDEOGRAPH WATER

0322D
PARENTHESIZED IDEOGRAPH WOOD

0322E
PARENTHESIZED IDEOGRAPH METAL

0322F
PARENTHESIZED IDEOGRAPH EARTH

03230
PARENTHESIZED IDEOGRAPH SUN

03231
PARENTHESIZED IDEOGRAPH STOCK

03232
PARENTHESIZED IDEOGRAPH HAVE

03233
PARENTHESIZED IDEOGRAPH SOCIETY

03234
PARENTHESIZED IDEOGRAPH NAME

03235
PARENTHESIZED IDEOGRAPH SPECIAL

03236
PARENTHESIZED IDEOGRAPH FINANCIAL

03237
PARENTHESIZED IDEOGRAPH CONGRATULATION

03238
PARENTHESIZED IDEOGRAPH LABOR

03239
PARENTHESIZED IDEOGRAPH REPRESENT

0323A
PARENTHESIZED IDEOGRAPH CALL

0323B
PARENTHESIZED IDEOGRAPH STUDY

0323C
PARENTHESIZED IDEOGRAPH SUPERVISE

0323D
PARENTHESIZED IDEOGRAPH ENTERPRISE

0323E
PARENTHESIZED IDEOGRAPH RESOURCE

0323F
PARENTHESIZED IDEOGRAPH ALLIANCE

03240
PARENTHESIZED IDEOGRAPH FESTIVAL

03241
PARENTHESIZED IDEOGRAPH REST

03242
PARENTHESIZED IDEOGRAPH SELF

03243
PARENTHESIZED IDEOGRAPH REACH

03244
CIRCLED IDEOGRAPH QUESTION

03245
CIRCLED IDEOGRAPH KINDERGARTEN

03246
CIRCLED IDEOGRAPH SCHOOL

03247
CIRCLED IDEOGRAPH KOTO

03248
CIRCLED NUMBER TEN ON BLACK SQUARE

03249
CIRCLED NUMBER TWENTY ON BLACK SQUARE

0324A
CIRCLED NUMBER THIRTY ON BLACK SQUARE

0324B
CIRCLED NUMBER FORTY ON BLACK SQUARE

0324C
CIRCLED NUMBER FIFTY ON BLACK SQUARE

0324D
CIRCLED NUMBER SIXTY ON BLACK SQUARE

0324E
CIRCLED NUMBER SEVENTY ON BLACK SQUARE

0324F
CIRCLED NUMBER EIGHTY ON BLACK SQUARE

03250
PARTNERSHIP SIGN

03251
CIRCLED NUMBER TWENTY ONE

03252
CIRCLED NUMBER TWENTY TWO

03253
CIRCLED NUMBER TWENTY THREE

03254
CIRCLED NUMBER TWENTY FOUR

03255
CIRCLED NUMBER TWENTY FIVE

03256
CIRCLED NUMBER TWENTY SIX

03257
CIRCLED NUMBER TWENTY SEVEN

03258
CIRCLED NUMBER TWENTY EIGHT

03259
CIRCLED NUMBER TWENTY NINE

0325A
CIRCLED NUMBER THIRTY

0325B
CIRCLED NUMBER THIRTY ONE

0325C
CIRCLED NUMBER THIRTY TWO

0325D
CIRCLED NUMBER THIRTY THREE

0325E
CIRCLED NUMBER THIRTY FOUR

0325F
CIRCLED NUMBER THIRTY FIVE

03260
CIRCLED HANGUL KIYEOK

03261
CIRCLED HANGUL NIEUN

03262
CIRCLED HANGUL TIKEUT

03263
CIRCLED HANGUL RIEUL

03264
CIRCLED HANGUL MIEUM

03265
CIRCLED HANGUL PIEUP

03266
CIRCLED HANGUL SIOS

03267
CIRCLED HANGUL IEUNG

03268
CIRCLED HANGUL CIEUC

03269
CIRCLED HANGUL CHIEUCH

0326A
CIRCLED HANGUL KHIEUKH

0326B
CIRCLED HANGUL THIEUTH

0326C
CIRCLED HANGUL PHIEUPH

0326D
CIRCLED HANGUL HIEUH

0326E
CIRCLED HANGUL KIYEOK A

0326F
CIRCLED HANGUL NIEUN A

03270
CIRCLED HANGUL TIKEUT A

03271
CIRCLED HANGUL RIEUL A

03272
CIRCLED HANGUL MIEUM A

03273
CIRCLED HANGUL PIEUP A

03274
CIRCLED HANGUL SIOS A

03275
CIRCLED HANGUL IEUNG A

03276
CIRCLED HANGUL CIEUC A

03277
CIRCLED HANGUL CHIEUCH A

03278
CIRCLED HANGUL KHIEUKH A

03279
CIRCLED HANGUL THIEUTH A

0327A
CIRCLED HANGUL PHIEUPH A

0327B
CIRCLED HANGUL HIEUH A

0327C
CIRCLED KOREAN CHARACTER CHAMKO

0327D
CIRCLED KOREAN CHARACTER JUEUI

0327E
CIRCLED HANGUL IEUNG U

0327F
KOREAN STANDARD SYMBOL

03280
CIRCLED IDEOGRAPH ONE

03281
CIRCLED IDEOGRAPH TWO

03282
CIRCLED IDEOGRAPH THREE

03283
CIRCLED IDEOGRAPH FOUR

03284
CIRCLED IDEOGRAPH FIVE

03285
CIRCLED IDEOGRAPH SIX

03286
CIRCLED IDEOGRAPH SEVEN

03287
CIRCLED IDEOGRAPH EIGHT

03288
CIRCLED IDEOGRAPH NINE

03289
CIRCLED IDEOGRAPH TEN

0328A
CIRCLED IDEOGRAPH MOON

0328B
CIRCLED IDEOGRAPH FIRE

0328C
CIRCLED IDEOGRAPH WATER

0328D
CIRCLED IDEOGRAPH WOOD

0328E
CIRCLED IDEOGRAPH METAL

0328F
CIRCLED IDEOGRAPH EARTH

03290
CIRCLED IDEOGRAPH SUN

03291
CIRCLED IDEOGRAPH STOCK

03292
CIRCLED IDEOGRAPH HAVE

03293
CIRCLED IDEOGRAPH SOCIETY

03294
CIRCLED IDEOGRAPH NAME

03295
CIRCLED IDEOGRAPH SPECIAL

03296
CIRCLED IDEOGRAPH FINANCIAL

03297
CIRCLED IDEOGRAPH CONGRATULATION

03298
CIRCLED IDEOGRAPH LABOR

03299
CIRCLED IDEOGRAPH SECRET

0329A
CIRCLED IDEOGRAPH MALE

0329B
CIRCLED IDEOGRAPH FEMALE

0329C
CIRCLED IDEOGRAPH SUITABLE

0329D
CIRCLED IDEOGRAPH EXCELLENT

0329E
CIRCLED IDEOGRAPH PRINT

0329F
CIRCLED IDEOGRAPH ATTENTION

032A0
CIRCLED IDEOGRAPH ITEM

032A1
CIRCLED IDEOGRAPH REST

032A2
CIRCLED IDEOGRAPH COPY

032A3
CIRCLED IDEOGRAPH CORRECT

032A4
CIRCLED IDEOGRAPH HIGH

032A5
CIRCLED IDEOGRAPH CENTRE

032A6
CIRCLED IDEOGRAPH LOW

032A7
CIRCLED IDEOGRAPH LEFT

032A8
CIRCLED IDEOGRAPH RIGHT

032A9
CIRCLED IDEOGRAPH MEDICINE

032AA
CIRCLED IDEOGRAPH RELIGION

032AB
CIRCLED IDEOGRAPH STUDY

032AC
CIRCLED IDEOGRAPH SUPERVISE

032AD
CIRCLED IDEOGRAPH ENTERPRISE

032AE
CIRCLED IDEOGRAPH RESOURCE

032AF
CIRCLED IDEOGRAPH ALLIANCE

032B0
CIRCLED IDEOGRAPH NIGHT

032B1
CIRCLED NUMBER THIRTY SIX

032B2
CIRCLED NUMBER THIRTY SEVEN

032B3
CIRCLED NUMBER THIRTY EIGHT

032B4
CIRCLED NUMBER THIRTY NINE

032B5
CIRCLED NUMBER FORTY

032B6
CIRCLED NUMBER FORTY ONE

032B7
CIRCLED NUMBER FORTY TWO

032B8
CIRCLED NUMBER FORTY THREE

032B9
CIRCLED NUMBER FORTY FOUR

032BA
CIRCLED NUMBER FORTY FIVE

032BB
CIRCLED NUMBER FORTY SIX

032BC
CIRCLED NUMBER FORTY SEVEN

032BD
CIRCLED NUMBER FORTY EIGHT

032BE
CIRCLED NUMBER FORTY NINE

032BF
CIRCLED NUMBER FIFTY

032C0
IDEOGRAPHIC TELEGRAPH SYMBOL FOR JANUARY

032C1
IDEOGRAPHIC TELEGRAPH SYMBOL FOR FEBRUARY

032C2
IDEOGRAPHIC TELEGRAPH SYMBOL FOR MARCH

032C3
IDEOGRAPHIC TELEGRAPH SYMBOL FOR APRIL

032C4
IDEOGRAPHIC TELEGRAPH SYMBOL FOR MAY

032C5
IDEOGRAPHIC TELEGRAPH SYMBOL FOR JUNE

032C6
IDEOGRAPHIC TELEGRAPH SYMBOL FOR JULY

032C7
IDEOGRAPHIC TELEGRAPH SYMBOL FOR AUGUST

032C8
IDEOGRAPHIC TELEGRAPH SYMBOL FOR SEPTEMBER

032C9
IDEOGRAPHIC TELEGRAPH SYMBOL FOR OCTOBER

032CA
IDEOGRAPHIC TELEGRAPH SYMBOL FOR NOVEMBER

032CB
IDEOGRAPHIC TELEGRAPH SYMBOL FOR DECEMBER

032CC
SQUARE HG

032CD
SQUARE ERG

032CE
SQUARE EV

032CF
LIMITED LIABILITY SIGN

032D0
CIRCLED KATAKANA A

032D1
CIRCLED KATAKANA I

032D2
CIRCLED KATAKANA U

032D3
CIRCLED KATAKANA E

032D4
CIRCLED KATAKANA O

032D5
CIRCLED KATAKANA KA

032D6
CIRCLED KATAKANA KI

032D7
CIRCLED KATAKANA KU

032D8
CIRCLED KATAKANA KE

032D9
CIRCLED KATAKANA KO

032DA
CIRCLED KATAKANA SA

032DB
CIRCLED KATAKANA SI

032DC
CIRCLED KATAKANA SU

032DD
CIRCLED KATAKANA SE

032DE
CIRCLED KATAKANA SO

032DF
CIRCLED KATAKANA TA

032E0
CIRCLED KATAKANA TI

032E1
CIRCLED KATAKANA TU

032E2
CIRCLED KATAKANA TE

032E3
CIRCLED KATAKANA TO

032E4
CIRCLED KATAKANA NA

032E5
CIRCLED KATAKANA NI

032E6
CIRCLED KATAKANA NU

032E7
CIRCLED KATAKANA NE

032E8
CIRCLED KATAKANA NO

032E9
CIRCLED KATAKANA HA

032EA
CIRCLED KATAKANA HI

032EB
CIRCLED KATAKANA HU

032EC
CIRCLED KATAKANA HE

032ED
CIRCLED KATAKANA HO

032EE
CIRCLED KATAKANA MA

032EF
CIRCLED KATAKANA MI

032F0
CIRCLED KATAKANA MU

032F1
CIRCLED KATAKANA ME

032F2
CIRCLED KATAKANA MO

032F3
CIRCLED KATAKANA YA

032F4
CIRCLED KATAKANA YU

032F5
CIRCLED KATAKANA YO

032F6
CIRCLED KATAKANA RA

032F7
CIRCLED KATAKANA RI

032F8
CIRCLED KATAKANA RU

032F9
CIRCLED KATAKANA RE

032FA
CIRCLED KATAKANA RO

032FB
CIRCLED KATAKANA WA

032FC
CIRCLED KATAKANA WI

032FD
CIRCLED KATAKANA WE

032FE
CIRCLED KATAKANA WO

032FF
SQUARE ERA NAME REIWA

03300
SQUARE APAATO

03301
SQUARE ARUHUA

03302
SQUARE ANPEA

03303
SQUARE AARU

03304
SQUARE ININGU

03305
SQUARE INTI

03306
SQUARE UON

03307
SQUARE ESUKUUDO

03308
SQUARE EEKAA

03309
SQUARE ONSU

0330A
SQUARE OOMU

0330B
SQUARE KAIRI

0330C
SQUARE KARATTO

0330D
SQUARE KARORII

0330E
SQUARE GARON

0330F
SQUARE GANMA

03310
SQUARE GIGA

03311
SQUARE GINII

03312
SQUARE KYURII

03313
SQUARE GIRUDAA

03314
SQUARE KIRO

03315
SQUARE KIROGURAMU

03316
SQUARE KIROMEETORU

03317
SQUARE KIROWATTO

03318
SQUARE GURAMU

03319
SQUARE GURAMUTON

0331A
SQUARE KURUZEIRO

0331B
SQUARE KUROONE

0331C
SQUARE KEESU

0331D
SQUARE KORUNA

0331E
SQUARE KOOPO

0331F
SQUARE SAIKURU

03320
SQUARE SANTIIMU

03321
SQUARE SIRINGU

03322
SQUARE SENTI

03323
SQUARE SENTO

03324
SQUARE DAASU

03325
SQUARE DESI

03326
SQUARE DORU

03327
SQUARE TON

03328
SQUARE NANO

03329
SQUARE NOTTO

0332A
SQUARE HAITU

0332B
SQUARE PAASENTO

0332C
SQUARE PAATU

0332D
SQUARE BAARERU

0332E
SQUARE PIASUTORU

0332F
SQUARE PIKURU

03330
SQUARE PIKO

03331
SQUARE BIRU

03332
SQUARE HUARADDO

03333
SQUARE HUIITO

03334
SQUARE BUSSYERU

03335
SQUARE HURAN

03336
SQUARE HEKUTAARU

03337
SQUARE PESO

03338
SQUARE PENIHI

03339
SQUARE HERUTU

0333A
SQUARE PENSU

0333B
SQUARE PEEZI

0333C
SQUARE BEETA

0333D
SQUARE POINTO

0333E
SQUARE BORUTO

0333F
SQUARE HON

03340
SQUARE PONDO

03341
SQUARE HOORU

03342
SQUARE HOON

03343
SQUARE MAIKURO

03344
SQUARE MAIRU

03345
SQUARE MAHHA

03346
SQUARE MARUKU

03347
SQUARE MANSYON

03348
SQUARE MIKURON

03349
SQUARE MIRI

0334A
SQUARE MIRIBAARU

0334B
SQUARE MEGA

0334C
SQUARE MEGATON

0334D
SQUARE MEETORU

0334E
SQUARE YAADO

0334F
SQUARE YAARU

03350
SQUARE YUAN

03351
SQUARE RITTORU

03352
SQUARE RIRA

03353
SQUARE RUPII

03354
SQUARE RUUBURU

03355
SQUARE REMU

03356
SQUARE RENTOGEN

03357
SQUARE WATTO

03358
IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR ZERO

03359
IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR ONE

0335A
IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TWO

0335B
IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR THREE

0335C
IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR FOUR

0335D
IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR FIVE

0335E
IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR SIX

0335F
IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR SEVEN

03360
IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR EIGHT

03361
IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR NINE

03362
IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TEN

03363
IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR ELEVEN

03364
IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TWELVE

03365
IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR THIRTEEN

03366
IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR FOURTEEN

03367
IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR FIFTEEN

03368
IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR SIXTEEN

03369
IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR SEVENTEEN

0336A
IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR EIGHTEEN

0336B
IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR NINETEEN

0336C
IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TWENTY

0336D
IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TWENTY-ONE

0336E
IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TWENTY-TWO

0336F
IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TWENTY-THREE

03370
IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TWENTY-FOUR

03371
SQUARE HPA

03372
SQUARE DA

03373
SQUARE AU

03374
SQUARE BAR

03375
SQUARE OV

03376
SQUARE PC

03377
SQUARE DM

03378
SQUARE DM SQUARED

03379
SQUARE DM CUBED

0337A
SQUARE IU

0337B
SQUARE ERA NAME HEISEI

0337C
SQUARE ERA NAME SYOUWA

0337D
SQUARE ERA NAME TAISYOU

0337E
SQUARE ERA NAME MEIZI

0337F
SQUARE CORPORATION

03380
SQUARE PA AMPS

03381
SQUARE NA

03382
SQUARE MU A

03383
SQUARE MA

03384
SQUARE KA

03385
SQUARE KB

03386
SQUARE MB

03387
SQUARE GB

03388
SQUARE CAL

03389
SQUARE KCAL

0338A
SQUARE PF

0338B
SQUARE NF

0338C
SQUARE MU F

0338D
SQUARE MU G

0338E
SQUARE MG

0338F
SQUARE KG

03390
SQUARE HZ

03391
SQUARE KHZ

03392
SQUARE MHZ

03393
SQUARE GHZ

03394
SQUARE THZ

03395
SQUARE MU L

03396
SQUARE ML

03397
SQUARE DL

03398
SQUARE KL

03399
SQUARE FM

0339A
SQUARE NM

0339B
SQUARE MU M

0339C
SQUARE MM

0339D
SQUARE CM

0339E
SQUARE KM

0339F
SQUARE MM SQUARED

033A0
SQUARE CM SQUARED

033A1
SQUARE M SQUARED

033A2
SQUARE KM SQUARED

033A3
SQUARE MM CUBED

033A4
SQUARE CM CUBED

033A5
SQUARE M CUBED

033A6
SQUARE KM CUBED

033A7
SQUARE M OVER S

033A8
SQUARE M OVER S SQUARED

033A9
SQUARE PA

033AA
SQUARE KPA

033AB
SQUARE MPA

033AC
SQUARE GPA

033AD
SQUARE RAD

033AE
SQUARE RAD OVER S

033AF
SQUARE RAD OVER S SQUARED

033B0
SQUARE PS

033B1
SQUARE NS

033B2
SQUARE MU S

033B3
SQUARE MS

033B4
SQUARE PV

033B5
SQUARE NV

033B6
SQUARE MU V

033B7
SQUARE MV

033B8
SQUARE KV

033B9
SQUARE MV MEGA

033BA
SQUARE PW

033BB
SQUARE NW

033BC
SQUARE MU W

033BD
SQUARE MW

033BE
SQUARE KW

033BF
SQUARE MW MEGA

033C0
SQUARE K OHM

033C1
SQUARE M OHM

033C2
SQUARE AM

033C3
SQUARE BQ

033C4
SQUARE CC

033C5
SQUARE CD

033C6
SQUARE C OVER KG

033C7
SQUARE CO

033C8
SQUARE DB

033C9
SQUARE GY

033CA
SQUARE HA

033CB
SQUARE HP

033CC
SQUARE IN

033CD
SQUARE KK

033CE
SQUARE KM CAPITAL

033CF
SQUARE KT

033D0
SQUARE LM

033D1
SQUARE LN

033D2
SQUARE LOG

033D3
SQUARE LX

033D4
SQUARE MB SMALL

033D5
SQUARE MIL

033D6
SQUARE MOL

033D7
SQUARE PH

033D8
SQUARE PM

033D9
SQUARE PPM

033DA
SQUARE PR

033DB
SQUARE SR

033DC
SQUARE SV

033DD
SQUARE WB

033DE
SQUARE V OVER M

033DF
SQUARE A OVER M

033E0
IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY ONE

033E1
IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWO

033E2
IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY THREE

033E3
IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY FOUR

033E4
IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY FIVE

033E5
IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY SIX

033E6
IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY SEVEN

033E7
IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY EIGHT

033E8
IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY NINE

033E9
IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TEN

033EA
IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY ELEVEN

033EB
IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWELVE

033EC
IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY THIRTEEN

033ED
IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY FOURTEEN

033EE
IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY FIFTEEN

033EF
IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY SIXTEEN

033F0
IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY SEVENTEEN

033F1
IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY EIGHTEEN

033F2
IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY NINETEEN

033F3
IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY

033F4
IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-ONE

033F5
IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-TWO

033F6
IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-THREE

033F7
IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-FOUR

033F8
IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-FIVE

033F9
IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-SIX

033FA
IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-SEVEN

033FB
IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-EIGHT

033FC
IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-NINE

033FD
IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY THIRTY

033FE
IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY THIRTY-ONE

033FF
SQUARE GAL

04DC0
HEXAGRAM FOR THE CREATIVE HEAVEN

04DC1
HEXAGRAM FOR THE RECEPTIVE EARTH

04DC2
HEXAGRAM FOR DIFFICULTY AT THE BEGINNING

04DC3
HEXAGRAM FOR YOUTHFUL FOLLY

04DC4
HEXAGRAM FOR WAITING

04DC5
HEXAGRAM FOR CONFLICT

04DC6
HEXAGRAM FOR THE ARMY

04DC7
HEXAGRAM FOR HOLDING TOGETHER

04DC8
HEXAGRAM FOR SMALL TAMING

04DC9
HEXAGRAM FOR TREADING

04DCA
HEXAGRAM FOR PEACE

04DCB
HEXAGRAM FOR STANDSTILL

04DCC
HEXAGRAM FOR FELLOWSHIP

04DCD
HEXAGRAM FOR GREAT POSSESSION

04DCE
HEXAGRAM FOR MODESTY

04DCF
HEXAGRAM FOR ENTHUSIASM

04DD0
HEXAGRAM FOR FOLLOWING

04DD1
HEXAGRAM FOR WORK ON THE DECAYED

04DD2
HEXAGRAM FOR APPROACH

04DD3
HEXAGRAM FOR CONTEMPLATION

04DD4
HEXAGRAM FOR BITING THROUGH

04DD5
HEXAGRAM FOR GRACE

04DD6
HEXAGRAM FOR SPLITTING APART

04DD7
HEXAGRAM FOR RETURN

04DD8
HEXAGRAM FOR INNOCENCE

04DD9
HEXAGRAM FOR GREAT TAMING

04DDA
HEXAGRAM FOR MOUTH CORNERS

04DDB
HEXAGRAM FOR GREAT PREPONDERANCE

04DDC
HEXAGRAM FOR THE ABYSMAL WATER

04DDD
HEXAGRAM FOR THE CLINGING FIRE

04DDE
HEXAGRAM FOR INFLUENCE

04DDF
HEXAGRAM FOR DURATION

04DE0
HEXAGRAM FOR RETREAT

04DE1
HEXAGRAM FOR GREAT POWER

04DE2
HEXAGRAM FOR PROGRESS

04DE3
HEXAGRAM FOR DARKENING OF THE LIGHT

04DE4
HEXAGRAM FOR THE FAMILY

04DE5
HEXAGRAM FOR OPPOSITION

04DE6
HEXAGRAM FOR OBSTRUCTION

04DE7
HEXAGRAM FOR DELIVERANCE

04DE8
HEXAGRAM FOR DECREASE

04DE9
HEXAGRAM FOR INCREASE

04DEA
HEXAGRAM FOR BREAKTHROUGH

04DEB
HEXAGRAM FOR COMING TO MEET

04DEC
HEXAGRAM FOR GATHERING TOGETHER

04DED
HEXAGRAM FOR PUSHING UPWARD

04DEE
HEXAGRAM FOR OPPRESSION

04DEF
HEXAGRAM FOR THE WELL

04DF0
HEXAGRAM FOR REVOLUTION

04DF1
HEXAGRAM FOR THE CAULDRON

04DF2
HEXAGRAM FOR THE AROUSING THUNDER

04DF3
HEXAGRAM FOR THE KEEPING STILL MOUNTAIN

04DF4
HEXAGRAM FOR DEVELOPMENT

04DF5
HEXAGRAM FOR THE MARRYING MAIDEN

04DF6
HEXAGRAM FOR ABUNDANCE

04DF7
HEXAGRAM FOR THE WANDERER

04DF8
HEXAGRAM FOR THE GENTLE WIND

04DF9
HEXAGRAM FOR THE JOYOUS LAKE

04DFA
HEXAGRAM FOR DISPERSION

04DFB
HEXAGRAM FOR LIMITATION

04DFC
HEXAGRAM FOR INNER TRUTH

04DFD
HEXAGRAM FOR SMALL PREPONDERANCE

04DFE
HEXAGRAM FOR AFTER COMPLETION

04DFF
HEXAGRAM FOR BEFORE COMPLETION

0A000
YI SYLLABLE IT

0A001
YI SYLLABLE IX

0A002
YI SYLLABLE I

0A003
YI SYLLABLE IP

0A004
YI SYLLABLE IET

0A005
YI SYLLABLE IEX

0A006
YI SYLLABLE IE

0A007
YI SYLLABLE IEP

0A008
YI SYLLABLE AT

0A009
YI SYLLABLE AX

0A00A
YI SYLLABLE A

0A00B
YI SYLLABLE AP

0A00C
YI SYLLABLE UOX

0A00D
YI SYLLABLE UO

0A00E
YI SYLLABLE UOP

0A00F
YI SYLLABLE OT

0A010
YI SYLLABLE OX

0A011
YI SYLLABLE O

0A012
YI SYLLABLE OP

0A013
YI SYLLABLE EX

0A014
YI SYLLABLE E

0A015
YI SYLLABLE ITERATION MARK

0A015
YI SYLLABLE WU

0A016
YI SYLLABLE BIT

0A017
YI SYLLABLE BIX

0A018
YI SYLLABLE BI

0A019
YI SYLLABLE BIP

0A01A
YI SYLLABLE BIET

0A01B
YI SYLLABLE BIEX

0A01C
YI SYLLABLE BIE

0A01D
YI SYLLABLE BIEP

0A01E
YI SYLLABLE BAT

0A01F
YI SYLLABLE BAX

0A020
YI SYLLABLE BA

0A021
YI SYLLABLE BAP

0A022
YI SYLLABLE BUOX

0A023
YI SYLLABLE BUO

0A024
YI SYLLABLE BUOP

0A025
YI SYLLABLE BOT

0A026
YI SYLLABLE BOX

0A027
YI SYLLABLE BO

0A028
YI SYLLABLE BOP

0A029
YI SYLLABLE BEX

0A02A
YI SYLLABLE BE

0A02B
YI SYLLABLE BEP

0A02C
YI SYLLABLE BUT

0A02D
YI SYLLABLE BUX

0A02E
YI SYLLABLE BU

0A02F
YI SYLLABLE BUP

0A030
YI SYLLABLE BURX

0A031
YI SYLLABLE BUR

0A032
YI SYLLABLE BYT

0A033
YI SYLLABLE BYX

0A034
YI SYLLABLE BY

0A035
YI SYLLABLE BYP

0A036
YI SYLLABLE BYRX

0A037
YI SYLLABLE BYR

0A038
YI SYLLABLE PIT

0A039
YI SYLLABLE PIX

0A03A
YI SYLLABLE PI

0A03B
YI SYLLABLE PIP

0A03C
YI SYLLABLE PIEX

0A03D
YI SYLLABLE PIE

0A03E
YI SYLLABLE PIEP

0A03F
YI SYLLABLE PAT

0A040
YI SYLLABLE PAX

0A041
YI SYLLABLE PA

0A042
YI SYLLABLE PAP

0A043
YI SYLLABLE PUOX

0A044
YI SYLLABLE PUO

0A045
YI SYLLABLE PUOP

0A046
YI SYLLABLE POT

0A047
YI SYLLABLE POX

0A048
YI SYLLABLE PO

0A049
YI SYLLABLE POP

0A04A
YI SYLLABLE PUT

0A04B
YI SYLLABLE PUX

0A04C
YI SYLLABLE PU

0A04D
YI SYLLABLE PUP

0A04E
YI SYLLABLE PURX

0A04F
YI SYLLABLE PUR

0A050
YI SYLLABLE PYT

0A051
YI SYLLABLE PYX

0A052
YI SYLLABLE PY

0A053
YI SYLLABLE PYP

0A054
YI SYLLABLE PYRX

0A055
YI SYLLABLE PYR

0A056
YI SYLLABLE BBIT

0A057
YI SYLLABLE BBIX

0A058
YI SYLLABLE BBI

0A059
YI SYLLABLE BBIP

0A05A
YI SYLLABLE BBIET

0A05B
YI SYLLABLE BBIEX

0A05C
YI SYLLABLE BBIE

0A05D
YI SYLLABLE BBIEP

0A05E
YI SYLLABLE BBAT

0A05F
YI SYLLABLE BBAX

0A060
YI SYLLABLE BBA

0A061
YI SYLLABLE BBAP

0A062
YI SYLLABLE BBUOX

0A063
YI SYLLABLE BBUO

0A064
YI SYLLABLE BBUOP

0A065
YI SYLLABLE BBOT

0A066
YI SYLLABLE BBOX

0A067
YI SYLLABLE BBO

0A068
YI SYLLABLE BBOP

0A069
YI SYLLABLE BBEX

0A06A
YI SYLLABLE BBE

0A06B
YI SYLLABLE BBEP

0A06C
YI SYLLABLE BBUT

0A06D
YI SYLLABLE BBUX

0A06E
YI SYLLABLE BBU

0A06F
YI SYLLABLE BBUP

0A070
YI SYLLABLE BBURX

0A071
YI SYLLABLE BBUR

0A072
YI SYLLABLE BBYT

0A073
YI SYLLABLE BBYX

0A074
YI SYLLABLE BBY

0A075
YI SYLLABLE BBYP

0A076
YI SYLLABLE NBIT

0A077
YI SYLLABLE NBIX

0A078
YI SYLLABLE NBI

0A079
YI SYLLABLE NBIP

0A07A
YI SYLLABLE NBIEX

0A07B
YI SYLLABLE NBIE

0A07C
YI SYLLABLE NBIEP

0A07D
YI SYLLABLE NBAT

0A07E
YI SYLLABLE NBAX

0A07F
YI SYLLABLE NBA

0A080
YI SYLLABLE NBAP

0A081
YI SYLLABLE NBOT

0A082
YI SYLLABLE NBOX

0A083
YI SYLLABLE NBO

0A084
YI SYLLABLE NBOP

0A085
YI SYLLABLE NBUT

0A086
YI SYLLABLE NBUX

0A087
YI SYLLABLE NBU

0A088
YI SYLLABLE NBUP

0A089
YI SYLLABLE NBURX

0A08A
YI SYLLABLE NBUR

0A08B
YI SYLLABLE NBYT

0A08C
YI SYLLABLE NBYX

0A08D
YI SYLLABLE NBY

0A08E
YI SYLLABLE NBYP

0A08F
YI SYLLABLE NBYRX

0A090
YI SYLLABLE NBYR

0A091
YI SYLLABLE HMIT

0A092
YI SYLLABLE HMIX

0A093
YI SYLLABLE HMI

0A094
YI SYLLABLE HMIP

0A095
YI SYLLABLE HMIEX

0A096
YI SYLLABLE HMIE

0A097
YI SYLLABLE HMIEP

0A098
YI SYLLABLE HMAT

0A099
YI SYLLABLE HMAX

0A09A
YI SYLLABLE HMA

0A09B
YI SYLLABLE HMAP

0A09C
YI SYLLABLE HMUOX

0A09D
YI SYLLABLE HMUO

0A09E
YI SYLLABLE HMUOP

0A09F
YI SYLLABLE HMOT

0A0A0
YI SYLLABLE HMOX

0A0A1
YI SYLLABLE HMO

0A0A2
YI SYLLABLE HMOP

0A0A3
YI SYLLABLE HMUT

0A0A4
YI SYLLABLE HMUX

0A0A5
YI SYLLABLE HMU

0A0A6
YI SYLLABLE HMUP

0A0A7
YI SYLLABLE HMURX

0A0A8
YI SYLLABLE HMUR

0A0A9
YI SYLLABLE HMYX

0A0AA
YI SYLLABLE HMY

0A0AB
YI SYLLABLE HMYP

0A0AC
YI SYLLABLE HMYRX

0A0AD
YI SYLLABLE HMYR

0A0AE
YI SYLLABLE MIT

0A0AF
YI SYLLABLE MIX

0A0B0
YI SYLLABLE MI

0A0B1
YI SYLLABLE MIP

0A0B2
YI SYLLABLE MIEX

0A0B3
YI SYLLABLE MIE

0A0B4
YI SYLLABLE MIEP

0A0B5
YI SYLLABLE MAT

0A0B6
YI SYLLABLE MAX

0A0B7
YI SYLLABLE MA

0A0B8
YI SYLLABLE MAP

0A0B9
YI SYLLABLE MUOT

0A0BA
YI SYLLABLE MUOX

0A0BB
YI SYLLABLE MUO

0A0BC
YI SYLLABLE MUOP

0A0BD
YI SYLLABLE MOT

0A0BE
YI SYLLABLE MOX

0A0BF
YI SYLLABLE MO

0A0C0
YI SYLLABLE MOP

0A0C1
YI SYLLABLE MEX

0A0C2
YI SYLLABLE ME

0A0C3
YI SYLLABLE MUT

0A0C4
YI SYLLABLE MUX

0A0C5
YI SYLLABLE MU

0A0C6
YI SYLLABLE MUP

0A0C7
YI SYLLABLE MURX

0A0C8
YI SYLLABLE MUR

0A0C9
YI SYLLABLE MYT

0A0CA
YI SYLLABLE MYX

0A0CB
YI SYLLABLE MY

0A0CC
YI SYLLABLE MYP

0A0CD
YI SYLLABLE FIT

0A0CE
YI SYLLABLE FIX

0A0CF
YI SYLLABLE FI

0A0D0
YI SYLLABLE FIP

0A0D1
YI SYLLABLE FAT

0A0D2
YI SYLLABLE FAX

0A0D3
YI SYLLABLE FA

0A0D4
YI SYLLABLE FAP

0A0D5
YI SYLLABLE FOX

0A0D6
YI SYLLABLE FO

0A0D7
YI SYLLABLE FOP

0A0D8
YI SYLLABLE FUT

0A0D9
YI SYLLABLE FUX

0A0DA
YI SYLLABLE FU

0A0DB
YI SYLLABLE FUP

0A0DC
YI SYLLABLE FURX

0A0DD
YI SYLLABLE FUR

0A0DE
YI SYLLABLE FYT

0A0DF
YI SYLLABLE FYX

0A0E0
YI SYLLABLE FY

0A0E1
YI SYLLABLE FYP

0A0E2
YI SYLLABLE VIT

0A0E3
YI SYLLABLE VIX

0A0E4
YI SYLLABLE VI

0A0E5
YI SYLLABLE VIP

0A0E6
YI SYLLABLE VIET

0A0E7
YI SYLLABLE VIEX

0A0E8
YI SYLLABLE VIE

0A0E9
YI SYLLABLE VIEP

0A0EA
YI SYLLABLE VAT

0A0EB
YI SYLLABLE VAX

0A0EC
YI SYLLABLE VA

0A0ED
YI SYLLABLE VAP

0A0EE
YI SYLLABLE VOT

0A0EF
YI SYLLABLE VOX

0A0F0
YI SYLLABLE VO

0A0F1
YI SYLLABLE VOP

0A0F2
YI SYLLABLE VEX

0A0F3
YI SYLLABLE VEP

0A0F4
YI SYLLABLE VUT

0A0F5
YI SYLLABLE VUX

0A0F6
YI SYLLABLE VU

0A0F7
YI SYLLABLE VUP

0A0F8
YI SYLLABLE VURX

0A0F9
YI SYLLABLE VUR

0A0FA
YI SYLLABLE VYT

0A0FB
YI SYLLABLE VYX

0A0FC
YI SYLLABLE VY

0A0FD
YI SYLLABLE VYP

0A0FE
YI SYLLABLE VYRX

0A0FF
YI SYLLABLE VYR

0A100
YI SYLLABLE DIT

0A101
YI SYLLABLE DIX

0A102
YI SYLLABLE DI

0A103
YI SYLLABLE DIP

0A104
YI SYLLABLE DIEX

0A105
YI SYLLABLE DIE

0A106
YI SYLLABLE DIEP

0A107
YI SYLLABLE DAT

0A108
YI SYLLABLE DAX

0A109
YI SYLLABLE DA

0A10A
YI SYLLABLE DAP

0A10B
YI SYLLABLE DUOX

0A10C
YI SYLLABLE DUO

0A10D
YI SYLLABLE DOT

0A10E
YI SYLLABLE DOX

0A10F
YI SYLLABLE DO

0A110
YI SYLLABLE DOP

0A111
YI SYLLABLE DEX

0A112
YI SYLLABLE DE

0A113
YI SYLLABLE DEP

0A114
YI SYLLABLE DUT

0A115
YI SYLLABLE DUX

0A116
YI SYLLABLE DU

0A117
YI SYLLABLE DUP

0A118
YI SYLLABLE DURX

0A119
YI SYLLABLE DUR

0A11A
YI SYLLABLE TIT

0A11B
YI SYLLABLE TIX

0A11C
YI SYLLABLE TI

0A11D
YI SYLLABLE TIP

0A11E
YI SYLLABLE TIEX

0A11F
YI SYLLABLE TIE

0A120
YI SYLLABLE TIEP

0A121
YI SYLLABLE TAT

0A122
YI SYLLABLE TAX

0A123
YI SYLLABLE TA

0A124
YI SYLLABLE TAP

0A125
YI SYLLABLE TUOT

0A126
YI SYLLABLE TUOX

0A127
YI SYLLABLE TUO

0A128
YI SYLLABLE TUOP

0A129
YI SYLLABLE TOT

0A12A
YI SYLLABLE TOX

0A12B
YI SYLLABLE TO

0A12C
YI SYLLABLE TOP

0A12D
YI SYLLABLE TEX

0A12E
YI SYLLABLE TE

0A12F
YI SYLLABLE TEP

0A130
YI SYLLABLE TUT

0A131
YI SYLLABLE TUX

0A132
YI SYLLABLE TU

0A133
YI SYLLABLE TUP

0A134
YI SYLLABLE TURX

0A135
YI SYLLABLE TUR

0A136
YI SYLLABLE DDIT

0A137
YI SYLLABLE DDIX

0A138
YI SYLLABLE DDI

0A139
YI SYLLABLE DDIP

0A13A
YI SYLLABLE DDIEX

0A13B
YI SYLLABLE DDIE

0A13C
YI SYLLABLE DDIEP

0A13D
YI SYLLABLE DDAT

0A13E
YI SYLLABLE DDAX

0A13F
YI SYLLABLE DDA

0A140
YI SYLLABLE DDAP

0A141
YI SYLLABLE DDUOX

0A142
YI SYLLABLE DDUO

0A143
YI SYLLABLE DDUOP

0A144
YI SYLLABLE DDOT

0A145
YI SYLLABLE DDOX

0A146
YI SYLLABLE DDO

0A147
YI SYLLABLE DDOP

0A148
YI SYLLABLE DDEX

0A149
YI SYLLABLE DDE

0A14A
YI SYLLABLE DDEP

0A14B
YI SYLLABLE DDUT

0A14C
YI SYLLABLE DDUX

0A14D
YI SYLLABLE DDU

0A14E
YI SYLLABLE DDUP

0A14F
YI SYLLABLE DDURX

0A150
YI SYLLABLE DDUR

0A151
YI SYLLABLE NDIT

0A152
YI SYLLABLE NDIX

0A153
YI SYLLABLE NDI

0A154
YI SYLLABLE NDIP

0A155
YI SYLLABLE NDIEX

0A156
YI SYLLABLE NDIE

0A157
YI SYLLABLE NDAT

0A158
YI SYLLABLE NDAX

0A159
YI SYLLABLE NDA

0A15A
YI SYLLABLE NDAP

0A15B
YI SYLLABLE NDOT

0A15C
YI SYLLABLE NDOX

0A15D
YI SYLLABLE NDO

0A15E
YI SYLLABLE NDOP

0A15F
YI SYLLABLE NDEX

0A160
YI SYLLABLE NDE

0A161
YI SYLLABLE NDEP

0A162
YI SYLLABLE NDUT

0A163
YI SYLLABLE NDUX

0A164
YI SYLLABLE NDU

0A165
YI SYLLABLE NDUP

0A166
YI SYLLABLE NDURX

0A167
YI SYLLABLE NDUR

0A168
YI SYLLABLE HNIT

0A169
YI SYLLABLE HNIX

0A16A
YI SYLLABLE HNI

0A16B
YI SYLLABLE HNIP

0A16C
YI SYLLABLE HNIET

0A16D
YI SYLLABLE HNIEX

0A16E
YI SYLLABLE HNIE

0A16F
YI SYLLABLE HNIEP

0A170
YI SYLLABLE HNAT

0A171
YI SYLLABLE HNAX

0A172
YI SYLLABLE HNA

0A173
YI SYLLABLE HNAP

0A174
YI SYLLABLE HNUOX

0A175
YI SYLLABLE HNUO

0A176
YI SYLLABLE HNOT

0A177
YI SYLLABLE HNOX

0A178
YI SYLLABLE HNOP

0A179
YI SYLLABLE HNEX

0A17A
YI SYLLABLE HNE

0A17B
YI SYLLABLE HNEP

0A17C
YI SYLLABLE HNUT

0A17D
YI SYLLABLE NIT

0A17E
YI SYLLABLE NIX

0A17F
YI SYLLABLE NI

0A180
YI SYLLABLE NIP

0A181
YI SYLLABLE NIEX

0A182
YI SYLLABLE NIE

0A183
YI SYLLABLE NIEP

0A184
YI SYLLABLE NAX

0A185
YI SYLLABLE NA

0A186
YI SYLLABLE NAP

0A187
YI SYLLABLE NUOX

0A188
YI SYLLABLE NUO

0A189
YI SYLLABLE NUOP

0A18A
YI SYLLABLE NOT

0A18B
YI SYLLABLE NOX

0A18C
YI SYLLABLE NO

0A18D
YI SYLLABLE NOP

0A18E
YI SYLLABLE NEX

0A18F
YI SYLLABLE NE

0A190
YI SYLLABLE NEP

0A191
YI SYLLABLE NUT

0A192
YI SYLLABLE NUX

0A193
YI SYLLABLE NU

0A194
YI SYLLABLE NUP

0A195
YI SYLLABLE NURX

0A196
YI SYLLABLE NUR

0A197
YI SYLLABLE HLIT

0A198
YI SYLLABLE HLIX

0A199
YI SYLLABLE HLI

0A19A
YI SYLLABLE HLIP

0A19B
YI SYLLABLE HLIEX

0A19C
YI SYLLABLE HLIE

0A19D
YI SYLLABLE HLIEP

0A19E
YI SYLLABLE HLAT

0A19F
YI SYLLABLE HLAX

0A1A0
YI SYLLABLE HLA

0A1A1
YI SYLLABLE HLAP

0A1A2
YI SYLLABLE HLUOX

0A1A3
YI SYLLABLE HLUO

0A1A4
YI SYLLABLE HLUOP

0A1A5
YI SYLLABLE HLOX

0A1A6
YI SYLLABLE HLO

0A1A7
YI SYLLABLE HLOP

0A1A8
YI SYLLABLE HLEX

0A1A9
YI SYLLABLE HLE

0A1AA
YI SYLLABLE HLEP

0A1AB
YI SYLLABLE HLUT

0A1AC
YI SYLLABLE HLUX

0A1AD
YI SYLLABLE HLU

0A1AE
YI SYLLABLE HLUP

0A1AF
YI SYLLABLE HLURX

0A1B0
YI SYLLABLE HLUR

0A1B1
YI SYLLABLE HLYT

0A1B2
YI SYLLABLE HLYX

0A1B3
YI SYLLABLE HLY

0A1B4
YI SYLLABLE HLYP

0A1B5
YI SYLLABLE HLYRX

0A1B6
YI SYLLABLE HLYR

0A1B7
YI SYLLABLE LIT

0A1B8
YI SYLLABLE LIX

0A1B9
YI SYLLABLE LI

0A1BA
YI SYLLABLE LIP

0A1BB
YI SYLLABLE LIET

0A1BC
YI SYLLABLE LIEX

0A1BD
YI SYLLABLE LIE

0A1BE
YI SYLLABLE LIEP

0A1BF
YI SYLLABLE LAT

0A1C0
YI SYLLABLE LAX

0A1C1
YI SYLLABLE LA

0A1C2
YI SYLLABLE LAP

0A1C3
YI SYLLABLE LUOT

0A1C4
YI SYLLABLE LUOX

0A1C5
YI SYLLABLE LUO

0A1C6
YI SYLLABLE LUOP

0A1C7
YI SYLLABLE LOT

0A1C8
YI SYLLABLE LOX

0A1C9
YI SYLLABLE LO

0A1CA
YI SYLLABLE LOP

0A1CB
YI SYLLABLE LEX

0A1CC
YI SYLLABLE LE

0A1CD
YI SYLLABLE LEP

0A1CE
YI SYLLABLE LUT

0A1CF
YI SYLLABLE LUX

0A1D0
YI SYLLABLE LU

0A1D1
YI SYLLABLE LUP

0A1D2
YI SYLLABLE LURX

0A1D3
YI SYLLABLE LUR

0A1D4
YI SYLLABLE LYT

0A1D5
YI SYLLABLE LYX

0A1D6
YI SYLLABLE LY

0A1D7
YI SYLLABLE LYP

0A1D8
YI SYLLABLE LYRX

0A1D9
YI SYLLABLE LYR

0A1DA
YI SYLLABLE GIT

0A1DB
YI SYLLABLE GIX

0A1DC
YI SYLLABLE GI

0A1DD
YI SYLLABLE GIP

0A1DE
YI SYLLABLE GIET

0A1DF
YI SYLLABLE GIEX

0A1E0
YI SYLLABLE GIE

0A1E1
YI SYLLABLE GIEP

0A1E2
YI SYLLABLE GAT

0A1E3
YI SYLLABLE GAX

0A1E4
YI SYLLABLE GA

0A1E5
YI SYLLABLE GAP

0A1E6
YI SYLLABLE GUOT

0A1E7
YI SYLLABLE GUOX

0A1E8
YI SYLLABLE GUO

0A1E9
YI SYLLABLE GUOP

0A1EA
YI SYLLABLE GOT

0A1EB
YI SYLLABLE GOX

0A1EC
YI SYLLABLE GO

0A1ED
YI SYLLABLE GOP

0A1EE
YI SYLLABLE GET

0A1EF
YI SYLLABLE GEX

0A1F0
YI SYLLABLE GE

0A1F1
YI SYLLABLE GEP

0A1F2
YI SYLLABLE GUT

0A1F3
YI SYLLABLE GUX

0A1F4
YI SYLLABLE GU

0A1F5
YI SYLLABLE GUP

0A1F6
YI SYLLABLE GURX

0A1F7
YI SYLLABLE GUR

0A1F8
YI SYLLABLE KIT

0A1F9
YI SYLLABLE KIX

0A1FA
YI SYLLABLE KI

0A1FB
YI SYLLABLE KIP

0A1FC
YI SYLLABLE KIEX

0A1FD
YI SYLLABLE KIE

0A1FE
YI SYLLABLE KIEP

0A1FF
YI SYLLABLE KAT

0A200
YI SYLLABLE KAX

0A201
YI SYLLABLE KA

0A202
YI SYLLABLE KAP

0A203
YI SYLLABLE KUOX

0A204
YI SYLLABLE KUO

0A205
YI SYLLABLE KUOP

0A206
YI SYLLABLE KOT

0A207
YI SYLLABLE KOX

0A208
YI SYLLABLE KO

0A209
YI SYLLABLE KOP

0A20A
YI SYLLABLE KET

0A20B
YI SYLLABLE KEX

0A20C
YI SYLLABLE KE

0A20D
YI SYLLABLE KEP

0A20E
YI SYLLABLE KUT

0A20F
YI SYLLABLE KUX

0A210
YI SYLLABLE KU

0A211
YI SYLLABLE KUP

0A212
YI SYLLABLE KURX

0A213
YI SYLLABLE KUR

0A214
YI SYLLABLE GGIT

0A215
YI SYLLABLE GGIX

0A216
YI SYLLABLE GGI

0A217
YI SYLLABLE GGIEX

0A218
YI SYLLABLE GGIE

0A219
YI SYLLABLE GGIEP

0A21A
YI SYLLABLE GGAT

0A21B
YI SYLLABLE GGAX

0A21C
YI SYLLABLE GGA

0A21D
YI SYLLABLE GGAP

0A21E
YI SYLLABLE GGUOT

0A21F
YI SYLLABLE GGUOX

0A220
YI SYLLABLE GGUO

0A221
YI SYLLABLE GGUOP

0A222
YI SYLLABLE GGOT

0A223
YI SYLLABLE GGOX

0A224
YI SYLLABLE GGO

0A225
YI SYLLABLE GGOP

0A226
YI SYLLABLE GGET

0A227
YI SYLLABLE GGEX

0A228
YI SYLLABLE GGE

0A229
YI SYLLABLE GGEP

0A22A
YI SYLLABLE GGUT

0A22B
YI SYLLABLE GGUX

0A22C
YI SYLLABLE GGU

0A22D
YI SYLLABLE GGUP

0A22E
YI SYLLABLE GGURX

0A22F
YI SYLLABLE GGUR

0A230
YI SYLLABLE MGIEX

0A231
YI SYLLABLE MGIE

0A232
YI SYLLABLE MGAT

0A233
YI SYLLABLE MGAX

0A234
YI SYLLABLE MGA

0A235
YI SYLLABLE MGAP

0A236
YI SYLLABLE MGUOX

0A237
YI SYLLABLE MGUO

0A238
YI SYLLABLE MGUOP

0A239
YI SYLLABLE MGOT

0A23A
YI SYLLABLE MGOX

0A23B
YI SYLLABLE MGO

0A23C
YI SYLLABLE MGOP

0A23D
YI SYLLABLE MGEX

0A23E
YI SYLLABLE MGE

0A23F
YI SYLLABLE MGEP

0A240
YI SYLLABLE MGUT

0A241
YI SYLLABLE MGUX

0A242
YI SYLLABLE MGU

0A243
YI SYLLABLE MGUP

0A244
YI SYLLABLE MGURX

0A245
YI SYLLABLE MGUR

0A246
YI SYLLABLE HXIT

0A247
YI SYLLABLE HXIX

0A248
YI SYLLABLE HXI

0A249
YI SYLLABLE HXIP

0A24A
YI SYLLABLE HXIET

0A24B
YI SYLLABLE HXIEX

0A24C
YI SYLLABLE HXIE

0A24D
YI SYLLABLE HXIEP

0A24E
YI SYLLABLE HXAT

0A24F
YI SYLLABLE HXAX

0A250
YI SYLLABLE HXA

0A251
YI SYLLABLE HXAP

0A252
YI SYLLABLE HXUOT

0A253
YI SYLLABLE HXUOX

0A254
YI SYLLABLE HXUO

0A255
YI SYLLABLE HXUOP

0A256
YI SYLLABLE HXOT

0A257
YI SYLLABLE HXOX

0A258
YI SYLLABLE HXO

0A259
YI SYLLABLE HXOP

0A25A
YI SYLLABLE HXEX

0A25B
YI SYLLABLE HXE

0A25C
YI SYLLABLE HXEP

0A25D
YI SYLLABLE NGIEX

0A25E
YI SYLLABLE NGIE

0A25F
YI SYLLABLE NGIEP

0A260
YI SYLLABLE NGAT

0A261
YI SYLLABLE NGAX

0A262
YI SYLLABLE NGA

0A263
YI SYLLABLE NGAP

0A264
YI SYLLABLE NGUOT

0A265
YI SYLLABLE NGUOX

0A266
YI SYLLABLE NGUO

0A267
YI SYLLABLE NGOT

0A268
YI SYLLABLE NGOX

0A269
YI SYLLABLE NGO

0A26A
YI SYLLABLE NGOP

0A26B
YI SYLLABLE NGEX

0A26C
YI SYLLABLE NGE

0A26D
YI SYLLABLE NGEP

0A26E
YI SYLLABLE HIT

0A26F
YI SYLLABLE HIEX

0A270
YI SYLLABLE HIE

0A271
YI SYLLABLE HAT

0A272
YI SYLLABLE HAX

0A273
YI SYLLABLE HA

0A274
YI SYLLABLE HAP

0A275
YI SYLLABLE HUOT

0A276
YI SYLLABLE HUOX

0A277
YI SYLLABLE HUO

0A278
YI SYLLABLE HUOP

0A279
YI SYLLABLE HOT

0A27A
YI SYLLABLE HOX

0A27B
YI SYLLABLE HO

0A27C
YI SYLLABLE HOP

0A27D
YI SYLLABLE HEX

0A27E
YI SYLLABLE HE

0A27F
YI SYLLABLE HEP

0A280
YI SYLLABLE WAT

0A281
YI SYLLABLE WAX

0A282
YI SYLLABLE WA

0A283
YI SYLLABLE WAP

0A284
YI SYLLABLE WUOX

0A285
YI SYLLABLE WUO

0A286
YI SYLLABLE WUOP

0A287
YI SYLLABLE WOX

0A288
YI SYLLABLE WO

0A289
YI SYLLABLE WOP

0A28A
YI SYLLABLE WEX

0A28B
YI SYLLABLE WE

0A28C
YI SYLLABLE WEP

0A28D
YI SYLLABLE ZIT

0A28E
YI SYLLABLE ZIX

0A28F
YI SYLLABLE ZI

0A290
YI SYLLABLE ZIP

0A291
YI SYLLABLE ZIEX

0A292
YI SYLLABLE ZIE

0A293
YI SYLLABLE ZIEP

0A294
YI SYLLABLE ZAT

0A295
YI SYLLABLE ZAX

0A296
YI SYLLABLE ZA

0A297
YI SYLLABLE ZAP

0A298
YI SYLLABLE ZUOX

0A299
YI SYLLABLE ZUO

0A29A
YI SYLLABLE ZUOP

0A29B
YI SYLLABLE ZOT

0A29C
YI SYLLABLE ZOX

0A29D
YI SYLLABLE ZO

0A29E
YI SYLLABLE ZOP

0A29F
YI SYLLABLE ZEX

0A2A0
YI SYLLABLE ZE

0A2A1
YI SYLLABLE ZEP

0A2A2
YI SYLLABLE ZUT

0A2A3
YI SYLLABLE ZUX

0A2A4
YI SYLLABLE ZU

0A2A5
YI SYLLABLE ZUP

0A2A6
YI SYLLABLE ZURX

0A2A7
YI SYLLABLE ZUR

0A2A8
YI SYLLABLE ZYT

0A2A9
YI SYLLABLE ZYX

0A2AA
YI SYLLABLE ZY

0A2AB
YI SYLLABLE ZYP

0A2AC
YI SYLLABLE ZYRX

0A2AD
YI SYLLABLE ZYR

0A2AE
YI SYLLABLE CIT

0A2AF
YI SYLLABLE CIX

0A2B0
YI SYLLABLE CI

0A2B1
YI SYLLABLE CIP

0A2B2
YI SYLLABLE CIET

0A2B3
YI SYLLABLE CIEX

0A2B4
YI SYLLABLE CIE

0A2B5
YI SYLLABLE CIEP

0A2B6
YI SYLLABLE CAT

0A2B7
YI SYLLABLE CAX

0A2B8
YI SYLLABLE CA

0A2B9
YI SYLLABLE CAP

0A2BA
YI SYLLABLE CUOX

0A2BB
YI SYLLABLE CUO

0A2BC
YI SYLLABLE CUOP

0A2BD
YI SYLLABLE COT

0A2BE
YI SYLLABLE COX

0A2BF
YI SYLLABLE CO

0A2C0
YI SYLLABLE COP

0A2C1
YI SYLLABLE CEX

0A2C2
YI SYLLABLE CE

0A2C3
YI SYLLABLE CEP

0A2C4
YI SYLLABLE CUT

0A2C5
YI SYLLABLE CUX

0A2C6
YI SYLLABLE CU

0A2C7
YI SYLLABLE CUP

0A2C8
YI SYLLABLE CURX

0A2C9
YI SYLLABLE CUR

0A2CA
YI SYLLABLE CYT

0A2CB
YI SYLLABLE CYX

0A2CC
YI SYLLABLE CY

0A2CD
YI SYLLABLE CYP

0A2CE
YI SYLLABLE CYRX

0A2CF
YI SYLLABLE CYR

0A2D0
YI SYLLABLE ZZIT

0A2D1
YI SYLLABLE ZZIX

0A2D2
YI SYLLABLE ZZI

0A2D3
YI SYLLABLE ZZIP

0A2D4
YI SYLLABLE ZZIET

0A2D5
YI SYLLABLE ZZIEX

0A2D6
YI SYLLABLE ZZIE

0A2D7
YI SYLLABLE ZZIEP

0A2D8
YI SYLLABLE ZZAT

0A2D9
YI SYLLABLE ZZAX

0A2DA
YI SYLLABLE ZZA

0A2DB
YI SYLLABLE ZZAP

0A2DC
YI SYLLABLE ZZOX

0A2DD
YI SYLLABLE ZZO

0A2DE
YI SYLLABLE ZZOP

0A2DF
YI SYLLABLE ZZEX

0A2E0
YI SYLLABLE ZZE

0A2E1
YI SYLLABLE ZZEP

0A2E2
YI SYLLABLE ZZUX

0A2E3
YI SYLLABLE ZZU

0A2E4
YI SYLLABLE ZZUP

0A2E5
YI SYLLABLE ZZURX

0A2E6
YI SYLLABLE ZZUR

0A2E7
YI SYLLABLE ZZYT

0A2E8
YI SYLLABLE ZZYX

0A2E9
YI SYLLABLE ZZY

0A2EA
YI SYLLABLE ZZYP

0A2EB
YI SYLLABLE ZZYRX

0A2EC
YI SYLLABLE ZZYR

0A2ED
YI SYLLABLE NZIT

0A2EE
YI SYLLABLE NZIX

0A2EF
YI SYLLABLE NZI

0A2F0
YI SYLLABLE NZIP

0A2F1
YI SYLLABLE NZIEX

0A2F2
YI SYLLABLE NZIE

0A2F3
YI SYLLABLE NZIEP

0A2F4
YI SYLLABLE NZAT

0A2F5
YI SYLLABLE NZAX

0A2F6
YI SYLLABLE NZA

0A2F7
YI SYLLABLE NZAP

0A2F8
YI SYLLABLE NZUOX

0A2F9
YI SYLLABLE NZUO

0A2FA
YI SYLLABLE NZOX

0A2FB
YI SYLLABLE NZOP

0A2FC
YI SYLLABLE NZEX

0A2FD
YI SYLLABLE NZE

0A2FE
YI SYLLABLE NZUX

0A2FF
YI SYLLABLE NZU

0A300
YI SYLLABLE NZUP

0A301
YI SYLLABLE NZURX

0A302
YI SYLLABLE NZUR

0A303
YI SYLLABLE NZYT

0A304
YI SYLLABLE NZYX

0A305
YI SYLLABLE NZY

0A306
YI SYLLABLE NZYP

0A307
YI SYLLABLE NZYRX

0A308
YI SYLLABLE NZYR

0A309
YI SYLLABLE SIT

0A30A
YI SYLLABLE SIX

0A30B
YI SYLLABLE SI

0A30C
YI SYLLABLE SIP

0A30D
YI SYLLABLE SIEX

0A30E
YI SYLLABLE SIE

0A30F
YI SYLLABLE SIEP

0A310
YI SYLLABLE SAT

0A311
YI SYLLABLE SAX

0A312
YI SYLLABLE SA

0A313
YI SYLLABLE SAP

0A314
YI SYLLABLE SUOX

0A315
YI SYLLABLE SUO

0A316
YI SYLLABLE SUOP

0A317
YI SYLLABLE SOT

0A318
YI SYLLABLE SOX

0A319
YI SYLLABLE SO

0A31A
YI SYLLABLE SOP

0A31B
YI SYLLABLE SEX

0A31C
YI SYLLABLE SE

0A31D
YI SYLLABLE SEP

0A31E
YI SYLLABLE SUT

0A31F
YI SYLLABLE SUX

0A320
YI SYLLABLE SU

0A321
YI SYLLABLE SUP

0A322
YI SYLLABLE SURX

0A323
YI SYLLABLE SUR

0A324
YI SYLLABLE SYT

0A325
YI SYLLABLE SYX

0A326
YI SYLLABLE SY

0A327
YI SYLLABLE SYP

0A328
YI SYLLABLE SYRX

0A329
YI SYLLABLE SYR

0A32A
YI SYLLABLE SSIT

0A32B
YI SYLLABLE SSIX

0A32C
YI SYLLABLE SSI

0A32D
YI SYLLABLE SSIP

0A32E
YI SYLLABLE SSIEX

0A32F
YI SYLLABLE SSIE

0A330
YI SYLLABLE SSIEP

0A331
YI SYLLABLE SSAT

0A332
YI SYLLABLE SSAX

0A333
YI SYLLABLE SSA

0A334
YI SYLLABLE SSAP

0A335
YI SYLLABLE SSOT

0A336
YI SYLLABLE SSOX

0A337
YI SYLLABLE SSO

0A338
YI SYLLABLE SSOP

0A339
YI SYLLABLE SSEX

0A33A
YI SYLLABLE SSE

0A33B
YI SYLLABLE SSEP

0A33C
YI SYLLABLE SSUT

0A33D
YI SYLLABLE SSUX

0A33E
YI SYLLABLE SSU

0A33F
YI SYLLABLE SSUP

0A340
YI SYLLABLE SSYT

0A341
YI SYLLABLE SSYX

0A342
YI SYLLABLE SSY

0A343
YI SYLLABLE SSYP

0A344
YI SYLLABLE SSYRX

0A345
YI SYLLABLE SSYR

0A346
YI SYLLABLE ZHAT

0A347
YI SYLLABLE ZHAX

0A348
YI SYLLABLE ZHA

0A349
YI SYLLABLE ZHAP

0A34A
YI SYLLABLE ZHUOX

0A34B
YI SYLLABLE ZHUO

0A34C
YI SYLLABLE ZHUOP

0A34D
YI SYLLABLE ZHOT

0A34E
YI SYLLABLE ZHOX

0A34F
YI SYLLABLE ZHO

0A350
YI SYLLABLE ZHOP

0A351
YI SYLLABLE ZHET

0A352
YI SYLLABLE ZHEX

0A353
YI SYLLABLE ZHE

0A354
YI SYLLABLE ZHEP

0A355
YI SYLLABLE ZHUT

0A356
YI SYLLABLE ZHUX

0A357
YI SYLLABLE ZHU

0A358
YI SYLLABLE ZHUP

0A359
YI SYLLABLE ZHURX

0A35A
YI SYLLABLE ZHUR

0A35B
YI SYLLABLE ZHYT

0A35C
YI SYLLABLE ZHYX

0A35D
YI SYLLABLE ZHY

0A35E
YI SYLLABLE ZHYP

0A35F
YI SYLLABLE ZHYRX

0A360
YI SYLLABLE ZHYR

0A361
YI SYLLABLE CHAT

0A362
YI SYLLABLE CHAX

0A363
YI SYLLABLE CHA

0A364
YI SYLLABLE CHAP

0A365
YI SYLLABLE CHUOT

0A366
YI SYLLABLE CHUOX

0A367
YI SYLLABLE CHUO

0A368
YI SYLLABLE CHUOP

0A369
YI SYLLABLE CHOT

0A36A
YI SYLLABLE CHOX

0A36B
YI SYLLABLE CHO

0A36C
YI SYLLABLE CHOP

0A36D
YI SYLLABLE CHET

0A36E
YI SYLLABLE CHEX

0A36F
YI SYLLABLE CHE

0A370
YI SYLLABLE CHEP

0A371
YI SYLLABLE CHUX

0A372
YI SYLLABLE CHU

0A373
YI SYLLABLE CHUP

0A374
YI SYLLABLE CHURX

0A375
YI SYLLABLE CHUR

0A376
YI SYLLABLE CHYT

0A377
YI SYLLABLE CHYX

0A378
YI SYLLABLE CHY

0A379
YI SYLLABLE CHYP

0A37A
YI SYLLABLE CHYRX

0A37B
YI SYLLABLE CHYR

0A37C
YI SYLLABLE RRAX

0A37D
YI SYLLABLE RRA

0A37E
YI SYLLABLE RRUOX

0A37F
YI SYLLABLE RRUO

0A380
YI SYLLABLE RROT

0A381
YI SYLLABLE RROX

0A382
YI SYLLABLE RRO

0A383
YI SYLLABLE RROP

0A384
YI SYLLABLE RRET

0A385
YI SYLLABLE RREX

0A386
YI SYLLABLE RRE

0A387
YI SYLLABLE RREP

0A388
YI SYLLABLE RRUT

0A389
YI SYLLABLE RRUX

0A38A
YI SYLLABLE RRU

0A38B
YI SYLLABLE RRUP

0A38C
YI SYLLABLE RRURX

0A38D
YI SYLLABLE RRUR

0A38E
YI SYLLABLE RRYT

0A38F
YI SYLLABLE RRYX

0A390
YI SYLLABLE RRY

0A391
YI SYLLABLE RRYP

0A392
YI SYLLABLE RRYRX

0A393
YI SYLLABLE RRYR

0A394
YI SYLLABLE NRAT

0A395
YI SYLLABLE NRAX

0A396
YI SYLLABLE NRA

0A397
YI SYLLABLE NRAP

0A398
YI SYLLABLE NROX

0A399
YI SYLLABLE NRO

0A39A
YI SYLLABLE NROP

0A39B
YI SYLLABLE NRET

0A39C
YI SYLLABLE NREX

0A39D
YI SYLLABLE NRE

0A39E
YI SYLLABLE NREP

0A39F
YI SYLLABLE NRUT

0A3A0
YI SYLLABLE NRUX

0A3A1
YI SYLLABLE NRU

0A3A2
YI SYLLABLE NRUP

0A3A3
YI SYLLABLE NRURX

0A3A4
YI SYLLABLE NRUR

0A3A5
YI SYLLABLE NRYT

0A3A6
YI SYLLABLE NRYX

0A3A7
YI SYLLABLE NRY

0A3A8
YI SYLLABLE NRYP

0A3A9
YI SYLLABLE NRYRX

0A3AA
YI SYLLABLE NRYR

0A3AB
YI SYLLABLE SHAT

0A3AC
YI SYLLABLE SHAX

0A3AD
YI SYLLABLE SHA

0A3AE
YI SYLLABLE SHAP

0A3AF
YI SYLLABLE SHUOX

0A3B0
YI SYLLABLE SHUO

0A3B1
YI SYLLABLE SHUOP

0A3B2
YI SYLLABLE SHOT

0A3B3
YI SYLLABLE SHOX

0A3B4
YI SYLLABLE SHO

0A3B5
YI SYLLABLE SHOP

0A3B6
YI SYLLABLE SHET

0A3B7
YI SYLLABLE SHEX

0A3B8
YI SYLLABLE SHE

0A3B9
YI SYLLABLE SHEP

0A3BA
YI SYLLABLE SHUT

0A3BB
YI SYLLABLE SHUX

0A3BC
YI SYLLABLE SHU

0A3BD
YI SYLLABLE SHUP

0A3BE
YI SYLLABLE SHURX

0A3BF
YI SYLLABLE SHUR

0A3C0
YI SYLLABLE SHYT

0A3C1
YI SYLLABLE SHYX

0A3C2
YI SYLLABLE SHY

0A3C3
YI SYLLABLE SHYP

0A3C4
YI SYLLABLE SHYRX

0A3C5
YI SYLLABLE SHYR

0A3C6
YI SYLLABLE RAT

0A3C7
YI SYLLABLE RAX

0A3C8
YI SYLLABLE RA

0A3C9
YI SYLLABLE RAP

0A3CA
YI SYLLABLE RUOX

0A3CB
YI SYLLABLE RUO

0A3CC
YI SYLLABLE RUOP

0A3CD
YI SYLLABLE ROT

0A3CE
YI SYLLABLE ROX

0A3CF
YI SYLLABLE RO

0A3D0
YI SYLLABLE ROP

0A3D1
YI SYLLABLE REX

0A3D2
YI SYLLABLE RE

0A3D3
YI SYLLABLE REP

0A3D4
YI SYLLABLE RUT

0A3D5
YI SYLLABLE RUX

0A3D6
YI SYLLABLE RU

0A3D7
YI SYLLABLE RUP

0A3D8
YI SYLLABLE RURX

0A3D9
YI SYLLABLE RUR

0A3DA
YI SYLLABLE RYT

0A3DB
YI SYLLABLE RYX

0A3DC
YI SYLLABLE RY

0A3DD
YI SYLLABLE RYP

0A3DE
YI SYLLABLE RYRX

0A3DF
YI SYLLABLE RYR

0A3E0
YI SYLLABLE JIT

0A3E1
YI SYLLABLE JIX

0A3E2
YI SYLLABLE JI

0A3E3
YI SYLLABLE JIP

0A3E4
YI SYLLABLE JIET

0A3E5
YI SYLLABLE JIEX

0A3E6
YI SYLLABLE JIE

0A3E7
YI SYLLABLE JIEP

0A3E8
YI SYLLABLE JUOT

0A3E9
YI SYLLABLE JUOX

0A3EA
YI SYLLABLE JUO

0A3EB
YI SYLLABLE JUOP

0A3EC
YI SYLLABLE JOT

0A3ED
YI SYLLABLE JOX

0A3EE
YI SYLLABLE JO

0A3EF
YI SYLLABLE JOP

0A3F0
YI SYLLABLE JUT

0A3F1
YI SYLLABLE JUX

0A3F2
YI SYLLABLE JU

0A3F3
YI SYLLABLE JUP

0A3F4
YI SYLLABLE JURX

0A3F5
YI SYLLABLE JUR

0A3F6
YI SYLLABLE JYT

0A3F7
YI SYLLABLE JYX

0A3F8
YI SYLLABLE JY

0A3F9
YI SYLLABLE JYP

0A3FA
YI SYLLABLE JYRX

0A3FB
YI SYLLABLE JYR

0A3FC
YI SYLLABLE QIT

0A3FD
YI SYLLABLE QIX

0A3FE
YI SYLLABLE QI

0A3FF
YI SYLLABLE QIP

0A400
YI SYLLABLE QIET

0A401
YI SYLLABLE QIEX

0A402
YI SYLLABLE QIE

0A403
YI SYLLABLE QIEP

0A404
YI SYLLABLE QUOT

0A405
YI SYLLABLE QUOX

0A406
YI SYLLABLE QUO

0A407
YI SYLLABLE QUOP

0A408
YI SYLLABLE QOT

0A409
YI SYLLABLE QOX

0A40A
YI SYLLABLE QO

0A40B
YI SYLLABLE QOP

0A40C
YI SYLLABLE QUT

0A40D
YI SYLLABLE QUX

0A40E
YI SYLLABLE QU

0A40F
YI SYLLABLE QUP

0A410
YI SYLLABLE QURX

0A411
YI SYLLABLE QUR

0A412
YI SYLLABLE QYT

0A413
YI SYLLABLE QYX

0A414
YI SYLLABLE QY

0A415
YI SYLLABLE QYP

0A416
YI SYLLABLE QYRX

0A417
YI SYLLABLE QYR

0A418
YI SYLLABLE JJIT

0A419
YI SYLLABLE JJIX

0A41A
YI SYLLABLE JJI

0A41B
YI SYLLABLE JJIP

0A41C
YI SYLLABLE JJIET

0A41D
YI SYLLABLE JJIEX

0A41E
YI SYLLABLE JJIE

0A41F
YI SYLLABLE JJIEP

0A420
YI SYLLABLE JJUOX

0A421
YI SYLLABLE JJUO

0A422
YI SYLLABLE JJUOP

0A423
YI SYLLABLE JJOT

0A424
YI SYLLABLE JJOX

0A425
YI SYLLABLE JJO

0A426
YI SYLLABLE JJOP

0A427
YI SYLLABLE JJUT

0A428
YI SYLLABLE JJUX

0A429
YI SYLLABLE JJU

0A42A
YI SYLLABLE JJUP

0A42B
YI SYLLABLE JJURX

0A42C
YI SYLLABLE JJUR

0A42D
YI SYLLABLE JJYT

0A42E
YI SYLLABLE JJYX

0A42F
YI SYLLABLE JJY

0A430
YI SYLLABLE JJYP

0A431
YI SYLLABLE NJIT

0A432
YI SYLLABLE NJIX

0A433
YI SYLLABLE NJI

0A434
YI SYLLABLE NJIP

0A435
YI SYLLABLE NJIET

0A436
YI SYLLABLE NJIEX

0A437
YI SYLLABLE NJIE

0A438
YI SYLLABLE NJIEP

0A439
YI SYLLABLE NJUOX

0A43A
YI SYLLABLE NJUO

0A43B
YI SYLLABLE NJOT

0A43C
YI SYLLABLE NJOX

0A43D
YI SYLLABLE NJO

0A43E
YI SYLLABLE NJOP

0A43F
YI SYLLABLE NJUX

0A440
YI SYLLABLE NJU

0A441
YI SYLLABLE NJUP

0A442
YI SYLLABLE NJURX

0A443
YI SYLLABLE NJUR

0A444
YI SYLLABLE NJYT

0A445
YI SYLLABLE NJYX

0A446
YI SYLLABLE NJY

0A447
YI SYLLABLE NJYP

0A448
YI SYLLABLE NJYRX

0A449
YI SYLLABLE NJYR

0A44A
YI SYLLABLE NYIT

0A44B
YI SYLLABLE NYIX

0A44C
YI SYLLABLE NYI

0A44D
YI SYLLABLE NYIP

0A44E
YI SYLLABLE NYIET

0A44F
YI SYLLABLE NYIEX

0A450
YI SYLLABLE NYIE

0A451
YI SYLLABLE NYIEP

0A452
YI SYLLABLE NYUOX

0A453
YI SYLLABLE NYUO

0A454
YI SYLLABLE NYUOP

0A455
YI SYLLABLE NYOT

0A456
YI SYLLABLE NYOX

0A457
YI SYLLABLE NYO

0A458
YI SYLLABLE NYOP

0A459
YI SYLLABLE NYUT

0A45A
YI SYLLABLE NYUX

0A45B
YI SYLLABLE NYU

0A45C
YI SYLLABLE NYUP

0A45D
YI SYLLABLE XIT

0A45E
YI SYLLABLE XIX

0A45F
YI SYLLABLE XI

0A460
YI SYLLABLE XIP

0A461
YI SYLLABLE XIET

0A462
YI SYLLABLE XIEX

0A463
YI SYLLABLE XIE

0A464
YI SYLLABLE XIEP

0A465
YI SYLLABLE XUOX

0A466
YI SYLLABLE XUO

0A467
YI SYLLABLE XOT

0A468
YI SYLLABLE XOX

0A469
YI SYLLABLE XO

0A46A
YI SYLLABLE XOP

0A46B
YI SYLLABLE XYT

0A46C
YI SYLLABLE XYX

0A46D
YI SYLLABLE XY

0A46E
YI SYLLABLE XYP

0A46F
YI SYLLABLE XYRX

0A470
YI SYLLABLE XYR

0A471
YI SYLLABLE YIT

0A472
YI SYLLABLE YIX

0A473
YI SYLLABLE YI

0A474
YI SYLLABLE YIP

0A475
YI SYLLABLE YIET

0A476
YI SYLLABLE YIEX

0A477
YI SYLLABLE YIE

0A478
YI SYLLABLE YIEP

0A479
YI SYLLABLE YUOT

0A47A
YI SYLLABLE YUOX

0A47B
YI SYLLABLE YUO

0A47C
YI SYLLABLE YUOP

0A47D
YI SYLLABLE YOT

0A47E
YI SYLLABLE YOX

0A47F
YI SYLLABLE YO

0A480
YI SYLLABLE YOP

0A481
YI SYLLABLE YUT

0A482
YI SYLLABLE YUX

0A483
YI SYLLABLE YU

0A484
YI SYLLABLE YUP

0A485
YI SYLLABLE YURX

0A486
YI SYLLABLE YUR

0A487
YI SYLLABLE YYT

0A488
YI SYLLABLE YYX

0A489
YI SYLLABLE YY

0A48A
YI SYLLABLE YYP

0A48B
YI SYLLABLE YYRX

0A48C
YI SYLLABLE YYR

0A490
YI RADICAL QOT

0A491
YI RADICAL LI

0A492
YI RADICAL KIT

0A493
YI RADICAL NYIP

0A494
YI RADICAL CYP

0A495
YI RADICAL SSI

0A496
YI RADICAL GGOP

0A497
YI RADICAL GEP

0A498
YI RADICAL MI

0A499
YI RADICAL HXIT

0A49A
YI RADICAL LYR

0A49B
YI RADICAL BBUT

0A49C
YI RADICAL MOP

0A49D
YI RADICAL YO

0A49E
YI RADICAL PUT

0A49F
YI RADICAL HXUO

0A4A0
YI RADICAL TAT

0A4A1
YI RADICAL GA

0A4A2
YI RADICAL ZUP

0A4A3
YI RADICAL CYT

0A4A4
YI RADICAL DDUR

0A4A5
YI RADICAL BUR

0A4A6
YI RADICAL GGUO

0A4A7
YI RADICAL NYOP

0A4A8
YI RADICAL TU

0A4A9
YI RADICAL OP

0A4AA
YI RADICAL JJUT

0A4AB
YI RADICAL ZOT

0A4AC
YI RADICAL PYT

0A4AD
YI RADICAL HMO

0A4AE
YI RADICAL YIT

0A4AF
YI RADICAL VUR

0A4B0
YI RADICAL SHY

0A4B1
YI RADICAL VEP

0A4B2
YI RADICAL ZA

0A4B3
YI RADICAL JO

0A4B4
YI RADICAL NZUP

0A4B5
YI RADICAL JJY

0A4B6
YI RADICAL GOT

0A4B7
YI RADICAL JJIE

0A4B8
YI RADICAL WO

0A4B9
YI RADICAL DU

0A4BA
YI RADICAL SHUR

0A4BB
YI RADICAL LIE

0A4BC
YI RADICAL CY

0A4BD
YI RADICAL CUOP

0A4BE
YI RADICAL CIP

0A4BF
YI RADICAL HXOP

0A4C0
YI RADICAL SHAT

0A4C1
YI RADICAL ZUR

0A4C2
YI RADICAL SHOP

0A4C3
YI RADICAL CHE

0A4C4
YI RADICAL ZZIET

0A4C5
YI RADICAL NBIE

0A4C6
YI RADICAL KE

0A4D0
LISU LETTER BA

0A4D1
LISU LETTER PA

0A4D2
LISU LETTER PHA

0A4D3
LISU LETTER DA

0A4D4
LISU LETTER TA

0A4D5
LISU LETTER THA

0A4D6
LISU LETTER GA

0A4D7
LISU LETTER KA

0A4D8
LISU LETTER KHA

0A4D9
LISU LETTER JA

0A4DA
LISU LETTER CA

0A4DB
LISU LETTER CHA

0A4DC
LISU LETTER DZA

0A4DD
LISU LETTER TSA

0A4DE
LISU LETTER TSHA

0A4DF
LISU LETTER MA

0A4E0
LISU LETTER NA

0A4E1
LISU LETTER LA

0A4E2
LISU LETTER SA

0A4E3
LISU LETTER ZHA

0A4E4
LISU LETTER ZA

0A4E5
LISU LETTER NGA

0A4E6
LISU LETTER HA

0A4E7
LISU LETTER XA

0A4E8
LISU LETTER HHA

0A4E9
LISU LETTER FA

0A4EA
LISU LETTER WA

0A4EB
LISU LETTER SHA

0A4EC
LISU LETTER YA

0A4ED
LISU LETTER GHA

0A4EE
LISU LETTER A

0A4EF
LISU LETTER AE

0A4F0
LISU LETTER E

0A4F1
LISU LETTER EU

0A4F2
LISU LETTER I

0A4F3
LISU LETTER O

0A4F4
LISU LETTER U

0A4F5
LISU LETTER UE

0A4F6
LISU LETTER UH

0A4F7
LISU LETTER OE

0A4F8
LISU LETTER TONE MYA TI

0A4F9
LISU LETTER TONE NA PO

0A4FA
LISU LETTER TONE MYA CYA

0A4FB
LISU LETTER TONE MYA BO

0A4FC
LISU LETTER TONE MYA NA

0A4FD
LISU LETTER TONE MYA JEU

0A4FE
LISU PUNCTUATION COMMA

0A4FF
LISU PUNCTUATION FULL STOP

0A500
VAI SYLLABLE EE

0A501
VAI SYLLABLE EEN

0A502
VAI SYLLABLE HEE

0A503
VAI SYLLABLE WEE

0A504
VAI SYLLABLE WEEN

0A505
VAI SYLLABLE PEE

0A506
VAI SYLLABLE BHEE

0A507
VAI SYLLABLE BEE

0A508
VAI SYLLABLE MBEE

0A509
VAI SYLLABLE KPEE

0A50A
VAI SYLLABLE MGBEE

0A50B
VAI SYLLABLE GBEE

0A50C
VAI SYLLABLE FEE

0A50D
VAI SYLLABLE VEE

0A50E
VAI SYLLABLE TEE

0A50F
VAI SYLLABLE THEE

0A510
VAI SYLLABLE DHEE

0A511
VAI SYLLABLE DHHEE

0A512
VAI SYLLABLE LEE

0A513
VAI SYLLABLE REE

0A514
VAI SYLLABLE DEE

0A515
VAI SYLLABLE NDEE

0A516
VAI SYLLABLE SEE

0A517
VAI SYLLABLE SHEE

0A518
VAI SYLLABLE ZEE

0A519
VAI SYLLABLE ZHEE

0A51A
VAI SYLLABLE CEE

0A51B
VAI SYLLABLE JEE

0A51C
VAI SYLLABLE NJEE

0A51D
VAI SYLLABLE YEE

0A51E
VAI SYLLABLE KEE

0A51F
VAI SYLLABLE NGGEE

0A520
VAI SYLLABLE GEE

0A521
VAI SYLLABLE MEE

0A522
VAI SYLLABLE NEE

0A523
VAI SYLLABLE NYEE

0A524
VAI SYLLABLE I

0A525
VAI SYLLABLE IN

0A526
VAI SYLLABLE HI

0A527
VAI SYLLABLE HIN

0A528
VAI SYLLABLE WI

0A529
VAI SYLLABLE WIN

0A52A
VAI SYLLABLE PI

0A52B
VAI SYLLABLE BHI

0A52C
VAI SYLLABLE BI

0A52D
VAI SYLLABLE MBI

0A52E
VAI SYLLABLE KPI

0A52F
VAI SYLLABLE MGBI

0A530
VAI SYLLABLE GBI

0A531
VAI SYLLABLE FI

0A532
VAI SYLLABLE VI

0A533
VAI SYLLABLE TI

0A534
VAI SYLLABLE THI

0A535
VAI SYLLABLE DHI

0A536
VAI SYLLABLE DHHI

0A537
VAI SYLLABLE LI

0A538
VAI SYLLABLE RI

0A539
VAI SYLLABLE DI

0A53A
VAI SYLLABLE NDI

0A53B
VAI SYLLABLE SI

0A53C
VAI SYLLABLE SHI

0A53D
VAI SYLLABLE ZI

0A53E
VAI SYLLABLE ZHI

0A53F
VAI SYLLABLE CI

0A540
VAI SYLLABLE JI

0A541
VAI SYLLABLE NJI

0A542
VAI SYLLABLE YI

0A543
VAI SYLLABLE KI

0A544
VAI SYLLABLE NGGI

0A545
VAI SYLLABLE GI

0A546
VAI SYLLABLE MI

0A547
VAI SYLLABLE NI

0A548
VAI SYLLABLE NYI

0A549
VAI SYLLABLE A

0A54A
VAI SYLLABLE AN

0A54B
VAI SYLLABLE NGAN

0A54C
VAI SYLLABLE HA

0A54D
VAI SYLLABLE HAN

0A54E
VAI SYLLABLE WA

0A54F
VAI SYLLABLE WAN

0A550
VAI SYLLABLE PA

0A551
VAI SYLLABLE BHA

0A552
VAI SYLLABLE BA

0A553
VAI SYLLABLE MBA

0A554
VAI SYLLABLE KPA

0A555
VAI SYLLABLE KPAN

0A556
VAI SYLLABLE MGBA

0A557
VAI SYLLABLE GBA

0A558
VAI SYLLABLE FA

0A559
VAI SYLLABLE VA

0A55A
VAI SYLLABLE TA

0A55B
VAI SYLLABLE THA

0A55C
VAI SYLLABLE DHA

0A55D
VAI SYLLABLE DHHA

0A55E
VAI SYLLABLE LA

0A55F
VAI SYLLABLE RA

0A560
VAI SYLLABLE DA

0A561
VAI SYLLABLE NDA

0A562
VAI SYLLABLE SA

0A563
VAI SYLLABLE SHA

0A564
VAI SYLLABLE ZA

0A565
VAI SYLLABLE ZHA

0A566
VAI SYLLABLE CA

0A567
VAI SYLLABLE JA

0A568
VAI SYLLABLE NJA

0A569
VAI SYLLABLE YA

0A56A
VAI SYLLABLE KA

0A56B
VAI SYLLABLE KAN

0A56C
VAI SYLLABLE NGGA

0A56D
VAI SYLLABLE GA

0A56E
VAI SYLLABLE MA

0A56F
VAI SYLLABLE NA

0A570
VAI SYLLABLE NYA

0A571
VAI SYLLABLE OO

0A572
VAI SYLLABLE OON

0A573
VAI SYLLABLE HOO

0A574
VAI SYLLABLE WOO

0A575
VAI SYLLABLE WOON

0A576
VAI SYLLABLE POO

0A577
VAI SYLLABLE BHOO

0A578
VAI SYLLABLE BOO

0A579
VAI SYLLABLE MBOO

0A57A
VAI SYLLABLE KPOO

0A57B
VAI SYLLABLE MGBOO

0A57C
VAI SYLLABLE GBOO

0A57D
VAI SYLLABLE FOO

0A57E
VAI SYLLABLE VOO

0A57F
VAI SYLLABLE TOO

0A580
VAI SYLLABLE THOO

0A581
VAI SYLLABLE DHOO

0A582
VAI SYLLABLE DHHOO

0A583
VAI SYLLABLE LOO

0A584
VAI SYLLABLE ROO

0A585
VAI SYLLABLE DOO

0A586
VAI SYLLABLE NDOO

0A587
VAI SYLLABLE SOO

0A588
VAI SYLLABLE SHOO

0A589
VAI SYLLABLE ZOO

0A58A
VAI SYLLABLE ZHOO

0A58B
VAI SYLLABLE COO

0A58C
VAI SYLLABLE JOO

0A58D
VAI SYLLABLE NJOO

0A58E
VAI SYLLABLE YOO

0A58F
VAI SYLLABLE KOO

0A590
VAI SYLLABLE NGGOO

0A591
VAI SYLLABLE GOO

0A592
VAI SYLLABLE MOO

0A593
VAI SYLLABLE NOO

0A594
VAI SYLLABLE NYOO

0A595
VAI SYLLABLE U

0A596
VAI SYLLABLE UN

0A597
VAI SYLLABLE HU

0A598
VAI SYLLABLE HUN

0A599
VAI SYLLABLE WU

0A59A
VAI SYLLABLE WUN

0A59B
VAI SYLLABLE PU

0A59C
VAI SYLLABLE BHU

0A59D
VAI SYLLABLE BU

0A59E
VAI SYLLABLE MBU

0A59F
VAI SYLLABLE KPU

0A5A0
VAI SYLLABLE MGBU

0A5A1
VAI SYLLABLE GBU

0A5A2
VAI SYLLABLE FU

0A5A3
VAI SYLLABLE VU

0A5A4
VAI SYLLABLE TU

0A5A5
VAI SYLLABLE THU

0A5A6
VAI SYLLABLE DHU

0A5A7
VAI SYLLABLE DHHU

0A5A8
VAI SYLLABLE LU

0A5A9
VAI SYLLABLE RU

0A5AA
VAI SYLLABLE DU

0A5AB
VAI SYLLABLE NDU

0A5AC
VAI SYLLABLE SU

0A5AD
VAI SYLLABLE SHU

0A5AE
VAI SYLLABLE ZU

0A5AF
VAI SYLLABLE ZHU

0A5B0
VAI SYLLABLE CU

0A5B1
VAI SYLLABLE JU

0A5B2
VAI SYLLABLE NJU

0A5B3
VAI SYLLABLE YU

0A5B4
VAI SYLLABLE KU

0A5B5
VAI SYLLABLE NGGU

0A5B6
VAI SYLLABLE GU

0A5B7
VAI SYLLABLE MU

0A5B8
VAI SYLLABLE NU

0A5B9
VAI SYLLABLE NYU

0A5BA
VAI SYLLABLE O

0A5BB
VAI SYLLABLE ON

0A5BC
VAI SYLLABLE NGON

0A5BD
VAI SYLLABLE HO

0A5BE
VAI SYLLABLE HON

0A5BF
VAI SYLLABLE WO

0A5C0
VAI SYLLABLE WON

0A5C1
VAI SYLLABLE PO

0A5C2
VAI SYLLABLE BHO

0A5C3
VAI SYLLABLE BO

0A5C4
VAI SYLLABLE MBO

0A5C5
VAI SYLLABLE KPO

0A5C6
VAI SYLLABLE MGBO

0A5C7
VAI SYLLABLE GBO

0A5C8
VAI SYLLABLE GBON

0A5C9
VAI SYLLABLE FO

0A5CA
VAI SYLLABLE VO

0A5CB
VAI SYLLABLE TO

0A5CC
VAI SYLLABLE THO

0A5CD
VAI SYLLABLE DHO

0A5CE
VAI SYLLABLE DHHO

0A5CF
VAI SYLLABLE LO

0A5D0
VAI SYLLABLE RO

0A5D1
VAI SYLLABLE DO

0A5D2
VAI SYLLABLE NDO

0A5D3
VAI SYLLABLE SO

0A5D4
VAI SYLLABLE SHO

0A5D5
VAI SYLLABLE ZO

0A5D6
VAI SYLLABLE ZHO

0A5D7
VAI SYLLABLE CO

0A5D8
VAI SYLLABLE JO

0A5D9
VAI SYLLABLE NJO

0A5DA
VAI SYLLABLE YO

0A5DB
VAI SYLLABLE KO

0A5DC
VAI SYLLABLE NGGO

0A5DD
VAI SYLLABLE GO

0A5DE
VAI SYLLABLE MO

0A5DF
VAI SYLLABLE NO

0A5E0
VAI SYLLABLE NYO

0A5E1
VAI SYLLABLE E

0A5E2
VAI SYLLABLE EN

0A5E3
VAI SYLLABLE NGEN

0A5E4
VAI SYLLABLE HE

0A5E5
VAI SYLLABLE HEN

0A5E6
VAI SYLLABLE WE

0A5E7
VAI SYLLABLE WEN

0A5E8
VAI SYLLABLE PE

0A5E9
VAI SYLLABLE BHE

0A5EA
VAI SYLLABLE BE

0A5EB
VAI SYLLABLE MBE

0A5EC
VAI SYLLABLE KPE

0A5ED
VAI SYLLABLE KPEN

0A5EE
VAI SYLLABLE MGBE

0A5EF
VAI SYLLABLE GBE

0A5F0
VAI SYLLABLE GBEN

0A5F1
VAI SYLLABLE FE

0A5F2
VAI SYLLABLE VE

0A5F3
VAI SYLLABLE TE

0A5F4
VAI SYLLABLE THE

0A5F5
VAI SYLLABLE DHE

0A5F6
VAI SYLLABLE DHHE

0A5F7
VAI SYLLABLE LE

0A5F8
VAI SYLLABLE RE

0A5F9
VAI SYLLABLE DE

0A5FA
VAI SYLLABLE NDE

0A5FB
VAI SYLLABLE SE

0A5FC
VAI SYLLABLE SHE

0A5FD
VAI SYLLABLE ZE

0A5FE
VAI SYLLABLE ZHE

0A5FF
VAI SYLLABLE CE

0A600
VAI SYLLABLE JE

0A601
VAI SYLLABLE NJE

0A602
VAI SYLLABLE YE

0A603
VAI SYLLABLE KE

0A604
VAI SYLLABLE NGGE

0A605
VAI SYLLABLE NGGEN

0A606
VAI SYLLABLE GE

0A607
VAI SYLLABLE GEN

0A608
VAI SYLLABLE ME

0A609
VAI SYLLABLE NE

0A60A
VAI SYLLABLE NYE

0A60B
VAI SYLLABLE NG

0A60C
VAI SYLLABLE LENGTHENER

0A60D
VAI COMMA

0A60E
VAI FULL STOP

0A60F
VAI QUESTION MARK

0A610
VAI SYLLABLE NDOLE FA

0A611
VAI SYLLABLE NDOLE KA

0A612
VAI SYLLABLE NDOLE SOO

0A613
VAI SYMBOL FEENG

0A614
VAI SYMBOL KEENG

0A615
VAI SYMBOL TING

0A616
VAI SYMBOL NII

0A617
VAI SYMBOL BANG

0A618
VAI SYMBOL FAA

0A619
VAI SYMBOL TAA

0A61A
VAI SYMBOL DANG

0A61B
VAI SYMBOL DOONG

0A61C
VAI SYMBOL KUNG

0A61D
VAI SYMBOL TONG

0A61E
VAI SYMBOL DO-O

0A61F
VAI SYMBOL JONG

0A620
VAI DIGIT ZERO

0A621
VAI DIGIT ONE

0A622
VAI DIGIT TWO

0A623
VAI DIGIT THREE

0A624
VAI DIGIT FOUR

0A625
VAI DIGIT FIVE

0A626
VAI DIGIT SIX

0A627
VAI DIGIT SEVEN

0A628
VAI DIGIT EIGHT

0A629
VAI DIGIT NINE

0A62A
VAI SYLLABLE NDOLE MA

0A62B
VAI SYLLABLE NDOLE DO

0A640
CYRILLIC CAPITAL LETTER ZEMLYA

0A641
CYRILLIC SMALL LETTER ZEMLYA

0A642
CYRILLIC CAPITAL LETTER DZELO

0A643
CYRILLIC SMALL LETTER DZELO

0A644
CYRILLIC CAPITAL LETTER REVERSED DZE

0A645
CYRILLIC SMALL LETTER REVERSED DZE

0A646
CYRILLIC CAPITAL LETTER IOTA

0A647
CYRILLIC SMALL LETTER IOTA

0A648
CYRILLIC CAPITAL LETTER DJERV

0A649
CYRILLIC SMALL LETTER DJERV

0A64A
CYRILLIC CAPITAL LETTER MONOGRAPH UK

0A64B
CYRILLIC SMALL LETTER MONOGRAPH UK

0A64C
CYRILLIC CAPITAL LETTER BROAD OMEGA

0A64D
CYRILLIC SMALL LETTER BROAD OMEGA

0A64E
CYRILLIC CAPITAL LETTER NEUTRAL YER

0A64F
CYRILLIC SMALL LETTER NEUTRAL YER

0A650
CYRILLIC CAPITAL LETTER YERU WITH BACK YER

0A651
CYRILLIC SMALL LETTER YERU WITH BACK YER

0A652
CYRILLIC CAPITAL LETTER IOTIFIED YAT

0A653
CYRILLIC SMALL LETTER IOTIFIED YAT

0A654
CYRILLIC CAPITAL LETTER REVERSED YU

0A655
CYRILLIC SMALL LETTER REVERSED YU

0A656
CYRILLIC CAPITAL LETTER IOTIFIED A

0A657
CYRILLIC SMALL LETTER IOTIFIED A

0A658
CYRILLIC CAPITAL LETTER CLOSED LITTLE YUS

0A659
CYRILLIC SMALL LETTER CLOSED LITTLE YUS

0A65A
CYRILLIC CAPITAL LETTER BLENDED YUS

0A65B
CYRILLIC SMALL LETTER BLENDED YUS

0A65C
CYRILLIC CAPITAL LETTER IOTIFIED CLOSED LITTLE YUS

0A65D
CYRILLIC SMALL LETTER IOTIFIED CLOSED LITTLE YUS

0A65E
CYRILLIC CAPITAL LETTER YN

0A65F
CYRILLIC SMALL LETTER YN

0A660
CYRILLIC CAPITAL LETTER REVERSED TSE

0A661
CYRILLIC SMALL LETTER REVERSED TSE

0A662
CYRILLIC CAPITAL LETTER SOFT DE

0A663
CYRILLIC SMALL LETTER SOFT DE

0A664
CYRILLIC CAPITAL LETTER SOFT EL

0A665
CYRILLIC SMALL LETTER SOFT EL

0A666
CYRILLIC CAPITAL LETTER SOFT EM

0A667
CYRILLIC SMALL LETTER SOFT EM

0A668
CYRILLIC CAPITAL LETTER MONOCULAR O

0A669
CYRILLIC SMALL LETTER MONOCULAR O

0A66A
CYRILLIC CAPITAL LETTER BINOCULAR O

0A66B
CYRILLIC SMALL LETTER BINOCULAR O

0A66C
CYRILLIC CAPITAL LETTER DOUBLE MONOCULAR O

0A66D
CYRILLIC SMALL LETTER DOUBLE MONOCULAR O

0A66E
CYRILLIC LETTER MULTIOCULAR O

0A66F
COMBINING CYRILLIC VZMET

0A670
COMBINING CYRILLIC TEN MILLIONS SIGN

0A671
COMBINING CYRILLIC HUNDRED MILLIONS SIGN

0A672
COMBINING CYRILLIC THOUSAND MILLIONS SIGN

0A673
SLAVONIC ASTERISK

0A674
COMBINING CYRILLIC LETTER UKRAINIAN IE

0A675
COMBINING CYRILLIC LETTER I

0A676
COMBINING CYRILLIC LETTER YI

0A677
COMBINING CYRILLIC LETTER U

0A678
COMBINING CYRILLIC LETTER HARD SIGN

0A679
COMBINING CYRILLIC LETTER YERU

0A67A
COMBINING CYRILLIC LETTER SOFT SIGN

0A67B
COMBINING CYRILLIC LETTER OMEGA

0A67C
COMBINING CYRILLIC KAVYKA

0A67D
COMBINING CYRILLIC PAYEROK

0A67E
CYRILLIC KAVYKA

0A67F
CYRILLIC PAYEROK

0A680
CYRILLIC CAPITAL LETTER DWE

0A681
CYRILLIC SMALL LETTER DWE

0A682
CYRILLIC CAPITAL LETTER DZWE

0A683
CYRILLIC SMALL LETTER DZWE

0A684
CYRILLIC CAPITAL LETTER ZHWE

0A685
CYRILLIC SMALL LETTER ZHWE

0A686
CYRILLIC CAPITAL LETTER CCHE

0A687
CYRILLIC SMALL LETTER CCHE

0A688
CYRILLIC CAPITAL LETTER DZZE

0A689
CYRILLIC SMALL LETTER DZZE

0A68A
CYRILLIC CAPITAL LETTER TE WITH MIDDLE HOOK

0A68B
CYRILLIC SMALL LETTER TE WITH MIDDLE HOOK

0A68C
CYRILLIC CAPITAL LETTER TWE

0A68D
CYRILLIC SMALL LETTER TWE

0A68E
CYRILLIC CAPITAL LETTER TSWE

0A68F
CYRILLIC SMALL LETTER TSWE

0A690
CYRILLIC CAPITAL LETTER TSSE

0A691
CYRILLIC SMALL LETTER TSSE

0A692
CYRILLIC CAPITAL LETTER TCHE

0A693
CYRILLIC SMALL LETTER TCHE

0A694
CYRILLIC CAPITAL LETTER HWE

0A695
CYRILLIC SMALL LETTER HWE

0A696
CYRILLIC CAPITAL LETTER SHWE

0A697
CYRILLIC SMALL LETTER SHWE

0A698
CYRILLIC CAPITAL LETTER DOUBLE O

0A699
CYRILLIC SMALL LETTER DOUBLE O

0A69A
CYRILLIC CAPITAL LETTER CROSSED O

0A69B
CYRILLIC SMALL LETTER CROSSED O

0A69C
MODIFIER LETTER CYRILLIC HARD SIGN

0A69D
MODIFIER LETTER CYRILLIC SOFT SIGN

0A69E
COMBINING CYRILLIC LETTER EF

0A69F
COMBINING CYRILLIC LETTER IOTIFIED E

0A6A0
BAMUM LETTER A

0A6A1
BAMUM LETTER KA

0A6A2
BAMUM LETTER U

0A6A3
BAMUM LETTER KU

0A6A4
BAMUM LETTER EE

0A6A5
BAMUM LETTER REE

0A6A6
BAMUM LETTER TAE

0A6A7
BAMUM LETTER O

0A6A8
BAMUM LETTER NYI

0A6A9
BAMUM LETTER I

0A6AA
BAMUM LETTER LA

0A6AB
BAMUM LETTER PA

0A6AC
BAMUM LETTER RII

0A6AD
BAMUM LETTER RIEE

0A6AE
BAMUM LETTER LEEEE

0A6AF
BAMUM LETTER MEEEE

0A6B0
BAMUM LETTER TAA

0A6B1
BAMUM LETTER NDAA

0A6B2
BAMUM LETTER NJAEM

0A6B3
BAMUM LETTER M

0A6B4
BAMUM LETTER SUU

0A6B5
BAMUM LETTER MU

0A6B6
BAMUM LETTER SHII

0A6B7
BAMUM LETTER SI

0A6B8
BAMUM LETTER SHEUX

0A6B9
BAMUM LETTER SEUX

0A6BA
BAMUM LETTER KYEE

0A6BB
BAMUM LETTER KET

0A6BC
BAMUM LETTER NUAE

0A6BD
BAMUM LETTER NU

0A6BE
BAMUM LETTER NJUAE

0A6BF
BAMUM LETTER YOQ

0A6C0
BAMUM LETTER SHU

0A6C1
BAMUM LETTER YUQ

0A6C2
BAMUM LETTER YA

0A6C3
BAMUM LETTER NSHA

0A6C4
BAMUM LETTER KEUX

0A6C5
BAMUM LETTER PEUX

0A6C6
BAMUM LETTER NJEE

0A6C7
BAMUM LETTER NTEE

0A6C8
BAMUM LETTER PUE

0A6C9
BAMUM LETTER WUE

0A6CA
BAMUM LETTER PEE

0A6CB
BAMUM LETTER FEE

0A6CC
BAMUM LETTER RU

0A6CD
BAMUM LETTER LU

0A6CE
BAMUM LETTER MI

0A6CF
BAMUM LETTER NI

0A6D0
BAMUM LETTER REUX

0A6D1
BAMUM LETTER RAE

0A6D2
BAMUM LETTER KEN

0A6D3
BAMUM LETTER NGKWAEN

0A6D4
BAMUM LETTER NGGA

0A6D5
BAMUM LETTER NGA

0A6D6
BAMUM LETTER SHO

0A6D7
BAMUM LETTER PUAE

0A6D8
BAMUM LETTER FU

0A6D9
BAMUM LETTER FOM

0A6DA
BAMUM LETTER WA

0A6DB
BAMUM LETTER NA

0A6DC
BAMUM LETTER LI

0A6DD
BAMUM LETTER PI

0A6DE
BAMUM LETTER LOQ

0A6DF
BAMUM LETTER KO

0A6E0
BAMUM LETTER MBEN

0A6E1
BAMUM LETTER REN

0A6E2
BAMUM LETTER MEN

0A6E3
BAMUM LETTER MA

0A6E4
BAMUM LETTER TI

0A6E5
BAMUM LETTER KI

0A6E6
BAMUM LETTER MO

0A6E7
BAMUM LETTER MBAA

0A6E8
BAMUM LETTER TET

0A6E9
BAMUM LETTER KPA

0A6EA
BAMUM LETTER TEN

0A6EB
BAMUM LETTER NTUU

0A6EC
BAMUM LETTER SAMBA

0A6ED
BAMUM LETTER FAAMAE

0A6EE
BAMUM LETTER KOVUU

0A6EF
BAMUM LETTER KOGHOM

0A6F0
BAMUM COMBINING MARK KOQNDON

0A6F1
BAMUM COMBINING MARK TUKWENTIS

0A6F2
BAMUM NJAEMLI

0A6F3
BAMUM FULL STOP

0A6F4
BAMUM COLON

0A6F5
BAMUM COMMA

0A6F6
BAMUM SEMICOLON

0A6F7
BAMUM QUESTION MARK

0A700
MODIFIER LETTER CHINESE TONE YIN PING

0A701
MODIFIER LETTER CHINESE TONE YANG PING

0A702
MODIFIER LETTER CHINESE TONE YIN SHANG

0A703
MODIFIER LETTER CHINESE TONE YANG SHANG

0A704
MODIFIER LETTER CHINESE TONE YIN QU

0A705
MODIFIER LETTER CHINESE TONE YANG QU

0A706
MODIFIER LETTER CHINESE TONE YIN RU

0A707
MODIFIER LETTER CHINESE TONE YANG RU

0A708
MODIFIER LETTER EXTRA-HIGH DOTTED TONE BAR

0A709
MODIFIER LETTER HIGH DOTTED TONE BAR

0A70A
MODIFIER LETTER MID DOTTED TONE BAR

0A70B
MODIFIER LETTER LOW DOTTED TONE BAR

0A70C
MODIFIER LETTER EXTRA-LOW DOTTED TONE BAR

0A70D
MODIFIER LETTER EXTRA-HIGH DOTTED LEFT-STEM TONE BAR

0A70E
MODIFIER LETTER HIGH DOTTED LEFT-STEM TONE BAR

0A70F
MODIFIER LETTER MID DOTTED LEFT-STEM TONE BAR

0A710
MODIFIER LETTER LOW DOTTED LEFT-STEM TONE BAR

0A711
MODIFIER LETTER EXTRA-LOW DOTTED LEFT-STEM TONE BAR

0A712
MODIFIER LETTER EXTRA-HIGH LEFT-STEM TONE BAR

0A713
MODIFIER LETTER HIGH LEFT-STEM TONE BAR

0A714
MODIFIER LETTER MID LEFT-STEM TONE BAR

0A715
MODIFIER LETTER LOW LEFT-STEM TONE BAR

0A716
MODIFIER LETTER EXTRA-LOW LEFT-STEM TONE BAR

0A717
MODIFIER LETTER DOT VERTICAL BAR

0A718
MODIFIER LETTER DOT SLASH

0A719
MODIFIER LETTER DOT HORIZONTAL BAR

0A71A
MODIFIER LETTER LOWER RIGHT CORNER ANGLE

0A71B
MODIFIER LETTER RAISED UP ARROW

0A71C
MODIFIER LETTER RAISED DOWN ARROW

0A71D
MODIFIER LETTER RAISED EXCLAMATION MARK

0A71E
MODIFIER LETTER RAISED INVERTED EXCLAMATION MARK

0A71F
MODIFIER LETTER LOW INVERTED EXCLAMATION MARK

0A720
MODIFIER LETTER STRESS AND HIGH TONE

0A721
MODIFIER LETTER STRESS AND LOW TONE

0A722
LATIN CAPITAL LETTER EGYPTOLOGICAL ALEF

0A723
LATIN SMALL LETTER EGYPTOLOGICAL ALEF

0A724
LATIN CAPITAL LETTER EGYPTOLOGICAL AIN

0A725
LATIN SMALL LETTER EGYPTOLOGICAL AIN

0A726
LATIN CAPITAL LETTER HENG

0A727
LATIN SMALL LETTER HENG

0A728
LATIN CAPITAL LETTER TZ

0A729
LATIN SMALL LETTER TZ

0A72A
LATIN CAPITAL LETTER TRESILLO

0A72B
LATIN SMALL LETTER TRESILLO

0A72C
LATIN CAPITAL LETTER CUATRILLO

0A72D
LATIN SMALL LETTER CUATRILLO

0A72E
LATIN CAPITAL LETTER CUATRILLO WITH COMMA

0A72F
LATIN SMALL LETTER CUATRILLO WITH COMMA

0A730
LATIN LETTER SMALL CAPITAL F

0A731
LATIN LETTER SMALL CAPITAL S

0A732
LATIN CAPITAL LETTER AA

0A733
LATIN SMALL LETTER AA

0A734
LATIN CAPITAL LETTER AO

0A735
LATIN SMALL LETTER AO

0A736
LATIN CAPITAL LETTER AU

0A737
LATIN SMALL LETTER AU

0A738
LATIN CAPITAL LETTER AV

0A739
LATIN SMALL LETTER AV

0A73A
LATIN CAPITAL LETTER AV WITH HORIZONTAL BAR

0A73B
LATIN SMALL LETTER AV WITH HORIZONTAL BAR

0A73C
LATIN CAPITAL LETTER AY

0A73D
LATIN SMALL LETTER AY

0A73E
LATIN CAPITAL LETTER REVERSED C WITH DOT

0A73F
LATIN SMALL LETTER REVERSED C WITH DOT

0A740
LATIN CAPITAL LETTER K WITH STROKE

0A741
LATIN SMALL LETTER K WITH STROKE

0A742
LATIN CAPITAL LETTER K WITH DIAGONAL STROKE

0A743
LATIN SMALL LETTER K WITH DIAGONAL STROKE

0A744
LATIN CAPITAL LETTER K WITH STROKE AND DIAGONAL STROKE

0A745
LATIN SMALL LETTER K WITH STROKE AND DIAGONAL STROKE

0A746
LATIN CAPITAL LETTER BROKEN L

0A747
LATIN SMALL LETTER BROKEN L

0A748
LATIN CAPITAL LETTER L WITH HIGH STROKE

0A749
LATIN SMALL LETTER L WITH HIGH STROKE

0A74A
LATIN CAPITAL LETTER O WITH LONG STROKE OVERLAY

0A74B
LATIN SMALL LETTER O WITH LONG STROKE OVERLAY

0A74C
LATIN CAPITAL LETTER O WITH LOOP

0A74D
LATIN SMALL LETTER O WITH LOOP

0A74E
LATIN CAPITAL LETTER OO

0A74F
LATIN SMALL LETTER OO

0A750
LATIN CAPITAL LETTER P WITH STROKE THROUGH DESCENDER

0A751
LATIN SMALL LETTER P WITH STROKE THROUGH DESCENDER

0A752
LATIN CAPITAL LETTER P WITH FLOURISH

0A753
LATIN SMALL LETTER P WITH FLOURISH

0A754
LATIN CAPITAL LETTER P WITH SQUIRREL TAIL

0A755
LATIN SMALL LETTER P WITH SQUIRREL TAIL

0A756
LATIN CAPITAL LETTER Q WITH STROKE THROUGH DESCENDER

0A757
LATIN SMALL LETTER Q WITH STROKE THROUGH DESCENDER

0A758
LATIN CAPITAL LETTER Q WITH DIAGONAL STROKE

0A759
LATIN SMALL LETTER Q WITH DIAGONAL STROKE

0A75A
LATIN CAPITAL LETTER R ROTUNDA

0A75B
LATIN SMALL LETTER R ROTUNDA

0A75C
LATIN CAPITAL LETTER RUM ROTUNDA

0A75D
LATIN SMALL LETTER RUM ROTUNDA

0A75E
LATIN CAPITAL LETTER V WITH DIAGONAL STROKE

0A75F
LATIN SMALL LETTER V WITH DIAGONAL STROKE

0A760
LATIN CAPITAL LETTER VY

0A761
LATIN SMALL LETTER VY

0A762
LATIN CAPITAL LETTER VISIGOTHIC Z

0A763
LATIN SMALL LETTER VISIGOTHIC Z

0A764
LATIN CAPITAL LETTER THORN WITH STROKE

0A765
LATIN SMALL LETTER THORN WITH STROKE

0A766
LATIN CAPITAL LETTER THORN WITH STROKE THROUGH DESCENDER

0A767
LATIN SMALL LETTER THORN WITH STROKE THROUGH DESCENDER

0A768
LATIN CAPITAL LETTER VEND

0A769
LATIN SMALL LETTER VEND

0A76A
LATIN CAPITAL LETTER ET

0A76B
LATIN SMALL LETTER ET

0A76C
LATIN CAPITAL LETTER IS

0A76D
LATIN SMALL LETTER IS

0A76E
LATIN CAPITAL LETTER CON

0A76F
LATIN SMALL LETTER CON

0A770
MODIFIER LETTER US

0A771
LATIN SMALL LETTER DUM

0A772
LATIN SMALL LETTER LUM

0A773
LATIN SMALL LETTER MUM

0A774
LATIN SMALL LETTER NUM

0A775
LATIN SMALL LETTER RUM

0A776
LATIN LETTER SMALL CAPITAL RUM

0A777
LATIN SMALL LETTER TUM

0A778
LATIN SMALL LETTER UM

0A779
LATIN CAPITAL LETTER INSULAR D

0A77A
LATIN SMALL LETTER INSULAR D

0A77B
LATIN CAPITAL LETTER INSULAR F

0A77C
LATIN SMALL LETTER INSULAR F

0A77D
LATIN CAPITAL LETTER INSULAR G

0A77E
LATIN CAPITAL LETTER TURNED INSULAR G

0A77F
LATIN SMALL LETTER TURNED INSULAR G

0A780
LATIN CAPITAL LETTER TURNED L

0A781
LATIN SMALL LETTER TURNED L

0A782
LATIN CAPITAL LETTER INSULAR R

0A783
LATIN SMALL LETTER INSULAR R

0A784
LATIN CAPITAL LETTER INSULAR S

0A785
LATIN SMALL LETTER INSULAR S

0A786
LATIN CAPITAL LETTER INSULAR T

0A787
LATIN SMALL LETTER INSULAR T

0A788
MODIFIER LETTER LOW CIRCUMFLEX ACCENT

0A789
MODIFIER LETTER COLON

0A78A
MODIFIER LETTER SHORT EQUALS SIGN

0A78B
LATIN CAPITAL LETTER SALTILLO

0A78C
LATIN SMALL LETTER SALTILLO

0A78D
LATIN CAPITAL LETTER TURNED H

0A78E
LATIN SMALL LETTER L WITH RETROFLEX HOOK AND BELT

0A78F
LATIN LETTER SINOLOGICAL DOT

0A790
LATIN CAPITAL LETTER N WITH DESCENDER

0A791
LATIN SMALL LETTER N WITH DESCENDER

0A792
LATIN CAPITAL LETTER C WITH BAR

0A793
LATIN SMALL LETTER C WITH BAR

0A794
LATIN SMALL LETTER C WITH PALATAL HOOK

0A795
LATIN SMALL LETTER H WITH PALATAL HOOK

0A796
LATIN CAPITAL LETTER B WITH FLOURISH

0A797
LATIN SMALL LETTER B WITH FLOURISH

0A798
LATIN CAPITAL LETTER F WITH STROKE

0A799
LATIN SMALL LETTER F WITH STROKE

0A79A
LATIN CAPITAL LETTER VOLAPUK AE

0A79B
LATIN SMALL LETTER VOLAPUK AE

0A79C
LATIN CAPITAL LETTER VOLAPUK OE

0A79D
LATIN SMALL LETTER VOLAPUK OE

0A79E
LATIN CAPITAL LETTER VOLAPUK UE

0A79F
LATIN SMALL LETTER VOLAPUK UE

0A7A0
LATIN CAPITAL LETTER G WITH OBLIQUE STROKE

0A7A1
LATIN SMALL LETTER G WITH OBLIQUE STROKE

0A7A2
LATIN CAPITAL LETTER K WITH OBLIQUE STROKE

0A7A3
LATIN SMALL LETTER K WITH OBLIQUE STROKE

0A7A4
LATIN CAPITAL LETTER N WITH OBLIQUE STROKE

0A7A5
LATIN SMALL LETTER N WITH OBLIQUE STROKE

0A7A6
LATIN CAPITAL LETTER R WITH OBLIQUE STROKE

0A7A7
LATIN SMALL LETTER R WITH OBLIQUE STROKE

0A7A8
LATIN CAPITAL LETTER S WITH OBLIQUE STROKE

0A7A9
LATIN SMALL LETTER S WITH OBLIQUE STROKE

0A7AA
LATIN CAPITAL LETTER H WITH HOOK

0A7AB
LATIN CAPITAL LETTER REVERSED OPEN E

0A7AC
LATIN CAPITAL LETTER SCRIPT G

0A7AD
LATIN CAPITAL LETTER L WITH BELT

0A7AE
LATIN CAPITAL LETTER SMALL CAPITAL I

0A7AF
LATIN LETTER SMALL CAPITAL Q

0A7B0
LATIN CAPITAL LETTER TURNED K

0A7B1
LATIN CAPITAL LETTER TURNED T

0A7B2
LATIN CAPITAL LETTER J WITH CROSSED-TAIL

0A7B3
LATIN CAPITAL LETTER CHI

0A7B4
LATIN CAPITAL LETTER BETA

0A7B5
LATIN SMALL LETTER BETA

0A7B6
LATIN CAPITAL LETTER OMEGA

0A7B7
LATIN SMALL LETTER OMEGA

0A7B8
LATIN CAPITAL LETTER U WITH STROKE

0A7B9
LATIN SMALL LETTER U WITH STROKE

0A7BA
LATIN CAPITAL LETTER GLOTTAL A

0A7BB
LATIN SMALL LETTER GLOTTAL A

0A7BC
LATIN CAPITAL LETTER GLOTTAL I

0A7BD
LATIN SMALL LETTER GLOTTAL I

0A7BE
LATIN CAPITAL LETTER GLOTTAL U

0A7BF
LATIN SMALL LETTER GLOTTAL U

0A7C0
LATIN CAPITAL LETTER OLD POLISH O

0A7C1
LATIN SMALL LETTER OLD POLISH O

0A7C2
LATIN CAPITAL LETTER ANGLICANA W

0A7C3
LATIN SMALL LETTER ANGLICANA W

0A7C4
LATIN CAPITAL LETTER C WITH PALATAL HOOK

0A7C5
LATIN CAPITAL LETTER S WITH HOOK

0A7C6
LATIN CAPITAL LETTER Z WITH PALATAL HOOK

0A7C7
LATIN CAPITAL LETTER D WITH SHORT STROKE OVERLAY

0A7C8
LATIN SMALL LETTER D WITH SHORT STROKE OVERLAY

0A7C9
LATIN CAPITAL LETTER S WITH SHORT STROKE OVERLAY

0A7CA
LATIN SMALL LETTER S WITH SHORT STROKE OVERLAY

0A7CB
LATIN CAPITAL LETTER RAMS HORN

0A7CC
LATIN CAPITAL LETTER S WITH DIAGONAL STROKE

0A7CD
LATIN SMALL LETTER S WITH DIAGONAL STROKE

0A7CE
LATIN CAPITAL LETTER PHARYNGEAL VOICED FRICATIVE

0A7CF
LATIN SMALL LETTER PHARYNGEAL VOICED FRICATIVE

0A7D0
LATIN CAPITAL LETTER CLOSED INSULAR G

0A7D1
LATIN SMALL LETTER CLOSED INSULAR G

0A7D2
LATIN CAPITAL LETTER DOUBLE THORN

0A7D3
LATIN SMALL LETTER DOUBLE THORN

0A7D4
LATIN CAPITAL LETTER DOUBLE WYNN

0A7D5
LATIN SMALL LETTER DOUBLE WYNN

0A7D6
LATIN CAPITAL LETTER MIDDLE SCOTS S

0A7D7
LATIN SMALL LETTER MIDDLE SCOTS S

0A7D8
LATIN CAPITAL LETTER SIGMOID S

0A7D9
LATIN SMALL LETTER SIGMOID S

0A7DA
LATIN CAPITAL LETTER LAMBDA

0A7DB
LATIN SMALL LETTER LAMBDA

0A7DC
LATIN CAPITAL LETTER LAMBDA WITH STROKE

0A7F1
MODIFIER LETTER CAPITAL S

0A7F2
MODIFIER LETTER CAPITAL C

0A7F3
MODIFIER LETTER CAPITAL F

0A7F4
MODIFIER LETTER CAPITAL Q

0A7F5
LATIN CAPITAL LETTER REVERSED HALF H

0A7F6
LATIN SMALL LETTER REVERSED HALF H

0A7F7
LATIN EPIGRAPHIC LETTER SIDEWAYS I

0A7F8
MODIFIER LETTER CAPITAL H WITH STROKE

0A7F9
MODIFIER LETTER SMALL LIGATURE OE

0A7FA
LATIN LETTER SMALL CAPITAL TURNED M

0A7FB
LATIN EPIGRAPHIC LETTER REVERSED F

0A7FC
LATIN EPIGRAPHIC LETTER REVERSED P

0A7FD
LATIN EPIGRAPHIC LETTER INVERTED M

0A7FE
LATIN EPIGRAPHIC LETTER I LONGA

0A7FF
LATIN EPIGRAPHIC LETTER ARCHAIC M

0A800
SYLOTI NAGRI LETTER A

0A801
SYLOTI NAGRI LETTER I

0A802
SYLOTI NAGRI SIGN DVISVARA

0A803
SYLOTI NAGRI LETTER U

0A804
SYLOTI NAGRI LETTER E

0A805
SYLOTI NAGRI LETTER O

0A806
SYLOTI NAGRI SIGN HASANTA

0A807
SYLOTI NAGRI LETTER KO

0A808
SYLOTI NAGRI LETTER KHO

0A809
SYLOTI NAGRI LETTER GO

0A80A
SYLOTI NAGRI LETTER GHO

0A80B
SYLOTI NAGRI SIGN ANUSVARA

0A80C
SYLOTI NAGRI LETTER CO

0A80D
SYLOTI NAGRI LETTER CHO

0A80E
SYLOTI NAGRI LETTER JO

0A80F
SYLOTI NAGRI LETTER JHO

0A810
SYLOTI NAGRI LETTER TTO

0A811
SYLOTI NAGRI LETTER TTHO

0A812
SYLOTI NAGRI LETTER DDO

0A813
SYLOTI NAGRI LETTER DDHO

0A814
SYLOTI NAGRI LETTER TO

0A815
SYLOTI NAGRI LETTER THO

0A816
SYLOTI NAGRI LETTER DO

0A817
SYLOTI NAGRI LETTER DHO

0A818
SYLOTI NAGRI LETTER NO

0A819
SYLOTI NAGRI LETTER PO

0A81A
SYLOTI NAGRI LETTER PHO

0A81B
SYLOTI NAGRI LETTER BO

0A81C
SYLOTI NAGRI LETTER BHO

0A81D
SYLOTI NAGRI LETTER MO

0A81E
SYLOTI NAGRI LETTER RO

0A81F
SYLOTI NAGRI LETTER LO

0A820
SYLOTI NAGRI LETTER RRO

0A821
SYLOTI NAGRI LETTER SO

0A822
SYLOTI NAGRI LETTER HO

0A823
SYLOTI NAGRI VOWEL SIGN A

0A824
SYLOTI NAGRI VOWEL SIGN I

0A825
SYLOTI NAGRI VOWEL SIGN U

0A826
SYLOTI NAGRI VOWEL SIGN E

0A827
SYLOTI NAGRI VOWEL SIGN OO

0A828
SYLOTI NAGRI POETRY MARK-1

0A829
SYLOTI NAGRI POETRY MARK-2

0A82A
SYLOTI NAGRI POETRY MARK-3

0A82B
SYLOTI NAGRI POETRY MARK-4

0A82C
SYLOTI NAGRI SIGN ALTERNATE HASANTA

0A830
NORTH INDIC FRACTION ONE QUARTER

0A831
NORTH INDIC FRACTION ONE HALF

0A832
NORTH INDIC FRACTION THREE QUARTERS

0A833
NORTH INDIC FRACTION ONE SIXTEENTH

0A834
NORTH INDIC FRACTION ONE EIGHTH

0A835
NORTH INDIC FRACTION THREE SIXTEENTHS

0A836
NORTH INDIC QUARTER MARK

0A837
NORTH INDIC PLACEHOLDER MARK

0A838
NORTH INDIC RUPEE MARK

0A839
NORTH INDIC QUANTITY MARK

0A840
PHAGS-PA LETTER KA

0A841
PHAGS-PA LETTER KHA

0A842
PHAGS-PA LETTER GA

0A843
PHAGS-PA LETTER NGA

0A844
PHAGS-PA LETTER CA

0A845
PHAGS-PA LETTER CHA

0A846
PHAGS-PA LETTER JA

0A847
PHAGS-PA LETTER NYA

0A848
PHAGS-PA LETTER TA

0A849
PHAGS-PA LETTER THA

0A84A
PHAGS-PA LETTER DA

0A84B
PHAGS-PA LETTER NA

0A84C
PHAGS-PA LETTER PA

0A84D
PHAGS-PA LETTER PHA

0A84E
PHAGS-PA LETTER BA

0A84F
PHAGS-PA LETTER MA

0A850
PHAGS-PA LETTER TSA

0A851
PHAGS-PA LETTER TSHA

0A852
PHAGS-PA LETTER DZA

0A853
PHAGS-PA LETTER WA

0A854
PHAGS-PA LETTER ZHA

0A855
PHAGS-PA LETTER ZA

0A856
PHAGS-PA LETTER SMALL A

0A857
PHAGS-PA LETTER YA

0A858
PHAGS-PA LETTER RA

0A859
PHAGS-PA LETTER LA

0A85A
PHAGS-PA LETTER SHA

0A85B
PHAGS-PA LETTER SA

0A85C
PHAGS-PA LETTER HA

0A85D
PHAGS-PA LETTER A

0A85E
PHAGS-PA LETTER I

0A85F
PHAGS-PA LETTER U

0A860
PHAGS-PA LETTER E

0A861
PHAGS-PA LETTER O

0A862
PHAGS-PA LETTER QA

0A863
PHAGS-PA LETTER XA

0A864
PHAGS-PA LETTER FA

0A865
PHAGS-PA LETTER GGA

0A866
PHAGS-PA LETTER EE

0A867
PHAGS-PA SUBJOINED LETTER WA

0A868
PHAGS-PA SUBJOINED LETTER YA

0A869
PHAGS-PA LETTER TTA

0A86A
PHAGS-PA LETTER TTHA

0A86B
PHAGS-PA LETTER DDA

0A86C
PHAGS-PA LETTER NNA

0A86D
PHAGS-PA LETTER ALTERNATE YA

0A86E
PHAGS-PA LETTER VOICELESS SHA

0A86F
PHAGS-PA LETTER VOICED HA

0A870
PHAGS-PA LETTER ASPIRATED FA

0A871
PHAGS-PA SUBJOINED LETTER RA

0A872
PHAGS-PA SUPERFIXED LETTER RA

0A873
PHAGS-PA LETTER CANDRABINDU

0A874
PHAGS-PA SINGLE HEAD MARK

0A875
PHAGS-PA DOUBLE HEAD MARK

0A876
PHAGS-PA MARK SHAD

0A877
PHAGS-PA MARK DOUBLE SHAD

0A880
SAURASHTRA SIGN ANUSVARA

0A881
SAURASHTRA SIGN VISARGA

0A882
SAURASHTRA LETTER A

0A883
SAURASHTRA LETTER AA

0A884
SAURASHTRA LETTER I

0A885
SAURASHTRA LETTER II

0A886
SAURASHTRA LETTER U

0A887
SAURASHTRA LETTER UU

0A888
SAURASHTRA LETTER VOCALIC R

0A889
SAURASHTRA LETTER VOCALIC RR

0A88A
SAURASHTRA LETTER VOCALIC L

0A88B
SAURASHTRA LETTER VOCALIC LL

0A88C
SAURASHTRA LETTER E

0A88D
SAURASHTRA LETTER EE

0A88E
SAURASHTRA LETTER AI

0A88F
SAURASHTRA LETTER O

0A890
SAURASHTRA LETTER OO

0A891
SAURASHTRA LETTER AU

0A892
SAURASHTRA LETTER KA

0A893
SAURASHTRA LETTER KHA

0A894
SAURASHTRA LETTER GA

0A895
SAURASHTRA LETTER GHA

0A896
SAURASHTRA LETTER NGA

0A897
SAURASHTRA LETTER CA

0A898
SAURASHTRA LETTER CHA

0A899
SAURASHTRA LETTER JA

0A89A
SAURASHTRA LETTER JHA

0A89B
SAURASHTRA LETTER NYA

0A89C
SAURASHTRA LETTER TTA

0A89D
SAURASHTRA LETTER TTHA

0A89E
SAURASHTRA LETTER DDA

0A89F
SAURASHTRA LETTER DDHA

0A8A0
SAURASHTRA LETTER NNA

0A8A1
SAURASHTRA LETTER TA

0A8A2
SAURASHTRA LETTER THA

0A8A3
SAURASHTRA LETTER DA

0A8A4
SAURASHTRA LETTER DHA

0A8A5
SAURASHTRA LETTER NA

0A8A6
SAURASHTRA LETTER PA

0A8A7
SAURASHTRA LETTER PHA

0A8A8
SAURASHTRA LETTER BA

0A8A9
SAURASHTRA LETTER BHA

0A8AA
SAURASHTRA LETTER MA

0A8AB
SAURASHTRA LETTER YA

0A8AC
SAURASHTRA LETTER RA

0A8AD
SAURASHTRA LETTER LA

0A8AE
SAURASHTRA LETTER VA

0A8AF
SAURASHTRA LETTER SHA

0A8B0
SAURASHTRA LETTER SSA

0A8B1
SAURASHTRA LETTER SA

0A8B2
SAURASHTRA LETTER HA

0A8B3
SAURASHTRA LETTER LLA

0A8B4
SAURASHTRA CONSONANT SIGN HAARU

0A8B5
SAURASHTRA VOWEL SIGN AA

0A8B6
SAURASHTRA VOWEL SIGN I

0A8B7
SAURASHTRA VOWEL SIGN II

0A8B8
SAURASHTRA VOWEL SIGN U

0A8B9
SAURASHTRA VOWEL SIGN UU

0A8BA
SAURASHTRA VOWEL SIGN VOCALIC R

0A8BB
SAURASHTRA VOWEL SIGN VOCALIC RR

0A8BC
SAURASHTRA VOWEL SIGN VOCALIC L

0A8BD
SAURASHTRA VOWEL SIGN VOCALIC LL

0A8BE
SAURASHTRA VOWEL SIGN E

0A8BF
SAURASHTRA VOWEL SIGN EE

0A8C0
SAURASHTRA VOWEL SIGN AI

0A8C1
SAURASHTRA VOWEL SIGN O

0A8C2
SAURASHTRA VOWEL SIGN OO

0A8C3
SAURASHTRA VOWEL SIGN AU

0A8C4
SAURASHTRA SIGN VIRAMA

0A8C5
SAURASHTRA SIGN CANDRABINDU

0A8CE
SAURASHTRA DANDA

0A8CF
SAURASHTRA DOUBLE DANDA

0A8D0
SAURASHTRA DIGIT ZERO

0A8D1
SAURASHTRA DIGIT ONE

0A8D2
SAURASHTRA DIGIT TWO

0A8D3
SAURASHTRA DIGIT THREE

0A8D4
SAURASHTRA DIGIT FOUR

0A8D5
SAURASHTRA DIGIT FIVE

0A8D6
SAURASHTRA DIGIT SIX

0A8D7
SAURASHTRA DIGIT SEVEN

0A8D8
SAURASHTRA DIGIT EIGHT

0A8D9
SAURASHTRA DIGIT NINE

0A8E0
COMBINING DEVANAGARI DIGIT ZERO

0A8E1
COMBINING DEVANAGARI DIGIT ONE

0A8E2
COMBINING DEVANAGARI DIGIT TWO

0A8E3
COMBINING DEVANAGARI DIGIT THREE

0A8E4
COMBINING DEVANAGARI DIGIT FOUR

0A8E5
COMBINING DEVANAGARI DIGIT FIVE

0A8E6
COMBINING DEVANAGARI DIGIT SIX

0A8E7
COMBINING DEVANAGARI DIGIT SEVEN

0A8E8
COMBINING DEVANAGARI DIGIT EIGHT

0A8E9
COMBINING DEVANAGARI DIGIT NINE

0A8EA
COMBINING DEVANAGARI LETTER A

0A8EB
COMBINING DEVANAGARI LETTER U

0A8EC
COMBINING DEVANAGARI LETTER KA

0A8ED
COMBINING DEVANAGARI LETTER NA

0A8EE
COMBINING DEVANAGARI LETTER PA

0A8EF
COMBINING DEVANAGARI LETTER RA

0A8F0
COMBINING DEVANAGARI LETTER VI

0A8F1
COMBINING DEVANAGARI SIGN AVAGRAHA

0A8F2
DEVANAGARI SIGN SPACING CANDRABINDU

0A8F3
DEVANAGARI SIGN CANDRABINDU VIRAMA

0A8F4
DEVANAGARI SIGN DOUBLE CANDRABINDU VIRAMA

0A8F5
DEVANAGARI SIGN CANDRABINDU TWO

0A8F6
DEVANAGARI SIGN CANDRABINDU THREE

0A8F7
DEVANAGARI SIGN CANDRABINDU AVAGRAHA

0A8F8
DEVANAGARI SIGN PUSHPIKA

0A8F9
DEVANAGARI GAP FILLER

0A8FA
DEVANAGARI CARET

0A8FB
DEVANAGARI HEADSTROKE

0A8FC
DEVANAGARI SIGN SIDDHAM

0A8FD
DEVANAGARI JAIN OM

0A8FE
DEVANAGARI LETTER AY

0A8FF
DEVANAGARI VOWEL SIGN AY

0A900
KAYAH LI DIGIT ZERO

0A901
KAYAH LI DIGIT ONE

0A902
KAYAH LI DIGIT TWO

0A903
KAYAH LI DIGIT THREE

0A904
KAYAH LI DIGIT FOUR

0A905
KAYAH LI DIGIT FIVE

0A906
KAYAH LI DIGIT SIX

0A907
KAYAH LI DIGIT SEVEN

0A908
KAYAH LI DIGIT EIGHT

0A909
KAYAH LI DIGIT NINE

0A90A
KAYAH LI LETTER KA

0A90B
KAYAH LI LETTER KHA

0A90C
KAYAH LI LETTER GA

0A90D
KAYAH LI LETTER NGA

0A90E
KAYAH LI LETTER SA

0A90F
KAYAH LI LETTER SHA

0A910
KAYAH LI LETTER ZA

0A911
KAYAH LI LETTER NYA

0A912
KAYAH LI LETTER TA

0A913
KAYAH LI LETTER HTA

0A914
KAYAH LI LETTER NA

0A915
KAYAH LI LETTER PA

0A916
KAYAH LI LETTER PHA

0A917
KAYAH LI LETTER MA

0A918
KAYAH LI LETTER DA

0A919
KAYAH LI LETTER BA

0A91A
KAYAH LI LETTER RA

0A91B
KAYAH LI LETTER YA

0A91C
KAYAH LI LETTER LA

0A91D
KAYAH LI LETTER WA

0A91E
KAYAH LI LETTER THA

0A91F
KAYAH LI LETTER HA

0A920
KAYAH LI LETTER VA

0A921
KAYAH LI LETTER CA

0A922
KAYAH LI LETTER A

0A923
KAYAH LI LETTER OE

0A924
KAYAH LI LETTER I

0A925
KAYAH LI LETTER OO

0A926
KAYAH LI VOWEL UE

0A927
KAYAH LI VOWEL E

0A928
KAYAH LI VOWEL U

0A929
KAYAH LI VOWEL EE

0A92A
KAYAH LI VOWEL O

0A92B
KAYAH LI TONE PLOPHU

0A92C
KAYAH LI TONE CALYA

0A92D
KAYAH LI TONE CALYA PLOPHU

0A92E
KAYAH LI SIGN CWI

0A92F
KAYAH LI SIGN SHYA

0A930
REJANG LETTER KA

0A931
REJANG LETTER GA

0A932
REJANG LETTER NGA

0A933
REJANG LETTER TA

0A934
REJANG LETTER DA

0A935
REJANG LETTER NA

0A936
REJANG LETTER PA

0A937
REJANG LETTER BA

0A938
REJANG LETTER MA

0A939
REJANG LETTER CA

0A93A
REJANG LETTER JA

0A93B
REJANG LETTER NYA

0A93C
REJANG LETTER SA

0A93D
REJANG LETTER RA

0A93E
REJANG LETTER LA

0A93F
REJANG LETTER YA

0A940
REJANG LETTER WA

0A941
REJANG LETTER HA

0A942
REJANG LETTER MBA

0A943
REJANG LETTER NGGA

0A944
REJANG LETTER NDA

0A945
REJANG LETTER NYJA

0A946
REJANG LETTER A

0A947
REJANG VOWEL SIGN I

0A948
REJANG VOWEL SIGN U

0A949
REJANG VOWEL SIGN E

0A94A
REJANG VOWEL SIGN AI

0A94B
REJANG VOWEL SIGN O

0A94C
REJANG VOWEL SIGN AU

0A94D
REJANG VOWEL SIGN EU

0A94E
REJANG VOWEL SIGN EA

0A94F
REJANG CONSONANT SIGN NG

0A950
REJANG CONSONANT SIGN N

0A951
REJANG CONSONANT SIGN R

0A952
REJANG CONSONANT SIGN H

0A953
REJANG VIRAMA

0A95F
REJANG SECTION MARK

0A960
HANGUL CHOSEONG TIKEUT-MIEUM

0A961
HANGUL CHOSEONG TIKEUT-PIEUP

0A962
HANGUL CHOSEONG TIKEUT-SIOS

0A963
HANGUL CHOSEONG TIKEUT-CIEUC

0A964
HANGUL CHOSEONG RIEUL-KIYEOK

0A965
HANGUL CHOSEONG RIEUL-SSANGKIYEOK

0A966
HANGUL CHOSEONG RIEUL-TIKEUT

0A967
HANGUL CHOSEONG RIEUL-SSANGTIKEUT

0A968
HANGUL CHOSEONG RIEUL-MIEUM

0A969
HANGUL CHOSEONG RIEUL-PIEUP

0A96A
HANGUL CHOSEONG RIEUL-SSANGPIEUP

0A96B
HANGUL CHOSEONG RIEUL-KAPYEOUNPIEUP

0A96C
HANGUL CHOSEONG RIEUL-SIOS

0A96D
HANGUL CHOSEONG RIEUL-CIEUC

0A96E
HANGUL CHOSEONG RIEUL-KHIEUKH

0A96F
HANGUL CHOSEONG MIEUM-KIYEOK

0A970
HANGUL CHOSEONG MIEUM-TIKEUT

0A971
HANGUL CHOSEONG MIEUM-SIOS

0A972
HANGUL CHOSEONG PIEUP-SIOS-THIEUTH

0A973
HANGUL CHOSEONG PIEUP-KHIEUKH

0A974
HANGUL CHOSEONG PIEUP-HIEUH

0A975
HANGUL CHOSEONG SSANGSIOS-PIEUP

0A976
HANGUL CHOSEONG IEUNG-RIEUL

0A977
HANGUL CHOSEONG IEUNG-HIEUH

0A978
HANGUL CHOSEONG SSANGCIEUC-HIEUH

0A979
HANGUL CHOSEONG SSANGTHIEUTH

0A97A
HANGUL CHOSEONG PHIEUPH-HIEUH

0A97B
HANGUL CHOSEONG HIEUH-SIOS

0A97C
HANGUL CHOSEONG SSANGYEORINHIEUH

0A980
JAVANESE SIGN PANYANGGA

0A981
JAVANESE SIGN CECAK

0A982
JAVANESE SIGN LAYAR

0A983
JAVANESE SIGN WIGNYAN

0A984
JAVANESE LETTER A

0A985
JAVANESE LETTER I KAWI

0A986
JAVANESE LETTER I

0A987
JAVANESE LETTER II

0A988
JAVANESE LETTER U

0A989
JAVANESE LETTER PA CEREK

0A98A
JAVANESE LETTER NGA LELET

0A98B
JAVANESE LETTER NGA LELET RASWADI

0A98C
JAVANESE LETTER E

0A98D
JAVANESE LETTER AI

0A98E
JAVANESE LETTER O

0A98F
JAVANESE LETTER KA

0A990
JAVANESE LETTER KA SASAK

0A991
JAVANESE LETTER KA MURDA

0A992
JAVANESE LETTER GA

0A993
JAVANESE LETTER GA MURDA

0A994
JAVANESE LETTER NGA

0A995
JAVANESE LETTER CA

0A996
JAVANESE LETTER CA MURDA

0A997
JAVANESE LETTER JA

0A998
JAVANESE LETTER NYA MURDA

0A999
JAVANESE LETTER JA MAHAPRANA

0A99A
JAVANESE LETTER NYA

0A99B
JAVANESE LETTER TTA

0A99C
JAVANESE LETTER TTA MAHAPRANA

0A99D
JAVANESE LETTER DDA

0A99E
JAVANESE LETTER DDA MAHAPRANA

0A99F
JAVANESE LETTER NA MURDA

0A9A0
JAVANESE LETTER TA

0A9A1
JAVANESE LETTER TA MURDA

0A9A2
JAVANESE LETTER DA

0A9A3
JAVANESE LETTER DA MAHAPRANA

0A9A4
JAVANESE LETTER NA

0A9A5
JAVANESE LETTER PA

0A9A6
JAVANESE LETTER PA MURDA

0A9A7
JAVANESE LETTER BA

0A9A8
JAVANESE LETTER BA MURDA

0A9A9
JAVANESE LETTER MA

0A9AA
JAVANESE LETTER YA

0A9AB
JAVANESE LETTER RA

0A9AC
JAVANESE LETTER RA AGUNG

0A9AD
JAVANESE LETTER LA

0A9AE
JAVANESE LETTER WA

0A9AF
JAVANESE LETTER SA MURDA

0A9B0
JAVANESE LETTER SA MAHAPRANA

0A9B1
JAVANESE LETTER SA

0A9B2
JAVANESE LETTER HA

0A9B3
JAVANESE SIGN CECAK TELU

0A9B4
JAVANESE VOWEL SIGN TARUNG

0A9B5
JAVANESE VOWEL SIGN TOLONG

0A9B6
JAVANESE VOWEL SIGN WULU

0A9B7
JAVANESE VOWEL SIGN WULU MELIK

0A9B8
JAVANESE VOWEL SIGN SUKU

0A9B9
JAVANESE VOWEL SIGN SUKU MENDUT

0A9BA
JAVANESE VOWEL SIGN TALING

0A9BB
JAVANESE VOWEL SIGN DIRGA MURE

0A9BC
JAVANESE VOWEL SIGN PEPET

0A9BD
JAVANESE CONSONANT SIGN KERET

0A9BE
JAVANESE CONSONANT SIGN PENGKAL

0A9BF
JAVANESE CONSONANT SIGN CAKRA

0A9C0
JAVANESE PANGKON

0A9C1
JAVANESE LEFT RERENGGAN

0A9C2
JAVANESE RIGHT RERENGGAN

0A9C3
JAVANESE PADA ANDAP

0A9C4
JAVANESE PADA MADYA

0A9C5
JAVANESE PADA LUHUR

0A9C6
JAVANESE PADA WINDU

0A9C7
JAVANESE PADA PANGKAT

0A9C8
JAVANESE PADA LINGSA

0A9C9
JAVANESE PADA LUNGSI

0A9CA
JAVANESE PADA ADEG

0A9CB
JAVANESE PADA ADEG ADEG

0A9CC
JAVANESE PADA PISELEH

0A9CD
JAVANESE TURNED PADA PISELEH

0A9CF
JAVANESE PANGRANGKEP

0A9D0
JAVANESE DIGIT ZERO

0A9D1
JAVANESE DIGIT ONE

0A9D2
JAVANESE DIGIT TWO

0A9D3
JAVANESE DIGIT THREE

0A9D4
JAVANESE DIGIT FOUR

0A9D5
JAVANESE DIGIT FIVE

0A9D6
JAVANESE DIGIT SIX

0A9D7
JAVANESE DIGIT SEVEN

0A9D8
JAVANESE DIGIT EIGHT

0A9D9
JAVANESE DIGIT NINE

0A9DE
JAVANESE PADA TIRTA TUMETES

0A9DF
JAVANESE PADA ISEN-ISEN

0A9E0
MYANMAR LETTER SHAN GHA

0A9E1
MYANMAR LETTER SHAN CHA

0A9E2
MYANMAR LETTER SHAN JHA

0A9E3
MYANMAR LETTER SHAN NNA

0A9E4
MYANMAR LETTER SHAN BHA

0A9E5
MYANMAR SIGN SHAN SAW

0A9E6
MYANMAR MODIFIER LETTER SHAN REDUPLICATION

0A9E7
MYANMAR LETTER TAI LAING NYA

0A9E8
MYANMAR LETTER TAI LAING FA

0A9E9
MYANMAR LETTER TAI LAING GA

0A9EA
MYANMAR LETTER TAI LAING GHA

0A9EB
MYANMAR LETTER TAI LAING JA

0A9EC
MYANMAR LETTER TAI LAING JHA

0A9ED
MYANMAR LETTER TAI LAING DDA

0A9EE
MYANMAR LETTER TAI LAING DDHA

0A9EF
MYANMAR LETTER TAI LAING NNA

0A9F0
MYANMAR TAI LAING DIGIT ZERO

0A9F1
MYANMAR TAI LAING DIGIT ONE

0A9F2
MYANMAR TAI LAING DIGIT TWO

0A9F3
MYANMAR TAI LAING DIGIT THREE

0A9F4
MYANMAR TAI LAING DIGIT FOUR

0A9F5
MYANMAR TAI LAING DIGIT FIVE

0A9F6
MYANMAR TAI LAING DIGIT SIX

0A9F7
MYANMAR TAI LAING DIGIT SEVEN

0A9F8
MYANMAR TAI LAING DIGIT EIGHT

0A9F9
MYANMAR TAI LAING DIGIT NINE

0A9FA
MYANMAR LETTER TAI LAING LLA

0A9FB
MYANMAR LETTER TAI LAING DA

0A9FC
MYANMAR LETTER TAI LAING DHA

0A9FD
MYANMAR LETTER TAI LAING BA

0A9FE
MYANMAR LETTER TAI LAING BHA

0AA00
CHAM LETTER A

0AA01
CHAM LETTER I

0AA02
CHAM LETTER U

0AA03
CHAM LETTER E

0AA04
CHAM LETTER AI

0AA05
CHAM LETTER O

0AA06
CHAM LETTER KA

0AA07
CHAM LETTER KHA

0AA08
CHAM LETTER GA

0AA09
CHAM LETTER GHA

0AA0A
CHAM LETTER NGUE

0AA0B
CHAM LETTER NGA

0AA0C
CHAM LETTER CHA

0AA0D
CHAM LETTER CHHA

0AA0E
CHAM LETTER JA

0AA0F
CHAM LETTER JHA

0AA10
CHAM LETTER NHUE

0AA11
CHAM LETTER NHA

0AA12
CHAM LETTER NHJA

0AA13
CHAM LETTER TA

0AA14
CHAM LETTER THA

0AA15
CHAM LETTER DA

0AA16
CHAM LETTER DHA

0AA17
CHAM LETTER NUE

0AA18
CHAM LETTER NA

0AA19
CHAM LETTER DDA

0AA1A
CHAM LETTER PA

0AA1B
CHAM LETTER PPA

0AA1C
CHAM LETTER PHA

0AA1D
CHAM LETTER BA

0AA1E
CHAM LETTER BHA

0AA1F
CHAM LETTER MUE

0AA20
CHAM LETTER MA

0AA21
CHAM LETTER BBA

0AA22
CHAM LETTER YA

0AA23
CHAM LETTER RA

0AA24
CHAM LETTER LA

0AA25
CHAM LETTER VA

0AA26
CHAM LETTER SSA

0AA27
CHAM LETTER SA

0AA28
CHAM LETTER HA

0AA29
CHAM VOWEL SIGN AA

0AA2A
CHAM VOWEL SIGN I

0AA2B
CHAM VOWEL SIGN II

0AA2C
CHAM VOWEL SIGN EI

0AA2D
CHAM VOWEL SIGN U

0AA2E
CHAM VOWEL SIGN OE

0AA2F
CHAM VOWEL SIGN O

0AA30
CHAM VOWEL SIGN AI

0AA31
CHAM VOWEL SIGN AU

0AA32
CHAM VOWEL SIGN UE

0AA33
CHAM CONSONANT SIGN YA

0AA34
CHAM CONSONANT SIGN RA

0AA35
CHAM CONSONANT SIGN LA

0AA36
CHAM CONSONANT SIGN WA

0AA40
CHAM LETTER FINAL K

0AA41
CHAM LETTER FINAL G

0AA42
CHAM LETTER FINAL NG

0AA43
CHAM CONSONANT SIGN FINAL NG

0AA44
CHAM LETTER FINAL CH

0AA45
CHAM LETTER FINAL T

0AA46
CHAM LETTER FINAL N

0AA47
CHAM LETTER FINAL P

0AA48
CHAM LETTER FINAL Y

0AA49
CHAM LETTER FINAL R

0AA4A
CHAM LETTER FINAL L

0AA4B
CHAM LETTER FINAL SS

0AA4C
CHAM CONSONANT SIGN FINAL M

0AA4D
CHAM CONSONANT SIGN FINAL H

0AA50
CHAM DIGIT ZERO

0AA51
CHAM DIGIT ONE

0AA52
CHAM DIGIT TWO

0AA53
CHAM DIGIT THREE

0AA54
CHAM DIGIT FOUR

0AA55
CHAM DIGIT FIVE

0AA56
CHAM DIGIT SIX

0AA57
CHAM DIGIT SEVEN

0AA58
CHAM DIGIT EIGHT

0AA59
CHAM DIGIT NINE

0AA5C
CHAM PUNCTUATION SPIRAL

0AA5D
CHAM PUNCTUATION DANDA

0AA5E
CHAM PUNCTUATION DOUBLE DANDA

0AA5F
CHAM PUNCTUATION TRIPLE DANDA

0AA60
MYANMAR LETTER KHAMTI GA

0AA61
MYANMAR LETTER KHAMTI CA

0AA62
MYANMAR LETTER KHAMTI CHA

0AA63
MYANMAR LETTER KHAMTI JA

0AA64
MYANMAR LETTER KHAMTI JHA

0AA65
MYANMAR LETTER KHAMTI NYA

0AA66
MYANMAR LETTER KHAMTI TTA

0AA67
MYANMAR LETTER KHAMTI TTHA

0AA68
MYANMAR LETTER KHAMTI DDA

0AA69
MYANMAR LETTER KHAMTI DDHA

0AA6A
MYANMAR LETTER KHAMTI DHA

0AA6B
MYANMAR LETTER KHAMTI NA

0AA6C
MYANMAR LETTER KHAMTI SA

0AA6D
MYANMAR LETTER KHAMTI HA

0AA6E
MYANMAR LETTER KHAMTI LLA

0AA6E
MYANMAR LETTER KHAMTI HHA

0AA6F
MYANMAR LETTER KHAMTI FA

0AA70
MYANMAR MODIFIER LETTER KHAMTI REDUPLICATION

0AA71
MYANMAR LETTER KHAMTI XA

0AA72
MYANMAR LETTER KHAMTI ZA

0AA73
MYANMAR LETTER KHAMTI RA

0AA74
MYANMAR LOGOGRAM KHAMTI OAY

0AA75
MYANMAR LOGOGRAM KHAMTI QN

0AA76
MYANMAR LOGOGRAM KHAMTI HM

0AA77
MYANMAR SYMBOL AITON EXCLAMATION

0AA78
MYANMAR SYMBOL AITON ONE

0AA79
MYANMAR SYMBOL AITON TWO

0AA7A
MYANMAR LETTER AITON RA

0AA7B
MYANMAR SIGN PAO KAREN TONE

0AA7C
MYANMAR SIGN TAI LAING TONE-2

0AA7D
MYANMAR SIGN TAI LAING TONE-5

0AA7E
MYANMAR LETTER SHWE PALAUNG CHA

0AA7F
MYANMAR LETTER SHWE PALAUNG SHA

0AA80
TAI VIET LETTER LOW KO

0AA81
TAI VIET LETTER HIGH KO

0AA82
TAI VIET LETTER LOW KHO

0AA83
TAI VIET LETTER HIGH KHO

0AA84
TAI VIET LETTER LOW KHHO

0AA85
TAI VIET LETTER HIGH KHHO

0AA86
TAI VIET LETTER LOW GO

0AA87
TAI VIET LETTER HIGH GO

0AA88
TAI VIET LETTER LOW NGO

0AA89
TAI VIET LETTER HIGH NGO

0AA8A
TAI VIET LETTER LOW CO

0AA8B
TAI VIET LETTER HIGH CO

0AA8C
TAI VIET LETTER LOW CHO

0AA8D
TAI VIET LETTER HIGH CHO

0AA8E
TAI VIET LETTER LOW SO

0AA8F
TAI VIET LETTER HIGH SO

0AA90
TAI VIET LETTER LOW NYO

0AA91
TAI VIET LETTER HIGH NYO

0AA92
TAI VIET LETTER LOW DO

0AA93
TAI VIET LETTER HIGH DO

0AA94
TAI VIET LETTER LOW TO

0AA95
TAI VIET LETTER HIGH TO

0AA96
TAI VIET LETTER LOW THO

0AA97
TAI VIET LETTER HIGH THO

0AA98
TAI VIET LETTER LOW NO

0AA99
TAI VIET LETTER HIGH NO

0AA9A
TAI VIET LETTER LOW BO

0AA9B
TAI VIET LETTER HIGH BO

0AA9C
TAI VIET LETTER LOW PO

0AA9D
TAI VIET LETTER HIGH PO

0AA9E
TAI VIET LETTER LOW PHO

0AA9F
TAI VIET LETTER HIGH PHO

0AAA0
TAI VIET LETTER LOW FO

0AAA1
TAI VIET LETTER HIGH FO

0AAA2
TAI VIET LETTER LOW MO

0AAA3
TAI VIET LETTER HIGH MO

0AAA4
TAI VIET LETTER LOW YO

0AAA5
TAI VIET LETTER HIGH YO

0AAA6
TAI VIET LETTER LOW RO

0AAA7
TAI VIET LETTER HIGH RO

0AAA8
TAI VIET LETTER LOW LO

0AAA9
TAI VIET LETTER HIGH LO

0AAAA
TAI VIET LETTER LOW VO

0AAAB
TAI VIET LETTER HIGH VO

0AAAC
TAI VIET LETTER LOW HO

0AAAD
TAI VIET LETTER HIGH HO

0AAAE
TAI VIET LETTER LOW O

0AAAF
TAI VIET LETTER HIGH O

0AAB0
TAI VIET MAI KANG

0AAB1
TAI VIET VOWEL AA

0AAB2
TAI VIET VOWEL I

0AAB3
TAI VIET VOWEL UE

0AAB4
TAI VIET VOWEL U

0AAB5
TAI VIET VOWEL E

0AAB6
TAI VIET VOWEL O

0AAB7
TAI VIET MAI KHIT

0AAB8
TAI VIET VOWEL IA

0AAB9
TAI VIET VOWEL UEA

0AABA
TAI VIET VOWEL UA

0AABB
TAI VIET VOWEL AUE

0AABC
TAI VIET VOWEL AY

0AABD
TAI VIET VOWEL AN

0AABE
TAI VIET VOWEL AM

0AABF
TAI VIET TONE MAI EK

0AAC0
TAI VIET TONE MAI NUENG

0AAC1
TAI VIET TONE MAI THO

0AAC2
TAI VIET TONE MAI SONG

0AADB
TAI VIET SYMBOL KON

0AADC
TAI VIET SYMBOL NUENG

0AADD
TAI VIET SYMBOL SAM

0AADE
TAI VIET SYMBOL HO HOI

0AADF
TAI VIET SYMBOL KOI KOI

0AAE0
MEETEI MAYEK LETTER E

0AAE1
MEETEI MAYEK LETTER O

0AAE2
MEETEI MAYEK LETTER CHA

0AAE3
MEETEI MAYEK LETTER NYA

0AAE4
MEETEI MAYEK LETTER TTA

0AAE5
MEETEI MAYEK LETTER TTHA

0AAE6
MEETEI MAYEK LETTER DDA

0AAE7
MEETEI MAYEK LETTER DDHA

0AAE8
MEETEI MAYEK LETTER NNA

0AAE9
MEETEI MAYEK LETTER SHA

0AAEA
MEETEI MAYEK LETTER SSA

0AAEB
MEETEI MAYEK VOWEL SIGN II

0AAEC
MEETEI MAYEK VOWEL SIGN UU

0AAED
MEETEI MAYEK VOWEL SIGN AAI

0AAEE
MEETEI MAYEK VOWEL SIGN AU

0AAEF
MEETEI MAYEK VOWEL SIGN AAU

0AAF0
MEETEI MAYEK CHEIKHAN

0AAF1
MEETEI MAYEK AHANG KHUDAM

0AAF2
MEETEI MAYEK ANJI

0AAF3
MEETEI MAYEK SYLLABLE REPETITION MARK

0AAF4
MEETEI MAYEK WORD REPETITION MARK

0AAF5
MEETEI MAYEK VOWEL SIGN VISARGA

0AAF6
MEETEI MAYEK VIRAMA

0AB01
ETHIOPIC SYLLABLE TTHU

0AB02
ETHIOPIC SYLLABLE TTHI

0AB03
ETHIOPIC SYLLABLE TTHAA

0AB04
ETHIOPIC SYLLABLE TTHEE

0AB05
ETHIOPIC SYLLABLE TTHE

0AB06
ETHIOPIC SYLLABLE TTHO

0AB09
ETHIOPIC SYLLABLE DDHU

0AB0A
ETHIOPIC SYLLABLE DDHI

0AB0B
ETHIOPIC SYLLABLE DDHAA

0AB0C
ETHIOPIC SYLLABLE DDHEE

0AB0D
ETHIOPIC SYLLABLE DDHE

0AB0E
ETHIOPIC SYLLABLE DDHO

0AB11
ETHIOPIC SYLLABLE DZU

0AB12
ETHIOPIC SYLLABLE DZI

0AB13
ETHIOPIC SYLLABLE DZAA

0AB14
ETHIOPIC SYLLABLE DZEE

0AB15
ETHIOPIC SYLLABLE DZE

0AB16
ETHIOPIC SYLLABLE DZO

0AB20
ETHIOPIC SYLLABLE CCHHA

0AB21
ETHIOPIC SYLLABLE CCHHU

0AB22
ETHIOPIC SYLLABLE CCHHI

0AB23
ETHIOPIC SYLLABLE CCHHAA

0AB24
ETHIOPIC SYLLABLE CCHHEE

0AB25
ETHIOPIC SYLLABLE CCHHE

0AB26
ETHIOPIC SYLLABLE CCHHO

0AB28
ETHIOPIC SYLLABLE BBA

0AB29
ETHIOPIC SYLLABLE BBU

0AB2A
ETHIOPIC SYLLABLE BBI

0AB2B
ETHIOPIC SYLLABLE BBAA

0AB2C
ETHIOPIC SYLLABLE BBEE

0AB2D
ETHIOPIC SYLLABLE BBE

0AB2E
ETHIOPIC SYLLABLE BBO

0AB30
LATIN SMALL LETTER BARRED ALPHA

0AB31
LATIN SMALL LETTER A REVERSED-SCHWA

0AB32
LATIN SMALL LETTER BLACKLETTER E

0AB33
LATIN SMALL LETTER BARRED E

0AB34
LATIN SMALL LETTER E WITH FLOURISH

0AB35
LATIN SMALL LETTER LENIS F

0AB36
LATIN SMALL LETTER SCRIPT G WITH CROSSED-TAIL

0AB37
LATIN SMALL LETTER L WITH INVERTED LAZY S

0AB38
LATIN SMALL LETTER L WITH DOUBLE MIDDLE TILDE

0AB39
LATIN SMALL LETTER L WITH MIDDLE RING

0AB3A
LATIN SMALL LETTER M WITH CROSSED-TAIL

0AB3B
LATIN SMALL LETTER N WITH CROSSED-TAIL

0AB3C
LATIN SMALL LETTER ENG WITH CROSSED-TAIL

0AB3D
LATIN SMALL LETTER BLACKLETTER O

0AB3E
LATIN SMALL LETTER BLACKLETTER O WITH STROKE

0AB3F
LATIN SMALL LETTER OPEN O WITH STROKE

0AB40
LATIN SMALL LETTER INVERTED OE

0AB41
LATIN SMALL LETTER TURNED OE WITH STROKE

0AB42
LATIN SMALL LETTER TURNED OE WITH HORIZONTAL STROKE

0AB43
LATIN SMALL LETTER TURNED O OPEN-O

0AB44
LATIN SMALL LETTER TURNED O OPEN-O WITH STROKE

0AB45
LATIN SMALL LETTER STIRRUP R

0AB46
LATIN LETTER SMALL CAPITAL R WITH RIGHT LEG

0AB47
LATIN SMALL LETTER R WITHOUT HANDLE

0AB48
LATIN SMALL LETTER DOUBLE R

0AB49
LATIN SMALL LETTER R WITH CROSSED-TAIL

0AB4A
LATIN SMALL LETTER DOUBLE R WITH CROSSED-TAIL

0AB4B
LATIN SMALL LETTER SCRIPT R

0AB4C
LATIN SMALL LETTER SCRIPT R WITH RING

0AB4D
LATIN SMALL LETTER BASELINE ESH

0AB4E
LATIN SMALL LETTER U WITH SHORT RIGHT LEG

0AB4F
LATIN SMALL LETTER U BAR WITH SHORT RIGHT LEG

0AB50
LATIN SMALL LETTER UI

0AB51
LATIN SMALL LETTER TURNED UI

0AB52
LATIN SMALL LETTER U WITH LEFT HOOK

0AB53
LATIN SMALL LETTER CHI

0AB54
LATIN SMALL LETTER CHI WITH LOW RIGHT RING

0AB55
LATIN SMALL LETTER CHI WITH LOW LEFT SERIF

0AB56
LATIN SMALL LETTER X WITH LOW RIGHT RING

0AB57
LATIN SMALL LETTER X WITH LONG LEFT LEG

0AB58
LATIN SMALL LETTER X WITH LONG LEFT LEG AND LOW RIGHT RING

0AB59
LATIN SMALL LETTER X WITH LONG LEFT LEG WITH SERIF

0AB5A
LATIN SMALL LETTER Y WITH SHORT RIGHT LEG

0AB5B
MODIFIER BREVE WITH INVERTED BREVE

0AB5C
MODIFIER LETTER SMALL HENG

0AB5D
MODIFIER LETTER SMALL L WITH INVERTED LAZY S

0AB5E
MODIFIER LETTER SMALL L WITH MIDDLE TILDE

0AB5F
MODIFIER LETTER SMALL U WITH LEFT HOOK

0AB60
LATIN SMALL LETTER SAKHA YAT

0AB61
LATIN SMALL LETTER IOTIFIED E

0AB62
LATIN SMALL LETTER OPEN OE

0AB63
LATIN SMALL LETTER UO

0AB64
LATIN SMALL LETTER INVERTED ALPHA

0AB65
GREEK LETTER SMALL CAPITAL OMEGA

0AB66
LATIN SMALL LETTER DZ DIGRAPH WITH RETROFLEX HOOK

0AB67
LATIN SMALL LETTER TS DIGRAPH WITH RETROFLEX HOOK

0AB68
LATIN SMALL LETTER TURNED R WITH MIDDLE TILDE

0AB69
MODIFIER LETTER SMALL TURNED W

0AB6A
MODIFIER LETTER LEFT TACK

0AB6B
MODIFIER LETTER RIGHT TACK

0AB70
CHEROKEE SMALL LETTER A

0AB71
CHEROKEE SMALL LETTER E

0AB72
CHEROKEE SMALL LETTER I

0AB73
CHEROKEE SMALL LETTER O

0AB74
CHEROKEE SMALL LETTER U

0AB75
CHEROKEE SMALL LETTER V

0AB76
CHEROKEE SMALL LETTER GA

0AB77
CHEROKEE SMALL LETTER KA

0AB78
CHEROKEE SMALL LETTER GE

0AB79
CHEROKEE SMALL LETTER GI

0AB7A
CHEROKEE SMALL LETTER GO

0AB7B
CHEROKEE SMALL LETTER GU

0AB7C
CHEROKEE SMALL LETTER GV

0AB7D
CHEROKEE SMALL LETTER HA

0AB7E
CHEROKEE SMALL LETTER HE

0AB7F
CHEROKEE SMALL LETTER HI

0AB80
CHEROKEE SMALL LETTER HO

0AB81
CHEROKEE SMALL LETTER HU

0AB82
CHEROKEE SMALL LETTER HV

0AB83
CHEROKEE SMALL LETTER LA

0AB84
CHEROKEE SMALL LETTER LE

0AB85
CHEROKEE SMALL LETTER LI

0AB86
CHEROKEE SMALL LETTER LO

0AB87
CHEROKEE SMALL LETTER LU

0AB88
CHEROKEE SMALL LETTER LV

0AB89
CHEROKEE SMALL LETTER MA

0AB8A
CHEROKEE SMALL LETTER ME

0AB8B
CHEROKEE SMALL LETTER MI

0AB8C
CHEROKEE SMALL LETTER MO

0AB8D
CHEROKEE SMALL LETTER MU

0AB8E
CHEROKEE SMALL LETTER NA

0AB8F
CHEROKEE SMALL LETTER HNA

0AB90
CHEROKEE SMALL LETTER NAH

0AB91
CHEROKEE SMALL LETTER NE

0AB92
CHEROKEE SMALL LETTER NI

0AB93
CHEROKEE SMALL LETTER NO

0AB94
CHEROKEE SMALL LETTER NU

0AB95
CHEROKEE SMALL LETTER NV

0AB96
CHEROKEE SMALL LETTER QUA

0AB97
CHEROKEE SMALL LETTER QUE

0AB98
CHEROKEE SMALL LETTER QUI

0AB99
CHEROKEE SMALL LETTER QUO

0AB9A
CHEROKEE SMALL LETTER QUU

0AB9B
CHEROKEE SMALL LETTER QUV

0AB9C
CHEROKEE SMALL LETTER SA

0AB9D
CHEROKEE SMALL LETTER S

0AB9E
CHEROKEE SMALL LETTER SE

0AB9F
CHEROKEE SMALL LETTER SI

0ABA0
CHEROKEE SMALL LETTER SO

0ABA1
CHEROKEE SMALL LETTER SU

0ABA2
CHEROKEE SMALL LETTER SV

0ABA3
CHEROKEE SMALL LETTER DA

0ABA4
CHEROKEE SMALL LETTER TA

0ABA5
CHEROKEE SMALL LETTER DE

0ABA6
CHEROKEE SMALL LETTER TE

0ABA7
CHEROKEE SMALL LETTER DI

0ABA8
CHEROKEE SMALL LETTER TI

0ABA9
CHEROKEE SMALL LETTER DO

0ABAA
CHEROKEE SMALL LETTER DU

0ABAB
CHEROKEE SMALL LETTER DV

0ABAC
CHEROKEE SMALL LETTER DLA

0ABAD
CHEROKEE SMALL LETTER TLA

0ABAE
CHEROKEE SMALL LETTER TLE

0ABAF
CHEROKEE SMALL LETTER TLI

0ABB0
CHEROKEE SMALL LETTER TLO

0ABB1
CHEROKEE SMALL LETTER TLU

0ABB2
CHEROKEE SMALL LETTER TLV

0ABB3
CHEROKEE SMALL LETTER TSA

0ABB4
CHEROKEE SMALL LETTER TSE

0ABB5
CHEROKEE SMALL LETTER TSI

0ABB6
CHEROKEE SMALL LETTER TSO

0ABB7
CHEROKEE SMALL LETTER TSU

0ABB8
CHEROKEE SMALL LETTER TSV

0ABB9
CHEROKEE SMALL LETTER WA

0ABBA
CHEROKEE SMALL LETTER WE

0ABBB
CHEROKEE SMALL LETTER WI

0ABBC
CHEROKEE SMALL LETTER WO

0ABBD
CHEROKEE SMALL LETTER WU

0ABBE
CHEROKEE SMALL LETTER WV

0ABBF
CHEROKEE SMALL LETTER YA

0ABC0
MEETEI MAYEK LETTER KOK

0ABC1
MEETEI MAYEK LETTER SAM

0ABC2
MEETEI MAYEK LETTER LAI

0ABC3
MEETEI MAYEK LETTER MIT

0ABC4
MEETEI MAYEK LETTER PA

0ABC5
MEETEI MAYEK LETTER NA

0ABC6
MEETEI MAYEK LETTER CHIL

0ABC7
MEETEI MAYEK LETTER TIL

0ABC8
MEETEI MAYEK LETTER KHOU

0ABC9
MEETEI MAYEK LETTER NGOU

0ABCA
MEETEI MAYEK LETTER THOU

0ABCB
MEETEI MAYEK LETTER WAI

0ABCC
MEETEI MAYEK LETTER YANG

0ABCD
MEETEI MAYEK LETTER HUK

0ABCE
MEETEI MAYEK LETTER UN

0ABCF
MEETEI MAYEK LETTER I

0ABD0
MEETEI MAYEK LETTER PHAM

0ABD1
MEETEI MAYEK LETTER ATIYA

0ABD2
MEETEI MAYEK LETTER GOK

0ABD3
MEETEI MAYEK LETTER JHAM

0ABD4
MEETEI MAYEK LETTER RAI

0ABD5
MEETEI MAYEK LETTER BA

0ABD6
MEETEI MAYEK LETTER JIL

0ABD7
MEETEI MAYEK LETTER DIL

0ABD8
MEETEI MAYEK LETTER GHOU

0ABD9
MEETEI MAYEK LETTER DHOU

0ABDA
MEETEI MAYEK LETTER BHAM

0ABDB
MEETEI MAYEK LETTER KOK LONSUM

0ABDC
MEETEI MAYEK LETTER LAI LONSUM

0ABDD
MEETEI MAYEK LETTER MIT LONSUM

0ABDE
MEETEI MAYEK LETTER PA LONSUM

0ABDF
MEETEI MAYEK LETTER NA LONSUM

0ABE0
MEETEI MAYEK LETTER TIL LONSUM

0ABE1
MEETEI MAYEK LETTER NGOU LONSUM

0ABE2
MEETEI MAYEK LETTER I LONSUM

0ABE3
MEETEI MAYEK VOWEL SIGN ONAP

0ABE4
MEETEI MAYEK VOWEL SIGN INAP

0ABE5
MEETEI MAYEK VOWEL SIGN ANAP

0ABE6
MEETEI MAYEK VOWEL SIGN YENAP

0ABE7
MEETEI MAYEK VOWEL SIGN SOUNAP

0ABE8
MEETEI MAYEK VOWEL SIGN UNAP

0ABE9
MEETEI MAYEK VOWEL SIGN CHEINAP

0ABEA
MEETEI MAYEK VOWEL SIGN NUNG

0ABEB
MEETEI MAYEK CHEIKHEI

0ABEC
MEETEI MAYEK LUM IYEK

0ABED
MEETEI MAYEK APUN IYEK

0ABF0
MEETEI MAYEK DIGIT ZERO

0ABF1
MEETEI MAYEK DIGIT ONE

0ABF2
MEETEI MAYEK DIGIT TWO

0ABF3
MEETEI MAYEK DIGIT THREE

0ABF4
MEETEI MAYEK DIGIT FOUR

0ABF5
MEETEI MAYEK DIGIT FIVE

0ABF6
MEETEI MAYEK DIGIT SIX

0ABF7
MEETEI MAYEK DIGIT SEVEN

0ABF8
MEETEI MAYEK DIGIT EIGHT

0ABF9
MEETEI MAYEK DIGIT NINE

0D7B0
HANGUL JUNGSEONG O-YEO

0D7B1
HANGUL JUNGSEONG O-O-I

0D7B2
HANGUL JUNGSEONG YO-A

0D7B3
HANGUL JUNGSEONG YO-AE

0D7B4
HANGUL JUNGSEONG YO-EO

0D7B5
HANGUL JUNGSEONG U-YEO

0D7B6
HANGUL JUNGSEONG U-I-I

0D7B7
HANGUL JUNGSEONG YU-AE

0D7B8
HANGUL JUNGSEONG YU-O

0D7B9
HANGUL JUNGSEONG EU-A

0D7BA
HANGUL JUNGSEONG EU-EO

0D7BB
HANGUL JUNGSEONG EU-E

0D7BC
HANGUL JUNGSEONG EU-O

0D7BD
HANGUL JUNGSEONG I-YA-O

0D7BE
HANGUL JUNGSEONG I-YAE

0D7BF
HANGUL JUNGSEONG I-YEO

0D7C0
HANGUL JUNGSEONG I-YE

0D7C1
HANGUL JUNGSEONG I-O-I

0D7C2
HANGUL JUNGSEONG I-YO

0D7C3
HANGUL JUNGSEONG I-YU

0D7C4
HANGUL JUNGSEONG I-I

0D7C5
HANGUL JUNGSEONG ARAEA-A

0D7C6
HANGUL JUNGSEONG ARAEA-E

0D7CB
HANGUL JONGSEONG NIEUN-RIEUL

0D7CC
HANGUL JONGSEONG NIEUN-CHIEUCH

0D7CD
HANGUL JONGSEONG SSANGTIKEUT

0D7CE
HANGUL JONGSEONG SSANGTIKEUT-PIEUP

0D7CF
HANGUL JONGSEONG TIKEUT-PIEUP

0D7D0
HANGUL JONGSEONG TIKEUT-SIOS

0D7D1
HANGUL JONGSEONG TIKEUT-SIOS-KIYEOK

0D7D2
HANGUL JONGSEONG TIKEUT-CIEUC

0D7D3
HANGUL JONGSEONG TIKEUT-CHIEUCH

0D7D4
HANGUL JONGSEONG TIKEUT-THIEUTH

0D7D5
HANGUL JONGSEONG RIEUL-SSANGKIYEOK

0D7D6
HANGUL JONGSEONG RIEUL-KIYEOK-HIEUH

0D7D7
HANGUL JONGSEONG SSANGRIEUL-KHIEUKH

0D7D8
HANGUL JONGSEONG RIEUL-MIEUM-HIEUH

0D7D9
HANGUL JONGSEONG RIEUL-PIEUP-TIKEUT

0D7DA
HANGUL JONGSEONG RIEUL-PIEUP-PHIEUPH

0D7DB
HANGUL JONGSEONG RIEUL-YESIEUNG

0D7DC
HANGUL JONGSEONG RIEUL-YEORINHIEUH-HIEUH

0D7DD
HANGUL JONGSEONG KAPYEOUNRIEUL

0D7DE
HANGUL JONGSEONG MIEUM-NIEUN

0D7DF
HANGUL JONGSEONG MIEUM-SSANGNIEUN

0D7E0
HANGUL JONGSEONG SSANGMIEUM

0D7E1
HANGUL JONGSEONG MIEUM-PIEUP-SIOS

0D7E2
HANGUL JONGSEONG MIEUM-CIEUC

0D7E3
HANGUL JONGSEONG PIEUP-TIKEUT

0D7E4
HANGUL JONGSEONG PIEUP-RIEUL-PHIEUPH

0D7E5
HANGUL JONGSEONG PIEUP-MIEUM

0D7E6
HANGUL JONGSEONG SSANGPIEUP

0D7E7
HANGUL JONGSEONG PIEUP-SIOS-TIKEUT

0D7E8
HANGUL JONGSEONG PIEUP-CIEUC

0D7E9
HANGUL JONGSEONG PIEUP-CHIEUCH

0D7EA
HANGUL JONGSEONG SIOS-MIEUM

0D7EB
HANGUL JONGSEONG SIOS-KAPYEOUNPIEUP

0D7EC
HANGUL JONGSEONG SSANGSIOS-KIYEOK

0D7ED
HANGUL JONGSEONG SSANGSIOS-TIKEUT

0D7EE
HANGUL JONGSEONG SIOS-PANSIOS

0D7EF
HANGUL JONGSEONG SIOS-CIEUC

0D7F0
HANGUL JONGSEONG SIOS-CHIEUCH

0D7F1
HANGUL JONGSEONG SIOS-THIEUTH

0D7F2
HANGUL JONGSEONG SIOS-HIEUH

0D7F3
HANGUL JONGSEONG PANSIOS-PIEUP

0D7F4
HANGUL JONGSEONG PANSIOS-KAPYEOUNPIEUP

0D7F5
HANGUL JONGSEONG YESIEUNG-MIEUM

0D7F6
HANGUL JONGSEONG YESIEUNG-HIEUH

0D7F7
HANGUL JONGSEONG CIEUC-PIEUP

0D7F8
HANGUL JONGSEONG CIEUC-SSANGPIEUP

0D7F9
HANGUL JONGSEONG SSANGCIEUC

0D7FA
HANGUL JONGSEONG PHIEUPH-SIOS

0D7FB
HANGUL JONGSEONG PHIEUPH-THIEUTH

0FB00
LATIN SMALL LIGATURE FF

0FB01
LATIN SMALL LIGATURE FI

0FB02
LATIN SMALL LIGATURE FL

0FB03
LATIN SMALL LIGATURE FFI

0FB04
LATIN SMALL LIGATURE FFL

0FB05
LATIN SMALL LIGATURE LONG S T

0FB06
LATIN SMALL LIGATURE ST

0FB13
ARMENIAN SMALL LIGATURE MEN NOW

0FB14
ARMENIAN SMALL LIGATURE MEN ECH

0FB15
ARMENIAN SMALL LIGATURE MEN INI

0FB16
ARMENIAN SMALL LIGATURE VEW NOW

0FB17
ARMENIAN SMALL LIGATURE MEN XEH

0FB1D
HEBREW LETTER YOD WITH HIRIQ

0FB1E
HEBREW POINT JUDEO-SPANISH VARIKA

0FB1F
HEBREW LIGATURE YIDDISH YOD YOD PATAH

0FB20
HEBREW LETTER ALTERNATIVE AYIN

0FB21
HEBREW LETTER WIDE ALEF

0FB22
HEBREW LETTER WIDE DALET

0FB23
HEBREW LETTER WIDE HE

0FB24
HEBREW LETTER WIDE KAF

0FB25
HEBREW LETTER WIDE LAMED

0FB26
HEBREW LETTER WIDE FINAL MEM

0FB27
HEBREW LETTER WIDE RESH

0FB28
HEBREW LETTER WIDE TAV

0FB29
HEBREW LETTER ALTERNATIVE PLUS SIGN

0FB2A
HEBREW LETTER SHIN WITH SHIN DOT

0FB2B
HEBREW LETTER SHIN WITH SIN DOT

0FB2C
HEBREW LETTER SHIN WITH DAGESH AND SHIN DOT

0FB2D
HEBREW LETTER SHIN WITH DAGESH AND SIN DOT

0FB2E
HEBREW LETTER ALEF WITH PATAH

0FB2F
HEBREW LETTER ALEF WITH QAMATS

0FB30
HEBREW LETTER ALEF WITH MAPIQ

0FB31
HEBREW LETTER BET WITH DAGESH

0FB32
HEBREW LETTER GIMEL WITH DAGESH

0FB33
HEBREW LETTER DALET WITH DAGESH

0FB34
HEBREW LETTER HE WITH MAPIQ

0FB35
HEBREW LETTER VAV WITH DAGESH

0FB36
HEBREW LETTER ZAYIN WITH DAGESH

0FB38
HEBREW LETTER TET WITH DAGESH

0FB39
HEBREW LETTER YOD WITH DAGESH

0FB3A
HEBREW LETTER FINAL KAF WITH DAGESH

0FB3B
HEBREW LETTER KAF WITH DAGESH

0FB3C
HEBREW LETTER LAMED WITH DAGESH

0FB3E
HEBREW LETTER MEM WITH DAGESH

0FB40
HEBREW LETTER NUN WITH DAGESH

0FB41
HEBREW LETTER SAMEKH WITH DAGESH

0FB43
HEBREW LETTER FINAL PE WITH DAGESH

0FB44
HEBREW LETTER PE WITH DAGESH

0FB46
HEBREW LETTER TSADI WITH DAGESH

0FB47
HEBREW LETTER QOF WITH DAGESH

0FB48
HEBREW LETTER RESH WITH DAGESH

0FB49
HEBREW LETTER SHIN WITH DAGESH

0FB4A
HEBREW LETTER TAV WITH DAGESH

0FB4B
HEBREW LETTER VAV WITH HOLAM

0FB4C
HEBREW LETTER BET WITH RAFE

0FB4D
HEBREW LETTER KAF WITH RAFE

0FB4E
HEBREW LETTER PE WITH RAFE

0FB4F
HEBREW LIGATURE ALEF LAMED

0FB50
ARABIC LETTER ALEF WASLA ISOLATED FORM

0FB51
ARABIC LETTER ALEF WASLA FINAL FORM

0FB52
ARABIC LETTER BEEH ISOLATED FORM

0FB53
ARABIC LETTER BEEH FINAL FORM

0FB54
ARABIC LETTER BEEH INITIAL FORM

0FB55
ARABIC LETTER BEEH MEDIAL FORM

0FB56
ARABIC LETTER PEH ISOLATED FORM

0FB57
ARABIC LETTER PEH FINAL FORM

0FB58
ARABIC LETTER PEH INITIAL FORM

0FB59
ARABIC LETTER PEH MEDIAL FORM

0FB5A
ARABIC LETTER BEHEH ISOLATED FORM

0FB5B
ARABIC LETTER BEHEH FINAL FORM

0FB5C
ARABIC LETTER BEHEH INITIAL FORM

0FB5D
ARABIC LETTER BEHEH MEDIAL FORM

0FB5E
ARABIC LETTER TTEHEH ISOLATED FORM

0FB5F
ARABIC LETTER TTEHEH FINAL FORM

0FB60
ARABIC LETTER TTEHEH INITIAL FORM

0FB61
ARABIC LETTER TTEHEH MEDIAL FORM

0FB62
ARABIC LETTER TEHEH ISOLATED FORM

0FB63
ARABIC LETTER TEHEH FINAL FORM

0FB64
ARABIC LETTER TEHEH INITIAL FORM

0FB65
ARABIC LETTER TEHEH MEDIAL FORM

0FB66
ARABIC LETTER TTEH ISOLATED FORM

0FB67
ARABIC LETTER TTEH FINAL FORM

0FB68
ARABIC LETTER TTEH INITIAL FORM

0FB69
ARABIC LETTER TTEH MEDIAL FORM

0FB6A
ARABIC LETTER VEH ISOLATED FORM

0FB6B
ARABIC LETTER VEH FINAL FORM

0FB6C
ARABIC LETTER VEH INITIAL FORM

0FB6D
ARABIC LETTER VEH MEDIAL FORM

0FB6E
ARABIC LETTER PEHEH ISOLATED FORM

0FB6F
ARABIC LETTER PEHEH FINAL FORM

0FB70
ARABIC LETTER PEHEH INITIAL FORM

0FB71
ARABIC LETTER PEHEH MEDIAL FORM

0FB72
ARABIC LETTER DYEH ISOLATED FORM

0FB73
ARABIC LETTER DYEH FINAL FORM

0FB74
ARABIC LETTER DYEH INITIAL FORM

0FB75
ARABIC LETTER DYEH MEDIAL FORM

0FB76
ARABIC LETTER NYEH ISOLATED FORM

0FB77
ARABIC LETTER NYEH FINAL FORM

0FB78
ARABIC LETTER NYEH INITIAL FORM

0FB79
ARABIC LETTER NYEH MEDIAL FORM

0FB7A
ARABIC LETTER TCHEH ISOLATED FORM

0FB7B
ARABIC LETTER TCHEH FINAL FORM

0FB7C
ARABIC LETTER TCHEH INITIAL FORM

0FB7D
ARABIC LETTER TCHEH MEDIAL FORM

0FB7E
ARABIC LETTER TCHEHEH ISOLATED FORM

0FB7F
ARABIC LETTER TCHEHEH FINAL FORM

0FB80
ARABIC LETTER TCHEHEH INITIAL FORM

0FB81
ARABIC LETTER TCHEHEH MEDIAL FORM

0FB82
ARABIC LETTER DDAHAL ISOLATED FORM

0FB83
ARABIC LETTER DDAHAL FINAL FORM

0FB84
ARABIC LETTER DAHAL ISOLATED FORM

0FB85
ARABIC LETTER DAHAL FINAL FORM

0FB86
ARABIC LETTER DUL ISOLATED FORM

0FB87
ARABIC LETTER DUL FINAL FORM

0FB88
ARABIC LETTER DDAL ISOLATED FORM

0FB89
ARABIC LETTER DDAL FINAL FORM

0FB8A
ARABIC LETTER JEH ISOLATED FORM

0FB8B
ARABIC LETTER JEH FINAL FORM

0FB8C
ARABIC LETTER RREH ISOLATED FORM

0FB8D
ARABIC LETTER RREH FINAL FORM

0FB8E
ARABIC LETTER KEHEH ISOLATED FORM

0FB8F
ARABIC LETTER KEHEH FINAL FORM

0FB90
ARABIC LETTER KEHEH INITIAL FORM

0FB91
ARABIC LETTER KEHEH MEDIAL FORM

0FB92
ARABIC LETTER GAF ISOLATED FORM

0FB93
ARABIC LETTER GAF FINAL FORM

0FB94
ARABIC LETTER GAF INITIAL FORM

0FB95
ARABIC LETTER GAF MEDIAL FORM

0FB96
ARABIC LETTER GUEH ISOLATED FORM

0FB97
ARABIC LETTER GUEH FINAL FORM

0FB98
ARABIC LETTER GUEH INITIAL FORM

0FB99
ARABIC LETTER GUEH MEDIAL FORM

0FB9A
ARABIC LETTER NGOEH ISOLATED FORM

0FB9B
ARABIC LETTER NGOEH FINAL FORM

0FB9C
ARABIC LETTER NGOEH INITIAL FORM

0FB9D
ARABIC LETTER NGOEH MEDIAL FORM

0FB9E
ARABIC LETTER NOON GHUNNA ISOLATED FORM

0FB9F
ARABIC LETTER NOON GHUNNA FINAL FORM

0FBA0
ARABIC LETTER RNOON ISOLATED FORM

0FBA1
ARABIC LETTER RNOON FINAL FORM

0FBA2
ARABIC LETTER RNOON INITIAL FORM

0FBA3
ARABIC LETTER RNOON MEDIAL FORM

0FBA4
ARABIC LETTER HEH WITH YEH ABOVE ISOLATED FORM

0FBA5
ARABIC LETTER HEH WITH YEH ABOVE FINAL FORM

0FBA6
ARABIC LETTER HEH GOAL ISOLATED FORM

0FBA7
ARABIC LETTER HEH GOAL FINAL FORM

0FBA8
ARABIC LETTER HEH GOAL INITIAL FORM

0FBA9
ARABIC LETTER HEH GOAL MEDIAL FORM

0FBAA
ARABIC LETTER HEH DOACHASHMEE ISOLATED FORM

0FBAB
ARABIC LETTER HEH DOACHASHMEE FINAL FORM

0FBAC
ARABIC LETTER HEH DOACHASHMEE INITIAL FORM

0FBAD
ARABIC LETTER HEH DOACHASHMEE MEDIAL FORM

0FBAE
ARABIC LETTER YEH BARREE ISOLATED FORM

0FBAF
ARABIC LETTER YEH BARREE FINAL FORM

0FBB0
ARABIC LETTER YEH BARREE WITH HAMZA ABOVE ISOLATED FORM

0FBB1
ARABIC LETTER YEH BARREE WITH HAMZA ABOVE FINAL FORM

0FBB2
ARABIC SYMBOL DOT ABOVE

0FBB3
ARABIC SYMBOL DOT BELOW

0FBB4
ARABIC SYMBOL TWO DOTS ABOVE

0FBB5
ARABIC SYMBOL TWO DOTS BELOW

0FBB6
ARABIC SYMBOL THREE DOTS ABOVE

0FBB7
ARABIC SYMBOL THREE DOTS BELOW

0FBB8
ARABIC SYMBOL THREE DOTS POINTING DOWNWARDS ABOVE

0FBB9
ARABIC SYMBOL THREE DOTS POINTING DOWNWARDS BELOW

0FBBA
ARABIC SYMBOL FOUR DOTS ABOVE

0FBBB
ARABIC SYMBOL FOUR DOTS BELOW

0FBBC
ARABIC SYMBOL DOUBLE VERTICAL BAR BELOW

0FBBD
ARABIC SYMBOL TWO DOTS VERTICALLY ABOVE

0FBBE
ARABIC SYMBOL TWO DOTS VERTICALLY BELOW

0FBBF
ARABIC SYMBOL RING

0FBC0
ARABIC SYMBOL SMALL TAH ABOVE

0FBC1
ARABIC SYMBOL SMALL TAH BELOW

0FBC2
ARABIC SYMBOL WASLA ABOVE

0FBC3
ARABIC LIGATURE JALLA WA-ALAA

0FBC4
ARABIC LIGATURE DAAMAT BARAKAATUHUM

0FBC5
ARABIC LIGATURE RAHMATU ALLAAHI TAAALAA ALAYH

0FBC6
ARABIC LIGATURE RAHMATU ALLAAHI ALAYHIM

0FBC7
ARABIC LIGATURE RAHMATU ALLAAHI ALAYHIMAA

0FBC8
ARABIC LIGATURE RAHIMAHUM ALLAAHU TAAALAA

0FBC9
ARABIC LIGATURE RAHIMAHUMAA ALLAAH

0FBCA
ARABIC LIGATURE RAHIMAHUMAA ALLAAHU TAAALAA

0FBCB
ARABIC LIGATURE RADI ALLAAHU TAAALAA ANHUM

0FBCC
ARABIC LIGATURE HAFIZAHU ALLAAH

0FBCD
ARABIC LIGATURE HAFIZAHU ALLAAHU TAAALAA

0FBCE
ARABIC LIGATURE HAFIZAHUM ALLAAHU TAAALAA

0FBCF
ARABIC LIGATURE HAFIZAHUMAA ALLAAHU TAAALAA

0FBD0
ARABIC LIGATURE SALLALLAAHU TAAALAA ALAYHI WA-SALLAM

0FBD1
ARABIC LIGATURE AJJAL ALLAAHU FARAJAHU ASH-SHAREEF

0FBD2
ARABIC LIGATURE ALAYHI AR-RAHMAH

0FBD3
ARABIC LETTER NG ISOLATED FORM

0FBD4
ARABIC LETTER NG FINAL FORM

0FBD5
ARABIC LETTER NG INITIAL FORM

0FBD6
ARABIC LETTER NG MEDIAL FORM

0FBD7
ARABIC LETTER U ISOLATED FORM

0FBD8
ARABIC LETTER U FINAL FORM

0FBD9
ARABIC LETTER OE ISOLATED FORM

0FBDA
ARABIC LETTER OE FINAL FORM

0FBDB
ARABIC LETTER YU ISOLATED FORM

0FBDC
ARABIC LETTER YU FINAL FORM

0FBDD
ARABIC LETTER U WITH HAMZA ABOVE ISOLATED FORM

0FBDE
ARABIC LETTER VE ISOLATED FORM

0FBDF
ARABIC LETTER VE FINAL FORM

0FBE0
ARABIC LETTER KIRGHIZ OE ISOLATED FORM

0FBE1
ARABIC LETTER KIRGHIZ OE FINAL FORM

0FBE2
ARABIC LETTER KIRGHIZ YU ISOLATED FORM

0FBE3
ARABIC LETTER KIRGHIZ YU FINAL FORM

0FBE4
ARABIC LETTER E ISOLATED FORM

0FBE5
ARABIC LETTER E FINAL FORM

0FBE6
ARABIC LETTER E INITIAL FORM

0FBE7
ARABIC LETTER E MEDIAL FORM

0FBE8
ARABIC LETTER UIGHUR KAZAKH KIRGHIZ ALEF MAKSURA INITIAL FORM

0FBE9
ARABIC LETTER UIGHUR KAZAKH KIRGHIZ ALEF MAKSURA MEDIAL FORM

0FBEA
ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH ALEF ISOLATED FORM

0FBEB
ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH ALEF FINAL FORM

0FBEC
ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH AE ISOLATED FORM

0FBED
ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH AE FINAL FORM

0FBEE
ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH WAW ISOLATED FORM

0FBEF
ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH WAW FINAL FORM

0FBF0
ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH U ISOLATED FORM

0FBF1
ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH U FINAL FORM

0FBF2
ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH OE ISOLATED FORM

0FBF3
ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH OE FINAL FORM

0FBF4
ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH YU ISOLATED FORM

0FBF5
ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH YU FINAL FORM

0FBF6
ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH E ISOLATED FORM

0FBF7
ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH E FINAL FORM

0FBF8
ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH E INITIAL FORM

0FBF9
ARABIC LIGATURE UIGHUR KIRGHIZ YEH WITH HAMZA ABOVE WITH ALEF MAKSURA ISOLATED FORM

0FBFA
ARABIC LIGATURE UIGHUR KIRGHIZ YEH WITH HAMZA ABOVE WITH ALEF MAKSURA FINAL FORM

0FBFB
ARABIC LIGATURE UIGHUR KIRGHIZ YEH WITH HAMZA ABOVE WITH ALEF MAKSURA INITIAL FORM

0FBFC
ARABIC LETTER FARSI YEH ISOLATED FORM

0FBFD
ARABIC LETTER FARSI YEH FINAL FORM

0FBFE
ARABIC LETTER FARSI YEH INITIAL FORM

0FBFF
ARABIC LETTER FARSI YEH MEDIAL FORM

0FC00
ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH JEEM ISOLATED FORM

0FC01
ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH HAH ISOLATED FORM

0FC02
ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH MEEM ISOLATED FORM

0FC03
ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH ALEF MAKSURA ISOLATED FORM

0FC04
ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH YEH ISOLATED FORM

0FC05
ARABIC LIGATURE BEH WITH JEEM ISOLATED FORM

0FC06
ARABIC LIGATURE BEH WITH HAH ISOLATED FORM

0FC07
ARABIC LIGATURE BEH WITH KHAH ISOLATED FORM

0FC08
ARABIC LIGATURE BEH WITH MEEM ISOLATED FORM

0FC09
ARABIC LIGATURE BEH WITH ALEF MAKSURA ISOLATED FORM

0FC0A
ARABIC LIGATURE BEH WITH YEH ISOLATED FORM

0FC0B
ARABIC LIGATURE TEH WITH JEEM ISOLATED FORM

0FC0C
ARABIC LIGATURE TEH WITH HAH ISOLATED FORM

0FC0D
ARABIC LIGATURE TEH WITH KHAH ISOLATED FORM

0FC0E
ARABIC LIGATURE TEH WITH MEEM ISOLATED FORM

0FC0F
ARABIC LIGATURE TEH WITH ALEF MAKSURA ISOLATED FORM

0FC10
ARABIC LIGATURE TEH WITH YEH ISOLATED FORM

0FC11
ARABIC LIGATURE THEH WITH JEEM ISOLATED FORM

0FC12
ARABIC LIGATURE THEH WITH MEEM ISOLATED FORM

0FC13
ARABIC LIGATURE THEH WITH ALEF MAKSURA ISOLATED FORM

0FC14
ARABIC LIGATURE THEH WITH YEH ISOLATED FORM

0FC15
ARABIC LIGATURE JEEM WITH HAH ISOLATED FORM

0FC16
ARABIC LIGATURE JEEM WITH MEEM ISOLATED FORM

0FC17
ARABIC LIGATURE HAH WITH JEEM ISOLATED FORM

0FC18
ARABIC LIGATURE HAH WITH MEEM ISOLATED FORM

0FC19
ARABIC LIGATURE KHAH WITH JEEM ISOLATED FORM

0FC1A
ARABIC LIGATURE KHAH WITH HAH ISOLATED FORM

0FC1B
ARABIC LIGATURE KHAH WITH MEEM ISOLATED FORM

0FC1C
ARABIC LIGATURE SEEN WITH JEEM ISOLATED FORM

0FC1D
ARABIC LIGATURE SEEN WITH HAH ISOLATED FORM

0FC1E
ARABIC LIGATURE SEEN WITH KHAH ISOLATED FORM

0FC1F
ARABIC LIGATURE SEEN WITH MEEM ISOLATED FORM

0FC20
ARABIC LIGATURE SAD WITH HAH ISOLATED FORM

0FC21
ARABIC LIGATURE SAD WITH MEEM ISOLATED FORM

0FC22
ARABIC LIGATURE DAD WITH JEEM ISOLATED FORM

0FC23
ARABIC LIGATURE DAD WITH HAH ISOLATED FORM

0FC24
ARABIC LIGATURE DAD WITH KHAH ISOLATED FORM

0FC25
ARABIC LIGATURE DAD WITH MEEM ISOLATED FORM

0FC26
ARABIC LIGATURE TAH WITH HAH ISOLATED FORM

0FC27
ARABIC LIGATURE TAH WITH MEEM ISOLATED FORM

0FC28
ARABIC LIGATURE ZAH WITH MEEM ISOLATED FORM

0FC29
ARABIC LIGATURE AIN WITH JEEM ISOLATED FORM

0FC2A
ARABIC LIGATURE AIN WITH MEEM ISOLATED FORM

0FC2B
ARABIC LIGATURE GHAIN WITH JEEM ISOLATED FORM

0FC2C
ARABIC LIGATURE GHAIN WITH MEEM ISOLATED FORM

0FC2D
ARABIC LIGATURE FEH WITH JEEM ISOLATED FORM

0FC2E
ARABIC LIGATURE FEH WITH HAH ISOLATED FORM

0FC2F
ARABIC LIGATURE FEH WITH KHAH ISOLATED FORM

0FC30
ARABIC LIGATURE FEH WITH MEEM ISOLATED FORM

0FC31
ARABIC LIGATURE FEH WITH ALEF MAKSURA ISOLATED FORM

0FC32
ARABIC LIGATURE FEH WITH YEH ISOLATED FORM

0FC33
ARABIC LIGATURE QAF WITH HAH ISOLATED FORM

0FC34
ARABIC LIGATURE QAF WITH MEEM ISOLATED FORM

0FC35
ARABIC LIGATURE QAF WITH ALEF MAKSURA ISOLATED FORM

0FC36
ARABIC LIGATURE QAF WITH YEH ISOLATED FORM

0FC37
ARABIC LIGATURE KAF WITH ALEF ISOLATED FORM

0FC38
ARABIC LIGATURE KAF WITH JEEM ISOLATED FORM

0FC39
ARABIC LIGATURE KAF WITH HAH ISOLATED FORM

0FC3A
ARABIC LIGATURE KAF WITH KHAH ISOLATED FORM

0FC3B
ARABIC LIGATURE KAF WITH LAM ISOLATED FORM

0FC3C
ARABIC LIGATURE KAF WITH MEEM ISOLATED FORM

0FC3D
ARABIC LIGATURE KAF WITH ALEF MAKSURA ISOLATED FORM

0FC3E
ARABIC LIGATURE KAF WITH YEH ISOLATED FORM

0FC3F
ARABIC LIGATURE LAM WITH JEEM ISOLATED FORM

0FC40
ARABIC LIGATURE LAM WITH HAH ISOLATED FORM

0FC41
ARABIC LIGATURE LAM WITH KHAH ISOLATED FORM

0FC42
ARABIC LIGATURE LAM WITH MEEM ISOLATED FORM

0FC43
ARABIC LIGATURE LAM WITH ALEF MAKSURA ISOLATED FORM

0FC44
ARABIC LIGATURE LAM WITH YEH ISOLATED FORM

0FC45
ARABIC LIGATURE MEEM WITH JEEM ISOLATED FORM

0FC46
ARABIC LIGATURE MEEM WITH HAH ISOLATED FORM

0FC47
ARABIC LIGATURE MEEM WITH KHAH ISOLATED FORM

0FC48
ARABIC LIGATURE MEEM WITH MEEM ISOLATED FORM

0FC49
ARABIC LIGATURE MEEM WITH ALEF MAKSURA ISOLATED FORM

0FC4A
ARABIC LIGATURE MEEM WITH YEH ISOLATED FORM

0FC4B
ARABIC LIGATURE NOON WITH JEEM ISOLATED FORM

0FC4C
ARABIC LIGATURE NOON WITH HAH ISOLATED FORM

0FC4D
ARABIC LIGATURE NOON WITH KHAH ISOLATED FORM

0FC4E
ARABIC LIGATURE NOON WITH MEEM ISOLATED FORM

0FC4F
ARABIC LIGATURE NOON WITH ALEF MAKSURA ISOLATED FORM

0FC50
ARABIC LIGATURE NOON WITH YEH ISOLATED FORM

0FC51
ARABIC LIGATURE HEH WITH JEEM ISOLATED FORM

0FC52
ARABIC LIGATURE HEH WITH MEEM ISOLATED FORM

0FC53
ARABIC LIGATURE HEH WITH ALEF MAKSURA ISOLATED FORM

0FC54
ARABIC LIGATURE HEH WITH YEH ISOLATED FORM

0FC55
ARABIC LIGATURE YEH WITH JEEM ISOLATED FORM

0FC56
ARABIC LIGATURE YEH WITH HAH ISOLATED FORM

0FC57
ARABIC LIGATURE YEH WITH KHAH ISOLATED FORM

0FC58
ARABIC LIGATURE YEH WITH MEEM ISOLATED FORM

0FC59
ARABIC LIGATURE YEH WITH ALEF MAKSURA ISOLATED FORM

0FC5A
ARABIC LIGATURE YEH WITH YEH ISOLATED FORM

0FC5B
ARABIC LIGATURE THAL WITH SUPERSCRIPT ALEF ISOLATED FORM

0FC5C
ARABIC LIGATURE REH WITH SUPERSCRIPT ALEF ISOLATED FORM

0FC5D
ARABIC LIGATURE ALEF MAKSURA WITH SUPERSCRIPT ALEF ISOLATED FORM

0FC5E
ARABIC LIGATURE SHADDA WITH DAMMATAN ISOLATED FORM

0FC5F
ARABIC LIGATURE SHADDA WITH KASRATAN ISOLATED FORM

0FC60
ARABIC LIGATURE SHADDA WITH FATHA ISOLATED FORM

0FC61
ARABIC LIGATURE SHADDA WITH DAMMA ISOLATED FORM

0FC62
ARABIC LIGATURE SHADDA WITH KASRA ISOLATED FORM

0FC63
ARABIC LIGATURE SHADDA WITH SUPERSCRIPT ALEF ISOLATED FORM

0FC64
ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH REH FINAL FORM

0FC65
ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH ZAIN FINAL FORM

0FC66
ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH MEEM FINAL FORM

0FC67
ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH NOON FINAL FORM

0FC68
ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH ALEF MAKSURA FINAL FORM

0FC69
ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH YEH FINAL FORM

0FC6A
ARABIC LIGATURE BEH WITH REH FINAL FORM

0FC6B
ARABIC LIGATURE BEH WITH ZAIN FINAL FORM

0FC6C
ARABIC LIGATURE BEH WITH MEEM FINAL FORM

0FC6D
ARABIC LIGATURE BEH WITH NOON FINAL FORM

0FC6E
ARABIC LIGATURE BEH WITH ALEF MAKSURA FINAL FORM

0FC6F
ARABIC LIGATURE BEH WITH YEH FINAL FORM

0FC70
ARABIC LIGATURE TEH WITH REH FINAL FORM

0FC71
ARABIC LIGATURE TEH WITH ZAIN FINAL FORM

0FC72
ARABIC LIGATURE TEH WITH MEEM FINAL FORM

0FC73
ARABIC LIGATURE TEH WITH NOON FINAL FORM

0FC74
ARABIC LIGATURE TEH WITH ALEF MAKSURA FINAL FORM

0FC75
ARABIC LIGATURE TEH WITH YEH FINAL FORM

0FC76
ARABIC LIGATURE THEH WITH REH FINAL FORM

0FC77
ARABIC LIGATURE THEH WITH ZAIN FINAL FORM

0FC78
ARABIC LIGATURE THEH WITH MEEM FINAL FORM

0FC79
ARABIC LIGATURE THEH WITH NOON FINAL FORM

0FC7A
ARABIC LIGATURE THEH WITH ALEF MAKSURA FINAL FORM

0FC7B
ARABIC LIGATURE THEH WITH YEH FINAL FORM

0FC7C
ARABIC LIGATURE FEH WITH ALEF MAKSURA FINAL FORM

0FC7D
ARABIC LIGATURE FEH WITH YEH FINAL FORM

0FC7E
ARABIC LIGATURE QAF WITH ALEF MAKSURA FINAL FORM

0FC7F
ARABIC LIGATURE QAF WITH YEH FINAL FORM

0FC80
ARABIC LIGATURE KAF WITH ALEF FINAL FORM

0FC81
ARABIC LIGATURE KAF WITH LAM FINAL FORM

0FC82
ARABIC LIGATURE KAF WITH MEEM FINAL FORM

0FC83
ARABIC LIGATURE KAF WITH ALEF MAKSURA FINAL FORM

0FC84
ARABIC LIGATURE KAF WITH YEH FINAL FORM

0FC85
ARABIC LIGATURE LAM WITH MEEM FINAL FORM

0FC86
ARABIC LIGATURE LAM WITH ALEF MAKSURA FINAL FORM

0FC87
ARABIC LIGATURE LAM WITH YEH FINAL FORM

0FC88
ARABIC LIGATURE MEEM WITH ALEF FINAL FORM

0FC89
ARABIC LIGATURE MEEM WITH MEEM FINAL FORM

0FC8A
ARABIC LIGATURE NOON WITH REH FINAL FORM

0FC8B
ARABIC LIGATURE NOON WITH ZAIN FINAL FORM

0FC8C
ARABIC LIGATURE NOON WITH MEEM FINAL FORM

0FC8D
ARABIC LIGATURE NOON WITH NOON FINAL FORM

0FC8E
ARABIC LIGATURE NOON WITH ALEF MAKSURA FINAL FORM

0FC8F
ARABIC LIGATURE NOON WITH YEH FINAL FORM

0FC90
ARABIC LIGATURE ALEF MAKSURA WITH SUPERSCRIPT ALEF FINAL FORM

0FC91
ARABIC LIGATURE YEH WITH REH FINAL FORM

0FC92
ARABIC LIGATURE YEH WITH ZAIN FINAL FORM

0FC93
ARABIC LIGATURE YEH WITH MEEM FINAL FORM

0FC94
ARABIC LIGATURE YEH WITH NOON FINAL FORM

0FC95
ARABIC LIGATURE YEH WITH ALEF MAKSURA FINAL FORM

0FC96
ARABIC LIGATURE YEH WITH YEH FINAL FORM

0FC97
ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH JEEM INITIAL FORM

0FC98
ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH HAH INITIAL FORM

0FC99
ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH KHAH INITIAL FORM

0FC9A
ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH MEEM INITIAL FORM

0FC9B
ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH HEH INITIAL FORM

0FC9C
ARABIC LIGATURE BEH WITH JEEM INITIAL FORM

0FC9D
ARABIC LIGATURE BEH WITH HAH INITIAL FORM

0FC9E
ARABIC LIGATURE BEH WITH KHAH INITIAL FORM

0FC9F
ARABIC LIGATURE BEH WITH MEEM INITIAL FORM

0FCA0
ARABIC LIGATURE BEH WITH HEH INITIAL FORM

0FCA1
ARABIC LIGATURE TEH WITH JEEM INITIAL FORM

0FCA2
ARABIC LIGATURE TEH WITH HAH INITIAL FORM

0FCA3
ARABIC LIGATURE TEH WITH KHAH INITIAL FORM

0FCA4
ARABIC LIGATURE TEH WITH MEEM INITIAL FORM

0FCA5
ARABIC LIGATURE TEH WITH HEH INITIAL FORM

0FCA6
ARABIC LIGATURE THEH WITH MEEM INITIAL FORM

0FCA7
ARABIC LIGATURE JEEM WITH HAH INITIAL FORM

0FCA8
ARABIC LIGATURE JEEM WITH MEEM INITIAL FORM

0FCA9
ARABIC LIGATURE HAH WITH JEEM INITIAL FORM

0FCAA
ARABIC LIGATURE HAH WITH MEEM INITIAL FORM

0FCAB
ARABIC LIGATURE KHAH WITH JEEM INITIAL FORM

0FCAC
ARABIC LIGATURE KHAH WITH MEEM INITIAL FORM

0FCAD
ARABIC LIGATURE SEEN WITH JEEM INITIAL FORM

0FCAE
ARABIC LIGATURE SEEN WITH HAH INITIAL FORM

0FCAF
ARABIC LIGATURE SEEN WITH KHAH INITIAL FORM

0FCB0
ARABIC LIGATURE SEEN WITH MEEM INITIAL FORM

0FCB1
ARABIC LIGATURE SAD WITH HAH INITIAL FORM

0FCB2
ARABIC LIGATURE SAD WITH KHAH INITIAL FORM

0FCB3
ARABIC LIGATURE SAD WITH MEEM INITIAL FORM

0FCB4
ARABIC LIGATURE DAD WITH JEEM INITIAL FORM

0FCB5
ARABIC LIGATURE DAD WITH HAH INITIAL FORM

0FCB6
ARABIC LIGATURE DAD WITH KHAH INITIAL FORM

0FCB7
ARABIC LIGATURE DAD WITH MEEM INITIAL FORM

0FCB8
ARABIC LIGATURE TAH WITH HAH INITIAL FORM

0FCB9
ARABIC LIGATURE ZAH WITH MEEM INITIAL FORM

0FCBA
ARABIC LIGATURE AIN WITH JEEM INITIAL FORM

0FCBB
ARABIC LIGATURE AIN WITH MEEM INITIAL FORM

0FCBC
ARABIC LIGATURE GHAIN WITH JEEM INITIAL FORM

0FCBD
ARABIC LIGATURE GHAIN WITH MEEM INITIAL FORM

0FCBE
ARABIC LIGATURE FEH WITH JEEM INITIAL FORM

0FCBF
ARABIC LIGATURE FEH WITH HAH INITIAL FORM

0FCC0
ARABIC LIGATURE FEH WITH KHAH INITIAL FORM

0FCC1
ARABIC LIGATURE FEH WITH MEEM INITIAL FORM

0FCC2
ARABIC LIGATURE QAF WITH HAH INITIAL FORM

0FCC3
ARABIC LIGATURE QAF WITH MEEM INITIAL FORM

0FCC4
ARABIC LIGATURE KAF WITH JEEM INITIAL FORM

0FCC5
ARABIC LIGATURE KAF WITH HAH INITIAL FORM

0FCC6
ARABIC LIGATURE KAF WITH KHAH INITIAL FORM

0FCC7
ARABIC LIGATURE KAF WITH LAM INITIAL FORM

0FCC8
ARABIC LIGATURE KAF WITH MEEM INITIAL FORM

0FCC9
ARABIC LIGATURE LAM WITH JEEM INITIAL FORM

0FCCA
ARABIC LIGATURE LAM WITH HAH INITIAL FORM

0FCCB
ARABIC LIGATURE LAM WITH KHAH INITIAL FORM

0FCCC
ARABIC LIGATURE LAM WITH MEEM INITIAL FORM

0FCCD
ARABIC LIGATURE LAM WITH HEH INITIAL FORM

0FCCE
ARABIC LIGATURE MEEM WITH JEEM INITIAL FORM

0FCCF
ARABIC LIGATURE MEEM WITH HAH INITIAL FORM

0FCD0
ARABIC LIGATURE MEEM WITH KHAH INITIAL FORM

0FCD1
ARABIC LIGATURE MEEM WITH MEEM INITIAL FORM

0FCD2
ARABIC LIGATURE NOON WITH JEEM INITIAL FORM

0FCD3
ARABIC LIGATURE NOON WITH HAH INITIAL FORM

0FCD4
ARABIC LIGATURE NOON WITH KHAH INITIAL FORM

0FCD5
ARABIC LIGATURE NOON WITH MEEM INITIAL FORM

0FCD6
ARABIC LIGATURE NOON WITH HEH INITIAL FORM

0FCD7
ARABIC LIGATURE HEH WITH JEEM INITIAL FORM

0FCD8
ARABIC LIGATURE HEH WITH MEEM INITIAL FORM

0FCD9
ARABIC LIGATURE HEH WITH SUPERSCRIPT ALEF INITIAL FORM

0FCDA
ARABIC LIGATURE YEH WITH JEEM INITIAL FORM

0FCDB
ARABIC LIGATURE YEH WITH HAH INITIAL FORM

0FCDC
ARABIC LIGATURE YEH WITH KHAH INITIAL FORM

0FCDD
ARABIC LIGATURE YEH WITH MEEM INITIAL FORM

0FCDE
ARABIC LIGATURE YEH WITH HEH INITIAL FORM

0FCDF
ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH MEEM MEDIAL FORM

0FCE0
ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH HEH MEDIAL FORM

0FCE1
ARABIC LIGATURE BEH WITH MEEM MEDIAL FORM

0FCE2
ARABIC LIGATURE BEH WITH HEH MEDIAL FORM

0FCE3
ARABIC LIGATURE TEH WITH MEEM MEDIAL FORM

0FCE4
ARABIC LIGATURE TEH WITH HEH MEDIAL FORM

0FCE5
ARABIC LIGATURE THEH WITH MEEM MEDIAL FORM

0FCE6
ARABIC LIGATURE THEH WITH HEH MEDIAL FORM

0FCE7
ARABIC LIGATURE SEEN WITH MEEM MEDIAL FORM

0FCE8
ARABIC LIGATURE SEEN WITH HEH MEDIAL FORM

0FCE9
ARABIC LIGATURE SHEEN WITH MEEM MEDIAL FORM

0FCEA
ARABIC LIGATURE SHEEN WITH HEH MEDIAL FORM

0FCEB
ARABIC LIGATURE KAF WITH LAM MEDIAL FORM

0FCEC
ARABIC LIGATURE KAF WITH MEEM MEDIAL FORM

0FCED
ARABIC LIGATURE LAM WITH MEEM MEDIAL FORM

0FCEE
ARABIC LIGATURE NOON WITH MEEM MEDIAL FORM

0FCEF
ARABIC LIGATURE NOON WITH HEH MEDIAL FORM

0FCF0
ARABIC LIGATURE YEH WITH MEEM MEDIAL FORM

0FCF1
ARABIC LIGATURE YEH WITH HEH MEDIAL FORM

0FCF2
ARABIC LIGATURE SHADDA WITH FATHA MEDIAL FORM

0FCF3
ARABIC LIGATURE SHADDA WITH DAMMA MEDIAL FORM

0FCF4
ARABIC LIGATURE SHADDA WITH KASRA MEDIAL FORM

0FCF5
ARABIC LIGATURE TAH WITH ALEF MAKSURA ISOLATED FORM

0FCF6
ARABIC LIGATURE TAH WITH YEH ISOLATED FORM

0FCF7
ARABIC LIGATURE AIN WITH ALEF MAKSURA ISOLATED FORM

0FCF8
ARABIC LIGATURE AIN WITH YEH ISOLATED FORM

0FCF9
ARABIC LIGATURE GHAIN WITH ALEF MAKSURA ISOLATED FORM

0FCFA
ARABIC LIGATURE GHAIN WITH YEH ISOLATED FORM

0FCFB
ARABIC LIGATURE SEEN WITH ALEF MAKSURA ISOLATED FORM

0FCFC
ARABIC LIGATURE SEEN WITH YEH ISOLATED FORM

0FCFD
ARABIC LIGATURE SHEEN WITH ALEF MAKSURA ISOLATED FORM

0FCFE
ARABIC LIGATURE SHEEN WITH YEH ISOLATED FORM

0FCFF
ARABIC LIGATURE HAH WITH ALEF MAKSURA ISOLATED FORM

0FD00
ARABIC LIGATURE HAH WITH YEH ISOLATED FORM

0FD01
ARABIC LIGATURE JEEM WITH ALEF MAKSURA ISOLATED FORM

0FD02
ARABIC LIGATURE JEEM WITH YEH ISOLATED FORM

0FD03
ARABIC LIGATURE KHAH WITH ALEF MAKSURA ISOLATED FORM

0FD04
ARABIC LIGATURE KHAH WITH YEH ISOLATED FORM

0FD05
ARABIC LIGATURE SAD WITH ALEF MAKSURA ISOLATED FORM

0FD06
ARABIC LIGATURE SAD WITH YEH ISOLATED FORM

0FD07
ARABIC LIGATURE DAD WITH ALEF MAKSURA ISOLATED FORM

0FD08
ARABIC LIGATURE DAD WITH YEH ISOLATED FORM

0FD09
ARABIC LIGATURE SHEEN WITH JEEM ISOLATED FORM

0FD0A
ARABIC LIGATURE SHEEN WITH HAH ISOLATED FORM

0FD0B
ARABIC LIGATURE SHEEN WITH KHAH ISOLATED FORM

0FD0C
ARABIC LIGATURE SHEEN WITH MEEM ISOLATED FORM

0FD0D
ARABIC LIGATURE SHEEN WITH REH ISOLATED FORM

0FD0E
ARABIC LIGATURE SEEN WITH REH ISOLATED FORM

0FD0F
ARABIC LIGATURE SAD WITH REH ISOLATED FORM

0FD10
ARABIC LIGATURE DAD WITH REH ISOLATED FORM

0FD11
ARABIC LIGATURE TAH WITH ALEF MAKSURA FINAL FORM

0FD12
ARABIC LIGATURE TAH WITH YEH FINAL FORM

0FD13
ARABIC LIGATURE AIN WITH ALEF MAKSURA FINAL FORM

0FD14
ARABIC LIGATURE AIN WITH YEH FINAL FORM

0FD15
ARABIC LIGATURE GHAIN WITH ALEF MAKSURA FINAL FORM

0FD16
ARABIC LIGATURE GHAIN WITH YEH FINAL FORM

0FD17
ARABIC LIGATURE SEEN WITH ALEF MAKSURA FINAL FORM

0FD18
ARABIC LIGATURE SEEN WITH YEH FINAL FORM

0FD19
ARABIC LIGATURE SHEEN WITH ALEF MAKSURA FINAL FORM

0FD1A
ARABIC LIGATURE SHEEN WITH YEH FINAL FORM

0FD1B
ARABIC LIGATURE HAH WITH ALEF MAKSURA FINAL FORM

0FD1C
ARABIC LIGATURE HAH WITH YEH FINAL FORM

0FD1D
ARABIC LIGATURE JEEM WITH ALEF MAKSURA FINAL FORM

0FD1E
ARABIC LIGATURE JEEM WITH YEH FINAL FORM

0FD1F
ARABIC LIGATURE KHAH WITH ALEF MAKSURA FINAL FORM

0FD20
ARABIC LIGATURE KHAH WITH YEH FINAL FORM

0FD21
ARABIC LIGATURE SAD WITH ALEF MAKSURA FINAL FORM

0FD22
ARABIC LIGATURE SAD WITH YEH FINAL FORM

0FD23
ARABIC LIGATURE DAD WITH ALEF MAKSURA FINAL FORM

0FD24
ARABIC LIGATURE DAD WITH YEH FINAL FORM

0FD25
ARABIC LIGATURE SHEEN WITH JEEM FINAL FORM

0FD26
ARABIC LIGATURE SHEEN WITH HAH FINAL FORM

0FD27
ARABIC LIGATURE SHEEN WITH KHAH FINAL FORM

0FD28
ARABIC LIGATURE SHEEN WITH MEEM FINAL FORM

0FD29
ARABIC LIGATURE SHEEN WITH REH FINAL FORM

0FD2A
ARABIC LIGATURE SEEN WITH REH FINAL FORM

0FD2B
ARABIC LIGATURE SAD WITH REH FINAL FORM

0FD2C
ARABIC LIGATURE DAD WITH REH FINAL FORM

0FD2D
ARABIC LIGATURE SHEEN WITH JEEM INITIAL FORM

0FD2E
ARABIC LIGATURE SHEEN WITH HAH INITIAL FORM

0FD2F
ARABIC LIGATURE SHEEN WITH KHAH INITIAL FORM

0FD30
ARABIC LIGATURE SHEEN WITH MEEM INITIAL FORM

0FD31
ARABIC LIGATURE SEEN WITH HEH INITIAL FORM

0FD32
ARABIC LIGATURE SHEEN WITH HEH INITIAL FORM

0FD33
ARABIC LIGATURE TAH WITH MEEM INITIAL FORM

0FD34
ARABIC LIGATURE SEEN WITH JEEM MEDIAL FORM

0FD35
ARABIC LIGATURE SEEN WITH HAH MEDIAL FORM

0FD36
ARABIC LIGATURE SEEN WITH KHAH MEDIAL FORM

0FD37
ARABIC LIGATURE SHEEN WITH JEEM MEDIAL FORM

0FD38
ARABIC LIGATURE SHEEN WITH HAH MEDIAL FORM

0FD39
ARABIC LIGATURE SHEEN WITH KHAH MEDIAL FORM

0FD3A
ARABIC LIGATURE TAH WITH MEEM MEDIAL FORM

0FD3B
ARABIC LIGATURE ZAH WITH MEEM MEDIAL FORM

0FD3C
ARABIC LIGATURE ALEF WITH FATHATAN FINAL FORM

0FD3D
ARABIC LIGATURE ALEF WITH FATHATAN ISOLATED FORM

0FD3E
ORNATE LEFT PARENTHESIS

0FD3F
ORNATE RIGHT PARENTHESIS

0FD40
ARABIC LIGATURE RAHIMAHU ALLAAH

0FD41
ARABIC LIGATURE RADI ALLAAHU ANH

0FD42
ARABIC LIGATURE RADI ALLAAHU ANHAA

0FD43
ARABIC LIGATURE RADI ALLAAHU ANHUM

0FD44
ARABIC LIGATURE RADI ALLAAHU ANHUMAA

0FD45
ARABIC LIGATURE RADI ALLAAHU ANHUNNA

0FD46
ARABIC LIGATURE SALLALLAAHU ALAYHI WA-AALIH

0FD47
ARABIC LIGATURE ALAYHI AS-SALAAM

0FD48
ARABIC LIGATURE ALAYHIM AS-SALAAM

0FD49
ARABIC LIGATURE ALAYHIMAA AS-SALAAM

0FD4A
ARABIC LIGATURE ALAYHI AS-SALAATU WAS-SALAAM

0FD4B
ARABIC LIGATURE QUDDISA SIRRAH

0FD4C
ARABIC LIGATURE SALLALLAHU ALAYHI WAAALIHEE WA-SALLAM

0FD4D
ARABIC LIGATURE ALAYHAA AS-SALAAM

0FD4E
ARABIC LIGATURE TABAARAKA WA-TAAALAA

0FD4F
ARABIC LIGATURE RAHIMAHUM ALLAAH

0FD50
ARABIC LIGATURE TEH WITH JEEM WITH MEEM INITIAL FORM

0FD51
ARABIC LIGATURE TEH WITH HAH WITH JEEM FINAL FORM

0FD52
ARABIC LIGATURE TEH WITH HAH WITH JEEM INITIAL FORM

0FD53
ARABIC LIGATURE TEH WITH HAH WITH MEEM INITIAL FORM

0FD54
ARABIC LIGATURE TEH WITH KHAH WITH MEEM INITIAL FORM

0FD55
ARABIC LIGATURE TEH WITH MEEM WITH JEEM INITIAL FORM

0FD56
ARABIC LIGATURE TEH WITH MEEM WITH HAH INITIAL FORM

0FD57
ARABIC LIGATURE TEH WITH MEEM WITH KHAH INITIAL FORM

0FD58
ARABIC LIGATURE JEEM WITH MEEM WITH HAH FINAL FORM

0FD59
ARABIC LIGATURE JEEM WITH MEEM WITH HAH INITIAL FORM

0FD5A
ARABIC LIGATURE HAH WITH MEEM WITH YEH FINAL FORM

0FD5B
ARABIC LIGATURE HAH WITH MEEM WITH ALEF MAKSURA FINAL FORM

0FD5C
ARABIC LIGATURE SEEN WITH HAH WITH JEEM INITIAL FORM

0FD5D
ARABIC LIGATURE SEEN WITH JEEM WITH HAH INITIAL FORM

0FD5E
ARABIC LIGATURE SEEN WITH JEEM WITH ALEF MAKSURA FINAL FORM

0FD5F
ARABIC LIGATURE SEEN WITH MEEM WITH HAH FINAL FORM

0FD60
ARABIC LIGATURE SEEN WITH MEEM WITH HAH INITIAL FORM

0FD61
ARABIC LIGATURE SEEN WITH MEEM WITH JEEM INITIAL FORM

0FD62
ARABIC LIGATURE SEEN WITH MEEM WITH MEEM FINAL FORM

0FD63
ARABIC LIGATURE SEEN WITH MEEM WITH MEEM INITIAL FORM

0FD64
ARABIC LIGATURE SAD WITH HAH WITH HAH FINAL FORM

0FD65
ARABIC LIGATURE SAD WITH HAH WITH HAH INITIAL FORM

0FD66
ARABIC LIGATURE SAD WITH MEEM WITH MEEM FINAL FORM

0FD67
ARABIC LIGATURE SHEEN WITH HAH WITH MEEM FINAL FORM

0FD68
ARABIC LIGATURE SHEEN WITH HAH WITH MEEM INITIAL FORM

0FD69
ARABIC LIGATURE SHEEN WITH JEEM WITH YEH FINAL FORM

0FD6A
ARABIC LIGATURE SHEEN WITH MEEM WITH KHAH FINAL FORM

0FD6B
ARABIC LIGATURE SHEEN WITH MEEM WITH KHAH INITIAL FORM

0FD6C
ARABIC LIGATURE SHEEN WITH MEEM WITH MEEM FINAL FORM

0FD6D
ARABIC LIGATURE SHEEN WITH MEEM WITH MEEM INITIAL FORM

0FD6E
ARABIC LIGATURE DAD WITH HAH WITH ALEF MAKSURA FINAL FORM

0FD6F
ARABIC LIGATURE DAD WITH KHAH WITH MEEM FINAL FORM

0FD70
ARABIC LIGATURE DAD WITH KHAH WITH MEEM INITIAL FORM

0FD71
ARABIC LIGATURE TAH WITH MEEM WITH HAH FINAL FORM

0FD72
ARABIC LIGATURE TAH WITH MEEM WITH HAH INITIAL FORM

0FD73
ARABIC LIGATURE TAH WITH MEEM WITH MEEM INITIAL FORM

0FD74
ARABIC LIGATURE TAH WITH MEEM WITH YEH FINAL FORM

0FD75
ARABIC LIGATURE AIN WITH JEEM WITH MEEM FINAL FORM

0FD76
ARABIC LIGATURE AIN WITH MEEM WITH MEEM FINAL FORM

0FD77
ARABIC LIGATURE AIN WITH MEEM WITH MEEM INITIAL FORM

0FD78
ARABIC LIGATURE AIN WITH MEEM WITH ALEF MAKSURA FINAL FORM

0FD79
ARABIC LIGATURE GHAIN WITH MEEM WITH MEEM FINAL FORM

0FD7A
ARABIC LIGATURE GHAIN WITH MEEM WITH YEH FINAL FORM

0FD7B
ARABIC LIGATURE GHAIN WITH MEEM WITH ALEF MAKSURA FINAL FORM

0FD7C
ARABIC LIGATURE FEH WITH KHAH WITH MEEM FINAL FORM

0FD7D
ARABIC LIGATURE FEH WITH KHAH WITH MEEM INITIAL FORM

0FD7E
ARABIC LIGATURE QAF WITH MEEM WITH HAH FINAL FORM

0FD7F
ARABIC LIGATURE QAF WITH MEEM WITH MEEM FINAL FORM

0FD80
ARABIC LIGATURE LAM WITH HAH WITH MEEM FINAL FORM

0FD81
ARABIC LIGATURE LAM WITH HAH WITH YEH FINAL FORM

0FD82
ARABIC LIGATURE LAM WITH HAH WITH ALEF MAKSURA FINAL FORM

0FD83
ARABIC LIGATURE LAM WITH JEEM WITH JEEM INITIAL FORM

0FD84
ARABIC LIGATURE LAM WITH JEEM WITH JEEM FINAL FORM

0FD85
ARABIC LIGATURE LAM WITH KHAH WITH MEEM FINAL FORM

0FD86
ARABIC LIGATURE LAM WITH KHAH WITH MEEM INITIAL FORM

0FD87
ARABIC LIGATURE LAM WITH MEEM WITH HAH FINAL FORM

0FD88
ARABIC LIGATURE LAM WITH MEEM WITH HAH INITIAL FORM

0FD89
ARABIC LIGATURE MEEM WITH HAH WITH JEEM INITIAL FORM

0FD8A
ARABIC LIGATURE MEEM WITH HAH WITH MEEM INITIAL FORM

0FD8B
ARABIC LIGATURE MEEM WITH HAH WITH YEH FINAL FORM

0FD8C
ARABIC LIGATURE MEEM WITH JEEM WITH HAH INITIAL FORM

0FD8D
ARABIC LIGATURE MEEM WITH JEEM WITH MEEM INITIAL FORM

0FD8E
ARABIC LIGATURE MEEM WITH KHAH WITH JEEM INITIAL FORM

0FD8F
ARABIC LIGATURE MEEM WITH KHAH WITH MEEM INITIAL FORM

0FD90
ARABIC LIGATURE RAHMATU ALLAAHI ALAYH

0FD91
ARABIC LIGATURE RAHMATU ALLAAHI ALAYHAA

0FD92
ARABIC LIGATURE MEEM WITH JEEM WITH KHAH INITIAL FORM

0FD93
ARABIC LIGATURE HEH WITH MEEM WITH JEEM INITIAL FORM

0FD94
ARABIC LIGATURE HEH WITH MEEM WITH MEEM INITIAL FORM

0FD95
ARABIC LIGATURE NOON WITH HAH WITH MEEM INITIAL FORM

0FD96
ARABIC LIGATURE NOON WITH HAH WITH ALEF MAKSURA FINAL FORM

0FD97
ARABIC LIGATURE NOON WITH JEEM WITH MEEM FINAL FORM

0FD98
ARABIC LIGATURE NOON WITH JEEM WITH MEEM INITIAL FORM

0FD99
ARABIC LIGATURE NOON WITH JEEM WITH ALEF MAKSURA FINAL FORM

0FD9A
ARABIC LIGATURE NOON WITH MEEM WITH YEH FINAL FORM

0FD9B
ARABIC LIGATURE NOON WITH MEEM WITH ALEF MAKSURA FINAL FORM

0FD9C
ARABIC LIGATURE YEH WITH MEEM WITH MEEM FINAL FORM

0FD9D
ARABIC LIGATURE YEH WITH MEEM WITH MEEM INITIAL FORM

0FD9E
ARABIC LIGATURE BEH WITH KHAH WITH YEH FINAL FORM

0FD9F
ARABIC LIGATURE TEH WITH JEEM WITH YEH FINAL FORM

0FDA0
ARABIC LIGATURE TEH WITH JEEM WITH ALEF MAKSURA FINAL FORM

0FDA1
ARABIC LIGATURE TEH WITH KHAH WITH YEH FINAL FORM

0FDA2
ARABIC LIGATURE TEH WITH KHAH WITH ALEF MAKSURA FINAL FORM

0FDA3
ARABIC LIGATURE TEH WITH MEEM WITH YEH FINAL FORM

0FDA4
ARABIC LIGATURE TEH WITH MEEM WITH ALEF MAKSURA FINAL FORM

0FDA5
ARABIC LIGATURE JEEM WITH MEEM WITH YEH FINAL FORM

0FDA6
ARABIC LIGATURE JEEM WITH HAH WITH ALEF MAKSURA FINAL FORM

0FDA7
ARABIC LIGATURE JEEM WITH MEEM WITH ALEF MAKSURA FINAL FORM

0FDA8
ARABIC LIGATURE SEEN WITH KHAH WITH ALEF MAKSURA FINAL FORM

0FDA9
ARABIC LIGATURE SAD WITH HAH WITH YEH FINAL FORM

0FDAA
ARABIC LIGATURE SHEEN WITH HAH WITH YEH FINAL FORM

0FDAB
ARABIC LIGATURE DAD WITH HAH WITH YEH FINAL FORM

0FDAC
ARABIC LIGATURE LAM WITH JEEM WITH YEH FINAL FORM

0FDAD
ARABIC LIGATURE LAM WITH MEEM WITH YEH FINAL FORM

0FDAE
ARABIC LIGATURE YEH WITH HAH WITH YEH FINAL FORM

0FDAF
ARABIC LIGATURE YEH WITH JEEM WITH YEH FINAL FORM

0FDB0
ARABIC LIGATURE YEH WITH MEEM WITH YEH FINAL FORM

0FDB1
ARABIC LIGATURE MEEM WITH MEEM WITH YEH FINAL FORM

0FDB2
ARABIC LIGATURE QAF WITH MEEM WITH YEH FINAL FORM

0FDB3
ARABIC LIGATURE NOON WITH HAH WITH YEH FINAL FORM

0FDB4
ARABIC LIGATURE QAF WITH MEEM WITH HAH INITIAL FORM

0FDB5
ARABIC LIGATURE LAM WITH HAH WITH MEEM INITIAL FORM

0FDB6
ARABIC LIGATURE AIN WITH MEEM WITH YEH FINAL FORM

0FDB7
ARABIC LIGATURE KAF WITH MEEM WITH YEH FINAL FORM

0FDB8
ARABIC LIGATURE NOON WITH JEEM WITH HAH INITIAL FORM

0FDB9
ARABIC LIGATURE MEEM WITH KHAH WITH YEH FINAL FORM

0FDBA
ARABIC LIGATURE LAM WITH JEEM WITH MEEM INITIAL FORM

0FDBB
ARABIC LIGATURE KAF WITH MEEM WITH MEEM FINAL FORM

0FDBC
ARABIC LIGATURE LAM WITH JEEM WITH MEEM FINAL FORM

0FDBD
ARABIC LIGATURE NOON WITH JEEM WITH HAH FINAL FORM

0FDBE
ARABIC LIGATURE JEEM WITH HAH WITH YEH FINAL FORM

0FDBF
ARABIC LIGATURE HAH WITH JEEM WITH YEH FINAL FORM

0FDC0
ARABIC LIGATURE MEEM WITH JEEM WITH YEH FINAL FORM

0FDC1
ARABIC LIGATURE FEH WITH MEEM WITH YEH FINAL FORM

0FDC2
ARABIC LIGATURE BEH WITH HAH WITH YEH FINAL FORM

0FDC3
ARABIC LIGATURE KAF WITH MEEM WITH MEEM INITIAL FORM

0FDC4
ARABIC LIGATURE AIN WITH JEEM WITH MEEM INITIAL FORM

0FDC5
ARABIC LIGATURE SAD WITH MEEM WITH MEEM INITIAL FORM

0FDC6
ARABIC LIGATURE SEEN WITH KHAH WITH YEH FINAL FORM

0FDC7
ARABIC LIGATURE NOON WITH JEEM WITH YEH FINAL FORM

0FDC8
ARABIC LIGATURE RAHIMAHU ALLAAH TAAALAA

0FDC9
ARABIC LIGATURE RADI ALLAAHU TAAALAA ANH

0FDCA
ARABIC LIGATURE RADI ALLAAHU TAAALAA ANHAA

0FDCB
ARABIC LIGATURE RADI ALLAAHU TAAALAA ANHUMAA

0FDCC
ARABIC LIGATURE SALLALLAHU ALAYHI WA-ALAA AALIHEE WA-SALLAM

0FDCD
ARABIC LIGATURE AJJAL ALLAAHU TAAALAA FARAJAHU ASH-SHAREEF

0FDCE
ARABIC LIGATURE KARRAMA ALLAAHU WAJHAH

0FDCF
ARABIC LIGATURE SALAAMUHU ALAYNAA

0FDF0
ARABIC LIGATURE SALLA USED AS KORANIC STOP SIGN ISOLATED FORM

0FDF1
ARABIC LIGATURE QALA USED AS KORANIC STOP SIGN ISOLATED FORM

0FDF2
ARABIC LIGATURE ALLAH ISOLATED FORM

0FDF3
ARABIC LIGATURE AKBAR ISOLATED FORM

0FDF4
ARABIC LIGATURE MOHAMMAD ISOLATED FORM

0FDF5
ARABIC LIGATURE SALAM ISOLATED FORM

0FDF6
ARABIC LIGATURE RASOUL ISOLATED FORM

0FDF7
ARABIC LIGATURE ALAYHE ISOLATED FORM

0FDF8
ARABIC LIGATURE WASALLAM ISOLATED FORM

0FDF9
ARABIC LIGATURE SALLA ISOLATED FORM

0FDFA
ARABIC LIGATURE SALLALLAHOU ALAYHE WASALLAM

0FDFB
ARABIC LIGATURE JALLAJALALOUHOU

0FDFC
RIAL SIGN

0FDFD
ARABIC LIGATURE BISMILLAH AR-RAHMAN AR-RAHEEM

0FDFE
ARABIC LIGATURE SUBHAANAHU WA TAAALAA

0FDFF
ARABIC LIGATURE AZZA WA JALL

0FE00
VARIATION SELECTOR-1

0FE00
VS1

0FE01
VARIATION SELECTOR-2

0FE01
VS2

0FE02
VARIATION SELECTOR-3

0FE02
VS3

0FE03
VARIATION SELECTOR-4

0FE03
VS4

0FE04
VARIATION SELECTOR-5

0FE04
VS5

0FE05
VARIATION SELECTOR-6

0FE05
VS6

0FE06
VARIATION SELECTOR-7

0FE06
VS7

0FE07
VARIATION SELECTOR-8

0FE07
VS8

0FE08
VARIATION SELECTOR-9

0FE08
VS9

0FE09
VARIATION SELECTOR-10

0FE09
VS10

0FE0A
VARIATION SELECTOR-11

0FE0A
VS11

0FE0B
VARIATION SELECTOR-12

0FE0B
VS12

0FE0C
VARIATION SELECTOR-13

0FE0C
VS13

0FE0D
VARIATION SELECTOR-14

0FE0D
VS14

0FE0E
VARIATION SELECTOR-15

0FE0E
VS15

0FE0F
VARIATION SELECTOR-16

0FE0F
VS16

0FE10
PRESENTATION FORM FOR VERTICAL COMMA

0FE11
PRESENTATION FORM FOR VERTICAL IDEOGRAPHIC COMMA

0FE12
PRESENTATION FORM FOR VERTICAL IDEOGRAPHIC FULL STOP

0FE13
PRESENTATION FORM FOR VERTICAL COLON

0FE14
PRESENTATION FORM FOR VERTICAL SEMICOLON

0FE15
PRESENTATION FORM FOR VERTICAL EXCLAMATION MARK

0FE16
PRESENTATION FORM FOR VERTICAL QUESTION MARK

0FE17
PRESENTATION FORM FOR VERTICAL LEFT WHITE LENTICULAR BRACKET

0FE18
PRESENTATION FORM FOR VERTICAL RIGHT WHITE LENTICULAR BRACKET

0FE18
PRESENTATION FORM FOR VERTICAL RIGHT WHITE LENTICULAR BRAKCET

0FE19
PRESENTATION FORM FOR VERTICAL HORIZONTAL ELLIPSIS

0FE20
COMBINING LIGATURE LEFT HALF

0FE21
COMBINING LIGATURE RIGHT HALF

0FE22
COMBINING DOUBLE TILDE LEFT HALF

0FE23
COMBINING DOUBLE TILDE RIGHT HALF

0FE24
COMBINING MACRON LEFT HALF

0FE25
COMBINING MACRON RIGHT HALF

0FE26
COMBINING CONJOINING MACRON

0FE27
COMBINING LIGATURE LEFT HALF BELOW

0FE28
COMBINING LIGATURE RIGHT HALF BELOW

0FE29
COMBINING TILDE LEFT HALF BELOW

0FE2A
COMBINING TILDE RIGHT HALF BELOW

0FE2B
COMBINING MACRON LEFT HALF BELOW

0FE2C
COMBINING MACRON RIGHT HALF BELOW

0FE2D
COMBINING CONJOINING MACRON BELOW

0FE2E
COMBINING CYRILLIC TITLO LEFT HALF

0FE2F
COMBINING CYRILLIC TITLO RIGHT HALF

0FE30
PRESENTATION FORM FOR VERTICAL TWO DOT LEADER

0FE31
PRESENTATION FORM FOR VERTICAL EM DASH

0FE32
PRESENTATION FORM FOR VERTICAL EN DASH

0FE33
PRESENTATION FORM FOR VERTICAL LOW LINE

0FE34
PRESENTATION FORM FOR VERTICAL WAVY LOW LINE

0FE35
PRESENTATION FORM FOR VERTICAL LEFT PARENTHESIS

0FE36
PRESENTATION FORM FOR VERTICAL RIGHT PARENTHESIS

0FE37
PRESENTATION FORM FOR VERTICAL LEFT CURLY BRACKET

0FE38
PRESENTATION FORM FOR VERTICAL RIGHT CURLY BRACKET

0FE39
PRESENTATION FORM FOR VERTICAL LEFT TORTOISE SHELL BRACKET

0FE3A
PRESENTATION FORM FOR VERTICAL RIGHT TORTOISE SHELL BRACKET

0FE3B
PRESENTATION FORM FOR VERTICAL LEFT BLACK LENTICULAR BRACKET

0FE3C
PRESENTATION FORM FOR VERTICAL RIGHT BLACK LENTICULAR BRACKET

0FE3D
PRESENTATION FORM FOR VERTICAL LEFT DOUBLE ANGLE BRACKET

0FE3E
PRESENTATION FORM FOR VERTICAL RIGHT DOUBLE ANGLE BRACKET

0FE3F
PRESENTATION FORM FOR VERTICAL LEFT ANGLE BRACKET

0FE40
PRESENTATION FORM FOR VERTICAL RIGHT ANGLE BRACKET

0FE41
PRESENTATION FORM FOR VERTICAL LEFT CORNER BRACKET

0FE42
PRESENTATION FORM FOR VERTICAL RIGHT CORNER BRACKET

0FE43
PRESENTATION FORM FOR VERTICAL LEFT WHITE CORNER BRACKET

0FE44
PRESENTATION FORM FOR VERTICAL RIGHT WHITE CORNER BRACKET

0FE45
SESAME DOT

0FE46
WHITE SESAME DOT

0FE47
PRESENTATION FORM FOR VERTICAL LEFT SQUARE BRACKET

0FE48
PRESENTATION FORM FOR VERTICAL RIGHT SQUARE BRACKET

0FE49
DASHED OVERLINE

0FE4A
CENTRELINE OVERLINE

0FE4B
WAVY OVERLINE

0FE4C
DOUBLE WAVY OVERLINE

0FE4D
DASHED LOW LINE

0FE4E
CENTRELINE LOW LINE

0FE4F
WAVY LOW LINE

0FE50
SMALL COMMA

0FE51
SMALL IDEOGRAPHIC COMMA

0FE52
SMALL FULL STOP

0FE54
SMALL SEMICOLON

0FE55
SMALL COLON

0FE56
SMALL QUESTION MARK

0FE57
SMALL EXCLAMATION MARK

0FE58
SMALL EM DASH

0FE59
SMALL LEFT PARENTHESIS

0FE5A
SMALL RIGHT PARENTHESIS

0FE5B
SMALL LEFT CURLY BRACKET

0FE5C
SMALL RIGHT CURLY BRACKET

0FE5D
SMALL LEFT TORTOISE SHELL BRACKET

0FE5E
SMALL RIGHT TORTOISE SHELL BRACKET

0FE5F
SMALL NUMBER SIGN

0FE60
SMALL AMPERSAND

0FE61
SMALL ASTERISK

0FE62
SMALL PLUS SIGN

0FE63
SMALL HYPHEN-MINUS

0FE64
SMALL LESS-THAN SIGN

0FE65
SMALL GREATER-THAN SIGN

0FE66
SMALL EQUALS SIGN

0FE68
SMALL REVERSE SOLIDUS

0FE69
SMALL DOLLAR SIGN

0FE6A
SMALL PERCENT SIGN

0FE6B
SMALL COMMERCIAL AT

0FE70
ARABIC FATHATAN ISOLATED FORM

0FE71
ARABIC TATWEEL WITH FATHATAN ABOVE

0FE72
ARABIC DAMMATAN ISOLATED FORM

0FE73
ARABIC TAIL FRAGMENT

0FE74
ARABIC KASRATAN ISOLATED FORM

0FE76
ARABIC FATHA ISOLATED FORM

0FE77
ARABIC FATHA MEDIAL FORM

0FE78
ARABIC DAMMA ISOLATED FORM

0FE79
ARABIC DAMMA MEDIAL FORM

0FE7A
ARABIC KASRA ISOLATED FORM

0FE7B
ARABIC KASRA MEDIAL FORM

0FE7C
ARABIC SHADDA ISOLATED FORM

0FE7D
ARABIC SHADDA MEDIAL FORM

0FE7E
ARABIC SUKUN ISOLATED FORM

0FE7F
ARABIC SUKUN MEDIAL FORM

0FE80
ARABIC LETTER HAMZA ISOLATED FORM

0FE81
ARABIC LETTER ALEF WITH MADDA ABOVE ISOLATED FORM

0FE82
ARABIC LETTER ALEF WITH MADDA ABOVE FINAL FORM

0FE83
ARABIC LETTER ALEF WITH HAMZA ABOVE ISOLATED FORM

0FE84
ARABIC LETTER ALEF WITH HAMZA ABOVE FINAL FORM

0FE85
ARABIC LETTER WAW WITH HAMZA ABOVE ISOLATED FORM

0FE86
ARABIC LETTER WAW WITH HAMZA ABOVE FINAL FORM

0FE87
ARABIC LETTER ALEF WITH HAMZA BELOW ISOLATED FORM

0FE88
ARABIC LETTER ALEF WITH HAMZA BELOW FINAL FORM

0FE89
ARABIC LETTER YEH WITH HAMZA ABOVE ISOLATED FORM

0FE8A
ARABIC LETTER YEH WITH HAMZA ABOVE FINAL FORM

0FE8B
ARABIC LETTER YEH WITH HAMZA ABOVE INITIAL FORM

0FE8C
ARABIC LETTER YEH WITH HAMZA ABOVE MEDIAL FORM

0FE8D
ARABIC LETTER ALEF ISOLATED FORM

0FE8E
ARABIC LETTER ALEF FINAL FORM

0FE8F
ARABIC LETTER BEH ISOLATED FORM

0FE90
ARABIC LETTER BEH FINAL FORM

0FE91
ARABIC LETTER BEH INITIAL FORM

0FE92
ARABIC LETTER BEH MEDIAL FORM

0FE93
ARABIC LETTER TEH MARBUTA ISOLATED FORM

0FE94
ARABIC LETTER TEH MARBUTA FINAL FORM

0FE95
ARABIC LETTER TEH ISOLATED FORM

0FE96
ARABIC LETTER TEH FINAL FORM

0FE97
ARABIC LETTER TEH INITIAL FORM

0FE98
ARABIC LETTER TEH MEDIAL FORM

0FE99
ARABIC LETTER THEH ISOLATED FORM

0FE9A
ARABIC LETTER THEH FINAL FORM

0FE9B
ARABIC LETTER THEH INITIAL FORM

0FE9C
ARABIC LETTER THEH MEDIAL FORM

0FE9D
ARABIC LETTER JEEM ISOLATED FORM

0FE9E
ARABIC LETTER JEEM FINAL FORM

0FE9F
ARABIC LETTER JEEM INITIAL FORM

0FEA0
ARABIC LETTER JEEM MEDIAL FORM

0FEA1
ARABIC LETTER HAH ISOLATED FORM

0FEA2
ARABIC LETTER HAH FINAL FORM

0FEA3
ARABIC LETTER HAH INITIAL FORM

0FEA4
ARABIC LETTER HAH MEDIAL FORM

0FEA5
ARABIC LETTER KHAH ISOLATED FORM

0FEA6
ARABIC LETTER KHAH FINAL FORM

0FEA7
ARABIC LETTER KHAH INITIAL FORM

0FEA8
ARABIC LETTER KHAH MEDIAL FORM

0FEA9
ARABIC LETTER DAL ISOLATED FORM

0FEAA
ARABIC LETTER DAL FINAL FORM

0FEAB
ARABIC LETTER THAL ISOLATED FORM

0FEAC
ARABIC LETTER THAL FINAL FORM

0FEAD
ARABIC LETTER REH ISOLATED FORM

0FEAE
ARABIC LETTER REH FINAL FORM

0FEAF
ARABIC LETTER ZAIN ISOLATED FORM

0FEB0
ARABIC LETTER ZAIN FINAL FORM

0FEB1
ARABIC LETTER SEEN ISOLATED FORM

0FEB2
ARABIC LETTER SEEN FINAL FORM

0FEB3
ARABIC LETTER SEEN INITIAL FORM

0FEB4
ARABIC LETTER SEEN MEDIAL FORM

0FEB5
ARABIC LETTER SHEEN ISOLATED FORM

0FEB6
ARABIC LETTER SHEEN FINAL FORM

0FEB7
ARABIC LETTER SHEEN INITIAL FORM

0FEB8
ARABIC LETTER SHEEN MEDIAL FORM

0FEB9
ARABIC LETTER SAD ISOLATED FORM

0FEBA
ARABIC LETTER SAD FINAL FORM

0FEBB
ARABIC LETTER SAD INITIAL FORM

0FEBC
ARABIC LETTER SAD MEDIAL FORM

0FEBD
ARABIC LETTER DAD ISOLATED FORM

0FEBE
ARABIC LETTER DAD FINAL FORM

0FEBF
ARABIC LETTER DAD INITIAL FORM

0FEC0
ARABIC LETTER DAD MEDIAL FORM

0FEC1
ARABIC LETTER TAH ISOLATED FORM

0FEC2
ARABIC LETTER TAH FINAL FORM

0FEC3
ARABIC LETTER TAH INITIAL FORM

0FEC4
ARABIC LETTER TAH MEDIAL FORM

0FEC5
ARABIC LETTER ZAH ISOLATED FORM

0FEC6
ARABIC LETTER ZAH FINAL FORM

0FEC7
ARABIC LETTER ZAH INITIAL FORM

0FEC8
ARABIC LETTER ZAH MEDIAL FORM

0FEC9
ARABIC LETTER AIN ISOLATED FORM

0FECA
ARABIC LETTER AIN FINAL FORM

0FECB
ARABIC LETTER AIN INITIAL FORM

0FECC
ARABIC LETTER AIN MEDIAL FORM

0FECD
ARABIC LETTER GHAIN ISOLATED FORM

0FECE
ARABIC LETTER GHAIN FINAL FORM

0FECF
ARABIC LETTER GHAIN INITIAL FORM

0FED0
ARABIC LETTER GHAIN MEDIAL FORM

0FED1
ARABIC LETTER FEH ISOLATED FORM

0FED2
ARABIC LETTER FEH FINAL FORM

0FED3
ARABIC LETTER FEH INITIAL FORM

0FED4
ARABIC LETTER FEH MEDIAL FORM

0FED5
ARABIC LETTER QAF ISOLATED FORM

0FED6
ARABIC LETTER QAF FINAL FORM

0FED7
ARABIC LETTER QAF INITIAL FORM

0FED8
ARABIC LETTER QAF MEDIAL FORM

0FED9
ARABIC LETTER KAF ISOLATED FORM

0FEDA
ARABIC LETTER KAF FINAL FORM

0FEDB
ARABIC LETTER KAF INITIAL FORM

0FEDC
ARABIC LETTER KAF MEDIAL FORM

0FEDD
ARABIC LETTER LAM ISOLATED FORM

0FEDE
ARABIC LETTER LAM FINAL FORM

0FEDF
ARABIC LETTER LAM INITIAL FORM

0FEE0
ARABIC LETTER LAM MEDIAL FORM

0FEE1
ARABIC LETTER MEEM ISOLATED FORM

0FEE2
ARABIC LETTER MEEM FINAL FORM

0FEE3
ARABIC LETTER MEEM INITIAL FORM

0FEE4
ARABIC LETTER MEEM MEDIAL FORM

0FEE5
ARABIC LETTER NOON ISOLATED FORM

0FEE6
ARABIC LETTER NOON FINAL FORM

0FEE7
ARABIC LETTER NOON INITIAL FORM

0FEE8
ARABIC LETTER NOON MEDIAL FORM

0FEE9
ARABIC LETTER HEH ISOLATED FORM

0FEEA
ARABIC LETTER HEH FINAL FORM

0FEEB
ARABIC LETTER HEH INITIAL FORM

0FEEC
ARABIC LETTER HEH MEDIAL FORM

0FEED
ARABIC LETTER WAW ISOLATED FORM

0FEEE
ARABIC LETTER WAW FINAL FORM

0FEEF
ARABIC LETTER ALEF MAKSURA ISOLATED FORM

0FEF0
ARABIC LETTER ALEF MAKSURA FINAL FORM

0FEF1
ARABIC LETTER YEH ISOLATED FORM

0FEF2
ARABIC LETTER YEH FINAL FORM

0FEF3
ARABIC LETTER YEH INITIAL FORM

0FEF4
ARABIC LETTER YEH MEDIAL FORM

0FEF5
ARABIC LIGATURE LAM WITH ALEF WITH MADDA ABOVE ISOLATED FORM

0FEF6
ARABIC LIGATURE LAM WITH ALEF WITH MADDA ABOVE FINAL FORM

0FEF7
ARABIC LIGATURE LAM WITH ALEF WITH HAMZA ABOVE ISOLATED FORM

0FEF8
ARABIC LIGATURE LAM WITH ALEF WITH HAMZA ABOVE FINAL FORM

0FEF9
ARABIC LIGATURE LAM WITH ALEF WITH HAMZA BELOW ISOLATED FORM

0FEFA
ARABIC LIGATURE LAM WITH ALEF WITH HAMZA BELOW FINAL FORM

0FEFB
ARABIC LIGATURE LAM WITH ALEF ISOLATED FORM

0FEFC
ARABIC LIGATURE LAM WITH ALEF FINAL FORM

0FEFF
ZERO WIDTH NO-BREAK SPACE

0FEFF
BYTE ORDER MARK

0FEFF
BOM

0FEFF
ZWNBSP

0FF01
FULLWIDTH EXCLAMATION MARK

0FF02
FULLWIDTH QUOTATION MARK

0FF03
FULLWIDTH NUMBER SIGN

0FF04
FULLWIDTH DOLLAR SIGN

0FF05
FULLWIDTH PERCENT SIGN

0FF06
FULLWIDTH AMPERSAND

0FF07
FULLWIDTH APOSTROPHE

0FF08
FULLWIDTH LEFT PARENTHESIS

0FF09
FULLWIDTH RIGHT PARENTHESIS

0FF0A
FULLWIDTH ASTERISK

0FF0B
FULLWIDTH PLUS SIGN

0FF0C
FULLWIDTH COMMA

0FF0D
FULLWIDTH HYPHEN-MINUS

0FF0E
FULLWIDTH FULL STOP

0FF0F
FULLWIDTH SOLIDUS

0FF10
FULLWIDTH DIGIT ZERO

0FF11
FULLWIDTH DIGIT ONE

0FF12
FULLWIDTH DIGIT TWO

0FF13
FULLWIDTH DIGIT THREE

0FF14
FULLWIDTH DIGIT FOUR

0FF15
FULLWIDTH DIGIT FIVE

0FF16
FULLWIDTH DIGIT SIX

0FF17
FULLWIDTH DIGIT SEVEN

0FF18
FULLWIDTH DIGIT EIGHT

0FF19
FULLWIDTH DIGIT NINE

0FF1A
FULLWIDTH COLON

0FF1B
FULLWIDTH SEMICOLON

0FF1C
FULLWIDTH LESS-THAN SIGN

0FF1D
FULLWIDTH EQUALS SIGN

0FF1E
FULLWIDTH GREATER-THAN SIGN

0FF1F
FULLWIDTH QUESTION MARK

0FF20
FULLWIDTH COMMERCIAL AT

0FF21
FULLWIDTH LATIN CAPITAL LETTER A

0FF22
FULLWIDTH LATIN CAPITAL LETTER B

0FF23
FULLWIDTH LATIN CAPITAL LETTER C

0FF24
FULLWIDTH LATIN CAPITAL LETTER D

0FF25
FULLWIDTH LATIN CAPITAL LETTER E

0FF26
FULLWIDTH LATIN CAPITAL LETTER F

0FF27
FULLWIDTH LATIN CAPITAL LETTER G

0FF28
FULLWIDTH LATIN CAPITAL LETTER H

0FF29
FULLWIDTH LATIN CAPITAL LETTER I

0FF2A
FULLWIDTH LATIN CAPITAL LETTER J

0FF2B
FULLWIDTH LATIN CAPITAL LETTER K

0FF2C
FULLWIDTH LATIN CAPITAL LETTER L

0FF2D
FULLWIDTH LATIN CAPITAL LETTER M

0FF2E
FULLWIDTH LATIN CAPITAL LETTER N

0FF2F
FULLWIDTH LATIN CAPITAL LETTER O

0FF30
FULLWIDTH LATIN CAPITAL LETTER P

0FF31
FULLWIDTH LATIN CAPITAL LETTER Q

0FF32
FULLWIDTH LATIN CAPITAL LETTER R

0FF33
FULLWIDTH LATIN CAPITAL LETTER S

0FF34
FULLWIDTH LATIN CAPITAL LETTER T

0FF35
FULLWIDTH LATIN CAPITAL LETTER U

0FF36
FULLWIDTH LATIN CAPITAL LETTER V

0FF37
FULLWIDTH LATIN CAPITAL LETTER W

0FF38
FULLWIDTH LATIN CAPITAL LETTER X

0FF39
FULLWIDTH LATIN CAPITAL LETTER Y

0FF3A
FULLWIDTH LATIN CAPITAL LETTER Z

0FF3B
FULLWIDTH LEFT SQUARE BRACKET

0FF3C
FULLWIDTH REVERSE SOLIDUS

0FF3D
FULLWIDTH RIGHT SQUARE BRACKET

0FF3E
FULLWIDTH CIRCUMFLEX ACCENT

0FF3F
FULLWIDTH LOW LINE

0FF40
FULLWIDTH GRAVE ACCENT

0FF41
FULLWIDTH LATIN SMALL LETTER A

0FF42
FULLWIDTH LATIN SMALL LETTER B

0FF43
FULLWIDTH LATIN SMALL LETTER C

0FF44
FULLWIDTH LATIN SMALL LETTER D

0FF45
FULLWIDTH LATIN SMALL LETTER E

0FF46
FULLWIDTH LATIN SMALL LETTER F

0FF47
FULLWIDTH LATIN SMALL LETTER G

0FF48
FULLWIDTH LATIN SMALL LETTER H

0FF49
FULLWIDTH LATIN SMALL LETTER I

0FF4A
FULLWIDTH LATIN SMALL LETTER J

0FF4B
FULLWIDTH LATIN SMALL LETTER K

0FF4C
FULLWIDTH LATIN SMALL LETTER L

0FF4D
FULLWIDTH LATIN SMALL LETTER M

0FF4E
FULLWIDTH LATIN SMALL LETTER N

0FF4F
FULLWIDTH LATIN SMALL LETTER O

0FF50
FULLWIDTH LATIN SMALL LETTER P

0FF51
FULLWIDTH LATIN SMALL LETTER Q

0FF52
FULLWIDTH LATIN SMALL LETTER R

0FF53
FULLWIDTH LATIN SMALL LETTER S

0FF54
FULLWIDTH LATIN SMALL LETTER T

0FF55
FULLWIDTH LATIN SMALL LETTER U

0FF56
FULLWIDTH LATIN SMALL LETTER V

0FF57
FULLWIDTH LATIN SMALL LETTER W

0FF58
FULLWIDTH LATIN SMALL LETTER X

0FF59
FULLWIDTH LATIN SMALL LETTER Y

0FF5A
FULLWIDTH LATIN SMALL LETTER Z

0FF5B
FULLWIDTH LEFT CURLY BRACKET

0FF5C
FULLWIDTH VERTICAL LINE

0FF5D
FULLWIDTH RIGHT CURLY BRACKET

0FF5E
FULLWIDTH TILDE

0FF5F
FULLWIDTH LEFT WHITE PARENTHESIS

0FF60
FULLWIDTH RIGHT WHITE PARENTHESIS

0FF61
HALFWIDTH IDEOGRAPHIC FULL STOP

0FF62
HALFWIDTH LEFT CORNER BRACKET

0FF63
HALFWIDTH RIGHT CORNER BRACKET

0FF64
HALFWIDTH IDEOGRAPHIC COMMA

0FF65
HALFWIDTH KATAKANA MIDDLE DOT

0FF66
HALFWIDTH KATAKANA LETTER WO

0FF67
HALFWIDTH KATAKANA LETTER SMALL A

0FF68
HALFWIDTH KATAKANA LETTER SMALL I

0FF69
HALFWIDTH KATAKANA LETTER SMALL U

0FF6A
HALFWIDTH KATAKANA LETTER SMALL E

0FF6B
HALFWIDTH KATAKANA LETTER SMALL O

0FF6C
HALFWIDTH KATAKANA LETTER SMALL YA

0FF6D
HALFWIDTH KATAKANA LETTER SMALL YU

0FF6E
HALFWIDTH KATAKANA LETTER SMALL YO

0FF6F
HALFWIDTH KATAKANA LETTER SMALL TU

0FF70
HALFWIDTH KATAKANA-HIRAGANA PROLONGED SOUND MARK

0FF71
HALFWIDTH KATAKANA LETTER A

0FF72
HALFWIDTH KATAKANA LETTER I

0FF73
HALFWIDTH KATAKANA LETTER U

0FF74
HALFWIDTH KATAKANA LETTER E

0FF75
HALFWIDTH KATAKANA LETTER O

0FF76
HALFWIDTH KATAKANA LETTER KA

0FF77
HALFWIDTH KATAKANA LETTER KI

0FF78
HALFWIDTH KATAKANA LETTER KU

0FF79
HALFWIDTH KATAKANA LETTER KE

0FF7A
HALFWIDTH KATAKANA LETTER KO

0FF7B
HALFWIDTH KATAKANA LETTER SA

0FF7C
HALFWIDTH KATAKANA LETTER SI

0FF7D
HALFWIDTH KATAKANA LETTER SU

0FF7E
HALFWIDTH KATAKANA LETTER SE

0FF7F
HALFWIDTH KATAKANA LETTER SO

0FF80
HALFWIDTH KATAKANA LETTER TA

0FF81
HALFWIDTH KATAKANA LETTER TI

0FF82
HALFWIDTH KATAKANA LETTER TU

0FF83
HALFWIDTH KATAKANA LETTER TE

0FF84
HALFWIDTH KATAKANA LETTER TO

0FF85
HALFWIDTH KATAKANA LETTER NA

0FF86
HALFWIDTH KATAKANA LETTER NI

0FF87
HALFWIDTH KATAKANA LETTER NU

0FF88
HALFWIDTH KATAKANA LETTER NE

0FF89
HALFWIDTH KATAKANA LETTER NO

0FF8A
HALFWIDTH KATAKANA LETTER HA

0FF8B
HALFWIDTH KATAKANA LETTER HI

0FF8C
HALFWIDTH KATAKANA LETTER HU

0FF8D
HALFWIDTH KATAKANA LETTER HE

0FF8E
HALFWIDTH KATAKANA LETTER HO

0FF8F
HALFWIDTH KATAKANA LETTER MA

0FF90
HALFWIDTH KATAKANA LETTER MI

0FF91
HALFWIDTH KATAKANA LETTER MU

0FF92
HALFWIDTH KATAKANA LETTER ME

0FF93
HALFWIDTH KATAKANA LETTER MO

0FF94
HALFWIDTH KATAKANA LETTER YA

0FF95
HALFWIDTH KATAKANA LETTER YU

0FF96
HALFWIDTH KATAKANA LETTER YO

0FF97
HALFWIDTH KATAKANA LETTER RA

0FF98
HALFWIDTH KATAKANA LETTER RI

0FF99
HALFWIDTH KATAKANA LETTER RU

0FF9A
HALFWIDTH KATAKANA LETTER RE

0FF9B
HALFWIDTH KATAKANA LETTER RO

0FF9C
HALFWIDTH KATAKANA LETTER WA

0FF9D
HALFWIDTH KATAKANA LETTER N

0FF9E
HALFWIDTH KATAKANA VOICED SOUND MARK

0FF9F
HALFWIDTH KATAKANA SEMI-VOICED SOUND MARK

0FFA0
HALFWIDTH HANGUL FILLER

0FFA1
HALFWIDTH HANGUL LETTER KIYEOK

0FFA2
HALFWIDTH HANGUL LETTER SSANGKIYEOK

0FFA3
HALFWIDTH HANGUL LETTER KIYEOK-SIOS

0FFA4
HALFWIDTH HANGUL LETTER NIEUN

0FFA5
HALFWIDTH HANGUL LETTER NIEUN-CIEUC

0FFA6
HALFWIDTH HANGUL LETTER NIEUN-HIEUH

0FFA7
HALFWIDTH HANGUL LETTER TIKEUT

0FFA8
HALFWIDTH HANGUL LETTER SSANGTIKEUT

0FFA9
HALFWIDTH HANGUL LETTER RIEUL

0FFAA
HALFWIDTH HANGUL LETTER RIEUL-KIYEOK

0FFAB
HALFWIDTH HANGUL LETTER RIEUL-MIEUM

0FFAC
HALFWIDTH HANGUL LETTER RIEUL-PIEUP

0FFAD
HALFWIDTH HANGUL LETTER RIEUL-SIOS

0FFAE
HALFWIDTH HANGUL LETTER RIEUL-THIEUTH

0FFAF
HALFWIDTH HANGUL LETTER RIEUL-PHIEUPH

0FFB0
HALFWIDTH HANGUL LETTER RIEUL-HIEUH

0FFB1
HALFWIDTH HANGUL LETTER MIEUM

0FFB2
HALFWIDTH HANGUL LETTER PIEUP

0FFB3
HALFWIDTH HANGUL LETTER SSANGPIEUP

0FFB4
HALFWIDTH HANGUL LETTER PIEUP-SIOS

0FFB5
HALFWIDTH HANGUL LETTER SIOS

0FFB6
HALFWIDTH HANGUL LETTER SSANGSIOS

0FFB7
HALFWIDTH HANGUL LETTER IEUNG

0FFB8
HALFWIDTH HANGUL LETTER CIEUC

0FFB9
HALFWIDTH HANGUL LETTER SSANGCIEUC

0FFBA
HALFWIDTH HANGUL LETTER CHIEUCH

0FFBB
HALFWIDTH HANGUL LETTER KHIEUKH

0FFBC
HALFWIDTH HANGUL LETTER THIEUTH

0FFBD
HALFWIDTH HANGUL LETTER PHIEUPH

0FFBE
HALFWIDTH HANGUL LETTER HIEUH

0FFC2
HALFWIDTH HANGUL LETTER A

0FFC3
HALFWIDTH HANGUL LETTER AE

0FFC4
HALFWIDTH HANGUL LETTER YA

0FFC5
HALFWIDTH HANGUL LETTER YAE

0FFC6
HALFWIDTH HANGUL LETTER EO

0FFC7
HALFWIDTH HANGUL LETTER E

0FFCA
HALFWIDTH HANGUL LETTER YEO

0FFCB
HALFWIDTH HANGUL LETTER YE

0FFCC
HALFWIDTH HANGUL LETTER O

0FFCD
HALFWIDTH HANGUL LETTER WA

0FFCE
HALFWIDTH HANGUL LETTER WAE

0FFCF
HALFWIDTH HANGUL LETTER OE

0FFD2
HALFWIDTH HANGUL LETTER YO

0FFD3
HALFWIDTH HANGUL LETTER U

0FFD4
HALFWIDTH HANGUL LETTER WEO

0FFD5
HALFWIDTH HANGUL LETTER WE

0FFD6
HALFWIDTH HANGUL LETTER WI

0FFD7
HALFWIDTH HANGUL LETTER YU

0FFDA
HALFWIDTH HANGUL LETTER EU

0FFDB
HALFWIDTH HANGUL LETTER YI

0FFDC
HALFWIDTH HANGUL LETTER I

0FFE0
FULLWIDTH CENT SIGN

0FFE1
FULLWIDTH POUND SIGN

0FFE2
FULLWIDTH NOT SIGN

0FFE3
FULLWIDTH MACRON

0FFE4
FULLWIDTH BROKEN BAR

0FFE5
FULLWIDTH YEN SIGN

0FFE6
FULLWIDTH WON SIGN

0FFE8
HALFWIDTH FORMS LIGHT VERTICAL

0FFE9
HALFWIDTH LEFTWARDS ARROW

0FFEA
HALFWIDTH UPWARDS ARROW

0FFEB
HALFWIDTH RIGHTWARDS ARROW

0FFEC
HALFWIDTH DOWNWARDS ARROW

0FFED
HALFWIDTH BLACK SQUARE

0FFEE
HALFWIDTH WHITE CIRCLE

0FFF9
INTERLINEAR ANNOTATION ANCHOR

0FFFA
INTERLINEAR ANNOTATION SEPARATOR

0FFFB
INTERLINEAR ANNOTATION TERMINATOR

0FFFC
OBJECT REPLACEMENT CHARACTER

0FFFD
REPLACEMENT CHARACTER

10000
LINEAR B SYLLABLE B008 A

10001
LINEAR B SYLLABLE B038 E

10002
LINEAR B SYLLABLE B028 I

10003
LINEAR B SYLLABLE B061 O

10004
LINEAR B SYLLABLE B010 U

10005
LINEAR B SYLLABLE B001 DA

10006
LINEAR B SYLLABLE B045 DE

10007
LINEAR B SYLLABLE B007 DI

10008
LINEAR B SYLLABLE B014 DO

10009
LINEAR B SYLLABLE B051 DU

1000A
LINEAR B SYLLABLE B057 JA

1000B
LINEAR B SYLLABLE B046 JE

1000D
LINEAR B SYLLABLE B036 JO

1000E
LINEAR B SYLLABLE B065 JU

1000F
LINEAR B SYLLABLE B077 KA

10010
LINEAR B SYLLABLE B044 KE

10011
LINEAR B SYLLABLE B067 KI

10012
LINEAR B SYLLABLE B070 KO

10013
LINEAR B SYLLABLE B081 KU

10014
LINEAR B SYLLABLE B080 MA

10015
LINEAR B SYLLABLE B013 ME

10016
LINEAR B SYLLABLE B073 MI

10017
LINEAR B SYLLABLE B015 MO

10018
LINEAR B SYLLABLE B023 MU

10019
LINEAR B SYLLABLE B006 NA

1001A
LINEAR B SYLLABLE B024 NE

1001B
LINEAR B SYLLABLE B030 NI

1001C
LINEAR B SYLLABLE B052 NO

1001D
LINEAR B SYLLABLE B055 NU

1001E
LINEAR B SYLLABLE B003 PA

1001F
LINEAR B SYLLABLE B072 PE

10020
LINEAR B SYLLABLE B039 PI

10021
LINEAR B SYLLABLE B011 PO

10022
LINEAR B SYLLABLE B050 PU

10023
LINEAR B SYLLABLE B016 QA

10024
LINEAR B SYLLABLE B078 QE

10025
LINEAR B SYLLABLE B021 QI

10026
LINEAR B SYLLABLE B032 QO

10028
LINEAR B SYLLABLE B060 RA

10029
LINEAR B SYLLABLE B027 RE

1002A
LINEAR B SYLLABLE B053 RI

1002B
LINEAR B SYLLABLE B002 RO

1002C
LINEAR B SYLLABLE B026 RU

1002D
LINEAR B SYLLABLE B031 SA

1002E
LINEAR B SYLLABLE B009 SE

1002F
LINEAR B SYLLABLE B041 SI

10030
LINEAR B SYLLABLE B012 SO

10031
LINEAR B SYLLABLE B058 SU

10032
LINEAR B SYLLABLE B059 TA

10033
LINEAR B SYLLABLE B004 TE

10034
LINEAR B SYLLABLE B037 TI

10035
LINEAR B SYLLABLE B005 TO

10036
LINEAR B SYLLABLE B069 TU

10037
LINEAR B SYLLABLE B054 WA

10038
LINEAR B SYLLABLE B075 WE

10039
LINEAR B SYLLABLE B040 WI

1003A
LINEAR B SYLLABLE B042 WO

1003C
LINEAR B SYLLABLE B017 ZA

1003D
LINEAR B SYLLABLE B074 ZE

1003F
LINEAR B SYLLABLE B020 ZO

10040
LINEAR B SYLLABLE B025 A2

10041
LINEAR B SYLLABLE B043 A3

10042
LINEAR B SYLLABLE B085 AU

10043
LINEAR B SYLLABLE B071 DWE

10044
LINEAR B SYLLABLE B090 DWO

10045
LINEAR B SYLLABLE B048 NWA

10046
LINEAR B SYLLABLE B029 PU2

10047
LINEAR B SYLLABLE B062 PTE

10048
LINEAR B SYLLABLE B076 RA2

10049
LINEAR B SYLLABLE B033 RA3

1004A
LINEAR B SYLLABLE B068 RO2

1004B
LINEAR B SYLLABLE B066 TA2

1004C
LINEAR B SYLLABLE B087 TWE

1004D
LINEAR B SYLLABLE B091 TWO

10050
LINEAR B SYMBOL B018

10051
LINEAR B SYMBOL B019

10052
LINEAR B SYMBOL B022

10053
LINEAR B SYMBOL B034

10054
LINEAR B SYMBOL B047

10055
LINEAR B SYMBOL B049

10056
LINEAR B SYMBOL B056

10057
LINEAR B SYMBOL B063

10058
LINEAR B SYMBOL B064

10059
LINEAR B SYMBOL B079

1005A
LINEAR B SYMBOL B082

1005B
LINEAR B SYMBOL B083

1005C
LINEAR B SYMBOL B086

1005D
LINEAR B SYMBOL B089

10080
LINEAR B IDEOGRAM B100 MAN

10081
LINEAR B IDEOGRAM B102 WOMAN

10082
LINEAR B IDEOGRAM B104 DEER

10083
LINEAR B IDEOGRAM B105 EQUID

10084
LINEAR B IDEOGRAM B105F MARE

10085
LINEAR B IDEOGRAM B105M STALLION

10086
LINEAR B IDEOGRAM B106F EWE

10087
LINEAR B IDEOGRAM B106M RAM

10088
LINEAR B IDEOGRAM B107F SHE-GOAT

10089
LINEAR B IDEOGRAM B107M HE-GOAT

1008A
LINEAR B IDEOGRAM B108F SOW

1008B
LINEAR B IDEOGRAM B108M BOAR

1008C
LINEAR B IDEOGRAM B109F COW

1008D
LINEAR B IDEOGRAM B109M BULL

1008E
LINEAR B IDEOGRAM B120 WHEAT

1008F
LINEAR B IDEOGRAM B121 BARLEY

10090
LINEAR B IDEOGRAM B122 OLIVE

10091
LINEAR B IDEOGRAM B123 SPICE

10092
LINEAR B IDEOGRAM B125 CYPERUS

10093
LINEAR B MONOGRAM B127 KAPO

10094
LINEAR B MONOGRAM B128 KANAKO

10095
LINEAR B IDEOGRAM B130 OIL

10096
LINEAR B IDEOGRAM B131 WINE

10097
LINEAR B IDEOGRAM B132

10098
LINEAR B MONOGRAM B133 AREPA

10099
LINEAR B MONOGRAM B135 MERI

1009A
LINEAR B IDEOGRAM B140 BRONZE

1009B
LINEAR B IDEOGRAM B141 GOLD

1009C
LINEAR B IDEOGRAM B142

1009D
LINEAR B IDEOGRAM B145 WOOL

1009E
LINEAR B IDEOGRAM B146

1009F
LINEAR B IDEOGRAM B150

100A0
LINEAR B IDEOGRAM B151 HORN

100A1
LINEAR B IDEOGRAM B152

100A2
LINEAR B IDEOGRAM B153

100A3
LINEAR B IDEOGRAM B154

100A4
LINEAR B MONOGRAM B156 TURO2

100A5
LINEAR B IDEOGRAM B157

100A6
LINEAR B IDEOGRAM B158

100A7
LINEAR B IDEOGRAM B159 CLOTH

100A8
LINEAR B IDEOGRAM B160

100A9
LINEAR B IDEOGRAM B161

100AA
LINEAR B IDEOGRAM B162 GARMENT

100AB
LINEAR B IDEOGRAM B163 ARMOUR

100AC
LINEAR B IDEOGRAM B164

100AD
LINEAR B IDEOGRAM B165

100AE
LINEAR B IDEOGRAM B166

100AF
LINEAR B IDEOGRAM B167

100B0
LINEAR B IDEOGRAM B168

100B1
LINEAR B IDEOGRAM B169

100B2
LINEAR B IDEOGRAM B170

100B3
LINEAR B IDEOGRAM B171

100B4
LINEAR B IDEOGRAM B172

100B5
LINEAR B IDEOGRAM B173 MONTH

100B6
LINEAR B IDEOGRAM B174

100B7
LINEAR B IDEOGRAM B176 TREE

100B8
LINEAR B IDEOGRAM B177

100B9
LINEAR B IDEOGRAM B178

100BA
LINEAR B IDEOGRAM B179

100BB
LINEAR B IDEOGRAM B180

100BC
LINEAR B IDEOGRAM B181

100BD
LINEAR B IDEOGRAM B182

100BE
LINEAR B IDEOGRAM B183

100BF
LINEAR B IDEOGRAM B184

100C0
LINEAR B IDEOGRAM B185

100C1
LINEAR B IDEOGRAM B189

100C2
LINEAR B IDEOGRAM B190

100C3
LINEAR B IDEOGRAM B191 HELMET

100C4
LINEAR B IDEOGRAM B220 FOOTSTOOL

100C5
LINEAR B IDEOGRAM B225 BATHTUB

100C6
LINEAR B IDEOGRAM B230 SPEAR

100C7
LINEAR B IDEOGRAM B231 ARROW

100C8
LINEAR B IDEOGRAM B232

100C9
LINEAR B IDEOGRAM B233 SWORD

100CA
LINEAR B IDEOGRAM B234

100CB
LINEAR B IDEOGRAM B236

100CC
LINEAR B IDEOGRAM B240 WHEELED CHARIOT

100CD
LINEAR B IDEOGRAM B241 CHARIOT

100CE
LINEAR B IDEOGRAM B242 CHARIOT FRAME

100CF
LINEAR B IDEOGRAM B243 WHEEL

100D0
LINEAR B IDEOGRAM B245

100D1
LINEAR B IDEOGRAM B246

100D2
LINEAR B MONOGRAM B247 DIPTE

100D3
LINEAR B IDEOGRAM B248

100D4
LINEAR B IDEOGRAM B249

100D5
LINEAR B IDEOGRAM B251

100D6
LINEAR B IDEOGRAM B252

100D7
LINEAR B IDEOGRAM B253

100D8
LINEAR B IDEOGRAM B254 DART

100D9
LINEAR B IDEOGRAM B255

100DA
LINEAR B IDEOGRAM B256

100DB
LINEAR B IDEOGRAM B257

100DC
LINEAR B IDEOGRAM B258

100DD
LINEAR B IDEOGRAM B259

100DE
LINEAR B IDEOGRAM VESSEL B155

100DF
LINEAR B IDEOGRAM VESSEL B200

100E0
LINEAR B IDEOGRAM VESSEL B201

100E1
LINEAR B IDEOGRAM VESSEL B202

100E2
LINEAR B IDEOGRAM VESSEL B203

100E3
LINEAR B IDEOGRAM VESSEL B204

100E4
LINEAR B IDEOGRAM VESSEL B205

100E5
LINEAR B IDEOGRAM VESSEL B206

100E6
LINEAR B IDEOGRAM VESSEL B207

100E7
LINEAR B IDEOGRAM VESSEL B208

100E8
LINEAR B IDEOGRAM VESSEL B209

100E9
LINEAR B IDEOGRAM VESSEL B210

100EA
LINEAR B IDEOGRAM VESSEL B211

100EB
LINEAR B IDEOGRAM VESSEL B212

100EC
LINEAR B IDEOGRAM VESSEL B213

100ED
LINEAR B IDEOGRAM VESSEL B214

100EE
LINEAR B IDEOGRAM VESSEL B215

100EF
LINEAR B IDEOGRAM VESSEL B216

100F0
LINEAR B IDEOGRAM VESSEL B217

100F1
LINEAR B IDEOGRAM VESSEL B218

100F2
LINEAR B IDEOGRAM VESSEL B219

100F3
LINEAR B IDEOGRAM VESSEL B221

100F4
LINEAR B IDEOGRAM VESSEL B222

100F5
LINEAR B IDEOGRAM VESSEL B226

100F6
LINEAR B IDEOGRAM VESSEL B227

100F7
LINEAR B IDEOGRAM VESSEL B228

100F8
LINEAR B IDEOGRAM VESSEL B229

100F9
LINEAR B IDEOGRAM VESSEL B250

100FA
LINEAR B IDEOGRAM VESSEL B305

10100
AEGEAN WORD SEPARATOR LINE

10101
AEGEAN WORD SEPARATOR DOT

10102
AEGEAN CHECK MARK

10107
AEGEAN NUMBER ONE

10108
AEGEAN NUMBER TWO

10109
AEGEAN NUMBER THREE

1010A
AEGEAN NUMBER FOUR

1010B
AEGEAN NUMBER FIVE

1010C
AEGEAN NUMBER SIX

1010D
AEGEAN NUMBER SEVEN

1010E
AEGEAN NUMBER EIGHT

1010F
AEGEAN NUMBER NINE

10110
AEGEAN NUMBER TEN

10111
AEGEAN NUMBER TWENTY

10112
AEGEAN NUMBER THIRTY

10113
AEGEAN NUMBER FORTY

10114
AEGEAN NUMBER FIFTY

10115
AEGEAN NUMBER SIXTY

10116
AEGEAN NUMBER SEVENTY

10117
AEGEAN NUMBER EIGHTY

10118
AEGEAN NUMBER NINETY

10119
AEGEAN NUMBER ONE HUNDRED

1011A
AEGEAN NUMBER TWO HUNDRED

1011B
AEGEAN NUMBER THREE HUNDRED

1011C
AEGEAN NUMBER FOUR HUNDRED

1011D
AEGEAN NUMBER FIVE HUNDRED

1011E
AEGEAN NUMBER SIX HUNDRED

1011F
AEGEAN NUMBER SEVEN HUNDRED

10120
AEGEAN NUMBER EIGHT HUNDRED

10121
AEGEAN NUMBER NINE HUNDRED

10122
AEGEAN NUMBER ONE THOUSAND

10123
AEGEAN NUMBER TWO THOUSAND

10124
AEGEAN NUMBER THREE THOUSAND

10125
AEGEAN NUMBER FOUR THOUSAND

10126
AEGEAN NUMBER FIVE THOUSAND

10127
AEGEAN NUMBER SIX THOUSAND

10128
AEGEAN NUMBER SEVEN THOUSAND

10129
AEGEAN NUMBER EIGHT THOUSAND

1012A
AEGEAN NUMBER NINE THOUSAND

1012B
AEGEAN NUMBER TEN THOUSAND

1012C
AEGEAN NUMBER TWENTY THOUSAND

1012D
AEGEAN NUMBER THIRTY THOUSAND

1012E
AEGEAN NUMBER FORTY THOUSAND

1012F
AEGEAN NUMBER FIFTY THOUSAND

10130
AEGEAN NUMBER SIXTY THOUSAND

10131
AEGEAN NUMBER SEVENTY THOUSAND

10132
AEGEAN NUMBER EIGHTY THOUSAND

10133
AEGEAN NUMBER NINETY THOUSAND

10137
AEGEAN WEIGHT BASE UNIT

10138
AEGEAN WEIGHT FIRST SUBUNIT

10139
AEGEAN WEIGHT SECOND SUBUNIT

1013A
AEGEAN WEIGHT THIRD SUBUNIT

1013B
AEGEAN WEIGHT FOURTH SUBUNIT

1013C
AEGEAN DRY MEASURE FIRST SUBUNIT

1013D
AEGEAN LIQUID MEASURE FIRST SUBUNIT

1013E
AEGEAN MEASURE SECOND SUBUNIT

1013F
AEGEAN MEASURE THIRD SUBUNIT

10140
GREEK ACROPHONIC ATTIC ONE QUARTER

10141
GREEK ACROPHONIC ATTIC ONE HALF

10142
GREEK ACROPHONIC ATTIC ONE DRACHMA

10143
GREEK ACROPHONIC ATTIC FIVE

10144
GREEK ACROPHONIC ATTIC FIFTY

10145
GREEK ACROPHONIC ATTIC FIVE HUNDRED

10146
GREEK ACROPHONIC ATTIC FIVE THOUSAND

10147
GREEK ACROPHONIC ATTIC FIFTY THOUSAND

10148
GREEK ACROPHONIC ATTIC FIVE TALENTS

10149
GREEK ACROPHONIC ATTIC TEN TALENTS

1014A
GREEK ACROPHONIC ATTIC FIFTY TALENTS

1014B
GREEK ACROPHONIC ATTIC ONE HUNDRED TALENTS

1014C
GREEK ACROPHONIC ATTIC FIVE HUNDRED TALENTS

1014D
GREEK ACROPHONIC ATTIC ONE THOUSAND TALENTS

1014E
GREEK ACROPHONIC ATTIC FIVE THOUSAND TALENTS

1014F
GREEK ACROPHONIC ATTIC FIVE STATERS

10150
GREEK ACROPHONIC ATTIC TEN STATERS

10151
GREEK ACROPHONIC ATTIC FIFTY STATERS

10152
GREEK ACROPHONIC ATTIC ONE HUNDRED STATERS

10153
GREEK ACROPHONIC ATTIC FIVE HUNDRED STATERS

10154
GREEK ACROPHONIC ATTIC ONE THOUSAND STATERS

10155
GREEK ACROPHONIC ATTIC TEN THOUSAND STATERS

10156
GREEK ACROPHONIC ATTIC FIFTY THOUSAND STATERS

10157
GREEK ACROPHONIC ATTIC TEN MNAS

10158
GREEK ACROPHONIC HERAEUM ONE PLETHRON

10159
GREEK ACROPHONIC THESPIAN ONE

1015A
GREEK ACROPHONIC HERMIONIAN ONE

1015B
GREEK ACROPHONIC EPIDAUREAN TWO

1015C
GREEK ACROPHONIC THESPIAN TWO

1015D
GREEK ACROPHONIC CYRENAIC TWO DRACHMAS

1015E
GREEK ACROPHONIC EPIDAUREAN TWO DRACHMAS

1015F
GREEK ACROPHONIC TROEZENIAN FIVE

10160
GREEK ACROPHONIC TROEZENIAN TEN

10161
GREEK ACROPHONIC TROEZENIAN TEN ALTERNATE FORM

10162
GREEK ACROPHONIC HERMIONIAN TEN

10163
GREEK ACROPHONIC MESSENIAN TEN

10164
GREEK ACROPHONIC THESPIAN TEN

10165
GREEK ACROPHONIC THESPIAN THIRTY

10166
GREEK ACROPHONIC TROEZENIAN FIFTY

10167
GREEK ACROPHONIC TROEZENIAN FIFTY ALTERNATE FORM

10168
GREEK ACROPHONIC HERMIONIAN FIFTY

10169
GREEK ACROPHONIC THESPIAN FIFTY

1016A
GREEK ACROPHONIC THESPIAN ONE HUNDRED

1016B
GREEK ACROPHONIC THESPIAN THREE HUNDRED

1016C
GREEK ACROPHONIC EPIDAUREAN FIVE HUNDRED

1016D
GREEK ACROPHONIC TROEZENIAN FIVE HUNDRED

1016E
GREEK ACROPHONIC THESPIAN FIVE HUNDRED

1016F
GREEK ACROPHONIC CARYSTIAN FIVE HUNDRED

10170
GREEK ACROPHONIC NAXIAN FIVE HUNDRED

10171
GREEK ACROPHONIC THESPIAN ONE THOUSAND

10172
GREEK ACROPHONIC THESPIAN FIVE THOUSAND

10173
GREEK ACROPHONIC DELPHIC FIVE MNAS

10174
GREEK ACROPHONIC STRATIAN FIFTY MNAS

10175
GREEK ONE HALF SIGN

10176
GREEK ONE HALF SIGN ALTERNATE FORM

10177
GREEK TWO THIRDS SIGN

10178
GREEK THREE QUARTERS SIGN

10179
GREEK YEAR SIGN

1017A
GREEK TALENT SIGN

1017B
GREEK DRACHMA SIGN

1017C
GREEK OBOL SIGN

1017D
GREEK TWO OBOLS SIGN

1017E
GREEK THREE OBOLS SIGN

1017F
GREEK FOUR OBOLS SIGN

10180
GREEK FIVE OBOLS SIGN

10181
GREEK METRETES SIGN

10182
GREEK KYATHOS BASE SIGN

10183
GREEK LITRA SIGN

10184
GREEK OUNKIA SIGN

10185
GREEK XESTES SIGN

10186
GREEK ARTABE SIGN

10187
GREEK AROURA SIGN

10188
GREEK GRAMMA SIGN

10189
GREEK TRYBLION BASE SIGN

1018A
GREEK ZERO SIGN

1018B
GREEK ONE QUARTER SIGN

1018C
GREEK SINUSOID SIGN

1018D
GREEK INDICTION SIGN

1018E
NOMISMA SIGN

10190
ROMAN SEXTANS SIGN

10191
ROMAN UNCIA SIGN

10192
ROMAN SEMUNCIA SIGN

10193
ROMAN SEXTULA SIGN

10194
ROMAN DIMIDIA SEXTULA SIGN

10195
ROMAN SILIQUA SIGN

10196
ROMAN DENARIUS SIGN

10197
ROMAN QUINARIUS SIGN

10198
ROMAN SESTERTIUS SIGN

10199
ROMAN DUPONDIUS SIGN

1019A
ROMAN AS SIGN

1019B
ROMAN CENTURIAL SIGN

1019C
ASCIA SYMBOL

101A0
GREEK SYMBOL TAU RHO

101D0
PHAISTOS DISC SIGN PEDESTRIAN

101D1
PHAISTOS DISC SIGN PLUMED HEAD

101D2
PHAISTOS DISC SIGN TATTOOED HEAD

101D3
PHAISTOS DISC SIGN CAPTIVE

101D4
PHAISTOS DISC SIGN CHILD

101D5
PHAISTOS DISC SIGN WOMAN

101D6
PHAISTOS DISC SIGN HELMET

101D7
PHAISTOS DISC SIGN GAUNTLET

101D8
PHAISTOS DISC SIGN TIARA

101D9
PHAISTOS DISC SIGN ARROW

101DA
PHAISTOS DISC SIGN BOW

101DB
PHAISTOS DISC SIGN SHIELD

101DC
PHAISTOS DISC SIGN CLUB

101DD
PHAISTOS DISC SIGN MANACLES

101DE
PHAISTOS DISC SIGN MATTOCK

101DF
PHAISTOS DISC SIGN SAW

101E0
PHAISTOS DISC SIGN LID

101E1
PHAISTOS DISC SIGN BOOMERANG

101E2
PHAISTOS DISC SIGN CARPENTRY PLANE

101E3
PHAISTOS DISC SIGN DOLIUM

101E4
PHAISTOS DISC SIGN COMB

101E5
PHAISTOS DISC SIGN SLING

101E6
PHAISTOS DISC SIGN COLUMN

101E7
PHAISTOS DISC SIGN BEEHIVE

101E8
PHAISTOS DISC SIGN SHIP

101E9
PHAISTOS DISC SIGN HORN

101EA
PHAISTOS DISC SIGN HIDE

101EB
PHAISTOS DISC SIGN BULLS LEG

101EC
PHAISTOS DISC SIGN CAT

101ED
PHAISTOS DISC SIGN RAM

101EE
PHAISTOS DISC SIGN EAGLE

101EF
PHAISTOS DISC SIGN DOVE

101F0
PHAISTOS DISC SIGN TUNNY

101F1
PHAISTOS DISC SIGN BEE

101F2
PHAISTOS DISC SIGN PLANE TREE

101F3
PHAISTOS DISC SIGN VINE

101F4
PHAISTOS DISC SIGN PAPYRUS

101F5
PHAISTOS DISC SIGN ROSETTE

101F6
PHAISTOS DISC SIGN LILY

101F7
PHAISTOS DISC SIGN OX BACK

101F8
PHAISTOS DISC SIGN FLUTE

101F9
PHAISTOS DISC SIGN GRATER

101FA
PHAISTOS DISC SIGN STRAINER

101FB
PHAISTOS DISC SIGN SMALL AXE

101FC
PHAISTOS DISC SIGN WAVY BAND

101FD
PHAISTOS DISC SIGN COMBINING OBLIQUE STROKE

10280
LYCIAN LETTER A

10281
LYCIAN LETTER E

10282
LYCIAN LETTER B

10283
LYCIAN LETTER BH

10284
LYCIAN LETTER G

10285
LYCIAN LETTER D

10286
LYCIAN LETTER I

10287
LYCIAN LETTER W

10288
LYCIAN LETTER Z

10289
LYCIAN LETTER TH

1028A
LYCIAN LETTER J

1028B
LYCIAN LETTER K

1028C
LYCIAN LETTER Q

1028D
LYCIAN LETTER L

1028E
LYCIAN LETTER M

1028F
LYCIAN LETTER N

10290
LYCIAN LETTER MM

10291
LYCIAN LETTER NN

10292
LYCIAN LETTER U

10293
LYCIAN LETTER P

10294
LYCIAN LETTER KK

10295
LYCIAN LETTER R

10296
LYCIAN LETTER S

10297
LYCIAN LETTER T

10298
LYCIAN LETTER TT

10299
LYCIAN LETTER AN

1029A
LYCIAN LETTER EN

1029B
LYCIAN LETTER H

1029C
LYCIAN LETTER X

102A0
CARIAN LETTER A

102A1
CARIAN LETTER P2

102A2
CARIAN LETTER D

102A3
CARIAN LETTER L

102A4
CARIAN LETTER UUU

102A5
CARIAN LETTER R

102A6
CARIAN LETTER LD

102A7
CARIAN LETTER A2

102A8
CARIAN LETTER Q

102A9
CARIAN LETTER B

102AA
CARIAN LETTER M

102AB
CARIAN LETTER O

102AC
CARIAN LETTER D2

102AD
CARIAN LETTER T

102AE
CARIAN LETTER SH

102AF
CARIAN LETTER SH2

102B0
CARIAN LETTER S

102B1
CARIAN LETTER C-18

102B2
CARIAN LETTER U

102B3
CARIAN LETTER NN

102B4
CARIAN LETTER X

102B5
CARIAN LETTER N

102B6
CARIAN LETTER TT2

102B7
CARIAN LETTER P

102B8
CARIAN LETTER SS

102B9
CARIAN LETTER I

102BA
CARIAN LETTER E

102BB
CARIAN LETTER UUUU

102BC
CARIAN LETTER K

102BD
CARIAN LETTER K2

102BE
CARIAN LETTER ND

102BF
CARIAN LETTER UU

102C0
CARIAN LETTER G

102C1
CARIAN LETTER G2

102C2
CARIAN LETTER ST

102C3
CARIAN LETTER ST2

102C4
CARIAN LETTER NG

102C5
CARIAN LETTER II

102C6
CARIAN LETTER C-39

102C7
CARIAN LETTER TT

102C8
CARIAN LETTER UUU2

102C9
CARIAN LETTER RR

102CA
CARIAN LETTER MB

102CB
CARIAN LETTER MB2

102CC
CARIAN LETTER MB3

102CD
CARIAN LETTER MB4

102CE
CARIAN LETTER LD2

102CF
CARIAN LETTER E2

102D0
CARIAN LETTER UUU3

102E0
COPTIC EPACT THOUSANDS MARK

102E1
COPTIC EPACT DIGIT ONE

102E2
COPTIC EPACT DIGIT TWO

102E3
COPTIC EPACT DIGIT THREE

102E4
COPTIC EPACT DIGIT FOUR

102E5
COPTIC EPACT DIGIT FIVE

102E6
COPTIC EPACT DIGIT SIX

102E7
COPTIC EPACT DIGIT SEVEN

102E8
COPTIC EPACT DIGIT EIGHT

102E9
COPTIC EPACT DIGIT NINE

102EA
COPTIC EPACT NUMBER TEN

102EB
COPTIC EPACT NUMBER TWENTY

102EC
COPTIC EPACT NUMBER THIRTY

102ED
COPTIC EPACT NUMBER FORTY

102EE
COPTIC EPACT NUMBER FIFTY

102EF
COPTIC EPACT NUMBER SIXTY

102F0
COPTIC EPACT NUMBER SEVENTY

102F1
COPTIC EPACT NUMBER EIGHTY

102F2
COPTIC EPACT NUMBER NINETY

102F3
COPTIC EPACT NUMBER ONE HUNDRED

102F4
COPTIC EPACT NUMBER TWO HUNDRED

102F5
COPTIC EPACT NUMBER THREE HUNDRED

102F6
COPTIC EPACT NUMBER FOUR HUNDRED

102F7
COPTIC EPACT NUMBER FIVE HUNDRED

102F8
COPTIC EPACT NUMBER SIX HUNDRED

102F9
COPTIC EPACT NUMBER SEVEN HUNDRED

102FA
COPTIC EPACT NUMBER EIGHT HUNDRED

102FB
COPTIC EPACT NUMBER NINE HUNDRED

10300
OLD ITALIC LETTER A

10301
OLD ITALIC LETTER BE

10302
OLD ITALIC LETTER KE

10303
OLD ITALIC LETTER DE

10304
OLD ITALIC LETTER E

10305
OLD ITALIC LETTER VE

10306
OLD ITALIC LETTER ZE

10307
OLD ITALIC LETTER HE

10308
OLD ITALIC LETTER THE

10309
OLD ITALIC LETTER I

1030A
OLD ITALIC LETTER KA

1030B
OLD ITALIC LETTER EL

1030C
OLD ITALIC LETTER EM

1030D
OLD ITALIC LETTER EN

1030E
OLD ITALIC LETTER ESH

1030F
OLD ITALIC LETTER O

10310
OLD ITALIC LETTER PE

10311
OLD ITALIC LETTER SHE

10312
OLD ITALIC LETTER KU

10313
OLD ITALIC LETTER ER

10314
OLD ITALIC LETTER ES

10315
OLD ITALIC LETTER TE

10316
OLD ITALIC LETTER U

10317
OLD ITALIC LETTER EKS

10318
OLD ITALIC LETTER PHE

10319
OLD ITALIC LETTER KHE

1031A
OLD ITALIC LETTER EF

1031B
OLD ITALIC LETTER ERS

1031C
OLD ITALIC LETTER CHE

1031D
OLD ITALIC LETTER II

1031E
OLD ITALIC LETTER UU

1031F
OLD ITALIC LETTER ESS

10320
OLD ITALIC NUMERAL ONE

10321
OLD ITALIC NUMERAL FIVE

10322
OLD ITALIC NUMERAL TEN

10323
OLD ITALIC NUMERAL FIFTY

1032D
OLD ITALIC LETTER YE

1032E
OLD ITALIC LETTER NORTHERN TSE

1032F
OLD ITALIC LETTER SOUTHERN TSE

10330
GOTHIC LETTER AHSA

10331
GOTHIC LETTER BAIRKAN

10332
GOTHIC LETTER GIBA

10333
GOTHIC LETTER DAGS

10334
GOTHIC LETTER AIHVUS

10335
GOTHIC LETTER QAIRTHRA

10336
GOTHIC LETTER IUJA

10337
GOTHIC LETTER HAGL

10338
GOTHIC LETTER THIUTH

10339
GOTHIC LETTER EIS

1033A
GOTHIC LETTER KUSMA

1033B
GOTHIC LETTER LAGUS

1033C
GOTHIC LETTER MANNA

1033D
GOTHIC LETTER NAUTHS

1033E
GOTHIC LETTER JER

1033F
GOTHIC LETTER URUS

10340
GOTHIC LETTER PAIRTHRA

10341
GOTHIC LETTER NINETY

10342
GOTHIC LETTER RAIDA

10343
GOTHIC LETTER SAUIL

10344
GOTHIC LETTER TEIWS

10345
GOTHIC LETTER WINJA

10346
GOTHIC LETTER FAIHU

10347
GOTHIC LETTER IGGWS

10348
GOTHIC LETTER HWAIR

10349
GOTHIC LETTER OTHAL

1034A
GOTHIC LETTER NINE HUNDRED

10350
OLD PERMIC LETTER AN

10351
OLD PERMIC LETTER BUR

10352
OLD PERMIC LETTER GAI

10353
OLD PERMIC LETTER DOI

10354
OLD PERMIC LETTER E

10355
OLD PERMIC LETTER ZHOI

10356
OLD PERMIC LETTER DZHOI

10357
OLD PERMIC LETTER ZATA

10358
OLD PERMIC LETTER DZITA

10359
OLD PERMIC LETTER I

1035A
OLD PERMIC LETTER KOKE

1035B
OLD PERMIC LETTER LEI

1035C
OLD PERMIC LETTER MENOE

1035D
OLD PERMIC LETTER NENOE

1035E
OLD PERMIC LETTER VOOI

1035F
OLD PERMIC LETTER PEEI

10360
OLD PERMIC LETTER REI

10361
OLD PERMIC LETTER SII

10362
OLD PERMIC LETTER TAI

10363
OLD PERMIC LETTER U

10364
OLD PERMIC LETTER CHERY

10365
OLD PERMIC LETTER SHOOI

10366
OLD PERMIC LETTER SHCHOOI

10367
OLD PERMIC LETTER YRY

10368
OLD PERMIC LETTER YERU

10369
OLD PERMIC LETTER O

1036A
OLD PERMIC LETTER OO

1036B
OLD PERMIC LETTER EF

1036C
OLD PERMIC LETTER HA

1036D
OLD PERMIC LETTER TSIU

1036E
OLD PERMIC LETTER VER

1036F
OLD PERMIC LETTER YER

10370
OLD PERMIC LETTER YERI

10371
OLD PERMIC LETTER YAT

10372
OLD PERMIC LETTER IE

10373
OLD PERMIC LETTER YU

10374
OLD PERMIC LETTER YA

10375
OLD PERMIC LETTER IA

10376
COMBINING OLD PERMIC LETTER AN

10377
COMBINING OLD PERMIC LETTER DOI

10378
COMBINING OLD PERMIC LETTER ZATA

10379
COMBINING OLD PERMIC LETTER NENOE

1037A
COMBINING OLD PERMIC LETTER SII

10380
UGARITIC LETTER ALPA

10381
UGARITIC LETTER BETA

10382
UGARITIC LETTER GAMLA

10383
UGARITIC LETTER KHA

10384
UGARITIC LETTER DELTA

10385
UGARITIC LETTER HO

10386
UGARITIC LETTER WO

10387
UGARITIC LETTER ZETA

10388
UGARITIC LETTER HOTA

10389
UGARITIC LETTER TET

1038A
UGARITIC LETTER YOD

1038B
UGARITIC LETTER KAF

1038C
UGARITIC LETTER SHIN

1038D
UGARITIC LETTER LAMDA

1038E
UGARITIC LETTER MEM

1038F
UGARITIC LETTER DHAL

10390
UGARITIC LETTER NUN

10391
UGARITIC LETTER ZU

10392
UGARITIC LETTER SAMKA

10393
UGARITIC LETTER AIN

10394
UGARITIC LETTER PU

10395
UGARITIC LETTER SADE

10396
UGARITIC LETTER QOPA

10397
UGARITIC LETTER RASHA

10398
UGARITIC LETTER THANNA

10399
UGARITIC LETTER GHAIN

1039A
UGARITIC LETTER TO

1039B
UGARITIC LETTER I

1039C
UGARITIC LETTER U

1039D
UGARITIC LETTER SSU

1039F
UGARITIC WORD DIVIDER

103A0
OLD PERSIAN SIGN A

103A1
OLD PERSIAN SIGN I

103A2
OLD PERSIAN SIGN U

103A3
OLD PERSIAN SIGN KA

103A4
OLD PERSIAN SIGN KU

103A5
OLD PERSIAN SIGN GA

103A6
OLD PERSIAN SIGN GU

103A7
OLD PERSIAN SIGN XA

103A8
OLD PERSIAN SIGN CA

103A9
OLD PERSIAN SIGN JA

103AA
OLD PERSIAN SIGN JI

103AB
OLD PERSIAN SIGN TA

103AC
OLD PERSIAN SIGN TU

103AD
OLD PERSIAN SIGN DA

103AE
OLD PERSIAN SIGN DI

103AF
OLD PERSIAN SIGN DU

103B0
OLD PERSIAN SIGN THA

103B1
OLD PERSIAN SIGN PA

103B2
OLD PERSIAN SIGN BA

103B3
OLD PERSIAN SIGN FA

103B4
OLD PERSIAN SIGN NA

103B5
OLD PERSIAN SIGN NU

103B6
OLD PERSIAN SIGN MA

103B7
OLD PERSIAN SIGN MI

103B8
OLD PERSIAN SIGN MU

103B9
OLD PERSIAN SIGN YA

103BA
OLD PERSIAN SIGN VA

103BB
OLD PERSIAN SIGN VI

103BC
OLD PERSIAN SIGN RA

103BD
OLD PERSIAN SIGN RU

103BE
OLD PERSIAN SIGN LA

103BF
OLD PERSIAN SIGN SA

103C0
OLD PERSIAN SIGN ZA

103C1
OLD PERSIAN SIGN SHA

103C2
OLD PERSIAN SIGN SSA

103C3
OLD PERSIAN SIGN HA

103C8
OLD PERSIAN SIGN AURAMAZDAA

103C9
OLD PERSIAN SIGN AURAMAZDAA-2

103CA
OLD PERSIAN SIGN AURAMAZDAAHA

103CB
OLD PERSIAN SIGN XSHAAYATHIYA

103CC
OLD PERSIAN SIGN DAHYAAUSH

103CD
OLD PERSIAN SIGN DAHYAAUSH-2

103CE
OLD PERSIAN SIGN BAGA

103CF
OLD PERSIAN SIGN BUUMISH

103D0
OLD PERSIAN WORD DIVIDER

103D1
OLD PERSIAN NUMBER ONE

103D2
OLD PERSIAN NUMBER TWO

103D3
OLD PERSIAN NUMBER TEN

103D4
OLD PERSIAN NUMBER TWENTY

103D5
OLD PERSIAN NUMBER HUNDRED

10400
DESERET CAPITAL LETTER LONG I

10401
DESERET CAPITAL LETTER LONG E

10402
DESERET CAPITAL LETTER LONG A

10403
DESERET CAPITAL LETTER LONG AH

10404
DESERET CAPITAL LETTER LONG O

10405
DESERET CAPITAL LETTER LONG OO

10406
DESERET CAPITAL LETTER SHORT I

10407
DESERET CAPITAL LETTER SHORT E

10408
DESERET CAPITAL LETTER SHORT A

10409
DESERET CAPITAL LETTER SHORT AH

1040A
DESERET CAPITAL LETTER SHORT O

1040B
DESERET CAPITAL LETTER SHORT OO

1040C
DESERET CAPITAL LETTER AY

1040D
DESERET CAPITAL LETTER OW

1040E
DESERET CAPITAL LETTER WU

1040F
DESERET CAPITAL LETTER YEE

10410
DESERET CAPITAL LETTER H

10411
DESERET CAPITAL LETTER PEE

10412
DESERET CAPITAL LETTER BEE

10413
DESERET CAPITAL LETTER TEE

10414
DESERET CAPITAL LETTER DEE

10415
DESERET CAPITAL LETTER CHEE

10416
DESERET CAPITAL LETTER JEE

10417
DESERET CAPITAL LETTER KAY

10418
DESERET CAPITAL LETTER GAY

10419
DESERET CAPITAL LETTER EF

1041A
DESERET CAPITAL LETTER VEE

1041B
DESERET CAPITAL LETTER ETH

1041C
DESERET CAPITAL LETTER THEE

1041D
DESERET CAPITAL LETTER ES

1041E
DESERET CAPITAL LETTER ZEE

1041F
DESERET CAPITAL LETTER ESH

10420
DESERET CAPITAL LETTER ZHEE

10421
DESERET CAPITAL LETTER ER

10422
DESERET CAPITAL LETTER EL

10423
DESERET CAPITAL LETTER EM

10424
DESERET CAPITAL LETTER EN

10425
DESERET CAPITAL LETTER ENG

10426
DESERET CAPITAL LETTER OI

10427
DESERET CAPITAL LETTER EW

10428
DESERET SMALL LETTER LONG I

10429
DESERET SMALL LETTER LONG E

1042A
DESERET SMALL LETTER LONG A

1042B
DESERET SMALL LETTER LONG AH

1042C
DESERET SMALL LETTER LONG O

1042D
DESERET SMALL LETTER LONG OO

1042E
DESERET SMALL LETTER SHORT I

1042F
DESERET SMALL LETTER SHORT E

10430
DESERET SMALL LETTER SHORT A

10431
DESERET SMALL LETTER SHORT AH

10432
DESERET SMALL LETTER SHORT O

10433
DESERET SMALL LETTER SHORT OO

10434
DESERET SMALL LETTER AY

10435
DESERET SMALL LETTER OW

10436
DESERET SMALL LETTER WU

10437
DESERET SMALL LETTER YEE

10438
DESERET SMALL LETTER H

10439
DESERET SMALL LETTER PEE

1043A
DESERET SMALL LETTER BEE

1043B
DESERET SMALL LETTER TEE

1043C
DESERET SMALL LETTER DEE

1043D
DESERET SMALL LETTER CHEE

1043E
DESERET SMALL LETTER JEE

1043F
DESERET SMALL LETTER KAY

10440
DESERET SMALL LETTER GAY

10441
DESERET SMALL LETTER EF

10442
DESERET SMALL LETTER VEE

10443
DESERET SMALL LETTER ETH

10444
DESERET SMALL LETTER THEE

10445
DESERET SMALL LETTER ES

10446
DESERET SMALL LETTER ZEE

10447
DESERET SMALL LETTER ESH

10448
DESERET SMALL LETTER ZHEE

10449
DESERET SMALL LETTER ER

1044A
DESERET SMALL LETTER EL

1044B
DESERET SMALL LETTER EM

1044C
DESERET SMALL LETTER EN

1044D
DESERET SMALL LETTER ENG

1044E
DESERET SMALL LETTER OI

1044F
DESERET SMALL LETTER EW

10450
SHAVIAN LETTER PEEP

10451
SHAVIAN LETTER TOT

10452
SHAVIAN LETTER KICK

10453
SHAVIAN LETTER FEE

10454
SHAVIAN LETTER THIGH

10455
SHAVIAN LETTER SO

10456
SHAVIAN LETTER SURE

10457
SHAVIAN LETTER CHURCH

10458
SHAVIAN LETTER YEA

10459
SHAVIAN LETTER HUNG

1045A
SHAVIAN LETTER BIB

1045B
SHAVIAN LETTER DEAD

1045C
SHAVIAN LETTER GAG

1045D
SHAVIAN LETTER VOW

1045E
SHAVIAN LETTER THEY

1045F
SHAVIAN LETTER ZOO

10460
SHAVIAN LETTER MEASURE

10461
SHAVIAN LETTER JUDGE

10462
SHAVIAN LETTER WOE

10463
SHAVIAN LETTER HA-HA

10464
SHAVIAN LETTER LOLL

10465
SHAVIAN LETTER MIME

10466
SHAVIAN LETTER IF

10467
SHAVIAN LETTER EGG

10468
SHAVIAN LETTER ASH

10469
SHAVIAN LETTER ADO

1046A
SHAVIAN LETTER ON

1046B
SHAVIAN LETTER WOOL

1046C
SHAVIAN LETTER OUT

1046D
SHAVIAN LETTER AH

1046E
SHAVIAN LETTER ROAR

1046F
SHAVIAN LETTER NUN

10470
SHAVIAN LETTER EAT

10471
SHAVIAN LETTER AGE

10472
SHAVIAN LETTER ICE

10473
SHAVIAN LETTER UP

10474
SHAVIAN LETTER OAK

10475
SHAVIAN LETTER OOZE

10476
SHAVIAN LETTER OIL

10477
SHAVIAN LETTER AWE

10478
SHAVIAN LETTER ARE

10479
SHAVIAN LETTER OR

1047A
SHAVIAN LETTER AIR

1047B
SHAVIAN LETTER ERR

1047C
SHAVIAN LETTER ARRAY

1047D
SHAVIAN LETTER EAR

1047E
SHAVIAN LETTER IAN

1047F
SHAVIAN LETTER YEW

10480
OSMANYA LETTER ALEF

10481
OSMANYA LETTER BA

10482
OSMANYA LETTER TA

10483
OSMANYA LETTER JA

10484
OSMANYA LETTER XA

10485
OSMANYA LETTER KHA

10486
OSMANYA LETTER DEEL

10487
OSMANYA LETTER RA

10488
OSMANYA LETTER SA

10489
OSMANYA LETTER SHIIN

1048A
OSMANYA LETTER DHA

1048B
OSMANYA LETTER CAYN

1048C
OSMANYA LETTER GA

1048D
OSMANYA LETTER FA

1048E
OSMANYA LETTER QAAF

1048F
OSMANYA LETTER KAAF

10490
OSMANYA LETTER LAAN

10491
OSMANYA LETTER MIIN

10492
OSMANYA LETTER NUUN

10493
OSMANYA LETTER WAW

10494
OSMANYA LETTER HA

10495
OSMANYA LETTER YA

10496
OSMANYA LETTER A

10497
OSMANYA LETTER E

10498
OSMANYA LETTER I

10499
OSMANYA LETTER O

1049A
OSMANYA LETTER U

1049B
OSMANYA LETTER AA

1049C
OSMANYA LETTER EE

1049D
OSMANYA LETTER OO

104A0
OSMANYA DIGIT ZERO

104A1
OSMANYA DIGIT ONE

104A2
OSMANYA DIGIT TWO

104A3
OSMANYA DIGIT THREE

104A4
OSMANYA DIGIT FOUR

104A5
OSMANYA DIGIT FIVE

104A6
OSMANYA DIGIT SIX

104A7
OSMANYA DIGIT SEVEN

104A8
OSMANYA DIGIT EIGHT

104A9
OSMANYA DIGIT NINE

104B0
OSAGE CAPITAL LETTER A

104B1
OSAGE CAPITAL LETTER AI

104B2
OSAGE CAPITAL LETTER AIN

104B3
OSAGE CAPITAL LETTER AH

104B4
OSAGE CAPITAL LETTER BRA

104B5
OSAGE CAPITAL LETTER CHA

104B6
OSAGE CAPITAL LETTER EHCHA

104B7
OSAGE CAPITAL LETTER E

104B8
OSAGE CAPITAL LETTER EIN

104B9
OSAGE CAPITAL LETTER HA

104BA
OSAGE CAPITAL LETTER HYA

104BB
OSAGE CAPITAL LETTER I

104BC
OSAGE CAPITAL LETTER KA

104BD
OSAGE CAPITAL LETTER EHKA

104BE
OSAGE CAPITAL LETTER KYA

104BF
OSAGE CAPITAL LETTER LA

104C0
OSAGE CAPITAL LETTER MA

104C1
OSAGE CAPITAL LETTER NA

104C2
OSAGE CAPITAL LETTER O

104C3
OSAGE CAPITAL LETTER OIN

104C4
OSAGE CAPITAL LETTER PA

104C5
OSAGE CAPITAL LETTER EHPA

104C6
OSAGE CAPITAL LETTER SA

104C7
OSAGE CAPITAL LETTER SHA

104C8
OSAGE CAPITAL LETTER TA

104C9
OSAGE CAPITAL LETTER EHTA

104CA
OSAGE CAPITAL LETTER TSA

104CB
OSAGE CAPITAL LETTER EHTSA

104CC
OSAGE CAPITAL LETTER TSHA

104CD
OSAGE CAPITAL LETTER DHA

104CE
OSAGE CAPITAL LETTER U

104CF
OSAGE CAPITAL LETTER WA

104D0
OSAGE CAPITAL LETTER KHA

104D1
OSAGE CAPITAL LETTER GHA

104D2
OSAGE CAPITAL LETTER ZA

104D3
OSAGE CAPITAL LETTER ZHA

104D8
OSAGE SMALL LETTER A

104D9
OSAGE SMALL LETTER AI

104DA
OSAGE SMALL LETTER AIN

104DB
OSAGE SMALL LETTER AH

104DC
OSAGE SMALL LETTER BRA

104DD
OSAGE SMALL LETTER CHA

104DE
OSAGE SMALL LETTER EHCHA

104DF
OSAGE SMALL LETTER E

104E0
OSAGE SMALL LETTER EIN

104E1
OSAGE SMALL LETTER HA

104E2
OSAGE SMALL LETTER HYA

104E3
OSAGE SMALL LETTER I

104E4
OSAGE SMALL LETTER KA

104E5
OSAGE SMALL LETTER EHKA

104E6
OSAGE SMALL LETTER KYA

104E7
OSAGE SMALL LETTER LA

104E8
OSAGE SMALL LETTER MA

104E9
OSAGE SMALL LETTER NA

104EA
OSAGE SMALL LETTER O

104EB
OSAGE SMALL LETTER OIN

104EC
OSAGE SMALL LETTER PA

104ED
OSAGE SMALL LETTER EHPA

104EE
OSAGE SMALL LETTER SA

104EF
OSAGE SMALL LETTER SHA

104F0
OSAGE SMALL LETTER TA

104F1
OSAGE SMALL LETTER EHTA

104F2
OSAGE SMALL LETTER TSA

104F3
OSAGE SMALL LETTER EHTSA

104F4
OSAGE SMALL LETTER TSHA

104F5
OSAGE SMALL LETTER DHA

104F6
OSAGE SMALL LETTER U

104F7
OSAGE SMALL LETTER WA

104F8
OSAGE SMALL LETTER KHA

104F9
OSAGE SMALL LETTER GHA

104FA
OSAGE SMALL LETTER ZA

104FB
OSAGE SMALL LETTER ZHA

10500
ELBASAN LETTER A

10501
ELBASAN LETTER BE

10502
ELBASAN LETTER CE

10503
ELBASAN LETTER CHE

10504
ELBASAN LETTER DE

10505
ELBASAN LETTER NDE

10506
ELBASAN LETTER DHE

10507
ELBASAN LETTER EI

10508
ELBASAN LETTER E

10509
ELBASAN LETTER FE

1050A
ELBASAN LETTER GE

1050B
ELBASAN LETTER GJE

1050C
ELBASAN LETTER HE

1050D
ELBASAN LETTER I

1050E
ELBASAN LETTER JE

1050F
ELBASAN LETTER KE

10510
ELBASAN LETTER LE

10511
ELBASAN LETTER LLE

10512
ELBASAN LETTER ME

10513
ELBASAN LETTER NE

10514
ELBASAN LETTER NA

10515
ELBASAN LETTER NJE

10516
ELBASAN LETTER O

10517
ELBASAN LETTER PE

10518
ELBASAN LETTER QE

10519
ELBASAN LETTER RE

1051A
ELBASAN LETTER RRE

1051B
ELBASAN LETTER SE

1051C
ELBASAN LETTER SHE

1051D
ELBASAN LETTER TE

1051E
ELBASAN LETTER THE

1051F
ELBASAN LETTER U

10520
ELBASAN LETTER VE

10521
ELBASAN LETTER XE

10522
ELBASAN LETTER Y

10523
ELBASAN LETTER ZE

10524
ELBASAN LETTER ZHE

10525
ELBASAN LETTER GHE

10526
ELBASAN LETTER GHAMMA

10527
ELBASAN LETTER KHE

10530
CAUCASIAN ALBANIAN LETTER ALT

10531
CAUCASIAN ALBANIAN LETTER BET

10532
CAUCASIAN ALBANIAN LETTER GIM

10533
CAUCASIAN ALBANIAN LETTER DAT

10534
CAUCASIAN ALBANIAN LETTER EB

10535
CAUCASIAN ALBANIAN LETTER ZARL

10536
CAUCASIAN ALBANIAN LETTER EYN

10537
CAUCASIAN ALBANIAN LETTER ZHIL

10538
CAUCASIAN ALBANIAN LETTER TAS

10539
CAUCASIAN ALBANIAN LETTER CHA

1053A
CAUCASIAN ALBANIAN LETTER YOWD

1053B
CAUCASIAN ALBANIAN LETTER ZHA

1053C
CAUCASIAN ALBANIAN LETTER IRB

1053D
CAUCASIAN ALBANIAN LETTER SHA

1053E
CAUCASIAN ALBANIAN LETTER LAN

1053F
CAUCASIAN ALBANIAN LETTER INYA

10540
CAUCASIAN ALBANIAN LETTER XEYN

10541
CAUCASIAN ALBANIAN LETTER DYAN

10542
CAUCASIAN ALBANIAN LETTER CAR

10543
CAUCASIAN ALBANIAN LETTER JHOX

10544
CAUCASIAN ALBANIAN LETTER KAR

10545
CAUCASIAN ALBANIAN LETTER LYIT

10546
CAUCASIAN ALBANIAN LETTER HEYT

10547
CAUCASIAN ALBANIAN LETTER QAY

10548
CAUCASIAN ALBANIAN LETTER AOR

10549
CAUCASIAN ALBANIAN LETTER CHOY

1054A
CAUCASIAN ALBANIAN LETTER CHI

1054B
CAUCASIAN ALBANIAN LETTER CYAY

1054C
CAUCASIAN ALBANIAN LETTER MAQ

1054D
CAUCASIAN ALBANIAN LETTER QAR

1054E
CAUCASIAN ALBANIAN LETTER NOWC

1054F
CAUCASIAN ALBANIAN LETTER DZYAY

10550
CAUCASIAN ALBANIAN LETTER SHAK

10551
CAUCASIAN ALBANIAN LETTER JAYN

10552
CAUCASIAN ALBANIAN LETTER ON

10553
CAUCASIAN ALBANIAN LETTER TYAY

10554
CAUCASIAN ALBANIAN LETTER FAM

10555
CAUCASIAN ALBANIAN LETTER DZAY

10556
CAUCASIAN ALBANIAN LETTER CHAT

10557
CAUCASIAN ALBANIAN LETTER PEN

10558
CAUCASIAN ALBANIAN LETTER GHEYS

10559
CAUCASIAN ALBANIAN LETTER RAT

1055A
CAUCASIAN ALBANIAN LETTER SEYK

1055B
CAUCASIAN ALBANIAN LETTER VEYZ

1055C
CAUCASIAN ALBANIAN LETTER TIWR

1055D
CAUCASIAN ALBANIAN LETTER SHOY

1055E
CAUCASIAN ALBANIAN LETTER IWN

1055F
CAUCASIAN ALBANIAN LETTER CYAW

10560
CAUCASIAN ALBANIAN LETTER CAYN

10561
CAUCASIAN ALBANIAN LETTER YAYD

10562
CAUCASIAN ALBANIAN LETTER PIWR

10563
CAUCASIAN ALBANIAN LETTER KIW

1056F
CAUCASIAN ALBANIAN CITATION MARK

10570
VITHKUQI CAPITAL LETTER A

10571
VITHKUQI CAPITAL LETTER BBE

10572
VITHKUQI CAPITAL LETTER BE

10573
VITHKUQI CAPITAL LETTER CE

10574
VITHKUQI CAPITAL LETTER CHE

10575
VITHKUQI CAPITAL LETTER DE

10576
VITHKUQI CAPITAL LETTER DHE

10577
VITHKUQI CAPITAL LETTER EI

10578
VITHKUQI CAPITAL LETTER E

10579
VITHKUQI CAPITAL LETTER FE

1057A
VITHKUQI CAPITAL LETTER GA

1057C
VITHKUQI CAPITAL LETTER HA

1057D
VITHKUQI CAPITAL LETTER HHA

1057E
VITHKUQI CAPITAL LETTER I

1057F
VITHKUQI CAPITAL LETTER IJE

10580
VITHKUQI CAPITAL LETTER JE

10581
VITHKUQI CAPITAL LETTER KA

10582
VITHKUQI CAPITAL LETTER LA

10583
VITHKUQI CAPITAL LETTER LLA

10584
VITHKUQI CAPITAL LETTER ME

10585
VITHKUQI CAPITAL LETTER NE

10586
VITHKUQI CAPITAL LETTER NJE

10587
VITHKUQI CAPITAL LETTER O

10588
VITHKUQI CAPITAL LETTER PE

10589
VITHKUQI CAPITAL LETTER QA

1058A
VITHKUQI CAPITAL LETTER RE

1058C
VITHKUQI CAPITAL LETTER SE

1058D
VITHKUQI CAPITAL LETTER SHE

1058E
VITHKUQI CAPITAL LETTER TE

1058F
VITHKUQI CAPITAL LETTER THE

10590
VITHKUQI CAPITAL LETTER U

10591
VITHKUQI CAPITAL LETTER VE

10592
VITHKUQI CAPITAL LETTER XE

10594
VITHKUQI CAPITAL LETTER Y

10595
VITHKUQI CAPITAL LETTER ZE

10597
VITHKUQI SMALL LETTER A

10598
VITHKUQI SMALL LETTER BBE

10599
VITHKUQI SMALL LETTER BE

1059A
VITHKUQI SMALL LETTER CE

1059B
VITHKUQI SMALL LETTER CHE

1059C
VITHKUQI SMALL LETTER DE

1059D
VITHKUQI SMALL LETTER DHE

1059E
VITHKUQI SMALL LETTER EI

1059F
VITHKUQI SMALL LETTER E

105A0
VITHKUQI SMALL LETTER FE

105A1
VITHKUQI SMALL LETTER GA

105A3
VITHKUQI SMALL LETTER HA

105A4
VITHKUQI SMALL LETTER HHA

105A5
VITHKUQI SMALL LETTER I

105A6
VITHKUQI SMALL LETTER IJE

105A7
VITHKUQI SMALL LETTER JE

105A8
VITHKUQI SMALL LETTER KA

105A9
VITHKUQI SMALL LETTER LA

105AA
VITHKUQI SMALL LETTER LLA

105AB
VITHKUQI SMALL LETTER ME

105AC
VITHKUQI SMALL LETTER NE

105AD
VITHKUQI SMALL LETTER NJE

105AE
VITHKUQI SMALL LETTER O

105AF
VITHKUQI SMALL LETTER PE

105B0
VITHKUQI SMALL LETTER QA

105B1
VITHKUQI SMALL LETTER RE

105B3
VITHKUQI SMALL LETTER SE

105B4
VITHKUQI SMALL LETTER SHE

105B5
VITHKUQI SMALL LETTER TE

105B6
VITHKUQI SMALL LETTER THE

105B7
VITHKUQI SMALL LETTER U

105B8
VITHKUQI SMALL LETTER VE

105B9
VITHKUQI SMALL LETTER XE

105BB
VITHKUQI SMALL LETTER Y

105BC
VITHKUQI SMALL LETTER ZE

105C0
TODHRI LETTER A

105C1
TODHRI LETTER AS

105C2
TODHRI LETTER BA

105C3
TODHRI LETTER MBA

105C4
TODHRI LETTER CA

105C5
TODHRI LETTER CHA

105C6
TODHRI LETTER DA

105C7
TODHRI LETTER NDA

105C8
TODHRI LETTER DHA

105C9
TODHRI LETTER EI

105CA
TODHRI LETTER E

105CB
TODHRI LETTER FA

105CC
TODHRI LETTER GA

105CD
TODHRI LETTER NGA

105CE
TODHRI LETTER GJA

105CF
TODHRI LETTER NGJA

105D0
TODHRI LETTER HA

105D1
TODHRI LETTER HJA

105D2
TODHRI LETTER I

105D3
TODHRI LETTER JA

105D4
TODHRI LETTER KA

105D5
TODHRI LETTER LA

105D6
TODHRI LETTER LLA

105D7
TODHRI LETTER MA

105D8
TODHRI LETTER NA

105D9
TODHRI LETTER NJAN

105DA
TODHRI LETTER O

105DB
TODHRI LETTER PA

105DC
TODHRI LETTER QA

105DD
TODHRI LETTER RA

105DE
TODHRI LETTER RRA

105DF
TODHRI LETTER SA

105E0
TODHRI LETTER SHA

105E1
TODHRI LETTER SHTA

105E2
TODHRI LETTER TA

105E3
TODHRI LETTER THA

105E4
TODHRI LETTER U

105E5
TODHRI LETTER VA

105E6
TODHRI LETTER XA

105E7
TODHRI LETTER NXA

105E8
TODHRI LETTER XHA

105E9
TODHRI LETTER NXHA

105EA
TODHRI LETTER Y

105EB
TODHRI LETTER JY

105EC
TODHRI LETTER ZA

105ED
TODHRI LETTER ZHA

105EE
TODHRI LETTER GHA

105EF
TODHRI LETTER STA

105F0
TODHRI LETTER SKAN

105F1
TODHRI LETTER KHA

105F2
TODHRI LETTER PSA

105F3
TODHRI LETTER OO

10600
LINEAR A SIGN AB001

10601
LINEAR A SIGN AB002

10602
LINEAR A SIGN AB003

10603
LINEAR A SIGN AB004

10604
LINEAR A SIGN AB005

10605
LINEAR A SIGN AB006

10606
LINEAR A SIGN AB007

10607
LINEAR A SIGN AB008

10608
LINEAR A SIGN AB009

10609
LINEAR A SIGN AB010

1060A
LINEAR A SIGN AB011

1060B
LINEAR A SIGN AB013

1060C
LINEAR A SIGN AB016

1060D
LINEAR A SIGN AB017

1060E
LINEAR A SIGN AB020

1060F
LINEAR A SIGN AB021

10610
LINEAR A SIGN AB021F

10611
LINEAR A SIGN AB021M

10612
LINEAR A SIGN AB022

10613
LINEAR A SIGN AB022F

10614
LINEAR A SIGN AB022M

10615
LINEAR A SIGN AB023

10616
LINEAR A SIGN AB023M

10617
LINEAR A SIGN AB024

10618
LINEAR A SIGN AB026

10619
LINEAR A SIGN AB027

1061A
LINEAR A SIGN AB028

1061B
LINEAR A SIGN A028B

1061C
LINEAR A SIGN AB029

1061D
LINEAR A SIGN AB030

1061E
LINEAR A SIGN AB031

1061F
LINEAR A SIGN AB034

10620
LINEAR A SIGN AB037

10621
LINEAR A SIGN AB038

10622
LINEAR A SIGN AB039

10623
LINEAR A SIGN AB040

10624
LINEAR A SIGN AB041

10625
LINEAR A SIGN AB044

10626
LINEAR A SIGN AB045

10627
LINEAR A SIGN AB046

10628
LINEAR A SIGN AB047

10629
LINEAR A SIGN AB048

1062A
LINEAR A SIGN AB049

1062B
LINEAR A SIGN AB050

1062C
LINEAR A SIGN AB051

1062D
LINEAR A SIGN AB053

1062E
LINEAR A SIGN AB054

1062F
LINEAR A SIGN AB055

10630
LINEAR A SIGN AB056

10631
LINEAR A SIGN AB057

10632
LINEAR A SIGN AB058

10633
LINEAR A SIGN AB059

10634
LINEAR A SIGN AB060

10635
LINEAR A SIGN AB061

10636
LINEAR A SIGN AB065

10637
LINEAR A SIGN AB066

10638
LINEAR A SIGN AB067

10639
LINEAR A SIGN AB069

1063A
LINEAR A SIGN AB070

1063B
LINEAR A SIGN AB073

1063C
LINEAR A SIGN AB074

1063D
LINEAR A SIGN AB076

1063E
LINEAR A SIGN AB077

1063F
LINEAR A SIGN AB078

10640
LINEAR A SIGN AB079

10641
LINEAR A SIGN AB080

10642
LINEAR A SIGN AB081

10643
LINEAR A SIGN AB082

10644
LINEAR A SIGN AB085

10645
LINEAR A SIGN AB086

10646
LINEAR A SIGN AB087

10647
LINEAR A SIGN A100-102

10648
LINEAR A SIGN AB118

10649
LINEAR A SIGN AB120

1064A
LINEAR A SIGN A120B

1064B
LINEAR A SIGN AB122

1064C
LINEAR A SIGN AB123

1064D
LINEAR A SIGN AB131A

1064E
LINEAR A SIGN AB131B

1064F
LINEAR A SIGN A131C

10650
LINEAR A SIGN AB164

10651
LINEAR A SIGN AB171

10652
LINEAR A SIGN AB180

10653
LINEAR A SIGN AB188

10654
LINEAR A SIGN AB191

10655
LINEAR A SIGN A301

10656
LINEAR A SIGN A302

10657
LINEAR A SIGN A303

10658
LINEAR A SIGN A304

10659
LINEAR A SIGN A305

1065A
LINEAR A SIGN A306

1065B
LINEAR A SIGN A307

1065C
LINEAR A SIGN A308

1065D
LINEAR A SIGN A309A

1065E
LINEAR A SIGN A309B

1065F
LINEAR A SIGN A309C

10660
LINEAR A SIGN A310

10661
LINEAR A SIGN A311

10662
LINEAR A SIGN A312

10663
LINEAR A SIGN A313A

10664
LINEAR A SIGN A313B

10665
LINEAR A SIGN A313C

10666
LINEAR A SIGN A314

10667
LINEAR A SIGN A315

10668
LINEAR A SIGN A316

10669
LINEAR A SIGN A317

1066A
LINEAR A SIGN A318

1066B
LINEAR A SIGN A319

1066C
LINEAR A SIGN A320

1066D
LINEAR A SIGN A321

1066E
LINEAR A SIGN A322

1066F
LINEAR A SIGN A323

10670
LINEAR A SIGN A324

10671
LINEAR A SIGN A325

10672
LINEAR A SIGN A326

10673
LINEAR A SIGN A327

10674
LINEAR A SIGN A328

10675
LINEAR A SIGN A329

10676
LINEAR A SIGN A330

10677
LINEAR A SIGN A331

10678
LINEAR A SIGN A332

10679
LINEAR A SIGN A333

1067A
LINEAR A SIGN A334

1067B
LINEAR A SIGN A335

1067C
LINEAR A SIGN A336

1067D
LINEAR A SIGN A337

1067E
LINEAR A SIGN A338

1067F
LINEAR A SIGN A339

10680
LINEAR A SIGN A340

10681
LINEAR A SIGN A341

10682
LINEAR A SIGN A342

10683
LINEAR A SIGN A343

10684
LINEAR A SIGN A344

10685
LINEAR A SIGN A345

10686
LINEAR A SIGN A346

10687
LINEAR A SIGN A347

10688
LINEAR A SIGN A348

10689
LINEAR A SIGN A349

1068A
LINEAR A SIGN A350

1068B
LINEAR A SIGN A351

1068C
LINEAR A SIGN A352

1068D
LINEAR A SIGN A353

1068E
LINEAR A SIGN A354

1068F
LINEAR A SIGN A355

10690
LINEAR A SIGN A356

10691
LINEAR A SIGN A357

10692
LINEAR A SIGN A358

10693
LINEAR A SIGN A359

10694
LINEAR A SIGN A360

10695
LINEAR A SIGN A361

10696
LINEAR A SIGN A362

10697
LINEAR A SIGN A363

10698
LINEAR A SIGN A364

10699
LINEAR A SIGN A365

1069A
LINEAR A SIGN A366

1069B
LINEAR A SIGN A367

1069C
LINEAR A SIGN A368

1069D
LINEAR A SIGN A369

1069E
LINEAR A SIGN A370

1069F
LINEAR A SIGN A371

106A0
LINEAR A SIGN A400-VAS

106A1
LINEAR A SIGN A401-VAS

106A2
LINEAR A SIGN A402-VAS

106A3
LINEAR A SIGN A403-VAS

106A4
LINEAR A SIGN A404-VAS

106A5
LINEAR A SIGN A405-VAS

106A6
LINEAR A SIGN A406-VAS

106A7
LINEAR A SIGN A407-VAS

106A8
LINEAR A SIGN A408-VAS

106A9
LINEAR A SIGN A409-VAS

106AA
LINEAR A SIGN A410-VAS

106AB
LINEAR A SIGN A411-VAS

106AC
LINEAR A SIGN A412-VAS

106AD
LINEAR A SIGN A413-VAS

106AE
LINEAR A SIGN A414-VAS

106AF
LINEAR A SIGN A415-VAS

106B0
LINEAR A SIGN A416-VAS

106B1
LINEAR A SIGN A417-VAS

106B2
LINEAR A SIGN A418-VAS

106B3
LINEAR A SIGN A501

106B4
LINEAR A SIGN A502

106B5
LINEAR A SIGN A503

106B6
LINEAR A SIGN A504

106B7
LINEAR A SIGN A505

106B8
LINEAR A SIGN A506

106B9
LINEAR A SIGN A508

106BA
LINEAR A SIGN A509

106BB
LINEAR A SIGN A510

106BC
LINEAR A SIGN A511

106BD
LINEAR A SIGN A512

106BE
LINEAR A SIGN A513

106BF
LINEAR A SIGN A515

106C0
LINEAR A SIGN A516

106C1
LINEAR A SIGN A520

106C2
LINEAR A SIGN A521

106C3
LINEAR A SIGN A523

106C4
LINEAR A SIGN A524

106C5
LINEAR A SIGN A525

106C6
LINEAR A SIGN A526

106C7
LINEAR A SIGN A527

106C8
LINEAR A SIGN A528

106C9
LINEAR A SIGN A529

106CA
LINEAR A SIGN A530

106CB
LINEAR A SIGN A531

106CC
LINEAR A SIGN A532

106CD
LINEAR A SIGN A534

106CE
LINEAR A SIGN A535

106CF
LINEAR A SIGN A536

106D0
LINEAR A SIGN A537

106D1
LINEAR A SIGN A538

106D2
LINEAR A SIGN A539

106D3
LINEAR A SIGN A540

106D4
LINEAR A SIGN A541

106D5
LINEAR A SIGN A542

106D6
LINEAR A SIGN A545

106D7
LINEAR A SIGN A547

106D8
LINEAR A SIGN A548

106D9
LINEAR A SIGN A549

106DA
LINEAR A SIGN A550

106DB
LINEAR A SIGN A551

106DC
LINEAR A SIGN A552

106DD
LINEAR A SIGN A553

106DE
LINEAR A SIGN A554

106DF
LINEAR A SIGN A555

106E0
LINEAR A SIGN A556

106E1
LINEAR A SIGN A557

106E2
LINEAR A SIGN A559

106E3
LINEAR A SIGN A563

106E4
LINEAR A SIGN A564

106E5
LINEAR A SIGN A565

106E6
LINEAR A SIGN A566

106E7
LINEAR A SIGN A568

106E8
LINEAR A SIGN A569

106E9
LINEAR A SIGN A570

106EA
LINEAR A SIGN A571

106EB
LINEAR A SIGN A572

106EC
LINEAR A SIGN A573

106ED
LINEAR A SIGN A574

106EE
LINEAR A SIGN A575

106EF
LINEAR A SIGN A576

106F0
LINEAR A SIGN A577

106F1
LINEAR A SIGN A578

106F2
LINEAR A SIGN A579

106F3
LINEAR A SIGN A580

106F4
LINEAR A SIGN A581

106F5
LINEAR A SIGN A582

106F6
LINEAR A SIGN A583

106F7
LINEAR A SIGN A584

106F8
LINEAR A SIGN A585

106F9
LINEAR A SIGN A586

106FA
LINEAR A SIGN A587

106FB
LINEAR A SIGN A588

106FC
LINEAR A SIGN A589

106FD
LINEAR A SIGN A591

106FE
LINEAR A SIGN A592

106FF
LINEAR A SIGN A594

10700
LINEAR A SIGN A595

10701
LINEAR A SIGN A596

10702
LINEAR A SIGN A598

10703
LINEAR A SIGN A600

10704
LINEAR A SIGN A601

10705
LINEAR A SIGN A602

10706
LINEAR A SIGN A603

10707
LINEAR A SIGN A604

10708
LINEAR A SIGN A606

10709
LINEAR A SIGN A608

1070A
LINEAR A SIGN A609

1070B
LINEAR A SIGN A610

1070C
LINEAR A SIGN A611

1070D
LINEAR A SIGN A612

1070E
LINEAR A SIGN A613

1070F
LINEAR A SIGN A614

10710
LINEAR A SIGN A615

10711
LINEAR A SIGN A616

10712
LINEAR A SIGN A617

10713
LINEAR A SIGN A618

10714
LINEAR A SIGN A619

10715
LINEAR A SIGN A620

10716
LINEAR A SIGN A621

10717
LINEAR A SIGN A622

10718
LINEAR A SIGN A623

10719
LINEAR A SIGN A624

1071A
LINEAR A SIGN A626

1071B
LINEAR A SIGN A627

1071C
LINEAR A SIGN A628

1071D
LINEAR A SIGN A629

1071E
LINEAR A SIGN A634

1071F
LINEAR A SIGN A637

10720
LINEAR A SIGN A638

10721
LINEAR A SIGN A640

10722
LINEAR A SIGN A642

10723
LINEAR A SIGN A643

10724
LINEAR A SIGN A644

10725
LINEAR A SIGN A645

10726
LINEAR A SIGN A646

10727
LINEAR A SIGN A648

10728
LINEAR A SIGN A649

10729
LINEAR A SIGN A651

1072A
LINEAR A SIGN A652

1072B
LINEAR A SIGN A653

1072C
LINEAR A SIGN A654

1072D
LINEAR A SIGN A655

1072E
LINEAR A SIGN A656

1072F
LINEAR A SIGN A657

10730
LINEAR A SIGN A658

10731
LINEAR A SIGN A659

10732
LINEAR A SIGN A660

10733
LINEAR A SIGN A661

10734
LINEAR A SIGN A662

10735
LINEAR A SIGN A663

10736
LINEAR A SIGN A664

10740
LINEAR A SIGN A701 A

10741
LINEAR A SIGN A702 B

10742
LINEAR A SIGN A703 D

10743
LINEAR A SIGN A704 E

10744
LINEAR A SIGN A705 F

10745
LINEAR A SIGN A706 H

10746
LINEAR A SIGN A707 J

10747
LINEAR A SIGN A708 K

10748
LINEAR A SIGN A709 L

10749
LINEAR A SIGN A709-2 L2

1074A
LINEAR A SIGN A709-3 L3

1074B
LINEAR A SIGN A709-4 L4

1074C
LINEAR A SIGN A709-6 L6

1074D
LINEAR A SIGN A710 W

1074E
LINEAR A SIGN A711 X

1074F
LINEAR A SIGN A712 Y

10750
LINEAR A SIGN A713 OMEGA

10751
LINEAR A SIGN A714 ABB

10752
LINEAR A SIGN A715 BB

10753
LINEAR A SIGN A717 DD

10754
LINEAR A SIGN A726 EYYY

10755
LINEAR A SIGN A732 JE

10760
LINEAR A SIGN A800

10761
LINEAR A SIGN A801

10762
LINEAR A SIGN A802

10763
LINEAR A SIGN A803

10764
LINEAR A SIGN A804

10765
LINEAR A SIGN A805

10766
LINEAR A SIGN A806

10767
LINEAR A SIGN A807

10780
MODIFIER LETTER SMALL CAPITAL AA

10781
MODIFIER LETTER SUPERSCRIPT TRIANGULAR COLON

10782
MODIFIER LETTER SUPERSCRIPT HALF TRIANGULAR COLON

10783
MODIFIER LETTER SMALL AE

10784
MODIFIER LETTER SMALL CAPITAL B

10785
MODIFIER LETTER SMALL B WITH HOOK

10787
MODIFIER LETTER SMALL DZ DIGRAPH

10788
MODIFIER LETTER SMALL DZ DIGRAPH WITH RETROFLEX HOOK

10789
MODIFIER LETTER SMALL DZ DIGRAPH WITH CURL

1078A
MODIFIER LETTER SMALL DEZH DIGRAPH

1078B
MODIFIER LETTER SMALL D WITH TAIL

1078C
MODIFIER LETTER SMALL D WITH HOOK

1078D
MODIFIER LETTER SMALL D WITH HOOK AND TAIL

1078E
MODIFIER LETTER SMALL REVERSED E

1078F
MODIFIER LETTER SMALL CLOSED REVERSED OPEN E

10790
MODIFIER LETTER SMALL FENG DIGRAPH

10791
MODIFIER LETTER SMALL RAMS HORN

10792
MODIFIER LETTER SMALL CAPITAL G

10793
MODIFIER LETTER SMALL G WITH HOOK

10794
MODIFIER LETTER SMALL CAPITAL G WITH HOOK

10795
MODIFIER LETTER SMALL H WITH STROKE

10796
MODIFIER LETTER SMALL CAPITAL H

10797
MODIFIER LETTER SMALL HENG WITH HOOK

10798
MODIFIER LETTER SMALL DOTLESS J WITH STROKE AND HOOK

10799
MODIFIER LETTER SMALL LS DIGRAPH

1079A
MODIFIER LETTER SMALL LZ DIGRAPH

1079B
MODIFIER LETTER SMALL L WITH BELT

1079C
MODIFIER LETTER SMALL CAPITAL L WITH BELT

1079D
MODIFIER LETTER SMALL L WITH RETROFLEX HOOK AND BELT

1079E
MODIFIER LETTER SMALL LEZH

1079F
MODIFIER LETTER SMALL LEZH WITH RETROFLEX HOOK

107A0
MODIFIER LETTER SMALL TURNED Y

107A1
MODIFIER LETTER SMALL TURNED Y WITH BELT

107A2
MODIFIER LETTER SMALL O WITH STROKE

107A3
MODIFIER LETTER SMALL CAPITAL OE

107A4
MODIFIER LETTER SMALL CLOSED OMEGA

107A5
MODIFIER LETTER SMALL Q

107A6
MODIFIER LETTER SMALL TURNED R WITH LONG LEG

107A7
MODIFIER LETTER SMALL TURNED R WITH LONG LEG AND RETROFLEX HOOK

107A8
MODIFIER LETTER SMALL R WITH TAIL

107A9
MODIFIER LETTER SMALL R WITH FISHHOOK

107AA
MODIFIER LETTER SMALL CAPITAL R

107AB
MODIFIER LETTER SMALL TC DIGRAPH WITH CURL

107AC
MODIFIER LETTER SMALL TS DIGRAPH

107AD
MODIFIER LETTER SMALL TS DIGRAPH WITH RETROFLEX HOOK

107AE
MODIFIER LETTER SMALL TESH DIGRAPH

107AF
MODIFIER LETTER SMALL T WITH RETROFLEX HOOK

107B0
MODIFIER LETTER SMALL V WITH RIGHT HOOK

107B2
MODIFIER LETTER SMALL CAPITAL Y

107B3
MODIFIER LETTER GLOTTAL STOP WITH STROKE

107B4
MODIFIER LETTER REVERSED GLOTTAL STOP WITH STROKE

107B5
MODIFIER LETTER BILABIAL CLICK

107B6
MODIFIER LETTER DENTAL CLICK

107B7
MODIFIER LETTER LATERAL CLICK

107B8
MODIFIER LETTER ALVEOLAR CLICK

107B9
MODIFIER LETTER RETROFLEX CLICK WITH RETROFLEX HOOK

107BA
MODIFIER LETTER SMALL S WITH CURL

10800
CYPRIOT SYLLABLE A

10801
CYPRIOT SYLLABLE E

10802
CYPRIOT SYLLABLE I

10803
CYPRIOT SYLLABLE O

10804
CYPRIOT SYLLABLE U

10805
CYPRIOT SYLLABLE JA

10808
CYPRIOT SYLLABLE JO

1080A
CYPRIOT SYLLABLE KA

1080B
CYPRIOT SYLLABLE KE

1080C
CYPRIOT SYLLABLE KI

1080D
CYPRIOT SYLLABLE KO

1080E
CYPRIOT SYLLABLE KU

1080F
CYPRIOT SYLLABLE LA

10810
CYPRIOT SYLLABLE LE

10811
CYPRIOT SYLLABLE LI

10812
CYPRIOT SYLLABLE LO

10813
CYPRIOT SYLLABLE LU

10814
CYPRIOT SYLLABLE MA

10815
CYPRIOT SYLLABLE ME

10816
CYPRIOT SYLLABLE MI

10817
CYPRIOT SYLLABLE MO

10818
CYPRIOT SYLLABLE MU

10819
CYPRIOT SYLLABLE NA

1081A
CYPRIOT SYLLABLE NE

1081B
CYPRIOT SYLLABLE NI

1081C
CYPRIOT SYLLABLE NO

1081D
CYPRIOT SYLLABLE NU

1081E
CYPRIOT SYLLABLE PA

1081F
CYPRIOT SYLLABLE PE

10820
CYPRIOT SYLLABLE PI

10821
CYPRIOT SYLLABLE PO

10822
CYPRIOT SYLLABLE PU

10823
CYPRIOT SYLLABLE RA

10824
CYPRIOT SYLLABLE RE

10825
CYPRIOT SYLLABLE RI

10826
CYPRIOT SYLLABLE RO

10827
CYPRIOT SYLLABLE RU

10828
CYPRIOT SYLLABLE SA

10829
CYPRIOT SYLLABLE SE

1082A
CYPRIOT SYLLABLE SI

1082B
CYPRIOT SYLLABLE SO

1082C
CYPRIOT SYLLABLE SU

1082D
CYPRIOT SYLLABLE TA

1082E
CYPRIOT SYLLABLE TE

1082F
CYPRIOT SYLLABLE TI

10830
CYPRIOT SYLLABLE TO

10831
CYPRIOT SYLLABLE TU

10832
CYPRIOT SYLLABLE WA

10833
CYPRIOT SYLLABLE WE

10834
CYPRIOT SYLLABLE WI

10835
CYPRIOT SYLLABLE WO

10837
CYPRIOT SYLLABLE XA

10838
CYPRIOT SYLLABLE XE

1083C
CYPRIOT SYLLABLE ZA

1083F
CYPRIOT SYLLABLE ZO

10840
IMPERIAL ARAMAIC LETTER ALEPH

10841
IMPERIAL ARAMAIC LETTER BETH

10842
IMPERIAL ARAMAIC LETTER GIMEL

10843
IMPERIAL ARAMAIC LETTER DALETH

10844
IMPERIAL ARAMAIC LETTER HE

10845
IMPERIAL ARAMAIC LETTER WAW

10846
IMPERIAL ARAMAIC LETTER ZAYIN

10847
IMPERIAL ARAMAIC LETTER HETH

10848
IMPERIAL ARAMAIC LETTER TETH

10849
IMPERIAL ARAMAIC LETTER YODH

1084A
IMPERIAL ARAMAIC LETTER KAPH

1084B
IMPERIAL ARAMAIC LETTER LAMEDH

1084C
IMPERIAL ARAMAIC LETTER MEM

1084D
IMPERIAL ARAMAIC LETTER NUN

1084E
IMPERIAL ARAMAIC LETTER SAMEKH

1084F
IMPERIAL ARAMAIC LETTER AYIN

10850
IMPERIAL ARAMAIC LETTER PE

10851
IMPERIAL ARAMAIC LETTER SADHE

10852
IMPERIAL ARAMAIC LETTER QOPH

10853
IMPERIAL ARAMAIC LETTER RESH

10854
IMPERIAL ARAMAIC LETTER SHIN

10855
IMPERIAL ARAMAIC LETTER TAW

10857
IMPERIAL ARAMAIC SECTION SIGN

10858
IMPERIAL ARAMAIC NUMBER ONE

10859
IMPERIAL ARAMAIC NUMBER TWO

1085A
IMPERIAL ARAMAIC NUMBER THREE

1085B
IMPERIAL ARAMAIC NUMBER TEN

1085C
IMPERIAL ARAMAIC NUMBER TWENTY

1085D
IMPERIAL ARAMAIC NUMBER ONE HUNDRED

1085E
IMPERIAL ARAMAIC NUMBER ONE THOUSAND

1085F
IMPERIAL ARAMAIC NUMBER TEN THOUSAND

10860
PALMYRENE LETTER ALEPH

10861
PALMYRENE LETTER BETH

10862
PALMYRENE LETTER GIMEL

10863
PALMYRENE LETTER DALETH

10864
PALMYRENE LETTER HE

10865
PALMYRENE LETTER WAW

10866
PALMYRENE LETTER ZAYIN

10867
PALMYRENE LETTER HETH

10868
PALMYRENE LETTER TETH

10869
PALMYRENE LETTER YODH

1086A
PALMYRENE LETTER KAPH

1086B
PALMYRENE LETTER LAMEDH

1086C
PALMYRENE LETTER MEM

1086D
PALMYRENE LETTER FINAL NUN

1086E
PALMYRENE LETTER NUN

1086F
PALMYRENE LETTER SAMEKH

10870
PALMYRENE LETTER AYIN

10871
PALMYRENE LETTER PE

10872
PALMYRENE LETTER SADHE

10873
PALMYRENE LETTER QOPH

10874
PALMYRENE LETTER RESH

10875
PALMYRENE LETTER SHIN

10876
PALMYRENE LETTER TAW

10877
PALMYRENE LEFT-POINTING FLEURON

10878
PALMYRENE RIGHT-POINTING FLEURON

10879
PALMYRENE NUMBER ONE

1087A
PALMYRENE NUMBER TWO

1087B
PALMYRENE NUMBER THREE

1087C
PALMYRENE NUMBER FOUR

1087D
PALMYRENE NUMBER FIVE

1087E
PALMYRENE NUMBER TEN

1087F
PALMYRENE NUMBER TWENTY

10880
NABATAEAN LETTER FINAL ALEPH

10881
NABATAEAN LETTER ALEPH

10882
NABATAEAN LETTER FINAL BETH

10883
NABATAEAN LETTER BETH

10884
NABATAEAN LETTER GIMEL

10885
NABATAEAN LETTER DALETH

10886
NABATAEAN LETTER FINAL HE

10887
NABATAEAN LETTER HE

10888
NABATAEAN LETTER WAW

10889
NABATAEAN LETTER ZAYIN

1088A
NABATAEAN LETTER HETH

1088B
NABATAEAN LETTER TETH

1088C
NABATAEAN LETTER FINAL YODH

1088D
NABATAEAN LETTER YODH

1088E
NABATAEAN LETTER FINAL KAPH

1088F
NABATAEAN LETTER KAPH

10890
NABATAEAN LETTER FINAL LAMEDH

10891
NABATAEAN LETTER LAMEDH

10892
NABATAEAN LETTER FINAL MEM

10893
NABATAEAN LETTER MEM

10894
NABATAEAN LETTER FINAL NUN

10895
NABATAEAN LETTER NUN

10896
NABATAEAN LETTER SAMEKH

10897
NABATAEAN LETTER AYIN

10898
NABATAEAN LETTER PE

10899
NABATAEAN LETTER SADHE

1089A
NABATAEAN LETTER QOPH

1089B
NABATAEAN LETTER RESH

1089C
NABATAEAN LETTER FINAL SHIN

1089D
NABATAEAN LETTER SHIN

1089E
NABATAEAN LETTER TAW

108A7
NABATAEAN NUMBER ONE

108A8
NABATAEAN NUMBER TWO

108A9
NABATAEAN NUMBER THREE

108AA
NABATAEAN NUMBER FOUR

108AB
NABATAEAN CRUCIFORM NUMBER FOUR

108AC
NABATAEAN NUMBER FIVE

108AD
NABATAEAN NUMBER TEN

108AE
NABATAEAN NUMBER TWENTY

108AF
NABATAEAN NUMBER ONE HUNDRED

108E0
HATRAN LETTER ALEPH

108E1
HATRAN LETTER BETH

108E2
HATRAN LETTER GIMEL

108E3
HATRAN LETTER DALETH-RESH

108E4
HATRAN LETTER HE

108E5
HATRAN LETTER WAW

108E6
HATRAN LETTER ZAYN

108E7
HATRAN LETTER HETH

108E8
HATRAN LETTER TETH

108E9
HATRAN LETTER YODH

108EA
HATRAN LETTER KAPH

108EB
HATRAN LETTER LAMEDH

108EC
HATRAN LETTER MEM

108ED
HATRAN LETTER NUN

108EE
HATRAN LETTER SAMEKH

108EF
HATRAN LETTER AYN

108F0
HATRAN LETTER PE

108F1
HATRAN LETTER SADHE

108F2
HATRAN LETTER QOPH

108F4
HATRAN LETTER SHIN

108F5
HATRAN LETTER TAW

108FB
HATRAN NUMBER ONE

108FC
HATRAN NUMBER FIVE

108FD
HATRAN NUMBER TEN

108FE
HATRAN NUMBER TWENTY

108FF
HATRAN NUMBER ONE HUNDRED

10900
PHOENICIAN LETTER ALF

10901
PHOENICIAN LETTER BET

10902
PHOENICIAN LETTER GAML

10903
PHOENICIAN LETTER DELT

10904
PHOENICIAN LETTER HE

10905
PHOENICIAN LETTER WAU

10906
PHOENICIAN LETTER ZAI

10907
PHOENICIAN LETTER HET

10908
PHOENICIAN LETTER TET

10909
PHOENICIAN LETTER YOD

1090A
PHOENICIAN LETTER KAF

1090B
PHOENICIAN LETTER LAMD

1090C
PHOENICIAN LETTER MEM

1090D
PHOENICIAN LETTER NUN

1090E
PHOENICIAN LETTER SEMK

1090F
PHOENICIAN LETTER AIN

10910
PHOENICIAN LETTER PE

10911
PHOENICIAN LETTER SADE

10912
PHOENICIAN LETTER QOF

10913
PHOENICIAN LETTER ROSH

10914
PHOENICIAN LETTER SHIN

10915
PHOENICIAN LETTER TAU

10916
PHOENICIAN NUMBER ONE

10917
PHOENICIAN NUMBER TEN

10918
PHOENICIAN NUMBER TWENTY

10919
PHOENICIAN NUMBER ONE HUNDRED

1091A
PHOENICIAN NUMBER TWO

1091B
PHOENICIAN NUMBER THREE

1091F
PHOENICIAN WORD SEPARATOR

10920
LYDIAN LETTER A

10921
LYDIAN LETTER B

10922
LYDIAN LETTER G

10923
LYDIAN LETTER D

10924
LYDIAN LETTER E

10925
LYDIAN LETTER V

10926
LYDIAN LETTER I

10927
LYDIAN LETTER Y

10928
LYDIAN LETTER K

10929
LYDIAN LETTER L

1092A
LYDIAN LETTER M

1092B
LYDIAN LETTER N

1092C
LYDIAN LETTER O

1092D
LYDIAN LETTER R

1092E
LYDIAN LETTER SS

1092F
LYDIAN LETTER T

10930
LYDIAN LETTER U

10931
LYDIAN LETTER F

10932
LYDIAN LETTER Q

10933
LYDIAN LETTER S

10934
LYDIAN LETTER TT

10935
LYDIAN LETTER AN

10936
LYDIAN LETTER EN

10937
LYDIAN LETTER LY

10938
LYDIAN LETTER NN

10939
LYDIAN LETTER C

1093F
LYDIAN TRIANGULAR MARK

10940
SIDETIC LETTER N01

10941
SIDETIC LETTER N02

10942
SIDETIC LETTER N03

10943
SIDETIC LETTER N04

10944
SIDETIC LETTER N05

10945
SIDETIC LETTER N06

10946
SIDETIC LETTER N07

10947
SIDETIC LETTER N08

10948
SIDETIC LETTER N09

10949
SIDETIC LETTER N10

1094A
SIDETIC LETTER N11

1094B
SIDETIC LETTER N12

1094C
SIDETIC LETTER N13

1094D
SIDETIC LETTER N14

1094E
SIDETIC LETTER N15

1094F
SIDETIC LETTER N16

10950
SIDETIC LETTER N17

10951
SIDETIC LETTER N18

10952
SIDETIC LETTER N19

10953
SIDETIC LETTER N20

10954
SIDETIC LETTER N21

10955
SIDETIC LETTER N22

10956
SIDETIC LETTER N23

10957
SIDETIC LETTER N24

10958
SIDETIC LETTER N25

10959
SIDETIC LETTER N26

10980
MEROITIC HIEROGLYPHIC LETTER A

10981
MEROITIC HIEROGLYPHIC LETTER E

10982
MEROITIC HIEROGLYPHIC LETTER I

10983
MEROITIC HIEROGLYPHIC LETTER O

10984
MEROITIC HIEROGLYPHIC LETTER YA

10985
MEROITIC HIEROGLYPHIC LETTER WA

10986
MEROITIC HIEROGLYPHIC LETTER BA

10987
MEROITIC HIEROGLYPHIC LETTER BA-2

10988
MEROITIC HIEROGLYPHIC LETTER PA

10989
MEROITIC HIEROGLYPHIC LETTER MA

1098A
MEROITIC HIEROGLYPHIC LETTER NA

1098B
MEROITIC HIEROGLYPHIC LETTER NA-2

1098C
MEROITIC HIEROGLYPHIC LETTER NE

1098D
MEROITIC HIEROGLYPHIC LETTER NE-2

1098E
MEROITIC HIEROGLYPHIC LETTER RA

1098F
MEROITIC HIEROGLYPHIC LETTER RA-2

10990
MEROITIC HIEROGLYPHIC LETTER LA

10991
MEROITIC HIEROGLYPHIC LETTER KHA

10992
MEROITIC HIEROGLYPHIC LETTER HHA

10993
MEROITIC HIEROGLYPHIC LETTER SA

10994
MEROITIC HIEROGLYPHIC LETTER SA-2

10995
MEROITIC HIEROGLYPHIC LETTER SE

10996
MEROITIC HIEROGLYPHIC LETTER KA

10997
MEROITIC HIEROGLYPHIC LETTER QA

10998
MEROITIC HIEROGLYPHIC LETTER TA

10999
MEROITIC HIEROGLYPHIC LETTER TA-2

1099A
MEROITIC HIEROGLYPHIC LETTER TE

1099B
MEROITIC HIEROGLYPHIC LETTER TE-2

1099C
MEROITIC HIEROGLYPHIC LETTER TO

1099D
MEROITIC HIEROGLYPHIC LETTER DA

1099E
MEROITIC HIEROGLYPHIC SYMBOL VIDJ

1099F
MEROITIC HIEROGLYPHIC SYMBOL VIDJ-2

109A0
MEROITIC CURSIVE LETTER A

109A1
MEROITIC CURSIVE LETTER E

109A2
MEROITIC CURSIVE LETTER I

109A3
MEROITIC CURSIVE LETTER O

109A4
MEROITIC CURSIVE LETTER YA

109A5
MEROITIC CURSIVE LETTER WA

109A6
MEROITIC CURSIVE LETTER BA

109A7
MEROITIC CURSIVE LETTER PA

109A8
MEROITIC CURSIVE LETTER MA

109A9
MEROITIC CURSIVE LETTER NA

109AA
MEROITIC CURSIVE LETTER NE

109AB
MEROITIC CURSIVE LETTER RA

109AC
MEROITIC CURSIVE LETTER LA

109AD
MEROITIC CURSIVE LETTER KHA

109AE
MEROITIC CURSIVE LETTER HHA

109AF
MEROITIC CURSIVE LETTER SA

109B0
MEROITIC CURSIVE LETTER ARCHAIC SA

109B1
MEROITIC CURSIVE LETTER SE

109B2
MEROITIC CURSIVE LETTER KA

109B3
MEROITIC CURSIVE LETTER QA

109B4
MEROITIC CURSIVE LETTER TA

109B5
MEROITIC CURSIVE LETTER TE

109B6
MEROITIC CURSIVE LETTER TO

109B7
MEROITIC CURSIVE LETTER DA

109BC
MEROITIC CURSIVE FRACTION ELEVEN TWELFTHS

109BD
MEROITIC CURSIVE FRACTION ONE HALF

109BE
MEROITIC CURSIVE LOGOGRAM RMT

109BF
MEROITIC CURSIVE LOGOGRAM IMN

109C0
MEROITIC CURSIVE NUMBER ONE

109C1
MEROITIC CURSIVE NUMBER TWO

109C2
MEROITIC CURSIVE NUMBER THREE

109C3
MEROITIC CURSIVE NUMBER FOUR

109C4
MEROITIC CURSIVE NUMBER FIVE

109C5
MEROITIC CURSIVE NUMBER SIX

109C6
MEROITIC CURSIVE NUMBER SEVEN

109C7
MEROITIC CURSIVE NUMBER EIGHT

109C8
MEROITIC CURSIVE NUMBER NINE

109C9
MEROITIC CURSIVE NUMBER TEN

109CA
MEROITIC CURSIVE NUMBER TWENTY

109CB
MEROITIC CURSIVE NUMBER THIRTY

109CC
MEROITIC CURSIVE NUMBER FORTY

109CD
MEROITIC CURSIVE NUMBER FIFTY

109CE
MEROITIC CURSIVE NUMBER SIXTY

109CF
MEROITIC CURSIVE NUMBER SEVENTY

109D2
MEROITIC CURSIVE NUMBER ONE HUNDRED

109D3
MEROITIC CURSIVE NUMBER TWO HUNDRED

109D4
MEROITIC CURSIVE NUMBER THREE HUNDRED

109D5
MEROITIC CURSIVE NUMBER FOUR HUNDRED

109D6
MEROITIC CURSIVE NUMBER FIVE HUNDRED

109D7
MEROITIC CURSIVE NUMBER SIX HUNDRED

109D8
MEROITIC CURSIVE NUMBER SEVEN HUNDRED

109D9
MEROITIC CURSIVE NUMBER EIGHT HUNDRED

109DA
MEROITIC CURSIVE NUMBER NINE HUNDRED

109DB
MEROITIC CURSIVE NUMBER ONE THOUSAND

109DC
MEROITIC CURSIVE NUMBER TWO THOUSAND

109DD
MEROITIC CURSIVE NUMBER THREE THOUSAND

109DE
MEROITIC CURSIVE NUMBER FOUR THOUSAND

109DF
MEROITIC CURSIVE NUMBER FIVE THOUSAND

109E0
MEROITIC CURSIVE NUMBER SIX THOUSAND

109E1
MEROITIC CURSIVE NUMBER SEVEN THOUSAND

109E2
MEROITIC CURSIVE NUMBER EIGHT THOUSAND

109E3
MEROITIC CURSIVE NUMBER NINE THOUSAND

109E4
MEROITIC CURSIVE NUMBER TEN THOUSAND

109E5
MEROITIC CURSIVE NUMBER TWENTY THOUSAND

109E6
MEROITIC CURSIVE NUMBER THIRTY THOUSAND

109E7
MEROITIC CURSIVE NUMBER FORTY THOUSAND

109E8
MEROITIC CURSIVE NUMBER FIFTY THOUSAND

109E9
MEROITIC CURSIVE NUMBER SIXTY THOUSAND

109EA
MEROITIC CURSIVE NUMBER SEVENTY THOUSAND

109EB
MEROITIC CURSIVE NUMBER EIGHTY THOUSAND

109EC
MEROITIC CURSIVE NUMBER NINETY THOUSAND

109ED
MEROITIC CURSIVE NUMBER ONE HUNDRED THOUSAND

109EE
MEROITIC CURSIVE NUMBER TWO HUNDRED THOUSAND

109EF
MEROITIC CURSIVE NUMBER THREE HUNDRED THOUSAND

109F0
MEROITIC CURSIVE NUMBER FOUR HUNDRED THOUSAND

109F1
MEROITIC CURSIVE NUMBER FIVE HUNDRED THOUSAND

109F2
MEROITIC CURSIVE NUMBER SIX HUNDRED THOUSAND

109F3
MEROITIC CURSIVE NUMBER SEVEN HUNDRED THOUSAND

109F4
MEROITIC CURSIVE NUMBER EIGHT HUNDRED THOUSAND

109F5
MEROITIC CURSIVE NUMBER NINE HUNDRED THOUSAND

109F6
MEROITIC CURSIVE FRACTION ONE TWELFTH

109F7
MEROITIC CURSIVE FRACTION TWO TWELFTHS

109F8
MEROITIC CURSIVE FRACTION THREE TWELFTHS

109F9
MEROITIC CURSIVE FRACTION FOUR TWELFTHS

109FA
MEROITIC CURSIVE FRACTION FIVE TWELFTHS

109FB
MEROITIC CURSIVE FRACTION SIX TWELFTHS

109FC
MEROITIC CURSIVE FRACTION SEVEN TWELFTHS

109FD
MEROITIC CURSIVE FRACTION EIGHT TWELFTHS

109FE
MEROITIC CURSIVE FRACTION NINE TWELFTHS

109FF
MEROITIC CURSIVE FRACTION TEN TWELFTHS

10A00
KHAROSHTHI LETTER A

10A01
KHAROSHTHI VOWEL SIGN I

10A02
KHAROSHTHI VOWEL SIGN U

10A03
KHAROSHTHI VOWEL SIGN VOCALIC R

10A05
KHAROSHTHI VOWEL SIGN E

10A06
KHAROSHTHI VOWEL SIGN O

10A0C
KHAROSHTHI VOWEL LENGTH MARK

10A0D
KHAROSHTHI SIGN DOUBLE RING BELOW

10A0E
KHAROSHTHI SIGN ANUSVARA

10A0F
KHAROSHTHI SIGN VISARGA

10A10
KHAROSHTHI LETTER KA

10A11
KHAROSHTHI LETTER KHA

10A12
KHAROSHTHI LETTER GA

10A13
KHAROSHTHI LETTER GHA

10A15
KHAROSHTHI LETTER CA

10A16
KHAROSHTHI LETTER CHA

10A17
KHAROSHTHI LETTER JA

10A19
KHAROSHTHI LETTER NYA

10A1A
KHAROSHTHI LETTER TTA

10A1B
KHAROSHTHI LETTER TTHA

10A1C
KHAROSHTHI LETTER DDA

10A1D
KHAROSHTHI LETTER DDHA

10A1E
KHAROSHTHI LETTER NNA

10A1F
KHAROSHTHI LETTER TA

10A20
KHAROSHTHI LETTER THA

10A21
KHAROSHTHI LETTER DA

10A22
KHAROSHTHI LETTER DHA

10A23
KHAROSHTHI LETTER NA

10A24
KHAROSHTHI LETTER PA

10A25
KHAROSHTHI LETTER PHA

10A26
KHAROSHTHI LETTER BA

10A27
KHAROSHTHI LETTER BHA

10A28
KHAROSHTHI LETTER MA

10A29
KHAROSHTHI LETTER YA

10A2A
KHAROSHTHI LETTER RA

10A2B
KHAROSHTHI LETTER LA

10A2C
KHAROSHTHI LETTER VA

10A2D
KHAROSHTHI LETTER SHA

10A2E
KHAROSHTHI LETTER SSA

10A2F
KHAROSHTHI LETTER SA

10A30
KHAROSHTHI LETTER ZA

10A31
KHAROSHTHI LETTER HA

10A32
KHAROSHTHI LETTER KKA

10A33
KHAROSHTHI LETTER TTTHA

10A34
KHAROSHTHI LETTER TTTA

10A35
KHAROSHTHI LETTER VHA

10A38
KHAROSHTHI SIGN BAR ABOVE

10A39
KHAROSHTHI SIGN CAUDA

10A3A
KHAROSHTHI SIGN DOT BELOW

10A3F
KHAROSHTHI VIRAMA

10A40
KHAROSHTHI DIGIT ONE

10A41
KHAROSHTHI DIGIT TWO

10A42
KHAROSHTHI DIGIT THREE

10A43
KHAROSHTHI DIGIT FOUR

10A44
KHAROSHTHI NUMBER TEN

10A45
KHAROSHTHI NUMBER TWENTY

10A46
KHAROSHTHI NUMBER ONE HUNDRED

10A47
KHAROSHTHI NUMBER ONE THOUSAND

10A48
KHAROSHTHI FRACTION ONE HALF

10A50
KHAROSHTHI PUNCTUATION DOT

10A51
KHAROSHTHI PUNCTUATION SMALL CIRCLE

10A52
KHAROSHTHI PUNCTUATION CIRCLE

10A53
KHAROSHTHI PUNCTUATION CRESCENT BAR

10A54
KHAROSHTHI PUNCTUATION MANGALAM

10A55
KHAROSHTHI PUNCTUATION LOTUS

10A56
KHAROSHTHI PUNCTUATION DANDA

10A57
KHAROSHTHI PUNCTUATION DOUBLE DANDA

10A58
KHAROSHTHI PUNCTUATION LINES

10A60
OLD SOUTH ARABIAN LETTER HE

10A61
OLD SOUTH ARABIAN LETTER LAMEDH

10A62
OLD SOUTH ARABIAN LETTER HETH

10A63
OLD SOUTH ARABIAN LETTER MEM

10A64
OLD SOUTH ARABIAN LETTER QOPH

10A65
OLD SOUTH ARABIAN LETTER WAW

10A66
OLD SOUTH ARABIAN LETTER SHIN

10A67
OLD SOUTH ARABIAN LETTER RESH

10A68
OLD SOUTH ARABIAN LETTER BETH

10A69
OLD SOUTH ARABIAN LETTER TAW

10A6A
OLD SOUTH ARABIAN LETTER SAT

10A6B
OLD SOUTH ARABIAN LETTER KAPH

10A6C
OLD SOUTH ARABIAN LETTER NUN

10A6D
OLD SOUTH ARABIAN LETTER KHETH

10A6E
OLD SOUTH ARABIAN LETTER SADHE

10A6F
OLD SOUTH ARABIAN LETTER SAMEKH

10A70
OLD SOUTH ARABIAN LETTER FE

10A71
OLD SOUTH ARABIAN LETTER ALEF

10A72
OLD SOUTH ARABIAN LETTER AYN

10A73
OLD SOUTH ARABIAN LETTER DHADHE

10A74
OLD SOUTH ARABIAN LETTER GIMEL

10A75
OLD SOUTH ARABIAN LETTER DALETH

10A76
OLD SOUTH ARABIAN LETTER GHAYN

10A77
OLD SOUTH ARABIAN LETTER TETH

10A78
OLD SOUTH ARABIAN LETTER ZAYN

10A79
OLD SOUTH ARABIAN LETTER DHALETH

10A7A
OLD SOUTH ARABIAN LETTER YODH

10A7B
OLD SOUTH ARABIAN LETTER THAW

10A7C
OLD SOUTH ARABIAN LETTER THETH

10A7D
OLD SOUTH ARABIAN NUMBER ONE

10A7E
OLD SOUTH ARABIAN NUMBER FIFTY

10A7F
OLD SOUTH ARABIAN NUMERIC INDICATOR

10A80
OLD NORTH ARABIAN LETTER HEH

10A81
OLD NORTH ARABIAN LETTER LAM

10A82
OLD NORTH ARABIAN LETTER HAH

10A83
OLD NORTH ARABIAN LETTER MEEM

10A84
OLD NORTH ARABIAN LETTER QAF

10A85
OLD NORTH ARABIAN LETTER WAW

10A86
OLD NORTH ARABIAN LETTER ES-2

10A87
OLD NORTH ARABIAN LETTER REH

10A88
OLD NORTH ARABIAN LETTER BEH

10A89
OLD NORTH ARABIAN LETTER TEH

10A8A
OLD NORTH ARABIAN LETTER ES-1

10A8B
OLD NORTH ARABIAN LETTER KAF

10A8C
OLD NORTH ARABIAN LETTER NOON

10A8D
OLD NORTH ARABIAN LETTER KHAH

10A8E
OLD NORTH ARABIAN LETTER SAD

10A8F
OLD NORTH ARABIAN LETTER ES-3

10A90
OLD NORTH ARABIAN LETTER FEH

10A91
OLD NORTH ARABIAN LETTER ALEF

10A92
OLD NORTH ARABIAN LETTER AIN

10A93
OLD NORTH ARABIAN LETTER DAD

10A94
OLD NORTH ARABIAN LETTER GEEM

10A95
OLD NORTH ARABIAN LETTER DAL

10A96
OLD NORTH ARABIAN LETTER GHAIN

10A97
OLD NORTH ARABIAN LETTER TAH

10A98
OLD NORTH ARABIAN LETTER ZAIN

10A99
OLD NORTH ARABIAN LETTER THAL

10A9A
OLD NORTH ARABIAN LETTER YEH

10A9B
OLD NORTH ARABIAN LETTER THEH

10A9C
OLD NORTH ARABIAN LETTER ZAH

10A9D
OLD NORTH ARABIAN NUMBER ONE

10A9E
OLD NORTH ARABIAN NUMBER TEN

10A9F
OLD NORTH ARABIAN NUMBER TWENTY

10AC0
MANICHAEAN LETTER ALEPH

10AC1
MANICHAEAN LETTER BETH

10AC2
MANICHAEAN LETTER BHETH

10AC3
MANICHAEAN LETTER GIMEL

10AC4
MANICHAEAN LETTER GHIMEL

10AC5
MANICHAEAN LETTER DALETH

10AC6
MANICHAEAN LETTER HE

10AC7
MANICHAEAN LETTER WAW

10AC8
MANICHAEAN SIGN UD

10AC9
MANICHAEAN LETTER ZAYIN

10ACA
MANICHAEAN LETTER ZHAYIN

10ACB
MANICHAEAN LETTER JAYIN

10ACC
MANICHAEAN LETTER JHAYIN

10ACD
MANICHAEAN LETTER HETH

10ACE
MANICHAEAN LETTER TETH

10ACF
MANICHAEAN LETTER YODH

10AD0
MANICHAEAN LETTER KAPH

10AD1
MANICHAEAN LETTER XAPH

10AD2
MANICHAEAN LETTER KHAPH

10AD3
MANICHAEAN LETTER LAMEDH

10AD4
MANICHAEAN LETTER DHAMEDH

10AD5
MANICHAEAN LETTER THAMEDH

10AD6
MANICHAEAN LETTER MEM

10AD7
MANICHAEAN LETTER NUN

10AD8
MANICHAEAN LETTER SAMEKH

10AD9
MANICHAEAN LETTER AYIN

10ADA
MANICHAEAN LETTER AAYIN

10ADB
MANICHAEAN LETTER PE

10ADC
MANICHAEAN LETTER FE

10ADD
MANICHAEAN LETTER SADHE

10ADE
MANICHAEAN LETTER QOPH

10ADF
MANICHAEAN LETTER XOPH

10AE0
MANICHAEAN LETTER QHOPH

10AE1
MANICHAEAN LETTER RESH

10AE2
MANICHAEAN LETTER SHIN

10AE3
MANICHAEAN LETTER SSHIN

10AE4
MANICHAEAN LETTER TAW

10AE5
MANICHAEAN ABBREVIATION MARK ABOVE

10AE6
MANICHAEAN ABBREVIATION MARK BELOW

10AEB
MANICHAEAN NUMBER ONE

10AEC
MANICHAEAN NUMBER FIVE

10AED
MANICHAEAN NUMBER TEN

10AEE
MANICHAEAN NUMBER TWENTY

10AEF
MANICHAEAN NUMBER ONE HUNDRED

10AF0
MANICHAEAN PUNCTUATION STAR

10AF1
MANICHAEAN PUNCTUATION FLEURON

10AF2
MANICHAEAN PUNCTUATION DOUBLE DOT WITHIN DOT

10AF3
MANICHAEAN PUNCTUATION DOT WITHIN DOT

10AF4
MANICHAEAN PUNCTUATION DOT

10AF5
MANICHAEAN PUNCTUATION TWO DOTS

10AF6
MANICHAEAN PUNCTUATION LINE FILLER

10B00
AVESTAN LETTER A

10B01
AVESTAN LETTER AA

10B02
AVESTAN LETTER AO

10B03
AVESTAN LETTER AAO

10B04
AVESTAN LETTER AN

10B05
AVESTAN LETTER AAN

10B06
AVESTAN LETTER AE

10B07
AVESTAN LETTER AEE

10B08
AVESTAN LETTER E

10B09
AVESTAN LETTER EE

10B0A
AVESTAN LETTER O

10B0B
AVESTAN LETTER OO

10B0C
AVESTAN LETTER I

10B0D
AVESTAN LETTER II

10B0E
AVESTAN LETTER U

10B0F
AVESTAN LETTER UU

10B10
AVESTAN LETTER KE

10B11
AVESTAN LETTER XE

10B12
AVESTAN LETTER XYE

10B13
AVESTAN LETTER XVE

10B14
AVESTAN LETTER GE

10B15
AVESTAN LETTER GGE

10B16
AVESTAN LETTER GHE

10B17
AVESTAN LETTER CE

10B18
AVESTAN LETTER JE

10B19
AVESTAN LETTER TE

10B1A
AVESTAN LETTER THE

10B1B
AVESTAN LETTER DE

10B1C
AVESTAN LETTER DHE

10B1D
AVESTAN LETTER TTE

10B1E
AVESTAN LETTER PE

10B1F
AVESTAN LETTER FE

10B20
AVESTAN LETTER BE

10B21
AVESTAN LETTER BHE

10B22
AVESTAN LETTER NGE

10B23
AVESTAN LETTER NGYE

10B24
AVESTAN LETTER NGVE

10B25
AVESTAN LETTER NE

10B26
AVESTAN LETTER NYE

10B27
AVESTAN LETTER NNE

10B28
AVESTAN LETTER ME

10B29
AVESTAN LETTER HME

10B2A
AVESTAN LETTER YYE

10B2B
AVESTAN LETTER YE

10B2C
AVESTAN LETTER VE

10B2D
AVESTAN LETTER RE

10B2E
AVESTAN LETTER LE

10B2F
AVESTAN LETTER SE

10B30
AVESTAN LETTER ZE

10B31
AVESTAN LETTER SHE

10B32
AVESTAN LETTER ZHE

10B33
AVESTAN LETTER SHYE

10B34
AVESTAN LETTER SSHE

10B35
AVESTAN LETTER HE

10B39
AVESTAN ABBREVIATION MARK

10B3A
TINY TWO DOTS OVER ONE DOT PUNCTUATION

10B3B
SMALL TWO DOTS OVER ONE DOT PUNCTUATION

10B3C
LARGE TWO DOTS OVER ONE DOT PUNCTUATION

10B3D
LARGE ONE DOT OVER TWO DOTS PUNCTUATION

10B3E
LARGE TWO RINGS OVER ONE RING PUNCTUATION

10B3F
LARGE ONE RING OVER TWO RINGS PUNCTUATION

10B40
INSCRIPTIONAL PARTHIAN LETTER ALEPH

10B41
INSCRIPTIONAL PARTHIAN LETTER BETH

10B42
INSCRIPTIONAL PARTHIAN LETTER GIMEL

10B43
INSCRIPTIONAL PARTHIAN LETTER DALETH

10B44
INSCRIPTIONAL PARTHIAN LETTER HE

10B45
INSCRIPTIONAL PARTHIAN LETTER WAW

10B46
INSCRIPTIONAL PARTHIAN LETTER ZAYIN

10B47
INSCRIPTIONAL PARTHIAN LETTER HETH

10B48
INSCRIPTIONAL PARTHIAN LETTER TETH

10B49
INSCRIPTIONAL PARTHIAN LETTER YODH

10B4A
INSCRIPTIONAL PARTHIAN LETTER KAPH

10B4B
INSCRIPTIONAL PARTHIAN LETTER LAMEDH

10B4C
INSCRIPTIONAL PARTHIAN LETTER MEM

10B4D
INSCRIPTIONAL PARTHIAN LETTER NUN

10B4E
INSCRIPTIONAL PARTHIAN LETTER SAMEKH

10B4F
INSCRIPTIONAL PARTHIAN LETTER AYIN

10B50
INSCRIPTIONAL PARTHIAN LETTER PE

10B51
INSCRIPTIONAL PARTHIAN LETTER SADHE

10B52
INSCRIPTIONAL PARTHIAN LETTER QOPH

10B53
INSCRIPTIONAL PARTHIAN LETTER RESH

10B54
INSCRIPTIONAL PARTHIAN LETTER SHIN

10B55
INSCRIPTIONAL PARTHIAN LETTER TAW

10B58
INSCRIPTIONAL PARTHIAN NUMBER ONE

10B59
INSCRIPTIONAL PARTHIAN NUMBER TWO

10B5A
INSCRIPTIONAL PARTHIAN NUMBER THREE

10B5B
INSCRIPTIONAL PARTHIAN NUMBER FOUR

10B5C
INSCRIPTIONAL PARTHIAN NUMBER TEN

10B5D
INSCRIPTIONAL PARTHIAN NUMBER TWENTY

10B5E
INSCRIPTIONAL PARTHIAN NUMBER ONE HUNDRED

10B5F
INSCRIPTIONAL PARTHIAN NUMBER ONE THOUSAND

10B60
INSCRIPTIONAL PAHLAVI LETTER ALEPH

10B61
INSCRIPTIONAL PAHLAVI LETTER BETH

10B62
INSCRIPTIONAL PAHLAVI LETTER GIMEL

10B63
INSCRIPTIONAL PAHLAVI LETTER DALETH

10B64
INSCRIPTIONAL PAHLAVI LETTER HE

10B65
INSCRIPTIONAL PAHLAVI LETTER WAW-AYIN-RESH

10B66
INSCRIPTIONAL PAHLAVI LETTER ZAYIN

10B67
INSCRIPTIONAL PAHLAVI LETTER HETH

10B68
INSCRIPTIONAL PAHLAVI LETTER TETH

10B69
INSCRIPTIONAL PAHLAVI LETTER YODH

10B6A
INSCRIPTIONAL PAHLAVI LETTER KAPH

10B6B
INSCRIPTIONAL PAHLAVI LETTER LAMEDH

10B6C
INSCRIPTIONAL PAHLAVI LETTER MEM-QOPH

10B6D
INSCRIPTIONAL PAHLAVI LETTER NUN

10B6E
INSCRIPTIONAL PAHLAVI LETTER SAMEKH

10B6F
INSCRIPTIONAL PAHLAVI LETTER PE

10B70
INSCRIPTIONAL PAHLAVI LETTER SADHE

10B71
INSCRIPTIONAL PAHLAVI LETTER SHIN

10B72
INSCRIPTIONAL PAHLAVI LETTER TAW

10B78
INSCRIPTIONAL PAHLAVI NUMBER ONE

10B79
INSCRIPTIONAL PAHLAVI NUMBER TWO

10B7A
INSCRIPTIONAL PAHLAVI NUMBER THREE

10B7B
INSCRIPTIONAL PAHLAVI NUMBER FOUR

10B7C
INSCRIPTIONAL PAHLAVI NUMBER TEN

10B7D
INSCRIPTIONAL PAHLAVI NUMBER TWENTY

10B7E
INSCRIPTIONAL PAHLAVI NUMBER ONE HUNDRED

10B7F
INSCRIPTIONAL PAHLAVI NUMBER ONE THOUSAND

10B80
PSALTER PAHLAVI LETTER ALEPH

10B81
PSALTER PAHLAVI LETTER BETH

10B82
PSALTER PAHLAVI LETTER GIMEL

10B83
PSALTER PAHLAVI LETTER DALETH

10B84
PSALTER PAHLAVI LETTER HE

10B85
PSALTER PAHLAVI LETTER WAW-AYIN-RESH

10B86
PSALTER PAHLAVI LETTER ZAYIN

10B87
PSALTER PAHLAVI LETTER HETH

10B88
PSALTER PAHLAVI LETTER YODH

10B89
PSALTER PAHLAVI LETTER KAPH

10B8A
PSALTER PAHLAVI LETTER LAMEDH

10B8B
PSALTER PAHLAVI LETTER MEM-QOPH

10B8C
PSALTER PAHLAVI LETTER NUN

10B8D
PSALTER PAHLAVI LETTER SAMEKH

10B8E
PSALTER PAHLAVI LETTER PE

10B8F
PSALTER PAHLAVI LETTER SADHE

10B90
PSALTER PAHLAVI LETTER SHIN

10B91
PSALTER PAHLAVI LETTER TAW

10B99
PSALTER PAHLAVI SECTION MARK

10B9A
PSALTER PAHLAVI TURNED SECTION MARK

10B9B
PSALTER PAHLAVI FOUR DOTS WITH CROSS

10B9C
PSALTER PAHLAVI FOUR DOTS WITH DOT

10BA9
PSALTER PAHLAVI NUMBER ONE

10BAA
PSALTER PAHLAVI NUMBER TWO

10BAB
PSALTER PAHLAVI NUMBER THREE

10BAC
PSALTER PAHLAVI NUMBER FOUR

10BAD
PSALTER PAHLAVI NUMBER TEN

10BAE
PSALTER PAHLAVI NUMBER TWENTY

10BAF
PSALTER PAHLAVI NUMBER ONE HUNDRED

10C00
OLD TURKIC LETTER ORKHON A

10C01
OLD TURKIC LETTER YENISEI A

10C02
OLD TURKIC LETTER YENISEI AE

10C03
OLD TURKIC LETTER ORKHON I

10C04
OLD TURKIC LETTER YENISEI I

10C05
OLD TURKIC LETTER YENISEI E

10C06
OLD TURKIC LETTER ORKHON O

10C07
OLD TURKIC LETTER ORKHON OE

10C08
OLD TURKIC LETTER YENISEI OE

10C09
OLD TURKIC LETTER ORKHON AB

10C0A
OLD TURKIC LETTER YENISEI AB

10C0B
OLD TURKIC LETTER ORKHON AEB

10C0C
OLD TURKIC LETTER YENISEI AEB

10C0D
OLD TURKIC LETTER ORKHON AG

10C0E
OLD TURKIC LETTER YENISEI AG

10C0F
OLD TURKIC LETTER ORKHON AEG

10C10
OLD TURKIC LETTER YENISEI AEG

10C11
OLD TURKIC LETTER ORKHON AD

10C12
OLD TURKIC LETTER YENISEI AD

10C13
OLD TURKIC LETTER ORKHON AED

10C14
OLD TURKIC LETTER ORKHON EZ

10C15
OLD TURKIC LETTER YENISEI EZ

10C16
OLD TURKIC LETTER ORKHON AY

10C17
OLD TURKIC LETTER YENISEI AY

10C18
OLD TURKIC LETTER ORKHON AEY

10C19
OLD TURKIC LETTER YENISEI AEY

10C1A
OLD TURKIC LETTER ORKHON AEK

10C1B
OLD TURKIC LETTER YENISEI AEK

10C1C
OLD TURKIC LETTER ORKHON OEK

10C1D
OLD TURKIC LETTER YENISEI OEK

10C1E
OLD TURKIC LETTER ORKHON AL

10C1F
OLD TURKIC LETTER YENISEI AL

10C20
OLD TURKIC LETTER ORKHON AEL

10C21
OLD TURKIC LETTER ORKHON ELT

10C22
OLD TURKIC LETTER ORKHON EM

10C23
OLD TURKIC LETTER ORKHON AN

10C24
OLD TURKIC LETTER ORKHON AEN

10C25
OLD TURKIC LETTER YENISEI AEN

10C26
OLD TURKIC LETTER ORKHON ENT

10C27
OLD TURKIC LETTER YENISEI ENT

10C28
OLD TURKIC LETTER ORKHON ENC

10C29
OLD TURKIC LETTER YENISEI ENC

10C2A
OLD TURKIC LETTER ORKHON ENY

10C2B
OLD TURKIC LETTER YENISEI ENY

10C2C
OLD TURKIC LETTER YENISEI ANG

10C2D
OLD TURKIC LETTER ORKHON ENG

10C2E
OLD TURKIC LETTER YENISEI AENG

10C2F
OLD TURKIC LETTER ORKHON EP

10C30
OLD TURKIC LETTER ORKHON OP

10C31
OLD TURKIC LETTER ORKHON IC

10C32
OLD TURKIC LETTER ORKHON EC

10C33
OLD TURKIC LETTER YENISEI EC

10C34
OLD TURKIC LETTER ORKHON AQ

10C35
OLD TURKIC LETTER YENISEI AQ

10C36
OLD TURKIC LETTER ORKHON IQ

10C37
OLD TURKIC LETTER YENISEI IQ

10C38
OLD TURKIC LETTER ORKHON OQ

10C39
OLD TURKIC LETTER YENISEI OQ

10C3A
OLD TURKIC LETTER ORKHON AR

10C3B
OLD TURKIC LETTER YENISEI AR

10C3C
OLD TURKIC LETTER ORKHON AER

10C3D
OLD TURKIC LETTER ORKHON AS

10C3E
OLD TURKIC LETTER ORKHON AES

10C3F
OLD TURKIC LETTER ORKHON ASH

10C40
OLD TURKIC LETTER YENISEI ASH

10C41
OLD TURKIC LETTER ORKHON ESH

10C42
OLD TURKIC LETTER YENISEI ESH

10C43
OLD TURKIC LETTER ORKHON AT

10C44
OLD TURKIC LETTER YENISEI AT

10C45
OLD TURKIC LETTER ORKHON AET

10C46
OLD TURKIC LETTER YENISEI AET

10C47
OLD TURKIC LETTER ORKHON OT

10C48
OLD TURKIC LETTER ORKHON BASH

10C80
OLD HUNGARIAN CAPITAL LETTER A

10C81
OLD HUNGARIAN CAPITAL LETTER AA

10C82
OLD HUNGARIAN CAPITAL LETTER EB

10C83
OLD HUNGARIAN CAPITAL LETTER AMB

10C84
OLD HUNGARIAN CAPITAL LETTER EC

10C85
OLD HUNGARIAN CAPITAL LETTER ENC

10C86
OLD HUNGARIAN CAPITAL LETTER ECS

10C87
OLD HUNGARIAN CAPITAL LETTER ED

10C88
OLD HUNGARIAN CAPITAL LETTER AND

10C89
OLD HUNGARIAN CAPITAL LETTER E

10C8A
OLD HUNGARIAN CAPITAL LETTER CLOSE E

10C8B
OLD HUNGARIAN CAPITAL LETTER EE

10C8C
OLD HUNGARIAN CAPITAL LETTER EF

10C8D
OLD HUNGARIAN CAPITAL LETTER EG

10C8E
OLD HUNGARIAN CAPITAL LETTER EGY

10C8F
OLD HUNGARIAN CAPITAL LETTER EH

10C90
OLD HUNGARIAN CAPITAL LETTER I

10C91
OLD HUNGARIAN CAPITAL LETTER II

10C92
OLD HUNGARIAN CAPITAL LETTER EJ

10C93
OLD HUNGARIAN CAPITAL LETTER EK

10C94
OLD HUNGARIAN CAPITAL LETTER AK

10C95
OLD HUNGARIAN CAPITAL LETTER UNK

10C96
OLD HUNGARIAN CAPITAL LETTER EL

10C97
OLD HUNGARIAN CAPITAL LETTER ELY

10C98
OLD HUNGARIAN CAPITAL LETTER EM

10C99
OLD HUNGARIAN CAPITAL LETTER EN

10C9A
OLD HUNGARIAN CAPITAL LETTER ENY

10C9B
OLD HUNGARIAN CAPITAL LETTER O

10C9C
OLD HUNGARIAN CAPITAL LETTER OO

10C9D
OLD HUNGARIAN CAPITAL LETTER NIKOLSBURG OE

10C9E
OLD HUNGARIAN CAPITAL LETTER RUDIMENTA OE

10C9F
OLD HUNGARIAN CAPITAL LETTER OEE

10CA0
OLD HUNGARIAN CAPITAL LETTER EP

10CA1
OLD HUNGARIAN CAPITAL LETTER EMP

10CA2
OLD HUNGARIAN CAPITAL LETTER ER

10CA3
OLD HUNGARIAN CAPITAL LETTER SHORT ER

10CA4
OLD HUNGARIAN CAPITAL LETTER ES

10CA5
OLD HUNGARIAN CAPITAL LETTER ESZ

10CA6
OLD HUNGARIAN CAPITAL LETTER ET

10CA7
OLD HUNGARIAN CAPITAL LETTER ENT

10CA8
OLD HUNGARIAN CAPITAL LETTER ETY

10CA9
OLD HUNGARIAN CAPITAL LETTER ECH

10CAA
OLD HUNGARIAN CAPITAL LETTER U

10CAB
OLD HUNGARIAN CAPITAL LETTER UU

10CAC
OLD HUNGARIAN CAPITAL LETTER NIKOLSBURG UE

10CAD
OLD HUNGARIAN CAPITAL LETTER RUDIMENTA UE

10CAE
OLD HUNGARIAN CAPITAL LETTER EV

10CAF
OLD HUNGARIAN CAPITAL LETTER EZ

10CB0
OLD HUNGARIAN CAPITAL LETTER EZS

10CB1
OLD HUNGARIAN CAPITAL LETTER ENT-SHAPED SIGN

10CB2
OLD HUNGARIAN CAPITAL LETTER US

10CC0
OLD HUNGARIAN SMALL LETTER A

10CC1
OLD HUNGARIAN SMALL LETTER AA

10CC2
OLD HUNGARIAN SMALL LETTER EB

10CC3
OLD HUNGARIAN SMALL LETTER AMB

10CC4
OLD HUNGARIAN SMALL LETTER EC

10CC5
OLD HUNGARIAN SMALL LETTER ENC

10CC6
OLD HUNGARIAN SMALL LETTER ECS

10CC7
OLD HUNGARIAN SMALL LETTER ED

10CC8
OLD HUNGARIAN SMALL LETTER AND

10CC9
OLD HUNGARIAN SMALL LETTER E

10CCA
OLD HUNGARIAN SMALL LETTER CLOSE E

10CCB
OLD HUNGARIAN SMALL LETTER EE

10CCC
OLD HUNGARIAN SMALL LETTER EF

10CCD
OLD HUNGARIAN SMALL LETTER EG

10CCE
OLD HUNGARIAN SMALL LETTER EGY

10CCF
OLD HUNGARIAN SMALL LETTER EH

10CD0
OLD HUNGARIAN SMALL LETTER I

10CD1
OLD HUNGARIAN SMALL LETTER II

10CD2
OLD HUNGARIAN SMALL LETTER EJ

10CD3
OLD HUNGARIAN SMALL LETTER EK

10CD4
OLD HUNGARIAN SMALL LETTER AK

10CD5
OLD HUNGARIAN SMALL LETTER UNK

10CD6
OLD HUNGARIAN SMALL LETTER EL

10CD7
OLD HUNGARIAN SMALL LETTER ELY

10CD8
OLD HUNGARIAN SMALL LETTER EM

10CD9
OLD HUNGARIAN SMALL LETTER EN

10CDA
OLD HUNGARIAN SMALL LETTER ENY

10CDB
OLD HUNGARIAN SMALL LETTER O

10CDC
OLD HUNGARIAN SMALL LETTER OO

10CDD
OLD HUNGARIAN SMALL LETTER NIKOLSBURG OE

10CDE
OLD HUNGARIAN SMALL LETTER RUDIMENTA OE

10CDF
OLD HUNGARIAN SMALL LETTER OEE

10CE0
OLD HUNGARIAN SMALL LETTER EP

10CE1
OLD HUNGARIAN SMALL LETTER EMP

10CE2
OLD HUNGARIAN SMALL LETTER ER

10CE3
OLD HUNGARIAN SMALL LETTER SHORT ER

10CE4
OLD HUNGARIAN SMALL LETTER ES

10CE5
OLD HUNGARIAN SMALL LETTER ESZ

10CE6
OLD HUNGARIAN SMALL LETTER ET

10CE7
OLD HUNGARIAN SMALL LETTER ENT

10CE8
OLD HUNGARIAN SMALL LETTER ETY

10CE9
OLD HUNGARIAN SMALL LETTER ECH

10CEA
OLD HUNGARIAN SMALL LETTER U

10CEB
OLD HUNGARIAN SMALL LETTER UU

10CEC
OLD HUNGARIAN SMALL LETTER NIKOLSBURG UE

10CED
OLD HUNGARIAN SMALL LETTER RUDIMENTA UE

10CEE
OLD HUNGARIAN SMALL LETTER EV

10CEF
OLD HUNGARIAN SMALL LETTER EZ

10CF0
OLD HUNGARIAN SMALL LETTER EZS

10CF1
OLD HUNGARIAN SMALL LETTER ENT-SHAPED SIGN

10CF2
OLD HUNGARIAN SMALL LETTER US

10CFA
OLD HUNGARIAN NUMBER ONE

10CFB
OLD HUNGARIAN NUMBER FIVE

10CFC
OLD HUNGARIAN NUMBER TEN

10CFD
OLD HUNGARIAN NUMBER FIFTY

10CFE
OLD HUNGARIAN NUMBER ONE HUNDRED

10CFF
OLD HUNGARIAN NUMBER ONE THOUSAND

10D00
HANIFI ROHINGYA LETTER A

10D01
HANIFI ROHINGYA LETTER BA

10D02
HANIFI ROHINGYA LETTER PA

10D03
HANIFI ROHINGYA LETTER TA

10D04
HANIFI ROHINGYA LETTER TTA

10D05
HANIFI ROHINGYA LETTER JA

10D06
HANIFI ROHINGYA LETTER CA

10D07
HANIFI ROHINGYA LETTER HA

10D08
HANIFI ROHINGYA LETTER KHA

10D09
HANIFI ROHINGYA LETTER FA

10D0A
HANIFI ROHINGYA LETTER DA

10D0B
HANIFI ROHINGYA LETTER DDA

10D0C
HANIFI ROHINGYA LETTER RA

10D0D
HANIFI ROHINGYA LETTER RRA

10D0E
HANIFI ROHINGYA LETTER ZA

10D0F
HANIFI ROHINGYA LETTER SA

10D10
HANIFI ROHINGYA LETTER SHA

10D11
HANIFI ROHINGYA LETTER KA

10D12
HANIFI ROHINGYA LETTER GA

10D13
HANIFI ROHINGYA LETTER LA

10D14
HANIFI ROHINGYA LETTER MA

10D15
HANIFI ROHINGYA LETTER NA

10D16
HANIFI ROHINGYA LETTER WA

10D17
HANIFI ROHINGYA LETTER KINNA WA

10D18
HANIFI ROHINGYA LETTER YA

10D19
HANIFI ROHINGYA LETTER KINNA YA

10D1A
HANIFI ROHINGYA LETTER NGA

10D1B
HANIFI ROHINGYA LETTER NYA

10D1C
HANIFI ROHINGYA LETTER VA

10D1D
HANIFI ROHINGYA VOWEL A

10D1E
HANIFI ROHINGYA VOWEL I

10D1F
HANIFI ROHINGYA VOWEL U

10D20
HANIFI ROHINGYA VOWEL E

10D21
HANIFI ROHINGYA VOWEL O

10D22
HANIFI ROHINGYA MARK SAKIN

10D23
HANIFI ROHINGYA MARK NA KHONNA

10D24
HANIFI ROHINGYA SIGN HARBAHAY

10D25
HANIFI ROHINGYA SIGN TAHALA

10D26
HANIFI ROHINGYA SIGN TANA

10D27
HANIFI ROHINGYA SIGN TASSI

10D30
HANIFI ROHINGYA DIGIT ZERO

10D31
HANIFI ROHINGYA DIGIT ONE

10D32
HANIFI ROHINGYA DIGIT TWO

10D33
HANIFI ROHINGYA DIGIT THREE

10D34
HANIFI ROHINGYA DIGIT FOUR

10D35
HANIFI ROHINGYA DIGIT FIVE

10D36
HANIFI ROHINGYA DIGIT SIX

10D37
HANIFI ROHINGYA DIGIT SEVEN

10D38
HANIFI ROHINGYA DIGIT EIGHT

10D39
HANIFI ROHINGYA DIGIT NINE

10D40
GARAY DIGIT ZERO

10D41
GARAY DIGIT ONE

10D42
GARAY DIGIT TWO

10D43
GARAY DIGIT THREE

10D44
GARAY DIGIT FOUR

10D45
GARAY DIGIT FIVE

10D46
GARAY DIGIT SIX

10D47
GARAY DIGIT SEVEN

10D48
GARAY DIGIT EIGHT

10D49
GARAY DIGIT NINE

10D4A
GARAY VOWEL SIGN A

10D4B
GARAY VOWEL SIGN I

10D4C
GARAY VOWEL SIGN O

10D4D
GARAY VOWEL SIGN EE

10D4E
GARAY VOWEL LENGTH MARK

10D4F
GARAY SUKUN

10D50
GARAY CAPITAL LETTER A

10D51
GARAY CAPITAL LETTER CA

10D52
GARAY CAPITAL LETTER MA

10D53
GARAY CAPITAL LETTER KA

10D54
GARAY CAPITAL LETTER BA

10D55
GARAY CAPITAL LETTER JA

10D56
GARAY CAPITAL LETTER SA

10D57
GARAY CAPITAL LETTER WA

10D58
GARAY CAPITAL LETTER LA

10D59
GARAY CAPITAL LETTER GA

10D5A
GARAY CAPITAL LETTER DA

10D5B
GARAY CAPITAL LETTER XA

10D5C
GARAY CAPITAL LETTER YA

10D5D
GARAY CAPITAL LETTER TA

10D5E
GARAY CAPITAL LETTER RA

10D5F
GARAY CAPITAL LETTER NYA

10D60
GARAY CAPITAL LETTER FA

10D61
GARAY CAPITAL LETTER NA

10D62
GARAY CAPITAL LETTER PA

10D63
GARAY CAPITAL LETTER HA

10D64
GARAY CAPITAL LETTER OLD KA

10D65
GARAY CAPITAL LETTER OLD NA

10D69
GARAY VOWEL SIGN E

10D6A
GARAY CONSONANT GEMINATION MARK

10D6B
GARAY COMBINING DOT ABOVE

10D6C
GARAY COMBINING DOUBLE DOT ABOVE

10D6D
GARAY CONSONANT NASALIZATION MARK

10D6E
GARAY HYPHEN

10D6F
GARAY REDUPLICATION MARK

10D70
GARAY SMALL LETTER A

10D71
GARAY SMALL LETTER CA

10D72
GARAY SMALL LETTER MA

10D73
GARAY SMALL LETTER KA

10D74
GARAY SMALL LETTER BA

10D75
GARAY SMALL LETTER JA

10D76
GARAY SMALL LETTER SA

10D77
GARAY SMALL LETTER WA

10D78
GARAY SMALL LETTER LA

10D79
GARAY SMALL LETTER GA

10D7A
GARAY SMALL LETTER DA

10D7B
GARAY SMALL LETTER XA

10D7C
GARAY SMALL LETTER YA

10D7D
GARAY SMALL LETTER TA

10D7E
GARAY SMALL LETTER RA

10D7F
GARAY SMALL LETTER NYA

10D80
GARAY SMALL LETTER FA

10D81
GARAY SMALL LETTER NA

10D82
GARAY SMALL LETTER PA

10D83
GARAY SMALL LETTER HA

10D84
GARAY SMALL LETTER OLD KA

10D85
GARAY SMALL LETTER OLD NA

10D8E
GARAY PLUS SIGN

10D8F
GARAY MINUS SIGN

10E60
RUMI DIGIT ONE

10E61
RUMI DIGIT TWO

10E62
RUMI DIGIT THREE

10E63
RUMI DIGIT FOUR

10E64
RUMI DIGIT FIVE

10E65
RUMI DIGIT SIX

10E66
RUMI DIGIT SEVEN

10E67
RUMI DIGIT EIGHT

10E68
RUMI DIGIT NINE

10E69
RUMI NUMBER TEN

10E6A
RUMI NUMBER TWENTY

10E6B
RUMI NUMBER THIRTY

10E6C
RUMI NUMBER FORTY

10E6D
RUMI NUMBER FIFTY

10E6E
RUMI NUMBER SIXTY

10E6F
RUMI NUMBER SEVENTY

10E70
RUMI NUMBER EIGHTY

10E71
RUMI NUMBER NINETY

10E72
RUMI NUMBER ONE HUNDRED

10E73
RUMI NUMBER TWO HUNDRED

10E74
RUMI NUMBER THREE HUNDRED

10E75
RUMI NUMBER FOUR HUNDRED

10E76
RUMI NUMBER FIVE HUNDRED

10E77
RUMI NUMBER SIX HUNDRED

10E78
RUMI NUMBER SEVEN HUNDRED

10E79
RUMI NUMBER EIGHT HUNDRED

10E7A
RUMI NUMBER NINE HUNDRED

10E7B
RUMI FRACTION ONE HALF

10E7C
RUMI FRACTION ONE QUARTER

10E7D
RUMI FRACTION ONE THIRD

10E7E
RUMI FRACTION TWO THIRDS

10E80
YEZIDI LETTER ELIF

10E81
YEZIDI LETTER BE

10E82
YEZIDI LETTER PE

10E83
YEZIDI LETTER PHE

10E84
YEZIDI LETTER THE

10E85
YEZIDI LETTER SE

10E86
YEZIDI LETTER CIM

10E87
YEZIDI LETTER CHIM

10E88
YEZIDI LETTER CHHIM

10E89
YEZIDI LETTER HHA

10E8A
YEZIDI LETTER XA

10E8B
YEZIDI LETTER DAL

10E8C
YEZIDI LETTER ZAL

10E8D
YEZIDI LETTER RA

10E8E
YEZIDI LETTER RHA

10E8F
YEZIDI LETTER ZA

10E90
YEZIDI LETTER JA

10E91
YEZIDI LETTER SIN

10E92
YEZIDI LETTER SHIN

10E93
YEZIDI LETTER SAD

10E94
YEZIDI LETTER DAD

10E95
YEZIDI LETTER TA

10E96
YEZIDI LETTER ZE

10E97
YEZIDI LETTER EYN

10E98
YEZIDI LETTER XHEYN

10E99
YEZIDI LETTER FA

10E9A
YEZIDI LETTER VA

10E9B
YEZIDI LETTER VA ALTERNATE FORM

10E9C
YEZIDI LETTER QAF

10E9D
YEZIDI LETTER KAF

10E9E
YEZIDI LETTER KHAF

10E9F
YEZIDI LETTER GAF

10EA0
YEZIDI LETTER LAM

10EA1
YEZIDI LETTER MIM

10EA2
YEZIDI LETTER NUN

10EA3
YEZIDI LETTER UM

10EA4
YEZIDI LETTER WAW

10EA5
YEZIDI LETTER OW

10EA6
YEZIDI LETTER EW

10EA7
YEZIDI LETTER HAY

10EA8
YEZIDI LETTER YOT

10EA9
YEZIDI LETTER ET

10EAB
YEZIDI COMBINING HAMZA MARK

10EAC
YEZIDI COMBINING MADDA MARK

10EAD
YEZIDI HYPHENATION MARK

10EB0
YEZIDI LETTER LAM WITH DOT ABOVE

10EB1
YEZIDI LETTER YOT WITH CIRCUMFLEX ABOVE

10EC2
ARABIC LETTER DAL WITH TWO DOTS VERTICALLY BELOW

10EC3
ARABIC LETTER TAH WITH TWO DOTS VERTICALLY BELOW

10EC4
ARABIC LETTER KAF WITH TWO DOTS VERTICALLY BELOW

10EC5
ARABIC SMALL YEH BARREE WITH TWO DOTS BELOW

10EC6
ARABIC LETTER THIN NOON

10EC7
ARABIC LETTER YEH WITH FOUR DOTS BELOW

10ED0
ARABIC BIBLICAL END OF VERSE

10ED1
ARABIC LIGATURE ALAYHAA AS-SALAATU WAS-SALAAM

10ED2
ARABIC LIGATURE ALAYHIM AS-SALAATU WAS-SALAAM

10ED3
ARABIC LIGATURE ALAYHIMAA AS-SALAATU WAS-SALAAM

10ED4
ARABIC LIGATURE QADDASA ALLAAHU SIRRAH

10ED5
ARABIC LIGATURE QUDDISA SIRRUHUM

10ED6
ARABIC LIGATURE QUDDISA SIRRUHUMAA

10ED7
ARABIC LIGATURE QUDDISAT ASRAARUHUM

10ED8
ARABIC LIGATURE NAWWARA ALLAAHU MARQADAH

10EFA
ARABIC DOUBLE VERTICAL BAR BELOW

10EFB
ARABIC SMALL LOW NOON

10EFC
ARABIC COMBINING ALEF OVERLAY

10EFD
ARABIC SMALL LOW WORD SAKTA

10EFE
ARABIC SMALL LOW WORD QASR

10EFF
ARABIC SMALL LOW WORD MADDA

10F00
OLD SOGDIAN LETTER ALEPH

10F01
OLD SOGDIAN LETTER FINAL ALEPH

10F02
OLD SOGDIAN LETTER BETH

10F03
OLD SOGDIAN LETTER FINAL BETH

10F04
OLD SOGDIAN LETTER GIMEL

10F05
OLD SOGDIAN LETTER HE

10F06
OLD SOGDIAN LETTER FINAL HE

10F07
OLD SOGDIAN LETTER WAW

10F08
OLD SOGDIAN LETTER ZAYIN

10F09
OLD SOGDIAN LETTER HETH

10F0A
OLD SOGDIAN LETTER YODH

10F0B
OLD SOGDIAN LETTER KAPH

10F0C
OLD SOGDIAN LETTER LAMEDH

10F0D
OLD SOGDIAN LETTER MEM

10F0E
OLD SOGDIAN LETTER NUN

10F0F
OLD SOGDIAN LETTER FINAL NUN

10F10
OLD SOGDIAN LETTER FINAL NUN WITH VERTICAL TAIL

10F11
OLD SOGDIAN LETTER SAMEKH

10F12
OLD SOGDIAN LETTER AYIN

10F13
OLD SOGDIAN LETTER ALTERNATE AYIN

10F14
OLD SOGDIAN LETTER PE

10F15
OLD SOGDIAN LETTER SADHE

10F16
OLD SOGDIAN LETTER FINAL SADHE

10F17
OLD SOGDIAN LETTER FINAL SADHE WITH VERTICAL TAIL

10F18
OLD SOGDIAN LETTER RESH-AYIN-DALETH

10F19
OLD SOGDIAN LETTER SHIN

10F1A
OLD SOGDIAN LETTER TAW

10F1B
OLD SOGDIAN LETTER FINAL TAW

10F1C
OLD SOGDIAN LETTER FINAL TAW WITH VERTICAL TAIL

10F1D
OLD SOGDIAN NUMBER ONE

10F1E
OLD SOGDIAN NUMBER TWO

10F1F
OLD SOGDIAN NUMBER THREE

10F20
OLD SOGDIAN NUMBER FOUR

10F21
OLD SOGDIAN NUMBER FIVE

10F22
OLD SOGDIAN NUMBER TEN

10F23
OLD SOGDIAN NUMBER TWENTY

10F24
OLD SOGDIAN NUMBER THIRTY

10F25
OLD SOGDIAN NUMBER ONE HUNDRED

10F26
OLD SOGDIAN FRACTION ONE HALF

10F27
OLD SOGDIAN LIGATURE AYIN-DALETH

10F30
SOGDIAN LETTER ALEPH

10F31
SOGDIAN LETTER BETH

10F32
SOGDIAN LETTER GIMEL

10F33
SOGDIAN LETTER HE

10F34
SOGDIAN LETTER WAW

10F35
SOGDIAN LETTER ZAYIN

10F36
SOGDIAN LETTER HETH

10F37
SOGDIAN LETTER YODH

10F38
SOGDIAN LETTER KAPH

10F39
SOGDIAN LETTER LAMEDH

10F3A
SOGDIAN LETTER MEM

10F3B
SOGDIAN LETTER NUN

10F3C
SOGDIAN LETTER SAMEKH

10F3D
SOGDIAN LETTER AYIN

10F3E
SOGDIAN LETTER PE

10F3F
SOGDIAN LETTER SADHE

10F40
SOGDIAN LETTER RESH-AYIN

10F41
SOGDIAN LETTER SHIN

10F42
SOGDIAN LETTER TAW

10F43
SOGDIAN LETTER FETH

10F44
SOGDIAN LETTER LESH

10F45
SOGDIAN INDEPENDENT SHIN

10F46
SOGDIAN COMBINING DOT BELOW

10F47
SOGDIAN COMBINING TWO DOTS BELOW

10F48
SOGDIAN COMBINING DOT ABOVE

10F49
SOGDIAN COMBINING TWO DOTS ABOVE

10F4A
SOGDIAN COMBINING CURVE ABOVE

10F4B
SOGDIAN COMBINING CURVE BELOW

10F4C
SOGDIAN COMBINING HOOK ABOVE

10F4D
SOGDIAN COMBINING HOOK BELOW

10F4E
SOGDIAN COMBINING LONG HOOK BELOW

10F4F
SOGDIAN COMBINING RESH BELOW

10F50
SOGDIAN COMBINING STROKE BELOW

10F51
SOGDIAN NUMBER ONE

10F52
SOGDIAN NUMBER TEN

10F53
SOGDIAN NUMBER TWENTY

10F54
SOGDIAN NUMBER ONE HUNDRED

10F55
SOGDIAN PUNCTUATION TWO VERTICAL BARS

10F56
SOGDIAN PUNCTUATION TWO VERTICAL BARS WITH DOTS

10F57
SOGDIAN PUNCTUATION CIRCLE WITH DOT

10F58
SOGDIAN PUNCTUATION TWO CIRCLES WITH DOTS

10F59
SOGDIAN PUNCTUATION HALF CIRCLE WITH DOT

10F70
OLD UYGHUR LETTER ALEPH

10F71
OLD UYGHUR LETTER BETH

10F72
OLD UYGHUR LETTER GIMEL-HETH

10F73
OLD UYGHUR LETTER WAW

10F74
OLD UYGHUR LETTER ZAYIN

10F75
OLD UYGHUR LETTER FINAL HETH

10F76
OLD UYGHUR LETTER YODH

10F77
OLD UYGHUR LETTER KAPH

10F78
OLD UYGHUR LETTER LAMEDH

10F79
OLD UYGHUR LETTER MEM

10F7A
OLD UYGHUR LETTER NUN

10F7B
OLD UYGHUR LETTER SAMEKH

10F7C
OLD UYGHUR LETTER PE

10F7D
OLD UYGHUR LETTER SADHE

10F7E
OLD UYGHUR LETTER RESH

10F7F
OLD UYGHUR LETTER SHIN

10F80
OLD UYGHUR LETTER TAW

10F81
OLD UYGHUR LETTER LESH

10F82
OLD UYGHUR COMBINING DOT ABOVE

10F83
OLD UYGHUR COMBINING DOT BELOW

10F84
OLD UYGHUR COMBINING TWO DOTS ABOVE

10F85
OLD UYGHUR COMBINING TWO DOTS BELOW

10F86
OLD UYGHUR PUNCTUATION BAR

10F87
OLD UYGHUR PUNCTUATION TWO BARS

10F88
OLD UYGHUR PUNCTUATION TWO DOTS

10F89
OLD UYGHUR PUNCTUATION FOUR DOTS

10FB0
CHORASMIAN LETTER ALEPH

10FB1
CHORASMIAN LETTER SMALL ALEPH

10FB2
CHORASMIAN LETTER BETH

10FB3
CHORASMIAN LETTER GIMEL

10FB4
CHORASMIAN LETTER DALETH

10FB5
CHORASMIAN LETTER HE

10FB6
CHORASMIAN LETTER WAW

10FB7
CHORASMIAN LETTER CURLED WAW

10FB8
CHORASMIAN LETTER ZAYIN

10FB9
CHORASMIAN LETTER HETH

10FBA
CHORASMIAN LETTER YODH

10FBB
CHORASMIAN LETTER KAPH

10FBC
CHORASMIAN LETTER LAMEDH

10FBD
CHORASMIAN LETTER MEM

10FBE
CHORASMIAN LETTER NUN

10FBF
CHORASMIAN LETTER SAMEKH

10FC0
CHORASMIAN LETTER AYIN

10FC1
CHORASMIAN LETTER PE

10FC2
CHORASMIAN LETTER RESH

10FC3
CHORASMIAN LETTER SHIN

10FC4
CHORASMIAN LETTER TAW

10FC5
CHORASMIAN NUMBER ONE

10FC6
CHORASMIAN NUMBER TWO

10FC7
CHORASMIAN NUMBER THREE

10FC8
CHORASMIAN NUMBER FOUR

10FC9
CHORASMIAN NUMBER TEN

10FCA
CHORASMIAN NUMBER TWENTY

10FCB
CHORASMIAN NUMBER ONE HUNDRED

10FE0
ELYMAIC LETTER ALEPH

10FE1
ELYMAIC LETTER BETH

10FE2
ELYMAIC LETTER GIMEL

10FE3
ELYMAIC LETTER DALETH

10FE4
ELYMAIC LETTER HE

10FE5
ELYMAIC LETTER WAW

10FE6
ELYMAIC LETTER ZAYIN

10FE7
ELYMAIC LETTER HETH

10FE8
ELYMAIC LETTER TETH

10FE9
ELYMAIC LETTER YODH

10FEA
ELYMAIC LETTER KAPH

10FEB
ELYMAIC LETTER LAMEDH

10FEC
ELYMAIC LETTER MEM

10FED
ELYMAIC LETTER NUN

10FEE
ELYMAIC LETTER SAMEKH

10FEF
ELYMAIC LETTER AYIN

10FF0
ELYMAIC LETTER PE

10FF1
ELYMAIC LETTER SADHE

10FF2
ELYMAIC LETTER QOPH

10FF3
ELYMAIC LETTER RESH

10FF4
ELYMAIC LETTER SHIN

10FF5
ELYMAIC LETTER TAW

10FF6
ELYMAIC LIGATURE ZAYIN-YODH

11000
BRAHMI SIGN CANDRABINDU

11001
BRAHMI SIGN ANUSVARA

11002
BRAHMI SIGN VISARGA

11003
BRAHMI SIGN JIHVAMULIYA

11004
BRAHMI SIGN UPADHMANIYA

11005
BRAHMI LETTER A

11006
BRAHMI LETTER AA

11007
BRAHMI LETTER I

11008
BRAHMI LETTER II

11009
BRAHMI LETTER U

1100A
BRAHMI LETTER UU

1100B
BRAHMI LETTER VOCALIC R

1100C
BRAHMI LETTER VOCALIC RR

1100D
BRAHMI LETTER VOCALIC L

1100E
BRAHMI LETTER VOCALIC LL

1100F
BRAHMI LETTER E

11010
BRAHMI LETTER AI

11011
BRAHMI LETTER O

11012
BRAHMI LETTER AU

11013
BRAHMI LETTER KA

11014
BRAHMI LETTER KHA

11015
BRAHMI LETTER GA

11016
BRAHMI LETTER GHA

11017
BRAHMI LETTER NGA

11018
BRAHMI LETTER CA

11019
BRAHMI LETTER CHA

1101A
BRAHMI LETTER JA

1101B
BRAHMI LETTER JHA

1101C
BRAHMI LETTER NYA

1101D
BRAHMI LETTER TTA

1101E
BRAHMI LETTER TTHA

1101F
BRAHMI LETTER DDA

11020
BRAHMI LETTER DDHA

11021
BRAHMI LETTER NNA

11022
BRAHMI LETTER TA

11023
BRAHMI LETTER THA

11024
BRAHMI LETTER DA

11025
BRAHMI LETTER DHA

11026
BRAHMI LETTER NA

11027
BRAHMI LETTER PA

11028
BRAHMI LETTER PHA

11029
BRAHMI LETTER BA

1102A
BRAHMI LETTER BHA

1102B
BRAHMI LETTER MA

1102C
BRAHMI LETTER YA

1102D
BRAHMI LETTER RA

1102E
BRAHMI LETTER LA

1102F
BRAHMI LETTER VA

11030
BRAHMI LETTER SHA

11031
BRAHMI LETTER SSA

11032
BRAHMI LETTER SA

11033
BRAHMI LETTER HA

11034
BRAHMI LETTER LLA

11035
BRAHMI LETTER OLD TAMIL LLLA

11036
BRAHMI LETTER OLD TAMIL RRA

11037
BRAHMI LETTER OLD TAMIL NNNA

11038
BRAHMI VOWEL SIGN AA

11039
BRAHMI VOWEL SIGN BHATTIPROLU AA

1103A
BRAHMI VOWEL SIGN I

1103B
BRAHMI VOWEL SIGN II

1103C
BRAHMI VOWEL SIGN U

1103D
BRAHMI VOWEL SIGN UU

1103E
BRAHMI VOWEL SIGN VOCALIC R

1103F
BRAHMI VOWEL SIGN VOCALIC RR

11040
BRAHMI VOWEL SIGN VOCALIC L

11041
BRAHMI VOWEL SIGN VOCALIC LL

11042
BRAHMI VOWEL SIGN E

11043
BRAHMI VOWEL SIGN AI

11044
BRAHMI VOWEL SIGN O

11045
BRAHMI VOWEL SIGN AU

11046
BRAHMI VIRAMA

11047
BRAHMI DANDA

11048
BRAHMI DOUBLE DANDA

11049
BRAHMI PUNCTUATION DOT

1104A
BRAHMI PUNCTUATION DOUBLE DOT

1104B
BRAHMI PUNCTUATION LINE

1104C
BRAHMI PUNCTUATION CRESCENT BAR

1104D
BRAHMI PUNCTUATION LOTUS

11052
BRAHMI NUMBER ONE

11053
BRAHMI NUMBER TWO

11054
BRAHMI NUMBER THREE

11055
BRAHMI NUMBER FOUR

11056
BRAHMI NUMBER FIVE

11057
BRAHMI NUMBER SIX

11058
BRAHMI NUMBER SEVEN

11059
BRAHMI NUMBER EIGHT

1105A
BRAHMI NUMBER NINE

1105B
BRAHMI NUMBER TEN

1105C
BRAHMI NUMBER TWENTY

1105D
BRAHMI NUMBER THIRTY

1105E
BRAHMI NUMBER FORTY

1105F
BRAHMI NUMBER FIFTY

11060
BRAHMI NUMBER SIXTY

11061
BRAHMI NUMBER SEVENTY

11062
BRAHMI NUMBER EIGHTY

11063
BRAHMI NUMBER NINETY

11064
BRAHMI NUMBER ONE HUNDRED

11065
BRAHMI NUMBER ONE THOUSAND

11066
BRAHMI DIGIT ZERO

11067
BRAHMI DIGIT ONE

11068
BRAHMI DIGIT TWO

11069
BRAHMI DIGIT THREE

1106A
BRAHMI DIGIT FOUR

1106B
BRAHMI DIGIT FIVE

1106C
BRAHMI DIGIT SIX

1106D
BRAHMI DIGIT SEVEN

1106E
BRAHMI DIGIT EIGHT

1106F
BRAHMI DIGIT NINE

11070
BRAHMI SIGN OLD TAMIL VIRAMA

11071
BRAHMI LETTER OLD TAMIL SHORT E

11072
BRAHMI LETTER OLD TAMIL SHORT O

11073
BRAHMI VOWEL SIGN OLD TAMIL SHORT E

11074
BRAHMI VOWEL SIGN OLD TAMIL SHORT O

11075
BRAHMI LETTER OLD TAMIL LLA

1107F
BRAHMI NUMBER JOINER

11080
KAITHI SIGN CANDRABINDU

11081
KAITHI SIGN ANUSVARA

11082
KAITHI SIGN VISARGA

11083
KAITHI LETTER A

11084
KAITHI LETTER AA

11085
KAITHI LETTER I

11086
KAITHI LETTER II

11087
KAITHI LETTER U

11088
KAITHI LETTER UU

11089
KAITHI LETTER E

1108A
KAITHI LETTER AI

1108B
KAITHI LETTER O

1108C
KAITHI LETTER AU

1108D
KAITHI LETTER KA

1108E
KAITHI LETTER KHA

1108F
KAITHI LETTER GA

11090
KAITHI LETTER GHA

11091
KAITHI LETTER NGA

11092
KAITHI LETTER CA

11093
KAITHI LETTER CHA

11094
KAITHI LETTER JA

11095
KAITHI LETTER JHA

11096
KAITHI LETTER NYA

11097
KAITHI LETTER TTA

11098
KAITHI LETTER TTHA

11099
KAITHI LETTER DDA

1109A
KAITHI LETTER DDDHA

1109B
KAITHI LETTER DDHA

1109C
KAITHI LETTER RHA

1109D
KAITHI LETTER NNA

1109E
KAITHI LETTER TA

1109F
KAITHI LETTER THA

110A0
KAITHI LETTER DA

110A1
KAITHI LETTER DHA

110A2
KAITHI LETTER NA

110A3
KAITHI LETTER PA

110A4
KAITHI LETTER PHA

110A5
KAITHI LETTER BA

110A6
KAITHI LETTER BHA

110A7
KAITHI LETTER MA

110A8
KAITHI LETTER YA

110A9
KAITHI LETTER RA

110AA
KAITHI LETTER LA

110AB
KAITHI LETTER VA

110AC
KAITHI LETTER SHA

110AD
KAITHI LETTER SSA

110AE
KAITHI LETTER SA

110AF
KAITHI LETTER HA

110B0
KAITHI VOWEL SIGN AA

110B1
KAITHI VOWEL SIGN I

110B2
KAITHI VOWEL SIGN II

110B3
KAITHI VOWEL SIGN U

110B4
KAITHI VOWEL SIGN UU

110B5
KAITHI VOWEL SIGN E

110B6
KAITHI VOWEL SIGN AI

110B7
KAITHI VOWEL SIGN O

110B8
KAITHI VOWEL SIGN AU

110B9
KAITHI SIGN VIRAMA

110BA
KAITHI SIGN NUKTA

110BB
KAITHI ABBREVIATION SIGN

110BC
KAITHI ENUMERATION SIGN

110BD
KAITHI NUMBER SIGN

110BE
KAITHI SECTION MARK

110BF
KAITHI DOUBLE SECTION MARK

110C0
KAITHI DANDA

110C1
KAITHI DOUBLE DANDA

110C2
KAITHI VOWEL SIGN VOCALIC R

110CD
KAITHI NUMBER SIGN ABOVE

110D0
SORA SOMPENG LETTER SAH

110D1
SORA SOMPENG LETTER TAH

110D2
SORA SOMPENG LETTER BAH

110D3
SORA SOMPENG LETTER CAH

110D4
SORA SOMPENG LETTER DAH

110D5
SORA SOMPENG LETTER GAH

110D6
SORA SOMPENG LETTER MAH

110D7
SORA SOMPENG LETTER NGAH

110D8
SORA SOMPENG LETTER LAH

110D9
SORA SOMPENG LETTER NAH

110DA
SORA SOMPENG LETTER VAH

110DB
SORA SOMPENG LETTER PAH

110DC
SORA SOMPENG LETTER YAH

110DD
SORA SOMPENG LETTER RAH

110DE
SORA SOMPENG LETTER HAH

110DF
SORA SOMPENG LETTER KAH

110E0
SORA SOMPENG LETTER JAH

110E1
SORA SOMPENG LETTER NYAH

110E2
SORA SOMPENG LETTER AH

110E3
SORA SOMPENG LETTER EEH

110E4
SORA SOMPENG LETTER IH

110E5
SORA SOMPENG LETTER UH

110E6
SORA SOMPENG LETTER OH

110E7
SORA SOMPENG LETTER EH

110E8
SORA SOMPENG LETTER MAE

110F0
SORA SOMPENG DIGIT ZERO

110F1
SORA SOMPENG DIGIT ONE

110F2
SORA SOMPENG DIGIT TWO

110F3
SORA SOMPENG DIGIT THREE

110F4
SORA SOMPENG DIGIT FOUR

110F5
SORA SOMPENG DIGIT FIVE

110F6
SORA SOMPENG DIGIT SIX

110F7
SORA SOMPENG DIGIT SEVEN

110F8
SORA SOMPENG DIGIT EIGHT

110F9
SORA SOMPENG DIGIT NINE

11100
CHAKMA SIGN CANDRABINDU

11101
CHAKMA SIGN ANUSVARA

11102
CHAKMA SIGN VISARGA

11103
CHAKMA LETTER AA

11104
CHAKMA LETTER I

11105
CHAKMA LETTER U

11106
CHAKMA LETTER E

11107
CHAKMA LETTER KAA

11108
CHAKMA LETTER KHAA

11109
CHAKMA LETTER GAA

1110A
CHAKMA LETTER GHAA

1110B
CHAKMA LETTER NGAA

1110C
CHAKMA LETTER CAA

1110D
CHAKMA LETTER CHAA

1110E
CHAKMA LETTER JAA

1110F
CHAKMA LETTER JHAA

11110
CHAKMA LETTER NYAA

11111
CHAKMA LETTER TTAA

11112
CHAKMA LETTER TTHAA

11113
CHAKMA LETTER DDAA

11114
CHAKMA LETTER DDHAA

11115
CHAKMA LETTER NNAA

11116
CHAKMA LETTER TAA

11117
CHAKMA LETTER THAA

11118
CHAKMA LETTER DAA

11119
CHAKMA LETTER DHAA

1111A
CHAKMA LETTER NAA

1111B
CHAKMA LETTER PAA

1111C
CHAKMA LETTER PHAA

1111D
CHAKMA LETTER BAA

1111E
CHAKMA LETTER BHAA

1111F
CHAKMA LETTER MAA

11120
CHAKMA LETTER YYAA

11121
CHAKMA LETTER YAA

11122
CHAKMA LETTER RAA

11123
CHAKMA LETTER LAA

11124
CHAKMA LETTER WAA

11125
CHAKMA LETTER SAA

11126
CHAKMA LETTER HAA

11127
CHAKMA VOWEL SIGN A

11128
CHAKMA VOWEL SIGN I

11129
CHAKMA VOWEL SIGN II

1112A
CHAKMA VOWEL SIGN U

1112B
CHAKMA VOWEL SIGN UU

1112C
CHAKMA VOWEL SIGN E

1112D
CHAKMA VOWEL SIGN AI

1112E
CHAKMA VOWEL SIGN O

1112F
CHAKMA VOWEL SIGN AU

11130
CHAKMA VOWEL SIGN OI

11131
CHAKMA O MARK

11132
CHAKMA AU MARK

11133
CHAKMA VIRAMA

11134
CHAKMA MAAYYAA

11136
CHAKMA DIGIT ZERO

11137
CHAKMA DIGIT ONE

11138
CHAKMA DIGIT TWO

11139
CHAKMA DIGIT THREE

1113A
CHAKMA DIGIT FOUR

1113B
CHAKMA DIGIT FIVE

1113C
CHAKMA DIGIT SIX

1113D
CHAKMA DIGIT SEVEN

1113E
CHAKMA DIGIT EIGHT

1113F
CHAKMA DIGIT NINE

11140
CHAKMA SECTION MARK

11141
CHAKMA DANDA

11142
CHAKMA DOUBLE DANDA

11143
CHAKMA QUESTION MARK

11144
CHAKMA LETTER LHAA

11145
CHAKMA VOWEL SIGN AA

11146
CHAKMA VOWEL SIGN EI

11147
CHAKMA LETTER VAA

11150
MAHAJANI LETTER A

11151
MAHAJANI LETTER I

11152
MAHAJANI LETTER U

11153
MAHAJANI LETTER E

11154
MAHAJANI LETTER O

11155
MAHAJANI LETTER KA

11156
MAHAJANI LETTER KHA

11157
MAHAJANI LETTER GA

11158
MAHAJANI LETTER GHA

11159
MAHAJANI LETTER CA

1115A
MAHAJANI LETTER CHA

1115B
MAHAJANI LETTER JA

1115C
MAHAJANI LETTER JHA

1115D
MAHAJANI LETTER NYA

1115E
MAHAJANI LETTER TTA

1115F
MAHAJANI LETTER TTHA

11160
MAHAJANI LETTER DDA

11161
MAHAJANI LETTER DDHA

11162
MAHAJANI LETTER NNA

11163
MAHAJANI LETTER TA

11164
MAHAJANI LETTER THA

11165
MAHAJANI LETTER DA

11166
MAHAJANI LETTER DHA

11167
MAHAJANI LETTER NA

11168
MAHAJANI LETTER PA

11169
MAHAJANI LETTER PHA

1116A
MAHAJANI LETTER BA

1116B
MAHAJANI LETTER BHA

1116C
MAHAJANI LETTER MA

1116D
MAHAJANI LETTER RA

1116E
MAHAJANI LETTER LA

1116F
MAHAJANI LETTER VA

11170
MAHAJANI LETTER SA

11171
MAHAJANI LETTER HA

11172
MAHAJANI LETTER RRA

11173
MAHAJANI SIGN NUKTA

11174
MAHAJANI ABBREVIATION SIGN

11175
MAHAJANI SECTION MARK

11176
MAHAJANI LIGATURE SHRI

11180
SHARADA SIGN CANDRABINDU

11181
SHARADA SIGN ANUSVARA

11182
SHARADA SIGN VISARGA

11183
SHARADA LETTER A

11184
SHARADA LETTER AA

11185
SHARADA LETTER I

11186
SHARADA LETTER II

11187
SHARADA LETTER U

11188
SHARADA LETTER UU

11189
SHARADA LETTER VOCALIC R

1118A
SHARADA LETTER VOCALIC RR

1118B
SHARADA LETTER VOCALIC L

1118C
SHARADA LETTER VOCALIC LL

1118D
SHARADA LETTER E

1118E
SHARADA LETTER AI

1118F
SHARADA LETTER O

11190
SHARADA LETTER AU

11191
SHARADA LETTER KA

11192
SHARADA LETTER KHA

11193
SHARADA LETTER GA

11194
SHARADA LETTER GHA

11195
SHARADA LETTER NGA

11196
SHARADA LETTER CA

11197
SHARADA LETTER CHA

11198
SHARADA LETTER JA

11199
SHARADA LETTER JHA

1119A
SHARADA LETTER NYA

1119B
SHARADA LETTER TTA

1119C
SHARADA LETTER TTHA

1119D
SHARADA LETTER DDA

1119E
SHARADA LETTER DDHA

1119F
SHARADA LETTER NNA

111A0
SHARADA LETTER TA

111A1
SHARADA LETTER THA

111A2
SHARADA LETTER DA

111A3
SHARADA LETTER DHA

111A4
SHARADA LETTER NA

111A5
SHARADA LETTER PA

111A6
SHARADA LETTER PHA

111A7
SHARADA LETTER BA

111A8
SHARADA LETTER BHA

111A9
SHARADA LETTER MA

111AA
SHARADA LETTER YA

111AB
SHARADA LETTER RA

111AC
SHARADA LETTER LA

111AD
SHARADA LETTER LLA

111AE
SHARADA LETTER VA

111AF
SHARADA LETTER SHA

111B0
SHARADA LETTER SSA

111B1
SHARADA LETTER SA

111B2
SHARADA LETTER HA

111B3
SHARADA VOWEL SIGN AA

111B4
SHARADA VOWEL SIGN I

111B5
SHARADA VOWEL SIGN II

111B6
SHARADA VOWEL SIGN U

111B7
SHARADA VOWEL SIGN UU

111B8
SHARADA VOWEL SIGN VOCALIC R

111B9
SHARADA VOWEL SIGN VOCALIC RR

111BA
SHARADA VOWEL SIGN VOCALIC L

111BB
SHARADA VOWEL SIGN VOCALIC LL

111BC
SHARADA VOWEL SIGN E

111BD
SHARADA VOWEL SIGN AI

111BE
SHARADA VOWEL SIGN O

111BF
SHARADA VOWEL SIGN AU

111C0
SHARADA SIGN VIRAMA

111C1
SHARADA SIGN AVAGRAHA

111C2
SHARADA SIGN JIHVAMULIYA

111C3
SHARADA SIGN UPADHMANIYA

111C4
SHARADA OM

111C5
SHARADA DANDA

111C6
SHARADA DOUBLE DANDA

111C7
SHARADA ABBREVIATION SIGN

111C8
SHARADA SEPARATOR

111C9
SHARADA SANDHI MARK

111CA
SHARADA SIGN NUKTA

111CB
SHARADA VOWEL MODIFIER MARK

111CC
SHARADA EXTRA SHORT VOWEL MARK

111CD
SHARADA SUTRA MARK

111CE
SHARADA VOWEL SIGN PRISHTHAMATRA E

111CF
SHARADA SIGN INVERTED CANDRABINDU

111D0
SHARADA DIGIT ZERO

111D1
SHARADA DIGIT ONE

111D2
SHARADA DIGIT TWO

111D3
SHARADA DIGIT THREE

111D4
SHARADA DIGIT FOUR

111D5
SHARADA DIGIT FIVE

111D6
SHARADA DIGIT SIX

111D7
SHARADA DIGIT SEVEN

111D8
SHARADA DIGIT EIGHT

111D9
SHARADA DIGIT NINE

111DA
SHARADA EKAM

111DB
SHARADA SIGN SIDDHAM

111DC
SHARADA HEADSTROKE

111DD
SHARADA CONTINUATION SIGN

111DE
SHARADA SECTION MARK-1

111DF
SHARADA SECTION MARK-2

111E1
SINHALA ARCHAIC DIGIT ONE

111E2
SINHALA ARCHAIC DIGIT TWO

111E3
SINHALA ARCHAIC DIGIT THREE

111E4
SINHALA ARCHAIC DIGIT FOUR

111E5
SINHALA ARCHAIC DIGIT FIVE

111E6
SINHALA ARCHAIC DIGIT SIX

111E7
SINHALA ARCHAIC DIGIT SEVEN

111E8
SINHALA ARCHAIC DIGIT EIGHT

111E9
SINHALA ARCHAIC DIGIT NINE

111EA
SINHALA ARCHAIC NUMBER TEN

111EB
SINHALA ARCHAIC NUMBER TWENTY

111EC
SINHALA ARCHAIC NUMBER THIRTY

111ED
SINHALA ARCHAIC NUMBER FORTY

111EE
SINHALA ARCHAIC NUMBER FIFTY

111EF
SINHALA ARCHAIC NUMBER SIXTY

111F0
SINHALA ARCHAIC NUMBER SEVENTY

111F1
SINHALA ARCHAIC NUMBER EIGHTY

111F2
SINHALA ARCHAIC NUMBER NINETY

111F3
SINHALA ARCHAIC NUMBER ONE HUNDRED

111F4
SINHALA ARCHAIC NUMBER ONE THOUSAND

11200
KHOJKI LETTER A

11201
KHOJKI LETTER AA

11202
KHOJKI LETTER I

11203
KHOJKI LETTER U

11204
KHOJKI LETTER E

11205
KHOJKI LETTER AI

11206
KHOJKI LETTER O

11207
KHOJKI LETTER AU

11208
KHOJKI LETTER KA

11209
KHOJKI LETTER KHA

1120A
KHOJKI LETTER GA

1120B
KHOJKI LETTER GGA

1120C
KHOJKI LETTER GHA

1120D
KHOJKI LETTER NGA

1120E
KHOJKI LETTER CA

1120F
KHOJKI LETTER CHA

11210
KHOJKI LETTER JA

11211
KHOJKI LETTER JJA

11213
KHOJKI LETTER NYA

11214
KHOJKI LETTER TTA

11215
KHOJKI LETTER TTHA

11216
KHOJKI LETTER DDA

11217
KHOJKI LETTER DDHA

11218
KHOJKI LETTER NNA

11219
KHOJKI LETTER TA

1121A
KHOJKI LETTER THA

1121B
KHOJKI LETTER DA

1121C
KHOJKI LETTER DDDA

1121D
KHOJKI LETTER DHA

1121E
KHOJKI LETTER NA

1121F
KHOJKI LETTER PA

11220
KHOJKI LETTER PHA

11221
KHOJKI LETTER BA

11222
KHOJKI LETTER BBA

11223
KHOJKI LETTER BHA

11224
KHOJKI LETTER MA

11225
KHOJKI LETTER YA

11226
KHOJKI LETTER RA

11227
KHOJKI LETTER LA

11228
KHOJKI LETTER VA

11229
KHOJKI LETTER SA

1122A
KHOJKI LETTER HA

1122B
KHOJKI LETTER LLA

1122C
KHOJKI VOWEL SIGN AA

1122D
KHOJKI VOWEL SIGN I

1122E
KHOJKI VOWEL SIGN II

1122F
KHOJKI VOWEL SIGN U

11230
KHOJKI VOWEL SIGN E

11231
KHOJKI VOWEL SIGN AI

11232
KHOJKI VOWEL SIGN O

11233
KHOJKI VOWEL SIGN AU

11234
KHOJKI SIGN ANUSVARA

11235
KHOJKI SIGN VIRAMA

11236
KHOJKI SIGN NUKTA

11237
KHOJKI SIGN SHADDA

11238
KHOJKI DANDA

11239
KHOJKI DOUBLE DANDA

1123A
KHOJKI WORD SEPARATOR

1123B
KHOJKI SECTION MARK

1123C
KHOJKI DOUBLE SECTION MARK

1123D
KHOJKI ABBREVIATION SIGN

1123E
KHOJKI SIGN SUKUN

1123F
KHOJKI LETTER QA

11240
KHOJKI LETTER SHORT I

11241
KHOJKI VOWEL SIGN VOCALIC R

11280
MULTANI LETTER A

11281
MULTANI LETTER I

11282
MULTANI LETTER U

11283
MULTANI LETTER E

11284
MULTANI LETTER KA

11285
MULTANI LETTER KHA

11286
MULTANI LETTER GA

11288
MULTANI LETTER GHA

1128A
MULTANI LETTER CA

1128B
MULTANI LETTER CHA

1128C
MULTANI LETTER JA

1128D
MULTANI LETTER JJA

1128F
MULTANI LETTER NYA

11290
MULTANI LETTER TTA

11291
MULTANI LETTER TTHA

11292
MULTANI LETTER DDA

11293
MULTANI LETTER DDDA

11294
MULTANI LETTER DDHA

11295
MULTANI LETTER NNA

11296
MULTANI LETTER TA

11297
MULTANI LETTER THA

11298
MULTANI LETTER DA

11299
MULTANI LETTER DHA

1129A
MULTANI LETTER NA

1129B
MULTANI LETTER PA

1129C
MULTANI LETTER PHA

1129D
MULTANI LETTER BA

1129F
MULTANI LETTER BHA

112A0
MULTANI LETTER MA

112A1
MULTANI LETTER YA

112A2
MULTANI LETTER RA

112A3
MULTANI LETTER LA

112A4
MULTANI LETTER VA

112A5
MULTANI LETTER SA

112A6
MULTANI LETTER HA

112A7
MULTANI LETTER RRA

112A8
MULTANI LETTER RHA

112A9
MULTANI SECTION MARK

112B0
KHUDAWADI LETTER A

112B1
KHUDAWADI LETTER AA

112B2
KHUDAWADI LETTER I

112B3
KHUDAWADI LETTER II

112B4
KHUDAWADI LETTER U

112B5
KHUDAWADI LETTER UU

112B6
KHUDAWADI LETTER E

112B7
KHUDAWADI LETTER AI

112B8
KHUDAWADI LETTER O

112B9
KHUDAWADI LETTER AU

112BA
KHUDAWADI LETTER KA

112BB
KHUDAWADI LETTER KHA

112BC
KHUDAWADI LETTER GA

112BD
KHUDAWADI LETTER GGA

112BE
KHUDAWADI LETTER GHA

112BF
KHUDAWADI LETTER NGA

112C0
KHUDAWADI LETTER CA

112C1
KHUDAWADI LETTER CHA

112C2
KHUDAWADI LETTER JA

112C3
KHUDAWADI LETTER JJA

112C4
KHUDAWADI LETTER JHA

112C5
KHUDAWADI LETTER NYA

112C6
KHUDAWADI LETTER TTA

112C7
KHUDAWADI LETTER TTHA

112C8
KHUDAWADI LETTER DDA

112C9
KHUDAWADI LETTER DDDA

112CA
KHUDAWADI LETTER RRA

112CB
KHUDAWADI LETTER DDHA

112CC
KHUDAWADI LETTER NNA

112CD
KHUDAWADI LETTER TA

112CE
KHUDAWADI LETTER THA

112CF
KHUDAWADI LETTER DA

112D0
KHUDAWADI LETTER DHA

112D1
KHUDAWADI LETTER NA

112D2
KHUDAWADI LETTER PA

112D3
KHUDAWADI LETTER PHA

112D4
KHUDAWADI LETTER BA

112D5
KHUDAWADI LETTER BBA

112D6
KHUDAWADI LETTER BHA

112D7
KHUDAWADI LETTER MA

112D8
KHUDAWADI LETTER YA

112D9
KHUDAWADI LETTER RA

112DA
KHUDAWADI LETTER LA

112DB
KHUDAWADI LETTER VA

112DC
KHUDAWADI LETTER SHA

112DD
KHUDAWADI LETTER SA

112DE
KHUDAWADI LETTER HA

112DF
KHUDAWADI SIGN ANUSVARA

112E0
KHUDAWADI VOWEL SIGN AA

112E1
KHUDAWADI VOWEL SIGN I

112E2
KHUDAWADI VOWEL SIGN II

112E3
KHUDAWADI VOWEL SIGN U

112E4
KHUDAWADI VOWEL SIGN UU

112E5
KHUDAWADI VOWEL SIGN E

112E6
KHUDAWADI VOWEL SIGN AI

112E7
KHUDAWADI VOWEL SIGN O

112E8
KHUDAWADI VOWEL SIGN AU

112E9
KHUDAWADI SIGN NUKTA

112EA
KHUDAWADI SIGN VIRAMA

112F0
KHUDAWADI DIGIT ZERO

112F1
KHUDAWADI DIGIT ONE

112F2
KHUDAWADI DIGIT TWO

112F3
KHUDAWADI DIGIT THREE

112F4
KHUDAWADI DIGIT FOUR

112F5
KHUDAWADI DIGIT FIVE

112F6
KHUDAWADI DIGIT SIX

112F7
KHUDAWADI DIGIT SEVEN

112F8
KHUDAWADI DIGIT EIGHT

112F9
KHUDAWADI DIGIT NINE

11300
GRANTHA SIGN COMBINING ANUSVARA ABOVE

11301
GRANTHA SIGN CANDRABINDU

11302
GRANTHA SIGN ANUSVARA

11303
GRANTHA SIGN VISARGA

11305
GRANTHA LETTER A

11306
GRANTHA LETTER AA

11307
GRANTHA LETTER I

11308
GRANTHA LETTER II

11309
GRANTHA LETTER U

1130A
GRANTHA LETTER UU

1130B
GRANTHA LETTER VOCALIC R

1130C
GRANTHA LETTER VOCALIC L

1130F
GRANTHA LETTER EE

11310
GRANTHA LETTER AI

11313
GRANTHA LETTER OO

11314
GRANTHA LETTER AU

11315
GRANTHA LETTER KA

11316
GRANTHA LETTER KHA

11317
GRANTHA LETTER GA

11318
GRANTHA LETTER GHA

11319
GRANTHA LETTER NGA

1131A
GRANTHA LETTER CA

1131B
GRANTHA LETTER CHA

1131C
GRANTHA LETTER JA

1131D
GRANTHA LETTER JHA

1131E
GRANTHA LETTER NYA

1131F
GRANTHA LETTER TTA

11320
GRANTHA LETTER TTHA

11321
GRANTHA LETTER DDA

11322
GRANTHA LETTER DDHA

11323
GRANTHA LETTER NNA

11324
GRANTHA LETTER TA

11325
GRANTHA LETTER THA

11326
GRANTHA LETTER DA

11327
GRANTHA LETTER DHA

11328
GRANTHA LETTER NA

1132A
GRANTHA LETTER PA

1132B
GRANTHA LETTER PHA

1132C
GRANTHA LETTER BA

1132D
GRANTHA LETTER BHA

1132E
GRANTHA LETTER MA

1132F
GRANTHA LETTER YA

11330
GRANTHA LETTER RA

11332
GRANTHA LETTER LA

11333
GRANTHA LETTER LLA

11335
GRANTHA LETTER VA

11336
GRANTHA LETTER SHA

11337
GRANTHA LETTER SSA

11338
GRANTHA LETTER SA

11339
GRANTHA LETTER HA

1133B
COMBINING BINDU BELOW

1133C
GRANTHA SIGN NUKTA

1133D
GRANTHA SIGN AVAGRAHA

1133E
GRANTHA VOWEL SIGN AA

1133F
GRANTHA VOWEL SIGN I

11340
GRANTHA VOWEL SIGN II

11341
GRANTHA VOWEL SIGN U

11342
GRANTHA VOWEL SIGN UU

11343
GRANTHA VOWEL SIGN VOCALIC R

11344
GRANTHA VOWEL SIGN VOCALIC RR

11347
GRANTHA VOWEL SIGN EE

11348
GRANTHA VOWEL SIGN AI

1134B
GRANTHA VOWEL SIGN OO

1134C
GRANTHA VOWEL SIGN AU

1134D
GRANTHA SIGN VIRAMA

11350
GRANTHA OM

11357
GRANTHA AU LENGTH MARK

1135D
GRANTHA SIGN PLUTA

1135E
GRANTHA LETTER VEDIC ANUSVARA

1135F
GRANTHA LETTER VEDIC DOUBLE ANUSVARA

11360
GRANTHA LETTER VOCALIC RR

11361
GRANTHA LETTER VOCALIC LL

11362
GRANTHA VOWEL SIGN VOCALIC L

11363
GRANTHA VOWEL SIGN VOCALIC LL

11366
COMBINING GRANTHA DIGIT ZERO

11367
COMBINING GRANTHA DIGIT ONE

11368
COMBINING GRANTHA DIGIT TWO

11369
COMBINING GRANTHA DIGIT THREE

1136A
COMBINING GRANTHA DIGIT FOUR

1136B
COMBINING GRANTHA DIGIT FIVE

1136C
COMBINING GRANTHA DIGIT SIX

11370
COMBINING GRANTHA LETTER A

11371
COMBINING GRANTHA LETTER KA

11372
COMBINING GRANTHA LETTER NA

11373
COMBINING GRANTHA LETTER VI

11374
COMBINING GRANTHA LETTER PA

11380
TULU-TIGALARI LETTER A

11381
TULU-TIGALARI LETTER AA

11382
TULU-TIGALARI LETTER I

11383
TULU-TIGALARI LETTER II

11384
TULU-TIGALARI LETTER U

11385
TULU-TIGALARI LETTER UU

11386
TULU-TIGALARI LETTER VOCALIC R

11387
TULU-TIGALARI LETTER VOCALIC RR

11388
TULU-TIGALARI LETTER VOCALIC L

11389
TULU-TIGALARI LETTER VOCALIC LL

1138B
TULU-TIGALARI LETTER EE

1138E
TULU-TIGALARI LETTER AI

11390
TULU-TIGALARI LETTER OO

11391
TULU-TIGALARI LETTER AU

11392
TULU-TIGALARI LETTER KA

11393
TULU-TIGALARI LETTER KHA

11394
TULU-TIGALARI LETTER GA

11395
TULU-TIGALARI LETTER GHA

11396
TULU-TIGALARI LETTER NGA

11397
TULU-TIGALARI LETTER CA

11398
TULU-TIGALARI LETTER CHA

11399
TULU-TIGALARI LETTER JA

1139A
TULU-TIGALARI LETTER JHA

1139B
TULU-TIGALARI LETTER NYA

1139C
TULU-TIGALARI LETTER TTA

1139D
TULU-TIGALARI LETTER TTHA

1139E
TULU-TIGALARI LETTER DDA

1139F
TULU-TIGALARI LETTER DDHA

113A0
TULU-TIGALARI LETTER NNA

113A1
TULU-TIGALARI LETTER TA

113A2
TULU-TIGALARI LETTER THA

113A3
TULU-TIGALARI LETTER DA

113A4
TULU-TIGALARI LETTER DHA

113A5
TULU-TIGALARI LETTER NA

113A6
TULU-TIGALARI LETTER PA

113A7
TULU-TIGALARI LETTER PHA

113A8
TULU-TIGALARI LETTER BA

113A9
TULU-TIGALARI LETTER BHA

113AA
TULU-TIGALARI LETTER MA

113AB
TULU-TIGALARI LETTER YA

113AC
TULU-TIGALARI LETTER RA

113AD
TULU-TIGALARI LETTER LA

113AE
TULU-TIGALARI LETTER VA

113AF
TULU-TIGALARI LETTER SHA

113B0
TULU-TIGALARI LETTER SSA

113B1
TULU-TIGALARI LETTER SA

113B2
TULU-TIGALARI LETTER HA

113B3
TULU-TIGALARI LETTER LLA

113B4
TULU-TIGALARI LETTER RRA

113B5
TULU-TIGALARI LETTER LLLA

113B7
TULU-TIGALARI SIGN AVAGRAHA

113B8
TULU-TIGALARI VOWEL SIGN AA

113B9
TULU-TIGALARI VOWEL SIGN I

113BA
TULU-TIGALARI VOWEL SIGN II

113BB
TULU-TIGALARI VOWEL SIGN U

113BC
TULU-TIGALARI VOWEL SIGN UU

113BD
TULU-TIGALARI VOWEL SIGN VOCALIC R

113BE
TULU-TIGALARI VOWEL SIGN VOCALIC RR

113BF
TULU-TIGALARI VOWEL SIGN VOCALIC L

113C0
TULU-TIGALARI VOWEL SIGN VOCALIC LL

113C2
TULU-TIGALARI VOWEL SIGN EE

113C5
TULU-TIGALARI VOWEL SIGN AI

113C7
TULU-TIGALARI VOWEL SIGN OO

113C8
TULU-TIGALARI VOWEL SIGN AU

113C9
TULU-TIGALARI AU LENGTH MARK

113CA
TULU-TIGALARI SIGN CANDRA ANUNASIKA

113CC
TULU-TIGALARI SIGN ANUSVARA

113CD
TULU-TIGALARI SIGN VISARGA

113CE
TULU-TIGALARI SIGN VIRAMA

113CF
TULU-TIGALARI SIGN LOOPED VIRAMA

113D0
TULU-TIGALARI CONJOINER

113D1
TULU-TIGALARI REPHA

113D2
TULU-TIGALARI GEMINATION MARK

113D3
TULU-TIGALARI SIGN PLUTA

113D4
TULU-TIGALARI DANDA

113D5
TULU-TIGALARI DOUBLE DANDA

113D7
TULU-TIGALARI SIGN OM PUSHPIKA

113D8
TULU-TIGALARI SIGN SHRII PUSHPIKA

113E1
TULU-TIGALARI VEDIC TONE SVARITA

113E2
TULU-TIGALARI VEDIC TONE ANUDATTA

11400
NEWA LETTER A

11401
NEWA LETTER AA

11402
NEWA LETTER I

11403
NEWA LETTER II

11404
NEWA LETTER U

11405
NEWA LETTER UU

11406
NEWA LETTER VOCALIC R

11407
NEWA LETTER VOCALIC RR

11408
NEWA LETTER VOCALIC L

11409
NEWA LETTER VOCALIC LL

1140A
NEWA LETTER E

1140B
NEWA LETTER AI

1140C
NEWA LETTER O

1140D
NEWA LETTER AU

1140E
NEWA LETTER KA

1140F
NEWA LETTER KHA

11410
NEWA LETTER GA

11411
NEWA LETTER GHA

11412
NEWA LETTER NGA

11413
NEWA LETTER NGHA

11414
NEWA LETTER CA

11415
NEWA LETTER CHA

11416
NEWA LETTER JA

11417
NEWA LETTER JHA

11418
NEWA LETTER NYA

11419
NEWA LETTER NYHA

1141A
NEWA LETTER TTA

1141B
NEWA LETTER TTHA

1141C
NEWA LETTER DDA

1141D
NEWA LETTER DDHA

1141E
NEWA LETTER NNA

1141F
NEWA LETTER TA

11420
NEWA LETTER THA

11421
NEWA LETTER DA

11422
NEWA LETTER DHA

11423
NEWA LETTER NA

11424
NEWA LETTER NHA

11425
NEWA LETTER PA

11426
NEWA LETTER PHA

11427
NEWA LETTER BA

11428
NEWA LETTER BHA

11429
NEWA LETTER MA

1142A
NEWA LETTER MHA

1142B
NEWA LETTER YA

1142C
NEWA LETTER RA

1142D
NEWA LETTER RHA

1142E
NEWA LETTER LA

1142F
NEWA LETTER LHA

11430
NEWA LETTER WA

11431
NEWA LETTER SHA

11432
NEWA LETTER SSA

11433
NEWA LETTER SA

11434
NEWA LETTER HA

11435
NEWA VOWEL SIGN AA

11436
NEWA VOWEL SIGN I

11437
NEWA VOWEL SIGN II

11438
NEWA VOWEL SIGN U

11439
NEWA VOWEL SIGN UU

1143A
NEWA VOWEL SIGN VOCALIC R

1143B
NEWA VOWEL SIGN VOCALIC RR

1143C
NEWA VOWEL SIGN VOCALIC L

1143D
NEWA VOWEL SIGN VOCALIC LL

1143E
NEWA VOWEL SIGN E

1143F
NEWA VOWEL SIGN AI

11440
NEWA VOWEL SIGN O

11441
NEWA VOWEL SIGN AU

11442
NEWA SIGN VIRAMA

11443
NEWA SIGN CANDRABINDU

11444
NEWA SIGN ANUSVARA

11445
NEWA SIGN VISARGA

11446
NEWA SIGN NUKTA

11447
NEWA SIGN AVAGRAHA

11448
NEWA SIGN FINAL ANUSVARA

11449
NEWA OM

1144A
NEWA SIDDHI

1144B
NEWA DANDA

1144C
NEWA DOUBLE DANDA

1144D
NEWA COMMA

1144E
NEWA GAP FILLER

1144F
NEWA ABBREVIATION SIGN

11450
NEWA DIGIT ZERO

11451
NEWA DIGIT ONE

11452
NEWA DIGIT TWO

11453
NEWA DIGIT THREE

11454
NEWA DIGIT FOUR

11455
NEWA DIGIT FIVE

11456
NEWA DIGIT SIX

11457
NEWA DIGIT SEVEN

11458
NEWA DIGIT EIGHT

11459
NEWA DIGIT NINE

1145A
NEWA DOUBLE COMMA

1145B
NEWA PLACEHOLDER MARK

1145D
NEWA INSERTION SIGN

1145E
NEWA SANDHI MARK

1145F
NEWA LETTER VEDIC ANUSVARA

11460
NEWA SIGN JIHVAMULIYA

11461
NEWA SIGN UPADHMANIYA

11480
TIRHUTA ANJI

11481
TIRHUTA LETTER A

11482
TIRHUTA LETTER AA

11483
TIRHUTA LETTER I

11484
TIRHUTA LETTER II

11485
TIRHUTA LETTER U

11486
TIRHUTA LETTER UU

11487
TIRHUTA LETTER VOCALIC R

11488
TIRHUTA LETTER VOCALIC RR

11489
TIRHUTA LETTER VOCALIC L

1148A
TIRHUTA LETTER VOCALIC LL

1148B
TIRHUTA LETTER E

1148C
TIRHUTA LETTER AI

1148D
TIRHUTA LETTER O

1148E
TIRHUTA LETTER AU

1148F
TIRHUTA LETTER KA

11490
TIRHUTA LETTER KHA

11491
TIRHUTA LETTER GA

11492
TIRHUTA LETTER GHA

11493
TIRHUTA LETTER NGA

11494
TIRHUTA LETTER CA

11495
TIRHUTA LETTER CHA

11496
TIRHUTA LETTER JA

11497
TIRHUTA LETTER JHA

11498
TIRHUTA LETTER NYA

11499
TIRHUTA LETTER TTA

1149A
TIRHUTA LETTER TTHA

1149B
TIRHUTA LETTER DDA

1149C
TIRHUTA LETTER DDHA

1149D
TIRHUTA LETTER NNA

1149E
TIRHUTA LETTER TA

1149F
TIRHUTA LETTER THA

114A0
TIRHUTA LETTER DA

114A1
TIRHUTA LETTER DHA

114A2
TIRHUTA LETTER NA

114A3
TIRHUTA LETTER PA

114A4
TIRHUTA LETTER PHA

114A5
TIRHUTA LETTER BA

114A6
TIRHUTA LETTER BHA

114A7
TIRHUTA LETTER MA

114A8
TIRHUTA LETTER YA

114A9
TIRHUTA LETTER RA

114AA
TIRHUTA LETTER LA

114AB
TIRHUTA LETTER VA

114AC
TIRHUTA LETTER SHA

114AD
TIRHUTA LETTER SSA

114AE
TIRHUTA LETTER SA

114AF
TIRHUTA LETTER HA

114B0
TIRHUTA VOWEL SIGN AA

114B1
TIRHUTA VOWEL SIGN I

114B2
TIRHUTA VOWEL SIGN II

114B3
TIRHUTA VOWEL SIGN U

114B4
TIRHUTA VOWEL SIGN UU

114B5
TIRHUTA VOWEL SIGN VOCALIC R

114B6
TIRHUTA VOWEL SIGN VOCALIC RR

114B7
TIRHUTA VOWEL SIGN VOCALIC L

114B8
TIRHUTA VOWEL SIGN VOCALIC LL

114B9
TIRHUTA VOWEL SIGN E

114BA
TIRHUTA VOWEL SIGN SHORT E

114BB
TIRHUTA VOWEL SIGN AI

114BC
TIRHUTA VOWEL SIGN O

114BD
TIRHUTA VOWEL SIGN SHORT O

114BE
TIRHUTA VOWEL SIGN AU

114BF
TIRHUTA SIGN CANDRABINDU

114C0
TIRHUTA SIGN ANUSVARA

114C1
TIRHUTA SIGN VISARGA

114C2
TIRHUTA SIGN VIRAMA

114C3
TIRHUTA SIGN NUKTA

114C4
TIRHUTA SIGN AVAGRAHA

114C5
TIRHUTA GVANG

114C6
TIRHUTA ABBREVIATION SIGN

114C7
TIRHUTA OM

114D0
TIRHUTA DIGIT ZERO

114D1
TIRHUTA DIGIT ONE

114D2
TIRHUTA DIGIT TWO

114D3
TIRHUTA DIGIT THREE

114D4
TIRHUTA DIGIT FOUR

114D5
TIRHUTA DIGIT FIVE

114D6
TIRHUTA DIGIT SIX

114D7
TIRHUTA DIGIT SEVEN

114D8
TIRHUTA DIGIT EIGHT

114D9
TIRHUTA DIGIT NINE

11580
SIDDHAM LETTER A

11581
SIDDHAM LETTER AA

11582
SIDDHAM LETTER I

11583
SIDDHAM LETTER II

11584
SIDDHAM LETTER U

11585
SIDDHAM LETTER UU

11586
SIDDHAM LETTER VOCALIC R

11587
SIDDHAM LETTER VOCALIC RR

11588
SIDDHAM LETTER VOCALIC L

11589
SIDDHAM LETTER VOCALIC LL

1158A
SIDDHAM LETTER E

1158B
SIDDHAM LETTER AI

1158C
SIDDHAM LETTER O

1158D
SIDDHAM LETTER AU

1158E
SIDDHAM LETTER KA

1158F
SIDDHAM LETTER KHA

11590
SIDDHAM LETTER GA

11591
SIDDHAM LETTER GHA

11592
SIDDHAM LETTER NGA

11593
SIDDHAM LETTER CA

11594
SIDDHAM LETTER CHA

11595
SIDDHAM LETTER JA

11596
SIDDHAM LETTER JHA

11597
SIDDHAM LETTER NYA

11598
SIDDHAM LETTER TTA

11599
SIDDHAM LETTER TTHA

1159A
SIDDHAM LETTER DDA

1159B
SIDDHAM LETTER DDHA

1159C
SIDDHAM LETTER NNA

1159D
SIDDHAM LETTER TA

1159E
SIDDHAM LETTER THA

1159F
SIDDHAM LETTER DA

115A0
SIDDHAM LETTER DHA

115A1
SIDDHAM LETTER NA

115A2
SIDDHAM LETTER PA

115A3
SIDDHAM LETTER PHA

115A4
SIDDHAM LETTER BA

115A5
SIDDHAM LETTER BHA

115A6
SIDDHAM LETTER MA

115A7
SIDDHAM LETTER YA

115A8
SIDDHAM LETTER RA

115A9
SIDDHAM LETTER LA

115AA
SIDDHAM LETTER VA

115AB
SIDDHAM LETTER SHA

115AC
SIDDHAM LETTER SSA

115AD
SIDDHAM LETTER SA

115AE
SIDDHAM LETTER HA

115AF
SIDDHAM VOWEL SIGN AA

115B0
SIDDHAM VOWEL SIGN I

115B1
SIDDHAM VOWEL SIGN II

115B2
SIDDHAM VOWEL SIGN U

115B3
SIDDHAM VOWEL SIGN UU

115B4
SIDDHAM VOWEL SIGN VOCALIC R

115B5
SIDDHAM VOWEL SIGN VOCALIC RR

115B8
SIDDHAM VOWEL SIGN E

115B9
SIDDHAM VOWEL SIGN AI

115BA
SIDDHAM VOWEL SIGN O

115BB
SIDDHAM VOWEL SIGN AU

115BC
SIDDHAM SIGN CANDRABINDU

115BD
SIDDHAM SIGN ANUSVARA

115BE
SIDDHAM SIGN VISARGA

115BF
SIDDHAM SIGN VIRAMA

115C0
SIDDHAM SIGN NUKTA

115C1
SIDDHAM SIGN SIDDHAM

115C2
SIDDHAM DANDA

115C3
SIDDHAM DOUBLE DANDA

115C4
SIDDHAM SEPARATOR DOT

115C5
SIDDHAM SEPARATOR BAR

115C6
SIDDHAM REPETITION MARK-1

115C7
SIDDHAM REPETITION MARK-2

115C8
SIDDHAM REPETITION MARK-3

115C9
SIDDHAM END OF TEXT MARK

115CA
SIDDHAM SECTION MARK WITH TRIDENT AND U-SHAPED ORNAMENTS

115CB
SIDDHAM SECTION MARK WITH TRIDENT AND DOTTED CRESCENTS

115CC
SIDDHAM SECTION MARK WITH RAYS AND DOTTED CRESCENTS

115CD
SIDDHAM SECTION MARK WITH RAYS AND DOTTED DOUBLE CRESCENTS

115CE
SIDDHAM SECTION MARK WITH RAYS AND DOTTED TRIPLE CRESCENTS

115CF
SIDDHAM SECTION MARK DOUBLE RING

115D0
SIDDHAM SECTION MARK DOUBLE RING WITH RAYS

115D1
SIDDHAM SECTION MARK WITH DOUBLE CRESCENTS

115D2
SIDDHAM SECTION MARK WITH TRIPLE CRESCENTS

115D3
SIDDHAM SECTION MARK WITH QUADRUPLE CRESCENTS

115D4
SIDDHAM SECTION MARK WITH SEPTUPLE CRESCENTS

115D5
SIDDHAM SECTION MARK WITH CIRCLES AND RAYS

115D6
SIDDHAM SECTION MARK WITH CIRCLES AND TWO ENCLOSURES

115D7
SIDDHAM SECTION MARK WITH CIRCLES AND FOUR ENCLOSURES

115D8
SIDDHAM LETTER THREE-CIRCLE ALTERNATE I

115D9
SIDDHAM LETTER TWO-CIRCLE ALTERNATE I

115DA
SIDDHAM LETTER TWO-CIRCLE ALTERNATE II

115DB
SIDDHAM LETTER ALTERNATE U

115DC
SIDDHAM VOWEL SIGN ALTERNATE U

115DD
SIDDHAM VOWEL SIGN ALTERNATE UU

11600
MODI LETTER A

11601
MODI LETTER AA

11602
MODI LETTER I

11603
MODI LETTER II

11604
MODI LETTER U

11605
MODI LETTER UU

11606
MODI LETTER VOCALIC R

11607
MODI LETTER VOCALIC RR

11608
MODI LETTER VOCALIC L

11609
MODI LETTER VOCALIC LL

1160A
MODI LETTER E

1160B
MODI LETTER AI

1160C
MODI LETTER O

1160D
MODI LETTER AU

1160E
MODI LETTER KA

1160F
MODI LETTER KHA

11610
MODI LETTER GA

11611
MODI LETTER GHA

11612
MODI LETTER NGA

11613
MODI LETTER CA

11614
MODI LETTER CHA

11615
MODI LETTER JA

11616
MODI LETTER JHA

11617
MODI LETTER NYA

11618
MODI LETTER TTA

11619
MODI LETTER TTHA

1161A
MODI LETTER DDA

1161B
MODI LETTER DDHA

1161C
MODI LETTER NNA

1161D
MODI LETTER TA

1161E
MODI LETTER THA

1161F
MODI LETTER DA

11620
MODI LETTER DHA

11621
MODI LETTER NA

11622
MODI LETTER PA

11623
MODI LETTER PHA

11624
MODI LETTER BA

11625
MODI LETTER BHA

11626
MODI LETTER MA

11627
MODI LETTER YA

11628
MODI LETTER RA

11629
MODI LETTER LA

1162A
MODI LETTER VA

1162B
MODI LETTER SHA

1162C
MODI LETTER SSA

1162D
MODI LETTER SA

1162E
MODI LETTER HA

1162F
MODI LETTER LLA

11630
MODI VOWEL SIGN AA

11631
MODI VOWEL SIGN I

11632
MODI VOWEL SIGN II

11633
MODI VOWEL SIGN U

11634
MODI VOWEL SIGN UU

11635
MODI VOWEL SIGN VOCALIC R

11636
MODI VOWEL SIGN VOCALIC RR

11637
MODI VOWEL SIGN VOCALIC L

11638
MODI VOWEL SIGN VOCALIC LL

11639
MODI VOWEL SIGN E

1163A
MODI VOWEL SIGN AI

1163B
MODI VOWEL SIGN O

1163C
MODI VOWEL SIGN AU

1163D
MODI SIGN ANUSVARA

1163E
MODI SIGN VISARGA

1163F
MODI SIGN VIRAMA

11640
MODI SIGN ARDHACANDRA

11641
MODI DANDA

11642
MODI DOUBLE DANDA

11643
MODI ABBREVIATION SIGN

11644
MODI SIGN HUVA

11650
MODI DIGIT ZERO

11651
MODI DIGIT ONE

11652
MODI DIGIT TWO

11653
MODI DIGIT THREE

11654
MODI DIGIT FOUR

11655
MODI DIGIT FIVE

11656
MODI DIGIT SIX

11657
MODI DIGIT SEVEN

11658
MODI DIGIT EIGHT

11659
MODI DIGIT NINE

11660
MONGOLIAN BIRGA WITH ORNAMENT

11661
MONGOLIAN ROTATED BIRGA

11662
MONGOLIAN DOUBLE BIRGA WITH ORNAMENT

11663
MONGOLIAN TRIPLE BIRGA WITH ORNAMENT

11664
MONGOLIAN BIRGA WITH DOUBLE ORNAMENT

11665
MONGOLIAN ROTATED BIRGA WITH ORNAMENT

11666
MONGOLIAN ROTATED BIRGA WITH DOUBLE ORNAMENT

11667
MONGOLIAN INVERTED BIRGA

11668
MONGOLIAN INVERTED BIRGA WITH DOUBLE ORNAMENT

11669
MONGOLIAN SWIRL BIRGA

1166A
MONGOLIAN SWIRL BIRGA WITH ORNAMENT

1166B
MONGOLIAN SWIRL BIRGA WITH DOUBLE ORNAMENT

1166C
MONGOLIAN TURNED SWIRL BIRGA WITH DOUBLE ORNAMENT

11680
TAKRI LETTER A

11681
TAKRI LETTER AA

11682
TAKRI LETTER I

11683
TAKRI LETTER II

11684
TAKRI LETTER U

11685
TAKRI LETTER UU

11686
TAKRI LETTER E

11687
TAKRI LETTER AI

11688
TAKRI LETTER O

11689
TAKRI LETTER AU

1168A
TAKRI LETTER KA

1168B
TAKRI LETTER KHA

1168C
TAKRI LETTER GA

1168D
TAKRI LETTER GHA

1168E
TAKRI LETTER NGA

1168F
TAKRI LETTER CA

11690
TAKRI LETTER CHA

11691
TAKRI LETTER JA

11692
TAKRI LETTER JHA

11693
TAKRI LETTER NYA

11694
TAKRI LETTER TTA

11695
TAKRI LETTER TTHA

11696
TAKRI LETTER DDA

11697
TAKRI LETTER DDHA

11698
TAKRI LETTER NNA

11699
TAKRI LETTER TA

1169A
TAKRI LETTER THA

1169B
TAKRI LETTER DA

1169C
TAKRI LETTER DHA

1169D
TAKRI LETTER NA

1169E
TAKRI LETTER PA

1169F
TAKRI LETTER PHA

116A0
TAKRI LETTER BA

116A1
TAKRI LETTER BHA

116A2
TAKRI LETTER MA

116A3
TAKRI LETTER YA

116A4
TAKRI LETTER RA

116A5
TAKRI LETTER LA

116A6
TAKRI LETTER VA

116A7
TAKRI LETTER SHA

116A8
TAKRI LETTER SA

116A9
TAKRI LETTER HA

116AA
TAKRI LETTER RRA

116AB
TAKRI SIGN ANUSVARA

116AC
TAKRI SIGN VISARGA

116AD
TAKRI VOWEL SIGN AA

116AE
TAKRI VOWEL SIGN I

116AF
TAKRI VOWEL SIGN II

116B0
TAKRI VOWEL SIGN U

116B1
TAKRI VOWEL SIGN UU

116B2
TAKRI VOWEL SIGN E

116B3
TAKRI VOWEL SIGN AI

116B4
TAKRI VOWEL SIGN O

116B5
TAKRI VOWEL SIGN AU

116B6
TAKRI SIGN VIRAMA

116B7
TAKRI SIGN NUKTA

116B8
TAKRI LETTER ARCHAIC KHA

116B9
TAKRI ABBREVIATION SIGN

116C0
TAKRI DIGIT ZERO

116C1
TAKRI DIGIT ONE

116C2
TAKRI DIGIT TWO

116C3
TAKRI DIGIT THREE

116C4
TAKRI DIGIT FOUR

116C5
TAKRI DIGIT FIVE

116C6
TAKRI DIGIT SIX

116C7
TAKRI DIGIT SEVEN

116C8
TAKRI DIGIT EIGHT

116C9
TAKRI DIGIT NINE

116D0
MYANMAR PAO DIGIT ZERO

116D1
MYANMAR PAO DIGIT ONE

116D2
MYANMAR PAO DIGIT TWO

116D3
MYANMAR PAO DIGIT THREE

116D4
MYANMAR PAO DIGIT FOUR

116D5
MYANMAR PAO DIGIT FIVE

116D6
MYANMAR PAO DIGIT SIX

116D7
MYANMAR PAO DIGIT SEVEN

116D8
MYANMAR PAO DIGIT EIGHT

116D9
MYANMAR PAO DIGIT NINE

116DA
MYANMAR EASTERN PWO KAREN DIGIT ZERO

116DB
MYANMAR EASTERN PWO KAREN DIGIT ONE

116DC
MYANMAR EASTERN PWO KAREN DIGIT TWO

116DD
MYANMAR EASTERN PWO KAREN DIGIT THREE

116DE
MYANMAR EASTERN PWO KAREN DIGIT FOUR

116DF
MYANMAR EASTERN PWO KAREN DIGIT FIVE

116E0
MYANMAR EASTERN PWO KAREN DIGIT SIX

116E1
MYANMAR EASTERN PWO KAREN DIGIT SEVEN

116E2
MYANMAR EASTERN PWO KAREN DIGIT EIGHT

116E3
MYANMAR EASTERN PWO KAREN DIGIT NINE

11700
AHOM LETTER KA

11701
AHOM LETTER KHA

11702
AHOM LETTER NGA

11703
AHOM LETTER NA

11704
AHOM LETTER TA

11705
AHOM LETTER ALTERNATE TA

11706
AHOM LETTER PA

11707
AHOM LETTER PHA

11708
AHOM LETTER BA

11709
AHOM LETTER MA

1170A
AHOM LETTER JA

1170B
AHOM LETTER CHA

1170C
AHOM LETTER THA

1170D
AHOM LETTER RA

1170E
AHOM LETTER LA

1170F
AHOM LETTER SA

11710
AHOM LETTER NYA

11711
AHOM LETTER HA

11712
AHOM LETTER A

11713
AHOM LETTER DA

11714
AHOM LETTER DHA

11715
AHOM LETTER GA

11716
AHOM LETTER ALTERNATE GA

11717
AHOM LETTER GHA

11718
AHOM LETTER BHA

11719
AHOM LETTER JHA

1171A
AHOM LETTER ALTERNATE BA

1171D
AHOM CONSONANT SIGN MEDIAL LA

1171E
AHOM CONSONANT SIGN MEDIAL RA

1171F
AHOM CONSONANT SIGN MEDIAL LIGATING RA

11720
AHOM VOWEL SIGN A

11721
AHOM VOWEL SIGN AA

11722
AHOM VOWEL SIGN I

11723
AHOM VOWEL SIGN II

11724
AHOM VOWEL SIGN U

11725
AHOM VOWEL SIGN UU

11726
AHOM VOWEL SIGN E

11727
AHOM VOWEL SIGN AW

11728
AHOM VOWEL SIGN O

11729
AHOM VOWEL SIGN AI

1172A
AHOM VOWEL SIGN AM

1172B
AHOM SIGN KILLER

11730
AHOM DIGIT ZERO

11731
AHOM DIGIT ONE

11732
AHOM DIGIT TWO

11733
AHOM DIGIT THREE

11734
AHOM DIGIT FOUR

11735
AHOM DIGIT FIVE

11736
AHOM DIGIT SIX

11737
AHOM DIGIT SEVEN

11738
AHOM DIGIT EIGHT

11739
AHOM DIGIT NINE

1173A
AHOM NUMBER TEN

1173B
AHOM NUMBER TWENTY

1173C
AHOM SIGN SMALL SECTION

1173D
AHOM SIGN SECTION

1173E
AHOM SIGN RULAI

1173F
AHOM SYMBOL VI

11740
AHOM LETTER CA

11741
AHOM LETTER TTA

11742
AHOM LETTER TTHA

11743
AHOM LETTER DDA

11744
AHOM LETTER DDHA

11745
AHOM LETTER NNA

11746
AHOM LETTER LLA

11800
DOGRA LETTER A

11801
DOGRA LETTER AA

11802
DOGRA LETTER I

11803
DOGRA LETTER II

11804
DOGRA LETTER U

11805
DOGRA LETTER UU

11806
DOGRA LETTER E

11807
DOGRA LETTER AI

11808
DOGRA LETTER O

11809
DOGRA LETTER AU

1180A
DOGRA LETTER KA

1180B
DOGRA LETTER KHA

1180C
DOGRA LETTER GA

1180D
DOGRA LETTER GHA

1180E
DOGRA LETTER NGA

1180F
DOGRA LETTER CA

11810
DOGRA LETTER CHA

11811
DOGRA LETTER JA

11812
DOGRA LETTER JHA

11813
DOGRA LETTER NYA

11814
DOGRA LETTER TTA

11815
DOGRA LETTER TTHA

11816
DOGRA LETTER DDA

11817
DOGRA LETTER DDHA

11818
DOGRA LETTER NNA

11819
DOGRA LETTER TA

1181A
DOGRA LETTER THA

1181B
DOGRA LETTER DA

1181C
DOGRA LETTER DHA

1181D
DOGRA LETTER NA

1181E
DOGRA LETTER PA

1181F
DOGRA LETTER PHA

11820
DOGRA LETTER BA

11821
DOGRA LETTER BHA

11822
DOGRA LETTER MA

11823
DOGRA LETTER YA

11824
DOGRA LETTER RA

11825
DOGRA LETTER LA

11826
DOGRA LETTER VA

11827
DOGRA LETTER SHA

11828
DOGRA LETTER SSA

11829
DOGRA LETTER SA

1182A
DOGRA LETTER HA

1182B
DOGRA LETTER RRA

1182C
DOGRA VOWEL SIGN AA

1182D
DOGRA VOWEL SIGN I

1182E
DOGRA VOWEL SIGN II

1182F
DOGRA VOWEL SIGN U

11830
DOGRA VOWEL SIGN UU

11831
DOGRA VOWEL SIGN VOCALIC R

11832
DOGRA VOWEL SIGN VOCALIC RR

11833
DOGRA VOWEL SIGN E

11834
DOGRA VOWEL SIGN AI

11835
DOGRA VOWEL SIGN O

11836
DOGRA VOWEL SIGN AU

11837
DOGRA SIGN ANUSVARA

11838
DOGRA SIGN VISARGA

11839
DOGRA SIGN VIRAMA

1183A
DOGRA SIGN NUKTA

1183B
DOGRA ABBREVIATION SIGN

118A0
WARANG CITI CAPITAL LETTER NGAA

118A1
WARANG CITI CAPITAL LETTER A

118A2
WARANG CITI CAPITAL LETTER WI

118A3
WARANG CITI CAPITAL LETTER YU

118A4
WARANG CITI CAPITAL LETTER YA

118A5
WARANG CITI CAPITAL LETTER YO

118A6
WARANG CITI CAPITAL LETTER II

118A7
WARANG CITI CAPITAL LETTER UU

118A8
WARANG CITI CAPITAL LETTER E

118A9
WARANG CITI CAPITAL LETTER O

118AA
WARANG CITI CAPITAL LETTER ANG

118AB
WARANG CITI CAPITAL LETTER GA

118AC
WARANG CITI CAPITAL LETTER KO

118AD
WARANG CITI CAPITAL LETTER ENY

118AE
WARANG CITI CAPITAL LETTER YUJ

118AF
WARANG CITI CAPITAL LETTER UC

118B0
WARANG CITI CAPITAL LETTER ENN

118B1
WARANG CITI CAPITAL LETTER ODD

118B2
WARANG CITI CAPITAL LETTER TTE

118B3
WARANG CITI CAPITAL LETTER NUNG

118B4
WARANG CITI CAPITAL LETTER DA

118B5
WARANG CITI CAPITAL LETTER AT

118B6
WARANG CITI CAPITAL LETTER AM

118B7
WARANG CITI CAPITAL LETTER BU

118B8
WARANG CITI CAPITAL LETTER PU

118B9
WARANG CITI CAPITAL LETTER HIYO

118BA
WARANG CITI CAPITAL LETTER HOLO

118BB
WARANG CITI CAPITAL LETTER HORR

118BC
WARANG CITI CAPITAL LETTER HAR

118BD
WARANG CITI CAPITAL LETTER SSUU

118BE
WARANG CITI CAPITAL LETTER SII

118BF
WARANG CITI CAPITAL LETTER VIYO

118C0
WARANG CITI SMALL LETTER NGAA

118C1
WARANG CITI SMALL LETTER A

118C2
WARANG CITI SMALL LETTER WI

118C3
WARANG CITI SMALL LETTER YU

118C4
WARANG CITI SMALL LETTER YA

118C5
WARANG CITI SMALL LETTER YO

118C6
WARANG CITI SMALL LETTER II

118C7
WARANG CITI SMALL LETTER UU

118C8
WARANG CITI SMALL LETTER E

118C9
WARANG CITI SMALL LETTER O

118CA
WARANG CITI SMALL LETTER ANG

118CB
WARANG CITI SMALL LETTER GA

118CC
WARANG CITI SMALL LETTER KO

118CD
WARANG CITI SMALL LETTER ENY

118CE
WARANG CITI SMALL LETTER YUJ

118CF
WARANG CITI SMALL LETTER UC

118D0
WARANG CITI SMALL LETTER ENN

118D1
WARANG CITI SMALL LETTER ODD

118D2
WARANG CITI SMALL LETTER TTE

118D3
WARANG CITI SMALL LETTER NUNG

118D4
WARANG CITI SMALL LETTER DA

118D5
WARANG CITI SMALL LETTER AT

118D6
WARANG CITI SMALL LETTER AM

118D7
WARANG CITI SMALL LETTER BU

118D8
WARANG CITI SMALL LETTER PU

118D9
WARANG CITI SMALL LETTER HIYO

118DA
WARANG CITI SMALL LETTER HOLO

118DB
WARANG CITI SMALL LETTER HORR

118DC
WARANG CITI SMALL LETTER HAR

118DD
WARANG CITI SMALL LETTER SSUU

118DE
WARANG CITI SMALL LETTER SII

118DF
WARANG CITI SMALL LETTER VIYO

118E0
WARANG CITI DIGIT ZERO

118E1
WARANG CITI DIGIT ONE

118E2
WARANG CITI DIGIT TWO

118E3
WARANG CITI DIGIT THREE

118E4
WARANG CITI DIGIT FOUR

118E5
WARANG CITI DIGIT FIVE

118E6
WARANG CITI DIGIT SIX

118E7
WARANG CITI DIGIT SEVEN

118E8
WARANG CITI DIGIT EIGHT

118E9
WARANG CITI DIGIT NINE

118EA
WARANG CITI NUMBER TEN

118EB
WARANG CITI NUMBER TWENTY

118EC
WARANG CITI NUMBER THIRTY

118ED
WARANG CITI NUMBER FORTY

118EE
WARANG CITI NUMBER FIFTY

118EF
WARANG CITI NUMBER SIXTY

118F0
WARANG CITI NUMBER SEVENTY

118F1
WARANG CITI NUMBER EIGHTY

118F2
WARANG CITI NUMBER NINETY

118FF
WARANG CITI OM

11900
DIVES AKURU LETTER A

11901
DIVES AKURU LETTER AA

11902
DIVES AKURU LETTER I

11903
DIVES AKURU LETTER II

11904
DIVES AKURU LETTER U

11905
DIVES AKURU LETTER UU

11906
DIVES AKURU LETTER E

11909
DIVES AKURU LETTER O

1190C
DIVES AKURU LETTER KA

1190D
DIVES AKURU LETTER KHA

1190E
DIVES AKURU LETTER GA

1190F
DIVES AKURU LETTER GHA

11910
DIVES AKURU LETTER NGA

11911
DIVES AKURU LETTER CA

11912
DIVES AKURU LETTER CHA

11913
DIVES AKURU LETTER JA

11915
DIVES AKURU LETTER NYA

11916
DIVES AKURU LETTER TTA

11918
DIVES AKURU LETTER DDA

11919
DIVES AKURU LETTER DDHA

1191A
DIVES AKURU LETTER NNA

1191B
DIVES AKURU LETTER TA

1191C
DIVES AKURU LETTER THA

1191D
DIVES AKURU LETTER DA

1191E
DIVES AKURU LETTER DHA

1191F
DIVES AKURU LETTER NA

11920
DIVES AKURU LETTER PA

11921
DIVES AKURU LETTER PHA

11922
DIVES AKURU LETTER BA

11923
DIVES AKURU LETTER BHA

11924
DIVES AKURU LETTER MA

11925
DIVES AKURU LETTER YA

11926
DIVES AKURU LETTER YYA

11927
DIVES AKURU LETTER RA

11928
DIVES AKURU LETTER LA

11929
DIVES AKURU LETTER VA

1192A
DIVES AKURU LETTER SHA

1192B
DIVES AKURU LETTER SSA

1192C
DIVES AKURU LETTER SA

1192D
DIVES AKURU LETTER HA

1192E
DIVES AKURU LETTER LLA

1192F
DIVES AKURU LETTER ZA

11930
DIVES AKURU VOWEL SIGN AA

11931
DIVES AKURU VOWEL SIGN I

11932
DIVES AKURU VOWEL SIGN II

11933
DIVES AKURU VOWEL SIGN U

11934
DIVES AKURU VOWEL SIGN UU

11935
DIVES AKURU VOWEL SIGN E

11937
DIVES AKURU VOWEL SIGN AI

11938
DIVES AKURU VOWEL SIGN O

1193B
DIVES AKURU SIGN ANUSVARA

1193C
DIVES AKURU SIGN CANDRABINDU

1193D
DIVES AKURU SIGN HALANTA

1193E
DIVES AKURU VIRAMA

1193F
DIVES AKURU PREFIXED NASAL SIGN

11940
DIVES AKURU MEDIAL YA

11941
DIVES AKURU INITIAL RA

11942
DIVES AKURU MEDIAL RA

11943
DIVES AKURU SIGN NUKTA

11944
DIVES AKURU DOUBLE DANDA

11945
DIVES AKURU GAP FILLER

11946
DIVES AKURU END OF TEXT MARK

11950
DIVES AKURU DIGIT ZERO

11951
DIVES AKURU DIGIT ONE

11952
DIVES AKURU DIGIT TWO

11953
DIVES AKURU DIGIT THREE

11954
DIVES AKURU DIGIT FOUR

11955
DIVES AKURU DIGIT FIVE

11956
DIVES AKURU DIGIT SIX

11957
DIVES AKURU DIGIT SEVEN

11958
DIVES AKURU DIGIT EIGHT

11959
DIVES AKURU DIGIT NINE

119A0
NANDINAGARI LETTER A

119A1
NANDINAGARI LETTER AA

119A2
NANDINAGARI LETTER I

119A3
NANDINAGARI LETTER II

119A4
NANDINAGARI LETTER U

119A5
NANDINAGARI LETTER UU

119A6
NANDINAGARI LETTER VOCALIC R

119A7
NANDINAGARI LETTER VOCALIC RR

119AA
NANDINAGARI LETTER E

119AB
NANDINAGARI LETTER AI

119AC
NANDINAGARI LETTER O

119AD
NANDINAGARI LETTER AU

119AE
NANDINAGARI LETTER KA

119AF
NANDINAGARI LETTER KHA

119B0
NANDINAGARI LETTER GA

119B1
NANDINAGARI LETTER GHA

119B2
NANDINAGARI LETTER NGA

119B3
NANDINAGARI LETTER CA

119B4
NANDINAGARI LETTER CHA

119B5
NANDINAGARI LETTER JA

119B6
NANDINAGARI LETTER JHA

119B7
NANDINAGARI LETTER NYA

119B8
NANDINAGARI LETTER TTA

119B9
NANDINAGARI LETTER TTHA

119BA
NANDINAGARI LETTER DDA

119BB
NANDINAGARI LETTER DDHA

119BC
NANDINAGARI LETTER NNA

119BD
NANDINAGARI LETTER TA

119BE
NANDINAGARI LETTER THA

119BF
NANDINAGARI LETTER DA

119C0
NANDINAGARI LETTER DHA

119C1
NANDINAGARI LETTER NA

119C2
NANDINAGARI LETTER PA

119C3
NANDINAGARI LETTER PHA

119C4
NANDINAGARI LETTER BA

119C5
NANDINAGARI LETTER BHA

119C6
NANDINAGARI LETTER MA

119C7
NANDINAGARI LETTER YA

119C8
NANDINAGARI LETTER RA

119C9
NANDINAGARI LETTER LA

119CA
NANDINAGARI LETTER VA

119CB
NANDINAGARI LETTER SHA

119CC
NANDINAGARI LETTER SSA

119CD
NANDINAGARI LETTER SA

119CE
NANDINAGARI LETTER HA

119CF
NANDINAGARI LETTER LLA

119D0
NANDINAGARI LETTER RRA

119D1
NANDINAGARI VOWEL SIGN AA

119D2
NANDINAGARI VOWEL SIGN I

119D3
NANDINAGARI VOWEL SIGN II

119D4
NANDINAGARI VOWEL SIGN U

119D5
NANDINAGARI VOWEL SIGN UU

119D6
NANDINAGARI VOWEL SIGN VOCALIC R

119D7
NANDINAGARI VOWEL SIGN VOCALIC RR

119DA
NANDINAGARI VOWEL SIGN E

119DB
NANDINAGARI VOWEL SIGN AI

119DC
NANDINAGARI VOWEL SIGN O

119DD
NANDINAGARI VOWEL SIGN AU

119DE
NANDINAGARI SIGN ANUSVARA

119DF
NANDINAGARI SIGN VISARGA

119E0
NANDINAGARI SIGN VIRAMA

119E1
NANDINAGARI SIGN AVAGRAHA

119E2
NANDINAGARI SIGN SIDDHAM

119E3
NANDINAGARI HEADSTROKE

119E4
NANDINAGARI VOWEL SIGN PRISHTHAMATRA E

11A00
ZANABAZAR SQUARE LETTER A

11A01
ZANABAZAR SQUARE VOWEL SIGN I

11A02
ZANABAZAR SQUARE VOWEL SIGN UE

11A03
ZANABAZAR SQUARE VOWEL SIGN U

11A04
ZANABAZAR SQUARE VOWEL SIGN E

11A05
ZANABAZAR SQUARE VOWEL SIGN OE

11A06
ZANABAZAR SQUARE VOWEL SIGN O

11A07
ZANABAZAR SQUARE VOWEL SIGN AI

11A08
ZANABAZAR SQUARE VOWEL SIGN AU

11A09
ZANABAZAR SQUARE VOWEL SIGN REVERSED I

11A0A
ZANABAZAR SQUARE VOWEL LENGTH MARK

11A0B
ZANABAZAR SQUARE LETTER KA

11A0C
ZANABAZAR SQUARE LETTER KHA

11A0D
ZANABAZAR SQUARE LETTER GA

11A0E
ZANABAZAR SQUARE LETTER GHA

11A0F
ZANABAZAR SQUARE LETTER NGA

11A10
ZANABAZAR SQUARE LETTER CA

11A11
ZANABAZAR SQUARE LETTER CHA

11A12
ZANABAZAR SQUARE LETTER JA

11A13
ZANABAZAR SQUARE LETTER NYA

11A14
ZANABAZAR SQUARE LETTER TTA

11A15
ZANABAZAR SQUARE LETTER TTHA

11A16
ZANABAZAR SQUARE LETTER DDA

11A17
ZANABAZAR SQUARE LETTER DDHA

11A18
ZANABAZAR SQUARE LETTER NNA

11A19
ZANABAZAR SQUARE LETTER TA

11A1A
ZANABAZAR SQUARE LETTER THA

11A1B
ZANABAZAR SQUARE LETTER DA

11A1C
ZANABAZAR SQUARE LETTER DHA

11A1D
ZANABAZAR SQUARE LETTER NA

11A1E
ZANABAZAR SQUARE LETTER PA

11A1F
ZANABAZAR SQUARE LETTER PHA

11A20
ZANABAZAR SQUARE LETTER BA

11A21
ZANABAZAR SQUARE LETTER BHA

11A22
ZANABAZAR SQUARE LETTER MA

11A23
ZANABAZAR SQUARE LETTER TSA

11A24
ZANABAZAR SQUARE LETTER TSHA

11A25
ZANABAZAR SQUARE LETTER DZA

11A26
ZANABAZAR SQUARE LETTER DZHA

11A27
ZANABAZAR SQUARE LETTER ZHA

11A28
ZANABAZAR SQUARE LETTER ZA

11A29
ZANABAZAR SQUARE LETTER -A

11A2A
ZANABAZAR SQUARE LETTER YA

11A2B
ZANABAZAR SQUARE LETTER RA

11A2C
ZANABAZAR SQUARE LETTER LA

11A2D
ZANABAZAR SQUARE LETTER VA

11A2E
ZANABAZAR SQUARE LETTER SHA

11A2F
ZANABAZAR SQUARE LETTER SSA

11A30
ZANABAZAR SQUARE LETTER SA

11A31
ZANABAZAR SQUARE LETTER HA

11A32
ZANABAZAR SQUARE LETTER KSSA

11A33
ZANABAZAR SQUARE FINAL CONSONANT MARK

11A34
ZANABAZAR SQUARE SIGN VIRAMA

11A35
ZANABAZAR SQUARE SIGN CANDRABINDU

11A36
ZANABAZAR SQUARE SIGN CANDRABINDU WITH ORNAMENT

11A37
ZANABAZAR SQUARE SIGN CANDRA WITH ORNAMENT

11A38
ZANABAZAR SQUARE SIGN ANUSVARA

11A39
ZANABAZAR SQUARE SIGN VISARGA

11A3A
ZANABAZAR SQUARE CLUSTER-INITIAL LETTER RA

11A3B
ZANABAZAR SQUARE CLUSTER-FINAL LETTER YA

11A3C
ZANABAZAR SQUARE CLUSTER-FINAL LETTER RA

11A3D
ZANABAZAR SQUARE CLUSTER-FINAL LETTER LA

11A3E
ZANABAZAR SQUARE CLUSTER-FINAL LETTER VA

11A3F
ZANABAZAR SQUARE INITIAL HEAD MARK

11A40
ZANABAZAR SQUARE CLOSING HEAD MARK

11A41
ZANABAZAR SQUARE MARK TSHEG

11A42
ZANABAZAR SQUARE MARK SHAD

11A43
ZANABAZAR SQUARE MARK DOUBLE SHAD

11A44
ZANABAZAR SQUARE MARK LONG TSHEG

11A45
ZANABAZAR SQUARE INITIAL DOUBLE-LINED HEAD MARK

11A46
ZANABAZAR SQUARE CLOSING DOUBLE-LINED HEAD MARK

11A47
ZANABAZAR SQUARE SUBJOINER

11A50
SOYOMBO LETTER A

11A51
SOYOMBO VOWEL SIGN I

11A52
SOYOMBO VOWEL SIGN UE

11A53
SOYOMBO VOWEL SIGN U

11A54
SOYOMBO VOWEL SIGN E

11A55
SOYOMBO VOWEL SIGN O

11A56
SOYOMBO VOWEL SIGN OE

11A57
SOYOMBO VOWEL SIGN AI

11A58
SOYOMBO VOWEL SIGN AU

11A59
SOYOMBO VOWEL SIGN VOCALIC R

11A5A
SOYOMBO VOWEL SIGN VOCALIC L

11A5B
SOYOMBO VOWEL LENGTH MARK

11A5C
SOYOMBO LETTER KA

11A5D
SOYOMBO LETTER KHA

11A5E
SOYOMBO LETTER GA

11A5F
SOYOMBO LETTER GHA

11A60
SOYOMBO LETTER NGA

11A61
SOYOMBO LETTER CA

11A62
SOYOMBO LETTER CHA

11A63
SOYOMBO LETTER JA

11A64
SOYOMBO LETTER JHA

11A65
SOYOMBO LETTER NYA

11A66
SOYOMBO LETTER TTA

11A67
SOYOMBO LETTER TTHA

11A68
SOYOMBO LETTER DDA

11A69
SOYOMBO LETTER DDHA

11A6A
SOYOMBO LETTER NNA

11A6B
SOYOMBO LETTER TA

11A6C
SOYOMBO LETTER THA

11A6D
SOYOMBO LETTER DA

11A6E
SOYOMBO LETTER DHA

11A6F
SOYOMBO LETTER NA

11A70
SOYOMBO LETTER PA

11A71
SOYOMBO LETTER PHA

11A72
SOYOMBO LETTER BA

11A73
SOYOMBO LETTER BHA

11A74
SOYOMBO LETTER MA

11A75
SOYOMBO LETTER TSA

11A76
SOYOMBO LETTER TSHA

11A77
SOYOMBO LETTER DZA

11A78
SOYOMBO LETTER ZHA

11A79
SOYOMBO LETTER ZA

11A7A
SOYOMBO LETTER -A

11A7B
SOYOMBO LETTER YA

11A7C
SOYOMBO LETTER RA

11A7D
SOYOMBO LETTER LA

11A7E
SOYOMBO LETTER VA

11A7F
SOYOMBO LETTER SHA

11A80
SOYOMBO LETTER SSA

11A81
SOYOMBO LETTER SA

11A82
SOYOMBO LETTER HA

11A83
SOYOMBO LETTER KSSA

11A84
SOYOMBO SIGN JIHVAMULIYA

11A85
SOYOMBO SIGN UPADHMANIYA

11A86
SOYOMBO CLUSTER-INITIAL LETTER RA

11A87
SOYOMBO CLUSTER-INITIAL LETTER LA

11A88
SOYOMBO CLUSTER-INITIAL LETTER SHA

11A89
SOYOMBO CLUSTER-INITIAL LETTER SA

11A8A
SOYOMBO FINAL CONSONANT SIGN G

11A8B
SOYOMBO FINAL CONSONANT SIGN K

11A8C
SOYOMBO FINAL CONSONANT SIGN NG

11A8D
SOYOMBO FINAL CONSONANT SIGN D

11A8E
SOYOMBO FINAL CONSONANT SIGN N

11A8F
SOYOMBO FINAL CONSONANT SIGN B

11A90
SOYOMBO FINAL CONSONANT SIGN M

11A91
SOYOMBO FINAL CONSONANT SIGN R

11A92
SOYOMBO FINAL CONSONANT SIGN L

11A93
SOYOMBO FINAL CONSONANT SIGN SH

11A94
SOYOMBO FINAL CONSONANT SIGN S

11A95
SOYOMBO FINAL CONSONANT SIGN -A

11A96
SOYOMBO SIGN ANUSVARA

11A97
SOYOMBO SIGN VISARGA

11A98
SOYOMBO GEMINATION MARK

11A99
SOYOMBO SUBJOINER

11A9A
SOYOMBO MARK TSHEG

11A9B
SOYOMBO MARK SHAD

11A9C
SOYOMBO MARK DOUBLE SHAD

11A9D
SOYOMBO MARK PLUTA

11A9E
SOYOMBO HEAD MARK WITH MOON AND SUN AND TRIPLE FLAME

11A9F
SOYOMBO HEAD MARK WITH MOON AND SUN AND FLAME

11AA0
SOYOMBO HEAD MARK WITH MOON AND SUN

11AA1
SOYOMBO TERMINAL MARK-1

11AA2
SOYOMBO TERMINAL MARK-2

11AB0
CANADIAN SYLLABICS NATTILIK HI

11AB1
CANADIAN SYLLABICS NATTILIK HII

11AB2
CANADIAN SYLLABICS NATTILIK HO

11AB3
CANADIAN SYLLABICS NATTILIK HOO

11AB4
CANADIAN SYLLABICS NATTILIK HA

11AB5
CANADIAN SYLLABICS NATTILIK HAA

11AB6
CANADIAN SYLLABICS NATTILIK SHRI

11AB7
CANADIAN SYLLABICS NATTILIK SHRII

11AB8
CANADIAN SYLLABICS NATTILIK SHRO

11AB9
CANADIAN SYLLABICS NATTILIK SHROO

11ABA
CANADIAN SYLLABICS NATTILIK SHRA

11ABB
CANADIAN SYLLABICS NATTILIK SHRAA

11ABC
CANADIAN SYLLABICS SPE

11ABD
CANADIAN SYLLABICS SPI

11ABE
CANADIAN SYLLABICS SPO

11ABF
CANADIAN SYLLABICS SPA

11AC0
PAU CIN HAU LETTER PA

11AC1
PAU CIN HAU LETTER KA

11AC2
PAU CIN HAU LETTER LA

11AC3
PAU CIN HAU LETTER MA

11AC4
PAU CIN HAU LETTER DA

11AC5
PAU CIN HAU LETTER ZA

11AC6
PAU CIN HAU LETTER VA

11AC7
PAU CIN HAU LETTER NGA

11AC8
PAU CIN HAU LETTER HA

11AC9
PAU CIN HAU LETTER GA

11ACA
PAU CIN HAU LETTER KHA

11ACB
PAU CIN HAU LETTER SA

11ACC
PAU CIN HAU LETTER BA

11ACD
PAU CIN HAU LETTER CA

11ACE
PAU CIN HAU LETTER TA

11ACF
PAU CIN HAU LETTER THA

11AD0
PAU CIN HAU LETTER NA

11AD1
PAU CIN HAU LETTER PHA

11AD2
PAU CIN HAU LETTER RA

11AD3
PAU CIN HAU LETTER FA

11AD4
PAU CIN HAU LETTER CHA

11AD5
PAU CIN HAU LETTER A

11AD6
PAU CIN HAU LETTER E

11AD7
PAU CIN HAU LETTER I

11AD8
PAU CIN HAU LETTER O

11AD9
PAU CIN HAU LETTER U

11ADA
PAU CIN HAU LETTER UA

11ADB
PAU CIN HAU LETTER IA

11ADC
PAU CIN HAU LETTER FINAL P

11ADD
PAU CIN HAU LETTER FINAL K

11ADE
PAU CIN HAU LETTER FINAL T

11ADF
PAU CIN HAU LETTER FINAL M

11AE0
PAU CIN HAU LETTER FINAL N

11AE1
PAU CIN HAU LETTER FINAL L

11AE2
PAU CIN HAU LETTER FINAL W

11AE3
PAU CIN HAU LETTER FINAL NG

11AE4
PAU CIN HAU LETTER FINAL Y

11AE5
PAU CIN HAU RISING TONE LONG

11AE6
PAU CIN HAU RISING TONE

11AE7
PAU CIN HAU SANDHI GLOTTAL STOP

11AE8
PAU CIN HAU RISING TONE LONG FINAL

11AE9
PAU CIN HAU RISING TONE FINAL

11AEA
PAU CIN HAU SANDHI GLOTTAL STOP FINAL

11AEB
PAU CIN HAU SANDHI TONE LONG

11AEC
PAU CIN HAU SANDHI TONE

11AED
PAU CIN HAU SANDHI TONE LONG FINAL

11AEE
PAU CIN HAU SANDHI TONE FINAL

11AEF
PAU CIN HAU MID-LEVEL TONE

11AF0
PAU CIN HAU GLOTTAL STOP VARIANT

11AF1
PAU CIN HAU MID-LEVEL TONE LONG FINAL

11AF2
PAU CIN HAU MID-LEVEL TONE FINAL

11AF3
PAU CIN HAU LOW-FALLING TONE LONG

11AF4
PAU CIN HAU LOW-FALLING TONE

11AF5
PAU CIN HAU GLOTTAL STOP

11AF6
PAU CIN HAU LOW-FALLING TONE LONG FINAL

11AF7
PAU CIN HAU LOW-FALLING TONE FINAL

11AF8
PAU CIN HAU GLOTTAL STOP FINAL

11B00
DEVANAGARI HEAD MARK

11B01
DEVANAGARI HEAD MARK WITH HEADSTROKE

11B02
DEVANAGARI SIGN BHALE

11B03
DEVANAGARI SIGN BHALE WITH HOOK

11B04
DEVANAGARI SIGN EXTENDED BHALE

11B05
DEVANAGARI SIGN EXTENDED BHALE WITH HOOK

11B06
DEVANAGARI SIGN WESTERN FIVE-LIKE BHALE

11B07
DEVANAGARI SIGN WESTERN NINE-LIKE BHALE

11B08
DEVANAGARI SIGN REVERSED NINE-LIKE BHALE

11B09
DEVANAGARI SIGN MINDU

11B60
SHARADA VOWEL SIGN OE

11B61
SHARADA VOWEL SIGN OOE

11B62
SHARADA VOWEL SIGN UE

11B63
SHARADA VOWEL SIGN UUE

11B64
SHARADA VOWEL SIGN SHORT E

11B65
SHARADA VOWEL SIGN SHORT O

11B66
SHARADA VOWEL SIGN CANDRA E

11B67
SHARADA VOWEL SIGN CANDRA O

11BC0
SUNUWAR LETTER DEVI

11BC1
SUNUWAR LETTER TASLA

11BC2
SUNUWAR LETTER EKO

11BC3
SUNUWAR LETTER IMAR

11BC4
SUNUWAR LETTER REU

11BC5
SUNUWAR LETTER UTTHI

11BC6
SUNUWAR LETTER KIK

11BC7
SUNUWAR LETTER MA

11BC8
SUNUWAR LETTER APPHO

11BC9
SUNUWAR LETTER PIP

11BCA
SUNUWAR LETTER GIL

11BCB
SUNUWAR LETTER HAMSO

11BCC
SUNUWAR LETTER CARMI

11BCD
SUNUWAR LETTER NAH

11BCE
SUNUWAR LETTER BUR

11BCF
SUNUWAR LETTER JYAH

11BD0
SUNUWAR LETTER LOACHA

11BD1
SUNUWAR LETTER OTTHI

11BD2
SUNUWAR LETTER SHYELE

11BD3
SUNUWAR LETTER VARCA

11BD4
SUNUWAR LETTER YAT

11BD5
SUNUWAR LETTER AVA

11BD6
SUNUWAR LETTER AAL

11BD7
SUNUWAR LETTER DONGA

11BD8
SUNUWAR LETTER THARI

11BD9
SUNUWAR LETTER PHAR

11BDA
SUNUWAR LETTER NGAR

11BDB
SUNUWAR LETTER KHA

11BDC
SUNUWAR LETTER SHYER

11BDD
SUNUWAR LETTER CHELAP

11BDE
SUNUWAR LETTER TENTU

11BDF
SUNUWAR LETTER THELE

11BE0
SUNUWAR LETTER KLOKO

11BE1
SUNUWAR SIGN PVO

11BF0
SUNUWAR DIGIT ZERO

11BF1
SUNUWAR DIGIT ONE

11BF2
SUNUWAR DIGIT TWO

11BF3
SUNUWAR DIGIT THREE

11BF4
SUNUWAR DIGIT FOUR

11BF5
SUNUWAR DIGIT FIVE

11BF6
SUNUWAR DIGIT SIX

11BF7
SUNUWAR DIGIT SEVEN

11BF8
SUNUWAR DIGIT EIGHT

11BF9
SUNUWAR DIGIT NINE

11C00
BHAIKSUKI LETTER A

11C01
BHAIKSUKI LETTER AA

11C02
BHAIKSUKI LETTER I

11C03
BHAIKSUKI LETTER II

11C04
BHAIKSUKI LETTER U

11C05
BHAIKSUKI LETTER UU

11C06
BHAIKSUKI LETTER VOCALIC R

11C07
BHAIKSUKI LETTER VOCALIC RR

11C08
BHAIKSUKI LETTER VOCALIC L

11C0A
BHAIKSUKI LETTER E

11C0B
BHAIKSUKI LETTER AI

11C0C
BHAIKSUKI LETTER O

11C0D
BHAIKSUKI LETTER AU

11C0E
BHAIKSUKI LETTER KA

11C0F
BHAIKSUKI LETTER KHA

11C10
BHAIKSUKI LETTER GA

11C11
BHAIKSUKI LETTER GHA

11C12
BHAIKSUKI LETTER NGA

11C13
BHAIKSUKI LETTER CA

11C14
BHAIKSUKI LETTER CHA

11C15
BHAIKSUKI LETTER JA

11C16
BHAIKSUKI LETTER JHA

11C17
BHAIKSUKI LETTER NYA

11C18
BHAIKSUKI LETTER TTA

11C19
BHAIKSUKI LETTER TTHA

11C1A
BHAIKSUKI LETTER DDA

11C1B
BHAIKSUKI LETTER DDHA

11C1C
BHAIKSUKI LETTER NNA

11C1D
BHAIKSUKI LETTER TA

11C1E
BHAIKSUKI LETTER THA

11C1F
BHAIKSUKI LETTER DA

11C20
BHAIKSUKI LETTER DHA

11C21
BHAIKSUKI LETTER NA

11C22
BHAIKSUKI LETTER PA

11C23
BHAIKSUKI LETTER PHA

11C24
BHAIKSUKI LETTER BA

11C25
BHAIKSUKI LETTER BHA

11C26
BHAIKSUKI LETTER MA

11C27
BHAIKSUKI LETTER YA

11C28
BHAIKSUKI LETTER RA

11C29
BHAIKSUKI LETTER LA

11C2A
BHAIKSUKI LETTER VA

11C2B
BHAIKSUKI LETTER SHA

11C2C
BHAIKSUKI LETTER SSA

11C2D
BHAIKSUKI LETTER SA

11C2E
BHAIKSUKI LETTER HA

11C2F
BHAIKSUKI VOWEL SIGN AA

11C30
BHAIKSUKI VOWEL SIGN I

11C31
BHAIKSUKI VOWEL SIGN II

11C32
BHAIKSUKI VOWEL SIGN U

11C33
BHAIKSUKI VOWEL SIGN UU

11C34
BHAIKSUKI VOWEL SIGN VOCALIC R

11C35
BHAIKSUKI VOWEL SIGN VOCALIC RR

11C36
BHAIKSUKI VOWEL SIGN VOCALIC L

11C38
BHAIKSUKI VOWEL SIGN E

11C39
BHAIKSUKI VOWEL SIGN AI

11C3A
BHAIKSUKI VOWEL SIGN O

11C3B
BHAIKSUKI VOWEL SIGN AU

11C3C
BHAIKSUKI SIGN CANDRABINDU

11C3D
BHAIKSUKI SIGN ANUSVARA

11C3E
BHAIKSUKI SIGN VISARGA

11C3F
BHAIKSUKI SIGN VIRAMA

11C40
BHAIKSUKI SIGN AVAGRAHA

11C41
BHAIKSUKI DANDA

11C42
BHAIKSUKI DOUBLE DANDA

11C43
BHAIKSUKI WORD SEPARATOR

11C44
BHAIKSUKI GAP FILLER-1

11C45
BHAIKSUKI GAP FILLER-2

11C50
BHAIKSUKI DIGIT ZERO

11C51
BHAIKSUKI DIGIT ONE

11C52
BHAIKSUKI DIGIT TWO

11C53
BHAIKSUKI DIGIT THREE

11C54
BHAIKSUKI DIGIT FOUR

11C55
BHAIKSUKI DIGIT FIVE

11C56
BHAIKSUKI DIGIT SIX

11C57
BHAIKSUKI DIGIT SEVEN

11C58
BHAIKSUKI DIGIT EIGHT

11C59
BHAIKSUKI DIGIT NINE

11C5A
BHAIKSUKI NUMBER ONE

11C5B
BHAIKSUKI NUMBER TWO

11C5C
BHAIKSUKI NUMBER THREE

11C5D
BHAIKSUKI NUMBER FOUR

11C5E
BHAIKSUKI NUMBER FIVE

11C5F
BHAIKSUKI NUMBER SIX

11C60
BHAIKSUKI NUMBER SEVEN

11C61
BHAIKSUKI NUMBER EIGHT

11C62
BHAIKSUKI NUMBER NINE

11C63
BHAIKSUKI NUMBER TEN

11C64
BHAIKSUKI NUMBER TWENTY

11C65
BHAIKSUKI NUMBER THIRTY

11C66
BHAIKSUKI NUMBER FORTY

11C67
BHAIKSUKI NUMBER FIFTY

11C68
BHAIKSUKI NUMBER SIXTY

11C69
BHAIKSUKI NUMBER SEVENTY

11C6A
BHAIKSUKI NUMBER EIGHTY

11C6B
BHAIKSUKI NUMBER NINETY

11C6C
BHAIKSUKI HUNDREDS UNIT MARK

11C70
MARCHEN HEAD MARK

11C71
MARCHEN MARK SHAD

11C72
MARCHEN LETTER KA

11C73
MARCHEN LETTER KHA

11C74
MARCHEN LETTER GA

11C75
MARCHEN LETTER NGA

11C76
MARCHEN LETTER CA

11C77
MARCHEN LETTER CHA

11C78
MARCHEN LETTER JA

11C79
MARCHEN LETTER NYA

11C7A
MARCHEN LETTER TA

11C7B
MARCHEN LETTER THA

11C7C
MARCHEN LETTER DA

11C7D
MARCHEN LETTER NA

11C7E
MARCHEN LETTER PA

11C7F
MARCHEN LETTER PHA

11C80
MARCHEN LETTER BA

11C81
MARCHEN LETTER MA

11C82
MARCHEN LETTER TSA

11C83
MARCHEN LETTER TSHA

11C84
MARCHEN LETTER DZA

11C85
MARCHEN LETTER WA

11C86
MARCHEN LETTER ZHA

11C87
MARCHEN LETTER ZA

11C88
MARCHEN LETTER -A

11C89
MARCHEN LETTER YA

11C8A
MARCHEN LETTER RA

11C8B
MARCHEN LETTER LA

11C8C
MARCHEN LETTER SHA

11C8D
MARCHEN LETTER SA

11C8E
MARCHEN LETTER HA

11C8F
MARCHEN LETTER A

11C92
MARCHEN SUBJOINED LETTER KA

11C93
MARCHEN SUBJOINED LETTER KHA

11C94
MARCHEN SUBJOINED LETTER GA

11C95
MARCHEN SUBJOINED LETTER NGA

11C96
MARCHEN SUBJOINED LETTER CA

11C97
MARCHEN SUBJOINED LETTER CHA

11C98
MARCHEN SUBJOINED LETTER JA

11C99
MARCHEN SUBJOINED LETTER NYA

11C9A
MARCHEN SUBJOINED LETTER TA

11C9B
MARCHEN SUBJOINED LETTER THA

11C9C
MARCHEN SUBJOINED LETTER DA

11C9D
MARCHEN SUBJOINED LETTER NA

11C9E
MARCHEN SUBJOINED LETTER PA

11C9F
MARCHEN SUBJOINED LETTER PHA

11CA0
MARCHEN SUBJOINED LETTER BA

11CA1
MARCHEN SUBJOINED LETTER MA

11CA2
MARCHEN SUBJOINED LETTER TSA

11CA3
MARCHEN SUBJOINED LETTER TSHA

11CA4
MARCHEN SUBJOINED LETTER DZA

11CA5
MARCHEN SUBJOINED LETTER WA

11CA6
MARCHEN SUBJOINED LETTER ZHA

11CA7
MARCHEN SUBJOINED LETTER ZA

11CA9
MARCHEN SUBJOINED LETTER YA

11CAA
MARCHEN SUBJOINED LETTER RA

11CAB
MARCHEN SUBJOINED LETTER LA

11CAC
MARCHEN SUBJOINED LETTER SHA

11CAD
MARCHEN SUBJOINED LETTER SA

11CAE
MARCHEN SUBJOINED LETTER HA

11CAF
MARCHEN SUBJOINED LETTER A

11CB0
MARCHEN VOWEL SIGN AA

11CB1
MARCHEN VOWEL SIGN I

11CB2
MARCHEN VOWEL SIGN U

11CB3
MARCHEN VOWEL SIGN E

11CB4
MARCHEN VOWEL SIGN O

11CB5
MARCHEN SIGN ANUSVARA

11CB6
MARCHEN SIGN CANDRABINDU

11D00
MASARAM GONDI LETTER A

11D01
MASARAM GONDI LETTER AA

11D02
MASARAM GONDI LETTER I

11D03
MASARAM GONDI LETTER II

11D04
MASARAM GONDI LETTER U

11D05
MASARAM GONDI LETTER UU

11D06
MASARAM GONDI LETTER E

11D08
MASARAM GONDI LETTER AI

11D09
MASARAM GONDI LETTER O

11D0B
MASARAM GONDI LETTER AU

11D0C
MASARAM GONDI LETTER KA

11D0D
MASARAM GONDI LETTER KHA

11D0E
MASARAM GONDI LETTER GA

11D0F
MASARAM GONDI LETTER GHA

11D10
MASARAM GONDI LETTER NGA

11D11
MASARAM GONDI LETTER CA

11D12
MASARAM GONDI LETTER CHA

11D13
MASARAM GONDI LETTER JA

11D14
MASARAM GONDI LETTER JHA

11D15
MASARAM GONDI LETTER NYA

11D16
MASARAM GONDI LETTER TTA

11D17
MASARAM GONDI LETTER TTHA

11D18
MASARAM GONDI LETTER DDA

11D19
MASARAM GONDI LETTER DDHA

11D1A
MASARAM GONDI LETTER NNA

11D1B
MASARAM GONDI LETTER TA

11D1C
MASARAM GONDI LETTER THA

11D1D
MASARAM GONDI LETTER DA

11D1E
MASARAM GONDI LETTER DHA

11D1F
MASARAM GONDI LETTER NA

11D20
MASARAM GONDI LETTER PA

11D21
MASARAM GONDI LETTER PHA

11D22
MASARAM GONDI LETTER BA

11D23
MASARAM GONDI LETTER BHA

11D24
MASARAM GONDI LETTER MA

11D25
MASARAM GONDI LETTER YA

11D26
MASARAM GONDI LETTER RA

11D27
MASARAM GONDI LETTER LA

11D28
MASARAM GONDI LETTER VA

11D29
MASARAM GONDI LETTER SHA

11D2A
MASARAM GONDI LETTER SSA

11D2B
MASARAM GONDI LETTER SA

11D2C
MASARAM GONDI LETTER HA

11D2D
MASARAM GONDI LETTER LLA

11D2E
MASARAM GONDI LETTER KSSA

11D2F
MASARAM GONDI LETTER JNYA

11D30
MASARAM GONDI LETTER TRA

11D31
MASARAM GONDI VOWEL SIGN AA

11D32
MASARAM GONDI VOWEL SIGN I

11D33
MASARAM GONDI VOWEL SIGN II

11D34
MASARAM GONDI VOWEL SIGN U

11D35
MASARAM GONDI VOWEL SIGN UU

11D36
MASARAM GONDI VOWEL SIGN VOCALIC R

11D3A
MASARAM GONDI VOWEL SIGN E

11D3C
MASARAM GONDI VOWEL SIGN AI

11D3D
MASARAM GONDI VOWEL SIGN O

11D3F
MASARAM GONDI VOWEL SIGN AU

11D40
MASARAM GONDI SIGN ANUSVARA

11D41
MASARAM GONDI SIGN VISARGA

11D42
MASARAM GONDI SIGN NUKTA

11D43
MASARAM GONDI SIGN CANDRA

11D44
MASARAM GONDI SIGN HALANTA

11D45
MASARAM GONDI VIRAMA

11D46
MASARAM GONDI REPHA

11D47
MASARAM GONDI RA-KARA

11D50
MASARAM GONDI DIGIT ZERO

11D51
MASARAM GONDI DIGIT ONE

11D52
MASARAM GONDI DIGIT TWO

11D53
MASARAM GONDI DIGIT THREE

11D54
MASARAM GONDI DIGIT FOUR

11D55
MASARAM GONDI DIGIT FIVE

11D56
MASARAM GONDI DIGIT SIX

11D57
MASARAM GONDI DIGIT SEVEN

11D58
MASARAM GONDI DIGIT EIGHT

11D59
MASARAM GONDI DIGIT NINE

11D60
GUNJALA GONDI LETTER A

11D61
GUNJALA GONDI LETTER AA

11D62
GUNJALA GONDI LETTER I

11D63
GUNJALA GONDI LETTER II

11D64
GUNJALA GONDI LETTER U

11D65
GUNJALA GONDI LETTER UU

11D67
GUNJALA GONDI LETTER EE

11D68
GUNJALA GONDI LETTER AI

11D6A
GUNJALA GONDI LETTER OO

11D6B
GUNJALA GONDI LETTER AU

11D6C
GUNJALA GONDI LETTER YA

11D6D
GUNJALA GONDI LETTER VA

11D6E
GUNJALA GONDI LETTER BA

11D6F
GUNJALA GONDI LETTER BHA

11D70
GUNJALA GONDI LETTER MA

11D71
GUNJALA GONDI LETTER KA

11D72
GUNJALA GONDI LETTER KHA

11D73
GUNJALA GONDI LETTER TA

11D74
GUNJALA GONDI LETTER THA

11D75
GUNJALA GONDI LETTER LA

11D76
GUNJALA GONDI LETTER GA

11D77
GUNJALA GONDI LETTER GHA

11D78
GUNJALA GONDI LETTER DA

11D79
GUNJALA GONDI LETTER DHA

11D7A
GUNJALA GONDI LETTER NA

11D7B
GUNJALA GONDI LETTER CA

11D7C
GUNJALA GONDI LETTER CHA

11D7D
GUNJALA GONDI LETTER TTA

11D7E
GUNJALA GONDI LETTER TTHA

11D7F
GUNJALA GONDI LETTER LLA

11D80
GUNJALA GONDI LETTER JA

11D81
GUNJALA GONDI LETTER JHA

11D82
GUNJALA GONDI LETTER DDA

11D83
GUNJALA GONDI LETTER DDHA

11D84
GUNJALA GONDI LETTER NGA

11D85
GUNJALA GONDI LETTER PA

11D86
GUNJALA GONDI LETTER PHA

11D87
GUNJALA GONDI LETTER HA

11D88
GUNJALA GONDI LETTER RA

11D89
GUNJALA GONDI LETTER SA

11D8A
GUNJALA GONDI VOWEL SIGN AA

11D8B
GUNJALA GONDI VOWEL SIGN I

11D8C
GUNJALA GONDI VOWEL SIGN II

11D8D
GUNJALA GONDI VOWEL SIGN U

11D8E
GUNJALA GONDI VOWEL SIGN UU

11D90
GUNJALA GONDI VOWEL SIGN EE

11D91
GUNJALA GONDI VOWEL SIGN AI

11D93
GUNJALA GONDI VOWEL SIGN OO

11D94
GUNJALA GONDI VOWEL SIGN AU

11D95
GUNJALA GONDI SIGN ANUSVARA

11D96
GUNJALA GONDI SIGN VISARGA

11D97
GUNJALA GONDI VIRAMA

11D98
GUNJALA GONDI OM

11DA0
GUNJALA GONDI DIGIT ZERO

11DA1
GUNJALA GONDI DIGIT ONE

11DA2
GUNJALA GONDI DIGIT TWO

11DA3
GUNJALA GONDI DIGIT THREE

11DA4
GUNJALA GONDI DIGIT FOUR

11DA5
GUNJALA GONDI DIGIT FIVE

11DA6
GUNJALA GONDI DIGIT SIX

11DA7
GUNJALA GONDI DIGIT SEVEN

11DA8
GUNJALA GONDI DIGIT EIGHT

11DA9
GUNJALA GONDI DIGIT NINE

11DB0
TOLONG SIKI LETTER I

11DB1
TOLONG SIKI LETTER E

11DB2
TOLONG SIKI LETTER U

11DB3
TOLONG SIKI LETTER O

11DB4
TOLONG SIKI LETTER A

11DB5
TOLONG SIKI LETTER AA

11DB6
TOLONG SIKI LETTER P

11DB7
TOLONG SIKI LETTER PH

11DB8
TOLONG SIKI LETTER B

11DB9
TOLONG SIKI LETTER BH

11DBA
TOLONG SIKI LETTER M

11DBB
TOLONG SIKI LETTER T

11DBC
TOLONG SIKI LETTER TH

11DBD
TOLONG SIKI LETTER D

11DBE
TOLONG SIKI LETTER DH

11DBF
TOLONG SIKI LETTER N

11DC0
TOLONG SIKI LETTER TT

11DC1
TOLONG SIKI LETTER TTH

11DC2
TOLONG SIKI LETTER DD

11DC3
TOLONG SIKI LETTER DDH

11DC4
TOLONG SIKI LETTER NN

11DC5
TOLONG SIKI LETTER C

11DC6
TOLONG SIKI LETTER CH

11DC7
TOLONG SIKI LETTER J

11DC8
TOLONG SIKI LETTER JH

11DC9
TOLONG SIKI LETTER NY

11DCA
TOLONG SIKI LETTER K

11DCB
TOLONG SIKI LETTER KH

11DCC
TOLONG SIKI LETTER G

11DCD
TOLONG SIKI LETTER GH

11DCE
TOLONG SIKI LETTER NG

11DCF
TOLONG SIKI LETTER Y

11DD0
TOLONG SIKI LETTER R

11DD1
TOLONG SIKI LETTER L

11DD2
TOLONG SIKI LETTER V

11DD3
TOLONG SIKI LETTER NNY

11DD4
TOLONG SIKI LETTER S

11DD5
TOLONG SIKI LETTER H

11DD6
TOLONG SIKI LETTER X

11DD7
TOLONG SIKI LETTER RR

11DD8
TOLONG SIKI LETTER RRH

11DD9
TOLONG SIKI SIGN SELA

11DDA
TOLONG SIKI SIGN HECAKA

11DDB
TOLONG SIKI UNGGA

11DE0
TOLONG SIKI DIGIT ZERO

11DE1
TOLONG SIKI DIGIT ONE

11DE2
TOLONG SIKI DIGIT TWO

11DE3
TOLONG SIKI DIGIT THREE

11DE4
TOLONG SIKI DIGIT FOUR

11DE5
TOLONG SIKI DIGIT FIVE

11DE6
TOLONG SIKI DIGIT SIX

11DE7
TOLONG SIKI DIGIT SEVEN

11DE8
TOLONG SIKI DIGIT EIGHT

11DE9
TOLONG SIKI DIGIT NINE

11EE0
MAKASAR LETTER KA

11EE1
MAKASAR LETTER GA

11EE2
MAKASAR LETTER NGA

11EE3
MAKASAR LETTER PA

11EE4
MAKASAR LETTER BA

11EE5
MAKASAR LETTER MA

11EE6
MAKASAR LETTER TA

11EE7
MAKASAR LETTER DA

11EE8
MAKASAR LETTER NA

11EE9
MAKASAR LETTER CA

11EEA
MAKASAR LETTER JA

11EEB
MAKASAR LETTER NYA

11EEC
MAKASAR LETTER YA

11EED
MAKASAR LETTER RA

11EEE
MAKASAR LETTER LA

11EEF
MAKASAR LETTER VA

11EF0
MAKASAR LETTER SA

11EF1
MAKASAR LETTER A

11EF2
MAKASAR ANGKA

11EF3
MAKASAR VOWEL SIGN I

11EF4
MAKASAR VOWEL SIGN U

11EF5
MAKASAR VOWEL SIGN E

11EF6
MAKASAR VOWEL SIGN O

11EF7
MAKASAR PASSIMBANG

11EF8
MAKASAR END OF SECTION

11F00
KAWI SIGN CANDRABINDU

11F01
KAWI SIGN ANUSVARA

11F02
KAWI SIGN REPHA

11F03
KAWI SIGN VISARGA

11F04
KAWI LETTER A

11F05
KAWI LETTER AA

11F06
KAWI LETTER I

11F07
KAWI LETTER II

11F08
KAWI LETTER U

11F09
KAWI LETTER UU

11F0A
KAWI LETTER VOCALIC R

11F0B
KAWI LETTER VOCALIC RR

11F0C
KAWI LETTER VOCALIC L

11F0D
KAWI LETTER VOCALIC LL

11F0E
KAWI LETTER E

11F0F
KAWI LETTER AI

11F10
KAWI LETTER O

11F12
KAWI LETTER KA

11F13
KAWI LETTER KHA

11F14
KAWI LETTER GA

11F15
KAWI LETTER GHA

11F16
KAWI LETTER NGA

11F17
KAWI LETTER CA

11F18
KAWI LETTER CHA

11F19
KAWI LETTER JA

11F1A
KAWI LETTER JHA

11F1B
KAWI LETTER NYA

11F1C
KAWI LETTER TTA

11F1D
KAWI LETTER TTHA

11F1E
KAWI LETTER DDA

11F1F
KAWI LETTER DDHA

11F20
KAWI LETTER NNA

11F21
KAWI LETTER TA

11F22
KAWI LETTER THA

11F23
KAWI LETTER DA

11F24
KAWI LETTER DHA

11F25
KAWI LETTER NA

11F26
KAWI LETTER PA

11F27
KAWI LETTER PHA

11F28
KAWI LETTER BA

11F29
KAWI LETTER BHA

11F2A
KAWI LETTER MA

11F2B
KAWI LETTER YA

11F2C
KAWI LETTER RA

11F2D
KAWI LETTER LA

11F2E
KAWI LETTER WA

11F2F
KAWI LETTER SHA

11F30
KAWI LETTER SSA

11F31
KAWI LETTER SA

11F32
KAWI LETTER HA

11F33
KAWI LETTER JNYA

11F34
KAWI VOWEL SIGN AA

11F35
KAWI VOWEL SIGN ALTERNATE AA

11F36
KAWI VOWEL SIGN I

11F37
KAWI VOWEL SIGN II

11F38
KAWI VOWEL SIGN U

11F39
KAWI VOWEL SIGN UU

11F3A
KAWI VOWEL SIGN VOCALIC R

11F3E
KAWI VOWEL SIGN E

11F3F
KAWI VOWEL SIGN AI

11F40
KAWI VOWEL SIGN EU

11F41
KAWI SIGN KILLER

11F42
KAWI CONJOINER

11F43
KAWI DANDA

11F44
KAWI DOUBLE DANDA

11F45
KAWI PUNCTUATION SECTION MARKER

11F46
KAWI PUNCTUATION ALTERNATE SECTION MARKER

11F47
KAWI PUNCTUATION FLOWER

11F48
KAWI PUNCTUATION SPACE FILLER

11F49
KAWI PUNCTUATION DOT

11F4A
KAWI PUNCTUATION DOUBLE DOT

11F4B
KAWI PUNCTUATION TRIPLE DOT

11F4C
KAWI PUNCTUATION CIRCLE

11F4D
KAWI PUNCTUATION FILLED CIRCLE

11F4E
KAWI PUNCTUATION SPIRAL

11F4F
KAWI PUNCTUATION CLOSING SPIRAL

11F50
KAWI DIGIT ZERO

11F51
KAWI DIGIT ONE

11F52
KAWI DIGIT TWO

11F53
KAWI DIGIT THREE

11F54
KAWI DIGIT FOUR

11F55
KAWI DIGIT FIVE

11F56
KAWI DIGIT SIX

11F57
KAWI DIGIT SEVEN

11F58
KAWI DIGIT EIGHT

11F59
KAWI DIGIT NINE

11F5A
KAWI SIGN NUKTA

11FB0
LISU LETTER YHA

11FC0
TAMIL FRACTION ONE THREE-HUNDRED-AND-TWENTIETH

11FC1
TAMIL FRACTION ONE ONE-HUNDRED-AND-SIXTIETH

11FC2
TAMIL FRACTION ONE EIGHTIETH

11FC3
TAMIL FRACTION ONE SIXTY-FOURTH

11FC4
TAMIL FRACTION ONE FORTIETH

11FC5
TAMIL FRACTION ONE THIRTY-SECOND

11FC6
TAMIL FRACTION THREE EIGHTIETHS

11FC7
TAMIL FRACTION THREE SIXTY-FOURTHS

11FC8
TAMIL FRACTION ONE TWENTIETH

11FC9
TAMIL FRACTION ONE SIXTEENTH-1

11FCA
TAMIL FRACTION ONE SIXTEENTH-2

11FCB
TAMIL FRACTION ONE TENTH

11FCC
TAMIL FRACTION ONE EIGHTH

11FCD
TAMIL FRACTION THREE TWENTIETHS

11FCE
TAMIL FRACTION THREE SIXTEENTHS

11FCF
TAMIL FRACTION ONE FIFTH

11FD0
TAMIL FRACTION ONE QUARTER

11FD1
TAMIL FRACTION ONE HALF-1

11FD2
TAMIL FRACTION ONE HALF-2

11FD3
TAMIL FRACTION THREE QUARTERS

11FD4
TAMIL FRACTION DOWNSCALING FACTOR KIIZH

11FD5
TAMIL SIGN NEL

11FD6
TAMIL SIGN CEVITU

11FD7
TAMIL SIGN AAZHAAKKU

11FD8
TAMIL SIGN UZHAKKU

11FD9
TAMIL SIGN MUUVUZHAKKU

11FDA
TAMIL SIGN KURUNI

11FDB
TAMIL SIGN PATHAKKU

11FDC
TAMIL SIGN MUKKURUNI

11FDD
TAMIL SIGN KAACU

11FDE
TAMIL SIGN PANAM

11FDF
TAMIL SIGN PON

11FE0
TAMIL SIGN VARAAKAN

11FE1
TAMIL SIGN PAARAM

11FE2
TAMIL SIGN KUZHI

11FE3
TAMIL SIGN VELI

11FE4
TAMIL WET CULTIVATION SIGN

11FE5
TAMIL DRY CULTIVATION SIGN

11FE6
TAMIL LAND SIGN

11FE7
TAMIL SALT PAN SIGN

11FE8
TAMIL TRADITIONAL CREDIT SIGN

11FE9
TAMIL TRADITIONAL NUMBER SIGN

11FEA
TAMIL CURRENT SIGN

11FEB
TAMIL AND ODD SIGN

11FEC
TAMIL SPENT SIGN

11FED
TAMIL TOTAL SIGN

11FEE
TAMIL IN POSSESSION SIGN

11FEF
TAMIL STARTING FROM SIGN

11FF0
TAMIL SIGN MUTHALIYA

11FF1
TAMIL SIGN VAKAIYARAA

11FFF
TAMIL PUNCTUATION END OF TEXT

12000
CUNEIFORM SIGN A

12001
CUNEIFORM SIGN A TIMES A

12002
CUNEIFORM SIGN A TIMES BAD

12003
CUNEIFORM SIGN A TIMES GAN2 TENU

12004
CUNEIFORM SIGN A TIMES HA

12005
CUNEIFORM SIGN A TIMES IGI

12006
CUNEIFORM SIGN A TIMES LAGAR GUNU

12007
CUNEIFORM SIGN A TIMES MUSH

12008
CUNEIFORM SIGN A TIMES SAG

12009
CUNEIFORM SIGN A2

1200A
CUNEIFORM SIGN AB

1200B
CUNEIFORM SIGN AB TIMES ASH2

1200C
CUNEIFORM SIGN AB TIMES DUN3 GUNU

1200D
CUNEIFORM SIGN AB TIMES GAL

1200E
CUNEIFORM SIGN AB TIMES GAN2 TENU

1200F
CUNEIFORM SIGN AB TIMES HA

12010
CUNEIFORM SIGN AB TIMES IGI GUNU

12011
CUNEIFORM SIGN AB TIMES IMIN

12012
CUNEIFORM SIGN AB TIMES LAGAB

12013
CUNEIFORM SIGN AB TIMES SHESH

12014
CUNEIFORM SIGN AB TIMES U PLUS U PLUS U

12015
CUNEIFORM SIGN AB GUNU

12016
CUNEIFORM SIGN AB2

12017
CUNEIFORM SIGN AB2 TIMES BALAG

12018
CUNEIFORM SIGN AB2 TIMES GAN2 TENU

12019
CUNEIFORM SIGN AB2 TIMES ME PLUS EN

1201A
CUNEIFORM SIGN AB2 TIMES SHA3

1201B
CUNEIFORM SIGN AB2 TIMES TAK4

1201C
CUNEIFORM SIGN AD

1201D
CUNEIFORM SIGN AK

1201E
CUNEIFORM SIGN AK TIMES ERIN2

1201F
CUNEIFORM SIGN AK TIMES SHITA PLUS GISH

12020
CUNEIFORM SIGN AL

12021
CUNEIFORM SIGN AL TIMES AL

12022
CUNEIFORM SIGN AL TIMES DIM2

12023
CUNEIFORM SIGN AL TIMES GISH

12024
CUNEIFORM SIGN AL TIMES HA

12025
CUNEIFORM SIGN AL TIMES KAD3

12026
CUNEIFORM SIGN AL TIMES KI

12027
CUNEIFORM SIGN AL TIMES SHE

12028
CUNEIFORM SIGN AL TIMES USH

12029
CUNEIFORM SIGN ALAN

1202A
CUNEIFORM SIGN ALEPH

1202B
CUNEIFORM SIGN AMAR

1202C
CUNEIFORM SIGN AMAR TIMES SHE

1202D
CUNEIFORM SIGN AN

1202E
CUNEIFORM SIGN AN OVER AN

1202F
CUNEIFORM SIGN AN THREE TIMES

12030
CUNEIFORM SIGN AN PLUS NAGA OPPOSING AN PLUS NAGA

12031
CUNEIFORM SIGN AN PLUS NAGA SQUARED

12032
CUNEIFORM SIGN ANSHE

12033
CUNEIFORM SIGN APIN

12034
CUNEIFORM SIGN ARAD

12035
CUNEIFORM SIGN ARAD TIMES KUR

12036
CUNEIFORM SIGN ARKAB

12037
CUNEIFORM SIGN ASAL2

12038
CUNEIFORM SIGN ASH

12039
CUNEIFORM SIGN ASH ZIDA TENU

1203A
CUNEIFORM SIGN ASH KABA TENU

1203B
CUNEIFORM SIGN ASH OVER ASH TUG2 OVER TUG2 TUG2 OVER TUG2 PAP

1203C
CUNEIFORM SIGN ASH OVER ASH OVER ASH

1203D
CUNEIFORM SIGN ASH OVER ASH OVER ASH CROSSING ASH OVER ASH OVER ASH

1203E
CUNEIFORM SIGN ASH2

1203F
CUNEIFORM SIGN ASHGAB

12040
CUNEIFORM SIGN BA

12041
CUNEIFORM SIGN BAD

12042
CUNEIFORM SIGN BAG3

12043
CUNEIFORM SIGN BAHAR2

12044
CUNEIFORM SIGN BAL

12045
CUNEIFORM SIGN BAL OVER BAL

12046
CUNEIFORM SIGN BALAG

12047
CUNEIFORM SIGN BAR

12048
CUNEIFORM SIGN BARA2

12049
CUNEIFORM SIGN BI

1204A
CUNEIFORM SIGN BI TIMES A

1204B
CUNEIFORM SIGN BI TIMES GAR

1204C
CUNEIFORM SIGN BI TIMES IGI GUNU

1204D
CUNEIFORM SIGN BU

1204E
CUNEIFORM SIGN BU OVER BU AB

1204F
CUNEIFORM SIGN BU OVER BU UN

12050
CUNEIFORM SIGN BU CROSSING BU

12051
CUNEIFORM SIGN BULUG

12052
CUNEIFORM SIGN BULUG OVER BULUG

12053
CUNEIFORM SIGN BUR

12054
CUNEIFORM SIGN BUR2

12055
CUNEIFORM SIGN DA

12056
CUNEIFORM SIGN DAG

12057
CUNEIFORM SIGN DAG KISIM5 TIMES A PLUS MASH

12058
CUNEIFORM SIGN DAG KISIM5 TIMES AMAR

12059
CUNEIFORM SIGN DAG KISIM5 TIMES BALAG

1205A
CUNEIFORM SIGN DAG KISIM5 TIMES BI

1205B
CUNEIFORM SIGN DAG KISIM5 TIMES GA

1205C
CUNEIFORM SIGN DAG KISIM5 TIMES GA PLUS MASH

1205D
CUNEIFORM SIGN DAG KISIM5 TIMES GI

1205E
CUNEIFORM SIGN DAG KISIM5 TIMES GIR2

1205F
CUNEIFORM SIGN DAG KISIM5 TIMES GUD

12060
CUNEIFORM SIGN DAG KISIM5 TIMES HA

12061
CUNEIFORM SIGN DAG KISIM5 TIMES IR

12062
CUNEIFORM SIGN DAG KISIM5 TIMES IR PLUS LU

12063
CUNEIFORM SIGN DAG KISIM5 TIMES KAK

12064
CUNEIFORM SIGN DAG KISIM5 TIMES LA

12065
CUNEIFORM SIGN DAG KISIM5 TIMES LU

12066
CUNEIFORM SIGN DAG KISIM5 TIMES LU PLUS MASH2

12067
CUNEIFORM SIGN DAG KISIM5 TIMES LUM

12068
CUNEIFORM SIGN DAG KISIM5 TIMES NE

12069
CUNEIFORM SIGN DAG KISIM5 TIMES PAP PLUS PAP

1206A
CUNEIFORM SIGN DAG KISIM5 TIMES SI

1206B
CUNEIFORM SIGN DAG KISIM5 TIMES TAK4

1206C
CUNEIFORM SIGN DAG KISIM5 TIMES U2 PLUS GIR2

1206D
CUNEIFORM SIGN DAG KISIM5 TIMES USH

1206E
CUNEIFORM SIGN DAM

1206F
CUNEIFORM SIGN DAR

12070
CUNEIFORM SIGN DARA3

12071
CUNEIFORM SIGN DARA4

12072
CUNEIFORM SIGN DI

12073
CUNEIFORM SIGN DIB

12074
CUNEIFORM SIGN DIM

12075
CUNEIFORM SIGN DIM TIMES SHE

12076
CUNEIFORM SIGN DIM2

12077
CUNEIFORM SIGN DIN

12078
CUNEIFORM SIGN DIN KASKAL U GUNU DISH

12079
CUNEIFORM SIGN DISH

1207A
CUNEIFORM SIGN DU

1207B
CUNEIFORM SIGN DU OVER DU

1207C
CUNEIFORM SIGN DU GUNU

1207D
CUNEIFORM SIGN DU SHESHIG

1207E
CUNEIFORM SIGN DUB

1207F
CUNEIFORM SIGN DUB TIMES ESH2

12080
CUNEIFORM SIGN DUB2

12081
CUNEIFORM SIGN DUG

12082
CUNEIFORM SIGN DUGUD

12083
CUNEIFORM SIGN DUH

12084
CUNEIFORM SIGN DUN

12085
CUNEIFORM SIGN DUN3

12086
CUNEIFORM SIGN DUN3 GUNU

12087
CUNEIFORM SIGN DUN3 GUNU GUNU

12088
CUNEIFORM SIGN DUN4

12089
CUNEIFORM SIGN DUR2

1208A
CUNEIFORM SIGN E

1208B
CUNEIFORM SIGN E TIMES PAP

1208C
CUNEIFORM SIGN E OVER E NUN OVER NUN

1208D
CUNEIFORM SIGN E2

1208E
CUNEIFORM SIGN E2 TIMES A PLUS HA PLUS DA

1208F
CUNEIFORM SIGN E2 TIMES GAR

12090
CUNEIFORM SIGN E2 TIMES MI

12091
CUNEIFORM SIGN E2 TIMES SAL

12092
CUNEIFORM SIGN E2 TIMES SHE

12093
CUNEIFORM SIGN E2 TIMES U

12094
CUNEIFORM SIGN EDIN

12095
CUNEIFORM SIGN EGIR

12096
CUNEIFORM SIGN EL

12097
CUNEIFORM SIGN EN

12098
CUNEIFORM SIGN EN TIMES GAN2

12099
CUNEIFORM SIGN EN TIMES GAN2 TENU

1209A
CUNEIFORM SIGN EN TIMES ME

1209B
CUNEIFORM SIGN EN CROSSING EN

1209C
CUNEIFORM SIGN EN OPPOSING EN

1209D
CUNEIFORM SIGN EN SQUARED

1209E
CUNEIFORM SIGN EREN

1209F
CUNEIFORM SIGN ERIN2

120A0
CUNEIFORM SIGN ESH2

120A1
CUNEIFORM SIGN EZEN

120A2
CUNEIFORM SIGN EZEN TIMES A

120A3
CUNEIFORM SIGN EZEN TIMES A PLUS LAL

120A4
CUNEIFORM SIGN EZEN TIMES A PLUS LAL TIMES LAL

120A5
CUNEIFORM SIGN EZEN TIMES AN

120A6
CUNEIFORM SIGN EZEN TIMES BAD

120A7
CUNEIFORM SIGN EZEN TIMES DUN3 GUNU

120A8
CUNEIFORM SIGN EZEN TIMES DUN3 GUNU GUNU

120A9
CUNEIFORM SIGN EZEN TIMES HA

120AA
CUNEIFORM SIGN EZEN TIMES HA GUNU

120AB
CUNEIFORM SIGN EZEN TIMES IGI GUNU

120AC
CUNEIFORM SIGN EZEN TIMES KASKAL

120AD
CUNEIFORM SIGN EZEN TIMES KASKAL SQUARED

120AE
CUNEIFORM SIGN EZEN TIMES KU3

120AF
CUNEIFORM SIGN EZEN TIMES LA

120B0
CUNEIFORM SIGN EZEN TIMES LAL TIMES LAL

120B1
CUNEIFORM SIGN EZEN TIMES LI

120B2
CUNEIFORM SIGN EZEN TIMES LU

120B3
CUNEIFORM SIGN EZEN TIMES U2

120B4
CUNEIFORM SIGN EZEN TIMES UD

120B5
CUNEIFORM SIGN GA

120B6
CUNEIFORM SIGN GA GUNU

120B7
CUNEIFORM SIGN GA2

120B8
CUNEIFORM SIGN GA2 TIMES A PLUS DA PLUS HA

120B9
CUNEIFORM SIGN GA2 TIMES A PLUS HA

120BA
CUNEIFORM SIGN GA2 TIMES A PLUS IGI

120BB
CUNEIFORM SIGN GA2 TIMES AB2 TENU PLUS TAB

120BC
CUNEIFORM SIGN GA2 TIMES AN

120BD
CUNEIFORM SIGN GA2 TIMES ASH

120BE
CUNEIFORM SIGN GA2 TIMES ASH2 PLUS GAL

120BF
CUNEIFORM SIGN GA2 TIMES BAD

120C0
CUNEIFORM SIGN GA2 TIMES BAR PLUS RA

120C1
CUNEIFORM SIGN GA2 TIMES BUR

120C2
CUNEIFORM SIGN GA2 TIMES BUR PLUS RA

120C3
CUNEIFORM SIGN GA2 TIMES DA

120C4
CUNEIFORM SIGN GA2 TIMES DI

120C5
CUNEIFORM SIGN GA2 TIMES DIM TIMES SHE

120C6
CUNEIFORM SIGN GA2 TIMES DUB

120C7
CUNEIFORM SIGN GA2 TIMES EL

120C8
CUNEIFORM SIGN GA2 TIMES EL PLUS LA

120C9
CUNEIFORM SIGN GA2 TIMES EN

120CA
CUNEIFORM SIGN GA2 TIMES EN TIMES GAN2 TENU

120CB
CUNEIFORM SIGN GA2 TIMES GAN2 TENU

120CC
CUNEIFORM SIGN GA2 TIMES GAR

120CD
CUNEIFORM SIGN GA2 TIMES GI

120CE
CUNEIFORM SIGN GA2 TIMES GI4

120CF
CUNEIFORM SIGN GA2 TIMES GI4 PLUS A

120D0
CUNEIFORM SIGN GA2 TIMES GIR2 PLUS SU

120D1
CUNEIFORM SIGN GA2 TIMES HA PLUS LU PLUS ESH2

120D2
CUNEIFORM SIGN GA2 TIMES HAL

120D3
CUNEIFORM SIGN GA2 TIMES HAL PLUS LA

120D4
CUNEIFORM SIGN GA2 TIMES HI PLUS LI

120D5
CUNEIFORM SIGN GA2 TIMES HUB2

120D6
CUNEIFORM SIGN GA2 TIMES IGI GUNU

120D7
CUNEIFORM SIGN GA2 TIMES ISH PLUS HU PLUS ASH

120D8
CUNEIFORM SIGN GA2 TIMES KAK

120D9
CUNEIFORM SIGN GA2 TIMES KASKAL

120DA
CUNEIFORM SIGN GA2 TIMES KID

120DB
CUNEIFORM SIGN GA2 TIMES KID PLUS LAL

120DC
CUNEIFORM SIGN GA2 TIMES KU3 PLUS AN

120DD
CUNEIFORM SIGN GA2 TIMES LA

120DE
CUNEIFORM SIGN GA2 TIMES ME PLUS EN

120DF
CUNEIFORM SIGN GA2 TIMES MI

120E0
CUNEIFORM SIGN GA2 TIMES NUN

120E1
CUNEIFORM SIGN GA2 TIMES NUN OVER NUN

120E2
CUNEIFORM SIGN GA2 TIMES PA

120E3
CUNEIFORM SIGN GA2 TIMES SAL

120E4
CUNEIFORM SIGN GA2 TIMES SAR

120E5
CUNEIFORM SIGN GA2 TIMES SHE

120E6
CUNEIFORM SIGN GA2 TIMES SHE PLUS TUR

120E7
CUNEIFORM SIGN GA2 TIMES SHID

120E8
CUNEIFORM SIGN GA2 TIMES SUM

120E9
CUNEIFORM SIGN GA2 TIMES TAK4

120EA
CUNEIFORM SIGN GA2 TIMES U

120EB
CUNEIFORM SIGN GA2 TIMES UD

120EC
CUNEIFORM SIGN GA2 TIMES UD PLUS DU

120ED
CUNEIFORM SIGN GA2 OVER GA2

120EE
CUNEIFORM SIGN GABA

120EF
CUNEIFORM SIGN GABA CROSSING GABA

120F0
CUNEIFORM SIGN GAD

120F1
CUNEIFORM SIGN GAD OVER GAD GAR OVER GAR

120F2
CUNEIFORM SIGN GAL

120F3
CUNEIFORM SIGN GAL GAD OVER GAD GAR OVER GAR

120F4
CUNEIFORM SIGN GALAM

120F5
CUNEIFORM SIGN GAM

120F6
CUNEIFORM SIGN GAN

120F7
CUNEIFORM SIGN GAN2

120F8
CUNEIFORM SIGN GAN2 TENU

120F9
CUNEIFORM SIGN GAN2 OVER GAN2

120FA
CUNEIFORM SIGN GAN2 CROSSING GAN2

120FB
CUNEIFORM SIGN GAR

120FC
CUNEIFORM SIGN GAR3

120FD
CUNEIFORM SIGN GASHAN

120FE
CUNEIFORM SIGN GESHTIN

120FF
CUNEIFORM SIGN GESHTIN TIMES KUR

12100
CUNEIFORM SIGN GI

12101
CUNEIFORM SIGN GI TIMES E

12102
CUNEIFORM SIGN GI TIMES U

12103
CUNEIFORM SIGN GI CROSSING GI

12104
CUNEIFORM SIGN GI4

12105
CUNEIFORM SIGN GI4 OVER GI4

12106
CUNEIFORM SIGN GI4 CROSSING GI4

12107
CUNEIFORM SIGN GIDIM

12108
CUNEIFORM SIGN GIR2

12109
CUNEIFORM SIGN GIR2 GUNU

1210A
CUNEIFORM SIGN GIR3

1210B
CUNEIFORM SIGN GIR3 TIMES A PLUS IGI

1210C
CUNEIFORM SIGN GIR3 TIMES GAN2 TENU

1210D
CUNEIFORM SIGN GIR3 TIMES IGI

1210E
CUNEIFORM SIGN GIR3 TIMES LU PLUS IGI

1210F
CUNEIFORM SIGN GIR3 TIMES PA

12110
CUNEIFORM SIGN GISAL

12111
CUNEIFORM SIGN GISH

12112
CUNEIFORM SIGN GISH CROSSING GISH

12113
CUNEIFORM SIGN GISH TIMES BAD

12114
CUNEIFORM SIGN GISH TIMES TAK4

12115
CUNEIFORM SIGN GISH TENU

12116
CUNEIFORM SIGN GU

12117
CUNEIFORM SIGN GU CROSSING GU

12118
CUNEIFORM SIGN GU2

12119
CUNEIFORM SIGN GU2 TIMES KAK

1211A
CUNEIFORM SIGN GU2 TIMES KAK TIMES IGI GUNU

1211B
CUNEIFORM SIGN GU2 TIMES NUN

1211C
CUNEIFORM SIGN GU2 TIMES SAL PLUS TUG2

1211D
CUNEIFORM SIGN GU2 GUNU

1211E
CUNEIFORM SIGN GUD

1211F
CUNEIFORM SIGN GUD TIMES A PLUS KUR

12120
CUNEIFORM SIGN GUD TIMES KUR

12121
CUNEIFORM SIGN GUD OVER GUD LUGAL

12122
CUNEIFORM SIGN GUL

12123
CUNEIFORM SIGN GUM

12124
CUNEIFORM SIGN GUM TIMES SHE

12125
CUNEIFORM SIGN GUR

12126
CUNEIFORM SIGN GUR7

12127
CUNEIFORM SIGN GURUN

12128
CUNEIFORM SIGN GURUSH

12129
CUNEIFORM SIGN HA

1212A
CUNEIFORM SIGN HA TENU

1212B
CUNEIFORM SIGN HA GUNU

1212C
CUNEIFORM SIGN HAL

1212D
CUNEIFORM SIGN HI

1212E
CUNEIFORM SIGN HI TIMES ASH

1212F
CUNEIFORM SIGN HI TIMES ASH2

12130
CUNEIFORM SIGN HI TIMES BAD

12131
CUNEIFORM SIGN HI TIMES DISH

12132
CUNEIFORM SIGN HI TIMES GAD

12133
CUNEIFORM SIGN HI TIMES KIN

12134
CUNEIFORM SIGN HI TIMES NUN

12135
CUNEIFORM SIGN HI TIMES SHE

12136
CUNEIFORM SIGN HI TIMES U

12137
CUNEIFORM SIGN HU

12138
CUNEIFORM SIGN HUB2

12139
CUNEIFORM SIGN HUB2 TIMES AN

1213A
CUNEIFORM SIGN HUB2 TIMES HAL

1213B
CUNEIFORM SIGN HUB2 TIMES KASKAL

1213C
CUNEIFORM SIGN HUB2 TIMES LISH

1213D
CUNEIFORM SIGN HUB2 TIMES UD

1213E
CUNEIFORM SIGN HUL2

1213F
CUNEIFORM SIGN I

12140
CUNEIFORM SIGN I A

12141
CUNEIFORM SIGN IB

12142
CUNEIFORM SIGN IDIM

12143
CUNEIFORM SIGN IDIM OVER IDIM BUR

12144
CUNEIFORM SIGN IDIM OVER IDIM SQUARED

12145
CUNEIFORM SIGN IG

12146
CUNEIFORM SIGN IGI

12147
CUNEIFORM SIGN IGI DIB

12148
CUNEIFORM SIGN IGI RI

12149
CUNEIFORM SIGN IGI OVER IGI SHIR OVER SHIR UD OVER UD

1214A
CUNEIFORM SIGN IGI GUNU

1214B
CUNEIFORM SIGN IL

1214C
CUNEIFORM SIGN IL TIMES GAN2 TENU

1214D
CUNEIFORM SIGN IL2

1214E
CUNEIFORM SIGN IM

1214F
CUNEIFORM SIGN IM TIMES TAK4

12150
CUNEIFORM SIGN IM CROSSING IM

12151
CUNEIFORM SIGN IM OPPOSING IM

12152
CUNEIFORM SIGN IM SQUARED

12153
CUNEIFORM SIGN IMIN

12154
CUNEIFORM SIGN IN

12155
CUNEIFORM SIGN IR

12156
CUNEIFORM SIGN ISH

12157
CUNEIFORM SIGN KA

12158
CUNEIFORM SIGN KA TIMES A

12159
CUNEIFORM SIGN KA TIMES AD

1215A
CUNEIFORM SIGN KA TIMES AD PLUS KU3

1215B
CUNEIFORM SIGN KA TIMES ASH2

1215C
CUNEIFORM SIGN KA TIMES BAD

1215D
CUNEIFORM SIGN KA TIMES BALAG

1215E
CUNEIFORM SIGN KA TIMES BAR

1215F
CUNEIFORM SIGN KA TIMES BI

12160
CUNEIFORM SIGN KA TIMES ERIN2

12161
CUNEIFORM SIGN KA TIMES ESH2

12162
CUNEIFORM SIGN KA TIMES GA

12163
CUNEIFORM SIGN KA TIMES GAL

12164
CUNEIFORM SIGN KA TIMES GAN2 TENU

12165
CUNEIFORM SIGN KA TIMES GAR

12166
CUNEIFORM SIGN KA TIMES GAR PLUS SHA3 PLUS A

12167
CUNEIFORM SIGN KA TIMES GI

12168
CUNEIFORM SIGN KA TIMES GIR2

12169
CUNEIFORM SIGN KA TIMES GISH PLUS SAR

1216A
CUNEIFORM SIGN KA TIMES GISH CROSSING GISH

1216B
CUNEIFORM SIGN KA TIMES GU

1216C
CUNEIFORM SIGN KA TIMES GUR7

1216D
CUNEIFORM SIGN KA TIMES IGI

1216E
CUNEIFORM SIGN KA TIMES IM

1216F
CUNEIFORM SIGN KA TIMES KAK

12170
CUNEIFORM SIGN KA TIMES KI

12171
CUNEIFORM SIGN KA TIMES KID

12172
CUNEIFORM SIGN KA TIMES LI

12173
CUNEIFORM SIGN KA TIMES LU

12174
CUNEIFORM SIGN KA TIMES ME

12175
CUNEIFORM SIGN KA TIMES ME PLUS DU

12176
CUNEIFORM SIGN KA TIMES ME PLUS GI

12177
CUNEIFORM SIGN KA TIMES ME PLUS TE

12178
CUNEIFORM SIGN KA TIMES MI

12179
CUNEIFORM SIGN KA TIMES MI PLUS NUNUZ

1217A
CUNEIFORM SIGN KA TIMES NE

1217B
CUNEIFORM SIGN KA TIMES NUN

1217C
CUNEIFORM SIGN KA TIMES PI

1217D
CUNEIFORM SIGN KA TIMES RU

1217E
CUNEIFORM SIGN KA TIMES SA

1217F
CUNEIFORM SIGN KA TIMES SAR

12180
CUNEIFORM SIGN KA TIMES SHA

12181
CUNEIFORM SIGN KA TIMES SHE

12182
CUNEIFORM SIGN KA TIMES SHID

12183
CUNEIFORM SIGN KA TIMES SHU

12184
CUNEIFORM SIGN KA TIMES SIG

12185
CUNEIFORM SIGN KA TIMES SUHUR

12186
CUNEIFORM SIGN KA TIMES TAR

12187
CUNEIFORM SIGN KA TIMES U

12188
CUNEIFORM SIGN KA TIMES U2

12189
CUNEIFORM SIGN KA TIMES UD

1218A
CUNEIFORM SIGN KA TIMES UMUM TIMES PA

1218B
CUNEIFORM SIGN KA TIMES USH

1218C
CUNEIFORM SIGN KA TIMES ZI

1218D
CUNEIFORM SIGN KA2

1218E
CUNEIFORM SIGN KA2 CROSSING KA2

1218F
CUNEIFORM SIGN KAB

12190
CUNEIFORM SIGN KAD2

12191
CUNEIFORM SIGN KAD3

12192
CUNEIFORM SIGN KAD4

12193
CUNEIFORM SIGN KAD5

12194
CUNEIFORM SIGN KAD5 OVER KAD5

12195
CUNEIFORM SIGN KAK

12196
CUNEIFORM SIGN KAK TIMES IGI GUNU

12197
CUNEIFORM SIGN KAL

12198
CUNEIFORM SIGN KAL TIMES BAD

12199
CUNEIFORM SIGN KAL CROSSING KAL

1219A
CUNEIFORM SIGN KAM2

1219B
CUNEIFORM SIGN KAM4

1219C
CUNEIFORM SIGN KASKAL

1219D
CUNEIFORM SIGN KASKAL LAGAB TIMES U OVER LAGAB TIMES U

1219E
CUNEIFORM SIGN KASKAL OVER KASKAL LAGAB TIMES U OVER LAGAB TIMES U

1219F
CUNEIFORM SIGN KESH2

121A0
CUNEIFORM SIGN KI

121A1
CUNEIFORM SIGN KI TIMES BAD

121A2
CUNEIFORM SIGN KI TIMES U

121A3
CUNEIFORM SIGN KI TIMES UD

121A4
CUNEIFORM SIGN KID

121A5
CUNEIFORM SIGN KIN

121A6
CUNEIFORM SIGN KISAL

121A7
CUNEIFORM SIGN KISH

121A8
CUNEIFORM SIGN KISIM5

121A9
CUNEIFORM SIGN KISIM5 OVER KISIM5

121AA
CUNEIFORM SIGN KU

121AB
CUNEIFORM SIGN KU OVER HI TIMES ASH2 KU OVER HI TIMES ASH2

121AC
CUNEIFORM SIGN KU3

121AD
CUNEIFORM SIGN KU4

121AE
CUNEIFORM SIGN KU4 VARIANT FORM

121AF
CUNEIFORM SIGN KU7

121B0
CUNEIFORM SIGN KUL

121B1
CUNEIFORM SIGN KUL GUNU

121B2
CUNEIFORM SIGN KUN

121B3
CUNEIFORM SIGN KUR

121B4
CUNEIFORM SIGN KUR OPPOSING KUR

121B5
CUNEIFORM SIGN KUSHU2

121B6
CUNEIFORM SIGN KWU318

121B7
CUNEIFORM SIGN LA

121B8
CUNEIFORM SIGN LAGAB

121B9
CUNEIFORM SIGN LAGAB TIMES A

121BA
CUNEIFORM SIGN LAGAB TIMES A PLUS DA PLUS HA

121BB
CUNEIFORM SIGN LAGAB TIMES A PLUS GAR

121BC
CUNEIFORM SIGN LAGAB TIMES A PLUS LAL

121BD
CUNEIFORM SIGN LAGAB TIMES AL

121BE
CUNEIFORM SIGN LAGAB TIMES AN

121BF
CUNEIFORM SIGN LAGAB TIMES ASH ZIDA TENU

121C0
CUNEIFORM SIGN LAGAB TIMES BAD

121C1
CUNEIFORM SIGN LAGAB TIMES BI

121C2
CUNEIFORM SIGN LAGAB TIMES DAR

121C3
CUNEIFORM SIGN LAGAB TIMES EN

121C4
CUNEIFORM SIGN LAGAB TIMES GA

121C5
CUNEIFORM SIGN LAGAB TIMES GAR

121C6
CUNEIFORM SIGN LAGAB TIMES GUD

121C7
CUNEIFORM SIGN LAGAB TIMES GUD PLUS GUD

121C8
CUNEIFORM SIGN LAGAB TIMES HA

121C9
CUNEIFORM SIGN LAGAB TIMES HAL

121CA
CUNEIFORM SIGN LAGAB TIMES HI TIMES NUN

121CB
CUNEIFORM SIGN LAGAB TIMES IGI GUNU

121CC
CUNEIFORM SIGN LAGAB TIMES IM

121CD
CUNEIFORM SIGN LAGAB TIMES IM PLUS HA

121CE
CUNEIFORM SIGN LAGAB TIMES IM PLUS LU

121CF
CUNEIFORM SIGN LAGAB TIMES KI

121D0
CUNEIFORM SIGN LAGAB TIMES KIN

121D1
CUNEIFORM SIGN LAGAB TIMES KU3

121D2
CUNEIFORM SIGN LAGAB TIMES KUL

121D3
CUNEIFORM SIGN LAGAB TIMES KUL PLUS HI PLUS A

121D4
CUNEIFORM SIGN LAGAB TIMES LAGAB

121D5
CUNEIFORM SIGN LAGAB TIMES LISH

121D6
CUNEIFORM SIGN LAGAB TIMES LU

121D7
CUNEIFORM SIGN LAGAB TIMES LUL

121D8
CUNEIFORM SIGN LAGAB TIMES ME

121D9
CUNEIFORM SIGN LAGAB TIMES ME PLUS EN

121DA
CUNEIFORM SIGN LAGAB TIMES MUSH

121DB
CUNEIFORM SIGN LAGAB TIMES NE

121DC
CUNEIFORM SIGN LAGAB TIMES SHE PLUS SUM

121DD
CUNEIFORM SIGN LAGAB TIMES SHITA PLUS GISH PLUS ERIN2

121DE
CUNEIFORM SIGN LAGAB TIMES SHITA PLUS GISH TENU

121DF
CUNEIFORM SIGN LAGAB TIMES SHU2

121E0
CUNEIFORM SIGN LAGAB TIMES SHU2 PLUS SHU2

121E1
CUNEIFORM SIGN LAGAB TIMES SUM

121E2
CUNEIFORM SIGN LAGAB TIMES TAG

121E3
CUNEIFORM SIGN LAGAB TIMES TAK4

121E4
CUNEIFORM SIGN LAGAB TIMES TE PLUS A PLUS SU PLUS NA

121E5
CUNEIFORM SIGN LAGAB TIMES U

121E6
CUNEIFORM SIGN LAGAB TIMES U PLUS A

121E7
CUNEIFORM SIGN LAGAB TIMES U PLUS U PLUS U

121E8
CUNEIFORM SIGN LAGAB TIMES U2 PLUS ASH

121E9
CUNEIFORM SIGN LAGAB TIMES UD

121EA
CUNEIFORM SIGN LAGAB TIMES USH

121EB
CUNEIFORM SIGN LAGAB SQUARED

121EC
CUNEIFORM SIGN LAGAR

121ED
CUNEIFORM SIGN LAGAR TIMES SHE

121EE
CUNEIFORM SIGN LAGAR TIMES SHE PLUS SUM

121EF
CUNEIFORM SIGN LAGAR GUNU

121F0
CUNEIFORM SIGN LAGAR GUNU OVER LAGAR GUNU SHE

121F1
CUNEIFORM SIGN LAHSHU

121F2
CUNEIFORM SIGN LAL

121F3
CUNEIFORM SIGN LAL TIMES LAL

121F4
CUNEIFORM SIGN LAM

121F5
CUNEIFORM SIGN LAM TIMES KUR

121F6
CUNEIFORM SIGN LAM TIMES KUR PLUS RU

121F7
CUNEIFORM SIGN LI

121F8
CUNEIFORM SIGN LIL

121F9
CUNEIFORM SIGN LIMMU2

121FA
CUNEIFORM SIGN LISH

121FB
CUNEIFORM SIGN LU

121FC
CUNEIFORM SIGN LU TIMES BAD

121FD
CUNEIFORM SIGN LU2

121FE
CUNEIFORM SIGN LU2 TIMES AL

121FF
CUNEIFORM SIGN LU2 TIMES BAD

12200
CUNEIFORM SIGN LU2 TIMES ESH2

12201
CUNEIFORM SIGN LU2 TIMES ESH2 TENU

12202
CUNEIFORM SIGN LU2 TIMES GAN2 TENU

12203
CUNEIFORM SIGN LU2 TIMES HI TIMES BAD

12204
CUNEIFORM SIGN LU2 TIMES IM

12205
CUNEIFORM SIGN LU2 TIMES KAD2

12206
CUNEIFORM SIGN LU2 TIMES KAD3

12207
CUNEIFORM SIGN LU2 TIMES KAD3 PLUS ASH

12208
CUNEIFORM SIGN LU2 TIMES KI

12209
CUNEIFORM SIGN LU2 TIMES LA PLUS ASH

1220A
CUNEIFORM SIGN LU2 TIMES LAGAB

1220B
CUNEIFORM SIGN LU2 TIMES ME PLUS EN

1220C
CUNEIFORM SIGN LU2 TIMES NE

1220D
CUNEIFORM SIGN LU2 TIMES NU

1220E
CUNEIFORM SIGN LU2 TIMES SI PLUS ASH

1220F
CUNEIFORM SIGN LU2 TIMES SIK2 PLUS BU

12210
CUNEIFORM SIGN LU2 TIMES TUG2

12211
CUNEIFORM SIGN LU2 TENU

12212
CUNEIFORM SIGN LU2 CROSSING LU2

12213
CUNEIFORM SIGN LU2 OPPOSING LU2

12214
CUNEIFORM SIGN LU2 SQUARED

12215
CUNEIFORM SIGN LU2 SHESHIG

12216
CUNEIFORM SIGN LU3

12217
CUNEIFORM SIGN LUGAL

12218
CUNEIFORM SIGN LUGAL OVER LUGAL

12219
CUNEIFORM SIGN LUGAL OPPOSING LUGAL

1221A
CUNEIFORM SIGN LUGAL SHESHIG

1221B
CUNEIFORM SIGN LUH

1221C
CUNEIFORM SIGN LUL

1221D
CUNEIFORM SIGN LUM

1221E
CUNEIFORM SIGN LUM OVER LUM

1221F
CUNEIFORM SIGN LUM OVER LUM GAR OVER GAR

12220
CUNEIFORM SIGN MA

12221
CUNEIFORM SIGN MA TIMES TAK4

12222
CUNEIFORM SIGN MA GUNU

12223
CUNEIFORM SIGN MA2

12224
CUNEIFORM SIGN MAH

12225
CUNEIFORM SIGN MAR

12226
CUNEIFORM SIGN MASH

12227
CUNEIFORM SIGN MASH2

12228
CUNEIFORM SIGN ME

12229
CUNEIFORM SIGN MES

1222A
CUNEIFORM SIGN MI

1222B
CUNEIFORM SIGN MIN

1222C
CUNEIFORM SIGN MU

1222D
CUNEIFORM SIGN MU OVER MU

1222E
CUNEIFORM SIGN MUG

1222F
CUNEIFORM SIGN MUG GUNU

12230
CUNEIFORM SIGN MUNSUB

12231
CUNEIFORM SIGN MURGU2

12232
CUNEIFORM SIGN MUSH

12233
CUNEIFORM SIGN MUSH TIMES A

12234
CUNEIFORM SIGN MUSH TIMES KUR

12235
CUNEIFORM SIGN MUSH TIMES ZA

12236
CUNEIFORM SIGN MUSH OVER MUSH

12237
CUNEIFORM SIGN MUSH OVER MUSH TIMES A PLUS NA

12238
CUNEIFORM SIGN MUSH CROSSING MUSH

12239
CUNEIFORM SIGN MUSH3

1223A
CUNEIFORM SIGN MUSH3 TIMES A

1223B
CUNEIFORM SIGN MUSH3 TIMES A PLUS DI

1223C
CUNEIFORM SIGN MUSH3 TIMES DI

1223D
CUNEIFORM SIGN MUSH3 GUNU

1223E
CUNEIFORM SIGN NA

1223F
CUNEIFORM SIGN NA2

12240
CUNEIFORM SIGN NAGA

12241
CUNEIFORM SIGN NAGA INVERTED

12242
CUNEIFORM SIGN NAGA TIMES SHU TENU

12243
CUNEIFORM SIGN NAGA OPPOSING NAGA

12244
CUNEIFORM SIGN NAGAR

12245
CUNEIFORM SIGN NAM NUTILLU

12246
CUNEIFORM SIGN NAM

12247
CUNEIFORM SIGN NAM2

12248
CUNEIFORM SIGN NE

12249
CUNEIFORM SIGN NE TIMES A

1224A
CUNEIFORM SIGN NE TIMES UD

1224B
CUNEIFORM SIGN NE SHESHIG

1224C
CUNEIFORM SIGN NI

1224D
CUNEIFORM SIGN NI TIMES E

1224E
CUNEIFORM SIGN NI2

1224F
CUNEIFORM SIGN NIM

12250
CUNEIFORM SIGN NIM TIMES GAN2 TENU

12251
CUNEIFORM SIGN NIM TIMES GAR PLUS GAN2 TENU

12252
CUNEIFORM SIGN NINDA2

12253
CUNEIFORM SIGN NINDA2 TIMES AN

12254
CUNEIFORM SIGN NINDA2 TIMES ASH

12255
CUNEIFORM SIGN NINDA2 TIMES ASH PLUS ASH

12256
CUNEIFORM SIGN NINDA2 TIMES GUD

12257
CUNEIFORM SIGN NINDA2 TIMES ME PLUS GAN2 TENU

12258
CUNEIFORM SIGN NINDA2 TIMES NE

12259
CUNEIFORM SIGN NINDA2 TIMES NUN

1225A
CUNEIFORM SIGN NINDA2 TIMES SHE

1225B
CUNEIFORM SIGN NINDA2 TIMES SHE PLUS A AN

1225C
CUNEIFORM SIGN NINDA2 TIMES SHE PLUS ASH

1225D
CUNEIFORM SIGN NINDA2 TIMES SHE PLUS ASH PLUS ASH

1225E
CUNEIFORM SIGN NINDA2 TIMES U2 PLUS ASH

1225F
CUNEIFORM SIGN NINDA2 TIMES USH

12260
CUNEIFORM SIGN NISAG

12261
CUNEIFORM SIGN NU

12262
CUNEIFORM SIGN NU11

12263
CUNEIFORM SIGN NUN

12264
CUNEIFORM SIGN NUN LAGAR TIMES GAR

12265
CUNEIFORM SIGN NUN LAGAR TIMES MASH

12266
CUNEIFORM SIGN NUN LAGAR TIMES SAL

12267
CUNEIFORM SIGN NUN LAGAR TIMES SAL OVER NUN LAGAR TIMES SAL

12268
CUNEIFORM SIGN NUN LAGAR TIMES USH

12269
CUNEIFORM SIGN NUN TENU

1226A
CUNEIFORM SIGN NUN OVER NUN

1226B
CUNEIFORM SIGN NUN CROSSING NUN

1226C
CUNEIFORM SIGN NUN CROSSING NUN LAGAR OVER LAGAR

1226D
CUNEIFORM SIGN NUNUZ

1226E
CUNEIFORM SIGN NUNUZ AB2 TIMES ASHGAB

1226F
CUNEIFORM SIGN NUNUZ AB2 TIMES BI

12270
CUNEIFORM SIGN NUNUZ AB2 TIMES DUG

12271
CUNEIFORM SIGN NUNUZ AB2 TIMES GUD

12272
CUNEIFORM SIGN NUNUZ AB2 TIMES IGI GUNU

12273
CUNEIFORM SIGN NUNUZ AB2 TIMES KAD3

12274
CUNEIFORM SIGN NUNUZ AB2 TIMES LA

12275
CUNEIFORM SIGN NUNUZ AB2 TIMES NE

12276
CUNEIFORM SIGN NUNUZ AB2 TIMES SILA3

12277
CUNEIFORM SIGN NUNUZ AB2 TIMES U2

12278
CUNEIFORM SIGN NUNUZ KISIM5 TIMES BI

12279
CUNEIFORM SIGN NUNUZ KISIM5 TIMES BI U

1227A
CUNEIFORM SIGN PA

1227B
CUNEIFORM SIGN PAD

1227C
CUNEIFORM SIGN PAN

1227D
CUNEIFORM SIGN PAP

1227E
CUNEIFORM SIGN PESH2

1227F
CUNEIFORM SIGN PI

12280
CUNEIFORM SIGN PI TIMES A

12281
CUNEIFORM SIGN PI TIMES AB

12282
CUNEIFORM SIGN PI TIMES BI

12283
CUNEIFORM SIGN PI TIMES BU

12284
CUNEIFORM SIGN PI TIMES E

12285
CUNEIFORM SIGN PI TIMES I

12286
CUNEIFORM SIGN PI TIMES IB

12287
CUNEIFORM SIGN PI TIMES U

12288
CUNEIFORM SIGN PI TIMES U2

12289
CUNEIFORM SIGN PI CROSSING PI

1228A
CUNEIFORM SIGN PIRIG

1228B
CUNEIFORM SIGN PIRIG TIMES KAL

1228C
CUNEIFORM SIGN PIRIG TIMES UD

1228D
CUNEIFORM SIGN PIRIG TIMES ZA

1228E
CUNEIFORM SIGN PIRIG OPPOSING PIRIG

1228F
CUNEIFORM SIGN RA

12290
CUNEIFORM SIGN RAB

12291
CUNEIFORM SIGN RI

12292
CUNEIFORM SIGN RU

12293
CUNEIFORM SIGN SA

12294
CUNEIFORM SIGN SAG NUTILLU

12295
CUNEIFORM SIGN SAG

12296
CUNEIFORM SIGN SAG TIMES A

12297
CUNEIFORM SIGN SAG TIMES DU

12298
CUNEIFORM SIGN SAG TIMES DUB

12299
CUNEIFORM SIGN SAG TIMES HA

1229A
CUNEIFORM SIGN SAG TIMES KAK

1229B
CUNEIFORM SIGN SAG TIMES KUR

1229C
CUNEIFORM SIGN SAG TIMES LUM

1229D
CUNEIFORM SIGN SAG TIMES MI

1229E
CUNEIFORM SIGN SAG TIMES NUN

1229F
CUNEIFORM SIGN SAG TIMES SAL

122A0
CUNEIFORM SIGN SAG TIMES SHID

122A1
CUNEIFORM SIGN SAG TIMES TAB

122A2
CUNEIFORM SIGN SAG TIMES U2

122A3
CUNEIFORM SIGN SAG TIMES UB

122A4
CUNEIFORM SIGN SAG TIMES UM

122A5
CUNEIFORM SIGN SAG TIMES UR

122A6
CUNEIFORM SIGN SAG TIMES USH

122A7
CUNEIFORM SIGN SAG OVER SAG

122A8
CUNEIFORM SIGN SAG GUNU

122A9
CUNEIFORM SIGN SAL

122AA
CUNEIFORM SIGN SAL LAGAB TIMES ASH2

122AB
CUNEIFORM SIGN SANGA2

122AC
CUNEIFORM SIGN SAR

122AD
CUNEIFORM SIGN SHA

122AE
CUNEIFORM SIGN SHA3

122AF
CUNEIFORM SIGN SHA3 TIMES A

122B0
CUNEIFORM SIGN SHA3 TIMES BAD

122B1
CUNEIFORM SIGN SHA3 TIMES GISH

122B2
CUNEIFORM SIGN SHA3 TIMES NE

122B3
CUNEIFORM SIGN SHA3 TIMES SHU2

122B4
CUNEIFORM SIGN SHA3 TIMES TUR

122B5
CUNEIFORM SIGN SHA3 TIMES U

122B6
CUNEIFORM SIGN SHA3 TIMES U PLUS A

122B7
CUNEIFORM SIGN SHA6

122B8
CUNEIFORM SIGN SHAB6

122B9
CUNEIFORM SIGN SHAR2

122BA
CUNEIFORM SIGN SHE

122BB
CUNEIFORM SIGN SHE HU

122BC
CUNEIFORM SIGN SHE OVER SHE GAD OVER GAD GAR OVER GAR

122BD
CUNEIFORM SIGN SHE OVER SHE TAB OVER TAB GAR OVER GAR

122BE
CUNEIFORM SIGN SHEG9

122BF
CUNEIFORM SIGN SHEN

122C0
CUNEIFORM SIGN SHESH

122C1
CUNEIFORM SIGN SHESH2

122C2
CUNEIFORM SIGN SHESHLAM

122C3
CUNEIFORM SIGN SHID

122C4
CUNEIFORM SIGN SHID TIMES A

122C5
CUNEIFORM SIGN SHID TIMES IM

122C6
CUNEIFORM SIGN SHIM

122C7
CUNEIFORM SIGN SHIM TIMES A

122C8
CUNEIFORM SIGN SHIM TIMES BAL

122C9
CUNEIFORM SIGN SHIM TIMES BULUG

122CA
CUNEIFORM SIGN SHIM TIMES DIN

122CB
CUNEIFORM SIGN SHIM TIMES GAR

122CC
CUNEIFORM SIGN SHIM TIMES IGI

122CD
CUNEIFORM SIGN SHIM TIMES IGI GUNU

122CE
CUNEIFORM SIGN SHIM TIMES KUSHU2

122CF
CUNEIFORM SIGN SHIM TIMES LUL

122D0
CUNEIFORM SIGN SHIM TIMES MUG

122D1
CUNEIFORM SIGN SHIM TIMES SAL

122D2
CUNEIFORM SIGN SHINIG

122D3
CUNEIFORM SIGN SHIR

122D4
CUNEIFORM SIGN NU11 TENU

122D4
CUNEIFORM SIGN SHIR TENU

122D5
CUNEIFORM SIGN NU11 OVER NU11 BUR OVER BUR

122D5
CUNEIFORM SIGN SHIR OVER SHIR BUR OVER BUR

122D6
CUNEIFORM SIGN SHITA

122D7
CUNEIFORM SIGN SHU

122D8
CUNEIFORM SIGN SHU OVER INVERTED SHU

122D9
CUNEIFORM SIGN SHU2

122DA
CUNEIFORM SIGN SHUBUR

122DB
CUNEIFORM SIGN SI

122DC
CUNEIFORM SIGN SI GUNU

122DD
CUNEIFORM SIGN SIG

122DE
CUNEIFORM SIGN SIG4

122DF
CUNEIFORM SIGN SIG4 OVER SIG4 SHU2

122E0
CUNEIFORM SIGN SIK2

122E1
CUNEIFORM SIGN SILA3

122E2
CUNEIFORM SIGN SU

122E3
CUNEIFORM SIGN SU OVER SU

122E4
CUNEIFORM SIGN SUD

122E5
CUNEIFORM SIGN SUD2

122E6
CUNEIFORM SIGN SUHUR

122E7
CUNEIFORM SIGN SUM

122E8
CUNEIFORM SIGN SUMASH

122E9
CUNEIFORM SIGN SUR

122EA
CUNEIFORM SIGN SUR9

122EB
CUNEIFORM SIGN TA

122EC
CUNEIFORM SIGN TA ASTERISK

122ED
CUNEIFORM SIGN TA TIMES HI

122EE
CUNEIFORM SIGN TA TIMES MI

122EF
CUNEIFORM SIGN TA GUNU

122F0
CUNEIFORM SIGN TAB

122F1
CUNEIFORM SIGN TAB OVER TAB NI OVER NI DISH OVER DISH

122F2
CUNEIFORM SIGN TAB SQUARED

122F3
CUNEIFORM SIGN TAG

122F4
CUNEIFORM SIGN TAG TIMES BI

122F5
CUNEIFORM SIGN TAG TIMES GUD

122F6
CUNEIFORM SIGN TAG TIMES SHE

122F7
CUNEIFORM SIGN TAG TIMES SHU

122F8
CUNEIFORM SIGN TAG TIMES TUG2

122F9
CUNEIFORM SIGN TAG TIMES UD

122FA
CUNEIFORM SIGN TAK4

122FB
CUNEIFORM SIGN TAR

122FC
CUNEIFORM SIGN TE

122FD
CUNEIFORM SIGN TE GUNU

122FE
CUNEIFORM SIGN TI

122FF
CUNEIFORM SIGN TI TENU

12300
CUNEIFORM SIGN TIL

12301
CUNEIFORM SIGN TIR

12302
CUNEIFORM SIGN TIR TIMES TAK4

12303
CUNEIFORM SIGN TIR OVER TIR

12304
CUNEIFORM SIGN TIR OVER TIR GAD OVER GAD GAR OVER GAR

12305
CUNEIFORM SIGN TU

12306
CUNEIFORM SIGN TUG2

12307
CUNEIFORM SIGN TUK

12308
CUNEIFORM SIGN TUM

12309
CUNEIFORM SIGN TUR

1230A
CUNEIFORM SIGN TUR OVER TUR ZA OVER ZA

1230B
CUNEIFORM SIGN U

1230C
CUNEIFORM SIGN U GUD

1230D
CUNEIFORM SIGN U U U

1230E
CUNEIFORM SIGN U OVER U PA OVER PA GAR OVER GAR

1230F
CUNEIFORM SIGN U OVER U SUR OVER SUR

12310
CUNEIFORM SIGN U OVER U U REVERSED OVER U REVERSED

12311
CUNEIFORM SIGN U2

12312
CUNEIFORM SIGN UB

12313
CUNEIFORM SIGN UD

12314
CUNEIFORM SIGN UD KUSHU2

12315
CUNEIFORM SIGN UD TIMES BAD

12316
CUNEIFORM SIGN UD TIMES MI

12317
CUNEIFORM SIGN UD TIMES U PLUS U PLUS U

12318
CUNEIFORM SIGN UD TIMES U PLUS U PLUS U GUNU

12319
CUNEIFORM SIGN UD GUNU

1231A
CUNEIFORM SIGN UD SHESHIG

1231B
CUNEIFORM SIGN UD SHESHIG TIMES BAD

1231C
CUNEIFORM SIGN UDUG

1231D
CUNEIFORM SIGN UM

1231E
CUNEIFORM SIGN UM TIMES LAGAB

1231F
CUNEIFORM SIGN UM TIMES ME PLUS DA

12320
CUNEIFORM SIGN UM TIMES SHA3

12321
CUNEIFORM SIGN UM TIMES U

12322
CUNEIFORM SIGN UMBIN

12323
CUNEIFORM SIGN UMUM

12324
CUNEIFORM SIGN UMUM TIMES KASKAL

12325
CUNEIFORM SIGN UMUM TIMES PA

12326
CUNEIFORM SIGN UN

12327
CUNEIFORM SIGN KALAM

12327
CUNEIFORM SIGN UN GUNU

12328
CUNEIFORM SIGN UR

12329
CUNEIFORM SIGN UR CROSSING UR

1232A
CUNEIFORM SIGN UR SHESHIG

1232B
CUNEIFORM SIGN UR2

1232C
CUNEIFORM SIGN UR2 TIMES A PLUS HA

1232D
CUNEIFORM SIGN UR2 TIMES A PLUS NA

1232E
CUNEIFORM SIGN UR2 TIMES AL

1232F
CUNEIFORM SIGN UR2 TIMES HA

12330
CUNEIFORM SIGN UR2 TIMES NUN

12331
CUNEIFORM SIGN UR2 TIMES U2

12332
CUNEIFORM SIGN UR2 TIMES U2 PLUS ASH

12333
CUNEIFORM SIGN UR2 TIMES U2 PLUS BI

12334
CUNEIFORM SIGN UR4

12335
CUNEIFORM SIGN URI

12336
CUNEIFORM SIGN URI3

12337
CUNEIFORM SIGN URU

12338
CUNEIFORM SIGN URU TIMES A

12339
CUNEIFORM SIGN URU TIMES ASHGAB

1233A
CUNEIFORM SIGN URU TIMES BAR

1233B
CUNEIFORM SIGN URU TIMES DUN

1233C
CUNEIFORM SIGN URU TIMES GA

1233D
CUNEIFORM SIGN URU TIMES GAL

1233E
CUNEIFORM SIGN URU TIMES GAN2 TENU

1233F
CUNEIFORM SIGN URU TIMES GAR

12340
CUNEIFORM SIGN URU TIMES GU

12341
CUNEIFORM SIGN URU TIMES HA

12342
CUNEIFORM SIGN URU TIMES IGI

12343
CUNEIFORM SIGN URU TIMES IM

12344
CUNEIFORM SIGN URU TIMES ISH

12345
CUNEIFORM SIGN URU TIMES KI

12346
CUNEIFORM SIGN URU TIMES LUM

12347
CUNEIFORM SIGN URU TIMES MIN

12348
CUNEIFORM SIGN URU TIMES PA

12349
CUNEIFORM SIGN URU TIMES SHE

1234A
CUNEIFORM SIGN URU TIMES SIG4

1234B
CUNEIFORM SIGN URU TIMES TU

1234C
CUNEIFORM SIGN URU TIMES U PLUS GUD

1234D
CUNEIFORM SIGN URU TIMES UD

1234E
CUNEIFORM SIGN URU TIMES URUDA

1234F
CUNEIFORM SIGN URUDA

12350
CUNEIFORM SIGN URUDA TIMES U

12351
CUNEIFORM SIGN USH

12352
CUNEIFORM SIGN USH TIMES A

12353
CUNEIFORM SIGN USH TIMES KU

12354
CUNEIFORM SIGN USH TIMES KUR

12355
CUNEIFORM SIGN USH TIMES TAK4

12356
CUNEIFORM SIGN USHX

12357
CUNEIFORM SIGN USH2

12358
CUNEIFORM SIGN USHUMX

12359
CUNEIFORM SIGN UTUKI

1235A
CUNEIFORM SIGN UZ3

1235B
CUNEIFORM SIGN UZ3 TIMES KASKAL

1235C
CUNEIFORM SIGN UZU

1235D
CUNEIFORM SIGN ZA

1235E
CUNEIFORM SIGN ZA TENU

1235F
CUNEIFORM SIGN ZA SQUARED TIMES KUR

12360
CUNEIFORM SIGN ZAG

12361
CUNEIFORM SIGN ZAMX

12362
CUNEIFORM SIGN ZE2

12363
CUNEIFORM SIGN ZI

12364
CUNEIFORM SIGN ZI OVER ZI

12365
CUNEIFORM SIGN ZI3

12366
CUNEIFORM SIGN ZIB

12367
CUNEIFORM SIGN ZIB KABA TENU

12368
CUNEIFORM SIGN ZIG

12369
CUNEIFORM SIGN ZIZ2

1236A
CUNEIFORM SIGN ZU

1236B
CUNEIFORM SIGN ZU5

1236C
CUNEIFORM SIGN ZU5 TIMES A

1236D
CUNEIFORM SIGN ZUBUR

1236E
CUNEIFORM SIGN ZUM

1236F
CUNEIFORM SIGN KAP ELAMITE

12370
CUNEIFORM SIGN AB TIMES NUN

12371
CUNEIFORM SIGN AB2 TIMES A

12372
CUNEIFORM SIGN AMAR TIMES KUG

12373
CUNEIFORM SIGN DAG KISIM5 TIMES U2 PLUS MASH

12374
CUNEIFORM SIGN DAG3

12375
CUNEIFORM SIGN DISH PLUS SHU

12376
CUNEIFORM SIGN DUB TIMES SHE

12377
CUNEIFORM SIGN EZEN TIMES GUD

12378
CUNEIFORM SIGN EZEN TIMES SHE

12379
CUNEIFORM SIGN GA2 TIMES AN PLUS KAK PLUS A

1237A
CUNEIFORM SIGN GA2 TIMES ASH2

1237B
CUNEIFORM SIGN GE22

1237C
CUNEIFORM SIGN GIG

1237D
CUNEIFORM SIGN HUSH

1237E
CUNEIFORM SIGN KA TIMES ANSHE

1237F
CUNEIFORM SIGN KA TIMES ASH3

12380
CUNEIFORM SIGN KA TIMES GISH

12381
CUNEIFORM SIGN KA TIMES GUD

12382
CUNEIFORM SIGN KA TIMES HI TIMES ASH2

12383
CUNEIFORM SIGN KA TIMES LUM

12384
CUNEIFORM SIGN KA TIMES PA

12385
CUNEIFORM SIGN KA TIMES SHUL

12386
CUNEIFORM SIGN KA TIMES TU

12387
CUNEIFORM SIGN KA TIMES UR2

12388
CUNEIFORM SIGN LAGAB TIMES GI

12389
CUNEIFORM SIGN LU2 SHESHIG TIMES BAD

1238A
CUNEIFORM SIGN LU2 TIMES ESH2 PLUS LAL

1238B
CUNEIFORM SIGN LU2 TIMES SHU

1238C
CUNEIFORM SIGN MESH

1238D
CUNEIFORM SIGN MUSH3 TIMES ZA

1238E
CUNEIFORM SIGN NA4

1238F
CUNEIFORM SIGN NIN

12390
CUNEIFORM SIGN NIN9

12391
CUNEIFORM SIGN NINDA2 TIMES BAL

12392
CUNEIFORM SIGN NINDA2 TIMES GI

12393
CUNEIFORM SIGN NU11 ROTATED NINETY DEGREES

12394
CUNEIFORM SIGN PESH2 ASTERISK

12395
CUNEIFORM SIGN PIR2

12396
CUNEIFORM SIGN SAG TIMES IGI GUNU

12397
CUNEIFORM SIGN TI2

12398
CUNEIFORM SIGN UM TIMES ME

12399
CUNEIFORM SIGN U U

12400
CUNEIFORM NUMERIC SIGN TWO ASH

12401
CUNEIFORM NUMERIC SIGN THREE ASH

12402
CUNEIFORM NUMERIC SIGN FOUR ASH

12403
CUNEIFORM NUMERIC SIGN FIVE ASH

12404
CUNEIFORM NUMERIC SIGN SIX ASH

12405
CUNEIFORM NUMERIC SIGN SEVEN ASH

12406
CUNEIFORM NUMERIC SIGN EIGHT ASH

12407
CUNEIFORM NUMERIC SIGN NINE ASH

12408
CUNEIFORM NUMERIC SIGN THREE DISH

12409
CUNEIFORM NUMERIC SIGN FOUR DISH

1240A
CUNEIFORM NUMERIC SIGN FIVE DISH

1240B
CUNEIFORM NUMERIC SIGN SIX DISH

1240C
CUNEIFORM NUMERIC SIGN SEVEN DISH

1240D
CUNEIFORM NUMERIC SIGN EIGHT DISH

1240E
CUNEIFORM NUMERIC SIGN NINE DISH

1240F
CUNEIFORM NUMERIC SIGN FOUR U

12410
CUNEIFORM NUMERIC SIGN FIVE U

12411
CUNEIFORM NUMERIC SIGN SIX U

12412
CUNEIFORM NUMERIC SIGN SEVEN U

12413
CUNEIFORM NUMERIC SIGN EIGHT U

12414
CUNEIFORM NUMERIC SIGN NINE U

12415
CUNEIFORM NUMERIC SIGN ONE GESH2

12416
CUNEIFORM NUMERIC SIGN TWO GESH2

12417
CUNEIFORM NUMERIC SIGN THREE GESH2

12418
CUNEIFORM NUMERIC SIGN FOUR GESH2

12419
CUNEIFORM NUMERIC SIGN FIVE GESH2

1241A
CUNEIFORM NUMERIC SIGN SIX GESH2

1241B
CUNEIFORM NUMERIC SIGN SEVEN GESH2

1241C
CUNEIFORM NUMERIC SIGN EIGHT GESH2

1241D
CUNEIFORM NUMERIC SIGN NINE GESH2

1241E
CUNEIFORM NUMERIC SIGN ONE GESHU

1241F
CUNEIFORM NUMERIC SIGN TWO GESHU

12420
CUNEIFORM NUMERIC SIGN THREE GESHU

12421
CUNEIFORM NUMERIC SIGN FOUR GESHU

12422
CUNEIFORM NUMERIC SIGN FIVE GESHU

12423
CUNEIFORM NUMERIC SIGN TWO SHAR2

12424
CUNEIFORM NUMERIC SIGN THREE SHAR2

12425
CUNEIFORM NUMERIC SIGN THREE SHAR2 VARIANT FORM

12426
CUNEIFORM NUMERIC SIGN FOUR SHAR2

12427
CUNEIFORM NUMERIC SIGN FIVE SHAR2

12428
CUNEIFORM NUMERIC SIGN SIX SHAR2

12429
CUNEIFORM NUMERIC SIGN SEVEN SHAR2

1242A
CUNEIFORM NUMERIC SIGN EIGHT SHAR2

1242B
CUNEIFORM NUMERIC SIGN NINE SHAR2

1242C
CUNEIFORM NUMERIC SIGN ONE SHARU

1242D
CUNEIFORM NUMERIC SIGN TWO SHARU

1242E
CUNEIFORM NUMERIC SIGN THREE SHARU

1242F
CUNEIFORM NUMERIC SIGN THREE SHARU VARIANT FORM

12430
CUNEIFORM NUMERIC SIGN FOUR SHARU

12431
CUNEIFORM NUMERIC SIGN FIVE SHARU

12432
CUNEIFORM NUMERIC SIGN SHAR2 TIMES GAL PLUS DISH

12433
CUNEIFORM NUMERIC SIGN SHAR2 TIMES GAL PLUS MIN

12434
CUNEIFORM NUMERIC SIGN ONE BURU

12435
CUNEIFORM NUMERIC SIGN TWO BURU

12436
CUNEIFORM NUMERIC SIGN THREE BURU

12437
CUNEIFORM NUMERIC SIGN THREE BURU VARIANT FORM

12438
CUNEIFORM NUMERIC SIGN FOUR BURU

12439
CUNEIFORM NUMERIC SIGN FIVE BURU

1243A
CUNEIFORM NUMERIC SIGN THREE VARIANT FORM ESH16

1243B
CUNEIFORM NUMERIC SIGN THREE VARIANT FORM ESH21

1243C
CUNEIFORM NUMERIC SIGN FOUR VARIANT FORM LIMMU

1243D
CUNEIFORM NUMERIC SIGN FOUR VARIANT FORM LIMMU4

1243E
CUNEIFORM NUMERIC SIGN FOUR VARIANT FORM LIMMU A

1243F
CUNEIFORM NUMERIC SIGN FOUR VARIANT FORM LIMMU B

12440
CUNEIFORM NUMERIC SIGN SIX VARIANT FORM ASH9

12441
CUNEIFORM NUMERIC SIGN SEVEN VARIANT FORM IMIN3

12442
CUNEIFORM NUMERIC SIGN SEVEN VARIANT FORM IMIN A

12443
CUNEIFORM NUMERIC SIGN SEVEN VARIANT FORM IMIN B

12444
CUNEIFORM NUMERIC SIGN EIGHT VARIANT FORM USSU

12445
CUNEIFORM NUMERIC SIGN EIGHT VARIANT FORM USSU3

12446
CUNEIFORM NUMERIC SIGN NINE VARIANT FORM ILIMMU

12447
CUNEIFORM NUMERIC SIGN NINE VARIANT FORM ILIMMU3

12448
CUNEIFORM NUMERIC SIGN NINE VARIANT FORM ILIMMU4

12449
CUNEIFORM NUMERIC SIGN NINE VARIANT FORM ILIMMU A

1244A
CUNEIFORM NUMERIC SIGN TWO ASH TENU

1244B
CUNEIFORM NUMERIC SIGN THREE ASH TENU

1244C
CUNEIFORM NUMERIC SIGN FOUR ASH TENU

1244D
CUNEIFORM NUMERIC SIGN FIVE ASH TENU

1244E
CUNEIFORM NUMERIC SIGN SIX ASH TENU

1244F
CUNEIFORM NUMERIC SIGN ONE BAN2

12450
CUNEIFORM NUMERIC SIGN TWO BAN2

12451
CUNEIFORM NUMERIC SIGN THREE BAN2

12452
CUNEIFORM NUMERIC SIGN FOUR BAN2

12453
CUNEIFORM NUMERIC SIGN FOUR BAN2 VARIANT FORM

12454
CUNEIFORM NUMERIC SIGN FIVE BAN2

12455
CUNEIFORM NUMERIC SIGN FIVE BAN2 VARIANT FORM

12456
CUNEIFORM NUMERIC SIGN NIGIDAMIN

12457
CUNEIFORM NUMERIC SIGN NIGIDAESH

12458
CUNEIFORM NUMERIC SIGN ONE ESHE3

12459
CUNEIFORM NUMERIC SIGN TWO ESHE3

1245A
CUNEIFORM NUMERIC SIGN ONE THIRD DISH

1245B
CUNEIFORM NUMERIC SIGN TWO THIRDS DISH

1245C
CUNEIFORM NUMERIC SIGN FIVE SIXTHS DISH

1245D
CUNEIFORM NUMERIC SIGN ONE THIRD VARIANT FORM A

1245E
CUNEIFORM NUMERIC SIGN TWO THIRDS VARIANT FORM A

1245F
CUNEIFORM NUMERIC SIGN ONE EIGHTH ASH

12460
CUNEIFORM NUMERIC SIGN ONE QUARTER ASH

12461
CUNEIFORM NUMERIC SIGN OLD ASSYRIAN ONE SIXTH

12462
CUNEIFORM NUMERIC SIGN OLD ASSYRIAN ONE QUARTER

12463
CUNEIFORM NUMERIC SIGN ONE QUARTER GUR

12464
CUNEIFORM NUMERIC SIGN ONE HALF GUR

12465
CUNEIFORM NUMERIC SIGN ELAMITE ONE THIRD

12466
CUNEIFORM NUMERIC SIGN ELAMITE TWO THIRDS

12467
CUNEIFORM NUMERIC SIGN ELAMITE FORTY

12468
CUNEIFORM NUMERIC SIGN ELAMITE FIFTY

12469
CUNEIFORM NUMERIC SIGN FOUR U VARIANT FORM

1246A
CUNEIFORM NUMERIC SIGN FIVE U VARIANT FORM

1246B
CUNEIFORM NUMERIC SIGN SIX U VARIANT FORM

1246C
CUNEIFORM NUMERIC SIGN SEVEN U VARIANT FORM

1246D
CUNEIFORM NUMERIC SIGN EIGHT U VARIANT FORM

1246E
CUNEIFORM NUMERIC SIGN NINE U VARIANT FORM

12470
CUNEIFORM PUNCTUATION SIGN OLD ASSYRIAN WORD DIVIDER

12471
CUNEIFORM PUNCTUATION SIGN VERTICAL COLON

12472
CUNEIFORM PUNCTUATION SIGN DIAGONAL COLON

12473
CUNEIFORM PUNCTUATION SIGN DIAGONAL TRICOLON

12474
CUNEIFORM PUNCTUATION SIGN DIAGONAL QUADCOLON

12480
CUNEIFORM SIGN AB TIMES NUN TENU

12481
CUNEIFORM SIGN AB TIMES SHU2

12482
CUNEIFORM SIGN AD TIMES ESH2

12483
CUNEIFORM SIGN BAD TIMES DISH TENU

12484
CUNEIFORM SIGN BAHAR2 TIMES AB2

12485
CUNEIFORM SIGN BAHAR2 TIMES NI

12486
CUNEIFORM SIGN BAHAR2 TIMES ZA

12487
CUNEIFORM SIGN BU OVER BU TIMES NA2

12488
CUNEIFORM SIGN DA TIMES TAK4

12489
CUNEIFORM SIGN DAG TIMES KUR

1248A
CUNEIFORM SIGN DIM TIMES IGI

1248B
CUNEIFORM SIGN DIM TIMES U U U

1248C
CUNEIFORM SIGN DIM2 TIMES UD

1248D
CUNEIFORM SIGN DUG TIMES ANSHE

1248E
CUNEIFORM SIGN DUG TIMES ASH

1248F
CUNEIFORM SIGN DUG TIMES ASH AT LEFT

12490
CUNEIFORM SIGN DUG TIMES DIN

12491
CUNEIFORM SIGN DUG TIMES DUN

12492
CUNEIFORM SIGN DUG TIMES ERIN2

12493
CUNEIFORM SIGN DUG TIMES GA

12494
CUNEIFORM SIGN DUG TIMES GI

12495
CUNEIFORM SIGN DUG TIMES GIR2 GUNU

12496
CUNEIFORM SIGN DUG TIMES GISH

12497
CUNEIFORM SIGN DUG TIMES HA

12498
CUNEIFORM SIGN DUG TIMES HI

12499
CUNEIFORM SIGN DUG TIMES IGI GUNU

1249A
CUNEIFORM SIGN DUG TIMES KASKAL

1249B
CUNEIFORM SIGN DUG TIMES KUR

1249C
CUNEIFORM SIGN DUG TIMES KUSHU2

1249D
CUNEIFORM SIGN DUG TIMES KUSHU2 PLUS KASKAL

1249E
CUNEIFORM SIGN DUG TIMES LAK-020

1249F
CUNEIFORM SIGN DUG TIMES LAM

124A0
CUNEIFORM SIGN DUG TIMES LAM TIMES KUR

124A1
CUNEIFORM SIGN DUG TIMES LUH PLUS GISH

124A2
CUNEIFORM SIGN DUG TIMES MASH

124A3
CUNEIFORM SIGN DUG TIMES MES

124A4
CUNEIFORM SIGN DUG TIMES MI

124A5
CUNEIFORM SIGN DUG TIMES NI

124A6
CUNEIFORM SIGN DUG TIMES PI

124A7
CUNEIFORM SIGN DUG TIMES SHE

124A8
CUNEIFORM SIGN DUG TIMES SI GUNU

124A9
CUNEIFORM SIGN E2 TIMES KUR

124AA
CUNEIFORM SIGN E2 TIMES PAP

124AB
CUNEIFORM SIGN ERIN2 X

124AC
CUNEIFORM SIGN ESH2 CROSSING ESH2

124AD
CUNEIFORM SIGN EZEN SHESHIG TIMES ASH

124AE
CUNEIFORM SIGN EZEN SHESHIG TIMES HI

124AF
CUNEIFORM SIGN EZEN SHESHIG TIMES IGI GUNU

124B0
CUNEIFORM SIGN EZEN SHESHIG TIMES LA

124B1
CUNEIFORM SIGN EZEN SHESHIG TIMES LAL

124B2
CUNEIFORM SIGN EZEN SHESHIG TIMES ME

124B3
CUNEIFORM SIGN EZEN SHESHIG TIMES MES

124B4
CUNEIFORM SIGN EZEN SHESHIG TIMES SU

124B5
CUNEIFORM SIGN EZEN TIMES SU

124B6
CUNEIFORM SIGN GA2 TIMES BAHAR2

124B7
CUNEIFORM SIGN GA2 TIMES DIM GUNU

124B8
CUNEIFORM SIGN GA2 TIMES DUG TIMES IGI GUNU

124B9
CUNEIFORM SIGN GA2 TIMES DUG TIMES KASKAL

124BA
CUNEIFORM SIGN GA2 TIMES EREN

124BB
CUNEIFORM SIGN GA2 TIMES GA

124BC
CUNEIFORM SIGN GA2 TIMES GAR PLUS DI

124BD
CUNEIFORM SIGN GA2 TIMES GAR PLUS NE

124BE
CUNEIFORM SIGN GA2 TIMES HA PLUS A

124BF
CUNEIFORM SIGN GA2 TIMES KUSHU2 PLUS KASKAL

124C0
CUNEIFORM SIGN GA2 TIMES LAM

124C1
CUNEIFORM SIGN GA2 TIMES LAM TIMES KUR

124C2
CUNEIFORM SIGN GA2 TIMES LUH

124C3
CUNEIFORM SIGN GA2 TIMES MUSH

124C4
CUNEIFORM SIGN GA2 TIMES NE

124C5
CUNEIFORM SIGN GA2 TIMES NE PLUS E2

124C6
CUNEIFORM SIGN GA2 TIMES NE PLUS GI

124C7
CUNEIFORM SIGN GA2 TIMES SHIM

124C8
CUNEIFORM SIGN GA2 TIMES ZIZ2

124C9
CUNEIFORM SIGN GABA ROTATED NINETY DEGREES

124CA
CUNEIFORM SIGN GESHTIN TIMES U

124CB
CUNEIFORM SIGN GISH TIMES GISH CROSSING GISH

124CC
CUNEIFORM SIGN GU2 TIMES IGI GUNU

124CD
CUNEIFORM SIGN GUD PLUS GISH TIMES TAK4

124CE
CUNEIFORM SIGN HA TENU GUNU

124CF
CUNEIFORM SIGN HI TIMES ASH OVER HI TIMES ASH

124D0
CUNEIFORM SIGN KA TIMES BU

124D1
CUNEIFORM SIGN KA TIMES KA

124D2
CUNEIFORM SIGN KA TIMES U U U

124D3
CUNEIFORM SIGN KA TIMES UR

124D4
CUNEIFORM SIGN LAGAB TIMES ZU OVER ZU

124D5
CUNEIFORM SIGN LAK-003

124D6
CUNEIFORM SIGN LAK-021

124D7
CUNEIFORM SIGN LAK-025

124D8
CUNEIFORM SIGN LAK-030

124D9
CUNEIFORM SIGN LAK-050

124DA
CUNEIFORM SIGN LAK-051

124DB
CUNEIFORM SIGN LAK-062

124DC
CUNEIFORM SIGN LAK-079 OVER LAK-079 GUNU

124DD
CUNEIFORM SIGN LAK-080

124DE
CUNEIFORM SIGN LAK-081 OVER LAK-081

124DF
CUNEIFORM SIGN LAK-092

124E0
CUNEIFORM SIGN LAK-130

124E1
CUNEIFORM SIGN LAK-142

124E2
CUNEIFORM SIGN LAK-210

124E3
CUNEIFORM SIGN LAK-219

124E4
CUNEIFORM SIGN LAK-220

124E5
CUNEIFORM SIGN LAK-225

124E6
CUNEIFORM SIGN LAK-228

124E7
CUNEIFORM SIGN LAK-238

124E8
CUNEIFORM SIGN LAK-265

124E9
CUNEIFORM SIGN LAK-266

124EA
CUNEIFORM SIGN LAK-343

124EB
CUNEIFORM SIGN LAK-347

124EC
CUNEIFORM SIGN LAK-348

124ED
CUNEIFORM SIGN LAK-383

124EE
CUNEIFORM SIGN LAK-384

124EF
CUNEIFORM SIGN LAK-390

124F0
CUNEIFORM SIGN LAK-441

124F1
CUNEIFORM SIGN LAK-449

124F2
CUNEIFORM SIGN LAK-449 TIMES GU

124F3
CUNEIFORM SIGN LAK-449 TIMES IGI

124F4
CUNEIFORM SIGN LAK-449 TIMES PAP PLUS LU3

124F5
CUNEIFORM SIGN LAK-449 TIMES PAP PLUS PAP PLUS LU3

124F6
CUNEIFORM SIGN LAK-449 TIMES U2 PLUS BA

124F7
CUNEIFORM SIGN LAK-450

124F8
CUNEIFORM SIGN LAK-457

124F9
CUNEIFORM SIGN LAK-470

124FA
CUNEIFORM SIGN LAK-483

124FB
CUNEIFORM SIGN LAK-490

124FC
CUNEIFORM SIGN LAK-492

124FD
CUNEIFORM SIGN LAK-493

124FE
CUNEIFORM SIGN LAK-495

124FF
CUNEIFORM SIGN LAK-550

12500
CUNEIFORM SIGN LAK-608

12501
CUNEIFORM SIGN LAK-617

12502
CUNEIFORM SIGN LAK-617 TIMES ASH

12503
CUNEIFORM SIGN LAK-617 TIMES BAD

12504
CUNEIFORM SIGN LAK-617 TIMES DUN3 GUNU GUNU

12505
CUNEIFORM SIGN LAK-617 TIMES KU3

12506
CUNEIFORM SIGN LAK-617 TIMES LA

12507
CUNEIFORM SIGN LAK-617 TIMES TAR

12508
CUNEIFORM SIGN LAK-617 TIMES TE

12509
CUNEIFORM SIGN LAK-617 TIMES U2

1250A
CUNEIFORM SIGN LAK-617 TIMES UD

1250B
CUNEIFORM SIGN LAK-617 TIMES URUDA

1250C
CUNEIFORM SIGN LAK-636

1250D
CUNEIFORM SIGN LAK-648

1250E
CUNEIFORM SIGN LAK-648 TIMES DUB

1250F
CUNEIFORM SIGN LAK-648 TIMES GA

12510
CUNEIFORM SIGN LAK-648 TIMES IGI

12511
CUNEIFORM SIGN LAK-648 TIMES IGI GUNU

12512
CUNEIFORM SIGN LAK-648 TIMES NI

12513
CUNEIFORM SIGN LAK-648 TIMES PAP PLUS PAP PLUS LU3

12514
CUNEIFORM SIGN LAK-648 TIMES SHESH PLUS KI

12515
CUNEIFORM SIGN LAK-648 TIMES UD

12516
CUNEIFORM SIGN LAK-648 TIMES URUDA

12517
CUNEIFORM SIGN LAK-724

12518
CUNEIFORM SIGN LAK-749

12519
CUNEIFORM SIGN LU2 GUNU TIMES ASH

1251A
CUNEIFORM SIGN LU2 TIMES DISH

1251B
CUNEIFORM SIGN LU2 TIMES HAL

1251C
CUNEIFORM SIGN LU2 TIMES PAP

1251D
CUNEIFORM SIGN LU2 TIMES PAP PLUS PAP PLUS LU3

1251E
CUNEIFORM SIGN LU2 TIMES TAK4

1251F
CUNEIFORM SIGN MI PLUS ZA7

12520
CUNEIFORM SIGN MUSH OVER MUSH TIMES GA

12521
CUNEIFORM SIGN MUSH OVER MUSH TIMES KAK

12522
CUNEIFORM SIGN NINDA2 TIMES DIM GUNU

12523
CUNEIFORM SIGN NINDA2 TIMES GISH

12524
CUNEIFORM SIGN NINDA2 TIMES GUL

12525
CUNEIFORM SIGN NINDA2 TIMES HI

12526
CUNEIFORM SIGN NINDA2 TIMES KESH2

12527
CUNEIFORM SIGN NINDA2 TIMES LAK-050

12528
CUNEIFORM SIGN NINDA2 TIMES MASH

12529
CUNEIFORM SIGN NINDA2 TIMES PAP PLUS PAP

1252A
CUNEIFORM SIGN NINDA2 TIMES U

1252B
CUNEIFORM SIGN NINDA2 TIMES U PLUS U

1252C
CUNEIFORM SIGN NINDA2 TIMES URUDA

1252D
CUNEIFORM SIGN SAG GUNU TIMES HA

1252E
CUNEIFORM SIGN SAG TIMES EN

1252F
CUNEIFORM SIGN SAG TIMES SHE AT LEFT

12530
CUNEIFORM SIGN SAG TIMES TAK4

12531
CUNEIFORM SIGN SHA6 TENU

12532
CUNEIFORM SIGN SHE OVER SHE

12533
CUNEIFORM SIGN SHE PLUS HUB2

12534
CUNEIFORM SIGN SHE PLUS NAM2

12535
CUNEIFORM SIGN SHE PLUS SAR

12536
CUNEIFORM SIGN SHU2 PLUS DUG TIMES NI

12537
CUNEIFORM SIGN SHU2 PLUS E2 TIMES AN

12538
CUNEIFORM SIGN SI TIMES TAK4

12539
CUNEIFORM SIGN TAK4 PLUS SAG

1253A
CUNEIFORM SIGN TUM TIMES GAN2 TENU

1253B
CUNEIFORM SIGN TUM TIMES THREE DISH

1253C
CUNEIFORM SIGN UR2 INVERTED

1253D
CUNEIFORM SIGN UR2 TIMES UD

1253E
CUNEIFORM SIGN URU TIMES DARA3

1253F
CUNEIFORM SIGN URU TIMES LAK-668

12540
CUNEIFORM SIGN URU TIMES LU3

12541
CUNEIFORM SIGN ZA7

12542
CUNEIFORM SIGN ZU OVER ZU PLUS SAR

12543
CUNEIFORM SIGN ZU5 TIMES THREE DISH TENU

12F90
CYPRO-MINOAN SIGN CM001

12F91
CYPRO-MINOAN SIGN CM002

12F92
CYPRO-MINOAN SIGN CM004

12F93
CYPRO-MINOAN SIGN CM005

12F94
CYPRO-MINOAN SIGN CM006

12F95
CYPRO-MINOAN SIGN CM007

12F96
CYPRO-MINOAN SIGN CM008

12F97
CYPRO-MINOAN SIGN CM009

12F98
CYPRO-MINOAN SIGN CM010

12F99
CYPRO-MINOAN SIGN CM011

12F9A
CYPRO-MINOAN SIGN CM012

12F9B
CYPRO-MINOAN SIGN CM012B

12F9C
CYPRO-MINOAN SIGN CM013

12F9D
CYPRO-MINOAN SIGN CM015

12F9E
CYPRO-MINOAN SIGN CM017

12F9F
CYPRO-MINOAN SIGN CM019

12FA0
CYPRO-MINOAN SIGN CM021

12FA1
CYPRO-MINOAN SIGN CM023

12FA2
CYPRO-MINOAN SIGN CM024

12FA3
CYPRO-MINOAN SIGN CM025

12FA4
CYPRO-MINOAN SIGN CM026

12FA5
CYPRO-MINOAN SIGN CM027

12FA6
CYPRO-MINOAN SIGN CM028

12FA7
CYPRO-MINOAN SIGN CM029

12FA8
CYPRO-MINOAN SIGN CM030

12FA9
CYPRO-MINOAN SIGN CM033

12FAA
CYPRO-MINOAN SIGN CM034

12FAB
CYPRO-MINOAN SIGN CM035

12FAC
CYPRO-MINOAN SIGN CM036

12FAD
CYPRO-MINOAN SIGN CM037

12FAE
CYPRO-MINOAN SIGN CM038

12FAF
CYPRO-MINOAN SIGN CM039

12FB0
CYPRO-MINOAN SIGN CM040

12FB1
CYPRO-MINOAN SIGN CM041

12FB2
CYPRO-MINOAN SIGN CM044

12FB3
CYPRO-MINOAN SIGN CM046

12FB4
CYPRO-MINOAN SIGN CM047

12FB5
CYPRO-MINOAN SIGN CM049

12FB6
CYPRO-MINOAN SIGN CM050

12FB7
CYPRO-MINOAN SIGN CM051

12FB8
CYPRO-MINOAN SIGN CM052

12FB9
CYPRO-MINOAN SIGN CM053

12FBA
CYPRO-MINOAN SIGN CM054

12FBB
CYPRO-MINOAN SIGN CM055

12FBC
CYPRO-MINOAN SIGN CM056

12FBD
CYPRO-MINOAN SIGN CM058

12FBE
CYPRO-MINOAN SIGN CM059

12FBF
CYPRO-MINOAN SIGN CM060

12FC0
CYPRO-MINOAN SIGN CM061

12FC1
CYPRO-MINOAN SIGN CM062

12FC2
CYPRO-MINOAN SIGN CM063

12FC3
CYPRO-MINOAN SIGN CM064

12FC4
CYPRO-MINOAN SIGN CM066

12FC5
CYPRO-MINOAN SIGN CM067

12FC6
CYPRO-MINOAN SIGN CM068

12FC7
CYPRO-MINOAN SIGN CM069

12FC8
CYPRO-MINOAN SIGN CM070

12FC9
CYPRO-MINOAN SIGN CM071

12FCA
CYPRO-MINOAN SIGN CM072

12FCB
CYPRO-MINOAN SIGN CM073

12FCC
CYPRO-MINOAN SIGN CM074

12FCD
CYPRO-MINOAN SIGN CM075

12FCE
CYPRO-MINOAN SIGN CM075B

12FCF
CYPRO-MINOAN SIGN CM076

12FD0
CYPRO-MINOAN SIGN CM078

12FD1
CYPRO-MINOAN SIGN CM079

12FD2
CYPRO-MINOAN SIGN CM080

12FD3
CYPRO-MINOAN SIGN CM081

12FD4
CYPRO-MINOAN SIGN CM082

12FD5
CYPRO-MINOAN SIGN CM083

12FD6
CYPRO-MINOAN SIGN CM084

12FD7
CYPRO-MINOAN SIGN CM085

12FD8
CYPRO-MINOAN SIGN CM086

12FD9
CYPRO-MINOAN SIGN CM087

12FDA
CYPRO-MINOAN SIGN CM088

12FDB
CYPRO-MINOAN SIGN CM089

12FDC
CYPRO-MINOAN SIGN CM090

12FDD
CYPRO-MINOAN SIGN CM091

12FDE
CYPRO-MINOAN SIGN CM092

12FDF
CYPRO-MINOAN SIGN CM094

12FE0
CYPRO-MINOAN SIGN CM095

12FE1
CYPRO-MINOAN SIGN CM096

12FE2
CYPRO-MINOAN SIGN CM097

12FE3
CYPRO-MINOAN SIGN CM098

12FE4
CYPRO-MINOAN SIGN CM099

12FE5
CYPRO-MINOAN SIGN CM100

12FE6
CYPRO-MINOAN SIGN CM101

12FE7
CYPRO-MINOAN SIGN CM102

12FE8
CYPRO-MINOAN SIGN CM103

12FE9
CYPRO-MINOAN SIGN CM104

12FEA
CYPRO-MINOAN SIGN CM105

12FEB
CYPRO-MINOAN SIGN CM107

12FEC
CYPRO-MINOAN SIGN CM108

12FED
CYPRO-MINOAN SIGN CM109

12FEE
CYPRO-MINOAN SIGN CM110

12FEF
CYPRO-MINOAN SIGN CM112

12FF0
CYPRO-MINOAN SIGN CM114

12FF1
CYPRO-MINOAN SIGN CM301

12FF2
CYPRO-MINOAN SIGN CM302

13000
EGYPTIAN HIEROGLYPH A001

13001
EGYPTIAN HIEROGLYPH A002

13002
EGYPTIAN HIEROGLYPH A003

13003
EGYPTIAN HIEROGLYPH A004

13004
EGYPTIAN HIEROGLYPH A005

13005
EGYPTIAN HIEROGLYPH A005A

13006
EGYPTIAN HIEROGLYPH A006

13007
EGYPTIAN HIEROGLYPH A006A

13008
EGYPTIAN HIEROGLYPH A006B

13009
EGYPTIAN HIEROGLYPH A007

1300A
EGYPTIAN HIEROGLYPH A008

1300B
EGYPTIAN HIEROGLYPH A009

1300C
EGYPTIAN HIEROGLYPH A010

1300D
EGYPTIAN HIEROGLYPH A011

1300E
EGYPTIAN HIEROGLYPH A012

1300F
EGYPTIAN HIEROGLYPH A013

13010
EGYPTIAN HIEROGLYPH A014

13011
EGYPTIAN HIEROGLYPH A014A

13012
EGYPTIAN HIEROGLYPH A015

13013
EGYPTIAN HIEROGLYPH A016

13014
EGYPTIAN HIEROGLYPH A017

13015
EGYPTIAN HIEROGLYPH A017A

13016
EGYPTIAN HIEROGLYPH A018

13017
EGYPTIAN HIEROGLYPH A019

13018
EGYPTIAN HIEROGLYPH A020

13019
EGYPTIAN HIEROGLYPH A021

1301A
EGYPTIAN HIEROGLYPH A022

1301B
EGYPTIAN HIEROGLYPH A023

1301C
EGYPTIAN HIEROGLYPH A024

1301D
EGYPTIAN HIEROGLYPH A025

1301E
EGYPTIAN HIEROGLYPH A026

1301F
EGYPTIAN HIEROGLYPH A027

13020
EGYPTIAN HIEROGLYPH A028

13021
EGYPTIAN HIEROGLYPH A029

13022
EGYPTIAN HIEROGLYPH A030

13023
EGYPTIAN HIEROGLYPH A031

13024
EGYPTIAN HIEROGLYPH A032

13025
EGYPTIAN HIEROGLYPH A032A

13026
EGYPTIAN HIEROGLYPH A033

13027
EGYPTIAN HIEROGLYPH A034

13028
EGYPTIAN HIEROGLYPH A035

13029
EGYPTIAN HIEROGLYPH A036

1302A
EGYPTIAN HIEROGLYPH A037

1302B
EGYPTIAN HIEROGLYPH A038

1302C
EGYPTIAN HIEROGLYPH A039

1302D
EGYPTIAN HIEROGLYPH A040

1302E
EGYPTIAN HIEROGLYPH A040A

1302F
EGYPTIAN HIEROGLYPH A041

13030
EGYPTIAN HIEROGLYPH A042

13031
EGYPTIAN HIEROGLYPH A042A

13032
EGYPTIAN HIEROGLYPH A043

13033
EGYPTIAN HIEROGLYPH A043A

13034
EGYPTIAN HIEROGLYPH A044

13035
EGYPTIAN HIEROGLYPH A045

13036
EGYPTIAN HIEROGLYPH A045A

13037
EGYPTIAN HIEROGLYPH A046

13038
EGYPTIAN HIEROGLYPH A047

13039
EGYPTIAN HIEROGLYPH A048

1303A
EGYPTIAN HIEROGLYPH A049

1303B
EGYPTIAN HIEROGLYPH A050

1303C
EGYPTIAN HIEROGLYPH A051

1303D
EGYPTIAN HIEROGLYPH A052

1303E
EGYPTIAN HIEROGLYPH A053

1303F
EGYPTIAN HIEROGLYPH A054

13040
EGYPTIAN HIEROGLYPH A055

13041
EGYPTIAN HIEROGLYPH A056

13042
EGYPTIAN HIEROGLYPH A057

13043
EGYPTIAN HIEROGLYPH A058

13044
EGYPTIAN HIEROGLYPH A059

13045
EGYPTIAN HIEROGLYPH A060

13046
EGYPTIAN HIEROGLYPH A061

13047
EGYPTIAN HIEROGLYPH A062

13048
EGYPTIAN HIEROGLYPH A063

13049
EGYPTIAN HIEROGLYPH A064

1304A
EGYPTIAN HIEROGLYPH A065

1304B
EGYPTIAN HIEROGLYPH A066

1304C
EGYPTIAN HIEROGLYPH A067

1304D
EGYPTIAN HIEROGLYPH A068

1304E
EGYPTIAN HIEROGLYPH A069

1304F
EGYPTIAN HIEROGLYPH A070

13050
EGYPTIAN HIEROGLYPH B001

13051
EGYPTIAN HIEROGLYPH B002

13052
EGYPTIAN HIEROGLYPH B003

13053
EGYPTIAN HIEROGLYPH B004

13054
EGYPTIAN HIEROGLYPH B005

13055
EGYPTIAN HIEROGLYPH B005A

13056
EGYPTIAN HIEROGLYPH B006

13057
EGYPTIAN HIEROGLYPH B007

13058
EGYPTIAN HIEROGLYPH B008

13059
EGYPTIAN HIEROGLYPH B009

1305A
EGYPTIAN HIEROGLYPH C001

1305B
EGYPTIAN HIEROGLYPH C002

1305C
EGYPTIAN HIEROGLYPH C002A

1305D
EGYPTIAN HIEROGLYPH C002B

1305E
EGYPTIAN HIEROGLYPH C002C

1305F
EGYPTIAN HIEROGLYPH C003

13060
EGYPTIAN HIEROGLYPH C004

13061
EGYPTIAN HIEROGLYPH C005

13062
EGYPTIAN HIEROGLYPH C006

13063
EGYPTIAN HIEROGLYPH C007

13064
EGYPTIAN HIEROGLYPH C008

13065
EGYPTIAN HIEROGLYPH C009

13066
EGYPTIAN HIEROGLYPH C010

13067
EGYPTIAN HIEROGLYPH C010A

13068
EGYPTIAN HIEROGLYPH C011

13069
EGYPTIAN HIEROGLYPH C012

1306A
EGYPTIAN HIEROGLYPH C013

1306B
EGYPTIAN HIEROGLYPH C014

1306C
EGYPTIAN HIEROGLYPH C015

1306D
EGYPTIAN HIEROGLYPH C016

1306E
EGYPTIAN HIEROGLYPH C017

1306F
EGYPTIAN HIEROGLYPH C018

13070
EGYPTIAN HIEROGLYPH C019

13071
EGYPTIAN HIEROGLYPH C020

13072
EGYPTIAN HIEROGLYPH C021

13073
EGYPTIAN HIEROGLYPH C022

13074
EGYPTIAN HIEROGLYPH C023

13075
EGYPTIAN HIEROGLYPH C024

13076
EGYPTIAN HIEROGLYPH D001

13077
EGYPTIAN HIEROGLYPH D002

13078
EGYPTIAN HIEROGLYPH D003

13079
EGYPTIAN HIEROGLYPH D004

1307A
EGYPTIAN HIEROGLYPH D005

1307B
EGYPTIAN HIEROGLYPH D006

1307C
EGYPTIAN HIEROGLYPH D007

1307D
EGYPTIAN HIEROGLYPH D008

1307E
EGYPTIAN HIEROGLYPH D008A

1307F
EGYPTIAN HIEROGLYPH D009

13080
EGYPTIAN HIEROGLYPH D010

13081
EGYPTIAN HIEROGLYPH D011

13082
EGYPTIAN HIEROGLYPH D012

13083
EGYPTIAN HIEROGLYPH D013

13084
EGYPTIAN HIEROGLYPH D014

13085
EGYPTIAN HIEROGLYPH D015

13086
EGYPTIAN HIEROGLYPH D016

13087
EGYPTIAN HIEROGLYPH D017

13088
EGYPTIAN HIEROGLYPH D018

13089
EGYPTIAN HIEROGLYPH D019

1308A
EGYPTIAN HIEROGLYPH D020

1308B
EGYPTIAN HIEROGLYPH D021

1308C
EGYPTIAN HIEROGLYPH D022

1308D
EGYPTIAN HIEROGLYPH D023

1308E
EGYPTIAN HIEROGLYPH D024

1308F
EGYPTIAN HIEROGLYPH D025

13090
EGYPTIAN HIEROGLYPH D026

13091
EGYPTIAN HIEROGLYPH D027

13092
EGYPTIAN HIEROGLYPH D027A

13093
EGYPTIAN HIEROGLYPH D028

13094
EGYPTIAN HIEROGLYPH D029

13095
EGYPTIAN HIEROGLYPH D030

13096
EGYPTIAN HIEROGLYPH D031

13097
EGYPTIAN HIEROGLYPH D031A

13098
EGYPTIAN HIEROGLYPH D032

13099
EGYPTIAN HIEROGLYPH D033

1309A
EGYPTIAN HIEROGLYPH D034

1309B
EGYPTIAN HIEROGLYPH D034A

1309C
EGYPTIAN HIEROGLYPH D035

1309D
EGYPTIAN HIEROGLYPH D036

1309E
EGYPTIAN HIEROGLYPH D037

1309F
EGYPTIAN HIEROGLYPH D038

130A0
EGYPTIAN HIEROGLYPH D039

130A1
EGYPTIAN HIEROGLYPH D040

130A2
EGYPTIAN HIEROGLYPH D041

130A3
EGYPTIAN HIEROGLYPH D042

130A4
EGYPTIAN HIEROGLYPH D043

130A5
EGYPTIAN HIEROGLYPH D044

130A6
EGYPTIAN HIEROGLYPH D045

130A7
EGYPTIAN HIEROGLYPH D046

130A8
EGYPTIAN HIEROGLYPH D046A

130A9
EGYPTIAN HIEROGLYPH D047

130AA
EGYPTIAN HIEROGLYPH D048

130AB
EGYPTIAN HIEROGLYPH D048A

130AC
EGYPTIAN HIEROGLYPH D049

130AD
EGYPTIAN HIEROGLYPH D050

130AE
EGYPTIAN HIEROGLYPH D050A

130AF
EGYPTIAN HIEROGLYPH D050B

130B0
EGYPTIAN HIEROGLYPH D050C

130B1
EGYPTIAN HIEROGLYPH D050D

130B2
EGYPTIAN HIEROGLYPH D050E

130B3
EGYPTIAN HIEROGLYPH D050F

130B4
EGYPTIAN HIEROGLYPH D050G

130B5
EGYPTIAN HIEROGLYPH D050H

130B6
EGYPTIAN HIEROGLYPH D050I

130B7
EGYPTIAN HIEROGLYPH D051

130B8
EGYPTIAN HIEROGLYPH D052

130B9
EGYPTIAN HIEROGLYPH D052A

130BA
EGYPTIAN HIEROGLYPH D053

130BB
EGYPTIAN HIEROGLYPH D054

130BC
EGYPTIAN HIEROGLYPH D054A

130BD
EGYPTIAN HIEROGLYPH D055

130BE
EGYPTIAN HIEROGLYPH D056

130BF
EGYPTIAN HIEROGLYPH D057

130C0
EGYPTIAN HIEROGLYPH D058

130C1
EGYPTIAN HIEROGLYPH D059

130C2
EGYPTIAN HIEROGLYPH D060

130C3
EGYPTIAN HIEROGLYPH D061

130C4
EGYPTIAN HIEROGLYPH D062

130C5
EGYPTIAN HIEROGLYPH D063

130C6
EGYPTIAN HIEROGLYPH D064

130C7
EGYPTIAN HIEROGLYPH D065

130C8
EGYPTIAN HIEROGLYPH D066

130C9
EGYPTIAN HIEROGLYPH D067

130CA
EGYPTIAN HIEROGLYPH D067A

130CB
EGYPTIAN HIEROGLYPH D067B

130CC
EGYPTIAN HIEROGLYPH D067C

130CD
EGYPTIAN HIEROGLYPH D067D

130CE
EGYPTIAN HIEROGLYPH D067E

130CF
EGYPTIAN HIEROGLYPH D067F

130D0
EGYPTIAN HIEROGLYPH D067G

130D1
EGYPTIAN HIEROGLYPH D067H

130D2
EGYPTIAN HIEROGLYPH E001

130D3
EGYPTIAN HIEROGLYPH E002

130D4
EGYPTIAN HIEROGLYPH E003

130D5
EGYPTIAN HIEROGLYPH E004

130D6
EGYPTIAN HIEROGLYPH E005

130D7
EGYPTIAN HIEROGLYPH E006

130D8
EGYPTIAN HIEROGLYPH E007

130D9
EGYPTIAN HIEROGLYPH E008

130DA
EGYPTIAN HIEROGLYPH E008A

130DB
EGYPTIAN HIEROGLYPH E009

130DC
EGYPTIAN HIEROGLYPH E009A

130DD
EGYPTIAN HIEROGLYPH E010

130DE
EGYPTIAN HIEROGLYPH E011

130DF
EGYPTIAN HIEROGLYPH E012

130E0
EGYPTIAN HIEROGLYPH E013

130E1
EGYPTIAN HIEROGLYPH E014

130E2
EGYPTIAN HIEROGLYPH E015

130E3
EGYPTIAN HIEROGLYPH E016

130E4
EGYPTIAN HIEROGLYPH E016A

130E5
EGYPTIAN HIEROGLYPH E017

130E6
EGYPTIAN HIEROGLYPH E017A

130E7
EGYPTIAN HIEROGLYPH E018

130E8
EGYPTIAN HIEROGLYPH E019

130E9
EGYPTIAN HIEROGLYPH E020

130EA
EGYPTIAN HIEROGLYPH E020A

130EB
EGYPTIAN HIEROGLYPH E021

130EC
EGYPTIAN HIEROGLYPH E022

130ED
EGYPTIAN HIEROGLYPH E023

130EE
EGYPTIAN HIEROGLYPH E024

130EF
EGYPTIAN HIEROGLYPH E025

130F0
EGYPTIAN HIEROGLYPH E026

130F1
EGYPTIAN HIEROGLYPH E027

130F2
EGYPTIAN HIEROGLYPH E028

130F3
EGYPTIAN HIEROGLYPH E028A

130F4
EGYPTIAN HIEROGLYPH E029

130F5
EGYPTIAN HIEROGLYPH E030

130F6
EGYPTIAN HIEROGLYPH E031

130F7
EGYPTIAN HIEROGLYPH E032

130F8
EGYPTIAN HIEROGLYPH E033

130F9
EGYPTIAN HIEROGLYPH E034

130FA
EGYPTIAN HIEROGLYPH E034A

130FB
EGYPTIAN HIEROGLYPH E036

130FC
EGYPTIAN HIEROGLYPH E037

130FD
EGYPTIAN HIEROGLYPH E038

130FE
EGYPTIAN HIEROGLYPH F001

130FF
EGYPTIAN HIEROGLYPH F001A

13100
EGYPTIAN HIEROGLYPH F002

13101
EGYPTIAN HIEROGLYPH F003

13102
EGYPTIAN HIEROGLYPH F004

13103
EGYPTIAN HIEROGLYPH F005

13104
EGYPTIAN HIEROGLYPH F006

13105
EGYPTIAN HIEROGLYPH F007

13106
EGYPTIAN HIEROGLYPH F008

13107
EGYPTIAN HIEROGLYPH F009

13108
EGYPTIAN HIEROGLYPH F010

13109
EGYPTIAN HIEROGLYPH F011

1310A
EGYPTIAN HIEROGLYPH F012

1310B
EGYPTIAN HIEROGLYPH F013

1310C
EGYPTIAN HIEROGLYPH F013A

1310D
EGYPTIAN HIEROGLYPH F014

1310E
EGYPTIAN HIEROGLYPH F015

1310F
EGYPTIAN HIEROGLYPH F016

13110
EGYPTIAN HIEROGLYPH F017

13111
EGYPTIAN HIEROGLYPH F018

13112
EGYPTIAN HIEROGLYPH F019

13113
EGYPTIAN HIEROGLYPH F020

13114
EGYPTIAN HIEROGLYPH F021

13115
EGYPTIAN HIEROGLYPH F021A

13116
EGYPTIAN HIEROGLYPH F022

13117
EGYPTIAN HIEROGLYPH F023

13118
EGYPTIAN HIEROGLYPH F024

13119
EGYPTIAN HIEROGLYPH F025

1311A
EGYPTIAN HIEROGLYPH F026

1311B
EGYPTIAN HIEROGLYPH F027

1311C
EGYPTIAN HIEROGLYPH F028

1311D
EGYPTIAN HIEROGLYPH F029

1311E
EGYPTIAN HIEROGLYPH F030

1311F
EGYPTIAN HIEROGLYPH F031

13120
EGYPTIAN HIEROGLYPH F031A

13121
EGYPTIAN HIEROGLYPH F032

13122
EGYPTIAN HIEROGLYPH F033

13123
EGYPTIAN HIEROGLYPH F034

13124
EGYPTIAN HIEROGLYPH F035

13125
EGYPTIAN HIEROGLYPH F036

13126
EGYPTIAN HIEROGLYPH F037

13127
EGYPTIAN HIEROGLYPH F037A

13128
EGYPTIAN HIEROGLYPH F038

13129
EGYPTIAN HIEROGLYPH F038A

1312A
EGYPTIAN HIEROGLYPH F039

1312B
EGYPTIAN HIEROGLYPH F040

1312C
EGYPTIAN HIEROGLYPH F041

1312D
EGYPTIAN HIEROGLYPH F042

1312E
EGYPTIAN HIEROGLYPH F043

1312F
EGYPTIAN HIEROGLYPH F044

13130
EGYPTIAN HIEROGLYPH F045

13131
EGYPTIAN HIEROGLYPH F045A

13132
EGYPTIAN HIEROGLYPH F046

13133
EGYPTIAN HIEROGLYPH F046A

13134
EGYPTIAN HIEROGLYPH F047

13135
EGYPTIAN HIEROGLYPH F047A

13136
EGYPTIAN HIEROGLYPH F048

13137
EGYPTIAN HIEROGLYPH F049

13138
EGYPTIAN HIEROGLYPH F050

13139
EGYPTIAN HIEROGLYPH F051

1313A
EGYPTIAN HIEROGLYPH F051A

1313B
EGYPTIAN HIEROGLYPH F051B

1313C
EGYPTIAN HIEROGLYPH F051C

1313D
EGYPTIAN HIEROGLYPH F052

1313E
EGYPTIAN HIEROGLYPH F053

1313F
EGYPTIAN HIEROGLYPH G001

13140
EGYPTIAN HIEROGLYPH G002

13141
EGYPTIAN HIEROGLYPH G003

13142
EGYPTIAN HIEROGLYPH G004

13143
EGYPTIAN HIEROGLYPH G005

13144
EGYPTIAN HIEROGLYPH G006

13145
EGYPTIAN HIEROGLYPH G006A

13146
EGYPTIAN HIEROGLYPH G007

13147
EGYPTIAN HIEROGLYPH G007A

13148
EGYPTIAN HIEROGLYPH G007B

13149
EGYPTIAN HIEROGLYPH G008

1314A
EGYPTIAN HIEROGLYPH G009

1314B
EGYPTIAN HIEROGLYPH G010

1314C
EGYPTIAN HIEROGLYPH G011

1314D
EGYPTIAN HIEROGLYPH G011A

1314E
EGYPTIAN HIEROGLYPH G012

1314F
EGYPTIAN HIEROGLYPH G013

13150
EGYPTIAN HIEROGLYPH G014

13151
EGYPTIAN HIEROGLYPH G015

13152
EGYPTIAN HIEROGLYPH G016

13153
EGYPTIAN HIEROGLYPH G017

13154
EGYPTIAN HIEROGLYPH G018

13155
EGYPTIAN HIEROGLYPH G019

13156
EGYPTIAN HIEROGLYPH G020

13157
EGYPTIAN HIEROGLYPH G020A

13158
EGYPTIAN HIEROGLYPH G021

13159
EGYPTIAN HIEROGLYPH G022

1315A
EGYPTIAN HIEROGLYPH G023

1315B
EGYPTIAN HIEROGLYPH G024

1315C
EGYPTIAN HIEROGLYPH G025

1315D
EGYPTIAN HIEROGLYPH G026

1315E
EGYPTIAN HIEROGLYPH G026A

1315F
EGYPTIAN HIEROGLYPH G027

13160
EGYPTIAN HIEROGLYPH G028

13161
EGYPTIAN HIEROGLYPH G029

13162
EGYPTIAN HIEROGLYPH G030

13163
EGYPTIAN HIEROGLYPH G031

13164
EGYPTIAN HIEROGLYPH G032

13165
EGYPTIAN HIEROGLYPH G033

13166
EGYPTIAN HIEROGLYPH G034

13167
EGYPTIAN HIEROGLYPH G035

13168
EGYPTIAN HIEROGLYPH G036

13169
EGYPTIAN HIEROGLYPH G036A

1316A
EGYPTIAN HIEROGLYPH G037

1316B
EGYPTIAN HIEROGLYPH G037A

1316C
EGYPTIAN HIEROGLYPH G038

1316D
EGYPTIAN HIEROGLYPH G039

1316E
EGYPTIAN HIEROGLYPH G040

1316F
EGYPTIAN HIEROGLYPH G041

13170
EGYPTIAN HIEROGLYPH G042

13171
EGYPTIAN HIEROGLYPH G043

13172
EGYPTIAN HIEROGLYPH G043A

13173
EGYPTIAN HIEROGLYPH G044

13174
EGYPTIAN HIEROGLYPH G045

13175
EGYPTIAN HIEROGLYPH G045A

13176
EGYPTIAN HIEROGLYPH G046

13177
EGYPTIAN HIEROGLYPH G047

13178
EGYPTIAN HIEROGLYPH G048

13179
EGYPTIAN HIEROGLYPH G049

1317A
EGYPTIAN HIEROGLYPH G050

1317B
EGYPTIAN HIEROGLYPH G051

1317C
EGYPTIAN HIEROGLYPH G052

1317D
EGYPTIAN HIEROGLYPH G053

1317E
EGYPTIAN HIEROGLYPH G054

1317F
EGYPTIAN HIEROGLYPH H001

13180
EGYPTIAN HIEROGLYPH H002

13181
EGYPTIAN HIEROGLYPH H003

13182
EGYPTIAN HIEROGLYPH H004

13183
EGYPTIAN HIEROGLYPH H005

13184
EGYPTIAN HIEROGLYPH H006

13185
EGYPTIAN HIEROGLYPH H006A

13186
EGYPTIAN HIEROGLYPH H007

13187
EGYPTIAN HIEROGLYPH H008

13188
EGYPTIAN HIEROGLYPH I001

13189
EGYPTIAN HIEROGLYPH I002

1318A
EGYPTIAN HIEROGLYPH I003

1318B
EGYPTIAN HIEROGLYPH I004

1318C
EGYPTIAN HIEROGLYPH I005

1318D
EGYPTIAN HIEROGLYPH I005A

1318E
EGYPTIAN HIEROGLYPH I006

1318F
EGYPTIAN HIEROGLYPH I007

13190
EGYPTIAN HIEROGLYPH I008

13191
EGYPTIAN HIEROGLYPH I009

13192
EGYPTIAN HIEROGLYPH I009A

13193
EGYPTIAN HIEROGLYPH I010

13194
EGYPTIAN HIEROGLYPH I010A

13195
EGYPTIAN HIEROGLYPH I011

13196
EGYPTIAN HIEROGLYPH I011A

13197
EGYPTIAN HIEROGLYPH I012

13198
EGYPTIAN HIEROGLYPH I013

13199
EGYPTIAN HIEROGLYPH I014

1319A
EGYPTIAN HIEROGLYPH I015

1319B
EGYPTIAN HIEROGLYPH K001

1319C
EGYPTIAN HIEROGLYPH K002

1319D
EGYPTIAN HIEROGLYPH K003

1319E
EGYPTIAN HIEROGLYPH K004

1319F
EGYPTIAN HIEROGLYPH K005

131A0
EGYPTIAN HIEROGLYPH K006

131A1
EGYPTIAN HIEROGLYPH K007

131A2
EGYPTIAN HIEROGLYPH K008

131A3
EGYPTIAN HIEROGLYPH L001

131A4
EGYPTIAN HIEROGLYPH L002

131A5
EGYPTIAN HIEROGLYPH L002A

131A6
EGYPTIAN HIEROGLYPH L003

131A7
EGYPTIAN HIEROGLYPH L004

131A8
EGYPTIAN HIEROGLYPH L005

131A9
EGYPTIAN HIEROGLYPH L006

131AA
EGYPTIAN HIEROGLYPH L006A

131AB
EGYPTIAN HIEROGLYPH L007

131AC
EGYPTIAN HIEROGLYPH L008

131AD
EGYPTIAN HIEROGLYPH M001

131AE
EGYPTIAN HIEROGLYPH M001A

131AF
EGYPTIAN HIEROGLYPH M001B

131B0
EGYPTIAN HIEROGLYPH M002

131B1
EGYPTIAN HIEROGLYPH M003

131B2
EGYPTIAN HIEROGLYPH M003A

131B3
EGYPTIAN HIEROGLYPH M004

131B4
EGYPTIAN HIEROGLYPH M005

131B5
EGYPTIAN HIEROGLYPH M006

131B6
EGYPTIAN HIEROGLYPH M007

131B7
EGYPTIAN HIEROGLYPH M008

131B8
EGYPTIAN HIEROGLYPH M009

131B9
EGYPTIAN HIEROGLYPH M010

131BA
EGYPTIAN HIEROGLYPH M010A

131BB
EGYPTIAN HIEROGLYPH M011

131BC
EGYPTIAN HIEROGLYPH M012

131BD
EGYPTIAN HIEROGLYPH M012A

131BE
EGYPTIAN HIEROGLYPH M012B

131BF
EGYPTIAN HIEROGLYPH M012C

131C0
EGYPTIAN HIEROGLYPH M012D

131C1
EGYPTIAN HIEROGLYPH M012E

131C2
EGYPTIAN HIEROGLYPH M012F

131C3
EGYPTIAN HIEROGLYPH M012G

131C4
EGYPTIAN HIEROGLYPH M012H

131C5
EGYPTIAN HIEROGLYPH M013

131C6
EGYPTIAN HIEROGLYPH M014

131C7
EGYPTIAN HIEROGLYPH M015

131C8
EGYPTIAN HIEROGLYPH M015A

131C9
EGYPTIAN HIEROGLYPH M016

131CA
EGYPTIAN HIEROGLYPH M016A

131CB
EGYPTIAN HIEROGLYPH M017

131CC
EGYPTIAN HIEROGLYPH M017A

131CD
EGYPTIAN HIEROGLYPH M018

131CE
EGYPTIAN HIEROGLYPH M019

131CF
EGYPTIAN HIEROGLYPH M020

131D0
EGYPTIAN HIEROGLYPH M021

131D1
EGYPTIAN HIEROGLYPH M022

131D2
EGYPTIAN HIEROGLYPH M022A

131D3
EGYPTIAN HIEROGLYPH M023

131D4
EGYPTIAN HIEROGLYPH M024

131D5
EGYPTIAN HIEROGLYPH M024A

131D6
EGYPTIAN HIEROGLYPH M025

131D7
EGYPTIAN HIEROGLYPH M026

131D8
EGYPTIAN HIEROGLYPH M027

131D9
EGYPTIAN HIEROGLYPH M028

131DA
EGYPTIAN HIEROGLYPH M028A

131DB
EGYPTIAN HIEROGLYPH M029

131DC
EGYPTIAN HIEROGLYPH M030

131DD
EGYPTIAN HIEROGLYPH M031

131DE
EGYPTIAN HIEROGLYPH M031A

131DF
EGYPTIAN HIEROGLYPH M032

131E0
EGYPTIAN HIEROGLYPH M033

131E1
EGYPTIAN HIEROGLYPH M033A

131E2
EGYPTIAN HIEROGLYPH M033B

131E3
EGYPTIAN HIEROGLYPH M034

131E4
EGYPTIAN HIEROGLYPH M035

131E5
EGYPTIAN HIEROGLYPH M036

131E6
EGYPTIAN HIEROGLYPH M037

131E7
EGYPTIAN HIEROGLYPH M038

131E8
EGYPTIAN HIEROGLYPH M039

131E9
EGYPTIAN HIEROGLYPH M040

131EA
EGYPTIAN HIEROGLYPH M040A

131EB
EGYPTIAN HIEROGLYPH M041

131EC
EGYPTIAN HIEROGLYPH M042

131ED
EGYPTIAN HIEROGLYPH M043

131EE
EGYPTIAN HIEROGLYPH M044

131EF
EGYPTIAN HIEROGLYPH N001

131F0
EGYPTIAN HIEROGLYPH N002

131F1
EGYPTIAN HIEROGLYPH N003

131F2
EGYPTIAN HIEROGLYPH N004

131F3
EGYPTIAN HIEROGLYPH N005

131F4
EGYPTIAN HIEROGLYPH N006

131F5
EGYPTIAN HIEROGLYPH N007

131F6
EGYPTIAN HIEROGLYPH N008

131F7
EGYPTIAN HIEROGLYPH N009

131F8
EGYPTIAN HIEROGLYPH N010

131F9
EGYPTIAN HIEROGLYPH N011

131FA
EGYPTIAN HIEROGLYPH N012

131FB
EGYPTIAN HIEROGLYPH N013

131FC
EGYPTIAN HIEROGLYPH N014

131FD
EGYPTIAN HIEROGLYPH N015

131FE
EGYPTIAN HIEROGLYPH N016

131FF
EGYPTIAN HIEROGLYPH N017

13200
EGYPTIAN HIEROGLYPH N018

13201
EGYPTIAN HIEROGLYPH N018A

13202
EGYPTIAN HIEROGLYPH N018B

13203
EGYPTIAN HIEROGLYPH N019

13204
EGYPTIAN HIEROGLYPH N020

13205
EGYPTIAN HIEROGLYPH N021

13206
EGYPTIAN HIEROGLYPH N022

13207
EGYPTIAN HIEROGLYPH N023

13208
EGYPTIAN HIEROGLYPH N024

13209
EGYPTIAN HIEROGLYPH N025

1320A
EGYPTIAN HIEROGLYPH N025A

1320B
EGYPTIAN HIEROGLYPH N026

1320C
EGYPTIAN HIEROGLYPH N027

1320D
EGYPTIAN HIEROGLYPH N028

1320E
EGYPTIAN HIEROGLYPH N029

1320F
EGYPTIAN HIEROGLYPH N030

13210
EGYPTIAN HIEROGLYPH N031

13211
EGYPTIAN HIEROGLYPH N032

13212
EGYPTIAN HIEROGLYPH N033

13213
EGYPTIAN HIEROGLYPH N033A

13214
EGYPTIAN HIEROGLYPH N034

13215
EGYPTIAN HIEROGLYPH N034A

13216
EGYPTIAN HIEROGLYPH N035

13217
EGYPTIAN HIEROGLYPH N035A

13218
EGYPTIAN HIEROGLYPH N036

13219
EGYPTIAN HIEROGLYPH N037

1321A
EGYPTIAN HIEROGLYPH N037A

1321B
EGYPTIAN HIEROGLYPH N038

1321C
EGYPTIAN HIEROGLYPH N039

1321D
EGYPTIAN HIEROGLYPH N040

1321E
EGYPTIAN HIEROGLYPH N041

1321F
EGYPTIAN HIEROGLYPH N042

13220
EGYPTIAN HIEROGLYPH NL001

13221
EGYPTIAN HIEROGLYPH NL002

13222
EGYPTIAN HIEROGLYPH NL003

13223
EGYPTIAN HIEROGLYPH NL004

13224
EGYPTIAN HIEROGLYPH NL005

13225
EGYPTIAN HIEROGLYPH NL005A

13226
EGYPTIAN HIEROGLYPH NL006

13227
EGYPTIAN HIEROGLYPH NL007

13228
EGYPTIAN HIEROGLYPH NL008

13229
EGYPTIAN HIEROGLYPH NL009

1322A
EGYPTIAN HIEROGLYPH NL010

1322B
EGYPTIAN HIEROGLYPH NL011

1322C
EGYPTIAN HIEROGLYPH NL012

1322D
EGYPTIAN HIEROGLYPH NL013

1322E
EGYPTIAN HIEROGLYPH NL014

1322F
EGYPTIAN HIEROGLYPH NL015

13230
EGYPTIAN HIEROGLYPH NL016

13231
EGYPTIAN HIEROGLYPH NL017

13232
EGYPTIAN HIEROGLYPH NL017A

13233
EGYPTIAN HIEROGLYPH NL018

13234
EGYPTIAN HIEROGLYPH NL019

13235
EGYPTIAN HIEROGLYPH NL020

13236
EGYPTIAN HIEROGLYPH NU001

13237
EGYPTIAN HIEROGLYPH NU002

13238
EGYPTIAN HIEROGLYPH NU003

13239
EGYPTIAN HIEROGLYPH NU004

1323A
EGYPTIAN HIEROGLYPH NU005

1323B
EGYPTIAN HIEROGLYPH NU006

1323C
EGYPTIAN HIEROGLYPH NU007

1323D
EGYPTIAN HIEROGLYPH NU008

1323E
EGYPTIAN HIEROGLYPH NU009

1323F
EGYPTIAN HIEROGLYPH NU010

13240
EGYPTIAN HIEROGLYPH NU010A

13241
EGYPTIAN HIEROGLYPH NU011

13242
EGYPTIAN HIEROGLYPH NU011A

13243
EGYPTIAN HIEROGLYPH NU012

13244
EGYPTIAN HIEROGLYPH NU013

13245
EGYPTIAN HIEROGLYPH NU014

13246
EGYPTIAN HIEROGLYPH NU015

13247
EGYPTIAN HIEROGLYPH NU016

13248
EGYPTIAN HIEROGLYPH NU017

13249
EGYPTIAN HIEROGLYPH NU018

1324A
EGYPTIAN HIEROGLYPH NU018A

1324B
EGYPTIAN HIEROGLYPH NU019

1324C
EGYPTIAN HIEROGLYPH NU020

1324D
EGYPTIAN HIEROGLYPH NU021

1324E
EGYPTIAN HIEROGLYPH NU022

1324F
EGYPTIAN HIEROGLYPH NU022A

13250
EGYPTIAN HIEROGLYPH O001

13251
EGYPTIAN HIEROGLYPH O001A

13252
EGYPTIAN HIEROGLYPH O002

13253
EGYPTIAN HIEROGLYPH O003

13254
EGYPTIAN HIEROGLYPH O004

13255
EGYPTIAN HIEROGLYPH O005

13256
EGYPTIAN HIEROGLYPH O005A

13257
EGYPTIAN HIEROGLYPH O006

13258
EGYPTIAN HIEROGLYPH O006A

13259
EGYPTIAN HIEROGLYPH O006B

1325A
EGYPTIAN HIEROGLYPH O006C

1325B
EGYPTIAN HIEROGLYPH O006D

1325C
EGYPTIAN HIEROGLYPH O006E

1325D
EGYPTIAN HIEROGLYPH O006F

1325E
EGYPTIAN HIEROGLYPH O007

1325F
EGYPTIAN HIEROGLYPH O008

13260
EGYPTIAN HIEROGLYPH O009

13261
EGYPTIAN HIEROGLYPH O010

13262
EGYPTIAN HIEROGLYPH O010A

13263
EGYPTIAN HIEROGLYPH O010B

13264
EGYPTIAN HIEROGLYPH O010C

13265
EGYPTIAN HIEROGLYPH O011

13266
EGYPTIAN HIEROGLYPH O012

13267
EGYPTIAN HIEROGLYPH O013

13268
EGYPTIAN HIEROGLYPH O014

13269
EGYPTIAN HIEROGLYPH O015

1326A
EGYPTIAN HIEROGLYPH O016

1326B
EGYPTIAN HIEROGLYPH O017

1326C
EGYPTIAN HIEROGLYPH O018

1326D
EGYPTIAN HIEROGLYPH O019

1326E
EGYPTIAN HIEROGLYPH O019A

1326F
EGYPTIAN HIEROGLYPH O020

13270
EGYPTIAN HIEROGLYPH O020A

13271
EGYPTIAN HIEROGLYPH O021

13272
EGYPTIAN HIEROGLYPH O022

13273
EGYPTIAN HIEROGLYPH O023

13274
EGYPTIAN HIEROGLYPH O024

13275
EGYPTIAN HIEROGLYPH O024A

13276
EGYPTIAN HIEROGLYPH O025

13277
EGYPTIAN HIEROGLYPH O025A

13278
EGYPTIAN HIEROGLYPH O026

13279
EGYPTIAN HIEROGLYPH O027

1327A
EGYPTIAN HIEROGLYPH O028

1327B
EGYPTIAN HIEROGLYPH O029

1327C
EGYPTIAN HIEROGLYPH O029A

1327D
EGYPTIAN HIEROGLYPH O030

1327E
EGYPTIAN HIEROGLYPH O030A

1327F
EGYPTIAN HIEROGLYPH O031

13280
EGYPTIAN HIEROGLYPH O032

13281
EGYPTIAN HIEROGLYPH O033

13282
EGYPTIAN HIEROGLYPH O033A

13283
EGYPTIAN HIEROGLYPH O034

13284
EGYPTIAN HIEROGLYPH O035

13285
EGYPTIAN HIEROGLYPH O036

13286
EGYPTIAN HIEROGLYPH O036A

13287
EGYPTIAN HIEROGLYPH O036B

13288
EGYPTIAN HIEROGLYPH O036C

13289
EGYPTIAN HIEROGLYPH O036D

1328A
EGYPTIAN HIEROGLYPH O037

1328B
EGYPTIAN HIEROGLYPH O038

1328C
EGYPTIAN HIEROGLYPH O039

1328D
EGYPTIAN HIEROGLYPH O040

1328E
EGYPTIAN HIEROGLYPH O041

1328F
EGYPTIAN HIEROGLYPH O042

13290
EGYPTIAN HIEROGLYPH O043

13291
EGYPTIAN HIEROGLYPH O044

13292
EGYPTIAN HIEROGLYPH O045

13293
EGYPTIAN HIEROGLYPH O046

13294
EGYPTIAN HIEROGLYPH O047

13295
EGYPTIAN HIEROGLYPH O048

13296
EGYPTIAN HIEROGLYPH O049

13297
EGYPTIAN HIEROGLYPH O050

13298
EGYPTIAN HIEROGLYPH O050A

13299
EGYPTIAN HIEROGLYPH O050B

1329A
EGYPTIAN HIEROGLYPH O051

1329B
EGYPTIAN HIEROGLYPH P001

1329C
EGYPTIAN HIEROGLYPH P001A

1329D
EGYPTIAN HIEROGLYPH P002

1329E
EGYPTIAN HIEROGLYPH P003

1329F
EGYPTIAN HIEROGLYPH P003A

132A0
EGYPTIAN HIEROGLYPH P004

132A1
EGYPTIAN HIEROGLYPH P005

132A2
EGYPTIAN HIEROGLYPH P006

132A3
EGYPTIAN HIEROGLYPH P007

132A4
EGYPTIAN HIEROGLYPH P008

132A5
EGYPTIAN HIEROGLYPH P009

132A6
EGYPTIAN HIEROGLYPH P010

132A7
EGYPTIAN HIEROGLYPH P011

132A8
EGYPTIAN HIEROGLYPH Q001

132A9
EGYPTIAN HIEROGLYPH Q002

132AA
EGYPTIAN HIEROGLYPH Q003

132AB
EGYPTIAN HIEROGLYPH Q004

132AC
EGYPTIAN HIEROGLYPH Q005

132AD
EGYPTIAN HIEROGLYPH Q006

132AE
EGYPTIAN HIEROGLYPH Q007

132AF
EGYPTIAN HIEROGLYPH R001

132B0
EGYPTIAN HIEROGLYPH R002

132B1
EGYPTIAN HIEROGLYPH R002A

132B2
EGYPTIAN HIEROGLYPH R003

132B3
EGYPTIAN HIEROGLYPH R003A

132B4
EGYPTIAN HIEROGLYPH R003B

132B5
EGYPTIAN HIEROGLYPH R004

132B6
EGYPTIAN HIEROGLYPH R005

132B7
EGYPTIAN HIEROGLYPH R006

132B8
EGYPTIAN HIEROGLYPH R007

132B9
EGYPTIAN HIEROGLYPH R008

132BA
EGYPTIAN HIEROGLYPH R009

132BB
EGYPTIAN HIEROGLYPH R010

132BC
EGYPTIAN HIEROGLYPH R010A

132BD
EGYPTIAN HIEROGLYPH R011

132BE
EGYPTIAN HIEROGLYPH R012

132BF
EGYPTIAN HIEROGLYPH R013

132C0
EGYPTIAN HIEROGLYPH R014

132C1
EGYPTIAN HIEROGLYPH R015

132C2
EGYPTIAN HIEROGLYPH R016

132C3
EGYPTIAN HIEROGLYPH R016A

132C4
EGYPTIAN HIEROGLYPH R017

132C5
EGYPTIAN HIEROGLYPH R018

132C6
EGYPTIAN HIEROGLYPH R019

132C7
EGYPTIAN HIEROGLYPH R020

132C8
EGYPTIAN HIEROGLYPH R021

132C9
EGYPTIAN HIEROGLYPH R022

132CA
EGYPTIAN HIEROGLYPH R023

132CB
EGYPTIAN HIEROGLYPH R024

132CC
EGYPTIAN HIEROGLYPH R025

132CD
EGYPTIAN HIEROGLYPH R026

132CE
EGYPTIAN HIEROGLYPH R027

132CF
EGYPTIAN HIEROGLYPH R028

132D0
EGYPTIAN HIEROGLYPH R029

132D1
EGYPTIAN HIEROGLYPH S001

132D2
EGYPTIAN HIEROGLYPH S002

132D3
EGYPTIAN HIEROGLYPH S002A

132D4
EGYPTIAN HIEROGLYPH S003

132D5
EGYPTIAN HIEROGLYPH S004

132D6
EGYPTIAN HIEROGLYPH S005

132D7
EGYPTIAN HIEROGLYPH S006

132D8
EGYPTIAN HIEROGLYPH S006A

132D9
EGYPTIAN HIEROGLYPH S007

132DA
EGYPTIAN HIEROGLYPH S008

132DB
EGYPTIAN HIEROGLYPH S009

132DC
EGYPTIAN HIEROGLYPH S010

132DD
EGYPTIAN HIEROGLYPH S011

132DE
EGYPTIAN HIEROGLYPH S012

132DF
EGYPTIAN HIEROGLYPH S013

132E0
EGYPTIAN HIEROGLYPH S014

132E1
EGYPTIAN HIEROGLYPH S014A

132E2
EGYPTIAN HIEROGLYPH S014B

132E3
EGYPTIAN HIEROGLYPH S015

132E4
EGYPTIAN HIEROGLYPH S016

132E5
EGYPTIAN HIEROGLYPH S017

132E6
EGYPTIAN HIEROGLYPH S017A

132E7
EGYPTIAN HIEROGLYPH S018

132E8
EGYPTIAN HIEROGLYPH S019

132E9
EGYPTIAN HIEROGLYPH S020

132EA
EGYPTIAN HIEROGLYPH S021

132EB
EGYPTIAN HIEROGLYPH S022

132EC
EGYPTIAN HIEROGLYPH S023

132ED
EGYPTIAN HIEROGLYPH S024

132EE
EGYPTIAN HIEROGLYPH S025

132EF
EGYPTIAN HIEROGLYPH S026

132F0
EGYPTIAN HIEROGLYPH S026A

132F1
EGYPTIAN HIEROGLYPH S026B

132F2
EGYPTIAN HIEROGLYPH S027

132F3
EGYPTIAN HIEROGLYPH S028

132F4
EGYPTIAN HIEROGLYPH S029

132F5
EGYPTIAN HIEROGLYPH S030

132F6
EGYPTIAN HIEROGLYPH S031

132F7
EGYPTIAN HIEROGLYPH S032

132F8
EGYPTIAN HIEROGLYPH S033

132F9
EGYPTIAN HIEROGLYPH S034

132FA
EGYPTIAN HIEROGLYPH S035

132FB
EGYPTIAN HIEROGLYPH S035A

132FC
EGYPTIAN HIEROGLYPH S036

132FD
EGYPTIAN HIEROGLYPH S037

132FE
EGYPTIAN HIEROGLYPH S038

132FF
EGYPTIAN HIEROGLYPH S039

13300
EGYPTIAN HIEROGLYPH S040

13301
EGYPTIAN HIEROGLYPH S041

13302
EGYPTIAN HIEROGLYPH S042

13303
EGYPTIAN HIEROGLYPH S043

13304
EGYPTIAN HIEROGLYPH S044

13305
EGYPTIAN HIEROGLYPH S045

13306
EGYPTIAN HIEROGLYPH S046

13307
EGYPTIAN HIEROGLYPH T001

13308
EGYPTIAN HIEROGLYPH T002

13309
EGYPTIAN HIEROGLYPH T003

1330A
EGYPTIAN HIEROGLYPH T003A

1330B
EGYPTIAN HIEROGLYPH T004

1330C
EGYPTIAN HIEROGLYPH T005

1330D
EGYPTIAN HIEROGLYPH T006

1330E
EGYPTIAN HIEROGLYPH T007

1330F
EGYPTIAN HIEROGLYPH T007A

13310
EGYPTIAN HIEROGLYPH T008

13311
EGYPTIAN HIEROGLYPH T008A

13312
EGYPTIAN HIEROGLYPH T009

13313
EGYPTIAN HIEROGLYPH T009A

13314
EGYPTIAN HIEROGLYPH T010

13315
EGYPTIAN HIEROGLYPH T011

13316
EGYPTIAN HIEROGLYPH T011A

13317
EGYPTIAN HIEROGLYPH T012

13318
EGYPTIAN HIEROGLYPH T013

13319
EGYPTIAN HIEROGLYPH T014

1331A
EGYPTIAN HIEROGLYPH T015

1331B
EGYPTIAN HIEROGLYPH T016

1331C
EGYPTIAN HIEROGLYPH T016A

1331D
EGYPTIAN HIEROGLYPH T017

1331E
EGYPTIAN HIEROGLYPH T018

1331F
EGYPTIAN HIEROGLYPH T019

13320
EGYPTIAN HIEROGLYPH T020

13321
EGYPTIAN HIEROGLYPH T021

13322
EGYPTIAN HIEROGLYPH T022

13323
EGYPTIAN HIEROGLYPH T023

13324
EGYPTIAN HIEROGLYPH T024

13325
EGYPTIAN HIEROGLYPH T025

13326
EGYPTIAN HIEROGLYPH T026

13327
EGYPTIAN HIEROGLYPH T027

13328
EGYPTIAN HIEROGLYPH T028

13329
EGYPTIAN HIEROGLYPH T029

1332A
EGYPTIAN HIEROGLYPH T030

1332B
EGYPTIAN HIEROGLYPH T031

1332C
EGYPTIAN HIEROGLYPH T032

1332D
EGYPTIAN HIEROGLYPH T032A

1332E
EGYPTIAN HIEROGLYPH T033

1332F
EGYPTIAN HIEROGLYPH T033A

13330
EGYPTIAN HIEROGLYPH T034

13331
EGYPTIAN HIEROGLYPH T035

13332
EGYPTIAN HIEROGLYPH T036

13333
EGYPTIAN HIEROGLYPH U001

13334
EGYPTIAN HIEROGLYPH U002

13335
EGYPTIAN HIEROGLYPH U003

13336
EGYPTIAN HIEROGLYPH U004

13337
EGYPTIAN HIEROGLYPH U005

13338
EGYPTIAN HIEROGLYPH U006

13339
EGYPTIAN HIEROGLYPH U006A

1333A
EGYPTIAN HIEROGLYPH U006B

1333B
EGYPTIAN HIEROGLYPH U007

1333C
EGYPTIAN HIEROGLYPH U008

1333D
EGYPTIAN HIEROGLYPH U009

1333E
EGYPTIAN HIEROGLYPH U010

1333F
EGYPTIAN HIEROGLYPH U011

13340
EGYPTIAN HIEROGLYPH U012

13341
EGYPTIAN HIEROGLYPH U013

13342
EGYPTIAN HIEROGLYPH U014

13343
EGYPTIAN HIEROGLYPH U015

13344
EGYPTIAN HIEROGLYPH U016

13345
EGYPTIAN HIEROGLYPH U017

13346
EGYPTIAN HIEROGLYPH U018

13347
EGYPTIAN HIEROGLYPH U019

13348
EGYPTIAN HIEROGLYPH U020

13349
EGYPTIAN HIEROGLYPH U021

1334A
EGYPTIAN HIEROGLYPH U022

1334B
EGYPTIAN HIEROGLYPH U023

1334C
EGYPTIAN HIEROGLYPH U023A

1334D
EGYPTIAN HIEROGLYPH U024

1334E
EGYPTIAN HIEROGLYPH U025

1334F
EGYPTIAN HIEROGLYPH U026

13350
EGYPTIAN HIEROGLYPH U027

13351
EGYPTIAN HIEROGLYPH U028

13352
EGYPTIAN HIEROGLYPH U029

13353
EGYPTIAN HIEROGLYPH U029A

13354
EGYPTIAN HIEROGLYPH U030

13355
EGYPTIAN HIEROGLYPH U031

13356
EGYPTIAN HIEROGLYPH U032

13357
EGYPTIAN HIEROGLYPH U032A

13358
EGYPTIAN HIEROGLYPH U033

13359
EGYPTIAN HIEROGLYPH U034

1335A
EGYPTIAN HIEROGLYPH U035

1335B
EGYPTIAN HIEROGLYPH U036

1335C
EGYPTIAN HIEROGLYPH U037

1335D
EGYPTIAN HIEROGLYPH U038

1335E
EGYPTIAN HIEROGLYPH U039

1335F
EGYPTIAN HIEROGLYPH U040

13360
EGYPTIAN HIEROGLYPH U041

13361
EGYPTIAN HIEROGLYPH U042

13362
EGYPTIAN HIEROGLYPH V001

13363
EGYPTIAN HIEROGLYPH V001A

13364
EGYPTIAN HIEROGLYPH V001B

13365
EGYPTIAN HIEROGLYPH V001C

13366
EGYPTIAN HIEROGLYPH V001D

13367
EGYPTIAN HIEROGLYPH V001E

13368
EGYPTIAN HIEROGLYPH V001F

13369
EGYPTIAN HIEROGLYPH V001G

1336A
EGYPTIAN HIEROGLYPH V001H

1336B
EGYPTIAN HIEROGLYPH V001I

1336C
EGYPTIAN HIEROGLYPH V002

1336D
EGYPTIAN HIEROGLYPH V002A

1336E
EGYPTIAN HIEROGLYPH V003

1336F
EGYPTIAN HIEROGLYPH V004

13370
EGYPTIAN HIEROGLYPH V005

13371
EGYPTIAN HIEROGLYPH V006

13372
EGYPTIAN HIEROGLYPH V007

13373
EGYPTIAN HIEROGLYPH V007A

13374
EGYPTIAN HIEROGLYPH V007B

13375
EGYPTIAN HIEROGLYPH V008

13376
EGYPTIAN HIEROGLYPH V009

13377
EGYPTIAN HIEROGLYPH V010

13378
EGYPTIAN HIEROGLYPH V011

13379
EGYPTIAN HIEROGLYPH V011A

1337A
EGYPTIAN HIEROGLYPH V011B

1337B
EGYPTIAN HIEROGLYPH V011C

1337C
EGYPTIAN HIEROGLYPH V012

1337D
EGYPTIAN HIEROGLYPH V012A

1337E
EGYPTIAN HIEROGLYPH V012B

1337F
EGYPTIAN HIEROGLYPH V013

13380
EGYPTIAN HIEROGLYPH V014

13381
EGYPTIAN HIEROGLYPH V015

13382
EGYPTIAN HIEROGLYPH V016

13383
EGYPTIAN HIEROGLYPH V017

13384
EGYPTIAN HIEROGLYPH V018

13385
EGYPTIAN HIEROGLYPH V019

13386
EGYPTIAN HIEROGLYPH V020

13387
EGYPTIAN HIEROGLYPH V020A

13388
EGYPTIAN HIEROGLYPH V020B

13389
EGYPTIAN HIEROGLYPH V020C

1338A
EGYPTIAN HIEROGLYPH V020D

1338B
EGYPTIAN HIEROGLYPH V020E

1338C
EGYPTIAN HIEROGLYPH V020F

1338D
EGYPTIAN HIEROGLYPH V020G

1338E
EGYPTIAN HIEROGLYPH V020H

1338F
EGYPTIAN HIEROGLYPH V020I

13390
EGYPTIAN HIEROGLYPH V020J

13391
EGYPTIAN HIEROGLYPH V020K

13392
EGYPTIAN HIEROGLYPH V020L

13393
EGYPTIAN HIEROGLYPH V021

13394
EGYPTIAN HIEROGLYPH V022

13395
EGYPTIAN HIEROGLYPH V023

13396
EGYPTIAN HIEROGLYPH V023A

13397
EGYPTIAN HIEROGLYPH V024

13398
EGYPTIAN HIEROGLYPH V025

13399
EGYPTIAN HIEROGLYPH V026

1339A
EGYPTIAN HIEROGLYPH V027

1339B
EGYPTIAN HIEROGLYPH V028

1339C
EGYPTIAN HIEROGLYPH V028A

1339D
EGYPTIAN HIEROGLYPH V029

1339E
EGYPTIAN HIEROGLYPH V029A

1339F
EGYPTIAN HIEROGLYPH V030

133A0
EGYPTIAN HIEROGLYPH V030A

133A1
EGYPTIAN HIEROGLYPH V031

133A2
EGYPTIAN HIEROGLYPH V031A

133A3
EGYPTIAN HIEROGLYPH V032

133A4
EGYPTIAN HIEROGLYPH V033

133A5
EGYPTIAN HIEROGLYPH V033A

133A6
EGYPTIAN HIEROGLYPH V034

133A7
EGYPTIAN HIEROGLYPH V035

133A8
EGYPTIAN HIEROGLYPH V036

133A9
EGYPTIAN HIEROGLYPH V037

133AA
EGYPTIAN HIEROGLYPH V037A

133AB
EGYPTIAN HIEROGLYPH V038

133AC
EGYPTIAN HIEROGLYPH V039

133AD
EGYPTIAN HIEROGLYPH V040

133AE
EGYPTIAN HIEROGLYPH V040A

133AF
EGYPTIAN HIEROGLYPH W001

133B0
EGYPTIAN HIEROGLYPH W002

133B1
EGYPTIAN HIEROGLYPH W003

133B2
EGYPTIAN HIEROGLYPH W003A

133B3
EGYPTIAN HIEROGLYPH W004

133B4
EGYPTIAN HIEROGLYPH W005

133B5
EGYPTIAN HIEROGLYPH W006

133B6
EGYPTIAN HIEROGLYPH W007

133B7
EGYPTIAN HIEROGLYPH W008

133B8
EGYPTIAN HIEROGLYPH W009

133B9
EGYPTIAN HIEROGLYPH W009A

133BA
EGYPTIAN HIEROGLYPH W010

133BB
EGYPTIAN HIEROGLYPH W010A

133BC
EGYPTIAN HIEROGLYPH W011

133BD
EGYPTIAN HIEROGLYPH W012

133BE
EGYPTIAN HIEROGLYPH W013

133BF
EGYPTIAN HIEROGLYPH W014

133C0
EGYPTIAN HIEROGLYPH W014A

133C1
EGYPTIAN HIEROGLYPH W015

133C2
EGYPTIAN HIEROGLYPH W016

133C3
EGYPTIAN HIEROGLYPH W017

133C4
EGYPTIAN HIEROGLYPH W017A

133C5
EGYPTIAN HIEROGLYPH W018

133C6
EGYPTIAN HIEROGLYPH W018A

133C7
EGYPTIAN HIEROGLYPH W019

133C8
EGYPTIAN HIEROGLYPH W020

133C9
EGYPTIAN HIEROGLYPH W021

133CA
EGYPTIAN HIEROGLYPH W022

133CB
EGYPTIAN HIEROGLYPH W023

133CC
EGYPTIAN HIEROGLYPH W024

133CD
EGYPTIAN HIEROGLYPH W024A

133CE
EGYPTIAN HIEROGLYPH W025

133CF
EGYPTIAN HIEROGLYPH X001

133D0
EGYPTIAN HIEROGLYPH X002

133D1
EGYPTIAN HIEROGLYPH X003

133D2
EGYPTIAN HIEROGLYPH X004

133D3
EGYPTIAN HIEROGLYPH X004A

133D4
EGYPTIAN HIEROGLYPH X004B

133D5
EGYPTIAN HIEROGLYPH X005

133D6
EGYPTIAN HIEROGLYPH X006

133D7
EGYPTIAN HIEROGLYPH X006A

133D8
EGYPTIAN HIEROGLYPH X007

133D9
EGYPTIAN HIEROGLYPH X008

133DA
EGYPTIAN HIEROGLYPH X008A

133DB
EGYPTIAN HIEROGLYPH Y001

133DC
EGYPTIAN HIEROGLYPH Y001A

133DD
EGYPTIAN HIEROGLYPH Y002

133DE
EGYPTIAN HIEROGLYPH Y003

133DF
EGYPTIAN HIEROGLYPH Y004

133E0
EGYPTIAN HIEROGLYPH Y005

133E1
EGYPTIAN HIEROGLYPH Y006

133E2
EGYPTIAN HIEROGLYPH Y007

133E3
EGYPTIAN HIEROGLYPH Y008

133E4
EGYPTIAN HIEROGLYPH Z001

133E5
EGYPTIAN HIEROGLYPH Z002

133E6
EGYPTIAN HIEROGLYPH Z002A

133E7
EGYPTIAN HIEROGLYPH Z002B

133E8
EGYPTIAN HIEROGLYPH Z002C

133E9
EGYPTIAN HIEROGLYPH Z002D

133EA
EGYPTIAN HIEROGLYPH Z003

133EB
EGYPTIAN HIEROGLYPH Z003A

133EC
EGYPTIAN HIEROGLYPH Z003B

133ED
EGYPTIAN HIEROGLYPH Z004

133EE
EGYPTIAN HIEROGLYPH Z004A

133EF
EGYPTIAN HIEROGLYPH Z005

133F0
EGYPTIAN HIEROGLYPH Z005A

133F1
EGYPTIAN HIEROGLYPH Z006

133F2
EGYPTIAN HIEROGLYPH Z007

133F3
EGYPTIAN HIEROGLYPH Z008

133F4
EGYPTIAN HIEROGLYPH Z009

133F5
EGYPTIAN HIEROGLYPH Z010

133F6
EGYPTIAN HIEROGLYPH Z011

133F7
EGYPTIAN HIEROGLYPH Z012

133F8
EGYPTIAN HIEROGLYPH Z013

133F9
EGYPTIAN HIEROGLYPH Z014

133FA
EGYPTIAN HIEROGLYPH Z015

133FB
EGYPTIAN HIEROGLYPH Z015A

133FC
EGYPTIAN HIEROGLYPH Z015B

133FD
EGYPTIAN HIEROGLYPH Z015C

133FE
EGYPTIAN HIEROGLYPH Z015D

133FF
EGYPTIAN HIEROGLYPH Z015E

13400
EGYPTIAN HIEROGLYPH Z015F

13401
EGYPTIAN HIEROGLYPH Z015G

13402
EGYPTIAN HIEROGLYPH Z015H

13403
EGYPTIAN HIEROGLYPH Z015I

13404
EGYPTIAN HIEROGLYPH Z016

13405
EGYPTIAN HIEROGLYPH Z016A

13406
EGYPTIAN HIEROGLYPH Z016B

13407
EGYPTIAN HIEROGLYPH Z016C

13408
EGYPTIAN HIEROGLYPH Z016D

13409
EGYPTIAN HIEROGLYPH Z016E

1340A
EGYPTIAN HIEROGLYPH Z016F

1340B
EGYPTIAN HIEROGLYPH Z016G

1340C
EGYPTIAN HIEROGLYPH Z016H

1340D
EGYPTIAN HIEROGLYPH AA001

1340E
EGYPTIAN HIEROGLYPH AA002

1340F
EGYPTIAN HIEROGLYPH AA003

13410
EGYPTIAN HIEROGLYPH AA004

13411
EGYPTIAN HIEROGLYPH AA005

13412
EGYPTIAN HIEROGLYPH AA006

13413
EGYPTIAN HIEROGLYPH AA007

13414
EGYPTIAN HIEROGLYPH AA007A

13415
EGYPTIAN HIEROGLYPH AA007B

13416
EGYPTIAN HIEROGLYPH AA008

13417
EGYPTIAN HIEROGLYPH AA009

13418
EGYPTIAN HIEROGLYPH AA010

13419
EGYPTIAN HIEROGLYPH AA011

1341A
EGYPTIAN HIEROGLYPH AA012

1341B
EGYPTIAN HIEROGLYPH AA013

1341C
EGYPTIAN HIEROGLYPH AA014

1341D
EGYPTIAN HIEROGLYPH AA015

1341E
EGYPTIAN HIEROGLYPH AA016

1341F
EGYPTIAN HIEROGLYPH AA017

13420
EGYPTIAN HIEROGLYPH AA018

13421
EGYPTIAN HIEROGLYPH AA019

13422
EGYPTIAN HIEROGLYPH AA020

13423
EGYPTIAN HIEROGLYPH AA021

13424
EGYPTIAN HIEROGLYPH AA022

13425
EGYPTIAN HIEROGLYPH AA023

13426
EGYPTIAN HIEROGLYPH AA024

13427
EGYPTIAN HIEROGLYPH AA025

13428
EGYPTIAN HIEROGLYPH AA026

13429
EGYPTIAN HIEROGLYPH AA027

1342A
EGYPTIAN HIEROGLYPH AA028

1342B
EGYPTIAN HIEROGLYPH AA029

1342C
EGYPTIAN HIEROGLYPH AA030

1342D
EGYPTIAN HIEROGLYPH AA031

1342E
EGYPTIAN HIEROGLYPH AA032

1342F
EGYPTIAN HIEROGLYPH V011D

13430
EGYPTIAN HIEROGLYPH VERTICAL JOINER

13431
EGYPTIAN HIEROGLYPH HORIZONTAL JOINER

13432
EGYPTIAN HIEROGLYPH INSERT AT TOP START

13433
EGYPTIAN HIEROGLYPH INSERT AT BOTTOM START

13434
EGYPTIAN HIEROGLYPH INSERT AT TOP END

13435
EGYPTIAN HIEROGLYPH INSERT AT BOTTOM END

13436
EGYPTIAN HIEROGLYPH OVERLAY MIDDLE

13437
EGYPTIAN HIEROGLYPH BEGIN SEGMENT

13438
EGYPTIAN HIEROGLYPH END SEGMENT

13439
EGYPTIAN HIEROGLYPH INSERT AT MIDDLE

1343A
EGYPTIAN HIEROGLYPH INSERT AT TOP

1343B
EGYPTIAN HIEROGLYPH INSERT AT BOTTOM

1343C
EGYPTIAN HIEROGLYPH BEGIN ENCLOSURE

1343D
EGYPTIAN HIEROGLYPH END ENCLOSURE

1343E
EGYPTIAN HIEROGLYPH BEGIN WALLED ENCLOSURE

1343F
EGYPTIAN HIEROGLYPH END WALLED ENCLOSURE

13440
EGYPTIAN HIEROGLYPH MIRROR HORIZONTALLY

13441
EGYPTIAN HIEROGLYPH FULL BLANK

13442
EGYPTIAN HIEROGLYPH HALF BLANK

13443
EGYPTIAN HIEROGLYPH LOST SIGN

13444
EGYPTIAN HIEROGLYPH HALF LOST SIGN

13445
EGYPTIAN HIEROGLYPH TALL LOST SIGN

13446
EGYPTIAN HIEROGLYPH WIDE LOST SIGN

13447
EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT TOP START

13448
EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT BOTTOM START

13449
EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT START

1344A
EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT TOP END

1344B
EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT TOP

1344C
EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT BOTTOM START AND TOP END

1344D
EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT START AND TOP

1344E
EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT BOTTOM END

1344F
EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT TOP START AND BOTTOM END

13450
EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT BOTTOM

13451
EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT START AND BOTTOM

13452
EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT END

13453
EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT TOP AND END

13454
EGYPTIAN HIEROGLYPH MODIFIER DAMAGED AT BOTTOM AND END

13455
EGYPTIAN HIEROGLYPH MODIFIER DAMAGED

14400
ANATOLIAN HIEROGLYPH A001

14401
ANATOLIAN HIEROGLYPH A002

14402
ANATOLIAN HIEROGLYPH A003

14403
ANATOLIAN HIEROGLYPH A004

14404
ANATOLIAN HIEROGLYPH A005

14405
ANATOLIAN HIEROGLYPH A006

14406
ANATOLIAN HIEROGLYPH A007

14407
ANATOLIAN HIEROGLYPH A008

14408
ANATOLIAN HIEROGLYPH A009

14409
ANATOLIAN HIEROGLYPH A010

1440A
ANATOLIAN HIEROGLYPH A010A

1440B
ANATOLIAN HIEROGLYPH A011

1440C
ANATOLIAN HIEROGLYPH A012

1440D
ANATOLIAN HIEROGLYPH A013

1440E
ANATOLIAN HIEROGLYPH A014

1440F
ANATOLIAN HIEROGLYPH A015

14410
ANATOLIAN HIEROGLYPH A016

14411
ANATOLIAN HIEROGLYPH A017

14412
ANATOLIAN HIEROGLYPH A018

14413
ANATOLIAN HIEROGLYPH A019

14414
ANATOLIAN HIEROGLYPH A020

14415
ANATOLIAN HIEROGLYPH A021

14416
ANATOLIAN HIEROGLYPH A022

14417
ANATOLIAN HIEROGLYPH A023

14418
ANATOLIAN HIEROGLYPH A024

14419
ANATOLIAN HIEROGLYPH A025

1441A
ANATOLIAN HIEROGLYPH A026

1441B
ANATOLIAN HIEROGLYPH A026A

1441C
ANATOLIAN HIEROGLYPH A027

1441D
ANATOLIAN HIEROGLYPH A028

1441E
ANATOLIAN HIEROGLYPH A029

1441F
ANATOLIAN HIEROGLYPH A030

14420
ANATOLIAN HIEROGLYPH A031

14421
ANATOLIAN HIEROGLYPH A032

14422
ANATOLIAN HIEROGLYPH A033

14423
ANATOLIAN HIEROGLYPH A034

14424
ANATOLIAN HIEROGLYPH A035

14425
ANATOLIAN HIEROGLYPH A036

14426
ANATOLIAN HIEROGLYPH A037

14427
ANATOLIAN HIEROGLYPH A038

14428
ANATOLIAN HIEROGLYPH A039

14429
ANATOLIAN HIEROGLYPH A039A

1442A
ANATOLIAN HIEROGLYPH A040

1442B
ANATOLIAN HIEROGLYPH A041

1442C
ANATOLIAN HIEROGLYPH A041A

1442D
ANATOLIAN HIEROGLYPH A042

1442E
ANATOLIAN HIEROGLYPH A043

1442F
ANATOLIAN HIEROGLYPH A044

14430
ANATOLIAN HIEROGLYPH A045

14431
ANATOLIAN HIEROGLYPH A045A

14432
ANATOLIAN HIEROGLYPH A046

14433
ANATOLIAN HIEROGLYPH A046A

14434
ANATOLIAN HIEROGLYPH A046B

14435
ANATOLIAN HIEROGLYPH A047

14436
ANATOLIAN HIEROGLYPH A048

14437
ANATOLIAN HIEROGLYPH A049

14438
ANATOLIAN HIEROGLYPH A050

14439
ANATOLIAN HIEROGLYPH A051

1443A
ANATOLIAN HIEROGLYPH A052

1443B
ANATOLIAN HIEROGLYPH A053

1443C
ANATOLIAN HIEROGLYPH A054

1443D
ANATOLIAN HIEROGLYPH A055

1443E
ANATOLIAN HIEROGLYPH A056

1443F
ANATOLIAN HIEROGLYPH A057

14440
ANATOLIAN HIEROGLYPH A058

14441
ANATOLIAN HIEROGLYPH A059

14442
ANATOLIAN HIEROGLYPH A060

14443
ANATOLIAN HIEROGLYPH A061

14444
ANATOLIAN HIEROGLYPH A062

14445
ANATOLIAN HIEROGLYPH A063

14446
ANATOLIAN HIEROGLYPH A064

14447
ANATOLIAN HIEROGLYPH A065

14448
ANATOLIAN HIEROGLYPH A066

14449
ANATOLIAN HIEROGLYPH A066A

1444A
ANATOLIAN HIEROGLYPH A066B

1444B
ANATOLIAN HIEROGLYPH A066C

1444C
ANATOLIAN HIEROGLYPH A067

1444D
ANATOLIAN HIEROGLYPH A068

1444E
ANATOLIAN HIEROGLYPH A069

1444F
ANATOLIAN HIEROGLYPH A070

14450
ANATOLIAN HIEROGLYPH A071

14451
ANATOLIAN HIEROGLYPH A072

14452
ANATOLIAN HIEROGLYPH A073

14453
ANATOLIAN HIEROGLYPH A074

14454
ANATOLIAN HIEROGLYPH A075

14455
ANATOLIAN HIEROGLYPH A076

14456
ANATOLIAN HIEROGLYPH A077

14457
ANATOLIAN HIEROGLYPH A078

14458
ANATOLIAN HIEROGLYPH A079

14459
ANATOLIAN HIEROGLYPH A080

1445A
ANATOLIAN HIEROGLYPH A081

1445B
ANATOLIAN HIEROGLYPH A082

1445C
ANATOLIAN HIEROGLYPH A083

1445D
ANATOLIAN HIEROGLYPH A084

1445E
ANATOLIAN HIEROGLYPH A085

1445F
ANATOLIAN HIEROGLYPH A086

14460
ANATOLIAN HIEROGLYPH A087

14461
ANATOLIAN HIEROGLYPH A088

14462
ANATOLIAN HIEROGLYPH A089

14463
ANATOLIAN HIEROGLYPH A090

14464
ANATOLIAN HIEROGLYPH A091

14465
ANATOLIAN HIEROGLYPH A092

14466
ANATOLIAN HIEROGLYPH A093

14467
ANATOLIAN HIEROGLYPH A094

14468
ANATOLIAN HIEROGLYPH A095

14469
ANATOLIAN HIEROGLYPH A096

1446A
ANATOLIAN HIEROGLYPH A097

1446B
ANATOLIAN HIEROGLYPH A097A

1446C
ANATOLIAN HIEROGLYPH A098

1446D
ANATOLIAN HIEROGLYPH A098A

1446E
ANATOLIAN HIEROGLYPH A099

1446F
ANATOLIAN HIEROGLYPH A100

14470
ANATOLIAN HIEROGLYPH A100A

14471
ANATOLIAN HIEROGLYPH A101

14472
ANATOLIAN HIEROGLYPH A101A

14473
ANATOLIAN HIEROGLYPH A102

14474
ANATOLIAN HIEROGLYPH A102A

14475
ANATOLIAN HIEROGLYPH A103

14476
ANATOLIAN HIEROGLYPH A104

14477
ANATOLIAN HIEROGLYPH A104A

14478
ANATOLIAN HIEROGLYPH A104B

14479
ANATOLIAN HIEROGLYPH A104C

1447A
ANATOLIAN HIEROGLYPH A105

1447B
ANATOLIAN HIEROGLYPH A105A

1447C
ANATOLIAN HIEROGLYPH A105B

1447D
ANATOLIAN HIEROGLYPH A106

1447E
ANATOLIAN HIEROGLYPH A107

1447F
ANATOLIAN HIEROGLYPH A107A

14480
ANATOLIAN HIEROGLYPH A107B

14481
ANATOLIAN HIEROGLYPH A107C

14482
ANATOLIAN HIEROGLYPH A108

14483
ANATOLIAN HIEROGLYPH A109

14484
ANATOLIAN HIEROGLYPH A110

14485
ANATOLIAN HIEROGLYPH A110A

14486
ANATOLIAN HIEROGLYPH A110B

14487
ANATOLIAN HIEROGLYPH A111

14488
ANATOLIAN HIEROGLYPH A112

14489
ANATOLIAN HIEROGLYPH A113

1448A
ANATOLIAN HIEROGLYPH A114

1448B
ANATOLIAN HIEROGLYPH A115

1448C
ANATOLIAN HIEROGLYPH A115A

1448D
ANATOLIAN HIEROGLYPH A116

1448E
ANATOLIAN HIEROGLYPH A117

1448F
ANATOLIAN HIEROGLYPH A118

14490
ANATOLIAN HIEROGLYPH A119

14491
ANATOLIAN HIEROGLYPH A120

14492
ANATOLIAN HIEROGLYPH A121

14493
ANATOLIAN HIEROGLYPH A122

14494
ANATOLIAN HIEROGLYPH A123

14495
ANATOLIAN HIEROGLYPH A124

14496
ANATOLIAN HIEROGLYPH A125

14497
ANATOLIAN HIEROGLYPH A125A

14498
ANATOLIAN HIEROGLYPH A126

14499
ANATOLIAN HIEROGLYPH A127

1449A
ANATOLIAN HIEROGLYPH A128

1449B
ANATOLIAN HIEROGLYPH A129

1449C
ANATOLIAN HIEROGLYPH A130

1449D
ANATOLIAN HIEROGLYPH A131

1449E
ANATOLIAN HIEROGLYPH A132

1449F
ANATOLIAN HIEROGLYPH A133

144A0
ANATOLIAN HIEROGLYPH A134

144A1
ANATOLIAN HIEROGLYPH A135

144A2
ANATOLIAN HIEROGLYPH A135A

144A3
ANATOLIAN HIEROGLYPH A136

144A4
ANATOLIAN HIEROGLYPH A137

144A5
ANATOLIAN HIEROGLYPH A138

144A6
ANATOLIAN HIEROGLYPH A139

144A7
ANATOLIAN HIEROGLYPH A140

144A8
ANATOLIAN HIEROGLYPH A141

144A9
ANATOLIAN HIEROGLYPH A142

144AA
ANATOLIAN HIEROGLYPH A143

144AB
ANATOLIAN HIEROGLYPH A144

144AC
ANATOLIAN HIEROGLYPH A145

144AD
ANATOLIAN HIEROGLYPH A146

144AE
ANATOLIAN HIEROGLYPH A147

144AF
ANATOLIAN HIEROGLYPH A148

144B0
ANATOLIAN HIEROGLYPH A149

144B1
ANATOLIAN HIEROGLYPH A150

144B2
ANATOLIAN HIEROGLYPH A151

144B3
ANATOLIAN HIEROGLYPH A152

144B4
ANATOLIAN HIEROGLYPH A153

144B5
ANATOLIAN HIEROGLYPH A154

144B6
ANATOLIAN HIEROGLYPH A155

144B7
ANATOLIAN HIEROGLYPH A156

144B8
ANATOLIAN HIEROGLYPH A157

144B9
ANATOLIAN HIEROGLYPH A158

144BA
ANATOLIAN HIEROGLYPH A159

144BB
ANATOLIAN HIEROGLYPH A160

144BC
ANATOLIAN HIEROGLYPH A161

144BD
ANATOLIAN HIEROGLYPH A162

144BE
ANATOLIAN HIEROGLYPH A163

144BF
ANATOLIAN HIEROGLYPH A164

144C0
ANATOLIAN HIEROGLYPH A165

144C1
ANATOLIAN HIEROGLYPH A166

144C2
ANATOLIAN HIEROGLYPH A167

144C3
ANATOLIAN HIEROGLYPH A168

144C4
ANATOLIAN HIEROGLYPH A169

144C5
ANATOLIAN HIEROGLYPH A170

144C6
ANATOLIAN HIEROGLYPH A171

144C7
ANATOLIAN HIEROGLYPH A172

144C8
ANATOLIAN HIEROGLYPH A173

144C9
ANATOLIAN HIEROGLYPH A174

144CA
ANATOLIAN HIEROGLYPH A175

144CB
ANATOLIAN HIEROGLYPH A176

144CC
ANATOLIAN HIEROGLYPH A177

144CD
ANATOLIAN HIEROGLYPH A178

144CE
ANATOLIAN HIEROGLYPH A179

144CF
ANATOLIAN HIEROGLYPH A180

144D0
ANATOLIAN HIEROGLYPH A181

144D1
ANATOLIAN HIEROGLYPH A182

144D2
ANATOLIAN HIEROGLYPH A183

144D3
ANATOLIAN HIEROGLYPH A184

144D4
ANATOLIAN HIEROGLYPH A185

144D5
ANATOLIAN HIEROGLYPH A186

144D6
ANATOLIAN HIEROGLYPH A187

144D7
ANATOLIAN HIEROGLYPH A188

144D8
ANATOLIAN HIEROGLYPH A189

144D9
ANATOLIAN HIEROGLYPH A190

144DA
ANATOLIAN HIEROGLYPH A191

144DB
ANATOLIAN HIEROGLYPH A192

144DC
ANATOLIAN HIEROGLYPH A193

144DD
ANATOLIAN HIEROGLYPH A194

144DE
ANATOLIAN HIEROGLYPH A195

144DF
ANATOLIAN HIEROGLYPH A196

144E0
ANATOLIAN HIEROGLYPH A197

144E1
ANATOLIAN HIEROGLYPH A198

144E2
ANATOLIAN HIEROGLYPH A199

144E3
ANATOLIAN HIEROGLYPH A200

144E4
ANATOLIAN HIEROGLYPH A201

144E5
ANATOLIAN HIEROGLYPH A202

144E6
ANATOLIAN HIEROGLYPH A202A

144E7
ANATOLIAN HIEROGLYPH A202B

144E8
ANATOLIAN HIEROGLYPH A203

144E9
ANATOLIAN HIEROGLYPH A204

144EA
ANATOLIAN HIEROGLYPH A205

144EB
ANATOLIAN HIEROGLYPH A206

144EC
ANATOLIAN HIEROGLYPH A207

144ED
ANATOLIAN HIEROGLYPH A207A

144EE
ANATOLIAN HIEROGLYPH A208

144EF
ANATOLIAN HIEROGLYPH A209

144F0
ANATOLIAN HIEROGLYPH A209A

144F1
ANATOLIAN HIEROGLYPH A210

144F2
ANATOLIAN HIEROGLYPH A211

144F3
ANATOLIAN HIEROGLYPH A212

144F4
ANATOLIAN HIEROGLYPH A213

144F5
ANATOLIAN HIEROGLYPH A214

144F6
ANATOLIAN HIEROGLYPH A215

144F7
ANATOLIAN HIEROGLYPH A215A

144F8
ANATOLIAN HIEROGLYPH A216

144F9
ANATOLIAN HIEROGLYPH A216A

144FA
ANATOLIAN HIEROGLYPH A217

144FB
ANATOLIAN HIEROGLYPH A218

144FC
ANATOLIAN HIEROGLYPH A219

144FD
ANATOLIAN HIEROGLYPH A220

144FE
ANATOLIAN HIEROGLYPH A221

144FF
ANATOLIAN HIEROGLYPH A222

14500
ANATOLIAN HIEROGLYPH A223

14501
ANATOLIAN HIEROGLYPH A224

14502
ANATOLIAN HIEROGLYPH A225

14503
ANATOLIAN HIEROGLYPH A226

14504
ANATOLIAN HIEROGLYPH A227

14505
ANATOLIAN HIEROGLYPH A227A

14506
ANATOLIAN HIEROGLYPH A228

14507
ANATOLIAN HIEROGLYPH A229

14508
ANATOLIAN HIEROGLYPH A230

14509
ANATOLIAN HIEROGLYPH A231

1450A
ANATOLIAN HIEROGLYPH A232

1450B
ANATOLIAN HIEROGLYPH A233

1450C
ANATOLIAN HIEROGLYPH A234

1450D
ANATOLIAN HIEROGLYPH A235

1450E
ANATOLIAN HIEROGLYPH A236

1450F
ANATOLIAN HIEROGLYPH A237

14510
ANATOLIAN HIEROGLYPH A238

14511
ANATOLIAN HIEROGLYPH A239

14512
ANATOLIAN HIEROGLYPH A240

14513
ANATOLIAN HIEROGLYPH A241

14514
ANATOLIAN HIEROGLYPH A242

14515
ANATOLIAN HIEROGLYPH A243

14516
ANATOLIAN HIEROGLYPH A244

14517
ANATOLIAN HIEROGLYPH A245

14518
ANATOLIAN HIEROGLYPH A246

14519
ANATOLIAN HIEROGLYPH A247

1451A
ANATOLIAN HIEROGLYPH A248

1451B
ANATOLIAN HIEROGLYPH A249

1451C
ANATOLIAN HIEROGLYPH A250

1451D
ANATOLIAN HIEROGLYPH A251

1451E
ANATOLIAN HIEROGLYPH A252

1451F
ANATOLIAN HIEROGLYPH A253

14520
ANATOLIAN HIEROGLYPH A254

14521
ANATOLIAN HIEROGLYPH A255

14522
ANATOLIAN HIEROGLYPH A256

14523
ANATOLIAN HIEROGLYPH A257

14524
ANATOLIAN HIEROGLYPH A258

14525
ANATOLIAN HIEROGLYPH A259

14526
ANATOLIAN HIEROGLYPH A260

14527
ANATOLIAN HIEROGLYPH A261

14528
ANATOLIAN HIEROGLYPH A262

14529
ANATOLIAN HIEROGLYPH A263

1452A
ANATOLIAN HIEROGLYPH A264

1452B
ANATOLIAN HIEROGLYPH A265

1452C
ANATOLIAN HIEROGLYPH A266

1452D
ANATOLIAN HIEROGLYPH A267

1452E
ANATOLIAN HIEROGLYPH A267A

1452F
ANATOLIAN HIEROGLYPH A268

14530
ANATOLIAN HIEROGLYPH A269

14531
ANATOLIAN HIEROGLYPH A270

14532
ANATOLIAN HIEROGLYPH A271

14533
ANATOLIAN HIEROGLYPH A272

14534
ANATOLIAN HIEROGLYPH A273

14535
ANATOLIAN HIEROGLYPH A274

14536
ANATOLIAN HIEROGLYPH A275

14537
ANATOLIAN HIEROGLYPH A276

14538
ANATOLIAN HIEROGLYPH A277

14539
ANATOLIAN HIEROGLYPH A278

1453A
ANATOLIAN HIEROGLYPH A279

1453B
ANATOLIAN HIEROGLYPH A280

1453C
ANATOLIAN HIEROGLYPH A281

1453D
ANATOLIAN HIEROGLYPH A282

1453E
ANATOLIAN HIEROGLYPH A283

1453F
ANATOLIAN HIEROGLYPH A284

14540
ANATOLIAN HIEROGLYPH A285

14541
ANATOLIAN HIEROGLYPH A286

14542
ANATOLIAN HIEROGLYPH A287

14543
ANATOLIAN HIEROGLYPH A288

14544
ANATOLIAN HIEROGLYPH A289

14545
ANATOLIAN HIEROGLYPH A289A

14546
ANATOLIAN HIEROGLYPH A290

14547
ANATOLIAN HIEROGLYPH A291

14548
ANATOLIAN HIEROGLYPH A292

14549
ANATOLIAN HIEROGLYPH A293

1454A
ANATOLIAN HIEROGLYPH A294

1454B
ANATOLIAN HIEROGLYPH A294A

1454C
ANATOLIAN HIEROGLYPH A295

1454D
ANATOLIAN HIEROGLYPH A296

1454E
ANATOLIAN HIEROGLYPH A297

1454F
ANATOLIAN HIEROGLYPH A298

14550
ANATOLIAN HIEROGLYPH A299

14551
ANATOLIAN HIEROGLYPH A299A

14552
ANATOLIAN HIEROGLYPH A300

14553
ANATOLIAN HIEROGLYPH A301

14554
ANATOLIAN HIEROGLYPH A302

14555
ANATOLIAN HIEROGLYPH A303

14556
ANATOLIAN HIEROGLYPH A304

14557
ANATOLIAN HIEROGLYPH A305

14558
ANATOLIAN HIEROGLYPH A306

14559
ANATOLIAN HIEROGLYPH A307

1455A
ANATOLIAN HIEROGLYPH A308

1455B
ANATOLIAN HIEROGLYPH A309

1455C
ANATOLIAN HIEROGLYPH A309A

1455D
ANATOLIAN HIEROGLYPH A310

1455E
ANATOLIAN HIEROGLYPH A311

1455F
ANATOLIAN HIEROGLYPH A312

14560
ANATOLIAN HIEROGLYPH A313

14561
ANATOLIAN HIEROGLYPH A314

14562
ANATOLIAN HIEROGLYPH A315

14563
ANATOLIAN HIEROGLYPH A316

14564
ANATOLIAN HIEROGLYPH A317

14565
ANATOLIAN HIEROGLYPH A318

14566
ANATOLIAN HIEROGLYPH A319

14567
ANATOLIAN HIEROGLYPH A320

14568
ANATOLIAN HIEROGLYPH A321

14569
ANATOLIAN HIEROGLYPH A322

1456A
ANATOLIAN HIEROGLYPH A323

1456B
ANATOLIAN HIEROGLYPH A324

1456C
ANATOLIAN HIEROGLYPH A325

1456D
ANATOLIAN HIEROGLYPH A326

1456E
ANATOLIAN HIEROGLYPH A327

1456F
ANATOLIAN HIEROGLYPH A328

14570
ANATOLIAN HIEROGLYPH A329

14571
ANATOLIAN HIEROGLYPH A329A

14572
ANATOLIAN HIEROGLYPH A330

14573
ANATOLIAN HIEROGLYPH A331

14574
ANATOLIAN HIEROGLYPH A332A

14575
ANATOLIAN HIEROGLYPH A332B

14576
ANATOLIAN HIEROGLYPH A332C

14577
ANATOLIAN HIEROGLYPH A333

14578
ANATOLIAN HIEROGLYPH A334

14579
ANATOLIAN HIEROGLYPH A335

1457A
ANATOLIAN HIEROGLYPH A336

1457B
ANATOLIAN HIEROGLYPH A336A

1457C
ANATOLIAN HIEROGLYPH A336B

1457D
ANATOLIAN HIEROGLYPH A336C

1457E
ANATOLIAN HIEROGLYPH A337

1457F
ANATOLIAN HIEROGLYPH A338

14580
ANATOLIAN HIEROGLYPH A339

14581
ANATOLIAN HIEROGLYPH A340

14582
ANATOLIAN HIEROGLYPH A341

14583
ANATOLIAN HIEROGLYPH A342

14584
ANATOLIAN HIEROGLYPH A343

14585
ANATOLIAN HIEROGLYPH A344

14586
ANATOLIAN HIEROGLYPH A345

14587
ANATOLIAN HIEROGLYPH A346

14588
ANATOLIAN HIEROGLYPH A347

14589
ANATOLIAN HIEROGLYPH A348

1458A
ANATOLIAN HIEROGLYPH A349

1458B
ANATOLIAN HIEROGLYPH A350

1458C
ANATOLIAN HIEROGLYPH A351

1458D
ANATOLIAN HIEROGLYPH A352

1458E
ANATOLIAN HIEROGLYPH A353

1458F
ANATOLIAN HIEROGLYPH A354

14590
ANATOLIAN HIEROGLYPH A355

14591
ANATOLIAN HIEROGLYPH A356

14592
ANATOLIAN HIEROGLYPH A357

14593
ANATOLIAN HIEROGLYPH A358

14594
ANATOLIAN HIEROGLYPH A359

14595
ANATOLIAN HIEROGLYPH A359A

14596
ANATOLIAN HIEROGLYPH A360

14597
ANATOLIAN HIEROGLYPH A361

14598
ANATOLIAN HIEROGLYPH A362

14599
ANATOLIAN HIEROGLYPH A363

1459A
ANATOLIAN HIEROGLYPH A364

1459B
ANATOLIAN HIEROGLYPH A364A

1459C
ANATOLIAN HIEROGLYPH A365

1459D
ANATOLIAN HIEROGLYPH A366

1459E
ANATOLIAN HIEROGLYPH A367

1459F
ANATOLIAN HIEROGLYPH A368

145A0
ANATOLIAN HIEROGLYPH A368A

145A1
ANATOLIAN HIEROGLYPH A369

145A2
ANATOLIAN HIEROGLYPH A370

145A3
ANATOLIAN HIEROGLYPH A371

145A4
ANATOLIAN HIEROGLYPH A371A

145A5
ANATOLIAN HIEROGLYPH A372

145A6
ANATOLIAN HIEROGLYPH A373

145A7
ANATOLIAN HIEROGLYPH A374

145A8
ANATOLIAN HIEROGLYPH A375

145A9
ANATOLIAN HIEROGLYPH A376

145AA
ANATOLIAN HIEROGLYPH A377

145AB
ANATOLIAN HIEROGLYPH A378

145AC
ANATOLIAN HIEROGLYPH A379

145AD
ANATOLIAN HIEROGLYPH A380

145AE
ANATOLIAN HIEROGLYPH A381

145AF
ANATOLIAN HIEROGLYPH A381A

145B0
ANATOLIAN HIEROGLYPH A382

145B1
ANATOLIAN HIEROGLYPH A383 RA OR RI

145B2
ANATOLIAN HIEROGLYPH A383A

145B3
ANATOLIAN HIEROGLYPH A384

145B4
ANATOLIAN HIEROGLYPH A385

145B5
ANATOLIAN HIEROGLYPH A386

145B6
ANATOLIAN HIEROGLYPH A386A

145B7
ANATOLIAN HIEROGLYPH A387

145B8
ANATOLIAN HIEROGLYPH A388

145B9
ANATOLIAN HIEROGLYPH A389

145BA
ANATOLIAN HIEROGLYPH A390

145BB
ANATOLIAN HIEROGLYPH A391

145BC
ANATOLIAN HIEROGLYPH A392

145BD
ANATOLIAN HIEROGLYPH A393 EIGHT

145BE
ANATOLIAN HIEROGLYPH A394

145BF
ANATOLIAN HIEROGLYPH A395

145C0
ANATOLIAN HIEROGLYPH A396

145C1
ANATOLIAN HIEROGLYPH A397

145C2
ANATOLIAN HIEROGLYPH A398

145C3
ANATOLIAN HIEROGLYPH A399

145C4
ANATOLIAN HIEROGLYPH A400

145C5
ANATOLIAN HIEROGLYPH A401

145C6
ANATOLIAN HIEROGLYPH A402

145C7
ANATOLIAN HIEROGLYPH A403

145C8
ANATOLIAN HIEROGLYPH A404

145C9
ANATOLIAN HIEROGLYPH A405

145CA
ANATOLIAN HIEROGLYPH A406

145CB
ANATOLIAN HIEROGLYPH A407

145CC
ANATOLIAN HIEROGLYPH A408

145CD
ANATOLIAN HIEROGLYPH A409

145CE
ANATOLIAN HIEROGLYPH A410 BEGIN LOGOGRAM MARK

145CF
ANATOLIAN HIEROGLYPH A410A END LOGOGRAM MARK

145D0
ANATOLIAN HIEROGLYPH A411

145D1
ANATOLIAN HIEROGLYPH A412

145D2
ANATOLIAN HIEROGLYPH A413

145D3
ANATOLIAN HIEROGLYPH A414

145D4
ANATOLIAN HIEROGLYPH A415

145D5
ANATOLIAN HIEROGLYPH A416

145D6
ANATOLIAN HIEROGLYPH A417

145D7
ANATOLIAN HIEROGLYPH A418

145D8
ANATOLIAN HIEROGLYPH A419

145D9
ANATOLIAN HIEROGLYPH A420

145DA
ANATOLIAN HIEROGLYPH A421

145DB
ANATOLIAN HIEROGLYPH A422

145DC
ANATOLIAN HIEROGLYPH A423

145DD
ANATOLIAN HIEROGLYPH A424

145DE
ANATOLIAN HIEROGLYPH A425

145DF
ANATOLIAN HIEROGLYPH A426

145E0
ANATOLIAN HIEROGLYPH A427

145E1
ANATOLIAN HIEROGLYPH A428

145E2
ANATOLIAN HIEROGLYPH A429

145E3
ANATOLIAN HIEROGLYPH A430

145E4
ANATOLIAN HIEROGLYPH A431

145E5
ANATOLIAN HIEROGLYPH A432

145E6
ANATOLIAN HIEROGLYPH A433

145E7
ANATOLIAN HIEROGLYPH A434

145E8
ANATOLIAN HIEROGLYPH A435

145E9
ANATOLIAN HIEROGLYPH A436

145EA
ANATOLIAN HIEROGLYPH A437

145EB
ANATOLIAN HIEROGLYPH A438

145EC
ANATOLIAN HIEROGLYPH A439

145ED
ANATOLIAN HIEROGLYPH A440

145EE
ANATOLIAN HIEROGLYPH A441

145EF
ANATOLIAN HIEROGLYPH A442

145F0
ANATOLIAN HIEROGLYPH A443

145F1
ANATOLIAN HIEROGLYPH A444

145F2
ANATOLIAN HIEROGLYPH A445

145F3
ANATOLIAN HIEROGLYPH A446

145F4
ANATOLIAN HIEROGLYPH A447

145F5
ANATOLIAN HIEROGLYPH A448

145F6
ANATOLIAN HIEROGLYPH A449

145F7
ANATOLIAN HIEROGLYPH A450

145F8
ANATOLIAN HIEROGLYPH A450A

145F9
ANATOLIAN HIEROGLYPH A451

145FA
ANATOLIAN HIEROGLYPH A452

145FB
ANATOLIAN HIEROGLYPH A453

145FC
ANATOLIAN HIEROGLYPH A454

145FD
ANATOLIAN HIEROGLYPH A455

145FE
ANATOLIAN HIEROGLYPH A456

145FF
ANATOLIAN HIEROGLYPH A457

14600
ANATOLIAN HIEROGLYPH A457A

14601
ANATOLIAN HIEROGLYPH A458

14602
ANATOLIAN HIEROGLYPH A459

14603
ANATOLIAN HIEROGLYPH A460

14604
ANATOLIAN HIEROGLYPH A461

14605
ANATOLIAN HIEROGLYPH A462

14606
ANATOLIAN HIEROGLYPH A463

14607
ANATOLIAN HIEROGLYPH A464

14608
ANATOLIAN HIEROGLYPH A465

14609
ANATOLIAN HIEROGLYPH A466

1460A
ANATOLIAN HIEROGLYPH A467

1460B
ANATOLIAN HIEROGLYPH A468

1460C
ANATOLIAN HIEROGLYPH A469

1460D
ANATOLIAN HIEROGLYPH A470

1460E
ANATOLIAN HIEROGLYPH A471

1460F
ANATOLIAN HIEROGLYPH A472

14610
ANATOLIAN HIEROGLYPH A473

14611
ANATOLIAN HIEROGLYPH A474

14612
ANATOLIAN HIEROGLYPH A475

14613
ANATOLIAN HIEROGLYPH A476

14614
ANATOLIAN HIEROGLYPH A477

14615
ANATOLIAN HIEROGLYPH A478

14616
ANATOLIAN HIEROGLYPH A479

14617
ANATOLIAN HIEROGLYPH A480

14618
ANATOLIAN HIEROGLYPH A481

14619
ANATOLIAN HIEROGLYPH A482

1461A
ANATOLIAN HIEROGLYPH A483

1461B
ANATOLIAN HIEROGLYPH A484

1461C
ANATOLIAN HIEROGLYPH A485

1461D
ANATOLIAN HIEROGLYPH A486

1461E
ANATOLIAN HIEROGLYPH A487

1461F
ANATOLIAN HIEROGLYPH A488

14620
ANATOLIAN HIEROGLYPH A489

14621
ANATOLIAN HIEROGLYPH A490

14622
ANATOLIAN HIEROGLYPH A491

14623
ANATOLIAN HIEROGLYPH A492

14624
ANATOLIAN HIEROGLYPH A493

14625
ANATOLIAN HIEROGLYPH A494

14626
ANATOLIAN HIEROGLYPH A495

14627
ANATOLIAN HIEROGLYPH A496

14628
ANATOLIAN HIEROGLYPH A497

14629
ANATOLIAN HIEROGLYPH A501

1462A
ANATOLIAN HIEROGLYPH A502

1462B
ANATOLIAN HIEROGLYPH A503

1462C
ANATOLIAN HIEROGLYPH A504

1462D
ANATOLIAN HIEROGLYPH A505

1462E
ANATOLIAN HIEROGLYPH A506

1462F
ANATOLIAN HIEROGLYPH A507

14630
ANATOLIAN HIEROGLYPH A508

14631
ANATOLIAN HIEROGLYPH A509

14632
ANATOLIAN HIEROGLYPH A510

14633
ANATOLIAN HIEROGLYPH A511

14634
ANATOLIAN HIEROGLYPH A512

14635
ANATOLIAN HIEROGLYPH A513

14636
ANATOLIAN HIEROGLYPH A514

14637
ANATOLIAN HIEROGLYPH A515

14638
ANATOLIAN HIEROGLYPH A516

14639
ANATOLIAN HIEROGLYPH A517

1463A
ANATOLIAN HIEROGLYPH A518

1463B
ANATOLIAN HIEROGLYPH A519

1463C
ANATOLIAN HIEROGLYPH A520

1463D
ANATOLIAN HIEROGLYPH A521

1463E
ANATOLIAN HIEROGLYPH A522

1463F
ANATOLIAN HIEROGLYPH A523

14640
ANATOLIAN HIEROGLYPH A524

14641
ANATOLIAN HIEROGLYPH A525

14642
ANATOLIAN HIEROGLYPH A526

14643
ANATOLIAN HIEROGLYPH A527

14644
ANATOLIAN HIEROGLYPH A528

14645
ANATOLIAN HIEROGLYPH A529

14646
ANATOLIAN HIEROGLYPH A530

16100
GURUNG KHEMA LETTER A

16101
GURUNG KHEMA LETTER KA

16102
GURUNG KHEMA LETTER KHA

16103
GURUNG KHEMA LETTER GA

16104
GURUNG KHEMA LETTER GHA

16105
GURUNG KHEMA LETTER NGA

16106
GURUNG KHEMA LETTER CA

16107
GURUNG KHEMA LETTER CHA

16108
GURUNG KHEMA LETTER JA

16109
GURUNG KHEMA LETTER JHA

1610A
GURUNG KHEMA LETTER HA

1610B
GURUNG KHEMA LETTER TTA

1610C
GURUNG KHEMA LETTER TTHA

1610D
GURUNG KHEMA LETTER DDA

1610E
GURUNG KHEMA LETTER DDHA

1610F
GURUNG KHEMA LETTER VA

16110
GURUNG KHEMA LETTER TA

16111
GURUNG KHEMA LETTER THA

16112
GURUNG KHEMA LETTER DA

16113
GURUNG KHEMA LETTER DHA

16114
GURUNG KHEMA LETTER NA

16115
GURUNG KHEMA LETTER PA

16116
GURUNG KHEMA LETTER PHA

16117
GURUNG KHEMA LETTER BA

16118
GURUNG KHEMA LETTER BHA

16119
GURUNG KHEMA LETTER MA

1611A
GURUNG KHEMA LETTER YA

1611B
GURUNG KHEMA LETTER RA

1611C
GURUNG KHEMA LETTER LA

1611D
GURUNG KHEMA LETTER SA

1611E
GURUNG KHEMA VOWEL SIGN AA

1611F
GURUNG KHEMA VOWEL SIGN I

16120
GURUNG KHEMA VOWEL SIGN II

16121
GURUNG KHEMA VOWEL SIGN U

16122
GURUNG KHEMA VOWEL SIGN UU

16123
GURUNG KHEMA VOWEL SIGN E

16124
GURUNG KHEMA VOWEL SIGN EE

16125
GURUNG KHEMA VOWEL SIGN AI

16126
GURUNG KHEMA VOWEL SIGN O

16127
GURUNG KHEMA VOWEL SIGN OO

16128
GURUNG KHEMA VOWEL SIGN AU

16129
GURUNG KHEMA VOWEL LENGTH MARK

1612A
GURUNG KHEMA CONSONANT SIGN MEDIAL YA

1612B
GURUNG KHEMA CONSONANT SIGN MEDIAL VA

1612C
GURUNG KHEMA CONSONANT SIGN MEDIAL HA

1612D
GURUNG KHEMA SIGN ANUSVARA

1612E
GURUNG KHEMA CONSONANT SIGN MEDIAL RA

1612F
GURUNG KHEMA SIGN THOLHOMA

16130
GURUNG KHEMA DIGIT ZERO

16131
GURUNG KHEMA DIGIT ONE

16132
GURUNG KHEMA DIGIT TWO

16133
GURUNG KHEMA DIGIT THREE

16134
GURUNG KHEMA DIGIT FOUR

16135
GURUNG KHEMA DIGIT FIVE

16136
GURUNG KHEMA DIGIT SIX

16137
GURUNG KHEMA DIGIT SEVEN

16138
GURUNG KHEMA DIGIT EIGHT

16139
GURUNG KHEMA DIGIT NINE

16800
BAMUM LETTER PHASE-A NGKUE MFON

16801
BAMUM LETTER PHASE-A GBIEE FON

16802
BAMUM LETTER PHASE-A PON MFON PIPAEMGBIEE

16803
BAMUM LETTER PHASE-A PON MFON PIPAEMBA

16804
BAMUM LETTER PHASE-A NAA MFON

16805
BAMUM LETTER PHASE-A SHUENSHUET

16806
BAMUM LETTER PHASE-A TITA MFON

16807
BAMUM LETTER PHASE-A NZA MFON

16808
BAMUM LETTER PHASE-A SHINDA PA NJI

16809
BAMUM LETTER PHASE-A PON PA NJI PIPAEMGBIEE

1680A
BAMUM LETTER PHASE-A PON PA NJI PIPAEMBA

1680B
BAMUM LETTER PHASE-A MAEMGBIEE

1680B
BAMUM LETTER PHASE-A MAEMBGBIEE

1680C
BAMUM LETTER PHASE-A TU MAEMBA

1680D
BAMUM LETTER PHASE-A NGANGU

1680E
BAMUM LETTER PHASE-A MAEMVEUX

1680F
BAMUM LETTER PHASE-A MANSUAE

16810
BAMUM LETTER PHASE-A MVEUAENGAM

16811
BAMUM LETTER PHASE-A SEUNYAM

16812
BAMUM LETTER PHASE-A NTOQPEN

16813
BAMUM LETTER PHASE-A KEUKEUTNDA

16814
BAMUM LETTER PHASE-A NKINDI

16815
BAMUM LETTER PHASE-A SUU

16816
BAMUM LETTER PHASE-A NGKUENZEUM

16817
BAMUM LETTER PHASE-A LAPAQ

16818
BAMUM LETTER PHASE-A LET KUT

16819
BAMUM LETTER PHASE-A NTAP MFAA

1681A
BAMUM LETTER PHASE-A MAEKEUP

1681B
BAMUM LETTER PHASE-A PASHAE

1681C
BAMUM LETTER PHASE-A GHEUAERAE

1681D
BAMUM LETTER PHASE-A PAMSHAE

1681E
BAMUM LETTER PHASE-A MON NGGEUAET

1681F
BAMUM LETTER PHASE-A NZUN MEUT

16820
BAMUM LETTER PHASE-A U YUQ NAE

16821
BAMUM LETTER PHASE-A GHEUAEGHEUAE

16822
BAMUM LETTER PHASE-A NTAP NTAA

16823
BAMUM LETTER PHASE-A SISA

16824
BAMUM LETTER PHASE-A MGBASA

16825
BAMUM LETTER PHASE-A MEUNJOMNDEUQ

16826
BAMUM LETTER PHASE-A MOOMPUQ

16827
BAMUM LETTER PHASE-A KAFA

16828
BAMUM LETTER PHASE-A PA LEERAEWA

16829
BAMUM LETTER PHASE-A NDA LEERAEWA

1682A
BAMUM LETTER PHASE-A PET

1682B
BAMUM LETTER PHASE-A MAEMKPEN

1682C
BAMUM LETTER PHASE-A NIKA

1682D
BAMUM LETTER PHASE-A PUP

1682E
BAMUM LETTER PHASE-A TUAEP

1682F
BAMUM LETTER PHASE-A LUAEP

16830
BAMUM LETTER PHASE-A SONJAM

16831
BAMUM LETTER PHASE-A TEUTEUWEN

16832
BAMUM LETTER PHASE-A MAENYI

16833
BAMUM LETTER PHASE-A KET

16834
BAMUM LETTER PHASE-A NDAANGGEUAET

16835
BAMUM LETTER PHASE-A KUOQ

16836
BAMUM LETTER PHASE-A MOOMEUT

16837
BAMUM LETTER PHASE-A SHUM

16838
BAMUM LETTER PHASE-A LOMMAE

16839
BAMUM LETTER PHASE-A FIRI

1683A
BAMUM LETTER PHASE-A ROM

1683B
BAMUM LETTER PHASE-A KPOQ

1683C
BAMUM LETTER PHASE-A SOQ

1683D
BAMUM LETTER PHASE-A MAP PIEET

1683E
BAMUM LETTER PHASE-A SHIRAE

1683F
BAMUM LETTER PHASE-A NTAP

16840
BAMUM LETTER PHASE-A SHOQ NSHUT YUM

16841
BAMUM LETTER PHASE-A NYIT MONGKEUAEQ

16842
BAMUM LETTER PHASE-A PAARAE

16843
BAMUM LETTER PHASE-A NKAARAE

16844
BAMUM LETTER PHASE-A UNKNOWN

16845
BAMUM LETTER PHASE-A NGGEN

16846
BAMUM LETTER PHASE-A MAESI

16847
BAMUM LETTER PHASE-A NJAM

16848
BAMUM LETTER PHASE-A MBANYI

16849
BAMUM LETTER PHASE-A NYET

1684A
BAMUM LETTER PHASE-A TEUAEN

1684B
BAMUM LETTER PHASE-A SOT

1684C
BAMUM LETTER PHASE-A PAAM

1684D
BAMUM LETTER PHASE-A NSHIEE

1684E
BAMUM LETTER PHASE-A MAEM

1684F
BAMUM LETTER PHASE-A NYI

16850
BAMUM LETTER PHASE-A KAQ

16851
BAMUM LETTER PHASE-A NSHA

16852
BAMUM LETTER PHASE-A VEE

16853
BAMUM LETTER PHASE-A LU

16854
BAMUM LETTER PHASE-A NEN

16855
BAMUM LETTER PHASE-A NAQ

16856
BAMUM LETTER PHASE-A MBAQ

16857
BAMUM LETTER PHASE-B NSHUET

16858
BAMUM LETTER PHASE-B TU MAEMGBIEE

16859
BAMUM LETTER PHASE-B SIEE

1685A
BAMUM LETTER PHASE-B SET TU

1685B
BAMUM LETTER PHASE-B LOM NTEUM

1685C
BAMUM LETTER PHASE-B MBA MAELEE

1685D
BAMUM LETTER PHASE-B KIEEM

1685E
BAMUM LETTER PHASE-B YEURAE

1685F
BAMUM LETTER PHASE-B MBAARAE

16860
BAMUM LETTER PHASE-B KAM

16861
BAMUM LETTER PHASE-B PEESHI

16862
BAMUM LETTER PHASE-B YAFU LEERAEWA

16863
BAMUM LETTER PHASE-B LAM NSHUT NYAM

16864
BAMUM LETTER PHASE-B NTIEE SHEUOQ

16865
BAMUM LETTER PHASE-B NDU NJAA

16866
BAMUM LETTER PHASE-B GHEUGHEUAEM

16867
BAMUM LETTER PHASE-B PIT

16868
BAMUM LETTER PHASE-B TU NSIEE

16869
BAMUM LETTER PHASE-B SHET NJAQ

1686A
BAMUM LETTER PHASE-B SHEUAEQTU

1686B
BAMUM LETTER PHASE-B MFON TEUAEQ

1686C
BAMUM LETTER PHASE-B MBIT MBAAKET

1686D
BAMUM LETTER PHASE-B NYI NTEUM

1686E
BAMUM LETTER PHASE-B KEUPUQ

1686F
BAMUM LETTER PHASE-B GHEUGHEN

16870
BAMUM LETTER PHASE-B KEUYEUX

16871
BAMUM LETTER PHASE-B LAANAE

16872
BAMUM LETTER PHASE-B PARUM

16873
BAMUM LETTER PHASE-B VEUM

16874
BAMUM LETTER PHASE-B NGKINDI MVOP

16875
BAMUM LETTER PHASE-B NGGEU MBU

16876
BAMUM LETTER PHASE-B WUAET

16877
BAMUM LETTER PHASE-B SAKEUAE

16878
BAMUM LETTER PHASE-B TAAM

16879
BAMUM LETTER PHASE-B MEUQ

1687A
BAMUM LETTER PHASE-B NGGUOQ

1687B
BAMUM LETTER PHASE-B NGGUOQ LARGE

1687C
BAMUM LETTER PHASE-B MFIYAQ

1687D
BAMUM LETTER PHASE-B SUE

1687E
BAMUM LETTER PHASE-B MBEURI

1687F
BAMUM LETTER PHASE-B MONTIEEN

16880
BAMUM LETTER PHASE-B NYAEMAE

16881
BAMUM LETTER PHASE-B PUNGGAAM

16881
BAMUM LETTER PHASE-B PUNGAAM

16882
BAMUM LETTER PHASE-B MEUT NGGEET

16883
BAMUM LETTER PHASE-B FEUX

16884
BAMUM LETTER PHASE-B MBUOQ

16885
BAMUM LETTER PHASE-B FEE

16886
BAMUM LETTER PHASE-B KEUAEM

16887
BAMUM LETTER PHASE-B MA NJEUAENA

16888
BAMUM LETTER PHASE-B MA NJUQA

16889
BAMUM LETTER PHASE-B LET

1688A
BAMUM LETTER PHASE-B NGGAAM

1688B
BAMUM LETTER PHASE-B NSEN

1688C
BAMUM LETTER PHASE-B MA

1688D
BAMUM LETTER PHASE-B KIQ

1688E
BAMUM LETTER PHASE-B NGGOM

1688E
BAMUM LETTER PHASE-B NGOM

1688F
BAMUM LETTER PHASE-C NGKUE MAEMBA

16890
BAMUM LETTER PHASE-C NZA

16891
BAMUM LETTER PHASE-C YUM

16892
BAMUM LETTER PHASE-C WANGKUOQ

16893
BAMUM LETTER PHASE-C NGGEN

16894
BAMUM LETTER PHASE-C NDEUAEREE

16895
BAMUM LETTER PHASE-C NGKAQ

16896
BAMUM LETTER PHASE-C GHARAE

16897
BAMUM LETTER PHASE-C MBEEKEET

16898
BAMUM LETTER PHASE-C GBAYI

16899
BAMUM LETTER PHASE-C NYIR MKPARAQ MEUN

1689A
BAMUM LETTER PHASE-C NTU MBIT

1689B
BAMUM LETTER PHASE-C MBEUM

1689C
BAMUM LETTER PHASE-C PIRIEEN

1689D
BAMUM LETTER PHASE-C NDOMBU

1689E
BAMUM LETTER PHASE-C MBAA CABBAGE-TREE

1689F
BAMUM LETTER PHASE-C KEUSHEUAEP

168A0
BAMUM LETTER PHASE-C GHAP

168A1
BAMUM LETTER PHASE-C KEUKAQ

168A2
BAMUM LETTER PHASE-C YU MUOMAE

168A3
BAMUM LETTER PHASE-C NZEUM

168A4
BAMUM LETTER PHASE-C MBUE

168A5
BAMUM LETTER PHASE-C NSEUAEN

168A6
BAMUM LETTER PHASE-C MBIT

168A7
BAMUM LETTER PHASE-C YEUQ

168A8
BAMUM LETTER PHASE-C KPARAQ

168A9
BAMUM LETTER PHASE-C KAA

168AA
BAMUM LETTER PHASE-C SEUX

168AB
BAMUM LETTER PHASE-C NDIDA

168AC
BAMUM LETTER PHASE-C TAASHAE

168AD
BAMUM LETTER PHASE-C NJUEQ

168AE
BAMUM LETTER PHASE-C TITA YUE

168AF
BAMUM LETTER PHASE-C SUAET

168B0
BAMUM LETTER PHASE-C NGGUAEN NYAM

168B1
BAMUM LETTER PHASE-C VEUX

168B2
BAMUM LETTER PHASE-C NANSANAQ

168B3
BAMUM LETTER PHASE-C MA KEUAERI

168B4
BAMUM LETTER PHASE-C NTAA

168B5
BAMUM LETTER PHASE-C NGGUON

168B6
BAMUM LETTER PHASE-C LAP

168B7
BAMUM LETTER PHASE-C MBIRIEEN

168B8
BAMUM LETTER PHASE-C MGBASAQ

168B9
BAMUM LETTER PHASE-C NTEUNGBA

168BA
BAMUM LETTER PHASE-C TEUTEUX

168BB
BAMUM LETTER PHASE-C NGGUM

168BC
BAMUM LETTER PHASE-C FUE

168BD
BAMUM LETTER PHASE-C NDEUT

168BE
BAMUM LETTER PHASE-C NSA

168BF
BAMUM LETTER PHASE-C NSHAQ

168C0
BAMUM LETTER PHASE-C BUNG

168C1
BAMUM LETTER PHASE-C VEUAEPEN

168C2
BAMUM LETTER PHASE-C MBERAE

168C3
BAMUM LETTER PHASE-C RU

168C4
BAMUM LETTER PHASE-C NJAEM

168C5
BAMUM LETTER PHASE-C LAM

168C6
BAMUM LETTER PHASE-C TITUAEP

168C7
BAMUM LETTER PHASE-C NSUOT NGOM

168C8
BAMUM LETTER PHASE-C NJEEEE

168C9
BAMUM LETTER PHASE-C KET

168CA
BAMUM LETTER PHASE-C NGGU

168CB
BAMUM LETTER PHASE-C MAESI

168CC
BAMUM LETTER PHASE-C MBUAEM

168CD
BAMUM LETTER PHASE-C LU

168CE
BAMUM LETTER PHASE-C KUT

168CF
BAMUM LETTER PHASE-C NJAM

168D0
BAMUM LETTER PHASE-C NGOM

168D1
BAMUM LETTER PHASE-C WUP

168D2
BAMUM LETTER PHASE-C NGGUEET

168D3
BAMUM LETTER PHASE-C NSOM

168D4
BAMUM LETTER PHASE-C NTEN

168D5
BAMUM LETTER PHASE-C KUOP NKAARAE

168D6
BAMUM LETTER PHASE-C NSUN

168D7
BAMUM LETTER PHASE-C NDAM

168D8
BAMUM LETTER PHASE-C MA NSIEE

168D9
BAMUM LETTER PHASE-C YAA

168DA
BAMUM LETTER PHASE-C NDAP

168DB
BAMUM LETTER PHASE-C SHUEQ

168DC
BAMUM LETTER PHASE-C SHETFON

168DC
BAMUM LETTER PHASE-C SETFON

168DD
BAMUM LETTER PHASE-C MBI

168DE
BAMUM LETTER PHASE-C MAEMBA

168DF
BAMUM LETTER PHASE-C MBANYI

168E0
BAMUM LETTER PHASE-C KEUSEUX

168E1
BAMUM LETTER PHASE-C MBEUX

168E2
BAMUM LETTER PHASE-C KEUM

168E3
BAMUM LETTER PHASE-C MBAA PICKET

168E4
BAMUM LETTER PHASE-C YUWOQ

168E5
BAMUM LETTER PHASE-C NJEUX

168E6
BAMUM LETTER PHASE-C MIEE

168E7
BAMUM LETTER PHASE-C MUAE

168E8
BAMUM LETTER PHASE-C SHIQ

168E9
BAMUM LETTER PHASE-C KEN LAW

168EA
BAMUM LETTER PHASE-C KEN FATIGUE

168EB
BAMUM LETTER PHASE-C NGAQ

168EC
BAMUM LETTER PHASE-C NAQ

168ED
BAMUM LETTER PHASE-C LIQ

168EE
BAMUM LETTER PHASE-C PIN

168EF
BAMUM LETTER PHASE-C PEN

168F0
BAMUM LETTER PHASE-C TET

168F1
BAMUM LETTER PHASE-D MBUO

168F2
BAMUM LETTER PHASE-D WAP

168F3
BAMUM LETTER PHASE-D NJI

168F4
BAMUM LETTER PHASE-D MFON

168F5
BAMUM LETTER PHASE-D NJIEE

168F6
BAMUM LETTER PHASE-D LIEE

168F7
BAMUM LETTER PHASE-D NJEUT

168F8
BAMUM LETTER PHASE-D NSHEE

168F9
BAMUM LETTER PHASE-D NGGAAMAE

168FA
BAMUM LETTER PHASE-D NYAM

168FB
BAMUM LETTER PHASE-D WUAEN

168FC
BAMUM LETTER PHASE-D NGKUN

168FD
BAMUM LETTER PHASE-D SHEE

168FE
BAMUM LETTER PHASE-D NGKAP

168FF
BAMUM LETTER PHASE-D KEUAETMEUN

16900
BAMUM LETTER PHASE-D TEUT

16901
BAMUM LETTER PHASE-D SHEUAE

16902
BAMUM LETTER PHASE-D NJAP

16903
BAMUM LETTER PHASE-D SUE

16904
BAMUM LETTER PHASE-D KET

16905
BAMUM LETTER PHASE-D YAEMMAE

16906
BAMUM LETTER PHASE-D KUOM

16907
BAMUM LETTER PHASE-D SAP

16908
BAMUM LETTER PHASE-D MFEUT

16909
BAMUM LETTER PHASE-D NDEUX

1690A
BAMUM LETTER PHASE-D MALEERI

1690B
BAMUM LETTER PHASE-D MEUT

1690C
BAMUM LETTER PHASE-D SEUAEQ

1690D
BAMUM LETTER PHASE-D YEN

1690E
BAMUM LETTER PHASE-D NJEUAEM

1690F
BAMUM LETTER PHASE-D KEUOT MBUAE

16910
BAMUM LETTER PHASE-D NGKEURI

16911
BAMUM LETTER PHASE-D TU

16912
BAMUM LETTER PHASE-D GHAA

16913
BAMUM LETTER PHASE-D NGKYEE

16914
BAMUM LETTER PHASE-D FEUFEUAET

16915
BAMUM LETTER PHASE-D NDEE

16916
BAMUM LETTER PHASE-D MGBOFUM

16917
BAMUM LETTER PHASE-D LEUAEP

16918
BAMUM LETTER PHASE-D NDON

16919
BAMUM LETTER PHASE-D MONI

1691A
BAMUM LETTER PHASE-D MGBEUN

1691B
BAMUM LETTER PHASE-D PUUT

1691C
BAMUM LETTER PHASE-D MGBIEE

1691D
BAMUM LETTER PHASE-D MFO

1691E
BAMUM LETTER PHASE-D LUM

1691F
BAMUM LETTER PHASE-D NSIEEP

16920
BAMUM LETTER PHASE-D MBAA

16921
BAMUM LETTER PHASE-D KWAET

16922
BAMUM LETTER PHASE-D NYET

16923
BAMUM LETTER PHASE-D TEUAEN

16924
BAMUM LETTER PHASE-D SOT

16925
BAMUM LETTER PHASE-D YUWOQ

16926
BAMUM LETTER PHASE-D KEUM

16927
BAMUM LETTER PHASE-D RAEM

16928
BAMUM LETTER PHASE-D TEEEE

16929
BAMUM LETTER PHASE-D NGKEUAEQ

1692A
BAMUM LETTER PHASE-D MFEUAE

1692B
BAMUM LETTER PHASE-D NSIEET

1692C
BAMUM LETTER PHASE-D KEUP

1692D
BAMUM LETTER PHASE-D PIP

1692E
BAMUM LETTER PHASE-D PEUTAE

1692F
BAMUM LETTER PHASE-D NYUE

16930
BAMUM LETTER PHASE-D LET

16931
BAMUM LETTER PHASE-D NGGAAM

16932
BAMUM LETTER PHASE-D MFIEE

16933
BAMUM LETTER PHASE-D NGGWAEN

16934
BAMUM LETTER PHASE-D YUOM

16935
BAMUM LETTER PHASE-D PAP

16936
BAMUM LETTER PHASE-D YUOP

16937
BAMUM LETTER PHASE-D NDAM

16938
BAMUM LETTER PHASE-D NTEUM

16939
BAMUM LETTER PHASE-D SUAE

1693A
BAMUM LETTER PHASE-D KUN

1693B
BAMUM LETTER PHASE-D NGGEUX

1693C
BAMUM LETTER PHASE-D NGKIEE

1693D
BAMUM LETTER PHASE-D TUOT

1693E
BAMUM LETTER PHASE-D MEUN

1693F
BAMUM LETTER PHASE-D KUQ

16940
BAMUM LETTER PHASE-D NSUM

16941
BAMUM LETTER PHASE-D TEUN

16942
BAMUM LETTER PHASE-D MAENJET

16943
BAMUM LETTER PHASE-D NGGAP

16944
BAMUM LETTER PHASE-D LEUM

16945
BAMUM LETTER PHASE-D NGGUOM

16946
BAMUM LETTER PHASE-D NSHUT

16947
BAMUM LETTER PHASE-D NJUEQ

16948
BAMUM LETTER PHASE-D GHEUAE

16949
BAMUM LETTER PHASE-D KU

1694A
BAMUM LETTER PHASE-D REN OLD

1694B
BAMUM LETTER PHASE-D TAE

1694C
BAMUM LETTER PHASE-D TOQ

1694D
BAMUM LETTER PHASE-D NYI

1694E
BAMUM LETTER PHASE-D RII

1694F
BAMUM LETTER PHASE-D LEEEE

16950
BAMUM LETTER PHASE-D MEEEE

16951
BAMUM LETTER PHASE-D M

16952
BAMUM LETTER PHASE-D SUU

16953
BAMUM LETTER PHASE-D MU

16954
BAMUM LETTER PHASE-D SHII

16955
BAMUM LETTER PHASE-D SHEUX

16956
BAMUM LETTER PHASE-D KYEE

16957
BAMUM LETTER PHASE-D NU

16958
BAMUM LETTER PHASE-D SHU

16959
BAMUM LETTER PHASE-D NTEE

1695A
BAMUM LETTER PHASE-D PEE

1695B
BAMUM LETTER PHASE-D NI

1695C
BAMUM LETTER PHASE-D SHOQ

1695D
BAMUM LETTER PHASE-D PUQ

1695E
BAMUM LETTER PHASE-D MVOP

1695F
BAMUM LETTER PHASE-D LOQ

16960
BAMUM LETTER PHASE-D REN MUCH

16961
BAMUM LETTER PHASE-D TI

16962
BAMUM LETTER PHASE-D NTUU

16963
BAMUM LETTER PHASE-D MBAA SEVEN

16964
BAMUM LETTER PHASE-D SAQ

16965
BAMUM LETTER PHASE-D FAA

16966
BAMUM LETTER PHASE-E NDAP

16967
BAMUM LETTER PHASE-E TOON

16968
BAMUM LETTER PHASE-E MBEUM

16969
BAMUM LETTER PHASE-E LAP

1696A
BAMUM LETTER PHASE-E VOM

1696B
BAMUM LETTER PHASE-E LOON

1696C
BAMUM LETTER PHASE-E PAA

1696D
BAMUM LETTER PHASE-E SOM

1696E
BAMUM LETTER PHASE-E RAQ

1696F
BAMUM LETTER PHASE-E NSHUOP

16970
BAMUM LETTER PHASE-E NDUN

16971
BAMUM LETTER PHASE-E PUAE

16972
BAMUM LETTER PHASE-E TAM

16973
BAMUM LETTER PHASE-E NGKA

16974
BAMUM LETTER PHASE-E KPEUX

16975
BAMUM LETTER PHASE-E WUO

16976
BAMUM LETTER PHASE-E SEE

16977
BAMUM LETTER PHASE-E NGGEUAET

16978
BAMUM LETTER PHASE-E PAAM

16979
BAMUM LETTER PHASE-E TOO

1697A
BAMUM LETTER PHASE-E KUOP

1697B
BAMUM LETTER PHASE-E LOM

1697C
BAMUM LETTER PHASE-E NSHIEE

1697D
BAMUM LETTER PHASE-E NGGOP

1697D
BAMUM LETTER PHASE-E NGOP

1697E
BAMUM LETTER PHASE-E MAEM

1697F
BAMUM LETTER PHASE-E NGKEUX

16980
BAMUM LETTER PHASE-E NGOQ

16981
BAMUM LETTER PHASE-E NSHUE

16982
BAMUM LETTER PHASE-E RIMGBA

16983
BAMUM LETTER PHASE-E NJEUX

16984
BAMUM LETTER PHASE-E PEEM

16985
BAMUM LETTER PHASE-E SAA

16986
BAMUM LETTER PHASE-E NGGURAE

16987
BAMUM LETTER PHASE-E MGBA

16988
BAMUM LETTER PHASE-E GHEUX

16989
BAMUM LETTER PHASE-E NGKEUAEM

1698A
BAMUM LETTER PHASE-E NJAEMLI

1698B
BAMUM LETTER PHASE-E MAP

1698C
BAMUM LETTER PHASE-E LOOT

1698D
BAMUM LETTER PHASE-E NGGEEEE

1698E
BAMUM LETTER PHASE-E NDIQ

1698F
BAMUM LETTER PHASE-E TAEN NTEUM

16990
BAMUM LETTER PHASE-E SET

16991
BAMUM LETTER PHASE-E PUM

16992
BAMUM LETTER PHASE-E NDAA SOFTNESS

16993
BAMUM LETTER PHASE-E NGGUAESHAE NYAM

16994
BAMUM LETTER PHASE-E YIEE

16995
BAMUM LETTER PHASE-E GHEUN

16996
BAMUM LETTER PHASE-E TUAE

16997
BAMUM LETTER PHASE-E YEUAE

16998
BAMUM LETTER PHASE-E PO

16999
BAMUM LETTER PHASE-E TUMAE

1699A
BAMUM LETTER PHASE-E KEUAE

1699B
BAMUM LETTER PHASE-E SUAEN

1699C
BAMUM LETTER PHASE-E TEUAEQ

1699D
BAMUM LETTER PHASE-E VEUAE

1699E
BAMUM LETTER PHASE-E WEUX

1699F
BAMUM LETTER PHASE-E LAAM

169A0
BAMUM LETTER PHASE-E PU

169A1
BAMUM LETTER PHASE-E TAAQ

169A2
BAMUM LETTER PHASE-E GHAAMAE

169A3
BAMUM LETTER PHASE-E NGEUREUT

169A4
BAMUM LETTER PHASE-E SHEUAEQ

169A5
BAMUM LETTER PHASE-E MGBEN

169A6
BAMUM LETTER PHASE-E MBEE

169A7
BAMUM LETTER PHASE-E NZAQ

169A8
BAMUM LETTER PHASE-E NKOM

169A9
BAMUM LETTER PHASE-E GBET

169AA
BAMUM LETTER PHASE-E TUM

169AB
BAMUM LETTER PHASE-E KUET

169AC
BAMUM LETTER PHASE-E YAP

169AD
BAMUM LETTER PHASE-E NYI CLEAVER

169AE
BAMUM LETTER PHASE-E YIT

169AF
BAMUM LETTER PHASE-E MFEUQ

169B0
BAMUM LETTER PHASE-E NDIAQ

169B1
BAMUM LETTER PHASE-E PIEEQ

169B2
BAMUM LETTER PHASE-E YUEQ

169B3
BAMUM LETTER PHASE-E LEUAEM

169B4
BAMUM LETTER PHASE-E FUE

169B5
BAMUM LETTER PHASE-E GBEUX

169B6
BAMUM LETTER PHASE-E NGKUP

169B7
BAMUM LETTER PHASE-E KET

169B8
BAMUM LETTER PHASE-E MAE

169B9
BAMUM LETTER PHASE-E NGKAAMI

169BA
BAMUM LETTER PHASE-E GHET

169BB
BAMUM LETTER PHASE-E FA

169BC
BAMUM LETTER PHASE-E NTUM

169BD
BAMUM LETTER PHASE-E PEUT

169BE
BAMUM LETTER PHASE-E YEUM

169BF
BAMUM LETTER PHASE-E NGGEUAE

169C0
BAMUM LETTER PHASE-E NYI BETWEEN

169C1
BAMUM LETTER PHASE-E NZUQ

169C2
BAMUM LETTER PHASE-E POON

169C3
BAMUM LETTER PHASE-E MIEE

169C4
BAMUM LETTER PHASE-E FUET

169C5
BAMUM LETTER PHASE-E NAE

169C6
BAMUM LETTER PHASE-E MUAE

169C7
BAMUM LETTER PHASE-E GHEUAE

169C8
BAMUM LETTER PHASE-E FU I

169C9
BAMUM LETTER PHASE-E MVI

169CA
BAMUM LETTER PHASE-E PUAQ

169CB
BAMUM LETTER PHASE-E NGKUM

169CC
BAMUM LETTER PHASE-E KUT

169CD
BAMUM LETTER PHASE-E PIET

169CE
BAMUM LETTER PHASE-E NTAP

169CF
BAMUM LETTER PHASE-E YEUAET

169D0
BAMUM LETTER PHASE-E NGGUP

169D1
BAMUM LETTER PHASE-E PA PEOPLE

169D2
BAMUM LETTER PHASE-E FU CALL

169D3
BAMUM LETTER PHASE-E FOM

169D4
BAMUM LETTER PHASE-E NJEE

169D5
BAMUM LETTER PHASE-E A

169D6
BAMUM LETTER PHASE-E TOQ

169D7
BAMUM LETTER PHASE-E O

169D8
BAMUM LETTER PHASE-E I

169D9
BAMUM LETTER PHASE-E LAQ

169DA
BAMUM LETTER PHASE-E PA PLURAL

169DB
BAMUM LETTER PHASE-E TAA

169DC
BAMUM LETTER PHASE-E TAQ

169DD
BAMUM LETTER PHASE-E NDAA MY HOUSE

169DE
BAMUM LETTER PHASE-E SHIQ

169DF
BAMUM LETTER PHASE-E YEUX

169E0
BAMUM LETTER PHASE-E NGUAE

169E1
BAMUM LETTER PHASE-E YUAEN

169E2
BAMUM LETTER PHASE-E YOQ SWIMMING

169E3
BAMUM LETTER PHASE-E YOQ COVER

169E4
BAMUM LETTER PHASE-E YUQ

169E5
BAMUM LETTER PHASE-E YUN

169E6
BAMUM LETTER PHASE-E KEUX

169E7
BAMUM LETTER PHASE-E PEUX

169E8
BAMUM LETTER PHASE-E NJEE EPOCH

169E9
BAMUM LETTER PHASE-E PUE

169EA
BAMUM LETTER PHASE-E WUE

169EB
BAMUM LETTER PHASE-E FEE

169EC
BAMUM LETTER PHASE-E VEE

169ED
BAMUM LETTER PHASE-E LU

169EE
BAMUM LETTER PHASE-E MI

169EF
BAMUM LETTER PHASE-E REUX

169F0
BAMUM LETTER PHASE-E RAE

169F1
BAMUM LETTER PHASE-E NGUAET

169F2
BAMUM LETTER PHASE-E NGA

169F3
BAMUM LETTER PHASE-E SHO

169F4
BAMUM LETTER PHASE-E SHOQ

169F5
BAMUM LETTER PHASE-E FU REMEDY

169F6
BAMUM LETTER PHASE-E NA

169F7
BAMUM LETTER PHASE-E PI

169F8
BAMUM LETTER PHASE-E LOQ

169F9
BAMUM LETTER PHASE-E KO

169FA
BAMUM LETTER PHASE-E MEN

169FB
BAMUM LETTER PHASE-E MA

169FC
BAMUM LETTER PHASE-E MAQ

169FD
BAMUM LETTER PHASE-E TEU

169FE
BAMUM LETTER PHASE-E KI

169FF
BAMUM LETTER PHASE-E MON

16A00
BAMUM LETTER PHASE-E TEN

16A01
BAMUM LETTER PHASE-E FAQ

16A02
BAMUM LETTER PHASE-E GHOM

16A03
BAMUM LETTER PHASE-F KA

16A04
BAMUM LETTER PHASE-F U

16A05
BAMUM LETTER PHASE-F KU

16A06
BAMUM LETTER PHASE-F EE

16A07
BAMUM LETTER PHASE-F REE

16A08
BAMUM LETTER PHASE-F TAE

16A09
BAMUM LETTER PHASE-F NYI

16A0A
BAMUM LETTER PHASE-F LA

16A0B
BAMUM LETTER PHASE-F RII

16A0C
BAMUM LETTER PHASE-F RIEE

16A0D
BAMUM LETTER PHASE-F MEEEE

16A0E
BAMUM LETTER PHASE-F TAA

16A0F
BAMUM LETTER PHASE-F NDAA

16A10
BAMUM LETTER PHASE-F NJAEM

16A11
BAMUM LETTER PHASE-F M

16A12
BAMUM LETTER PHASE-F SUU

16A13
BAMUM LETTER PHASE-F SHII

16A14
BAMUM LETTER PHASE-F SI

16A15
BAMUM LETTER PHASE-F SEUX

16A16
BAMUM LETTER PHASE-F KYEE

16A17
BAMUM LETTER PHASE-F KET

16A18
BAMUM LETTER PHASE-F NUAE

16A19
BAMUM LETTER PHASE-F NU

16A1A
BAMUM LETTER PHASE-F NJUAE

16A1B
BAMUM LETTER PHASE-F YOQ

16A1C
BAMUM LETTER PHASE-F SHU

16A1D
BAMUM LETTER PHASE-F YA

16A1E
BAMUM LETTER PHASE-F NSHA

16A1F
BAMUM LETTER PHASE-F PEUX

16A20
BAMUM LETTER PHASE-F NTEE

16A21
BAMUM LETTER PHASE-F WUE

16A22
BAMUM LETTER PHASE-F PEE

16A23
BAMUM LETTER PHASE-F RU

16A24
BAMUM LETTER PHASE-F NI

16A25
BAMUM LETTER PHASE-F REUX

16A26
BAMUM LETTER PHASE-F KEN

16A27
BAMUM LETTER PHASE-F NGKWAEN

16A28
BAMUM LETTER PHASE-F NGGA

16A29
BAMUM LETTER PHASE-F SHO

16A2A
BAMUM LETTER PHASE-F PUAE

16A2B
BAMUM LETTER PHASE-F FOM

16A2C
BAMUM LETTER PHASE-F WA

16A2D
BAMUM LETTER PHASE-F LI

16A2E
BAMUM LETTER PHASE-F LOQ

16A2F
BAMUM LETTER PHASE-F KO

16A30
BAMUM LETTER PHASE-F MBEN

16A31
BAMUM LETTER PHASE-F REN

16A32
BAMUM LETTER PHASE-F MA

16A33
BAMUM LETTER PHASE-F MO

16A34
BAMUM LETTER PHASE-F MBAA

16A35
BAMUM LETTER PHASE-F TET

16A36
BAMUM LETTER PHASE-F KPA

16A37
BAMUM LETTER PHASE-F SAMBA

16A38
BAMUM LETTER PHASE-F VUEQ

16A40
MRO LETTER TA

16A41
MRO LETTER NGI

16A42
MRO LETTER YO

16A43
MRO LETTER MIM

16A44
MRO LETTER BA

16A45
MRO LETTER DA

16A46
MRO LETTER A

16A47
MRO LETTER PHI

16A48
MRO LETTER KHAI

16A49
MRO LETTER HAO

16A4A
MRO LETTER DAI

16A4B
MRO LETTER CHU

16A4C
MRO LETTER KEAAE

16A4D
MRO LETTER OL

16A4E
MRO LETTER MAEM

16A4F
MRO LETTER NIN

16A50
MRO LETTER PA

16A51
MRO LETTER OO

16A52
MRO LETTER O

16A53
MRO LETTER RO

16A54
MRO LETTER SHI

16A55
MRO LETTER THEA

16A56
MRO LETTER EA

16A57
MRO LETTER WA

16A58
MRO LETTER E

16A59
MRO LETTER KO

16A5A
MRO LETTER LAN

16A5B
MRO LETTER LA

16A5C
MRO LETTER HAI

16A5D
MRO LETTER RI

16A5E
MRO LETTER TEK

16A60
MRO DIGIT ZERO

16A61
MRO DIGIT ONE

16A62
MRO DIGIT TWO

16A63
MRO DIGIT THREE

16A64
MRO DIGIT FOUR

16A65
MRO DIGIT FIVE

16A66
MRO DIGIT SIX

16A67
MRO DIGIT SEVEN

16A68
MRO DIGIT EIGHT

16A69
MRO DIGIT NINE

16A6E
MRO DANDA

16A6F
MRO DOUBLE DANDA

16A70
TANGSA LETTER OZ

16A71
TANGSA LETTER OC

16A72
TANGSA LETTER OQ

16A73
TANGSA LETTER OX

16A74
TANGSA LETTER AZ

16A75
TANGSA LETTER AC

16A76
TANGSA LETTER AQ

16A77
TANGSA LETTER AX

16A78
TANGSA LETTER VZ

16A79
TANGSA LETTER VC

16A7A
TANGSA LETTER VQ

16A7B
TANGSA LETTER VX

16A7C
TANGSA LETTER EZ

16A7D
TANGSA LETTER EC

16A7E
TANGSA LETTER EQ

16A7F
TANGSA LETTER EX

16A80
TANGSA LETTER IZ

16A81
TANGSA LETTER IC

16A82
TANGSA LETTER IQ

16A83
TANGSA LETTER IX

16A84
TANGSA LETTER UZ

16A85
TANGSA LETTER UC

16A86
TANGSA LETTER UQ

16A87
TANGSA LETTER UX

16A88
TANGSA LETTER AWZ

16A89
TANGSA LETTER AWC

16A8A
TANGSA LETTER AWQ

16A8B
TANGSA LETTER AWX

16A8C
TANGSA LETTER UIZ

16A8D
TANGSA LETTER UIC

16A8E
TANGSA LETTER UIQ

16A8F
TANGSA LETTER UIX

16A90
TANGSA LETTER FINAL NG

16A91
TANGSA LETTER LONG UEX

16A92
TANGSA LETTER SHORT UEZ

16A93
TANGSA LETTER SHORT AWX

16A94
TANGSA LETTER UEC

16A95
TANGSA LETTER UEZ

16A96
TANGSA LETTER UEQ

16A97
TANGSA LETTER UEX

16A98
TANGSA LETTER UIUZ

16A99
TANGSA LETTER UIUC

16A9A
TANGSA LETTER UIUQ

16A9B
TANGSA LETTER UIUX

16A9C
TANGSA LETTER MZ

16A9D
TANGSA LETTER MC

16A9E
TANGSA LETTER MQ

16A9F
TANGSA LETTER MX

16AA0
TANGSA LETTER KA

16AA1
TANGSA LETTER KHA

16AA2
TANGSA LETTER GA

16AA3
TANGSA LETTER NGA

16AA4
TANGSA LETTER SA

16AA5
TANGSA LETTER YA

16AA6
TANGSA LETTER WA

16AA7
TANGSA LETTER PA

16AA8
TANGSA LETTER NYA

16AA9
TANGSA LETTER PHA

16AAA
TANGSA LETTER BA

16AAB
TANGSA LETTER MA

16AAC
TANGSA LETTER NA

16AAD
TANGSA LETTER HA

16AAE
TANGSA LETTER LA

16AAF
TANGSA LETTER HTA

16AB0
TANGSA LETTER TA

16AB1
TANGSA LETTER DA

16AB2
TANGSA LETTER RA

16AB3
TANGSA LETTER NHA

16AB4
TANGSA LETTER SHA

16AB5
TANGSA LETTER CA

16AB6
TANGSA LETTER TSA

16AB7
TANGSA LETTER GHA

16AB8
TANGSA LETTER HTTA

16AB9
TANGSA LETTER THA

16ABA
TANGSA LETTER XA

16ABB
TANGSA LETTER FA

16ABC
TANGSA LETTER DHA

16ABD
TANGSA LETTER CHA

16ABE
TANGSA LETTER ZA

16AC0
TANGSA DIGIT ZERO

16AC1
TANGSA DIGIT ONE

16AC2
TANGSA DIGIT TWO

16AC3
TANGSA DIGIT THREE

16AC4
TANGSA DIGIT FOUR

16AC5
TANGSA DIGIT FIVE

16AC6
TANGSA DIGIT SIX

16AC7
TANGSA DIGIT SEVEN

16AC8
TANGSA DIGIT EIGHT

16AC9
TANGSA DIGIT NINE

16AD0
BASSA VAH LETTER ENNI

16AD1
BASSA VAH LETTER KA

16AD2
BASSA VAH LETTER SE

16AD3
BASSA VAH LETTER FA

16AD4
BASSA VAH LETTER MBE

16AD5
BASSA VAH LETTER YIE

16AD6
BASSA VAH LETTER GAH

16AD7
BASSA VAH LETTER DHII

16AD8
BASSA VAH LETTER KPAH

16AD9
BASSA VAH LETTER JO

16ADA
BASSA VAH LETTER HWAH

16ADB
BASSA VAH LETTER WA

16ADC
BASSA VAH LETTER ZO

16ADD
BASSA VAH LETTER GBU

16ADE
BASSA VAH LETTER DO

16ADF
BASSA VAH LETTER CE

16AE0
BASSA VAH LETTER UWU

16AE1
BASSA VAH LETTER TO

16AE2
BASSA VAH LETTER BA

16AE3
BASSA VAH LETTER VU

16AE4
BASSA VAH LETTER YEIN

16AE5
BASSA VAH LETTER PA

16AE6
BASSA VAH LETTER WADDA

16AE7
BASSA VAH LETTER A

16AE8
BASSA VAH LETTER O

16AE9
BASSA VAH LETTER OO

16AEA
BASSA VAH LETTER U

16AEB
BASSA VAH LETTER EE

16AEC
BASSA VAH LETTER E

16AED
BASSA VAH LETTER I

16AF0
BASSA VAH COMBINING HIGH TONE

16AF1
BASSA VAH COMBINING LOW TONE

16AF2
BASSA VAH COMBINING MID TONE

16AF3
BASSA VAH COMBINING LOW-MID TONE

16AF4
BASSA VAH COMBINING HIGH-LOW TONE

16AF5
BASSA VAH FULL STOP

16B00
PAHAWH HMONG VOWEL KEEB

16B01
PAHAWH HMONG VOWEL KEEV

16B02
PAHAWH HMONG VOWEL KIB

16B03
PAHAWH HMONG VOWEL KIV

16B04
PAHAWH HMONG VOWEL KAUB

16B05
PAHAWH HMONG VOWEL KAUV

16B06
PAHAWH HMONG VOWEL KUB

16B07
PAHAWH HMONG VOWEL KUV

16B08
PAHAWH HMONG VOWEL KEB

16B09
PAHAWH HMONG VOWEL KEV

16B0A
PAHAWH HMONG VOWEL KAIB

16B0B
PAHAWH HMONG VOWEL KAIV

16B0C
PAHAWH HMONG VOWEL KOOB

16B0D
PAHAWH HMONG VOWEL KOOV

16B0E
PAHAWH HMONG VOWEL KAWB

16B0F
PAHAWH HMONG VOWEL KAWV

16B10
PAHAWH HMONG VOWEL KUAB

16B11
PAHAWH HMONG VOWEL KUAV

16B12
PAHAWH HMONG VOWEL KOB

16B13
PAHAWH HMONG VOWEL KOV

16B14
PAHAWH HMONG VOWEL KIAB

16B15
PAHAWH HMONG VOWEL KIAV

16B16
PAHAWH HMONG VOWEL KAB

16B17
PAHAWH HMONG VOWEL KAV

16B18
PAHAWH HMONG VOWEL KWB

16B19
PAHAWH HMONG VOWEL KWV

16B1A
PAHAWH HMONG VOWEL KAAB

16B1B
PAHAWH HMONG VOWEL KAAV

16B1C
PAHAWH HMONG CONSONANT VAU

16B1D
PAHAWH HMONG CONSONANT NTSAU

16B1E
PAHAWH HMONG CONSONANT LAU

16B1F
PAHAWH HMONG CONSONANT HAU

16B20
PAHAWH HMONG CONSONANT NLAU

16B21
PAHAWH HMONG CONSONANT RAU

16B22
PAHAWH HMONG CONSONANT NKAU

16B23
PAHAWH HMONG CONSONANT QHAU

16B24
PAHAWH HMONG CONSONANT YAU

16B25
PAHAWH HMONG CONSONANT HLAU

16B26
PAHAWH HMONG CONSONANT MAU

16B27
PAHAWH HMONG CONSONANT CHAU

16B28
PAHAWH HMONG CONSONANT NCHAU

16B29
PAHAWH HMONG CONSONANT HNAU

16B2A
PAHAWH HMONG CONSONANT PLHAU

16B2B
PAHAWH HMONG CONSONANT NTHAU

16B2C
PAHAWH HMONG CONSONANT NAU

16B2D
PAHAWH HMONG CONSONANT AU

16B2E
PAHAWH HMONG CONSONANT XAU

16B2F
PAHAWH HMONG CONSONANT CAU

16B30
PAHAWH HMONG MARK CIM TUB

16B31
PAHAWH HMONG MARK CIM SO

16B32
PAHAWH HMONG MARK CIM KES

16B33
PAHAWH HMONG MARK CIM KHAV

16B34
PAHAWH HMONG MARK CIM SUAM

16B35
PAHAWH HMONG MARK CIM HOM

16B36
PAHAWH HMONG MARK CIM TAUM

16B37
PAHAWH HMONG SIGN VOS THOM

16B38
PAHAWH HMONG SIGN VOS TSHAB CEEB

16B39
PAHAWH HMONG SIGN CIM CHEEM

16B3A
PAHAWH HMONG SIGN VOS THIAB

16B3B
PAHAWH HMONG SIGN VOS FEEM

16B3C
PAHAWH HMONG SIGN XYEEM NTXIV

16B3D
PAHAWH HMONG SIGN XYEEM RHO

16B3E
PAHAWH HMONG SIGN XYEEM TOV

16B3F
PAHAWH HMONG SIGN XYEEM FAIB

16B40
PAHAWH HMONG SIGN VOS SEEV

16B41
PAHAWH HMONG SIGN MEEJ SUAB

16B42
PAHAWH HMONG SIGN VOS NRUA

16B43
PAHAWH HMONG SIGN IB YAM

16B44
PAHAWH HMONG SIGN XAUS

16B45
PAHAWH HMONG SIGN CIM TSOV ROG

16B50
PAHAWH HMONG DIGIT ZERO

16B51
PAHAWH HMONG DIGIT ONE

16B52
PAHAWH HMONG DIGIT TWO

16B53
PAHAWH HMONG DIGIT THREE

16B54
PAHAWH HMONG DIGIT FOUR

16B55
PAHAWH HMONG DIGIT FIVE

16B56
PAHAWH HMONG DIGIT SIX

16B57
PAHAWH HMONG DIGIT SEVEN

16B58
PAHAWH HMONG DIGIT EIGHT

16B59
PAHAWH HMONG DIGIT NINE

16B5B
PAHAWH HMONG NUMBER TENS

16B5C
PAHAWH HMONG NUMBER HUNDREDS

16B5D
PAHAWH HMONG NUMBER TEN THOUSANDS

16B5E
PAHAWH HMONG NUMBER MILLIONS

16B5F
PAHAWH HMONG NUMBER HUNDRED MILLIONS

16B60
PAHAWH HMONG NUMBER TEN BILLIONS

16B61
PAHAWH HMONG NUMBER TRILLIONS

16B63
PAHAWH HMONG SIGN VOS LUB

16B64
PAHAWH HMONG SIGN XYOO

16B65
PAHAWH HMONG SIGN HLI

16B66
PAHAWH HMONG SIGN THIRD-STAGE HLI

16B67
PAHAWH HMONG SIGN ZWJ THAJ

16B68
PAHAWH HMONG SIGN HNUB

16B69
PAHAWH HMONG SIGN NQIG

16B6A
PAHAWH HMONG SIGN XIAB

16B6B
PAHAWH HMONG SIGN NTUJ

16B6C
PAHAWH HMONG SIGN AV

16B6D
PAHAWH HMONG SIGN TXHEEJ CEEV

16B6E
PAHAWH HMONG SIGN MEEJ TSEEB

16B6F
PAHAWH HMONG SIGN TAU

16B70
PAHAWH HMONG SIGN LOS

16B71
PAHAWH HMONG SIGN MUS

16B72
PAHAWH HMONG SIGN CIM HAIS LUS NTOG NTOG

16B73
PAHAWH HMONG SIGN CIM CUAM TSHOOJ

16B74
PAHAWH HMONG SIGN CIM TXWV

16B75
PAHAWH HMONG SIGN CIM TXWV CHWV

16B76
PAHAWH HMONG SIGN CIM PUB DAWB

16B77
PAHAWH HMONG SIGN CIM NRES TOS

16B7D
PAHAWH HMONG CLAN SIGN TSHEEJ

16B7E
PAHAWH HMONG CLAN SIGN YEEG

16B7F
PAHAWH HMONG CLAN SIGN LIS

16B80
PAHAWH HMONG CLAN SIGN LAUJ

16B81
PAHAWH HMONG CLAN SIGN XYOOJ

16B82
PAHAWH HMONG CLAN SIGN KOO

16B83
PAHAWH HMONG CLAN SIGN HAWJ

16B84
PAHAWH HMONG CLAN SIGN MUAS

16B85
PAHAWH HMONG CLAN SIGN THOJ

16B86
PAHAWH HMONG CLAN SIGN TSAB

16B87
PAHAWH HMONG CLAN SIGN PHAB

16B88
PAHAWH HMONG CLAN SIGN KHAB

16B89
PAHAWH HMONG CLAN SIGN HAM

16B8A
PAHAWH HMONG CLAN SIGN VAJ

16B8B
PAHAWH HMONG CLAN SIGN FAJ

16B8C
PAHAWH HMONG CLAN SIGN YAJ

16B8D
PAHAWH HMONG CLAN SIGN TSWB

16B8E
PAHAWH HMONG CLAN SIGN KWM

16B8F
PAHAWH HMONG CLAN SIGN VWJ

16D40
KIRAT RAI SIGN ANUSVARA

16D41
KIRAT RAI SIGN TONPI

16D42
KIRAT RAI SIGN VISARGA

16D43
KIRAT RAI LETTER A

16D44
KIRAT RAI LETTER KA

16D45
KIRAT RAI LETTER KHA

16D46
KIRAT RAI LETTER GA

16D47
KIRAT RAI LETTER GHA

16D48
KIRAT RAI LETTER NGA

16D49
KIRAT RAI LETTER CA

16D4A
KIRAT RAI LETTER CHA

16D4B
KIRAT RAI LETTER JA

16D4C
KIRAT RAI LETTER JHA

16D4D
KIRAT RAI LETTER NYA

16D4E
KIRAT RAI LETTER TTA

16D4F
KIRAT RAI LETTER TTHA

16D50
KIRAT RAI LETTER DDA

16D51
KIRAT RAI LETTER DDHA

16D52
KIRAT RAI LETTER TA

16D53
KIRAT RAI LETTER THA

16D54
KIRAT RAI LETTER DA

16D55
KIRAT RAI LETTER DHA

16D56
KIRAT RAI LETTER NA

16D57
KIRAT RAI LETTER PA

16D58
KIRAT RAI LETTER PHA

16D59
KIRAT RAI LETTER BA

16D5A
KIRAT RAI LETTER BHA

16D5B
KIRAT RAI LETTER MA

16D5C
KIRAT RAI LETTER YA

16D5D
KIRAT RAI LETTER RA

16D5E
KIRAT RAI LETTER LA

16D5F
KIRAT RAI LETTER VA

16D60
KIRAT RAI LETTER SA

16D61
KIRAT RAI LETTER SHA

16D62
KIRAT RAI LETTER HA

16D63
KIRAT RAI VOWEL SIGN AA

16D64
KIRAT RAI VOWEL SIGN I

16D65
KIRAT RAI VOWEL SIGN U

16D66
KIRAT RAI VOWEL SIGN UE

16D67
KIRAT RAI VOWEL SIGN E

16D68
KIRAT RAI VOWEL SIGN AI

16D69
KIRAT RAI VOWEL SIGN O

16D6A
KIRAT RAI VOWEL SIGN AU

16D6B
KIRAT RAI SIGN VIRAMA

16D6C
KIRAT RAI SIGN SAAT

16D6D
KIRAT RAI SIGN YUPI

16D6E
KIRAT RAI DANDA

16D6F
KIRAT RAI DOUBLE DANDA

16D70
KIRAT RAI DIGIT ZERO

16D71
KIRAT RAI DIGIT ONE

16D72
KIRAT RAI DIGIT TWO

16D73
KIRAT RAI DIGIT THREE

16D74
KIRAT RAI DIGIT FOUR

16D75
KIRAT RAI DIGIT FIVE

16D76
KIRAT RAI DIGIT SIX

16D77
KIRAT RAI DIGIT SEVEN

16D78
KIRAT RAI DIGIT EIGHT

16D79
KIRAT RAI DIGIT NINE

16E40
MEDEFAIDRIN CAPITAL LETTER M

16E41
MEDEFAIDRIN CAPITAL LETTER S

16E42
MEDEFAIDRIN CAPITAL LETTER V

16E43
MEDEFAIDRIN CAPITAL LETTER W

16E44
MEDEFAIDRIN CAPITAL LETTER ATIU

16E45
MEDEFAIDRIN CAPITAL LETTER Z

16E46
MEDEFAIDRIN CAPITAL LETTER KP

16E47
MEDEFAIDRIN CAPITAL LETTER P

16E48
MEDEFAIDRIN CAPITAL LETTER T

16E49
MEDEFAIDRIN CAPITAL LETTER G

16E4A
MEDEFAIDRIN CAPITAL LETTER F

16E4B
MEDEFAIDRIN CAPITAL LETTER I

16E4C
MEDEFAIDRIN CAPITAL LETTER K

16E4D
MEDEFAIDRIN CAPITAL LETTER A

16E4E
MEDEFAIDRIN CAPITAL LETTER J

16E4F
MEDEFAIDRIN CAPITAL LETTER E

16E50
MEDEFAIDRIN CAPITAL LETTER B

16E51
MEDEFAIDRIN CAPITAL LETTER C

16E52
MEDEFAIDRIN CAPITAL LETTER U

16E53
MEDEFAIDRIN CAPITAL LETTER YU

16E54
MEDEFAIDRIN CAPITAL LETTER L

16E55
MEDEFAIDRIN CAPITAL LETTER Q

16E56
MEDEFAIDRIN CAPITAL LETTER H

16E56
MEDEFAIDRIN CAPITAL LETTER HP

16E57
MEDEFAIDRIN CAPITAL LETTER NG

16E57
MEDEFAIDRIN CAPITAL LETTER NY

16E58
MEDEFAIDRIN CAPITAL LETTER X

16E59
MEDEFAIDRIN CAPITAL LETTER D

16E5A
MEDEFAIDRIN CAPITAL LETTER OE

16E5B
MEDEFAIDRIN CAPITAL LETTER N

16E5C
MEDEFAIDRIN CAPITAL LETTER R

16E5D
MEDEFAIDRIN CAPITAL LETTER O

16E5E
MEDEFAIDRIN CAPITAL LETTER AI

16E5F
MEDEFAIDRIN CAPITAL LETTER Y

16E60
MEDEFAIDRIN SMALL LETTER M

16E61
MEDEFAIDRIN SMALL LETTER S

16E62
MEDEFAIDRIN SMALL LETTER V

16E63
MEDEFAIDRIN SMALL LETTER W

16E64
MEDEFAIDRIN SMALL LETTER ATIU

16E65
MEDEFAIDRIN SMALL LETTER Z

16E66
MEDEFAIDRIN SMALL LETTER KP

16E67
MEDEFAIDRIN SMALL LETTER P

16E68
MEDEFAIDRIN SMALL LETTER T

16E69
MEDEFAIDRIN SMALL LETTER G

16E6A
MEDEFAIDRIN SMALL LETTER F

16E6B
MEDEFAIDRIN SMALL LETTER I

16E6C
MEDEFAIDRIN SMALL LETTER K

16E6D
MEDEFAIDRIN SMALL LETTER A

16E6E
MEDEFAIDRIN SMALL LETTER J

16E6F
MEDEFAIDRIN SMALL LETTER E

16E70
MEDEFAIDRIN SMALL LETTER B

16E71
MEDEFAIDRIN SMALL LETTER C

16E72
MEDEFAIDRIN SMALL LETTER U

16E73
MEDEFAIDRIN SMALL LETTER YU

16E74
MEDEFAIDRIN SMALL LETTER L

16E75
MEDEFAIDRIN SMALL LETTER Q

16E76
MEDEFAIDRIN SMALL LETTER H

16E76
MEDEFAIDRIN SMALL LETTER HP

16E77
MEDEFAIDRIN SMALL LETTER NG

16E77
MEDEFAIDRIN SMALL LETTER NY

16E78
MEDEFAIDRIN SMALL LETTER X

16E79
MEDEFAIDRIN SMALL LETTER D

16E7A
MEDEFAIDRIN SMALL LETTER OE

16E7B
MEDEFAIDRIN SMALL LETTER N

16E7C
MEDEFAIDRIN SMALL LETTER R

16E7D
MEDEFAIDRIN SMALL LETTER O

16E7E
MEDEFAIDRIN SMALL LETTER AI

16E7F
MEDEFAIDRIN SMALL LETTER Y

16E80
MEDEFAIDRIN DIGIT ZERO

16E81
MEDEFAIDRIN DIGIT ONE

16E82
MEDEFAIDRIN DIGIT TWO

16E83
MEDEFAIDRIN DIGIT THREE

16E84
MEDEFAIDRIN DIGIT FOUR

16E85
MEDEFAIDRIN DIGIT FIVE

16E86
MEDEFAIDRIN DIGIT SIX

16E87
MEDEFAIDRIN DIGIT SEVEN

16E88
MEDEFAIDRIN DIGIT EIGHT

16E89
MEDEFAIDRIN DIGIT NINE

16E8A
MEDEFAIDRIN NUMBER TEN

16E8B
MEDEFAIDRIN NUMBER ELEVEN

16E8C
MEDEFAIDRIN NUMBER TWELVE

16E8D
MEDEFAIDRIN NUMBER THIRTEEN

16E8E
MEDEFAIDRIN NUMBER FOURTEEN

16E8F
MEDEFAIDRIN NUMBER FIFTEEN

16E90
MEDEFAIDRIN NUMBER SIXTEEN

16E91
MEDEFAIDRIN NUMBER SEVENTEEN

16E92
MEDEFAIDRIN NUMBER EIGHTEEN

16E93
MEDEFAIDRIN NUMBER NINETEEN

16E94
MEDEFAIDRIN DIGIT ONE ALTERNATE FORM

16E95
MEDEFAIDRIN DIGIT TWO ALTERNATE FORM

16E96
MEDEFAIDRIN DIGIT THREE ALTERNATE FORM

16E97
MEDEFAIDRIN COMMA

16E98
MEDEFAIDRIN FULL STOP

16E99
MEDEFAIDRIN SYMBOL AIVA

16E9A
MEDEFAIDRIN EXCLAMATION OH

16EA0
BERIA ERFE CAPITAL LETTER ARKAB

16EA1
BERIA ERFE CAPITAL LETTER BASIGNA

16EA2
BERIA ERFE CAPITAL LETTER DARBAI

16EA3
BERIA ERFE CAPITAL LETTER EH

16EA4
BERIA ERFE CAPITAL LETTER FITKO

16EA5
BERIA ERFE CAPITAL LETTER GOWAY

16EA6
BERIA ERFE CAPITAL LETTER HIRDEABO

16EA7
BERIA ERFE CAPITAL LETTER I

16EA8
BERIA ERFE CAPITAL LETTER DJAI

16EA9
BERIA ERFE CAPITAL LETTER KOBO

16EAA
BERIA ERFE CAPITAL LETTER LAKKO

16EAB
BERIA ERFE CAPITAL LETTER MERI

16EAC
BERIA ERFE CAPITAL LETTER NINI

16EAD
BERIA ERFE CAPITAL LETTER GNA

16EAE
BERIA ERFE CAPITAL LETTER NGAY

16EAF
BERIA ERFE CAPITAL LETTER OI

16EB0
BERIA ERFE CAPITAL LETTER PI

16EB1
BERIA ERFE CAPITAL LETTER ERIGO

16EB2
BERIA ERFE CAPITAL LETTER ERIGO TAMURA

16EB3
BERIA ERFE CAPITAL LETTER SERI

16EB4
BERIA ERFE CAPITAL LETTER SHEP

16EB5
BERIA ERFE CAPITAL LETTER TATASOUE

16EB6
BERIA ERFE CAPITAL LETTER UI

16EB7
BERIA ERFE CAPITAL LETTER WASSE

16EB8
BERIA ERFE CAPITAL LETTER AY

16EBB
BERIA ERFE SMALL LETTER ARKAB

16EBC
BERIA ERFE SMALL LETTER BASIGNA

16EBD
BERIA ERFE SMALL LETTER DARBAI

16EBE
BERIA ERFE SMALL LETTER EH

16EBF
BERIA ERFE SMALL LETTER FITKO

16EC0
BERIA ERFE SMALL LETTER GOWAY

16EC1
BERIA ERFE SMALL LETTER HIRDEABO

16EC2
BERIA ERFE SMALL LETTER I

16EC3
BERIA ERFE SMALL LETTER DJAI

16EC4
BERIA ERFE SMALL LETTER KOBO

16EC5
BERIA ERFE SMALL LETTER LAKKO

16EC6
BERIA ERFE SMALL LETTER MERI

16EC7
BERIA ERFE SMALL LETTER NINI

16EC8
BERIA ERFE SMALL LETTER GNA

16EC9
BERIA ERFE SMALL LETTER NGAY

16ECA
BERIA ERFE SMALL LETTER OI

16ECB
BERIA ERFE SMALL LETTER PI

16ECC
BERIA ERFE SMALL LETTER ERIGO

16ECD
BERIA ERFE SMALL LETTER ERIGO TAMURA

16ECE
BERIA ERFE SMALL LETTER SERI

16ECF
BERIA ERFE SMALL LETTER SHEP

16ED0
BERIA ERFE SMALL LETTER TATASOUE

16ED1
BERIA ERFE SMALL LETTER UI

16ED2
BERIA ERFE SMALL LETTER WASSE

16ED3
BERIA ERFE SMALL LETTER AY

16F00
MIAO LETTER PA

16F01
MIAO LETTER BA

16F02
MIAO LETTER YI PA

16F03
MIAO LETTER PLA

16F04
MIAO LETTER MA

16F05
MIAO LETTER MHA

16F06
MIAO LETTER ARCHAIC MA

16F07
MIAO LETTER FA

16F08
MIAO LETTER VA

16F09
MIAO LETTER VFA

16F0A
MIAO LETTER TA

16F0B
MIAO LETTER DA

16F0C
MIAO LETTER YI TTA

16F0D
MIAO LETTER YI TA

16F0E
MIAO LETTER TTA

16F0F
MIAO LETTER DDA

16F10
MIAO LETTER NA

16F11
MIAO LETTER NHA

16F12
MIAO LETTER YI NNA

16F13
MIAO LETTER ARCHAIC NA

16F14
MIAO LETTER NNA

16F15
MIAO LETTER NNHA

16F16
MIAO LETTER LA

16F17
MIAO LETTER LYA

16F18
MIAO LETTER LHA

16F19
MIAO LETTER LHYA

16F1A
MIAO LETTER TLHA

16F1B
MIAO LETTER DLHA

16F1C
MIAO LETTER TLHYA

16F1D
MIAO LETTER DLHYA

16F1E
MIAO LETTER KA

16F1F
MIAO LETTER GA

16F20
MIAO LETTER YI KA

16F21
MIAO LETTER QA

16F22
MIAO LETTER QGA

16F23
MIAO LETTER NGA

16F24
MIAO LETTER NGHA

16F25
MIAO LETTER ARCHAIC NGA

16F26
MIAO LETTER HA

16F27
MIAO LETTER XA

16F28
MIAO LETTER GHA

16F29
MIAO LETTER GHHA

16F2A
MIAO LETTER TSSA

16F2B
MIAO LETTER DZZA

16F2C
MIAO LETTER NYA

16F2D
MIAO LETTER NYHA

16F2E
MIAO LETTER TSHA

16F2F
MIAO LETTER DZHA

16F30
MIAO LETTER YI TSHA

16F31
MIAO LETTER YI DZHA

16F32
MIAO LETTER REFORMED TSHA

16F33
MIAO LETTER SHA

16F34
MIAO LETTER SSA

16F35
MIAO LETTER ZHA

16F36
MIAO LETTER ZSHA

16F37
MIAO LETTER TSA

16F38
MIAO LETTER DZA

16F39
MIAO LETTER YI TSA

16F3A
MIAO LETTER SA

16F3B
MIAO LETTER ZA

16F3C
MIAO LETTER ZSA

16F3D
MIAO LETTER ZZA

16F3E
MIAO LETTER ZZSA

16F3F
MIAO LETTER ARCHAIC ZZA

16F40
MIAO LETTER ZZYA

16F41
MIAO LETTER ZZSYA

16F42
MIAO LETTER WA

16F43
MIAO LETTER AH

16F44
MIAO LETTER HHA

16F45
MIAO LETTER BRI

16F46
MIAO LETTER SYI

16F47
MIAO LETTER DZYI

16F48
MIAO LETTER TE

16F49
MIAO LETTER TSE

16F4A
MIAO LETTER RTE

16F4F
MIAO SIGN CONSONANT MODIFIER BAR

16F50
MIAO LETTER NASALIZATION

16F51
MIAO SIGN ASPIRATION

16F52
MIAO SIGN REFORMED VOICING

16F53
MIAO SIGN REFORMED ASPIRATION

16F54
MIAO VOWEL SIGN A

16F55
MIAO VOWEL SIGN AA

16F56
MIAO VOWEL SIGN AHH

16F57
MIAO VOWEL SIGN AN

16F58
MIAO VOWEL SIGN ANG

16F59
MIAO VOWEL SIGN O

16F5A
MIAO VOWEL SIGN OO

16F5B
MIAO VOWEL SIGN WO

16F5C
MIAO VOWEL SIGN W

16F5D
MIAO VOWEL SIGN E

16F5E
MIAO VOWEL SIGN EN

16F5F
MIAO VOWEL SIGN ENG

16F60
MIAO VOWEL SIGN OEY

16F61
MIAO VOWEL SIGN I

16F62
MIAO VOWEL SIGN IA

16F63
MIAO VOWEL SIGN IAN

16F64
MIAO VOWEL SIGN IANG

16F65
MIAO VOWEL SIGN IO

16F66
MIAO VOWEL SIGN IE

16F67
MIAO VOWEL SIGN II

16F68
MIAO VOWEL SIGN IU

16F69
MIAO VOWEL SIGN ING

16F6A
MIAO VOWEL SIGN U

16F6B
MIAO VOWEL SIGN UA

16F6C
MIAO VOWEL SIGN UAN

16F6D
MIAO VOWEL SIGN UANG

16F6E
MIAO VOWEL SIGN UU

16F6F
MIAO VOWEL SIGN UEI

16F70
MIAO VOWEL SIGN UNG

16F71
MIAO VOWEL SIGN Y

16F72
MIAO VOWEL SIGN YI

16F73
MIAO VOWEL SIGN AE

16F74
MIAO VOWEL SIGN AEE

16F75
MIAO VOWEL SIGN ERR

16F76
MIAO VOWEL SIGN ROUNDED ERR

16F77
MIAO VOWEL SIGN ER

16F78
MIAO VOWEL SIGN ROUNDED ER

16F79
MIAO VOWEL SIGN AI

16F7A
MIAO VOWEL SIGN EI

16F7B
MIAO VOWEL SIGN AU

16F7C
MIAO VOWEL SIGN OU

16F7D
MIAO VOWEL SIGN N

16F7E
MIAO VOWEL SIGN NG

16F7F
MIAO VOWEL SIGN UOG

16F80
MIAO VOWEL SIGN YUI

16F81
MIAO VOWEL SIGN OG

16F82
MIAO VOWEL SIGN OER

16F83
MIAO VOWEL SIGN VW

16F84
MIAO VOWEL SIGN IG

16F85
MIAO VOWEL SIGN EA

16F86
MIAO VOWEL SIGN IONG

16F87
MIAO VOWEL SIGN UI

16F8F
MIAO TONE RIGHT

16F90
MIAO TONE TOP RIGHT

16F91
MIAO TONE ABOVE

16F92
MIAO TONE BELOW

16F93
MIAO LETTER TONE-2

16F94
MIAO LETTER TONE-3

16F95
MIAO LETTER TONE-4

16F96
MIAO LETTER TONE-5

16F97
MIAO LETTER TONE-6

16F98
MIAO LETTER TONE-7

16F99
MIAO LETTER TONE-8

16F9A
MIAO LETTER REFORMED TONE-1

16F9B
MIAO LETTER REFORMED TONE-2

16F9C
MIAO LETTER REFORMED TONE-4

16F9D
MIAO LETTER REFORMED TONE-5

16F9E
MIAO LETTER REFORMED TONE-6

16F9F
MIAO LETTER REFORMED TONE-8

16FE0
TANGUT ITERATION MARK

16FE1
NUSHU ITERATION MARK

16FE2
OLD CHINESE HOOK MARK

16FE3
OLD CHINESE ITERATION MARK

16FE4
KHITAN SMALL SCRIPT FILLER

16FF0
VIETNAMESE ALTERNATE READING MARK CA

16FF1
VIETNAMESE ALTERNATE READING MARK NHAY

16FF2
CHINESE SMALL SIMPLIFIED ER

16FF3
CHINESE SMALL TRADITIONAL ER

16FF4
YANGQIN SIGN SLOW ONE BEAT

16FF5
YANGQIN SIGN SLOW THREE HALF BEATS

16FF6
YANGQIN SIGN SLOW TWO BEATS

18800
TANGUT COMPONENT-001

18801
TANGUT COMPONENT-002

18802
TANGUT COMPONENT-003

18803
TANGUT COMPONENT-004

18804
TANGUT COMPONENT-005

18805
TANGUT COMPONENT-006

18806
TANGUT COMPONENT-007

18807
TANGUT COMPONENT-008

18808
TANGUT COMPONENT-009

18809
TANGUT COMPONENT-010

1880A
TANGUT COMPONENT-011

1880B
TANGUT COMPONENT-012

1880C
TANGUT COMPONENT-013

1880D
TANGUT COMPONENT-014

1880E
TANGUT COMPONENT-015

1880F
TANGUT COMPONENT-016

18810
TANGUT COMPONENT-017

18811
TANGUT COMPONENT-018

18812
TANGUT COMPONENT-019

18813
TANGUT COMPONENT-020

18814
TANGUT COMPONENT-021

18815
TANGUT COMPONENT-022

18816
TANGUT COMPONENT-023

18817
TANGUT COMPONENT-024

18818
TANGUT COMPONENT-025

18819
TANGUT COMPONENT-026

1881A
TANGUT COMPONENT-027

1881B
TANGUT COMPONENT-028

1881C
TANGUT COMPONENT-029

1881D
TANGUT COMPONENT-030

1881E
TANGUT COMPONENT-031

1881F
TANGUT COMPONENT-032

18820
TANGUT COMPONENT-033

18821
TANGUT COMPONENT-034

18822
TANGUT COMPONENT-035

18823
TANGUT COMPONENT-036

18824
TANGUT COMPONENT-037

18825
TANGUT COMPONENT-038

18826
TANGUT COMPONENT-039

18827
TANGUT COMPONENT-040

18828
TANGUT COMPONENT-041

18829
TANGUT COMPONENT-042

1882A
TANGUT COMPONENT-043

1882B
TANGUT COMPONENT-044

1882C
TANGUT COMPONENT-045

1882D
TANGUT COMPONENT-046

1882E
TANGUT COMPONENT-047

1882F
TANGUT COMPONENT-048

18830
TANGUT COMPONENT-049

18831
TANGUT COMPONENT-050

18832
TANGUT COMPONENT-051

18833
TANGUT COMPONENT-052

18834
TANGUT COMPONENT-053

18835
TANGUT COMPONENT-054

18836
TANGUT COMPONENT-055

18837
TANGUT COMPONENT-056

18838
TANGUT COMPONENT-057

18839
TANGUT COMPONENT-058

1883A
TANGUT COMPONENT-059

1883B
TANGUT COMPONENT-060

1883C
TANGUT COMPONENT-061

1883D
TANGUT COMPONENT-062

1883E
TANGUT COMPONENT-063

1883F
TANGUT COMPONENT-064

18840
TANGUT COMPONENT-065

18841
TANGUT COMPONENT-066

18842
TANGUT COMPONENT-067

18843
TANGUT COMPONENT-068

18844
TANGUT COMPONENT-069

18845
TANGUT COMPONENT-070

18846
TANGUT COMPONENT-071

18847
TANGUT COMPONENT-072

18848
TANGUT COMPONENT-073

18849
TANGUT COMPONENT-074

1884A
TANGUT COMPONENT-075

1884B
TANGUT COMPONENT-076

1884C
TANGUT COMPONENT-077

1884D
TANGUT COMPONENT-078

1884E
TANGUT COMPONENT-079

1884F
TANGUT COMPONENT-080

18850
TANGUT COMPONENT-081

18851
TANGUT COMPONENT-082

18852
TANGUT COMPONENT-083

18853
TANGUT COMPONENT-084

18854
TANGUT COMPONENT-085

18855
TANGUT COMPONENT-086

18856
TANGUT COMPONENT-087

18857
TANGUT COMPONENT-088

18858
TANGUT COMPONENT-089

18859
TANGUT COMPONENT-090

1885A
TANGUT COMPONENT-091

1885B
TANGUT COMPONENT-092

1885C
TANGUT COMPONENT-093

1885D
TANGUT COMPONENT-094

1885E
TANGUT COMPONENT-095

1885F
TANGUT COMPONENT-096

18860
TANGUT COMPONENT-097

18861
TANGUT COMPONENT-098

18862
TANGUT COMPONENT-099

18863
TANGUT COMPONENT-100

18864
TANGUT COMPONENT-101

18865
TANGUT COMPONENT-102

18866
TANGUT COMPONENT-103

18867
TANGUT COMPONENT-104

18868
TANGUT COMPONENT-105

18869
TANGUT COMPONENT-106

1886A
TANGUT COMPONENT-107

1886B
TANGUT COMPONENT-108

1886C
TANGUT COMPONENT-109

1886D
TANGUT COMPONENT-110

1886E
TANGUT COMPONENT-111

1886F
TANGUT COMPONENT-112

18870
TANGUT COMPONENT-113

18871
TANGUT COMPONENT-114

18872
TANGUT COMPONENT-115

18873
TANGUT COMPONENT-116

18874
TANGUT COMPONENT-117

18875
TANGUT COMPONENT-118

18876
TANGUT COMPONENT-119

18877
TANGUT COMPONENT-120

18878
TANGUT COMPONENT-121

18879
TANGUT COMPONENT-122

1887A
TANGUT COMPONENT-123

1887B
TANGUT COMPONENT-124

1887C
TANGUT COMPONENT-125

1887D
TANGUT COMPONENT-126

1887E
TANGUT COMPONENT-127

1887F
TANGUT COMPONENT-128

18880
TANGUT COMPONENT-129

18881
TANGUT COMPONENT-130

18882
TANGUT COMPONENT-131

18883
TANGUT COMPONENT-132

18884
TANGUT COMPONENT-133

18885
TANGUT COMPONENT-134

18886
TANGUT COMPONENT-135

18887
TANGUT COMPONENT-136

18888
TANGUT COMPONENT-137

18889
TANGUT COMPONENT-138

1888A
TANGUT COMPONENT-139

1888B
TANGUT COMPONENT-140

1888C
TANGUT COMPONENT-141

1888D
TANGUT COMPONENT-142

1888E
TANGUT COMPONENT-143

1888F
TANGUT COMPONENT-144

18890
TANGUT COMPONENT-145

18891
TANGUT COMPONENT-146

18892
TANGUT COMPONENT-147

18893
TANGUT COMPONENT-148

18894
TANGUT COMPONENT-149

18895
TANGUT COMPONENT-150

18896
TANGUT COMPONENT-151

18897
TANGUT COMPONENT-152

18898
TANGUT COMPONENT-153

18899
TANGUT COMPONENT-154

1889A
TANGUT COMPONENT-155

1889B
TANGUT COMPONENT-156

1889C
TANGUT COMPONENT-157

1889D
TANGUT COMPONENT-158

1889E
TANGUT COMPONENT-159

1889F
TANGUT COMPONENT-160

188A0
TANGUT COMPONENT-161

188A1
TANGUT COMPONENT-162

188A2
TANGUT COMPONENT-163

188A3
TANGUT COMPONENT-164

188A4
TANGUT COMPONENT-165

188A5
TANGUT COMPONENT-166

188A6
TANGUT COMPONENT-167

188A7
TANGUT COMPONENT-168

188A8
TANGUT COMPONENT-169

188A9
TANGUT COMPONENT-170

188AA
TANGUT COMPONENT-171

188AB
TANGUT COMPONENT-172

188AC
TANGUT COMPONENT-173

188AD
TANGUT COMPONENT-174

188AE
TANGUT COMPONENT-175

188AF
TANGUT COMPONENT-176

188B0
TANGUT COMPONENT-177

188B1
TANGUT COMPONENT-178

188B2
TANGUT COMPONENT-179

188B3
TANGUT COMPONENT-180

188B4
TANGUT COMPONENT-181

188B5
TANGUT COMPONENT-182

188B6
TANGUT COMPONENT-183

188B7
TANGUT COMPONENT-184

188B8
TANGUT COMPONENT-185

188B9
TANGUT COMPONENT-186

188BA
TANGUT COMPONENT-187

188BB
TANGUT COMPONENT-188

188BC
TANGUT COMPONENT-189

188BD
TANGUT COMPONENT-190

188BE
TANGUT COMPONENT-191

188BF
TANGUT COMPONENT-192

188C0
TANGUT COMPONENT-193

188C1
TANGUT COMPONENT-194

188C2
TANGUT COMPONENT-195

188C3
TANGUT COMPONENT-196

188C4
TANGUT COMPONENT-197

188C5
TANGUT COMPONENT-198

188C6
TANGUT COMPONENT-199

188C7
TANGUT COMPONENT-200

188C8
TANGUT COMPONENT-201

188C9
TANGUT COMPONENT-202

188CA
TANGUT COMPONENT-203

188CB
TANGUT COMPONENT-204

188CC
TANGUT COMPONENT-205

188CD
TANGUT COMPONENT-206

188CE
TANGUT COMPONENT-207

188CF
TANGUT COMPONENT-208

188D0
TANGUT COMPONENT-209

188D1
TANGUT COMPONENT-210

188D2
TANGUT COMPONENT-211

188D3
TANGUT COMPONENT-212

188D4
TANGUT COMPONENT-213

188D5
TANGUT COMPONENT-214

188D6
TANGUT COMPONENT-215

188D7
TANGUT COMPONENT-216

188D8
TANGUT COMPONENT-217

188D9
TANGUT COMPONENT-218

188DA
TANGUT COMPONENT-219

188DB
TANGUT COMPONENT-220

188DC
TANGUT COMPONENT-221

188DD
TANGUT COMPONENT-222

188DE
TANGUT COMPONENT-223

188DF
TANGUT COMPONENT-224

188E0
TANGUT COMPONENT-225

188E1
TANGUT COMPONENT-226

188E2
TANGUT COMPONENT-227

188E3
TANGUT COMPONENT-228

188E4
TANGUT COMPONENT-229

188E5
TANGUT COMPONENT-230

188E6
TANGUT COMPONENT-231

188E7
TANGUT COMPONENT-232

188E8
TANGUT COMPONENT-233

188E9
TANGUT COMPONENT-234

188EA
TANGUT COMPONENT-235

188EB
TANGUT COMPONENT-236

188EC
TANGUT COMPONENT-237

188ED
TANGUT COMPONENT-238

188EE
TANGUT COMPONENT-239

188EF
TANGUT COMPONENT-240

188F0
TANGUT COMPONENT-241

188F1
TANGUT COMPONENT-242

188F2
TANGUT COMPONENT-243

188F3
TANGUT COMPONENT-244

188F4
TANGUT COMPONENT-245

188F5
TANGUT COMPONENT-246

188F6
TANGUT COMPONENT-247

188F7
TANGUT COMPONENT-248

188F8
TANGUT COMPONENT-249

188F9
TANGUT COMPONENT-250

188FA
TANGUT COMPONENT-251

188FB
TANGUT COMPONENT-252

188FC
TANGUT COMPONENT-253

188FD
TANGUT COMPONENT-254

188FE
TANGUT COMPONENT-255

188FF
TANGUT COMPONENT-256

18900
TANGUT COMPONENT-257

18901
TANGUT COMPONENT-258

18902
TANGUT COMPONENT-259

18903
TANGUT COMPONENT-260

18904
TANGUT COMPONENT-261

18905
TANGUT COMPONENT-262

18906
TANGUT COMPONENT-263

18907
TANGUT COMPONENT-264

18908
TANGUT COMPONENT-265

18909
TANGUT COMPONENT-266

1890A
TANGUT COMPONENT-267

1890B
TANGUT COMPONENT-268

1890C
TANGUT COMPONENT-269

1890D
TANGUT COMPONENT-270

1890E
TANGUT COMPONENT-271

1890F
TANGUT COMPONENT-272

18910
TANGUT COMPONENT-273

18911
TANGUT COMPONENT-274

18912
TANGUT COMPONENT-275

18913
TANGUT COMPONENT-276

18914
TANGUT COMPONENT-277

18915
TANGUT COMPONENT-278

18916
TANGUT COMPONENT-279

18917
TANGUT COMPONENT-280

18918
TANGUT COMPONENT-281

18919
TANGUT COMPONENT-282

1891A
TANGUT COMPONENT-283

1891B
TANGUT COMPONENT-284

1891C
TANGUT COMPONENT-285

1891D
TANGUT COMPONENT-286

1891E
TANGUT COMPONENT-287

1891F
TANGUT COMPONENT-288

18920
TANGUT COMPONENT-289

18921
TANGUT COMPONENT-290

18922
TANGUT COMPONENT-291

18923
TANGUT COMPONENT-292

18924
TANGUT COMPONENT-293

18925
TANGUT COMPONENT-294

18926
TANGUT COMPONENT-295

18927
TANGUT COMPONENT-296

18928
TANGUT COMPONENT-297

18929
TANGUT COMPONENT-298

1892A
TANGUT COMPONENT-299

1892B
TANGUT COMPONENT-300

1892C
TANGUT COMPONENT-301

1892D
TANGUT COMPONENT-302

1892E
TANGUT COMPONENT-303

1892F
TANGUT COMPONENT-304

18930
TANGUT COMPONENT-305

18931
TANGUT COMPONENT-306

18932
TANGUT COMPONENT-307

18933
TANGUT COMPONENT-308

18934
TANGUT COMPONENT-309

18935
TANGUT COMPONENT-310

18936
TANGUT COMPONENT-311

18937
TANGUT COMPONENT-312

18938
TANGUT COMPONENT-313

18939
TANGUT COMPONENT-314

1893A
TANGUT COMPONENT-315

1893B
TANGUT COMPONENT-316

1893C
TANGUT COMPONENT-317

1893D
TANGUT COMPONENT-318

1893E
TANGUT COMPONENT-319

1893F
TANGUT COMPONENT-320

18940
TANGUT COMPONENT-321

18941
TANGUT COMPONENT-322

18942
TANGUT COMPONENT-323

18943
TANGUT COMPONENT-324

18944
TANGUT COMPONENT-325

18945
TANGUT COMPONENT-326

18946
TANGUT COMPONENT-327

18947
TANGUT COMPONENT-328

18948
TANGUT COMPONENT-329

18949
TANGUT COMPONENT-330

1894A
TANGUT COMPONENT-331

1894B
TANGUT COMPONENT-332

1894C
TANGUT COMPONENT-333

1894D
TANGUT COMPONENT-334

1894E
TANGUT COMPONENT-335

1894F
TANGUT COMPONENT-336

18950
TANGUT COMPONENT-337

18951
TANGUT COMPONENT-338

18952
TANGUT COMPONENT-339

18953
TANGUT COMPONENT-340

18954
TANGUT COMPONENT-341

18955
TANGUT COMPONENT-342

18956
TANGUT COMPONENT-343

18957
TANGUT COMPONENT-344

18958
TANGUT COMPONENT-345

18959
TANGUT COMPONENT-346

1895A
TANGUT COMPONENT-347

1895B
TANGUT COMPONENT-348

1895C
TANGUT COMPONENT-349

1895D
TANGUT COMPONENT-350

1895E
TANGUT COMPONENT-351

1895F
TANGUT COMPONENT-352

18960
TANGUT COMPONENT-353

18961
TANGUT COMPONENT-354

18962
TANGUT COMPONENT-355

18963
TANGUT COMPONENT-356

18964
TANGUT COMPONENT-357

18965
TANGUT COMPONENT-358

18966
TANGUT COMPONENT-359

18967
TANGUT COMPONENT-360

18968
TANGUT COMPONENT-361

18969
TANGUT COMPONENT-362

1896A
TANGUT COMPONENT-363

1896B
TANGUT COMPONENT-364

1896C
TANGUT COMPONENT-365

1896D
TANGUT COMPONENT-366

1896E
TANGUT COMPONENT-367

1896F
TANGUT COMPONENT-368

18970
TANGUT COMPONENT-369

18971
TANGUT COMPONENT-370

18972
TANGUT COMPONENT-371

18973
TANGUT COMPONENT-372

18974
TANGUT COMPONENT-373

18975
TANGUT COMPONENT-374

18976
TANGUT COMPONENT-375

18977
TANGUT COMPONENT-376

18978
TANGUT COMPONENT-377

18979
TANGUT COMPONENT-378

1897A
TANGUT COMPONENT-379

1897B
TANGUT COMPONENT-380

1897C
TANGUT COMPONENT-381

1897D
TANGUT COMPONENT-382

1897E
TANGUT COMPONENT-383

1897F
TANGUT COMPONENT-384

18980
TANGUT COMPONENT-385

18981
TANGUT COMPONENT-386

18982
TANGUT COMPONENT-387

18983
TANGUT COMPONENT-388

18984
TANGUT COMPONENT-389

18985
TANGUT COMPONENT-390

18986
TANGUT COMPONENT-391

18987
TANGUT COMPONENT-392

18988
TANGUT COMPONENT-393

18989
TANGUT COMPONENT-394

1898A
TANGUT COMPONENT-395

1898B
TANGUT COMPONENT-396

1898C
TANGUT COMPONENT-397

1898D
TANGUT COMPONENT-398

1898E
TANGUT COMPONENT-399

1898F
TANGUT COMPONENT-400

18990
TANGUT COMPONENT-401

18991
TANGUT COMPONENT-402

18992
TANGUT COMPONENT-403

18993
TANGUT COMPONENT-404

18994
TANGUT COMPONENT-405

18995
TANGUT COMPONENT-406

18996
TANGUT COMPONENT-407

18997
TANGUT COMPONENT-408

18998
TANGUT COMPONENT-409

18999
TANGUT COMPONENT-410

1899A
TANGUT COMPONENT-411

1899B
TANGUT COMPONENT-412

1899C
TANGUT COMPONENT-413

1899D
TANGUT COMPONENT-414

1899E
TANGUT COMPONENT-415

1899F
TANGUT COMPONENT-416

189A0
TANGUT COMPONENT-417

189A1
TANGUT COMPONENT-418

189A2
TANGUT COMPONENT-419

189A3
TANGUT COMPONENT-420

189A4
TANGUT COMPONENT-421

189A5
TANGUT COMPONENT-422

189A6
TANGUT COMPONENT-423

189A7
TANGUT COMPONENT-424

189A8
TANGUT COMPONENT-425

189A9
TANGUT COMPONENT-426

189AA
TANGUT COMPONENT-427

189AB
TANGUT COMPONENT-428

189AC
TANGUT COMPONENT-429

189AD
TANGUT COMPONENT-430

189AE
TANGUT COMPONENT-431

189AF
TANGUT COMPONENT-432

189B0
TANGUT COMPONENT-433

189B1
TANGUT COMPONENT-434

189B2
TANGUT COMPONENT-435

189B3
TANGUT COMPONENT-436

189B4
TANGUT COMPONENT-437

189B5
TANGUT COMPONENT-438

189B6
TANGUT COMPONENT-439

189B7
TANGUT COMPONENT-440

189B8
TANGUT COMPONENT-441

189B9
TANGUT COMPONENT-442

189BA
TANGUT COMPONENT-443

189BB
TANGUT COMPONENT-444

189BC
TANGUT COMPONENT-445

189BD
TANGUT COMPONENT-446

189BE
TANGUT COMPONENT-447

189BF
TANGUT COMPONENT-448

189C0
TANGUT COMPONENT-449

189C1
TANGUT COMPONENT-450

189C2
TANGUT COMPONENT-451

189C3
TANGUT COMPONENT-452

189C4
TANGUT COMPONENT-453

189C5
TANGUT COMPONENT-454

189C6
TANGUT COMPONENT-455

189C7
TANGUT COMPONENT-456

189C8
TANGUT COMPONENT-457

189C9
TANGUT COMPONENT-458

189CA
TANGUT COMPONENT-459

189CB
TANGUT COMPONENT-460

189CC
TANGUT COMPONENT-461

189CD
TANGUT COMPONENT-462

189CE
TANGUT COMPONENT-463

189CF
TANGUT COMPONENT-464

189D0
TANGUT COMPONENT-465

189D1
TANGUT COMPONENT-466

189D2
TANGUT COMPONENT-467

189D3
TANGUT COMPONENT-468

189D4
TANGUT COMPONENT-469

189D5
TANGUT COMPONENT-470

189D6
TANGUT COMPONENT-471

189D7
TANGUT COMPONENT-472

189D8
TANGUT COMPONENT-473

189D9
TANGUT COMPONENT-474

189DA
TANGUT COMPONENT-475

189DB
TANGUT COMPONENT-476

189DC
TANGUT COMPONENT-477

189DD
TANGUT COMPONENT-478

189DE
TANGUT COMPONENT-479

189DF
TANGUT COMPONENT-480

189E0
TANGUT COMPONENT-481

189E1
TANGUT COMPONENT-482

189E2
TANGUT COMPONENT-483

189E3
TANGUT COMPONENT-484

189E4
TANGUT COMPONENT-485

189E5
TANGUT COMPONENT-486

189E6
TANGUT COMPONENT-487

189E7
TANGUT COMPONENT-488

189E8
TANGUT COMPONENT-489

189E9
TANGUT COMPONENT-490

189EA
TANGUT COMPONENT-491

189EB
TANGUT COMPONENT-492

189EC
TANGUT COMPONENT-493

189ED
TANGUT COMPONENT-494

189EE
TANGUT COMPONENT-495

189EF
TANGUT COMPONENT-496

189F0
TANGUT COMPONENT-497

189F1
TANGUT COMPONENT-498

189F2
TANGUT COMPONENT-499

189F3
TANGUT COMPONENT-500

189F4
TANGUT COMPONENT-501

189F5
TANGUT COMPONENT-502

189F6
TANGUT COMPONENT-503

189F7
TANGUT COMPONENT-504

189F8
TANGUT COMPONENT-505

189F9
TANGUT COMPONENT-506

189FA
TANGUT COMPONENT-507

189FB
TANGUT COMPONENT-508

189FC
TANGUT COMPONENT-509

189FD
TANGUT COMPONENT-510

189FE
TANGUT COMPONENT-511

189FF
TANGUT COMPONENT-512

18A00
TANGUT COMPONENT-513

18A01
TANGUT COMPONENT-514

18A02
TANGUT COMPONENT-515

18A03
TANGUT COMPONENT-516

18A04
TANGUT COMPONENT-517

18A05
TANGUT COMPONENT-518

18A06
TANGUT COMPONENT-519

18A07
TANGUT COMPONENT-520

18A08
TANGUT COMPONENT-521

18A09
TANGUT COMPONENT-522

18A0A
TANGUT COMPONENT-523

18A0B
TANGUT COMPONENT-524

18A0C
TANGUT COMPONENT-525

18A0D
TANGUT COMPONENT-526

18A0E
TANGUT COMPONENT-527

18A0F
TANGUT COMPONENT-528

18A10
TANGUT COMPONENT-529

18A11
TANGUT COMPONENT-530

18A12
TANGUT COMPONENT-531

18A13
TANGUT COMPONENT-532

18A14
TANGUT COMPONENT-533

18A15
TANGUT COMPONENT-534

18A16
TANGUT COMPONENT-535

18A17
TANGUT COMPONENT-536

18A18
TANGUT COMPONENT-537

18A19
TANGUT COMPONENT-538

18A1A
TANGUT COMPONENT-539

18A1B
TANGUT COMPONENT-540

18A1C
TANGUT COMPONENT-541

18A1D
TANGUT COMPONENT-542

18A1E
TANGUT COMPONENT-543

18A1F
TANGUT COMPONENT-544

18A20
TANGUT COMPONENT-545

18A21
TANGUT COMPONENT-546

18A22
TANGUT COMPONENT-547

18A23
TANGUT COMPONENT-548

18A24
TANGUT COMPONENT-549

18A25
TANGUT COMPONENT-550

18A26
TANGUT COMPONENT-551

18A27
TANGUT COMPONENT-552

18A28
TANGUT COMPONENT-553

18A29
TANGUT COMPONENT-554

18A2A
TANGUT COMPONENT-555

18A2B
TANGUT COMPONENT-556

18A2C
TANGUT COMPONENT-557

18A2D
TANGUT COMPONENT-558

18A2E
TANGUT COMPONENT-559

18A2F
TANGUT COMPONENT-560

18A30
TANGUT COMPONENT-561

18A31
TANGUT COMPONENT-562

18A32
TANGUT COMPONENT-563

18A33
TANGUT COMPONENT-564

18A34
TANGUT COMPONENT-565

18A35
TANGUT COMPONENT-566

18A36
TANGUT COMPONENT-567

18A37
TANGUT COMPONENT-568

18A38
TANGUT COMPONENT-569

18A39
TANGUT COMPONENT-570

18A3A
TANGUT COMPONENT-571

18A3B
TANGUT COMPONENT-572

18A3C
TANGUT COMPONENT-573

18A3D
TANGUT COMPONENT-574

18A3E
TANGUT COMPONENT-575

18A3F
TANGUT COMPONENT-576

18A40
TANGUT COMPONENT-577

18A41
TANGUT COMPONENT-578

18A42
TANGUT COMPONENT-579

18A43
TANGUT COMPONENT-580

18A44
TANGUT COMPONENT-581

18A45
TANGUT COMPONENT-582

18A46
TANGUT COMPONENT-583

18A47
TANGUT COMPONENT-584

18A48
TANGUT COMPONENT-585

18A49
TANGUT COMPONENT-586

18A4A
TANGUT COMPONENT-587

18A4B
TANGUT COMPONENT-588

18A4C
TANGUT COMPONENT-589

18A4D
TANGUT COMPONENT-590

18A4E
TANGUT COMPONENT-591

18A4F
TANGUT COMPONENT-592

18A50
TANGUT COMPONENT-593

18A51
TANGUT COMPONENT-594

18A52
TANGUT COMPONENT-595

18A53
TANGUT COMPONENT-596

18A54
TANGUT COMPONENT-597

18A55
TANGUT COMPONENT-598

18A56
TANGUT COMPONENT-599

18A57
TANGUT COMPONENT-600

18A58
TANGUT COMPONENT-601

18A59
TANGUT COMPONENT-602

18A5A
TANGUT COMPONENT-603

18A5B
TANGUT COMPONENT-604

18A5C
TANGUT COMPONENT-605

18A5D
TANGUT COMPONENT-606

18A5E
TANGUT COMPONENT-607

18A5F
TANGUT COMPONENT-608

18A60
TANGUT COMPONENT-609

18A61
TANGUT COMPONENT-610

18A62
TANGUT COMPONENT-611

18A63
TANGUT COMPONENT-612

18A64
TANGUT COMPONENT-613

18A65
TANGUT COMPONENT-614

18A66
TANGUT COMPONENT-615

18A67
TANGUT COMPONENT-616

18A68
TANGUT COMPONENT-617

18A69
TANGUT COMPONENT-618

18A6A
TANGUT COMPONENT-619

18A6B
TANGUT COMPONENT-620

18A6C
TANGUT COMPONENT-621

18A6D
TANGUT COMPONENT-622

18A6E
TANGUT COMPONENT-623

18A6F
TANGUT COMPONENT-624

18A70
TANGUT COMPONENT-625

18A71
TANGUT COMPONENT-626

18A72
TANGUT COMPONENT-627

18A73
TANGUT COMPONENT-628

18A74
TANGUT COMPONENT-629

18A75
TANGUT COMPONENT-630

18A76
TANGUT COMPONENT-631

18A77
TANGUT COMPONENT-632

18A78
TANGUT COMPONENT-633

18A79
TANGUT COMPONENT-634

18A7A
TANGUT COMPONENT-635

18A7B
TANGUT COMPONENT-636

18A7C
TANGUT COMPONENT-637

18A7D
TANGUT COMPONENT-638

18A7E
TANGUT COMPONENT-639

18A7F
TANGUT COMPONENT-640

18A80
TANGUT COMPONENT-641

18A81
TANGUT COMPONENT-642

18A82
TANGUT COMPONENT-643

18A83
TANGUT COMPONENT-644

18A84
TANGUT COMPONENT-645

18A85
TANGUT COMPONENT-646

18A86
TANGUT COMPONENT-647

18A87
TANGUT COMPONENT-648

18A88
TANGUT COMPONENT-649

18A89
TANGUT COMPONENT-650

18A8A
TANGUT COMPONENT-651

18A8B
TANGUT COMPONENT-652

18A8C
TANGUT COMPONENT-653

18A8D
TANGUT COMPONENT-654

18A8E
TANGUT COMPONENT-655

18A8F
TANGUT COMPONENT-656

18A90
TANGUT COMPONENT-657

18A91
TANGUT COMPONENT-658

18A92
TANGUT COMPONENT-659

18A93
TANGUT COMPONENT-660

18A94
TANGUT COMPONENT-661

18A95
TANGUT COMPONENT-662

18A96
TANGUT COMPONENT-663

18A97
TANGUT COMPONENT-664

18A98
TANGUT COMPONENT-665

18A99
TANGUT COMPONENT-666

18A9A
TANGUT COMPONENT-667

18A9B
TANGUT COMPONENT-668

18A9C
TANGUT COMPONENT-669

18A9D
TANGUT COMPONENT-670

18A9E
TANGUT COMPONENT-671

18A9F
TANGUT COMPONENT-672

18AA0
TANGUT COMPONENT-673

18AA1
TANGUT COMPONENT-674

18AA2
TANGUT COMPONENT-675

18AA3
TANGUT COMPONENT-676

18AA4
TANGUT COMPONENT-677

18AA5
TANGUT COMPONENT-678

18AA6
TANGUT COMPONENT-679

18AA7
TANGUT COMPONENT-680

18AA8
TANGUT COMPONENT-681

18AA9
TANGUT COMPONENT-682

18AAA
TANGUT COMPONENT-683

18AAB
TANGUT COMPONENT-684

18AAC
TANGUT COMPONENT-685

18AAD
TANGUT COMPONENT-686

18AAE
TANGUT COMPONENT-687

18AAF
TANGUT COMPONENT-688

18AB0
TANGUT COMPONENT-689

18AB1
TANGUT COMPONENT-690

18AB2
TANGUT COMPONENT-691

18AB3
TANGUT COMPONENT-692

18AB4
TANGUT COMPONENT-693

18AB5
TANGUT COMPONENT-694

18AB6
TANGUT COMPONENT-695

18AB7
TANGUT COMPONENT-696

18AB8
TANGUT COMPONENT-697

18AB9
TANGUT COMPONENT-698

18ABA
TANGUT COMPONENT-699

18ABB
TANGUT COMPONENT-700

18ABC
TANGUT COMPONENT-701

18ABD
TANGUT COMPONENT-702

18ABE
TANGUT COMPONENT-703

18ABF
TANGUT COMPONENT-704

18AC0
TANGUT COMPONENT-705

18AC1
TANGUT COMPONENT-706

18AC2
TANGUT COMPONENT-707

18AC3
TANGUT COMPONENT-708

18AC4
TANGUT COMPONENT-709

18AC5
TANGUT COMPONENT-710

18AC6
TANGUT COMPONENT-711

18AC7
TANGUT COMPONENT-712

18AC8
TANGUT COMPONENT-713

18AC9
TANGUT COMPONENT-714

18ACA
TANGUT COMPONENT-715

18ACB
TANGUT COMPONENT-716

18ACC
TANGUT COMPONENT-717

18ACD
TANGUT COMPONENT-718

18ACE
TANGUT COMPONENT-719

18ACF
TANGUT COMPONENT-720

18AD0
TANGUT COMPONENT-721

18AD1
TANGUT COMPONENT-722

18AD2
TANGUT COMPONENT-723

18AD3
TANGUT COMPONENT-724

18AD4
TANGUT COMPONENT-725

18AD5
TANGUT COMPONENT-726

18AD6
TANGUT COMPONENT-727

18AD7
TANGUT COMPONENT-728

18AD8
TANGUT COMPONENT-729

18AD9
TANGUT COMPONENT-730

18ADA
TANGUT COMPONENT-731

18ADB
TANGUT COMPONENT-732

18ADC
TANGUT COMPONENT-733

18ADD
TANGUT COMPONENT-734

18ADE
TANGUT COMPONENT-735

18ADF
TANGUT COMPONENT-736

18AE0
TANGUT COMPONENT-737

18AE1
TANGUT COMPONENT-738

18AE2
TANGUT COMPONENT-739

18AE3
TANGUT COMPONENT-740

18AE4
TANGUT COMPONENT-741

18AE5
TANGUT COMPONENT-742

18AE6
TANGUT COMPONENT-743

18AE7
TANGUT COMPONENT-744

18AE8
TANGUT COMPONENT-745

18AE9
TANGUT COMPONENT-746

18AEA
TANGUT COMPONENT-747

18AEB
TANGUT COMPONENT-748

18AEC
TANGUT COMPONENT-749

18AED
TANGUT COMPONENT-750

18AEE
TANGUT COMPONENT-751

18AEF
TANGUT COMPONENT-752

18AF0
TANGUT COMPONENT-753

18AF1
TANGUT COMPONENT-754

18AF2
TANGUT COMPONENT-755

18AF3
TANGUT COMPONENT-756

18AF4
TANGUT COMPONENT-757

18AF5
TANGUT COMPONENT-758

18AF6
TANGUT COMPONENT-759

18AF7
TANGUT COMPONENT-760

18AF8
TANGUT COMPONENT-761

18AF9
TANGUT COMPONENT-762

18AFA
TANGUT COMPONENT-763

18AFB
TANGUT COMPONENT-764

18AFC
TANGUT COMPONENT-765

18AFD
TANGUT COMPONENT-766

18AFE
TANGUT COMPONENT-767

18AFF
TANGUT COMPONENT-768

18D80
TANGUT COMPONENT-769

18D81
TANGUT COMPONENT-770

18D82
TANGUT COMPONENT-771

18D83
TANGUT COMPONENT-772

18D84
TANGUT COMPONENT-773

18D85
TANGUT COMPONENT-774

18D86
TANGUT COMPONENT-775

18D87
TANGUT COMPONENT-776

18D88
TANGUT COMPONENT-777

18D89
TANGUT COMPONENT-778

18D8A
TANGUT COMPONENT-779

18D8B
TANGUT COMPONENT-780

18D8C
TANGUT COMPONENT-781

18D8D
TANGUT COMPONENT-782

18D8E
TANGUT COMPONENT-783

18D8F
TANGUT COMPONENT-784

18D90
TANGUT COMPONENT-785

18D91
TANGUT COMPONENT-786

18D92
TANGUT COMPONENT-787

18D93
TANGUT COMPONENT-788

18D94
TANGUT COMPONENT-789

18D95
TANGUT COMPONENT-790

18D96
TANGUT COMPONENT-791

18D97
TANGUT COMPONENT-792

18D98
TANGUT COMPONENT-793

18D99
TANGUT COMPONENT-794

18D9A
TANGUT COMPONENT-795

18D9B
TANGUT COMPONENT-796

18D9C
TANGUT COMPONENT-797

18D9D
TANGUT COMPONENT-798

18D9E
TANGUT COMPONENT-799

18D9F
TANGUT COMPONENT-800

18DA0
TANGUT COMPONENT-801

18DA1
TANGUT COMPONENT-802

18DA2
TANGUT COMPONENT-803

18DA3
TANGUT COMPONENT-804

18DA4
TANGUT COMPONENT-805

18DA5
TANGUT COMPONENT-806

18DA6
TANGUT COMPONENT-807

18DA7
TANGUT COMPONENT-808

18DA8
TANGUT COMPONENT-809

18DA9
TANGUT COMPONENT-810

18DAA
TANGUT COMPONENT-811

18DAB
TANGUT COMPONENT-812

18DAC
TANGUT COMPONENT-813

18DAD
TANGUT COMPONENT-814

18DAE
TANGUT COMPONENT-815

18DAF
TANGUT COMPONENT-816

18DB0
TANGUT COMPONENT-817

18DB1
TANGUT COMPONENT-818

18DB2
TANGUT COMPONENT-819

18DB3
TANGUT COMPONENT-820

18DB4
TANGUT COMPONENT-821

18DB5
TANGUT COMPONENT-822

18DB6
TANGUT COMPONENT-823

18DB7
TANGUT COMPONENT-824

18DB8
TANGUT COMPONENT-825

18DB9
TANGUT COMPONENT-826

18DBA
TANGUT COMPONENT-827

18DBB
TANGUT COMPONENT-828

18DBC
TANGUT COMPONENT-829

18DBD
TANGUT COMPONENT-830

18DBE
TANGUT COMPONENT-831

18DBF
TANGUT COMPONENT-832

18DC0
TANGUT COMPONENT-833

18DC1
TANGUT COMPONENT-834

18DC2
TANGUT COMPONENT-835

18DC3
TANGUT COMPONENT-836

18DC4
TANGUT COMPONENT-837

18DC5
TANGUT COMPONENT-838

18DC6
TANGUT COMPONENT-839

18DC7
TANGUT COMPONENT-840

18DC8
TANGUT COMPONENT-841

18DC9
TANGUT COMPONENT-842

18DCA
TANGUT COMPONENT-843

18DCB
TANGUT COMPONENT-844

18DCC
TANGUT COMPONENT-845

18DCD
TANGUT COMPONENT-846

18DCE
TANGUT COMPONENT-847

18DCF
TANGUT COMPONENT-848

18DD0
TANGUT COMPONENT-849

18DD1
TANGUT COMPONENT-850

18DD2
TANGUT COMPONENT-851

18DD3
TANGUT COMPONENT-852

18DD4
TANGUT COMPONENT-853

18DD5
TANGUT COMPONENT-854

18DD6
TANGUT COMPONENT-855

18DD7
TANGUT COMPONENT-856

18DD8
TANGUT COMPONENT-857

18DD9
TANGUT COMPONENT-858

18DDA
TANGUT COMPONENT-859

18DDB
TANGUT COMPONENT-860

18DDC
TANGUT COMPONENT-861

18DDD
TANGUT COMPONENT-862

18DDE
TANGUT COMPONENT-863

18DDF
TANGUT COMPONENT-864

18DE0
TANGUT COMPONENT-865

18DE1
TANGUT COMPONENT-866

18DE2
TANGUT COMPONENT-867

18DE3
TANGUT COMPONENT-868

18DE4
TANGUT COMPONENT-869

18DE5
TANGUT COMPONENT-870

18DE6
TANGUT COMPONENT-871

18DE7
TANGUT COMPONENT-872

18DE8
TANGUT COMPONENT-873

18DE9
TANGUT COMPONENT-874

18DEA
TANGUT COMPONENT-875

18DEB
TANGUT COMPONENT-876

18DEC
TANGUT COMPONENT-877

18DED
TANGUT COMPONENT-878

18DEE
TANGUT COMPONENT-879

18DEF
TANGUT COMPONENT-880

18DF0
TANGUT COMPONENT-881

18DF1
TANGUT COMPONENT-882

18DF2
TANGUT COMPONENT-883

1AFF0
KATAKANA LETTER MINNAN TONE-2

1AFF1
KATAKANA LETTER MINNAN TONE-3

1AFF2
KATAKANA LETTER MINNAN TONE-4

1AFF3
KATAKANA LETTER MINNAN TONE-5

1AFF5
KATAKANA LETTER MINNAN TONE-7

1AFF6
KATAKANA LETTER MINNAN TONE-8

1AFF7
KATAKANA LETTER MINNAN NASALIZED TONE-1

1AFF8
KATAKANA LETTER MINNAN NASALIZED TONE-2

1AFF9
KATAKANA LETTER MINNAN NASALIZED TONE-3

1AFFA
KATAKANA LETTER MINNAN NASALIZED TONE-4

1AFFB
KATAKANA LETTER MINNAN NASALIZED TONE-5

1AFFD
KATAKANA LETTER MINNAN NASALIZED TONE-7

1AFFE
KATAKANA LETTER MINNAN NASALIZED TONE-8

1B000
KATAKANA LETTER ARCHAIC E

1B001
HENTAIGANA LETTER E-1

1B001
HIRAGANA LETTER ARCHAIC YE

1B002
HENTAIGANA LETTER A-1

1B003
HENTAIGANA LETTER A-2

1B004
HENTAIGANA LETTER A-3

1B005
HENTAIGANA LETTER A-WO

1B006
HENTAIGANA LETTER I-1

1B007
HENTAIGANA LETTER I-2

1B008
HENTAIGANA LETTER I-3

1B009
HENTAIGANA LETTER I-4

1B00A
HENTAIGANA LETTER U-1

1B00B
HENTAIGANA LETTER U-2

1B00C
HENTAIGANA LETTER U-3

1B00D
HENTAIGANA LETTER U-4

1B00E
HENTAIGANA LETTER U-5

1B00F
HENTAIGANA LETTER E-2

1B010
HENTAIGANA LETTER E-3

1B011
HENTAIGANA LETTER E-4

1B012
HENTAIGANA LETTER E-5

1B013
HENTAIGANA LETTER E-6

1B014
HENTAIGANA LETTER O-1

1B015
HENTAIGANA LETTER O-2

1B016
HENTAIGANA LETTER O-3

1B017
HENTAIGANA LETTER KA-1

1B018
HENTAIGANA LETTER KA-2

1B019
HENTAIGANA LETTER KA-3

1B01A
HENTAIGANA LETTER KA-4

1B01B
HENTAIGANA LETTER KA-5

1B01C
HENTAIGANA LETTER KA-6

1B01D
HENTAIGANA LETTER KA-7

1B01E
HENTAIGANA LETTER KA-8

1B01F
HENTAIGANA LETTER KA-9

1B020
HENTAIGANA LETTER KA-10

1B021
HENTAIGANA LETTER KA-11

1B022
HENTAIGANA LETTER KA-KE

1B023
HENTAIGANA LETTER KI-1

1B024
HENTAIGANA LETTER KI-2

1B025
HENTAIGANA LETTER KI-3

1B026
HENTAIGANA LETTER KI-4

1B027
HENTAIGANA LETTER KI-5

1B028
HENTAIGANA LETTER KI-6

1B029
HENTAIGANA LETTER KI-7

1B02A
HENTAIGANA LETTER KI-8

1B02B
HENTAIGANA LETTER KU-1

1B02C
HENTAIGANA LETTER KU-2

1B02D
HENTAIGANA LETTER KU-3

1B02E
HENTAIGANA LETTER KU-4

1B02F
HENTAIGANA LETTER KU-5

1B030
HENTAIGANA LETTER KU-6

1B031
HENTAIGANA LETTER KU-7

1B032
HENTAIGANA LETTER KE-1

1B033
HENTAIGANA LETTER KE-2

1B034
HENTAIGANA LETTER KE-3

1B035
HENTAIGANA LETTER KE-4

1B036
HENTAIGANA LETTER KE-5

1B037
HENTAIGANA LETTER KE-6

1B038
HENTAIGANA LETTER KO-1

1B039
HENTAIGANA LETTER KO-2

1B03A
HENTAIGANA LETTER KO-3

1B03B
HENTAIGANA LETTER KO-KI

1B03C
HENTAIGANA LETTER SA-1

1B03D
HENTAIGANA LETTER SA-2

1B03E
HENTAIGANA LETTER SA-3

1B03F
HENTAIGANA LETTER SA-4

1B040
HENTAIGANA LETTER SA-5

1B041
HENTAIGANA LETTER SA-6

1B042
HENTAIGANA LETTER SA-7

1B043
HENTAIGANA LETTER SA-8

1B044
HENTAIGANA LETTER SI-1

1B045
HENTAIGANA LETTER SI-2

1B046
HENTAIGANA LETTER SI-3

1B047
HENTAIGANA LETTER SI-4

1B048
HENTAIGANA LETTER SI-5

1B049
HENTAIGANA LETTER SI-6

1B04A
HENTAIGANA LETTER SU-1

1B04B
HENTAIGANA LETTER SU-2

1B04C
HENTAIGANA LETTER SU-3

1B04D
HENTAIGANA LETTER SU-4

1B04E
HENTAIGANA LETTER SU-5

1B04F
HENTAIGANA LETTER SU-6

1B050
HENTAIGANA LETTER SU-7

1B051
HENTAIGANA LETTER SU-8

1B052
HENTAIGANA LETTER SE-1

1B053
HENTAIGANA LETTER SE-2

1B054
HENTAIGANA LETTER SE-3

1B055
HENTAIGANA LETTER SE-4

1B056
HENTAIGANA LETTER SE-5

1B057
HENTAIGANA LETTER SO-1

1B058
HENTAIGANA LETTER SO-2

1B059
HENTAIGANA LETTER SO-3

1B05A
HENTAIGANA LETTER SO-4

1B05B
HENTAIGANA LETTER SO-5

1B05C
HENTAIGANA LETTER SO-6

1B05D
HENTAIGANA LETTER SO-7

1B05E
HENTAIGANA LETTER TA-1

1B05F
HENTAIGANA LETTER TA-2

1B060
HENTAIGANA LETTER TA-3

1B061
HENTAIGANA LETTER TA-4

1B062
HENTAIGANA LETTER TI-1

1B063
HENTAIGANA LETTER TI-2

1B064
HENTAIGANA LETTER TI-3

1B065
HENTAIGANA LETTER TI-4

1B066
HENTAIGANA LETTER TI-5

1B067
HENTAIGANA LETTER TI-6

1B068
HENTAIGANA LETTER TI-7

1B069
HENTAIGANA LETTER TU-1

1B06A
HENTAIGANA LETTER TU-2

1B06B
HENTAIGANA LETTER TU-3

1B06C
HENTAIGANA LETTER TU-4

1B06D
HENTAIGANA LETTER TU-TO

1B06E
HENTAIGANA LETTER TE-1

1B06F
HENTAIGANA LETTER TE-2

1B070
HENTAIGANA LETTER TE-3

1B071
HENTAIGANA LETTER TE-4

1B072
HENTAIGANA LETTER TE-5

1B073
HENTAIGANA LETTER TE-6

1B074
HENTAIGANA LETTER TE-7

1B075
HENTAIGANA LETTER TE-8

1B076
HENTAIGANA LETTER TE-9

1B077
HENTAIGANA LETTER TO-1

1B078
HENTAIGANA LETTER TO-2

1B079
HENTAIGANA LETTER TO-3

1B07A
HENTAIGANA LETTER TO-4

1B07B
HENTAIGANA LETTER TO-5

1B07C
HENTAIGANA LETTER TO-6

1B07D
HENTAIGANA LETTER TO-RA

1B07E
HENTAIGANA LETTER NA-1

1B07F
HENTAIGANA LETTER NA-2

1B080
HENTAIGANA LETTER NA-3

1B081
HENTAIGANA LETTER NA-4

1B082
HENTAIGANA LETTER NA-5

1B083
HENTAIGANA LETTER NA-6

1B084
HENTAIGANA LETTER NA-7

1B085
HENTAIGANA LETTER NA-8

1B086
HENTAIGANA LETTER NA-9

1B087
HENTAIGANA LETTER NI-1

1B088
HENTAIGANA LETTER NI-2

1B089
HENTAIGANA LETTER NI-3

1B08A
HENTAIGANA LETTER NI-4

1B08B
HENTAIGANA LETTER NI-5

1B08C
HENTAIGANA LETTER NI-6

1B08D
HENTAIGANA LETTER NI-7

1B08E
HENTAIGANA LETTER NI-TE

1B08F
HENTAIGANA LETTER NU-1

1B090
HENTAIGANA LETTER NU-2

1B091
HENTAIGANA LETTER NU-3

1B092
HENTAIGANA LETTER NE-1

1B093
HENTAIGANA LETTER NE-2

1B094
HENTAIGANA LETTER NE-3

1B095
HENTAIGANA LETTER NE-4

1B096
HENTAIGANA LETTER NE-5

1B097
HENTAIGANA LETTER NE-6

1B098
HENTAIGANA LETTER NE-KO

1B099
HENTAIGANA LETTER NO-1

1B09A
HENTAIGANA LETTER NO-2

1B09B
HENTAIGANA LETTER NO-3

1B09C
HENTAIGANA LETTER NO-4

1B09D
HENTAIGANA LETTER NO-5

1B09E
HENTAIGANA LETTER HA-1

1B09F
HENTAIGANA LETTER HA-2

1B0A0
HENTAIGANA LETTER HA-3

1B0A1
HENTAIGANA LETTER HA-4

1B0A2
HENTAIGANA LETTER HA-5

1B0A3
HENTAIGANA LETTER HA-6

1B0A4
HENTAIGANA LETTER HA-7

1B0A5
HENTAIGANA LETTER HA-8

1B0A6
HENTAIGANA LETTER HA-9

1B0A7
HENTAIGANA LETTER HA-10

1B0A8
HENTAIGANA LETTER HA-11

1B0A9
HENTAIGANA LETTER HI-1

1B0AA
HENTAIGANA LETTER HI-2

1B0AB
HENTAIGANA LETTER HI-3

1B0AC
HENTAIGANA LETTER HI-4

1B0AD
HENTAIGANA LETTER HI-5

1B0AE
HENTAIGANA LETTER HI-6

1B0AF
HENTAIGANA LETTER HI-7

1B0B0
HENTAIGANA LETTER HU-1

1B0B1
HENTAIGANA LETTER HU-2

1B0B2
HENTAIGANA LETTER HU-3

1B0B3
HENTAIGANA LETTER HE-1

1B0B4
HENTAIGANA LETTER HE-2

1B0B5
HENTAIGANA LETTER HE-3

1B0B6
HENTAIGANA LETTER HE-4

1B0B7
HENTAIGANA LETTER HE-5

1B0B8
HENTAIGANA LETTER HE-6

1B0B9
HENTAIGANA LETTER HE-7

1B0BA
HENTAIGANA LETTER HO-1

1B0BB
HENTAIGANA LETTER HO-2

1B0BC
HENTAIGANA LETTER HO-3

1B0BD
HENTAIGANA LETTER HO-4

1B0BE
HENTAIGANA LETTER HO-5

1B0BF
HENTAIGANA LETTER HO-6

1B0C0
HENTAIGANA LETTER HO-7

1B0C1
HENTAIGANA LETTER HO-8

1B0C2
HENTAIGANA LETTER MA-1

1B0C3
HENTAIGANA LETTER MA-2

1B0C4
HENTAIGANA LETTER MA-3

1B0C5
HENTAIGANA LETTER MA-4

1B0C6
HENTAIGANA LETTER MA-5

1B0C7
HENTAIGANA LETTER MA-6

1B0C8
HENTAIGANA LETTER MA-7

1B0C9
HENTAIGANA LETTER MI-1

1B0CA
HENTAIGANA LETTER MI-2

1B0CB
HENTAIGANA LETTER MI-3

1B0CC
HENTAIGANA LETTER MI-4

1B0CD
HENTAIGANA LETTER MI-5

1B0CE
HENTAIGANA LETTER MI-6

1B0CF
HENTAIGANA LETTER MI-7

1B0D0
HENTAIGANA LETTER MU-1

1B0D1
HENTAIGANA LETTER MU-2

1B0D2
HENTAIGANA LETTER MU-3

1B0D3
HENTAIGANA LETTER MU-4

1B0D4
HENTAIGANA LETTER ME-1

1B0D5
HENTAIGANA LETTER ME-2

1B0D6
HENTAIGANA LETTER ME-MA

1B0D7
HENTAIGANA LETTER MO-1

1B0D8
HENTAIGANA LETTER MO-2

1B0D9
HENTAIGANA LETTER MO-3

1B0DA
HENTAIGANA LETTER MO-4

1B0DB
HENTAIGANA LETTER MO-5

1B0DC
HENTAIGANA LETTER MO-6

1B0DD
HENTAIGANA LETTER YA-1

1B0DE
HENTAIGANA LETTER YA-2

1B0DF
HENTAIGANA LETTER YA-3

1B0E0
HENTAIGANA LETTER YA-4

1B0E1
HENTAIGANA LETTER YA-5

1B0E2
HENTAIGANA LETTER YA-YO

1B0E3
HENTAIGANA LETTER YU-1

1B0E4
HENTAIGANA LETTER YU-2

1B0E5
HENTAIGANA LETTER YU-3

1B0E6
HENTAIGANA LETTER YU-4

1B0E7
HENTAIGANA LETTER YO-1

1B0E8
HENTAIGANA LETTER YO-2

1B0E9
HENTAIGANA LETTER YO-3

1B0EA
HENTAIGANA LETTER YO-4

1B0EB
HENTAIGANA LETTER YO-5

1B0EC
HENTAIGANA LETTER YO-6

1B0ED
HENTAIGANA LETTER RA-1

1B0EE
HENTAIGANA LETTER RA-2

1B0EF
HENTAIGANA LETTER RA-3

1B0F0
HENTAIGANA LETTER RA-4

1B0F1
HENTAIGANA LETTER RI-1

1B0F2
HENTAIGANA LETTER RI-2

1B0F3
HENTAIGANA LETTER RI-3

1B0F4
HENTAIGANA LETTER RI-4

1B0F5
HENTAIGANA LETTER RI-5

1B0F6
HENTAIGANA LETTER RI-6

1B0F7
HENTAIGANA LETTER RI-7

1B0F8
HENTAIGANA LETTER RU-1

1B0F9
HENTAIGANA LETTER RU-2

1B0FA
HENTAIGANA LETTER RU-3

1B0FB
HENTAIGANA LETTER RU-4

1B0FC
HENTAIGANA LETTER RU-5

1B0FD
HENTAIGANA LETTER RU-6

1B0FE
HENTAIGANA LETTER RE-1

1B0FF
HENTAIGANA LETTER RE-2

1B100
HENTAIGANA LETTER RE-3

1B101
HENTAIGANA LETTER RE-4

1B102
HENTAIGANA LETTER RO-1

1B103
HENTAIGANA LETTER RO-2

1B104
HENTAIGANA LETTER RO-3

1B105
HENTAIGANA LETTER RO-4

1B106
HENTAIGANA LETTER RO-5

1B107
HENTAIGANA LETTER RO-6

1B108
HENTAIGANA LETTER WA-1

1B109
HENTAIGANA LETTER WA-2

1B10A
HENTAIGANA LETTER WA-3

1B10B
HENTAIGANA LETTER WA-4

1B10C
HENTAIGANA LETTER WA-5

1B10D
HENTAIGANA LETTER WI-1

1B10E
HENTAIGANA LETTER WI-2

1B10F
HENTAIGANA LETTER WI-3

1B110
HENTAIGANA LETTER WI-4

1B111
HENTAIGANA LETTER WI-5

1B112
HENTAIGANA LETTER WE-1

1B113
HENTAIGANA LETTER WE-2

1B114
HENTAIGANA LETTER WE-3

1B115
HENTAIGANA LETTER WE-4

1B116
HENTAIGANA LETTER WO-1

1B117
HENTAIGANA LETTER WO-2

1B118
HENTAIGANA LETTER WO-3

1B119
HENTAIGANA LETTER WO-4

1B11A
HENTAIGANA LETTER WO-5

1B11B
HENTAIGANA LETTER WO-6

1B11C
HENTAIGANA LETTER WO-7

1B11D
HENTAIGANA LETTER N-MU-MO-1

1B11E
HENTAIGANA LETTER N-MU-MO-2

1B11F
HIRAGANA LETTER ARCHAIC WU

1B120
KATAKANA LETTER ARCHAIC YI

1B121
KATAKANA LETTER ARCHAIC YE

1B122
KATAKANA LETTER ARCHAIC WU

1B132
HIRAGANA LETTER SMALL KO

1B150
HIRAGANA LETTER SMALL WI

1B151
HIRAGANA LETTER SMALL WE

1B152
HIRAGANA LETTER SMALL WO

1B155
KATAKANA LETTER SMALL KO

1B164
KATAKANA LETTER SMALL WI

1B165
KATAKANA LETTER SMALL WE

1B166
KATAKANA LETTER SMALL WO

1B167
KATAKANA LETTER SMALL N

1BC00
DUPLOYAN LETTER H

1BC01
DUPLOYAN LETTER X

1BC02
DUPLOYAN LETTER P

1BC03
DUPLOYAN LETTER T

1BC04
DUPLOYAN LETTER F

1BC05
DUPLOYAN LETTER K

1BC06
DUPLOYAN LETTER L

1BC07
DUPLOYAN LETTER B

1BC08
DUPLOYAN LETTER D

1BC09
DUPLOYAN LETTER V

1BC0A
DUPLOYAN LETTER G

1BC0B
DUPLOYAN LETTER R

1BC0C
DUPLOYAN LETTER P N

1BC0D
DUPLOYAN LETTER D S

1BC0E
DUPLOYAN LETTER F N

1BC0F
DUPLOYAN LETTER K M

1BC10
DUPLOYAN LETTER R S

1BC11
DUPLOYAN LETTER TH

1BC12
DUPLOYAN LETTER SLOAN DH

1BC13
DUPLOYAN LETTER DH

1BC14
DUPLOYAN LETTER KK

1BC15
DUPLOYAN LETTER SLOAN J

1BC16
DUPLOYAN LETTER HL

1BC17
DUPLOYAN LETTER LH

1BC18
DUPLOYAN LETTER RH

1BC19
DUPLOYAN LETTER M

1BC1A
DUPLOYAN LETTER N

1BC1B
DUPLOYAN LETTER J

1BC1C
DUPLOYAN LETTER S

1BC1D
DUPLOYAN LETTER M N

1BC1E
DUPLOYAN LETTER N M

1BC1F
DUPLOYAN LETTER J M

1BC20
DUPLOYAN LETTER S J

1BC21
DUPLOYAN LETTER M WITH DOT

1BC22
DUPLOYAN LETTER N WITH DOT

1BC23
DUPLOYAN LETTER J WITH DOT

1BC24
DUPLOYAN LETTER J WITH DOTS INSIDE AND ABOVE

1BC25
DUPLOYAN LETTER S WITH DOT

1BC26
DUPLOYAN LETTER S WITH DOT BELOW

1BC27
DUPLOYAN LETTER M S

1BC28
DUPLOYAN LETTER N S

1BC29
DUPLOYAN LETTER J S

1BC2A
DUPLOYAN LETTER S S

1BC2B
DUPLOYAN LETTER M N S

1BC2C
DUPLOYAN LETTER N M S

1BC2D
DUPLOYAN LETTER J M S

1BC2E
DUPLOYAN LETTER S J S

1BC2F
DUPLOYAN LETTER J S WITH DOT

1BC30
DUPLOYAN LETTER J N

1BC31
DUPLOYAN LETTER J N S

1BC32
DUPLOYAN LETTER S T

1BC33
DUPLOYAN LETTER S T R

1BC34
DUPLOYAN LETTER S P

1BC35
DUPLOYAN LETTER S P R

1BC36
DUPLOYAN LETTER T S

1BC37
DUPLOYAN LETTER T R S

1BC38
DUPLOYAN LETTER W

1BC39
DUPLOYAN LETTER WH

1BC3A
DUPLOYAN LETTER W R

1BC3B
DUPLOYAN LETTER S N

1BC3C
DUPLOYAN LETTER S M

1BC3D
DUPLOYAN LETTER K R S

1BC3E
DUPLOYAN LETTER G R S

1BC3F
DUPLOYAN LETTER S K

1BC40
DUPLOYAN LETTER S K R

1BC41
DUPLOYAN LETTER A

1BC42
DUPLOYAN LETTER SLOAN OW

1BC43
DUPLOYAN LETTER OA

1BC44
DUPLOYAN LETTER O

1BC45
DUPLOYAN LETTER AOU

1BC46
DUPLOYAN LETTER I

1BC47
DUPLOYAN LETTER E

1BC48
DUPLOYAN LETTER IE

1BC49
DUPLOYAN LETTER SHORT I

1BC4A
DUPLOYAN LETTER UI

1BC4B
DUPLOYAN LETTER EE

1BC4C
DUPLOYAN LETTER SLOAN EH

1BC4D
DUPLOYAN LETTER ROMANIAN I

1BC4E
DUPLOYAN LETTER SLOAN EE

1BC4F
DUPLOYAN LETTER LONG I

1BC50
DUPLOYAN LETTER YE

1BC51
DUPLOYAN LETTER U

1BC52
DUPLOYAN LETTER EU

1BC53
DUPLOYAN LETTER XW

1BC54
DUPLOYAN LETTER U N

1BC55
DUPLOYAN LETTER LONG U

1BC56
DUPLOYAN LETTER ROMANIAN U

1BC57
DUPLOYAN LETTER UH

1BC58
DUPLOYAN LETTER SLOAN U

1BC59
DUPLOYAN LETTER OOH

1BC5A
DUPLOYAN LETTER OW

1BC5B
DUPLOYAN LETTER OU

1BC5C
DUPLOYAN LETTER WA

1BC5D
DUPLOYAN LETTER WO

1BC5E
DUPLOYAN LETTER WI

1BC5F
DUPLOYAN LETTER WEI

1BC60
DUPLOYAN LETTER WOW

1BC61
DUPLOYAN LETTER NASAL U

1BC62
DUPLOYAN LETTER NASAL O

1BC63
DUPLOYAN LETTER NASAL I

1BC64
DUPLOYAN LETTER NASAL A

1BC65
DUPLOYAN LETTER PERNIN AN

1BC66
DUPLOYAN LETTER PERNIN AM

1BC67
DUPLOYAN LETTER SLOAN EN

1BC68
DUPLOYAN LETTER SLOAN AN

1BC69
DUPLOYAN LETTER SLOAN ON

1BC6A
DUPLOYAN LETTER VOCALIC M

1BC70
DUPLOYAN AFFIX LEFT HORIZONTAL SECANT

1BC71
DUPLOYAN AFFIX MID HORIZONTAL SECANT

1BC72
DUPLOYAN AFFIX RIGHT HORIZONTAL SECANT

1BC73
DUPLOYAN AFFIX LOW VERTICAL SECANT

1BC74
DUPLOYAN AFFIX MID VERTICAL SECANT

1BC75
DUPLOYAN AFFIX HIGH VERTICAL SECANT

1BC76
DUPLOYAN AFFIX ATTACHED SECANT

1BC77
DUPLOYAN AFFIX ATTACHED LEFT-TO-RIGHT SECANT

1BC78
DUPLOYAN AFFIX ATTACHED TANGENT

1BC79
DUPLOYAN AFFIX ATTACHED TAIL

1BC7A
DUPLOYAN AFFIX ATTACHED E HOOK

1BC7B
DUPLOYAN AFFIX ATTACHED I HOOK

1BC7C
DUPLOYAN AFFIX ATTACHED TANGENT HOOK

1BC80
DUPLOYAN AFFIX HIGH ACUTE

1BC81
DUPLOYAN AFFIX HIGH TIGHT ACUTE

1BC82
DUPLOYAN AFFIX HIGH GRAVE

1BC83
DUPLOYAN AFFIX HIGH LONG GRAVE

1BC84
DUPLOYAN AFFIX HIGH DOT

1BC85
DUPLOYAN AFFIX HIGH CIRCLE

1BC86
DUPLOYAN AFFIX HIGH LINE

1BC87
DUPLOYAN AFFIX HIGH WAVE

1BC88
DUPLOYAN AFFIX HIGH VERTICAL

1BC90
DUPLOYAN AFFIX LOW ACUTE

1BC91
DUPLOYAN AFFIX LOW TIGHT ACUTE

1BC92
DUPLOYAN AFFIX LOW GRAVE

1BC93
DUPLOYAN AFFIX LOW LONG GRAVE

1BC94
DUPLOYAN AFFIX LOW DOT

1BC95
DUPLOYAN AFFIX LOW CIRCLE

1BC96
DUPLOYAN AFFIX LOW LINE

1BC97
DUPLOYAN AFFIX LOW WAVE

1BC98
DUPLOYAN AFFIX LOW VERTICAL

1BC99
DUPLOYAN AFFIX LOW ARROW

1BC9C
DUPLOYAN SIGN O WITH CROSS

1BC9D
DUPLOYAN THICK LETTER SELECTOR

1BC9E
DUPLOYAN DOUBLE MARK

1BC9F
DUPLOYAN PUNCTUATION CHINOOK FULL STOP

1BCA0
SHORTHAND FORMAT LETTER OVERLAP

1BCA1
SHORTHAND FORMAT CONTINUING OVERLAP

1BCA2
SHORTHAND FORMAT DOWN STEP

1BCA3
SHORTHAND FORMAT UP STEP

1CC00
UP-POINTING GO-KART

1CC01
RIGHT-POINTING GO-KART

1CC02
LEFT-POINTING STICK FIGURE

1CC03
RIGHT-POINTING STICK FIGURE

1CC04
DOWN-POINTING STICK FIGURE

1CC05
LOWER HORIZONTAL RULER SEGMENT

1CC06
RIGHT VERTICAL RULER SEGMENT

1CC07
LOWER RIGHT RULER SEGMENT

1CC08
ANTENNA

1CC09
HORIZONTAL RESISTOR SEGMENT

1CC0A
VERTICAL RESISTOR SEGMENT

1CC0B
LEFT THIRD INDUCTOR

1CC0C
MIDDLE THIRD INDUCTOR

1CC0D
RIGHT THIRD INDUCTOR

1CC0E
LEFT-POINTING DIODE

1CC0F
RIGHT-POINTING DIODE

1CC10
NPN TRANSISTOR

1CC11
PNP TRANSISTOR

1CC12
RECEPTACLE

1CC13
HORIZONTAL CAPACITOR

1CC14
VERTICAL CAPACITOR

1CC15
LOGIC GATE OR

1CC16
LOGIC GATE AND

1CC17
LOGIC GATE INVERTED INPUTS

1CC18
LOGIC GATE INVERTED OUTPUT

1CC19
LOGIC GATE BUFFER

1CC1A
LOGIC GATE BUFFER WITH INVERTED INPUT

1CC1B
BOX DRAWINGS LIGHT HORIZONTAL AND UPPER RIGHT

1CC1C
BOX DRAWINGS LIGHT HORIZONTAL AND LOWER RIGHT

1CC1D
BOX DRAWINGS LIGHT TOP AND UPPER LEFT

1CC1E
BOX DRAWINGS LIGHT BOTTOM AND LOWER LEFT

1CC1F
BOX DRAWINGS DOUBLE DIAGONAL UPPER RIGHT TO LOWER LEFT

1CC20
BOX DRAWINGS DOUBLE DIAGONAL UPPER LEFT TO LOWER RIGHT

1CC21
SEPARATED BLOCK QUADRANT-1

1CC22
SEPARATED BLOCK QUADRANT-2

1CC23
SEPARATED BLOCK QUADRANT-12

1CC24
SEPARATED BLOCK QUADRANT-3

1CC25
SEPARATED BLOCK QUADRANT-13

1CC26
SEPARATED BLOCK QUADRANT-23

1CC27
SEPARATED BLOCK QUADRANT-123

1CC28
SEPARATED BLOCK QUADRANT-4

1CC29
SEPARATED BLOCK QUADRANT-14

1CC2A
SEPARATED BLOCK QUADRANT-24

1CC2B
SEPARATED BLOCK QUADRANT-124

1CC2C
SEPARATED BLOCK QUADRANT-34

1CC2D
SEPARATED BLOCK QUADRANT-134

1CC2E
SEPARATED BLOCK QUADRANT-234

1CC2F
SEPARATED BLOCK QUADRANT-1234

1CC30
UPPER LEFT TWELFTH CIRCLE

1CC31
UPPER CENTRE LEFT TWELFTH CIRCLE

1CC32
UPPER CENTRE RIGHT TWELFTH CIRCLE

1CC33
UPPER RIGHT TWELFTH CIRCLE

1CC34
UPPER MIDDLE LEFT TWELFTH CIRCLE

1CC35
UPPER LEFT QUARTER CIRCLE

1CC36
UPPER RIGHT QUARTER CIRCLE

1CC37
UPPER MIDDLE RIGHT TWELFTH CIRCLE

1CC38
LOWER MIDDLE LEFT TWELFTH CIRCLE

1CC39
LOWER LEFT QUARTER CIRCLE

1CC3A
LOWER RIGHT QUARTER CIRCLE

1CC3B
LOWER MIDDLE RIGHT TWELFTH CIRCLE

1CC3C
LOWER LEFT TWELFTH CIRCLE

1CC3D
LOWER CENTRE LEFT TWELFTH CIRCLE

1CC3E
LOWER CENTRE RIGHT TWELFTH CIRCLE

1CC3F
LOWER RIGHT TWELFTH CIRCLE

1CC40
SPARSE HORIZONTAL FILL

1CC41
SPARSE VERTICAL FILL

1CC42
ORTHOGONAL CROSSHATCH FILL

1CC43
DIAGONAL CROSSHATCH FILL

1CC44
DENSE VERTICAL FILL

1CC45
DENSE HORIZONTAL FILL

1CC46
SPECKLE FILL FRAME-1

1CC47
SPECKLE FILL FRAME-2

1CC48
LEFT-FACING BASSINET

1CC49
RIGHT-FACING BASSINET

1CC4A
FLYING SAUCER WITH BEAMS

1CC4B
FLYING SAUCER WITHOUT BEAMS

1CC4C
ALIEN MONSTER OPEN JAWS

1CC4D
ALIEN MONSTER CLOSED JAWS

1CC4E
ALIEN SQUID OPEN TENTACLES

1CC4F
ALIEN SQUID CLOSED TENTACLES

1CC50
ALIEN CRAB STEPPING RIGHT

1CC51
ALIEN CRAB STEPPING LEFT

1CC52
ALIEN SPIDER CROUCHING

1CC53
ALIEN SPIDER SPREAD

1CC54
ALIEN MONSTER STEP-1

1CC55
ALIEN MONSTER STEP-2

1CC56
LEFT-POINTING ROCKET SHIP

1CC57
UP-POINTING ROCKET SHIP

1CC58
RIGHT-POINTING ROCKET SHIP

1CC59
DOWN-POINTING ROCKET SHIP

1CC5A
TOP HALF LEFT-FACING ROBOT

1CC5B
TOP HALF FORWARD-FACING ROBOT

1CC5C
TOP HALF RIGHT-FACING ROBOT

1CC5D
BOTTOM HALF LEFT-FACING ROBOT

1CC5E
BOTTOM HALF FORWARD-FACING ROBOT

1CC5F
BOTTOM HALF RIGHT-FACING ROBOT

1CC60
LEFT-POINTING ATOMIC BOMB

1CC61
UP-POINTING ATOMIC BOMB

1CC62
RIGHT-POINTING ATOMIC BOMB

1CC63
DOWN-POINTING ATOMIC BOMB

1CC64
MUSHROOM CLOUD

1CC65
LEFT-POINTING RIFLE

1CC66
UP-POINTING RIFLE

1CC67
RIGHT-POINTING RIFLE

1CC68
DOWN-POINTING RIFLE

1CC69
EIGHT RAYS INWARD

1CC6A
EIGHT RAYS OUTWARD

1CC6B
BLACK LARGE CIRCLE MINUS LEFT QUARTER SECTION

1CC6C
BLACK LARGE CIRCLE MINUS UPPER QUARTER SECTION

1CC6D
BLACK LARGE CIRCLE MINUS RIGHT QUARTER SECTION

1CC6E
BLACK LARGE CIRCLE MINUS LOWER QUARTER SECTION

1CC6F
BLACK NEUTRAL FACE

1CC70
LEFT-FACING SNAKE HEAD WITH OPEN MOUTH

1CC71
UP-FACING SNAKE HEAD WITH OPEN MOUTH

1CC72
RIGHT-FACING SNAKE HEAD WITH OPEN MOUTH

1CC73
DOWN-FACING SNAKE HEAD WITH OPEN MOUTH

1CC74
LEFT-FACING SNAKE HEAD WITH CLOSED MOUTH

1CC75
UP-FACING SNAKE HEAD WITH CLOSED MOUTH

1CC76
RIGHT-FACING SNAKE HEAD WITH CLOSED MOUTH

1CC77
DOWN-FACING SNAKE HEAD WITH CLOSED MOUTH

1CC78
LEFT-POINTING ENERGY WAVE

1CC79
UP-POINTING ENERGY WAVE

1CC7A
RIGHT-POINTING ENERGY WAVE

1CC7B
DOWN-POINTING ENERGY WAVE

1CC7C
SQUARE SPIRAL FROM TOP LEFT

1CC7D
SQUARE SPIRAL FROM TOP RIGHT

1CC7E
SQUARE SPIRAL FROM BOTTOM RIGHT

1CC7F
SQUARE SPIRAL FROM BOTTOM LEFT

1CC80
STRIPED LEFT-POINTING TRIANGLE

1CC81
STRIPED UP-POINTING TRIANGLE

1CC82
STRIPED RIGHT-POINTING TRIANGLE

1CC83
STRIPED DOWN-POINTING TRIANGLE

1CC84
VERTICAL LADDER

1CC85
HORIZONTAL LADDER

1CC86
WHITE LOWER LEFT POINTER

1CC87
WHITE LOWER RIGHT POINTER

1CC88
TWO RINGS ALIGNED HORIZONTALLY

1CC89
SQUARE FOUR CORNER SALTIRES

1CC8A
SQUARE FOUR CORNER DIAGONALS

1CC8B
SQUARE FOUR CORNER BLACK TRIANGLES

1CC8C
SQUARE APERTURE

1CC8D
INVERSE BLACK DIAMOND

1CC8E
LEFT AND UPPER ONE EIGHTH BLOCK CONTAINING BLACK SMALL SQUARE

1CC8F
INVERSE BLACK SMALL SQUARE

1CC90
VERTICAL LINE WITH FOUR TICK MARKS

1CC91
HORIZONTAL LINE WITH FOUR TICK MARKS

1CC92
LEFT-FACING FISH

1CC93
RIGHT-FACING FISH

1CC94
LEFT-FACING FISH WITH OPEN MOUTH

1CC95
RIGHT-FACING FISH WITH OPEN MOUTH

1CC96
FLAPPING BIRD

1CC97
LEFT-POINTING RACING CAR

1CC98
UP-POINTING RACING CAR

1CC99
RIGHT-POINTING RACING CAR

1CC9A
DOWN-POINTING RACING CAR

1CC9B
HORIZONTAL RACING CAR

1CC9C
VERTICAL RACING CAR

1CC9D
VERTICAL GO-KART

1CC9E
LEFT-POINTING TANK

1CC9F
RIGHT-POINTING TANK

1CCA0
LEFT-POINTING ROCKET BOOSTER

1CCA1
RIGHT-POINTING ROCKET BOOSTER

1CCA2
LEFT-POINTING ROLLER COASTER CAR

1CCA3
RIGHT-POINTING ROLLER COASTER CAR

1CCA4
LEFT HALF FLYING SAUCER

1CCA5
RIGHT HALF FLYING SAUCER

1CCA6
UPPER LEFT QUADRANT FACE WITH OPEN EYES

1CCA7
UPPER RIGHT QUADRANT FACE WITH OPEN EYES

1CCA8
UPPER LEFT QUADRANT FACE WITH CLOSED EYES

1CCA9
UPPER RIGHT QUADRANT FACE WITH CLOSED EYES

1CCAA
LOWER LEFT QUADRANT SMILING FACE

1CCAB
LOWER RIGHT QUADRANT SMILING FACE

1CCAC
LOWER LEFT QUADRANT NEUTRAL FACE

1CCAD
LOWER RIGHT QUADRANT NEUTRAL FACE

1CCAE
LOWER LEFT QUADRANT FACE WITH OPEN MOUTH

1CCAF
LOWER RIGHT QUADRANT FACE WITH OPEN MOUTH

1CCB0
LOWER LEFT QUADRANT FROWNING FACE

1CCB1
LOWER RIGHT QUADRANT FROWNING FACE

1CCB2
UPPER LEFT QUADRANT TELEVISION

1CCB3
UPPER RIGHT QUADRANT TELEVISION

1CCB4
LOWER LEFT QUADRANT TELEVISION

1CCB5
LOWER RIGHT QUADRANT TELEVISION

1CCB6
UPPER LEFT QUADRANT MICROCOMPUTER

1CCB7
UPPER RIGHT QUADRANT MICROCOMPUTER

1CCB8
LOWER LEFT QUADRANT MICROCOMPUTER

1CCB9
LOWER RIGHT QUADRANT MICROCOMPUTER

1CCBA
UPPER LEFT QUADRANT CHESS KING

1CCBB
UPPER RIGHT QUADRANT CHESS KING

1CCBC
LOWER LEFT QUADRANT CHESS KING

1CCBD
LOWER RIGHT QUADRANT CHESS KING

1CCBE
UPPER LEFT QUADRANT CHESS QUEEN

1CCBF
UPPER RIGHT QUADRANT CHESS QUEEN

1CCC0
LOWER LEFT QUADRANT CHESS QUEEN

1CCC1
LOWER RIGHT QUADRANT CHESS QUEEN

1CCC2
UPPER LEFT QUADRANT CHESS ROOK

1CCC3
UPPER RIGHT QUADRANT CHESS ROOK

1CCC4
LOWER LEFT QUADRANT CHESS ROOK

1CCC5
LOWER RIGHT QUADRANT CHESS ROOK

1CCC6
UPPER LEFT QUADRANT CHESS BISHOP

1CCC7
UPPER RIGHT QUADRANT CHESS BISHOP

1CCC8
LOWER LEFT QUADRANT CHESS BISHOP

1CCC9
LOWER RIGHT QUADRANT CHESS BISHOP

1CCCA
UPPER LEFT QUADRANT CHESS KNIGHT

1CCCB
UPPER RIGHT QUADRANT CHESS KNIGHT

1CCCC
LOWER LEFT QUADRANT CHESS KNIGHT

1CCCD
LOWER RIGHT QUADRANT CHESS KNIGHT

1CCCE
UPPER LEFT QUADRANT CHESS PAWN

1CCCF
UPPER RIGHT QUADRANT CHESS PAWN

1CCD0
LOWER LEFT QUADRANT CHESS PAWN

1CCD1
LOWER RIGHT QUADRANT CHESS PAWN

1CCD2
UPPER LEFT QUADRANT STANDING KNIGHT

1CCD3
UPPER RIGHT QUADRANT STANDING KNIGHT

1CCD4
LOWER LEFT QUADRANT STANDING KNIGHT

1CCD5
LOWER RIGHT QUADRANT STANDING KNIGHT

1CCD6
OUTLINED LATIN CAPITAL LETTER A

1CCD7
OUTLINED LATIN CAPITAL LETTER B

1CCD8
OUTLINED LATIN CAPITAL LETTER C

1CCD9
OUTLINED LATIN CAPITAL LETTER D

1CCDA
OUTLINED LATIN CAPITAL LETTER E

1CCDB
OUTLINED LATIN CAPITAL LETTER F

1CCDC
OUTLINED LATIN CAPITAL LETTER G

1CCDD
OUTLINED LATIN CAPITAL LETTER H

1CCDE
OUTLINED LATIN CAPITAL LETTER I

1CCDF
OUTLINED LATIN CAPITAL LETTER J

1CCE0
OUTLINED LATIN CAPITAL LETTER K

1CCE1
OUTLINED LATIN CAPITAL LETTER L

1CCE2
OUTLINED LATIN CAPITAL LETTER M

1CCE3
OUTLINED LATIN CAPITAL LETTER N

1CCE4
OUTLINED LATIN CAPITAL LETTER O

1CCE5
OUTLINED LATIN CAPITAL LETTER P

1CCE6
OUTLINED LATIN CAPITAL LETTER Q

1CCE7
OUTLINED LATIN CAPITAL LETTER R

1CCE8
OUTLINED LATIN CAPITAL LETTER S

1CCE9
OUTLINED LATIN CAPITAL LETTER T

1CCEA
OUTLINED LATIN CAPITAL LETTER U

1CCEB
OUTLINED LATIN CAPITAL LETTER V

1CCEC
OUTLINED LATIN CAPITAL LETTER W

1CCED
OUTLINED LATIN CAPITAL LETTER X

1CCEE
OUTLINED LATIN CAPITAL LETTER Y

1CCEF
OUTLINED LATIN CAPITAL LETTER Z

1CCF0
OUTLINED DIGIT ZERO

1CCF1
OUTLINED DIGIT ONE

1CCF2
OUTLINED DIGIT TWO

1CCF3
OUTLINED DIGIT THREE

1CCF4
OUTLINED DIGIT FOUR

1CCF5
OUTLINED DIGIT FIVE

1CCF6
OUTLINED DIGIT SIX

1CCF7
OUTLINED DIGIT SEVEN

1CCF8
OUTLINED DIGIT EIGHT

1CCF9
OUTLINED DIGIT NINE

1CCFA
SNAKE SYMBOL

1CCFB
FLYING SAUCER SYMBOL

1CCFC
NOSE SYMBOL

1CD00
BLOCK OCTANT-3

1CD01
BLOCK OCTANT-23

1CD02
BLOCK OCTANT-123

1CD03
BLOCK OCTANT-4

1CD04
BLOCK OCTANT-14

1CD05
BLOCK OCTANT-124

1CD06
BLOCK OCTANT-34

1CD07
BLOCK OCTANT-134

1CD08
BLOCK OCTANT-234

1CD09
BLOCK OCTANT-5

1CD0A
BLOCK OCTANT-15

1CD0B
BLOCK OCTANT-25

1CD0C
BLOCK OCTANT-125

1CD0D
BLOCK OCTANT-135

1CD0E
BLOCK OCTANT-235

1CD0F
BLOCK OCTANT-1235

1CD10
BLOCK OCTANT-45

1CD11
BLOCK OCTANT-145

1CD12
BLOCK OCTANT-245

1CD13
BLOCK OCTANT-1245

1CD14
BLOCK OCTANT-345

1CD15
BLOCK OCTANT-1345

1CD16
BLOCK OCTANT-2345

1CD17
BLOCK OCTANT-12345

1CD18
BLOCK OCTANT-6

1CD19
BLOCK OCTANT-16

1CD1A
BLOCK OCTANT-26

1CD1B
BLOCK OCTANT-126

1CD1C
BLOCK OCTANT-36

1CD1D
BLOCK OCTANT-136

1CD1E
BLOCK OCTANT-236

1CD1F
BLOCK OCTANT-1236

1CD20
BLOCK OCTANT-146

1CD21
BLOCK OCTANT-246

1CD22
BLOCK OCTANT-1246

1CD23
BLOCK OCTANT-346

1CD24
BLOCK OCTANT-1346

1CD25
BLOCK OCTANT-2346

1CD26
BLOCK OCTANT-12346

1CD27
BLOCK OCTANT-56

1CD28
BLOCK OCTANT-156

1CD29
BLOCK OCTANT-256

1CD2A
BLOCK OCTANT-1256

1CD2B
BLOCK OCTANT-356

1CD2C
BLOCK OCTANT-1356

1CD2D
BLOCK OCTANT-2356

1CD2E
BLOCK OCTANT-12356

1CD2F
BLOCK OCTANT-456

1CD30
BLOCK OCTANT-1456

1CD31
BLOCK OCTANT-2456

1CD32
BLOCK OCTANT-12456

1CD33
BLOCK OCTANT-3456

1CD34
BLOCK OCTANT-13456

1CD35
BLOCK OCTANT-23456

1CD36
BLOCK OCTANT-17

1CD37
BLOCK OCTANT-27

1CD38
BLOCK OCTANT-127

1CD39
BLOCK OCTANT-37

1CD3A
BLOCK OCTANT-137

1CD3B
BLOCK OCTANT-237

1CD3C
BLOCK OCTANT-1237

1CD3D
BLOCK OCTANT-47

1CD3E
BLOCK OCTANT-147

1CD3F
BLOCK OCTANT-247

1CD40
BLOCK OCTANT-1247

1CD41
BLOCK OCTANT-347

1CD42
BLOCK OCTANT-1347

1CD43
BLOCK OCTANT-2347

1CD44
BLOCK OCTANT-12347

1CD45
BLOCK OCTANT-157

1CD46
BLOCK OCTANT-257

1CD47
BLOCK OCTANT-1257

1CD48
BLOCK OCTANT-357

1CD49
BLOCK OCTANT-2357

1CD4A
BLOCK OCTANT-12357

1CD4B
BLOCK OCTANT-457

1CD4C
BLOCK OCTANT-1457

1CD4D
BLOCK OCTANT-12457

1CD4E
BLOCK OCTANT-3457

1CD4F
BLOCK OCTANT-13457

1CD50
BLOCK OCTANT-23457

1CD51
BLOCK OCTANT-67

1CD52
BLOCK OCTANT-167

1CD53
BLOCK OCTANT-267

1CD54
BLOCK OCTANT-1267

1CD55
BLOCK OCTANT-367

1CD56
BLOCK OCTANT-1367

1CD57
BLOCK OCTANT-2367

1CD58
BLOCK OCTANT-12367

1CD59
BLOCK OCTANT-467

1CD5A
BLOCK OCTANT-1467

1CD5B
BLOCK OCTANT-2467

1CD5C
BLOCK OCTANT-12467

1CD5D
BLOCK OCTANT-3467

1CD5E
BLOCK OCTANT-13467

1CD5F
BLOCK OCTANT-23467

1CD60
BLOCK OCTANT-123467

1CD61
BLOCK OCTANT-567

1CD62
BLOCK OCTANT-1567

1CD63
BLOCK OCTANT-2567

1CD64
BLOCK OCTANT-12567

1CD65
BLOCK OCTANT-3567

1CD66
BLOCK OCTANT-13567

1CD67
BLOCK OCTANT-23567

1CD68
BLOCK OCTANT-123567

1CD69
BLOCK OCTANT-4567

1CD6A
BLOCK OCTANT-14567

1CD6B
BLOCK OCTANT-24567

1CD6C
BLOCK OCTANT-124567

1CD6D
BLOCK OCTANT-34567

1CD6E
BLOCK OCTANT-134567

1CD6F
BLOCK OCTANT-234567

1CD70
BLOCK OCTANT-1234567

1CD71
BLOCK OCTANT-18

1CD72
BLOCK OCTANT-28

1CD73
BLOCK OCTANT-128

1CD74
BLOCK OCTANT-38

1CD75
BLOCK OCTANT-138

1CD76
BLOCK OCTANT-238

1CD77
BLOCK OCTANT-1238

1CD78
BLOCK OCTANT-48

1CD79
BLOCK OCTANT-148

1CD7A
BLOCK OCTANT-248

1CD7B
BLOCK OCTANT-1248

1CD7C
BLOCK OCTANT-348

1CD7D
BLOCK OCTANT-1348

1CD7E
BLOCK OCTANT-2348

1CD7F
BLOCK OCTANT-12348

1CD80
BLOCK OCTANT-58

1CD81
BLOCK OCTANT-158

1CD82
BLOCK OCTANT-258

1CD83
BLOCK OCTANT-1258

1CD84
BLOCK OCTANT-358

1CD85
BLOCK OCTANT-1358

1CD86
BLOCK OCTANT-2358

1CD87
BLOCK OCTANT-12358

1CD88
BLOCK OCTANT-458

1CD89
BLOCK OCTANT-1458

1CD8A
BLOCK OCTANT-2458

1CD8B
BLOCK OCTANT-12458

1CD8C
BLOCK OCTANT-3458

1CD8D
BLOCK OCTANT-13458

1CD8E
BLOCK OCTANT-23458

1CD8F
BLOCK OCTANT-123458

1CD90
BLOCK OCTANT-168

1CD91
BLOCK OCTANT-268

1CD92
BLOCK OCTANT-1268

1CD93
BLOCK OCTANT-368

1CD94
BLOCK OCTANT-2368

1CD95
BLOCK OCTANT-12368

1CD96
BLOCK OCTANT-468

1CD97
BLOCK OCTANT-1468

1CD98
BLOCK OCTANT-12468

1CD99
BLOCK OCTANT-3468

1CD9A
BLOCK OCTANT-13468

1CD9B
BLOCK OCTANT-23468

1CD9C
BLOCK OCTANT-568

1CD9D
BLOCK OCTANT-1568

1CD9E
BLOCK OCTANT-2568

1CD9F
BLOCK OCTANT-12568

1CDA0
BLOCK OCTANT-3568

1CDA1
BLOCK OCTANT-13568

1CDA2
BLOCK OCTANT-23568

1CDA3
BLOCK OCTANT-123568

1CDA4
BLOCK OCTANT-4568

1CDA5
BLOCK OCTANT-14568

1CDA6
BLOCK OCTANT-24568

1CDA7
BLOCK OCTANT-124568

1CDA8
BLOCK OCTANT-34568

1CDA9
BLOCK OCTANT-134568

1CDAA
BLOCK OCTANT-234568

1CDAB
BLOCK OCTANT-1234568

1CDAC
BLOCK OCTANT-178

1CDAD
BLOCK OCTANT-278

1CDAE
BLOCK OCTANT-1278

1CDAF
BLOCK OCTANT-378

1CDB0
BLOCK OCTANT-1378

1CDB1
BLOCK OCTANT-2378

1CDB2
BLOCK OCTANT-12378

1CDB3
BLOCK OCTANT-478

1CDB4
BLOCK OCTANT-1478

1CDB5
BLOCK OCTANT-2478

1CDB6
BLOCK OCTANT-12478

1CDB7
BLOCK OCTANT-3478

1CDB8
BLOCK OCTANT-13478

1CDB9
BLOCK OCTANT-23478

1CDBA
BLOCK OCTANT-123478

1CDBB
BLOCK OCTANT-578

1CDBC
BLOCK OCTANT-1578

1CDBD
BLOCK OCTANT-2578

1CDBE
BLOCK OCTANT-12578

1CDBF
BLOCK OCTANT-3578

1CDC0
BLOCK OCTANT-13578

1CDC1
BLOCK OCTANT-23578

1CDC2
BLOCK OCTANT-123578

1CDC3
BLOCK OCTANT-4578

1CDC4
BLOCK OCTANT-14578

1CDC5
BLOCK OCTANT-24578

1CDC6
BLOCK OCTANT-124578

1CDC7
BLOCK OCTANT-34578

1CDC8
BLOCK OCTANT-134578

1CDC9
BLOCK OCTANT-234578

1CDCA
BLOCK OCTANT-1234578

1CDCB
BLOCK OCTANT-678

1CDCC
BLOCK OCTANT-1678

1CDCD
BLOCK OCTANT-2678

1CDCE
BLOCK OCTANT-12678

1CDCF
BLOCK OCTANT-3678

1CDD0
BLOCK OCTANT-13678

1CDD1
BLOCK OCTANT-23678

1CDD2
BLOCK OCTANT-123678

1CDD3
BLOCK OCTANT-4678

1CDD4
BLOCK OCTANT-14678

1CDD5
BLOCK OCTANT-24678

1CDD6
BLOCK OCTANT-124678

1CDD7
BLOCK OCTANT-34678

1CDD8
BLOCK OCTANT-134678

1CDD9
BLOCK OCTANT-234678

1CDDA
BLOCK OCTANT-1234678

1CDDB
BLOCK OCTANT-15678

1CDDC
BLOCK OCTANT-25678

1CDDD
BLOCK OCTANT-125678

1CDDE
BLOCK OCTANT-35678

1CDDF
BLOCK OCTANT-235678

1CDE0
BLOCK OCTANT-1235678

1CDE1
BLOCK OCTANT-45678

1CDE2
BLOCK OCTANT-145678

1CDE3
BLOCK OCTANT-1245678

1CDE4
BLOCK OCTANT-1345678

1CDE5
BLOCK OCTANT-2345678

1CDE6
TOP HALF STANDING PERSON

1CDE7
BOTTOM HALF STANDING PERSON

1CDE8
TOP HALF RIGHT-FACING RUNNER FRAME-1

1CDE9
BOTTOM HALF RIGHT-FACING RUNNER FRAME-1

1CDEA
TOP HALF RIGHT-FACING RUNNER FRAME-2

1CDEB
BOTTOM HALF RIGHT-FACING RUNNER FRAME-2

1CDEC
TOP HALF LEFT-FACING RUNNER FRAME-1

1CDED
BOTTOM HALF LEFT-FACING RUNNER FRAME-1

1CDEE
TOP HALF LEFT-FACING RUNNER FRAME-2

1CDEF
BOTTOM HALF LEFT-FACING RUNNER FRAME-2

1CDF0
TOP HALF FORWARD-FACING RUNNER

1CDF1
BOTTOM HALF FORWARD-FACING RUNNER FRAME-1

1CDF2
BOTTOM HALF FORWARD-FACING RUNNER FRAME-2

1CDF3
BOTTOM HALF FORWARD-FACING RUNNER FRAME-3

1CDF4
BOTTOM HALF FORWARD-FACING RUNNER FRAME-4

1CDF5
MOON LANDER

1CDF6
TOP HALF FLAILING ROBOT FRAME-1

1CDF7
TOP HALF FLAILING ROBOT FRAME-2

1CDF8
DOWN-POINTING AIRPLANE

1CDF9
LEFT-POINTING AIRPLANE

1CDFA
SMALL UP-POINTING AIRPLANE

1CDFB
UP-POINTING FROG

1CDFC
DOWN-POINTING FROG

1CDFD
EXPLOSION FRAME-1

1CDFE
EXPLOSION FRAME-2

1CDFF
EXPLOSION FRAME-3

1CE00
RIGHT HALF AND LEFT HALF WHITE CIRCLE

1CE01
LOWER HALF AND UPPER HALF WHITE CIRCLE

1CE02
EXPLOSION AT HORIZON

1CE03
UPPER HALF HEAVY WHITE SQUARE

1CE04
LOWER HALF HEAVY WHITE SQUARE

1CE05
HEAVY WHITE SQUARE CONTAINING BLACK VERY SMALL SQUARE

1CE06
WHITE VERTICAL RECTANGLE WITH HORIZONTAL BAR

1CE07
TOP LEFT BLACK LEFT-POINTING SMALL TRIANGLE

1CE08
FUNNEL

1CE09
BOX DRAWINGS DOUBLE DIAGONAL LOWER LEFT TO MIDDLE CENTRE TO LOWER RIGHT

1CE0A
BOX DRAWINGS DOUBLE DIAGONAL UPPER LEFT TO MIDDLE CENTRE TO UPPER RIGHT

1CE0B
LEFT HALF WHITE ELLIPSE

1CE0C
RIGHT HALF WHITE ELLIPSE

1CE0D
LEFT HALF TRIPLE DASH HORIZONTAL

1CE0E
RIGHT HALF TRIPLE DASH HORIZONTAL

1CE0F
HORIZONTAL LINE WITH TICK MARK

1CE10
LEFT HALF HORIZONTAL LINE WITH THREE TICK MARKS

1CE11
RIGHT HALF HORIZONTAL LINE WITH THREE TICK MARKS

1CE12
HORIZONTAL LINE WITH THREE TICK MARKS

1CE13
LOWER HALF VERTICAL LINE WITH THREE TICK MARKS

1CE14
UPPER HALF VERTICAL LINE WITH THREE TICK MARKS

1CE15
VERTICAL LINE WITH THREE TICK MARKS

1CE16
BOX DRAWINGS LIGHT VERTICAL AND TOP RIGHT

1CE17
BOX DRAWINGS LIGHT VERTICAL AND BOTTOM RIGHT

1CE18
BOX DRAWINGS LIGHT VERTICAL AND TOP LEFT

1CE19
BOX DRAWINGS LIGHT VERTICAL AND BOTTOM LEFT

1CE1A
LARGE TYPE PIECE UPPER LEFT ARC

1CE1B
LARGE TYPE PIECE UPPER LEFT CORNER

1CE1C
LARGE TYPE PIECE UPPER TERMINAL

1CE1D
LARGE TYPE PIECE UPPER LEFT CROTCH

1CE1E
LARGE TYPE PIECE LEFT ARM

1CE1F
LARGE TYPE PIECE CROSSBAR

1CE20
LARGE TYPE PIECE CROSSBAR WITH LOWER STEM

1CE21
LARGE TYPE PIECE UPPER HALF VERTEX OF M

1CE22
LARGE TYPE PIECE DIAGONAL LOWER LEFT

1CE23
LARGE TYPE PIECE SHORT UPPER TERMINAL

1CE24
LARGE TYPE PIECE UPPER RIGHT ARC

1CE25
LARGE TYPE PIECE RIGHT ARM

1CE26
LARGE TYPE PIECE UPPER RIGHT CROTCH

1CE27
LARGE TYPE PIECE UPPER RIGHT CORNER

1CE28
LARGE TYPE PIECE STEM WITH RIGHT CROSSBAR

1CE29
LARGE TYPE PIECE STEM

1CE2A
LARGE TYPE PIECE DIAGONAL UPPER RIGHT AND LOWER RIGHT

1CE2B
LARGE TYPE PIECE DIAGONAL UPPER RIGHT

1CE2C
LARGE TYPE PIECE DIAGONAL LOWER RIGHT

1CE2D
LARGE TYPE PIECE SHORT LOWER TERMINAL

1CE2E
LARGE TYPE PIECE LOWER LEFT AND UPPER LEFT ARC

1CE2F
LARGE TYPE PIECE CENTRE OF K

1CE30
LARGE TYPE PIECE LOWER HALF VERTEX OF M

1CE31
LARGE TYPE PIECE UPPER HALF VERTEX OF W

1CE32
LARGE TYPE PIECE CENTRE OF X

1CE33
LARGE TYPE PIECE CENTRE OF Y

1CE34
LARGE TYPE PIECE CENTRE OF Z WITH CROSSBAR

1CE35
LARGE TYPE PIECE RAISED UPPER LEFT ARC

1CE36
LARGE TYPE PIECE STEM WITH LEFT CROSSBAR

1CE37
LARGE TYPE PIECE LOWER RIGHT AND UPPER RIGHT ARC

1CE38
LARGE TYPE PIECE DIAGONAL UPPER LEFT AND LOWER LEFT

1CE39
LARGE TYPE PIECE STEM WITH LEFT JOINT

1CE3A
LARGE TYPE PIECE STEM WITH CROSSBAR

1CE3B
LARGE TYPE PIECE DIAGONAL UPPER LEFT

1CE3C
LARGE TYPE PIECE LOWER TERMINAL

1CE3D
LARGE TYPE PIECE LOWER LEFT CORNER

1CE3E
LARGE TYPE PIECE LOWER LEFT ARC

1CE3F
LARGE TYPE PIECE LOWER LEFT CROTCH

1CE40
LARGE TYPE PIECE CROSSBAR WITH UPPER STEM

1CE41
LARGE TYPE PIECE VERTEX OF V

1CE42
LARGE TYPE PIECE LOWER HALF VERTEX OF W

1CE43
LARGE TYPE PIECE LOWER RIGHT ARC

1CE44
LARGE TYPE PIECE LOWER RIGHT CORNER

1CE45
LARGE TYPE PIECE LOWER RIGHT ARC WITH TAIL

1CE46
LARGE TYPE PIECE LOWER RIGHT CROTCH

1CE47
LARGE TYPE PIECE STEM-45

1CE48
LARGE TYPE PIECE STEM-2345

1CE49
LARGE TYPE PIECE STEM-4

1CE4A
LARGE TYPE PIECE STEM-34

1CE4B
LARGE TYPE PIECE STEM-234

1CE4C
LARGE TYPE PIECE STEM-1234

1CE4D
LARGE TYPE PIECE STEM-3

1CE4E
LARGE TYPE PIECE STEM-23

1CE4F
LARGE TYPE PIECE STEM-2

1CE50
LARGE TYPE PIECE STEM-12

1CE51
SEPARATED BLOCK SEXTANT-1

1CE52
SEPARATED BLOCK SEXTANT-2

1CE53
SEPARATED BLOCK SEXTANT-12

1CE54
SEPARATED BLOCK SEXTANT-3

1CE55
SEPARATED BLOCK SEXTANT-13

1CE56
SEPARATED BLOCK SEXTANT-23

1CE57
SEPARATED BLOCK SEXTANT-123

1CE58
SEPARATED BLOCK SEXTANT-4

1CE59
SEPARATED BLOCK SEXTANT-14

1CE5A
SEPARATED BLOCK SEXTANT-24

1CE5B
SEPARATED BLOCK SEXTANT-124

1CE5C
SEPARATED BLOCK SEXTANT-34

1CE5D
SEPARATED BLOCK SEXTANT-134

1CE5E
SEPARATED BLOCK SEXTANT-234

1CE5F
SEPARATED BLOCK SEXTANT-1234

1CE60
SEPARATED BLOCK SEXTANT-5

1CE61
SEPARATED BLOCK SEXTANT-15

1CE62
SEPARATED BLOCK SEXTANT-25

1CE63
SEPARATED BLOCK SEXTANT-125

1CE64
SEPARATED BLOCK SEXTANT-35

1CE65
SEPARATED BLOCK SEXTANT-135

1CE66
SEPARATED BLOCK SEXTANT-235

1CE67
SEPARATED BLOCK SEXTANT-1235

1CE68
SEPARATED BLOCK SEXTANT-45

1CE69
SEPARATED BLOCK SEXTANT-145

1CE6A
SEPARATED BLOCK SEXTANT-245

1CE6B
SEPARATED BLOCK SEXTANT-1245

1CE6C
SEPARATED BLOCK SEXTANT-345

1CE6D
SEPARATED BLOCK SEXTANT-1345

1CE6E
SEPARATED BLOCK SEXTANT-2345

1CE6F
SEPARATED BLOCK SEXTANT-12345

1CE70
SEPARATED BLOCK SEXTANT-6

1CE71
SEPARATED BLOCK SEXTANT-16

1CE72
SEPARATED BLOCK SEXTANT-26

1CE73
SEPARATED BLOCK SEXTANT-126

1CE74
SEPARATED BLOCK SEXTANT-36

1CE75
SEPARATED BLOCK SEXTANT-136

1CE76
SEPARATED BLOCK SEXTANT-236

1CE77
SEPARATED BLOCK SEXTANT-1236

1CE78
SEPARATED BLOCK SEXTANT-46

1CE79
SEPARATED BLOCK SEXTANT-146

1CE7A
SEPARATED BLOCK SEXTANT-246

1CE7B
SEPARATED BLOCK SEXTANT-1246

1CE7C
SEPARATED BLOCK SEXTANT-346

1CE7D
SEPARATED BLOCK SEXTANT-1346

1CE7E
SEPARATED BLOCK SEXTANT-2346

1CE7F
SEPARATED BLOCK SEXTANT-12346

1CE80
SEPARATED BLOCK SEXTANT-56

1CE81
SEPARATED BLOCK SEXTANT-156

1CE82
SEPARATED BLOCK SEXTANT-256

1CE83
SEPARATED BLOCK SEXTANT-1256

1CE84
SEPARATED BLOCK SEXTANT-356

1CE85
SEPARATED BLOCK SEXTANT-1356

1CE86
SEPARATED BLOCK SEXTANT-2356

1CE87
SEPARATED BLOCK SEXTANT-12356

1CE88
SEPARATED BLOCK SEXTANT-456

1CE89
SEPARATED BLOCK SEXTANT-1456

1CE8A
SEPARATED BLOCK SEXTANT-2456

1CE8B
SEPARATED BLOCK SEXTANT-12456

1CE8C
SEPARATED BLOCK SEXTANT-3456

1CE8D
SEPARATED BLOCK SEXTANT-13456

1CE8E
SEPARATED BLOCK SEXTANT-23456

1CE8F
SEPARATED BLOCK SEXTANT-123456

1CE90
UPPER LEFT ONE SIXTEENTH BLOCK

1CE91
UPPER CENTRE LEFT ONE SIXTEENTH BLOCK

1CE92
UPPER CENTRE RIGHT ONE SIXTEENTH BLOCK

1CE93
UPPER RIGHT ONE SIXTEENTH BLOCK

1CE94
UPPER MIDDLE LEFT ONE SIXTEENTH BLOCK

1CE95
UPPER MIDDLE CENTRE LEFT ONE SIXTEENTH BLOCK

1CE96
UPPER MIDDLE CENTRE RIGHT ONE SIXTEENTH BLOCK

1CE97
UPPER MIDDLE RIGHT ONE SIXTEENTH BLOCK

1CE98
LOWER MIDDLE LEFT ONE SIXTEENTH BLOCK

1CE99
LOWER MIDDLE CENTRE LEFT ONE SIXTEENTH BLOCK

1CE9A
LOWER MIDDLE CENTRE RIGHT ONE SIXTEENTH BLOCK

1CE9B
LOWER MIDDLE RIGHT ONE SIXTEENTH BLOCK

1CE9C
LOWER LEFT ONE SIXTEENTH BLOCK

1CE9D
LOWER CENTRE LEFT ONE SIXTEENTH BLOCK

1CE9E
LOWER CENTRE RIGHT ONE SIXTEENTH BLOCK

1CE9F
LOWER RIGHT ONE SIXTEENTH BLOCK

1CEA0
RIGHT HALF LOWER ONE QUARTER BLOCK

1CEA1
RIGHT THREE QUARTERS LOWER ONE QUARTER BLOCK

1CEA2
LEFT THREE QUARTERS LOWER ONE QUARTER BLOCK

1CEA3
LEFT HALF LOWER ONE QUARTER BLOCK

1CEA4
LOWER HALF LEFT ONE QUARTER BLOCK

1CEA5
LOWER THREE QUARTERS LEFT ONE QUARTER BLOCK

1CEA6
UPPER THREE QUARTERS LEFT ONE QUARTER BLOCK

1CEA7
UPPER HALF LEFT ONE QUARTER BLOCK

1CEA8
LEFT HALF UPPER ONE QUARTER BLOCK

1CEA9
LEFT THREE QUARTERS UPPER ONE QUARTER BLOCK

1CEAA
RIGHT THREE QUARTERS UPPER ONE QUARTER BLOCK

1CEAB
RIGHT HALF UPPER ONE QUARTER BLOCK

1CEAC
UPPER HALF RIGHT ONE QUARTER BLOCK

1CEAD
UPPER THREE QUARTERS RIGHT ONE QUARTER BLOCK

1CEAE
LOWER THREE QUARTERS RIGHT ONE QUARTER BLOCK

1CEAF
LOWER HALF RIGHT ONE QUARTER BLOCK

1CEB0
HORIZONTAL ZIGZAG LINE

1CEB1
KEYHOLE

1CEB2
OLD PERSONAL COMPUTER WITH MONITOR IN PORTRAIT ORIENTATION

1CEB3
BLACK RIGHT TRIANGLE CARET

1CEBA
FRAGILE SYMBOL

1CEBB
OFFICE BUILDING SYMBOL

1CEBC
TREE SYMBOL

1CEBD
APPLE SYMBOL

1CEBE
CHERRY SYMBOL

1CEBF
STRAWBERRY SYMBOL

1CEC0
HEBE

1CEC1
IRIS

1CEC2
FLORA

1CEC3
METIS

1CEC4
PARTHENOPE

1CEC5
VICTORIA

1CEC6
EGERIA

1CEC7
IRENE

1CEC8
EUNOMIA

1CEC9
PSYCHE

1CECA
THETIS

1CECB
MELPOMENE

1CECC
FORTUNA

1CECD
ASTRONOMICAL SYMBOL FOR ASTEROID PROSERPINA

1CECE
BELLONA

1CECF
AMPHITRITE

1CED0
LEUKOTHEA

1CEE0
GEOMANTIC FIGURE POPULUS

1CEE1
GEOMANTIC FIGURE TRISTITIA

1CEE2
GEOMANTIC FIGURE ALBUS

1CEE3
GEOMANTIC FIGURE FORTUNA MAJOR

1CEE4
GEOMANTIC FIGURE RUBEUS

1CEE5
GEOMANTIC FIGURE ACQUISITIO

1CEE6
GEOMANTIC FIGURE CONJUNCTIO

1CEE7
GEOMANTIC FIGURE CAPUT DRACONIS

1CEE8
GEOMANTIC FIGURE LAETITIA

1CEE9
GEOMANTIC FIGURE CARCER

1CEEA
GEOMANTIC FIGURE AMISSIO

1CEEB
GEOMANTIC FIGURE PUELLA

1CEEC
GEOMANTIC FIGURE FORTUNA MINOR

1CEED
GEOMANTIC FIGURE PUER

1CEEE
GEOMANTIC FIGURE CAUDA DRACONIS

1CEEF
GEOMANTIC FIGURE VIA

1CEF0
MEDIUM SMALL WHITE CIRCLE WITH HORIZONTAL BAR

1CF00
ZNAMENNY COMBINING MARK GORAZDO NIZKO S KRYZHEM ON LEFT

1CF01
ZNAMENNY COMBINING MARK NIZKO S KRYZHEM ON LEFT

1CF02
ZNAMENNY COMBINING MARK TSATA ON LEFT

1CF03
ZNAMENNY COMBINING MARK GORAZDO NIZKO ON LEFT

1CF04
ZNAMENNY COMBINING MARK NIZKO ON LEFT

1CF05
ZNAMENNY COMBINING MARK SREDNE ON LEFT

1CF06
ZNAMENNY COMBINING MARK MALO POVYSHE ON LEFT

1CF07
ZNAMENNY COMBINING MARK POVYSHE ON LEFT

1CF08
ZNAMENNY COMBINING MARK VYSOKO ON LEFT

1CF09
ZNAMENNY COMBINING MARK MALO POVYSHE S KHOKHLOM ON LEFT

1CF0A
ZNAMENNY COMBINING MARK POVYSHE S KHOKHLOM ON LEFT

1CF0B
ZNAMENNY COMBINING MARK VYSOKO S KHOKHLOM ON LEFT

1CF0C
ZNAMENNY COMBINING MARK GORAZDO NIZKO S KRYZHEM ON RIGHT

1CF0D
ZNAMENNY COMBINING MARK NIZKO S KRYZHEM ON RIGHT

1CF0E
ZNAMENNY COMBINING MARK TSATA ON RIGHT

1CF0F
ZNAMENNY COMBINING MARK GORAZDO NIZKO ON RIGHT

1CF10
ZNAMENNY COMBINING MARK NIZKO ON RIGHT

1CF11
ZNAMENNY COMBINING MARK SREDNE ON RIGHT

1CF12
ZNAMENNY COMBINING MARK MALO POVYSHE ON RIGHT

1CF13
ZNAMENNY COMBINING MARK POVYSHE ON RIGHT

1CF14
ZNAMENNY COMBINING MARK VYSOKO ON RIGHT

1CF15
ZNAMENNY COMBINING MARK MALO POVYSHE S KHOKHLOM ON RIGHT

1CF16
ZNAMENNY COMBINING MARK POVYSHE S KHOKHLOM ON RIGHT

1CF17
ZNAMENNY COMBINING MARK VYSOKO S KHOKHLOM ON RIGHT

1CF18
ZNAMENNY COMBINING MARK TSATA S KRYZHEM

1CF19
ZNAMENNY COMBINING MARK MALO POVYSHE S KRYZHEM

1CF1A
ZNAMENNY COMBINING MARK STRANNO MALO POVYSHE

1CF1B
ZNAMENNY COMBINING MARK POVYSHE S KRYZHEM

1CF1C
ZNAMENNY COMBINING MARK POVYSHE STRANNO

1CF1D
ZNAMENNY COMBINING MARK VYSOKO S KRYZHEM

1CF1E
ZNAMENNY COMBINING MARK MALO POVYSHE STRANNO

1CF1F
ZNAMENNY COMBINING MARK GORAZDO VYSOKO

1CF20
ZNAMENNY COMBINING MARK ZELO

1CF21
ZNAMENNY COMBINING MARK ON

1CF22
ZNAMENNY COMBINING MARK RAVNO

1CF23
ZNAMENNY COMBINING MARK TIKHAYA

1CF24
ZNAMENNY COMBINING MARK BORZAYA

1CF25
ZNAMENNY COMBINING MARK UDARKA

1CF26
ZNAMENNY COMBINING MARK PODVERTKA

1CF27
ZNAMENNY COMBINING MARK LOMKA

1CF28
ZNAMENNY COMBINING MARK KUPNAYA

1CF29
ZNAMENNY COMBINING MARK KACHKA

1CF2A
ZNAMENNY COMBINING MARK ZEVOK

1CF2B
ZNAMENNY COMBINING MARK SKOBA

1CF2C
ZNAMENNY COMBINING MARK RAZSEKA

1CF2D
ZNAMENNY COMBINING MARK KRYZH ON LEFT

1CF30
ZNAMENNY COMBINING TONAL RANGE MARK MRACHNO

1CF31
ZNAMENNY COMBINING TONAL RANGE MARK SVETLO

1CF32
ZNAMENNY COMBINING TONAL RANGE MARK TRESVETLO

1CF33
ZNAMENNY COMBINING MARK ZADERZHKA

1CF34
ZNAMENNY COMBINING MARK DEMESTVENNY ZADERZHKA

1CF35
ZNAMENNY COMBINING MARK OTSECHKA

1CF36
ZNAMENNY COMBINING MARK PODCHASHIE

1CF37
ZNAMENNY COMBINING MARK PODCHASHIE WITH VERTICAL STROKE

1CF38
ZNAMENNY COMBINING MARK CHASHKA

1CF39
ZNAMENNY COMBINING MARK CHASHKA POLNAYA

1CF3A
ZNAMENNY COMBINING MARK OBLACHKO

1CF3B
ZNAMENNY COMBINING MARK SOROCHYA NOZHKA

1CF3C
ZNAMENNY COMBINING MARK TOCHKA

1CF3D
ZNAMENNY COMBINING MARK DVOETOCHIE

1CF3E
ZNAMENNY COMBINING ATTACHING VERTICAL OMET

1CF3F
ZNAMENNY COMBINING MARK CURVED OMET

1CF40
ZNAMENNY COMBINING MARK KRYZH

1CF41
ZNAMENNY COMBINING LOWER TONAL RANGE INDICATOR

1CF42
ZNAMENNY PRIZNAK MODIFIER LEVEL-2

1CF43
ZNAMENNY PRIZNAK MODIFIER LEVEL-3

1CF44
ZNAMENNY PRIZNAK MODIFIER DIRECTION FLIP

1CF45
ZNAMENNY PRIZNAK MODIFIER KRYZH

1CF46
ZNAMENNY PRIZNAK MODIFIER ROG

1CF50
ZNAMENNY NEUME KRYUK

1CF51
ZNAMENNY NEUME KRYUK TIKHY

1CF52
ZNAMENNY NEUME PARAKLIT

1CF53
ZNAMENNY NEUME DVA V CHELNU

1CF54
ZNAMENNY NEUME KLYUCH

1CF55
ZNAMENNY NEUME ZANOZHEK

1CF56
ZNAMENNY NEUME STOPITSA

1CF57
ZNAMENNY NEUME STOPITSA S OCHKOM

1CF58
ZNAMENNY NEUME PEREVODKA

1CF59
ZNAMENNY NEUME PEREVODKA NEPOSTOYANNAYA

1CF5A
ZNAMENNY NEUME STOPITSA WITH SOROCHYA NOZHKA

1CF5B
ZNAMENNY NEUME CHELYUSTKA

1CF5C
ZNAMENNY NEUME PALKA

1CF5D
ZNAMENNY NEUME ZAPYATAYA

1CF5E
ZNAMENNY NEUME GOLUBCHIK BORZY

1CF5F
ZNAMENNY NEUME GOLUBCHIK TIKHY

1CF60
ZNAMENNY NEUME GOLUBCHIK MRACHNY

1CF61
ZNAMENNY NEUME GOLUBCHIK SVETLY

1CF62
ZNAMENNY NEUME GOLUBCHIK TRESVETLY

1CF63
ZNAMENNY NEUME VRAKHIYA PROSTAYA

1CF64
ZNAMENNY NEUME VRAKHIYA MRACHNAYA

1CF65
ZNAMENNY NEUME VRAKHIYA SVETLAYA

1CF66
ZNAMENNY NEUME VRAKHIYA TRESVETLAYA

1CF67
ZNAMENNY NEUME VRAKHIYA KLYUCHEVAYA PROSTAYA

1CF68
ZNAMENNY NEUME VRAKHIYA KLYUCHEVAYA MRACHNAYA

1CF69
ZNAMENNY NEUME VRAKHIYA KLYUCHEVAYA SVETLAYA

1CF6A
ZNAMENNY NEUME VRAKHIYA KLYUCHEVAYA TRESVETLAYA

1CF6B
ZNAMENNY NEUME DOUBLE ZAPYATAYA

1CF6C
ZNAMENNY NEUME REVERSED CHELYUSTKA

1CF6D
ZNAMENNY NEUME DERBITSA

1CF6E
ZNAMENNY NEUME KHAMILO

1CF6F
ZNAMENNY NEUME CHASHKA

1CF70
ZNAMENNY NEUME PODCHASHIE

1CF71
ZNAMENNY NEUME SKAMEYTSA MRACHNAYA

1CF72
ZNAMENNY NEUME SKAMEYTSA SVETLAYA

1CF73
ZNAMENNY NEUME SKAMEYTSA TRESVETLAYA

1CF74
ZNAMENNY NEUME SKAMEYTSA TIKHAYA

1CF75
ZNAMENNY NEUME DEMESTVENNY KLYUCH

1CF76
ZNAMENNY NEUME SKAMEYTSA KLYUCHEVAYA SVETLAYA

1CF77
ZNAMENNY NEUME SKAMEYTSA KLYUCHENEPOSTOYANNAYA

1CF78
ZNAMENNY NEUME SKAMEYTSA KLYUCHEVAYA TIKHAYA

1CF79
ZNAMENNY NEUME SKAMEYTSA DVOECHELNAYA PROSTAYA

1CF7A
ZNAMENNY NEUME SKAMEYTSA DVOECHELNAYA SVETLAYA

1CF7B
ZNAMENNY NEUME SKAMEYTSA DVOECHELNAYA NEPOSTOYANNAYA

1CF7C
ZNAMENNY NEUME SKAMEYTSA DVOECHELNAYA KLYUCHEVAYA

1CF7D
ZNAMENNY NEUME SLOZHITIE

1CF7E
ZNAMENNY NEUME SLOZHITIE S ZAPYATOY

1CF7F
ZNAMENNY NEUME SLOZHITIE ZAKRYTOE

1CF80
ZNAMENNY NEUME SLOZHITIE S KRYZHEM

1CF81
ZNAMENNY NEUME KRYZH

1CF82
ZNAMENNY NEUME ROG

1CF83
ZNAMENNY NEUME FITA

1CF84
ZNAMENNY NEUME KOBYLA

1CF85
ZNAMENNY NEUME ZMEYTSA

1CF86
ZNAMENNY NEUME STATYA

1CF87
ZNAMENNY NEUME STATYA S ZAPYATOY

1CF88
ZNAMENNY NEUME STATYA S KRYZHEM

1CF89
ZNAMENNY NEUME STATYA S ZAPYATOY I KRYZHEM

1CF8A
ZNAMENNY NEUME STATYA S KRYZHEM I ZAPYATOY

1CF8B
ZNAMENNY NEUME STATYA ZAKRYTAYA

1CF8C
ZNAMENNY NEUME STATYA ZAKRYTAYA S ZAPYATOY

1CF8D
ZNAMENNY NEUME STATYA S ROGOM

1CF8E
ZNAMENNY NEUME STATYA S DVUMYA ZAPYATYMI

1CF8F
ZNAMENNY NEUME STATYA S ZAPYATOY I PODCHASHIEM

1CF90
ZNAMENNY NEUME POLKULIZMY

1CF91
ZNAMENNY NEUME STATYA NEPOSTOYANNAYA

1CF92
ZNAMENNY NEUME STRELA PROSTAYA

1CF93
ZNAMENNY NEUME STRELA MRACHNOTIKHAYA

1CF94
ZNAMENNY NEUME STRELA KRYZHEVAYA

1CF95
ZNAMENNY NEUME STRELA POLUPOVODNAYA

1CF96
ZNAMENNY NEUME STRELA POVODNAYA

1CF97
ZNAMENNY NEUME STRELA NEPOSTOYANNAYA

1CF98
ZNAMENNY NEUME STRELA KLYUCHEPOVODNAYA

1CF99
ZNAMENNY NEUME STRELA KLYUCHENEPOSTOYANNAYA

1CF9A
ZNAMENNY NEUME STRELA TIKHAYA PUTNAYA

1CF9B
ZNAMENNY NEUME STRELA DVOECHELNAYA

1CF9C
ZNAMENNY NEUME STRELA DVOECHELNOKRYZHEVAYA

1CF9D
ZNAMENNY NEUME STRELA DVOECHELNOPOVODNAYA

1CF9E
ZNAMENNY NEUME STRELA DVOECHELNAYA KLYUCHEVAYA

1CF9F
ZNAMENNY NEUME STRELA DVOECHELNOPOVODNAYA KLYUCHEVAYA

1CFA0
ZNAMENNY NEUME STRELA GROMNAYA WITH SINGLE ZAPYATAYA

1CFA1
ZNAMENNY NEUME STRELA GROMOPOVODNAYA WITH SINGLE ZAPYATAYA

1CFA2
ZNAMENNY NEUME STRELA GROMNAYA

1CFA3
ZNAMENNY NEUME STRELA GROMOPOVODNAYA

1CFA4
ZNAMENNY NEUME STRELA GROMOPOVODNAYA WITH DOUBLE ZAPYATAYA

1CFA5
ZNAMENNY NEUME STRELA GROMOKRYZHEVAYA

1CFA6
ZNAMENNY NEUME STRELA GROMOKRYZHEVAYA POVODNAYA

1CFA7
ZNAMENNY NEUME MECHIK

1CFA8
ZNAMENNY NEUME MECHIK POVODNY

1CFA9
ZNAMENNY NEUME MECHIK KLYUCHEVOY

1CFAA
ZNAMENNY NEUME MECHIK KLYUCHEPOVODNY

1CFAB
ZNAMENNY NEUME MECHIK KLYUCHENEPOSTOYANNY

1CFAC
ZNAMENNY NEUME STRELA TRYASOGLASNAYA

1CFAD
ZNAMENNY NEUME STRELA TRYASOPOVODNAYA

1CFAE
ZNAMENNY NEUME STRELA TRYASOSTRELNAYA

1CFAF
ZNAMENNY NEUME OSOKA

1CFB0
ZNAMENNY NEUME OSOKA SVETLAYA

1CFB1
ZNAMENNY NEUME OSOKA TRESVETLAYA

1CFB2
ZNAMENNY NEUME OSOKA KRYUKOVAYA SVETLAYA

1CFB3
ZNAMENNY NEUME OSOKA KLYUCHEVAYA SVETLAYA

1CFB4
ZNAMENNY NEUME OSOKA KLYUCHEVAYA NEPOSTOYANNAYA

1CFB5
ZNAMENNY NEUME STRELA KRYUKOVAYA

1CFB6
ZNAMENNY NEUME STRELA KRYUKOVAYA POVODNAYA

1CFB7
ZNAMENNY NEUME STRELA KRYUKOVAYA GROMNAYA WITH SINGLE ZAPYATAYA

1CFB8
ZNAMENNY NEUME STRELA KRYUKOVAYA GROMOPOVODNAYA WITH SINGLE ZAPYATAYA

1CFB9
ZNAMENNY NEUME STRELA KRYUKOVAYA GROMNAYA

1CFBA
ZNAMENNY NEUME STRELA KRYUKOVAYA GROMOPOVODNAYA

1CFBB
ZNAMENNY NEUME STRELA KRYUKOVAYA GROMOPOVODNAYA WITH DOUBLE ZAPYATAYA

1CFBC
ZNAMENNY NEUME STRELA KRYUKOVAYA GROMOKRYZHEVAYA

1CFBD
ZNAMENNY NEUME STRELA KRYUKOVAYA GROMOKRYZHEVAYA POVODNAYA

1CFBE
ZNAMENNY NEUME STRELA KRYUKOVAYA TRYASKA

1CFBF
ZNAMENNY NEUME KUFISMA

1CFC0
ZNAMENNY NEUME OBLAKO

1CFC1
ZNAMENNY NEUME DUDA

1CFC2
ZNAMENNY NEUME NEMKA

1CFC3
ZNAMENNY NEUME PAUK

1D000
BYZANTINE MUSICAL SYMBOL PSILI

1D001
BYZANTINE MUSICAL SYMBOL DASEIA

1D002
BYZANTINE MUSICAL SYMBOL PERISPOMENI

1D003
BYZANTINE MUSICAL SYMBOL OXEIA EKFONITIKON

1D004
BYZANTINE MUSICAL SYMBOL OXEIA DIPLI

1D005
BYZANTINE MUSICAL SYMBOL VAREIA EKFONITIKON

1D006
BYZANTINE MUSICAL SYMBOL VAREIA DIPLI

1D007
BYZANTINE MUSICAL SYMBOL KATHISTI

1D008
BYZANTINE MUSICAL SYMBOL SYRMATIKI

1D009
BYZANTINE MUSICAL SYMBOL PARAKLITIKI

1D00A
BYZANTINE MUSICAL SYMBOL YPOKRISIS

1D00B
BYZANTINE MUSICAL SYMBOL YPOKRISIS DIPLI

1D00C
BYZANTINE MUSICAL SYMBOL KREMASTI

1D00D
BYZANTINE MUSICAL SYMBOL APESO EKFONITIKON

1D00E
BYZANTINE MUSICAL SYMBOL EXO EKFONITIKON

1D00F
BYZANTINE MUSICAL SYMBOL TELEIA

1D010
BYZANTINE MUSICAL SYMBOL KENTIMATA

1D011
BYZANTINE MUSICAL SYMBOL APOSTROFOS

1D012
BYZANTINE MUSICAL SYMBOL APOSTROFOS DIPLI

1D013
BYZANTINE MUSICAL SYMBOL SYNEVMA

1D014
BYZANTINE MUSICAL SYMBOL THITA

1D015
BYZANTINE MUSICAL SYMBOL OLIGON ARCHAION

1D016
BYZANTINE MUSICAL SYMBOL GORGON ARCHAION

1D017
BYZANTINE MUSICAL SYMBOL PSILON

1D018
BYZANTINE MUSICAL SYMBOL CHAMILON

1D019
BYZANTINE MUSICAL SYMBOL VATHY

1D01A
BYZANTINE MUSICAL SYMBOL ISON ARCHAION

1D01B
BYZANTINE MUSICAL SYMBOL KENTIMA ARCHAION

1D01C
BYZANTINE MUSICAL SYMBOL KENTIMATA ARCHAION

1D01D
BYZANTINE MUSICAL SYMBOL SAXIMATA

1D01E
BYZANTINE MUSICAL SYMBOL PARICHON

1D01F
BYZANTINE MUSICAL SYMBOL STAVROS APODEXIA

1D020
BYZANTINE MUSICAL SYMBOL OXEIAI ARCHAION

1D021
BYZANTINE MUSICAL SYMBOL VAREIAI ARCHAION

1D022
BYZANTINE MUSICAL SYMBOL APODERMA ARCHAION

1D023
BYZANTINE MUSICAL SYMBOL APOTHEMA

1D024
BYZANTINE MUSICAL SYMBOL KLASMA

1D025
BYZANTINE MUSICAL SYMBOL REVMA

1D026
BYZANTINE MUSICAL SYMBOL PIASMA ARCHAION

1D027
BYZANTINE MUSICAL SYMBOL TINAGMA

1D028
BYZANTINE MUSICAL SYMBOL ANATRICHISMA

1D029
BYZANTINE MUSICAL SYMBOL SEISMA

1D02A
BYZANTINE MUSICAL SYMBOL SYNAGMA ARCHAION

1D02B
BYZANTINE MUSICAL SYMBOL SYNAGMA META STAVROU

1D02C
BYZANTINE MUSICAL SYMBOL OYRANISMA ARCHAION

1D02D
BYZANTINE MUSICAL SYMBOL THEMA

1D02E
BYZANTINE MUSICAL SYMBOL LEMOI

1D02F
BYZANTINE MUSICAL SYMBOL DYO

1D030
BYZANTINE MUSICAL SYMBOL TRIA

1D031
BYZANTINE MUSICAL SYMBOL TESSERA

1D032
BYZANTINE MUSICAL SYMBOL KRATIMATA

1D033
BYZANTINE MUSICAL SYMBOL APESO EXO NEO

1D034
BYZANTINE MUSICAL SYMBOL FTHORA ARCHAION

1D035
BYZANTINE MUSICAL SYMBOL IMIFTHORA

1D036
BYZANTINE MUSICAL SYMBOL TROMIKON ARCHAION

1D037
BYZANTINE MUSICAL SYMBOL KATAVA TROMIKON

1D038
BYZANTINE MUSICAL SYMBOL PELASTON

1D039
BYZANTINE MUSICAL SYMBOL PSIFISTON

1D03A
BYZANTINE MUSICAL SYMBOL KONTEVMA

1D03B
BYZANTINE MUSICAL SYMBOL CHOREVMA ARCHAION

1D03C
BYZANTINE MUSICAL SYMBOL RAPISMA

1D03D
BYZANTINE MUSICAL SYMBOL PARAKALESMA ARCHAION

1D03E
BYZANTINE MUSICAL SYMBOL PARAKLITIKI ARCHAION

1D03F
BYZANTINE MUSICAL SYMBOL ICHADIN

1D040
BYZANTINE MUSICAL SYMBOL NANA

1D041
BYZANTINE MUSICAL SYMBOL PETASMA

1D042
BYZANTINE MUSICAL SYMBOL KONTEVMA ALLO

1D043
BYZANTINE MUSICAL SYMBOL TROMIKON ALLO

1D044
BYZANTINE MUSICAL SYMBOL STRAGGISMATA

1D045
BYZANTINE MUSICAL SYMBOL GRONTHISMATA

1D046
BYZANTINE MUSICAL SYMBOL ISON NEO

1D047
BYZANTINE MUSICAL SYMBOL OLIGON NEO

1D048
BYZANTINE MUSICAL SYMBOL OXEIA NEO

1D049
BYZANTINE MUSICAL SYMBOL PETASTI

1D04A
BYZANTINE MUSICAL SYMBOL KOUFISMA

1D04B
BYZANTINE MUSICAL SYMBOL PETASTOKOUFISMA

1D04C
BYZANTINE MUSICAL SYMBOL KRATIMOKOUFISMA

1D04D
BYZANTINE MUSICAL SYMBOL PELASTON NEO

1D04E
BYZANTINE MUSICAL SYMBOL KENTIMATA NEO ANO

1D04F
BYZANTINE MUSICAL SYMBOL KENTIMA NEO ANO

1D050
BYZANTINE MUSICAL SYMBOL YPSILI

1D051
BYZANTINE MUSICAL SYMBOL APOSTROFOS NEO

1D052
BYZANTINE MUSICAL SYMBOL APOSTROFOI SYNDESMOS NEO

1D053
BYZANTINE MUSICAL SYMBOL YPORROI

1D054
BYZANTINE MUSICAL SYMBOL KRATIMOYPORROON

1D055
BYZANTINE MUSICAL SYMBOL ELAFRON

1D056
BYZANTINE MUSICAL SYMBOL CHAMILI

1D057
BYZANTINE MUSICAL SYMBOL MIKRON ISON

1D058
BYZANTINE MUSICAL SYMBOL VAREIA NEO

1D059
BYZANTINE MUSICAL SYMBOL PIASMA NEO

1D05A
BYZANTINE MUSICAL SYMBOL PSIFISTON NEO

1D05B
BYZANTINE MUSICAL SYMBOL OMALON

1D05C
BYZANTINE MUSICAL SYMBOL ANTIKENOMA

1D05D
BYZANTINE MUSICAL SYMBOL LYGISMA

1D05E
BYZANTINE MUSICAL SYMBOL PARAKLITIKI NEO

1D05F
BYZANTINE MUSICAL SYMBOL PARAKALESMA NEO

1D060
BYZANTINE MUSICAL SYMBOL ETERON PARAKALESMA

1D061
BYZANTINE MUSICAL SYMBOL KYLISMA

1D062
BYZANTINE MUSICAL SYMBOL ANTIKENOKYLISMA

1D063
BYZANTINE MUSICAL SYMBOL TROMIKON NEO

1D064
BYZANTINE MUSICAL SYMBOL EKSTREPTON

1D065
BYZANTINE MUSICAL SYMBOL SYNAGMA NEO

1D066
BYZANTINE MUSICAL SYMBOL SYRMA

1D067
BYZANTINE MUSICAL SYMBOL CHOREVMA NEO

1D068
BYZANTINE MUSICAL SYMBOL EPEGERMA

1D069
BYZANTINE MUSICAL SYMBOL SEISMA NEO

1D06A
BYZANTINE MUSICAL SYMBOL XIRON KLASMA

1D06B
BYZANTINE MUSICAL SYMBOL TROMIKOPSIFISTON

1D06C
BYZANTINE MUSICAL SYMBOL PSIFISTOLYGISMA

1D06D
BYZANTINE MUSICAL SYMBOL TROMIKOLYGISMA

1D06E
BYZANTINE MUSICAL SYMBOL TROMIKOPARAKALESMA

1D06F
BYZANTINE MUSICAL SYMBOL PSIFISTOPARAKALESMA

1D070
BYZANTINE MUSICAL SYMBOL TROMIKOSYNAGMA

1D071
BYZANTINE MUSICAL SYMBOL PSIFISTOSYNAGMA

1D072
BYZANTINE MUSICAL SYMBOL GORGOSYNTHETON

1D073
BYZANTINE MUSICAL SYMBOL ARGOSYNTHETON

1D074
BYZANTINE MUSICAL SYMBOL ETERON ARGOSYNTHETON

1D075
BYZANTINE MUSICAL SYMBOL OYRANISMA NEO

1D076
BYZANTINE MUSICAL SYMBOL THEMATISMOS ESO

1D077
BYZANTINE MUSICAL SYMBOL THEMATISMOS EXO

1D078
BYZANTINE MUSICAL SYMBOL THEMA APLOUN

1D079
BYZANTINE MUSICAL SYMBOL THES KAI APOTHES

1D07A
BYZANTINE MUSICAL SYMBOL KATAVASMA

1D07B
BYZANTINE MUSICAL SYMBOL ENDOFONON

1D07C
BYZANTINE MUSICAL SYMBOL YFEN KATO

1D07D
BYZANTINE MUSICAL SYMBOL YFEN ANO

1D07E
BYZANTINE MUSICAL SYMBOL STAVROS

1D07F
BYZANTINE MUSICAL SYMBOL KLASMA ANO

1D080
BYZANTINE MUSICAL SYMBOL DIPLI ARCHAION

1D081
BYZANTINE MUSICAL SYMBOL KRATIMA ARCHAION

1D082
BYZANTINE MUSICAL SYMBOL KRATIMA ALLO

1D083
BYZANTINE MUSICAL SYMBOL KRATIMA NEO

1D084
BYZANTINE MUSICAL SYMBOL APODERMA NEO

1D085
BYZANTINE MUSICAL SYMBOL APLI

1D086
BYZANTINE MUSICAL SYMBOL DIPLI

1D087
BYZANTINE MUSICAL SYMBOL TRIPLI

1D088
BYZANTINE MUSICAL SYMBOL TETRAPLI

1D089
BYZANTINE MUSICAL SYMBOL KORONIS

1D08A
BYZANTINE MUSICAL SYMBOL LEIMMA ENOS CHRONOU

1D08B
BYZANTINE MUSICAL SYMBOL LEIMMA DYO CHRONON

1D08C
BYZANTINE MUSICAL SYMBOL LEIMMA TRION CHRONON

1D08D
BYZANTINE MUSICAL SYMBOL LEIMMA TESSARON CHRONON

1D08E
BYZANTINE MUSICAL SYMBOL LEIMMA IMISEOS CHRONOU

1D08F
BYZANTINE MUSICAL SYMBOL GORGON NEO ANO

1D090
BYZANTINE MUSICAL SYMBOL GORGON PARESTIGMENON ARISTERA

1D091
BYZANTINE MUSICAL SYMBOL GORGON PARESTIGMENON DEXIA

1D092
BYZANTINE MUSICAL SYMBOL DIGORGON

1D093
BYZANTINE MUSICAL SYMBOL DIGORGON PARESTIGMENON ARISTERA KATO

1D094
BYZANTINE MUSICAL SYMBOL DIGORGON PARESTIGMENON ARISTERA ANO

1D095
BYZANTINE MUSICAL SYMBOL DIGORGON PARESTIGMENON DEXIA

1D096
BYZANTINE MUSICAL SYMBOL TRIGORGON

1D097
BYZANTINE MUSICAL SYMBOL ARGON

1D098
BYZANTINE MUSICAL SYMBOL IMIDIARGON

1D099
BYZANTINE MUSICAL SYMBOL DIARGON

1D09A
BYZANTINE MUSICAL SYMBOL AGOGI POLI ARGI

1D09B
BYZANTINE MUSICAL SYMBOL AGOGI ARGOTERI

1D09C
BYZANTINE MUSICAL SYMBOL AGOGI ARGI

1D09D
BYZANTINE MUSICAL SYMBOL AGOGI METRIA

1D09E
BYZANTINE MUSICAL SYMBOL AGOGI MESI

1D09F
BYZANTINE MUSICAL SYMBOL AGOGI GORGI

1D0A0
BYZANTINE MUSICAL SYMBOL AGOGI GORGOTERI

1D0A1
BYZANTINE MUSICAL SYMBOL AGOGI POLI GORGI

1D0A2
BYZANTINE MUSICAL SYMBOL MARTYRIA PROTOS ICHOS

1D0A3
BYZANTINE MUSICAL SYMBOL MARTYRIA ALLI PROTOS ICHOS

1D0A4
BYZANTINE MUSICAL SYMBOL MARTYRIA DEYTEROS ICHOS

1D0A5
BYZANTINE MUSICAL SYMBOL MARTYRIA ALLI DEYTEROS ICHOS

1D0A6
BYZANTINE MUSICAL SYMBOL MARTYRIA TRITOS ICHOS

1D0A7
BYZANTINE MUSICAL SYMBOL MARTYRIA TRIFONIAS

1D0A8
BYZANTINE MUSICAL SYMBOL MARTYRIA TETARTOS ICHOS

1D0A9
BYZANTINE MUSICAL SYMBOL MARTYRIA TETARTOS LEGETOS ICHOS

1D0AA
BYZANTINE MUSICAL SYMBOL MARTYRIA LEGETOS ICHOS

1D0AB
BYZANTINE MUSICAL SYMBOL MARTYRIA PLAGIOS ICHOS

1D0AC
BYZANTINE MUSICAL SYMBOL ISAKIA TELOUS ICHIMATOS

1D0AD
BYZANTINE MUSICAL SYMBOL APOSTROFOI TELOUS ICHIMATOS

1D0AE
BYZANTINE MUSICAL SYMBOL FANEROSIS TETRAFONIAS

1D0AF
BYZANTINE MUSICAL SYMBOL FANEROSIS MONOFONIAS

1D0B0
BYZANTINE MUSICAL SYMBOL FANEROSIS DIFONIAS

1D0B1
BYZANTINE MUSICAL SYMBOL MARTYRIA VARYS ICHOS

1D0B2
BYZANTINE MUSICAL SYMBOL MARTYRIA PROTOVARYS ICHOS

1D0B3
BYZANTINE MUSICAL SYMBOL MARTYRIA PLAGIOS TETARTOS ICHOS

1D0B4
BYZANTINE MUSICAL SYMBOL GORTHMIKON N APLOUN

1D0B5
BYZANTINE MUSICAL SYMBOL GORTHMIKON N DIPLOUN

1D0B6
BYZANTINE MUSICAL SYMBOL ENARXIS KAI FTHORA VOU

1D0B7
BYZANTINE MUSICAL SYMBOL IMIFONON

1D0B8
BYZANTINE MUSICAL SYMBOL IMIFTHORON

1D0B9
BYZANTINE MUSICAL SYMBOL FTHORA ARCHAION DEYTEROU ICHOU

1D0BA
BYZANTINE MUSICAL SYMBOL FTHORA DIATONIKI PA

1D0BB
BYZANTINE MUSICAL SYMBOL FTHORA DIATONIKI NANA

1D0BC
BYZANTINE MUSICAL SYMBOL FTHORA NAOS ICHOS

1D0BD
BYZANTINE MUSICAL SYMBOL FTHORA DIATONIKI DI

1D0BE
BYZANTINE MUSICAL SYMBOL FTHORA SKLIRON DIATONON DI

1D0BF
BYZANTINE MUSICAL SYMBOL FTHORA DIATONIKI KE

1D0C0
BYZANTINE MUSICAL SYMBOL FTHORA DIATONIKI ZO

1D0C1
BYZANTINE MUSICAL SYMBOL FTHORA DIATONIKI NI KATO

1D0C2
BYZANTINE MUSICAL SYMBOL FTHORA DIATONIKI NI ANO

1D0C3
BYZANTINE MUSICAL SYMBOL FTHORA MALAKON CHROMA DIFONIAS

1D0C4
BYZANTINE MUSICAL SYMBOL FTHORA MALAKON CHROMA MONOFONIAS

1D0C5
BYZANTINE MUSICAL SYMBOL FTHORA SKLIRON CHROMA VASIS

1D0C5
BYZANTINE MUSICAL SYMBOL FHTORA SKLIRON CHROMA VASIS

1D0C6
BYZANTINE MUSICAL SYMBOL FTHORA SKLIRON CHROMA SYNAFI

1D0C7
BYZANTINE MUSICAL SYMBOL FTHORA NENANO

1D0C8
BYZANTINE MUSICAL SYMBOL CHROA ZYGOS

1D0C9
BYZANTINE MUSICAL SYMBOL CHROA KLITON

1D0CA
BYZANTINE MUSICAL SYMBOL CHROA SPATHI

1D0CB
BYZANTINE MUSICAL SYMBOL FTHORA I YFESIS TETARTIMORION

1D0CC
BYZANTINE MUSICAL SYMBOL FTHORA ENARMONIOS ANTIFONIA

1D0CD
BYZANTINE MUSICAL SYMBOL YFESIS TRITIMORION

1D0CE
BYZANTINE MUSICAL SYMBOL DIESIS TRITIMORION

1D0CF
BYZANTINE MUSICAL SYMBOL DIESIS TETARTIMORION

1D0D0
BYZANTINE MUSICAL SYMBOL DIESIS APLI DYO DODEKATA

1D0D1
BYZANTINE MUSICAL SYMBOL DIESIS MONOGRAMMOS TESSERA DODEKATA

1D0D2
BYZANTINE MUSICAL SYMBOL DIESIS DIGRAMMOS EX DODEKATA

1D0D3
BYZANTINE MUSICAL SYMBOL DIESIS TRIGRAMMOS OKTO DODEKATA

1D0D4
BYZANTINE MUSICAL SYMBOL YFESIS APLI DYO DODEKATA

1D0D5
BYZANTINE MUSICAL SYMBOL YFESIS MONOGRAMMOS TESSERA DODEKATA

1D0D6
BYZANTINE MUSICAL SYMBOL YFESIS DIGRAMMOS EX DODEKATA

1D0D7
BYZANTINE MUSICAL SYMBOL YFESIS TRIGRAMMOS OKTO DODEKATA

1D0D8
BYZANTINE MUSICAL SYMBOL GENIKI DIESIS

1D0D9
BYZANTINE MUSICAL SYMBOL GENIKI YFESIS

1D0DA
BYZANTINE MUSICAL SYMBOL DIASTOLI APLI MIKRI

1D0DB
BYZANTINE MUSICAL SYMBOL DIASTOLI APLI MEGALI

1D0DC
BYZANTINE MUSICAL SYMBOL DIASTOLI DIPLI

1D0DD
BYZANTINE MUSICAL SYMBOL DIASTOLI THESEOS

1D0DE
BYZANTINE MUSICAL SYMBOL SIMANSIS THESEOS

1D0DF
BYZANTINE MUSICAL SYMBOL SIMANSIS THESEOS DISIMOU

1D0E0
BYZANTINE MUSICAL SYMBOL SIMANSIS THESEOS TRISIMOU

1D0E1
BYZANTINE MUSICAL SYMBOL SIMANSIS THESEOS TETRASIMOU

1D0E2
BYZANTINE MUSICAL SYMBOL SIMANSIS ARSEOS

1D0E3
BYZANTINE MUSICAL SYMBOL SIMANSIS ARSEOS DISIMOU

1D0E4
BYZANTINE MUSICAL SYMBOL SIMANSIS ARSEOS TRISIMOU

1D0E5
BYZANTINE MUSICAL SYMBOL SIMANSIS ARSEOS TETRASIMOU

1D0E6
BYZANTINE MUSICAL SYMBOL DIGRAMMA GG

1D0E7
BYZANTINE MUSICAL SYMBOL DIFTOGGOS OU

1D0E8
BYZANTINE MUSICAL SYMBOL STIGMA

1D0E9
BYZANTINE MUSICAL SYMBOL ARKTIKO PA

1D0EA
BYZANTINE MUSICAL SYMBOL ARKTIKO VOU

1D0EB
BYZANTINE MUSICAL SYMBOL ARKTIKO GA

1D0EC
BYZANTINE MUSICAL SYMBOL ARKTIKO DI

1D0ED
BYZANTINE MUSICAL SYMBOL ARKTIKO KE

1D0EE
BYZANTINE MUSICAL SYMBOL ARKTIKO ZO

1D0EF
BYZANTINE MUSICAL SYMBOL ARKTIKO NI

1D0F0
BYZANTINE MUSICAL SYMBOL KENTIMATA NEO MESO

1D0F1
BYZANTINE MUSICAL SYMBOL KENTIMA NEO MESO

1D0F2
BYZANTINE MUSICAL SYMBOL KENTIMATA NEO KATO

1D0F3
BYZANTINE MUSICAL SYMBOL KENTIMA NEO KATO

1D0F4
BYZANTINE MUSICAL SYMBOL KLASMA KATO

1D0F5
BYZANTINE MUSICAL SYMBOL GORGON NEO KATO

1D100
MUSICAL SYMBOL SINGLE BARLINE

1D101
MUSICAL SYMBOL DOUBLE BARLINE

1D102
MUSICAL SYMBOL FINAL BARLINE

1D103
MUSICAL SYMBOL REVERSE FINAL BARLINE

1D104
MUSICAL SYMBOL DASHED BARLINE

1D105
MUSICAL SYMBOL SHORT BARLINE

1D106
MUSICAL SYMBOL LEFT REPEAT SIGN

1D107
MUSICAL SYMBOL RIGHT REPEAT SIGN

1D108
MUSICAL SYMBOL REPEAT DOTS

1D109
MUSICAL SYMBOL DAL SEGNO

1D10A
MUSICAL SYMBOL DA CAPO

1D10B
MUSICAL SYMBOL SEGNO

1D10C
MUSICAL SYMBOL CODA

1D10D
MUSICAL SYMBOL REPEATED FIGURE-1

1D10E
MUSICAL SYMBOL REPEATED FIGURE-2

1D10F
MUSICAL SYMBOL REPEATED FIGURE-3

1D110
MUSICAL SYMBOL FERMATA

1D111
MUSICAL SYMBOL FERMATA BELOW

1D112
MUSICAL SYMBOL BREATH MARK

1D113
MUSICAL SYMBOL CAESURA

1D114
MUSICAL SYMBOL BRACE

1D115
MUSICAL SYMBOL BRACKET

1D116
MUSICAL SYMBOL ONE-LINE STAFF

1D117
MUSICAL SYMBOL TWO-LINE STAFF

1D118
MUSICAL SYMBOL THREE-LINE STAFF

1D119
MUSICAL SYMBOL FOUR-LINE STAFF

1D11A
MUSICAL SYMBOL FIVE-LINE STAFF

1D11B
MUSICAL SYMBOL SIX-LINE STAFF

1D11C
MUSICAL SYMBOL SIX-STRING FRETBOARD

1D11D
MUSICAL SYMBOL FOUR-STRING FRETBOARD

1D11E
MUSICAL SYMBOL G CLEF

1D11F
MUSICAL SYMBOL G CLEF OTTAVA ALTA

1D120
MUSICAL SYMBOL G CLEF OTTAVA BASSA

1D121
MUSICAL SYMBOL C CLEF

1D122
MUSICAL SYMBOL F CLEF

1D123
MUSICAL SYMBOL F CLEF OTTAVA ALTA

1D124
MUSICAL SYMBOL F CLEF OTTAVA BASSA

1D125
MUSICAL SYMBOL DRUM CLEF-1

1D126
MUSICAL SYMBOL DRUM CLEF-2

1D129
MUSICAL SYMBOL MULTIPLE MEASURE REST

1D12A
MUSICAL SYMBOL DOUBLE SHARP

1D12B
MUSICAL SYMBOL DOUBLE FLAT

1D12C
MUSICAL SYMBOL FLAT UP

1D12D
MUSICAL SYMBOL FLAT DOWN

1D12E
MUSICAL SYMBOL NATURAL UP

1D12F
MUSICAL SYMBOL NATURAL DOWN

1D130
MUSICAL SYMBOL SHARP UP

1D131
MUSICAL SYMBOL SHARP DOWN

1D132
MUSICAL SYMBOL QUARTER TONE SHARP

1D133
MUSICAL SYMBOL QUARTER TONE FLAT

1D134
MUSICAL SYMBOL COMMON TIME

1D135
MUSICAL SYMBOL CUT TIME

1D136
MUSICAL SYMBOL OTTAVA ALTA

1D137
MUSICAL SYMBOL OTTAVA BASSA

1D138
MUSICAL SYMBOL QUINDICESIMA ALTA

1D139
MUSICAL SYMBOL QUINDICESIMA BASSA

1D13A
MUSICAL SYMBOL MULTI REST

1D13B
MUSICAL SYMBOL WHOLE REST

1D13C
MUSICAL SYMBOL HALF REST

1D13D
MUSICAL SYMBOL QUARTER REST

1D13E
MUSICAL SYMBOL EIGHTH REST

1D13F
MUSICAL SYMBOL SIXTEENTH REST

1D140
MUSICAL SYMBOL THIRTY-SECOND REST

1D141
MUSICAL SYMBOL SIXTY-FOURTH REST

1D142
MUSICAL SYMBOL ONE HUNDRED TWENTY-EIGHTH REST

1D143
MUSICAL SYMBOL X NOTEHEAD

1D144
MUSICAL SYMBOL PLUS NOTEHEAD

1D145
MUSICAL SYMBOL CIRCLE X NOTEHEAD

1D146
MUSICAL SYMBOL SQUARE NOTEHEAD WHITE

1D147
MUSICAL SYMBOL SQUARE NOTEHEAD BLACK

1D148
MUSICAL SYMBOL TRIANGLE NOTEHEAD UP WHITE

1D149
MUSICAL SYMBOL TRIANGLE NOTEHEAD UP BLACK

1D14A
MUSICAL SYMBOL TRIANGLE NOTEHEAD LEFT WHITE

1D14B
MUSICAL SYMBOL TRIANGLE NOTEHEAD LEFT BLACK

1D14C
MUSICAL SYMBOL TRIANGLE NOTEHEAD RIGHT WHITE

1D14D
MUSICAL SYMBOL TRIANGLE NOTEHEAD RIGHT BLACK

1D14E
MUSICAL SYMBOL TRIANGLE NOTEHEAD DOWN WHITE

1D14F
MUSICAL SYMBOL TRIANGLE NOTEHEAD DOWN BLACK

1D150
MUSICAL SYMBOL TRIANGLE NOTEHEAD UP RIGHT WHITE

1D151
MUSICAL SYMBOL TRIANGLE NOTEHEAD UP RIGHT BLACK

1D152
MUSICAL SYMBOL MOON NOTEHEAD WHITE

1D153
MUSICAL SYMBOL MOON NOTEHEAD BLACK

1D154
MUSICAL SYMBOL TRIANGLE-ROUND NOTEHEAD DOWN WHITE

1D155
MUSICAL SYMBOL TRIANGLE-ROUND NOTEHEAD DOWN BLACK

1D156
MUSICAL SYMBOL PARENTHESIS NOTEHEAD

1D157
MUSICAL SYMBOL VOID NOTEHEAD

1D158
MUSICAL SYMBOL NOTEHEAD BLACK

1D159
MUSICAL SYMBOL NULL NOTEHEAD

1D15A
MUSICAL SYMBOL CLUSTER NOTEHEAD WHITE

1D15B
MUSICAL SYMBOL CLUSTER NOTEHEAD BLACK

1D15C
MUSICAL SYMBOL BREVE

1D15D
MUSICAL SYMBOL WHOLE NOTE

1D15E
MUSICAL SYMBOL HALF NOTE

1D15F
MUSICAL SYMBOL QUARTER NOTE

1D160
MUSICAL SYMBOL EIGHTH NOTE

1D161
MUSICAL SYMBOL SIXTEENTH NOTE

1D162
MUSICAL SYMBOL THIRTY-SECOND NOTE

1D163
MUSICAL SYMBOL SIXTY-FOURTH NOTE

1D164
MUSICAL SYMBOL ONE HUNDRED TWENTY-EIGHTH NOTE

1D165
MUSICAL SYMBOL COMBINING STEM

1D166
MUSICAL SYMBOL COMBINING SPRECHGESANG STEM

1D167
MUSICAL SYMBOL COMBINING TREMOLO-1

1D168
MUSICAL SYMBOL COMBINING TREMOLO-2

1D169
MUSICAL SYMBOL COMBINING TREMOLO-3

1D16A
MUSICAL SYMBOL FINGERED TREMOLO-1

1D16B
MUSICAL SYMBOL FINGERED TREMOLO-2

1D16C
MUSICAL SYMBOL FINGERED TREMOLO-3

1D16D
MUSICAL SYMBOL COMBINING AUGMENTATION DOT

1D16E
MUSICAL SYMBOL COMBINING FLAG-1

1D16F
MUSICAL SYMBOL COMBINING FLAG-2

1D170
MUSICAL SYMBOL COMBINING FLAG-3

1D171
MUSICAL SYMBOL COMBINING FLAG-4

1D172
MUSICAL SYMBOL COMBINING FLAG-5

1D173
MUSICAL SYMBOL BEGIN BEAM

1D174
MUSICAL SYMBOL END BEAM

1D175
MUSICAL SYMBOL BEGIN TIE

1D176
MUSICAL SYMBOL END TIE

1D177
MUSICAL SYMBOL BEGIN SLUR

1D178
MUSICAL SYMBOL END SLUR

1D179
MUSICAL SYMBOL BEGIN PHRASE

1D17A
MUSICAL SYMBOL END PHRASE

1D17B
MUSICAL SYMBOL COMBINING ACCENT

1D17C
MUSICAL SYMBOL COMBINING STACCATO

1D17D
MUSICAL SYMBOL COMBINING TENUTO

1D17E
MUSICAL SYMBOL COMBINING STACCATISSIMO

1D17F
MUSICAL SYMBOL COMBINING MARCATO

1D180
MUSICAL SYMBOL COMBINING MARCATO-STACCATO

1D181
MUSICAL SYMBOL COMBINING ACCENT-STACCATO

1D182
MUSICAL SYMBOL COMBINING LOURE

1D183
MUSICAL SYMBOL ARPEGGIATO UP

1D184
MUSICAL SYMBOL ARPEGGIATO DOWN

1D185
MUSICAL SYMBOL COMBINING DOIT

1D186
MUSICAL SYMBOL COMBINING RIP

1D187
MUSICAL SYMBOL COMBINING FLIP

1D188
MUSICAL SYMBOL COMBINING SMEAR

1D189
MUSICAL SYMBOL COMBINING BEND

1D18A
MUSICAL SYMBOL COMBINING DOUBLE TONGUE

1D18B
MUSICAL SYMBOL COMBINING TRIPLE TONGUE

1D18C
MUSICAL SYMBOL RINFORZANDO

1D18D
MUSICAL SYMBOL SUBITO

1D18E
MUSICAL SYMBOL Z

1D18F
MUSICAL SYMBOL PIANO

1D190
MUSICAL SYMBOL MEZZO

1D191
MUSICAL SYMBOL FORTE

1D192
MUSICAL SYMBOL CRESCENDO

1D193
MUSICAL SYMBOL DECRESCENDO

1D194
MUSICAL SYMBOL GRACE NOTE SLASH

1D195
MUSICAL SYMBOL GRACE NOTE NO SLASH

1D196
MUSICAL SYMBOL TR

1D197
MUSICAL SYMBOL TURN

1D198
MUSICAL SYMBOL INVERTED TURN

1D199
MUSICAL SYMBOL TURN SLASH

1D19A
MUSICAL SYMBOL TURN UP

1D19B
MUSICAL SYMBOL ORNAMENT STROKE-1

1D19C
MUSICAL SYMBOL ORNAMENT STROKE-2

1D19D
MUSICAL SYMBOL ORNAMENT STROKE-3

1D19E
MUSICAL SYMBOL ORNAMENT STROKE-4

1D19F
MUSICAL SYMBOL ORNAMENT STROKE-5

1D1A0
MUSICAL SYMBOL ORNAMENT STROKE-6

1D1A1
MUSICAL SYMBOL ORNAMENT STROKE-7

1D1A2
MUSICAL SYMBOL ORNAMENT STROKE-8

1D1A3
MUSICAL SYMBOL ORNAMENT STROKE-9

1D1A4
MUSICAL SYMBOL ORNAMENT STROKE-10

1D1A5
MUSICAL SYMBOL ORNAMENT STROKE-11

1D1A6
MUSICAL SYMBOL HAUPTSTIMME

1D1A7
MUSICAL SYMBOL NEBENSTIMME

1D1A8
MUSICAL SYMBOL END OF STIMME

1D1A9
MUSICAL SYMBOL DEGREE SLASH

1D1AA
MUSICAL SYMBOL COMBINING DOWN BOW

1D1AB
MUSICAL SYMBOL COMBINING UP BOW

1D1AC
MUSICAL SYMBOL COMBINING HARMONIC

1D1AD
MUSICAL SYMBOL COMBINING SNAP PIZZICATO

1D1AE
MUSICAL SYMBOL PEDAL MARK

1D1AF
MUSICAL SYMBOL PEDAL UP MARK

1D1B0
MUSICAL SYMBOL HALF PEDAL MARK

1D1B1
MUSICAL SYMBOL GLISSANDO UP

1D1B2
MUSICAL SYMBOL GLISSANDO DOWN

1D1B3
MUSICAL SYMBOL WITH FINGERNAILS

1D1B4
MUSICAL SYMBOL DAMP

1D1B5
MUSICAL SYMBOL DAMP ALL

1D1B6
MUSICAL SYMBOL MAXIMA

1D1B7
MUSICAL SYMBOL LONGA

1D1B8
MUSICAL SYMBOL BREVIS

1D1B9
MUSICAL SYMBOL SEMIBREVIS WHITE

1D1BA
MUSICAL SYMBOL SEMIBREVIS BLACK

1D1BB
MUSICAL SYMBOL MINIMA

1D1BC
MUSICAL SYMBOL MINIMA BLACK

1D1BD
MUSICAL SYMBOL SEMIMINIMA WHITE

1D1BE
MUSICAL SYMBOL SEMIMINIMA BLACK

1D1BF
MUSICAL SYMBOL FUSA WHITE

1D1C0
MUSICAL SYMBOL FUSA BLACK

1D1C1
MUSICAL SYMBOL LONGA PERFECTA REST

1D1C2
MUSICAL SYMBOL LONGA IMPERFECTA REST

1D1C3
MUSICAL SYMBOL BREVIS REST

1D1C4
MUSICAL SYMBOL SEMIBREVIS REST

1D1C5
MUSICAL SYMBOL MINIMA REST

1D1C6
MUSICAL SYMBOL SEMIMINIMA REST

1D1C7
MUSICAL SYMBOL TEMPUS PERFECTUM CUM PROLATIONE PERFECTA

1D1C8
MUSICAL SYMBOL TEMPUS PERFECTUM CUM PROLATIONE IMPERFECTA

1D1C9
MUSICAL SYMBOL TEMPUS PERFECTUM CUM PROLATIONE PERFECTA DIMINUTION-1

1D1CA
MUSICAL SYMBOL TEMPUS IMPERFECTUM CUM PROLATIONE PERFECTA

1D1CB
MUSICAL SYMBOL TEMPUS IMPERFECTUM CUM PROLATIONE IMPERFECTA

1D1CC
MUSICAL SYMBOL TEMPUS IMPERFECTUM CUM PROLATIONE IMPERFECTA DIMINUTION-1

1D1CD
MUSICAL SYMBOL TEMPUS IMPERFECTUM CUM PROLATIONE IMPERFECTA DIMINUTION-2

1D1CE
MUSICAL SYMBOL TEMPUS IMPERFECTUM CUM PROLATIONE IMPERFECTA DIMINUTION-3

1D1CF
MUSICAL SYMBOL CROIX

1D1D0
MUSICAL SYMBOL GREGORIAN C CLEF

1D1D1
MUSICAL SYMBOL GREGORIAN F CLEF

1D1D2
MUSICAL SYMBOL SQUARE B

1D1D3
MUSICAL SYMBOL VIRGA

1D1D4
MUSICAL SYMBOL PODATUS

1D1D5
MUSICAL SYMBOL CLIVIS

1D1D6
MUSICAL SYMBOL SCANDICUS

1D1D7
MUSICAL SYMBOL CLIMACUS

1D1D8
MUSICAL SYMBOL TORCULUS

1D1D9
MUSICAL SYMBOL PORRECTUS

1D1DA
MUSICAL SYMBOL PORRECTUS FLEXUS

1D1DB
MUSICAL SYMBOL SCANDICUS FLEXUS

1D1DC
MUSICAL SYMBOL TORCULUS RESUPINUS

1D1DD
MUSICAL SYMBOL PES SUBPUNCTIS

1D1DE
MUSICAL SYMBOL KIEVAN C CLEF

1D1DF
MUSICAL SYMBOL KIEVAN END OF PIECE

1D1E0
MUSICAL SYMBOL KIEVAN FINAL NOTE

1D1E1
MUSICAL SYMBOL KIEVAN RECITATIVE MARK

1D1E2
MUSICAL SYMBOL KIEVAN WHOLE NOTE

1D1E3
MUSICAL SYMBOL KIEVAN HALF NOTE

1D1E4
MUSICAL SYMBOL KIEVAN QUARTER NOTE STEM DOWN

1D1E5
MUSICAL SYMBOL KIEVAN QUARTER NOTE STEM UP

1D1E6
MUSICAL SYMBOL KIEVAN EIGHTH NOTE STEM DOWN

1D1E7
MUSICAL SYMBOL KIEVAN EIGHTH NOTE STEM UP

1D1E8
MUSICAL SYMBOL KIEVAN FLAT SIGN

1D1E9
MUSICAL SYMBOL SORI

1D1EA
MUSICAL SYMBOL KORON

1D200
GREEK VOCAL NOTATION SYMBOL-1

1D201
GREEK VOCAL NOTATION SYMBOL-2

1D202
GREEK VOCAL NOTATION SYMBOL-3

1D203
GREEK VOCAL NOTATION SYMBOL-4

1D204
GREEK VOCAL NOTATION SYMBOL-5

1D205
GREEK VOCAL NOTATION SYMBOL-6

1D206
GREEK VOCAL NOTATION SYMBOL-7

1D207
GREEK VOCAL NOTATION SYMBOL-8

1D208
GREEK VOCAL NOTATION SYMBOL-9

1D209
GREEK VOCAL NOTATION SYMBOL-10

1D20A
GREEK VOCAL NOTATION SYMBOL-11

1D20B
GREEK VOCAL NOTATION SYMBOL-12

1D20C
GREEK VOCAL NOTATION SYMBOL-13

1D20D
GREEK VOCAL NOTATION SYMBOL-14

1D20E
GREEK VOCAL NOTATION SYMBOL-15

1D20F
GREEK VOCAL NOTATION SYMBOL-16

1D210
GREEK VOCAL NOTATION SYMBOL-17

1D211
GREEK VOCAL NOTATION SYMBOL-18

1D212
GREEK VOCAL NOTATION SYMBOL-19

1D213
GREEK VOCAL NOTATION SYMBOL-20

1D214
GREEK VOCAL NOTATION SYMBOL-21

1D215
GREEK VOCAL NOTATION SYMBOL-22

1D216
GREEK VOCAL NOTATION SYMBOL-23

1D217
GREEK VOCAL NOTATION SYMBOL-24

1D218
GREEK VOCAL NOTATION SYMBOL-50

1D219
GREEK VOCAL NOTATION SYMBOL-51

1D21A
GREEK VOCAL NOTATION SYMBOL-52

1D21B
GREEK VOCAL NOTATION SYMBOL-53

1D21C
GREEK VOCAL NOTATION SYMBOL-54

1D21D
GREEK INSTRUMENTAL NOTATION SYMBOL-1

1D21E
GREEK INSTRUMENTAL NOTATION SYMBOL-2

1D21F
GREEK INSTRUMENTAL NOTATION SYMBOL-4

1D220
GREEK INSTRUMENTAL NOTATION SYMBOL-5

1D221
GREEK INSTRUMENTAL NOTATION SYMBOL-7

1D222
GREEK INSTRUMENTAL NOTATION SYMBOL-8

1D223
GREEK INSTRUMENTAL NOTATION SYMBOL-11

1D224
GREEK INSTRUMENTAL NOTATION SYMBOL-12

1D225
GREEK INSTRUMENTAL NOTATION SYMBOL-13

1D226
GREEK INSTRUMENTAL NOTATION SYMBOL-14

1D227
GREEK INSTRUMENTAL NOTATION SYMBOL-17

1D228
GREEK INSTRUMENTAL NOTATION SYMBOL-18

1D229
GREEK INSTRUMENTAL NOTATION SYMBOL-19

1D22A
GREEK INSTRUMENTAL NOTATION SYMBOL-23

1D22B
GREEK INSTRUMENTAL NOTATION SYMBOL-24

1D22C
GREEK INSTRUMENTAL NOTATION SYMBOL-25

1D22D
GREEK INSTRUMENTAL NOTATION SYMBOL-26

1D22E
GREEK INSTRUMENTAL NOTATION SYMBOL-27

1D22F
GREEK INSTRUMENTAL NOTATION SYMBOL-29

1D230
GREEK INSTRUMENTAL NOTATION SYMBOL-30

1D231
GREEK INSTRUMENTAL NOTATION SYMBOL-32

1D232
GREEK INSTRUMENTAL NOTATION SYMBOL-36

1D233
GREEK INSTRUMENTAL NOTATION SYMBOL-37

1D234
GREEK INSTRUMENTAL NOTATION SYMBOL-38

1D235
GREEK INSTRUMENTAL NOTATION SYMBOL-39

1D236
GREEK INSTRUMENTAL NOTATION SYMBOL-40

1D237
GREEK INSTRUMENTAL NOTATION SYMBOL-42

1D238
GREEK INSTRUMENTAL NOTATION SYMBOL-43

1D239
GREEK INSTRUMENTAL NOTATION SYMBOL-45

1D23A
GREEK INSTRUMENTAL NOTATION SYMBOL-47

1D23B
GREEK INSTRUMENTAL NOTATION SYMBOL-48

1D23C
GREEK INSTRUMENTAL NOTATION SYMBOL-49

1D23D
GREEK INSTRUMENTAL NOTATION SYMBOL-50

1D23E
GREEK INSTRUMENTAL NOTATION SYMBOL-51

1D23F
GREEK INSTRUMENTAL NOTATION SYMBOL-52

1D240
GREEK INSTRUMENTAL NOTATION SYMBOL-53

1D241
GREEK INSTRUMENTAL NOTATION SYMBOL-54

1D242
COMBINING GREEK MUSICAL TRISEME

1D243
COMBINING GREEK MUSICAL TETRASEME

1D244
COMBINING GREEK MUSICAL PENTASEME

1D245
GREEK MUSICAL LEIMMA

1D2C0
KAKTOVIK NUMERAL ZERO

1D2C1
KAKTOVIK NUMERAL ONE

1D2C2
KAKTOVIK NUMERAL TWO

1D2C3
KAKTOVIK NUMERAL THREE

1D2C4
KAKTOVIK NUMERAL FOUR

1D2C5
KAKTOVIK NUMERAL FIVE

1D2C6
KAKTOVIK NUMERAL SIX

1D2C7
KAKTOVIK NUMERAL SEVEN

1D2C8
KAKTOVIK NUMERAL EIGHT

1D2C9
KAKTOVIK NUMERAL NINE

1D2CA
KAKTOVIK NUMERAL TEN

1D2CB
KAKTOVIK NUMERAL ELEVEN

1D2CC
KAKTOVIK NUMERAL TWELVE

1D2CD
KAKTOVIK NUMERAL THIRTEEN

1D2CE
KAKTOVIK NUMERAL FOURTEEN

1D2CF
KAKTOVIK NUMERAL FIFTEEN

1D2D0
KAKTOVIK NUMERAL SIXTEEN

1D2D1
KAKTOVIK NUMERAL SEVENTEEN

1D2D2
KAKTOVIK NUMERAL EIGHTEEN

1D2D3
KAKTOVIK NUMERAL NINETEEN

1D2E0
MAYAN NUMERAL ZERO

1D2E1
MAYAN NUMERAL ONE

1D2E2
MAYAN NUMERAL TWO

1D2E3
MAYAN NUMERAL THREE

1D2E4
MAYAN NUMERAL FOUR

1D2E5
MAYAN NUMERAL FIVE

1D2E6
MAYAN NUMERAL SIX

1D2E7
MAYAN NUMERAL SEVEN

1D2E8
MAYAN NUMERAL EIGHT

1D2E9
MAYAN NUMERAL NINE

1D2EA
MAYAN NUMERAL TEN

1D2EB
MAYAN NUMERAL ELEVEN

1D2EC
MAYAN NUMERAL TWELVE

1D2ED
MAYAN NUMERAL THIRTEEN

1D2EE
MAYAN NUMERAL FOURTEEN

1D2EF
MAYAN NUMERAL FIFTEEN

1D2F0
MAYAN NUMERAL SIXTEEN

1D2F1
MAYAN NUMERAL SEVENTEEN

1D2F2
MAYAN NUMERAL EIGHTEEN

1D2F3
MAYAN NUMERAL NINETEEN

1D300
MONOGRAM FOR EARTH

1D301
DIGRAM FOR HEAVENLY EARTH

1D302
DIGRAM FOR HUMAN EARTH

1D303
DIGRAM FOR EARTHLY HEAVEN

1D304
DIGRAM FOR EARTHLY HUMAN

1D305
DIGRAM FOR EARTH

1D306
TETRAGRAM FOR CENTRE

1D307
TETRAGRAM FOR FULL CIRCLE

1D308
TETRAGRAM FOR MIRED

1D309
TETRAGRAM FOR BARRIER

1D30A
TETRAGRAM FOR KEEPING SMALL

1D30B
TETRAGRAM FOR CONTRARIETY

1D30C
TETRAGRAM FOR ASCENT

1D30D
TETRAGRAM FOR OPPOSITION

1D30E
TETRAGRAM FOR BRANCHING OUT

1D30F
TETRAGRAM FOR DEFECTIVENESS OR DISTORTION

1D310
TETRAGRAM FOR DIVERGENCE

1D311
TETRAGRAM FOR YOUTHFULNESS

1D312
TETRAGRAM FOR INCREASE

1D313
TETRAGRAM FOR PENETRATION

1D314
TETRAGRAM FOR REACH

1D315
TETRAGRAM FOR CONTACT

1D316
TETRAGRAM FOR HOLDING BACK

1D317
TETRAGRAM FOR WAITING

1D318
TETRAGRAM FOR FOLLOWING

1D319
TETRAGRAM FOR ADVANCE

1D31A
TETRAGRAM FOR RELEASE

1D31B
TETRAGRAM FOR RESISTANCE

1D31C
TETRAGRAM FOR EASE

1D31D
TETRAGRAM FOR JOY

1D31E
TETRAGRAM FOR CONTENTION

1D31F
TETRAGRAM FOR ENDEAVOUR

1D320
TETRAGRAM FOR DUTIES

1D321
TETRAGRAM FOR CHANGE

1D322
TETRAGRAM FOR DECISIVENESS

1D323
TETRAGRAM FOR BOLD RESOLUTION

1D324
TETRAGRAM FOR PACKING

1D325
TETRAGRAM FOR LEGION

1D326
TETRAGRAM FOR CLOSENESS

1D327
TETRAGRAM FOR KINSHIP

1D328
TETRAGRAM FOR GATHERING

1D329
TETRAGRAM FOR STRENGTH

1D32A
TETRAGRAM FOR PURITY

1D32B
TETRAGRAM FOR FULLNESS

1D32C
TETRAGRAM FOR RESIDENCE

1D32D
TETRAGRAM FOR LAW OR MODEL

1D32E
TETRAGRAM FOR RESPONSE

1D32F
TETRAGRAM FOR GOING TO MEET

1D330
TETRAGRAM FOR ENCOUNTERS

1D331
TETRAGRAM FOR STOVE

1D332
TETRAGRAM FOR GREATNESS

1D333
TETRAGRAM FOR ENLARGEMENT

1D334
TETRAGRAM FOR PATTERN

1D335
TETRAGRAM FOR RITUAL

1D336
TETRAGRAM FOR FLIGHT

1D337
TETRAGRAM FOR VASTNESS OR WASTING

1D338
TETRAGRAM FOR CONSTANCY

1D339
TETRAGRAM FOR MEASURE

1D33A
TETRAGRAM FOR ETERNITY

1D33B
TETRAGRAM FOR UNITY

1D33C
TETRAGRAM FOR DIMINISHMENT

1D33D
TETRAGRAM FOR CLOSED MOUTH

1D33E
TETRAGRAM FOR GUARDEDNESS

1D33F
TETRAGRAM FOR GATHERING IN

1D340
TETRAGRAM FOR MASSING

1D341
TETRAGRAM FOR ACCUMULATION

1D342
TETRAGRAM FOR EMBELLISHMENT

1D343
TETRAGRAM FOR DOUBT

1D344
TETRAGRAM FOR WATCH

1D345
TETRAGRAM FOR SINKING

1D346
TETRAGRAM FOR INNER

1D347
TETRAGRAM FOR DEPARTURE

1D348
TETRAGRAM FOR DARKENING

1D349
TETRAGRAM FOR DIMMING

1D34A
TETRAGRAM FOR EXHAUSTION

1D34B
TETRAGRAM FOR SEVERANCE

1D34C
TETRAGRAM FOR STOPPAGE

1D34D
TETRAGRAM FOR HARDNESS

1D34E
TETRAGRAM FOR COMPLETION

1D34F
TETRAGRAM FOR CLOSURE

1D350
TETRAGRAM FOR FAILURE

1D351
TETRAGRAM FOR AGGRAVATION

1D352
TETRAGRAM FOR COMPLIANCE

1D353
TETRAGRAM FOR ON THE VERGE

1D354
TETRAGRAM FOR DIFFICULTIES

1D355
TETRAGRAM FOR LABOURING

1D356
TETRAGRAM FOR FOSTERING

1D360
COUNTING ROD UNIT DIGIT ONE

1D361
COUNTING ROD UNIT DIGIT TWO

1D362
COUNTING ROD UNIT DIGIT THREE

1D363
COUNTING ROD UNIT DIGIT FOUR

1D364
COUNTING ROD UNIT DIGIT FIVE

1D365
COUNTING ROD UNIT DIGIT SIX

1D366
COUNTING ROD UNIT DIGIT SEVEN

1D367
COUNTING ROD UNIT DIGIT EIGHT

1D368
COUNTING ROD UNIT DIGIT NINE

1D369
COUNTING ROD TENS DIGIT ONE

1D36A
COUNTING ROD TENS DIGIT TWO

1D36B
COUNTING ROD TENS DIGIT THREE

1D36C
COUNTING ROD TENS DIGIT FOUR

1D36D
COUNTING ROD TENS DIGIT FIVE

1D36E
COUNTING ROD TENS DIGIT SIX

1D36F
COUNTING ROD TENS DIGIT SEVEN

1D370
COUNTING ROD TENS DIGIT EIGHT

1D371
COUNTING ROD TENS DIGIT NINE

1D372
IDEOGRAPHIC TALLY MARK ONE

1D373
IDEOGRAPHIC TALLY MARK TWO

1D374
IDEOGRAPHIC TALLY MARK THREE

1D375
IDEOGRAPHIC TALLY MARK FOUR

1D376
IDEOGRAPHIC TALLY MARK FIVE

1D377
TALLY MARK ONE

1D378
TALLY MARK FIVE

1D400
MATHEMATICAL BOLD CAPITAL A

1D401
MATHEMATICAL BOLD CAPITAL B

1D402
MATHEMATICAL BOLD CAPITAL C

1D403
MATHEMATICAL BOLD CAPITAL D

1D404
MATHEMATICAL BOLD CAPITAL E

1D405
MATHEMATICAL BOLD CAPITAL F

1D406
MATHEMATICAL BOLD CAPITAL G

1D407
MATHEMATICAL BOLD CAPITAL H

1D408
MATHEMATICAL BOLD CAPITAL I

1D409
MATHEMATICAL BOLD CAPITAL J

1D40A
MATHEMATICAL BOLD CAPITAL K

1D40B
MATHEMATICAL BOLD CAPITAL L

1D40C
MATHEMATICAL BOLD CAPITAL M

1D40D
MATHEMATICAL BOLD CAPITAL N

1D40E
MATHEMATICAL BOLD CAPITAL O

1D40F
MATHEMATICAL BOLD CAPITAL P

1D410
MATHEMATICAL BOLD CAPITAL Q

1D411
MATHEMATICAL BOLD CAPITAL R

1D412
MATHEMATICAL BOLD CAPITAL S

1D413
MATHEMATICAL BOLD CAPITAL T

1D414
MATHEMATICAL BOLD CAPITAL U

1D415
MATHEMATICAL BOLD CAPITAL V

1D416
MATHEMATICAL BOLD CAPITAL W

1D417
MATHEMATICAL BOLD CAPITAL X

1D418
MATHEMATICAL BOLD CAPITAL Y

1D419
MATHEMATICAL BOLD CAPITAL Z

1D41A
MATHEMATICAL BOLD SMALL A

1D41B
MATHEMATICAL BOLD SMALL B

1D41C
MATHEMATICAL BOLD SMALL C

1D41D
MATHEMATICAL BOLD SMALL D

1D41E
MATHEMATICAL BOLD SMALL E

1D41F
MATHEMATICAL BOLD SMALL F

1D420
MATHEMATICAL BOLD SMALL G

1D421
MATHEMATICAL BOLD SMALL H

1D422
MATHEMATICAL BOLD SMALL I

1D423
MATHEMATICAL BOLD SMALL J

1D424
MATHEMATICAL BOLD SMALL K

1D425
MATHEMATICAL BOLD SMALL L

1D426
MATHEMATICAL BOLD SMALL M

1D427
MATHEMATICAL BOLD SMALL N

1D428
MATHEMATICAL BOLD SMALL O

1D429
MATHEMATICAL BOLD SMALL P

1D42A
MATHEMATICAL BOLD SMALL Q

1D42B
MATHEMATICAL BOLD SMALL R

1D42C
MATHEMATICAL BOLD SMALL S

1D42D
MATHEMATICAL BOLD SMALL T

1D42E
MATHEMATICAL BOLD SMALL U

1D42F
MATHEMATICAL BOLD SMALL V

1D430
MATHEMATICAL BOLD SMALL W

1D431
MATHEMATICAL BOLD SMALL X

1D432
MATHEMATICAL BOLD SMALL Y

1D433
MATHEMATICAL BOLD SMALL Z

1D434
MATHEMATICAL ITALIC CAPITAL A

1D435
MATHEMATICAL ITALIC CAPITAL B

1D436
MATHEMATICAL ITALIC CAPITAL C

1D437
MATHEMATICAL ITALIC CAPITAL D

1D438
MATHEMATICAL ITALIC CAPITAL E

1D439
MATHEMATICAL ITALIC CAPITAL F

1D43A
MATHEMATICAL ITALIC CAPITAL G

1D43B
MATHEMATICAL ITALIC CAPITAL H

1D43C
MATHEMATICAL ITALIC CAPITAL I

1D43D
MATHEMATICAL ITALIC CAPITAL J

1D43E
MATHEMATICAL ITALIC CAPITAL K

1D43F
MATHEMATICAL ITALIC CAPITAL L

1D440
MATHEMATICAL ITALIC CAPITAL M

1D441
MATHEMATICAL ITALIC CAPITAL N

1D442
MATHEMATICAL ITALIC CAPITAL O

1D443
MATHEMATICAL ITALIC CAPITAL P

1D444
MATHEMATICAL ITALIC CAPITAL Q

1D445
MATHEMATICAL ITALIC CAPITAL R

1D446
MATHEMATICAL ITALIC CAPITAL S

1D447
MATHEMATICAL ITALIC CAPITAL T

1D448
MATHEMATICAL ITALIC CAPITAL U

1D449
MATHEMATICAL ITALIC CAPITAL V

1D44A
MATHEMATICAL ITALIC CAPITAL W

1D44B
MATHEMATICAL ITALIC CAPITAL X

1D44C
MATHEMATICAL ITALIC CAPITAL Y

1D44D
MATHEMATICAL ITALIC CAPITAL Z

1D44E
MATHEMATICAL ITALIC SMALL A

1D44F
MATHEMATICAL ITALIC SMALL B

1D450
MATHEMATICAL ITALIC SMALL C

1D451
MATHEMATICAL ITALIC SMALL D

1D452
MATHEMATICAL ITALIC SMALL E

1D453
MATHEMATICAL ITALIC SMALL F

1D454
MATHEMATICAL ITALIC SMALL G

1D456
MATHEMATICAL ITALIC SMALL I

1D457
MATHEMATICAL ITALIC SMALL J

1D458
MATHEMATICAL ITALIC SMALL K

1D459
MATHEMATICAL ITALIC SMALL L

1D45A
MATHEMATICAL ITALIC SMALL M

1D45B
MATHEMATICAL ITALIC SMALL N

1D45C
MATHEMATICAL ITALIC SMALL O

1D45D
MATHEMATICAL ITALIC SMALL P

1D45E
MATHEMATICAL ITALIC SMALL Q

1D45F
MATHEMATICAL ITALIC SMALL R

1D460
MATHEMATICAL ITALIC SMALL S

1D461
MATHEMATICAL ITALIC SMALL T

1D462
MATHEMATICAL ITALIC SMALL U

1D463
MATHEMATICAL ITALIC SMALL V

1D464
MATHEMATICAL ITALIC SMALL W

1D465
MATHEMATICAL ITALIC SMALL X

1D466
MATHEMATICAL ITALIC SMALL Y

1D467
MATHEMATICAL ITALIC SMALL Z

1D468
MATHEMATICAL BOLD ITALIC CAPITAL A

1D469
MATHEMATICAL BOLD ITALIC CAPITAL B

1D46A
MATHEMATICAL BOLD ITALIC CAPITAL C

1D46B
MATHEMATICAL BOLD ITALIC CAPITAL D

1D46C
MATHEMATICAL BOLD ITALIC CAPITAL E

1D46D
MATHEMATICAL BOLD ITALIC CAPITAL F

1D46E
MATHEMATICAL BOLD ITALIC CAPITAL G

1D46F
MATHEMATICAL BOLD ITALIC CAPITAL H

1D470
MATHEMATICAL BOLD ITALIC CAPITAL I

1D471
MATHEMATICAL BOLD ITALIC CAPITAL J

1D472
MATHEMATICAL BOLD ITALIC CAPITAL K

1D473
MATHEMATICAL BOLD ITALIC CAPITAL L

1D474
MATHEMATICAL BOLD ITALIC CAPITAL M

1D475
MATHEMATICAL BOLD ITALIC CAPITAL N

1D476
MATHEMATICAL BOLD ITALIC CAPITAL O

1D477
MATHEMATICAL BOLD ITALIC CAPITAL P

1D478
MATHEMATICAL BOLD ITALIC CAPITAL Q

1D479
MATHEMATICAL BOLD ITALIC CAPITAL R

1D47A
MATHEMATICAL BOLD ITALIC CAPITAL S

1D47B
MATHEMATICAL BOLD ITALIC CAPITAL T

1D47C
MATHEMATICAL BOLD ITALIC CAPITAL U

1D47D
MATHEMATICAL BOLD ITALIC CAPITAL V

1D47E
MATHEMATICAL BOLD ITALIC CAPITAL W

1D47F
MATHEMATICAL BOLD ITALIC CAPITAL X

1D480
MATHEMATICAL BOLD ITALIC CAPITAL Y

1D481
MATHEMATICAL BOLD ITALIC CAPITAL Z

1D482
MATHEMATICAL BOLD ITALIC SMALL A

1D483
MATHEMATICAL BOLD ITALIC SMALL B

1D484
MATHEMATICAL BOLD ITALIC SMALL C

1D485
MATHEMATICAL BOLD ITALIC SMALL D

1D486
MATHEMATICAL BOLD ITALIC SMALL E

1D487
MATHEMATICAL BOLD ITALIC SMALL F

1D488
MATHEMATICAL BOLD ITALIC SMALL G

1D489
MATHEMATICAL BOLD ITALIC SMALL H

1D48A
MATHEMATICAL BOLD ITALIC SMALL I

1D48B
MATHEMATICAL BOLD ITALIC SMALL J

1D48C
MATHEMATICAL BOLD ITALIC SMALL K

1D48D
MATHEMATICAL BOLD ITALIC SMALL L

1D48E
MATHEMATICAL BOLD ITALIC SMALL M

1D48F
MATHEMATICAL BOLD ITALIC SMALL N

1D490
MATHEMATICAL BOLD ITALIC SMALL O

1D491
MATHEMATICAL BOLD ITALIC SMALL P

1D492
MATHEMATICAL BOLD ITALIC SMALL Q

1D493
MATHEMATICAL BOLD ITALIC SMALL R

1D494
MATHEMATICAL BOLD ITALIC SMALL S

1D495
MATHEMATICAL BOLD ITALIC SMALL T

1D496
MATHEMATICAL BOLD ITALIC SMALL U

1D497
MATHEMATICAL BOLD ITALIC SMALL V

1D498
MATHEMATICAL BOLD ITALIC SMALL W

1D499
MATHEMATICAL BOLD ITALIC SMALL X

1D49A
MATHEMATICAL BOLD ITALIC SMALL Y

1D49B
MATHEMATICAL BOLD ITALIC SMALL Z

1D49C
MATHEMATICAL SCRIPT CAPITAL A

1D49E
MATHEMATICAL SCRIPT CAPITAL C

1D49F
MATHEMATICAL SCRIPT CAPITAL D

1D4A2
MATHEMATICAL SCRIPT CAPITAL G

1D4A5
MATHEMATICAL SCRIPT CAPITAL J

1D4A6
MATHEMATICAL SCRIPT CAPITAL K

1D4A9
MATHEMATICAL SCRIPT CAPITAL N

1D4AA
MATHEMATICAL SCRIPT CAPITAL O

1D4AB
MATHEMATICAL SCRIPT CAPITAL P

1D4AC
MATHEMATICAL SCRIPT CAPITAL Q

1D4AE
MATHEMATICAL SCRIPT CAPITAL S

1D4AF
MATHEMATICAL SCRIPT CAPITAL T

1D4B0
MATHEMATICAL SCRIPT CAPITAL U

1D4B1
MATHEMATICAL SCRIPT CAPITAL V

1D4B2
MATHEMATICAL SCRIPT CAPITAL W

1D4B3
MATHEMATICAL SCRIPT CAPITAL X

1D4B4
MATHEMATICAL SCRIPT CAPITAL Y

1D4B5
MATHEMATICAL SCRIPT CAPITAL Z

1D4B6
MATHEMATICAL SCRIPT SMALL A

1D4B7
MATHEMATICAL SCRIPT SMALL B

1D4B8
MATHEMATICAL SCRIPT SMALL C

1D4B9
MATHEMATICAL SCRIPT SMALL D

1D4BB
MATHEMATICAL SCRIPT SMALL F

1D4BD
MATHEMATICAL SCRIPT SMALL H

1D4BE
MATHEMATICAL SCRIPT SMALL I

1D4BF
MATHEMATICAL SCRIPT SMALL J

1D4C0
MATHEMATICAL SCRIPT SMALL K

1D4C1
MATHEMATICAL SCRIPT SMALL L

1D4C2
MATHEMATICAL SCRIPT SMALL M

1D4C3
MATHEMATICAL SCRIPT SMALL N

1D4C5
MATHEMATICAL SCRIPT SMALL P

1D4C6
MATHEMATICAL SCRIPT SMALL Q

1D4C7
MATHEMATICAL SCRIPT SMALL R

1D4C8
MATHEMATICAL SCRIPT SMALL S

1D4C9
MATHEMATICAL SCRIPT SMALL T

1D4CA
MATHEMATICAL SCRIPT SMALL U

1D4CB
MATHEMATICAL SCRIPT SMALL V

1D4CC
MATHEMATICAL SCRIPT SMALL W

1D4CD
MATHEMATICAL SCRIPT SMALL X

1D4CE
MATHEMATICAL SCRIPT SMALL Y

1D4CF
MATHEMATICAL SCRIPT SMALL Z

1D4D0
MATHEMATICAL BOLD SCRIPT CAPITAL A

1D4D1
MATHEMATICAL BOLD SCRIPT CAPITAL B

1D4D2
MATHEMATICAL BOLD SCRIPT CAPITAL C

1D4D3
MATHEMATICAL BOLD SCRIPT CAPITAL D

1D4D4
MATHEMATICAL BOLD SCRIPT CAPITAL E

1D4D5
MATHEMATICAL BOLD SCRIPT CAPITAL F

1D4D6
MATHEMATICAL BOLD SCRIPT CAPITAL G

1D4D7
MATHEMATICAL BOLD SCRIPT CAPITAL H

1D4D8
MATHEMATICAL BOLD SCRIPT CAPITAL I

1D4D9
MATHEMATICAL BOLD SCRIPT CAPITAL J

1D4DA
MATHEMATICAL BOLD SCRIPT CAPITAL K

1D4DB
MATHEMATICAL BOLD SCRIPT CAPITAL L

1D4DC
MATHEMATICAL BOLD SCRIPT CAPITAL M

1D4DD
MATHEMATICAL BOLD SCRIPT CAPITAL N

1D4DE
MATHEMATICAL BOLD SCRIPT CAPITAL O

1D4DF
MATHEMATICAL BOLD SCRIPT CAPITAL P

1D4E0
MATHEMATICAL BOLD SCRIPT CAPITAL Q

1D4E1
MATHEMATICAL BOLD SCRIPT CAPITAL R

1D4E2
MATHEMATICAL BOLD SCRIPT CAPITAL S

1D4E3
MATHEMATICAL BOLD SCRIPT CAPITAL T

1D4E4
MATHEMATICAL BOLD SCRIPT CAPITAL U

1D4E5
MATHEMATICAL BOLD SCRIPT CAPITAL V

1D4E6
MATHEMATICAL BOLD SCRIPT CAPITAL W

1D4E7
MATHEMATICAL BOLD SCRIPT CAPITAL X

1D4E8
MATHEMATICAL BOLD SCRIPT CAPITAL Y

1D4E9
MATHEMATICAL BOLD SCRIPT CAPITAL Z

1D4EA
MATHEMATICAL BOLD SCRIPT SMALL A

1D4EB
MATHEMATICAL BOLD SCRIPT SMALL B

1D4EC
MATHEMATICAL BOLD SCRIPT SMALL C

1D4ED
MATHEMATICAL BOLD SCRIPT SMALL D

1D4EE
MATHEMATICAL BOLD SCRIPT SMALL E

1D4EF
MATHEMATICAL BOLD SCRIPT SMALL F

1D4F0
MATHEMATICAL BOLD SCRIPT SMALL G

1D4F1
MATHEMATICAL BOLD SCRIPT SMALL H

1D4F2
MATHEMATICAL BOLD SCRIPT SMALL I

1D4F3
MATHEMATICAL BOLD SCRIPT SMALL J

1D4F4
MATHEMATICAL BOLD SCRIPT SMALL K

1D4F5
MATHEMATICAL BOLD SCRIPT SMALL L

1D4F6
MATHEMATICAL BOLD SCRIPT SMALL M

1D4F7
MATHEMATICAL BOLD SCRIPT SMALL N

1D4F8
MATHEMATICAL BOLD SCRIPT SMALL O

1D4F9
MATHEMATICAL BOLD SCRIPT SMALL P

1D4FA
MATHEMATICAL BOLD SCRIPT SMALL Q

1D4FB
MATHEMATICAL BOLD SCRIPT SMALL R

1D4FC
MATHEMATICAL BOLD SCRIPT SMALL S

1D4FD
MATHEMATICAL BOLD SCRIPT SMALL T

1D4FE
MATHEMATICAL BOLD SCRIPT SMALL U

1D4FF
MATHEMATICAL BOLD SCRIPT SMALL V

1D500
MATHEMATICAL BOLD SCRIPT SMALL W

1D501
MATHEMATICAL BOLD SCRIPT SMALL X

1D502
MATHEMATICAL BOLD SCRIPT SMALL Y

1D503
MATHEMATICAL BOLD SCRIPT SMALL Z

1D504
MATHEMATICAL FRAKTUR CAPITAL A

1D505
MATHEMATICAL FRAKTUR CAPITAL B

1D507
MATHEMATICAL FRAKTUR CAPITAL D

1D508
MATHEMATICAL FRAKTUR CAPITAL E

1D509
MATHEMATICAL FRAKTUR CAPITAL F

1D50A
MATHEMATICAL FRAKTUR CAPITAL G

1D50D
MATHEMATICAL FRAKTUR CAPITAL J

1D50E
MATHEMATICAL FRAKTUR CAPITAL K

1D50F
MATHEMATICAL FRAKTUR CAPITAL L

1D510
MATHEMATICAL FRAKTUR CAPITAL M

1D511
MATHEMATICAL FRAKTUR CAPITAL N

1D512
MATHEMATICAL FRAKTUR CAPITAL O

1D513
MATHEMATICAL FRAKTUR CAPITAL P

1D514
MATHEMATICAL FRAKTUR CAPITAL Q

1D516
MATHEMATICAL FRAKTUR CAPITAL S

1D517
MATHEMATICAL FRAKTUR CAPITAL T

1D518
MATHEMATICAL FRAKTUR CAPITAL U

1D519
MATHEMATICAL FRAKTUR CAPITAL V

1D51A
MATHEMATICAL FRAKTUR CAPITAL W

1D51B
MATHEMATICAL FRAKTUR CAPITAL X

1D51C
MATHEMATICAL FRAKTUR CAPITAL Y

1D51E
MATHEMATICAL FRAKTUR SMALL A

1D51F
MATHEMATICAL FRAKTUR SMALL B

1D520
MATHEMATICAL FRAKTUR SMALL C

1D521
MATHEMATICAL FRAKTUR SMALL D

1D522
MATHEMATICAL FRAKTUR SMALL E

1D523
MATHEMATICAL FRAKTUR SMALL F

1D524
MATHEMATICAL FRAKTUR SMALL G

1D525
MATHEMATICAL FRAKTUR SMALL H

1D526
MATHEMATICAL FRAKTUR SMALL I

1D527
MATHEMATICAL FRAKTUR SMALL J

1D528
MATHEMATICAL FRAKTUR SMALL K

1D529
MATHEMATICAL FRAKTUR SMALL L

1D52A
MATHEMATICAL FRAKTUR SMALL M

1D52B
MATHEMATICAL FRAKTUR SMALL N

1D52C
MATHEMATICAL FRAKTUR SMALL O

1D52D
MATHEMATICAL FRAKTUR SMALL P

1D52E
MATHEMATICAL FRAKTUR SMALL Q

1D52F
MATHEMATICAL FRAKTUR SMALL R

1D530
MATHEMATICAL FRAKTUR SMALL S

1D531
MATHEMATICAL FRAKTUR SMALL T

1D532
MATHEMATICAL FRAKTUR SMALL U

1D533
MATHEMATICAL FRAKTUR SMALL V

1D534
MATHEMATICAL FRAKTUR SMALL W

1D535
MATHEMATICAL FRAKTUR SMALL X

1D536
MATHEMATICAL FRAKTUR SMALL Y

1D537
MATHEMATICAL FRAKTUR SMALL Z

1D538
MATHEMATICAL DOUBLE-STRUCK CAPITAL A

1D539
MATHEMATICAL DOUBLE-STRUCK CAPITAL B

1D53B
MATHEMATICAL DOUBLE-STRUCK CAPITAL D

1D53C
MATHEMATICAL DOUBLE-STRUCK CAPITAL E

1D53D
MATHEMATICAL DOUBLE-STRUCK CAPITAL F

1D53E
MATHEMATICAL DOUBLE-STRUCK CAPITAL G

1D540
MATHEMATICAL DOUBLE-STRUCK CAPITAL I

1D541
MATHEMATICAL DOUBLE-STRUCK CAPITAL J

1D542
MATHEMATICAL DOUBLE-STRUCK CAPITAL K

1D543
MATHEMATICAL DOUBLE-STRUCK CAPITAL L

1D544
MATHEMATICAL DOUBLE-STRUCK CAPITAL M

1D546
MATHEMATICAL DOUBLE-STRUCK CAPITAL O

1D54A
MATHEMATICAL DOUBLE-STRUCK CAPITAL S

1D54B
MATHEMATICAL DOUBLE-STRUCK CAPITAL T

1D54C
MATHEMATICAL DOUBLE-STRUCK CAPITAL U

1D54D
MATHEMATICAL DOUBLE-STRUCK CAPITAL V

1D54E
MATHEMATICAL DOUBLE-STRUCK CAPITAL W

1D54F
MATHEMATICAL DOUBLE-STRUCK CAPITAL X

1D550
MATHEMATICAL DOUBLE-STRUCK CAPITAL Y

1D552
MATHEMATICAL DOUBLE-STRUCK SMALL A

1D553
MATHEMATICAL DOUBLE-STRUCK SMALL B

1D554
MATHEMATICAL DOUBLE-STRUCK SMALL C

1D555
MATHEMATICAL DOUBLE-STRUCK SMALL D

1D556
MATHEMATICAL DOUBLE-STRUCK SMALL E

1D557
MATHEMATICAL DOUBLE-STRUCK SMALL F

1D558
MATHEMATICAL DOUBLE-STRUCK SMALL G

1D559
MATHEMATICAL DOUBLE-STRUCK SMALL H

1D55A
MATHEMATICAL DOUBLE-STRUCK SMALL I

1D55B
MATHEMATICAL DOUBLE-STRUCK SMALL J

1D55C
MATHEMATICAL DOUBLE-STRUCK SMALL K

1D55D
MATHEMATICAL DOUBLE-STRUCK SMALL L

1D55E
MATHEMATICAL DOUBLE-STRUCK SMALL M

1D55F
MATHEMATICAL DOUBLE-STRUCK SMALL N

1D560
MATHEMATICAL DOUBLE-STRUCK SMALL O

1D561
MATHEMATICAL DOUBLE-STRUCK SMALL P

1D562
MATHEMATICAL DOUBLE-STRUCK SMALL Q

1D563
MATHEMATICAL DOUBLE-STRUCK SMALL R

1D564
MATHEMATICAL DOUBLE-STRUCK SMALL S

1D565
MATHEMATICAL DOUBLE-STRUCK SMALL T

1D566
MATHEMATICAL DOUBLE-STRUCK SMALL U

1D567
MATHEMATICAL DOUBLE-STRUCK SMALL V

1D568
MATHEMATICAL DOUBLE-STRUCK SMALL W

1D569
MATHEMATICAL DOUBLE-STRUCK SMALL X

1D56A
MATHEMATICAL DOUBLE-STRUCK SMALL Y

1D56B
MATHEMATICAL DOUBLE-STRUCK SMALL Z

1D56C
MATHEMATICAL BOLD FRAKTUR CAPITAL A

1D56D
MATHEMATICAL BOLD FRAKTUR CAPITAL B

1D56E
MATHEMATICAL BOLD FRAKTUR CAPITAL C

1D56F
MATHEMATICAL BOLD FRAKTUR CAPITAL D

1D570
MATHEMATICAL BOLD FRAKTUR CAPITAL E

1D571
MATHEMATICAL BOLD FRAKTUR CAPITAL F

1D572
MATHEMATICAL BOLD FRAKTUR CAPITAL G

1D573
MATHEMATICAL BOLD FRAKTUR CAPITAL H

1D574
MATHEMATICAL BOLD FRAKTUR CAPITAL I

1D575
MATHEMATICAL BOLD FRAKTUR CAPITAL J

1D576
MATHEMATICAL BOLD FRAKTUR CAPITAL K

1D577
MATHEMATICAL BOLD FRAKTUR CAPITAL L

1D578
MATHEMATICAL BOLD FRAKTUR CAPITAL M

1D579
MATHEMATICAL BOLD FRAKTUR CAPITAL N

1D57A
MATHEMATICAL BOLD FRAKTUR CAPITAL O

1D57B
MATHEMATICAL BOLD FRAKTUR CAPITAL P

1D57C
MATHEMATICAL BOLD FRAKTUR CAPITAL Q

1D57D
MATHEMATICAL BOLD FRAKTUR CAPITAL R

1D57E
MATHEMATICAL BOLD FRAKTUR CAPITAL S

1D57F
MATHEMATICAL BOLD FRAKTUR CAPITAL T

1D580
MATHEMATICAL BOLD FRAKTUR CAPITAL U

1D581
MATHEMATICAL BOLD FRAKTUR CAPITAL V

1D582
MATHEMATICAL BOLD FRAKTUR CAPITAL W

1D583
MATHEMATICAL BOLD FRAKTUR CAPITAL X

1D584
MATHEMATICAL BOLD FRAKTUR CAPITAL Y

1D585
MATHEMATICAL BOLD FRAKTUR CAPITAL Z

1D586
MATHEMATICAL BOLD FRAKTUR SMALL A

1D587
MATHEMATICAL BOLD FRAKTUR SMALL B

1D588
MATHEMATICAL BOLD FRAKTUR SMALL C

1D589
MATHEMATICAL BOLD FRAKTUR SMALL D

1D58A
MATHEMATICAL BOLD FRAKTUR SMALL E

1D58B
MATHEMATICAL BOLD FRAKTUR SMALL F

1D58C
MATHEMATICAL BOLD FRAKTUR SMALL G

1D58D
MATHEMATICAL BOLD FRAKTUR SMALL H

1D58E
MATHEMATICAL BOLD FRAKTUR SMALL I

1D58F
MATHEMATICAL BOLD FRAKTUR SMALL J

1D590
MATHEMATICAL BOLD FRAKTUR SMALL K

1D591
MATHEMATICAL BOLD FRAKTUR SMALL L

1D592
MATHEMATICAL BOLD FRAKTUR SMALL M

1D593
MATHEMATICAL BOLD FRAKTUR SMALL N

1D594
MATHEMATICAL BOLD FRAKTUR SMALL O

1D595
MATHEMATICAL BOLD FRAKTUR SMALL P

1D596
MATHEMATICAL BOLD FRAKTUR SMALL Q

1D597
MATHEMATICAL BOLD FRAKTUR SMALL R

1D598
MATHEMATICAL BOLD FRAKTUR SMALL S

1D599
MATHEMATICAL BOLD FRAKTUR SMALL T

1D59A
MATHEMATICAL BOLD FRAKTUR SMALL U

1D59B
MATHEMATICAL BOLD FRAKTUR SMALL V

1D59C
MATHEMATICAL BOLD FRAKTUR SMALL W

1D59D
MATHEMATICAL BOLD FRAKTUR SMALL X

1D59E
MATHEMATICAL BOLD FRAKTUR SMALL Y

1D59F
MATHEMATICAL BOLD FRAKTUR SMALL Z

1D5A0
MATHEMATICAL SANS-SERIF CAPITAL A

1D5A1
MATHEMATICAL SANS-SERIF CAPITAL B

1D5A2
MATHEMATICAL SANS-SERIF CAPITAL C

1D5A3
MATHEMATICAL SANS-SERIF CAPITAL D

1D5A4
MATHEMATICAL SANS-SERIF CAPITAL E

1D5A5
MATHEMATICAL SANS-SERIF CAPITAL F

1D5A6
MATHEMATICAL SANS-SERIF CAPITAL G

1D5A7
MATHEMATICAL SANS-SERIF CAPITAL H

1D5A8
MATHEMATICAL SANS-SERIF CAPITAL I

1D5A9
MATHEMATICAL SANS-SERIF CAPITAL J

1D5AA
MATHEMATICAL SANS-SERIF CAPITAL K

1D5AB
MATHEMATICAL SANS-SERIF CAPITAL L

1D5AC
MATHEMATICAL SANS-SERIF CAPITAL M

1D5AD
MATHEMATICAL SANS-SERIF CAPITAL N

1D5AE
MATHEMATICAL SANS-SERIF CAPITAL O

1D5AF
MATHEMATICAL SANS-SERIF CAPITAL P

1D5B0
MATHEMATICAL SANS-SERIF CAPITAL Q

1D5B1
MATHEMATICAL SANS-SERIF CAPITAL R

1D5B2
MATHEMATICAL SANS-SERIF CAPITAL S

1D5B3
MATHEMATICAL SANS-SERIF CAPITAL T

1D5B4
MATHEMATICAL SANS-SERIF CAPITAL U

1D5B5
MATHEMATICAL SANS-SERIF CAPITAL V

1D5B6
MATHEMATICAL SANS-SERIF CAPITAL W

1D5B7
MATHEMATICAL SANS-SERIF CAPITAL X

1D5B8
MATHEMATICAL SANS-SERIF CAPITAL Y

1D5B9
MATHEMATICAL SANS-SERIF CAPITAL Z

1D5BA
MATHEMATICAL SANS-SERIF SMALL A

1D5BB
MATHEMATICAL SANS-SERIF SMALL B

1D5BC
MATHEMATICAL SANS-SERIF SMALL C

1D5BD
MATHEMATICAL SANS-SERIF SMALL D

1D5BE
MATHEMATICAL SANS-SERIF SMALL E

1D5BF
MATHEMATICAL SANS-SERIF SMALL F

1D5C0
MATHEMATICAL SANS-SERIF SMALL G

1D5C1
MATHEMATICAL SANS-SERIF SMALL H

1D5C2
MATHEMATICAL SANS-SERIF SMALL I

1D5C3
MATHEMATICAL SANS-SERIF SMALL J

1D5C4
MATHEMATICAL SANS-SERIF SMALL K

1D5C5
MATHEMATICAL SANS-SERIF SMALL L

1D5C6
MATHEMATICAL SANS-SERIF SMALL M

1D5C7
MATHEMATICAL SANS-SERIF SMALL N

1D5C8
MATHEMATICAL SANS-SERIF SMALL O

1D5C9
MATHEMATICAL SANS-SERIF SMALL P

1D5CA
MATHEMATICAL SANS-SERIF SMALL Q

1D5CB
MATHEMATICAL SANS-SERIF SMALL R

1D5CC
MATHEMATICAL SANS-SERIF SMALL S

1D5CD
MATHEMATICAL SANS-SERIF SMALL T

1D5CE
MATHEMATICAL SANS-SERIF SMALL U

1D5CF
MATHEMATICAL SANS-SERIF SMALL V

1D5D0
MATHEMATICAL SANS-SERIF SMALL W

1D5D1
MATHEMATICAL SANS-SERIF SMALL X

1D5D2
MATHEMATICAL SANS-SERIF SMALL Y

1D5D3
MATHEMATICAL SANS-SERIF SMALL Z

1D5D4
MATHEMATICAL SANS-SERIF BOLD CAPITAL A

1D5D5
MATHEMATICAL SANS-SERIF BOLD CAPITAL B

1D5D6
MATHEMATICAL SANS-SERIF BOLD CAPITAL C

1D5D7
MATHEMATICAL SANS-SERIF BOLD CAPITAL D

1D5D8
MATHEMATICAL SANS-SERIF BOLD CAPITAL E

1D5D9
MATHEMATICAL SANS-SERIF BOLD CAPITAL F

1D5DA
MATHEMATICAL SANS-SERIF BOLD CAPITAL G

1D5DB
MATHEMATICAL SANS-SERIF BOLD CAPITAL H

1D5DC
MATHEMATICAL SANS-SERIF BOLD CAPITAL I

1D5DD
MATHEMATICAL SANS-SERIF BOLD CAPITAL J

1D5DE
MATHEMATICAL SANS-SERIF BOLD CAPITAL K

1D5DF
MATHEMATICAL SANS-SERIF BOLD CAPITAL L

1D5E0
MATHEMATICAL SANS-SERIF BOLD CAPITAL M

1D5E1
MATHEMATICAL SANS-SERIF BOLD CAPITAL N

1D5E2
MATHEMATICAL SANS-SERIF BOLD CAPITAL O

1D5E3
MATHEMATICAL SANS-SERIF BOLD CAPITAL P

1D5E4
MATHEMATICAL SANS-SERIF BOLD CAPITAL Q

1D5E5
MATHEMATICAL SANS-SERIF BOLD CAPITAL R

1D5E6
MATHEMATICAL SANS-SERIF BOLD CAPITAL S

1D5E7
MATHEMATICAL SANS-SERIF BOLD CAPITAL T

1D5E8
MATHEMATICAL SANS-SERIF BOLD CAPITAL U

1D5E9
MATHEMATICAL SANS-SERIF BOLD CAPITAL V

1D5EA
MATHEMATICAL SANS-SERIF BOLD CAPITAL W

1D5EB
MATHEMATICAL SANS-SERIF BOLD CAPITAL X

1D5EC
MATHEMATICAL SANS-SERIF BOLD CAPITAL Y

1D5ED
MATHEMATICAL SANS-SERIF BOLD CAPITAL Z

1D5EE
MATHEMATICAL SANS-SERIF BOLD SMALL A

1D5EF
MATHEMATICAL SANS-SERIF BOLD SMALL B

1D5F0
MATHEMATICAL SANS-SERIF BOLD SMALL C

1D5F1
MATHEMATICAL SANS-SERIF BOLD SMALL D

1D5F2
MATHEMATICAL SANS-SERIF BOLD SMALL E

1D5F3
MATHEMATICAL SANS-SERIF BOLD SMALL F

1D5F4
MATHEMATICAL SANS-SERIF BOLD SMALL G

1D5F5
MATHEMATICAL SANS-SERIF BOLD SMALL H

1D5F6
MATHEMATICAL SANS-SERIF BOLD SMALL I

1D5F7
MATHEMATICAL SANS-SERIF BOLD SMALL J

1D5F8
MATHEMATICAL SANS-SERIF BOLD SMALL K

1D5F9
MATHEMATICAL SANS-SERIF BOLD SMALL L

1D5FA
MATHEMATICAL SANS-SERIF BOLD SMALL M

1D5FB
MATHEMATICAL SANS-SERIF BOLD SMALL N

1D5FC
MATHEMATICAL SANS-SERIF BOLD SMALL O

1D5FD
MATHEMATICAL SANS-SERIF BOLD SMALL P

1D5FE
MATHEMATICAL SANS-SERIF BOLD SMALL Q

1D5FF
MATHEMATICAL SANS-SERIF BOLD SMALL R

1D600
MATHEMATICAL SANS-SERIF BOLD SMALL S

1D601
MATHEMATICAL SANS-SERIF BOLD SMALL T

1D602
MATHEMATICAL SANS-SERIF BOLD SMALL U

1D603
MATHEMATICAL SANS-SERIF BOLD SMALL V

1D604
MATHEMATICAL SANS-SERIF BOLD SMALL W

1D605
MATHEMATICAL SANS-SERIF BOLD SMALL X

1D606
MATHEMATICAL SANS-SERIF BOLD SMALL Y

1D607
MATHEMATICAL SANS-SERIF BOLD SMALL Z

1D608
MATHEMATICAL SANS-SERIF ITALIC CAPITAL A

1D609
MATHEMATICAL SANS-SERIF ITALIC CAPITAL B

1D60A
MATHEMATICAL SANS-SERIF ITALIC CAPITAL C

1D60B
MATHEMATICAL SANS-SERIF ITALIC CAPITAL D

1D60C
MATHEMATICAL SANS-SERIF ITALIC CAPITAL E

1D60D
MATHEMATICAL SANS-SERIF ITALIC CAPITAL F

1D60E
MATHEMATICAL SANS-SERIF ITALIC CAPITAL G

1D60F
MATHEMATICAL SANS-SERIF ITALIC CAPITAL H

1D610
MATHEMATICAL SANS-SERIF ITALIC CAPITAL I

1D611
MATHEMATICAL SANS-SERIF ITALIC CAPITAL J

1D612
MATHEMATICAL SANS-SERIF ITALIC CAPITAL K

1D613
MATHEMATICAL SANS-SERIF ITALIC CAPITAL L

1D614
MATHEMATICAL SANS-SERIF ITALIC CAPITAL M

1D615
MATHEMATICAL SANS-SERIF ITALIC CAPITAL N

1D616
MATHEMATICAL SANS-SERIF ITALIC CAPITAL O

1D617
MATHEMATICAL SANS-SERIF ITALIC CAPITAL P

1D618
MATHEMATICAL SANS-SERIF ITALIC CAPITAL Q

1D619
MATHEMATICAL SANS-SERIF ITALIC CAPITAL R

1D61A
MATHEMATICAL SANS-SERIF ITALIC CAPITAL S

1D61B
MATHEMATICAL SANS-SERIF ITALIC CAPITAL T

1D61C
MATHEMATICAL SANS-SERIF ITALIC CAPITAL U

1D61D
MATHEMATICAL SANS-SERIF ITALIC CAPITAL V

1D61E
MATHEMATICAL SANS-SERIF ITALIC CAPITAL W

1D61F
MATHEMATICAL SANS-SERIF ITALIC CAPITAL X

1D620
MATHEMATICAL SANS-SERIF ITALIC CAPITAL Y

1D621
MATHEMATICAL SANS-SERIF ITALIC CAPITAL Z

1D622
MATHEMATICAL SANS-SERIF ITALIC SMALL A

1D623
MATHEMATICAL SANS-SERIF ITALIC SMALL B

1D624
MATHEMATICAL SANS-SERIF ITALIC SMALL C

1D625
MATHEMATICAL SANS-SERIF ITALIC SMALL D

1D626
MATHEMATICAL SANS-SERIF ITALIC SMALL E

1D627
MATHEMATICAL SANS-SERIF ITALIC SMALL F

1D628
MATHEMATICAL SANS-SERIF ITALIC SMALL G

1D629
MATHEMATICAL SANS-SERIF ITALIC SMALL H

1D62A
MATHEMATICAL SANS-SERIF ITALIC SMALL I

1D62B
MATHEMATICAL SANS-SERIF ITALIC SMALL J

1D62C
MATHEMATICAL SANS-SERIF ITALIC SMALL K

1D62D
MATHEMATICAL SANS-SERIF ITALIC SMALL L

1D62E
MATHEMATICAL SANS-SERIF ITALIC SMALL M

1D62F
MATHEMATICAL SANS-SERIF ITALIC SMALL N

1D630
MATHEMATICAL SANS-SERIF ITALIC SMALL O

1D631
MATHEMATICAL SANS-SERIF ITALIC SMALL P

1D632
MATHEMATICAL SANS-SERIF ITALIC SMALL Q

1D633
MATHEMATICAL SANS-SERIF ITALIC SMALL R

1D634
MATHEMATICAL SANS-SERIF ITALIC SMALL S

1D635
MATHEMATICAL SANS-SERIF ITALIC SMALL T

1D636
MATHEMATICAL SANS-SERIF ITALIC SMALL U

1D637
MATHEMATICAL SANS-SERIF ITALIC SMALL V

1D638
MATHEMATICAL SANS-SERIF ITALIC SMALL W

1D639
MATHEMATICAL SANS-SERIF ITALIC SMALL X

1D63A
MATHEMATICAL SANS-SERIF ITALIC SMALL Y

1D63B
MATHEMATICAL SANS-SERIF ITALIC SMALL Z

1D63C
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL A

1D63D
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL B

1D63E
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL C

1D63F
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL D

1D640
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL E

1D641
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL F

1D642
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL G

1D643
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL H

1D644
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL I

1D645
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL J

1D646
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL K

1D647
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL L

1D648
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL M

1D649
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL N

1D64A
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL O

1D64B
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL P

1D64C
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL Q

1D64D
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL R

1D64E
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL S

1D64F
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL T

1D650
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL U

1D651
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL V

1D652
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL W

1D653
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL X

1D654
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL Y

1D655
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL Z

1D656
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL A

1D657
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL B

1D658
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL C

1D659
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL D

1D65A
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL E

1D65B
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL F

1D65C
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL G

1D65D
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL H

1D65E
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL I

1D65F
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL J

1D660
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL K

1D661
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL L

1D662
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL M

1D663
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL N

1D664
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL O

1D665
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL P

1D666
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL Q

1D667
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL R

1D668
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL S

1D669
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL T

1D66A
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL U

1D66B
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL V

1D66C
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL W

1D66D
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL X

1D66E
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL Y

1D66F
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL Z

1D670
MATHEMATICAL MONOSPACE CAPITAL A

1D671
MATHEMATICAL MONOSPACE CAPITAL B

1D672
MATHEMATICAL MONOSPACE CAPITAL C

1D673
MATHEMATICAL MONOSPACE CAPITAL D

1D674
MATHEMATICAL MONOSPACE CAPITAL E

1D675
MATHEMATICAL MONOSPACE CAPITAL F

1D676
MATHEMATICAL MONOSPACE CAPITAL G

1D677
MATHEMATICAL MONOSPACE CAPITAL H

1D678
MATHEMATICAL MONOSPACE CAPITAL I

1D679
MATHEMATICAL MONOSPACE CAPITAL J

1D67A
MATHEMATICAL MONOSPACE CAPITAL K

1D67B
MATHEMATICAL MONOSPACE CAPITAL L

1D67C
MATHEMATICAL MONOSPACE CAPITAL M

1D67D
MATHEMATICAL MONOSPACE CAPITAL N

1D67E
MATHEMATICAL MONOSPACE CAPITAL O

1D67F
MATHEMATICAL MONOSPACE CAPITAL P

1D680
MATHEMATICAL MONOSPACE CAPITAL Q

1D681
MATHEMATICAL MONOSPACE CAPITAL R

1D682
MATHEMATICAL MONOSPACE CAPITAL S

1D683
MATHEMATICAL MONOSPACE CAPITAL T

1D684
MATHEMATICAL MONOSPACE CAPITAL U

1D685
MATHEMATICAL MONOSPACE CAPITAL V

1D686
MATHEMATICAL MONOSPACE CAPITAL W

1D687
MATHEMATICAL MONOSPACE CAPITAL X

1D688
MATHEMATICAL MONOSPACE CAPITAL Y

1D689
MATHEMATICAL MONOSPACE CAPITAL Z

1D68A
MATHEMATICAL MONOSPACE SMALL A

1D68B
MATHEMATICAL MONOSPACE SMALL B

1D68C
MATHEMATICAL MONOSPACE SMALL C

1D68D
MATHEMATICAL MONOSPACE SMALL D

1D68E
MATHEMATICAL MONOSPACE SMALL E

1D68F
MATHEMATICAL MONOSPACE SMALL F

1D690
MATHEMATICAL MONOSPACE SMALL G

1D691
MATHEMATICAL MONOSPACE SMALL H

1D692
MATHEMATICAL MONOSPACE SMALL I

1D693
MATHEMATICAL MONOSPACE SMALL J

1D694
MATHEMATICAL MONOSPACE SMALL K

1D695
MATHEMATICAL MONOSPACE SMALL L

1D696
MATHEMATICAL MONOSPACE SMALL M

1D697
MATHEMATICAL MONOSPACE SMALL N

1D698
MATHEMATICAL MONOSPACE SMALL O

1D699
MATHEMATICAL MONOSPACE SMALL P

1D69A
MATHEMATICAL MONOSPACE SMALL Q

1D69B
MATHEMATICAL MONOSPACE SMALL R

1D69C
MATHEMATICAL MONOSPACE SMALL S

1D69D
MATHEMATICAL MONOSPACE SMALL T

1D69E
MATHEMATICAL MONOSPACE SMALL U

1D69F
MATHEMATICAL MONOSPACE SMALL V

1D6A0
MATHEMATICAL MONOSPACE SMALL W

1D6A1
MATHEMATICAL MONOSPACE SMALL X

1D6A2
MATHEMATICAL MONOSPACE SMALL Y

1D6A3
MATHEMATICAL MONOSPACE SMALL Z

1D6A4
MATHEMATICAL ITALIC SMALL DOTLESS I

1D6A5
MATHEMATICAL ITALIC SMALL DOTLESS J

1D6A8
MATHEMATICAL BOLD CAPITAL ALPHA

1D6A9
MATHEMATICAL BOLD CAPITAL BETA

1D6AA
MATHEMATICAL BOLD CAPITAL GAMMA

1D6AB
MATHEMATICAL BOLD CAPITAL DELTA

1D6AC
MATHEMATICAL BOLD CAPITAL EPSILON

1D6AD
MATHEMATICAL BOLD CAPITAL ZETA

1D6AE
MATHEMATICAL BOLD CAPITAL ETA

1D6AF
MATHEMATICAL BOLD CAPITAL THETA

1D6B0
MATHEMATICAL BOLD CAPITAL IOTA

1D6B1
MATHEMATICAL BOLD CAPITAL KAPPA

1D6B2
MATHEMATICAL BOLD CAPITAL LAMDA

1D6B3
MATHEMATICAL BOLD CAPITAL MU

1D6B4
MATHEMATICAL BOLD CAPITAL NU

1D6B5
MATHEMATICAL BOLD CAPITAL XI

1D6B6
MATHEMATICAL BOLD CAPITAL OMICRON

1D6B7
MATHEMATICAL BOLD CAPITAL PI

1D6B8
MATHEMATICAL BOLD CAPITAL RHO

1D6B9
MATHEMATICAL BOLD CAPITAL THETA SYMBOL

1D6BA
MATHEMATICAL BOLD CAPITAL SIGMA

1D6BB
MATHEMATICAL BOLD CAPITAL TAU

1D6BC
MATHEMATICAL BOLD CAPITAL UPSILON

1D6BD
MATHEMATICAL BOLD CAPITAL PHI

1D6BE
MATHEMATICAL BOLD CAPITAL CHI

1D6BF
MATHEMATICAL BOLD CAPITAL PSI

1D6C0
MATHEMATICAL BOLD CAPITAL OMEGA

1D6C1
MATHEMATICAL BOLD NABLA

1D6C2
MATHEMATICAL BOLD SMALL ALPHA

1D6C3
MATHEMATICAL BOLD SMALL BETA

1D6C4
MATHEMATICAL BOLD SMALL GAMMA

1D6C5
MATHEMATICAL BOLD SMALL DELTA

1D6C6
MATHEMATICAL BOLD SMALL EPSILON

1D6C7
MATHEMATICAL BOLD SMALL ZETA

1D6C8
MATHEMATICAL BOLD SMALL ETA

1D6C9
MATHEMATICAL BOLD SMALL THETA

1D6CA
MATHEMATICAL BOLD SMALL IOTA

1D6CB
MATHEMATICAL BOLD SMALL KAPPA

1D6CC
MATHEMATICAL BOLD SMALL LAMDA

1D6CD
MATHEMATICAL BOLD SMALL MU

1D6CE
MATHEMATICAL BOLD SMALL NU

1D6CF
MATHEMATICAL BOLD SMALL XI

1D6D0
MATHEMATICAL BOLD SMALL OMICRON

1D6D1
MATHEMATICAL BOLD SMALL PI

1D6D2
MATHEMATICAL BOLD SMALL RHO

1D6D3
MATHEMATICAL BOLD SMALL FINAL SIGMA

1D6D4
MATHEMATICAL BOLD SMALL SIGMA

1D6D5
MATHEMATICAL BOLD SMALL TAU

1D6D6
MATHEMATICAL BOLD SMALL UPSILON

1D6D7
MATHEMATICAL BOLD SMALL PHI

1D6D8
MATHEMATICAL BOLD SMALL CHI

1D6D9
MATHEMATICAL BOLD SMALL PSI

1D6DA
MATHEMATICAL BOLD SMALL OMEGA

1D6DB
MATHEMATICAL BOLD PARTIAL DIFFERENTIAL

1D6DC
MATHEMATICAL BOLD EPSILON SYMBOL

1D6DD
MATHEMATICAL BOLD THETA SYMBOL

1D6DE
MATHEMATICAL BOLD KAPPA SYMBOL

1D6DF
MATHEMATICAL BOLD PHI SYMBOL

1D6E0
MATHEMATICAL BOLD RHO SYMBOL

1D6E1
MATHEMATICAL BOLD PI SYMBOL

1D6E2
MATHEMATICAL ITALIC CAPITAL ALPHA

1D6E3
MATHEMATICAL ITALIC CAPITAL BETA

1D6E4
MATHEMATICAL ITALIC CAPITAL GAMMA

1D6E5
MATHEMATICAL ITALIC CAPITAL DELTA

1D6E6
MATHEMATICAL ITALIC CAPITAL EPSILON

1D6E7
MATHEMATICAL ITALIC CAPITAL ZETA

1D6E8
MATHEMATICAL ITALIC CAPITAL ETA

1D6E9
MATHEMATICAL ITALIC CAPITAL THETA

1D6EA
MATHEMATICAL ITALIC CAPITAL IOTA

1D6EB
MATHEMATICAL ITALIC CAPITAL KAPPA

1D6EC
MATHEMATICAL ITALIC CAPITAL LAMDA

1D6ED
MATHEMATICAL ITALIC CAPITAL MU

1D6EE
MATHEMATICAL ITALIC CAPITAL NU

1D6EF
MATHEMATICAL ITALIC CAPITAL XI

1D6F0
MATHEMATICAL ITALIC CAPITAL OMICRON

1D6F1
MATHEMATICAL ITALIC CAPITAL PI

1D6F2
MATHEMATICAL ITALIC CAPITAL RHO

1D6F3
MATHEMATICAL ITALIC CAPITAL THETA SYMBOL

1D6F4
MATHEMATICAL ITALIC CAPITAL SIGMA

1D6F5
MATHEMATICAL ITALIC CAPITAL TAU

1D6F6
MATHEMATICAL ITALIC CAPITAL UPSILON

1D6F7
MATHEMATICAL ITALIC CAPITAL PHI

1D6F8
MATHEMATICAL ITALIC CAPITAL CHI

1D6F9
MATHEMATICAL ITALIC CAPITAL PSI

1D6FA
MATHEMATICAL ITALIC CAPITAL OMEGA

1D6FB
MATHEMATICAL ITALIC NABLA

1D6FC
MATHEMATICAL ITALIC SMALL ALPHA

1D6FD
MATHEMATICAL ITALIC SMALL BETA

1D6FE
MATHEMATICAL ITALIC SMALL GAMMA

1D6FF
MATHEMATICAL ITALIC SMALL DELTA

1D700
MATHEMATICAL ITALIC SMALL EPSILON

1D701
MATHEMATICAL ITALIC SMALL ZETA

1D702
MATHEMATICAL ITALIC SMALL ETA

1D703
MATHEMATICAL ITALIC SMALL THETA

1D704
MATHEMATICAL ITALIC SMALL IOTA

1D705
MATHEMATICAL ITALIC SMALL KAPPA

1D706
MATHEMATICAL ITALIC SMALL LAMDA

1D707
MATHEMATICAL ITALIC SMALL MU

1D708
MATHEMATICAL ITALIC SMALL NU

1D709
MATHEMATICAL ITALIC SMALL XI

1D70A
MATHEMATICAL ITALIC SMALL OMICRON

1D70B
MATHEMATICAL ITALIC SMALL PI

1D70C
MATHEMATICAL ITALIC SMALL RHO

1D70D
MATHEMATICAL ITALIC SMALL FINAL SIGMA

1D70E
MATHEMATICAL ITALIC SMALL SIGMA

1D70F
MATHEMATICAL ITALIC SMALL TAU

1D710
MATHEMATICAL ITALIC SMALL UPSILON

1D711
MATHEMATICAL ITALIC SMALL PHI

1D712
MATHEMATICAL ITALIC SMALL CHI

1D713
MATHEMATICAL ITALIC SMALL PSI

1D714
MATHEMATICAL ITALIC SMALL OMEGA

1D715
MATHEMATICAL ITALIC PARTIAL DIFFERENTIAL

1D716
MATHEMATICAL ITALIC EPSILON SYMBOL

1D717
MATHEMATICAL ITALIC THETA SYMBOL

1D718
MATHEMATICAL ITALIC KAPPA SYMBOL

1D719
MATHEMATICAL ITALIC PHI SYMBOL

1D71A
MATHEMATICAL ITALIC RHO SYMBOL

1D71B
MATHEMATICAL ITALIC PI SYMBOL

1D71C
MATHEMATICAL BOLD ITALIC CAPITAL ALPHA

1D71D
MATHEMATICAL BOLD ITALIC CAPITAL BETA

1D71E
MATHEMATICAL BOLD ITALIC CAPITAL GAMMA

1D71F
MATHEMATICAL BOLD ITALIC CAPITAL DELTA

1D720
MATHEMATICAL BOLD ITALIC CAPITAL EPSILON

1D721
MATHEMATICAL BOLD ITALIC CAPITAL ZETA

1D722
MATHEMATICAL BOLD ITALIC CAPITAL ETA

1D723
MATHEMATICAL BOLD ITALIC CAPITAL THETA

1D724
MATHEMATICAL BOLD ITALIC CAPITAL IOTA

1D725
MATHEMATICAL BOLD ITALIC CAPITAL KAPPA

1D726
MATHEMATICAL BOLD ITALIC CAPITAL LAMDA

1D727
MATHEMATICAL BOLD ITALIC CAPITAL MU

1D728
MATHEMATICAL BOLD ITALIC CAPITAL NU

1D729
MATHEMATICAL BOLD ITALIC CAPITAL XI

1D72A
MATHEMATICAL BOLD ITALIC CAPITAL OMICRON

1D72B
MATHEMATICAL BOLD ITALIC CAPITAL PI

1D72C
MATHEMATICAL BOLD ITALIC CAPITAL RHO

1D72D
MATHEMATICAL BOLD ITALIC CAPITAL THETA SYMBOL

1D72E
MATHEMATICAL BOLD ITALIC CAPITAL SIGMA

1D72F
MATHEMATICAL BOLD ITALIC CAPITAL TAU

1D730
MATHEMATICAL BOLD ITALIC CAPITAL UPSILON

1D731
MATHEMATICAL BOLD ITALIC CAPITAL PHI

1D732
MATHEMATICAL BOLD ITALIC CAPITAL CHI

1D733
MATHEMATICAL BOLD ITALIC CAPITAL PSI

1D734
MATHEMATICAL BOLD ITALIC CAPITAL OMEGA

1D735
MATHEMATICAL BOLD ITALIC NABLA

1D736
MATHEMATICAL BOLD ITALIC SMALL ALPHA

1D737
MATHEMATICAL BOLD ITALIC SMALL BETA

1D738
MATHEMATICAL BOLD ITALIC SMALL GAMMA

1D739
MATHEMATICAL BOLD ITALIC SMALL DELTA

1D73A
MATHEMATICAL BOLD ITALIC SMALL EPSILON

1D73B
MATHEMATICAL BOLD ITALIC SMALL ZETA

1D73C
MATHEMATICAL BOLD ITALIC SMALL ETA

1D73D
MATHEMATICAL BOLD ITALIC SMALL THETA

1D73E
MATHEMATICAL BOLD ITALIC SMALL IOTA

1D73F
MATHEMATICAL BOLD ITALIC SMALL KAPPA

1D740
MATHEMATICAL BOLD ITALIC SMALL LAMDA

1D741
MATHEMATICAL BOLD ITALIC SMALL MU

1D742
MATHEMATICAL BOLD ITALIC SMALL NU

1D743
MATHEMATICAL BOLD ITALIC SMALL XI

1D744
MATHEMATICAL BOLD ITALIC SMALL OMICRON

1D745
MATHEMATICAL BOLD ITALIC SMALL PI

1D746
MATHEMATICAL BOLD ITALIC SMALL RHO

1D747
MATHEMATICAL BOLD ITALIC SMALL FINAL SIGMA

1D748
MATHEMATICAL BOLD ITALIC SMALL SIGMA

1D749
MATHEMATICAL BOLD ITALIC SMALL TAU

1D74A
MATHEMATICAL BOLD ITALIC SMALL UPSILON

1D74B
MATHEMATICAL BOLD ITALIC SMALL PHI

1D74C
MATHEMATICAL BOLD ITALIC SMALL CHI

1D74D
MATHEMATICAL BOLD ITALIC SMALL PSI

1D74E
MATHEMATICAL BOLD ITALIC SMALL OMEGA

1D74F
MATHEMATICAL BOLD ITALIC PARTIAL DIFFERENTIAL

1D750
MATHEMATICAL BOLD ITALIC EPSILON SYMBOL

1D751
MATHEMATICAL BOLD ITALIC THETA SYMBOL

1D752
MATHEMATICAL BOLD ITALIC KAPPA SYMBOL

1D753
MATHEMATICAL BOLD ITALIC PHI SYMBOL

1D754
MATHEMATICAL BOLD ITALIC RHO SYMBOL

1D755
MATHEMATICAL BOLD ITALIC PI SYMBOL

1D756
MATHEMATICAL SANS-SERIF BOLD CAPITAL ALPHA

1D757
MATHEMATICAL SANS-SERIF BOLD CAPITAL BETA

1D758
MATHEMATICAL SANS-SERIF BOLD CAPITAL GAMMA

1D759
MATHEMATICAL SANS-SERIF BOLD CAPITAL DELTA

1D75A
MATHEMATICAL SANS-SERIF BOLD CAPITAL EPSILON

1D75B
MATHEMATICAL SANS-SERIF BOLD CAPITAL ZETA

1D75C
MATHEMATICAL SANS-SERIF BOLD CAPITAL ETA

1D75D
MATHEMATICAL SANS-SERIF BOLD CAPITAL THETA

1D75E
MATHEMATICAL SANS-SERIF BOLD CAPITAL IOTA

1D75F
MATHEMATICAL SANS-SERIF BOLD CAPITAL KAPPA

1D760
MATHEMATICAL SANS-SERIF BOLD CAPITAL LAMDA

1D761
MATHEMATICAL SANS-SERIF BOLD CAPITAL MU

1D762
MATHEMATICAL SANS-SERIF BOLD CAPITAL NU

1D763
MATHEMATICAL SANS-SERIF BOLD CAPITAL XI

1D764
MATHEMATICAL SANS-SERIF BOLD CAPITAL OMICRON

1D765
MATHEMATICAL SANS-SERIF BOLD CAPITAL PI

1D766
MATHEMATICAL SANS-SERIF BOLD CAPITAL RHO

1D767
MATHEMATICAL SANS-SERIF BOLD CAPITAL THETA SYMBOL

1D768
MATHEMATICAL SANS-SERIF BOLD CAPITAL SIGMA

1D769
MATHEMATICAL SANS-SERIF BOLD CAPITAL TAU

1D76A
MATHEMATICAL SANS-SERIF BOLD CAPITAL UPSILON

1D76B
MATHEMATICAL SANS-SERIF BOLD CAPITAL PHI

1D76C
MATHEMATICAL SANS-SERIF BOLD CAPITAL CHI

1D76D
MATHEMATICAL SANS-SERIF BOLD CAPITAL PSI

1D76E
MATHEMATICAL SANS-SERIF BOLD CAPITAL OMEGA

1D76F
MATHEMATICAL SANS-SERIF BOLD NABLA

1D770
MATHEMATICAL SANS-SERIF BOLD SMALL ALPHA

1D771
MATHEMATICAL SANS-SERIF BOLD SMALL BETA

1D772
MATHEMATICAL SANS-SERIF BOLD SMALL GAMMA

1D773
MATHEMATICAL SANS-SERIF BOLD SMALL DELTA

1D774
MATHEMATICAL SANS-SERIF BOLD SMALL EPSILON

1D775
MATHEMATICAL SANS-SERIF BOLD SMALL ZETA

1D776
MATHEMATICAL SANS-SERIF BOLD SMALL ETA

1D777
MATHEMATICAL SANS-SERIF BOLD SMALL THETA

1D778
MATHEMATICAL SANS-SERIF BOLD SMALL IOTA

1D779
MATHEMATICAL SANS-SERIF BOLD SMALL KAPPA

1D77A
MATHEMATICAL SANS-SERIF BOLD SMALL LAMDA

1D77B
MATHEMATICAL SANS-SERIF BOLD SMALL MU

1D77C
MATHEMATICAL SANS-SERIF BOLD SMALL NU

1D77D
MATHEMATICAL SANS-SERIF BOLD SMALL XI

1D77E
MATHEMATICAL SANS-SERIF BOLD SMALL OMICRON

1D77F
MATHEMATICAL SANS-SERIF BOLD SMALL PI

1D780
MATHEMATICAL SANS-SERIF BOLD SMALL RHO

1D781
MATHEMATICAL SANS-SERIF BOLD SMALL FINAL SIGMA

1D782
MATHEMATICAL SANS-SERIF BOLD SMALL SIGMA

1D783
MATHEMATICAL SANS-SERIF BOLD SMALL TAU

1D784
MATHEMATICAL SANS-SERIF BOLD SMALL UPSILON

1D785
MATHEMATICAL SANS-SERIF BOLD SMALL PHI

1D786
MATHEMATICAL SANS-SERIF BOLD SMALL CHI

1D787
MATHEMATICAL SANS-SERIF BOLD SMALL PSI

1D788
MATHEMATICAL SANS-SERIF BOLD SMALL OMEGA

1D789
MATHEMATICAL SANS-SERIF BOLD PARTIAL DIFFERENTIAL

1D78A
MATHEMATICAL SANS-SERIF BOLD EPSILON SYMBOL

1D78B
MATHEMATICAL SANS-SERIF BOLD THETA SYMBOL

1D78C
MATHEMATICAL SANS-SERIF BOLD KAPPA SYMBOL

1D78D
MATHEMATICAL SANS-SERIF BOLD PHI SYMBOL

1D78E
MATHEMATICAL SANS-SERIF BOLD RHO SYMBOL

1D78F
MATHEMATICAL SANS-SERIF BOLD PI SYMBOL

1D790
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL ALPHA

1D791
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL BETA

1D792
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL GAMMA

1D793
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL DELTA

1D794
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL EPSILON

1D795
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL ZETA

1D796
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL ETA

1D797
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL THETA

1D798
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL IOTA

1D799
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL KAPPA

1D79A
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL LAMDA

1D79B
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL MU

1D79C
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL NU

1D79D
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL XI

1D79E
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL OMICRON

1D79F
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL PI

1D7A0
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL RHO

1D7A1
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL THETA SYMBOL

1D7A2
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL SIGMA

1D7A3
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL TAU

1D7A4
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL UPSILON

1D7A5
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL PHI

1D7A6
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL CHI

1D7A7
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL PSI

1D7A8
MATHEMATICAL SANS-SERIF BOLD ITALIC CAPITAL OMEGA

1D7A9
MATHEMATICAL SANS-SERIF BOLD ITALIC NABLA

1D7AA
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL ALPHA

1D7AB
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL BETA

1D7AC
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL GAMMA

1D7AD
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL DELTA

1D7AE
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL EPSILON

1D7AF
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL ZETA

1D7B0
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL ETA

1D7B1
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL THETA

1D7B2
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL IOTA

1D7B3
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL KAPPA

1D7B4
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL LAMDA

1D7B5
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL MU

1D7B6
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL NU

1D7B7
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL XI

1D7B8
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL OMICRON

1D7B9
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL PI

1D7BA
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL RHO

1D7BB
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL FINAL SIGMA

1D7BC
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL SIGMA

1D7BD
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL TAU

1D7BE
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL UPSILON

1D7BF
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL PHI

1D7C0
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL CHI

1D7C1
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL PSI

1D7C2
MATHEMATICAL SANS-SERIF BOLD ITALIC SMALL OMEGA

1D7C3
MATHEMATICAL SANS-SERIF BOLD ITALIC PARTIAL DIFFERENTIAL

1D7C4
MATHEMATICAL SANS-SERIF BOLD ITALIC EPSILON SYMBOL

1D7C5
MATHEMATICAL SANS-SERIF BOLD ITALIC THETA SYMBOL

1D7C6
MATHEMATICAL SANS-SERIF BOLD ITALIC KAPPA SYMBOL

1D7C7
MATHEMATICAL SANS-SERIF BOLD ITALIC PHI SYMBOL

1D7C8
MATHEMATICAL SANS-SERIF BOLD ITALIC RHO SYMBOL

1D7C9
MATHEMATICAL SANS-SERIF BOLD ITALIC PI SYMBOL

1D7CA
MATHEMATICAL BOLD CAPITAL DIGAMMA

1D7CB
MATHEMATICAL BOLD SMALL DIGAMMA

1D7CE
MATHEMATICAL BOLD DIGIT ZERO

1D7CF
MATHEMATICAL BOLD DIGIT ONE

1D7D0
MATHEMATICAL BOLD DIGIT TWO

1D7D1
MATHEMATICAL BOLD DIGIT THREE

1D7D2
MATHEMATICAL BOLD DIGIT FOUR

1D7D3
MATHEMATICAL BOLD DIGIT FIVE

1D7D4
MATHEMATICAL BOLD DIGIT SIX

1D7D5
MATHEMATICAL BOLD DIGIT SEVEN

1D7D6
MATHEMATICAL BOLD DIGIT EIGHT

1D7D7
MATHEMATICAL BOLD DIGIT NINE

1D7D8
MATHEMATICAL DOUBLE-STRUCK DIGIT ZERO

1D7D9
MATHEMATICAL DOUBLE-STRUCK DIGIT ONE

1D7DA
MATHEMATICAL DOUBLE-STRUCK DIGIT TWO

1D7DB
MATHEMATICAL DOUBLE-STRUCK DIGIT THREE

1D7DC
MATHEMATICAL DOUBLE-STRUCK DIGIT FOUR

1D7DD
MATHEMATICAL DOUBLE-STRUCK DIGIT FIVE

1D7DE
MATHEMATICAL DOUBLE-STRUCK DIGIT SIX

1D7DF
MATHEMATICAL DOUBLE-STRUCK DIGIT SEVEN

1D7E0
MATHEMATICAL DOUBLE-STRUCK DIGIT EIGHT

1D7E1
MATHEMATICAL DOUBLE-STRUCK DIGIT NINE

1D7E2
MATHEMATICAL SANS-SERIF DIGIT ZERO

1D7E3
MATHEMATICAL SANS-SERIF DIGIT ONE

1D7E4
MATHEMATICAL SANS-SERIF DIGIT TWO

1D7E5
MATHEMATICAL SANS-SERIF DIGIT THREE

1D7E6
MATHEMATICAL SANS-SERIF DIGIT FOUR

1D7E7
MATHEMATICAL SANS-SERIF DIGIT FIVE

1D7E8
MATHEMATICAL SANS-SERIF DIGIT SIX

1D7E9
MATHEMATICAL SANS-SERIF DIGIT SEVEN

1D7EA
MATHEMATICAL SANS-SERIF DIGIT EIGHT

1D7EB
MATHEMATICAL SANS-SERIF DIGIT NINE

1D7EC
MATHEMATICAL SANS-SERIF BOLD DIGIT ZERO

1D7ED
MATHEMATICAL SANS-SERIF BOLD DIGIT ONE

1D7EE
MATHEMATICAL SANS-SERIF BOLD DIGIT TWO

1D7EF
MATHEMATICAL SANS-SERIF BOLD DIGIT THREE

1D7F0
MATHEMATICAL SANS-SERIF BOLD DIGIT FOUR

1D7F1
MATHEMATICAL SANS-SERIF BOLD DIGIT FIVE

1D7F2
MATHEMATICAL SANS-SERIF BOLD DIGIT SIX

1D7F3
MATHEMATICAL SANS-SERIF BOLD DIGIT SEVEN

1D7F4
MATHEMATICAL SANS-SERIF BOLD DIGIT EIGHT

1D7F5
MATHEMATICAL SANS-SERIF BOLD DIGIT NINE

1D7F6
MATHEMATICAL MONOSPACE DIGIT ZERO

1D7F7
MATHEMATICAL MONOSPACE DIGIT ONE

1D7F8
MATHEMATICAL MONOSPACE DIGIT TWO

1D7F9
MATHEMATICAL MONOSPACE DIGIT THREE

1D7FA
MATHEMATICAL MONOSPACE DIGIT FOUR

1D7FB
MATHEMATICAL MONOSPACE DIGIT FIVE

1D7FC
MATHEMATICAL MONOSPACE DIGIT SIX

1D7FD
MATHEMATICAL MONOSPACE DIGIT SEVEN

1D7FE
MATHEMATICAL MONOSPACE DIGIT EIGHT

1D7FF
MATHEMATICAL MONOSPACE DIGIT NINE

1D800
SIGNWRITING HAND-FIST INDEX

1D801
SIGNWRITING HAND-CIRCLE INDEX

1D802
SIGNWRITING HAND-CUP INDEX

1D803
SIGNWRITING HAND-OVAL INDEX

1D804
SIGNWRITING HAND-HINGE INDEX

1D805
SIGNWRITING HAND-ANGLE INDEX

1D806
SIGNWRITING HAND-FIST INDEX BENT

1D807
SIGNWRITING HAND-CIRCLE INDEX BENT

1D808
SIGNWRITING HAND-FIST THUMB UNDER INDEX BENT

1D809
SIGNWRITING HAND-FIST INDEX RAISED KNUCKLE

1D80A
SIGNWRITING HAND-FIST INDEX CUPPED

1D80B
SIGNWRITING HAND-FIST INDEX HINGED

1D80C
SIGNWRITING HAND-FIST INDEX HINGED LOW

1D80D
SIGNWRITING HAND-CIRCLE INDEX HINGE

1D80E
SIGNWRITING HAND-FIST INDEX MIDDLE

1D80F
SIGNWRITING HAND-CIRCLE INDEX MIDDLE

1D810
SIGNWRITING HAND-FIST INDEX MIDDLE BENT

1D811
SIGNWRITING HAND-FIST INDEX MIDDLE RAISED KNUCKLES

1D812
SIGNWRITING HAND-FIST INDEX MIDDLE HINGED

1D813
SIGNWRITING HAND-FIST INDEX UP MIDDLE HINGED

1D814
SIGNWRITING HAND-FIST INDEX HINGED MIDDLE UP

1D815
SIGNWRITING HAND-FIST INDEX MIDDLE CONJOINED

1D816
SIGNWRITING HAND-FIST INDEX MIDDLE CONJOINED INDEX BENT

1D817
SIGNWRITING HAND-FIST INDEX MIDDLE CONJOINED MIDDLE BENT

1D818
SIGNWRITING HAND-FIST INDEX MIDDLE CONJOINED CUPPED

1D819
SIGNWRITING HAND-FIST INDEX MIDDLE CONJOINED HINGED

1D81A
SIGNWRITING HAND-FIST INDEX MIDDLE CROSSED

1D81B
SIGNWRITING HAND-CIRCLE INDEX MIDDLE CROSSED

1D81C
SIGNWRITING HAND-FIST MIDDLE BENT OVER INDEX

1D81D
SIGNWRITING HAND-FIST INDEX BENT OVER MIDDLE

1D81E
SIGNWRITING HAND-FIST INDEX MIDDLE THUMB

1D81F
SIGNWRITING HAND-CIRCLE INDEX MIDDLE THUMB

1D820
SIGNWRITING HAND-FIST INDEX MIDDLE STRAIGHT THUMB BENT

1D821
SIGNWRITING HAND-FIST INDEX MIDDLE BENT THUMB STRAIGHT

1D822
SIGNWRITING HAND-FIST INDEX MIDDLE THUMB BENT

1D823
SIGNWRITING HAND-FIST INDEX MIDDLE HINGED SPREAD THUMB SIDE

1D824
SIGNWRITING HAND-FIST INDEX UP MIDDLE HINGED THUMB SIDE

1D825
SIGNWRITING HAND-FIST INDEX UP MIDDLE HINGED THUMB CONJOINED

1D826
SIGNWRITING HAND-FIST INDEX HINGED MIDDLE UP THUMB SIDE

1D827
SIGNWRITING HAND-FIST INDEX MIDDLE UP SPREAD THUMB FORWARD

1D828
SIGNWRITING HAND-FIST INDEX MIDDLE THUMB CUPPED

1D829
SIGNWRITING HAND-FIST INDEX MIDDLE THUMB CIRCLED

1D82A
SIGNWRITING HAND-FIST INDEX MIDDLE THUMB HOOKED

1D82B
SIGNWRITING HAND-FIST INDEX MIDDLE THUMB HINGED

1D82C
SIGNWRITING HAND-FIST THUMB BETWEEN INDEX MIDDLE STRAIGHT

1D82D
SIGNWRITING HAND-FIST INDEX MIDDLE CONJOINED THUMB SIDE

1D82E
SIGNWRITING HAND-FIST INDEX MIDDLE CONJOINED THUMB SIDE CONJOINED

1D82F
SIGNWRITING HAND-FIST INDEX MIDDLE CONJOINED THUMB SIDE BENT

1D830
SIGNWRITING HAND-FIST MIDDLE THUMB HOOKED INDEX UP

1D831
SIGNWRITING HAND-FIST INDEX THUMB HOOKED MIDDLE UP

1D832
SIGNWRITING HAND-FIST INDEX MIDDLE CONJOINED HINGED THUMB SIDE

1D833
SIGNWRITING HAND-FIST INDEX MIDDLE CROSSED THUMB SIDE

1D834
SIGNWRITING HAND-FIST INDEX MIDDLE CONJOINED THUMB FORWARD

1D835
SIGNWRITING HAND-FIST INDEX MIDDLE CONJOINED CUPPED THUMB FORWARD

1D836
SIGNWRITING HAND-FIST MIDDLE THUMB CUPPED INDEX UP

1D837
SIGNWRITING HAND-FIST INDEX THUMB CUPPED MIDDLE UP

1D838
SIGNWRITING HAND-FIST MIDDLE THUMB CIRCLED INDEX UP

1D839
SIGNWRITING HAND-FIST MIDDLE THUMB CIRCLED INDEX HINGED

1D83A
SIGNWRITING HAND-FIST INDEX THUMB ANGLED OUT MIDDLE UP

1D83B
SIGNWRITING HAND-FIST INDEX THUMB ANGLED IN MIDDLE UP

1D83C
SIGNWRITING HAND-FIST INDEX THUMB CIRCLED MIDDLE UP

1D83D
SIGNWRITING HAND-FIST INDEX MIDDLE THUMB CONJOINED HINGED

1D83E
SIGNWRITING HAND-FIST INDEX MIDDLE THUMB ANGLED OUT

1D83F
SIGNWRITING HAND-FIST INDEX MIDDLE THUMB ANGLED

1D840
SIGNWRITING HAND-FIST MIDDLE THUMB ANGLED OUT INDEX UP

1D841
SIGNWRITING HAND-FIST MIDDLE THUMB ANGLED OUT INDEX CROSSED

1D842
SIGNWRITING HAND-FIST MIDDLE THUMB ANGLED INDEX UP

1D843
SIGNWRITING HAND-FIST INDEX THUMB HOOKED MIDDLE HINGED

1D844
SIGNWRITING HAND-FLAT FOUR FINGERS

1D845
SIGNWRITING HAND-FLAT FOUR FINGERS BENT

1D846
SIGNWRITING HAND-FLAT FOUR FINGERS HINGED

1D847
SIGNWRITING HAND-FLAT FOUR FINGERS CONJOINED

1D848
SIGNWRITING HAND-FLAT FOUR FINGERS CONJOINED SPLIT

1D849
SIGNWRITING HAND-CLAW FOUR FINGERS CONJOINED

1D84A
SIGNWRITING HAND-FIST FOUR FINGERS CONJOINED BENT

1D84B
SIGNWRITING HAND-HINGE FOUR FINGERS CONJOINED

1D84C
SIGNWRITING HAND-FLAT FIVE FINGERS SPREAD

1D84D
SIGNWRITING HAND-FLAT HEEL FIVE FINGERS SPREAD

1D84E
SIGNWRITING HAND-FLAT FIVE FINGERS SPREAD FOUR BENT

1D84F
SIGNWRITING HAND-FLAT HEEL FIVE FINGERS SPREAD FOUR BENT

1D850
SIGNWRITING HAND-FLAT FIVE FINGERS SPREAD BENT

1D851
SIGNWRITING HAND-FLAT HEEL FIVE FINGERS SPREAD BENT

1D852
SIGNWRITING HAND-FLAT FIVE FINGERS SPREAD THUMB FORWARD

1D853
SIGNWRITING HAND-CUP FIVE FINGERS SPREAD

1D854
SIGNWRITING HAND-CUP FIVE FINGERS SPREAD OPEN

1D855
SIGNWRITING HAND-HINGE FIVE FINGERS SPREAD OPEN

1D856
SIGNWRITING HAND-OVAL FIVE FINGERS SPREAD

1D857
SIGNWRITING HAND-FLAT FIVE FINGERS SPREAD HINGED

1D858
SIGNWRITING HAND-FLAT FIVE FINGERS SPREAD HINGED THUMB SIDE

1D859
SIGNWRITING HAND-FLAT FIVE FINGERS SPREAD HINGED NO THUMB

1D85A
SIGNWRITING HAND-FLAT

1D85B
SIGNWRITING HAND-FLAT BETWEEN PALM FACINGS

1D85C
SIGNWRITING HAND-FLAT HEEL

1D85D
SIGNWRITING HAND-FLAT THUMB SIDE

1D85E
SIGNWRITING HAND-FLAT HEEL THUMB SIDE

1D85F
SIGNWRITING HAND-FLAT THUMB BENT

1D860
SIGNWRITING HAND-FLAT THUMB FORWARD

1D861
SIGNWRITING HAND-FLAT SPLIT INDEX THUMB SIDE

1D862
SIGNWRITING HAND-FLAT SPLIT CENTRE

1D863
SIGNWRITING HAND-FLAT SPLIT CENTRE THUMB SIDE

1D864
SIGNWRITING HAND-FLAT SPLIT CENTRE THUMB SIDE BENT

1D865
SIGNWRITING HAND-FLAT SPLIT LITTLE

1D866
SIGNWRITING HAND-CLAW

1D867
SIGNWRITING HAND-CLAW THUMB SIDE

1D868
SIGNWRITING HAND-CLAW NO THUMB

1D869
SIGNWRITING HAND-CLAW THUMB FORWARD

1D86A
SIGNWRITING HAND-HOOK CURLICUE

1D86B
SIGNWRITING HAND-HOOK

1D86C
SIGNWRITING HAND-CUP OPEN

1D86D
SIGNWRITING HAND-CUP

1D86E
SIGNWRITING HAND-CUP OPEN THUMB SIDE

1D86F
SIGNWRITING HAND-CUP THUMB SIDE

1D870
SIGNWRITING HAND-CUP OPEN NO THUMB

1D871
SIGNWRITING HAND-CUP NO THUMB

1D872
SIGNWRITING HAND-CUP OPEN THUMB FORWARD

1D873
SIGNWRITING HAND-CUP THUMB FORWARD

1D874
SIGNWRITING HAND-CURLICUE OPEN

1D875
SIGNWRITING HAND-CURLICUE

1D876
SIGNWRITING HAND-CIRCLE

1D877
SIGNWRITING HAND-OVAL

1D878
SIGNWRITING HAND-OVAL THUMB SIDE

1D879
SIGNWRITING HAND-OVAL NO THUMB

1D87A
SIGNWRITING HAND-OVAL THUMB FORWARD

1D87B
SIGNWRITING HAND-HINGE OPEN

1D87C
SIGNWRITING HAND-HINGE OPEN THUMB FORWARD

1D87D
SIGNWRITING HAND-HINGE

1D87E
SIGNWRITING HAND-HINGE SMALL

1D87F
SIGNWRITING HAND-HINGE OPEN THUMB SIDE

1D880
SIGNWRITING HAND-HINGE THUMB SIDE

1D881
SIGNWRITING HAND-HINGE OPEN NO THUMB

1D882
SIGNWRITING HAND-HINGE NO THUMB

1D883
SIGNWRITING HAND-HINGE THUMB SIDE TOUCHING INDEX

1D884
SIGNWRITING HAND-HINGE THUMB BETWEEN MIDDLE RING

1D885
SIGNWRITING HAND-ANGLE

1D886
SIGNWRITING HAND-FIST INDEX MIDDLE RING

1D887
SIGNWRITING HAND-CIRCLE INDEX MIDDLE RING

1D888
SIGNWRITING HAND-HINGE INDEX MIDDLE RING

1D889
SIGNWRITING HAND-ANGLE INDEX MIDDLE RING

1D88A
SIGNWRITING HAND-HINGE LITTLE

1D88B
SIGNWRITING HAND-FIST INDEX MIDDLE RING BENT

1D88C
SIGNWRITING HAND-FIST INDEX MIDDLE RING CONJOINED

1D88D
SIGNWRITING HAND-HINGE INDEX MIDDLE RING CONJOINED

1D88E
SIGNWRITING HAND-FIST LITTLE DOWN

1D88F
SIGNWRITING HAND-FIST LITTLE DOWN RIPPLE STRAIGHT

1D890
SIGNWRITING HAND-FIST LITTLE DOWN RIPPLE CURVED

1D891
SIGNWRITING HAND-FIST LITTLE DOWN OTHERS CIRCLED

1D892
SIGNWRITING HAND-FIST LITTLE UP

1D893
SIGNWRITING HAND-FIST THUMB UNDER LITTLE UP

1D894
SIGNWRITING HAND-CIRCLE LITTLE UP

1D895
SIGNWRITING HAND-OVAL LITTLE UP

1D896
SIGNWRITING HAND-ANGLE LITTLE UP

1D897
SIGNWRITING HAND-FIST LITTLE RAISED KNUCKLE

1D898
SIGNWRITING HAND-FIST LITTLE BENT

1D899
SIGNWRITING HAND-FIST LITTLE TOUCHES THUMB

1D89A
SIGNWRITING HAND-FIST LITTLE THUMB

1D89B
SIGNWRITING HAND-HINGE LITTLE THUMB

1D89C
SIGNWRITING HAND-FIST LITTLE INDEX THUMB

1D89D
SIGNWRITING HAND-HINGE LITTLE INDEX THUMB

1D89E
SIGNWRITING HAND-ANGLE LITTLE INDEX THUMB INDEX THUMB OUT

1D89F
SIGNWRITING HAND-ANGLE LITTLE INDEX THUMB INDEX THUMB

1D8A0
SIGNWRITING HAND-FIST LITTLE INDEX

1D8A1
SIGNWRITING HAND-CIRCLE LITTLE INDEX

1D8A2
SIGNWRITING HAND-HINGE LITTLE INDEX

1D8A3
SIGNWRITING HAND-ANGLE LITTLE INDEX

1D8A4
SIGNWRITING HAND-FIST INDEX MIDDLE LITTLE

1D8A5
SIGNWRITING HAND-CIRCLE INDEX MIDDLE LITTLE

1D8A6
SIGNWRITING HAND-HINGE INDEX MIDDLE LITTLE

1D8A7
SIGNWRITING HAND-HINGE RING

1D8A8
SIGNWRITING HAND-ANGLE INDEX MIDDLE LITTLE

1D8A9
SIGNWRITING HAND-FIST INDEX MIDDLE CROSS LITTLE

1D8AA
SIGNWRITING HAND-CIRCLE INDEX MIDDLE CROSS LITTLE

1D8AB
SIGNWRITING HAND-FIST RING DOWN

1D8AC
SIGNWRITING HAND-HINGE RING DOWN INDEX THUMB HOOK MIDDLE

1D8AD
SIGNWRITING HAND-ANGLE RING DOWN MIDDLE THUMB INDEX CROSS

1D8AE
SIGNWRITING HAND-FIST RING UP

1D8AF
SIGNWRITING HAND-FIST RING RAISED KNUCKLE

1D8B0
SIGNWRITING HAND-FIST RING LITTLE

1D8B1
SIGNWRITING HAND-CIRCLE RING LITTLE

1D8B2
SIGNWRITING HAND-OVAL RING LITTLE

1D8B3
SIGNWRITING HAND-ANGLE RING LITTLE

1D8B4
SIGNWRITING HAND-FIST RING MIDDLE

1D8B5
SIGNWRITING HAND-FIST RING MIDDLE CONJOINED

1D8B6
SIGNWRITING HAND-FIST RING MIDDLE RAISED KNUCKLES

1D8B7
SIGNWRITING HAND-FIST RING INDEX

1D8B8
SIGNWRITING HAND-FIST RING THUMB

1D8B9
SIGNWRITING HAND-HOOK RING THUMB

1D8BA
SIGNWRITING HAND-FIST INDEX RING LITTLE

1D8BB
SIGNWRITING HAND-CIRCLE INDEX RING LITTLE

1D8BC
SIGNWRITING HAND-CURLICUE INDEX RING LITTLE ON

1D8BD
SIGNWRITING HAND-HOOK INDEX RING LITTLE OUT

1D8BE
SIGNWRITING HAND-HOOK INDEX RING LITTLE IN

1D8BF
SIGNWRITING HAND-HOOK INDEX RING LITTLE UNDER

1D8C0
SIGNWRITING HAND-CUP INDEX RING LITTLE

1D8C1
SIGNWRITING HAND-HINGE INDEX RING LITTLE

1D8C2
SIGNWRITING HAND-ANGLE INDEX RING LITTLE OUT

1D8C3
SIGNWRITING HAND-ANGLE INDEX RING LITTLE

1D8C4
SIGNWRITING HAND-FIST MIDDLE DOWN

1D8C5
SIGNWRITING HAND-HINGE MIDDLE

1D8C6
SIGNWRITING HAND-FIST MIDDLE UP

1D8C7
SIGNWRITING HAND-CIRCLE MIDDLE UP

1D8C8
SIGNWRITING HAND-FIST MIDDLE RAISED KNUCKLE

1D8C9
SIGNWRITING HAND-FIST MIDDLE UP THUMB SIDE

1D8CA
SIGNWRITING HAND-HOOK MIDDLE THUMB

1D8CB
SIGNWRITING HAND-FIST MIDDLE THUMB LITTLE

1D8CC
SIGNWRITING HAND-FIST MIDDLE LITTLE

1D8CD
SIGNWRITING HAND-FIST MIDDLE RING LITTLE

1D8CE
SIGNWRITING HAND-CIRCLE MIDDLE RING LITTLE

1D8CF
SIGNWRITING HAND-CURLICUE MIDDLE RING LITTLE ON

1D8D0
SIGNWRITING HAND-CUP MIDDLE RING LITTLE

1D8D1
SIGNWRITING HAND-HINGE MIDDLE RING LITTLE

1D8D2
SIGNWRITING HAND-ANGLE MIDDLE RING LITTLE OUT

1D8D3
SIGNWRITING HAND-ANGLE MIDDLE RING LITTLE IN

1D8D4
SIGNWRITING HAND-ANGLE MIDDLE RING LITTLE

1D8D5
SIGNWRITING HAND-CIRCLE MIDDLE RING LITTLE BENT

1D8D6
SIGNWRITING HAND-CLAW MIDDLE RING LITTLE CONJOINED

1D8D7
SIGNWRITING HAND-CLAW MIDDLE RING LITTLE CONJOINED SIDE

1D8D8
SIGNWRITING HAND-HOOK MIDDLE RING LITTLE CONJOINED OUT

1D8D9
SIGNWRITING HAND-HOOK MIDDLE RING LITTLE CONJOINED IN

1D8DA
SIGNWRITING HAND-HOOK MIDDLE RING LITTLE CONJOINED

1D8DB
SIGNWRITING HAND-HINGE INDEX HINGED

1D8DC
SIGNWRITING HAND-FIST INDEX THUMB SIDE

1D8DD
SIGNWRITING HAND-HINGE INDEX THUMB SIDE

1D8DE
SIGNWRITING HAND-FIST INDEX THUMB SIDE THUMB DIAGONAL

1D8DF
SIGNWRITING HAND-FIST INDEX THUMB SIDE THUMB CONJOINED

1D8E0
SIGNWRITING HAND-FIST INDEX THUMB SIDE THUMB BENT

1D8E1
SIGNWRITING HAND-FIST INDEX THUMB SIDE INDEX BENT

1D8E2
SIGNWRITING HAND-FIST INDEX THUMB SIDE BOTH BENT

1D8E3
SIGNWRITING HAND-FIST INDEX THUMB SIDE INDEX HINGE

1D8E4
SIGNWRITING HAND-FIST INDEX THUMB FORWARD INDEX STRAIGHT

1D8E5
SIGNWRITING HAND-FIST INDEX THUMB FORWARD INDEX BENT

1D8E6
SIGNWRITING HAND-FIST INDEX THUMB HOOK

1D8E7
SIGNWRITING HAND-FIST INDEX THUMB CURLICUE

1D8E8
SIGNWRITING HAND-FIST INDEX THUMB CURVE THUMB INSIDE

1D8E9
SIGNWRITING HAND-CLAW INDEX THUMB CURVE THUMB INSIDE

1D8EA
SIGNWRITING HAND-FIST INDEX THUMB CURVE THUMB UNDER

1D8EB
SIGNWRITING HAND-FIST INDEX THUMB CIRCLE

1D8EC
SIGNWRITING HAND-CUP INDEX THUMB

1D8ED
SIGNWRITING HAND-CUP INDEX THUMB OPEN

1D8EE
SIGNWRITING HAND-HINGE INDEX THUMB OPEN

1D8EF
SIGNWRITING HAND-HINGE INDEX THUMB LARGE

1D8F0
SIGNWRITING HAND-HINGE INDEX THUMB

1D8F1
SIGNWRITING HAND-HINGE INDEX THUMB SMALL

1D8F2
SIGNWRITING HAND-ANGLE INDEX THUMB OUT

1D8F3
SIGNWRITING HAND-ANGLE INDEX THUMB IN

1D8F4
SIGNWRITING HAND-ANGLE INDEX THUMB

1D8F5
SIGNWRITING HAND-FIST THUMB

1D8F6
SIGNWRITING HAND-FIST THUMB HEEL

1D8F7
SIGNWRITING HAND-FIST THUMB SIDE DIAGONAL

1D8F8
SIGNWRITING HAND-FIST THUMB SIDE CONJOINED

1D8F9
SIGNWRITING HAND-FIST THUMB SIDE BENT

1D8FA
SIGNWRITING HAND-FIST THUMB FORWARD

1D8FB
SIGNWRITING HAND-FIST THUMB BETWEEN INDEX MIDDLE

1D8FC
SIGNWRITING HAND-FIST THUMB BETWEEN MIDDLE RING

1D8FD
SIGNWRITING HAND-FIST THUMB BETWEEN RING LITTLE

1D8FE
SIGNWRITING HAND-FIST THUMB UNDER TWO FINGERS

1D8FF
SIGNWRITING HAND-FIST THUMB OVER TWO FINGERS

1D900
SIGNWRITING HAND-FIST THUMB UNDER THREE FINGERS

1D901
SIGNWRITING HAND-FIST THUMB UNDER FOUR FINGERS

1D902
SIGNWRITING HAND-FIST THUMB OVER FOUR RAISED KNUCKLES

1D903
SIGNWRITING HAND-FIST

1D904
SIGNWRITING HAND-FIST HEEL

1D905
SIGNWRITING TOUCH SINGLE

1D906
SIGNWRITING TOUCH MULTIPLE

1D907
SIGNWRITING TOUCH BETWEEN

1D908
SIGNWRITING GRASP SINGLE

1D909
SIGNWRITING GRASP MULTIPLE

1D90A
SIGNWRITING GRASP BETWEEN

1D90B
SIGNWRITING STRIKE SINGLE

1D90C
SIGNWRITING STRIKE MULTIPLE

1D90D
SIGNWRITING STRIKE BETWEEN

1D90E
SIGNWRITING BRUSH SINGLE

1D90F
SIGNWRITING BRUSH MULTIPLE

1D910
SIGNWRITING BRUSH BETWEEN

1D911
SIGNWRITING RUB SINGLE

1D912
SIGNWRITING RUB MULTIPLE

1D913
SIGNWRITING RUB BETWEEN

1D914
SIGNWRITING SURFACE SYMBOLS

1D915
SIGNWRITING SURFACE BETWEEN

1D916
SIGNWRITING SQUEEZE LARGE SINGLE

1D917
SIGNWRITING SQUEEZE SMALL SINGLE

1D918
SIGNWRITING SQUEEZE LARGE MULTIPLE

1D919
SIGNWRITING SQUEEZE SMALL MULTIPLE

1D91A
SIGNWRITING SQUEEZE SEQUENTIAL

1D91B
SIGNWRITING FLICK LARGE SINGLE

1D91C
SIGNWRITING FLICK SMALL SINGLE

1D91D
SIGNWRITING FLICK LARGE MULTIPLE

1D91E
SIGNWRITING FLICK SMALL MULTIPLE

1D91F
SIGNWRITING FLICK SEQUENTIAL

1D920
SIGNWRITING SQUEEZE FLICK ALTERNATING

1D921
SIGNWRITING MOVEMENT-HINGE UP DOWN LARGE

1D922
SIGNWRITING MOVEMENT-HINGE UP DOWN SMALL

1D923
SIGNWRITING MOVEMENT-HINGE UP SEQUENTIAL

1D924
SIGNWRITING MOVEMENT-HINGE DOWN SEQUENTIAL

1D925
SIGNWRITING MOVEMENT-HINGE UP DOWN ALTERNATING LARGE

1D926
SIGNWRITING MOVEMENT-HINGE UP DOWN ALTERNATING SMALL

1D927
SIGNWRITING MOVEMENT-HINGE SIDE TO SIDE SCISSORS

1D928
SIGNWRITING MOVEMENT-WALLPLANE FINGER CONTACT

1D929
SIGNWRITING MOVEMENT-FLOORPLANE FINGER CONTACT

1D92A
SIGNWRITING MOVEMENT-WALLPLANE SINGLE STRAIGHT SMALL

1D92B
SIGNWRITING MOVEMENT-WALLPLANE SINGLE STRAIGHT MEDIUM

1D92C
SIGNWRITING MOVEMENT-WALLPLANE SINGLE STRAIGHT LARGE

1D92D
SIGNWRITING MOVEMENT-WALLPLANE SINGLE STRAIGHT LARGEST

1D92E
SIGNWRITING MOVEMENT-WALLPLANE SINGLE WRIST FLEX

1D92F
SIGNWRITING MOVEMENT-WALLPLANE DOUBLE STRAIGHT

1D930
SIGNWRITING MOVEMENT-WALLPLANE DOUBLE WRIST FLEX

1D931
SIGNWRITING MOVEMENT-WALLPLANE DOUBLE ALTERNATING

1D932
SIGNWRITING MOVEMENT-WALLPLANE DOUBLE ALTERNATING WRIST FLEX

1D933
SIGNWRITING MOVEMENT-WALLPLANE CROSS

1D934
SIGNWRITING MOVEMENT-WALLPLANE TRIPLE STRAIGHT MOVEMENT

1D935
SIGNWRITING MOVEMENT-WALLPLANE TRIPLE WRIST FLEX

1D936
SIGNWRITING MOVEMENT-WALLPLANE TRIPLE ALTERNATING

1D937
SIGNWRITING MOVEMENT-WALLPLANE TRIPLE ALTERNATING WRIST FLEX

1D938
SIGNWRITING MOVEMENT-WALLPLANE BEND SMALL

1D939
SIGNWRITING MOVEMENT-WALLPLANE BEND MEDIUM

1D93A
SIGNWRITING MOVEMENT-WALLPLANE BEND LARGE

1D93B
SIGNWRITING MOVEMENT-WALLPLANE CORNER SMALL

1D93C
SIGNWRITING MOVEMENT-WALLPLANE CORNER MEDIUM

1D93D
SIGNWRITING MOVEMENT-WALLPLANE CORNER LARGE

1D93E
SIGNWRITING MOVEMENT-WALLPLANE CORNER ROTATION

1D93F
SIGNWRITING MOVEMENT-WALLPLANE CHECK SMALL

1D940
SIGNWRITING MOVEMENT-WALLPLANE CHECK MEDIUM

1D941
SIGNWRITING MOVEMENT-WALLPLANE CHECK LARGE

1D942
SIGNWRITING MOVEMENT-WALLPLANE BOX SMALL

1D943
SIGNWRITING MOVEMENT-WALLPLANE BOX MEDIUM

1D944
SIGNWRITING MOVEMENT-WALLPLANE BOX LARGE

1D945
SIGNWRITING MOVEMENT-WALLPLANE ZIGZAG SMALL

1D946
SIGNWRITING MOVEMENT-WALLPLANE ZIGZAG MEDIUM

1D947
SIGNWRITING MOVEMENT-WALLPLANE ZIGZAG LARGE

1D948
SIGNWRITING MOVEMENT-WALLPLANE PEAKS SMALL

1D949
SIGNWRITING MOVEMENT-WALLPLANE PEAKS MEDIUM

1D94A
SIGNWRITING MOVEMENT-WALLPLANE PEAKS LARGE

1D94B
SIGNWRITING TRAVEL-WALLPLANE ROTATION-WALLPLANE SINGLE

1D94C
SIGNWRITING TRAVEL-WALLPLANE ROTATION-WALLPLANE DOUBLE

1D94D
SIGNWRITING TRAVEL-WALLPLANE ROTATION-WALLPLANE ALTERNATING

1D94E
SIGNWRITING TRAVEL-WALLPLANE ROTATION-FLOORPLANE SINGLE

1D94F
SIGNWRITING TRAVEL-WALLPLANE ROTATION-FLOORPLANE DOUBLE

1D950
SIGNWRITING TRAVEL-WALLPLANE ROTATION-FLOORPLANE ALTERNATING

1D951
SIGNWRITING TRAVEL-WALLPLANE SHAKING

1D952
SIGNWRITING TRAVEL-WALLPLANE ARM SPIRAL SINGLE

1D953
SIGNWRITING TRAVEL-WALLPLANE ARM SPIRAL DOUBLE

1D954
SIGNWRITING TRAVEL-WALLPLANE ARM SPIRAL TRIPLE

1D955
SIGNWRITING MOVEMENT-DIAGONAL AWAY SMALL

1D956
SIGNWRITING MOVEMENT-DIAGONAL AWAY MEDIUM

1D957
SIGNWRITING MOVEMENT-DIAGONAL AWAY LARGE

1D958
SIGNWRITING MOVEMENT-DIAGONAL AWAY LARGEST

1D959
SIGNWRITING MOVEMENT-DIAGONAL TOWARDS SMALL

1D95A
SIGNWRITING MOVEMENT-DIAGONAL TOWARDS MEDIUM

1D95B
SIGNWRITING MOVEMENT-DIAGONAL TOWARDS LARGE

1D95C
SIGNWRITING MOVEMENT-DIAGONAL TOWARDS LARGEST

1D95D
SIGNWRITING MOVEMENT-DIAGONAL BETWEEN AWAY SMALL

1D95E
SIGNWRITING MOVEMENT-DIAGONAL BETWEEN AWAY MEDIUM

1D95F
SIGNWRITING MOVEMENT-DIAGONAL BETWEEN AWAY LARGE

1D960
SIGNWRITING MOVEMENT-DIAGONAL BETWEEN AWAY LARGEST

1D961
SIGNWRITING MOVEMENT-DIAGONAL BETWEEN TOWARDS SMALL

1D962
SIGNWRITING MOVEMENT-DIAGONAL BETWEEN TOWARDS MEDIUM

1D963
SIGNWRITING MOVEMENT-DIAGONAL BETWEEN TOWARDS LARGE

1D964
SIGNWRITING MOVEMENT-DIAGONAL BETWEEN TOWARDS LARGEST

1D965
SIGNWRITING MOVEMENT-FLOORPLANE SINGLE STRAIGHT SMALL

1D966
SIGNWRITING MOVEMENT-FLOORPLANE SINGLE STRAIGHT MEDIUM

1D967
SIGNWRITING MOVEMENT-FLOORPLANE SINGLE STRAIGHT LARGE

1D968
SIGNWRITING MOVEMENT-FLOORPLANE SINGLE STRAIGHT LARGEST

1D969
SIGNWRITING MOVEMENT-FLOORPLANE SINGLE WRIST FLEX

1D96A
SIGNWRITING MOVEMENT-FLOORPLANE DOUBLE STRAIGHT

1D96B
SIGNWRITING MOVEMENT-FLOORPLANE DOUBLE WRIST FLEX

1D96C
SIGNWRITING MOVEMENT-FLOORPLANE DOUBLE ALTERNATING

1D96D
SIGNWRITING MOVEMENT-FLOORPLANE DOUBLE ALTERNATING WRIST FLEX

1D96E
SIGNWRITING MOVEMENT-FLOORPLANE CROSS

1D96F
SIGNWRITING MOVEMENT-FLOORPLANE TRIPLE STRAIGHT MOVEMENT

1D970
SIGNWRITING MOVEMENT-FLOORPLANE TRIPLE WRIST FLEX

1D971
SIGNWRITING MOVEMENT-FLOORPLANE TRIPLE ALTERNATING MOVEMENT

1D972
SIGNWRITING MOVEMENT-FLOORPLANE TRIPLE ALTERNATING WRIST FLEX

1D973
SIGNWRITING MOVEMENT-FLOORPLANE BEND

1D974
SIGNWRITING MOVEMENT-FLOORPLANE CORNER SMALL

1D975
SIGNWRITING MOVEMENT-FLOORPLANE CORNER MEDIUM

1D976
SIGNWRITING MOVEMENT-FLOORPLANE CORNER LARGE

1D977
SIGNWRITING MOVEMENT-FLOORPLANE CHECK

1D978
SIGNWRITING MOVEMENT-FLOORPLANE BOX SMALL

1D979
SIGNWRITING MOVEMENT-FLOORPLANE BOX MEDIUM

1D97A
SIGNWRITING MOVEMENT-FLOORPLANE BOX LARGE

1D97B
SIGNWRITING MOVEMENT-FLOORPLANE ZIGZAG SMALL

1D97C
SIGNWRITING MOVEMENT-FLOORPLANE ZIGZAG MEDIUM

1D97D
SIGNWRITING MOVEMENT-FLOORPLANE ZIGZAG LARGE

1D97E
SIGNWRITING MOVEMENT-FLOORPLANE PEAKS SMALL

1D97F
SIGNWRITING MOVEMENT-FLOORPLANE PEAKS MEDIUM

1D980
SIGNWRITING MOVEMENT-FLOORPLANE PEAKS LARGE

1D981
SIGNWRITING TRAVEL-FLOORPLANE ROTATION-FLOORPLANE SINGLE

1D982
SIGNWRITING TRAVEL-FLOORPLANE ROTATION-FLOORPLANE DOUBLE

1D983
SIGNWRITING TRAVEL-FLOORPLANE ROTATION-FLOORPLANE ALTERNATING

1D984
SIGNWRITING TRAVEL-FLOORPLANE ROTATION-WALLPLANE SINGLE

1D985
SIGNWRITING TRAVEL-FLOORPLANE ROTATION-WALLPLANE DOUBLE

1D986
SIGNWRITING TRAVEL-FLOORPLANE ROTATION-WALLPLANE ALTERNATING

1D987
SIGNWRITING TRAVEL-FLOORPLANE SHAKING

1D988
SIGNWRITING MOVEMENT-WALLPLANE CURVE QUARTER SMALL

1D989
SIGNWRITING MOVEMENT-WALLPLANE CURVE QUARTER MEDIUM

1D98A
SIGNWRITING MOVEMENT-WALLPLANE CURVE QUARTER LARGE

1D98B
SIGNWRITING MOVEMENT-WALLPLANE CURVE QUARTER LARGEST

1D98C
SIGNWRITING MOVEMENT-WALLPLANE CURVE HALF-CIRCLE SMALL

1D98D
SIGNWRITING MOVEMENT-WALLPLANE CURVE HALF-CIRCLE MEDIUM

1D98E
SIGNWRITING MOVEMENT-WALLPLANE CURVE HALF-CIRCLE LARGE

1D98F
SIGNWRITING MOVEMENT-WALLPLANE CURVE HALF-CIRCLE LARGEST

1D990
SIGNWRITING MOVEMENT-WALLPLANE CURVE THREE-QUARTER CIRCLE SMALL

1D991
SIGNWRITING MOVEMENT-WALLPLANE CURVE THREE-QUARTER CIRCLE MEDIUM

1D992
SIGNWRITING MOVEMENT-WALLPLANE HUMP SMALL

1D993
SIGNWRITING MOVEMENT-WALLPLANE HUMP MEDIUM

1D994
SIGNWRITING MOVEMENT-WALLPLANE HUMP LARGE

1D995
SIGNWRITING MOVEMENT-WALLPLANE LOOP SMALL

1D996
SIGNWRITING MOVEMENT-WALLPLANE LOOP MEDIUM

1D997
SIGNWRITING MOVEMENT-WALLPLANE LOOP LARGE

1D998
SIGNWRITING MOVEMENT-WALLPLANE LOOP SMALL DOUBLE

1D999
SIGNWRITING MOVEMENT-WALLPLANE WAVE CURVE DOUBLE SMALL

1D99A
SIGNWRITING MOVEMENT-WALLPLANE WAVE CURVE DOUBLE MEDIUM

1D99B
SIGNWRITING MOVEMENT-WALLPLANE WAVE CURVE DOUBLE LARGE

1D99C
SIGNWRITING MOVEMENT-WALLPLANE WAVE CURVE TRIPLE SMALL

1D99D
SIGNWRITING MOVEMENT-WALLPLANE WAVE CURVE TRIPLE MEDIUM

1D99E
SIGNWRITING MOVEMENT-WALLPLANE WAVE CURVE TRIPLE LARGE

1D99F
SIGNWRITING MOVEMENT-WALLPLANE CURVE THEN STRAIGHT

1D9A0
SIGNWRITING MOVEMENT-WALLPLANE CURVED CROSS SMALL

1D9A1
SIGNWRITING MOVEMENT-WALLPLANE CURVED CROSS MEDIUM

1D9A2
SIGNWRITING ROTATION-WALLPLANE SINGLE

1D9A3
SIGNWRITING ROTATION-WALLPLANE DOUBLE

1D9A4
SIGNWRITING ROTATION-WALLPLANE ALTERNATE

1D9A5
SIGNWRITING MOVEMENT-WALLPLANE SHAKING

1D9A6
SIGNWRITING MOVEMENT-WALLPLANE CURVE HITTING FRONT WALL

1D9A7
SIGNWRITING MOVEMENT-WALLPLANE HUMP HITTING FRONT WALL

1D9A8
SIGNWRITING MOVEMENT-WALLPLANE LOOP HITTING FRONT WALL

1D9A9
SIGNWRITING MOVEMENT-WALLPLANE WAVE HITTING FRONT WALL

1D9AA
SIGNWRITING ROTATION-WALLPLANE SINGLE HITTING FRONT WALL

1D9AB
SIGNWRITING ROTATION-WALLPLANE DOUBLE HITTING FRONT WALL

1D9AC
SIGNWRITING ROTATION-WALLPLANE ALTERNATING HITTING FRONT WALL

1D9AD
SIGNWRITING MOVEMENT-WALLPLANE CURVE HITTING CHEST

1D9AE
SIGNWRITING MOVEMENT-WALLPLANE HUMP HITTING CHEST

1D9AF
SIGNWRITING MOVEMENT-WALLPLANE LOOP HITTING CHEST

1D9B0
SIGNWRITING MOVEMENT-WALLPLANE WAVE HITTING CHEST

1D9B1
SIGNWRITING ROTATION-WALLPLANE SINGLE HITTING CHEST

1D9B2
SIGNWRITING ROTATION-WALLPLANE DOUBLE HITTING CHEST

1D9B3
SIGNWRITING ROTATION-WALLPLANE ALTERNATING HITTING CHEST

1D9B4
SIGNWRITING MOVEMENT-WALLPLANE WAVE DIAGONAL PATH SMALL

1D9B5
SIGNWRITING MOVEMENT-WALLPLANE WAVE DIAGONAL PATH MEDIUM

1D9B6
SIGNWRITING MOVEMENT-WALLPLANE WAVE DIAGONAL PATH LARGE

1D9B7
SIGNWRITING MOVEMENT-FLOORPLANE CURVE HITTING CEILING SMALL

1D9B8
SIGNWRITING MOVEMENT-FLOORPLANE CURVE HITTING CEILING LARGE

1D9B9
SIGNWRITING MOVEMENT-FLOORPLANE HUMP HITTING CEILING SMALL DOUBLE

1D9BA
SIGNWRITING MOVEMENT-FLOORPLANE HUMP HITTING CEILING LARGE DOUBLE

1D9BB
SIGNWRITING MOVEMENT-FLOORPLANE HUMP HITTING CEILING SMALL TRIPLE

1D9BC
SIGNWRITING MOVEMENT-FLOORPLANE HUMP HITTING CEILING LARGE TRIPLE

1D9BD
SIGNWRITING MOVEMENT-FLOORPLANE LOOP HITTING CEILING SMALL SINGLE

1D9BE
SIGNWRITING MOVEMENT-FLOORPLANE LOOP HITTING CEILING LARGE SINGLE

1D9BF
SIGNWRITING MOVEMENT-FLOORPLANE LOOP HITTING CEILING SMALL DOUBLE

1D9C0
SIGNWRITING MOVEMENT-FLOORPLANE LOOP HITTING CEILING LARGE DOUBLE

1D9C1
SIGNWRITING MOVEMENT-FLOORPLANE WAVE HITTING CEILING SMALL

1D9C2
SIGNWRITING MOVEMENT-FLOORPLANE WAVE HITTING CEILING LARGE

1D9C3
SIGNWRITING ROTATION-FLOORPLANE SINGLE HITTING CEILING

1D9C4
SIGNWRITING ROTATION-FLOORPLANE DOUBLE HITTING CEILING

1D9C5
SIGNWRITING ROTATION-FLOORPLANE ALTERNATING HITTING CEILING

1D9C6
SIGNWRITING MOVEMENT-FLOORPLANE CURVE HITTING FLOOR SMALL

1D9C7
SIGNWRITING MOVEMENT-FLOORPLANE CURVE HITTING FLOOR LARGE

1D9C8
SIGNWRITING MOVEMENT-FLOORPLANE HUMP HITTING FLOOR SMALL DOUBLE

1D9C9
SIGNWRITING MOVEMENT-FLOORPLANE HUMP HITTING FLOOR LARGE DOUBLE

1D9CA
SIGNWRITING MOVEMENT-FLOORPLANE HUMP HITTING FLOOR TRIPLE SMALL TRIPLE

1D9CB
SIGNWRITING MOVEMENT-FLOORPLANE HUMP HITTING FLOOR TRIPLE LARGE TRIPLE

1D9CC
SIGNWRITING MOVEMENT-FLOORPLANE LOOP HITTING FLOOR SMALL SINGLE

1D9CD
SIGNWRITING MOVEMENT-FLOORPLANE LOOP HITTING FLOOR LARGE SINGLE

1D9CE
SIGNWRITING MOVEMENT-FLOORPLANE LOOP HITTING FLOOR SMALL DOUBLE

1D9CF
SIGNWRITING MOVEMENT-FLOORPLANE LOOP HITTING FLOOR LARGE DOUBLE

1D9D0
SIGNWRITING MOVEMENT-FLOORPLANE WAVE HITTING FLOOR SMALL

1D9D1
SIGNWRITING MOVEMENT-FLOORPLANE WAVE HITTING FLOOR LARGE

1D9D2
SIGNWRITING ROTATION-FLOORPLANE SINGLE HITTING FLOOR

1D9D3
SIGNWRITING ROTATION-FLOORPLANE DOUBLE HITTING FLOOR

1D9D4
SIGNWRITING ROTATION-FLOORPLANE ALTERNATING HITTING FLOOR

1D9D5
SIGNWRITING MOVEMENT-FLOORPLANE CURVE SMALL

1D9D6
SIGNWRITING MOVEMENT-FLOORPLANE CURVE MEDIUM

1D9D7
SIGNWRITING MOVEMENT-FLOORPLANE CURVE LARGE

1D9D8
SIGNWRITING MOVEMENT-FLOORPLANE CURVE LARGEST

1D9D9
SIGNWRITING MOVEMENT-FLOORPLANE CURVE COMBINED

1D9DA
SIGNWRITING MOVEMENT-FLOORPLANE HUMP SMALL

1D9DB
SIGNWRITING MOVEMENT-FLOORPLANE LOOP SMALL

1D9DC
SIGNWRITING MOVEMENT-FLOORPLANE WAVE SNAKE

1D9DD
SIGNWRITING MOVEMENT-FLOORPLANE WAVE SMALL

1D9DE
SIGNWRITING MOVEMENT-FLOORPLANE WAVE LARGE

1D9DF
SIGNWRITING ROTATION-FLOORPLANE SINGLE

1D9E0
SIGNWRITING ROTATION-FLOORPLANE DOUBLE

1D9E1
SIGNWRITING ROTATION-FLOORPLANE ALTERNATING

1D9E2
SIGNWRITING MOVEMENT-FLOORPLANE SHAKING PARALLEL

1D9E3
SIGNWRITING MOVEMENT-WALLPLANE ARM CIRCLE SMALL SINGLE

1D9E4
SIGNWRITING MOVEMENT-WALLPLANE ARM CIRCLE MEDIUM SINGLE

1D9E5
SIGNWRITING MOVEMENT-WALLPLANE ARM CIRCLE SMALL DOUBLE

1D9E6
SIGNWRITING MOVEMENT-WALLPLANE ARM CIRCLE MEDIUM DOUBLE

1D9E7
SIGNWRITING MOVEMENT-FLOORPLANE ARM CIRCLE HITTING WALL SMALL SINGLE

1D9E8
SIGNWRITING MOVEMENT-FLOORPLANE ARM CIRCLE HITTING WALL MEDIUM SINGLE

1D9E9
SIGNWRITING MOVEMENT-FLOORPLANE ARM CIRCLE HITTING WALL LARGE SINGLE

1D9EA
SIGNWRITING MOVEMENT-FLOORPLANE ARM CIRCLE HITTING WALL SMALL DOUBLE

1D9EB
SIGNWRITING MOVEMENT-FLOORPLANE ARM CIRCLE HITTING WALL MEDIUM DOUBLE

1D9EC
SIGNWRITING MOVEMENT-FLOORPLANE ARM CIRCLE HITTING WALL LARGE DOUBLE

1D9ED
SIGNWRITING MOVEMENT-WALLPLANE WRIST CIRCLE FRONT SINGLE

1D9EE
SIGNWRITING MOVEMENT-WALLPLANE WRIST CIRCLE FRONT DOUBLE

1D9EF
SIGNWRITING MOVEMENT-FLOORPLANE WRIST CIRCLE HITTING WALL SINGLE

1D9F0
SIGNWRITING MOVEMENT-FLOORPLANE WRIST CIRCLE HITTING WALL DOUBLE

1D9F1
SIGNWRITING MOVEMENT-WALLPLANE FINGER CIRCLES SINGLE

1D9F2
SIGNWRITING MOVEMENT-WALLPLANE FINGER CIRCLES DOUBLE

1D9F3
SIGNWRITING MOVEMENT-FLOORPLANE FINGER CIRCLES HITTING WALL SINGLE

1D9F4
SIGNWRITING MOVEMENT-FLOORPLANE FINGER CIRCLES HITTING WALL DOUBLE

1D9F5
SIGNWRITING DYNAMIC ARROWHEAD SMALL

1D9F6
SIGNWRITING DYNAMIC ARROWHEAD LARGE

1D9F7
SIGNWRITING DYNAMIC FAST

1D9F8
SIGNWRITING DYNAMIC SLOW

1D9F9
SIGNWRITING DYNAMIC TENSE

1D9FA
SIGNWRITING DYNAMIC RELAXED

1D9FB
SIGNWRITING DYNAMIC SIMULTANEOUS

1D9FC
SIGNWRITING DYNAMIC SIMULTANEOUS ALTERNATING

1D9FD
SIGNWRITING DYNAMIC EVERY OTHER TIME

1D9FE
SIGNWRITING DYNAMIC GRADUAL

1D9FF
SIGNWRITING HEAD

1DA00
SIGNWRITING HEAD RIM

1DA01
SIGNWRITING HEAD MOVEMENT-WALLPLANE STRAIGHT

1DA02
SIGNWRITING HEAD MOVEMENT-WALLPLANE TILT

1DA03
SIGNWRITING HEAD MOVEMENT-FLOORPLANE STRAIGHT

1DA04
SIGNWRITING HEAD MOVEMENT-WALLPLANE CURVE

1DA05
SIGNWRITING HEAD MOVEMENT-FLOORPLANE CURVE

1DA06
SIGNWRITING HEAD MOVEMENT CIRCLE

1DA07
SIGNWRITING FACE DIRECTION POSITION NOSE FORWARD TILTING

1DA08
SIGNWRITING FACE DIRECTION POSITION NOSE UP OR DOWN

1DA09
SIGNWRITING FACE DIRECTION POSITION NOSE UP OR DOWN TILTING

1DA0A
SIGNWRITING EYEBROWS STRAIGHT UP

1DA0B
SIGNWRITING EYEBROWS STRAIGHT NEUTRAL

1DA0C
SIGNWRITING EYEBROWS STRAIGHT DOWN

1DA0D
SIGNWRITING DREAMY EYEBROWS NEUTRAL DOWN

1DA0E
SIGNWRITING DREAMY EYEBROWS DOWN NEUTRAL

1DA0F
SIGNWRITING DREAMY EYEBROWS UP NEUTRAL

1DA10
SIGNWRITING DREAMY EYEBROWS NEUTRAL UP

1DA11
SIGNWRITING FOREHEAD NEUTRAL

1DA12
SIGNWRITING FOREHEAD CONTACT

1DA13
SIGNWRITING FOREHEAD WRINKLED

1DA14
SIGNWRITING EYES OPEN

1DA15
SIGNWRITING EYES SQUEEZED

1DA16
SIGNWRITING EYES CLOSED

1DA17
SIGNWRITING EYE BLINK SINGLE

1DA18
SIGNWRITING EYE BLINK MULTIPLE

1DA19
SIGNWRITING EYES HALF OPEN

1DA1A
SIGNWRITING EYES WIDE OPEN

1DA1B
SIGNWRITING EYES HALF CLOSED

1DA1C
SIGNWRITING EYES WIDENING MOVEMENT

1DA1D
SIGNWRITING EYE WINK

1DA1E
SIGNWRITING EYELASHES UP

1DA1F
SIGNWRITING EYELASHES DOWN

1DA20
SIGNWRITING EYELASHES FLUTTERING

1DA21
SIGNWRITING EYEGAZE-WALLPLANE STRAIGHT

1DA22
SIGNWRITING EYEGAZE-WALLPLANE STRAIGHT DOUBLE

1DA23
SIGNWRITING EYEGAZE-WALLPLANE STRAIGHT ALTERNATING

1DA24
SIGNWRITING EYEGAZE-FLOORPLANE STRAIGHT

1DA25
SIGNWRITING EYEGAZE-FLOORPLANE STRAIGHT DOUBLE

1DA26
SIGNWRITING EYEGAZE-FLOORPLANE STRAIGHT ALTERNATING

1DA27
SIGNWRITING EYEGAZE-WALLPLANE CURVED

1DA28
SIGNWRITING EYEGAZE-FLOORPLANE CURVED

1DA29
SIGNWRITING EYEGAZE-WALLPLANE CIRCLING

1DA2A
SIGNWRITING CHEEKS PUFFED

1DA2B
SIGNWRITING CHEEKS NEUTRAL

1DA2C
SIGNWRITING CHEEKS SUCKED

1DA2D
SIGNWRITING TENSE CHEEKS HIGH

1DA2E
SIGNWRITING TENSE CHEEKS MIDDLE

1DA2F
SIGNWRITING TENSE CHEEKS LOW

1DA30
SIGNWRITING EARS

1DA31
SIGNWRITING NOSE NEUTRAL

1DA32
SIGNWRITING NOSE CONTACT

1DA33
SIGNWRITING NOSE WRINKLES

1DA34
SIGNWRITING NOSE WIGGLES

1DA35
SIGNWRITING AIR BLOWING OUT

1DA36
SIGNWRITING AIR SUCKING IN

1DA37
SIGNWRITING AIR BLOW SMALL ROTATIONS

1DA38
SIGNWRITING AIR SUCK SMALL ROTATIONS

1DA39
SIGNWRITING BREATH INHALE

1DA3A
SIGNWRITING BREATH EXHALE

1DA3B
SIGNWRITING MOUTH CLOSED NEUTRAL

1DA3C
SIGNWRITING MOUTH CLOSED FORWARD

1DA3D
SIGNWRITING MOUTH CLOSED CONTACT

1DA3E
SIGNWRITING MOUTH SMILE

1DA3F
SIGNWRITING MOUTH SMILE WRINKLED

1DA40
SIGNWRITING MOUTH SMILE OPEN

1DA41
SIGNWRITING MOUTH FROWN

1DA42
SIGNWRITING MOUTH FROWN WRINKLED

1DA43
SIGNWRITING MOUTH FROWN OPEN

1DA44
SIGNWRITING MOUTH OPEN CIRCLE

1DA45
SIGNWRITING MOUTH OPEN FORWARD

1DA46
SIGNWRITING MOUTH OPEN WRINKLED

1DA47
SIGNWRITING MOUTH OPEN OVAL

1DA48
SIGNWRITING MOUTH OPEN OVAL WRINKLED

1DA49
SIGNWRITING MOUTH OPEN OVAL YAWN

1DA4A
SIGNWRITING MOUTH OPEN RECTANGLE

1DA4B
SIGNWRITING MOUTH OPEN RECTANGLE WRINKLED

1DA4C
SIGNWRITING MOUTH OPEN RECTANGLE YAWN

1DA4D
SIGNWRITING MOUTH KISS

1DA4E
SIGNWRITING MOUTH KISS FORWARD

1DA4F
SIGNWRITING MOUTH KISS WRINKLED

1DA50
SIGNWRITING MOUTH TENSE

1DA51
SIGNWRITING MOUTH TENSE FORWARD

1DA52
SIGNWRITING MOUTH TENSE SUCKED

1DA53
SIGNWRITING LIPS PRESSED TOGETHER

1DA54
SIGNWRITING LIP LOWER OVER UPPER

1DA55
SIGNWRITING LIP UPPER OVER LOWER

1DA56
SIGNWRITING MOUTH CORNERS

1DA57
SIGNWRITING MOUTH WRINKLES SINGLE

1DA58
SIGNWRITING MOUTH WRINKLES DOUBLE

1DA59
SIGNWRITING TONGUE STICKING OUT FAR

1DA5A
SIGNWRITING TONGUE LICKING LIPS

1DA5B
SIGNWRITING TONGUE TIP BETWEEN LIPS

1DA5C
SIGNWRITING TONGUE TIP TOUCHING INSIDE MOUTH

1DA5D
SIGNWRITING TONGUE INSIDE MOUTH RELAXED

1DA5E
SIGNWRITING TONGUE MOVES AGAINST CHEEK

1DA5F
SIGNWRITING TONGUE CENTRE STICKING OUT

1DA60
SIGNWRITING TONGUE CENTRE INSIDE MOUTH

1DA61
SIGNWRITING TEETH

1DA62
SIGNWRITING TEETH MOVEMENT

1DA63
SIGNWRITING TEETH ON TONGUE

1DA64
SIGNWRITING TEETH ON TONGUE MOVEMENT

1DA65
SIGNWRITING TEETH ON LIPS

1DA66
SIGNWRITING TEETH ON LIPS MOVEMENT

1DA67
SIGNWRITING TEETH BITE LIPS

1DA68
SIGNWRITING MOVEMENT-WALLPLANE JAW

1DA69
SIGNWRITING MOVEMENT-FLOORPLANE JAW

1DA6A
SIGNWRITING NECK

1DA6B
SIGNWRITING HAIR

1DA6C
SIGNWRITING EXCITEMENT

1DA6D
SIGNWRITING SHOULDER HIP SPINE

1DA6E
SIGNWRITING SHOULDER HIP POSITIONS

1DA6F
SIGNWRITING WALLPLANE SHOULDER HIP MOVE

1DA70
SIGNWRITING FLOORPLANE SHOULDER HIP MOVE

1DA71
SIGNWRITING SHOULDER TILTING FROM WAIST

1DA72
SIGNWRITING TORSO-WALLPLANE STRAIGHT STRETCH

1DA73
SIGNWRITING TORSO-WALLPLANE CURVED BEND

1DA74
SIGNWRITING TORSO-FLOORPLANE TWISTING

1DA75
SIGNWRITING UPPER BODY TILTING FROM HIP JOINTS

1DA76
SIGNWRITING LIMB COMBINATION

1DA77
SIGNWRITING LIMB LENGTH-1

1DA78
SIGNWRITING LIMB LENGTH-2

1DA79
SIGNWRITING LIMB LENGTH-3

1DA7A
SIGNWRITING LIMB LENGTH-4

1DA7B
SIGNWRITING LIMB LENGTH-5

1DA7C
SIGNWRITING LIMB LENGTH-6

1DA7D
SIGNWRITING LIMB LENGTH-7

1DA7E
SIGNWRITING FINGER

1DA7F
SIGNWRITING LOCATION-WALLPLANE SPACE

1DA80
SIGNWRITING LOCATION-FLOORPLANE SPACE

1DA81
SIGNWRITING LOCATION HEIGHT

1DA82
SIGNWRITING LOCATION WIDTH

1DA83
SIGNWRITING LOCATION DEPTH

1DA84
SIGNWRITING LOCATION HEAD NECK

1DA85
SIGNWRITING LOCATION TORSO

1DA86
SIGNWRITING LOCATION LIMBS DIGITS

1DA87
SIGNWRITING COMMA

1DA88
SIGNWRITING FULL STOP

1DA89
SIGNWRITING SEMICOLON

1DA8A
SIGNWRITING COLON

1DA8B
SIGNWRITING PARENTHESIS

1DA9B
SIGNWRITING FILL MODIFIER-2

1DA9C
SIGNWRITING FILL MODIFIER-3

1DA9D
SIGNWRITING FILL MODIFIER-4

1DA9E
SIGNWRITING FILL MODIFIER-5

1DA9F
SIGNWRITING FILL MODIFIER-6

1DAA1
SIGNWRITING ROTATION MODIFIER-2

1DAA2
SIGNWRITING ROTATION MODIFIER-3

1DAA3
SIGNWRITING ROTATION MODIFIER-4

1DAA4
SIGNWRITING ROTATION MODIFIER-5

1DAA5
SIGNWRITING ROTATION MODIFIER-6

1DAA6
SIGNWRITING ROTATION MODIFIER-7

1DAA7
SIGNWRITING ROTATION MODIFIER-8

1DAA8
SIGNWRITING ROTATION MODIFIER-9

1DAA9
SIGNWRITING ROTATION MODIFIER-10

1DAAA
SIGNWRITING ROTATION MODIFIER-11

1DAAB
SIGNWRITING ROTATION MODIFIER-12

1DAAC
SIGNWRITING ROTATION MODIFIER-13

1DAAD
SIGNWRITING ROTATION MODIFIER-14

1DAAE
SIGNWRITING ROTATION MODIFIER-15

1DAAF
SIGNWRITING ROTATION MODIFIER-16

1DF00
LATIN SMALL LETTER FENG DIGRAPH WITH TRILL

1DF01
LATIN SMALL LETTER REVERSED SCRIPT G

1DF02
LATIN LETTER SMALL CAPITAL TURNED G

1DF03
LATIN SMALL LETTER REVERSED K

1DF04
LATIN LETTER SMALL CAPITAL L WITH BELT

1DF05
LATIN SMALL LETTER LEZH WITH RETROFLEX HOOK

1DF06
LATIN SMALL LETTER TURNED Y WITH BELT

1DF07
LATIN SMALL LETTER REVERSED ENG

1DF08
LATIN SMALL LETTER TURNED R WITH LONG LEG AND RETROFLEX HOOK

1DF09
LATIN SMALL LETTER T WITH HOOK AND RETROFLEX HOOK

1DF0A
LATIN LETTER RETROFLEX CLICK WITH RETROFLEX HOOK

1DF0B
LATIN SMALL LETTER ESH WITH DOUBLE BAR

1DF0C
LATIN SMALL LETTER ESH WITH DOUBLE BAR AND CURL

1DF0D
LATIN SMALL LETTER TURNED T WITH CURL

1DF0E
LATIN LETTER INVERTED GLOTTAL STOP WITH CURL

1DF0F
LATIN LETTER STRETCHED C WITH CURL

1DF10
LATIN LETTER SMALL CAPITAL TURNED K

1DF11
LATIN SMALL LETTER L WITH FISHHOOK

1DF12
LATIN SMALL LETTER DEZH DIGRAPH WITH PALATAL HOOK

1DF13
LATIN SMALL LETTER L WITH BELT AND PALATAL HOOK

1DF14
LATIN SMALL LETTER ENG WITH PALATAL HOOK

1DF15
LATIN SMALL LETTER TURNED R WITH PALATAL HOOK

1DF16
LATIN SMALL LETTER R WITH FISHHOOK AND PALATAL HOOK

1DF17
LATIN SMALL LETTER TESH DIGRAPH WITH PALATAL HOOK

1DF18
LATIN SMALL LETTER EZH WITH PALATAL HOOK

1DF19
LATIN SMALL LETTER DEZH DIGRAPH WITH RETROFLEX HOOK

1DF1A
LATIN SMALL LETTER I WITH STROKE AND RETROFLEX HOOK

1DF1B
LATIN SMALL LETTER O WITH RETROFLEX HOOK

1DF1C
LATIN SMALL LETTER TESH DIGRAPH WITH RETROFLEX HOOK

1DF1D
LATIN SMALL LETTER C WITH RETROFLEX HOOK

1DF1E
LATIN SMALL LETTER S WITH CURL

1DF25
LATIN SMALL LETTER D WITH MID-HEIGHT LEFT HOOK

1DF26
LATIN SMALL LETTER L WITH MID-HEIGHT LEFT HOOK

1DF27
LATIN SMALL LETTER N WITH MID-HEIGHT LEFT HOOK

1DF28
LATIN SMALL LETTER R WITH MID-HEIGHT LEFT HOOK

1DF29
LATIN SMALL LETTER S WITH MID-HEIGHT LEFT HOOK

1DF2A
LATIN SMALL LETTER T WITH MID-HEIGHT LEFT HOOK

1E000
COMBINING GLAGOLITIC LETTER AZU

1E001
COMBINING GLAGOLITIC LETTER BUKY

1E002
COMBINING GLAGOLITIC LETTER VEDE

1E003
COMBINING GLAGOLITIC LETTER GLAGOLI

1E004
COMBINING GLAGOLITIC LETTER DOBRO

1E005
COMBINING GLAGOLITIC LETTER YESTU

1E006
COMBINING GLAGOLITIC LETTER ZHIVETE

1E008
COMBINING GLAGOLITIC LETTER ZEMLJA

1E009
COMBINING GLAGOLITIC LETTER IZHE

1E00A
COMBINING GLAGOLITIC LETTER INITIAL IZHE

1E00B
COMBINING GLAGOLITIC LETTER I

1E00C
COMBINING GLAGOLITIC LETTER DJERVI

1E00D
COMBINING GLAGOLITIC LETTER KAKO

1E00E
COMBINING GLAGOLITIC LETTER LJUDIJE

1E00F
COMBINING GLAGOLITIC LETTER MYSLITE

1E010
COMBINING GLAGOLITIC LETTER NASHI

1E011
COMBINING GLAGOLITIC LETTER ONU

1E012
COMBINING GLAGOLITIC LETTER POKOJI

1E013
COMBINING GLAGOLITIC LETTER RITSI

1E014
COMBINING GLAGOLITIC LETTER SLOVO

1E015
COMBINING GLAGOLITIC LETTER TVRIDO

1E016
COMBINING GLAGOLITIC LETTER UKU

1E017
COMBINING GLAGOLITIC LETTER FRITU

1E018
COMBINING GLAGOLITIC LETTER HERU

1E01B
COMBINING GLAGOLITIC LETTER SHTA

1E01C
COMBINING GLAGOLITIC LETTER TSI

1E01D
COMBINING GLAGOLITIC LETTER CHRIVI

1E01E
COMBINING GLAGOLITIC LETTER SHA

1E01F
COMBINING GLAGOLITIC LETTER YERU

1E020
COMBINING GLAGOLITIC LETTER YERI

1E021
COMBINING GLAGOLITIC LETTER YATI

1E023
COMBINING GLAGOLITIC LETTER YU

1E024
COMBINING GLAGOLITIC LETTER SMALL YUS

1E026
COMBINING GLAGOLITIC LETTER YO

1E027
COMBINING GLAGOLITIC LETTER IOTATED SMALL YUS

1E028
COMBINING GLAGOLITIC LETTER BIG YUS

1E029
COMBINING GLAGOLITIC LETTER IOTATED BIG YUS

1E02A
COMBINING GLAGOLITIC LETTER FITA

1E030
MODIFIER LETTER CYRILLIC SMALL A

1E031
MODIFIER LETTER CYRILLIC SMALL BE

1E032
MODIFIER LETTER CYRILLIC SMALL VE

1E033
MODIFIER LETTER CYRILLIC SMALL GHE

1E034
MODIFIER LETTER CYRILLIC SMALL DE

1E035
MODIFIER LETTER CYRILLIC SMALL IE

1E036
MODIFIER LETTER CYRILLIC SMALL ZHE

1E037
MODIFIER LETTER CYRILLIC SMALL ZE

1E038
MODIFIER LETTER CYRILLIC SMALL I

1E039
MODIFIER LETTER CYRILLIC SMALL KA

1E03A
MODIFIER LETTER CYRILLIC SMALL EL

1E03B
MODIFIER LETTER CYRILLIC SMALL EM

1E03C
MODIFIER LETTER CYRILLIC SMALL O

1E03D
MODIFIER LETTER CYRILLIC SMALL PE

1E03E
MODIFIER LETTER CYRILLIC SMALL ER

1E03F
MODIFIER LETTER CYRILLIC SMALL ES

1E040
MODIFIER LETTER CYRILLIC SMALL TE

1E041
MODIFIER LETTER CYRILLIC SMALL U

1E042
MODIFIER LETTER CYRILLIC SMALL EF

1E043
MODIFIER LETTER CYRILLIC SMALL HA

1E044
MODIFIER LETTER CYRILLIC SMALL TSE

1E045
MODIFIER LETTER CYRILLIC SMALL CHE

1E046
MODIFIER LETTER CYRILLIC SMALL SHA

1E047
MODIFIER LETTER CYRILLIC SMALL YERU

1E048
MODIFIER LETTER CYRILLIC SMALL E

1E049
MODIFIER LETTER CYRILLIC SMALL YU

1E04A
MODIFIER LETTER CYRILLIC SMALL DZZE

1E04B
MODIFIER LETTER CYRILLIC SMALL SCHWA

1E04C
MODIFIER LETTER CYRILLIC SMALL BYELORUSSIAN-UKRAINIAN I

1E04D
MODIFIER LETTER CYRILLIC SMALL JE

1E04E
MODIFIER LETTER CYRILLIC SMALL BARRED O

1E04F
MODIFIER LETTER CYRILLIC SMALL STRAIGHT U

1E050
MODIFIER LETTER CYRILLIC SMALL PALOCHKA

1E051
CYRILLIC SUBSCRIPT SMALL LETTER A

1E052
CYRILLIC SUBSCRIPT SMALL LETTER BE

1E053
CYRILLIC SUBSCRIPT SMALL LETTER VE

1E054
CYRILLIC SUBSCRIPT SMALL LETTER GHE

1E055
CYRILLIC SUBSCRIPT SMALL LETTER DE

1E056
CYRILLIC SUBSCRIPT SMALL LETTER IE

1E057
CYRILLIC SUBSCRIPT SMALL LETTER ZHE

1E058
CYRILLIC SUBSCRIPT SMALL LETTER ZE

1E059
CYRILLIC SUBSCRIPT SMALL LETTER I

1E05A
CYRILLIC SUBSCRIPT SMALL LETTER KA

1E05B
CYRILLIC SUBSCRIPT SMALL LETTER EL

1E05C
CYRILLIC SUBSCRIPT SMALL LETTER O

1E05D
CYRILLIC SUBSCRIPT SMALL LETTER PE

1E05E
CYRILLIC SUBSCRIPT SMALL LETTER ES

1E05F
CYRILLIC SUBSCRIPT SMALL LETTER U

1E060
CYRILLIC SUBSCRIPT SMALL LETTER EF

1E061
CYRILLIC SUBSCRIPT SMALL LETTER HA

1E062
CYRILLIC SUBSCRIPT SMALL LETTER TSE

1E063
CYRILLIC SUBSCRIPT SMALL LETTER CHE

1E064
CYRILLIC SUBSCRIPT SMALL LETTER SHA

1E065
CYRILLIC SUBSCRIPT SMALL LETTER HARD SIGN

1E066
CYRILLIC SUBSCRIPT SMALL LETTER YERU

1E067
CYRILLIC SUBSCRIPT SMALL LETTER GHE WITH UPTURN

1E068
CYRILLIC SUBSCRIPT SMALL LETTER BYELORUSSIAN-UKRAINIAN I

1E069
CYRILLIC SUBSCRIPT SMALL LETTER DZE

1E06A
CYRILLIC SUBSCRIPT SMALL LETTER DZHE

1E06B
MODIFIER LETTER CYRILLIC SMALL ES WITH DESCENDER

1E06C
MODIFIER LETTER CYRILLIC SMALL YERU WITH BACK YER

1E06D
MODIFIER LETTER CYRILLIC SMALL STRAIGHT U WITH STROKE

1E08F
COMBINING CYRILLIC SMALL LETTER BYELORUSSIAN-UKRAINIAN I

1E100
NYIAKENG PUACHUE HMONG LETTER MA

1E101
NYIAKENG PUACHUE HMONG LETTER TSA

1E102
NYIAKENG PUACHUE HMONG LETTER NTA

1E103
NYIAKENG PUACHUE HMONG LETTER TA

1E104
NYIAKENG PUACHUE HMONG LETTER HA

1E105
NYIAKENG PUACHUE HMONG LETTER NA

1E106
NYIAKENG PUACHUE HMONG LETTER XA

1E107
NYIAKENG PUACHUE HMONG LETTER NKA

1E108
NYIAKENG PUACHUE HMONG LETTER CA

1E109
NYIAKENG PUACHUE HMONG LETTER LA

1E10A
NYIAKENG PUACHUE HMONG LETTER SA

1E10B
NYIAKENG PUACHUE HMONG LETTER ZA

1E10C
NYIAKENG PUACHUE HMONG LETTER NCA

1E10D
NYIAKENG PUACHUE HMONG LETTER NTSA

1E10E
NYIAKENG PUACHUE HMONG LETTER KA

1E10F
NYIAKENG PUACHUE HMONG LETTER DA

1E110
NYIAKENG PUACHUE HMONG LETTER NYA

1E111
NYIAKENG PUACHUE HMONG LETTER NRA

1E112
NYIAKENG PUACHUE HMONG LETTER VA

1E113
NYIAKENG PUACHUE HMONG LETTER NTXA

1E114
NYIAKENG PUACHUE HMONG LETTER TXA

1E115
NYIAKENG PUACHUE HMONG LETTER FA

1E116
NYIAKENG PUACHUE HMONG LETTER RA

1E117
NYIAKENG PUACHUE HMONG LETTER QA

1E118
NYIAKENG PUACHUE HMONG LETTER YA

1E119
NYIAKENG PUACHUE HMONG LETTER NQA

1E11A
NYIAKENG PUACHUE HMONG LETTER PA

1E11B
NYIAKENG PUACHUE HMONG LETTER XYA

1E11C
NYIAKENG PUACHUE HMONG LETTER NPA

1E11D
NYIAKENG PUACHUE HMONG LETTER DLA

1E11E
NYIAKENG PUACHUE HMONG LETTER NPLA

1E11F
NYIAKENG PUACHUE HMONG LETTER HAH

1E120
NYIAKENG PUACHUE HMONG LETTER MLA

1E121
NYIAKENG PUACHUE HMONG LETTER PLA

1E122
NYIAKENG PUACHUE HMONG LETTER GA

1E123
NYIAKENG PUACHUE HMONG LETTER RRA

1E124
NYIAKENG PUACHUE HMONG LETTER A

1E125
NYIAKENG PUACHUE HMONG LETTER AA

1E126
NYIAKENG PUACHUE HMONG LETTER I

1E127
NYIAKENG PUACHUE HMONG LETTER U

1E128
NYIAKENG PUACHUE HMONG LETTER O

1E129
NYIAKENG PUACHUE HMONG LETTER OO

1E12A
NYIAKENG PUACHUE HMONG LETTER E

1E12B
NYIAKENG PUACHUE HMONG LETTER EE

1E12C
NYIAKENG PUACHUE HMONG LETTER W

1E130
NYIAKENG PUACHUE HMONG TONE-B

1E131
NYIAKENG PUACHUE HMONG TONE-M

1E132
NYIAKENG PUACHUE HMONG TONE-J

1E133
NYIAKENG PUACHUE HMONG TONE-V

1E134
NYIAKENG PUACHUE HMONG TONE-S

1E135
NYIAKENG PUACHUE HMONG TONE-G

1E136
NYIAKENG PUACHUE HMONG TONE-D

1E137
NYIAKENG PUACHUE HMONG SIGN FOR PERSON

1E138
NYIAKENG PUACHUE HMONG SIGN FOR THING

1E139
NYIAKENG PUACHUE HMONG SIGN FOR LOCATION

1E13A
NYIAKENG PUACHUE HMONG SIGN FOR ANIMAL

1E13B
NYIAKENG PUACHUE HMONG SIGN FOR INVERTEBRATE

1E13C
NYIAKENG PUACHUE HMONG SIGN XW XW

1E13D
NYIAKENG PUACHUE HMONG SYLLABLE LENGTHENER

1E140
NYIAKENG PUACHUE HMONG DIGIT ZERO

1E141
NYIAKENG PUACHUE HMONG DIGIT ONE

1E142
NYIAKENG PUACHUE HMONG DIGIT TWO

1E143
NYIAKENG PUACHUE HMONG DIGIT THREE

1E144
NYIAKENG PUACHUE HMONG DIGIT FOUR

1E145
NYIAKENG PUACHUE HMONG DIGIT FIVE

1E146
NYIAKENG PUACHUE HMONG DIGIT SIX

1E147
NYIAKENG PUACHUE HMONG DIGIT SEVEN

1E148
NYIAKENG PUACHUE HMONG DIGIT EIGHT

1E149
NYIAKENG PUACHUE HMONG DIGIT NINE

1E14E
NYIAKENG PUACHUE HMONG LOGOGRAM NYAJ

1E14F
NYIAKENG PUACHUE HMONG CIRCLED CA

1E290
TOTO LETTER PA

1E291
TOTO LETTER BA

1E292
TOTO LETTER TA

1E293
TOTO LETTER DA

1E294
TOTO LETTER KA

1E295
TOTO LETTER GA

1E296
TOTO LETTER MA

1E297
TOTO LETTER NA

1E298
TOTO LETTER NGA

1E299
TOTO LETTER SA

1E29A
TOTO LETTER CHA

1E29B
TOTO LETTER YA

1E29C
TOTO LETTER WA

1E29D
TOTO LETTER JA

1E29E
TOTO LETTER HA

1E29F
TOTO LETTER RA

1E2A0
TOTO LETTER LA

1E2A1
TOTO LETTER I

1E2A2
TOTO LETTER BREATHY I

1E2A3
TOTO LETTER IU

1E2A4
TOTO LETTER BREATHY IU

1E2A5
TOTO LETTER U

1E2A6
TOTO LETTER E

1E2A7
TOTO LETTER BREATHY E

1E2A8
TOTO LETTER EO

1E2A9
TOTO LETTER BREATHY EO

1E2AA
TOTO LETTER O

1E2AB
TOTO LETTER AE

1E2AC
TOTO LETTER BREATHY AE

1E2AD
TOTO LETTER A

1E2AE
TOTO SIGN RISING TONE

1E2C0
WANCHO LETTER AA

1E2C1
WANCHO LETTER A

1E2C2
WANCHO LETTER BA

1E2C3
WANCHO LETTER CA

1E2C4
WANCHO LETTER DA

1E2C5
WANCHO LETTER GA

1E2C6
WANCHO LETTER YA

1E2C7
WANCHO LETTER PHA

1E2C8
WANCHO LETTER LA

1E2C9
WANCHO LETTER NA

1E2CA
WANCHO LETTER PA

1E2CB
WANCHO LETTER TA

1E2CC
WANCHO LETTER THA

1E2CD
WANCHO LETTER FA

1E2CE
WANCHO LETTER SA

1E2CF
WANCHO LETTER SHA

1E2D0
WANCHO LETTER JA

1E2D1
WANCHO LETTER ZA

1E2D2
WANCHO LETTER WA

1E2D3
WANCHO LETTER VA

1E2D4
WANCHO LETTER KA

1E2D5
WANCHO LETTER O

1E2D6
WANCHO LETTER AU

1E2D7
WANCHO LETTER RA

1E2D8
WANCHO LETTER MA

1E2D9
WANCHO LETTER KHA

1E2DA
WANCHO LETTER HA

1E2DB
WANCHO LETTER E

1E2DC
WANCHO LETTER I

1E2DD
WANCHO LETTER NGA

1E2DE
WANCHO LETTER U

1E2DF
WANCHO LETTER LLHA

1E2E0
WANCHO LETTER TSA

1E2E1
WANCHO LETTER TRA

1E2E2
WANCHO LETTER ONG

1E2E3
WANCHO LETTER AANG

1E2E4
WANCHO LETTER ANG

1E2E5
WANCHO LETTER ING

1E2E6
WANCHO LETTER ON

1E2E7
WANCHO LETTER EN

1E2E8
WANCHO LETTER AAN

1E2E9
WANCHO LETTER NYA

1E2EA
WANCHO LETTER UEN

1E2EB
WANCHO LETTER YIH

1E2EC
WANCHO TONE TUP

1E2ED
WANCHO TONE TUPNI

1E2EE
WANCHO TONE KOI

1E2EF
WANCHO TONE KOINI

1E2F0
WANCHO DIGIT ZERO

1E2F1
WANCHO DIGIT ONE

1E2F2
WANCHO DIGIT TWO

1E2F3
WANCHO DIGIT THREE

1E2F4
WANCHO DIGIT FOUR

1E2F5
WANCHO DIGIT FIVE

1E2F6
WANCHO DIGIT SIX

1E2F7
WANCHO DIGIT SEVEN

1E2F8
WANCHO DIGIT EIGHT

1E2F9
WANCHO DIGIT NINE

1E2FF
WANCHO NGUN SIGN

1E4D0
NAG MUNDARI LETTER O

1E4D1
NAG MUNDARI LETTER OP

1E4D2
NAG MUNDARI LETTER OL

1E4D3
NAG MUNDARI LETTER OY

1E4D4
NAG MUNDARI LETTER ONG

1E4D5
NAG MUNDARI LETTER A

1E4D6
NAG MUNDARI LETTER AJ

1E4D7
NAG MUNDARI LETTER AB

1E4D8
NAG MUNDARI LETTER ANY

1E4D9
NAG MUNDARI LETTER AH

1E4DA
NAG MUNDARI LETTER I

1E4DB
NAG MUNDARI LETTER IS

1E4DC
NAG MUNDARI LETTER IDD

1E4DD
NAG MUNDARI LETTER IT

1E4DE
NAG MUNDARI LETTER IH

1E4DF
NAG MUNDARI LETTER U

1E4E0
NAG MUNDARI LETTER UC

1E4E1
NAG MUNDARI LETTER UD

1E4E2
NAG MUNDARI LETTER UK

1E4E3
NAG MUNDARI LETTER UR

1E4E4
NAG MUNDARI LETTER E

1E4E5
NAG MUNDARI LETTER ENN

1E4E6
NAG MUNDARI LETTER EG

1E4E7
NAG MUNDARI LETTER EM

1E4E8
NAG MUNDARI LETTER EN

1E4E9
NAG MUNDARI LETTER ETT

1E4EA
NAG MUNDARI LETTER ELL

1E4EB
NAG MUNDARI SIGN OJOD

1E4EC
NAG MUNDARI SIGN MUHOR

1E4ED
NAG MUNDARI SIGN TOYOR

1E4EE
NAG MUNDARI SIGN IKIR

1E4EF
NAG MUNDARI SIGN SUTUH

1E4F0
NAG MUNDARI DIGIT ZERO

1E4F1
NAG MUNDARI DIGIT ONE

1E4F2
NAG MUNDARI DIGIT TWO

1E4F3
NAG MUNDARI DIGIT THREE

1E4F4
NAG MUNDARI DIGIT FOUR

1E4F5
NAG MUNDARI DIGIT FIVE

1E4F6
NAG MUNDARI DIGIT SIX

1E4F7
NAG MUNDARI DIGIT SEVEN

1E4F8
NAG MUNDARI DIGIT EIGHT

1E4F9
NAG MUNDARI DIGIT NINE

1E5D0
OL ONAL LETTER O

1E5D1
OL ONAL LETTER OM

1E5D2
OL ONAL LETTER ONG

1E5D3
OL ONAL LETTER ORR

1E5D4
OL ONAL LETTER OO

1E5D5
OL ONAL LETTER OY

1E5D6
OL ONAL LETTER A

1E5D7
OL ONAL LETTER AD

1E5D8
OL ONAL LETTER AB

1E5D9
OL ONAL LETTER AH

1E5DA
OL ONAL LETTER AL

1E5DB
OL ONAL LETTER AW

1E5DC
OL ONAL LETTER I

1E5DD
OL ONAL LETTER IT

1E5DE
OL ONAL LETTER IP

1E5DF
OL ONAL LETTER ITT

1E5E0
OL ONAL LETTER ID

1E5E1
OL ONAL LETTER IN

1E5E2
OL ONAL LETTER U

1E5E3
OL ONAL LETTER UK

1E5E4
OL ONAL LETTER UDD

1E5E5
OL ONAL LETTER UJ

1E5E6
OL ONAL LETTER UNY

1E5E7
OL ONAL LETTER UR

1E5E8
OL ONAL LETTER E

1E5E9
OL ONAL LETTER ES

1E5EA
OL ONAL LETTER EH

1E5EB
OL ONAL LETTER EC

1E5EC
OL ONAL LETTER ENN

1E5ED
OL ONAL LETTER EG

1E5EE
OL ONAL SIGN MU

1E5EF
OL ONAL SIGN IKIR

1E5F0
OL ONAL SIGN HODDOND

1E5F1
OL ONAL DIGIT ZERO

1E5F2
OL ONAL DIGIT ONE

1E5F3
OL ONAL DIGIT TWO

1E5F4
OL ONAL DIGIT THREE

1E5F5
OL ONAL DIGIT FOUR

1E5F6
OL ONAL DIGIT FIVE

1E5F7
OL ONAL DIGIT SIX

1E5F8
OL ONAL DIGIT SEVEN

1E5F9
OL ONAL DIGIT EIGHT

1E5FA
OL ONAL DIGIT NINE

1E5FF
OL ONAL ABBREVIATION SIGN

1E6C0
TAI YO LETTER LOW KO

1E6C1
TAI YO LETTER HIGH KO

1E6C2
TAI YO LETTER LOW KHO

1E6C3
TAI YO LETTER HIGH KHO

1E6C4
TAI YO LETTER GO

1E6C5
TAI YO LETTER NGO

1E6C6
TAI YO LETTER CO

1E6C7
TAI YO LETTER LOW XO

1E6C8
TAI YO LETTER HIGH XO

1E6C9
TAI YO LETTER LOW NYO

1E6CA
TAI YO LETTER HIGH NYO

1E6CB
TAI YO LETTER DO

1E6CC
TAI YO LETTER LOW TO

1E6CD
TAI YO LETTER HIGH TO

1E6CE
TAI YO LETTER THO

1E6CF
TAI YO LETTER NO

1E6D0
TAI YO LETTER BO

1E6D1
TAI YO LETTER LOW PO

1E6D2
TAI YO LETTER HIGH PO

1E6D3
TAI YO LETTER PHO

1E6D4
TAI YO LETTER LOW FO

1E6D5
TAI YO LETTER HIGH FO

1E6D6
TAI YO LETTER MO

1E6D7
TAI YO LETTER YO

1E6D8
TAI YO LETTER LO

1E6D9
TAI YO LETTER VO

1E6DA
TAI YO LETTER LOW HO

1E6DB
TAI YO LETTER HIGH HO

1E6DC
TAI YO LETTER QO

1E6DD
TAI YO LETTER LOW KVO

1E6DE
TAI YO LETTER HIGH KVO

1E6E0
TAI YO LETTER AA

1E6E1
TAI YO LETTER I

1E6E2
TAI YO LETTER UE

1E6E3
TAI YO SIGN UE

1E6E4
TAI YO LETTER U

1E6E5
TAI YO LETTER AE

1E6E6
TAI YO SIGN AU

1E6E7
TAI YO LETTER O

1E6E8
TAI YO LETTER E

1E6E9
TAI YO LETTER IA

1E6EA
TAI YO LETTER UEA

1E6EB
TAI YO LETTER UA

1E6EC
TAI YO LETTER OO

1E6ED
TAI YO LETTER AUE

1E6EE
TAI YO SIGN AY

1E6EF
TAI YO SIGN ANG

1E6F0
TAI YO LETTER AN

1E6F1
TAI YO LETTER AM

1E6F2
TAI YO LETTER AK

1E6F3
TAI YO LETTER AT

1E6F4
TAI YO LETTER AP

1E6F5
TAI YO SIGN OM

1E6FE
TAI YO SYMBOL MUEANG

1E6FF
TAI YO XAM LAI

1E7E0
ETHIOPIC SYLLABLE HHYA

1E7E1
ETHIOPIC SYLLABLE HHYU

1E7E2
ETHIOPIC SYLLABLE HHYI

1E7E3
ETHIOPIC SYLLABLE HHYAA

1E7E4
ETHIOPIC SYLLABLE HHYEE

1E7E5
ETHIOPIC SYLLABLE HHYE

1E7E6
ETHIOPIC SYLLABLE HHYO

1E7E8
ETHIOPIC SYLLABLE GURAGE HHWA

1E7E9
ETHIOPIC SYLLABLE HHWI

1E7EA
ETHIOPIC SYLLABLE HHWEE

1E7EB
ETHIOPIC SYLLABLE HHWE

1E7ED
ETHIOPIC SYLLABLE GURAGE MWI

1E7EE
ETHIOPIC SYLLABLE GURAGE MWEE

1E7F0
ETHIOPIC SYLLABLE GURAGE QWI

1E7F1
ETHIOPIC SYLLABLE GURAGE QWEE

1E7F2
ETHIOPIC SYLLABLE GURAGE QWE

1E7F3
ETHIOPIC SYLLABLE GURAGE BWI

1E7F4
ETHIOPIC SYLLABLE GURAGE BWEE

1E7F5
ETHIOPIC SYLLABLE GURAGE KWI

1E7F6
ETHIOPIC SYLLABLE GURAGE KWEE

1E7F7
ETHIOPIC SYLLABLE GURAGE KWE

1E7F8
ETHIOPIC SYLLABLE GURAGE GWI

1E7F9
ETHIOPIC SYLLABLE GURAGE GWEE

1E7FA
ETHIOPIC SYLLABLE GURAGE GWE

1E7FB
ETHIOPIC SYLLABLE GURAGE FWI

1E7FC
ETHIOPIC SYLLABLE GURAGE FWEE

1E7FD
ETHIOPIC SYLLABLE GURAGE PWI

1E7FE
ETHIOPIC SYLLABLE GURAGE PWEE

1E800
MENDE KIKAKUI SYLLABLE M001 KI

1E801
MENDE KIKAKUI SYLLABLE M002 KA

1E802
MENDE KIKAKUI SYLLABLE M003 KU

1E803
MENDE KIKAKUI SYLLABLE M065 KEE

1E804
MENDE KIKAKUI SYLLABLE M095 KE

1E805
MENDE KIKAKUI SYLLABLE M076 KOO

1E806
MENDE KIKAKUI SYLLABLE M048 KO

1E807
MENDE KIKAKUI SYLLABLE M179 KUA

1E808
MENDE KIKAKUI SYLLABLE M004 WI

1E809
MENDE KIKAKUI SYLLABLE M005 WA

1E80A
MENDE KIKAKUI SYLLABLE M006 WU

1E80B
MENDE KIKAKUI SYLLABLE M126 WEE

1E80C
MENDE KIKAKUI SYLLABLE M118 WE

1E80D
MENDE KIKAKUI SYLLABLE M114 WOO

1E80E
MENDE KIKAKUI SYLLABLE M045 WO

1E80F
MENDE KIKAKUI SYLLABLE M194 WUI

1E810
MENDE KIKAKUI SYLLABLE M143 WEI

1E811
MENDE KIKAKUI SYLLABLE M061 WVI

1E812
MENDE KIKAKUI SYLLABLE M049 WVA

1E813
MENDE KIKAKUI SYLLABLE M139 WVE

1E814
MENDE KIKAKUI SYLLABLE M007 MIN

1E815
MENDE KIKAKUI SYLLABLE M008 MAN

1E816
MENDE KIKAKUI SYLLABLE M009 MUN

1E817
MENDE KIKAKUI SYLLABLE M059 MEN

1E818
MENDE KIKAKUI SYLLABLE M094 MON

1E819
MENDE KIKAKUI SYLLABLE M154 MUAN

1E81A
MENDE KIKAKUI SYLLABLE M189 MUEN

1E81B
MENDE KIKAKUI SYLLABLE M010 BI

1E81C
MENDE KIKAKUI SYLLABLE M011 BA

1E81D
MENDE KIKAKUI SYLLABLE M012 BU

1E81E
MENDE KIKAKUI SYLLABLE M150 BEE

1E81F
MENDE KIKAKUI SYLLABLE M097 BE

1E820
MENDE KIKAKUI SYLLABLE M103 BOO

1E821
MENDE KIKAKUI SYLLABLE M138 BO

1E822
MENDE KIKAKUI SYLLABLE M013 I

1E823
MENDE KIKAKUI SYLLABLE M014 A

1E824
MENDE KIKAKUI SYLLABLE M015 U

1E825
MENDE KIKAKUI SYLLABLE M163 EE

1E826
MENDE KIKAKUI SYLLABLE M100 E

1E827
MENDE KIKAKUI SYLLABLE M165 OO

1E828
MENDE KIKAKUI SYLLABLE M147 O

1E829
MENDE KIKAKUI SYLLABLE M137 EI

1E82A
MENDE KIKAKUI SYLLABLE M131 IN

1E82B
MENDE KIKAKUI SYLLABLE M135 IN

1E82C
MENDE KIKAKUI SYLLABLE M195 AN

1E82D
MENDE KIKAKUI SYLLABLE M178 EN

1E82E
MENDE KIKAKUI SYLLABLE M019 SI

1E82F
MENDE KIKAKUI SYLLABLE M020 SA

1E830
MENDE KIKAKUI SYLLABLE M021 SU

1E831
MENDE KIKAKUI SYLLABLE M162 SEE

1E832
MENDE KIKAKUI SYLLABLE M116 SE

1E833
MENDE KIKAKUI SYLLABLE M136 SOO

1E834
MENDE KIKAKUI SYLLABLE M079 SO

1E835
MENDE KIKAKUI SYLLABLE M196 SIA

1E836
MENDE KIKAKUI SYLLABLE M025 LI

1E837
MENDE KIKAKUI SYLLABLE M026 LA

1E838
MENDE KIKAKUI SYLLABLE M027 LU

1E839
MENDE KIKAKUI SYLLABLE M084 LEE

1E83A
MENDE KIKAKUI SYLLABLE M073 LE

1E83B
MENDE KIKAKUI SYLLABLE M054 LOO

1E83C
MENDE KIKAKUI SYLLABLE M153 LO

1E83D
MENDE KIKAKUI SYLLABLE M110 LONG LE

1E83E
MENDE KIKAKUI SYLLABLE M016 DI

1E83F
MENDE KIKAKUI SYLLABLE M017 DA

1E840
MENDE KIKAKUI SYLLABLE M018 DU

1E841
MENDE KIKAKUI SYLLABLE M089 DEE

1E842
MENDE KIKAKUI SYLLABLE M180 DOO

1E843
MENDE KIKAKUI SYLLABLE M181 DO

1E844
MENDE KIKAKUI SYLLABLE M022 TI

1E845
MENDE KIKAKUI SYLLABLE M023 TA

1E846
MENDE KIKAKUI SYLLABLE M024 TU

1E847
MENDE KIKAKUI SYLLABLE M091 TEE

1E848
MENDE KIKAKUI SYLLABLE M055 TE

1E849
MENDE KIKAKUI SYLLABLE M104 TOO

1E84A
MENDE KIKAKUI SYLLABLE M069 TO

1E84B
MENDE KIKAKUI SYLLABLE M028 JI

1E84C
MENDE KIKAKUI SYLLABLE M029 JA

1E84D
MENDE KIKAKUI SYLLABLE M030 JU

1E84E
MENDE KIKAKUI SYLLABLE M157 JEE

1E84F
MENDE KIKAKUI SYLLABLE M113 JE

1E850
MENDE KIKAKUI SYLLABLE M160 JOO

1E851
MENDE KIKAKUI SYLLABLE M063 JO

1E852
MENDE KIKAKUI SYLLABLE M175 LONG JO

1E853
MENDE KIKAKUI SYLLABLE M031 YI

1E854
MENDE KIKAKUI SYLLABLE M032 YA

1E855
MENDE KIKAKUI SYLLABLE M033 YU

1E856
MENDE KIKAKUI SYLLABLE M109 YEE

1E857
MENDE KIKAKUI SYLLABLE M080 YE

1E858
MENDE KIKAKUI SYLLABLE M141 YOO

1E859
MENDE KIKAKUI SYLLABLE M121 YO

1E85A
MENDE KIKAKUI SYLLABLE M034 FI

1E85B
MENDE KIKAKUI SYLLABLE M035 FA

1E85C
MENDE KIKAKUI SYLLABLE M036 FU

1E85D
MENDE KIKAKUI SYLLABLE M078 FEE

1E85E
MENDE KIKAKUI SYLLABLE M075 FE

1E85F
MENDE KIKAKUI SYLLABLE M133 FOO

1E860
MENDE KIKAKUI SYLLABLE M088 FO

1E861
MENDE KIKAKUI SYLLABLE M197 FUA

1E862
MENDE KIKAKUI SYLLABLE M101 FAN

1E863
MENDE KIKAKUI SYLLABLE M037 NIN

1E864
MENDE KIKAKUI SYLLABLE M038 NAN

1E865
MENDE KIKAKUI SYLLABLE M039 NUN

1E866
MENDE KIKAKUI SYLLABLE M117 NEN

1E867
MENDE KIKAKUI SYLLABLE M169 NON

1E868
MENDE KIKAKUI SYLLABLE M176 HI

1E869
MENDE KIKAKUI SYLLABLE M041 HA

1E86A
MENDE KIKAKUI SYLLABLE M186 HU

1E86B
MENDE KIKAKUI SYLLABLE M040 HEE

1E86C
MENDE KIKAKUI SYLLABLE M096 HE

1E86D
MENDE KIKAKUI SYLLABLE M042 HOO

1E86E
MENDE KIKAKUI SYLLABLE M140 HO

1E86F
MENDE KIKAKUI SYLLABLE M083 HEEI

1E870
MENDE KIKAKUI SYLLABLE M128 HOOU

1E871
MENDE KIKAKUI SYLLABLE M053 HIN

1E872
MENDE KIKAKUI SYLLABLE M130 HAN

1E873
MENDE KIKAKUI SYLLABLE M087 HUN

1E874
MENDE KIKAKUI SYLLABLE M052 HEN

1E875
MENDE KIKAKUI SYLLABLE M193 HON

1E876
MENDE KIKAKUI SYLLABLE M046 HUAN

1E877
MENDE KIKAKUI SYLLABLE M090 NGGI

1E878
MENDE KIKAKUI SYLLABLE M043 NGGA

1E879
MENDE KIKAKUI SYLLABLE M082 NGGU

1E87A
MENDE KIKAKUI SYLLABLE M115 NGGEE

1E87B
MENDE KIKAKUI SYLLABLE M146 NGGE

1E87C
MENDE KIKAKUI SYLLABLE M156 NGGOO

1E87D
MENDE KIKAKUI SYLLABLE M120 NGGO

1E87E
MENDE KIKAKUI SYLLABLE M159 NGGAA

1E87F
MENDE KIKAKUI SYLLABLE M127 NGGUA

1E880
MENDE KIKAKUI SYLLABLE M086 LONG NGGE

1E881
MENDE KIKAKUI SYLLABLE M106 LONG NGGOO

1E882
MENDE KIKAKUI SYLLABLE M183 LONG NGGO

1E883
MENDE KIKAKUI SYLLABLE M155 GI

1E884
MENDE KIKAKUI SYLLABLE M111 GA

1E885
MENDE KIKAKUI SYLLABLE M168 GU

1E886
MENDE KIKAKUI SYLLABLE M190 GEE

1E887
MENDE KIKAKUI SYLLABLE M166 GUEI

1E888
MENDE KIKAKUI SYLLABLE M167 GUAN

1E889
MENDE KIKAKUI SYLLABLE M184 NGEN

1E88A
MENDE KIKAKUI SYLLABLE M057 NGON

1E88B
MENDE KIKAKUI SYLLABLE M177 NGUAN

1E88C
MENDE KIKAKUI SYLLABLE M068 PI

1E88D
MENDE KIKAKUI SYLLABLE M099 PA

1E88E
MENDE KIKAKUI SYLLABLE M050 PU

1E88F
MENDE KIKAKUI SYLLABLE M081 PEE

1E890
MENDE KIKAKUI SYLLABLE M051 PE

1E891
MENDE KIKAKUI SYLLABLE M102 POO

1E892
MENDE KIKAKUI SYLLABLE M066 PO

1E893
MENDE KIKAKUI SYLLABLE M145 MBI

1E894
MENDE KIKAKUI SYLLABLE M062 MBA

1E895
MENDE KIKAKUI SYLLABLE M122 MBU

1E896
MENDE KIKAKUI SYLLABLE M047 MBEE

1E897
MENDE KIKAKUI SYLLABLE M188 MBEE

1E898
MENDE KIKAKUI SYLLABLE M072 MBE

1E899
MENDE KIKAKUI SYLLABLE M172 MBO

1E899
MENDE KIKAKUI SYLLABLE M172 MBOO

1E89A
MENDE KIKAKUI SYLLABLE M174 MBOO

1E89A
MENDE KIKAKUI SYLLABLE M174 MBO

1E89B
MENDE KIKAKUI SYLLABLE M187 MBUU

1E89C
MENDE KIKAKUI SYLLABLE M161 LONG MBE

1E89D
MENDE KIKAKUI SYLLABLE M105 LONG MBOO

1E89E
MENDE KIKAKUI SYLLABLE M142 LONG MBO

1E89F
MENDE KIKAKUI SYLLABLE M132 KPI

1E8A0
MENDE KIKAKUI SYLLABLE M092 KPA

1E8A1
MENDE KIKAKUI SYLLABLE M074 KPU

1E8A2
MENDE KIKAKUI SYLLABLE M044 KPEE

1E8A3
MENDE KIKAKUI SYLLABLE M108 KPE

1E8A4
MENDE KIKAKUI SYLLABLE M112 KPOO

1E8A5
MENDE KIKAKUI SYLLABLE M158 KPO

1E8A6
MENDE KIKAKUI SYLLABLE M124 GBI

1E8A7
MENDE KIKAKUI SYLLABLE M056 GBA

1E8A8
MENDE KIKAKUI SYLLABLE M148 GBU

1E8A9
MENDE KIKAKUI SYLLABLE M093 GBEE

1E8AA
MENDE KIKAKUI SYLLABLE M107 GBE

1E8AB
MENDE KIKAKUI SYLLABLE M071 GBOO

1E8AC
MENDE KIKAKUI SYLLABLE M070 GBO

1E8AD
MENDE KIKAKUI SYLLABLE M171 RA

1E8AE
MENDE KIKAKUI SYLLABLE M123 NDI

1E8AF
MENDE KIKAKUI SYLLABLE M129 NDA

1E8B0
MENDE KIKAKUI SYLLABLE M125 NDU

1E8B1
MENDE KIKAKUI SYLLABLE M191 NDEE

1E8B2
MENDE KIKAKUI SYLLABLE M119 NDE

1E8B3
MENDE KIKAKUI SYLLABLE M067 NDOO

1E8B4
MENDE KIKAKUI SYLLABLE M064 NDO

1E8B5
MENDE KIKAKUI SYLLABLE M152 NJA

1E8B6
MENDE KIKAKUI SYLLABLE M192 NJU

1E8B7
MENDE KIKAKUI SYLLABLE M149 NJEE

1E8B8
MENDE KIKAKUI SYLLABLE M134 NJOO

1E8B9
MENDE KIKAKUI SYLLABLE M182 VI

1E8BA
MENDE KIKAKUI SYLLABLE M185 VA

1E8BB
MENDE KIKAKUI SYLLABLE M151 VU

1E8BC
MENDE KIKAKUI SYLLABLE M173 VEE

1E8BD
MENDE KIKAKUI SYLLABLE M085 VE

1E8BE
MENDE KIKAKUI SYLLABLE M144 VOO

1E8BF
MENDE KIKAKUI SYLLABLE M077 VO

1E8C0
MENDE KIKAKUI SYLLABLE M164 NYIN

1E8C1
MENDE KIKAKUI SYLLABLE M058 NYAN

1E8C2
MENDE KIKAKUI SYLLABLE M170 NYUN

1E8C3
MENDE KIKAKUI SYLLABLE M098 NYEN

1E8C4
MENDE KIKAKUI SYLLABLE M060 NYON

1E8C7
MENDE KIKAKUI DIGIT ONE

1E8C8
MENDE KIKAKUI DIGIT TWO

1E8C9
MENDE KIKAKUI DIGIT THREE

1E8CA
MENDE KIKAKUI DIGIT FOUR

1E8CB
MENDE KIKAKUI DIGIT FIVE

1E8CC
MENDE KIKAKUI DIGIT SIX

1E8CD
MENDE KIKAKUI DIGIT SEVEN

1E8CE
MENDE KIKAKUI DIGIT EIGHT

1E8CF
MENDE KIKAKUI DIGIT NINE

1E8D0
MENDE KIKAKUI COMBINING NUMBER TEENS

1E8D1
MENDE KIKAKUI COMBINING NUMBER TENS

1E8D2
MENDE KIKAKUI COMBINING NUMBER HUNDREDS

1E8D3
MENDE KIKAKUI COMBINING NUMBER THOUSANDS

1E8D4
MENDE KIKAKUI COMBINING NUMBER TEN THOUSANDS

1E8D5
MENDE KIKAKUI COMBINING NUMBER HUNDRED THOUSANDS

1E8D6
MENDE KIKAKUI COMBINING NUMBER MILLIONS

1E900
ADLAM CAPITAL LETTER ALIF

1E901
ADLAM CAPITAL LETTER DAALI

1E902
ADLAM CAPITAL LETTER LAAM

1E903
ADLAM CAPITAL LETTER MIIM

1E904
ADLAM CAPITAL LETTER BA

1E905
ADLAM CAPITAL LETTER SINNYIIYHE

1E906
ADLAM CAPITAL LETTER PE

1E907
ADLAM CAPITAL LETTER BHE

1E908
ADLAM CAPITAL LETTER RA

1E909
ADLAM CAPITAL LETTER E

1E90A
ADLAM CAPITAL LETTER FA

1E90B
ADLAM CAPITAL LETTER I

1E90C
ADLAM CAPITAL LETTER O

1E90D
ADLAM CAPITAL LETTER DHA

1E90E
ADLAM CAPITAL LETTER YHE

1E90F
ADLAM CAPITAL LETTER WAW

1E910
ADLAM CAPITAL LETTER NUN

1E911
ADLAM CAPITAL LETTER KAF

1E912
ADLAM CAPITAL LETTER YA

1E913
ADLAM CAPITAL LETTER U

1E914
ADLAM CAPITAL LETTER JIIM

1E915
ADLAM CAPITAL LETTER CHI

1E916
ADLAM CAPITAL LETTER HA

1E917
ADLAM CAPITAL LETTER QAAF

1E918
ADLAM CAPITAL LETTER GA

1E919
ADLAM CAPITAL LETTER NYA

1E91A
ADLAM CAPITAL LETTER TU

1E91B
ADLAM CAPITAL LETTER NHA

1E91C
ADLAM CAPITAL LETTER VA

1E91D
ADLAM CAPITAL LETTER KHA

1E91E
ADLAM CAPITAL LETTER GBE

1E91F
ADLAM CAPITAL LETTER ZAL

1E920
ADLAM CAPITAL LETTER KPO

1E921
ADLAM CAPITAL LETTER SHA

1E922
ADLAM SMALL LETTER ALIF

1E923
ADLAM SMALL LETTER DAALI

1E924
ADLAM SMALL LETTER LAAM

1E925
ADLAM SMALL LETTER MIIM

1E926
ADLAM SMALL LETTER BA

1E927
ADLAM SMALL LETTER SINNYIIYHE

1E928
ADLAM SMALL LETTER PE

1E929
ADLAM SMALL LETTER BHE

1E92A
ADLAM SMALL LETTER RA

1E92B
ADLAM SMALL LETTER E

1E92C
ADLAM SMALL LETTER FA

1E92D
ADLAM SMALL LETTER I

1E92E
ADLAM SMALL LETTER O

1E92F
ADLAM SMALL LETTER DHA

1E930
ADLAM SMALL LETTER YHE

1E931
ADLAM SMALL LETTER WAW

1E932
ADLAM SMALL LETTER NUN

1E933
ADLAM SMALL LETTER KAF

1E934
ADLAM SMALL LETTER YA

1E935
ADLAM SMALL LETTER U

1E936
ADLAM SMALL LETTER JIIM

1E937
ADLAM SMALL LETTER CHI

1E938
ADLAM SMALL LETTER HA

1E939
ADLAM SMALL LETTER QAAF

1E93A
ADLAM SMALL LETTER GA

1E93B
ADLAM SMALL LETTER NYA

1E93C
ADLAM SMALL LETTER TU

1E93D
ADLAM SMALL LETTER NHA

1E93E
ADLAM SMALL LETTER VA

1E93F
ADLAM SMALL LETTER KHA

1E940
ADLAM SMALL LETTER GBE

1E941
ADLAM SMALL LETTER ZAL

1E942
ADLAM SMALL LETTER KPO

1E943
ADLAM SMALL LETTER SHA

1E944
ADLAM ALIF LENGTHENER

1E945
ADLAM VOWEL LENGTHENER

1E946
ADLAM GEMINATION MARK

1E947
ADLAM HAMZA

1E948
ADLAM CONSONANT MODIFIER

1E949
ADLAM GEMINATE CONSONANT MODIFIER

1E94A
ADLAM NUKTA

1E94B
ADLAM NASALIZATION MARK

1E950
ADLAM DIGIT ZERO

1E951
ADLAM DIGIT ONE

1E952
ADLAM DIGIT TWO

1E953
ADLAM DIGIT THREE

1E954
ADLAM DIGIT FOUR

1E955
ADLAM DIGIT FIVE

1E956
ADLAM DIGIT SIX

1E957
ADLAM DIGIT SEVEN

1E958
ADLAM DIGIT EIGHT

1E959
ADLAM DIGIT NINE

1E95E
ADLAM INITIAL EXCLAMATION MARK

1E95F
ADLAM INITIAL QUESTION MARK

1EC71
INDIC SIYAQ NUMBER ONE

1EC72
INDIC SIYAQ NUMBER TWO

1EC73
INDIC SIYAQ NUMBER THREE

1EC74
INDIC SIYAQ NUMBER FOUR

1EC75
INDIC SIYAQ NUMBER FIVE

1EC76
INDIC SIYAQ NUMBER SIX

1EC77
INDIC SIYAQ NUMBER SEVEN

1EC78
INDIC SIYAQ NUMBER EIGHT

1EC79
INDIC SIYAQ NUMBER NINE

1EC7A
INDIC SIYAQ NUMBER TEN

1EC7B
INDIC SIYAQ NUMBER TWENTY

1EC7C
INDIC SIYAQ NUMBER THIRTY

1EC7D
INDIC SIYAQ NUMBER FORTY

1EC7E
INDIC SIYAQ NUMBER FIFTY

1EC7F
INDIC SIYAQ NUMBER SIXTY

1EC80
INDIC SIYAQ NUMBER SEVENTY

1EC81
INDIC SIYAQ NUMBER EIGHTY

1EC82
INDIC SIYAQ NUMBER NINETY

1EC83
INDIC SIYAQ NUMBER ONE HUNDRED

1EC84
INDIC SIYAQ NUMBER TWO HUNDRED

1EC85
INDIC SIYAQ NUMBER THREE HUNDRED

1EC86
INDIC SIYAQ NUMBER FOUR HUNDRED

1EC87
INDIC SIYAQ NUMBER FIVE HUNDRED

1EC88
INDIC SIYAQ NUMBER SIX HUNDRED

1EC89
INDIC SIYAQ NUMBER SEVEN HUNDRED

1EC8A
INDIC SIYAQ NUMBER EIGHT HUNDRED

1EC8B
INDIC SIYAQ NUMBER NINE HUNDRED

1EC8C
INDIC SIYAQ NUMBER ONE THOUSAND

1EC8D
INDIC SIYAQ NUMBER TWO THOUSAND

1EC8E
INDIC SIYAQ NUMBER THREE THOUSAND

1EC8F
INDIC SIYAQ NUMBER FOUR THOUSAND

1EC90
INDIC SIYAQ NUMBER FIVE THOUSAND

1EC91
INDIC SIYAQ NUMBER SIX THOUSAND

1EC92
INDIC SIYAQ NUMBER SEVEN THOUSAND

1EC93
INDIC SIYAQ NUMBER EIGHT THOUSAND

1EC94
INDIC SIYAQ NUMBER NINE THOUSAND

1EC95
INDIC SIYAQ NUMBER TEN THOUSAND

1EC96
INDIC SIYAQ NUMBER TWENTY THOUSAND

1EC97
INDIC SIYAQ NUMBER THIRTY THOUSAND

1EC98
INDIC SIYAQ NUMBER FORTY THOUSAND

1EC99
INDIC SIYAQ NUMBER FIFTY THOUSAND

1EC9A
INDIC SIYAQ NUMBER SIXTY THOUSAND

1EC9B
INDIC SIYAQ NUMBER SEVENTY THOUSAND

1EC9C
INDIC SIYAQ NUMBER EIGHTY THOUSAND

1EC9D
INDIC SIYAQ NUMBER NINETY THOUSAND

1EC9E
INDIC SIYAQ NUMBER LAKH

1EC9F
INDIC SIYAQ NUMBER LAKHAN

1ECA0
INDIC SIYAQ LAKH MARK

1ECA1
INDIC SIYAQ NUMBER KAROR

1ECA2
INDIC SIYAQ NUMBER KARORAN

1ECA3
INDIC SIYAQ NUMBER PREFIXED ONE

1ECA4
INDIC SIYAQ NUMBER PREFIXED TWO

1ECA5
INDIC SIYAQ NUMBER PREFIXED THREE

1ECA6
INDIC SIYAQ NUMBER PREFIXED FOUR

1ECA7
INDIC SIYAQ NUMBER PREFIXED FIVE

1ECA8
INDIC SIYAQ NUMBER PREFIXED SIX

1ECA9
INDIC SIYAQ NUMBER PREFIXED SEVEN

1ECAA
INDIC SIYAQ NUMBER PREFIXED EIGHT

1ECAB
INDIC SIYAQ NUMBER PREFIXED NINE

1ECAC
INDIC SIYAQ PLACEHOLDER

1ECAD
INDIC SIYAQ FRACTION ONE QUARTER

1ECAE
INDIC SIYAQ FRACTION ONE HALF

1ECAF
INDIC SIYAQ FRACTION THREE QUARTERS

1ECB0
INDIC SIYAQ RUPEE MARK

1ECB1
INDIC SIYAQ NUMBER ALTERNATE ONE

1ECB2
INDIC SIYAQ NUMBER ALTERNATE TWO

1ECB3
INDIC SIYAQ NUMBER ALTERNATE TEN THOUSAND

1ECB4
INDIC SIYAQ ALTERNATE LAKH MARK

1ED01
OTTOMAN SIYAQ NUMBER ONE

1ED02
OTTOMAN SIYAQ NUMBER TWO

1ED03
OTTOMAN SIYAQ NUMBER THREE

1ED04
OTTOMAN SIYAQ NUMBER FOUR

1ED05
OTTOMAN SIYAQ NUMBER FIVE

1ED06
OTTOMAN SIYAQ NUMBER SIX

1ED07
OTTOMAN SIYAQ NUMBER SEVEN

1ED08
OTTOMAN SIYAQ NUMBER EIGHT

1ED09
OTTOMAN SIYAQ NUMBER NINE

1ED0A
OTTOMAN SIYAQ NUMBER TEN

1ED0B
OTTOMAN SIYAQ NUMBER TWENTY

1ED0C
OTTOMAN SIYAQ NUMBER THIRTY

1ED0D
OTTOMAN SIYAQ NUMBER FORTY

1ED0E
OTTOMAN SIYAQ NUMBER FIFTY

1ED0F
OTTOMAN SIYAQ NUMBER SIXTY

1ED10
OTTOMAN SIYAQ NUMBER SEVENTY

1ED11
OTTOMAN SIYAQ NUMBER EIGHTY

1ED12
OTTOMAN SIYAQ NUMBER NINETY

1ED13
OTTOMAN SIYAQ NUMBER ONE HUNDRED

1ED14
OTTOMAN SIYAQ NUMBER TWO HUNDRED

1ED15
OTTOMAN SIYAQ NUMBER THREE HUNDRED

1ED16
OTTOMAN SIYAQ NUMBER FOUR HUNDRED

1ED17
OTTOMAN SIYAQ NUMBER FIVE HUNDRED

1ED18
OTTOMAN SIYAQ NUMBER SIX HUNDRED

1ED19
OTTOMAN SIYAQ NUMBER SEVEN HUNDRED

1ED1A
OTTOMAN SIYAQ NUMBER EIGHT HUNDRED

1ED1B
OTTOMAN SIYAQ NUMBER NINE HUNDRED

1ED1C
OTTOMAN SIYAQ NUMBER ONE THOUSAND

1ED1D
OTTOMAN SIYAQ NUMBER TWO THOUSAND

1ED1E
OTTOMAN SIYAQ NUMBER THREE THOUSAND

1ED1F
OTTOMAN SIYAQ NUMBER FOUR THOUSAND

1ED20
OTTOMAN SIYAQ NUMBER FIVE THOUSAND

1ED21
OTTOMAN SIYAQ NUMBER SIX THOUSAND

1ED22
OTTOMAN SIYAQ NUMBER SEVEN THOUSAND

1ED23
OTTOMAN SIYAQ NUMBER EIGHT THOUSAND

1ED24
OTTOMAN SIYAQ NUMBER NINE THOUSAND

1ED25
OTTOMAN SIYAQ NUMBER TEN THOUSAND

1ED26
OTTOMAN SIYAQ NUMBER TWENTY THOUSAND

1ED27
OTTOMAN SIYAQ NUMBER THIRTY THOUSAND

1ED28
OTTOMAN SIYAQ NUMBER FORTY THOUSAND

1ED29
OTTOMAN SIYAQ NUMBER FIFTY THOUSAND

1ED2A
OTTOMAN SIYAQ NUMBER SIXTY THOUSAND

1ED2B
OTTOMAN SIYAQ NUMBER SEVENTY THOUSAND

1ED2C
OTTOMAN SIYAQ NUMBER EIGHTY THOUSAND

1ED2D
OTTOMAN SIYAQ NUMBER NINETY THOUSAND

1ED2E
OTTOMAN SIYAQ MARRATAN

1ED2F
OTTOMAN SIYAQ ALTERNATE NUMBER TWO

1ED30
OTTOMAN SIYAQ ALTERNATE NUMBER THREE

1ED31
OTTOMAN SIYAQ ALTERNATE NUMBER FOUR

1ED32
OTTOMAN SIYAQ ALTERNATE NUMBER FIVE

1ED33
OTTOMAN SIYAQ ALTERNATE NUMBER SIX

1ED34
OTTOMAN SIYAQ ALTERNATE NUMBER SEVEN

1ED35
OTTOMAN SIYAQ ALTERNATE NUMBER EIGHT

1ED36
OTTOMAN SIYAQ ALTERNATE NUMBER NINE

1ED37
OTTOMAN SIYAQ ALTERNATE NUMBER TEN

1ED38
OTTOMAN SIYAQ ALTERNATE NUMBER FOUR HUNDRED

1ED39
OTTOMAN SIYAQ ALTERNATE NUMBER SIX HUNDRED

1ED3A
OTTOMAN SIYAQ ALTERNATE NUMBER TWO THOUSAND

1ED3B
OTTOMAN SIYAQ ALTERNATE NUMBER TEN THOUSAND

1ED3C
OTTOMAN SIYAQ FRACTION ONE HALF

1ED3D
OTTOMAN SIYAQ FRACTION ONE SIXTH

1EE00
ARABIC MATHEMATICAL ALEF

1EE01
ARABIC MATHEMATICAL BEH

1EE02
ARABIC MATHEMATICAL JEEM

1EE03
ARABIC MATHEMATICAL DAL

1EE05
ARABIC MATHEMATICAL WAW

1EE06
ARABIC MATHEMATICAL ZAIN

1EE07
ARABIC MATHEMATICAL HAH

1EE08
ARABIC MATHEMATICAL TAH

1EE09
ARABIC MATHEMATICAL YEH

1EE0A
ARABIC MATHEMATICAL KAF

1EE0B
ARABIC MATHEMATICAL LAM

1EE0C
ARABIC MATHEMATICAL MEEM

1EE0D
ARABIC MATHEMATICAL NOON

1EE0E
ARABIC MATHEMATICAL SEEN

1EE0F
ARABIC MATHEMATICAL AIN

1EE10
ARABIC MATHEMATICAL FEH

1EE11
ARABIC MATHEMATICAL SAD

1EE12
ARABIC MATHEMATICAL QAF

1EE13
ARABIC MATHEMATICAL REH

1EE14
ARABIC MATHEMATICAL SHEEN

1EE15
ARABIC MATHEMATICAL TEH

1EE16
ARABIC MATHEMATICAL THEH

1EE17
ARABIC MATHEMATICAL KHAH

1EE18
ARABIC MATHEMATICAL THAL

1EE19
ARABIC MATHEMATICAL DAD

1EE1A
ARABIC MATHEMATICAL ZAH

1EE1B
ARABIC MATHEMATICAL GHAIN

1EE1C
ARABIC MATHEMATICAL DOTLESS BEH

1EE1D
ARABIC MATHEMATICAL DOTLESS NOON

1EE1E
ARABIC MATHEMATICAL DOTLESS FEH

1EE1F
ARABIC MATHEMATICAL DOTLESS QAF

1EE21
ARABIC MATHEMATICAL INITIAL BEH

1EE22
ARABIC MATHEMATICAL INITIAL JEEM

1EE24
ARABIC MATHEMATICAL INITIAL HEH

1EE27
ARABIC MATHEMATICAL INITIAL HAH

1EE29
ARABIC MATHEMATICAL INITIAL YEH

1EE2A
ARABIC MATHEMATICAL INITIAL KAF

1EE2B
ARABIC MATHEMATICAL INITIAL LAM

1EE2C
ARABIC MATHEMATICAL INITIAL MEEM

1EE2D
ARABIC MATHEMATICAL INITIAL NOON

1EE2E
ARABIC MATHEMATICAL INITIAL SEEN

1EE2F
ARABIC MATHEMATICAL INITIAL AIN

1EE30
ARABIC MATHEMATICAL INITIAL FEH

1EE31
ARABIC MATHEMATICAL INITIAL SAD

1EE32
ARABIC MATHEMATICAL INITIAL QAF

1EE34
ARABIC MATHEMATICAL INITIAL SHEEN

1EE35
ARABIC MATHEMATICAL INITIAL TEH

1EE36
ARABIC MATHEMATICAL INITIAL THEH

1EE37
ARABIC MATHEMATICAL INITIAL KHAH

1EE39
ARABIC MATHEMATICAL INITIAL DAD

1EE3B
ARABIC MATHEMATICAL INITIAL GHAIN

1EE42
ARABIC MATHEMATICAL TAILED JEEM

1EE47
ARABIC MATHEMATICAL TAILED HAH

1EE49
ARABIC MATHEMATICAL TAILED YEH

1EE4B
ARABIC MATHEMATICAL TAILED LAM

1EE4D
ARABIC MATHEMATICAL TAILED NOON

1EE4E
ARABIC MATHEMATICAL TAILED SEEN

1EE4F
ARABIC MATHEMATICAL TAILED AIN

1EE51
ARABIC MATHEMATICAL TAILED SAD

1EE52
ARABIC MATHEMATICAL TAILED QAF

1EE54
ARABIC MATHEMATICAL TAILED SHEEN

1EE57
ARABIC MATHEMATICAL TAILED KHAH

1EE59
ARABIC MATHEMATICAL TAILED DAD

1EE5B
ARABIC MATHEMATICAL TAILED GHAIN

1EE5D
ARABIC MATHEMATICAL TAILED DOTLESS NOON

1EE5F
ARABIC MATHEMATICAL TAILED DOTLESS QAF

1EE61
ARABIC MATHEMATICAL STRETCHED BEH

1EE62
ARABIC MATHEMATICAL STRETCHED JEEM

1EE64
ARABIC MATHEMATICAL STRETCHED HEH

1EE67
ARABIC MATHEMATICAL STRETCHED HAH

1EE68
ARABIC MATHEMATICAL STRETCHED TAH

1EE69
ARABIC MATHEMATICAL STRETCHED YEH

1EE6A
ARABIC MATHEMATICAL STRETCHED KAF

1EE6C
ARABIC MATHEMATICAL STRETCHED MEEM

1EE6D
ARABIC MATHEMATICAL STRETCHED NOON

1EE6E
ARABIC MATHEMATICAL STRETCHED SEEN

1EE6F
ARABIC MATHEMATICAL STRETCHED AIN

1EE70
ARABIC MATHEMATICAL STRETCHED FEH

1EE71
ARABIC MATHEMATICAL STRETCHED SAD

1EE72
ARABIC MATHEMATICAL STRETCHED QAF

1EE74
ARABIC MATHEMATICAL STRETCHED SHEEN

1EE75
ARABIC MATHEMATICAL STRETCHED TEH

1EE76
ARABIC MATHEMATICAL STRETCHED THEH

1EE77
ARABIC MATHEMATICAL STRETCHED KHAH

1EE79
ARABIC MATHEMATICAL STRETCHED DAD

1EE7A
ARABIC MATHEMATICAL STRETCHED ZAH

1EE7B
ARABIC MATHEMATICAL STRETCHED GHAIN

1EE7C
ARABIC MATHEMATICAL STRETCHED DOTLESS BEH

1EE7E
ARABIC MATHEMATICAL STRETCHED DOTLESS FEH

1EE80
ARABIC MATHEMATICAL LOOPED ALEF

1EE81
ARABIC MATHEMATICAL LOOPED BEH

1EE82
ARABIC MATHEMATICAL LOOPED JEEM

1EE83
ARABIC MATHEMATICAL LOOPED DAL

1EE84
ARABIC MATHEMATICAL LOOPED HEH

1EE85
ARABIC MATHEMATICAL LOOPED WAW

1EE86
ARABIC MATHEMATICAL LOOPED ZAIN

1EE87
ARABIC MATHEMATICAL LOOPED HAH

1EE88
ARABIC MATHEMATICAL LOOPED TAH

1EE89
ARABIC MATHEMATICAL LOOPED YEH

1EE8B
ARABIC MATHEMATICAL LOOPED LAM

1EE8C
ARABIC MATHEMATICAL LOOPED MEEM

1EE8D
ARABIC MATHEMATICAL LOOPED NOON

1EE8E
ARABIC MATHEMATICAL LOOPED SEEN

1EE8F
ARABIC MATHEMATICAL LOOPED AIN

1EE90
ARABIC MATHEMATICAL LOOPED FEH

1EE91
ARABIC MATHEMATICAL LOOPED SAD

1EE92
ARABIC MATHEMATICAL LOOPED QAF

1EE93
ARABIC MATHEMATICAL LOOPED REH

1EE94
ARABIC MATHEMATICAL LOOPED SHEEN

1EE95
ARABIC MATHEMATICAL LOOPED TEH

1EE96
ARABIC MATHEMATICAL LOOPED THEH

1EE97
ARABIC MATHEMATICAL LOOPED KHAH

1EE98
ARABIC MATHEMATICAL LOOPED THAL

1EE99
ARABIC MATHEMATICAL LOOPED DAD

1EE9A
ARABIC MATHEMATICAL LOOPED ZAH

1EE9B
ARABIC MATHEMATICAL LOOPED GHAIN

1EEA1
ARABIC MATHEMATICAL DOUBLE-STRUCK BEH

1EEA2
ARABIC MATHEMATICAL DOUBLE-STRUCK JEEM

1EEA3
ARABIC MATHEMATICAL DOUBLE-STRUCK DAL

1EEA5
ARABIC MATHEMATICAL DOUBLE-STRUCK WAW

1EEA6
ARABIC MATHEMATICAL DOUBLE-STRUCK ZAIN

1EEA7
ARABIC MATHEMATICAL DOUBLE-STRUCK HAH

1EEA8
ARABIC MATHEMATICAL DOUBLE-STRUCK TAH

1EEA9
ARABIC MATHEMATICAL DOUBLE-STRUCK YEH

1EEAB
ARABIC MATHEMATICAL DOUBLE-STRUCK LAM

1EEAC
ARABIC MATHEMATICAL DOUBLE-STRUCK MEEM

1EEAD
ARABIC MATHEMATICAL DOUBLE-STRUCK NOON

1EEAE
ARABIC MATHEMATICAL DOUBLE-STRUCK SEEN

1EEAF
ARABIC MATHEMATICAL DOUBLE-STRUCK AIN

1EEB0
ARABIC MATHEMATICAL DOUBLE-STRUCK FEH

1EEB1
ARABIC MATHEMATICAL DOUBLE-STRUCK SAD

1EEB2
ARABIC MATHEMATICAL DOUBLE-STRUCK QAF

1EEB3
ARABIC MATHEMATICAL DOUBLE-STRUCK REH

1EEB4
ARABIC MATHEMATICAL DOUBLE-STRUCK SHEEN

1EEB5
ARABIC MATHEMATICAL DOUBLE-STRUCK TEH

1EEB6
ARABIC MATHEMATICAL DOUBLE-STRUCK THEH

1EEB7
ARABIC MATHEMATICAL DOUBLE-STRUCK KHAH

1EEB8
ARABIC MATHEMATICAL DOUBLE-STRUCK THAL

1EEB9
ARABIC MATHEMATICAL DOUBLE-STRUCK DAD

1EEBA
ARABIC MATHEMATICAL DOUBLE-STRUCK ZAH

1EEBB
ARABIC MATHEMATICAL DOUBLE-STRUCK GHAIN

1EEF0
ARABIC MATHEMATICAL OPERATOR MEEM WITH HAH WITH TATWEEL

1EEF1
ARABIC MATHEMATICAL OPERATOR HAH WITH DAL

1F000
MAHJONG TILE EAST WIND

1F001
MAHJONG TILE SOUTH WIND

1F002
MAHJONG TILE WEST WIND

1F003
MAHJONG TILE NORTH WIND

1F004
MAHJONG TILE RED DRAGON

1F005
MAHJONG TILE GREEN DRAGON

1F006
MAHJONG TILE WHITE DRAGON

1F007
MAHJONG TILE ONE OF CHARACTERS

1F008
MAHJONG TILE TWO OF CHARACTERS

1F009
MAHJONG TILE THREE OF CHARACTERS

1F00A
MAHJONG TILE FOUR OF CHARACTERS

1F00B
MAHJONG TILE FIVE OF CHARACTERS

1F00C
MAHJONG TILE SIX OF CHARACTERS

1F00D
MAHJONG TILE SEVEN OF CHARACTERS

1F00E
MAHJONG TILE EIGHT OF CHARACTERS

1F00F
MAHJONG TILE NINE OF CHARACTERS

1F010
MAHJONG TILE ONE OF BAMBOOS

1F011
MAHJONG TILE TWO OF BAMBOOS

1F012
MAHJONG TILE THREE OF BAMBOOS

1F013
MAHJONG TILE FOUR OF BAMBOOS

1F014
MAHJONG TILE FIVE OF BAMBOOS

1F015
MAHJONG TILE SIX OF BAMBOOS

1F016
MAHJONG TILE SEVEN OF BAMBOOS

1F017
MAHJONG TILE EIGHT OF BAMBOOS

1F018
MAHJONG TILE NINE OF BAMBOOS

1F019
MAHJONG TILE ONE OF CIRCLES

1F01A
MAHJONG TILE TWO OF CIRCLES

1F01B
MAHJONG TILE THREE OF CIRCLES

1F01C
MAHJONG TILE FOUR OF CIRCLES

1F01D
MAHJONG TILE FIVE OF CIRCLES

1F01E
MAHJONG TILE SIX OF CIRCLES

1F01F
MAHJONG TILE SEVEN OF CIRCLES

1F020
MAHJONG TILE EIGHT OF CIRCLES

1F021
MAHJONG TILE NINE OF CIRCLES

1F022
MAHJONG TILE PLUM

1F023
MAHJONG TILE ORCHID

1F024
MAHJONG TILE BAMBOO

1F025
MAHJONG TILE CHRYSANTHEMUM

1F026
MAHJONG TILE SPRING

1F027
MAHJONG TILE SUMMER

1F028
MAHJONG TILE AUTUMN

1F029
MAHJONG TILE WINTER

1F02A
MAHJONG TILE JOKER

1F02B
MAHJONG TILE BACK

1F030
DOMINO TILE HORIZONTAL BACK

1F031
DOMINO TILE HORIZONTAL-00-00

1F032
DOMINO TILE HORIZONTAL-00-01

1F033
DOMINO TILE HORIZONTAL-00-02

1F034
DOMINO TILE HORIZONTAL-00-03

1F035
DOMINO TILE HORIZONTAL-00-04

1F036
DOMINO TILE HORIZONTAL-00-05

1F037
DOMINO TILE HORIZONTAL-00-06

1F038
DOMINO TILE HORIZONTAL-01-00

1F039
DOMINO TILE HORIZONTAL-01-01

1F03A
DOMINO TILE HORIZONTAL-01-02

1F03B
DOMINO TILE HORIZONTAL-01-03

1F03C
DOMINO TILE HORIZONTAL-01-04

1F03D
DOMINO TILE HORIZONTAL-01-05

1F03E
DOMINO TILE HORIZONTAL-01-06

1F03F
DOMINO TILE HORIZONTAL-02-00

1F040
DOMINO TILE HORIZONTAL-02-01

1F041
DOMINO TILE HORIZONTAL-02-02

1F042
DOMINO TILE HORIZONTAL-02-03

1F043
DOMINO TILE HORIZONTAL-02-04

1F044
DOMINO TILE HORIZONTAL-02-05

1F045
DOMINO TILE HORIZONTAL-02-06

1F046
DOMINO TILE HORIZONTAL-03-00

1F047
DOMINO TILE HORIZONTAL-03-01

1F048
DOMINO TILE HORIZONTAL-03-02

1F049
DOMINO TILE HORIZONTAL-03-03

1F04A
DOMINO TILE HORIZONTAL-03-04

1F04B
DOMINO TILE HORIZONTAL-03-05

1F04C
DOMINO TILE HORIZONTAL-03-06

1F04D
DOMINO TILE HORIZONTAL-04-00

1F04E
DOMINO TILE HORIZONTAL-04-01

1F04F
DOMINO TILE HORIZONTAL-04-02

1F050
DOMINO TILE HORIZONTAL-04-03

1F051
DOMINO TILE HORIZONTAL-04-04

1F052
DOMINO TILE HORIZONTAL-04-05

1F053
DOMINO TILE HORIZONTAL-04-06

1F054
DOMINO TILE HORIZONTAL-05-00

1F055
DOMINO TILE HORIZONTAL-05-01

1F056
DOMINO TILE HORIZONTAL-05-02

1F057
DOMINO TILE HORIZONTAL-05-03

1F058
DOMINO TILE HORIZONTAL-05-04

1F059
DOMINO TILE HORIZONTAL-05-05

1F05A
DOMINO TILE HORIZONTAL-05-06

1F05B
DOMINO TILE HORIZONTAL-06-00

1F05C
DOMINO TILE HORIZONTAL-06-01

1F05D
DOMINO TILE HORIZONTAL-06-02

1F05E
DOMINO TILE HORIZONTAL-06-03

1F05F
DOMINO TILE HORIZONTAL-06-04

1F060
DOMINO TILE HORIZONTAL-06-05

1F061
DOMINO TILE HORIZONTAL-06-06

1F062
DOMINO TILE VERTICAL BACK

1F063
DOMINO TILE VERTICAL-00-00

1F064
DOMINO TILE VERTICAL-00-01

1F065
DOMINO TILE VERTICAL-00-02

1F066
DOMINO TILE VERTICAL-00-03

1F067
DOMINO TILE VERTICAL-00-04

1F068
DOMINO TILE VERTICAL-00-05

1F069
DOMINO TILE VERTICAL-00-06

1F06A
DOMINO TILE VERTICAL-01-00

1F06B
DOMINO TILE VERTICAL-01-01

1F06C
DOMINO TILE VERTICAL-01-02

1F06D
DOMINO TILE VERTICAL-01-03

1F06E
DOMINO TILE VERTICAL-01-04

1F06F
DOMINO TILE VERTICAL-01-05

1F070
DOMINO TILE VERTICAL-01-06

1F071
DOMINO TILE VERTICAL-02-00

1F072
DOMINO TILE VERTICAL-02-01

1F073
DOMINO TILE VERTICAL-02-02

1F074
DOMINO TILE VERTICAL-02-03

1F075
DOMINO TILE VERTICAL-02-04

1F076
DOMINO TILE VERTICAL-02-05

1F077
DOMINO TILE VERTICAL-02-06

1F078
DOMINO TILE VERTICAL-03-00

1F079
DOMINO TILE VERTICAL-03-01

1F07A
DOMINO TILE VERTICAL-03-02

1F07B
DOMINO TILE VERTICAL-03-03

1F07C
DOMINO TILE VERTICAL-03-04

1F07D
DOMINO TILE VERTICAL-03-05

1F07E
DOMINO TILE VERTICAL-03-06

1F07F
DOMINO TILE VERTICAL-04-00

1F080
DOMINO TILE VERTICAL-04-01

1F081
DOMINO TILE VERTICAL-04-02

1F082
DOMINO TILE VERTICAL-04-03

1F083
DOMINO TILE VERTICAL-04-04

1F084
DOMINO TILE VERTICAL-04-05

1F085
DOMINO TILE VERTICAL-04-06

1F086
DOMINO TILE VERTICAL-05-00

1F087
DOMINO TILE VERTICAL-05-01

1F088
DOMINO TILE VERTICAL-05-02

1F089
DOMINO TILE VERTICAL-05-03

1F08A
DOMINO TILE VERTICAL-05-04

1F08B
DOMINO TILE VERTICAL-05-05

1F08C
DOMINO TILE VERTICAL-05-06

1F08D
DOMINO TILE VERTICAL-06-00

1F08E
DOMINO TILE VERTICAL-06-01

1F08F
DOMINO TILE VERTICAL-06-02

1F090
DOMINO TILE VERTICAL-06-03

1F091
DOMINO TILE VERTICAL-06-04

1F092
DOMINO TILE VERTICAL-06-05

1F093
DOMINO TILE VERTICAL-06-06

1F0A0
PLAYING CARD BACK

1F0A1
PLAYING CARD ACE OF SPADES

1F0A2
PLAYING CARD TWO OF SPADES

1F0A3
PLAYING CARD THREE OF SPADES

1F0A4
PLAYING CARD FOUR OF SPADES

1F0A5
PLAYING CARD FIVE OF SPADES

1F0A6
PLAYING CARD SIX OF SPADES

1F0A7
PLAYING CARD SEVEN OF SPADES

1F0A8
PLAYING CARD EIGHT OF SPADES

1F0A9
PLAYING CARD NINE OF SPADES

1F0AA
PLAYING CARD TEN OF SPADES

1F0AB
PLAYING CARD JACK OF SPADES

1F0AC
PLAYING CARD KNIGHT OF SPADES

1F0AD
PLAYING CARD QUEEN OF SPADES

1F0AE
PLAYING CARD KING OF SPADES

1F0B1
PLAYING CARD ACE OF HEARTS

1F0B2
PLAYING CARD TWO OF HEARTS

1F0B3
PLAYING CARD THREE OF HEARTS

1F0B4
PLAYING CARD FOUR OF HEARTS

1F0B5
PLAYING CARD FIVE OF HEARTS

1F0B6
PLAYING CARD SIX OF HEARTS

1F0B7
PLAYING CARD SEVEN OF HEARTS

1F0B8
PLAYING CARD EIGHT OF HEARTS

1F0B9
PLAYING CARD NINE OF HEARTS

1F0BA
PLAYING CARD TEN OF HEARTS

1F0BB
PLAYING CARD JACK OF HEARTS

1F0BC
PLAYING CARD KNIGHT OF HEARTS

1F0BD
PLAYING CARD QUEEN OF HEARTS

1F0BE
PLAYING CARD KING OF HEARTS

1F0BF
PLAYING CARD RED JOKER

1F0C1
PLAYING CARD ACE OF DIAMONDS

1F0C2
PLAYING CARD TWO OF DIAMONDS

1F0C3
PLAYING CARD THREE OF DIAMONDS

1F0C4
PLAYING CARD FOUR OF DIAMONDS

1F0C5
PLAYING CARD FIVE OF DIAMONDS

1F0C6
PLAYING CARD SIX OF DIAMONDS

1F0C7
PLAYING CARD SEVEN OF DIAMONDS

1F0C8
PLAYING CARD EIGHT OF DIAMONDS

1F0C9
PLAYING CARD NINE OF DIAMONDS

1F0CA
PLAYING CARD TEN OF DIAMONDS

1F0CB
PLAYING CARD JACK OF DIAMONDS

1F0CC
PLAYING CARD KNIGHT OF DIAMONDS

1F0CD
PLAYING CARD QUEEN OF DIAMONDS

1F0CE
PLAYING CARD KING OF DIAMONDS

1F0CF
PLAYING CARD BLACK JOKER

1F0D1
PLAYING CARD ACE OF CLUBS

1F0D2
PLAYING CARD TWO OF CLUBS

1F0D3
PLAYING CARD THREE OF CLUBS

1F0D4
PLAYING CARD FOUR OF CLUBS

1F0D5
PLAYING CARD FIVE OF CLUBS

1F0D6
PLAYING CARD SIX OF CLUBS

1F0D7
PLAYING CARD SEVEN OF CLUBS

1F0D8
PLAYING CARD EIGHT OF CLUBS

1F0D9
PLAYING CARD NINE OF CLUBS

1F0DA
PLAYING CARD TEN OF CLUBS

1F0DB
PLAYING CARD JACK OF CLUBS

1F0DC
PLAYING CARD KNIGHT OF CLUBS

1F0DD
PLAYING CARD QUEEN OF CLUBS

1F0DE
PLAYING CARD KING OF CLUBS

1F0DF
PLAYING CARD WHITE JOKER

1F0E0
PLAYING CARD FOOL

1F0E1
PLAYING CARD TRUMP-1

1F0E2
PLAYING CARD TRUMP-2

1F0E3
PLAYING CARD TRUMP-3

1F0E4
PLAYING CARD TRUMP-4

1F0E5
PLAYING CARD TRUMP-5

1F0E6
PLAYING CARD TRUMP-6

1F0E7
PLAYING CARD TRUMP-7

1F0E8
PLAYING CARD TRUMP-8

1F0E9
PLAYING CARD TRUMP-9

1F0EA
PLAYING CARD TRUMP-10

1F0EB
PLAYING CARD TRUMP-11

1F0EC
PLAYING CARD TRUMP-12

1F0ED
PLAYING CARD TRUMP-13

1F0EE
PLAYING CARD TRUMP-14

1F0EF
PLAYING CARD TRUMP-15

1F0F0
PLAYING CARD TRUMP-16

1F0F1
PLAYING CARD TRUMP-17

1F0F2
PLAYING CARD TRUMP-18

1F0F3
PLAYING CARD TRUMP-19

1F0F4
PLAYING CARD TRUMP-20

1F0F5
PLAYING CARD TRUMP-21

1F100
DIGIT ZERO FULL STOP

1F101
DIGIT ZERO COMMA

1F102
DIGIT ONE COMMA

1F103
DIGIT TWO COMMA

1F104
DIGIT THREE COMMA

1F105
DIGIT FOUR COMMA

1F106
DIGIT FIVE COMMA

1F107
DIGIT SIX COMMA

1F108
DIGIT SEVEN COMMA

1F109
DIGIT EIGHT COMMA

1F10A
DIGIT NINE COMMA

1F10B
DINGBAT CIRCLED SANS-SERIF DIGIT ZERO

1F10C
DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT ZERO

1F10D
CIRCLED ZERO WITH SLASH

1F10E
CIRCLED ANTICLOCKWISE ARROW

1F10F
CIRCLED DOLLAR SIGN WITH OVERLAID BACKSLASH

1F110
PARENTHESIZED LATIN CAPITAL LETTER A

1F111
PARENTHESIZED LATIN CAPITAL LETTER B

1F112
PARENTHESIZED LATIN CAPITAL LETTER C

1F113
PARENTHESIZED LATIN CAPITAL LETTER D

1F114
PARENTHESIZED LATIN CAPITAL LETTER E

1F115
PARENTHESIZED LATIN CAPITAL LETTER F

1F116
PARENTHESIZED LATIN CAPITAL LETTER G

1F117
PARENTHESIZED LATIN CAPITAL LETTER H

1F118
PARENTHESIZED LATIN CAPITAL LETTER I

1F119
PARENTHESIZED LATIN CAPITAL LETTER J

1F11A
PARENTHESIZED LATIN CAPITAL LETTER K

1F11B
PARENTHESIZED LATIN CAPITAL LETTER L

1F11C
PARENTHESIZED LATIN CAPITAL LETTER M

1F11D
PARENTHESIZED LATIN CAPITAL LETTER N

1F11E
PARENTHESIZED LATIN CAPITAL LETTER O

1F11F
PARENTHESIZED LATIN CAPITAL LETTER P

1F120
PARENTHESIZED LATIN CAPITAL LETTER Q

1F121
PARENTHESIZED LATIN CAPITAL LETTER R

1F122
PARENTHESIZED LATIN CAPITAL LETTER S

1F123
PARENTHESIZED LATIN CAPITAL LETTER T

1F124
PARENTHESIZED LATIN CAPITAL LETTER U

1F125
PARENTHESIZED LATIN CAPITAL LETTER V

1F126
PARENTHESIZED LATIN CAPITAL LETTER W

1F127
PARENTHESIZED LATIN CAPITAL LETTER X

1F128
PARENTHESIZED LATIN CAPITAL LETTER Y

1F129
PARENTHESIZED LATIN CAPITAL LETTER Z

1F12A
TORTOISE SHELL BRACKETED LATIN CAPITAL LETTER S

1F12B
CIRCLED ITALIC LATIN CAPITAL LETTER C

1F12C
CIRCLED ITALIC LATIN CAPITAL LETTER R

1F12D
CIRCLED CD

1F12E
CIRCLED WZ

1F12F
COPYLEFT SYMBOL

1F130
SQUARED LATIN CAPITAL LETTER A

1F131
SQUARED LATIN CAPITAL LETTER B

1F132
SQUARED LATIN CAPITAL LETTER C

1F133
SQUARED LATIN CAPITAL LETTER D

1F134
SQUARED LATIN CAPITAL LETTER E

1F135
SQUARED LATIN CAPITAL LETTER F

1F136
SQUARED LATIN CAPITAL LETTER G

1F137
SQUARED LATIN CAPITAL LETTER H

1F138
SQUARED LATIN CAPITAL LETTER I

1F139
SQUARED LATIN CAPITAL LETTER J

1F13A
SQUARED LATIN CAPITAL LETTER K

1F13B
SQUARED LATIN CAPITAL LETTER L

1F13C
SQUARED LATIN CAPITAL LETTER M

1F13D
SQUARED LATIN CAPITAL LETTER N

1F13E
SQUARED LATIN CAPITAL LETTER O

1F13F
SQUARED LATIN CAPITAL LETTER P

1F140
SQUARED LATIN CAPITAL LETTER Q

1F141
SQUARED LATIN CAPITAL LETTER R

1F142
SQUARED LATIN CAPITAL LETTER S

1F143
SQUARED LATIN CAPITAL LETTER T

1F144
SQUARED LATIN CAPITAL LETTER U

1F145
SQUARED LATIN CAPITAL LETTER V

1F146
SQUARED LATIN CAPITAL LETTER W

1F147
SQUARED LATIN CAPITAL LETTER X

1F148
SQUARED LATIN CAPITAL LETTER Y

1F149
SQUARED LATIN CAPITAL LETTER Z

1F14A
SQUARED HV

1F14B
SQUARED MV

1F14C
SQUARED SD

1F14D
SQUARED SS

1F14E
SQUARED PPV

1F14F
SQUARED WC

1F150
NEGATIVE CIRCLED LATIN CAPITAL LETTER A

1F151
NEGATIVE CIRCLED LATIN CAPITAL LETTER B

1F152
NEGATIVE CIRCLED LATIN CAPITAL LETTER C

1F153
NEGATIVE CIRCLED LATIN CAPITAL LETTER D

1F154
NEGATIVE CIRCLED LATIN CAPITAL LETTER E

1F155
NEGATIVE CIRCLED LATIN CAPITAL LETTER F

1F156
NEGATIVE CIRCLED LATIN CAPITAL LETTER G

1F157
NEGATIVE CIRCLED LATIN CAPITAL LETTER H

1F158
NEGATIVE CIRCLED LATIN CAPITAL LETTER I

1F159
NEGATIVE CIRCLED LATIN CAPITAL LETTER J

1F15A
NEGATIVE CIRCLED LATIN CAPITAL LETTER K

1F15B
NEGATIVE CIRCLED LATIN CAPITAL LETTER L

1F15C
NEGATIVE CIRCLED LATIN CAPITAL LETTER M

1F15D
NEGATIVE CIRCLED LATIN CAPITAL LETTER N

1F15E
NEGATIVE CIRCLED LATIN CAPITAL LETTER O

1F15F
NEGATIVE CIRCLED LATIN CAPITAL LETTER P

1F160
NEGATIVE CIRCLED LATIN CAPITAL LETTER Q

1F161
NEGATIVE CIRCLED LATIN CAPITAL LETTER R

1F162
NEGATIVE CIRCLED LATIN CAPITAL LETTER S

1F163
NEGATIVE CIRCLED LATIN CAPITAL LETTER T

1F164
NEGATIVE CIRCLED LATIN CAPITAL LETTER U

1F165
NEGATIVE CIRCLED LATIN CAPITAL LETTER V

1F166
NEGATIVE CIRCLED LATIN CAPITAL LETTER W

1F167
NEGATIVE CIRCLED LATIN CAPITAL LETTER X

1F168
NEGATIVE CIRCLED LATIN CAPITAL LETTER Y

1F169
NEGATIVE CIRCLED LATIN CAPITAL LETTER Z

1F16A
RAISED MC SIGN

1F16B
RAISED MD SIGN

1F16C
RAISED MR SIGN

1F16D
CIRCLED CC

1F16E
CIRCLED C WITH OVERLAID BACKSLASH

1F16F
CIRCLED HUMAN FIGURE

1F170
NEGATIVE SQUARED LATIN CAPITAL LETTER A

1F171
NEGATIVE SQUARED LATIN CAPITAL LETTER B

1F172
NEGATIVE SQUARED LATIN CAPITAL LETTER C

1F173
NEGATIVE SQUARED LATIN CAPITAL LETTER D

1F174
NEGATIVE SQUARED LATIN CAPITAL LETTER E

1F175
NEGATIVE SQUARED LATIN CAPITAL LETTER F

1F176
NEGATIVE SQUARED LATIN CAPITAL LETTER G

1F177
NEGATIVE SQUARED LATIN CAPITAL LETTER H

1F178
NEGATIVE SQUARED LATIN CAPITAL LETTER I

1F179
NEGATIVE SQUARED LATIN CAPITAL LETTER J

1F17A
NEGATIVE SQUARED LATIN CAPITAL LETTER K

1F17B
NEGATIVE SQUARED LATIN CAPITAL LETTER L

1F17C
NEGATIVE SQUARED LATIN CAPITAL LETTER M

1F17D
NEGATIVE SQUARED LATIN CAPITAL LETTER N

1F17E
NEGATIVE SQUARED LATIN CAPITAL LETTER O

1F17F
NEGATIVE SQUARED LATIN CAPITAL LETTER P

1F180
NEGATIVE SQUARED LATIN CAPITAL LETTER Q

1F181
NEGATIVE SQUARED LATIN CAPITAL LETTER R

1F182
NEGATIVE SQUARED LATIN CAPITAL LETTER S

1F183
NEGATIVE SQUARED LATIN CAPITAL LETTER T

1F184
NEGATIVE SQUARED LATIN CAPITAL LETTER U

1F185
NEGATIVE SQUARED LATIN CAPITAL LETTER V

1F186
NEGATIVE SQUARED LATIN CAPITAL LETTER W

1F187
NEGATIVE SQUARED LATIN CAPITAL LETTER X

1F188
NEGATIVE SQUARED LATIN CAPITAL LETTER Y

1F189
NEGATIVE SQUARED LATIN CAPITAL LETTER Z

1F18A
CROSSED NEGATIVE SQUARED LATIN CAPITAL LETTER P

1F18B
NEGATIVE SQUARED IC

1F18C
NEGATIVE SQUARED PA

1F18D
NEGATIVE SQUARED SA

1F18E
NEGATIVE SQUARED AB

1F18F
NEGATIVE SQUARED WC

1F190
SQUARE DJ

1F191
SQUARED CL

1F192
SQUARED COOL

1F193
SQUARED FREE

1F194
SQUARED ID

1F195
SQUARED NEW

1F196
SQUARED NG

1F197
SQUARED OK

1F198
SQUARED SOS

1F199
SQUARED UP WITH EXCLAMATION MARK

1F19A
SQUARED VS

1F19B
SQUARED THREE D

1F19C
SQUARED SECOND SCREEN

1F19D
SQUARED TWO K

1F19E
SQUARED FOUR K

1F19F
SQUARED EIGHT K

1F1A0
SQUARED FIVE POINT ONE

1F1A1
SQUARED SEVEN POINT ONE

1F1A2
SQUARED TWENTY-TWO POINT TWO

1F1A3
SQUARED SIXTY P

1F1A4
SQUARED ONE HUNDRED TWENTY P

1F1A5
SQUARED LATIN SMALL LETTER D

1F1A6
SQUARED HC

1F1A7
SQUARED HDR

1F1A8
SQUARED HI-RES

1F1A9
SQUARED LOSSLESS

1F1AA
SQUARED SHV

1F1AB
SQUARED UHD

1F1AC
SQUARED VOD

1F1AD
MASK WORK SYMBOL

1F1E6
REGIONAL INDICATOR SYMBOL LETTER A

1F1E7
REGIONAL INDICATOR SYMBOL LETTER B

1F1E8
REGIONAL INDICATOR SYMBOL LETTER C

1F1E9
REGIONAL INDICATOR SYMBOL LETTER D

1F1EA
REGIONAL INDICATOR SYMBOL LETTER E

1F1EB
REGIONAL INDICATOR SYMBOL LETTER F

1F1EC
REGIONAL INDICATOR SYMBOL LETTER G

1F1ED
REGIONAL INDICATOR SYMBOL LETTER H

1F1EE
REGIONAL INDICATOR SYMBOL LETTER I

1F1EF
REGIONAL INDICATOR SYMBOL LETTER J

1F1F0
REGIONAL INDICATOR SYMBOL LETTER K

1F1F1
REGIONAL INDICATOR SYMBOL LETTER L

1F1F2
REGIONAL INDICATOR SYMBOL LETTER M

1F1F3
REGIONAL INDICATOR SYMBOL LETTER N

1F1F4
REGIONAL INDICATOR SYMBOL LETTER O

1F1F5
REGIONAL INDICATOR SYMBOL LETTER P

1F1F6
REGIONAL INDICATOR SYMBOL LETTER Q

1F1F7
REGIONAL INDICATOR SYMBOL LETTER R

1F1F8
REGIONAL INDICATOR SYMBOL LETTER S

1F1F9
REGIONAL INDICATOR SYMBOL LETTER T

1F1FA
REGIONAL INDICATOR SYMBOL LETTER U

1F1FB
REGIONAL INDICATOR SYMBOL LETTER V

1F1FC
REGIONAL INDICATOR SYMBOL LETTER W

1F1FD
REGIONAL INDICATOR SYMBOL LETTER X

1F1FE
REGIONAL INDICATOR SYMBOL LETTER Y

1F1FF
REGIONAL INDICATOR SYMBOL LETTER Z

1F200
SQUARE HIRAGANA HOKA

1F201
SQUARED KATAKANA KOKO

1F202
SQUARED KATAKANA SA

1F210
SQUARED CJK UNIFIED IDEOGRAPH-624B

1F211
SQUARED CJK UNIFIED IDEOGRAPH-5B57

1F212
SQUARED CJK UNIFIED IDEOGRAPH-53CC

1F213
SQUARED KATAKANA DE

1F214
SQUARED CJK UNIFIED IDEOGRAPH-4E8C

1F215
SQUARED CJK UNIFIED IDEOGRAPH-591A

1F216
SQUARED CJK UNIFIED IDEOGRAPH-89E3

1F217
SQUARED CJK UNIFIED IDEOGRAPH-5929

1F218
SQUARED CJK UNIFIED IDEOGRAPH-4EA4

1F219
SQUARED CJK UNIFIED IDEOGRAPH-6620

1F21A
SQUARED CJK UNIFIED IDEOGRAPH-7121

1F21B
SQUARED CJK UNIFIED IDEOGRAPH-6599

1F21C
SQUARED CJK UNIFIED IDEOGRAPH-524D

1F21D
SQUARED CJK UNIFIED IDEOGRAPH-5F8C

1F21E
SQUARED CJK UNIFIED IDEOGRAPH-518D

1F21F
SQUARED CJK UNIFIED IDEOGRAPH-65B0

1F220
SQUARED CJK UNIFIED IDEOGRAPH-521D

1F221
SQUARED CJK UNIFIED IDEOGRAPH-7D42

1F222
SQUARED CJK UNIFIED IDEOGRAPH-751F

1F223
SQUARED CJK UNIFIED IDEOGRAPH-8CA9

1F224
SQUARED CJK UNIFIED IDEOGRAPH-58F0

1F225
SQUARED CJK UNIFIED IDEOGRAPH-5439

1F226
SQUARED CJK UNIFIED IDEOGRAPH-6F14

1F227
SQUARED CJK UNIFIED IDEOGRAPH-6295

1F228
SQUARED CJK UNIFIED IDEOGRAPH-6355

1F229
SQUARED CJK UNIFIED IDEOGRAPH-4E00

1F22A
SQUARED CJK UNIFIED IDEOGRAPH-4E09

1F22B
SQUARED CJK UNIFIED IDEOGRAPH-904A

1F22C
SQUARED CJK UNIFIED IDEOGRAPH-5DE6

1F22D
SQUARED CJK UNIFIED IDEOGRAPH-4E2D

1F22E
SQUARED CJK UNIFIED IDEOGRAPH-53F3

1F22F
SQUARED CJK UNIFIED IDEOGRAPH-6307

1F230
SQUARED CJK UNIFIED IDEOGRAPH-8D70

1F231
SQUARED CJK UNIFIED IDEOGRAPH-6253

1F232
SQUARED CJK UNIFIED IDEOGRAPH-7981

1F233
SQUARED CJK UNIFIED IDEOGRAPH-7A7A

1F234
SQUARED CJK UNIFIED IDEOGRAPH-5408

1F235
SQUARED CJK UNIFIED IDEOGRAPH-6E80

1F236
SQUARED CJK UNIFIED IDEOGRAPH-6709

1F237
SQUARED CJK UNIFIED IDEOGRAPH-6708

1F238
SQUARED CJK UNIFIED IDEOGRAPH-7533

1F239
SQUARED CJK UNIFIED IDEOGRAPH-5272

1F23A
SQUARED CJK UNIFIED IDEOGRAPH-55B6

1F23B
SQUARED CJK UNIFIED IDEOGRAPH-914D

1F240
TORTOISE SHELL BRACKETED CJK UNIFIED IDEOGRAPH-672C

1F241
TORTOISE SHELL BRACKETED CJK UNIFIED IDEOGRAPH-4E09

1F242
TORTOISE SHELL BRACKETED CJK UNIFIED IDEOGRAPH-4E8C

1F243
TORTOISE SHELL BRACKETED CJK UNIFIED IDEOGRAPH-5B89

1F244
TORTOISE SHELL BRACKETED CJK UNIFIED IDEOGRAPH-70B9

1F245
TORTOISE SHELL BRACKETED CJK UNIFIED IDEOGRAPH-6253

1F246
TORTOISE SHELL BRACKETED CJK UNIFIED IDEOGRAPH-76D7

1F247
TORTOISE SHELL BRACKETED CJK UNIFIED IDEOGRAPH-52DD

1F248
TORTOISE SHELL BRACKETED CJK UNIFIED IDEOGRAPH-6557

1F250
CIRCLED IDEOGRAPH ADVANTAGE

1F251
CIRCLED IDEOGRAPH ACCEPT

1F260
ROUNDED SYMBOL FOR FU

1F261
ROUNDED SYMBOL FOR LU

1F262
ROUNDED SYMBOL FOR SHOU

1F263
ROUNDED SYMBOL FOR XI

1F264
ROUNDED SYMBOL FOR SHUANGXI

1F265
ROUNDED SYMBOL FOR CAI

1F300
CYCLONE

1F301
FOGGY

1F302
CLOSED UMBRELLA

1F303
NIGHT WITH STARS

1F304
SUNRISE OVER MOUNTAINS

1F305
SUNRISE

1F306
CITYSCAPE AT DUSK

1F307
SUNSET OVER BUILDINGS

1F308
RAINBOW

1F309
BRIDGE AT NIGHT

1F30A
WATER WAVE

1F30B
VOLCANO

1F30C
MILKY WAY

1F30D
EARTH GLOBE EUROPE-AFRICA

1F30E
EARTH GLOBE AMERICAS

1F30F
EARTH GLOBE ASIA-AUSTRALIA

1F310
GLOBE WITH MERIDIANS

1F311
NEW MOON SYMBOL

1F312
WAXING CRESCENT MOON SYMBOL

1F313
FIRST QUARTER MOON SYMBOL

1F314
WAXING GIBBOUS MOON SYMBOL

1F315
FULL MOON SYMBOL

1F316
WANING GIBBOUS MOON SYMBOL

1F317
LAST QUARTER MOON SYMBOL

1F318
WANING CRESCENT MOON SYMBOL

1F319
CRESCENT MOON

1F31A
NEW MOON WITH FACE

1F31B
FIRST QUARTER MOON WITH FACE

1F31C
LAST QUARTER MOON WITH FACE

1F31D
FULL MOON WITH FACE

1F31E
SUN WITH FACE

1F31F
GLOWING STAR

1F320
SHOOTING STAR

1F321
THERMOMETER

1F322
BLACK DROPLET

1F323
WHITE SUN

1F324
WHITE SUN WITH SMALL CLOUD

1F325
WHITE SUN BEHIND CLOUD

1F326
WHITE SUN BEHIND CLOUD WITH RAIN

1F327
CLOUD WITH RAIN

1F328
CLOUD WITH SNOW

1F329
CLOUD WITH LIGHTNING

1F32A
CLOUD WITH TORNADO

1F32B
FOG

1F32C
WIND BLOWING FACE

1F32D
HOT DOG

1F32E
TACO

1F32F
BURRITO

1F330
CHESTNUT

1F331
SEEDLING

1F332
EVERGREEN TREE

1F333
DECIDUOUS TREE

1F334
PALM TREE

1F335
CACTUS

1F336
HOT PEPPER

1F337
TULIP

1F338
CHERRY BLOSSOM

1F339
ROSE

1F33A
HIBISCUS

1F33B
SUNFLOWER

1F33C
BLOSSOM

1F33D
EAR OF MAIZE

1F33E
EAR OF RICE

1F33F
HERB

1F340
FOUR LEAF CLOVER

1F341
MAPLE LEAF

1F342
FALLEN LEAF

1F343
LEAF FLUTTERING IN WIND

1F344
MUSHROOM

1F345
TOMATO

1F346
AUBERGINE

1F347
GRAPES

1F348
MELON

1F349
WATERMELON

1F34A
TANGERINE

1F34B
LEMON

1F34C
BANANA

1F34D
PINEAPPLE

1F34E
RED APPLE

1F34F
GREEN APPLE

1F350
PEAR

1F351
PEACH

1F352
CHERRIES

1F353
STRAWBERRY

1F354
HAMBURGER

1F355
SLICE OF PIZZA

1F356
MEAT ON BONE

1F357
POULTRY LEG

1F358
RICE CRACKER

1F359
RICE BALL

1F35A
COOKED RICE

1F35B
CURRY AND RICE

1F35C
STEAMING BOWL

1F35D
SPAGHETTI

1F35E
BREAD

1F35F
FRENCH FRIES

1F360
ROASTED SWEET POTATO

1F361
DANGO

1F362
ODEN

1F363
SUSHI

1F364
FRIED SHRIMP

1F365
FISH CAKE WITH SWIRL DESIGN

1F366
SOFT ICE CREAM

1F367
SHAVED ICE

1F368
ICE CREAM

1F369
DOUGHNUT

1F36A
COOKIE

1F36B
CHOCOLATE BAR

1F36C
CANDY

1F36D
LOLLIPOP

1F36E
CUSTARD

1F36F
HONEY POT

1F370
SHORTCAKE

1F371
BENTO BOX

1F372
POT OF FOOD

1F373
COOKING

1F374
FORK AND KNIFE

1F375
TEACUP WITHOUT HANDLE

1F376
SAKE BOTTLE AND CUP

1F377
WINE GLASS

1F378
COCKTAIL GLASS

1F379
TROPICAL DRINK

1F37A
BEER MUG

1F37B
CLINKING BEER MUGS

1F37C
BABY BOTTLE

1F37D
FORK AND KNIFE WITH PLATE

1F37E
BOTTLE WITH POPPING CORK

1F37F
POPCORN

1F380
RIBBON

1F381
WRAPPED PRESENT

1F382
BIRTHDAY CAKE

1F383
JACK-O-LANTERN

1F384
CHRISTMAS TREE

1F385
FATHER CHRISTMAS

1F386
FIREWORKS

1F387
FIREWORK SPARKLER

1F388
BALLOON

1F389
PARTY POPPER

1F38A
CONFETTI BALL

1F38B
TANABATA TREE

1F38C
CROSSED FLAGS

1F38D
PINE DECORATION

1F38E
JAPANESE DOLLS

1F38F
CARP STREAMER

1F390
WIND CHIME

1F391
MOON VIEWING CEREMONY

1F392
SCHOOL SATCHEL

1F393
GRADUATION CAP

1F394
HEART WITH TIP ON THE LEFT

1F395
BOUQUET OF FLOWERS

1F396
MILITARY MEDAL

1F397
REMINDER RIBBON

1F398
MUSICAL KEYBOARD WITH JACKS

1F399
STUDIO MICROPHONE

1F39A
LEVEL SLIDER

1F39B
CONTROL KNOBS

1F39C
BEAMED ASCENDING MUSICAL NOTES

1F39D
BEAMED DESCENDING MUSICAL NOTES

1F39E
FILM FRAMES

1F39F
ADMISSION TICKETS

1F3A0
CAROUSEL HORSE

1F3A1
FERRIS WHEEL

1F3A2
ROLLER COASTER

1F3A3
FISHING POLE AND FISH

1F3A4
MICROPHONE

1F3A5
MOVIE CAMERA

1F3A6
CINEMA

1F3A7
HEADPHONE

1F3A8
ARTIST PALETTE

1F3A9
TOP HAT

1F3AA
CIRCUS TENT

1F3AB
TICKET

1F3AC
CLAPPER BOARD

1F3AD
PERFORMING ARTS

1F3AE
VIDEO GAME

1F3AF
DIRECT HIT

1F3B0
SLOT MACHINE

1F3B1
BILLIARDS

1F3B2
GAME DIE

1F3B3
BOWLING

1F3B4
FLOWER PLAYING CARDS

1F3B5
MUSICAL NOTE

1F3B6
MULTIPLE MUSICAL NOTES

1F3B7
SAXOPHONE

1F3B8
GUITAR

1F3B9
MUSICAL KEYBOARD

1F3BA
TRUMPET

1F3BB
VIOLIN

1F3BC
MUSICAL SCORE

1F3BD
RUNNING SHIRT WITH SASH

1F3BE
TENNIS RACQUET AND BALL

1F3BF
SKI AND SKI BOOT

1F3C0
BASKETBALL AND HOOP

1F3C1
CHEQUERED FLAG

1F3C2
SNOWBOARDER

1F3C3
RUNNER

1F3C4
SURFER

1F3C5
SPORTS MEDAL

1F3C6
TROPHY

1F3C7
HORSE RACING

1F3C8
AMERICAN FOOTBALL

1F3C9
RUGBY FOOTBALL

1F3CA
SWIMMER

1F3CB
WEIGHT LIFTER

1F3CC
GOLFER

1F3CD
RACING MOTORCYCLE

1F3CE
RACING CAR

1F3CF
CRICKET BAT AND BALL

1F3D0
VOLLEYBALL

1F3D1
FIELD HOCKEY STICK AND BALL

1F3D2
ICE HOCKEY STICK AND PUCK

1F3D3
TABLE TENNIS PADDLE AND BALL

1F3D4
SNOW CAPPED MOUNTAIN

1F3D5
CAMPING

1F3D6
BEACH WITH UMBRELLA

1F3D7
BUILDING CONSTRUCTION

1F3D8
HOUSE BUILDINGS

1F3D9
CITYSCAPE

1F3DA
DERELICT HOUSE BUILDING

1F3DB
CLASSICAL BUILDING

1F3DC
DESERT

1F3DD
DESERT ISLAND

1F3DE
NATIONAL PARK

1F3DF
STADIUM

1F3E0
HOUSE BUILDING

1F3E1
HOUSE WITH GARDEN

1F3E2
OFFICE BUILDING

1F3E3
JAPANESE POST OFFICE

1F3E4
EUROPEAN POST OFFICE

1F3E5
HOSPITAL

1F3E6
BANK

1F3E7
AUTOMATED TELLER MACHINE

1F3E8
HOTEL

1F3E9
LOVE HOTEL

1F3EA
CONVENIENCE STORE

1F3EB
SCHOOL

1F3EC
DEPARTMENT STORE

1F3ED
FACTORY

1F3EE
IZAKAYA LANTERN

1F3EF
JAPANESE CASTLE

1F3F0
EUROPEAN CASTLE

1F3F1
WHITE PENNANT

1F3F2
BLACK PENNANT

1F3F3
WAVING WHITE FLAG

1F3F4
WAVING BLACK FLAG

1F3F5
ROSETTE

1F3F6
BLACK ROSETTE

1F3F7
LABEL

1F3F8
BADMINTON RACQUET AND SHUTTLECOCK

1F3F9
BOW AND ARROW

1F3FA
AMPHORA

1F3FB
EMOJI MODIFIER FITZPATRICK TYPE-1-2

1F3FC
EMOJI MODIFIER FITZPATRICK TYPE-3

1F3FD
EMOJI MODIFIER FITZPATRICK TYPE-4

1F3FE
EMOJI MODIFIER FITZPATRICK TYPE-5

1F3FF
EMOJI MODIFIER FITZPATRICK TYPE-6

1F400
RAT

1F401
MOUSE

1F402
OX

1F403
WATER BUFFALO

1F404
COW

1F405
TIGER

1F406
LEOPARD

1F407
RABBIT

1F408
CAT

1F409
DRAGON

1F40A
CROCODILE

1F40B
WHALE

1F40C
SNAIL

1F40D
SNAKE

1F40E
HORSE

1F40F
RAM

1F410
GOAT

1F411
SHEEP

1F412
MONKEY

1F413
ROOSTER

1F414
CHICKEN

1F415
DOG

1F416
PIG

1F417
BOAR

1F418
ELEPHANT

1F419
OCTOPUS

1F41A
SPIRAL SHELL

1F41B
BUG

1F41C
ANT

1F41D
HONEYBEE

1F41E
LADY BEETLE

1F41F
FISH

1F420
TROPICAL FISH

1F421
BLOWFISH

1F422
TURTLE

1F423
HATCHING CHICK

1F424
BABY CHICK

1F425
FRONT-FACING BABY CHICK

1F426
BIRD

1F427
PENGUIN

1F428
KOALA

1F429
POODLE

1F42A
DROMEDARY CAMEL

1F42B
BACTRIAN CAMEL

1F42C
DOLPHIN

1F42D
MOUSE FACE

1F42E
COW FACE

1F42F
TIGER FACE

1F430
RABBIT FACE

1F431
CAT FACE

1F432
DRAGON FACE

1F433
SPOUTING WHALE

1F434
HORSE FACE

1F435
MONKEY FACE

1F436
DOG FACE

1F437
PIG FACE

1F438
FROG FACE

1F439
HAMSTER FACE

1F43A
WOLF FACE

1F43B
BEAR FACE

1F43C
PANDA FACE

1F43D
PIG NOSE

1F43E
PAW PRINTS

1F43F
CHIPMUNK

1F440
EYES

1F441
EYE

1F442
EAR

1F443
NOSE

1F444
MOUTH

1F445
TONGUE

1F446
WHITE UP POINTING BACKHAND INDEX

1F447
WHITE DOWN POINTING BACKHAND INDEX

1F448
WHITE LEFT POINTING BACKHAND INDEX

1F449
WHITE RIGHT POINTING BACKHAND INDEX

1F44A
FISTED HAND SIGN

1F44B
WAVING HAND SIGN

1F44C
OK HAND SIGN

1F44D
THUMBS UP SIGN

1F44E
THUMBS DOWN SIGN

1F44F
CLAPPING HANDS SIGN

1F450
OPEN HANDS SIGN

1F451
CROWN

1F452
WOMANS HAT

1F453
EYEGLASSES

1F454
NECKTIE

1F455
T-SHIRT

1F456
JEANS

1F457
DRESS

1F458
KIMONO

1F459
BIKINI

1F45A
WOMANS CLOTHES

1F45B
PURSE

1F45C
HANDBAG

1F45D
POUCH

1F45E
MANS SHOE

1F45F
ATHLETIC SHOE

1F460
HIGH-HEELED SHOE

1F461
WOMANS SANDAL

1F462
WOMANS BOOTS

1F463
FOOTPRINTS

1F464
BUST IN SILHOUETTE

1F465
BUSTS IN SILHOUETTE

1F466
BOY

1F467
GIRL

1F468
MAN

1F469
WOMAN

1F46A
FAMILY

1F46B
MAN AND WOMAN HOLDING HANDS

1F46C
TWO MEN HOLDING HANDS

1F46D
TWO WOMEN HOLDING HANDS

1F46E
POLICE OFFICER

1F46F
WOMAN WITH BUNNY EARS

1F470
BRIDE WITH VEIL

1F471
PERSON WITH BLOND HAIR

1F472
MAN WITH GUA PI MAO

1F473
MAN WITH TURBAN

1F474
OLDER MAN

1F475
OLDER WOMAN

1F476
BABY

1F477
CONSTRUCTION WORKER

1F478
PRINCESS

1F479
JAPANESE OGRE

1F47A
JAPANESE GOBLIN

1F47B
GHOST

1F47C
BABY ANGEL

1F47D
EXTRATERRESTRIAL ALIEN

1F47E
ALIEN MONSTER

1F47F
IMP

1F480
SKULL

1F481
INFORMATION DESK PERSON

1F482
GUARDSMAN

1F483
DANCER

1F484
LIPSTICK

1F485
NAIL POLISH

1F486
FACE MASSAGE

1F487
HAIRCUT

1F488
BARBER POLE

1F489
SYRINGE

1F48A
PILL

1F48B
KISS MARK

1F48C
LOVE LETTER

1F48D
RING

1F48E
GEM STONE

1F48F
KISS

1F490
BOUQUET

1F491
COUPLE WITH HEART

1F492
WEDDING

1F493
BEATING HEART

1F494
BROKEN HEART

1F495
TWO HEARTS

1F496
SPARKLING HEART

1F497
GROWING HEART

1F498
HEART WITH ARROW

1F499
BLUE HEART

1F49A
GREEN HEART

1F49B
YELLOW HEART

1F49C
PURPLE HEART

1F49D
HEART WITH RIBBON

1F49E
REVOLVING HEARTS

1F49F
HEART DECORATION

1F4A0
DIAMOND SHAPE WITH A DOT INSIDE

1F4A1
ELECTRIC LIGHT BULB

1F4A2
ANGER SYMBOL

1F4A3
BOMB

1F4A4
SLEEPING SYMBOL

1F4A5
COLLISION SYMBOL

1F4A6
SPLASHING SWEAT SYMBOL

1F4A7
DROPLET

1F4A8
DASH SYMBOL

1F4A9
PILE OF POO

1F4AA
FLEXED BICEPS

1F4AB
DIZZY SYMBOL

1F4AC
SPEECH BALLOON

1F4AD
THOUGHT BALLOON

1F4AE
WHITE FLOWER

1F4AF
HUNDRED POINTS SYMBOL

1F4B0
MONEY BAG

1F4B1
CURRENCY EXCHANGE

1F4B2
HEAVY DOLLAR SIGN

1F4B3
CREDIT CARD

1F4B4
BANKNOTE WITH YEN SIGN

1F4B5
BANKNOTE WITH DOLLAR SIGN

1F4B6
BANKNOTE WITH EURO SIGN

1F4B7
BANKNOTE WITH POUND SIGN

1F4B8
MONEY WITH WINGS

1F4B9
CHART WITH UPWARDS TREND AND YEN SIGN

1F4BA
SEAT

1F4BB
PERSONAL COMPUTER

1F4BC
BRIEFCASE

1F4BD
MINIDISC

1F4BE
FLOPPY DISK

1F4BF
OPTICAL DISC

1F4C0
DVD

1F4C1
FILE FOLDER

1F4C2
OPEN FILE FOLDER

1F4C3
PAGE WITH CURL

1F4C4
PAGE FACING UP

1F4C5
CALENDAR

1F4C6
TEAR-OFF CALENDAR

1F4C7
CARD INDEX

1F4C8
CHART WITH UPWARDS TREND

1F4C9
CHART WITH DOWNWARDS TREND

1F4CA
BAR CHART

1F4CB
CLIPBOARD

1F4CC
PUSHPIN

1F4CD
ROUND PUSHPIN

1F4CE
PAPERCLIP

1F4CF
STRAIGHT RULER

1F4D0
TRIANGULAR RULER

1F4D1
BOOKMARK TABS

1F4D2
LEDGER

1F4D3
NOTEBOOK

1F4D4
NOTEBOOK WITH DECORATIVE COVER

1F4D5
CLOSED BOOK

1F4D6
OPEN BOOK

1F4D7
GREEN BOOK

1F4D8
BLUE BOOK

1F4D9
ORANGE BOOK

1F4DA
BOOKS

1F4DB
NAME BADGE

1F4DC
SCROLL

1F4DD
MEMO

1F4DE
TELEPHONE RECEIVER

1F4DF
PAGER

1F4E0
FAX MACHINE

1F4E1
SATELLITE ANTENNA

1F4E2
PUBLIC ADDRESS LOUDSPEAKER

1F4E3
CHEERING MEGAPHONE

1F4E4
OUTBOX TRAY

1F4E5
INBOX TRAY

1F4E6
PACKAGE

1F4E7
E-MAIL SYMBOL

1F4E8
INCOMING ENVELOPE

1F4E9
ENVELOPE WITH DOWNWARDS ARROW ABOVE

1F4EA
CLOSED MAILBOX WITH LOWERED FLAG

1F4EB
CLOSED MAILBOX WITH RAISED FLAG

1F4EC
OPEN MAILBOX WITH RAISED FLAG

1F4ED
OPEN MAILBOX WITH LOWERED FLAG

1F4EE
POSTBOX

1F4EF
POSTAL HORN

1F4F0
NEWSPAPER

1F4F1
MOBILE PHONE

1F4F2
MOBILE PHONE WITH RIGHTWARDS ARROW AT LEFT

1F4F3
VIBRATION MODE

1F4F4
MOBILE PHONE OFF

1F4F5
NO MOBILE PHONES

1F4F6
ANTENNA WITH BARS

1F4F7
CAMERA

1F4F8
CAMERA WITH FLASH

1F4F9
VIDEO CAMERA

1F4FA
TELEVISION

1F4FB
RADIO

1F4FC
VIDEOCASSETTE

1F4FD
FILM PROJECTOR

1F4FE
PORTABLE STEREO

1F4FF
PRAYER BEADS

1F500
TWISTED RIGHTWARDS ARROWS

1F501
CLOCKWISE RIGHTWARDS AND LEFTWARDS OPEN CIRCLE ARROWS

1F502
CLOCKWISE RIGHTWARDS AND LEFTWARDS OPEN CIRCLE ARROWS WITH CIRCLED ONE OVERLAY

1F503
CLOCKWISE DOWNWARDS AND UPWARDS OPEN CIRCLE ARROWS

1F504
ANTICLOCKWISE DOWNWARDS AND UPWARDS OPEN CIRCLE ARROWS

1F505
LOW BRIGHTNESS SYMBOL

1F506
HIGH BRIGHTNESS SYMBOL

1F507
SPEAKER WITH CANCELLATION STROKE

1F508
SPEAKER

1F509
SPEAKER WITH ONE SOUND WAVE

1F50A
SPEAKER WITH THREE SOUND WAVES

1F50B
BATTERY

1F50C
ELECTRIC PLUG

1F50D
LEFT-POINTING MAGNIFYING GLASS

1F50E
RIGHT-POINTING MAGNIFYING GLASS

1F50F
LOCK WITH INK PEN

1F510
CLOSED LOCK WITH KEY

1F511
KEY

1F512
LOCK

1F513
OPEN LOCK

1F514
BELL

1F515
BELL WITH CANCELLATION STROKE

1F516
BOOKMARK

1F517
LINK SYMBOL

1F518
RADIO BUTTON

1F519
BACK WITH LEFTWARDS ARROW ABOVE

1F51A
END WITH LEFTWARDS ARROW ABOVE

1F51B
ON WITH EXCLAMATION MARK WITH LEFT RIGHT ARROW ABOVE

1F51C
SOON WITH RIGHTWARDS ARROW ABOVE

1F51D
TOP WITH UPWARDS ARROW ABOVE

1F51E
NO ONE UNDER EIGHTEEN SYMBOL

1F51F
KEYCAP TEN

1F520
INPUT SYMBOL FOR LATIN CAPITAL LETTERS

1F521
INPUT SYMBOL FOR LATIN SMALL LETTERS

1F522
INPUT SYMBOL FOR NUMBERS

1F523
INPUT SYMBOL FOR SYMBOLS

1F524
INPUT SYMBOL FOR LATIN LETTERS

1F525
FIRE

1F526
ELECTRIC TORCH

1F527
WRENCH

1F528
HAMMER

1F529
NUT AND BOLT

1F52A
HOCHO

1F52B
PISTOL

1F52C
MICROSCOPE

1F52D
TELESCOPE

1F52E
CRYSTAL BALL

1F52F
SIX POINTED STAR WITH MIDDLE DOT

1F530
JAPANESE SYMBOL FOR BEGINNER

1F531
TRIDENT EMBLEM

1F532
BLACK SQUARE BUTTON

1F533
WHITE SQUARE BUTTON

1F534
LARGE RED CIRCLE

1F535
LARGE BLUE CIRCLE

1F536
LARGE ORANGE DIAMOND

1F537
LARGE BLUE DIAMOND

1F538
SMALL ORANGE DIAMOND

1F539
SMALL BLUE DIAMOND

1F53A
UP-POINTING RED TRIANGLE

1F53B
DOWN-POINTING RED TRIANGLE

1F53C
UP-POINTING SMALL RED TRIANGLE

1F53D
DOWN-POINTING SMALL RED TRIANGLE

1F53E
LOWER RIGHT SHADOWED WHITE CIRCLE

1F53F
UPPER RIGHT SHADOWED WHITE CIRCLE

1F540
CIRCLED CROSS POMMEE

1F541
CROSS POMMEE WITH HALF-CIRCLE BELOW

1F542
CROSS POMMEE

1F543
NOTCHED LEFT SEMICIRCLE WITH THREE DOTS

1F544
NOTCHED RIGHT SEMICIRCLE WITH THREE DOTS

1F545
SYMBOL FOR MARKS CHAPTER

1F546
WHITE LATIN CROSS

1F547
HEAVY LATIN CROSS

1F548
CELTIC CROSS

1F549
OM SYMBOL

1F54A
DOVE OF PEACE

1F54B
KAABA

1F54C
MOSQUE

1F54D
SYNAGOGUE

1F54E
MENORAH WITH NINE BRANCHES

1F54F
BOWL OF HYGIEIA

1F550
CLOCK FACE ONE OCLOCK

1F551
CLOCK FACE TWO OCLOCK

1F552
CLOCK FACE THREE OCLOCK

1F553
CLOCK FACE FOUR OCLOCK

1F554
CLOCK FACE FIVE OCLOCK

1F555
CLOCK FACE SIX OCLOCK

1F556
CLOCK FACE SEVEN OCLOCK

1F557
CLOCK FACE EIGHT OCLOCK

1F558
CLOCK FACE NINE OCLOCK

1F559
CLOCK FACE TEN OCLOCK

1F55A
CLOCK FACE ELEVEN OCLOCK

1F55B
CLOCK FACE TWELVE OCLOCK

1F55C
CLOCK FACE ONE-THIRTY

1F55D
CLOCK FACE TWO-THIRTY

1F55E
CLOCK FACE THREE-THIRTY

1F55F
CLOCK FACE FOUR-THIRTY

1F560
CLOCK FACE FIVE-THIRTY

1F561
CLOCK FACE SIX-THIRTY

1F562
CLOCK FACE SEVEN-THIRTY

1F563
CLOCK FACE EIGHT-THIRTY

1F564
CLOCK FACE NINE-THIRTY

1F565
CLOCK FACE TEN-THIRTY

1F566
CLOCK FACE ELEVEN-THIRTY

1F567
CLOCK FACE TWELVE-THIRTY

1F568
RIGHT SPEAKER

1F569
RIGHT SPEAKER WITH ONE SOUND WAVE

1F56A
RIGHT SPEAKER WITH THREE SOUND WAVES

1F56B
BULLHORN

1F56C
BULLHORN WITH SOUND WAVES

1F56D
RINGING BELL

1F56E
BOOK

1F56F
CANDLE

1F570
MANTELPIECE CLOCK

1F571
BLACK SKULL AND CROSSBONES

1F572
NO PIRACY

1F573
HOLE

1F574
MAN IN BUSINESS SUIT LEVITATING

1F575
SLEUTH OR SPY

1F576
DARK SUNGLASSES

1F577
SPIDER

1F578
SPIDER WEB

1F579
JOYSTICK

1F57A
MAN DANCING

1F57B
LEFT HAND TELEPHONE RECEIVER

1F57C
TELEPHONE RECEIVER WITH PAGE

1F57D
RIGHT HAND TELEPHONE RECEIVER

1F57E
WHITE TOUCHTONE TELEPHONE

1F57F
BLACK TOUCHTONE TELEPHONE

1F580
TELEPHONE ON TOP OF MODEM

1F581
CLAMSHELL MOBILE PHONE

1F582
BACK OF ENVELOPE

1F583
STAMPED ENVELOPE

1F584
ENVELOPE WITH LIGHTNING

1F585
FLYING ENVELOPE

1F586
PEN OVER STAMPED ENVELOPE

1F587
LINKED PAPERCLIPS

1F588
BLACK PUSHPIN

1F589
LOWER LEFT PENCIL

1F58A
LOWER LEFT BALLPOINT PEN

1F58B
LOWER LEFT FOUNTAIN PEN

1F58C
LOWER LEFT PAINTBRUSH

1F58D
LOWER LEFT CRAYON

1F58E
LEFT WRITING HAND

1F58F
TURNED OK HAND SIGN

1F590
RAISED HAND WITH FINGERS SPLAYED

1F591
REVERSED RAISED HAND WITH FINGERS SPLAYED

1F592
REVERSED THUMBS UP SIGN

1F593
REVERSED THUMBS DOWN SIGN

1F594
REVERSED VICTORY HAND

1F595
REVERSED HAND WITH MIDDLE FINGER EXTENDED

1F596
RAISED HAND WITH PART BETWEEN MIDDLE AND RING FINGERS

1F597
WHITE DOWN POINTING LEFT HAND INDEX

1F598
SIDEWAYS WHITE LEFT POINTING INDEX

1F599
SIDEWAYS WHITE RIGHT POINTING INDEX

1F59A
SIDEWAYS BLACK LEFT POINTING INDEX

1F59B
SIDEWAYS BLACK RIGHT POINTING INDEX

1F59C
BLACK LEFT POINTING BACKHAND INDEX

1F59D
BLACK RIGHT POINTING BACKHAND INDEX

1F59E
SIDEWAYS WHITE UP POINTING INDEX

1F59F
SIDEWAYS WHITE DOWN POINTING INDEX

1F5A0
SIDEWAYS BLACK UP POINTING INDEX

1F5A1
SIDEWAYS BLACK DOWN POINTING INDEX

1F5A2
BLACK UP POINTING BACKHAND INDEX

1F5A3
BLACK DOWN POINTING BACKHAND INDEX

1F5A4
BLACK HEART

1F5A5
DESKTOP COMPUTER

1F5A6
KEYBOARD AND MOUSE

1F5A7
THREE NETWORKED COMPUTERS

1F5A8
PRINTER

1F5A9
POCKET CALCULATOR

1F5AA
BLACK HARD SHELL FLOPPY DISK

1F5AB
WHITE HARD SHELL FLOPPY DISK

1F5AC
SOFT SHELL FLOPPY DISK

1F5AD
TAPE CARTRIDGE

1F5AE
WIRED KEYBOARD

1F5AF
ONE BUTTON MOUSE

1F5B0
TWO BUTTON MOUSE

1F5B1
THREE BUTTON MOUSE

1F5B2
TRACKBALL

1F5B3
OLD PERSONAL COMPUTER

1F5B4
HARD DISK

1F5B5
SCREEN

1F5B6
PRINTER ICON

1F5B7
FAX ICON

1F5B8
OPTICAL DISC ICON

1F5B9
DOCUMENT WITH TEXT

1F5BA
DOCUMENT WITH TEXT AND PICTURE

1F5BB
DOCUMENT WITH PICTURE

1F5BC
FRAME WITH PICTURE

1F5BD
FRAME WITH TILES

1F5BE
FRAME WITH AN X

1F5BF
BLACK FOLDER

1F5C0
FOLDER

1F5C1
OPEN FOLDER

1F5C2
CARD INDEX DIVIDERS

1F5C3
CARD FILE BOX

1F5C4
FILE CABINET

1F5C5
EMPTY NOTE

1F5C6
EMPTY NOTE PAGE

1F5C7
EMPTY NOTE PAD

1F5C8
NOTE

1F5C9
NOTE PAGE

1F5CA
NOTE PAD

1F5CB
EMPTY DOCUMENT

1F5CC
EMPTY PAGE

1F5CD
EMPTY PAGES

1F5CE
DOCUMENT

1F5CF
PAGE

1F5D0
PAGES

1F5D1
WASTEBASKET

1F5D2
SPIRAL NOTE PAD

1F5D3
SPIRAL CALENDAR PAD

1F5D4
DESKTOP WINDOW

1F5D5
MINIMIZE

1F5D6
MAXIMIZE

1F5D7
OVERLAP

1F5D8
CLOCKWISE RIGHT AND LEFT SEMICIRCLE ARROWS

1F5D9
CANCELLATION X

1F5DA
INCREASE FONT SIZE SYMBOL

1F5DB
DECREASE FONT SIZE SYMBOL

1F5DC
COMPRESSION

1F5DD
OLD KEY

1F5DE
ROLLED-UP NEWSPAPER

1F5DF
PAGE WITH CIRCLED TEXT

1F5E0
STOCK CHART

1F5E1
DAGGER KNIFE

1F5E2
LIPS

1F5E3
SPEAKING HEAD IN SILHOUETTE

1F5E4
THREE RAYS ABOVE

1F5E5
THREE RAYS BELOW

1F5E6
THREE RAYS LEFT

1F5E7
THREE RAYS RIGHT

1F5E8
LEFT SPEECH BUBBLE

1F5E9
RIGHT SPEECH BUBBLE

1F5EA
TWO SPEECH BUBBLES

1F5EB
THREE SPEECH BUBBLES

1F5EC
LEFT THOUGHT BUBBLE

1F5ED
RIGHT THOUGHT BUBBLE

1F5EE
LEFT ANGER BUBBLE

1F5EF
RIGHT ANGER BUBBLE

1F5F0
MOOD BUBBLE

1F5F1
LIGHTNING MOOD BUBBLE

1F5F2
LIGHTNING MOOD

1F5F3
BALLOT BOX WITH BALLOT

1F5F4
BALLOT SCRIPT X

1F5F5
BALLOT BOX WITH SCRIPT X

1F5F6
BALLOT BOLD SCRIPT X

1F5F7
BALLOT BOX WITH BOLD SCRIPT X

1F5F8
LIGHT CHECK MARK

1F5F9
BALLOT BOX WITH BOLD CHECK

1F5FA
WORLD MAP

1F5FB
MOUNT FUJI

1F5FC
TOKYO TOWER

1F5FD
STATUE OF LIBERTY

1F5FE
SILHOUETTE OF JAPAN

1F5FF
MOYAI

1F600
GRINNING FACE

1F601
GRINNING FACE WITH SMILING EYES

1F602
FACE WITH TEARS OF JOY

1F603
SMILING FACE WITH OPEN MOUTH

1F604
SMILING FACE WITH OPEN MOUTH AND SMILING EYES

1F605
SMILING FACE WITH OPEN MOUTH AND COLD SWEAT

1F606
SMILING FACE WITH OPEN MOUTH AND TIGHTLY-CLOSED EYES

1F607
SMILING FACE WITH HALO

1F608
SMILING FACE WITH HORNS

1F609
WINKING FACE

1F60A
SMILING FACE WITH SMILING EYES

1F60B
FACE SAVOURING DELICIOUS FOOD

1F60C
RELIEVED FACE

1F60D
SMILING FACE WITH HEART-SHAPED EYES

1F60E
SMILING FACE WITH SUNGLASSES

1F60F
SMIRKING FACE

1F610
NEUTRAL FACE

1F611
EXPRESSIONLESS FACE

1F612
UNAMUSED FACE

1F613
FACE WITH COLD SWEAT

1F614
PENSIVE FACE

1F615
CONFUSED FACE

1F616
CONFOUNDED FACE

1F617
KISSING FACE

1F618
FACE THROWING A KISS

1F619
KISSING FACE WITH SMILING EYES

1F61A
KISSING FACE WITH CLOSED EYES

1F61B
FACE WITH STUCK-OUT TONGUE

1F61C
FACE WITH STUCK-OUT TONGUE AND WINKING EYE

1F61D
FACE WITH STUCK-OUT TONGUE AND TIGHTLY-CLOSED EYES

1F61E
DISAPPOINTED FACE

1F61F
WORRIED FACE

1F620
ANGRY FACE

1F621
POUTING FACE

1F622
CRYING FACE

1F623
PERSEVERING FACE

1F624
FACE WITH LOOK OF TRIUMPH

1F625
DISAPPOINTED BUT RELIEVED FACE

1F626
FROWNING FACE WITH OPEN MOUTH

1F627
ANGUISHED FACE

1F628
FEARFUL FACE

1F629
WEARY FACE

1F62A
SLEEPY FACE

1F62B
TIRED FACE

1F62C
GRIMACING FACE

1F62D
LOUDLY CRYING FACE

1F62E
FACE WITH OPEN MOUTH

1F62F
HUSHED FACE

1F630
FACE WITH OPEN MOUTH AND COLD SWEAT

1F631
FACE SCREAMING IN FEAR

1F632
ASTONISHED FACE

1F633
FLUSHED FACE

1F634
SLEEPING FACE

1F635
DIZZY FACE

1F636
FACE WITHOUT MOUTH

1F637
FACE WITH MEDICAL MASK

1F638
GRINNING CAT FACE WITH SMILING EYES

1F639
CAT FACE WITH TEARS OF JOY

1F63A
SMILING CAT FACE WITH OPEN MOUTH

1F63B
SMILING CAT FACE WITH HEART-SHAPED EYES

1F63C
CAT FACE WITH WRY SMILE

1F63D
KISSING CAT FACE WITH CLOSED EYES

1F63E
POUTING CAT FACE

1F63F
CRYING CAT FACE

1F640
WEARY CAT FACE

1F641
SLIGHTLY FROWNING FACE

1F642
SLIGHTLY SMILING FACE

1F643
UPSIDE-DOWN FACE

1F644
FACE WITH ROLLING EYES

1F645
FACE WITH NO GOOD GESTURE

1F646
FACE WITH OK GESTURE

1F647
PERSON BOWING DEEPLY

1F648
SEE-NO-EVIL MONKEY

1F649
HEAR-NO-EVIL MONKEY

1F64A
SPEAK-NO-EVIL MONKEY

1F64B
HAPPY PERSON RAISING ONE HAND

1F64C
PERSON RAISING BOTH HANDS IN CELEBRATION

1F64D
PERSON FROWNING

1F64E
PERSON WITH POUTING FACE

1F64F
PERSON WITH FOLDED HANDS

1F650
NORTH WEST POINTING LEAF

1F651
SOUTH WEST POINTING LEAF

1F652
NORTH EAST POINTING LEAF

1F653
SOUTH EAST POINTING LEAF

1F654
TURNED NORTH WEST POINTING LEAF

1F655
TURNED SOUTH WEST POINTING LEAF

1F656
TURNED NORTH EAST POINTING LEAF

1F657
TURNED SOUTH EAST POINTING LEAF

1F658
NORTH WEST POINTING VINE LEAF

1F659
SOUTH WEST POINTING VINE LEAF

1F65A
NORTH EAST POINTING VINE LEAF

1F65B
SOUTH EAST POINTING VINE LEAF

1F65C
HEAVY NORTH WEST POINTING VINE LEAF

1F65D
HEAVY SOUTH WEST POINTING VINE LEAF

1F65E
HEAVY NORTH EAST POINTING VINE LEAF

1F65F
HEAVY SOUTH EAST POINTING VINE LEAF

1F660
NORTH WEST POINTING BUD

1F661
SOUTH WEST POINTING BUD

1F662
NORTH EAST POINTING BUD

1F663
SOUTH EAST POINTING BUD

1F664
HEAVY NORTH WEST POINTING BUD

1F665
HEAVY SOUTH WEST POINTING BUD

1F666
HEAVY NORTH EAST POINTING BUD

1F667
HEAVY SOUTH EAST POINTING BUD

1F668
HOLLOW QUILT SQUARE ORNAMENT

1F669
HOLLOW QUILT SQUARE ORNAMENT IN BLACK SQUARE

1F66A
SOLID QUILT SQUARE ORNAMENT

1F66B
SOLID QUILT SQUARE ORNAMENT IN BLACK SQUARE

1F66C
LEFTWARDS ROCKET

1F66D
UPWARDS ROCKET

1F66E
RIGHTWARDS ROCKET

1F66F
DOWNWARDS ROCKET

1F670
SCRIPT LIGATURE ET ORNAMENT

1F671
HEAVY SCRIPT LIGATURE ET ORNAMENT

1F672
LIGATURE OPEN ET ORNAMENT

1F673
HEAVY LIGATURE OPEN ET ORNAMENT

1F674
HEAVY AMPERSAND ORNAMENT

1F675
SWASH AMPERSAND ORNAMENT

1F676
SANS-SERIF HEAVY DOUBLE TURNED COMMA QUOTATION MARK ORNAMENT

1F677
SANS-SERIF HEAVY DOUBLE COMMA QUOTATION MARK ORNAMENT

1F678
SANS-SERIF HEAVY LOW DOUBLE COMMA QUOTATION MARK ORNAMENT

1F679
HEAVY INTERROBANG ORNAMENT

1F67A
SANS-SERIF INTERROBANG ORNAMENT

1F67B
HEAVY SANS-SERIF INTERROBANG ORNAMENT

1F67C
VERY HEAVY SOLIDUS

1F67D
VERY HEAVY REVERSE SOLIDUS

1F67E
CHECKER BOARD

1F67F
REVERSE CHECKER BOARD

1F680
ROCKET

1F681
HELICOPTER

1F682
STEAM LOCOMOTIVE

1F683
RAILWAY CAR

1F684
HIGH-SPEED TRAIN

1F685
HIGH-SPEED TRAIN WITH BULLET NOSE

1F686
TRAIN

1F687
METRO

1F688
LIGHT RAIL

1F689
STATION

1F68A
TRAM

1F68B
TRAM CAR

1F68C
BUS

1F68D
ONCOMING BUS

1F68E
TROLLEYBUS

1F68F
BUS STOP

1F690
MINIBUS

1F691
AMBULANCE

1F692
FIRE ENGINE

1F693
POLICE CAR

1F694
ONCOMING POLICE CAR

1F695
TAXI

1F696
ONCOMING TAXI

1F697
AUTOMOBILE

1F698
ONCOMING AUTOMOBILE

1F699
RECREATIONAL VEHICLE

1F69A
DELIVERY TRUCK

1F69B
ARTICULATED LORRY

1F69C
TRACTOR

1F69D
MONORAIL

1F69E
MOUNTAIN RAILWAY

1F69F
SUSPENSION RAILWAY

1F6A0
MOUNTAIN CABLEWAY

1F6A1
AERIAL TRAMWAY

1F6A2
SHIP

1F6A3
ROWBOAT

1F6A4
SPEEDBOAT

1F6A5
HORIZONTAL TRAFFIC LIGHT

1F6A6
VERTICAL TRAFFIC LIGHT

1F6A7
CONSTRUCTION SIGN

1F6A8
POLICE CARS REVOLVING LIGHT

1F6A9
TRIANGULAR FLAG ON POST

1F6AA
DOOR

1F6AB
NO ENTRY SIGN

1F6AC
SMOKING SYMBOL

1F6AD
NO SMOKING SYMBOL

1F6AE
PUT LITTER IN ITS PLACE SYMBOL

1F6AF
DO NOT LITTER SYMBOL

1F6B0
POTABLE WATER SYMBOL

1F6B1
NON-POTABLE WATER SYMBOL

1F6B2
BICYCLE

1F6B3
NO BICYCLES

1F6B4
BICYCLIST

1F6B5
MOUNTAIN BICYCLIST

1F6B6
PEDESTRIAN

1F6B7
NO PEDESTRIANS

1F6B8
CHILDREN CROSSING

1F6B9
MENS SYMBOL

1F6BA
WOMENS SYMBOL

1F6BB
RESTROOM

1F6BC
BABY SYMBOL

1F6BD
TOILET

1F6BE
WATER CLOSET

1F6BF
SHOWER

1F6C0
BATH

1F6C1
BATHTUB

1F6C2
PASSPORT CONTROL

1F6C3
CUSTOMS

1F6C4
BAGGAGE CLAIM

1F6C5
LEFT LUGGAGE

1F6C6
TRIANGLE WITH ROUNDED CORNERS

1F6C7
PROHIBITED SIGN

1F6C8
CIRCLED INFORMATION SOURCE

1F6C9
BOYS SYMBOL

1F6CA
GIRLS SYMBOL

1F6CB
COUCH AND LAMP

1F6CC
SLEEPING ACCOMMODATION

1F6CD
SHOPPING BAGS

1F6CE
BELLHOP BELL

1F6CF
BED

1F6D0
PLACE OF WORSHIP

1F6D1
OCTAGONAL SIGN

1F6D2
SHOPPING TROLLEY

1F6D3
STUPA

1F6D4
PAGODA

1F6D5
HINDU TEMPLE

1F6D6
HUT

1F6D7
ELEVATOR

1F6D8
LANDSLIDE

1F6DC
WIRELESS

1F6DD
PLAYGROUND SLIDE

1F6DE
WHEEL

1F6DF
RING BUOY

1F6E0
HAMMER AND WRENCH

1F6E1
SHIELD

1F6E2
OIL DRUM

1F6E3
MOTORWAY

1F6E4
RAILWAY TRACK

1F6E5
MOTOR BOAT

1F6E6
UP-POINTING MILITARY AIRPLANE

1F6E7
UP-POINTING AIRPLANE

1F6E8
UP-POINTING SMALL AIRPLANE

1F6E9
SMALL AIRPLANE

1F6EA
NORTHEAST-POINTING AIRPLANE

1F6EB
AIRPLANE DEPARTURE

1F6EC
AIRPLANE ARRIVING

1F6F0
SATELLITE

1F6F1
ONCOMING FIRE ENGINE

1F6F2
DIESEL LOCOMOTIVE

1F6F3
PASSENGER SHIP

1F6F4
SCOOTER

1F6F5
MOTOR SCOOTER

1F6F6
CANOE

1F6F7
SLED

1F6F8
FLYING SAUCER

1F6F9
SKATEBOARD

1F6FA
AUTO RICKSHAW

1F6FB
PICKUP TRUCK

1F6FC
ROLLER SKATE

1F700
ALCHEMICAL SYMBOL FOR QUINTESSENCE

1F701
ALCHEMICAL SYMBOL FOR AIR

1F702
ALCHEMICAL SYMBOL FOR FIRE

1F703
ALCHEMICAL SYMBOL FOR EARTH

1F704
ALCHEMICAL SYMBOL FOR WATER

1F705
ALCHEMICAL SYMBOL FOR AQUAFORTIS

1F706
ALCHEMICAL SYMBOL FOR AQUA REGIA

1F707
ALCHEMICAL SYMBOL FOR AQUA REGIA-2

1F708
ALCHEMICAL SYMBOL FOR AQUA VITAE

1F709
ALCHEMICAL SYMBOL FOR AQUA VITAE-2

1F70A
ALCHEMICAL SYMBOL FOR VINEGAR

1F70B
ALCHEMICAL SYMBOL FOR VINEGAR-2

1F70C
ALCHEMICAL SYMBOL FOR VINEGAR-3

1F70D
ALCHEMICAL SYMBOL FOR SULFUR

1F70E
ALCHEMICAL SYMBOL FOR PHILOSOPHERS SULFUR

1F70F
ALCHEMICAL SYMBOL FOR BLACK SULFUR

1F710
ALCHEMICAL SYMBOL FOR MERCURY SUBLIMATE

1F711
ALCHEMICAL SYMBOL FOR MERCURY SUBLIMATE-2

1F712
ALCHEMICAL SYMBOL FOR MERCURY SUBLIMATE-3

1F713
ALCHEMICAL SYMBOL FOR CINNABAR

1F714
ALCHEMICAL SYMBOL FOR SALT

1F715
ALCHEMICAL SYMBOL FOR NITRE

1F716
ALCHEMICAL SYMBOL FOR VITRIOL

1F717
ALCHEMICAL SYMBOL FOR VITRIOL-2

1F718
ALCHEMICAL SYMBOL FOR ROCK SALT

1F719
ALCHEMICAL SYMBOL FOR ROCK SALT-2

1F71A
ALCHEMICAL SYMBOL FOR GOLD

1F71B
ALCHEMICAL SYMBOL FOR SILVER

1F71C
ALCHEMICAL SYMBOL FOR IRON ORE

1F71D
ALCHEMICAL SYMBOL FOR IRON ORE-2

1F71E
ALCHEMICAL SYMBOL FOR CROCUS OF IRON

1F71F
ALCHEMICAL SYMBOL FOR REGULUS OF IRON

1F720
ALCHEMICAL SYMBOL FOR COPPER ORE

1F721
ALCHEMICAL SYMBOL FOR IRON-COPPER ORE

1F722
ALCHEMICAL SYMBOL FOR SUBLIMATE OF COPPER

1F723
ALCHEMICAL SYMBOL FOR CROCUS OF COPPER

1F724
ALCHEMICAL SYMBOL FOR CROCUS OF COPPER-2

1F725
ALCHEMICAL SYMBOL FOR COPPER ANTIMONIATE

1F726
ALCHEMICAL SYMBOL FOR SALT OF COPPER ANTIMONIATE

1F727
ALCHEMICAL SYMBOL FOR SUBLIMATE OF SALT OF COPPER

1F728
ALCHEMICAL SYMBOL FOR VERDIGRIS

1F729
ALCHEMICAL SYMBOL FOR TIN ORE

1F72A
ALCHEMICAL SYMBOL FOR LEAD ORE

1F72B
ALCHEMICAL SYMBOL FOR ANTIMONY ORE

1F72C
ALCHEMICAL SYMBOL FOR SUBLIMATE OF ANTIMONY

1F72D
ALCHEMICAL SYMBOL FOR SALT OF ANTIMONY

1F72E
ALCHEMICAL SYMBOL FOR SUBLIMATE OF SALT OF ANTIMONY

1F72F
ALCHEMICAL SYMBOL FOR VINEGAR OF ANTIMONY

1F730
ALCHEMICAL SYMBOL FOR REGULUS OF ANTIMONY

1F731
ALCHEMICAL SYMBOL FOR REGULUS OF ANTIMONY-2

1F732
ALCHEMICAL SYMBOL FOR REGULUS

1F733
ALCHEMICAL SYMBOL FOR REGULUS-2

1F734
ALCHEMICAL SYMBOL FOR REGULUS-3

1F735
ALCHEMICAL SYMBOL FOR REGULUS-4

1F736
ALCHEMICAL SYMBOL FOR ALKALI

1F737
ALCHEMICAL SYMBOL FOR ALKALI-2

1F738
ALCHEMICAL SYMBOL FOR MARCASITE

1F739
ALCHEMICAL SYMBOL FOR SAL-AMMONIAC

1F73A
ALCHEMICAL SYMBOL FOR ARSENIC

1F73B
ALCHEMICAL SYMBOL FOR REALGAR

1F73C
ALCHEMICAL SYMBOL FOR REALGAR-2

1F73D
ALCHEMICAL SYMBOL FOR AURIPIGMENT

1F73E
ALCHEMICAL SYMBOL FOR BISMUTH ORE

1F73F
ALCHEMICAL SYMBOL FOR TARTAR

1F740
ALCHEMICAL SYMBOL FOR TARTAR-2

1F741
ALCHEMICAL SYMBOL FOR QUICK LIME

1F742
ALCHEMICAL SYMBOL FOR BORAX

1F743
ALCHEMICAL SYMBOL FOR BORAX-2

1F744
ALCHEMICAL SYMBOL FOR BORAX-3

1F745
ALCHEMICAL SYMBOL FOR ALUM

1F746
ALCHEMICAL SYMBOL FOR OIL

1F747
ALCHEMICAL SYMBOL FOR SPIRIT

1F748
ALCHEMICAL SYMBOL FOR TINCTURE

1F749
ALCHEMICAL SYMBOL FOR GUM

1F74A
ALCHEMICAL SYMBOL FOR WAX

1F74B
ALCHEMICAL SYMBOL FOR POWDER

1F74C
ALCHEMICAL SYMBOL FOR CALX

1F74D
ALCHEMICAL SYMBOL FOR TUTTY

1F74E
ALCHEMICAL SYMBOL FOR CAPUT MORTUUM

1F74F
ALCHEMICAL SYMBOL FOR SCEPTER OF JOVE

1F750
ALCHEMICAL SYMBOL FOR CADUCEUS

1F751
ALCHEMICAL SYMBOL FOR TRIDENT

1F752
ALCHEMICAL SYMBOL FOR STARRED TRIDENT

1F753
ALCHEMICAL SYMBOL FOR LODESTONE

1F754
ALCHEMICAL SYMBOL FOR SOAP

1F755
ALCHEMICAL SYMBOL FOR URINE

1F756
ALCHEMICAL SYMBOL FOR HORSE DUNG

1F757
ALCHEMICAL SYMBOL FOR ASHES

1F758
ALCHEMICAL SYMBOL FOR POT ASHES

1F759
ALCHEMICAL SYMBOL FOR BRICK

1F75A
ALCHEMICAL SYMBOL FOR POWDERED BRICK

1F75B
ALCHEMICAL SYMBOL FOR AMALGAM

1F75C
ALCHEMICAL SYMBOL FOR STRATUM SUPER STRATUM

1F75D
ALCHEMICAL SYMBOL FOR STRATUM SUPER STRATUM-2

1F75E
ALCHEMICAL SYMBOL FOR SUBLIMATION

1F75F
ALCHEMICAL SYMBOL FOR PRECIPITATE

1F760
ALCHEMICAL SYMBOL FOR DISTILL

1F761
ALCHEMICAL SYMBOL FOR DISSOLVE

1F762
ALCHEMICAL SYMBOL FOR DISSOLVE-2

1F763
ALCHEMICAL SYMBOL FOR PURIFY

1F764
ALCHEMICAL SYMBOL FOR PUTREFACTION

1F765
ALCHEMICAL SYMBOL FOR CRUCIBLE

1F766
ALCHEMICAL SYMBOL FOR CRUCIBLE-2

1F767
ALCHEMICAL SYMBOL FOR CRUCIBLE-3

1F768
ALCHEMICAL SYMBOL FOR CRUCIBLE-4

1F769
ALCHEMICAL SYMBOL FOR CRUCIBLE-5

1F76A
ALCHEMICAL SYMBOL FOR ALEMBIC

1F76B
ALCHEMICAL SYMBOL FOR BATH OF MARY

1F76C
ALCHEMICAL SYMBOL FOR BATH OF VAPOURS

1F76D
ALCHEMICAL SYMBOL FOR RETORT

1F76E
ALCHEMICAL SYMBOL FOR HOUR

1F76F
ALCHEMICAL SYMBOL FOR NIGHT

1F770
ALCHEMICAL SYMBOL FOR DAY-NIGHT

1F771
ALCHEMICAL SYMBOL FOR MONTH

1F772
ALCHEMICAL SYMBOL FOR HALF DRAM

1F773
ALCHEMICAL SYMBOL FOR HALF OUNCE

1F774
LOT OF FORTUNE

1F775
OCCULTATION

1F776
LUNAR ECLIPSE

1F777
VESTA FORM TWO

1F778
ASTRAEA FORM TWO

1F779
HYGIEA FORM TWO

1F77A
PARTHENOPE FORM TWO

1F77B
HAUMEA

1F77C
MAKEMAKE

1F77D
GONGGONG

1F77E
QUAOAR

1F77F
ORCUS

1F780
BLACK LEFT-POINTING ISOSCELES RIGHT TRIANGLE

1F781
BLACK UP-POINTING ISOSCELES RIGHT TRIANGLE

1F782
BLACK RIGHT-POINTING ISOSCELES RIGHT TRIANGLE

1F783
BLACK DOWN-POINTING ISOSCELES RIGHT TRIANGLE

1F784
BLACK SLIGHTLY SMALL CIRCLE

1F785
MEDIUM BOLD WHITE CIRCLE

1F786
BOLD WHITE CIRCLE

1F787
HEAVY WHITE CIRCLE

1F788
VERY HEAVY WHITE CIRCLE

1F789
EXTREMELY HEAVY WHITE CIRCLE

1F78A
WHITE CIRCLE CONTAINING BLACK SMALL CIRCLE

1F78B
ROUND TARGET

1F78C
BLACK TINY SQUARE

1F78D
BLACK SLIGHTLY SMALL SQUARE

1F78E
LIGHT WHITE SQUARE

1F78F
MEDIUM WHITE SQUARE

1F790
BOLD WHITE SQUARE

1F791
HEAVY WHITE SQUARE

1F792
VERY HEAVY WHITE SQUARE

1F793
EXTREMELY HEAVY WHITE SQUARE

1F794
WHITE SQUARE CONTAINING BLACK VERY SMALL SQUARE

1F795
WHITE SQUARE CONTAINING BLACK MEDIUM SQUARE

1F796
SQUARE TARGET

1F797
BLACK TINY DIAMOND

1F798
BLACK VERY SMALL DIAMOND

1F799
BLACK MEDIUM SMALL DIAMOND

1F79A
WHITE DIAMOND CONTAINING BLACK VERY SMALL DIAMOND

1F79B
WHITE DIAMOND CONTAINING BLACK MEDIUM DIAMOND

1F79C
DIAMOND TARGET

1F79D
BLACK TINY LOZENGE

1F79E
BLACK VERY SMALL LOZENGE

1F79F
BLACK MEDIUM SMALL LOZENGE

1F7A0
WHITE LOZENGE CONTAINING BLACK SMALL LOZENGE

1F7A1
THIN GREEK CROSS

1F7A2
LIGHT GREEK CROSS

1F7A3
MEDIUM GREEK CROSS

1F7A4
BOLD GREEK CROSS

1F7A5
VERY BOLD GREEK CROSS

1F7A6
VERY HEAVY GREEK CROSS

1F7A7
EXTREMELY HEAVY GREEK CROSS

1F7A8
THIN SALTIRE

1F7A9
LIGHT SALTIRE

1F7AA
MEDIUM SALTIRE

1F7AB
BOLD SALTIRE

1F7AC
HEAVY SALTIRE

1F7AD
VERY HEAVY SALTIRE

1F7AE
EXTREMELY HEAVY SALTIRE

1F7AF
LIGHT FIVE SPOKED ASTERISK

1F7B0
MEDIUM FIVE SPOKED ASTERISK

1F7B1
BOLD FIVE SPOKED ASTERISK

1F7B2
HEAVY FIVE SPOKED ASTERISK

1F7B3
VERY HEAVY FIVE SPOKED ASTERISK

1F7B4
EXTREMELY HEAVY FIVE SPOKED ASTERISK

1F7B5
LIGHT SIX SPOKED ASTERISK

1F7B6
MEDIUM SIX SPOKED ASTERISK

1F7B7
BOLD SIX SPOKED ASTERISK

1F7B8
HEAVY SIX SPOKED ASTERISK

1F7B9
VERY HEAVY SIX SPOKED ASTERISK

1F7BA
EXTREMELY HEAVY SIX SPOKED ASTERISK

1F7BB
LIGHT EIGHT SPOKED ASTERISK

1F7BC
MEDIUM EIGHT SPOKED ASTERISK

1F7BD
BOLD EIGHT SPOKED ASTERISK

1F7BE
HEAVY EIGHT SPOKED ASTERISK

1F7BF
VERY HEAVY EIGHT SPOKED ASTERISK

1F7C0
LIGHT THREE POINTED BLACK STAR

1F7C1
MEDIUM THREE POINTED BLACK STAR

1F7C2
THREE POINTED BLACK STAR

1F7C3
MEDIUM THREE POINTED PINWHEEL STAR

1F7C4
LIGHT FOUR POINTED BLACK STAR

1F7C5
MEDIUM FOUR POINTED BLACK STAR

1F7C6
FOUR POINTED BLACK STAR

1F7C7
MEDIUM FOUR POINTED PINWHEEL STAR

1F7C8
REVERSE LIGHT FOUR POINTED PINWHEEL STAR

1F7C9
LIGHT FIVE POINTED BLACK STAR

1F7CA
HEAVY FIVE POINTED BLACK STAR

1F7CB
MEDIUM SIX POINTED BLACK STAR

1F7CC
HEAVY SIX POINTED BLACK STAR

1F7CD
SIX POINTED PINWHEEL STAR

1F7CE
MEDIUM EIGHT POINTED BLACK STAR

1F7CF
HEAVY EIGHT POINTED BLACK STAR

1F7D0
VERY HEAVY EIGHT POINTED BLACK STAR

1F7D1
HEAVY EIGHT POINTED PINWHEEL STAR

1F7D2
LIGHT TWELVE POINTED BLACK STAR

1F7D3
HEAVY TWELVE POINTED BLACK STAR

1F7D4
HEAVY TWELVE POINTED PINWHEEL STAR

1F7D5
CIRCLED TRIANGLE

1F7D6
NEGATIVE CIRCLED TRIANGLE

1F7D7
CIRCLED SQUARE

1F7D8
NEGATIVE CIRCLED SQUARE

1F7D9
NINE POINTED WHITE STAR

1F7E0
LARGE ORANGE CIRCLE

1F7E1
LARGE YELLOW CIRCLE

1F7E2
LARGE GREEN CIRCLE

1F7E3
LARGE PURPLE CIRCLE

1F7E4
LARGE BROWN CIRCLE

1F7E5
LARGE RED SQUARE

1F7E6
LARGE BLUE SQUARE

1F7E7
LARGE ORANGE SQUARE

1F7E8
LARGE YELLOW SQUARE

1F7E9
LARGE GREEN SQUARE

1F7EA
LARGE PURPLE SQUARE

1F7EB
LARGE BROWN SQUARE

1F7F0
HEAVY EQUALS SIGN

1F800
LEFTWARDS ARROW WITH SMALL TRIANGLE ARROWHEAD

1F801
UPWARDS ARROW WITH SMALL TRIANGLE ARROWHEAD

1F802
RIGHTWARDS ARROW WITH SMALL TRIANGLE ARROWHEAD

1F803
DOWNWARDS ARROW WITH SMALL TRIANGLE ARROWHEAD

1F804
LEFTWARDS ARROW WITH MEDIUM TRIANGLE ARROWHEAD

1F805
UPWARDS ARROW WITH MEDIUM TRIANGLE ARROWHEAD

1F806
RIGHTWARDS ARROW WITH MEDIUM TRIANGLE ARROWHEAD

1F807
DOWNWARDS ARROW WITH MEDIUM TRIANGLE ARROWHEAD

1F808
LEFTWARDS ARROW WITH LARGE TRIANGLE ARROWHEAD

1F809
UPWARDS ARROW WITH LARGE TRIANGLE ARROWHEAD

1F80A
RIGHTWARDS ARROW WITH LARGE TRIANGLE ARROWHEAD

1F80B
DOWNWARDS ARROW WITH LARGE TRIANGLE ARROWHEAD

1F810
LEFTWARDS ARROW WITH SMALL EQUILATERAL ARROWHEAD

1F811
UPWARDS ARROW WITH SMALL EQUILATERAL ARROWHEAD

1F812
RIGHTWARDS ARROW WITH SMALL EQUILATERAL ARROWHEAD

1F813
DOWNWARDS ARROW WITH SMALL EQUILATERAL ARROWHEAD

1F814
LEFTWARDS ARROW WITH EQUILATERAL ARROWHEAD

1F815
UPWARDS ARROW WITH EQUILATERAL ARROWHEAD

1F816
RIGHTWARDS ARROW WITH EQUILATERAL ARROWHEAD

1F817
DOWNWARDS ARROW WITH EQUILATERAL ARROWHEAD

1F818
HEAVY LEFTWARDS ARROW WITH EQUILATERAL ARROWHEAD

1F819
HEAVY UPWARDS ARROW WITH EQUILATERAL ARROWHEAD

1F81A
HEAVY RIGHTWARDS ARROW WITH EQUILATERAL ARROWHEAD

1F81B
HEAVY DOWNWARDS ARROW WITH EQUILATERAL ARROWHEAD

1F81C
HEAVY LEFTWARDS ARROW WITH LARGE EQUILATERAL ARROWHEAD

1F81D
HEAVY UPWARDS ARROW WITH LARGE EQUILATERAL ARROWHEAD

1F81E
HEAVY RIGHTWARDS ARROW WITH LARGE EQUILATERAL ARROWHEAD

1F81F
HEAVY DOWNWARDS ARROW WITH LARGE EQUILATERAL ARROWHEAD

1F820
LEFTWARDS TRIANGLE-HEADED ARROW WITH NARROW SHAFT

1F821
UPWARDS TRIANGLE-HEADED ARROW WITH NARROW SHAFT

1F822
RIGHTWARDS TRIANGLE-HEADED ARROW WITH NARROW SHAFT

1F823
DOWNWARDS TRIANGLE-HEADED ARROW WITH NARROW SHAFT

1F824
LEFTWARDS TRIANGLE-HEADED ARROW WITH MEDIUM SHAFT

1F825
UPWARDS TRIANGLE-HEADED ARROW WITH MEDIUM SHAFT

1F826
RIGHTWARDS TRIANGLE-HEADED ARROW WITH MEDIUM SHAFT

1F827
DOWNWARDS TRIANGLE-HEADED ARROW WITH MEDIUM SHAFT

1F828
LEFTWARDS TRIANGLE-HEADED ARROW WITH BOLD SHAFT

1F829
UPWARDS TRIANGLE-HEADED ARROW WITH BOLD SHAFT

1F82A
RIGHTWARDS TRIANGLE-HEADED ARROW WITH BOLD SHAFT

1F82B
DOWNWARDS TRIANGLE-HEADED ARROW WITH BOLD SHAFT

1F82C
LEFTWARDS TRIANGLE-HEADED ARROW WITH HEAVY SHAFT

1F82D
UPWARDS TRIANGLE-HEADED ARROW WITH HEAVY SHAFT

1F82E
RIGHTWARDS TRIANGLE-HEADED ARROW WITH HEAVY SHAFT

1F82F
DOWNWARDS TRIANGLE-HEADED ARROW WITH HEAVY SHAFT

1F830
LEFTWARDS TRIANGLE-HEADED ARROW WITH VERY HEAVY SHAFT

1F831
UPWARDS TRIANGLE-HEADED ARROW WITH VERY HEAVY SHAFT

1F832
RIGHTWARDS TRIANGLE-HEADED ARROW WITH VERY HEAVY SHAFT

1F833
DOWNWARDS TRIANGLE-HEADED ARROW WITH VERY HEAVY SHAFT

1F834
LEFTWARDS FINGER-POST ARROW

1F835
UPWARDS FINGER-POST ARROW

1F836
RIGHTWARDS FINGER-POST ARROW

1F837
DOWNWARDS FINGER-POST ARROW

1F838
LEFTWARDS SQUARED ARROW

1F839
UPWARDS SQUARED ARROW

1F83A
RIGHTWARDS SQUARED ARROW

1F83B
DOWNWARDS SQUARED ARROW

1F83C
LEFTWARDS COMPRESSED ARROW

1F83D
UPWARDS COMPRESSED ARROW

1F83E
RIGHTWARDS COMPRESSED ARROW

1F83F
DOWNWARDS COMPRESSED ARROW

1F840
LEFTWARDS HEAVY COMPRESSED ARROW

1F841
UPWARDS HEAVY COMPRESSED ARROW

1F842
RIGHTWARDS HEAVY COMPRESSED ARROW

1F843
DOWNWARDS HEAVY COMPRESSED ARROW

1F844
LEFTWARDS HEAVY ARROW

1F845
UPWARDS HEAVY ARROW

1F846
RIGHTWARDS HEAVY ARROW

1F847
DOWNWARDS HEAVY ARROW

1F850
LEFTWARDS SANS-SERIF ARROW

1F851
UPWARDS SANS-SERIF ARROW

1F852
RIGHTWARDS SANS-SERIF ARROW

1F853
DOWNWARDS SANS-SERIF ARROW

1F854
NORTH WEST SANS-SERIF ARROW

1F855
NORTH EAST SANS-SERIF ARROW

1F856
SOUTH EAST SANS-SERIF ARROW

1F857
SOUTH WEST SANS-SERIF ARROW

1F858
LEFT RIGHT SANS-SERIF ARROW

1F859
UP DOWN SANS-SERIF ARROW

1F860
WIDE-HEADED LEFTWARDS LIGHT BARB ARROW

1F861
WIDE-HEADED UPWARDS LIGHT BARB ARROW

1F862
WIDE-HEADED RIGHTWARDS LIGHT BARB ARROW

1F863
WIDE-HEADED DOWNWARDS LIGHT BARB ARROW

1F864
WIDE-HEADED NORTH WEST LIGHT BARB ARROW

1F865
WIDE-HEADED NORTH EAST LIGHT BARB ARROW

1F866
WIDE-HEADED SOUTH EAST LIGHT BARB ARROW

1F867
WIDE-HEADED SOUTH WEST LIGHT BARB ARROW

1F868
WIDE-HEADED LEFTWARDS BARB ARROW

1F869
WIDE-HEADED UPWARDS BARB ARROW

1F86A
WIDE-HEADED RIGHTWARDS BARB ARROW

1F86B
WIDE-HEADED DOWNWARDS BARB ARROW

1F86C
WIDE-HEADED NORTH WEST BARB ARROW

1F86D
WIDE-HEADED NORTH EAST BARB ARROW

1F86E
WIDE-HEADED SOUTH EAST BARB ARROW

1F86F
WIDE-HEADED SOUTH WEST BARB ARROW

1F870
WIDE-HEADED LEFTWARDS MEDIUM BARB ARROW

1F871
WIDE-HEADED UPWARDS MEDIUM BARB ARROW

1F872
WIDE-HEADED RIGHTWARDS MEDIUM BARB ARROW

1F873
WIDE-HEADED DOWNWARDS MEDIUM BARB ARROW

1F874
WIDE-HEADED NORTH WEST MEDIUM BARB ARROW

1F875
WIDE-HEADED NORTH EAST MEDIUM BARB ARROW

1F876
WIDE-HEADED SOUTH EAST MEDIUM BARB ARROW

1F877
WIDE-HEADED SOUTH WEST MEDIUM BARB ARROW

1F878
WIDE-HEADED LEFTWARDS HEAVY BARB ARROW

1F879
WIDE-HEADED UPWARDS HEAVY BARB ARROW

1F87A
WIDE-HEADED RIGHTWARDS HEAVY BARB ARROW

1F87B
WIDE-HEADED DOWNWARDS HEAVY BARB ARROW

1F87C
WIDE-HEADED NORTH WEST HEAVY BARB ARROW

1F87D
WIDE-HEADED NORTH EAST HEAVY BARB ARROW

1F87E
WIDE-HEADED SOUTH EAST HEAVY BARB ARROW

1F87F
WIDE-HEADED SOUTH WEST HEAVY BARB ARROW

1F880
WIDE-HEADED LEFTWARDS VERY HEAVY BARB ARROW

1F881
WIDE-HEADED UPWARDS VERY HEAVY BARB ARROW

1F882
WIDE-HEADED RIGHTWARDS VERY HEAVY BARB ARROW

1F883
WIDE-HEADED DOWNWARDS VERY HEAVY BARB ARROW

1F884
WIDE-HEADED NORTH WEST VERY HEAVY BARB ARROW

1F885
WIDE-HEADED NORTH EAST VERY HEAVY BARB ARROW

1F886
WIDE-HEADED SOUTH EAST VERY HEAVY BARB ARROW

1F887
WIDE-HEADED SOUTH WEST VERY HEAVY BARB ARROW

1F890
LEFTWARDS TRIANGLE ARROWHEAD

1F891
UPWARDS TRIANGLE ARROWHEAD

1F892
RIGHTWARDS TRIANGLE ARROWHEAD

1F893
DOWNWARDS TRIANGLE ARROWHEAD

1F894
LEFTWARDS WHITE ARROW WITHIN TRIANGLE ARROWHEAD

1F895
UPWARDS WHITE ARROW WITHIN TRIANGLE ARROWHEAD

1F896
RIGHTWARDS WHITE ARROW WITHIN TRIANGLE ARROWHEAD

1F897
DOWNWARDS WHITE ARROW WITHIN TRIANGLE ARROWHEAD

1F898
LEFTWARDS ARROW WITH NOTCHED TAIL

1F899
UPWARDS ARROW WITH NOTCHED TAIL

1F89A
RIGHTWARDS ARROW WITH NOTCHED TAIL

1F89B
DOWNWARDS ARROW WITH NOTCHED TAIL

1F89C
HEAVY ARROW SHAFT WIDTH ONE

1F89D
HEAVY ARROW SHAFT WIDTH TWO THIRDS

1F89E
HEAVY ARROW SHAFT WIDTH ONE HALF

1F89F
HEAVY ARROW SHAFT WIDTH ONE THIRD

1F8A0
LEFTWARDS BOTTOM-SHADED WHITE ARROW

1F8A1
RIGHTWARDS BOTTOM SHADED WHITE ARROW

1F8A2
LEFTWARDS TOP SHADED WHITE ARROW

1F8A3
RIGHTWARDS TOP SHADED WHITE ARROW

1F8A4
LEFTWARDS LEFT-SHADED WHITE ARROW

1F8A5
RIGHTWARDS RIGHT-SHADED WHITE ARROW

1F8A6
LEFTWARDS RIGHT-SHADED WHITE ARROW

1F8A7
RIGHTWARDS LEFT-SHADED WHITE ARROW

1F8A8
LEFTWARDS BACK-TILTED SHADOWED WHITE ARROW

1F8A9
RIGHTWARDS BACK-TILTED SHADOWED WHITE ARROW

1F8AA
LEFTWARDS FRONT-TILTED SHADOWED WHITE ARROW

1F8AB
RIGHTWARDS FRONT-TILTED SHADOWED WHITE ARROW

1F8AC
WHITE ARROW SHAFT WIDTH ONE

1F8AD
WHITE ARROW SHAFT WIDTH TWO THIRDS

1F8B0
ARROW POINTING UPWARDS THEN NORTH WEST

1F8B1
ARROW POINTING RIGHTWARDS THEN CURVING SOUTH WEST

1F8B2
RIGHTWARDS ARROW WITH LOWER HOOK

1F8B3
DOWNWARDS BLACK ARROW TO BAR

1F8B4
NEGATIVE SQUARED LEFTWARDS ARROW

1F8B5
NEGATIVE SQUARED UPWARDS ARROW

1F8B6
NEGATIVE SQUARED RIGHTWARDS ARROW

1F8B7
NEGATIVE SQUARED DOWNWARDS ARROW

1F8B8
NORTH WEST ARROW FROM BAR

1F8B9
NORTH EAST ARROW FROM BAR

1F8BA
SOUTH EAST ARROW FROM BAR

1F8BB
SOUTH WEST ARROW FROM BAR

1F8C0
LEFTWARDS ARROW FROM DOWNWARDS ARROW

1F8C1
RIGHTWARDS ARROW FROM DOWNWARDS ARROW

1F8D0
LONG RIGHTWARDS ARROW OVER LONG LEFTWARDS ARROW

1F8D1
LONG RIGHTWARDS HARPOON OVER LONG LEFTWARDS HARPOON

1F8D2
LONG RIGHTWARDS HARPOON ABOVE SHORT LEFTWARDS HARPOON

1F8D3
SHORT RIGHTWARDS HARPOON ABOVE LONG LEFTWARDS HARPOON

1F8D4
LONG LEFTWARDS HARPOON ABOVE SHORT RIGHTWARDS HARPOON

1F8D5
SHORT LEFTWARDS HARPOON ABOVE LONG RIGHTWARDS HARPOON

1F8D6
LONG RIGHTWARDS ARROW THROUGH X

1F8D7
LONG RIGHTWARDS ARROW WITH DOUBLE SLASH

1F8D8
LONG LEFT RIGHT ARROW WITH DEPENDENT LOBE

1F900
CIRCLED CROSS FORMEE WITH FOUR DOTS

1F901
CIRCLED CROSS FORMEE WITH TWO DOTS

1F902
CIRCLED CROSS FORMEE

1F903
LEFT HALF CIRCLE WITH FOUR DOTS

1F904
LEFT HALF CIRCLE WITH THREE DOTS

1F905
LEFT HALF CIRCLE WITH TWO DOTS

1F906
LEFT HALF CIRCLE WITH DOT

1F907
LEFT HALF CIRCLE

1F908
DOWNWARD FACING HOOK

1F909
DOWNWARD FACING NOTCHED HOOK

1F90A
DOWNWARD FACING HOOK WITH DOT

1F90B
DOWNWARD FACING NOTCHED HOOK WITH DOT

1F90C
PINCHED FINGERS

1F90D
WHITE HEART

1F90E
BROWN HEART

1F90F
PINCHING HAND

1F910
ZIPPER-MOUTH FACE

1F911
MONEY-MOUTH FACE

1F912
FACE WITH THERMOMETER

1F913
NERD FACE

1F914
THINKING FACE

1F915
FACE WITH HEAD-BANDAGE

1F916
ROBOT FACE

1F917
HUGGING FACE

1F918
SIGN OF THE HORNS

1F919
CALL ME HAND

1F91A
RAISED BACK OF HAND

1F91B
LEFT-FACING FIST

1F91C
RIGHT-FACING FIST

1F91D
HANDSHAKE

1F91E
HAND WITH INDEX AND MIDDLE FINGERS CROSSED

1F91F
I LOVE YOU HAND SIGN

1F920
FACE WITH COWBOY HAT

1F921
CLOWN FACE

1F922
NAUSEATED FACE

1F923
ROLLING ON THE FLOOR LAUGHING

1F924
DROOLING FACE

1F925
LYING FACE

1F926
FACE PALM

1F927
SNEEZING FACE

1F928
FACE WITH ONE EYEBROW RAISED

1F929
GRINNING FACE WITH STAR EYES

1F92A
GRINNING FACE WITH ONE LARGE AND ONE SMALL EYE

1F92B
FACE WITH FINGER COVERING CLOSED LIPS

1F92C
SERIOUS FACE WITH SYMBOLS COVERING MOUTH

1F92D
SMILING FACE WITH SMILING EYES AND HAND COVERING MOUTH

1F92E
FACE WITH OPEN MOUTH VOMITING

1F92F
SHOCKED FACE WITH EXPLODING HEAD

1F930
PREGNANT WOMAN

1F931
BREAST-FEEDING

1F932
PALMS UP TOGETHER

1F933
SELFIE

1F934
PRINCE

1F935
MAN IN TUXEDO

1F936
MOTHER CHRISTMAS

1F937
SHRUG

1F938
PERSON DOING CARTWHEEL

1F939
JUGGLING

1F93A
FENCER

1F93B
MODERN PENTATHLON

1F93C
WRESTLERS

1F93D
WATER POLO

1F93E
HANDBALL

1F93F
DIVING MASK

1F940
WILTED FLOWER

1F941
DRUM WITH DRUMSTICKS

1F942
CLINKING GLASSES

1F943
TUMBLER GLASS

1F944
SPOON

1F945
GOAL NET

1F946
RIFLE

1F947
FIRST PLACE MEDAL

1F948
SECOND PLACE MEDAL

1F949
THIRD PLACE MEDAL

1F94A
BOXING GLOVE

1F94B
MARTIAL ARTS UNIFORM

1F94C
CURLING STONE

1F94D
LACROSSE STICK AND BALL

1F94E
SOFTBALL

1F94F
FLYING DISC

1F950
CROISSANT

1F951
AVOCADO

1F952
CUCUMBER

1F953
BACON

1F954
POTATO

1F955
CARROT

1F956
BAGUETTE BREAD

1F957
GREEN SALAD

1F958
SHALLOW PAN OF FOOD

1F959
STUFFED FLATBREAD

1F95A
EGG

1F95B
GLASS OF MILK

1F95C
PEANUTS

1F95D
KIWIFRUIT

1F95E
PANCAKES

1F95F
DUMPLING

1F960
FORTUNE COOKIE

1F961
TAKEOUT BOX

1F962
CHOPSTICKS

1F963
BOWL WITH SPOON

1F964
CUP WITH STRAW

1F965
COCONUT

1F966
BROCCOLI

1F967
PIE

1F968
PRETZEL

1F969
CUT OF MEAT

1F96A
SANDWICH

1F96B
CANNED FOOD

1F96C
LEAFY GREEN

1F96D
MANGO

1F96E
MOON CAKE

1F96F
BAGEL

1F970
SMILING FACE WITH SMILING EYES AND THREE HEARTS

1F971
YAWNING FACE

1F972
SMILING FACE WITH TEAR

1F973
FACE WITH PARTY HORN AND PARTY HAT

1F974
FACE WITH UNEVEN EYES AND WAVY MOUTH

1F975
OVERHEATED FACE

1F976
FREEZING FACE

1F977
NINJA

1F978
DISGUISED FACE

1F979
FACE HOLDING BACK TEARS

1F97A
FACE WITH PLEADING EYES

1F97B
SARI

1F97C
LAB COAT

1F97D
GOGGLES

1F97E
HIKING BOOT

1F97F
FLAT SHOE

1F980
CRAB

1F981
LION FACE

1F982
SCORPION

1F983
TURKEY

1F984
UNICORN FACE

1F985
EAGLE

1F986
DUCK

1F987
BAT

1F988
SHARK

1F989
OWL

1F98A
FOX FACE

1F98B
BUTTERFLY

1F98C
DEER

1F98D
GORILLA

1F98E
LIZARD

1F98F
RHINOCEROS

1F990
SHRIMP

1F991
SQUID

1F992
GIRAFFE FACE

1F993
ZEBRA FACE

1F994
HEDGEHOG

1F995
SAUROPOD

1F996
T-REX

1F997
CRICKET

1F998
KANGAROO

1F999
LLAMA

1F99A
PEACOCK

1F99B
HIPPOPOTAMUS

1F99C
PARROT

1F99D
RACCOON

1F99E
LOBSTER

1F99F
MOSQUITO

1F9A0
MICROBE

1F9A1
BADGER

1F9A2
SWAN

1F9A3
MAMMOTH

1F9A4
DODO

1F9A5
SLOTH

1F9A6
OTTER

1F9A7
ORANGUTAN

1F9A8
SKUNK

1F9A9
FLAMINGO

1F9AA
OYSTER

1F9AB
BEAVER

1F9AC
BISON

1F9AD
SEAL

1F9AE
GUIDE DOG

1F9AF
PROBING CANE

1F9B0
EMOJI COMPONENT RED HAIR

1F9B1
EMOJI COMPONENT CURLY HAIR

1F9B2
EMOJI COMPONENT BALD

1F9B3
EMOJI COMPONENT WHITE HAIR

1F9B4
BONE

1F9B5
LEG

1F9B6
FOOT

1F9B7
TOOTH

1F9B8
SUPERHERO

1F9B9
SUPERVILLAIN

1F9BA
SAFETY VEST

1F9BB
EAR WITH HEARING AID

1F9BC
MOTORIZED WHEELCHAIR

1F9BD
MANUAL WHEELCHAIR

1F9BE
MECHANICAL ARM

1F9BF
MECHANICAL LEG

1F9C0
CHEESE WEDGE

1F9C1
CUPCAKE

1F9C2
SALT SHAKER

1F9C3
BEVERAGE BOX

1F9C4
GARLIC

1F9C5
ONION

1F9C6
FALAFEL

1F9C7
WAFFLE

1F9C8
BUTTER

1F9C9
MATE DRINK

1F9CA
ICE CUBE

1F9CB
BUBBLE TEA

1F9CC
TROLL

1F9CD
STANDING PERSON

1F9CE
KNEELING PERSON

1F9CF
DEAF PERSON

1F9D0
FACE WITH MONOCLE

1F9D1
ADULT

1F9D2
CHILD

1F9D3
OLDER ADULT

1F9D4
BEARDED PERSON

1F9D5
PERSON WITH HEADSCARF

1F9D6
PERSON IN STEAMY ROOM

1F9D7
PERSON CLIMBING

1F9D8
PERSON IN LOTUS POSITION

1F9D9
MAGE

1F9DA
FAIRY

1F9DB
VAMPIRE

1F9DC
MERPERSON

1F9DD
ELF

1F9DE
GENIE

1F9DF
ZOMBIE

1F9E0
BRAIN

1F9E1
ORANGE HEART

1F9E2
BILLED CAP

1F9E3
SCARF

1F9E4
GLOVES

1F9E5
COAT

1F9E6
SOCKS

1F9E7
RED GIFT ENVELOPE

1F9E8
FIRECRACKER

1F9E9
JIGSAW PUZZLE PIECE

1F9EA
TEST TUBE

1F9EB
PETRI DISH

1F9EC
DNA DOUBLE HELIX

1F9ED
COMPASS

1F9EE
ABACUS

1F9EF
FIRE EXTINGUISHER

1F9F0
TOOLBOX

1F9F1
BRICK

1F9F2
MAGNET

1F9F3
LUGGAGE

1F9F4
LOTION BOTTLE

1F9F5
SPOOL OF THREAD

1F9F6
BALL OF YARN

1F9F7
SAFETY PIN

1F9F8
TEDDY BEAR

1F9F9
BROOM

1F9FA
BASKET

1F9FB
ROLL OF PAPER

1F9FC
BAR OF SOAP

1F9FD
SPONGE

1F9FE
RECEIPT

1F9FF
NAZAR AMULET

1FA00
NEUTRAL CHESS KING

1FA01
NEUTRAL CHESS QUEEN

1FA02
NEUTRAL CHESS ROOK

1FA03
NEUTRAL CHESS BISHOP

1FA04
NEUTRAL CHESS KNIGHT

1FA05
NEUTRAL CHESS PAWN

1FA06
WHITE CHESS KNIGHT ROTATED FORTY-FIVE DEGREES

1FA07
BLACK CHESS KNIGHT ROTATED FORTY-FIVE DEGREES

1FA08
NEUTRAL CHESS KNIGHT ROTATED FORTY-FIVE DEGREES

1FA09
WHITE CHESS KING ROTATED NINETY DEGREES

1FA0A
WHITE CHESS QUEEN ROTATED NINETY DEGREES

1FA0B
WHITE CHESS ROOK ROTATED NINETY DEGREES

1FA0C
WHITE CHESS BISHOP ROTATED NINETY DEGREES

1FA0D
WHITE CHESS KNIGHT ROTATED NINETY DEGREES

1FA0E
WHITE CHESS PAWN ROTATED NINETY DEGREES

1FA0F
BLACK CHESS KING ROTATED NINETY DEGREES

1FA10
BLACK CHESS QUEEN ROTATED NINETY DEGREES

1FA11
BLACK CHESS ROOK ROTATED NINETY DEGREES

1FA12
BLACK CHESS BISHOP ROTATED NINETY DEGREES

1FA13
BLACK CHESS KNIGHT ROTATED NINETY DEGREES

1FA14
BLACK CHESS PAWN ROTATED NINETY DEGREES

1FA15
NEUTRAL CHESS KING ROTATED NINETY DEGREES

1FA16
NEUTRAL CHESS QUEEN ROTATED NINETY DEGREES

1FA17
NEUTRAL CHESS ROOK ROTATED NINETY DEGREES

1FA18
NEUTRAL CHESS BISHOP ROTATED NINETY DEGREES

1FA19
NEUTRAL CHESS KNIGHT ROTATED NINETY DEGREES

1FA1A
NEUTRAL CHESS PAWN ROTATED NINETY DEGREES

1FA1B
WHITE CHESS KNIGHT ROTATED ONE HUNDRED THIRTY-FIVE DEGREES

1FA1C
BLACK CHESS KNIGHT ROTATED ONE HUNDRED THIRTY-FIVE DEGREES

1FA1D
NEUTRAL CHESS KNIGHT ROTATED ONE HUNDRED THIRTY-FIVE DEGREES

1FA1E
WHITE CHESS TURNED KING

1FA1F
WHITE CHESS TURNED QUEEN

1FA20
WHITE CHESS TURNED ROOK

1FA21
WHITE CHESS TURNED BISHOP

1FA22
WHITE CHESS TURNED KNIGHT

1FA23
WHITE CHESS TURNED PAWN

1FA24
BLACK CHESS TURNED KING

1FA25
BLACK CHESS TURNED QUEEN

1FA26
BLACK CHESS TURNED ROOK

1FA27
BLACK CHESS TURNED BISHOP

1FA28
BLACK CHESS TURNED KNIGHT

1FA29
BLACK CHESS TURNED PAWN

1FA2A
NEUTRAL CHESS TURNED KING

1FA2B
NEUTRAL CHESS TURNED QUEEN

1FA2C
NEUTRAL CHESS TURNED ROOK

1FA2D
NEUTRAL CHESS TURNED BISHOP

1FA2E
NEUTRAL CHESS TURNED KNIGHT

1FA2F
NEUTRAL CHESS TURNED PAWN

1FA30
WHITE CHESS KNIGHT ROTATED TWO HUNDRED TWENTY-FIVE DEGREES

1FA31
BLACK CHESS KNIGHT ROTATED TWO HUNDRED TWENTY-FIVE DEGREES

1FA32
NEUTRAL CHESS KNIGHT ROTATED TWO HUNDRED TWENTY-FIVE DEGREES

1FA33
WHITE CHESS KING ROTATED TWO HUNDRED SEVENTY DEGREES

1FA34
WHITE CHESS QUEEN ROTATED TWO HUNDRED SEVENTY DEGREES

1FA35
WHITE CHESS ROOK ROTATED TWO HUNDRED SEVENTY DEGREES

1FA36
WHITE CHESS BISHOP ROTATED TWO HUNDRED SEVENTY DEGREES

1FA37
WHITE CHESS KNIGHT ROTATED TWO HUNDRED SEVENTY DEGREES

1FA38
WHITE CHESS PAWN ROTATED TWO HUNDRED SEVENTY DEGREES

1FA39
BLACK CHESS KING ROTATED TWO HUNDRED SEVENTY DEGREES

1FA3A
BLACK CHESS QUEEN ROTATED TWO HUNDRED SEVENTY DEGREES

1FA3B
BLACK CHESS ROOK ROTATED TWO HUNDRED SEVENTY DEGREES

1FA3C
BLACK CHESS BISHOP ROTATED TWO HUNDRED SEVENTY DEGREES

1FA3D
BLACK CHESS KNIGHT ROTATED TWO HUNDRED SEVENTY DEGREES

1FA3E
BLACK CHESS PAWN ROTATED TWO HUNDRED SEVENTY DEGREES

1FA3F
NEUTRAL CHESS KING ROTATED TWO HUNDRED SEVENTY DEGREES

1FA40
NEUTRAL CHESS QUEEN ROTATED TWO HUNDRED SEVENTY DEGREES

1FA41
NEUTRAL CHESS ROOK ROTATED TWO HUNDRED SEVENTY DEGREES

1FA42
NEUTRAL CHESS BISHOP ROTATED TWO HUNDRED SEVENTY DEGREES

1FA43
NEUTRAL CHESS KNIGHT ROTATED TWO HUNDRED SEVENTY DEGREES

1FA44
NEUTRAL CHESS PAWN ROTATED TWO HUNDRED SEVENTY DEGREES

1FA45
WHITE CHESS KNIGHT ROTATED THREE HUNDRED FIFTEEN DEGREES

1FA46
BLACK CHESS KNIGHT ROTATED THREE HUNDRED FIFTEEN DEGREES

1FA47
NEUTRAL CHESS KNIGHT ROTATED THREE HUNDRED FIFTEEN DEGREES

1FA48
WHITE CHESS EQUIHOPPER

1FA49
BLACK CHESS EQUIHOPPER

1FA4A
NEUTRAL CHESS EQUIHOPPER

1FA4B
WHITE CHESS EQUIHOPPER ROTATED NINETY DEGREES

1FA4C
BLACK CHESS EQUIHOPPER ROTATED NINETY DEGREES

1FA4D
NEUTRAL CHESS EQUIHOPPER ROTATED NINETY DEGREES

1FA4E
WHITE CHESS KNIGHT-QUEEN

1FA4F
WHITE CHESS KNIGHT-ROOK

1FA50
WHITE CHESS KNIGHT-BISHOP

1FA51
BLACK CHESS KNIGHT-QUEEN

1FA52
BLACK CHESS KNIGHT-ROOK

1FA53
BLACK CHESS KNIGHT-BISHOP

1FA54
WHITE CHESS FERZ

1FA55
WHITE CHESS ALFIL

1FA56
BLACK CHESS FERZ

1FA57
BLACK CHESS ALFIL

1FA60
XIANGQI RED GENERAL

1FA61
XIANGQI RED MANDARIN

1FA62
XIANGQI RED ELEPHANT

1FA63
XIANGQI RED HORSE

1FA64
XIANGQI RED CHARIOT

1FA65
XIANGQI RED CANNON

1FA66
XIANGQI RED SOLDIER

1FA67
XIANGQI BLACK GENERAL

1FA68
XIANGQI BLACK MANDARIN

1FA69
XIANGQI BLACK ELEPHANT

1FA6A
XIANGQI BLACK HORSE

1FA6B
XIANGQI BLACK CHARIOT

1FA6C
XIANGQI BLACK CANNON

1FA6D
XIANGQI BLACK SOLDIER

1FA70
BALLET SHOES

1FA71
ONE-PIECE SWIMSUIT

1FA72
BRIEFS

1FA73
SHORTS

1FA74
THONG SANDAL

1FA75
LIGHT BLUE HEART

1FA76
GREY HEART

1FA77
PINK HEART

1FA78
DROP OF BLOOD

1FA79
ADHESIVE BANDAGE

1FA7A
STETHOSCOPE

1FA7B
X-RAY

1FA7C
CRUTCH

1FA80
YO-YO

1FA81
KITE

1FA82
PARACHUTE

1FA83
BOOMERANG

1FA84
MAGIC WAND

1FA85
PINATA

1FA86
NESTING DOLLS

1FA87
MARACAS

1FA88
FLUTE

1FA89
HARP

1FA8A
TROMBONE

1FA8E
TREASURE CHEST

1FA8F
SHOVEL

1FA90
RINGED PLANET

1FA91
CHAIR

1FA92
RAZOR

1FA93
AXE

1FA94
DIYA LAMP

1FA95
BANJO

1FA96
MILITARY HELMET

1FA97
ACCORDION

1FA98
LONG DRUM

1FA99
COIN

1FA9A
CARPENTRY SAW

1FA9B
SCREWDRIVER

1FA9C
LADDER

1FA9D
HOOK

1FA9E
MIRROR

1FA9F
WINDOW

1FAA0
PLUNGER

1FAA1
SEWING NEEDLE

1FAA2
KNOT

1FAA3
BUCKET

1FAA4
MOUSE TRAP

1FAA5
TOOTHBRUSH

1FAA6
HEADSTONE

1FAA7
PLACARD

1FAA8
ROCK

1FAA9
MIRROR BALL

1FAAA
IDENTIFICATION CARD

1FAAB
LOW BATTERY

1FAAC
HAMSA

1FAAD
FOLDING HAND FAN

1FAAE
HAIR PICK

1FAAF
KHANDA

1FAB0
FLY

1FAB1
WORM

1FAB2
BEETLE

1FAB3
COCKROACH

1FAB4
POTTED PLANT

1FAB5
WOOD

1FAB6
FEATHER

1FAB7
LOTUS

1FAB8
CORAL

1FAB9
EMPTY NEST

1FABA
NEST WITH EGGS

1FABB
HYACINTH

1FABC
JELLYFISH

1FABD
WING

1FABE
LEAFLESS TREE

1FABF
GOOSE

1FAC0
ANATOMICAL HEART

1FAC1
LUNGS

1FAC2
PEOPLE HUGGING

1FAC3
PREGNANT MAN

1FAC4
PREGNANT PERSON

1FAC5
PERSON WITH CROWN

1FAC6
FINGERPRINT

1FAC8
HAIRY CREATURE

1FACD
ORCA

1FACE
MOOSE

1FACF
DONKEY

1FAD0
BLUEBERRIES

1FAD1
BELL PEPPER

1FAD2
OLIVE

1FAD3
FLATBREAD

1FAD4
TAMALE

1FAD5
FONDUE

1FAD6
TEAPOT

1FAD7
POURING LIQUID

1FAD8
BEANS

1FAD9
JAR

1FADA
GINGER ROOT

1FADB
PEA POD

1FADC
ROOT VEGETABLE

1FADF
SPLATTER

1FAE0
MELTING FACE

1FAE1
SALUTING FACE

1FAE2
FACE WITH OPEN EYES AND HAND OVER MOUTH

1FAE3
FACE WITH PEEKING EYE

1FAE4
FACE WITH DIAGONAL MOUTH

1FAE5
DOTTED LINE FACE

1FAE6
BITING LIP

1FAE7
BUBBLES

1FAE8
SHAKING FACE

1FAE9
FACE WITH BAGS UNDER EYES

1FAEA
DISTORTED FACE

1FAEF
FIGHT CLOUD

1FAF0
HAND WITH INDEX FINGER AND THUMB CROSSED

1FAF1
RIGHTWARDS HAND

1FAF2
LEFTWARDS HAND

1FAF3
PALM DOWN HAND

1FAF4
PALM UP HAND

1FAF5
INDEX POINTING AT THE VIEWER

1FAF6
HEART HANDS

1FAF7
LEFTWARDS PUSHING HAND

1FAF8
RIGHTWARDS PUSHING HAND

1FB00
BLOCK SEXTANT-1

1FB01
BLOCK SEXTANT-2

1FB02
BLOCK SEXTANT-12

1FB03
BLOCK SEXTANT-3

1FB04
BLOCK SEXTANT-13

1FB05
BLOCK SEXTANT-23

1FB06
BLOCK SEXTANT-123

1FB07
BLOCK SEXTANT-4

1FB08
BLOCK SEXTANT-14

1FB09
BLOCK SEXTANT-24

1FB0A
BLOCK SEXTANT-124

1FB0B
BLOCK SEXTANT-34

1FB0C
BLOCK SEXTANT-134

1FB0D
BLOCK SEXTANT-234

1FB0E
BLOCK SEXTANT-1234

1FB0F
BLOCK SEXTANT-5

1FB10
BLOCK SEXTANT-15

1FB11
BLOCK SEXTANT-25

1FB12
BLOCK SEXTANT-125

1FB13
BLOCK SEXTANT-35

1FB14
BLOCK SEXTANT-235

1FB15
BLOCK SEXTANT-1235

1FB16
BLOCK SEXTANT-45

1FB17
BLOCK SEXTANT-145

1FB18
BLOCK SEXTANT-245

1FB19
BLOCK SEXTANT-1245

1FB1A
BLOCK SEXTANT-345

1FB1B
BLOCK SEXTANT-1345

1FB1C
BLOCK SEXTANT-2345

1FB1D
BLOCK SEXTANT-12345

1FB1E
BLOCK SEXTANT-6

1FB1F
BLOCK SEXTANT-16

1FB20
BLOCK SEXTANT-26

1FB21
BLOCK SEXTANT-126

1FB22
BLOCK SEXTANT-36

1FB23
BLOCK SEXTANT-136

1FB24
BLOCK SEXTANT-236

1FB25
BLOCK SEXTANT-1236

1FB26
BLOCK SEXTANT-46

1FB27
BLOCK SEXTANT-146

1FB28
BLOCK SEXTANT-1246

1FB29
BLOCK SEXTANT-346

1FB2A
BLOCK SEXTANT-1346

1FB2B
BLOCK SEXTANT-2346

1FB2C
BLOCK SEXTANT-12346

1FB2D
BLOCK SEXTANT-56

1FB2E
BLOCK SEXTANT-156

1FB2F
BLOCK SEXTANT-256

1FB30
BLOCK SEXTANT-1256

1FB31
BLOCK SEXTANT-356

1FB32
BLOCK SEXTANT-1356

1FB33
BLOCK SEXTANT-2356

1FB34
BLOCK SEXTANT-12356

1FB35
BLOCK SEXTANT-456

1FB36
BLOCK SEXTANT-1456

1FB37
BLOCK SEXTANT-2456

1FB38
BLOCK SEXTANT-12456

1FB39
BLOCK SEXTANT-3456

1FB3A
BLOCK SEXTANT-13456

1FB3B
BLOCK SEXTANT-23456

1FB3C
LOWER LEFT BLOCK DIAGONAL LOWER MIDDLE LEFT TO LOWER CENTRE

1FB3D
LOWER LEFT BLOCK DIAGONAL LOWER MIDDLE LEFT TO LOWER RIGHT

1FB3E
LOWER LEFT BLOCK DIAGONAL UPPER MIDDLE LEFT TO LOWER CENTRE

1FB3F
LOWER LEFT BLOCK DIAGONAL UPPER MIDDLE LEFT TO LOWER RIGHT

1FB40
LOWER LEFT BLOCK DIAGONAL UPPER LEFT TO LOWER CENTRE

1FB41
LOWER RIGHT BLOCK DIAGONAL UPPER MIDDLE LEFT TO UPPER CENTRE

1FB42
LOWER RIGHT BLOCK DIAGONAL UPPER MIDDLE LEFT TO UPPER RIGHT

1FB43
LOWER RIGHT BLOCK DIAGONAL LOWER MIDDLE LEFT TO UPPER CENTRE

1FB44
LOWER RIGHT BLOCK DIAGONAL LOWER MIDDLE LEFT TO UPPER RIGHT

1FB45
LOWER RIGHT BLOCK DIAGONAL LOWER LEFT TO UPPER CENTRE

1FB46
LOWER RIGHT BLOCK DIAGONAL LOWER MIDDLE LEFT TO UPPER MIDDLE RIGHT

1FB47
LOWER RIGHT BLOCK DIAGONAL LOWER CENTRE TO LOWER MIDDLE RIGHT

1FB48
LOWER RIGHT BLOCK DIAGONAL LOWER LEFT TO LOWER MIDDLE RIGHT

1FB49
LOWER RIGHT BLOCK DIAGONAL LOWER CENTRE TO UPPER MIDDLE RIGHT

1FB4A
LOWER RIGHT BLOCK DIAGONAL LOWER LEFT TO UPPER MIDDLE RIGHT

1FB4B
LOWER RIGHT BLOCK DIAGONAL LOWER CENTRE TO UPPER RIGHT

1FB4C
LOWER LEFT BLOCK DIAGONAL UPPER CENTRE TO UPPER MIDDLE RIGHT

1FB4D
LOWER LEFT BLOCK DIAGONAL UPPER LEFT TO UPPER MIDDLE RIGHT

1FB4E
LOWER LEFT BLOCK DIAGONAL UPPER CENTRE TO LOWER MIDDLE RIGHT

1FB4F
LOWER LEFT BLOCK DIAGONAL UPPER LEFT TO LOWER MIDDLE RIGHT

1FB50
LOWER LEFT BLOCK DIAGONAL UPPER CENTRE TO LOWER RIGHT

1FB51
LOWER LEFT BLOCK DIAGONAL UPPER MIDDLE LEFT TO LOWER MIDDLE RIGHT

1FB52
UPPER RIGHT BLOCK DIAGONAL LOWER MIDDLE LEFT TO LOWER CENTRE

1FB53
UPPER RIGHT BLOCK DIAGONAL LOWER MIDDLE LEFT TO LOWER RIGHT

1FB54
UPPER RIGHT BLOCK DIAGONAL UPPER MIDDLE LEFT TO LOWER CENTRE

1FB55
UPPER RIGHT BLOCK DIAGONAL UPPER MIDDLE LEFT TO LOWER RIGHT

1FB56
UPPER RIGHT BLOCK DIAGONAL UPPER LEFT TO LOWER CENTRE

1FB57
UPPER LEFT BLOCK DIAGONAL UPPER MIDDLE LEFT TO UPPER CENTRE

1FB58
UPPER LEFT BLOCK DIAGONAL UPPER MIDDLE LEFT TO UPPER RIGHT

1FB59
UPPER LEFT BLOCK DIAGONAL LOWER MIDDLE LEFT TO UPPER CENTRE

1FB5A
UPPER LEFT BLOCK DIAGONAL LOWER MIDDLE LEFT TO UPPER RIGHT

1FB5B
UPPER LEFT BLOCK DIAGONAL LOWER LEFT TO UPPER CENTRE

1FB5C
UPPER LEFT BLOCK DIAGONAL LOWER MIDDLE LEFT TO UPPER MIDDLE RIGHT

1FB5D
UPPER LEFT BLOCK DIAGONAL LOWER CENTRE TO LOWER MIDDLE RIGHT

1FB5E
UPPER LEFT BLOCK DIAGONAL LOWER LEFT TO LOWER MIDDLE RIGHT

1FB5F
UPPER LEFT BLOCK DIAGONAL LOWER CENTRE TO UPPER MIDDLE RIGHT

1FB60
UPPER LEFT BLOCK DIAGONAL LOWER LEFT TO UPPER MIDDLE RIGHT

1FB61
UPPER LEFT BLOCK DIAGONAL LOWER CENTRE TO UPPER RIGHT

1FB62
UPPER RIGHT BLOCK DIAGONAL UPPER CENTRE TO UPPER MIDDLE RIGHT

1FB63
UPPER RIGHT BLOCK DIAGONAL UPPER LEFT TO UPPER MIDDLE RIGHT

1FB64
UPPER RIGHT BLOCK DIAGONAL UPPER CENTRE TO LOWER MIDDLE RIGHT

1FB65
UPPER RIGHT BLOCK DIAGONAL UPPER LEFT TO LOWER MIDDLE RIGHT

1FB66
UPPER RIGHT BLOCK DIAGONAL UPPER CENTRE TO LOWER RIGHT

1FB67
UPPER RIGHT BLOCK DIAGONAL UPPER MIDDLE LEFT TO LOWER MIDDLE RIGHT

1FB68
UPPER AND RIGHT AND LOWER TRIANGULAR THREE QUARTERS BLOCK

1FB69
LEFT AND LOWER AND RIGHT TRIANGULAR THREE QUARTERS BLOCK

1FB6A
UPPER AND LEFT AND LOWER TRIANGULAR THREE QUARTERS BLOCK

1FB6B
LEFT AND UPPER AND RIGHT TRIANGULAR THREE QUARTERS BLOCK

1FB6C
LEFT TRIANGULAR ONE QUARTER BLOCK

1FB6D
UPPER TRIANGULAR ONE QUARTER BLOCK

1FB6E
RIGHT TRIANGULAR ONE QUARTER BLOCK

1FB6F
LOWER TRIANGULAR ONE QUARTER BLOCK

1FB70
VERTICAL ONE EIGHTH BLOCK-2

1FB71
VERTICAL ONE EIGHTH BLOCK-3

1FB72
VERTICAL ONE EIGHTH BLOCK-4

1FB73
VERTICAL ONE EIGHTH BLOCK-5

1FB74
VERTICAL ONE EIGHTH BLOCK-6

1FB75
VERTICAL ONE EIGHTH BLOCK-7

1FB76
HORIZONTAL ONE EIGHTH BLOCK-2

1FB77
HORIZONTAL ONE EIGHTH BLOCK-3

1FB78
HORIZONTAL ONE EIGHTH BLOCK-4

1FB79
HORIZONTAL ONE EIGHTH BLOCK-5

1FB7A
HORIZONTAL ONE EIGHTH BLOCK-6

1FB7B
HORIZONTAL ONE EIGHTH BLOCK-7

1FB7C
LEFT AND LOWER ONE EIGHTH BLOCK

1FB7D
LEFT AND UPPER ONE EIGHTH BLOCK

1FB7E
RIGHT AND UPPER ONE EIGHTH BLOCK

1FB7F
RIGHT AND LOWER ONE EIGHTH BLOCK

1FB80
UPPER AND LOWER ONE EIGHTH BLOCK

1FB81
HORIZONTAL ONE EIGHTH BLOCK-1358

1FB82
UPPER ONE QUARTER BLOCK

1FB83
UPPER THREE EIGHTHS BLOCK

1FB84
UPPER FIVE EIGHTHS BLOCK

1FB85
UPPER THREE QUARTERS BLOCK

1FB86
UPPER SEVEN EIGHTHS BLOCK

1FB87
RIGHT ONE QUARTER BLOCK

1FB88
RIGHT THREE EIGHTHS BLOCK

1FB89
RIGHT FIVE EIGHTHS BLOCK

1FB8A
RIGHT THREE QUARTERS BLOCK

1FB8B
RIGHT SEVEN EIGHTHS BLOCK

1FB8C
LEFT HALF MEDIUM SHADE

1FB8D
RIGHT HALF MEDIUM SHADE

1FB8E
UPPER HALF MEDIUM SHADE

1FB8F
LOWER HALF MEDIUM SHADE

1FB90
INVERSE MEDIUM SHADE

1FB91
UPPER HALF BLOCK AND LOWER HALF INVERSE MEDIUM SHADE

1FB92
UPPER HALF INVERSE MEDIUM SHADE AND LOWER HALF BLOCK

1FB94
LEFT HALF INVERSE MEDIUM SHADE AND RIGHT HALF BLOCK

1FB95
CHECKER BOARD FILL

1FB96
INVERSE CHECKER BOARD FILL

1FB97
HEAVY HORIZONTAL FILL

1FB98
UPPER LEFT TO LOWER RIGHT FILL

1FB99
UPPER RIGHT TO LOWER LEFT FILL

1FB9A
UPPER AND LOWER TRIANGULAR HALF BLOCK

1FB9B
LEFT AND RIGHT TRIANGULAR HALF BLOCK

1FB9C
UPPER LEFT TRIANGULAR MEDIUM SHADE

1FB9D
UPPER RIGHT TRIANGULAR MEDIUM SHADE

1FB9E
LOWER RIGHT TRIANGULAR MEDIUM SHADE

1FB9F
LOWER LEFT TRIANGULAR MEDIUM SHADE

1FBA0
BOX DRAWINGS LIGHT DIAGONAL UPPER CENTRE TO MIDDLE LEFT

1FBA1
BOX DRAWINGS LIGHT DIAGONAL UPPER CENTRE TO MIDDLE RIGHT

1FBA2
BOX DRAWINGS LIGHT DIAGONAL MIDDLE LEFT TO LOWER CENTRE

1FBA3
BOX DRAWINGS LIGHT DIAGONAL MIDDLE RIGHT TO LOWER CENTRE

1FBA4
BOX DRAWINGS LIGHT DIAGONAL UPPER CENTRE TO MIDDLE LEFT TO LOWER CENTRE

1FBA5
BOX DRAWINGS LIGHT DIAGONAL UPPER CENTRE TO MIDDLE RIGHT TO LOWER CENTRE

1FBA6
BOX DRAWINGS LIGHT DIAGONAL MIDDLE LEFT TO LOWER CENTRE TO MIDDLE RIGHT

1FBA7
BOX DRAWINGS LIGHT DIAGONAL MIDDLE LEFT TO UPPER CENTRE TO MIDDLE RIGHT

1FBA8
BOX DRAWINGS LIGHT DIAGONAL UPPER CENTRE TO MIDDLE LEFT AND MIDDLE RIGHT TO LOWER CENTRE

1FBA9
BOX DRAWINGS LIGHT DIAGONAL UPPER CENTRE TO MIDDLE RIGHT AND MIDDLE LEFT TO LOWER CENTRE

1FBAA
BOX DRAWINGS LIGHT DIAGONAL UPPER CENTRE TO MIDDLE RIGHT TO LOWER CENTRE TO MIDDLE LEFT

1FBAB
BOX DRAWINGS LIGHT DIAGONAL UPPER CENTRE TO MIDDLE LEFT TO LOWER CENTRE TO MIDDLE RIGHT

1FBAC
BOX DRAWINGS LIGHT DIAGONAL MIDDLE LEFT TO UPPER CENTRE TO MIDDLE RIGHT TO LOWER CENTRE

1FBAD
BOX DRAWINGS LIGHT DIAGONAL MIDDLE RIGHT TO UPPER CENTRE TO MIDDLE LEFT TO LOWER CENTRE

1FBAE
BOX DRAWINGS LIGHT DIAGONAL DIAMOND

1FBAF
BOX DRAWINGS LIGHT HORIZONTAL WITH VERTICAL STROKE

1FBB0
ARROWHEAD-SHAPED POINTER

1FBB1
INVERSE CHECK MARK

1FBB2
LEFT HALF RUNNING MAN

1FBB3
RIGHT HALF RUNNING MAN

1FBB4
INVERSE DOWNWARDS ARROW WITH TIP LEFTWARDS

1FBB5
LEFTWARDS ARROW AND UPPER AND LOWER ONE EIGHTH BLOCK

1FBB6
RIGHTWARDS ARROW AND UPPER AND LOWER ONE EIGHTH BLOCK

1FBB7
DOWNWARDS ARROW AND RIGHT ONE EIGHTH BLOCK

1FBB8
UPWARDS ARROW AND RIGHT ONE EIGHTH BLOCK

1FBB9
LEFT HALF FOLDER

1FBBA
RIGHT HALF FOLDER

1FBBB
VOIDED GREEK CROSS

1FBBC
RIGHT OPEN SQUARED DOT

1FBBD
NEGATIVE DIAGONAL CROSS

1FBBE
NEGATIVE DIAGONAL MIDDLE RIGHT TO LOWER CENTRE

1FBBF
NEGATIVE DIAGONAL DIAMOND

1FBC0
WHITE HEAVY SALTIRE WITH ROUNDED CORNERS

1FBC1
LEFT THIRD WHITE RIGHT POINTING INDEX

1FBC2
MIDDLE THIRD WHITE RIGHT POINTING INDEX

1FBC3
RIGHT THIRD WHITE RIGHT POINTING INDEX

1FBC4
NEGATIVE SQUARED QUESTION MARK

1FBC5
STICK FIGURE

1FBC6
STICK FIGURE WITH ARMS RAISED

1FBC7
STICK FIGURE LEANING LEFT

1FBC8
STICK FIGURE LEANING RIGHT

1FBC9
STICK FIGURE WITH DRESS

1FBCA
WHITE UP-POINTING CHEVRON

1FBCB
WHITE CROSS MARK

1FBCC
RAISED SMALL LEFT SQUARE BRACKET

1FBCD
BLACK SMALL UP-POINTING CHEVRON

1FBCE
LEFT TWO THIRDS BLOCK

1FBCF
LEFT ONE THIRD BLOCK

1FBD0
BOX DRAWINGS LIGHT DIAGONAL MIDDLE RIGHT TO LOWER LEFT

1FBD1
BOX DRAWINGS LIGHT DIAGONAL UPPER RIGHT TO MIDDLE LEFT

1FBD2
BOX DRAWINGS LIGHT DIAGONAL UPPER LEFT TO MIDDLE RIGHT

1FBD3
BOX DRAWINGS LIGHT DIAGONAL MIDDLE LEFT TO LOWER RIGHT

1FBD4
BOX DRAWINGS LIGHT DIAGONAL UPPER LEFT TO LOWER CENTRE

1FBD5
BOX DRAWINGS LIGHT DIAGONAL UPPER CENTRE TO LOWER RIGHT

1FBD6
BOX DRAWINGS LIGHT DIAGONAL UPPER RIGHT TO LOWER CENTRE

1FBD7
BOX DRAWINGS LIGHT DIAGONAL UPPER CENTRE TO LOWER LEFT

1FBD8
BOX DRAWINGS LIGHT DIAGONAL UPPER LEFT TO MIDDLE CENTRE TO UPPER RIGHT

1FBD9
BOX DRAWINGS LIGHT DIAGONAL UPPER RIGHT TO MIDDLE CENTRE TO LOWER RIGHT

1FBDA
BOX DRAWINGS LIGHT DIAGONAL LOWER LEFT TO MIDDLE CENTRE TO LOWER RIGHT

1FBDB
BOX DRAWINGS LIGHT DIAGONAL UPPER LEFT TO MIDDLE CENTRE TO LOWER LEFT

1FBDC
BOX DRAWINGS LIGHT DIAGONAL UPPER LEFT TO LOWER CENTRE TO UPPER RIGHT

1FBDD
BOX DRAWINGS LIGHT DIAGONAL UPPER RIGHT TO MIDDLE LEFT TO LOWER RIGHT

1FBDE
BOX DRAWINGS LIGHT DIAGONAL LOWER LEFT TO UPPER CENTRE TO LOWER RIGHT

1FBDF
BOX DRAWINGS LIGHT DIAGONAL UPPER LEFT TO MIDDLE RIGHT TO LOWER LEFT

1FBE0
TOP JUSTIFIED LOWER HALF WHITE CIRCLE

1FBE1
RIGHT JUSTIFIED LEFT HALF WHITE CIRCLE

1FBE2
BOTTOM JUSTIFIED UPPER HALF WHITE CIRCLE

1FBE3
LEFT JUSTIFIED RIGHT HALF WHITE CIRCLE

1FBE4
UPPER CENTRE ONE QUARTER BLOCK

1FBE5
LOWER CENTRE ONE QUARTER BLOCK

1FBE6
MIDDLE LEFT ONE QUARTER BLOCK

1FBE7
MIDDLE RIGHT ONE QUARTER BLOCK

1FBE8
TOP JUSTIFIED LOWER HALF BLACK CIRCLE

1FBE9
RIGHT JUSTIFIED LEFT HALF BLACK CIRCLE

1FBEA
BOTTOM JUSTIFIED UPPER HALF BLACK CIRCLE

1FBEB
LEFT JUSTIFIED RIGHT HALF BLACK CIRCLE

1FBEC
TOP RIGHT JUSTIFIED LOWER LEFT QUARTER BLACK CIRCLE

1FBED
BOTTOM LEFT JUSTIFIED UPPER RIGHT QUARTER BLACK CIRCLE

1FBEE
BOTTOM RIGHT JUSTIFIED UPPER LEFT QUARTER BLACK CIRCLE

1FBEF
TOP LEFT JUSTIFIED LOWER RIGHT QUARTER BLACK CIRCLE

1FBF0
SEGMENTED DIGIT ZERO

1FBF1
SEGMENTED DIGIT ONE

1FBF2
SEGMENTED DIGIT TWO

1FBF3
SEGMENTED DIGIT THREE

1FBF4
SEGMENTED DIGIT FOUR

1FBF5
SEGMENTED DIGIT FIVE

1FBF6
SEGMENTED DIGIT SIX

1FBF7
SEGMENTED DIGIT SEVEN

1FBF8
SEGMENTED DIGIT EIGHT

1FBF9
SEGMENTED DIGIT NINE

1FBFA
ALARM BELL SYMBOL

E0001
LANGUAGE TAG

E0020
TAG SPACE

E0021
TAG EXCLAMATION MARK

E0022
TAG QUOTATION MARK

E0023
TAG NUMBER SIGN

E0024
TAG DOLLAR SIGN

E0025
TAG PERCENT SIGN

E0026
TAG AMPERSAND

E0027
TAG APOSTROPHE

E0028
TAG LEFT PARENTHESIS

E0029
TAG RIGHT PARENTHESIS

E002A
TAG ASTERISK

E002B
TAG PLUS SIGN

E002C
TAG COMMA

E002D
TAG HYPHEN-MINUS

E002E
TAG FULL STOP

E002F
TAG SOLIDUS

E0030
TAG DIGIT ZERO

E0031
TAG DIGIT ONE

E0032
TAG DIGIT TWO

E0033
TAG DIGIT THREE

E0034
TAG DIGIT FOUR

E0035
TAG DIGIT FIVE

E0036
TAG DIGIT SIX

E0037
TAG DIGIT SEVEN

E0038
TAG DIGIT EIGHT

E0039
TAG DIGIT NINE

E003A
TAG COLON

E003B
TAG SEMICOLON

E003C
TAG LESS-THAN SIGN

E003D
TAG EQUALS SIGN

E003E
TAG GREATER-THAN SIGN

E003F
TAG QUESTION MARK

E0040
TAG COMMERCIAL AT

E0041
TAG LATIN CAPITAL LETTER A

E0042
TAG LATIN CAPITAL LETTER B

E0043
TAG LATIN CAPITAL LETTER C

E0044
TAG LATIN CAPITAL LETTER D

E0045
TAG LATIN CAPITAL LETTER E

E0046
TAG LATIN CAPITAL LETTER F

E0047
TAG LATIN CAPITAL LETTER G

E0048
TAG LATIN CAPITAL LETTER H

E0049
TAG LATIN CAPITAL LETTER I

E004A
TAG LATIN CAPITAL LETTER J

E004B
TAG LATIN CAPITAL LETTER K

E004C
TAG LATIN CAPITAL LETTER L

E004D
TAG LATIN CAPITAL LETTER M

E004E
TAG LATIN CAPITAL LETTER N

E004F
TAG LATIN CAPITAL LETTER O

E0050
TAG LATIN CAPITAL LETTER P

E0051
TAG LATIN CAPITAL LETTER Q

E0052
TAG LATIN CAPITAL LETTER R

E0053
TAG LATIN CAPITAL LETTER S

E0054
TAG LATIN CAPITAL LETTER T

E0055
TAG LATIN CAPITAL LETTER U

E0056
TAG LATIN CAPITAL LETTER V

E0057
TAG LATIN CAPITAL LETTER W

E0058
TAG LATIN CAPITAL LETTER X

E0059
TAG LATIN CAPITAL LETTER Y

E005A
TAG LATIN CAPITAL LETTER Z

E005B
TAG LEFT SQUARE BRACKET

E005C
TAG REVERSE SOLIDUS

E005D
TAG RIGHT SQUARE BRACKET

E005E
TAG CIRCUMFLEX ACCENT

E005F
TAG LOW LINE

E0060
TAG GRAVE ACCENT

E0061
TAG LATIN SMALL LETTER A

E0062
TAG LATIN SMALL LETTER B

E0063
TAG LATIN SMALL LETTER C

E0064
TAG LATIN SMALL LETTER D

E0065
TAG LATIN SMALL LETTER E

E0066
TAG LATIN SMALL LETTER F

E0067
TAG LATIN SMALL LETTER G

E0068
TAG LATIN SMALL LETTER H

E0069
TAG LATIN SMALL LETTER I

E006A
TAG LATIN SMALL LETTER J

E006B
TAG LATIN SMALL LETTER K

E006C
TAG LATIN SMALL LETTER L

E006D
TAG LATIN SMALL LETTER M

E006E
TAG LATIN SMALL LETTER N

E006F
TAG LATIN SMALL LETTER O

E0070
TAG LATIN SMALL LETTER P

E0071
TAG LATIN SMALL LETTER Q

E0072
TAG LATIN SMALL LETTER R

E0073
TAG LATIN SMALL LETTER S

E0074
TAG LATIN SMALL LETTER T

E0075
TAG LATIN SMALL LETTER U

E0076
TAG LATIN SMALL LETTER V

E0077
TAG LATIN SMALL LETTER W

E0078
TAG LATIN SMALL LETTER X

E0079
TAG LATIN SMALL LETTER Y

E007A
TAG LATIN SMALL LETTER Z

E007B
TAG LEFT CURLY BRACKET

E007C
TAG VERTICAL LINE

E007D
TAG RIGHT CURLY BRACKET

E007E
TAG TILDE

E007F
CANCEL TAG

E0100
VARIATION SELECTOR-17

E0100
VS17

E0101
VARIATION SELECTOR-18

E0101
VS18

E0102
VARIATION SELECTOR-19

E0102
VS19

E0103
VARIATION SELECTOR-20

E0103
VS20

E0104
VARIATION SELECTOR-21

E0104
VS21

E0105
VARIATION SELECTOR-22

E0105
VS22

E0106
VARIATION SELECTOR-23

E0106
VS23

E0107
VARIATION SELECTOR-24

E0107
VS24

E0108
VARIATION SELECTOR-25

E0108
VS25

E0109
VARIATION SELECTOR-26

E0109
VS26

E010A
VARIATION SELECTOR-27

E010A
VS27

E010B
VARIATION SELECTOR-28

E010B
VS28

E010C
VARIATION SELECTOR-29

E010C
VS29

E010D
VARIATION SELECTOR-30

E010D
VS30

E010E
VARIATION SELECTOR-31

E010E
VS31

E010F
VARIATION SELECTOR-32

E010F
VS32

E0110
VARIATION SELECTOR-33

E0110
VS33

E0111
VARIATION SELECTOR-34

E0111
VS34

E0112
VARIATION SELECTOR-35

E0112
VS35

E0113
VARIATION SELECTOR-36

E0113
VS36

E0114
VARIATION SELECTOR-37

E0114
VS37

E0115
VARIATION SELECTOR-38

E0115
VS38

E0116
VARIATION SELECTOR-39

E0116
VS39

E0117
VARIATION SELECTOR-40

E0117
VS40

E0118
VARIATION SELECTOR-41

E0118
VS41

E0119
VARIATION SELECTOR-42

E0119
VS42

E011A
VARIATION SELECTOR-43

E011A
VS43

E011B
VARIATION SELECTOR-44

E011B
VS44

E011C
VARIATION SELECTOR-45

E011C
VS45

E011D
VARIATION SELECTOR-46

E011D
VS46

E011E
VARIATION SELECTOR-47

E011E
VS47

E011F
VARIATION SELECTOR-48

E011F
VS48

E0120
VARIATION SELECTOR-49

E0120
VS49

E0121
VARIATION SELECTOR-50

E0121
VS50

E0122
VARIATION SELECTOR-51

E0122
VS51

E0123
VARIATION SELECTOR-52

E0123
VS52

E0124
VARIATION SELECTOR-53

E0124
VS53

E0125
VARIATION SELECTOR-54

E0125
VS54

E0126
VARIATION SELECTOR-55

E0126
VS55

E0127
VARIATION SELECTOR-56

E0127
VS56

E0128
VARIATION SELECTOR-57

E0128
VS57

E0129
VARIATION SELECTOR-58

E0129
VS58

E012A
VARIATION SELECTOR-59

E012A
VS59

E012B
VARIATION SELECTOR-60

E012B
VS60

E012C
VARIATION SELECTOR-61

E012C
VS61

E012D
VARIATION SELECTOR-62

E012D
VS62

E012E
VARIATION SELECTOR-63

E012E
VS63

E012F
VARIATION SELECTOR-64

E012F
VS64

E0130
VARIATION SELECTOR-65

E0130
VS65

E0131
VARIATION SELECTOR-66

E0131
VS66

E0132
VARIATION SELECTOR-67

E0132
VS67

E0133
VARIATION SELECTOR-68

E0133
VS68

E0134
VARIATION SELECTOR-69

E0134
VS69

E0135
VARIATION SELECTOR-70

E0135
VS70

E0136
VARIATION SELECTOR-71

E0136
VS71

E0137
VARIATION SELECTOR-72

E0137
VS72

E0138
VARIATION SELECTOR-73

E0138
VS73

E0139
VARIATION SELECTOR-74

E0139
VS74

E013A
VARIATION SELECTOR-75

E013A
VS75

E013B
VARIATION SELECTOR-76

E013B
VS76

E013C
VARIATION SELECTOR-77

E013C
VS77

E013D
VARIATION SELECTOR-78

E013D
VS78

E013E
VARIATION SELECTOR-79

E013E
VS79

E013F
VARIATION SELECTOR-80

E013F
VS80

E0140
VARIATION SELECTOR-81

E0140
VS81

E0141
VARIATION SELECTOR-82

E0141
VS82

E0142
VARIATION SELECTOR-83

E0142
VS83

E0143
VARIATION SELECTOR-84

E0143
VS84

E0144
VARIATION SELECTOR-85

E0144
VS85

E0145
VARIATION SELECTOR-86

E0145
VS86

E0146
VARIATION SELECTOR-87

E0146
VS87

E0147
VARIATION SELECTOR-88

E0147
VS88

E0148
VARIATION SELECTOR-89

E0148
VS89

E0149
VARIATION SELECTOR-90

E0149
VS90

E014A
VARIATION SELECTOR-91

E014A
VS91

E014B
VARIATION SELECTOR-92

E014B
VS92

E014C
VARIATION SELECTOR-93

E014C
VS93

E014D
VARIATION SELECTOR-94

E014D
VS94

E014E
VARIATION SELECTOR-95

E014E
VS95

E014F
VARIATION SELECTOR-96

E014F
VS96

E0150
VARIATION SELECTOR-97

E0150
VS97

E0151
VARIATION SELECTOR-98

E0151
VS98

E0152
VARIATION SELECTOR-99

E0152
VS99

E0153
VARIATION SELECTOR-100

E0153
VS100

E0154
VARIATION SELECTOR-101

E0154
VS101

E0155
VARIATION SELECTOR-102

E0155
VS102

E0156
VARIATION SELECTOR-103

E0156
VS103

E0157
VARIATION SELECTOR-104

E0157
VS104

E0158
VARIATION SELECTOR-105

E0158
VS105

E0159
VARIATION SELECTOR-106

E0159
VS106

E015A
VARIATION SELECTOR-107

E015A
VS107

E015B
VARIATION SELECTOR-108

E015B
VS108

E015C
VARIATION SELECTOR-109

E015C
VS109

E015D
VARIATION SELECTOR-110

E015D
VS110

E015E
VARIATION SELECTOR-111

E015E
VS111

E015F
VARIATION SELECTOR-112

E015F
VS112

E0160
VARIATION SELECTOR-113

E0160
VS113

E0161
VARIATION SELECTOR-114

E0161
VS114

E0162
VARIATION SELECTOR-115

E0162
VS115

E0163
VARIATION SELECTOR-116

E0163
VS116

E0164
VARIATION SELECTOR-117

E0164
VS117

E0165
VARIATION SELECTOR-118

E0165
VS118

E0166
VARIATION SELECTOR-119

E0166
VS119

E0167
VARIATION SELECTOR-120

E0167
VS120

E0168
VARIATION SELECTOR-121

E0168
VS121

E0169
VARIATION SELECTOR-122

E0169
VS122

E016A
VARIATION SELECTOR-123

E016A
VS123

E016B
VARIATION SELECTOR-124

E016B
VS124

E016C
VARIATION SELECTOR-125

E016C
VS125

E016D
VARIATION SELECTOR-126

E016D
VS126

E016E
VARIATION SELECTOR-127

E016E
VS127

E016F
VARIATION SELECTOR-128

E016F
VS128

E0170
VARIATION SELECTOR-129

E0170
VS129

E0171
VARIATION SELECTOR-130

E0171
VS130

E0172
VARIATION SELECTOR-131

E0172
VS131

E0173
VARIATION SELECTOR-132

E0173
VS132

E0174
VARIATION SELECTOR-133

E0174
VS133

E0175
VARIATION SELECTOR-134

E0175
VS134

E0176
VARIATION SELECTOR-135

E0176
VS135

E0177
VARIATION SELECTOR-136

E0177
VS136

E0178
VARIATION SELECTOR-137

E0178
VS137

E0179
VARIATION SELECTOR-138

E0179
VS138

E017A
VARIATION SELECTOR-139

E017A
VS139

E017B
VARIATION SELECTOR-140

E017B
VS140

E017C
VARIATION SELECTOR-141

E017C
VS141

E017D
VARIATION SELECTOR-142

E017D
VS142

E017E
VARIATION SELECTOR-143

E017E
VS143

E017F
VARIATION SELECTOR-144

E017F
VS144

E0180
VARIATION SELECTOR-145

E0180
VS145

E0181
VARIATION SELECTOR-146

E0181
VS146

E0182
VARIATION SELECTOR-147

E0182
VS147

E0183
VARIATION SELECTOR-148

E0183
VS148

E0184
VARIATION SELECTOR-149

E0184
VS149

E0185
VARIATION SELECTOR-150

E0185
VS150

E0186
VARIATION SELECTOR-151

E0186
VS151

E0187
VARIATION SELECTOR-152

E0187
VS152

E0188
VARIATION SELECTOR-153

E0188
VS153

E0189
VARIATION SELECTOR-154

E0189
VS154

E018A
VARIATION SELECTOR-155

E018A
VS155

E018B
VARIATION SELECTOR-156

E018B
VS156

E018C
VARIATION SELECTOR-157

E018C
VS157

E018D
VARIATION SELECTOR-158

E018D
VS158

E018E
VARIATION SELECTOR-159

E018E
VS159

E018F
VARIATION SELECTOR-160

E018F
VS160

E0190
VARIATION SELECTOR-161

E0190
VS161

E0191
VARIATION SELECTOR-162

E0191
VS162

E0192
VARIATION SELECTOR-163

E0192
VS163

E0193
VARIATION SELECTOR-164

E0193
VS164

E0194
VARIATION SELECTOR-165

E0194
VS165

E0195
VARIATION SELECTOR-166

E0195
VS166

E0196
VARIATION SELECTOR-167

E0196
VS167

E0197
VARIATION SELECTOR-168

E0197
VS168

E0198
VARIATION SELECTOR-169

E0198
VS169

E0199
VARIATION SELECTOR-170

E0199
VS170

E019A
VARIATION SELECTOR-171

E019A
VS171

E019B
VARIATION SELECTOR-172

E019B
VS172

E019C
VARIATION SELECTOR-173

E019C
VS173

E019D
VARIATION SELECTOR-174

E019D
VS174

E019E
VARIATION SELECTOR-175

E019E
VS175

E019F
VARIATION SELECTOR-176

E019F
VS176

E01A0
VARIATION SELECTOR-177

E01A0
VS177

E01A1
VARIATION SELECTOR-178

E01A1
VS178

E01A2
VARIATION SELECTOR-179

E01A2
VS179

E01A3
VARIATION SELECTOR-180

E01A3
VS180

E01A4
VARIATION SELECTOR-181

E01A4
VS181

E01A5
VARIATION SELECTOR-182

E01A5
VS182

E01A6
VARIATION SELECTOR-183

E01A6
VS183

E01A7
VARIATION SELECTOR-184

E01A7
VS184

E01A8
VARIATION SELECTOR-185

E01A8
VS185

E01A9
VARIATION SELECTOR-186

E01A9
VS186

E01AA
VARIATION SELECTOR-187

E01AA
VS187

E01AB
VARIATION SELECTOR-188

E01AB
VS188

E01AC
VARIATION SELECTOR-189

E01AC
VS189

E01AD
VARIATION SELECTOR-190

E01AD
VS190

E01AE
VARIATION SELECTOR-191

E01AE
VS191

E01AF
VARIATION SELECTOR-192

E01AF
VS192

E01B0
VARIATION SELECTOR-193

E01B0
VS193

E01B1
VARIATION SELECTOR-194

E01B1
VS194

E01B2
VARIATION SELECTOR-195

E01B2
VS195

E01B3
VARIATION SELECTOR-196

E01B3
VS196

E01B4
VARIATION SELECTOR-197

E01B4
VS197

E01B5
VARIATION SELECTOR-198

E01B5
VS198

E01B6
VARIATION SELECTOR-199

E01B6
VS199

E01B7
VARIATION SELECTOR-200

E01B7
VS200

E01B8
VARIATION SELECTOR-201

E01B8
VS201

E01B9
VARIATION SELECTOR-202

E01B9
VS202

E01BA
VARIATION SELECTOR-203

E01BA
VS203

E01BB
VARIATION SELECTOR-204

E01BB
VS204

E01BC
VARIATION SELECTOR-205

E01BC
VS205

E01BD
VARIATION SELECTOR-206

E01BD
VS206

E01BE
VARIATION SELECTOR-207

E01BE
VS207

E01BF
VARIATION SELECTOR-208

E01BF
VS208

E01C0
VARIATION SELECTOR-209

E01C0
VS209

E01C1
VARIATION SELECTOR-210

E01C1
VS210

E01C2
VARIATION SELECTOR-211

E01C2
VS211

E01C3
VARIATION SELECTOR-212

E01C3
VS212

E01C4
VARIATION SELECTOR-213

E01C4
VS213

E01C5
VARIATION SELECTOR-214

E01C5
VS214

E01C6
VARIATION SELECTOR-215

E01C6
VS215

E01C7
VARIATION SELECTOR-216

E01C7
VS216

E01C8
VARIATION SELECTOR-217

E01C8
VS217

E01C9
VARIATION SELECTOR-218

E01C9
VS218

E01CA
VARIATION SELECTOR-219

E01CA
VS219

E01CB
VARIATION SELECTOR-220

E01CB
VS220

E01CC
VARIATION SELECTOR-221

E01CC
VS221

E01CD
VARIATION SELECTOR-222

E01CD
VS222

E01CE
VARIATION SELECTOR-223

E01CE
VS223

E01CF
VARIATION SELECTOR-224

E01CF
VS224

E01D0
VARIATION SELECTOR-225

E01D0
VS225

E01D1
VARIATION SELECTOR-226

E01D1
VS226

E01D2
VARIATION SELECTOR-227

E01D2
VS227

E01D3
VARIATION SELECTOR-228

E01D3
VS228

E01D4
VARIATION SELECTOR-229

E01D4
VS229

E01D5
VARIATION SELECTOR-230

E01D5
VS230

E01D6
VARIATION SELECTOR-231

E01D6
VS231

E01D7
VARIATION SELECTOR-232

E01D7
VS232

E01D8
VARIATION SELECTOR-233

E01D8
VS233

E01D9
VARIATION SELECTOR-234

E01D9
VS234

E01DA
VARIATION SELECTOR-235

E01DA
VS235

E01DB
VARIATION SELECTOR-236

E01DB
VS236

E01DC
VARIATION SELECTOR-237

E01DC
VS237

E01DD
VARIATION SELECTOR-238

E01DD
VS238

E01DE
VARIATION SELECTOR-239

E01DE
VS239

E01DF
VARIATION SELECTOR-240

E01DF
VS240

E01E0
VARIATION SELECTOR-241

E01E0
VS241

E01E1
VARIATION SELECTOR-242

E01E1
VS242

E01E2
VARIATION SELECTOR-243

E01E2
VS243

E01E3
VARIATION SELECTOR-244

E01E3
VS244

E01E4
VARIATION SELECTOR-245

E01E4
VS245

E01E5
VARIATION SELECTOR-246

E01E5
VS246

E01E6
VARIATION SELECTOR-247

E01E6
VS247

E01E7
VARIATION SELECTOR-248

E01E7
VS248

E01E8
VARIATION SELECTOR-249

E01E8
VS249

E01E9
VARIATION SELECTOR-250

E01E9
VS250

E01EA
VARIATION SELECTOR-251

E01EA
VS251

E01EB
VARIATION SELECTOR-252

E01EB
VS252

E01EC
VARIATION SELECTOR-253

E01EC
VS253

E01ED
VARIATION SELECTOR-254

E01ED
VS254

E01EE
VARIATION SELECTOR-255

E01EE
VS255

E01EF
VARIATION SELECTOR-256

E01EF
VS256

00023 0FE0F 020E3
KEYCAP NUMBER SIGN

0002A 0FE0F 020E3
KEYCAP ASTERISK

00030 0FE0F 020E3
KEYCAP DIGIT ZERO

00031 0FE0F 020E3
KEYCAP DIGIT ONE

00032 0FE0F 020E3
KEYCAP DIGIT TWO

00033 0FE0F 020E3
KEYCAP DIGIT THREE

00034 0FE0F 020E3
KEYCAP DIGIT FOUR

00035 0FE0F 020E3
KEYCAP DIGIT FIVE

00036 0FE0F 020E3
KEYCAP DIGIT SIX

00037 0FE0F 020E3
KEYCAP DIGIT SEVEN

00038 0FE0F 020E3
KEYCAP DIGIT EIGHT

00039 0FE0F 020E3
KEYCAP DIGIT NINE

00100 00300
LATIN CAPITAL LETTER A WITH MACRON AND GRAVE

00101 00300
LATIN SMALL LETTER A WITH MACRON AND GRAVE

0012A 00300
LATIN CAPITAL LETTER I WITH MACRON AND GRAVE

0012B 00300
LATIN SMALL LETTER I WITH MACRON AND GRAVE

0016A 00300
LATIN CAPITAL LETTER U WITH MACRON AND GRAVE

0016B 00300
LATIN SMALL LETTER U WITH MACRON AND GRAVE

00045 00329
LATIN CAPITAL LETTER E WITH VERTICAL LINE BELOW

00065 00329
LATIN SMALL LETTER E WITH VERTICAL LINE BELOW

000C8 00329
LATIN CAPITAL LETTER E WITH VERTICAL LINE BELOW AND GRAVE

000E8 00329
LATIN SMALL LETTER E WITH VERTICAL LINE BELOW AND GRAVE

000C9 00329
LATIN CAPITAL LETTER E WITH VERTICAL LINE BELOW AND ACUTE

000E9 00329
LATIN SMALL LETTER E WITH VERTICAL LINE BELOW AND ACUTE

0004F 00329
LATIN CAPITAL LETTER O WITH VERTICAL LINE BELOW

0006F 00329
LATIN SMALL LETTER O WITH VERTICAL LINE BELOW

000D2 00329
LATIN CAPITAL LETTER O WITH VERTICAL LINE BELOW AND GRAVE

000F2 00329
LATIN SMALL LETTER O WITH VERTICAL LINE BELOW AND GRAVE

000D3 00329
LATIN CAPITAL LETTER O WITH VERTICAL LINE BELOW AND ACUTE

000F3 00329
LATIN SMALL LETTER O WITH VERTICAL LINE BELOW AND ACUTE

00053 00329
LATIN CAPITAL LETTER S WITH VERTICAL LINE BELOW

00073 00329
LATIN SMALL LETTER S WITH VERTICAL LINE BELOW

000CA 00304
LATIN CAPITAL LETTER E WITH CIRCUMFLEX AND MACRON

000EA 00304
LATIN SMALL LETTER E WITH CIRCUMFLEX AND MACRON

000CA 0030C
LATIN CAPITAL LETTER E WITH CIRCUMFLEX AND CARON

000EA 0030C
LATIN SMALL LETTER E WITH CIRCUMFLEX AND CARON

00069 00307 00301
LATIN SMALL LETTER I WITH DOT ABOVE AND ACUTE

0006E 00360 00067
LATIN SMALL LETTER NG WITH TILDE ABOVE

00104 00301
LATIN CAPITAL LETTER A WITH OGONEK AND ACUTE

00105 00301
LATIN SMALL LETTER A WITH OGONEK AND ACUTE

00104 00303
LATIN CAPITAL LETTER A WITH OGONEK AND TILDE

00105 00303
LATIN SMALL LETTER A WITH OGONEK AND TILDE

00118 00301
LATIN CAPITAL LETTER E WITH OGONEK AND ACUTE

00119 00301
LATIN SMALL LETTER E WITH OGONEK AND ACUTE

00118 00303
LATIN CAPITAL LETTER E WITH OGONEK AND TILDE

00119 00303
LATIN SMALL LETTER E WITH OGONEK AND TILDE

00116 00301
LATIN CAPITAL LETTER E WITH DOT ABOVE AND ACUTE

00117 00301
LATIN SMALL LETTER E WITH DOT ABOVE AND ACUTE

00116 00303
LATIN CAPITAL LETTER E WITH DOT ABOVE AND TILDE

00117 00303
LATIN SMALL LETTER E WITH DOT ABOVE AND TILDE

00069 00307 00300
LATIN SMALL LETTER I WITH DOT ABOVE AND GRAVE

00069 00307 00303
LATIN SMALL LETTER I WITH DOT ABOVE AND TILDE

0012E 00301
LATIN CAPITAL LETTER I WITH OGONEK AND ACUTE

0012F 00307 00301
LATIN SMALL LETTER I WITH OGONEK AND DOT ABOVE AND ACUTE

0012E 00303
LATIN CAPITAL LETTER I WITH OGONEK AND TILDE

0012F 00307 00303
LATIN SMALL LETTER I WITH OGONEK AND DOT ABOVE AND TILDE

0004A 00303
LATIN CAPITAL LETTER J WITH TILDE

0006A 00307 00303
LATIN SMALL LETTER J WITH DOT ABOVE AND TILDE

0004C 00303
LATIN CAPITAL LETTER L WITH TILDE

0006C 00303
LATIN SMALL LETTER L WITH TILDE

0004D 00303
LATIN CAPITAL LETTER M WITH TILDE

0006D 00303
LATIN SMALL LETTER M WITH TILDE

00052 00303
LATIN CAPITAL LETTER R WITH TILDE

00072 00303
LATIN SMALL LETTER R WITH TILDE

00172 00301
LATIN CAPITAL LETTER U WITH OGONEK AND ACUTE

00173 00301
LATIN SMALL LETTER U WITH OGONEK AND ACUTE

00172 00303
LATIN CAPITAL LETTER U WITH OGONEK AND TILDE

00173 00303
LATIN SMALL LETTER U WITH OGONEK AND TILDE

0016A 00301
LATIN CAPITAL LETTER U WITH MACRON AND ACUTE

0016B 00301
LATIN SMALL LETTER U WITH MACRON AND ACUTE

0016A 00303
LATIN CAPITAL LETTER U WITH MACRON AND TILDE

0016B 00303
LATIN SMALL LETTER U WITH MACRON AND TILDE

000E6 00300
LATIN SMALL LETTER AE WITH GRAVE

00254 00300
LATIN SMALL LETTER OPEN O WITH GRAVE

00254 00301
LATIN SMALL LETTER OPEN O WITH ACUTE

0028C 00300
LATIN SMALL LETTER TURNED V WITH GRAVE

0028C 00301
LATIN SMALL LETTER TURNED V WITH ACUTE

00259 00300
LATIN SMALL LETTER SCHWA WITH GRAVE

00259 00301
LATIN SMALL LETTER SCHWA WITH ACUTE

0025A 00300
LATIN SMALL LETTER HOOKED SCHWA WITH GRAVE

0025A 00301
LATIN SMALL LETTER HOOKED SCHWA WITH ACUTE

00626 00627
ARABIC SEQUENCE YEH WITH HAMZA ABOVE WITH ALEF

00626 00648
ARABIC SEQUENCE YEH WITH HAMZA ABOVE WITH WAW

00626 00649
ARABIC SEQUENCE YEH WITH HAMZA ABOVE WITH ALEF MAKSURA

00626 006C6
ARABIC SEQUENCE YEH WITH HAMZA ABOVE WITH OE

00626 006C7
ARABIC SEQUENCE YEH WITH HAMZA ABOVE WITH U

00626 006C8
ARABIC SEQUENCE YEH WITH HAMZA ABOVE WITH YU

00626 006D0
ARABIC SEQUENCE YEH WITH HAMZA ABOVE WITH E

00626 006D5
ARABIC SEQUENCE YEH WITH HAMZA ABOVE WITH AE

00646 006A9
ARABIC SEQUENCE NOON WITH KEHEH

00915 0093C
DEVANAGARI SEQUENCE FOR LETTER QA

00916 0093C
DEVANAGARI SEQUENCE FOR LETTER KHHA

00917 0093C
DEVANAGARI SEQUENCE FOR LETTER GHHA

0091C 0093C
DEVANAGARI SEQUENCE FOR LETTER ZA

00921 0093C
DEVANAGARI SEQUENCE FOR LETTER DDDHA

00922 0093C
DEVANAGARI SEQUENCE FOR LETTER RHA

0092B 0093C
DEVANAGARI SEQUENCE FOR LETTER FA

0092F 0093C
DEVANAGARI SEQUENCE FOR LETTER YYA

009A1 009BC
BENGALI SEQUENCE FOR LETTER RRA

009A2 009BC
BENGALI SEQUENCE FOR LETTER RHA

009AF 009BC
BENGALI SEQUENCE FOR LETTER YYA

00A32 00A3C
GURMUKHI SEQUENCE FOR LETTER LLA

00A38 00A3C
GURMUKHI SEQUENCE FOR LETTER SHA

00A16 00A3C
GURMUKHI SEQUENCE FOR LETTER KHHA

00A17 00A3C
GURMUKHI SEQUENCE FOR LETTER GHHA

00A1C 00A3C
GURMUKHI SEQUENCE FOR LETTER ZA

00A2B 00A3C
GURMUKHI SEQUENCE FOR LETTER FA

00B21 00B3C
ORIYA SEQUENCE FOR LETTER RRA

00B22 00B3C
ORIYA SEQUENCE FOR LETTER RHA

00995 009CD 009B7
BENGALI LETTER KHINYA

00B95 00BCD
TAMIL CONSONANT K

00B99 00BCD
TAMIL CONSONANT NG

00B9A 00BCD
TAMIL CONSONANT C

00B9E 00BCD
TAMIL CONSONANT NY

00B9F 00BCD
TAMIL CONSONANT TT

00BA3 00BCD
TAMIL CONSONANT NN

00BA4 00BCD
TAMIL CONSONANT T

00BA8 00BCD
TAMIL CONSONANT N

00BAA 00BCD
TAMIL CONSONANT P

00BAE 00BCD
TAMIL CONSONANT M

00BAF 00BCD
TAMIL CONSONANT Y

00BB0 00BCD
TAMIL CONSONANT R

00BB2 00BCD
TAMIL CONSONANT L

00BB5 00BCD
TAMIL CONSONANT V

00BB4 00BCD
TAMIL CONSONANT LLL

00BB3 00BCD
TAMIL CONSONANT LL

00BB1 00BCD
TAMIL CONSONANT RR

00BA9 00BCD
TAMIL CONSONANT NNN

00B9C 00BCD
TAMIL CONSONANT J

00BB6 00BCD
TAMIL CONSONANT SH

00BB7 00BCD
TAMIL CONSONANT SS

00BB8 00BCD
TAMIL CONSONANT S

00BB9 00BCD
TAMIL CONSONANT H

00B95 00BCD 00BB7 00BCD
TAMIL CONSONANT KSS

00B95 00BBE
TAMIL SYLLABLE KAA

00B95 00BBF
TAMIL SYLLABLE KI

00B95 00BC0
TAMIL SYLLABLE KII

00B95 00BC1
TAMIL SYLLABLE KU

00B95 00BC2
TAMIL SYLLABLE KUU

00B95 00BC6
TAMIL SYLLABLE KE

00B95 00BC7
TAMIL SYLLABLE KEE

00B95 00BC8
TAMIL SYLLABLE KAI

00B95 00BCA
TAMIL SYLLABLE KO

00B95 00BCB
TAMIL SYLLABLE KOO

00B95 00BCC
TAMIL SYLLABLE KAU

00B99 00BBE
TAMIL SYLLABLE NGAA

00B99 00BBF
TAMIL SYLLABLE NGI

00B99 00BC0
TAMIL SYLLABLE NGII

00B99 00BC1
TAMIL SYLLABLE NGU

00B99 00BC2
TAMIL SYLLABLE NGUU

00B99 00BC6
TAMIL SYLLABLE NGE

00B99 00BC7
TAMIL SYLLABLE NGEE

00B99 00BC8
TAMIL SYLLABLE NGAI

00B99 00BCA
TAMIL SYLLABLE NGO

00B99 00BCB
TAMIL SYLLABLE NGOO

00B99 00BCC
TAMIL SYLLABLE NGAU

00B9A 00BBE
TAMIL SYLLABLE CAA

00B9A 00BBF
TAMIL SYLLABLE CI

00B9A 00BC0
TAMIL SYLLABLE CII

00B9A 00BC1
TAMIL SYLLABLE CU

00B9A 00BC2
TAMIL SYLLABLE CUU

00B9A 00BC6
TAMIL SYLLABLE CE

00B9A 00BC7
TAMIL SYLLABLE CEE

00B9A 00BC8
TAMIL SYLLABLE CAI

00B9A 00BCA
TAMIL SYLLABLE CO

00B9A 00BCB
TAMIL SYLLABLE COO

00B9A 00BCC
TAMIL SYLLABLE CAU

00B9E 00BBE
TAMIL SYLLABLE NYAA

00B9E 00BBF
TAMIL SYLLABLE NYI

00B9E 00BC0
TAMIL SYLLABLE NYII

00B9E 00BC1
TAMIL SYLLABLE NYU

00B9E 00BC2
TAMIL SYLLABLE NYUU

00B9E 00BC6
TAMIL SYLLABLE NYE

00B9E 00BC7
TAMIL SYLLABLE NYEE

00B9E 00BC8
TAMIL SYLLABLE NYAI

00B9E 00BCA
TAMIL SYLLABLE NYO

00B9E 00BCB
TAMIL SYLLABLE NYOO

00B9E 00BCC
TAMIL SYLLABLE NYAU

00B9F 00BBE
TAMIL SYLLABLE TTAA

00B9F 00BBF
TAMIL SYLLABLE TTI

00B9F 00BC0
TAMIL SYLLABLE TTII

00B9F 00BC1
TAMIL SYLLABLE TTU

00B9F 00BC2
TAMIL SYLLABLE TTUU

00B9F 00BC6
TAMIL SYLLABLE TTE

00B9F 00BC7
TAMIL SYLLABLE TTEE

00B9F 00BC8
TAMIL SYLLABLE TTAI

00B9F 00BCA
TAMIL SYLLABLE TTO

00B9F 00BCB
TAMIL SYLLABLE TTOO

00B9F 00BCC
TAMIL SYLLABLE TTAU

00BA3 00BBE
TAMIL SYLLABLE NNAA

00BA3 00BBF
TAMIL SYLLABLE NNI

00BA3 00BC0
TAMIL SYLLABLE NNII

00BA3 00BC1
TAMIL SYLLABLE NNU

00BA3 00BC2
TAMIL SYLLABLE NNUU

00BA3 00BC6
TAMIL SYLLABLE NNE

00BA3 00BC7
TAMIL SYLLABLE NNEE

00BA3 00BC8
TAMIL SYLLABLE NNAI

00BA3 00BCA
TAMIL SYLLABLE NNO

00BA3 00BCB
TAMIL SYLLABLE NNOO

00BA3 00BCC
TAMIL SYLLABLE NNAU

00BA4 00BBE
TAMIL SYLLABLE TAA

00BA4 00BBF
TAMIL SYLLABLE TI

00BA4 00BC0
TAMIL SYLLABLE TII

00BA4 00BC1
TAMIL SYLLABLE TU

00BA4 00BC2
TAMIL SYLLABLE TUU

00BA4 00BC6
TAMIL SYLLABLE TE

00BA4 00BC7
TAMIL SYLLABLE TEE

00BA4 00BC8
TAMIL SYLLABLE TAI

00BA4 00BCA
TAMIL SYLLABLE TO

00BA4 00BCB
TAMIL SYLLABLE TOO

00BA4 00BCC
TAMIL SYLLABLE TAU

00BA8 00BBE
TAMIL SYLLABLE NAA

00BA8 00BBF
TAMIL SYLLABLE NI

00BA8 00BC0
TAMIL SYLLABLE NII

00BA8 00BC1
TAMIL SYLLABLE NU

00BA8 00BC2
TAMIL SYLLABLE NUU

00BA8 00BC6
TAMIL SYLLABLE NE

00BA8 00BC7
TAMIL SYLLABLE NEE

00BA8 00BC8
TAMIL SYLLABLE NAI

00BA8 00BCA
TAMIL SYLLABLE NO

00BA8 00BCB
TAMIL SYLLABLE NOO

00BA8 00BCC
TAMIL SYLLABLE NAU

00BAA 00BBE
TAMIL SYLLABLE PAA

00BAA 00BBF
TAMIL SYLLABLE PI

00BAA 00BC0
TAMIL SYLLABLE PII

00BAA 00BC1
TAMIL SYLLABLE PU

00BAA 00BC2
TAMIL SYLLABLE PUU

00BAA 00BC6
TAMIL SYLLABLE PE

00BAA 00BC7
TAMIL SYLLABLE PEE

00BAA 00BC8
TAMIL SYLLABLE PAI

00BAA 00BCA
TAMIL SYLLABLE PO

00BAA 00BCB
TAMIL SYLLABLE POO

00BAA 00BCC
TAMIL SYLLABLE PAU

00BAE 00BBE
TAMIL SYLLABLE MAA

00BAE 00BBF
TAMIL SYLLABLE MI

00BAE 00BC0
TAMIL SYLLABLE MII

00BAE 00BC1
TAMIL SYLLABLE MU

00BAE 00BC2
TAMIL SYLLABLE MUU

00BAE 00BC6
TAMIL SYLLABLE ME

00BAE 00BC7
TAMIL SYLLABLE MEE

00BAE 00BC8
TAMIL SYLLABLE MAI

00BAE 00BCA
TAMIL SYLLABLE MO

00BAE 00BCB
TAMIL SYLLABLE MOO

00BAE 00BCC
TAMIL SYLLABLE MAU

00BAF 00BBE
TAMIL SYLLABLE YAA

00BAF 00BBF
TAMIL SYLLABLE YI

00BAF 00BC0
TAMIL SYLLABLE YII

00BAF 00BC1
TAMIL SYLLABLE YU

00BAF 00BC2
TAMIL SYLLABLE YUU

00BAF 00BC6
TAMIL SYLLABLE YE

00BAF 00BC7
TAMIL SYLLABLE YEE

00BAF 00BC8
TAMIL SYLLABLE YAI

00BAF 00BCA
TAMIL SYLLABLE YO

00BAF 00BCB
TAMIL SYLLABLE YOO

00BAF 00BCC
TAMIL SYLLABLE YAU

00BB0 00BBE
TAMIL SYLLABLE RAA

00BB0 00BBF
TAMIL SYLLABLE RI

00BB0 00BC0
TAMIL SYLLABLE RII

00BB0 00BC1
TAMIL SYLLABLE RU

00BB0 00BC2
TAMIL SYLLABLE RUU

00BB0 00BC6
TAMIL SYLLABLE RE

00BB0 00BC7
TAMIL SYLLABLE REE

00BB0 00BC8
TAMIL SYLLABLE RAI

00BB0 00BCA
TAMIL SYLLABLE RO

00BB0 00BCB
TAMIL SYLLABLE ROO

00BB0 00BCC
TAMIL SYLLABLE RAU

00BB2 00BBE
TAMIL SYLLABLE LAA

00BB2 00BBF
TAMIL SYLLABLE LI

00BB2 00BC0
TAMIL SYLLABLE LII

00BB2 00BC1
TAMIL SYLLABLE LU

00BB2 00BC2
TAMIL SYLLABLE LUU

00BB2 00BC6
TAMIL SYLLABLE LE

00BB2 00BC7
TAMIL SYLLABLE LEE

00BB2 00BC8
TAMIL SYLLABLE LAI

00BB2 00BCA
TAMIL SYLLABLE LO

00BB2 00BCB
TAMIL SYLLABLE LOO

00BB2 00BCC
TAMIL SYLLABLE LAU

00BB5 00BBE
TAMIL SYLLABLE VAA

00BB5 00BBF
TAMIL SYLLABLE VI

00BB5 00BC0
TAMIL SYLLABLE VII

00BB5 00BC1
TAMIL SYLLABLE VU

00BB5 00BC2
TAMIL SYLLABLE VUU

00BB5 00BC6
TAMIL SYLLABLE VE

00BB5 00BC7
TAMIL SYLLABLE VEE

00BB5 00BC8
TAMIL SYLLABLE VAI

00BB5 00BCA
TAMIL SYLLABLE VO

00BB5 00BCB
TAMIL SYLLABLE VOO

00BB5 00BCC
TAMIL SYLLABLE VAU

00BB4 00BBE
TAMIL SYLLABLE LLLAA

00BB4 00BBF
TAMIL SYLLABLE LLLI

00BB4 00BC0
TAMIL SYLLABLE LLLII

00BB4 00BC1
TAMIL SYLLABLE LLLU

00BB4 00BC2
TAMIL SYLLABLE LLLUU

00BB4 00BC6
TAMIL SYLLABLE LLLE

00BB4 00BC7
TAMIL SYLLABLE LLLEE

00BB4 00BC8
TAMIL SYLLABLE LLLAI

00BB4 00BCA
TAMIL SYLLABLE LLLO

00BB4 00BCB
TAMIL SYLLABLE LLLOO

00BB4 00BCC
TAMIL SYLLABLE LLLAU

00BB3 00BBE
TAMIL SYLLABLE LLAA

00BB3 00BBF
TAMIL SYLLABLE LLI

00BB3 00BC0
TAMIL SYLLABLE LLII

00BB3 00BC1
TAMIL SYLLABLE LLU

00BB3 00BC2
TAMIL SYLLABLE LLUU

00BB3 00BC6
TAMIL SYLLABLE LLE

00BB3 00BC7
TAMIL SYLLABLE LLEE

00BB3 00BC8
TAMIL SYLLABLE LLAI

00BB3 00BCA
TAMIL SYLLABLE LLO

00BB3 00BCB
TAMIL SYLLABLE LLOO

00BB3 00BCC
TAMIL SYLLABLE LLAU

00BB1 00BBE
TAMIL SYLLABLE RRAA

00BB1 00BBF
TAMIL SYLLABLE RRI

00BB1 00BC0
TAMIL SYLLABLE RRII

00BB1 00BC1
TAMIL SYLLABLE RRU

00BB1 00BC2
TAMIL SYLLABLE RRUU

00BB1 00BC6
TAMIL SYLLABLE RRE

00BB1 00BC7
TAMIL SYLLABLE RREE

00BB1 00BC8
TAMIL SYLLABLE RRAI

00BB1 00BCA
TAMIL SYLLABLE RRO

00BB1 00BCB
TAMIL SYLLABLE RROO

00BB1 00BCC
TAMIL SYLLABLE RRAU

00BA9 00BBE
TAMIL SYLLABLE NNNAA

00BA9 00BBF
TAMIL SYLLABLE NNNI

00BA9 00BC0
TAMIL SYLLABLE NNNII

00BA9 00BC1
TAMIL SYLLABLE NNNU

00BA9 00BC2
TAMIL SYLLABLE NNNUU

00BA9 00BC6
TAMIL SYLLABLE NNNE

00BA9 00BC7
TAMIL SYLLABLE NNNEE

00BA9 00BC8
TAMIL SYLLABLE NNNAI

00BA9 00BCA
TAMIL SYLLABLE NNNO

00BA9 00BCB
TAMIL SYLLABLE NNNOO

00BA9 00BCC
TAMIL SYLLABLE NNNAU

00B9C 00BBE
TAMIL SYLLABLE JAA

00B9C 00BBF
TAMIL SYLLABLE JI

00B9C 00BC0
TAMIL SYLLABLE JII

00B9C 00BC1
TAMIL SYLLABLE JU

00B9C 00BC2
TAMIL SYLLABLE JUU

00B9C 00BC6
TAMIL SYLLABLE JE

00B9C 00BC7
TAMIL SYLLABLE JEE

00B9C 00BC8
TAMIL SYLLABLE JAI

00B9C 00BCA
TAMIL SYLLABLE JO

00B9C 00BCB
TAMIL SYLLABLE JOO

00B9C 00BCC
TAMIL SYLLABLE JAU

00BB6 00BBE
TAMIL SYLLABLE SHAA

00BB6 00BBF
TAMIL SYLLABLE SHI

00BB6 00BC0
TAMIL SYLLABLE SHII

00BB6 00BC1
TAMIL SYLLABLE SHU

00BB6 00BC2
TAMIL SYLLABLE SHUU

00BB6 00BC6
TAMIL SYLLABLE SHE

00BB6 00BC7
TAMIL SYLLABLE SHEE

00BB6 00BC8
TAMIL SYLLABLE SHAI

00BB6 00BCA
TAMIL SYLLABLE SHO

00BB6 00BCB
TAMIL SYLLABLE SHOO

00BB6 00BCC
TAMIL SYLLABLE SHAU

00BB7 00BBE
TAMIL SYLLABLE SSAA

00BB7 00BBF
TAMIL SYLLABLE SSI

00BB7 00BC0
TAMIL SYLLABLE SSII

00BB7 00BC1
TAMIL SYLLABLE SSU

00BB7 00BC2
TAMIL SYLLABLE SSUU

00BB7 00BC6
TAMIL SYLLABLE SSE

00BB7 00BC7
TAMIL SYLLABLE SSEE

00BB7 00BC8
TAMIL SYLLABLE SSAI

00BB7 00BCA
TAMIL SYLLABLE SSO

00BB7 00BCB
TAMIL SYLLABLE SSOO

00BB7 00BCC
TAMIL SYLLABLE SSAU

00BB8 00BBE
TAMIL SYLLABLE SAA

00BB8 00BBF
TAMIL SYLLABLE SI

00BB8 00BC0
TAMIL SYLLABLE SII

00BB8 00BC1
TAMIL SYLLABLE SU

00BB8 00BC2
TAMIL SYLLABLE SUU

00BB8 00BC6
TAMIL SYLLABLE SE

00BB8 00BC7
TAMIL SYLLABLE SEE

00BB8 00BC8
TAMIL SYLLABLE SAI

00BB8 00BCA
TAMIL SYLLABLE SO

00BB8 00BCB
TAMIL SYLLABLE SOO

00BB8 00BCC
TAMIL SYLLABLE SAU

00BB9 00BBE
TAMIL SYLLABLE HAA

00BB9 00BBF
TAMIL SYLLABLE HI

00BB9 00BC0
TAMIL SYLLABLE HII

00BB9 00BC1
TAMIL SYLLABLE HU

00BB9 00BC2
TAMIL SYLLABLE HUU

00BB9 00BC6
TAMIL SYLLABLE HE

00BB9 00BC7
TAMIL SYLLABLE HEE

00BB9 00BC8
TAMIL SYLLABLE HAI

00BB9 00BCA
TAMIL SYLLABLE HO

00BB9 00BCB
TAMIL SYLLABLE HOO

00BB9 00BCC
TAMIL SYLLABLE HAU

00B95 00BCD 00BB7
TAMIL SYLLABLE KSSA

00B95 00BCD 00BB7 00BBE
TAMIL SYLLABLE KSSAA

00B95 00BCD 00BB7 00BBF
TAMIL SYLLABLE KSSI

00B95 00BCD 00BB7 00BC0
TAMIL SYLLABLE KSSII

00B95 00BCD 00BB7 00BC1
TAMIL SYLLABLE KSSU

00B95 00BCD 00BB7 00BC2
TAMIL SYLLABLE KSSUU

00B95 00BCD 00BB7 00BC6
TAMIL SYLLABLE KSSE

00B95 00BCD 00BB7 00BC7
TAMIL SYLLABLE KSSEE

00B95 00BCD 00BB7 00BC8
TAMIL SYLLABLE KSSAI

00B95 00BCD 00BB7 00BCA
TAMIL SYLLABLE KSSO

00B95 00BCD 00BB7 00BCB
TAMIL SYLLABLE KSSOO

00B95 00BCD 00BB7 00BCC
TAMIL SYLLABLE KSSAU

00BB6 00BCD 00BB0 00BC0
TAMIL SYLLABLE SHRII

00DCA 0200D 00DBA
SINHALA CONSONANT SIGN YANSAYA

00DCA 0200D 00DBB
SINHALA CONSONANT SIGN RAKAARAANSAYA

00DBB 00DCA 0200D
SINHALA CONSONANT SIGN REPAYA

010E3 00302
GEORGIAN LETTER U-BRJGU

017D2 01780
KHMER CONSONANT SIGN COENG KA

017D2 01781
KHMER CONSONANT SIGN COENG KHA

017D2 01782
KHMER CONSONANT SIGN COENG KO

017D2 01783
KHMER CONSONANT SIGN COENG KHO

017D2 01784
KHMER CONSONANT SIGN COENG NGO

017D2 01785
KHMER CONSONANT SIGN COENG CA

017D2 01786
KHMER CONSONANT SIGN COENG CHA

017D2 01787
KHMER CONSONANT SIGN COENG CO

017D2 01788
KHMER CONSONANT SIGN COENG CHO

017D2 01789
KHMER CONSONANT SIGN COENG NYO

017D2 0178A
KHMER CONSONANT SIGN COENG DA

017D2 0178B
KHMER CONSONANT SIGN COENG TTHA

017D2 0178C
KHMER CONSONANT SIGN COENG DO

017D2 0178D
KHMER CONSONANT SIGN COENG TTHO

017D2 0178E
KHMER CONSONANT SIGN COENG NA

017D2 0178F
KHMER CONSONANT SIGN COENG TA

017D2 01790
KHMER CONSONANT SIGN COENG THA

017D2 01791
KHMER CONSONANT SIGN COENG TO

017D2 01792
KHMER CONSONANT SIGN COENG THO

017D2 01793
KHMER CONSONANT SIGN COENG NO

017D2 01794
KHMER CONSONANT SIGN COENG BA

017D2 01795
KHMER CONSONANT SIGN COENG PHA

017D2 01796
KHMER CONSONANT SIGN COENG PO

017D2 01797
KHMER CONSONANT SIGN COENG PHO

017D2 01798
KHMER CONSONANT SIGN COENG MO

017D2 01799
KHMER CONSONANT SIGN COENG YO

017D2 0179A
KHMER CONSONANT SIGN COENG RO

017D2 0179B
KHMER CONSONANT SIGN COENG LO

017D2 0179C
KHMER CONSONANT SIGN COENG VO

017D2 0179D
KHMER CONSONANT SIGN COENG SHA

017D2 0179E
KHMER CONSONANT SIGN COENG SSA

017D2 0179F
KHMER CONSONANT SIGN COENG SA

017D2 017A0
KHMER CONSONANT SIGN COENG HA

017D2 017A1
KHMER CONSONANT SIGN COENG LA

017D2 017A2
KHMER VOWEL SIGN COENG QA

017D2 017A7
KHMER INDEPENDENT VOWEL SIGN COENG QU

017D2 017AB
KHMER INDEPENDENT VOWEL SIGN COENG RY

017D2 017AC
KHMER INDEPENDENT VOWEL SIGN COENG RYY

017D2 017AF
KHMER INDEPENDENT VOWEL SIGN COENG QE

017BB 017C6
KHMER VOWEL SIGN OM

017B6 017C6
KHMER VOWEL SIGN AAM

0304B 0309A
HIRAGANA LETTER BIDAKUON NGA

0304D 0309A
HIRAGANA LETTER BIDAKUON NGI

0304F 0309A
HIRAGANA LETTER BIDAKUON NGU

03051 0309A
HIRAGANA LETTER BIDAKUON NGE

03053 0309A
HIRAGANA LETTER BIDAKUON NGO

030AB 0309A
KATAKANA LETTER BIDAKUON NGA

030AD 0309A
KATAKANA LETTER BIDAKUON NGI

030AF 0309A
KATAKANA LETTER BIDAKUON NGU

030B1 0309A
KATAKANA LETTER BIDAKUON NGE

030B3 0309A
KATAKANA LETTER BIDAKUON NGO

030BB 0309A
KATAKANA LETTER AINU CE

030C4 0309A
KATAKANA LETTER AINU TU

030C8 0309A
KATAKANA LETTER AINU TO

031F7 0309A
KATAKANA LETTER AINU P

002E5 002E9
MODIFIER LETTER EXTRA-HIGH EXTRA-LOW CONTOUR TONE BAR

002E9 002E5
MODIFIER LETTER EXTRA-LOW EXTRA-HIGH CONTOUR TONE BAR

END
